/*
	The legend of Zelda: Breath of the wild - Master editor v20170705
	by Marc Robledo 2017
*/
var currentEditingItem=0;

SavegameEditor={
	Name:'The legend of Zelda: Breath of the wild (Master editor)',
	Filename:'game_data.sav',
	Version:20170705,


	/* Constants */
	Constants:{
		STRING_SIZE:0x80,

		/*						 v1.0    v1.1    v1.2    v1.3  */
		FILESIZE:				[896976, 897160, 897112, 907824],
		HEADER:					[0x24e2, 0x24ee, 0x2588, 0x29c0],
	},

	Hashes:[],
	HashesTypes:{},
	HashesNames:{},



	/* private functions */
	_toHexInt:function(i){var s=i.toString(16);while(s.length<8)s='0'+s;return '0x'+s},
	_writeBoolean:function(offset,val,arrayPos){if(arrayPos)tempFile.writeInt(offset+8*arrayPos,val?1:0);else tempFile.writeInt(offset,val?1:0)},
	_writeValue:function(offset,val,arrayPos){if(arrayPos)tempFile.writeInt(offset+8*arrayPos,val);else tempFile.writeInt(offset,val)},
	_writeString:function(offset,str){
		for(var i=0; i<16; i++){
			tempFile.writeBytes(offset,[0,0,0,0]);
			var fourBytes=str.substr(i*4, 4);
			for(j=0; j<fourBytes.length; j++){
				tempFile.writeByte(offset+j, fourBytes.charCodeAt(j));
			}
			offset+=8;
		}
	},

	_readString:function(offset){
		var txt='';
		for(var j=0; j<16; j++){
			txt+=tempFile.readString(offset,4);
			offset+=8;
		}
		return txt
	},


	/* check if savegame is valid */
	checkValidSavegame:function(){
		tempFile.littleEndian=false;
		for(var i=0; i<this.Constants.FILESIZE.length; i++){
			if(tempFile.fileSize===this.Constants.FILESIZE[i] && tempFile.readInt(0)===this.Constants.HEADER[i] && tempFile.readInt(4)===0xffffffff){
				setValue('version', 'v1.'+i+'.x');
				return true;
			}
		}
		return false
	},


	preload:function(){
		for(var i=0; i<HASHES.length; i++){
			var hashGroup=HASHES[i];
			for(var j=0; j<hashGroup.hashes.length; j+=2){
				var hashValue=hashGroup.hashes[j];
				var hashId=hashGroup.id?hashGroup.id+hashGroup.hashes[j+1]:hashGroup.hashes[j+1];

				this.HashesNames[hashValue]=hashId;
				this.HashesTypes[hashValue]=hashGroup.type;
			}
		}

		/* import new data from MrCheezy json */
		/*for(prop in TEST){
			var hashValue=parseInt(prop)>>>0;
			if(!this.HashesNames[hashValue]){
				this.HashesTypes[hashValue]=TEST[prop][0];
				this.HashesNames[hashValue]=TEST[prop][1]+' (NEW!)';
			}
		}*/
	},


	/* load function */
	load:function(){
		tempFile.littleEndian=false;
		tempFile.fileName='game_data.sav';

		this.Hashes=[];

		var textarea=document.createElement('textarea');
		document.body.appendChild(textarea);
		var previousHashValue=0;
		for(var i=0x0c; i<tempFile.fileSize-4; i+=8){
			var hashValue=tempFile.readInt(i);
			if(hashValue===previousHashValue)
				continue;
			var hashId=this.HashesNames[hashValue] || '___Unknown: '+this._toHexInt(hashValue);
			var hashType=this.HashesTypes[hashValue] || 's32';
			this.Hashes.push({id:hashId, hash:hashValue, type:hashType, offset:i});
			previousHashValue=hashValue;

			/*if(/ \(NEW!\)/.test(hashId)){
				textarea.value+=this._toHexInt(hashValue)+',\"'+hashId+'\",//'+hashType+'\n';
			}*/
		}
		this.Hashes.sort(function(a,b){
		if(a.id<b.id)return -1;
		if(a.id>b.id)return 1;
		return 0;
		});
		
		empty('the-editor');
		for(var i=0; i<this.Hashes.length; i++){
			var hash=this.Hashes[i];

			if(hash.type==='bool'){
				var input=checkbox(hash.id);
				input.offset=hash.offset+4;
				input.addEventListener('change', setBoolean);
				if(tempFile.readInt(hash.offset+4))
					input.checked=true;
				get('the-editor').appendChild(row(
					[8,4],
					label(input.id,hash.id),
					input
				));
			}else if(hash.type==='s32'){
				var input=inputNumber(hash.id, 0, 0xffffffff, tempFile.readInt(hash.offset+4));
				input.offset=hash.offset+4;
				input.addEventListener('change', setS32);
				get('the-editor').appendChild(row(
					[8,4],
					label(input.id,hash.id),
					input
				));
			}else{
				get('the-editor').appendChild(row(
					[12],
					span(hash.id+' ('+hash.type+')')
				));
			}
		}
	},

	/* save function */
	save:function(){}
}




function onScroll(){
	var h=document.getElementById('header-top').getBoundingClientRect().height;
	if(window.scrollY>h){
		document.getElementById('header').style.position='fixed';
		document.getElementById('header').style.top='-'+h+'px';
	}else{
		document.getElementById('header').style.position='absolute';
		document.getElementById('header').style.top='0px';
	}
}

window.addEventListener('load',function(){
	window.addEventListener('scroll',onScroll,false);
}, false);


function setBoolean(){
	SavegameEditor._writeBoolean(this.offset, this.checked, 0);
}
function setS32(){
	SavegameEditor._writeValue(this.offset, this.value, 0);
}








/*
	The legend of Zelda: Breath of the wild Savegame Editor - Hashes Module - v20170521
	this is just for debug purposes
	by Marc Robledo 2017

	all data here was datamined by MrCheeze:
	https://raw.githubusercontent.com/MrCheeze/botw-tools/master/gamedata.json
*/

var HASHES=[
	{type:"bool_array",hashes:[0x54e85ab6,"Enemy_Dragon_Ice_GrudgeAlive",0x87430c9d,"Horse_IsFamiliarityChecked",0xabd3efd8,"IsOpenItemCategory",0x824892be,"PorchItem_EquipFlag"]},
	{type:"string",hashes:[0x81068a31,"PlayerName",0x0bee9e46,"PlayerSavePosMapName",0xd913b769,"PlayerSavePosMapType"]},
	{type:"string64",hashes:[0xddb52297,"ActorName_SeakSensor_Slot0",0xaab21201,"ActorName_SeakSensor_Slot1",0x33bb43bb,"ActorName_SeakSensor_Slot2",0x1c10782f,"WolfLink_TodayFirstDetectedUID"]},
	{type:"string64_array",hashes:[0xf43eeba6,"AlbumPictureActorName",0x3863b57d,"AmiiboTouchHistory",0x750e9d0e,"AmiiboTouchHistoryTotal",0xccab71fd,"CookMaterialName0",0xbbac416b,"CookMaterialName1",0x22a510d1,"CookMaterialName2",0x55a22047,"CookMaterialName3",0xcbc6b5e4,"CookMaterialName4",0x71fe692e,"DeadHorse_ActorName",0x39d08876,"DeadHorse_AmiiboUidHash",0x43c96bc7,"DeadHorse_ManeName",0xd2e91906,"DeadHorse_ReinsName",0xf6332484,"DeadHorse_SaddleName",0xa4d177ef,"DeadHorse_UserName",0xfacfd421,"EquipStandBow_Slot",0x0f9674ff,"EquipStandShield_Slot",0x708311bc,"EquipStandSword_Slot",0xc247b696,"Horse_ActorName",0x00ac6bc9,"Horse_AmiiboUidHash",0x9c6cfd3f,"Horse_ManeName",0x6150c6be,"Horse_ReinsName",0x333aa6e5,"Horse_SaddleName",0x17cb2b11,"Horse_UsedAmiiboUidHash",0x7b74e117,"Horse_UserName",0x5f283289,"PorchItem"]},
	{type:"string256_array",hashes:[0x9b280a3b,"AlbumSaveDistrictName",0x2f95768f,"AlbumSaveLocationName",0x530dd198,"MemoryPhotoSaveDistrictName",0xe7b0ad2c,"MemoryPhotoSaveLocationName"]},
	{type:"s32_array",hashes:[0xd095066a,"AlbumPictureSize",0x3c5d9f91,"App_Menu_ButtonOrder",0x45f65a4f,"DeadHorse_CollarType",0x09597c61,"DeadHorse_DeadCause",0x0abd4874,"DeadHorse_FootType",0x39d1158f,"DeadHorse_RideTimeSec",0x20cf6d87,"EquipStandBow_FlagSp",0x16a5185b,"EquipStandBow_Life_Slot",0xb6747da9,"EquipStandBow_ValueSp",0xe9a2d3a7,"EquipStandShield_FlagSp",0x67adc0a1,"EquipStandShield_Life_Slot",0x8dd330df,"EquipStandShield_ValueSp",0x0f2e25a3,"EquipStandSword_FlagSp",0xc64ce1ab,"EquipStandSword_Life_Slot",0x8a587830,"EquipStandSword_ValueSp",0xd28041ad,"GodBless_LastUseWeek",0x80ffd82e,"Horse_CollarType",0xd518de8c,"Horse_FootType",0x03a14dc3,"Horse_RideTimeSec",0x9383490e,"MapApp_MapIconNo",0x0cbf052a,"PorchBow_FlagSp",0x1e3fd294,"PorchBow_ValueSp",0x6a09fc59,"PorchItem_Value1",0xc5238d2b,"PorchShield_FlagSp",0x69f17e8a,"PorchShield_ValueSp",0x57ee221d,"PorchSword_FlagSp",0xa6d926bc,"PorchSword_ValueSp",0x62a19a1f,"SeakSensorPictureSize",0xf7ec3c3a,"SmallKey",0x8e955bf2,"climateWeather",0x49e8b866,"climateWeather2",0x3eef88f0,"climateWeather3",]},
	{type:"s32",id:"Defeated_Enemy_",hashes:[0x37e3e861,"Assassin_Junior_Num",0x3b880649,"Assassin_Middle_Num",0x40980032,"Assassin_Senior_Num",0x20f2d141,"Bokoblin_Bone_Junior_Num",0x0539f312,"Bokoblin_Dark_Num",0xba93a7af,"Bokoblin_Junior_Num",0xb6f84987,"Bokoblin_Middle_Num",0xcde84ffc,"Bokoblin_Senior_Num",0xfe2980e1,"Chuchu_Electric_Junior_Num",0xeab6b9cc,"Chuchu_Fire_Junior_Num",0x48286f4c,"Chuchu_Ice_Junior_Num",0x4f3b99ff,"Chuchu_Junior_Num",0x6ecd67f2,"GanonBeast_Num",0x40a19377,"GanonGrudge_01_Num",0x0701e9a7,"GanonGrudge_02_Num",0xea8fa792,"GanonGrudge_Num",0xab93ff4c,"Giant_Bone_Num",0x29eac118,"Giant_Junior_Num",0x25812f30,"Giant_Middle_Num",0x5e91294b,"Giant_Senior_Num",0xfecdf539,"Golem_Fire_Num",0x3e3b8122,"Golem_Ice_Num",0xca920d7f,"Golem_Junior_Num",0xf662285d,"Golem_Little_Fire_Num",0xfcdbc86f,"Golem_Little_Ice_Num",0x325e258c,"Golem_Little_Num",0xc6f9e357,"Golem_Middle_Num",0xbde9e52c,"Golem_Senior_Num",0x5147851d,"Guardian_A_Fixed_Moss_Num",0x71aebd1c,"Guardian_A_Num",0x360ec7cc,"Guardian_B_Num",0x0b6eee7c,"Guardian_C_Num",0x4df1c130,"Guardian_Mini_Baby_Num",0x5412344c,"Guardian_Mini_Junior_Num",0x5879da64,"Guardian_Mini_Middle_Num",0x2369dc1f,"Guardian_Mini_Senior_Num",0xa52d2d74,"Keese_Electric_Num",0x265a76df,"Keese_Fire_Num",0xa493bd8a,"Keese_Ice_Num",0xaff7b730,"Keese_Num",0xad66391d,"Lizalfos_Bone_Junior_Num",0x0bba578d,"Lizalfos_Dark_Num",0x783dcd5b,"Lizalfos_Electric_Num",0xc540a0f6,"Lizalfos_Fire_Num",0xdd4e8602,"Lizalfos_Ice_Num",0x9df15808,"Lizalfos_Junior_Num",0x919ab620,"Lizalfos_Middle_Num",0xea8ab05b,"Lizalfos_Senior_Num",0x6814b509,"Lynel_Dark_Num",0x8c1ae5bd,"Lynel_Junior_Num",0x80710b95,"Lynel_Middle_Num",0xfb610dee,"Lynel_Senior_Num",0x3246d7cf,"Moriblin_Bone_Junior_Num",0x09690351,"Moriblin_Dark_Num",0xdc7a4525,"Moriblin_Junior_Num",0xd011ab0d,"Moriblin_Middle_Num",0xab01ad76,"Moriblin_Senior_Num",0x58f06192,"Octarock_Desert_Num",0xcedde80a,"Octarock_Forest_Num",0x090cb02d,"Octarock_Num",0x36a54e6d,"Octarock_Snow_Num",0xb2873686,"Octarock_Stone_Num",0x2d43d705,"Sandworm_Num",0x669be7d6,"SiteBoss_Bow_Num",0x93ace2c7,"SiteBoss_Lsword_Num",0x88da8d9b,"SiteBoss_Spear_Num",0x4c8dc9db,"SiteBoss_Sword_Num",0x573111b7,"Wizzrobe_Electric_Num",0x660fbef7,"Wizzrobe_Electric_Senior_Num",0x7c7d0643,"Wizzrobe_Fire_Num",0x02dd0f69,"Wizzrobe_Fire_Senior_Num",0x863c3a74,"Wizzrobe_Ice_Num",0x90e9d459,"Wizzrobe_Ice_Senior_Num"]},
	{type:"s32",id:"EquipTime_",hashes:[0x4412792f,"Armor_001_Head",0xdfedc63f,"Armor_001_Head_B",0x5c11789a,"Armor_001_Lower",0x3cd4dc25,"Armor_001_Upper",0xc2860b81,"Armor_002_Head",0x5162c1dc,"Armor_002_Head_B",0x6df96207,"Armor_002_Lower",0x0d3cc6b8,"Armor_002_Upper",0x09dad824,"Armor_003_Head",0x9dc8c142,"Armor_003_Head_B",0xcb8e69b3,"Armor_003_Lower",0xab4bcd0c,"Armor_003_Upper",0x14dfe89c,"Armor_004_Head",0x970dc85b,"Armor_004_Head_B",0x0e29573d,"Armor_004_Lower",0x6eecf382,"Armor_004_Upper",0xdf833b39,"Armor_005_Head",0xa85e5c89,"Armor_005_Lower",0xc89bf836,"Armor_005_Upper",0x59174997,"Armor_006_Head",0x99b64614,"Armor_006_Lower",0xf973e2ab,"Armor_006_Upper",0x924b9a32,"Armor_007_Head",0x3fc14da0,"Armor_007_Lower",0x5f04e91f,"Armor_007_Upper",0x631d28e7,"Armor_008_Head",0xc9893d49,"Armor_008_Lower",0xa94c99f6,"Armor_008_Upper",0xa841fb42,"Armor_009_Head",0x6ffe36fd,"Armor_009_Lower",0x0f3b9242,"Armor_009_Upper",0xe265729b,"Armor_011_Head",0x90bb7804,"Armor_011_Lower",0xf07edcbb,"Armor_011_Upper",0x64f10035,"Armor_012_Head",0x4619d59f,"Armor_012_Head_B",0xa1536299,"Armor_012_Lower",0xc196c626,"Armor_012_Upper",0xb2a8e328,"Armor_014_Head",0xc28357a3,"Armor_014_Lower",0xa246f31c,"Armor_014_Upper",0x79f4308d,"Armor_015_Head",0x4cdcdc86,"Armor_015_Head_B",0x64f45c17,"Armor_015_Lower",0x0431f8a8,"Armor_015_Upper",0x343c9186,"Armor_017_Head",0xf36b4d3e,"Armor_017_Lower",0x93aee981,"Armor_017_Upper",0x18d1bba3,"Armor_020_Head",0xb8437453,"Armor_020_Lower",0xd886d0ec,"Armor_020_Upper",0xd38d6806,"Armor_021_Head",0x1e347fe7,"Armor_021_Lower",0x7ef1db58,"Armor_021_Upper",0x55191aa8,"Armor_022_Head",0x7f94e95a,"Armor_022_Head_B",0x8340f9b5,"Armor_024_Head",0x481c2a10,"Armor_025_Head",0xce8858be,"Armor_026_Head",0x05d48b1b,"Armor_027_Head",0xf48239ce,"Armor_028_Head",0x3fdeea6b,"Armor_029_Head",0xee6b21a4,"Armor_035_Head",0x26d15b6a,"Armor_035_Lower",0x4614ffd5,"Armor_035_Upper",0x68ff530a,"Armor_036_Head",0x173941f7,"Armor_036_Lower",0x77fce548,"Armor_036_Upper",0xa3a380af,"Armor_037_Head",0xb14e4a43,"Armor_037_Lower",0xd18beefc,"Armor_037_Upper",0x99a9e1df,"Armor_039_Head",0xe171311e,"Armor_039_Lower",0x81b495a1,"Armor_039_Upper",0x7b018e99,"Armor_040_Head",0x7e2c7dd4,"Armor_040_Lower",0x1ee9d96b,"Armor_040_Upper",0x36c92f92,"Armor_042_Head",0x0c8e90d0,"Armor_042_Head_B",0xe9b36cfd,"Armor_042_Lower",0x8976c842,"Armor_042_Upper",0x4fc46749,"Armor_043_Lower",0x2f01c3f6,"Armor_043_Upper",0xeaa6fd78,"Armor_044_Upper",0x2bcc1f2a,"Armor_045_Head",0xad586d84,"Armor_046_Head",0x1dfc48ee,"Armor_046_Lower",0x7d39ec51,"Armor_046_Upper",0x97520cf4,"Armor_048_Head",0x4dc333b3,"Armor_048_Lower",0x2d06970c,"Armor_048_Upper",0xebb43807,"Armor_049_Lower",0x5be2f783,"Armor_053_Head",0x836e67d7,"Armor_053_Lower",0xe3abc368,"Armor_053_Upper",0x8dbb149e,"Armor_055_Head",0x0b2f6630,"Armor_056_Head",0xec9e9fb0,"Armor_060_Head",0x3c097aa9,"Armor_060_Lower",0x5cccde16,"Armor_060_Upper",0x27c24c15,"Armor_061_Head",0x9a7e711d,"Armor_061_Lower",0xfabbd5a2,"Armor_061_Upper",0xa1563ebb,"Armor_062_Head",0xab966b80,"Armor_062_Lower",0xcb53cf3f,"Armor_062_Upper",0x6a0aed1e,"Armor_063_Head",0x0de16034,"Armor_063_Lower",0x6d24c48b,"Armor_063_Upper",0x770fdda6,"Armor_064_Head",0xc8465eba,"Armor_064_Lower",0xa883fa05,"Armor_064_Upper",0xbc530e03,"Armor_065_Head",0x6e31550e,"Armor_065_Lower",0x0ef4f1b1,"Armor_065_Upper",0x3ac77cad,"Armor_066_Head",0x5fd94f93,"Armor_066_Lower",0x3f1ceb2c,"Armor_066_Upper",0xf19baf08,"Armor_067_Head",0xf9ae4427,"Armor_067_Lower",0x996be098,"Armor_067_Upper",0x81b547a1,"Armor_071_Head",0x56d47183,"Armor_071_Lower",0x3611d53c,"Armor_071_Upper",0x0721350f,"Armor_072_Head",0x673c6b1e,"Armor_072_Lower",0x07f9cfa1,"Armor_072_Upper",0xcc7de6aa,"Armor_073_Head",0xc14b60aa,"Armor_073_Lower",0xa18ec415,"Armor_073_Upper",0xd178d612,"Armor_074_Head",0x04ec5e24,"Armor_074_Lower",0x6429fa9b,"Armor_074_Upper",0x1a2405b7,"Armor_075_Head",0xa29b5590,"Armor_075_Lower",0xc25ef12f,"Armor_075_Upper",0x9cb07719,"Armor_076_Head",0x93734f0d,"Armor_076_Lower",0xf3b6ebb2,"Armor_076_Upper",0x57eca4bc,"Armor_077_Head",0x7de3a271,"Armor_077_Head_B",0x350444b9,"Armor_077_Lower",0x55c1e006,"Armor_077_Upper",0xa6ba1669,"Armor_078_Head",0xa4c3b0dd,"Armor_078_Head_B",0xc34c3450,"Armor_078_Lower",0xa38990ef,"Armor_078_Upper",0x6de6c5cc,"Armor_079_Head",0x6869b043,"Armor_079_Head_B",0x653b3fe4,"Armor_079_Lower",0x05fe9b5b,"Armor_079_Upper",0x3a359643,"Armor_083_Head",0x186b7206,"Armor_083_Lower",0x78aed6b9,"Armor_083_Upper",0x2730a6fb,"Armor_084_Head",0xddcc4c88,"Armor_084_Lower",0xbd09e837,"Armor_084_Upper",0xec6c755e,"Armor_085_Head",0x7bbb473c,"Armor_085_Lower",0x1b7ee383,"Armor_085_Upper",0x6af807f0,"Armor_086_Head",0x4a535da1,"Armor_086_Lower",0x2a96f91e,"Armor_086_Upper",0xa1a4d455,"Armor_087_Head",0xec245615,"Armor_087_Lower",0x8ce1f2aa,"Armor_087_Upper",0x50f26680,"Armor_088_Head",0x1a6c26fc,"Armor_088_Lower",0x7aa98243,"Armor_088_Upper",0x9baeb525,"Armor_089_Head",0xbc1b2d48,"Armor_089_Lower",0xdcde89f7,"Armor_089_Upper",0x1ad6ef59,"Armor_090_Head",0xe5296805,"Armor_090_Lower",0x85ecccba,"Armor_090_Upper",0x4a1b7eea,"Armor_095_Head",0xb71147a2,"Armor_095_Lower",0xd7d4e31d,"Armor_095_Upper",0xcc8f0c44,"Armor_096_Head",0x86f95d3f,"Armor_096_Lower",0xe63cf980,"Armor_096_Upper",0x07d3dfe1,"Armor_097_Head",0x208e568b,"Armor_097_Lower",0x404bf234,"Armor_097_Upper",0xf6856d34,"Armor_098_Head",0xd6c62662,"Armor_098_Lower",0xb60382dd,"Armor_098_Upper",0x3dd9be91,"Armor_099_Head",0x70b12dd6,"Armor_099_Lower",0x10748969,"Armor_099_Upper",0x43e4aa14,"Armor_100_Head",0xed1d676d,"Armor_100_Lower",0x8dd8c3d2,"Armor_100_Upper",0x88b879b1,"Armor_101_Head",0x4b6a6cd9,"Armor_101_Lower",0x2bafc866,"Armor_101_Upper",0x0e2c0b1f,"Armor_102_Head",0x7a827644,"Armor_102_Lower",0x1a47d2fb,"Armor_102_Upper",0xc570d8ba,"Armor_103_Head",0xdcf57df0,"Armor_103_Lower",0xbc30d94f,"Armor_103_Upper",0xd875e802,"Armor_104_Head",0x1952437e,"Armor_104_Lower",0x7997e7c1,"Armor_104_Upper",0x13293ba7,"Armor_105_Head",0xbf2548ca,"Armor_105_Lower",0xdfe0ec75,"Armor_105_Upper",0x95bd4909,"Armor_106_Head",0x8ecd5257,"Armor_106_Lower",0xee08f6e8,"Armor_106_Upper",0x2ecf7205,"Armor_111_Head",0x87c06c47,"Armor_111_Lower",0xe705c8f8,"Armor_111_Upper",0xa85b00ab,"Armor_112_Head",0xb62876da,"Armor_112_Lower",0xd6edd265,"Armor_112_Upper",0x6307d30e,"Armor_113_Head",0x105f7d6e,"Armor_113_Lower",0x709ad9d1,"Armor_113_Upper",0x7e02e3b6,"Armor_114_Head",0xd5f843e0,"Armor_114_Lower",0xb53de75f,"Armor_114_Upper",0xb55e3013,"Armor_115_Head",0x22a2f676,"Armor_116_Upper",0xf8969118,"Armor_117_Head",0x09c023cd,"Armor_118_Head",0xc29cf068,"Armor_119_Head",0xd47bbb3d,"Armor_120_Head",0x1f276898,"Armor_121_Head",0x99b31a36,"Armor_122_Head",0x52efc993,"Armor_123_Head",0x4feaf92b,"Armor_124_Head",0x84b62a8e,"Armor_125_Head",0x02225820,"Armor_126_Head",0xc97e8b85,"Armor_127_Head",0x38283950,"Armor_128_Head",0xf374eaf5,"Armor_129_Head",0x720cb089,"Armor_130_Head",0xb950632c,"Armor_131_Head",0x3fc41182,"Armor_132_Head",0xf498c227,"Armor_133_Head",0xe99df29f,"Armor_134_Head",0x22c1213a,"Armor_135_Head",0xa4555394,"Armor_136_Head",0x6f098031,"Armor_137_Head",0x9e5f32e4,"Armor_138_Head",0x5503e141,"Armor_139_Head",0xb7ab8e07,"Armor_140_Head",0x69576997,"Armor_140_Lower",0xcf206223,"Armor_141_Lower",0x3a7d834f,"Armor_148_Upper",0x9c0a88fb,"Armor_149_Upper",0xc538cdb6,"Armor_150_Upper",0x634fc602,"Armor_151_Upper",0x32627820,"Armor_152_Lower",0x94157394,"Armor_153_Lower",0x51b24d1a,"Armor_154_Lower",0xf7c546ae,"Armor_155_Lower",0xc62d5c33,"Armor_156_Lower",0x605a5787,"Armor_157_Lower",0x9612276e,"Armor_158_Lower",0x30652cda,"Armor_159_Lower",0x20349f2e,"Armor_160_Head",0x2b726eea,"Armor_160_Lower",0x4bb7ca55,"Armor_160_Upper",0x871dcacb,"Armor_170_Upper",0xcd6badf7,"Armor_200_Head",0xd4905ba8,"Armor_200_Lower",0xb455ff17,"Armor_200_Upper",0x06377e52,"Armor_201_Head",0x72e7501c,"Armor_201_Lower",0x1222f4a3,"Armor_201_Upper",0x80a30cfc,"Armor_202_Head",0x430f4a81,"Armor_202_Lower",0x23caee3e,"Armor_202_Upper",0x4bffdf59,"Armor_203_Head",0xe5784135,"Armor_203_Lower",0x85bde58a,"Armor_203_Upper",0x56faefe1,"Armor_204_Head",0x20df7fbb,"Armor_204_Lower",0x401adb04,"Armor_204_Upper",0x9da63c44,"Armor_205_Head",0x86a8740f,"Armor_205_Lower",0xe66dd0b0,"Armor_205_Upper",0x1b324eea,"Armor_206_Head",0xb7406e92,"Armor_206_Lower",0xd785ca2d,"Armor_206_Upper",0xd06e9d4f,"Armor_207_Head",0x11376526,"Armor_207_Lower",0x71f2c199,"Armor_207_Upper",0x21382f9a,"Armor_208_Head",0xe77f15cf,"Armor_208_Lower",0x87bab170,"Armor_208_Upper",0xea64fc3f,"Armor_209_Head",0x41081e7b,"Armor_209_Lower",0x21cdbac4,"Armor_209_Upper",0x6b1ca643,"Armor_210_Head",0x183a5b36,"Armor_210_Lower",0x78ffff89,"Armor_210_Upper",0xa04075e6,"Armor_211_Head",0xbe4d5082,"Armor_211_Lower",0xde88f43d,"Armor_211_Upper",0x26d40748,"Armor_212_Head",0x8fa54a1f,"Armor_212_Lower",0xef60eea0,"Armor_212_Upper",0xed88d4ed,"Armor_213_Head",0x29d241ab,"Armor_213_Lower",0x4917e514,"Armor_213_Upper",0xf08de455,"Armor_214_Head",0xec757f25,"Armor_214_Lower",0x8cb0db9a,"Armor_214_Upper",0x3bd137f0,"Armor_215_Head",0x4a027491,"Armor_215_Lower",0x2ac7d02e,"Armor_215_Upper",0xbd45455e,"Armor_216_Head",0x7bea6e0c,"Armor_216_Lower",0x1b2fcab3,"Armor_216_Upper",0x761996fb,"Armor_217_Head",0xdd9d65b8,"Armor_217_Lower",0xbd58c107,"Armor_217_Upper",0x874f242e,"Armor_218_Head",0x2bd51551,"Armor_218_Lower",0x4b10b1ee,"Armor_218_Upper",0x4c13f78b,"Armor_219_Head",0x8da21ee5,"Armor_219_Lower",0xed67ba5a,"Armor_219_Upper",0x5af4bcde,"Armor_220_Head",0x39443e1a,"Armor_220_Head_B",0x91a86f7b,"Armor_221_Head",0xf5ee3e84,"Armor_221_Head_B",0x173c1dd5,"Armor_222_Head",0x7b613967,"Armor_222_Head_B",0xdc60ce70,"Armor_223_Head",0xb7cb39f9,"Armor_223_Head_B",0xc165fec8,"Armor_224_Head",0xbd0e30e0,"Armor_224_Head_B",0x0a392d6d,"Armor_225_Head",0xc48d7372,"Armor_225_Lower",0xa448d7cd,"Armor_225_Upper",0x8cad5fc3,"Armor_226_Head",0xf56569ef,"Armor_226_Lower",0x95a0cd50,"Armor_226_Upper",0x47f18c66,"Armor_227_Head",0x5312625b,"Armor_227_Lower",0x33d7c6e4,"Armor_227_Upper",0xb6a73eb3,"Armor_228_Head",0xa55a12b2,"Armor_228_Lower",0xc59fb60d,"Armor_228_Upper",0x7dfbed16,"Armor_229_Head",0x032d1906,"Armor_229_Lower",0x63e8bdb9,"Armor_229_Upper",0xfc83b76a,"Armor_230_Head",0x5a1f5c4b,"Armor_230_Lower",0x3adaf8f4,"Armor_230_Upper",0x37df64cf,"Armor_231_Head",0xfc6857ff,"Armor_231_Lower",0x9cadf340,"Armor_231_Upper",0xb14b1661,"Armor_232_Head",0xcd804d62,"Armor_232_Lower",0xad45e9dd,"Armor_232_Upper",0x7a17c5c4,"Armor_233_Head",0x6bf746d6,"Armor_233_Lower",0x0b32e269,"Armor_233_Upper",0x6712f57c,"Armor_234_Head",0xae507858,"Armor_234_Lower",0xce95dce7,"Armor_234_Upper",0xc7aea4ee,"Armor_500_Head",0xb0f13661,"Armor_500_Lower",0xd03492de,"Armor_500_Upper",0x0cf2774b,"Armor_501_Head",0x16863dd5,"Armor_501_Lower",0x7643996a,"Armor_501_Upper",0x47ab83f7,"Armor_502_Upper",0x413ad640,"Armor_503_Head",0x30d11413,"Armor_Default_Head",0x13cf63cf,"Armor_Default_Upper",0xf54c5d91,"Obj_HeroSoul_Gerudo",0x73b645bf,"Obj_HeroSoul_Goron",0x50293056,"Obj_HeroSoul_Rito",0x20f2ee8a,"Obj_HeroSoul_Zora",0x041458e1,"TestSakookaBow",0x4f9eb00d,"TestSakookaWeapon",0xef02418d,"TestSakooka_Head",0x69fd2534,"Test_Sakooka_Shield",0x09baa0a6,"Test_Zero_Shield",0x6a193976,"Weapon_Bow_001",0xf31068cc,"Weapon_Bow_002",0x8417585a,"Weapon_Bow_003",0x1a73cdf9,"Weapon_Bow_004",0xf47dacd5,"Weapon_Bow_006",0x64c2b144,"Weapon_Bow_009",0x73020837,"Weapon_Bow_011",0x9d0c691b,"Weapon_Bow_013",0x0368fcb8,"Weapon_Bow_014",0x746fcc2e,"Weapon_Bow_015",0xed669d94,"Weapon_Bow_016",0x9a61ad02,"Weapon_Bow_017",0xb6213ad8,"Weapon_Bow_023",0xc64bce57,"Weapon_Bow_026",0xb14cfec1,"Weapon_Bow_027",0x21f3e350,"Weapon_Bow_028",0x56f4d3c6,"Weapon_Bow_029",0x36335a23,"Weapon_Bow_030",0xd83d3b0f,"Weapon_Bow_032",0xaf3a0b99,"Weapon_Bow_033",0x4659aeac,"Weapon_Bow_035",0xdf50ff16,"Weapon_Bow_036",0x38e8d211,"Weapon_Bow_038",0x7972cce4,"Weapon_Bow_040",0x2558afb1,"Weapon_Bow_071",0xbc51fe0b,"Weapon_Bow_072",0xc7855d61,"Weapon_Lsword_001",0x5e8c0cdb,"Weapon_Lsword_002",0x298b3c4d,"Weapon_Lsword_003",0xb7efa9ee,"Weapon_Lsword_004",0xc0e89978,"Weapon_Lsword_005",0x59e1c8c2,"Weapon_Lsword_006",0xa9995cb6,"Weapon_Lsword_010",0xde9e6c20,"Weapon_Lsword_011",0x47973d9a,"Weapon_Lsword_012",0x30900d0c,"Weapon_Lsword_013",0xaef498af,"Weapon_Lsword_014",0xd9f3a839,"Weapon_Lsword_015",0x40faf983,"Weapon_Lsword_016",0x37fdc915,"Weapon_Lsword_017",0xa742d484,"Weapon_Lsword_018",0xd045e412,"Weapon_Lsword_019",0x82b40f75,"Weapon_Lsword_020",0x1bbd5ecf,"Weapon_Lsword_023",0x85d9cb6c,"Weapon_Lsword_024",0x1cd09ad6,"Weapon_Lsword_027",0xfb68b7d1,"Weapon_Lsword_029",0x9baf3e34,"Weapon_Lsword_030",0xeca80ea2,"Weapon_Lsword_031",0x75a15f18,"Weapon_Lsword_032",0x02a66f8e,"Weapon_Lsword_033",0x9cc2fa2d,"Weapon_Lsword_034",0xebc5cabb,"Weapon_Lsword_035",0x72cc9b01,"Weapon_Lsword_036",0x05cbab97,"Weapon_Lsword_037",0x9574b606,"Weapon_Lsword_038",0xa3e99865,"Weapon_Lsword_041",0xa4845c7c,"Weapon_Lsword_045",0x4a8a3d50,"Weapon_Lsword_047",0xbaf2a924,"Weapon_Lsword_051",0xca985dab,"Weapon_Lsword_054",0xbd9f6d3d,"Weapon_Lsword_055",0x24963c87,"Weapon_Lsword_056",0x53910c11,"Weapon_Lsword_057",0xb4292116,"Weapon_Lsword_059",0xe6d8ca71,"Weapon_Lsword_060",0xf8ae3f29,"Weapon_Lsword_074",0xcb284414,"Weapon_Shield_001",0x522115ae,"Weapon_Shield_002",0x25262538,"Weapon_Shield_003",0xbb42b09b,"Weapon_Shield_004",0xcc45800d,"Weapon_Shield_005",0x554cd1b7,"Weapon_Shield_006",0x224be121,"Weapon_Shield_007",0xb2f4fcb0,"Weapon_Shield_008",0xc5f3cc26,"Weapon_Shield_009",0x3c3d1479,"Weapon_Shield_013",0xa25981da,"Weapon_Shield_014",0xd55eb14c,"Weapon_Shield_015",0x4c57e0f6,"Weapon_Shield_016",0x3b50d060,"Weapon_Shield_017",0xabefcdf1,"Weapon_Shield_018",0xf91e2696,"Weapon_Shield_021",0x6017772c,"Weapon_Shield_022",0x171047ba,"Weapon_Shield_023",0xfe73e28f,"Weapon_Shield_025",0x677ab335,"Weapon_Shield_026",0x97022741,"Weapon_Shield_030",0xe00517d7,"Weapon_Shield_031",0x790c466d,"Weapon_Shield_032",0x0e0b76fb,"Weapon_Shield_033",0x906fe358,"Weapon_Shield_034",0xe768d3ce,"Weapon_Shield_035",0x7e618274,"Weapon_Shield_036",0x0966b2e2,"Weapon_Shield_037",0x99d9af73,"Weapon_Shield_038",0xd843b186,"Weapon_Shield_040",0xaf448110,"Weapon_Shield_041",0x364dd0aa,"Weapon_Shield_042",0x5f3c1564,"Weapon_Shield_057",0x79840093,"Weapon_Spear_001",0xe08d5129,"Weapon_Spear_002",0x978a61bf,"Weapon_Spear_003",0x09eef41c,"Weapon_Spear_004",0x7ee9c48a,"Weapon_Spear_005",0xe7e09530,"Weapon_Spear_006",0x90e7a5a6,"Weapon_Spear_007",0x0058b837,"Weapon_Spear_008",0x775f88a1,"Weapon_Spear_009",0x17980144,"Weapon_Spear_010",0x609f31d2,"Weapon_Spear_011",0xf9966068,"Weapon_Spear_012",0x8e9150fe,"Weapon_Spear_013",0x10f5c55d,"Weapon_Spear_014",0x67f2f5cb,"Weapon_Spear_015",0xfefba471,"Weapon_Spear_016",0x89fc94e7,"Weapon_Spear_017",0x19438976,"Weapon_Spear_018",0x4bb26211,"Weapon_Spear_021",0xd2bb33ab,"Weapon_Spear_022",0xa5bc033d,"Weapon_Spear_023",0x3bd8969e,"Weapon_Spear_024",0x4cdfa608,"Weapon_Spear_025",0xa2d1c724,"Weapon_Spear_027",0x326edab5,"Weapon_Spear_028",0x4569ea23,"Weapon_Spear_029",0x25ae63c6,"Weapon_Spear_030",0x52a95350,"Weapon_Spear_031",0xcba002ea,"Weapon_Spear_032",0xbca7327c,"Weapon_Spear_033",0x22c3a7df,"Weapon_Spear_034",0x55c49749,"Weapon_Spear_035",0xcccdc6f3,"Weapon_Spear_036",0xbbcaf665,"Weapon_Spear_037",0x2b75ebf4,"Weapon_Spear_038",0xf48b60a2,"Weapon_Spear_047",0x13334da5,"Weapon_Spear_049",0x73f4c440,"Weapon_Spear_050",0xbdd344d3,"Weapon_Sword_001",0x24da1569,"Weapon_Sword_002",0x53dd25ff,"Weapon_Sword_003",0xcdb9b05c,"Weapon_Sword_004",0xbabe80ca,"Weapon_Sword_005",0x23b7d170,"Weapon_Sword_006",0x54b0e1e6,"Weapon_Sword_007",0xc40ffc77,"Weapon_Sword_008",0xb308cce1,"Weapon_Sword_009",0x4ac614be,"Weapon_Sword_013",0xd4a2811d,"Weapon_Sword_014",0xa3a5b18b,"Weapon_Sword_015",0x3aace031,"Weapon_Sword_016",0x4dabd0a7,"Weapon_Sword_017",0xdd14cd36,"Weapon_Sword_018",0xaa13fda0,"Weapon_Sword_019",0xf8e216c7,"Weapon_Sword_020",0x8fe52651,"Weapon_Sword_021",0x16ec77eb,"Weapon_Sword_022",0x61eb477d,"Weapon_Sword_023",0xff8fd2de,"Weapon_Sword_024",0x8888e248,"Weapon_Sword_025",0x66868364,"Weapon_Sword_027",0x813eae63,"Weapon_Sword_029",0xe1f92786,"Weapon_Sword_030",0x96fe1710,"Weapon_Sword_031",0x78f0763c,"Weapon_Sword_033",0xe694e39f,"Weapon_Sword_034",0x9193d309,"Weapon_Sword_035",0xaeb8b141,"Weapon_Sword_040",0xd9bf81d7,"Weapon_Sword_041",0x37b1e0fb,"Weapon_Sword_043",0xa9d57558,"Weapon_Sword_044",0x30dc24e2,"Weapon_Sword_047",0xa0633973,"Weapon_Sword_048",0xd76409e5,"Weapon_Sword_049",0xb7a38000,"Weapon_Sword_050",0xc0a4b096,"Weapon_Sword_051",0x59ade12c,"Weapon_Sword_052",0x2eaad1ba,"Weapon_Sword_053",0x29c715a3,"Weapon_Sword_057",0xb9780832,"Weapon_Sword_058",0xce7f38a4,"Weapon_Sword_059",0x9c8ed3c3,"Weapon_Sword_060",0xeb89e355,"Weapon_Sword_061",0x7280b2ef,"Weapon_Sword_062",0x8595e282,"Weapon_Sword_070",0x1c9cb338,"Weapon_Sword_073",0xe74f6fa2,"Weapon_Sword_DemoCheck"]},
	{type:"s32",id:"Location_",hashes:[0xbd432e2b,"AdeyaVillage",0x4c2a4c64,"AkkareBigBridge",0xc982b698,"AkkareGarrison",0xceec3555,"AkkareZhai",0x5ccee1d1,"AkkareZhaiBridge",0xe03f8aa7,"AncientLabo",0x01459d59,"AnoleTunnel",0xba8fa3ec,"Assassin",0xab69ff8d,"BigBrotherBridge",0x852308c6,"BigTree",0x763a58d2,"BlindForest",0x1855d45c,"BraveFountain",0x7b7427f3,"BrosRock",0x73fcf871,"CelebrationPark",0xf4b71776,"CentralExchange",0xd9393ccd,"ChirakaVillage",0x95db306c,"Cohorint",0x629dcd44,"Cokiri",0xf70a4075,"Corosseo",0x237a4d8a,"DeathMountainHatago",0xba52d9e9,"DeathMountain_Entrance",0x2a87cd61,"DodoraBridge",0xa8c3e430,"DragonAltar",0x1e4224f7,"DragonMarsh",0x9f65e395,"Dungeon000",0xe862d303,"Dungeon001",0x716b82b9,"Dungeon002",0x066cb22f,"Dungeon003",0x9808278c,"Dungeon004",0xef0f171a,"Dungeon005",0x760646a0,"Dungeon006",0x01017636,"Dungeon007",0x91be6ba7,"Dungeon008",0xe6b95b31,"Dungeon009",0x867ed2d4,"Dungeon010",0xf179e242,"Dungeon011",0x6870b3f8,"Dungeon012",0x1f77836e,"Dungeon013",0x811316cd,"Dungeon014",0xf614265b,"Dungeon015",0x6f1d77e1,"Dungeon016",0x181a4777,"Dungeon017",0x88a55ae6,"Dungeon018",0xffa26a70,"Dungeon019",0xad538117,"Dungeon020",0xda54b181,"Dungeon021",0x435de03b,"Dungeon022",0x345ad0ad,"Dungeon023",0xaa3e450e,"Dungeon024",0xdd397598,"Dungeon025",0x44302422,"Dungeon026",0x333714b4,"Dungeon027",0xa3880925,"Dungeon028",0xd48f39b3,"Dungeon029",0xb448b056,"Dungeon030",0xc34f80c0,"Dungeon031",0x5a46d17a,"Dungeon032",0x2d41e1ec,"Dungeon033",0xb325744f,"Dungeon034",0xc42244d9,"Dungeon035",0x5d2b1563,"Dungeon036",0x2a2c25f5,"Dungeon037",0xba933864,"Dungeon038",0xcd9408f2,"Dungeon039",0xfb092691,"Dungeon040",0x8c0e1607,"Dungeon041",0x150747bd,"Dungeon042",0x6200772b,"Dungeon043",0xfc64e288,"Dungeon044",0x8b63d21e,"Dungeon045",0x126a83a4,"Dungeon046",0x656db332,"Dungeon047",0xf5d2aea3,"Dungeon048",0x82d59e35,"Dungeon049",0xe21217d0,"Dungeon050",0x95152746,"Dungeon051",0x0c1c76fc,"Dungeon052",0x7b1b466a,"Dungeon053",0xe57fd3c9,"Dungeon054",0x9278e35f,"Dungeon055",0x0b71b2e5,"Dungeon056",0x7c768273,"Dungeon057",0xecc99fe2,"Dungeon058",0x9bceaf74,"Dungeon059",0xc93f4413,"Dungeon060",0xbe387485,"Dungeon061",0x2731253f,"Dungeon062",0x503615a9,"Dungeon063",0xce52800a,"Dungeon064",0xb955b09c,"Dungeon065",0x205ce126,"Dungeon066",0x575bd1b0,"Dungeon067",0xc7e4cc21,"Dungeon068",0xb0e3fcb7,"Dungeon069",0xd0247552,"Dungeon070",0xa72345c4,"Dungeon071",0x3e2a147e,"Dungeon072",0x492d24e8,"Dungeon073",0xd749b14b,"Dungeon074",0xa04e81dd,"Dungeon075",0x3947d067,"Dungeon076",0x4e40e0f1,"Dungeon077",0xdefffd60,"Dungeon078",0xa9f8cdf6,"Dungeon079",0x57bc699d,"Dungeon080",0x20bb590b,"Dungeon081",0xb9b208b1,"Dungeon082",0xceb53827,"Dungeon083",0x50d1ad84,"Dungeon084",0x27d69d12,"Dungeon085",0xbedfcca8,"Dungeon086",0xc9d8fc3e,"Dungeon087",0x5967e1af,"Dungeon088",0x2e60d139,"Dungeon089",0x4ea758dc,"Dungeon090",0x39a0684a,"Dungeon091",0xa0a939f0,"Dungeon092",0xd7ae0966,"Dungeon093",0x49ca9cc5,"Dungeon094",0x3ecdac53,"Dungeon095",0xa7c4fde9,"Dungeon096",0xd0c3cd7f,"Dungeon097",0x407cd0ee,"Dungeon098",0x377be078,"Dungeon099",0x9ea789a2,"Dungeon100",0xe9a0b934,"Dungeon101",0x70a9e88e,"Dungeon102",0x07aed818,"Dungeon103",0x99ca4dbb,"Dungeon104",0xeecd7d2d,"Dungeon105",0x77c42c97,"Dungeon106",0x00c31c01,"Dungeon107",0x907c0190,"Dungeon108",0xe77b3106,"Dungeon109",0x87bcb8e3,"Dungeon110",0xf0bb8875,"Dungeon111",0x69b2d9cf,"Dungeon112",0x1eb5e959,"Dungeon113",0x80d17cfa,"Dungeon114",0xf7d64c6c,"Dungeon115",0x6edf1dd6,"Dungeon116",0x19d82d40,"Dungeon117",0x896730d1,"Dungeon118",0xfe600047,"Dungeon119",0x25c12f63,"DungeonMush",0xd3d4fe4d,"EastDam",0x4edc4932,"EastGerudoRuins",0x90efa510,"EastPostTownRuin",0x93c23bdf,"Execution",0x3dee43cc,"EzzuBridge",0xda0163d5,"FaronHatago000",0xad065343,"FaronHatago001",0x340f02f9,"FaronHatago002",0x382af36e,"FironeForestEntrance",0xe27ea0e4,"FirstColony",0x8d8982b9,"FlamingoSpa",0xdbe04653,"FloriaBridge",0x5b3d301c,"FlowerSandbank",0x42d8f1bb,"ForestHatago",0x3046920f,"FrontGatePosttown",0x8a36f4dd,"FrozenCave",0xc17f8588,"GameBalling",0xdcbf49ee,"GameBitalockGolf",0x37d054d1,"GameGutsCriff",0xede11b4b,"GameMarathon",0x13df032f,"GameShiledSurf",0xb444af7f,"GameYabusame",0xb07ff371,"GameZarashiRace",0x86377b7e,"GarakishiVillage",0xc7d9e847,"GatakaraGarrison",0x7e6ac510,"Gerudo",0x11ca12b8,"GerudoCanyon_Entrance",0x3cb06c8d,"GerudoDesert_Entrance",0x8470bba5,"GerudoFossil",0x84b55727,"GerudoHatago",0xeb962ee5,"Goron",0x6b3090c4,"GoronBridge",0xf105fc93,"GutinizaPlainTomb",0x61ecf5d8,"HachijoBridge",0xb5bacc44,"Hatago001",0xb91ec523,"Hateno",0x5b065726,"HatenoGate",0x704a8cee,"HatenoLabo",0xfaab284d,"HebraBottomRemains",0xe66f047b,"HebraRemains",0x3ba1e6cd,"HeburaFossil",0xb46d4340,"HeburaLodge",0xa4e25e17,"HopesPlateau",0x506d96b6,"HyralBridge",0xcb844395,"HyruleCastle",0x47cf0ce2,"HyruleCastleTown",0xf579c518,"HyruleCastle_Hall_0",0x827ef58e,"HyruleCastle_Hall_1",0x1b77a434,"HyruleCastle_Hall_2",0xfb739f5d,"HyruleCastle_Room_0",0x8c74afcb,"HyruleCastle_Room_1",0xf8e1b0b6,"HyruleCastle_Room_10",0x8fe68020,"HyruleCastle_Room_11",0x157dfe71,"HyruleCastle_Room_2",0x627acee7,"HyruleCastle_Room_3",0xfc1e5b44,"HyruleCastle_Room_4",0x8b196bd2,"HyruleCastle_Room_5",0x12103a68,"HyruleCastle_Room_6",0x65170afe,"HyruleCastle_Room_7",0xf5a8176f,"HyruleCastle_Room_8",0x82af27f9,"HyruleCastle_Room_9",0xefc46622,"HyruleCentralPlace",0x30dce738,"HyruleDepthHatago",0x05768b36,"HyruleGarrison",0x29693cd5,"HyruleGround",0xc19b4064,"HyruleLabo",0x96fb7e0b,"HyrulePark",0x5c4d1750,"HyrulePlace",0x97185f06,"HyruleTemple",0x765ae3fd,"HyruleTower",0xd3635047,"HyruleWater",0xcab23d6f,"HyruleWestHatago",0x2e573fa2,"IceHouse",0x493163ed,"IceLake",0x0ea2f805,"JogoBridge",0x9aa32eca,"JokuBridge",0xd8b7a1e2,"Kakariko",0x384fc3a6,"KaturaCastle",0x1e278d65,"KingValley",0xc1d8c92c,"KingfisherSpa",0xb58d951c,"KochuBridge",0x94e41520,"KumSpa",0xcb7f8efb,"KunaiBridge",0xd8ce1bad,"LakeFloria",0xb106e3d2,"LakeHylia",0x64556cdf,"LanayruEastEntrance",0x34af6c3e,"LanayruWestEntrance",0xd3acb37d,"LeMarbeBridge",0x658ed94c,"LightningPlateau",0x791c7d90,"LittleBrotherBridge",0x9f1ff60d,"LomeiIslands",0x5fb1238c,"LostForest",0x40f556e8,"MapTower01",0xd9fc0752,"MapTower02",0xaefb37c4,"MapTower03",0x309fa267,"MapTower04",0x479892f1,"MapTower05",0xde91c34b,"MapTower06",0xa996f3dd,"MapTower07",0x3929ee4c,"MapTower08",0x4e2ededa,"MapTower09",0x2ee9573f,"MapTower10",0x59ee67a9,"MapTower11",0xc0e73613,"MapTower12",0xb7e00685,"MapTower13",0x29849326,"MapTower14",0x5e83a3b0,"MapTower15",0x9510e305,"MaronSpring",0x7cf7d4ad,"MarritaExchange",0xa89c3e9d,"MasazuBridge",0x33dedaf1,"MinakkareBridge",0x894e01b7,"MinakkareBridgeEast",0x7457367f,"MinakkareBridgeWest",0xfae08218,"Mine",0x8367f7f5,"NantanCastle",0x7939d480,"NishojiBridge",0x5c907154,"NorthHatelHatago",0x0fa7b37f,"Oasis",0x1a1649a5,"OrdinBridge",0xd865cfa4,"OrdinFossil",0xf1cab1a7,"OsanjoBridge",0x6360e32f,"PostTownRuin",0x1cc23004,"PowerFountain",0x18f92f82,"RemainsElectric",0x8ad5ef4a,"RemainsFire",0x66336893,"RemainsWater",0xf300b27b,"RemainsWind",0xd451777c,"RirikendoBridge",0xa262d614,"Rito",0x79431cd8,"RitoHatago",0xa313005e,"RiverSideHatago",0x8f067283,"RonronCity",0x80f9fd06,"SaihokuCastle",0x71a002e7,"SarjonBridge",0xe12f0afe,"ShichijoBridge",0x4e798832,"ShigonDam",0x57c1e588,"ShijoBridge",0x40060961,"ShinyarkiVillage",0xa31a50aa,"SnowStatue",0x8ae6edd7,"SojijiPark",0x46f6e384,"SotonkaBridge",0xd8e21026,"SouthGerudoLegacy",0xd8f19a2a,"SouthMine",0xc409b5e2,"StartPoint",0x0fe7e6ab,"StonePillers",0x94ddf95c,"SusukadiBridge",0x2fa52b7e,"TabantaBridge",0x62fffa64,"TabantaBridgeHatago",0x8c8e9401,"TabantaCrater",0xed1341fd,"TabantaHatago",0x99277060,"TabantaVillage",0x16b9c598,"TamourHatago",0xe4171cbe,"TamourHatago2",0x1a2cadf2,"TaserakaBridge",0x4695d38c,"Taura",0x095288b6,"TimesShrine",0x6c034b60,"ToriaBridge",0x8b62694a,"TotsugeBridge",0xd78f2eac,"TurasuBridge",0x26daeef2,"UMiiVillage",0x3187bba5,"UMiiVillageShopBougu",0x3b930e2c,"UMiiVillageShopJewel",0xd2180ce8,"UMiiVillageShopYadoya",0x29ad3d74,"UMiiVillageShopYorozu",0x1f4bc09c,"ValleyVillage",0xbfdd2802,"WeaponCureSpring",0x298b03fe,"WeaponCureSpring_02",0x5e8c3368,"WeaponCureSpring_03",0xc0e8a6cb,"WeaponCureSpring_04",0x3f3d559e,"WetGarrison",0xff6f7ae4,"WhiteZora",0xea6e852b,"WisdomFountain",0x2cd6f581,"XekuBridge",0x5c7246c7,"YashinoLegacy",0x4a46741a,"ZonauLegacy"]},
	{type:"s32",id:"MainField_PartsDrop_",hashes:[0x1bb8ea45,1009936452,0xe7eef679,1026285922,0xb949476b,1165739025,0x9b9f5085,1190835183,0xeef5808f,128179700,0x998d629b,129785380,0xe231092d,1329210538,0x94a07e95,1347313845,0xb9c39e29,1349980722,0x304ef42d,1410780922,0xc990e660,1429678381,0x6f51b340,144669291,0x465a24a8,1469234556,0x3e974293,1543401762,0x6bdd7263,1577003372,0xe4dfec71,1736524803,0x96691dee,1805862402,0xd7f315ea,1981635416,0x0b3a3f8f,1983585774,0x66c2caef,2116637744,0x68cd03a1,220762433,0x99e1287b,2249200231,0x73f2e84e,2293343495,0x7dfbf540,2325055349,0xab1e872d,2328048419,0x8e08adc8,2373028017,0x73032ffd,238847606,0x3793a813,2450204935,0xc662a942,2479341154,0x2c0b18da,2479542683,0xd47173d0,2485240059,0x93a4604f,2540234944,0xa0b41f0b,2541954314,0x95233e47,2557205703,0xe1493646,2643409954,0x41a9ef32,2700810178,0x7bc560f8,2772275816,0x07e2a82c,2802402972,0x6cdffb99,2900500761,0x2c77772b,2994951342,0xb19c8be1,3060564958,0x04b03bed,3132578758,0xc12bffc3,3140496781,0x077188fd,3231674089,0x772aa081,3251855747,0x2fdce6a8,3346995733,0x96b15ca2,3348421845,0xe0c9a0d8,3458921750,0x3dc76423,3534493673,0x685dec5f,3559217329,0xf6386daa,367948831,0x81eec3b9,3707116099,0x7b611c5f,3774681053,0xe13b3a60,3800503521,0xd6551021,3810979949,0x3ea1093d,3862084975,0x7e209a71,3916617587,0x180a6499,3980550772,0x0c02d42e,4006920142,0xcd70ff68,4060806309,0x5d70f378,4117946208,0xd48ab93d,4213135951,0x1c3bff7b,4259292662,0x85d1922c,510946050,0xb9314e56,557056845,0x13ee2ddd,568632014,0x9ef79501,626309166,0x99974649,645337006,0x20fbb90b,684303411,0x4964b5ee,743552386,0xc0bf53ee,744877403,0x531c048e,82492342,0x9e57b198,856671268,0xef9b8a22,887589490,0xf1cc6e34,888231907]},
	{type:"s32",id:"Npc_",hashes:[0x5fffb25a,"AncientAssistant003_Armor_014_Head",0x7c6510e2,"AncientAssistant003_Armor_014_Lower",0x1ca0b45d,"AncientAssistant003_Armor_014_Upper",0xf1222008,"AncientAssistant003_Armor_043_Lower",0x91e784b7,"AncientAssistant003_Armor_043_Upper",0x400f3cf6,"AncientAssistant003_Armor_046_Head",0xa31a0faf,"AncientAssistant003_Armor_046_Lower",0xc3dfab10,"AncientAssistant003_Armor_046_Upper",0x7a055d86,"AncientAssistant003_Armor_048_Head",0xf32574f2,"AncientAssistant003_Armor_048_Lower",0x93e0d04d,"AncientAssistant003_Armor_048_Upper",0x55527f46,"AncientAssistant003_Armor_049_Lower",0x71c62562,"AncientAssistant003_Armor_141_Lower",0x96d833d0,"AncientAssistant003_Weapon_Shield_030",0xb3a26b50,"AncientDoctor_AncientArrow",0x47cebe04,"AncientDoctor_Hateno_Armor_021_Head",0xf0ae5d1d,"AncientDoctor_Hateno_Armor_021_Lower",0x906bf9a2,"AncientDoctor_Hateno_Armor_021_Upper",0xc7b20f7c,"AncientDoctor_Hateno_Obj_AncientArrow_A_01",0xd507a092,"AncientDoctor_Hateno_Obj_AncientArrow_B_01",0x6dbbc7f7,"AncientDoctor_Hateno_Obj_AncientArrow_C_01",0x23596045,"AncientDoctor_Obj_AncientArrow_C_01",0x61833bcb,"AncientDoctor_Weapon_Lsword_023",0x35d4ebf6,"AncientDoctor_Weapon_Sword_023",0xb5f380df,"Assassin_005_Item_Fruit_H",0x45e1255f,"Attacked_001_BeeHome",0xcfe9a486,"Attacked_001_Item_Material_01",0x518d3125,"Attacked_001_Item_Material_06",0x5a9c737e,"Attacked_002_Item_Fruit_D",0xb4921252,"Attacked_002_Item_Fruit_F",0xc39522c4,"Attacked_002_Item_Fruit_G",0x532a3f55,"Attacked_002_Item_Fruit_H",0xd8fc28c2,"Attacked_003_Animal_Insect_A",0xa1209066,"Attacked_003_Animal_Insect_H",0x3829c1dc,"Attacked_003_Animal_Insect_K",0x370b028d,"Attacked_003_Item_Enemy_27",0x2eef3996,"Attacked_004_Obj_ArrowBundle_A_01",0x822b0713,"Attacked_004_Obj_FireArrow_A_03",0xf23680db,"Attacked_004_Obj_IceArrow_A_03",0xbd9dcb24,"Attacked_005_Item_Material_08",0x9433a197,"Attacked_006_Item_Enemy_28",0x932c58b8,"Attacked_006_Item_Ore_F",0x749475bf,"Attacked_006_Item_Ore_H",0x36fb5cd5,"Attacked_007_BeeHome",0x589c49b9,"Attacked_007_Item_Material_02",0x2f9b792f,"Attacked_007_Item_Material_03",0x28f6bd36,"Attacked_007_Item_Material_07",0xee8c3956,"Attacked_008_Item_Mushroom_A",0x9ee6cdd9,"Attacked_008_Item_Mushroom_D",0xe9e1fd4f,"Attacked_008_Item_Mushroom_E",0xfef82b9d,"Attacked_009_Item_Mushroom_J",0x609cbe3e,"Attacked_009_Item_Mushroom_M",0x8e92df12,"Attacked_009_Item_Mushroom_O",0xae65fe65,"Attacked_010_Item_Plant_G",0x3edae3f4,"Attacked_010_Item_Plant_H",0x49ddd362,"Attacked_010_Item_Plant_I",0xc76d89c2,"Attacked_012_Obj_ArrowBundle_A_01",0x7d8a8018,"Attacked_012_Obj_IceArrow_A_02",0x0a8db08e,"Attacked_012_Obj_IceArrow_A_03",0x648c1e54,"DemonStatue_ClearSeal",0xf0e7d122,"FaronWoods007_Item_Mushroom_A",0x6e834481,"FaronWoods007_Item_Mushroom_F",0x6058ccb3,"FaronWoods007_Item_Mushroom_N",0xf6418b9d,"FaronWoods007_Item_Ore_H",0xf23e9ccb,"FaronWoods007_Item_Plant_M",0x1c30fde7,"FaronWoods007_Item_Plant_O",0x4457b850,"FaronWoods007_Obj_ArrowBundle_A_01",0x8fe5c4d0,"FaronWoods009_Obj_ArrowBundle_A_01",0x16ec956a,"FaronWoods009_Obj_ArrowBundle_A_02",0xa610dd70,"FaronWoods009_Obj_ArrowNormal_A_01",0x572a26dd,"FaronWoods009_Obj_BombArrow_A_01",0xd694fc51,"FaronWoods009_Obj_ElectricArrow_A_03",0xbaba9597,"Goddess_UtuwaSum",0xf457202f,"Goron005_Item_Roast_03",0x599699e9,"HatenoVillage019_Armor_001_Head",0x2e0b9bff,"HatenoVillage019_Armor_001_Lower",0x4ece3f40,"HatenoVillage019_Armor_001_Upper",0xc380d1d0,"MamonoShop_Armor_022_Head",0xbd55d452,"MamonoShop_Armor_045_Head",0x1b22dfe6,"MamonoShop_Armor_055_Head",0x9db6ad48,"MamonoShop_Armor_056_Head",0xb6ad5456,"MamonoShop_Armor_160_Head",0x753a0e2f,"MamonoShop_Armor_160_Lower",0x15ffaa90,"MamonoShop_Armor_160_Upper",0x9f9b7d70,"MamonoShop_GameRomHorseReins_04",0xaa90f8e9,"MamonoShop_GameRomHorseSaddle_04",0x5d6aa814,"MamonoShop_Item_Material_08",0x279db8ac,"MamonoShop_Weapon_Spear_036",0x45e8cf1e,"MamonoShop_Weapon_Sword_040",0xa785563c,"Road_001_Obj_ArrowBundle_A_01",0x3e8c0786,"Road_001_Obj_ArrowBundle_A_02",0x8e704f9c,"Road_001_Obj_ArrowNormal_A_01",0x4ebbf649,"Road_001_Obj_FireArrow_A_01",0xa0b59765,"Road_001_Obj_FireArrow_A_03",0x6eb04f6d,"Road_001_Obj_IceArrow_A_03",0xd0c28604,"Road_002_Item_Meat_01",0x49cbd7be,"Road_002_Item_Meat_02",0x4ea613a7,"Road_002_Item_Meat_06",0x39a12331,"Road_002_Item_Meat_07",0xc9d9b745,"Road_002_Item_Meat_11",0xd7c52848,"Road_002_Item_Ore_H",0x304a4fcd,"Road_004_Item_Material_04",0x474d7f5b,"Road_004_Item_Material_05",0xde442ee1,"Road_004_Item_Material_06",0xa41d4d14,"Road_005_Item_Mushroom_A",0x3d141cae,"Road_005_Item_Mushroom_B",0x4a132c38,"Road_005_Item_Mushroom_C",0x3a79d8b7,"Road_005_Item_Mushroom_F",0xddc1f5b0,"Road_005_Item_Mushroom_H",0xdaac31a9,"Road_005_Item_Mushroom_L",0xadab013f,"Road_005_Item_Mushroom_M",0x34a25085,"Road_005_Item_Mushroom_N",0x43a56013,"Road_005_Item_Mushroom_O",0x36bae7fc,"Road_007_Item_Ore_A",0xafb3b646,"Road_007_Item_Ore_B",0xd8b486d0,"Road_007_Item_Ore_C",0x46d01373,"Road_007_Item_Ore_D",0x31d723e5,"Road_007_Item_Ore_E",0xa8de725f,"Road_007_Item_Ore_F",0xdfd942c9,"Road_007_Item_Ore_G",0x4f665f58,"Road_007_Item_Ore_H",0x38616fce,"Road_007_Item_Ore_I",0x9c6aec1c,"Road_010_Item_Material_01",0x0563bda6,"Road_010_Item_Material_02",0x72648d30,"Road_010_Item_Material_03",0xec001893,"Road_010_Item_Material_04",0x9b072805,"Road_010_Item_Material_05",0x020e79bf,"Road_010_Item_Material_06",0x75094929,"Road_010_Item_Material_07",0x72101f49,"Road_015_Item_FishGet_F",0x05172fdf,"Road_015_Item_FishGet_G",0x0ca163f4,"Road_015_Item_FishGet_K",0xd121742f,"Road_015_Item_Fruit_D",0x48282595,"Road_015_Item_Fruit_G",0x1c56bd95,"Road_015_Item_InsectGet_Z",0x42af5b7f,"Road_019_Animal_Fish_L",0x083ffacb,"Road_019_Animal_Insect_AA",0xc4a4c790,"Road_019_Animal_Insect_B",0xd7c85fc6,"Road_019_Animal_Insect_Z",0xde5793a4,"Road_019_Item_Mushroom_A",0x39efbea3,"Road_019_Item_Mushroom_O",0xa36cc20d,"Road_019_Item_Plant_Q",0xff5f3fef,"Road_020_Item_Enemy_26",0x88580f79,"Road_020_Item_Enemy_27",0x18e712e8,"Road_020_Item_Enemy_28",0x6fe0227e,"Road_020_Item_Enemy_29",0x0f27ab9b,"Road_020_Item_Enemy_30",0xbaebd227,"Road_024_Item_Fruit_J",0x80867c23,"Road_024_Item_Meat_01",0x198f2d99,"Road_024_Item_Meat_02",0xb80b7877,"Road_024_Item_Mushroom_F",0xb6d0f045,"Road_024_Item_Mushroom_N",0x11ea8460,"Road_024_Item_Plant_M",0x8488e4d7,"Road_030_Animal_Insect_AA",0x4013c1d0,"Road_030_Animal_Insect_H",0x8d6e5f5c,"Road_030_Item_Fruit_A",0x6dbbb642,"Road_030_Item_Fruit_K",0xefbd1261,"Road_030_Obj_ArrowNormal_A_01",0x5067d31b,"Road_031_Animal_Insect_C",0xa3dea253,"Road_031_Animal_Insect_Q",0xd3b456dc,"Road_031_Animal_Insect_T",0x21602e7f,"Road_044_Item_FishGet_J",0x08ae1b44,"Road_044_Item_Fruit_E",0x3041e71a,"Road_044_Item_Material_02",0x3b75f96b,"Road_044_Item_Meat_02",0xbb679990,"Road_044_Item_Meat_11",0x56a22dec,"Road_044_Item_Mushroom_L",0x21a51d7a,"Road_044_Item_Mushroom_M",0xc65bd953,"SmallOasis003_Obj_ArrowBundle_A_01",0x5d52cb06,"SmallOasis008_Item_Ore_F",0xbd15662a,"SmallOasis009_Item_Fruit_A",0xaab84282,"SmallOasis010_Item_Fruit_A",0x263b6145,"SouthernVillage005_Obj_ArrowBundle_A_01",0xe59c5aa9,"Test_Tamura_011_Armor_001_Upper",0x9f8a42d0,"Test_Tamura_011_Obj_ApplePie_A_01",0xaabf6fdc,"Test_Tamura_011_Obj_ArrowBundle_A_01",0x3b25ec67,"Test_Tamura_011_Obj_ElectricArrow_A_03",0x9bde3811,"Test_Tamura_011_Obj_FireArrow_A_03",0x6b3ff34d,"Test_Tamura_011_Obj_IceArrow_A_03",0x65e929ee,"Test_Tamura_011_Weapon_Sword_029",0x4ca526e9,"TripMaster_00_Animal_Insect_A",0x35799e4d,"TripMaster_00_Animal_Insect_H",0x9570ca2f,"TripMaster_00_Item_Enemy_57",0x6bb834d1,"TripMaster_00_Obj_ArrowBundle_A_01",0x424d2d71,"TripMaster_00_Obj_ArrowNormal_A_01",0x520abc10,"TripMaster_01_Animal_Insect_B",0xc2b5a181,"TripMaster_01_Animal_Insect_M",0x7c905011,"TripMaster_01_Obj_ArrowBundle_A_01",0x556549b1,"TripMaster_01_Obj_ArrowNormal_A_01",0x912ffa05,"TripMaster_02_Animal_Insect_M",0x0826abbf,"TripMaster_02_Animal_Insect_N",0xf54452c5,"TripMaster_02_Animal_Insect_T",0xdce1aceb,"TripMaster_02_Obj_ArrowBundle_A_02",0x6c1de4f1,"TripMaster_02_Obj_ArrowNormal_A_01",0x79cceda9,"TripMaster_03_Animal_Insect_AB",0xe5299482,"TripMaster_03_Item_Plant_B",0x75968913,"TripMaster_03_Item_Plant_M",0xcbc9c82b,"TripMaster_03_Obj_ArrowBundle_A_02",0x7b358031,"TripMaster_03_Obj_ArrowNormal_A_01",0xaf121cb7,"TripMaster_04_Animal_Insect_N",0x221a1142,"TripMaster_04_Animal_Insect_Q",0xbb1340f8,"TripMaster_04_Animal_Insect_R",0xae10f66b,"TripMaster_04_Obj_ArrowBundle_A_02",0x1eecbe71,"TripMaster_04_Obj_ArrowNormal_A_01",0x21029bdf,"TripMaster_05_Animal_Insect_B",0xbf660e7c,"TripMaster_05_Animal_Insect_E",0xd5d62e8e,"TripMaster_05_Animal_Insect_T",0xb93892ab,"TripMaster_05_Obj_ArrowBundle_A_02",0x09c4dab1,"TripMaster_05_Obj_ArrowNormal_A_01",0xe227ddca,"TripMaster_06_Animal_Insect_M",0x1828e0a9,"TripMaster_06_Animal_Insect_S",0x8ffa3921,"TripMaster_06_Animal_Insect_X",0x80403feb,"TripMaster_06_Obj_ArrowBundle_A_02",0x30bc77f1,"TripMaster_06_Obj_ArrowNormal_A_01",0xf53e0b18,"TripMaster_07_Animal_Insect_B",0x6b5a9ebb,"TripMaster_07_Animal_Insect_E",0x01eabe49,"TripMaster_07_Animal_Insect_T",0x97685b2b,"TripMaster_07_Obj_ArrowBundle_A_02",0x27941331,"TripMaster_07_Obj_ArrowNormal_A_01",0x33bc38cd,"TripMaster_08_Animal_Insect_B",0xa46ee145,"TripMaster_08_Animal_Insect_I",0x2e0b28a9,"TripMaster_08_Animal_Insect_R",0xd2fb12d1,"TripMaster_08_Obj_ArrowBundle_A_01",0xfb0e0b71,"TripMaster_08_Obj_ArrowNormal_A_01",0xb41af38e,"TripMaster_09_Animal_Insect_B",0xa9ade3ea,"TripMaster_09_Animal_Insect_R",0xdeaad37c,"TripMaster_09_Animal_Insect_S",0xc5d37611,"TripMaster_09_Obj_ArrowBundle_A_01",0xec266fb1,"TripMaster_09_Obj_ArrowNormal_A_01",0xa3f79008,"TripMaster_10_Animal_Insect_A",0xda2b28ac,"TripMaster_10_Animal_Insect_H",0xaa41dc23,"TripMaster_10_Animal_Insect_M",0xf0cbde05,"TripMaster_10_Obj_ArrowBundle_A_01",0xd93ec7a5,"TripMaster_10_Obj_ArrowNormal_A_01",0xca5f3a67,"TripMaster_11_Animal_Insect_C",0x233c9f52,"TripMaster_11_Animal_Insect_E",0x2de71760,"TripMaster_11_Animal_Insect_M",0xe7e3bac5,"TripMaster_11_Obj_ArrowBundle_A_01",0xce16a365,"TripMaster_11_Obj_ArrowNormal_A_01",0x77cb00cf,"TripMaster_12_Animal_Insect_A",0x417cdb10,"TripMaster_12_Animal_Insect_AB",0xeec25175,"TripMaster_12_Animal_Insect_B",0xde9b1785,"TripMaster_12_Obj_ArrowBundle_A_01",0xf76e0e25,"TripMaster_12_Obj_ArrowNormal_A_01",0x69649a36,"TripMaster_13_Animal_Insect_B",0xeddadbe8,"TripMaster_13_Animal_Insect_Q",0x9db02f67,"TripMaster_13_Animal_Insect_T",0x50ba22ff,"TripMaster_13_Obj_ArrowBundle_A_02",0xe0466ae5,"TripMaster_13_Obj_ArrowNormal_A_01",0xd79273de,"TripMaster_14_Animal_Insect_E",0xd949fbec,"TripMaster_14_Animal_Insect_M",0xb1643edd,"TripMaster_14_Item_Enemy_57",0x35631cbf,"TripMaster_14_Obj_ArrowBundle_A_02",0x859f54a5,"TripMaster_14_Obj_ArrowNormal_A_01",0xce502d3e,"TripMaster_15_Animal_Insect_B",0xb9571da8,"TripMaster_15_Animal_Insect_C",0x4aee6ce0,"TripMaster_15_Animal_Insect_Q",0x224b787f,"TripMaster_15_Obj_ArrowBundle_A_02",0x92b73065,"TripMaster_15_Obj_ArrowNormal_A_01",0x19901826,"UMiiVillage009_Obj_ArrowBundle_A_01",0x66b48aa9,"UMiiVillage009_Obj_BombArrow_A_02",0x1b17b2e0,"UMiiVillage009_Obj_ElectricArrow_A_03",0xff4d8df7,"UMiiVillage009_Obj_FireArrow_A_03",0x9322d554,"UMiiVillage009_Obj_IceArrow_A_03",0x20beed99,"UMiiVillage010_Item_Roast_03",0x2b7fe2de,"ValleyVillage010_Obj_BombArrow_A_01",0xb276b364,"ValleyVillage010_Obj_BombArrow_A_02",0x2a211998,"Zora002_Obj_ArrowBundle_A_01",0xb0e3ff86,"Zora002_Obj_BombArrow_A_03",0x65f32263,"Zora002_Obj_IceArrow_A_02",0x1778f80a,"Zora009_Item_Roast_03",0x6d23dd5b,"goron004_Armor_011_Head",0x0b50fc1b,"goron004_Armor_011_Lower",0x6b9558a4,"goron004_Armor_011_Upper",0x73f3e791,"kokiri003_Item_Fruit_A",0xe96d100d,"kokiri004_Armor_005_Head",0x89dc4617,"kokiri004_Armor_005_Lower",0xe919e2a8,"kokiri004_Armor_005_Upper",0x07548d01,"kokiri004_Obj_ArrowBundle_A_01",0x04507315,"oasis001_Armor_027_Head",0xf506c1c0,"oasis001_Armor_028_Head",0x3e5a1265,"oasis001_Armor_029_Head",0x2caa167e,"oasis002_Item_Ore_F",0x5a5301f3,"oasis005_Item_Fruit_A",0xe3a8da1b,"oasis015_Item_Fruit_A",0x9f6df4dc,"oasis022_Armor_008_Head",0x787ea4b1,"oasis022_Armor_008_Lower",0x18bb000e,"oasis022_Armor_008_Upper",0x28ed2f26,"oasis043_Item_Fruit_A",0x4d2085e6,"oasis044_Item_Fruit_A",0xd02f6490,"oasis045_Item_Fruit_A",0xac4e414b,"oasis046_Item_Fruit_A",0x351e21a9,"oasis050_2_Armor_014_Head",0x58bb52d7,"oasis050_2_Armor_014_Lower",0x387ef668,"oasis050_2_Armor_014_Upper",0x2aeeaf05,"oasis050_2_Armor_046_Head",0x87c44d9a,"oasis050_2_Armor_046_Lower",0xe701e925,"oasis050_2_Armor_046_Upper",0x10e4ce75,"oasis050_2_Armor_048_Head",0xd7fb36c7,"oasis050_2_Armor_048_Lower",0xb73e9278,"oasis050_2_Armor_048_Upper",0x718c3d73,"oasis050_2_Armor_049_Lower",0x55186757,"oasis050_2_Armor_141_Lower",0xe9ea663b,"oasis050_Armor_014_Head",0x466654f8,"oasis050_Armor_014_Lower",0x26a3f047,"oasis050_Armor_014_Upper",0xf61ae897,"oasis050_Armor_046_Head",0x99194bb5,"oasis050_Armor_046_Lower",0xf9dcef0a,"oasis050_Armor_046_Upper",0xcc1089e7,"oasis050_Armor_048_Head",0xc92630e8,"oasis050_Armor_048_Lower",0xa9e39457,"oasis050_Armor_048_Upper",0x6f513b5c,"oasis050_Armor_049_Lower",0x4bc56178,"oasis050_Armor_141_Lower"]},
	{type:"s32",id:"PictureBookSize_",hashes:[0x5c7400bc,"AncientArrow",0x773e8901,"Animal_Bear_A",0xee37d8bb,"Animal_Bear_B",0x3d8e91a0,"Animal_Boar_A",0xa487c01a,"Animal_Boar_B",0xf30c4824,"Animal_Bull_A",0xdb9d012e,"Animal_Cassowary_A",0xa68096fc,"Animal_Cow_A",0x88b6a4ee,"Animal_Crow_A",0x2e05fd4b,"Animal_Deer_A",0x0b023df2,"Animal_Doe_A",0x0886e99c,"Animal_Dog_A",0x2e4996a6,"Animal_Elk_A",0x653c5eb1,"Animal_Fox_A",0xfc350f0b,"Animal_Fox_B",0x69ce7ca1,"Animal_Goat_A",0xa3c1d997,"Animal_Gull_A",0x45a09c1b,"Animal_Hawk_A",0x74021db2,"Animal_Heron_A",0xed0b4c08,"Animal_Heron_B",0xabb09f55,"Animal_Insect_A",0xc8738b6f,"Animal_Insect_AA",0x517adad5,"Animal_Insect_AB",0x32b9ceef,"Animal_Insect_B",0x45befe79,"Animal_Insect_C",0xacdd5b4c,"Animal_Insect_E",0x35d40af6,"Animal_Insect_F",0x42d33a60,"Animal_Insect_G",0xd26c27f1,"Animal_Insect_H",0xa56b1767,"Animal_Insect_I",0xa206d37e,"Animal_Insect_M",0x3b0f82c4,"Animal_Insect_N",0xc100bfa7,"Animal_Insect_P",0xb6078f31,"Animal_Insect_Q",0x2f0ede8b,"Animal_Insect_R",0x5809ee1d,"Animal_Insect_S",0xc66d7bbe,"Animal_Insect_T",0xcfdb3795,"Animal_Insect_X",0x07212cda,"Animal_Kokko_A",0x8286ef61,"Animal_LittleBird_A",0x1b8fbedb,"Animal_LittleBird_B",0x6c888e4d,"Animal_LittleBird_C",0xf2ec1bee,"Animal_LittleBird_D",0x85eb2b78,"Animal_LittleBird_E",0x1ce27ac2,"Animal_LittleBird_F",0xcec35248,"Animal_Pigeon_A",0x57ca03f2,"Animal_Pigeon_B",0x195449c7,"Animal_Ptarmigan_A",0x805d187d,"Animal_Ptarmigan_B",0xa57c662d,"Animal_Rhino_A",0x13423c95,"Animal_RupeeRabbit_A",0x0680e3d4,"Animal_Sheep_A",0x70aeb88c,"Animal_Squirrel_A",0x24d5095b,"Animal_SunazarashiSP_C",0x62e46e10,"Animal_Sunazarashi_A",0x2c59c2a6,"Animal_WildDuck_A",0x3dee13ed,"Animal_WildGoat_A",0x87d0d1f9,"Animal_Wolf_A",0x1ed98043,"Animal_Wolf_B",0x69deb0d5,"Animal_Wolf_C",0xb3281fb0,"BeeHome",0x51f46518,"BombArrow_A",0xa15b6f7f,"ElectricArrow",0x735524a5,"Enemy_Assassin_Junior",0x04dbcbad,"Enemy_Assassin_Middle",0x376ce074,"Enemy_Assassin_Senior",0x2f883be5,"Enemy_Bokoblin_Bone_Junior",0xd7639b4c,"Enemy_Bokoblin_Dark",0x9087845a,"Enemy_Bokoblin_Junior",0xe7096b52,"Enemy_Bokoblin_Middle",0xd4be408b,"Enemy_Bokoblin_Senior",0xacf11d4a,"Enemy_Chuchu_Electric_Junior",0xf04391bf,"Enemy_Chuchu_Fire_Junior",0x9a4129c2,"Enemy_Chuchu_Ice_Junior",0xe86f3e48,"Enemy_Chuchu_Junior",0xfadc4035,"Enemy_Dragon_Electric",0xf20698c8,"Enemy_Dragon_Fire",0x1992dc60,"Enemy_Dragon_Ice",0x3b35855c,"Enemy_Ganon",0xcbf06982,"Enemy_GanonBeast",0xc95786b1,"Enemy_GanonGrudge",0x4a73e922,"Enemy_GanonGrudge_01",0xd37ab898,"Enemy_GanonGrudge_02",0x97e5562f,"Enemy_Giant_Bone",0x42badae4,"Enemy_Giant_Junior",0x353435ec,"Enemy_Giant_Middle",0x06831e35,"Enemy_Giant_Senior",0x34bc8b65,"Enemy_Golem_Fire",0xaf58c35a,"Enemy_Golem_Ice",0xbe502fca,"Enemy_Golem_Junior",0x4ebde788,"Enemy_Golem_Little",0xcc8b5bf0,"Enemy_Golem_Little_Fire",0xfa263f87,"Enemy_Golem_Little_Ice",0xc9dec0c2,"Enemy_Golem_Middle",0xfa69eb1b,"Enemy_Golem_Senior",0x19c3d7c9,"Enemy_Guardian_A",0x0ec4b121,"Enemy_Guardian_A_Fixed_Moss",0x80ca8673,"Enemy_Guardian_B",0xf7cdb6e5,"Enemy_Guardian_C",0xf598afff,"Enemy_Guardian_Mini_Baby",0xe7396cf3,"Enemy_Guardian_Mini_Junior",0x90b783fb,"Enemy_Guardian_Mini_Middle",0xa300a822,"Enemy_Guardian_Mini_Senior",0x0c57943e,"Enemy_Keese",0xef073e95,"Enemy_Keese_Electric",0x4cbcd1c7,"Enemy_Keese_Fire",0xa8cbc312,"Enemy_Keese_Ice",0x671d6f51,"Enemy_Lizalfos_Bone_Junior",0x3693fa81,"Enemy_Lizalfos_Dark",0x0956c741,"Enemy_Lizalfos_Electric",0x75314eb5,"Enemy_Lizalfos_Fire",0x26e773f1,"Enemy_Lizalfos_Ice",0xd4669d67,"Enemy_Lizalfos_Junior",0xa3e8726f,"Enemy_Lizalfos_Middle",0x905f59b6,"Enemy_Lizalfos_Senior",0xd95d2b85,"Enemy_Lynel_Dark",0x41de55ef,"Enemy_Lynel_Junior",0x3650bae7,"Enemy_Lynel_Middle",0x05e7913e,"Enemy_Lynel_Senior",0x2283b839,"Enemy_Moriblin_Bone_Junior",0x28b6c470,"Enemy_Moriblin_Dark",0x18a04e15,"Enemy_Moriblin_Junior",0x6f2ea11d,"Enemy_Moriblin_Middle",0x5c998ac4,"Enemy_Moriblin_Senior",0x1e53d75e,"Enemy_Octarock",0x43b97545,"Enemy_Octarock_Desert",0xe5669acb,"Enemy_Octarock_Forest",0x4d8f252a,"Enemy_Octarock_Snow",0x9088224a,"Enemy_Octarock_Stone",0x45113a89,"Enemy_Sandworm",0x45674631,"Enemy_SiteBoss_Bow",0x245a6353,"Enemy_SiteBoss_Lsword",0x908e709c,"Enemy_SiteBoss_Spear",0x95f53930,"Enemy_SiteBoss_Sword",0xb06b61f4,"Enemy_Wizzrobe_Electric",0xf3a11950,"Enemy_Wizzrobe_Electric_Senior",0x36a42f5d,"Enemy_Wizzrobe_Fire",0xb7b7721c,"Enemy_Wizzrobe_Fire_Senior",0xda6ff199,"Enemy_Wizzrobe_Ice",0xa5bd9391,"Enemy_Wizzrobe_Ice_Senior",0x78fa02df,"FireArrow",0x6f475198,"GameRomHorse",0xc31e56cb,"GameRomHorse00L",0x4e165b3e,"GameRomHorse00S",0xe2cb388e,"GameRomHorseBone",0xb78a47fa,"GameRomHorseNushi",0xc7e70254,"GameRomHorseZelda",0xabfd5819,"IceArrow",0xef797df7,"Item_FishGet_A",0x76702c4d,"Item_FishGet_B",0x01771cdb,"Item_FishGet_C",0x9f138978,"Item_FishGet_D",0xe814b9ee,"Item_FishGet_E",0x711de854,"Item_FishGet_F",0x061ad8c2,"Item_FishGet_G",0x96a5c553,"Item_FishGet_H",0xe1a2f5c5,"Item_FishGet_I",0x78aba47f,"Item_FishGet_J",0x0fac94e9,"Item_FishGet_K",0x91c8014a,"Item_FishGet_L",0xe6cf31dc,"Item_FishGet_M",0x8b12d537,"Item_FishGet_X",0x651cb41b,"Item_FishGet_Z",0xbf1d0115,"Item_Fruit_A",0x261450af,"Item_Fruit_B",0x51136039,"Item_Fruit_C",0xcf77f59a,"Item_Fruit_D",0xb870c50c,"Item_Fruit_E",0x217994b6,"Item_Fruit_F",0x567ea420,"Item_Fruit_G",0xc6c1b9b1,"Item_Fruit_H",0xb1c68927,"Item_Fruit_I",0x28cfd89d,"Item_Fruit_J",0xaa5395c9,"Item_InsectGet_K",0xad3e51d0,"Item_InsectGet_O",0xc0e3b53b,"Item_InsectGet_Z",0x03902941,"Item_MushroomGet_D",0x284a9adb,"Item_Mushroom_A",0xb143cb61,"Item_Mushroom_B",0xc644fbf7,"Item_Mushroom_C",0x2f275ec2,"Item_Mushroom_E",0xb62e0f78,"Item_Mushroom_F",0x5196227f,"Item_Mushroom_H",0xbf984353,"Item_Mushroom_J",0x56fbe666,"Item_Mushroom_L",0x21fcd6f0,"Item_Mushroom_M",0xb8f5874a,"Item_Mushroom_N",0xcff2b7dc,"Item_Mushroom_O",0xfc959886,"Item_PlantGet_A",0x659cc93c,"Item_PlantGet_B",0x129bf9aa,"Item_PlantGet_C",0xfbf85c9f,"Item_PlantGet_E",0x62f10d25,"Item_PlantGet_F",0x15f63db3,"Item_PlantGet_G",0x85492022,"Item_PlantGet_H",0xf24e10b4,"Item_PlantGet_I",0x6b47410e,"Item_PlantGet_J",0x8224e43b,"Item_PlantGet_L",0xf523d4ad,"Item_PlantGet_M",0x1b2db581,"Item_PlantGet_O",0xe12288e2,"Item_PlantGet_Q",0x55131e31,"NormalArrow",0x008797c1,"Obj_Mineral_A_01",0x1232382f,"Obj_Mineral_B_01",0xaa8e5f4a,"Obj_Mineral_C_01",0x531a45c0,"RemainsFire_Drone_A_01",0x03439478,"TBox_Field_Iron",0xb7883cdb,"Weapon_Bow_001",0x2e816d61,"Weapon_Bow_002",0x59865df7,"Weapon_Bow_003",0xc7e2c854,"Weapon_Bow_004",0x29eca978,"Weapon_Bow_006",0xb953b4e9,"Weapon_Bow_009",0xae930d9a,"Weapon_Bow_011",0x409d6cb6,"Weapon_Bow_013",0xdef9f915,"Weapon_Bow_014",0xa9fec983,"Weapon_Bow_015",0x30f79839,"Weapon_Bow_016",0x47f0a8af,"Weapon_Bow_017",0x6bb03f75,"Weapon_Bow_023",0x1bdacbfa,"Weapon_Bow_026",0x6cddfb6c,"Weapon_Bow_027",0xfc62e6fd,"Weapon_Bow_028",0x8b65d66b,"Weapon_Bow_029",0xeba25f8e,"Weapon_Bow_030",0x05ac3ea2,"Weapon_Bow_032",0x72ab0e34,"Weapon_Bow_033",0x9bc8ab01,"Weapon_Bow_035",0x02c1fabb,"Weapon_Bow_036",0xe579d7bc,"Weapon_Bow_038",0xa4e3c949,"Weapon_Bow_040",0xf8c9aa1c,"Weapon_Bow_071",0xec5ac418,"Weapon_Lsword_001",0x755395a2,"Weapon_Lsword_002",0x0254a534,"Weapon_Lsword_003",0x9c303097,"Weapon_Lsword_004",0xeb370001,"Weapon_Lsword_005",0x723e51bb,"Weapon_Lsword_006",0x8246c5cf,"Weapon_Lsword_010",0xf541f559,"Weapon_Lsword_011",0x6c48a4e3,"Weapon_Lsword_012",0x1b4f9475,"Weapon_Lsword_013",0x852b01d6,"Weapon_Lsword_014",0xf22c3140,"Weapon_Lsword_015",0x6b2560fa,"Weapon_Lsword_016",0x1c22506c,"Weapon_Lsword_017",0x8c9d4dfd,"Weapon_Lsword_018",0xfb9a7d6b,"Weapon_Lsword_019",0xa96b960c,"Weapon_Lsword_020",0x3062c7b6,"Weapon_Lsword_023",0xae065215,"Weapon_Lsword_024",0x370f03af,"Weapon_Lsword_027",0xd0b72ea8,"Weapon_Lsword_029",0xb070a74d,"Weapon_Lsword_030",0xc77797db,"Weapon_Lsword_031",0x5e7ec661,"Weapon_Lsword_032",0x2979f6f7,"Weapon_Lsword_033",0xb71d6354,"Weapon_Lsword_034",0xc01a53c2,"Weapon_Lsword_035",0x59130278,"Weapon_Lsword_036",0x2e1432ee,"Weapon_Lsword_037",0xbeab2f7f,"Weapon_Lsword_038",0x8836011c,"Weapon_Lsword_041",0x8f5bc505,"Weapon_Lsword_045",0x6155a429,"Weapon_Lsword_047",0x912d305d,"Weapon_Lsword_051",0xe147c4d2,"Weapon_Lsword_054",0x9640f444,"Weapon_Lsword_055",0x0f49a5fe,"Weapon_Lsword_056",0xd371a650,"Weapon_Lsword_074",0xe0f7dd6d,"Weapon_Shield_001",0x79fe8cd7,"Weapon_Shield_002",0x0ef9bc41,"Weapon_Shield_003",0x909d29e2,"Weapon_Shield_004",0xe79a1974,"Weapon_Shield_005",0x7e9348ce,"Weapon_Shield_006",0x09947858,"Weapon_Shield_007",0x992b65c9,"Weapon_Shield_008",0xee2c555f,"Weapon_Shield_009",0x17e28d00,"Weapon_Shield_013",0x898618a3,"Weapon_Shield_014",0xfe812835,"Weapon_Shield_015",0x6788798f,"Weapon_Shield_016",0x108f4919,"Weapon_Shield_017",0x80305488,"Weapon_Shield_018",0xd2c1bfef,"Weapon_Shield_021",0x4bc8ee55,"Weapon_Shield_022",0x3ccfdec3,"Weapon_Shield_023",0xd5ac7bf6,"Weapon_Shield_025",0x4ca52a4c,"Weapon_Shield_026",0xbcddbe38,"Weapon_Shield_030",0xcbda8eae,"Weapon_Shield_031",0x52d3df14,"Weapon_Shield_032",0x25d4ef82,"Weapon_Shield_033",0xbbb07a21,"Weapon_Shield_034",0xccb74ab7,"Weapon_Shield_035",0x55be1b0d,"Weapon_Shield_036",0x22b92b9b,"Weapon_Shield_037",0xb206360a,"Weapon_Shield_038",0xf39c28ff,"Weapon_Shield_040",0x849b1869,"Weapon_Shield_041",0x1d9249d3,"Weapon_Shield_042",0x1247eb29,"Weapon_Spear_001",0x8b4eba93,"Weapon_Spear_002",0xfc498a05,"Weapon_Spear_003",0x622d1fa6,"Weapon_Spear_004",0x152a2f30,"Weapon_Spear_005",0x8c237e8a,"Weapon_Spear_006",0xfb244e1c,"Weapon_Spear_007",0x6b9b538d,"Weapon_Spear_008",0x1c9c631b,"Weapon_Spear_009",0x7c5beafe,"Weapon_Spear_010",0x0b5cda68,"Weapon_Spear_011",0x92558bd2,"Weapon_Spear_012",0xe552bb44,"Weapon_Spear_013",0x7b362ee7,"Weapon_Spear_014",0x0c311e71,"Weapon_Spear_015",0x95384fcb,"Weapon_Spear_016",0xe23f7f5d,"Weapon_Spear_017",0x728062cc,"Weapon_Spear_018",0x207189ab,"Weapon_Spear_021",0xb978d811,"Weapon_Spear_022",0xce7fe887,"Weapon_Spear_023",0x501b7d24,"Weapon_Spear_024",0x271c4db2,"Weapon_Spear_025",0xc9122c9e,"Weapon_Spear_027",0x59ad310f,"Weapon_Spear_028",0x2eaa0199,"Weapon_Spear_029",0x4e6d887c,"Weapon_Spear_030",0x396ab8ea,"Weapon_Spear_031",0xa063e950,"Weapon_Spear_032",0xd764d9c6,"Weapon_Spear_033",0x49004c65,"Weapon_Spear_034",0x3e077cf3,"Weapon_Spear_035",0xa70e2d49,"Weapon_Spear_036",0xd0091ddf,"Weapon_Spear_037",0x40b6004e,"Weapon_Spear_038",0x9f488b18,"Weapon_Spear_047",0x78f0a61f,"Weapon_Spear_049",0x18372ffa,"Weapon_Spear_050",0xd610af69,"Weapon_Sword_001",0x4f19fed3,"Weapon_Sword_002",0x381ece45,"Weapon_Sword_003",0xa67a5be6,"Weapon_Sword_004",0xd17d6b70,"Weapon_Sword_005",0x48743aca,"Weapon_Sword_006",0x3f730a5c,"Weapon_Sword_007",0xafcc17cd,"Weapon_Sword_008",0xd8cb275b,"Weapon_Sword_009",0x2105ff04,"Weapon_Sword_013",0xbf616aa7,"Weapon_Sword_014",0xc8665a31,"Weapon_Sword_015",0x516f0b8b,"Weapon_Sword_016",0x26683b1d,"Weapon_Sword_017",0xb6d7268c,"Weapon_Sword_018",0xc1d0161a,"Weapon_Sword_019",0x9321fd7d,"Weapon_Sword_020",0xe426cdeb,"Weapon_Sword_021",0x7d2f9c51,"Weapon_Sword_022",0x0a28acc7,"Weapon_Sword_023",0x944c3964,"Weapon_Sword_024",0xe34b09f2,"Weapon_Sword_025",0x0d4568de,"Weapon_Sword_027",0xeafd45d9,"Weapon_Sword_029",0x8a3acc3c,"Weapon_Sword_030",0xfd3dfcaa,"Weapon_Sword_031",0x13339d86,"Weapon_Sword_033",0x8d570825,"Weapon_Sword_034",0xfa5038b3,"Weapon_Sword_035",0xc57b5afb,"Weapon_Sword_040",0xb27c6a6d,"Weapon_Sword_041",0x5c720b41,"Weapon_Sword_043",0xc2169ee2,"Weapon_Sword_044",0x5b1fcf58,"Weapon_Sword_047",0xcba0d2c9,"Weapon_Sword_048",0xbca7e25f,"Weapon_Sword_049",0xdc606bba,"Weapon_Sword_050",0xab675b2c,"Weapon_Sword_051",0x326e0a96,"Weapon_Sword_052",0x45693a00,"Weapon_Sword_053",0xf74d3879,"Weapon_Sword_060",0x804a08ef,"Weapon_Sword_061",0x19435955,"Weapon_Sword_062",0xee560938,"Weapon_Sword_070",0x775f5882,"Weapon_Sword_073"]},
	{type:"s32",id:"PorchTime_",hashes:[0xe6a901a6,"Armor_001_Head",0x6b72be88,"Armor_001_Head_B",0xc8d7f866,"Armor_001_Lower",0xa8125cd9,"Armor_001_Upper",0x603d7308,"Armor_002_Head",0xe5fdb96b,"Armor_002_Head_B",0xf93fe2fb,"Armor_002_Lower",0x99fa4644,"Armor_002_Upper",0xab61a0ad,"Armor_003_Head",0x2957b9f5,"Armor_003_Head_B",0x5f48e94f,"Armor_003_Lower",0x3f8d4df0,"Armor_003_Upper",0xb6649015,"Armor_004_Head",0x2392b0ec,"Armor_004_Head_B",0x9aefd7c1,"Armor_004_Lower",0xfa2a737e,"Armor_004_Upper",0x7d3843b0,"Armor_005_Head",0x3c98dc75,"Armor_005_Lower",0x5c5d78ca,"Armor_005_Upper",0xfbac311e,"Armor_006_Head",0x0d70c6e8,"Armor_006_Lower",0x6db56257,"Armor_006_Upper",0x30f0e2bb,"Armor_007_Head",0xab07cd5c,"Armor_007_Lower",0xcbc269e3,"Armor_007_Upper",0xc1a6506e,"Armor_008_Head",0x5d4fbdb5,"Armor_008_Lower",0x3d8a190a,"Armor_008_Upper",0x0afa83cb,"Armor_009_Head",0xfb38b601,"Armor_009_Lower",0x9bfd12be,"Armor_009_Upper",0x40de0a12,"Armor_011_Head",0x047df8f8,"Armor_011_Lower",0x64b85c47,"Armor_011_Upper",0xc64a78bc,"Armor_012_Head",0xf286ad28,"Armor_012_Head_B",0x3595e265,"Armor_012_Lower",0x555046da,"Armor_012_Upper",0x10139ba1,"Armor_014_Head",0x5645d75f,"Armor_014_Lower",0x368073e0,"Armor_014_Upper",0xdb4f4804,"Armor_015_Head",0xf843a431,"Armor_015_Head_B",0xf032dceb,"Armor_015_Lower",0x90f77854,"Armor_015_Upper",0x9687e90f,"Armor_017_Head",0x67adcdc2,"Armor_017_Lower",0x0768697d,"Armor_017_Upper",0xba6ac32a,"Armor_020_Head",0x2c85f4af,"Armor_020_Lower",0x4c405010,"Armor_020_Upper",0x7136108f,"Armor_021_Head",0x8af2ff1b,"Armor_021_Lower",0xea375ba4,"Armor_021_Upper",0xf7a26221,"Armor_022_Head",0xcb0b91ed,"Armor_022_Head_B",0x21fb813c,"Armor_024_Head",0xeaa75299,"Armor_025_Head",0x6c332037,"Armor_026_Head",0xa76ff392,"Armor_027_Head",0x56394147,"Armor_028_Head",0x9d6592e2,"Armor_029_Head",0x4cd0592d,"Armor_035_Head",0xb217db96,"Armor_035_Lower",0xd2d27f29,"Armor_035_Upper",0xca442b83,"Armor_036_Head",0x83ffc10b,"Armor_036_Lower",0xe33a65b4,"Armor_036_Upper",0x0118f826,"Armor_037_Head",0x2588cabf,"Armor_037_Lower",0x454d6e00,"Armor_037_Upper",0x3b129956,"Armor_039_Head",0x75b7b1e2,"Armor_039_Lower",0x1572155d,"Armor_039_Upper",0xd9baf610,"Armor_040_Head",0xeaeafd28,"Armor_040_Lower",0x8a2f5997,"Armor_040_Upper",0x9472571b,"Armor_042_Head",0xb811e867,"Armor_042_Head_B",0x7d75ec01,"Armor_042_Lower",0x1db048be,"Armor_042_Upper",0xdb02e7b5,"Armor_043_Lower",0xbbc7430a,"Armor_043_Upper",0x7e607d84,"Armor_044_Upper",0x897767a3,"Armor_045_Head",0x0fe3150d,"Armor_046_Head",0x893ac812,"Armor_046_Lower",0xe9ff6cad,"Armor_046_Upper",0x35e9747d,"Armor_048_Head",0xd905b34f,"Armor_048_Lower",0xb9c017f0,"Armor_048_Upper",0x7f72b8fb,"Armor_049_Lower",0xf9598f0a,"Armor_053_Head",0x17a8e72b,"Armor_053_Lower",0x776d4394,"Armor_053_Upper",0x2f006c17,"Armor_055_Head",0xa9941eb9,"Armor_056_Head",0x4e25e739,"Armor_060_Head",0xa8cffa55,"Armor_060_Lower",0xc80a5eea,"Armor_060_Upper",0x8579349c,"Armor_061_Head",0x0eb8f1e1,"Armor_061_Lower",0x6e7d555e,"Armor_061_Upper",0x03ed4632,"Armor_062_Head",0x3f50eb7c,"Armor_062_Lower",0x5f954fc3,"Armor_062_Upper",0xc8b19597,"Armor_063_Head",0x9927e0c8,"Armor_063_Lower",0xf9e24477,"Armor_063_Upper",0xd5b4a52f,"Armor_064_Head",0x5c80de46,"Armor_064_Lower",0x3c457af9,"Armor_064_Upper",0x1ee8768a,"Armor_065_Head",0xfaf7d5f2,"Armor_065_Lower",0x9a32714d,"Armor_065_Upper",0x987c0424,"Armor_066_Head",0xcb1fcf6f,"Armor_066_Lower",0xabda6bd0,"Armor_066_Upper",0x5320d781,"Armor_067_Head",0x6d68c4db,"Armor_067_Lower",0x0dad6064,"Armor_067_Upper",0x230e3f28,"Armor_071_Head",0xc212f17f,"Armor_071_Lower",0xa2d755c0,"Armor_071_Upper",0xa59a4d86,"Armor_072_Head",0xf3faebe2,"Armor_072_Lower",0x933f4f5d,"Armor_072_Upper",0x6ec69e23,"Armor_073_Head",0x558de056,"Armor_073_Lower",0x354844e9,"Armor_073_Upper",0x73c3ae9b,"Armor_074_Head",0x902aded8,"Armor_074_Lower",0xf0ef7a67,"Armor_074_Upper",0xb89f7d3e,"Armor_075_Head",0x365dd56c,"Armor_075_Lower",0x569871d3,"Armor_075_Upper",0x3e0b0f90,"Armor_076_Head",0x07b5cff1,"Armor_076_Lower",0x67706b4e,"Armor_076_Upper",0xf557dc35,"Armor_077_Head",0xc97cdac6,"Armor_077_Head_B",0xa1c2c445,"Armor_077_Lower",0xc10760fa,"Armor_077_Upper",0x04016ee0,"Armor_078_Head",0x105cc86a,"Armor_078_Head_B",0x578ab4ac,"Armor_078_Lower",0x374f1013,"Armor_078_Upper",0xcf5dbd45,"Armor_079_Head",0xdcf6c8f4,"Armor_079_Head_B",0xf1fdbf18,"Armor_079_Lower",0x91381ba7,"Armor_079_Upper",0x988eeeca,"Armor_083_Head",0x8cadf2fa,"Armor_083_Lower",0xec685645,"Armor_083_Upper",0x858bde72,"Armor_084_Head",0x490acc74,"Armor_084_Lower",0x29cf68cb,"Armor_084_Upper",0x4ed70dd7,"Armor_085_Head",0xef7dc7c0,"Armor_085_Lower",0x8fb8637f,"Armor_085_Upper",0xc8437f79,"Armor_086_Head",0xde95dd5d,"Armor_086_Lower",0xbe5079e2,"Armor_086_Upper",0x031facdc,"Armor_087_Head",0x78e2d6e9,"Armor_087_Lower",0x18277256,"Armor_087_Upper",0xf2491e09,"Armor_088_Head",0x8eaaa600,"Armor_088_Lower",0xee6f02bf,"Armor_088_Upper",0x3915cdac,"Armor_089_Head",0x28ddadb4,"Armor_089_Lower",0x4818090b,"Armor_089_Upper",0xb86d97d0,"Armor_090_Head",0x71efe8f9,"Armor_090_Lower",0x112a4c46,"Armor_090_Upper",0xe8a00663,"Armor_095_Head",0x23d7c75e,"Armor_095_Lower",0x431263e1,"Armor_095_Upper",0x6e3474cd,"Armor_096_Head",0x123fddc3,"Armor_096_Lower",0x72fa797c,"Armor_096_Upper",0xa568a768,"Armor_097_Head",0xb448d677,"Armor_097_Lower",0xd48d72c8,"Armor_097_Upper",0x543e15bd,"Armor_098_Head",0x4200a69e,"Armor_098_Lower",0x22c50221,"Armor_098_Upper",0x9f62c618,"Armor_099_Head",0xe477ad2a,"Armor_099_Lower",0x84b20995,"Armor_099_Upper",0xe15fd29d,"Armor_100_Head",0x79dbe791,"Armor_100_Lower",0x191e432e,"Armor_100_Upper",0x2a030138,"Armor_101_Head",0xdfacec25,"Armor_101_Lower",0xbf69489a,"Armor_101_Upper",0xac977396,"Armor_102_Head",0xee44f6b8,"Armor_102_Lower",0x8e815207,"Armor_102_Upper",0x67cba033,"Armor_103_Head",0x4833fd0c,"Armor_103_Lower",0x28f659b3,"Armor_103_Upper",0x7ace908b,"Armor_104_Head",0x8d94c382,"Armor_104_Lower",0xed51673d,"Armor_104_Upper",0xb192432e,"Armor_105_Head",0x2be3c836,"Armor_105_Lower",0x4b266c89,"Armor_105_Upper",0x37063180,"Armor_106_Head",0x1a0bd2ab,"Armor_106_Lower",0x7ace7614,"Armor_106_Upper",0x8c740a8c,"Armor_111_Head",0x1306ecbb,"Armor_111_Lower",0x73c34804,"Armor_111_Upper",0x0ae07822,"Armor_112_Head",0x22eef626,"Armor_112_Lower",0x422b5299,"Armor_112_Upper",0xc1bcab87,"Armor_113_Head",0x8499fd92,"Armor_113_Lower",0xe45c592d,"Armor_113_Upper",0xdcb99b3f,"Armor_114_Head",0x413ec31c,"Armor_114_Lower",0x21fb67a3,"Armor_114_Upper",0x17e5489a,"Armor_115_Head",0xb664768a,"Armor_116_Upper",0x5a2de991,"Armor_117_Head",0xab7b5b44,"Armor_118_Head",0x602788e1,"Armor_119_Head",0x76c0c3b4,"Armor_120_Head",0xbd9c1011,"Armor_121_Head",0x3b0862bf,"Armor_122_Head",0xf054b11a,"Armor_123_Head",0xed5181a2,"Armor_124_Head",0x260d5207,"Armor_125_Head",0xa09920a9,"Armor_126_Head",0x6bc5f30c,"Armor_127_Head",0x9a9341d9,"Armor_128_Head",0x51cf927c,"Armor_129_Head",0xd0b7c800,"Armor_130_Head",0x1beb1ba5,"Armor_131_Head",0x9d7f690b,"Armor_132_Head",0x5623baae,"Armor_133_Head",0x4b268a16,"Armor_134_Head",0x807a59b3,"Armor_135_Head",0x06ee2b1d,"Armor_136_Head",0xcdb2f8b8,"Armor_137_Head",0x3ce44a6d,"Armor_138_Head",0xf7b899c8,"Armor_139_Head",0x1510f68e,"Armor_140_Head",0xfd91e96b,"Armor_140_Lower",0x5be6e2df,"Armor_141_Lower",0xaebb03b3,"Armor_148_Upper",0x08cc0807,"Armor_149_Upper",0x51fe4d4a,"Armor_150_Upper",0xf78946fe,"Armor_151_Upper",0xa6a4f8dc,"Armor_152_Lower",0x00d3f368,"Armor_153_Lower",0xc574cde6,"Armor_154_Lower",0x6303c652,"Armor_155_Lower",0x52ebdccf,"Armor_156_Lower",0xf49cd77b,"Armor_157_Lower",0x02d4a792,"Armor_158_Lower",0xa4a3ac26,"Armor_159_Lower",0x828fe7a7,"Armor_160_Head",0xbfb4ee16,"Armor_160_Lower",0xdf714aa9,"Armor_160_Upper",0x13db4a37,"Armor_170_Upper",0x6fd0d57e,"Armor_200_Head",0x4056db54,"Armor_200_Lower",0x20937feb,"Armor_200_Upper",0xa48c06db,"Armor_201_Head",0xe621d0e0,"Armor_201_Lower",0x86e4745f,"Armor_201_Upper",0x22187475,"Armor_202_Head",0xd7c9ca7d,"Armor_202_Lower",0xb70c6ec2,"Armor_202_Upper",0xe944a7d0,"Armor_203_Head",0x71bec1c9,"Armor_203_Lower",0x117b6576,"Armor_203_Upper",0xf4419768,"Armor_204_Head",0xb419ff47,"Armor_204_Lower",0xd4dc5bf8,"Armor_204_Upper",0x3f1d44cd,"Armor_205_Head",0x126ef4f3,"Armor_205_Lower",0x72ab504c,"Armor_205_Upper",0xb9893663,"Armor_206_Head",0x2386ee6e,"Armor_206_Lower",0x43434ad1,"Armor_206_Upper",0x72d5e5c6,"Armor_207_Head",0x85f1e5da,"Armor_207_Lower",0xe5344165,"Armor_207_Upper",0x83835713,"Armor_208_Head",0x73b99533,"Armor_208_Lower",0x137c318c,"Armor_208_Upper",0x48df84b6,"Armor_209_Head",0xd5ce9e87,"Armor_209_Lower",0xb50b3a38,"Armor_209_Upper",0xc9a7deca,"Armor_210_Head",0x8cfcdbca,"Armor_210_Lower",0xec397f75,"Armor_210_Upper",0x02fb0d6f,"Armor_211_Head",0x2a8bd07e,"Armor_211_Lower",0x4a4e74c1,"Armor_211_Upper",0x846f7fc1,"Armor_212_Head",0x1b63cae3,"Armor_212_Lower",0x7ba66e5c,"Armor_212_Upper",0x4f33ac64,"Armor_213_Head",0xbd14c157,"Armor_213_Lower",0xddd165e8,"Armor_213_Upper",0x52369cdc,"Armor_214_Head",0x78b3ffd9,"Armor_214_Lower",0x18765b66,"Armor_214_Upper",0x996a4f79,"Armor_215_Head",0xdec4f46d,"Armor_215_Lower",0xbe0150d2,"Armor_215_Upper",0x1ffe3dd7,"Armor_216_Head",0xef2ceef0,"Armor_216_Lower",0x8fe94a4f,"Armor_216_Upper",0xd4a2ee72,"Armor_217_Head",0x495be544,"Armor_217_Lower",0x299e41fb,"Armor_217_Upper",0x25f45ca7,"Armor_218_Head",0xbf1395ad,"Armor_218_Lower",0xdfd63112,"Armor_218_Upper",0xeea88f02,"Armor_219_Head",0x19649e19,"Armor_219_Lower",0x79a13aa6,"Armor_219_Upper",0xf84fc457,"Armor_220_Head",0x8ddb46ad,"Armor_220_Head_B",0x331317f2,"Armor_221_Head",0x41714633,"Armor_221_Head_B",0xb587655c,"Armor_222_Head",0xcffe41d0,"Armor_222_Head_B",0x7edbb6f9,"Armor_223_Head",0x0354414e,"Armor_223_Head_B",0x63de8641,"Armor_224_Head",0x09914857,"Armor_224_Head_B",0xa88255e4,"Armor_225_Head",0x504bf38e,"Armor_225_Lower",0x308e5731,"Armor_225_Upper",0x2e16274a,"Armor_226_Head",0x61a3e913,"Armor_226_Lower",0x01664dac,"Armor_226_Upper",0xe54af4ef,"Armor_227_Head",0xc7d4e2a7,"Armor_227_Lower",0xa7114618,"Armor_227_Upper",0x141c463a,"Armor_228_Head",0x319c924e,"Armor_228_Lower",0x515936f1,"Armor_228_Upper",0xdf40959f,"Armor_229_Head",0x97eb99fa,"Armor_229_Lower",0xf72e3d45,"Armor_229_Upper",0x5e38cfe3,"Armor_230_Head",0xced9dcb7,"Armor_230_Lower",0xae1c7808,"Armor_230_Upper",0x95641c46,"Armor_231_Head",0x68aed703,"Armor_231_Lower",0x086b73bc,"Armor_231_Upper",0x13f06ee8,"Armor_232_Head",0x5946cd9e,"Armor_232_Lower",0x39836921,"Armor_232_Upper",0xd8acbd4d,"Armor_233_Head",0xff31c62a,"Armor_233_Lower",0x9ff46295,"Armor_233_Upper",0xc5a98df5,"Armor_234_Head",0x3a96f8a4,"Armor_234_Lower",0x5a535c1b,"Armor_234_Upper",0x6515dc67,"Armor_500_Head",0x2437b69d,"Armor_500_Lower",0x44f21222,"Armor_500_Upper",0xae490fc2,"Armor_501_Head",0x8240bd29,"Armor_501_Lower",0xe2851996,"Armor_501_Upper",0xd36d030b,"Armor_502_Upper",0xe381aec9,"Armor_503_Head",0xc58b61fd,"Armor_Default_Head",0x5488f6c5,"Armor_Default_Upper",0xb20bc89b,"Obj_HeroSoul_Gerudo",0x86ec3051,"Obj_HeroSoul_Goron",0x05988901,"Obj_HeroSoul_Rito",0x754357dd,"Obj_HeroSoul_Zora",0xa6af2068,"TestSakookaBow",0x1a2f095a,"TestSakookaWeapon",0x5b9d393a,"TestSakooka_Head",0x2ebab03e,"Test_Sakooka_Shield",0xbd25d811,"Test_Zero_Shield",0xc8a241ff,"Weapon_Bow_001",0x51ab1045,"Weapon_Bow_002",0x26ac20d3,"Weapon_Bow_003",0xb8c8b570,"Weapon_Bow_004",0x56c6d45c,"Weapon_Bow_006",0xc679c9cd,"Weapon_Bow_009",0xd1b970be,"Weapon_Bow_011",0x3fb71192,"Weapon_Bow_013",0xa1d38431,"Weapon_Bow_014",0xd6d4b4a7,"Weapon_Bow_015",0x4fdde51d,"Weapon_Bow_016",0x38dad58b,"Weapon_Bow_017",0x149a4251,"Weapon_Bow_023",0x64f0b6de,"Weapon_Bow_026",0x13f78648,"Weapon_Bow_027",0x83489bd9,"Weapon_Bow_028",0xf44fab4f,"Weapon_Bow_029",0x948822aa,"Weapon_Bow_030",0x7a864386,"Weapon_Bow_032",0x0d817310,"Weapon_Bow_033",0xe4e2d625,"Weapon_Bow_035",0x7deb879f,"Weapon_Bow_036",0x9a53aa98,"Weapon_Bow_038",0xdbc9b46d,"Weapon_Bow_040",0x87e3d738,"Weapon_Bow_071",0x1eea8682,"Weapon_Bow_072",0x9234e436,"Weapon_Lsword_001",0x0b3db58c,"Weapon_Lsword_002",0x7c3a851a,"Weapon_Lsword_003",0xe25e10b9,"Weapon_Lsword_004",0x9559202f,"Weapon_Lsword_005",0x0c507195,"Weapon_Lsword_006",0xfc28e5e1,"Weapon_Lsword_010",0x8b2fd577,"Weapon_Lsword_011",0x122684cd,"Weapon_Lsword_012",0x6521b45b,"Weapon_Lsword_013",0xfb4521f8,"Weapon_Lsword_014",0x8c42116e,"Weapon_Lsword_015",0x154b40d4,"Weapon_Lsword_016",0x624c7042,"Weapon_Lsword_017",0xf2f36dd3,"Weapon_Lsword_018",0x85f45d45,"Weapon_Lsword_019",0xd705b622,"Weapon_Lsword_020",0x4e0ce798,"Weapon_Lsword_023",0xd068723b,"Weapon_Lsword_024",0x49612381,"Weapon_Lsword_027",0xaed90e86,"Weapon_Lsword_029",0xce1e8763,"Weapon_Lsword_030",0xb919b7f5,"Weapon_Lsword_031",0x2010e64f,"Weapon_Lsword_032",0x5717d6d9,"Weapon_Lsword_033",0xc973437a,"Weapon_Lsword_034",0xbe7473ec,"Weapon_Lsword_035",0x277d2256,"Weapon_Lsword_036",0x507a12c0,"Weapon_Lsword_037",0xc0c50f51,"Weapon_Lsword_038",0xf6582132,"Weapon_Lsword_041",0xf135e52b,"Weapon_Lsword_045",0x1f3b8407,"Weapon_Lsword_047",0xef431073,"Weapon_Lsword_051",0x9f29e4fc,"Weapon_Lsword_054",0xe82ed46a,"Weapon_Lsword_055",0x712785d0,"Weapon_Lsword_056",0x0620b546,"Weapon_Lsword_057",0xe1989841,"Weapon_Lsword_059",0xb3697326,"Weapon_Lsword_060",0xad1f867e,"Weapon_Lsword_074",0x9e99fd43,"Weapon_Shield_001",0x0790acf9,"Weapon_Shield_002",0x70979c6f,"Weapon_Shield_003",0xeef309cc,"Weapon_Shield_004",0x99f4395a,"Weapon_Shield_005",0x00fd68e0,"Weapon_Shield_006",0x77fa5876,"Weapon_Shield_007",0xe74545e7,"Weapon_Shield_008",0x90427571,"Weapon_Shield_009",0x698cad2e,"Weapon_Shield_013",0xf7e8388d,"Weapon_Shield_014",0x80ef081b,"Weapon_Shield_015",0x19e659a1,"Weapon_Shield_016",0x6ee16937,"Weapon_Shield_017",0xfe5e74a6,"Weapon_Shield_018",0xacaf9fc1,"Weapon_Shield_021",0x35a6ce7b,"Weapon_Shield_022",0x42a1feed,"Weapon_Shield_023",0xabc25bd8,"Weapon_Shield_025",0x32cb0a62,"Weapon_Shield_026",0xc2b39e16,"Weapon_Shield_030",0xb5b4ae80,"Weapon_Shield_031",0x2cbdff3a,"Weapon_Shield_032",0x5bbacfac,"Weapon_Shield_033",0xc5de5a0f,"Weapon_Shield_034",0xb2d96a99,"Weapon_Shield_035",0x2bd03b23,"Weapon_Shield_036",0x5cd70bb5,"Weapon_Shield_037",0xcc681624,"Weapon_Shield_038",0x8df208d1,"Weapon_Shield_040",0xfaf53847,"Weapon_Shield_041",0x63fc69fd,"Weapon_Shield_042",0x0a8dac33,"Weapon_Shield_057",0xcd1b7824,"Weapon_Spear_001",0x5412299e,"Weapon_Spear_002",0x23151908,"Weapon_Spear_003",0xbd718cab,"Weapon_Spear_004",0xca76bc3d,"Weapon_Spear_005",0x537fed87,"Weapon_Spear_006",0x2478dd11,"Weapon_Spear_007",0xb4c7c080,"Weapon_Spear_008",0xc3c0f016,"Weapon_Spear_009",0xa30779f3,"Weapon_Spear_010",0xd4004965,"Weapon_Spear_011",0x4d0918df,"Weapon_Spear_012",0x3a0e2849,"Weapon_Spear_013",0xa46abdea,"Weapon_Spear_014",0xd36d8d7c,"Weapon_Spear_015",0x4a64dcc6,"Weapon_Spear_016",0x3d63ec50,"Weapon_Spear_017",0xaddcf1c1,"Weapon_Spear_018",0xff2d1aa6,"Weapon_Spear_021",0x66244b1c,"Weapon_Spear_022",0x11237b8a,"Weapon_Spear_023",0x8f47ee29,"Weapon_Spear_024",0xf840debf,"Weapon_Spear_025",0x164ebf93,"Weapon_Spear_027",0x86f1a202,"Weapon_Spear_028",0xf1f69294,"Weapon_Spear_029",0x91311b71,"Weapon_Spear_030",0xe6362be7,"Weapon_Spear_031",0x7f3f7a5d,"Weapon_Spear_032",0x08384acb,"Weapon_Spear_033",0x965cdf68,"Weapon_Spear_034",0xe15beffe,"Weapon_Spear_035",0x7852be44,"Weapon_Spear_036",0x0f558ed2,"Weapon_Spear_037",0x9fea9343,"Weapon_Spear_038",0x40141815,"Weapon_Spear_047",0xa7ac3512,"Weapon_Spear_049",0xc76bbcf7,"Weapon_Spear_050",0x094c3c64,"Weapon_Sword_001",0x90456dde,"Weapon_Sword_002",0xe7425d48,"Weapon_Sword_003",0x7926c8eb,"Weapon_Sword_004",0x0e21f87d,"Weapon_Sword_005",0x9728a9c7,"Weapon_Sword_006",0xe02f9951,"Weapon_Sword_007",0x709084c0,"Weapon_Sword_008",0x0797b456,"Weapon_Sword_009",0xfe596c09,"Weapon_Sword_013",0x603df9aa,"Weapon_Sword_014",0x173ac93c,"Weapon_Sword_015",0x8e339886,"Weapon_Sword_016",0xf934a810,"Weapon_Sword_017",0x698bb581,"Weapon_Sword_018",0x1e8c8517,"Weapon_Sword_019",0x4c7d6e70,"Weapon_Sword_020",0x3b7a5ee6,"Weapon_Sword_021",0xa2730f5c,"Weapon_Sword_022",0xd5743fca,"Weapon_Sword_023",0x4b10aa69,"Weapon_Sword_024",0x3c179aff,"Weapon_Sword_025",0xd219fbd3,"Weapon_Sword_027",0x35a1d6d4,"Weapon_Sword_029",0x55665f31,"Weapon_Sword_030",0x22616fa7,"Weapon_Sword_031",0xcc6f0e8b,"Weapon_Sword_033",0x520b9b28,"Weapon_Sword_034",0x250cabbe,"Weapon_Sword_035",0x1a27c9f6,"Weapon_Sword_040",0x6d20f960,"Weapon_Sword_041",0x832e984c,"Weapon_Sword_043",0x1d4a0def,"Weapon_Sword_044",0x84435c55,"Weapon_Sword_047",0x14fc41c4,"Weapon_Sword_048",0x63fb7152,"Weapon_Sword_049",0x033cf8b7,"Weapon_Sword_050",0x743bc821,"Weapon_Sword_051",0xed32999b,"Weapon_Sword_052",0x9a35a90d,"Weapon_Sword_053",0x9d586d14,"Weapon_Sword_057",0x0de77085,"Weapon_Sword_058",0x7ae04013,"Weapon_Sword_059",0x2811ab74,"Weapon_Sword_060",0x5f169be2,"Weapon_Sword_061",0xc61fca58,"Weapon_Sword_062",0x310a9a35,"Weapon_Sword_070",0xa803cb8f,"Weapon_Sword_073",0x0e56405f,"Weapon_Sword_DemoCheck"]},
	{type:"s32",hashes:[0x0a577f65,"AmiiboLastTouchDate",0x07911dce,"AmountRupeeForNextRevivalFountain",0x99cfcfac,"AmountRupeeForRevivalFountain1",0x00c69e16,"AmountRupeeForRevivalFountain2",0x77c1ae80,"AmountRupeeForRevivalFountain3",0xe9a53b23,"AmountRupeeForRevivalFountain4",0x9ea20bb5,"AmountRupeeForRevivalFountain5",0x29cc99a0,"AoCVerAtLastPlay",0x04a90712,"App_Award_StepCountTotal",0x96bd8326,"App_ExtendUsePoint",0x8f4a4f6e,"App_Menu_NewFlag",0xcd9e34ac,"App_Pedometer_StepCount",0xf40c2b0e,"App_SeakSensor_SearchMode",0x530f70f0,"App_SeakSensor_SlotIdx",0x53b00a7d,"BokoblinSeries_Counter",0xe7ce4453,"BowPorchStockNum",0x2bb321f1,"BreakHartValue",0xcd62df38,"Carnivorous_Boy_Stuffed",0x4c2dfe7e,"Counter_TerminalElectric",0xee506ae7,"Counter_TerminalFire",0xce303243,"Counter_TerminalWater",0x978537d6,"Counter_TerminalWind",0xbe9bc993,"CurrentHart",0xce7afed3,"CurrentMamo",0x23149bf8,"CurrentRupee",0x54679940,"DefeatedForestGiantNum",0x698266be,"DefeatedGolemNum",0x441b7231,"DefeatedSandwormNum",0x7f8727de,"Defeated_RemainsFire_Drone_A_01_Num",0xe605ce62,"DungeonClearCounter",0xf11ff7e1,"DungeonClearSealNum",0x97f925c3,"ElectricSupportUseNum",0x26eaaa25,"FairyRevivalNum",0xf1cf4807,"FireSupportUseNum",0x34666354,"FreezeHartValue",0xb1c7ef6d,"GerudoQuestCount",0x9abff7fd,"GoronCamp_BestScore",0x2f9804a8,"HatenoNPC022_TotalInsects",0xcec599f3,"Hateno_SheikPad_Lv2_Total",0x8a94e07a,"HiddenKorok_Number",0xbf19f40c,"Horse_ActiveIndex",0x7082e8bc,"Horse_Extra_Charge_Num",0x310a2c2d,"Item_AttackRate",0xcfefd7b6,"Item_BurnLevelAdd",0x8f6b1117,"Item_ClimbVelRate",0xffd624ac,"Item_ColdResistLevelAdd",0x205c9c65,"Item_DefenceRate",0x02573860,"Item_ElectricLevelAdd",0x831f2bd8,"Item_FreezeLevelAdd",0x1d0ba186,"Item_HotLevelAdd",0xeebb4764,"Item_LifeMaxAdd",0x81f7862a,"Item_LightningLevelAdd",0xa4fcd17b,"Item_NoiseRate",0xc853a2c5,"Item_SwimVelRate",0x9464b5e1,"Item_VelRate",0xd4e98bcb,"JoyConWirelessStyleWarningCount",0xaca57ad6,"Last_Ridden_Horse_Extra_Charge_Num",0x86f1c752,"LatestAoCVerPlayed",0x8891f228,"LizarfosSeries_Counter",0x2906f327,"MaxHartValue",0x7fd1cca2,"MiniGAme_HorsebackArchery_BestRecord",0x15169768,"MiniGame_HorseRace_BestResultMiliSecond",0x3c3193eb,"MiniGame_HorseRace_BestResultMinute",0xe4ec2937,"MiniGame_HorseRace_BestResultSecond",0xfc227172,"MiniGame_ParasailRide_BestRecord",0x0a8c5fff,"MoriblinSeries_Counter",0x2ea347af,"MsterSword_TalkLv",0x7c586c55,"NPC_oasis063_Item_Fruit_A",0x2e67d50b,"NPC_oasis065_Armor_008_Head",0x6078e9e7,"NPC_oasis065_Armor_008_Lower",0x00bd4d58,"NPC_oasis065_Armor_008_Upper",0xeb56d4ab,"NpcDemonStatue_UtuwaSum",0x72d8a917,"NpcTestIshikawa_Item_Meat_01",0xebd1f8ad,"NpcTestIshikawa_Item_Meat_02",0xecbc3cb4,"NpcTestIshikawa_Item_Meat_06",0x9bbb0c22,"NpcTestIshikawa_Item_Meat_07",0x6bc39856,"NpcTestIshikawa_Item_Meat_11",0x8e3abac7,"NpcTestIshikawa_Item_Ore_H",0x7e15ce4f,"OneHundred_KillCount",0x626848d6,"PaidRupeeInFountain1",0xfb61196c,"PaidRupeeInFountain2",0x8c6629fa,"PaidRupeeInFountain3",0x1202bc59,"PaidRupeeInFountain4",0x65058ccf,"PaidRupeeInFountain5",0x73f13fbc,"PictureMemory_Spot_Int",0x73c29681,"PlayReport_PlayTime",0xa439d800,"PlayerEquipItemType",0x5ed42d75,"QuestIndexSetPointGuide",0x6be4d0ea,"RemainsElectric_SmallKeyNum",0x0ce1b2ce,"RemainsFire_SmallKeyNum",0x698dafc0,"RemainsWater_SmallKeyNum",0xd57f6905,"RemainsWind_SmallKeyNum",0xd3674806,"RinelSeries_Counter",0xc3080fba,"SetItemCurrentNo",0x2fc0d2ab,"ShieldPorchStockNum",0x20e9fb84,"ShieldofKolog_mini_BestTime_L",0x57eecb12,"ShieldofKolog_mini_BestTime_M",0xade1f671,"ShieldofKolog_mini_BestTime_S",0xd3cafe11,"ShieldofKolog_mini_ThisTime_L",0xa4cdce87,"ShieldofKolog_mini_ThisTime_M",0x5ec2f3e4,"ShieldofKolog_mini_ThisTime_S",0x7d7c2a4d,"StalfosSeries_Counter",0xdcd63370,"SunazarashiRace_BestTime_L",0xabd103e6,"SunazarashiRace_BestTime_M",0x51de3e85,"SunazarashiRace_BestTime_S",0x2ff536e5,"SunazarashiRace_ThisTime_L",0x58f20673,"SunazarashiRace_ThisTime_M",0xa2fd3b10,"SunazarashiRace_ThisTime_S",0x33c52e44,"TartnackSeries_Counter",0x5f4ee4de,"Test_Npc_MamonoShop_Armor_022_Head",0x219be15c,"Test_Npc_MamonoShop_Armor_045_Head",0x87eceae8,"Test_Npc_MamonoShop_Armor_055_Head",0x01789846,"Test_Npc_MamonoShop_Armor_056_Head",0x2a636158,"Test_Npc_MamonoShop_Armor_160_Head",0x921eed1d,"Test_Npc_MamonoShop_Armor_160_Lower",0xf2db49a2,"Test_Npc_MamonoShop_Armor_160_Upper",0xaecbf357,"Test_Npc_MamonoShop_GameRomHorseReins_04",0x0fab1d0c,"Test_Npc_MamonoShop_GameRomHorseSaddle_04",0x955add77,"Test_Npc_MamonoShop_Item_Material_08",0xefadcdcf,"Test_Npc_MamonoShop_Weapon_Spear_036",0x8dd8ba7d,"Test_Npc_MamonoShop_Weapon_Sword_040",0xa02ac4e2,"Test_Npc_Road_Obj_ArrowBundle_A_01",0x221a530e,"Test_Npc_TripMaster_00_Animal_Insect_A",0x5bc6ebaa,"Test_Npc_TripMaster_00_Animal_Insect_H",0x5d40bf4c,"Test_Npc_TripMaster_00_Item_Enemy_57",0x98d180b4,"Test_Npc_TripMaster_00_Obj_ArrowBundle_A_01",0xb1249914,"Test_Npc_TripMaster_00_Obj_ArrowNormal_A_01",0x16541ebd,"Test_Npc_ValleyVillage010_Obj_BombArrow_A_01",0x8f5d4f07,"Test_Npc_ValleyVillage010_Obj_BombArrow_A_02",0xe05f036e,"Test_Npc_oasis001_Armor_027_Head",0x1109b1bb,"Test_Npc_oasis001_Armor_028_Head",0xda55621e,"Test_Npc_oasis001_Armor_029_Head",0xb0b8f5ef,"WM_TimeDivision",0x7467513e,"WM_bloodyEndReserveTimer",0x8c270c56,"WeaponPorchStockNum",0x7b4be851,"WM_NumberOfDays",0xfda0cde4,"WindSupportUseNum",0x71af2f23,"WolfLink_CurrentLife",0x8e879d96,"WolfLink_LastRecordedHearts",0xa4ddfd61,"ZoraMini_ReliefSearch_Count"]},
	{type:"f32",hashes:[0x3491d38f,"App_Map_Scale",0x4eccf571,"DRC_AppBatteryValue",0x53c8d421,"DebugMoveDistance",0x121684ab,"ElectricSupportReuseTimeInCastle",0x3738e32c,"ElectricSupportReuseTimeInField",0x6cd044fd,"FireSupportReuseTimeInCastle",0x40840821,"FireSupportReuseTimeInField",0x47d9e569,"Horse_RotY",0x81728b51,"Item_AttackTime",0xad44358a,"Item_BurnLevelAddTime",0x3f13b66b,"Item_ClimbVelTime",0xb7374988,"Item_ColdLevelAddTime",0x90243b19,"Item_DefenceTime",0xc50cdeef,"Item_ElectricLevelAddTime",0xf9024f9e,"Item_EnergyMaxAdd",0x1fa8432b,"Item_EnergyMaxAddTime",0xa1b5d308,"Item_FreezeLevelAddTime",0x5fdc5cd9,"Item_HotLevelAddTime",0x4eb47466,"Item_LifeMaxAddTime",0x24d3dbdf,"Item_LightningLevelAddTime",0x14847607,"Item_NoiseTime",0x782b05b9,"Item_SwimVelTime",0x241c129d,"Item_VelTime",0x526243a5,"Last_Ridden_Horse_RotY",0xb81b6da5,"MasterSwordRecoverTime",0xe1f39ed8,"MiniGame_BirdMan_HillTower_RecordDistance",0x6293b955,"PlayerSavePosAngleYDegree",0x58af5eac,"StaminaCurrentMax",0x3adff047,"StaminaMax",0x4234e199,"WM_BloodyMoonTimer",0x9c79a188,"WM_Time",0xa0b0eaed,"WaterSupportReuseTimeInCastle",0x2c5d8d95,"WaterSupportReuseTimeInField",0x065e1824,"WindSupportReuseTimeInCastle",0x7ef82330,"WindSupportReuseTimeInField",0xd687a3fc,"add_test"]},
	{type:"f32_array",hashes:[0xdbd09218,"DeadHorse_Familiarity",0xe64eb028,"GodBless_LastUseTime",0xe1a0ca54,"Horse_Familiarity"]},
	{type:"vector2f_array",hashes:[0xff95a905,"CookEffect0",0x88929993,"CookEffect1",0xda60b8cd,"StaminaRecover"]},
	{type:"vector3f",hashes:[0xf9a0275c,"Horse_Pos",0x982ba201,"Last_Ridden_Horse_Pos",0x1e4707b0,"MapApp_PointGuidePoint",0xa40ba103,"PlayerSavePos",0x87778dc9,"ShootingStarPosition"]},
	{type:"vector3f_array",hashes:[0xea9def3f,"MapApp_MapIconPos"]},
	{type:"bool",id:"C-5_Animal_RupeeRabbit_A_",hashes:[0xfaf6d0d1,1158573626,0x0add61ce,119676254,0x954f1340,1207652058,0xee4fe3fd,125156133,0xae7e41b4,1326369154,0x5580b554,1452972064,0x41ba5c58,2133969819,0xe4a87fb8,2216052439,0xf09322c3,2444329800,0xfe257789,2479244960,0xd4b41f8a,2847406036,0xad233624,2949923055,0x99273095,2987031642,0x44725ee8,3038019600,0x61b9d801,3058004865,0xa49d9125,3182492758,0xd6efa157,3266955628,0xb215807f,3289973653,0xb9464b88,3349720697,0x062c719d,3385779494,0x9c1b5daa,3430815441,0x8a8813e4,3453661316,0x43899775,3494308151,0x9b4df4b7,352056706,0x497caad5,3567214278,0xedcbac92,3698220879,0xe0eb917f,3835373149,0x1f7b36e2,3844573003,0x6b058274,3870968455,0x35947878,4061398525,0xfc694f58,4107212023,0xdc9afbdf,581551889,0x6174da1d,647652707,0x74f15487,654864030,0x6a8ef264,67622795,0xcdfb5872,750180744,0x8e240afc,789917160,0x2cb37f9b,8112710,0xee2a549f,823897486,0x7fd59159,956029213]},
	{type:"bool",id:"Enemy_Guardian_Mini_",hashes:[
0x6ad80327,"Baby_12074734",
0x66095dc8,"Baby_122748007",
0x75df8917,"Baby_1266903253",
0xc9ba4da4,"Baby_1506956104",
0x589c89bc,"Baby_1516711804",
0x1f0a8806,"Baby_1677477271",
0x0be85ec8,"Baby_1693900535",
0x35dc7b80,"Baby_1857268230",
0x6efb741c,"Baby_1931794335",
0x58a6c45d,"Baby_2119697256",
0x71f7f7da,"Baby_232436891",
0x0aae81c1,"Baby_2428658775",
0x07dc9639,"Baby_2588373674",
0x5b18de57,"Baby_2603013512",
0x3c7adfc0,"Baby_32907362",
0x42fcf405,"Baby_3402023080",
0x93051b76,"Baby_3954664609",
0x467f474c,"Baby_4081276934",
0xd04e1c29,"Baby_4189079613",
0x52361a99,"Baby_4209173229",
0x1d804c40,"Baby_4266961270",
0x290a4c21,"Baby_434930801",
0xcb1c2d28,"Baby_449976565",
0x2cf2e9aa,"Baby_464140840",
0xe14ebe9d,"Baby_675606334",
0xf0b3f079,"Baby_711623739",
0x9cd2471d,"Baby_859295607",
0xbd899657,"Junior_1022694805",
0x73029f32,"Junior_1618174821",
0x0a28dd2d,"Junior_1696615707",
0xb540e254,"Junior_2078709021",
0x0de513a6,"Junior_216114571",
0xbfd24611,"Junior_2299577936",
0xa3d52fea,"Junior_2476865048",
0xc4b75346,"Junior_304577933",
0xb2fa8da2,"Junior_3222881940",
0x042dd7a2,"Junior_337478064",
0xca9de2d5,"Junior_3466938473",
0xfa03356a,"Junior_3515525301",
0x63fbc16e,"Junior_4262983366",
0xf5543ebb,"Junior_55452432",
0x0905b85d,"Junior_751042018",
0x24b086f4,"Junior_766169018",
0x77dd0634,"Junior_927560093",
0xf0fa93a2,"Junior_Wipe_1387514970",
0xe111ea6d,"Junior_Wipe_2315392512",
0x49f8b5ef,"Junior_Wipe_3285708974",
0xe200cc3d,"Junior_Wipe_4225218943",
0x40e3652c,"Junior_Wipe_455548141",
0x6718bb46,"Junior_Wipe_587524306",
0xea26679d,"Middle_2352190641",
0xe16d304c,"Middle_3468909086",
0xb723e4f1,"Middle_3508831506",
0x1d61e750,"Middle_3809547592",
0x09d94553,"Middle_68391124",
0x91b6f98e,"Middle_962960752",
0xd351bf11,"Practice_373343272",
0xc41302b3,"Senior_1520705379",
0xad5b3252,"Senior_1860818910",
0x02c0fbe0,"Senior_2845352289",
0x02d77ef2,"Senior_3334538678",
0x81362950,"Senior_3632635303",
0x7aeace70,"Senior_3780796169",
0x3b61fff0,"Senior_4062528233",
0xdb8739ce,"Senior_567100857",
	]},
	{type:"bool",id:"TBox_Dungeon_",hashes:[
0x161656b2,"Iron_050_923248764",
0x6d2ce695,"Iron_1114850190",
0x154af266,"Iron_114738522",
0xfadfba31,"Iron_1297119120",
0x4db32bff,"Iron_1396031621",
0x9b3ef3b5,"Iron_1547680732",
0xe64c4f22,"Iron_1597604003",
0xf59647aa,"Iron_1708460808",
0xd093e98d,"Iron_1735097170",
0xdeb9482e,"Iron_1961782765",
0xd44a1f39,"Iron_2001786182",
0x31843e59,"Iron_223462799",
0xfa1a9001,"Iron_2489506180",
0xc5bd7840,"Iron_2731610075",
0x87e751b2,"Iron_2733670140",
0x6f7e5eee,"Iron_2905879506",
0x13f2fe03,"Iron_2953061699",
0x1ec2f62c,"Iron_296010651",
0xe0bf0f1d,"Iron_3003260476",
0x32d7bf68,"Iron_32387787",
0x9c9d3a32,"Iron_3264264754",
0xd793718c,"Iron_3284159718",
0x91045933,"Iron_3298610380",
0x4fd3c0dd,"Iron_3348821939",
0xbe8be031,"Iron_3375369818",
0x3d261c8d,"Iron_3658282587",
0x06f0e5d5,"Iron_3852730446",
0x73bab34a,"Iron_393155234",
0x596a279f,"Iron_4120858762",
0x8a783372,"Iron_418217937",
0xd64bb2a5,"Iron_651697158",
0xa0022914,"Iron_682054227",
0xd414517b,"Iron_731998508",
0x2a8df522,"Iron_771359327",
0x6782a903,"Iron_896824088",
0x29f0f0a4,"Stone_1015560256",
0x19f12105,"Stone_1101189494",
0x1f7062fc,"Stone_1161464155",
0x98756fcc,"Stone_1168499415",
0x0c6483e0,"Stone_1178382884",
0x53b00040,"Stone_1182477274",
0xdb3af095,"Stone_1203109046",
0xe1bee900,"Stone_1203458803",
0x8f3be5c9,"Stone_1221975918",
0x002a339e,"Stone_1223731997",
0xf7b86f1b,"Stone_1249330112",
0xa9c2801c,"Stone_1249779666",
0x2f9cdd6f,"Stone_1274382149",
0xea60d53e,"Stone_130001776",
0x38c0495f,"Stone_1306359972",
0x18f96105,"Stone_1319070279",
0xfa30fab0,"Stone_1345302867",
0x9883229c,"Stone_1366576292",
0x8c93f9e0,"Stone_1402060224",
0x5151f179,"Stone_1490683918",
0xc734abcd,"Stone_1505890884",
0xff8f61e8,"Stone_159148495",
0x0e8ea398,"Stone_1648376382",
0x45d659b9,"Stone_1749408352",
0xc97c6d4e,"Stone_1835622564",
0x13c1ef28,"Stone_1852894353",
0x76660c8a,"Stone_1908915428",
0x502d1c29,"Stone_1922807999",
0xc9ea6c74,"Stone_1956105793",
0xdafdd9d5,"Stone_2006054206",
0x6a46a2e4,"Stone_2042129072",
0xd049bf90,"Stone_2075859436",
0x4727d5dd,"Stone_2081482182",
0x88d1c02b,"Stone_2144851240",
0x90a191d3,"Stone_2150412173",
0xf87431e1,"Stone_215071769",
0xd20ab57f,"Stone_2175434126",
0x4d7883ff,"Stone_2202095126",
0x92ca4953,"Stone_2231710576",
0xcf47ebcf,"Stone_2260374746",
0xc9e3f305,"Stone_2298702348",
0xa3863a6d,"Stone_2319591679",
0x6b3f7560,"Stone_2340212918",
0x0ecdfa11,"Stone_235083489",
0x523ac97a,"Stone_2374996813",
0x1adfc763,"Stone_2402635144",
0x01647f7d,"Stone_2441275272",
0x3401866b,"Stone_246330360",
0x6ba67300,"Stone_2541023350",
0x7c844d21,"Stone_2561429726",
0x5fed87de,"Stone_2577142135",
0x680e56cc,"Stone_2591211974",
0xe41d998f,"Stone_2611637153",
0xf52f431d,"Stone_262007905",
0x45a0caf7,"Stone_2677842568",
0xd82fab13,"Stone_2699775799",
0xafc5d7fe,"Stone_2758576440",
0x0ded825c,"Stone_2775322850",
0xdd020460,"Stone_2783782463",
0x1e53b570,"Stone_2835421124",
0xacb79c6f,"Stone_2852596923",
0xdccbe049,"Stone_2855684808",
0xf2250a55,"Stone_287351360",
0x0a260a74,"Stone_2875625519",
0xfb53f705,"Stone_289382806",
0xf9583421,"Stone_2914122144",
0xf1c18fd6,"Stone_2917484848",
0x1486ff9c,"Stone_2928701109",
0x83be87f2,"Stone_2936889658",
0xa7fe747d,"Stone_294526304",
0xda72d7fc,"Stone_2966651740",
0x1eca9e17,"Stone_2975470194",
0x9c943aa8,"Stone_3032109463",
0xd1e24963,"Stone_3047743907",
0x23cacfd8,"Stone_304910307",
0x8484c68e,"Stone_3055684308",
0x1644df04,"Stone_3073200707",
0xd9daf6b2,"Stone_309690636",
0xe9def848,"Stone_3147218512",
0xdda2e997,"Stone_3163225088",
0x34650832,"Stone_3173256521",
0x61407f96,"Stone_319620032",
0xa9b4a886,"Stone_322360761",
0xd057c3d3,"Stone_3281368662",
0x0ea61331,"Stone_3287407663",
0xc9fac1df,"Stone_3410031031",
0x8472ff1c,"Stone_3450862395",
0xa1221bab,"Stone_3463086557",
0x58a96048,"Stone_3495362814",
0xae85be2b,"Stone_3497573834",
0xa43ee733,"Stone_3513544036",
0x8a3474c8,"Stone_3513966293",
0xdd02c07c,"Stone_3517324741",
0x05689c34,"Stone_3566506040",
0x7074f063,"Stone_359699135",
0x3653c785,"Stone_3612472744",
0x4286de8d,"Stone_3705447709",
0x64482ab3,"Stone_3717856843",
0x86e2912f,"Stone_3735012660",
0x8bbd814c,"Stone_3776874027",
0x0a0b55e8,"Stone_3819328534",
0x9aec118e,"Stone_3826616279",
0xb970a383,"Stone_3884443295",
0x81b31476,"Stone_3913217232",
0x88292599,"Stone_3918894197",
0x2f9fd008,"Stone_3962878289",
0x62bf661b,"Stone_396797432",
0x373f5939,"Stone_4012296241",
0x9c0e6fe0,"Stone_4013064750",
0xa19e183d,"Stone_4049467582",
0xea379a1b,"Stone_4073019342",
0xca9bf71e,"Stone_4073483487",
0x20d6b2b3,"Stone_4140941963",
0x1597aca2,"Stone_4153981767",
0x901f286e,"Stone_4192706961",
0x603fb3af,"Stone_4196993938",
0x01593981,"Stone_4223476174",
0x70d65d6a,"Stone_4248817833",
0x0a0f61c8,"Stone_4270154666",
0xdaf853da,"Stone_4272783493",
0x2dc654f6,"Stone_445242426",
0xd5c16a85,"Stone_450234134",
0xceaad5bd,"Stone_50928817",
0xd3b422fe,"Stone_56897618",
0x732ba397,"Stone_582159315",
0x41264207,"Stone_616546705",
0x3251aa6d,"Stone_628803076",
0x1c46e721,"Stone_638977289",
0x1c2f24da,"Stone_654507101",
0xff33f4a6,"Stone_699967431",
0x80eb9d08,"Stone_718693766",
0xfcc378c6,"Stone_787648808",
0xac515935,"Stone_79753044",
0xae862d63,"Stone_821435950",
0x9e7fb3c3,"Stone_832725204",
0xc03bf09d,"Stone_849632171",
0x7e6a6f99,"Stone_883436885",
0x864a6a89,"Stone_889586639",
0x9b573ab9,"Stone_944710900",
0x2e42bb91,"Stone_962037740",
0x0dbdf62d,"Stone_987962398",
0x2afecf3b,"Stone_992999892",
0xcd32bd79,"Stone_993230",
0x1c88a715,"Wood_1061208573",
0xd4999451,"Wood_1608635741",
0x2ee3be7b,"Wood_2545300147",
0x95cc97e2,"Wood_3299660229",
0x25dea6ac,"Wood_3617692800",
0x20e5ddec,"Wood_3647020470",
0xe99de1e0,"Wood_3853713592",
0x87fea0cf,"Wood_4214849996"
	]},
	{type:"bool",id:"CDungeon_",hashes:[
0xaff602b5,"Barrel_1092770907",
0x0e6b7c65,"Barrel_2083647576",
0x5823d7c4,"Barrel_3761156419",
0xf65a6bd1,"Barrel_CDungeon_1042770100",
0x225094f7,"Barrel_CDungeon_1160610760",
0x82fb4bad,"Barrel_CDungeon_126194817",
0x497d4b1a,"Barrel_CDungeon_144471684",
0x4821edfb,"Barrel_CDungeon_1705074755",
0x64b74bb9,"Barrel_CDungeon_1825997779",
0x752da6f2,"Barrel_CDungeon_1887609879",
0x245a700d,"Barrel_CDungeon_2128295552",
0xeeb5126b,"Barrel_CDungeon_2912899686",
0x65984632,"Barrel_CDungeon_2929227045",
0xbf0d96ec,"Barrel_CDungeon_3092003215",
0x527cb629,"Barrel_CDungeon_312889557",
0x2ff7b5d3,"Barrel_CDungeon_376146241",
0x7acf82c3,"Barrel_CDungeon_3856948289",
0x3d68cfff,"Barrel_CDungeon_4125086313",
0xa16fe848,"Barrel_CDungeon_594101259",
0xcf4055e1,"Barrel_CDungeon_705128812",
0x200d50d3,"Barrel_CDungeon_882246620",
0x84c2f710,"DgnObj_DownloadTerminal_A_01_1421474480",
0x943da9b1,"DgnObj_DownloadTerminal_A_01_2308174590",
0xc17bf254,"DgnObj_DownloadTerminal_A_01_3030563290",
0x5326b4a9,"DgnObj_DownloadTerminal_A_01_714582554",
0xf0b72bfd,"PutRupee_1171253277",
0x63284448,"PutRupee_1538858309",
0x264fd038,"PutRupee_1677703956",
0xaeecfc12,"PutRupee_1748322686",
0x827befd4,"PutRupee_1868074343",
0xdeb80296,"PutRupee_2169957451",
0xa358f13b,"PutRupee_2252023890",
0xdc8a2e6c,"PutRupee_2748183459",
0x9d0960cc,"PutRupee_2896973096",
0xaa8fae88,"PutRupee_3048701033",
0x305d38a1,"PutRupee_3154194498",
0x57def746,"PutRupee_3266333951",
0xe0972e5b,"PutRupee_351832962",
0x97dcc307,"PutRupee_3685449150",
0x1ec63c55,"PutRupee_4047247556",
0x149d99a4,"PutRupee_4081100421",
0x614c3384,"PutRupee_408927729",
0x71462ca1,"PutRupee_4132560093",
0x2de6f41f,"PutRupee_4293366006",
0x99a7751d,"PutRupee_523401704",
0x55c08625,"PutRupee_735308234",
0x73fdaa60,"PutRupee_750661075",
0xd16e9416,"PutRupee_852023435",
0xe05ad169,"PutRupee_Blue_1557453148",
0x83a65076,"PutRupee_Blue_1687322381",
0xd45dc05d,"PutRupee_Blue_2381437496",
0xa38d3cf0,"PutRupee_Blue_2881587505",
0x0ebf0185,"PutRupee_Blue_2963670577",
0x59ddb41c,"PutRupee_Blue_3000678512",
0x13fcbc38,"PutRupee_Blue_3406168276",
0xb6e057a9,"PutRupee_Blue_4210485934",
0x3c590483,"PutRupee_Blue_538516384",
0xf131346c,"PutRupee_Red_1461594934",
0xe29f8021,"PutRupee_Red_2061797973",
0x9779185b,"PutRupee_Red_2219283987",
0x7df19db6,"PutRupee_Red_2325487137",
0xa08f31f6,"PutRupee_Red_233028291",
0x68ee8dd4,"PutRupee_Red_2640095058",
0xb242afb7,"PutRupee_Red_3319630054",
0xc4d1dfec,"PutRupee_Red_3457645196",
0x120570de,"PutRupee_Red_3704011175",
0xfb076166,"PutRupee_Red_3931871172",
0xc0267760,"PutRupee_Red_4254805687",
0xccaf6e14,"ShutterKey_1183819034",
0xe3ce205d,"ShutterKey_1336221908",
0x05eb25a7,"ShutterKey_1366654233",
0x73a67ec1,"ShutterKey_1627093465",
0xe811a869,"ShutterKey_2271520545",
0x2eb56657,"ShutterKey_2648027468",
0x9e11fb3b,"ShutterKey_3210151054",
0x3fa05090,"ShutterKey_3727694420",
0x6fe42d84,"ShutterKey_4003336557",
0x418383b6,"ShutterKey_476606135",
0x12031ecc,"ShutterKey_48098650",
0x8e1a5c5f,"ShutterKey_968152827",
0x4b7e4c09,"SwitchCandlestickAncientOn_3142240707",
0x8b4249ad,"SwitchCandlestickAncient_1067647571",
0xf6e088ac,"SwitchCandlestickAncient_1068076307",
0x57581ac0,"SwitchCandlestickAncient_1168177335",
0x9acb2f7a,"SwitchCandlestickAncient_1316041193",
0x9a793d29,"SwitchCandlestickAncient_1565971740",
0x4168334f,"SwitchCandlestickAncient_200942711",
0x8f704986,"SwitchCandlestickAncient_2106658532",
0xe9c73c55,"SwitchCandlestickAncient_2282119702",
0x9889337d,"SwitchCandlestickAncient_2333832143",
0xacb1cd5a,"SwitchCandlestickAncient_2357333882",
0xe3dbbbdc,"SwitchCandlestickAncient_255818066",
0xeb076a30,"SwitchCandlestickAncient_2781927598",
0x4b0a0799,"SwitchCandlestickAncient_2919143501",
0x0d79b190,"SwitchCandlestickAncient_308709310",
0xe0262763,"SwitchCandlestickAncient_3155171738",
0x8854714c,"SwitchCandlestickAncient_3190822737",
0x0bd5a1ec,"SwitchCandlestickAncient_3838561915",
0x0978012a,"SwitchCandlestickAncient_3900610278",
0xc6244d05,"SwitchCandlestickAncient_4055371989",
0x2a5be03d,"SwitchCandlestickAncient_891246988",
0xf09b6d66,"Weapon_Lsword_031_2868413613",
0x84cf3a54,"Weapon_Sword_043_159970568",
	]},
	{type:"bool",id:"Clear_Dungeon",hashes:[0xf7a9cd64,"000",0x80aefdf2,"001",0x19a7ac48,"002",0x6ea09cde,"003",0xf0c4097d,"004",0x87c339eb,"005",0x1eca6851,"006",0x69cd58c7,"007",0xf9724556,"008",0x8e7575c0,"009",0xeeb2fc25,"010",0x99b5ccb3,"011",0x00bc9d09,"012",0x77bbad9f,"013",0xe9df383c,"014",0x9ed808aa,"015",0x07d15910,"016",0x70d66986,"017",0xe0697417,"018",0x976e4481,"019",0xc59fafe6,"020",0xb2989f70,"021",0x2b91ceca,"022",0x5c96fe5c,"023",0xc2f26bff,"024",0xb5f55b69,"025",0x2cfc0ad3,"026",0x5bfb3a45,"027",0xcb4427d4,"028",0xbc431742,"029",0xdc849ea7,"030",0xab83ae31,"031",0x328aff8b,"032",0x458dcf1d,"033",0xdbe95abe,"034",0xacee6a28,"035",0x35e73b92,"036",0x42e00b04,"037",0xd25f1695,"038",0xa5582603,"039",0x93c50860,"040",0xe4c238f6,"041",0x7dcb694c,"042",0x0acc59da,"043",0x94a8cc79,"044",0xe3affcef,"045",0x7aa6ad55,"046",0x0da19dc3,"047",0x9d1e8052,"048",0xea19b0c4,"049",0x8ade3921,"050",0xfdd909b7,"051",0x64d0580d,"052",0x13d7689b,"053",0x8db3fd38,"054",0xfab4cdae,"055",0x63bd9c14,"056",0x14baac82,"057",0x8405b113,"058",0xf3028185,"059",0xa1f36ae2,"060",0xd6f45a74,"061",0x4ffd0bce,"062",0x38fa3b58,"063",0xa69eaefb,"064",0xd1999e6d,"065",0x4890cfd7,"066",0x3f97ff41,"067",0xaf28e2d0,"068",0xd82fd246,"069",0xb8e85ba3,"070",0xcfef6b35,"071",0x56e63a8f,"072",0x21e10a19,"073",0xbf859fba,"074",0xc882af2c,"075",0x518bfe96,"076",0x268cce00,"077",0xb633d391,"078",0xc134e307,"079",0x3f70476c,"080",0x487777fa,"081",0xd17e2640,"082",0xa67916d6,"083",0x381d8375,"084",0x4f1ab3e3,"085",0xd613e259,"086",0xa114d2cf,"087",0x31abcf5e,"088",0x46acffc8,"089",0x266b762d,"090",0x516c46bb,"091",0xc8651701,"092",0xbf622797,"093",0x2106b234,"094",0x560182a2,"095",0xcf08d318,"096",0xb80fe38e,"097",0x28b0fe1f,"098",0x5fb7ce89,"099",0xf66ba753,100,0x816c97c5,101,0x1865c67f,102,0x6f62f6e9,103,0xf106634a,104,0x860153dc,105,0x1f080266,106,0x680f32f0,107,0xf8b02f61,108,0x8fb71ff7,109,0xef709612,110,0x9877a684,111,0x017ef73e,112,0x7679c7a8,113,0xe81d520b,114,0x9f1a629d,115,0x06133327,116,0x711403b1,117,0xe1ab1e20,118,0x96ac2eb6,119]},
	{type:"bool",id:"MainField_Npc_HiddenKorokFly_",hashes:[0x2e2b3cc3,1022207347,0xc9f260d3,1050589629,0x621d08df,1064779083,0xfd72c546,1090454606,0xbcbec70f,1112735978,0xdfc6edc1,1124420693,0xc239ed48,112511260,0xb0a896f6,1127673713,0xf450426c,1136402706,0x26743533,1151365424,0x32db0e32,1155006988,0x13025056,116485171,0x8c6caeff,1183281111,0xdb4d9121,1193735031,0xab4471a0,119564837,0xeac72197,1208482038,0x5b2750bd,1232628477,0x35ddecf1,1249482226,0xb3d00fa3,1253008712,0x437626cf,1257881160,0xf1888b2f,1261470678,0x07f53fae,1262205077,0xd40caa6e,1266640872,0xb2c7a150,1296891611,0xf9342a29,1319766043,0xcb8bdf6e,1325768340,0x399ffe15,1337298610,0x9b7f8797,1339515579,0xa0d80366,1352570001,0xec777d7c,1362729370,0xc1d550bc,1422977152,0xbb03c92f,1465600658,0x9812135c,1469044075,0xdaea9644,1474527610,0x70b00c66,1499375658,0x497b3aa1,1520984455,0xbbba17c1,1524700646,0x52eabbf9,1530191271,0x506472d9,1543038647,0xcf88b5d2,1551975449,0x78ca3e57,1561723616,0xb50f57b1,1566295243,0x4e86f173,1576838573,0x8cb6e831,1593037753,0x8a1c1396,1608828770,0xf9eb79ab,1636091100,0x0d90eabf,1651524906,0x40c903b6,1677622965,0x5fdb248a,1722524507,0x578f3784,1725820915,0x7fc87ad6,1754988899,0x49305266,1823309323,0x9a2a7ab3,1829369460,0x04774091,1840487990,0x285be209,1875716049,0x9f8b8360,1887181827,0xe73804b3,1919917625,0x93ea4614,1927658737,0x830f1ec0,1992448098,0x61895378,2011220059,0x8dec90ea,2026553014,0x1be2b3ed,2046521464,0x417748c0,2053359120,0x0c66c8aa,2057390424,0xcf4a869e,2057988876,0x095d5540,2072356967,0x3b8acb25,2120438097,0x9db69b95,2140588052,0x00753066,2151114466,0xf58b5c80,215198497,0x90aea807,2153057456,0xa26bf7c8,2154547408,0x29531d38,2164384451,0x485ae219,2191856877,0x0d62d541,219442737,0x7bfe2497,2211004746,0xdf664163,2236402541,0x74d76fb9,2251374971,0xbe4a5e9b,2252948811,0x19b169e1,2263320851,0xbb45ccb7,2263906631,0x7186e8b8,2269908490,0x22374558,227483275,0xfa7b263f,2285399707,0x424fc160,2290462947,0x60ebf61e,2292564599,0x542ca5dd,2297712141,0x5f208f59,2302345046,0xa3cecb89,2309251859,0x75910bc5,2326201195,0xfe508967,2329548229,0x1317a6a4,2338426315,0xc74e2a27,2354973447,0xbf572908,2363841502,0x64406b98,2408228336,0x31a00e68,2413762907,0x32441ef9,2425510130,0x31c92a64,244721166,0xbbb11045,245869383,0x2b306b21,2465190314,0x0e6e49e6,2474498880,0xa89818b7,2480656265,0xcc5c6d8c,2490721015,0x7dd35902,2515119845,0xdd836be1,2542645876,0xcf8c8a6b,2546320076,0xf6099a1a,2579365830,0x42cf300d,2620796805,0x17558872,2621942732,0xb0978dc0,2628974252,0x33d12189,2636187612,0x450bc0fc,2640143091,0x63a4612c,2641663225,0x4c34123f,2645385560,0x2a956d11,2645476262,0x3bb376d3,264665993,0xba2cee6a,2651170340,0xeadbf30c,2651467232,0xfd840f72,2672225838,0x1ffc5f3f,2675639802,0xf5b246cc,2679491751,0xb539c88a,2690380187,0x2ba19d1e,2694065779,0x6044b661,2702637595,0x2685f3a7,2703244554,0xba58f3fc,2717799946,0xb6f62fe4,2742864145,0x6134dc7d,2743740244,0xc6fe441e,2745514599,0x3e1546e4,2747244218,0xca1474b6,275156452,0x8c2a33fa,2759372052,0x80826056,2767708004,0xd186c389,2780109106,0x5a4a3917,2793303281,0xd652c264,2804399739,0xced040d3,281281735,0x8f706bf5,2832530704,0x63643f49,2863238498,0xbad8620b,2867399667,0x0f42e520,2875183771,0x450b9f2b,2879961570,0xe1f3f35a,288440097,0x6f884090,2885402839,0x655e5c8b,2892366278,0xc6a34951,2916314180,0x9b7cd7b0,2917136623,0xfa2f3eec,2927891737,0x2c758cd9,2929328639,0x88cc839d,2936523584,0xf7812399,2946306338,0x7169f507,2948090468,0x1e5870f3,2951543377,0x6e9bbc97,2958858019,0xfb765fbc,2981673548,0x3c5e4ee6,2987318336,0xbeceb922,2992517687,0x8e4adad4,3026959853,0x0105ef12,3032367431,0x9bebaec0,3034687397,0x74e51faa,3038274648,0xf5163503,3044351923,0x279bcc46,3053310424,0xa956dc79,3058032502,0xe2810b71,3061761754,0xe0ec0587,3062918566,0xcc2f05cf,3063389586,0x31893753,3079321771,0xfc5b8f9f,3090117635,0xe02da94d,3113994354,0x7c0867ac,3131977703,0x475ded63,3152951751,0x7012ae43,3160306273,0x526ee929,3182217296,0x99996f7e,318480047,0xc9ef2e6c,3190071149,0x881a3647,3198020249,0x2ad0d275,3214040014,0xee48aa58,3224577198,0x8a07fb55,3225537006,0x5893c1cd,3231139354,0x0c1a9b21,3245995596,0xc4a46604,3259789333,0xa92b45ef,3262263077,0x471abe01,3271497649,0x17997994,3275551752,0xb3fa9939,3278671227,0xdeb178e3,3297847254,0x2b2674ca,3303592459,0xd62e065c,3308270212,0xd5d14cc9,3318184333,0xe973a2c6,333517756,0x89facd12,3336627681,0xbf74ae6d,3382019260,0xece4af29,3384687735,0x645bf0a5,3384722244,0x52d18fa9,3389003106,0xff7758a0,3394048784,0x9c1bfb39,3400343405,0x4cbf31a5,3401422434,0x05828e12,3408673367,0x032d8700,3434613859,0x2e36e82b,344836657,0x3efdd10b,3459563376,0x8490d920,3488513756,0x712b0ea5,3491610976,0xa44b516c,3493303056,0x581282c5,3536227717,0x35d2e144,3540012681,0xd7c86b91,3544845441,0xd61e3051,3550885106,0x45f2d306,3552444757,0x945bd1cd,3564894439,0xc57a665c,3578794647,0xd9852d2a,3580495079,0xb42b2abb,3590025154,0xd1f96d16,3608445349,0x2a685c1e,3618750795,0x9988d511,362609360,0xa664790a,3633198989,0xc0547b5f,3635956701,0x9596e203,3653179270,0xf8d0c224,3665083853,0x48faa931,3697763237,0xf8497c57,3700933592,0x28c6a403,3729755327,0x2a7115b9,3740970746,0x5911354c,3748954375,0x6c4af732,3749378092,0xca72f256,3750286411,0x655a0c0f,3751086134,0x5d4170ff,3755199878,0x98143e94,376113665,0xe80a9eda,3763036580,0xda11e11a,377746506,0xfca2f664,3793810333,0x57ebd46f,3795652841,0xee425e1d,3796948896,0x8ac957d4,3809656229,0xf93afc9a,3810548171,0x87c85b50,3816840535,0x6d65fa2a,3825262801,0xf9e81bfa,3826192622,0x492c8b33,3831503070,0x4e05dfc4,3831879330,0x6720557f,3842820246,0xed01da5f,3859942761,0x963411fc,3879211127,0x72d01780,3890843455,0xb6a4fe6e,3891957745,0xd6db8c19,3896630968,0x5a432e0e,3901382303,0x27d9e48f,3927102937,0x4e1bec8d,3945127084,0xec9bb374,3953845336,0x139f0bdc,3996960662,0xfe3efb1d,400790366,0xbd828e2c,401467884,0x8c3a4e37,4017749282,0xa91201aa,4022716825,0xf6853f20,4032872334,0xdcc8ba61,4048451037,0x900525b6,4057147142,0x5ae18d84,4059460495,0x3b7ce629,4071789712,0x2067839b,4072119604,0x3cd4b5f0,4072231486,0x992cf785,407624430,0x7dd43b3d,4105883532,0xc54c483d,4107170268,0x02621e0d,4111462616,0x2ef650ea,4118141301,0xcacc2df3,4118900724,0xfbee67d4,4131985077,0xdd8e3d43,4134973300,0x219f8191,4138083952,0x80cfe359,414107933,0xa2b4a213,4154014246,0xdf2a8fb0,415897415,0xef6dfdb7,416115277,0x85fd0726,4164980717,0xaf46339e,41878054,0x40041a33,4204787349,0x70e63ec6,4205786892,0xa6a81087,425273904,0xcbf505e2,4282342925,0x134475fc,4283895671,0xec65fd0e,438124275,0xf22a43d9,472969006,0x68de4eed,478180247,0x9b5dda92,498318607,0x2e04bfa8,517129623,0x27729224,568190560,0x35d44e1e,569815132,0x3aaaf546,570246006,0xa7cbccfa,585183142,0x71e8a107,611457876,0x4ea8552b,612095039,0x86f2438b,617619040,0xd2fb91a3,63874103,0x82fad086,639218460,0xfa2ecee2,642507168,0x97e79802,673868809,0xd8c72da7,679910969,0xb1ae7c07,686028788,0x141b4dec,70019679,0x4b864ab7,702311689,0x47079416,707729943,0x1b44308f,728604987,0x7f8291b4,733512211,0xf036238f,740738517,0xb5f30638,74370041,0x12915bc4,766972157,0xc3ae20b1,768302005,0xd3eb5666,790923402,0xbde4dc1c,809627027,0x2e72c9b2,80988521,0x909ff681,819083735,0x56323b35,839851619,0x44260d03,857151736,0xf9531c7b,874628698,0x76caf587,885502085,0x667c2c2c,892486826,0x1911ba9b,914646713,0x7166a639,922416713,0x7489ba85,938409442,0x4a4be4b2,951238180,0xd022178b,959088135,0x98753d6c,968561627,0xc22ceff6,980785323,0x86b9cf34,981322004,0x2c596e0b,988839085,0x655ac3b3,993744813]},
	{type:"bool",id:"MainField_Npc_HiddenKorokGround_",hashes:[0x2b617218,1003109007,0x4963817f,1022760820,0x2bb51da0,1025441222,0xf6264d79,1034472920,0x1b50b160,1043683511,0x86e964bc,1055044777,0xf99b49af,1055964112,0x8c119354,1057880102,0x8ef728de,1066755147,0x286b87c1,1077827977,0xc4c62b2c,1090614442,0x5ec0d1ab,1099857635,0x21fa3907,1100557100,0xd24e7a25,1104919848,0x03d6015e,1107534550,0xcc29f2e1,1108415697,0x8c323e3b,1112505156,0x778f207d,1114715454,0xe4dfc1c0,1123836292,0x6801dca3,1124214851,0xcc35335a,1126570130,0xa394e375,1126579071,0x2f3c2ea2,1133279720,0x14a7d3b8,1142365941,0x35ff3af3,1151237411,0x5700b68a,1151900837,0x3e7a2c72,1165807816,0xe06124cb,1170208993,0xdfbafa88,1172987202,0x921c662a,117346372,0x7d787c89,1174894888,0xa5235679,1176465784,0xe0ec0c23,1187507014,0x8818e264,1191497114,0xc7b919e2,1192097443,0xfdd548dc,1196662160,0x97c9c11b,1205524554,0x07975126,1243204944,0x2adc59a7,1253263107,0x2a134c17,1256790451,0xf7fb7294,1274438551,0x58932cbd,1274917796,0x82ea3f2d,1277675559,0x70799bef,1287762421,0x4df49e60,1292505585,0x5af84790,1295414834,0x224c2d20,1316665502,0x6789cfcd,1354368149,0xbde42185,1357373610,0x7b10dfc2,1363103568,0x478bf467,1370935518,0xd2ac12eb,1390458183,0x8f2a30e1,1394129791,0xb29d5968,1408587618,0x2c016340,1410368125,0xcf43cc73,1419469252,0x45cafb59,1432044597,0xf9e10545,1457359469,0x38f48417,1458809933,0x62ef4882,1464881268,0x6d9a328b,1467735548,0x69c59d49,1486641369,0xb6514b9c,1489387789,0xfea08520,1490648511,0x2c8c2594,1495013638,0x649e315a,149758959,0x12c054b3,1501347682,0x8e40d3ee,1501878663,0xece1fa6a,1505263910,0x0b03cec2,1511540266,0xf0988858,1521470425,0xdd512e1b,1528393793,0xa2217391,1548021227,0x9ffb6db1,1555046530,0x2435519f,1557183492,0x01a77626,1559946760,0xd9d7b018,1574823026,0xb88a8351,1581054943,0xc5001ffb,1582805261,0x55d95f2b,1583540048,0x0066ded0,158421429,0x911bcd90,1591207013,0x5e2633fd,1603606398,0x6a047ec0,1618789613,0xb50397e3,1620165436,0xe23f09b9,1630771924,0x7e4d566b,1633736747,0xffa754fa,163673645,0xcf9d7c89,1641590512,0x05a246a2,1655461943,0x24d7197c,1689860325,0x18c5e539,1694444760,0x3a627ebc,1695768344,0x985f3beb,1704962879,0xca4be8f1,1708513594,0xa6f93c08,1719101522,0x037dfc51,1732815399,0x030bd873,1740805455,0x2fe77b9a,1743599302,0x98e17eee,1745306666,0x53bf9a40,1748713013,0xfa3e6133,175130178,0x944bb152,1762539812,0x9c77f87c,1769446694,0x07edf88b,1772183038,0x9fbac800,1788998636,0xdfabfc2e,1802155716,0x1d26bfc6,1806954803,0xb38f3294,1807988134,0xc12bdd2b,1810771052,0x6115ec62,1813209038,0xb97c0a67,182660367,0xfe750c11,1839771309,0xdfbd654f,1841374467,0x564f942b,1842521568,0x09af9809,1845056285,0x4dcf791e,1853561126,0x9f8c6f32,1873122871,0xf349734f,1880224914,0xd3d6c91c,1890767374,0xdf2fbf5f,1891870089,0x5f622fdc,1900984731,0x9ad72bfb,190942052,0x4d57c17f,1910443219,0x83bf0a9b,1912185042,0x3b854d84,1925951157,0x53449f3a,1926768837,0xdfda0f23,1946017359,0x7815fb3a,1955626705,0x4ddac174,1955931518,0x23166cb2,195808965,0x022e0e12,196795324,0x919a1346,1971282201,0x5bc2358d,1974856924,0xb0d722c0,1975203569,0x5ef432e2,1983659870,0x8ee513d1,1986291887,0xd656293a,1995415056,0xeed4a595,1995726509,0x9969542a,1996277818,0x0e6aa63d,2001041275,0x48eb753e,2033992327,0xb93698b0,2034029049,0x881390d3,20346760,0x59ab55bb,203862516,0x8b47a5fb,2040463177,0x4a9fe113,2040853061,0xd60dddfc,2040966359,0xf443fae0,2046074012,0xcf58e63d,2055319932,0xf80d5e7e,206037345,0x86d7be5a,2065742646,0x293f9601,2075344539,0xd0cd4001,2087107570,0x1a51b775,2090152705,0x6e998846,2092993004,0x99736b62,2109849091,0x8f3ab788,2110712400,0xe11fff55,211206137,0xdd5a8ff0,212164614,0xcef4746e,2124463981,0xcd93538a,2134832354,0x4c71cd7d,2136635167,0x15c3415d,214066177,0x8a4b420c,2143483470,0xdd777485,2145653167,0xc0c5b569,2155566002,0x6b93ced0,2166299771,0x0f9242ec,2169697373,0x44e4d73d,2185082907,0xff6a74af,2191142149,0x2bdafa92,220020778,0x3814a1ea,2203056445,0xdc0848c2,2206451776,0xf26db476,221338425,0xb29ba230,2229769454,0x34b9ea2d,2235158994,0x93dbf4f8,2242915368,0xa1a2edef,2250578196,0xb690acc0,2262193132,0x12a50513,2267408466,0x3a031153,2274271458,0xd910c140,2280888457,0xbe8daba7,2289203164,0xf691cf9e,2295461653,0xaed4da21,2298821431,0x914e5c8b,230531253,0x947bb87d,2312047993,0xc826318a,2333949492,0x17277806,234440360,0xf8d0d057,2352343899,0x9ffc1594,2353605806,0x71f54da2,2357147485,0x44bae869,2379000428,0x5fcdbd75,2381085384,0xd97128ce,2388081487,0x44760b6f,2400505191,0x610076d6,2407067424,0x58ed79fc,2410435147,0x4cf9af7a,2419732119,0x8ad9e8a7,2421908755,0xd71066f2,242784875,0x3b1391d4,2436092716,0x0bfa2096,2440708377,0x4784fd6b,2443011612,0x6c88172d,2448672352,0xce90cbd2,244961247,0x0b19b851,246591735,0xc2e1762c,2474186744,0x707072d3,248138722,0xdec72884,2494265121,0x9295df6c,2496367816,0x66a49116,2503614591,0xaeaa31fe,2504962267,0xac952baa,2514879761,0xd6e38e1e,2519166858,0xf9d1d011,2538992739,0x8eb3216e,2541805457,0xbb4bfa62,2546145880,0xa1300acf,2549908863,0xb77452ca,2551989172,0xdd4e2b38,2555543453,0x4bf81bb3,256319512,0xd9957080,2593866834,0x71e86829,2599870871,0xab116079,260387914,0x458327c7,2605263844,0x09533a19,2612760048,0xad635dca,2621974643,0x2172b5da,2627245617,0x3f8cebc3,2637046008,0x721f1852,2644584578,0xfffbdf04,2646642296,0xed80c7a5,2658985941,0xeb323cb8,2663662021,0xbdf2209a,2669349393,0x52e200aa,2678726617,0x751e528f,2685202162,0xcfd54587,2685534709,0x0e2d38d1,2699416523,0x96c02c14,2702338466,0x9c75e5f8,2705200596,0x567c4331,2712416928,0x604688d4,271822572,0xc61db84f,2725379834,0x36919813,2742708325,0xe7538675,2748078203,0x77efae60,2753014908,0x8a6c5698,2758390980,0xa79e3be3,2760820393,0xd00a5f8c,2766809335,0x7017c7d1,2779331030,0x92ae480a,2784382045,0x95cdfa70,2786704051,0x30abfb13,2794689043,0x75e0dfc6,2797164973,0x2789911a,281388505,0x853ee307,2834210857,0x8f633554,283772496,0x10ff5446,2840116265,0xc9fb4ba8,2841297783,0x7de6bf97,2842325967,0x91d09be7,284253909,0xd0fd5dce,2849759708,0xe1091dcc,2854638940,0x2482e3e2,2857143253,0x0ea24f4c,2861571087,0x17620397,2885083477,0xf634a5b7,2887292348,0x8583858c,2890610849,0xd2ba8c37,2904641484,0xf0274e1a,2907410757,0x883875aa,2907452236,0x79c7e150,2907491806,0x92ccabab,2922395813,0x31ed5300,292575428,0xc8ba33c8,2926065930,0x397c7666,2932670875,0x4c435209,293574068,0x938cdf69,2941760483,0xaf1381a7,2945927680,0x6071fa85,2953229461,0xe7bd6e0d,2960562638,0x0c91457e,2960596332,0x9696539d,2967053528,0xc88dcc27,2976628381,0x0a467aec,2977157730,0xaaf7b209,3002746880,0x1c2cebb5,3004822553,0x670c85ff,3005464103,0x1af5fbf1,3009490630,0x5ced9ad3,3025931676,0x0aab98c9,3036442541,0x042c6214,303801086,0xd4031665,3040996015,0xb092991a,3058629034,0x8c5a8b69,3068267418,0x4c676609,3071871579,0x89e19447,3077381654,0x4a1b0c3e,3082143218,0x8a3b5037,3084299899,0xa5b0ddd5,3089143643,0xe195562e,3089971833,0x8292f970,3099274170,0x9615d0d6,3113887708,0xcb09ff3f,3115869048,0x88e54d6b,3115870248,0x70665f74,311620516,0x03443957,3121333541,0x956be08e,312593884,0x4ae88291,3143270869,0x002c5b5d,3148789636,0x5b374fba,3152561036,0x8f03405e,3157046612,0xbfbebf6e,3158302180,0x07518cd7,3164740547,0xc31f0b04,316827338,0xd122eec3,3172460254,0x67e0b113,3188965979,0x5205d39d,3195731426,0x80d58d7e,3204990874,0x34e7350a,3207671420,0x7733f2b8,3223206605,0x05388d71,3227057467,0x04b6719b,3234651588,0x0de05681,3245194446,0x86ceb585,3288214517,0xd5210e5e,3289721783,0x4a68f549,329355923,0x2761df77,330732818,0x787bd471,331221107,0x03924391,3334334492,0x15905a70,3336956392,0xa63ff4cc,3338388419,0x3c716aa0,3342103247,0x6c199e80,3346017786,0x46ef8e1d,3351176543,0xbaba4538,3353180179,0x80e46a95,3358086753,0x2d7d823c,3358990058,0x604e6f4c,336637632,0xdb42b09c,3381823686,0x789ebea5,3382350609,0x32594185,3390101064,0x45c63ebb,3394776434,0x731a1644,3397528570,0x8b14d443,3407740780,0x8ec92e81,3431772484,0x9966d589,3433640415,0x3dc85c5a,3435847008,0x82ac968e,3440400112,0xf04e0c73,3449585398,0x35833d5f,3487360897,0x9b3310be,3493940551,0x4c624fdd,3510151725,0x1f9d7be0,3534313189,0x56a7b9b9,3535585095,0x8df41fab,3539006063,0x08546066,3541081322,0x1e92767b,3547235410,0x62cd12c3,3555530849,0x1f7e6066,3564984563,0xcb637a36,3565452126,0x0b727016,356547860,0xf1aa8b46,3567389425,0x181364a8,3577946435,0x70b4b53c,3577973525,0xa169ce01,358757673,0xf29f26c9,3590791728,0x59e0685e,3599610778,0x520b5971,3609162944,0xec46d3c7,3610068861,0x4ba7a2f7,3619749036,0x8ebae5b4,3621970781,0x470abbf6,3627156372,0x90931b5a,3637762495,0x80ec4d12,3640706254,0x50ab1278,3644262451,0x4b225629,3646955892,0xc8679196,3659537522,0x79fdfdf9,3662455251,0x63fbbcc0,3662938290,0xc55a580a,3669956414,0x535989d9,3671270425,0x8df0c4a7,3689020847,0x9fadacb6,3692404893,0x5ff70e36,3703681540,0x6fc72def,3708859739,0xae2177a0,3711673802,0x31776e42,3712478777,0xc232a5db,3712727875,0x84c082c4,3712748576,0x245527c7,3717672089,0xe7d81530,372397034,0xf90a90fd,3734640316,0x6e05e450,374681246,0x3432070d,3747962265,0x3b3f009f,3749556288,0xca31bdfa,3757948559,0x3ef6c241,3765893959,0xe27c26e0,3768037107,0xc0514ef5,3770952398,0xed7f50ce,3774592489,0x273b4962,3776995100,0x9b88be59,3779803099,0x2cc21c7d,3780330715,0x4a0d4d30,3784004966,0xe1edbaea,3789982093,0x71d5d765,379311799,0x13fe59ca,3796467789,0x8c5bda77,3803558448,0xd9f18bf9,3815190551,0xcaa658ea,3817401844,0x3fe2d9ba,3820454034,0x140f447d,3830003951,0x0b358ef9,3846600985,0x0bdebc55,3855289307,0xc0851403,3861969347,0xafd6ac26,3862225850,0x349f93b4,3863692092,0xdc2dd997,3872040196,0xd89c4741,3883998887,0x794f242d,3898015673,0x62939b45,39234779,0x94875ae7,3925702323,0x583a94ae,3929982099,0xea312f16,3940333182,0xdfc9ba84,3948649027,0x138ac4c9,3953108647,0xeafbc191,395395035,0xbcf9ae6f,3967712623,0x27ee227b,3969701293,0xa4e8182d,3971792230,0x21049e6d,3985379601,0xaf068a5a,3992708719,0xc5e9f609,3993982263,0xad58d3ce,3997389486,0x999c8205,4015062112,0x5603fd19,4030967171,0x6e4b275d,4033861990,0x35da7e79,4043539571,0x78fbe96b,4074379310,0x5987853d,407897388,0xf0b1e9b3,4089135994,0x45e64655,4092649321,0x6e425edd,4106157183,0x847d126e,4121205598,0x83dd4e76,4127642197,0x060b5dd9,4128059100,0x04bcb461,4129944370,0x8c750182,4135785646,0x59e4f211,4140427144,0x32cd82e1,4165601708,0x2bcf5168,4170728970,0x836fa5ed,4172831163,0x8d908351,4178784826,0xc996c2e9,4182772697,0x879574c2,4201794121,0xa9bb8861,4219585535,0x7901258d,4221382154,0xdad9a49f,4226676178,0xc88964c5,4229566114,0xe340e086,4234506345,0x9ebdaa4c,4234718147,0x8d17595f,4259169456,0x4437570a,4260692804,0x0e1bab2b,4265463744,0xb52de9f9,4267434510,0xb2972d36,4271368865,0x421ccbdc,4276174586,0xbc7389bd,4280094245,0x97b2deb4,4287327845,0xa206700f,429680985,0x1e37b9d3,44128826,0x93269172,445850678,0x9f0488fe,449189269,0x1b9c14c4,44966965,0xb92bcf9a,44975391,0xec69ed31,45276234,0xab7ee389,454163719,0x7f7d12eb,454324056,0x3ed4373c,45876839,0x4a278879,46603899,0xf7bfdb93,487631302,0xc5b90c80,500339770,0x6cd70d96,523107054,0xf366a0bd,525648079,0x26bfbead,540716452,0xfb1f386a,554041592,0x5d8f46c1,559266783,0x0664971e,565344768,0x60c729d3,570662527,0x81f28000,571853634,0xba424efe,577144012,0x8e6ff49f,579516840,0x5947fa66,584265466,0x1536edde,584764314,0x5848c075,597969040,0xf32bd5d4,59905512,0x4b1622c8,601648908,0x754daf9f,605449596,0x47f4c0e2,619473234,0x42baf164,622042688,0xd8927c64,626251567,0x975b52db,632570915,0xa5f5aed7,633079222,0x71afefbb,639282337,0x2a6e88c5,66646874,0x350236a8,678441573,0x2da020a7,682141153,0x144479ea,690493317,0x6ce27815,694540230,0xc32c01ee,70727143,0xc919c75f,711094671,0xca074b9f,717060064,0x8bfd9414,718629062,0xef0a866a,723908044,0x595194e8,735546514,0x9f5bf5e7,750653880,0x7bb7d834,755005670,0xa8810dca,766049103,0xa3c26a27,771357632,0x2bf2bfa3,772824033,0x4083454e,774319719,0x0059aae8,779408426,0xc0cd8a7f,782408967,0x7b7ba39e,784206734,0xacfc460d,792334001,0x3b48064c,797387577,0x3e4f52e6,805441109,0x782c2165,808719777,0xdae8ff9a,810709137,0x1ed6d88c,823673326,0x4d7068c2,836679309,0x22008f4a,837653176,0x49cb2b7f,837667204,0x907adc44,851072772,0xb692e371,852480808,0xfa5c61d9,854420948,0x325137af,858302191,0xb30d16ad,8592007,0x90da6b48,859996013,0x11e0cf95,864140626,0x7ca4ffb4,874580969,0xc2cb5826,876507772,0x69c884cb,884157545,0x7d12a804,889492032,0x467ee8b3,890643527,0x8b8a196a,915027338,0x79333e04,916331450,0xd93aea04,919247258,0x26acfebb,926509343,0x4add63de,928165852,0xebf2367a,94134284,0x297ac38f,960666893,0xf3610321,971644082,0x59613b7f,971801201,0x093d1f93,973939047,0xef9ceab3,976159134,0x89ca3c5d,980629983,0xe5293f22,984942515,0xc30ec0c8,98506835,0x6d337dd5,990032374,0xc3b32918,991886639,0xd7a55949,996758819,0x19e1da3a,99696691,0x5498e0e8,999512680]},
	/*{type:"bool",id:"xxxx",hashes:[
	]},*/
	{type:"bool",hashes:[
0x65cc1dcc,"3Relic_ZeldaVoice",
0x65908e33,"Access_AllTerminalElectric",
0xb840c3e2,"Access_AllTerminalFire",
0xbe0cd665,"Access_AllTerminalWater",
0xc1959ed3,"Access_AllTerminalWind",
0xdec8c8e7,"Active_RemainsElectric",
0x8f45157e,"Active_RemainsFire",
0x001cd869,"Active_RemainsFireBefore",
0x47820cdc,"Active_RemainsWater",
0xf690484f,"Active_RemainsWind",
0x4f6383ed,"AfterTalk_WaterRelic",
0xa9054109,"AlwaysSleepingBed",
0xfa130271,"AncientLabo_AncientAssistant001_Are",
0xeabfabe7,"AncientLabo_AncientAssistant001_ExplainStudy",
0xe6d8e71c,"AncientLabo_AncientAssistant001_First",
0xc5b553da,"AncientLabo_AncientAssistant001_FullCompFirst",
0x8e2092cd,"AncientLabo_AncientAssistant001_GetOut",
0x6eaccc6b,"AncientLabo_AncientAssistant001_Intro",
0x31904b59,"AncientLabo_AncientAssistant001_IrregularFirst",
0x39da26b0,"AncientLabo_AncientAssistant001_SSLv2Get",
0xb78768eb,"AncientLabo_AncientDoctor001_2nd",
0x660531dd,"AncientLabo_AncientDoctor001_Damnation",
0x66645be4,"AncientLabo_AncientDoctor001_Diary",
0x656e5851,"AncientLabo_AncientDoctor001_First",
0xb001638c,"AncientLabo_AncientDoctor001_First_PowUP",
0x3a1b9d69,"AncientLabo_AncientDoctor001_GetSSLv1",
0x259add49,"AncientLabo_AncientDoctor001_Irregular",
0xaa0c6f7b,"AncientLabo_AncientDoctor001_NoCheckIt",
0xe4f9492e,"AncientLabo_AncientDoctor001_Rikona",
0xba9493ae,"AncientLabo_AncientDoctor001_Tripper",
0x000fd871,"AncientLabo_AncientDoctor001_Wait",
0xc91d803e,"AncientLabo_AncientDoctor001_Yoro",
0x3c87dc70,"AncientLabo_AncientDoctor002_Clear_First",
0xaf03268d,"AncientLabo_AncientDoctor002_ExplainWife",
0xa04bde0c,"AncientLabo_AncientDoctor002_Explain_First",
0x548642cc,"AncientLabo_AncientDoctor002_First",
0x4a6b665b,"AncientLabo_AncientDoctor002_Naked_First",
0x8684b49f,"AncientLabo_AncientDoctor_Irregular",
0x47327ec5,"AncientLabo_AncientDoctor_MoveA1",
0x65620ae8,"AncientLabo_AncientOven_First",
0x1c24f2ae,"AncientLabo_NPC002_First",
0x4d59d839,"AncientLabo_NPC002_ReadyFirst",
0x42e966d3,"AncientLabo_NPC002_Roberry",
0x1caf96e9,"AncientLabo_NPC002_Son",
0x4d330277,"Animal_Forest_Act",
0x9ca7a7d2,"Animal_Forest_Activated",
0x4b75080c,"Animal_Forest_Finish",
0xff2765ff,"Animal_Forest_Ready",
0xba83e59a,"Animal_Forest_step1",
0xcd357720,"App_SeakSensor_IsEnabled",
0xd6fb97fb,"App_Slowtimer_OnOff",
0x7217a3ac,"Appear_PGanonElectric",
0xf03e90fc,"Appear_PGanonFire",
0x444b9555,"Appear_PGanonWater",
0x89ebcdcd,"Appear_PGanonWind",
0x9758f9a8,"Armor_Check",
0xf81d1874,"Armor_Get",
0x6602414c,"ArrivalZoraVillage",
0x94ca6a29,"Artist_FirstChallenge",
0xbf698e74,"Artist_FirstFirstFirstTalk",
0xa156e993,"Artist_FirstFirstTalk",
0x6b8f98bd,"Artist_FirstTalk",
0x7cafbf30,"Artist_Found10",
0xa33d67bf,"AssassinBoss_Create",
0xa7cb8a6a,"AssassinBoss_TreasureBoxCreate",
0xcac3867b,"AttackedNPC_Set1_FirstTalk",
0x447717a7,"AttackedNPC_Set1_Saved1",
0xdd7e461d,"AttackedNPC_Set1_Saved2",
0xaa79768b,"AttackedNPC_Set1_Saved3",
0x5321e07a,"AttackedNPC_Set2_FirstTalk",
0xcaf81044,"AttackedNPC_Set2_Saved1",
0x53f141fe,"AttackedNPC_Set2_Saved2",
0x24f67168,"AttackedNPC_Set2_Saved3",
0x92af3fba,"AttackedNPC_Set3_FirstTalk",
0x065210da,"AttackedNPC_Set3_Saved1",
0x9f5b4160,"AttackedNPC_Set3_Saved2",
0xe85c71f6,"AttackedNPC_Set3_Saved3",
0xbb942a39,"AttackedNPC_Set4_FirstTalk",
0x0c9719c3,"AttackedNPC_Set4_Saved1",
0x959e4879,"AttackedNPC_Set4_Saved2",
0xe29978ef,"AttackedNPC_Set4_Saved3",
0x7a1af5f9,"AttackedNPC_Set5_FirstTalk",
0xc03d195d,"AttackedNPC_Set5_Saved1",
0x593448e7,"AttackedNPC_Set5_Saved2",
0x2e337871,"AttackedNPC_Set5_Saved3",
0xb057edd2,"AttackedNPC_Set5_Saved4",
0xe3f893f8,"AttackedNPC_Set6_FirstTalk",
0x4eb21ebe,"AttackedNPC_Set6_Saved1",
0xd7bb4f04,"AttackedNPC_Set6_Saved2",
0xa0bc7f92,"AttackedNPC_Set6_Saved3",
0x22764c38,"AttackedNPC_Set7_FirstTalk",
0x82181e20,"AttackedNPC_Set7_Saved1",
0xb18eb8fe,"AttackedNPC_Set8_FirstTalk",
0x4061efae,"AttackedNPC_Set8_FirstTalk2",
0x5b380c8c,"AttackedNPC_Set8_Saved1",
0xafdfdb9a,"B6VarettaGenFlag",
0x43627a90,"BarrelErrand_Activated",
0x6b541d3e,"BarrelErrand_Finish_Finished",
0x1fa1b8bf,"BarrelErrand_Intro_Finished",
0x9936e13e,"BarrelErrand_Ready",
0xc82f231f,"BattleArena_Level1",
0x512672a5,"BattleArena_Level2",
0x26214233,"BattleArena_Level3",
0xb845d790,"BattleArena_Level4",
0xcfd03855,"BigGoddess_Camera_First",
0x10a832e0,"BigWhales_Ready_Gamyo",
0x4b69918c,"BigWhales_Ready_Negyui",
0xf9fac151,"BigWhales_Ready_Orak",
0x4b08b969,"BloodyMoonReliefFirstTime",
0xe9d6c31a,"BloodyMoonRelief_Activated",
0xb615ef36,"BloodyMoonRelief_Finish",
0x12a18623,"BloodyMoonRelief_NPC_AfterTalk",
0xa2215aff,"BloodyMoonRelief_NPC_Talk",
0xfd3925e5,"BloodyMoonRelief_Ready",
0x5d5f25ac,"BoguQuest2_1_3",
0x0c2b7c41,"BoguQuest_1_3",
0xf2dfa241,"Boko_Away",
0xe6294c09,"Boko_Away_Orei",
0x652507a8,"Bokoblin_Complete",
0xa2b25731,"Bottle_Mes_Activated",
0x50bf3d33,"Bottle_Mes_Demo30",
0x5b7634a6,"Bottle_Mes_Finish",
0x50b0f2c2,"Bottle_Mes_Go",
0x3083dd16,"Bottle_Mes_Item",
0x26619565,"Bottle_Mes_Ready",
0x918d4545,"Bottle_Mes_Start",
0xf3141f02,"Bottle_Mes_Step10",
0xc1227d80,"Bottle_Mes_Step30",
0x8e63eb47,"Bottle_Mes_Step40",
0x5c0e7062,"Bottle_Mes_Think",
0xa747c99d,"CDungeon087_NetaClear",
0x7e9a4c88,"CDungeon114_NetaClear",
0x42b2de81,"Carnivorous_Boy_Activated",
0xb49816a6,"Carnivorous_Boy_Finish",
0x1d53bf26,"Carnivorous_Boy_Ready",
0xcda3fa17,"CarryingBlueFireEXMini_Activated",
0xcfd4b174,"CarryingBlueFireEXMini_Camera",
0xe4652422,"CarryingBlueFireEXMini_Carry",
0xd401b469,"CarryingBlueFireEXMini_Finish",
0x8fb46bf2,"CarryingBlueFireEXMini_Fired",
0x7d945e81,"CarryingBlueFireEXMini_Permit",
0x347fa5b9,"CarryingBlueFireEXMini_Ready",
0x7357b7f4,"CarryingBlueFireEXMini_Repaired",
0x7e82e443,"CastleHarbor_DungeonAppear",
0xbaac25e7,"Challenge_Learning_Konishi2_Activated",
0x87d9383f,"Challenge_Learning_Konishi2_Finish",
0x5f60c34b,"Challenge_Learning_Konishi2_Ready",
0x844d4fd3,"Challenge_Learning_Konishi_Activated",
0xf1084677,"Challenge_Learning_Konishi_Finish",
0x525f1b0b,"Challenge_Learning_Konishi_Ready",
0xad0046f3,"ChampionshipBallGenFlg",
0x4b402325,"ClearTutorial_DungeonAll",
0x9d59003e,"Clear_NishibeTest",
0x18d100e9,"Clear_RemainsElectric",
0x4007e1ee,"Clear_RemainsFire",
0xb742dd6c,"Clear_RemainsWater",
0x39d2bcdf,"Clear_RemainsWind",
0xb32a60b6,"Cliff_Maze_Activated",
0x64bfcd8e,"Cliff_Maze_Finish",
0x5a9e60c3,"Cliff_Maze_Ready",
0x73cb413e,"CompleteDungeon_Activated",
0xe622ef03,"CompleteDungeon_Finish",
0xcf482c95,"CompleteDungeon_Ready",
0x5392b522,"CompleteTreasure_Dungeon000",
0x249585b4,"CompleteTreasure_Dungeon001",
0xbd9cd40e,"CompleteTreasure_Dungeon002",
0xca9be498,"CompleteTreasure_Dungeon003",
0x54ff713b,"CompleteTreasure_Dungeon004",
0x23f841ad,"CompleteTreasure_Dungeon005",
0xbaf11017,"CompleteTreasure_Dungeon006",
0xcdf62081,"CompleteTreasure_Dungeon007",
0x5d493d10,"CompleteTreasure_Dungeon008",
0x2a4e0d86,"CompleteTreasure_Dungeon009",
0x4a898463,"CompleteTreasure_Dungeon010",
0x3d8eb4f5,"CompleteTreasure_Dungeon011",
0xa487e54f,"CompleteTreasure_Dungeon012",
0xd380d5d9,"CompleteTreasure_Dungeon013",
0x4de4407a,"CompleteTreasure_Dungeon014",
0x3ae370ec,"CompleteTreasure_Dungeon015",
0xa3ea2156,"CompleteTreasure_Dungeon016",
0xd4ed11c0,"CompleteTreasure_Dungeon017",
0x44520c51,"CompleteTreasure_Dungeon018",
0x33553cc7,"CompleteTreasure_Dungeon019",
0x61a4d7a0,"CompleteTreasure_Dungeon020",
0x16a3e736,"CompleteTreasure_Dungeon021",
0x8faab68c,"CompleteTreasure_Dungeon022",
0xf8ad861a,"CompleteTreasure_Dungeon023",
0x66c913b9,"CompleteTreasure_Dungeon024",
0x11ce232f,"CompleteTreasure_Dungeon025",
0x88c77295,"CompleteTreasure_Dungeon026",
0xffc04203,"CompleteTreasure_Dungeon027",
0x6f7f5f92,"CompleteTreasure_Dungeon028",
0x18786f04,"CompleteTreasure_Dungeon029",
0x78bfe6e1,"CompleteTreasure_Dungeon030",
0x0fb8d677,"CompleteTreasure_Dungeon031",
0x96b187cd,"CompleteTreasure_Dungeon032",
0xe1b6b75b,"CompleteTreasure_Dungeon033",
0x7fd222f8,"CompleteTreasure_Dungeon034",
0x08d5126e,"CompleteTreasure_Dungeon035",
0x91dc43d4,"CompleteTreasure_Dungeon036",
0xe6db7342,"CompleteTreasure_Dungeon037",
0x76646ed3,"CompleteTreasure_Dungeon038",
0x01635e45,"CompleteTreasure_Dungeon039",
0x37fe7026,"CompleteTreasure_Dungeon040",
0x40f940b0,"CompleteTreasure_Dungeon041",
0xd9f0110a,"CompleteTreasure_Dungeon042",
0xaef7219c,"CompleteTreasure_Dungeon043",
0x3093b43f,"CompleteTreasure_Dungeon044",
0x479484a9,"CompleteTreasure_Dungeon045",
0xde9dd513,"CompleteTreasure_Dungeon046",
0xa99ae585,"CompleteTreasure_Dungeon047",
0x3925f814,"CompleteTreasure_Dungeon048",
0x4e22c882,"CompleteTreasure_Dungeon049",
0x2ee54167,"CompleteTreasure_Dungeon050",
0x59e271f1,"CompleteTreasure_Dungeon051",
0xc0eb204b,"CompleteTreasure_Dungeon052",
0xb7ec10dd,"CompleteTreasure_Dungeon053",
0x2988857e,"CompleteTreasure_Dungeon054",
0x5e8fb5e8,"CompleteTreasure_Dungeon055",
0xc786e452,"CompleteTreasure_Dungeon056",
0xb081d4c4,"CompleteTreasure_Dungeon057",
0x203ec955,"CompleteTreasure_Dungeon058",
0x5739f9c3,"CompleteTreasure_Dungeon059",
0x05c812a4,"CompleteTreasure_Dungeon060",
0x72cf2232,"CompleteTreasure_Dungeon061",
0xebc67388,"CompleteTreasure_Dungeon062",
0x9cc1431e,"CompleteTreasure_Dungeon063",
0x02a5d6bd,"CompleteTreasure_Dungeon064",
0x75a2e62b,"CompleteTreasure_Dungeon065",
0xecabb791,"CompleteTreasure_Dungeon066",
0x9bac8707,"CompleteTreasure_Dungeon067",
0x0b139a96,"CompleteTreasure_Dungeon068",
0x7c14aa00,"CompleteTreasure_Dungeon069",
0x1cd323e5,"CompleteTreasure_Dungeon070",
0x6bd41373,"CompleteTreasure_Dungeon071",
0xf2dd42c9,"CompleteTreasure_Dungeon072",
0x85da725f,"CompleteTreasure_Dungeon073",
0x1bbee7fc,"CompleteTreasure_Dungeon074",
0x6cb9d76a,"CompleteTreasure_Dungeon075",
0xf5b086d0,"CompleteTreasure_Dungeon076",
0x82b7b646,"CompleteTreasure_Dungeon077",
0x1208abd7,"CompleteTreasure_Dungeon078",
0x650f9b41,"CompleteTreasure_Dungeon079",
0x9b4b3f2a,"CompleteTreasure_Dungeon080",
0xec4c0fbc,"CompleteTreasure_Dungeon081",
0x75455e06,"CompleteTreasure_Dungeon082",
0x02426e90,"CompleteTreasure_Dungeon083",
0x9c26fb33,"CompleteTreasure_Dungeon084",
0xeb21cba5,"CompleteTreasure_Dungeon085",
0x72289a1f,"CompleteTreasure_Dungeon086",
0x052faa89,"CompleteTreasure_Dungeon087",
0x9590b718,"CompleteTreasure_Dungeon088",
0xe297878e,"CompleteTreasure_Dungeon089",
0x82500e6b,"CompleteTreasure_Dungeon090",
0xf5573efd,"CompleteTreasure_Dungeon091",
0x6c5e6f47,"CompleteTreasure_Dungeon092",
0x1b595fd1,"CompleteTreasure_Dungeon093",
0x853dca72,"CompleteTreasure_Dungeon094",
0xf23afae4,"CompleteTreasure_Dungeon095",
0x6b33ab5e,"CompleteTreasure_Dungeon096",
0x1c349bc8,"CompleteTreasure_Dungeon097",
0x8c8b8659,"CompleteTreasure_Dungeon098",
0xfb8cb6cf,"CompleteTreasure_Dungeon099",
0x5250df15,"CompleteTreasure_Dungeon100",
0x2557ef83,"CompleteTreasure_Dungeon101",
0xbc5ebe39,"CompleteTreasure_Dungeon102",
0xcb598eaf,"CompleteTreasure_Dungeon103",
0x553d1b0c,"CompleteTreasure_Dungeon104",
0x223a2b9a,"CompleteTreasure_Dungeon105",
0xbb337a20,"CompleteTreasure_Dungeon106",
0xcc344ab6,"CompleteTreasure_Dungeon107",
0x5c8b5727,"CompleteTreasure_Dungeon108",
0x2b8c67b1,"CompleteTreasure_Dungeon109",
0x4b4bee54,"CompleteTreasure_Dungeon110",
0x3c4cdec2,"CompleteTreasure_Dungeon111",
0xa5458f78,"CompleteTreasure_Dungeon112",
0xd242bfee,"CompleteTreasure_Dungeon113",
0x4c262a4d,"CompleteTreasure_Dungeon114",
0x3b211adb,"CompleteTreasure_Dungeon115",
0xa2284b61,"CompleteTreasure_Dungeon116",
0xd52f7bf7,"CompleteTreasure_Dungeon117",
0x45906666,"CompleteTreasure_Dungeon118",
0x329756f0,"CompleteTreasure_Dungeon119",
0x85074c8a,"CourageFirstDemoKill",
0xca55f5d9,"CrackWall_Thorn_Delete",
0xa416de05,"CursedStatue_Activated",
0x27db87fc,"CursedStatue_Finish",
0xe1db8ec0,"CursedStatue_Game",
0x573d78ab,"CursedStatue_Ready",
0xb223f793,"CursedStatue_StatueBroken",
0x8d7f7020,"D-5_Animal_RupeeRabbit_A_1355702776",
0xbbb9140f,"D-5_Animal_RupeeRabbit_A_2206925384",
0xd1fe54d3,"D-5_Animal_RupeeRabbit_A_2462742580",
0x520f3867,"D-5_Animal_RupeeRabbit_A_3491694435",
0x13ceaad6,"DarkWoods_Activated",
0x00ab93c5,"DarkWoods_Dungeon",
0xc44363be,"DarkWoods_Finish",
0xb77401da,"DarkWoods_Giant_Clear",
0x4037a397,"DarkWoods_Giant_Dead",
0xacd22823,"DarkWoods_Ready",
0xde927902,"DarkWoods_SetBall",
0x872d048c,"Demo004_0",
0x37893d6c,"Demo004_0_1st",
0xf930cd7e,"Demo008_1",
0x60399cc4,"Demo008_2",
0x3898cb35,"Demo010_0",
0x4f9ffba3,"Demo010_1",
0x9598ab62,"Demo034_0",
0x945ac155,"Demo035_0",
0x961c7f0c,"Demo036_0",
0x0cc2ef69,"Demo042_0",
0x7bc5dfff,"Demo042_1",
0xbd4485e0,"Demo100_0",
0xca43b576,"Demo100_1",
0xbc86efd7,"Demo101_0",
0xbec0518e,"Demo102_0",
0xbf023bb9,"Demo103_0",
0xba4d2d3c,"Demo104_0",
0xbb8f470b,"Demo105_0",
0xb9c9f952,"Demo106_0",
0xb80b9365,"Demo107_0",
0xb357d458,"Demo108_0",
0xb295be6f,"Demo109_0",
0xc5928ef9,"Demo109_1",
0x05f8e285,"Demo110_0",
0x043a88b2,"Demo111_0",
0x067c36eb,"Demo112_0",
0x07be5cdc,"Demo113_0",
0x02f14a59,"Demo114_0",
0x75f67acf,"Demo114_1",
0x0333206e,"Demo115_0",
0x743410f8,"Demo115_1",
0xed3d4142,"Demo115_2",
0x01759e37,"Demo116_0",
0x00b7f400,"Demo117_0",
0x77b0c496,"Demo117_1",
0x0bebb33d,"Demo118_0",
0x7cec83ab,"Demo118_1",
0xe5e5d211,"Demo118_2",
0x0a29d90a,"Demo119_0",
0x174d4d6b,"Demo120_0",
0x604a7dfd,"Demo120_1",
0x168f275c,"Demo121_0",
0x618817ca,"Demo121_1",
0xf8814670,"Demo121_2",
0x14c99905,"Demo122_0",
0x150bf332,"Demo123_0",
0x620cc3a4,"Demo123_1",
0x1044e5b7,"Demo124_0",
0x6743d521,"Demo124_1",
0xfe4a849b,"Demo124_2",
0x11868f80,"Demo125_0",
0x13c031d9,"Demo126_0",
0x12025bee,"Demo127_0",
0x195e1cd3,"Demo128_0",
0x189c76e4,"Demo129_0",
0xaff12a0e,"Demo130_0",
0xae334039,"Demo131_0",
0xac75fe60,"Demo132_0",
0xadb79457,"Demo133_0",
0xa8f882d2,"Demo134_0",
0xa93ae8e5,"Demo135_0",
0xab7c56bc,"Demo136_0",
0xaabe3c8b,"Demo137_0",
0xa1e27bb6,"Demo138_0",
0xa0201181,"Demo139_0",
0x322612b7,"Demo140_0",
0x33e47880,"Demo141_0",
0x31a2c6d9,"Demo142_0",
0x34edd05c,"Demo145_0",
0x36ab6e05,"Demo146_0",
0x37690432,"Demo147_0",
0x3c35430f,"Demo148_0",
0x3df72938,"Demo149_0",
0x8a9a75d2,"Demo150_0",
0x8b581fe5,"Demo151_0",
0x891ea1bc,"Demo152_0",
0x88dccb8b,"Demo153_0",
0x8d93dd0e,"Demo154_0",
0x8c51b739,"Demo155_0",
0x8e170960,"Demo156_0",
0x8fd56357,"Demo157_0",
0x8489246a,"Demo158_0",
0x854b4e5d,"Demo159_0",
0x982fda3c,"Demo160_0",
0x99edb00b,"Demo161_0",
0x9bab0e52,"Demo162_0",
0x9a696465,"Demo163_0",
0x9f2672e0,"Demo164_0",
0x9ee418d7,"Demo165_0",
0x9ca2a68e,"Demo166_0",
0x241ec1eb,"Demo176_0",
0xc855ed0f,"Demo309_0",
0xbf52dd99,"Demo309_1",
0x4fefe90b,"Demo344_0",
0xcfba3ccb,"Demo344_2_LatterHalf",
0x496c21b5,"Demo347_2RougeGenFlg",
0xf6f74340,"Demo995_0",
0xf573972e,"Demo997_0",
0xd9006169,"Demo_RitoTest_001",
0xccc6cba5,"Desert_Maze_Activated",
0x7484814c,"Desert_Maze_Finish",
0xbba8f859,"Desert_Maze_Ready",
0xc7e51aaf,"Die_PGanonElectric",
0x99589134,"Die_PGanonFire",
0xd19db9d6,"Die_PGanonWater",
0xe08dcc05,"Die_PGanonWind",
0x5c362296,"DokuroEye_Activated",
0x9083b0f6,"DokuroEye_Finish",
0xb25649d5,"DokuroEye_Ready",
0xb2c07f9e,"Domino_001",
0xb7f619d4,"DontDamageFlower_Activated",
0xee5d2ecd,"DontDamageFlower_Fail",
0x0f4c2aa2,"DontDamageFlower_Finish",
0x4b513408,"DontDamageFlower_Game",
0x4b43d0ca,"DontDamageFlower_OneMiss",
0xc228b893,"DontDamageFlower_Ready",
0x6a9a68a3,"DontDamageFlower_ReadyFail",
0x2eb45c34,"Drag_Hero_Activated",
0xfdaa2df8,"Drag_Hero_Debug01",
0x64e8546c,"Drag_Hero_Demo378_1_ElectlicRelic",
0x229b9cbe,"Drag_Hero_DragMerchant",
0x03463841,"Drag_Hero_Dressing",
0x5b725538,"Drag_Hero_Finish",
0xa14c6ef6,"Drag_Hero_Finish_first",
0x0c9de8bc,"Drag_Hero_FirstTime",
0x9cd6a641,"Drag_Hero_GerdoKeepOut",
0x22000b65,"Drag_Hero_Ready",
0x764a1ef8,"Drag_Hero_Rob",
0xa665a404,"Drag_Hero_Step1",
0x187c9e44,"Drag_Hero_rob_first",
0x3711e6d5,"Drag_Hero_warning01",
0xae18b76f,"Drag_Hero_warning02",
0x5a64ed23,"Dungeon028_1",
0xc36dbc99,"Dungeon028_2",
0xb46a8c0f,"Dungeon028_3",
0x2a0e19ac,"Dungeon028_4",
0x5d09293a,"Dungeon028_5",
0xc4007880,"Dungeon028_6",
0xb3074816,"Dungeon028_7",
0x5f9edb95,"Dungeon040_DoorStone_1",
0xc6978a2f,"Dungeon040_DoorStone_2",
0xb190bab9,"Dungeon040_DoorStone_3",
0x2ff42f1a,"Dungeon040_DoorStone_4",
0x58f31f8c,"Dungeon040_DoorStone_5",
0x8ba90826,"Dungeon109Test1",
0x12a0599c,"Dungeon109Test2",
0x737ca087,"Dungeon109_Appear",
0xab9a7862,"Dungeon_Clear_1",
0xdbf08ced,"Dungeon_Clear_4",
0x2727da68,"Dungeon000_SmallKey00",
0x5020eafe,"Dungeon000_SmallKey01",
0xc929bb44,"Dungeon000_SmallKey02",
0xbe2e8bd2,"Dungeon000_SmallKey03",
0x204a1e71,"Dungeon000_SmallKey04",
0x574d2ee7,"Dungeon000_SmallKey05",
0xce447f5d,"Dungeon000_SmallKey06",
0xb9434fcb,"Dungeon000_SmallKey07",
0x29fc525a,"Dungeon000_SmallKey08",
0x5efb62cc,"Dungeon000_SmallKey09",
0xbc829607,"Dungeon001_SmallKey00",
0xcb85a691,"Dungeon001_SmallKey01",
0x528cf72b,"Dungeon001_SmallKey02",
0x258bc7bd,"Dungeon001_SmallKey03",
0xbbef521e,"Dungeon001_SmallKey04",
0xcce86288,"Dungeon001_SmallKey05",
0x55e13332,"Dungeon001_SmallKey06",
0x22e603a4,"Dungeon001_SmallKey07",
0xb2591e35,"Dungeon001_SmallKey08",
0xc55e2ea3,"Dungeon001_SmallKey09",
0xcb1c44f7,"Dungeon002_SmallKey00",
0xbc1b7461,"Dungeon002_SmallKey01",
0x251225db,"Dungeon002_SmallKey02",
0x5215154d,"Dungeon002_SmallKey03",
0xcc7180ee,"Dungeon002_SmallKey04",
0xbb76b078,"Dungeon002_SmallKey05",
0x227fe1c2,"Dungeon002_SmallKey06",
0x5578d154,"Dungeon002_SmallKey07",
0xc5c7ccc5,"Dungeon002_SmallKey08",
0xb2c0fc53,"Dungeon002_SmallKey09",
0x50b90898,"Dungeon003_SmallKey00",
0x27be380e,"Dungeon003_SmallKey01",
0xbeb769b4,"Dungeon003_SmallKey02",
0xc9b05922,"Dungeon003_SmallKey03",
0x57d4cc81,"Dungeon003_SmallKey04",
0x20d3fc17,"Dungeon003_SmallKey05",
0xb9daadad,"Dungeon003_SmallKey06",
0xcedd9d3b,"Dungeon003_SmallKey07",
0x5e6280aa,"Dungeon003_SmallKey08",
0x2965b03c,"Dungeon003_SmallKey09",
0x2421e117,"Dungeon004_SmallKey00",
0x5326d181,"Dungeon004_SmallKey01",
0xca2f803b,"Dungeon004_SmallKey02",
0xbd28b0ad,"Dungeon004_SmallKey03",
0x234c250e,"Dungeon004_SmallKey04",
0x544b1598,"Dungeon004_SmallKey05",
0xcd424422,"Dungeon004_SmallKey06",
0xba4574b4,"Dungeon004_SmallKey07",
0x2afa6925,"Dungeon004_SmallKey08",
0x5dfd59b3,"Dungeon004_SmallKey09",
0xbf84ad78,"Dungeon005_SmallKey00",
0xc8839dee,"Dungeon005_SmallKey01",
0x518acc54,"Dungeon005_SmallKey02",
0x268dfcc2,"Dungeon005_SmallKey03",
0xb8e96961,"Dungeon005_SmallKey04",
0xcfee59f7,"Dungeon005_SmallKey05",
0x56e7084d,"Dungeon005_SmallKey06",
0x21e038db,"Dungeon005_SmallKey07",
0xb15f254a,"Dungeon005_SmallKey08",
0xc65815dc,"Dungeon005_SmallKey09",
0xc81a7f88,"Dungeon006_SmallKey00",
0xbf1d4f1e,"Dungeon006_SmallKey01",
0x26141ea4,"Dungeon006_SmallKey02",
0x51132e32,"Dungeon006_SmallKey03",
0xcf77bb91,"Dungeon006_SmallKey04",
0xb8708b07,"Dungeon006_SmallKey05",
0x2179dabd,"Dungeon006_SmallKey06",
0x567eea2b,"Dungeon006_SmallKey07",
0xc6c1f7ba,"Dungeon006_SmallKey08",
0xb1c6c72c,"Dungeon006_SmallKey09",
0x53bf33e7,"Dungeon007_SmallKey00",
0x24b80371,"Dungeon007_SmallKey01",
0xbdb152cb,"Dungeon007_SmallKey02",
0xcab6625d,"Dungeon007_SmallKey03",
0x54d2f7fe,"Dungeon007_SmallKey04",
0x23d5c768,"Dungeon007_SmallKey05",
0xbadc96d2,"Dungeon007_SmallKey06",
0xcddba644,"Dungeon007_SmallKey07",
0x5d64bbd5,"Dungeon007_SmallKey08",
0x2a638b43,"Dungeon007_SmallKey09",
0x212bac96,"Dungeon008_SmallKey00",
0x562c9c00,"Dungeon008_SmallKey01",
0xcf25cdba,"Dungeon008_SmallKey02",
0xb822fd2c,"Dungeon008_SmallKey03",
0x2646688f,"Dungeon008_SmallKey04",
0x51415819,"Dungeon008_SmallKey05",
0xc84809a3,"Dungeon008_SmallKey06",
0xbf4f3935,"Dungeon008_SmallKey07",
0x2ff024a4,"Dungeon008_SmallKey08",
0x58f71432,"Dungeon008_SmallKey09",
0xba8ee0f9,"Dungeon009_SmallKey00",
0xcd89d06f,"Dungeon009_SmallKey01",
0x548081d5,"Dungeon009_SmallKey02",
0x2387b143,"Dungeon009_SmallKey03",
0xbde324e0,"Dungeon009_SmallKey04",
0xcae41476,"Dungeon009_SmallKey05",
0x53ed45cc,"Dungeon009_SmallKey06",
0x24ea755a,"Dungeon009_SmallKey07",
0xb45568cb,"Dungeon009_SmallKey08",
0xc352585d,"Dungeon009_SmallKey09",
0xfab103ed,"Dungeon010_SmallKey00",
0x8db6337b,"Dungeon010_SmallKey01",
0x14bf62c1,"Dungeon010_SmallKey02",
0x63b85257,"Dungeon010_SmallKey03",
0xfddcc7f4,"Dungeon010_SmallKey04",
0x8adbf762,"Dungeon010_SmallKey05",
0x13d2a6d8,"Dungeon010_SmallKey06",
0x64d5964e,"Dungeon010_SmallKey07",
0xf46a8bdf,"Dungeon010_SmallKey08",
0x836dbb49,"Dungeon010_SmallKey09",
0x61144f82,"Dungeon011_SmallKey00",
0x16137f14,"Dungeon011_SmallKey01",
0x8f1a2eae,"Dungeon011_SmallKey02",
0xf81d1e38,"Dungeon011_SmallKey03",
0x66798b9b,"Dungeon011_SmallKey04",
0x117ebb0d,"Dungeon011_SmallKey05",
0x8877eab7,"Dungeon011_SmallKey06",
0xff70da21,"Dungeon011_SmallKey07",
0x6fcfc7b0,"Dungeon011_SmallKey08",
0x18c8f726,"Dungeon011_SmallKey09",
0x168a9d72,"Dungeon012_SmallKey00",
0x618dade4,"Dungeon012_SmallKey01",
0xf884fc5e,"Dungeon012_SmallKey02",
0x8f83ccc8,"Dungeon012_SmallKey03",
0x11e7596b,"Dungeon012_SmallKey04",
0x66e069fd,"Dungeon012_SmallKey05",
0xffe93847,"Dungeon012_SmallKey06",
0x88ee08d1,"Dungeon012_SmallKey07",
0x18511540,"Dungeon012_SmallKey08",
0x6f5625d6,"Dungeon012_SmallKey09",
0x8d2fd11d,"Dungeon013_SmallKey00",
0xfa28e18b,"Dungeon013_SmallKey01",
0x6321b031,"Dungeon013_SmallKey02",
0x142680a7,"Dungeon013_SmallKey03",
0x8a421504,"Dungeon013_SmallKey04",
0xfd452592,"Dungeon013_SmallKey05",
0x644c7428,"Dungeon013_SmallKey06",
0x134b44be,"Dungeon013_SmallKey07",
0x83f4592f,"Dungeon013_SmallKey08",
0xf4f369b9,"Dungeon013_SmallKey09",
0xf9b73892,"Dungeon014_SmallKey00",
0x8eb00804,"Dungeon014_SmallKey01",
0x17b959be,"Dungeon014_SmallKey02",
0x60be6928,"Dungeon014_SmallKey03",
0xfedafc8b,"Dungeon014_SmallKey04",
0x89ddcc1d,"Dungeon014_SmallKey05",
0x10d49da7,"Dungeon014_SmallKey06",
0x67d3ad31,"Dungeon014_SmallKey07",
0xf76cb0a0,"Dungeon014_SmallKey08",
0x806b8036,"Dungeon014_SmallKey09",
0x621274fd,"Dungeon015_SmallKey00",
0x1515446b,"Dungeon015_SmallKey01",
0x8c1c15d1,"Dungeon015_SmallKey02",
0xfb1b2547,"Dungeon015_SmallKey03",
0x657fb0e4,"Dungeon015_SmallKey04",
0x12788072,"Dungeon015_SmallKey05",
0x8b71d1c8,"Dungeon015_SmallKey06",
0xfc76e15e,"Dungeon015_SmallKey07",
0x6cc9fccf,"Dungeon015_SmallKey08",
0x1bcecc59,"Dungeon015_SmallKey09",
0x158ca60d,"Dungeon016_SmallKey00",
0x628b969b,"Dungeon016_SmallKey01",
0xfb82c721,"Dungeon016_SmallKey02",
0x8c85f7b7,"Dungeon016_SmallKey03",
0x12e16214,"Dungeon016_SmallKey04",
0x65e65282,"Dungeon016_SmallKey05",
0xfcef0338,"Dungeon016_SmallKey06",
0x8be833ae,"Dungeon016_SmallKey07",
0x1b572e3f,"Dungeon016_SmallKey08",
0x6c501ea9,"Dungeon016_SmallKey09",
0x8e29ea62,"Dungeon017_SmallKey00",
0xf92edaf4,"Dungeon017_SmallKey01",
0x60278b4e,"Dungeon017_SmallKey02",
0x1720bbd8,"Dungeon017_SmallKey03",
0x89442e7b,"Dungeon017_SmallKey04",
0xfe431eed,"Dungeon017_SmallKey05",
0x674a4f57,"Dungeon017_SmallKey06",
0x104d7fc1,"Dungeon017_SmallKey07",
0x80f26250,"Dungeon017_SmallKey08",
0xf7f552c6,"Dungeon017_SmallKey09",
0xfcbd7513,"Dungeon018_SmallKey00",
0x8bba4585,"Dungeon018_SmallKey01",
0x12b3143f,"Dungeon018_SmallKey02",
0x65b424a9,"Dungeon018_SmallKey03",
0xfbd0b10a,"Dungeon018_SmallKey04",
0x8cd7819c,"Dungeon018_SmallKey05",
0x15ded026,"Dungeon018_SmallKey06",
0x62d9e0b0,"Dungeon018_SmallKey07",
0xf266fd21,"Dungeon018_SmallKey08",
0x8561cdb7,"Dungeon018_SmallKey09",
0x6718397c,"Dungeon019_SmallKey00",
0x101f09ea,"Dungeon019_SmallKey01",
0x89165850,"Dungeon019_SmallKey02",
0xfe1168c6,"Dungeon019_SmallKey03",
0x6075fd65,"Dungeon019_SmallKey04",
0x1772cdf3,"Dungeon019_SmallKey05",
0x8e7b9c49,"Dungeon019_SmallKey06",
0xf97cacdf,"Dungeon019_SmallKey07",
0x69c3b14e,"Dungeon019_SmallKey08",
0x1ec481d8,"Dungeon019_SmallKey09",
0x477b6f23,"Dungeon020_SmallKey00",
0x307c5fb5,"Dungeon020_SmallKey01",
0xa9750e0f,"Dungeon020_SmallKey02",
0xde723e99,"Dungeon020_SmallKey03",
0x4016ab3a,"Dungeon020_SmallKey04",
0x37119bac,"Dungeon020_SmallKey05",
0xae18ca16,"Dungeon020_SmallKey06",
0xd91ffa80,"Dungeon020_SmallKey07",
0x49a0e711,"Dungeon020_SmallKey08",
0x3ea7d787,"Dungeon020_SmallKey09",
0xdcde234c,"Dungeon021_SmallKey00",
0xabd913da,"Dungeon021_SmallKey01",
0x32d04260,"Dungeon021_SmallKey02",
0x45d772f6,"Dungeon021_SmallKey03",
0xdbb3e755,"Dungeon021_SmallKey04",
0xacb4d7c3,"Dungeon021_SmallKey05",
0x35bd8679,"Dungeon021_SmallKey06",
0x42bab6ef,"Dungeon021_SmallKey07",
0xd205ab7e,"Dungeon021_SmallKey08",
0xa5029be8,"Dungeon021_SmallKey09",
0xab40f1bc,"Dungeon022_SmallKey00",
0xdc47c12a,"Dungeon022_SmallKey01",
0x454e9090,"Dungeon022_SmallKey02",
0x3249a006,"Dungeon022_SmallKey03",
0xac2d35a5,"Dungeon022_SmallKey04",
0xdb2a0533,"Dungeon022_SmallKey05",
0x42235489,"Dungeon022_SmallKey06",
0x3524641f,"Dungeon022_SmallKey07",
0xa59b798e,"Dungeon022_SmallKey08",
0xd29c4918,"Dungeon022_SmallKey09",
0x30e5bdd3,"Dungeon023_SmallKey00",
0x47e28d45,"Dungeon023_SmallKey01",
0xdeebdcff,"Dungeon023_SmallKey02",
0xa9ecec69,"Dungeon023_SmallKey03",
0x378879ca,"Dungeon023_SmallKey04",
0x408f495c,"Dungeon023_SmallKey05",
0xd98618e6,"Dungeon023_SmallKey06",
0xae812870,"Dungeon023_SmallKey07",
0x3e3e35e1,"Dungeon023_SmallKey08",
0x49390577,"Dungeon023_SmallKey09",
0x447d545c,"Dungeon024_SmallKey00",
0x337a64ca,"Dungeon024_SmallKey01",
0xaa733570,"Dungeon024_SmallKey02",
0xdd7405e6,"Dungeon024_SmallKey03",
0x43109045,"Dungeon024_SmallKey04",
0x3417a0d3,"Dungeon024_SmallKey05",
0xad1ef169,"Dungeon024_SmallKey06",
0xda19c1ff,"Dungeon024_SmallKey07",
0x4aa6dc6e,"Dungeon024_SmallKey08",
0x3da1ecf8,"Dungeon024_SmallKey09",
0xdfd81833,"Dungeon025_SmallKey00",
0xa8df28a5,"Dungeon025_SmallKey01",
0x31d6791f,"Dungeon025_SmallKey02",
0x46d14989,"Dungeon025_SmallKey03",
0xd8b5dc2a,"Dungeon025_SmallKey04",
0xafb2ecbc,"Dungeon025_SmallKey05",
0x36bbbd06,"Dungeon025_SmallKey06",
0x41bc8d90,"Dungeon025_SmallKey07",
0xd1039001,"Dungeon025_SmallKey08",
0xa604a097,"Dungeon025_SmallKey09",
0xa846cac3,"Dungeon026_SmallKey00",
0xdf41fa55,"Dungeon026_SmallKey01",
0x4648abef,"Dungeon026_SmallKey02",
0x314f9b79,"Dungeon026_SmallKey03",
0xaf2b0eda,"Dungeon026_SmallKey04",
0xd82c3e4c,"Dungeon026_SmallKey05",
0x41256ff6,"Dungeon026_SmallKey06",
0x36225f60,"Dungeon026_SmallKey07",
0xa69d42f1,"Dungeon026_SmallKey08",
0xd19a7267,"Dungeon026_SmallKey09",
0x33e386ac,"Dungeon027_SmallKey00",
0x44e4b63a,"Dungeon027_SmallKey01",
0xddede780,"Dungeon027_SmallKey02",
0xaaead716,"Dungeon027_SmallKey03",
0x348e42b5,"Dungeon027_SmallKey04",
0x43897223,"Dungeon027_SmallKey05",
0xda802399,"Dungeon027_SmallKey06",
0xad87130f,"Dungeon027_SmallKey07",
0x3d380e9e,"Dungeon027_SmallKey08",
0x4a3f3e08,"Dungeon027_SmallKey09",
0x417719dd,"Dungeon028_SmallKey00",
0x3670294b,"Dungeon028_SmallKey01",
0xaf7978f1,"Dungeon028_SmallKey02",
0xd87e4867,"Dungeon028_SmallKey03",
0x461addc4,"Dungeon028_SmallKey04",
0x311ded52,"Dungeon028_SmallKey05",
0xa814bce8,"Dungeon028_SmallKey06",
0xdf138c7e,"Dungeon028_SmallKey07",
0x4fac91ef,"Dungeon028_SmallKey08",
0x38aba179,"Dungeon028_SmallKey09",
0xdad255b2,"Dungeon029_SmallKey00",
0xadd56524,"Dungeon029_SmallKey01",
0x34dc349e,"Dungeon029_SmallKey02",
0x43db0408,"Dungeon029_SmallKey03",
0xddbf91ab,"Dungeon029_SmallKey04",
0xaab8a13d,"Dungeon029_SmallKey05",
0x33b1f087,"Dungeon029_SmallKey06",
0x44b6c011,"Dungeon029_SmallKey07",
0xd409dd80,"Dungeon029_SmallKey08",
0xa30eed16,"Dungeon029_SmallKey09",
0x9aedb6a6,"Dungeon030_SmallKey00",
0xedea8630,"Dungeon030_SmallKey01",
0x74e3d78a,"Dungeon030_SmallKey02",
0x03e4e71c,"Dungeon030_SmallKey03",
0x9d8072bf,"Dungeon030_SmallKey04",
0xea874229,"Dungeon030_SmallKey05",
0x738e1393,"Dungeon030_SmallKey06",
0x04892305,"Dungeon030_SmallKey07",
0x94363e94,"Dungeon030_SmallKey08",
0xe3310e02,"Dungeon030_SmallKey09",
0x0148fac9,"Dungeon031_SmallKey00",
0x764fca5f,"Dungeon031_SmallKey01",
0xef469be5,"Dungeon031_SmallKey02",
0x9841ab73,"Dungeon031_SmallKey03",
0x06253ed0,"Dungeon031_SmallKey04",
0x71220e46,"Dungeon031_SmallKey05",
0xe82b5ffc,"Dungeon031_SmallKey06",
0x9f2c6f6a,"Dungeon031_SmallKey07",
0x0f9372fb,"Dungeon031_SmallKey08",
0x7894426d,"Dungeon031_SmallKey09",
0x76d62839,"Dungeon032_SmallKey00",
0x01d118af,"Dungeon032_SmallKey01",
0x98d84915,"Dungeon032_SmallKey02",
0xefdf7983,"Dungeon032_SmallKey03",
0x71bbec20,"Dungeon032_SmallKey04",
0x06bcdcb6,"Dungeon032_SmallKey05",
0x9fb58d0c,"Dungeon032_SmallKey06",
0xe8b2bd9a,"Dungeon032_SmallKey07",
0x780da00b,"Dungeon032_SmallKey08",
0x0f0a909d,"Dungeon032_SmallKey09",
0xed736456,"Dungeon033_SmallKey00",
0x9a7454c0,"Dungeon033_SmallKey01",
0x037d057a,"Dungeon033_SmallKey02",
0x747a35ec,"Dungeon033_SmallKey03",
0xea1ea04f,"Dungeon033_SmallKey04",
0x9d1990d9,"Dungeon033_SmallKey05",
0x0410c163,"Dungeon033_SmallKey06",
0x7317f1f5,"Dungeon033_SmallKey07",
0xe3a8ec64,"Dungeon033_SmallKey08",
0x94afdcf2,"Dungeon033_SmallKey09",
0x99eb8dd9,"Dungeon034_SmallKey00",
0xeeecbd4f,"Dungeon034_SmallKey01",
0x77e5ecf5,"Dungeon034_SmallKey02",
0x00e2dc63,"Dungeon034_SmallKey03",
0x9e8649c0,"Dungeon034_SmallKey04",
0xe9817956,"Dungeon034_SmallKey05",
0x708828ec,"Dungeon034_SmallKey06",
0x078f187a,"Dungeon034_SmallKey07",
0x973005eb,"Dungeon034_SmallKey08",
0xe037357d,"Dungeon034_SmallKey09",
0x024ec1b6,"Dungeon035_SmallKey00",
0x7549f120,"Dungeon035_SmallKey01",
0xec40a09a,"Dungeon035_SmallKey02",
0x9b47900c,"Dungeon035_SmallKey03",
0x052305af,"Dungeon035_SmallKey04",
0x72243539,"Dungeon035_SmallKey05",
0xeb2d6483,"Dungeon035_SmallKey06",
0x9c2a5415,"Dungeon035_SmallKey07",
0x0c954984,"Dungeon035_SmallKey08",
0x7b927912,"Dungeon035_SmallKey09",
0x75d01346,"Dungeon036_SmallKey00",
0x02d723d0,"Dungeon036_SmallKey01",
0x9bde726a,"Dungeon036_SmallKey02",
0xecd942fc,"Dungeon036_SmallKey03",
0x72bdd75f,"Dungeon036_SmallKey04",
0x05bae7c9,"Dungeon036_SmallKey05",
0x9cb3b673,"Dungeon036_SmallKey06",
0xebb486e5,"Dungeon036_SmallKey07",
0x7b0b9b74,"Dungeon036_SmallKey08",
0x0c0cabe2,"Dungeon036_SmallKey09",
0xee755f29,"Dungeon037_SmallKey00",
0x99726fbf,"Dungeon037_SmallKey01",
0x007b3e05,"Dungeon037_SmallKey02",
0x777c0e93,"Dungeon037_SmallKey03",
0xe9189b30,"Dungeon037_SmallKey04",
0x9e1faba6,"Dungeon037_SmallKey05",
0x0716fa1c,"Dungeon037_SmallKey06",
0x7011ca8a,"Dungeon037_SmallKey07",
0xe0aed71b,"Dungeon037_SmallKey08",
0x97a9e78d,"Dungeon037_SmallKey09",
0x9ce1c058,"Dungeon038_SmallKey00",
0xebe6f0ce,"Dungeon038_SmallKey01",
0x72efa174,"Dungeon038_SmallKey02",
0x05e891e2,"Dungeon038_SmallKey03",
0x9b8c0441,"Dungeon038_SmallKey04",
0xec8b34d7,"Dungeon038_SmallKey05",
0x7582656d,"Dungeon038_SmallKey06",
0x028555fb,"Dungeon038_SmallKey07",
0x923a486a,"Dungeon038_SmallKey08",
0xe53d78fc,"Dungeon038_SmallKey09",
0x07448c37,"Dungeon039_SmallKey00",
0x7043bca1,"Dungeon039_SmallKey01",
0xe94aed1b,"Dungeon039_SmallKey02",
0x9e4ddd8d,"Dungeon039_SmallKey03",
0x0029482e,"Dungeon039_SmallKey04",
0x772e78b8,"Dungeon039_SmallKey05",
0xee272902,"Dungeon039_SmallKey06",
0x99201994,"Dungeon039_SmallKey07",
0x099f0405,"Dungeon039_SmallKey08",
0x7e983493,"Dungeon039_SmallKey09",
0xe79eb0fe,"Dungeon040_SmallKey00",
0x90998068,"Dungeon040_SmallKey01",
0x0990d1d2,"Dungeon040_SmallKey02",
0x7e97e144,"Dungeon040_SmallKey03",
0xe0f374e7,"Dungeon040_SmallKey04",
0x97f44471,"Dungeon040_SmallKey05",
0x0efd15cb,"Dungeon040_SmallKey06",
0x79fa255d,"Dungeon040_SmallKey07",
0xe94538cc,"Dungeon040_SmallKey08",
0x9e42085a,"Dungeon040_SmallKey09",
0x7c3bfc91,"Dungeon041_SmallKey00",
0x0b3ccc07,"Dungeon041_SmallKey01",
0x92359dbd,"Dungeon041_SmallKey02",
0xe532ad2b,"Dungeon041_SmallKey03",
0x7b563888,"Dungeon041_SmallKey04",
0x0c51081e,"Dungeon041_SmallKey05",
0x955859a4,"Dungeon041_SmallKey06",
0xe25f6932,"Dungeon041_SmallKey07",
0x72e074a3,"Dungeon041_SmallKey08",
0x05e74435,"Dungeon041_SmallKey09",
0x0ba52e61,"Dungeon042_SmallKey00",
0x7ca21ef7,"Dungeon042_SmallKey01",
0xe5ab4f4d,"Dungeon042_SmallKey02",
0x92ac7fdb,"Dungeon042_SmallKey03",
0x0cc8ea78,"Dungeon042_SmallKey04",
0x7bcfdaee,"Dungeon042_SmallKey05",
0xe2c68b54,"Dungeon042_SmallKey06",
0x95c1bbc2,"Dungeon042_SmallKey07",
0x057ea653,"Dungeon042_SmallKey08",
0x727996c5,"Dungeon042_SmallKey09",
0x9000620e,"Dungeon043_SmallKey00",
0xe7075298,"Dungeon043_SmallKey01",
0x7e0e0322,"Dungeon043_SmallKey02",
0x090933b4,"Dungeon043_SmallKey03",
0x976da617,"Dungeon043_SmallKey04",
0xe06a9681,"Dungeon043_SmallKey05",
0x7963c73b,"Dungeon043_SmallKey06",
0x0e64f7ad,"Dungeon043_SmallKey07",
0x9edbea3c,"Dungeon043_SmallKey08",
0xe9dcdaaa,"Dungeon043_SmallKey09",
0xe4988b81,"Dungeon044_SmallKey00",
0x939fbb17,"Dungeon044_SmallKey01",
0x0a96eaad,"Dungeon044_SmallKey02",
0x7d91da3b,"Dungeon044_SmallKey03",
0xe3f54f98,"Dungeon044_SmallKey04",
0x94f27f0e,"Dungeon044_SmallKey05",
0x0dfb2eb4,"Dungeon044_SmallKey06",
0x7afc1e22,"Dungeon044_SmallKey07",
0xea4303b3,"Dungeon044_SmallKey08",
0x9d443325,"Dungeon044_SmallKey09",
0x7f3dc7ee,"Dungeon045_SmallKey00",
0x083af778,"Dungeon045_SmallKey01",
0x9133a6c2,"Dungeon045_SmallKey02",
0xe6349654,"Dungeon045_SmallKey03",
0x785003f7,"Dungeon045_SmallKey04",
0x0f573361,"Dungeon045_SmallKey05",
0x965e62db,"Dungeon045_SmallKey06",
0xe159524d,"Dungeon045_SmallKey07",
0x71e64fdc,"Dungeon045_SmallKey08",
0x06e17f4a,"Dungeon045_SmallKey09",
0x08a3151e,"Dungeon046_SmallKey00",
0x7fa42588,"Dungeon046_SmallKey01",
0xe6ad7432,"Dungeon046_SmallKey02",
0x91aa44a4,"Dungeon046_SmallKey03",
0x0fced107,"Dungeon046_SmallKey04",
0x78c9e191,"Dungeon046_SmallKey05",
0xe1c0b02b,"Dungeon046_SmallKey06",
0x96c780bd,"Dungeon046_SmallKey07",
0x06789d2c,"Dungeon046_SmallKey08",
0x717fadba,"Dungeon046_SmallKey09",
0x93065971,"Dungeon047_SmallKey00",
0xe40169e7,"Dungeon047_SmallKey01",
0x7d08385d,"Dungeon047_SmallKey02",
0x0a0f08cb,"Dungeon047_SmallKey03",
0x946b9d68,"Dungeon047_SmallKey04",
0xe36cadfe,"Dungeon047_SmallKey05",
0x7a65fc44,"Dungeon047_SmallKey06",
0x0d62ccd2,"Dungeon047_SmallKey07",
0x9dddd143,"Dungeon047_SmallKey08",
0xeadae1d5,"Dungeon047_SmallKey09",
0xe192c600,"Dungeon048_SmallKey00",
0x9695f696,"Dungeon048_SmallKey01",
0x0f9ca72c,"Dungeon048_SmallKey02",
0x789b97ba,"Dungeon048_SmallKey03",
0xe6ff0219,"Dungeon048_SmallKey04",
0x91f8328f,"Dungeon048_SmallKey05",
0x08f16335,"Dungeon048_SmallKey06",
0x7ff653a3,"Dungeon048_SmallKey07",
0xef494e32,"Dungeon048_SmallKey08",
0x984e7ea4,"Dungeon048_SmallKey09",
0x7a378a6f,"Dungeon049_SmallKey00",
0x0d30baf9,"Dungeon049_SmallKey01",
0x9439eb43,"Dungeon049_SmallKey02",
0xe33edbd5,"Dungeon049_SmallKey03",
0x7d5a4e76,"Dungeon049_SmallKey04",
0x0a5d7ee0,"Dungeon049_SmallKey05",
0x93542f5a,"Dungeon049_SmallKey06",
0xe4531fcc,"Dungeon049_SmallKey07",
0x74ec025d,"Dungeon049_SmallKey08",
0x03eb32cb,"Dungeon049_SmallKey09",
0x3a08697b,"Dungeon050_SmallKey00",
0x4d0f59ed,"Dungeon050_SmallKey01",
0xd4060857,"Dungeon050_SmallKey02",
0xa30138c1,"Dungeon050_SmallKey03",
0x3d65ad62,"Dungeon050_SmallKey04",
0x4a629df4,"Dungeon050_SmallKey05",
0xd36bcc4e,"Dungeon050_SmallKey06",
0xa46cfcd8,"Dungeon050_SmallKey07",
0x34d3e149,"Dungeon050_SmallKey08",
0x43d4d1df,"Dungeon050_SmallKey09",
0xa1ad2514,"Dungeon051_SmallKey00",
0xd6aa1582,"Dungeon051_SmallKey01",
0x4fa34438,"Dungeon051_SmallKey02",
0x38a474ae,"Dungeon051_SmallKey03",
0xa6c0e10d,"Dungeon051_SmallKey04",
0xd1c7d19b,"Dungeon051_SmallKey05",
0x48ce8021,"Dungeon051_SmallKey06",
0x3fc9b0b7,"Dungeon051_SmallKey07",
0xaf76ad26,"Dungeon051_SmallKey08",
0xd8719db0,"Dungeon051_SmallKey09",
0xd633f7e4,"Dungeon052_SmallKey00",
0xa134c772,"Dungeon052_SmallKey01",
0x383d96c8,"Dungeon052_SmallKey02",
0x4f3aa65e,"Dungeon052_SmallKey03",
0xd15e33fd,"Dungeon052_SmallKey04",
0xa659036b,"Dungeon052_SmallKey05",
0x3f5052d1,"Dungeon052_SmallKey06",
0x48576247,"Dungeon052_SmallKey07",
0xd8e87fd6,"Dungeon052_SmallKey08",
0xafef4f40,"Dungeon052_SmallKey09",
0x4d96bb8b,"Dungeon053_SmallKey00",
0x3a918b1d,"Dungeon053_SmallKey01",
0xa398daa7,"Dungeon053_SmallKey02",
0xd49fea31,"Dungeon053_SmallKey03",
0x4afb7f92,"Dungeon053_SmallKey04",
0x3dfc4f04,"Dungeon053_SmallKey05",
0xa4f51ebe,"Dungeon053_SmallKey06",
0xd3f22e28,"Dungeon053_SmallKey07",
0x434d33b9,"Dungeon053_SmallKey08",
0x344a032f,"Dungeon053_SmallKey09",
0x390e5204,"Dungeon054_SmallKey00",
0x4e096292,"Dungeon054_SmallKey01",
0xd7003328,"Dungeon054_SmallKey02",
0xa00703be,"Dungeon054_SmallKey03",
0x3e63961d,"Dungeon054_SmallKey04",
0x4964a68b,"Dungeon054_SmallKey05",
0xd06df731,"Dungeon054_SmallKey06",
0xa76ac7a7,"Dungeon054_SmallKey07",
0x37d5da36,"Dungeon054_SmallKey08",
0x40d2eaa0,"Dungeon054_SmallKey09",
0xa2ab1e6b,"Dungeon055_SmallKey00",
0xd5ac2efd,"Dungeon055_SmallKey01",
0x4ca57f47,"Dungeon055_SmallKey02",
0x3ba24fd1,"Dungeon055_SmallKey03",
0xa5c6da72,"Dungeon055_SmallKey04",
0xd2c1eae4,"Dungeon055_SmallKey05",
0x4bc8bb5e,"Dungeon055_SmallKey06",
0x3ccf8bc8,"Dungeon055_SmallKey07",
0xac709659,"Dungeon055_SmallKey08",
0xdb77a6cf,"Dungeon055_SmallKey09",
0xd535cc9b,"Dungeon056_SmallKey00",
0xa232fc0d,"Dungeon056_SmallKey01",
0x3b3badb7,"Dungeon056_SmallKey02",
0x4c3c9d21,"Dungeon056_SmallKey03",
0xd2580882,"Dungeon056_SmallKey04",
0xa55f3814,"Dungeon056_SmallKey05",
0x3c5669ae,"Dungeon056_SmallKey06",
0x4b515938,"Dungeon056_SmallKey07",
0xdbee44a9,"Dungeon056_SmallKey08",
0xace9743f,"Dungeon056_SmallKey09",
0x4e9080f4,"Dungeon057_SmallKey00",
0x3997b062,"Dungeon057_SmallKey01",
0xa09ee1d8,"Dungeon057_SmallKey02",
0xd799d14e,"Dungeon057_SmallKey03",
0x49fd44ed,"Dungeon057_SmallKey04",
0x3efa747b,"Dungeon057_SmallKey05",
0xa7f325c1,"Dungeon057_SmallKey06",
0xd0f41557,"Dungeon057_SmallKey07",
0x404b08c6,"Dungeon057_SmallKey08",
0x374c3850,"Dungeon057_SmallKey09",
0x3c041f85,"Dungeon058_SmallKey00",
0x4b032f13,"Dungeon058_SmallKey01",
0xd20a7ea9,"Dungeon058_SmallKey02",
0xa50d4e3f,"Dungeon058_SmallKey03",
0x3b69db9c,"Dungeon058_SmallKey04",
0x4c6eeb0a,"Dungeon058_SmallKey05",
0xd567bab0,"Dungeon058_SmallKey06",
0xa2608a26,"Dungeon058_SmallKey07",
0x32df97b7,"Dungeon058_SmallKey08",
0x45d8a721,"Dungeon058_SmallKey09",
0xa7a153ea,"Dungeon059_SmallKey00",
0xd0a6637c,"Dungeon059_SmallKey01",
0x49af32c6,"Dungeon059_SmallKey02",
0x3ea80250,"Dungeon059_SmallKey03",
0xa0cc97f3,"Dungeon059_SmallKey04",
0xd7cba765,"Dungeon059_SmallKey05",
0x4ec2f6df,"Dungeon059_SmallKey06",
0x39c5c649,"Dungeon059_SmallKey07",
0xa97adbd8,"Dungeon059_SmallKey08",
0xde7deb4e,"Dungeon059_SmallKey09",
0x87c205b5,"Dungeon060_SmallKey00",
0xf0c53523,"Dungeon060_SmallKey01",
0x69cc6499,"Dungeon060_SmallKey02",
0x1ecb540f,"Dungeon060_SmallKey03",
0x80afc1ac,"Dungeon060_SmallKey04",
0xf7a8f13a,"Dungeon060_SmallKey05",
0x6ea1a080,"Dungeon060_SmallKey06",
0x19a69016,"Dungeon060_SmallKey07",
0x89198d87,"Dungeon060_SmallKey08",
0xfe1ebd11,"Dungeon060_SmallKey09",
0x1c6749da,"Dungeon061_SmallKey00",
0x6b60794c,"Dungeon061_SmallKey01",
0xf26928f6,"Dungeon061_SmallKey02",
0x856e1860,"Dungeon061_SmallKey03",
0x1b0a8dc3,"Dungeon061_SmallKey04",
0x6c0dbd55,"Dungeon061_SmallKey05",
0xf504ecef,"Dungeon061_SmallKey06",
0x8203dc79,"Dungeon061_SmallKey07",
0x12bcc1e8,"Dungeon061_SmallKey08",
0x65bbf17e,"Dungeon061_SmallKey09",
0x6bf99b2a,"Dungeon062_SmallKey00",
0x1cfeabbc,"Dungeon062_SmallKey01",
0x85f7fa06,"Dungeon062_SmallKey02",
0xf2f0ca90,"Dungeon062_SmallKey03",
0x6c945f33,"Dungeon062_SmallKey04",
0x1b936fa5,"Dungeon062_SmallKey05",
0x829a3e1f,"Dungeon062_SmallKey06",
0xf59d0e89,"Dungeon062_SmallKey07",
0x65221318,"Dungeon062_SmallKey08",
0x1225238e,"Dungeon062_SmallKey09",
0xf05cd745,"Dungeon063_SmallKey00",
0x875be7d3,"Dungeon063_SmallKey01",
0x1e52b669,"Dungeon063_SmallKey02",
0x695586ff,"Dungeon063_SmallKey03",
0xf731135c,"Dungeon063_SmallKey04",
0x803623ca,"Dungeon063_SmallKey05",
0x193f7270,"Dungeon063_SmallKey06",
0x6e3842e6,"Dungeon063_SmallKey07",
0xfe875f77,"Dungeon063_SmallKey08",
0x89806fe1,"Dungeon063_SmallKey09",
0x84c43eca,"Dungeon064_SmallKey00",
0xf3c30e5c,"Dungeon064_SmallKey01",
0x6aca5fe6,"Dungeon064_SmallKey02",
0x1dcd6f70,"Dungeon064_SmallKey03",
0x83a9fad3,"Dungeon064_SmallKey04",
0xf4aeca45,"Dungeon064_SmallKey05",
0x6da79bff,"Dungeon064_SmallKey06",
0x1aa0ab69,"Dungeon064_SmallKey07",
0x8a1fb6f8,"Dungeon064_SmallKey08",
0xfd18866e,"Dungeon064_SmallKey09",
0x1f6172a5,"Dungeon065_SmallKey00",
0x68664233,"Dungeon065_SmallKey01",
0xf16f1389,"Dungeon065_SmallKey02",
0x8668231f,"Dungeon065_SmallKey03",
0x180cb6bc,"Dungeon065_SmallKey04",
0x6f0b862a,"Dungeon065_SmallKey05",
0xf602d790,"Dungeon065_SmallKey06",
0x8105e706,"Dungeon065_SmallKey07",
0x11bafa97,"Dungeon065_SmallKey08",
0x66bdca01,"Dungeon065_SmallKey09",
0x68ffa055,"Dungeon066_SmallKey00",
0x1ff890c3,"Dungeon066_SmallKey01",
0x86f1c179,"Dungeon066_SmallKey02",
0xf1f6f1ef,"Dungeon066_SmallKey03",
0x6f92644c,"Dungeon066_SmallKey04",
0x189554da,"Dungeon066_SmallKey05",
0x819c0560,"Dungeon066_SmallKey06",
0xf69b35f6,"Dungeon066_SmallKey07",
0x66242867,"Dungeon066_SmallKey08",
0x112318f1,"Dungeon066_SmallKey09",
0xf35aec3a,"Dungeon067_SmallKey00",
0x845ddcac,"Dungeon067_SmallKey01",
0x1d548d16,"Dungeon067_SmallKey02",
0x6a53bd80,"Dungeon067_SmallKey03",
0xf4372823,"Dungeon067_SmallKey04",
0x833018b5,"Dungeon067_SmallKey05",
0x1a39490f,"Dungeon067_SmallKey06",
0x6d3e7999,"Dungeon067_SmallKey07",
0xfd816408,"Dungeon067_SmallKey08",
0x8a86549e,"Dungeon067_SmallKey09",
0x81ce734b,"Dungeon068_SmallKey00",
0xf6c943dd,"Dungeon068_SmallKey01",
0x6fc01267,"Dungeon068_SmallKey02",
0x18c722f1,"Dungeon068_SmallKey03",
0x86a3b752,"Dungeon068_SmallKey04",
0xf1a487c4,"Dungeon068_SmallKey05",
0x68add67e,"Dungeon068_SmallKey06",
0x1faae6e8,"Dungeon068_SmallKey07",
0x8f15fb79,"Dungeon068_SmallKey08",
0xf812cbef,"Dungeon068_SmallKey09",
0x1a6b3f24,"Dungeon069_SmallKey00",
0x6d6c0fb2,"Dungeon069_SmallKey01",
0xf4655e08,"Dungeon069_SmallKey02",
0x83626e9e,"Dungeon069_SmallKey03",
0x1d06fb3d,"Dungeon069_SmallKey04",
0x6a01cbab,"Dungeon069_SmallKey05",
0xf3089a11,"Dungeon069_SmallKey06",
0x840faa87,"Dungeon069_SmallKey07",
0x14b0b716,"Dungeon069_SmallKey08",
0x63b78780,"Dungeon069_SmallKey09",
0x5a54dc30,"Dungeon070_SmallKey00",
0x2d53eca6,"Dungeon070_SmallKey01",
0xb45abd1c,"Dungeon070_SmallKey02",
0xc35d8d8a,"Dungeon070_SmallKey03",
0x5d391829,"Dungeon070_SmallKey04",
0x2a3e28bf,"Dungeon070_SmallKey05",
0xb3377905,"Dungeon070_SmallKey06",
0xc4304993,"Dungeon070_SmallKey07",
0x548f5402,"Dungeon070_SmallKey08",
0x23886494,"Dungeon070_SmallKey09",
0xc1f1905f,"Dungeon071_SmallKey00",
0xb6f6a0c9,"Dungeon071_SmallKey01",
0x2ffff173,"Dungeon071_SmallKey02",
0x58f8c1e5,"Dungeon071_SmallKey03",
0xc69c5446,"Dungeon071_SmallKey04",
0xb19b64d0,"Dungeon071_SmallKey05",
0x2892356a,"Dungeon071_SmallKey06",
0x5f9505fc,"Dungeon071_SmallKey07",
0xcf2a186d,"Dungeon071_SmallKey08",
0xb82d28fb,"Dungeon071_SmallKey09",
0xb66f42af,"Dungeon072_SmallKey00",
0xc1687239,"Dungeon072_SmallKey01",
0x58612383,"Dungeon072_SmallKey02",
0x2f661315,"Dungeon072_SmallKey03",
0xb10286b6,"Dungeon072_SmallKey04",
0xc605b620,"Dungeon072_SmallKey05",
0x5f0ce79a,"Dungeon072_SmallKey06",
0x280bd70c,"Dungeon072_SmallKey07",
0xb8b4ca9d,"Dungeon072_SmallKey08",
0xcfb3fa0b,"Dungeon072_SmallKey09",
0x2dca0ec0,"Dungeon073_SmallKey00",
0x5acd3e56,"Dungeon073_SmallKey01",
0xc3c46fec,"Dungeon073_SmallKey02",
0xb4c35f7a,"Dungeon073_SmallKey03",
0x2aa7cad9,"Dungeon073_SmallKey04",
0x5da0fa4f,"Dungeon073_SmallKey05",
0xc4a9abf5,"Dungeon073_SmallKey06",
0xb3ae9b63,"Dungeon073_SmallKey07",
0x231186f2,"Dungeon073_SmallKey08",
0x5416b664,"Dungeon073_SmallKey09",
0x5952e74f,"Dungeon074_SmallKey00",
0x2e55d7d9,"Dungeon074_SmallKey01",
0xb75c8663,"Dungeon074_SmallKey02",
0xc05bb6f5,"Dungeon074_SmallKey03",
0x5e3f2356,"Dungeon074_SmallKey04",
0x293813c0,"Dungeon074_SmallKey05",
0xb031427a,"Dungeon074_SmallKey06",
0xc73672ec,"Dungeon074_SmallKey07",
0x57896f7d,"Dungeon074_SmallKey08",
0x208e5feb,"Dungeon074_SmallKey09",
0xc2f7ab20,"Dungeon075_SmallKey00",
0xb5f09bb6,"Dungeon075_SmallKey01",
0x2cf9ca0c,"Dungeon075_SmallKey02",
0x5bfefa9a,"Dungeon075_SmallKey03",
0xc59a6f39,"Dungeon075_SmallKey04",
0xb29d5faf,"Dungeon075_SmallKey05",
0x2b940e15,"Dungeon075_SmallKey06",
0x5c933e83,"Dungeon075_SmallKey07",
0xcc2c2312,"Dungeon075_SmallKey08",
0xbb2b1384,"Dungeon075_SmallKey09",
0xb56979d0,"Dungeon076_SmallKey00",
0xc26e4946,"Dungeon076_SmallKey01",
0x5b6718fc,"Dungeon076_SmallKey02",
0x2c60286a,"Dungeon076_SmallKey03",
0xb204bdc9,"Dungeon076_SmallKey04",
0xc5038d5f,"Dungeon076_SmallKey05",
0x5c0adce5,"Dungeon076_SmallKey06",
0x2b0dec73,"Dungeon076_SmallKey07",
0xbbb2f1e2,"Dungeon076_SmallKey08",
0xccb5c174,"Dungeon076_SmallKey09",
0x2ecc35bf,"Dungeon077_SmallKey00",
0x59cb0529,"Dungeon077_SmallKey01",
0xc0c25493,"Dungeon077_SmallKey02",
0xb7c56405,"Dungeon077_SmallKey03",
0x29a1f1a6,"Dungeon077_SmallKey04",
0x5ea6c130,"Dungeon077_SmallKey05",
0xc7af908a,"Dungeon077_SmallKey06",
0xb0a8a01c,"Dungeon077_SmallKey07",
0x2017bd8d,"Dungeon077_SmallKey08",
0x57108d1b,"Dungeon077_SmallKey09",
0x5c58aace,"Dungeon078_SmallKey00",
0x2b5f9a58,"Dungeon078_SmallKey01",
0xb256cbe2,"Dungeon078_SmallKey02",
0xc551fb74,"Dungeon078_SmallKey03",
0x5b356ed7,"Dungeon078_SmallKey04",
0x2c325e41,"Dungeon078_SmallKey05",
0xb53b0ffb,"Dungeon078_SmallKey06",
0xc23c3f6d,"Dungeon078_SmallKey07",
0x528322fc,"Dungeon078_SmallKey08",
0x2584126a,"Dungeon078_SmallKey09",
0xc7fde6a1,"Dungeon079_SmallKey00",
0xb0fad637,"Dungeon079_SmallKey01",
0x29f3878d,"Dungeon079_SmallKey02",
0x5ef4b71b,"Dungeon079_SmallKey03",
0xc09022b8,"Dungeon079_SmallKey04",
0xb797122e,"Dungeon079_SmallKey05",
0x2e9e4394,"Dungeon079_SmallKey06",
0x59997302,"Dungeon079_SmallKey07",
0xc9266e93,"Dungeon079_SmallKey08",
0xbe215e05,"Dungeon079_SmallKey09",
0x7d240905,"Dungeon080_SmallKey00",
0x0a233993,"Dungeon080_SmallKey01",
0x932a6829,"Dungeon080_SmallKey02",
0xe42d58bf,"Dungeon080_SmallKey03",
0x7a49cd1c,"Dungeon080_SmallKey04",
0x0d4efd8a,"Dungeon080_SmallKey05",
0x9447ac30,"Dungeon080_SmallKey06",
0xe3409ca6,"Dungeon080_SmallKey07",
0x73ff8137,"Dungeon080_SmallKey08",
0x04f8b1a1,"Dungeon080_SmallKey09",
0xe681456a,"Dungeon081_SmallKey00",
0x918675fc,"Dungeon081_SmallKey01",
0x088f2446,"Dungeon081_SmallKey02",
0x7f8814d0,"Dungeon081_SmallKey03",
0xe1ec8173,"Dungeon081_SmallKey04",
0x96ebb1e5,"Dungeon081_SmallKey05",
0x0fe2e05f,"Dungeon081_SmallKey06",
0x78e5d0c9,"Dungeon081_SmallKey07",
0xe85acd58,"Dungeon081_SmallKey08",
0x9f5dfdce,"Dungeon081_SmallKey09",
0x911f979a,"Dungeon082_SmallKey00",
0xe618a70c,"Dungeon082_SmallKey01",
0x7f11f6b6,"Dungeon082_SmallKey02",
0x0816c620,"Dungeon082_SmallKey03",
0x96725383,"Dungeon082_SmallKey04",
0xe1756315,"Dungeon082_SmallKey05",
0x787c32af,"Dungeon082_SmallKey06",
0x0f7b0239,"Dungeon082_SmallKey07",
0x9fc41fa8,"Dungeon082_SmallKey08",
0xe8c32f3e,"Dungeon082_SmallKey09",
0x0abadbf5,"Dungeon083_SmallKey00",
0x7dbdeb63,"Dungeon083_SmallKey01",
0xe4b4bad9,"Dungeon083_SmallKey02",
0x93b38a4f,"Dungeon083_SmallKey03",
0x0dd71fec,"Dungeon083_SmallKey04",
0x7ad02f7a,"Dungeon083_SmallKey05",
0xe3d97ec0,"Dungeon083_SmallKey06",
0x94de4e56,"Dungeon083_SmallKey07",
0x046153c7,"Dungeon083_SmallKey08",
0x73666351,"Dungeon083_SmallKey09",
0x7e22327a,"Dungeon084_SmallKey00",
0x092502ec,"Dungeon084_SmallKey01",
0x902c5356,"Dungeon084_SmallKey02",
0xe72b63c0,"Dungeon084_SmallKey03",
0x794ff663,"Dungeon084_SmallKey04",
0x0e48c6f5,"Dungeon084_SmallKey05",
0x9741974f,"Dungeon084_SmallKey06",
0xe046a7d9,"Dungeon084_SmallKey07",
0x70f9ba48,"Dungeon084_SmallKey08",
0x07fe8ade,"Dungeon084_SmallKey09",
0xe5877e15,"Dungeon085_SmallKey00",
0x92804e83,"Dungeon085_SmallKey01",
0x0b891f39,"Dungeon085_SmallKey02",
0x7c8e2faf,"Dungeon085_SmallKey03",
0xe2eaba0c,"Dungeon085_SmallKey04",
0x95ed8a9a,"Dungeon085_SmallKey05",
0x0ce4db20,"Dungeon085_SmallKey06",
0x7be3ebb6,"Dungeon085_SmallKey07",
0xeb5cf627,"Dungeon085_SmallKey08",
0x9c5bc6b1,"Dungeon085_SmallKey09",
0x9219ace5,"Dungeon086_SmallKey00",
0xe51e9c73,"Dungeon086_SmallKey01",
0x7c17cdc9,"Dungeon086_SmallKey02",
0x0b10fd5f,"Dungeon086_SmallKey03",
0x957468fc,"Dungeon086_SmallKey04",
0xe273586a,"Dungeon086_SmallKey05",
0x7b7a09d0,"Dungeon086_SmallKey06",
0x0c7d3946,"Dungeon086_SmallKey07",
0x9cc224d7,"Dungeon086_SmallKey08",
0xebc51441,"Dungeon086_SmallKey09",
0x09bce08a,"Dungeon087_SmallKey00",
0x7ebbd01c,"Dungeon087_SmallKey01",
0xe7b281a6,"Dungeon087_SmallKey02",
0x90b5b130,"Dungeon087_SmallKey03",
0x0ed12493,"Dungeon087_SmallKey04",
0x79d61405,"Dungeon087_SmallKey05",
0xe0df45bf,"Dungeon087_SmallKey06",
0x97d87529,"Dungeon087_SmallKey07",
0x076768b8,"Dungeon087_SmallKey08",
0x7060582e,"Dungeon087_SmallKey09",
0x7b287ffb,"Dungeon088_SmallKey00",
0x0c2f4f6d,"Dungeon088_SmallKey01",
0x95261ed7,"Dungeon088_SmallKey02",
0xe2212e41,"Dungeon088_SmallKey03",
0x7c45bbe2,"Dungeon088_SmallKey04",
0x0b428b74,"Dungeon088_SmallKey05",
0x924bdace,"Dungeon088_SmallKey06",
0xe54cea58,"Dungeon088_SmallKey07",
0x75f3f7c9,"Dungeon088_SmallKey08",
0x02f4c75f,"Dungeon088_SmallKey09",
0xe08d3394,"Dungeon089_SmallKey00",
0x978a0302,"Dungeon089_SmallKey01",
0x0e8352b8,"Dungeon089_SmallKey02",
0x7984622e,"Dungeon089_SmallKey03",
0xe7e0f78d,"Dungeon089_SmallKey04",
0x90e7c71b,"Dungeon089_SmallKey05",
0x09ee96a1,"Dungeon089_SmallKey06",
0x7ee9a637,"Dungeon089_SmallKey07",
0xee56bba6,"Dungeon089_SmallKey08",
0x99518b30,"Dungeon089_SmallKey09",
0xa0b2d080,"Dungeon090_SmallKey00",
0xd7b5e016,"Dungeon090_SmallKey01",
0x4ebcb1ac,"Dungeon090_SmallKey02",
0x39bb813a,"Dungeon090_SmallKey03",
0xa7df1499,"Dungeon090_SmallKey04",
0xd0d8240f,"Dungeon090_SmallKey05",
0x49d175b5,"Dungeon090_SmallKey06",
0x3ed64523,"Dungeon090_SmallKey07",
0xae6958b2,"Dungeon090_SmallKey08",
0xd96e6824,"Dungeon090_SmallKey09",
0x3b179cef,"Dungeon091_SmallKey00",
0x4c10ac79,"Dungeon091_SmallKey01",
0xd519fdc3,"Dungeon091_SmallKey02",
0xa21ecd55,"Dungeon091_SmallKey03",
0x3c7a58f6,"Dungeon091_SmallKey04",
0x4b7d6860,"Dungeon091_SmallKey05",
0xd27439da,"Dungeon091_SmallKey06",
0xa573094c,"Dungeon091_SmallKey07",
0x35cc14dd,"Dungeon091_SmallKey08",
0x42cb244b,"Dungeon091_SmallKey09",
0x4c894e1f,"Dungeon092_SmallKey00",
0x3b8e7e89,"Dungeon092_SmallKey01",
0xa2872f33,"Dungeon092_SmallKey02",
0xd5801fa5,"Dungeon092_SmallKey03",
0x4be48a06,"Dungeon092_SmallKey04",
0x3ce3ba90,"Dungeon092_SmallKey05",
0xa5eaeb2a,"Dungeon092_SmallKey06",
0xd2eddbbc,"Dungeon092_SmallKey07",
0x4252c62d,"Dungeon092_SmallKey08",
0x3555f6bb,"Dungeon092_SmallKey09",
0xd72c0270,"Dungeon093_SmallKey00",
0xa02b32e6,"Dungeon093_SmallKey01",
0x3922635c,"Dungeon093_SmallKey02",
0x4e2553ca,"Dungeon093_SmallKey03",
0xd041c669,"Dungeon093_SmallKey04",
0xa746f6ff,"Dungeon093_SmallKey05",
0x3e4fa745,"Dungeon093_SmallKey06",
0x494897d3,"Dungeon093_SmallKey07",
0xd9f78a42,"Dungeon093_SmallKey08",
0xaef0bad4,"Dungeon093_SmallKey09",
0xa3b4ebff,"Dungeon094_SmallKey00",
0xd4b3db69,"Dungeon094_SmallKey01",
0x4dba8ad3,"Dungeon094_SmallKey02",
0x3abdba45,"Dungeon094_SmallKey03",
0xa4d92fe6,"Dungeon094_SmallKey04",
0xd3de1f70,"Dungeon094_SmallKey05",
0x4ad74eca,"Dungeon094_SmallKey06",
0x3dd07e5c,"Dungeon094_SmallKey07",
0xad6f63cd,"Dungeon094_SmallKey08",
0xda68535b,"Dungeon094_SmallKey09",
0x3811a790,"Dungeon095_SmallKey00",
0x4f169706,"Dungeon095_SmallKey01",
0xd61fc6bc,"Dungeon095_SmallKey02",
0xa118f62a,"Dungeon095_SmallKey03",
0x3f7c6389,"Dungeon095_SmallKey04",
0x487b531f,"Dungeon095_SmallKey05",
0xd17202a5,"Dungeon095_SmallKey06",
0xa6753233,"Dungeon095_SmallKey07",
0x36ca2fa2,"Dungeon095_SmallKey08",
0x41cd1f34,"Dungeon095_SmallKey09",
0x4f8f7560,"Dungeon096_SmallKey00",
0x388845f6,"Dungeon096_SmallKey01",
0xa181144c,"Dungeon096_SmallKey02",
0xd68624da,"Dungeon096_SmallKey03",
0x48e2b179,"Dungeon096_SmallKey04",
0x3fe581ef,"Dungeon096_SmallKey05",
0xa6ecd055,"Dungeon096_SmallKey06",
0xd1ebe0c3,"Dungeon096_SmallKey07",
0x4154fd52,"Dungeon096_SmallKey08",
0x3653cdc4,"Dungeon096_SmallKey09",
0xd42a390f,"Dungeon097_SmallKey00",
0xa32d0999,"Dungeon097_SmallKey01",
0x3a245823,"Dungeon097_SmallKey02",
0x4d2368b5,"Dungeon097_SmallKey03",
0xd347fd16,"Dungeon097_SmallKey04",
0xa440cd80,"Dungeon097_SmallKey05",
0x3d499c3a,"Dungeon097_SmallKey06",
0x4a4eacac,"Dungeon097_SmallKey07",
0xdaf1b13d,"Dungeon097_SmallKey08",
0xadf681ab,"Dungeon097_SmallKey09",
0xa6bea67e,"Dungeon098_SmallKey00",
0xd1b996e8,"Dungeon098_SmallKey01",
0x48b0c752,"Dungeon098_SmallKey02",
0x3fb7f7c4,"Dungeon098_SmallKey03",
0xa1d36267,"Dungeon098_SmallKey04",
0xd6d452f1,"Dungeon098_SmallKey05",
0x4fdd034b,"Dungeon098_SmallKey06",
0x38da33dd,"Dungeon098_SmallKey07",
0xa8652e4c,"Dungeon098_SmallKey08",
0xdf621eda,"Dungeon098_SmallKey09",
0x3d1bea11,"Dungeon099_SmallKey00",
0x4a1cda87,"Dungeon099_SmallKey01",
0xd3158b3d,"Dungeon099_SmallKey02",
0xa412bbab,"Dungeon099_SmallKey03",
0x3a762e08,"Dungeon099_SmallKey04",
0x4d711e9e,"Dungeon099_SmallKey05",
0xd4784f24,"Dungeon099_SmallKey06",
0xa37f7fb2,"Dungeon099_SmallKey07",
0x33c06223,"Dungeon099_SmallKey08",
0x44c752b5,"Dungeon099_SmallKey09",
0x0adca951,"E-6_Animal_RupeeRabbit_A_2487515977",
0xa087d848,"Electlic_Relic_Rush",
0x9dc3f061,"Electlic_Relic_Sunazarashi",
0x1d740cc7,"Electlic_Relic_raimeinokabuto",
0x55c43d86,"Electric_Relic_Activated",
0x4bf1a6fc,"Electric_Relic_Arrival",
0x601f5853,"Electric_Relic_AssassinBoss",
0xa641d695,"Electric_Relic_AssassinFirst",
0xe3d58e14,"Electric_Relic_AssassunBossEnd",
0x3482807e,"Electric_Relic_BomArrowGet",
0xad5c60b6,"Electric_Relic_BomArrowGet2",
0xda5b5020,"Electric_Relic_BomArrowGet3",
0xc62a489f,"Electric_Relic_Defeat",
0x46a96afc,"Electric_Relic_Deliver",
0xde587664,"Electric_Relic_Finished",
0x875dc4f0,"Electric_Relic_FirstTalk",
0x7bba3eae,"Electric_Relic_GetBack",
0x06d2fad2,"Electric_Relic_Invasion",
0xc96380d8,"Electric_Relic_Ready",
0x6661178d,"Electric_Relic_RunningStop",
0x7a70c93d,"Electric_Relic_Search",
0x6c5198b5,"Electric_Relic_Strategy",
0x177d718e,"Electric_Relic_SunazarashiRouge",
0x14dfab75,"Electric_Relic_Varetta_first",
0x6fef7573,"Electric_Relic_Varetta_first_lady",
0xb1f1ce01,"Enable_Surfing",
0x618edd8e,"Enemy_Dragon_Electric_Appearance_Today",
0x95f77256,"Enemy_Dragon_Electric_DoorOpen",
0x52557a82,"Enemy_Dragon_Electric_DropItem",
0xe03e640a,"Enemy_Dragon_Electric_Item_Splashdown",
0x0f614303,"Enemy_Dragon_Fire_Appearance_Today",
0xae3e866b,"Enemy_Dragon_Fire_DoorOpen",
0x699c8ebf,"Enemy_Dragon_Fire_DropItem",
0x64a60e9d,"Enemy_Dragon_Fire_Item_Splashdown",
0xf96a0cc8,"Enemy_Dragon_Ice_Appearance_Today",
0xc775d89c,"Enemy_Dragon_Ice_ChallengeClear",
0xb35ceef8,"Enemy_Dragon_Ice_CreateRunelDragon",
0xf7dae8ed,"Enemy_Dragon_Ice_DoorOpen",
0x3078e039,"Enemy_Dragon_Ice_DropItem",
0xda11221f,"Enemy_Dragon_Ice_GrudgeDestroyComplete",
0xdb6951c9,"Enemy_Dragon_Ice_GrudgeDestroyDemoEnd",
0x6e4a7aba,"Enemy_Dragon_Ice_GrudgeLeft_1",
0xf7432b00,"Enemy_Dragon_Ice_GrudgeLeft_2",
0x80441b96,"Enemy_Dragon_Ice_GrudgeLeft_3",
0x1e208e35,"Enemy_Dragon_Ice_GrudgeLeft_4",
0xd6cf9e48,"Enemy_Dragon_Ice_Item_Splashdown",
0xce5429fb,"Enemy_Dragon_Ice_ReturnToSky",
0x0b8ad079,"Enemy_Icebreak_SpotFlag",
0x37baa29d,"Enter_Dungeon000",
0x40bd920b,"Enter_Dungeon001",
0xd9b4c3b1,"Enter_Dungeon002",
0xaeb3f327,"Enter_Dungeon003",
0x30d76684,"Enter_Dungeon004",
0x47d05612,"Enter_Dungeon005",
0xded907a8,"Enter_Dungeon006",
0xa9de373e,"Enter_Dungeon007",
0x39612aaf,"Enter_Dungeon008",
0x4e661a39,"Enter_Dungeon009",
0x2ea193dc,"Enter_Dungeon010",
0x59a6a34a,"Enter_Dungeon011",
0xc0aff2f0,"Enter_Dungeon012",
0xb7a8c266,"Enter_Dungeon013",
0x29cc57c5,"Enter_Dungeon014",
0x5ecb6753,"Enter_Dungeon015",
0xc7c236e9,"Enter_Dungeon016",
0xb0c5067f,"Enter_Dungeon017",
0x207a1bee,"Enter_Dungeon018",
0x577d2b78,"Enter_Dungeon019",
0x058cc01f,"Enter_Dungeon020",
0x728bf089,"Enter_Dungeon021",
0xeb82a133,"Enter_Dungeon022",
0x9c8591a5,"Enter_Dungeon023",
0x02e10406,"Enter_Dungeon024",
0x75e63490,"Enter_Dungeon025",
0xecef652a,"Enter_Dungeon026",
0x9be855bc,"Enter_Dungeon027",
0x0b57482d,"Enter_Dungeon028",
0x7c5078bb,"Enter_Dungeon029",
0x1c97f15e,"Enter_Dungeon030",
0x6b90c1c8,"Enter_Dungeon031",
0xf2999072,"Enter_Dungeon032",
0x859ea0e4,"Enter_Dungeon033",
0x1bfa3547,"Enter_Dungeon034",
0x6cfd05d1,"Enter_Dungeon035",
0xf5f4546b,"Enter_Dungeon036",
0x82f364fd,"Enter_Dungeon037",
0x124c796c,"Enter_Dungeon038",
0x654b49fa,"Enter_Dungeon039",
0x53d66799,"Enter_Dungeon040",
0x24d1570f,"Enter_Dungeon041",
0xbdd806b5,"Enter_Dungeon042",
0xcadf3623,"Enter_Dungeon043",
0x54bba380,"Enter_Dungeon044",
0x23bc9316,"Enter_Dungeon045",
0xbab5c2ac,"Enter_Dungeon046",
0xcdb2f23a,"Enter_Dungeon047",
0x5d0defab,"Enter_Dungeon048",
0x2a0adf3d,"Enter_Dungeon049",
0x4acd56d8,"Enter_Dungeon050",
0x3dca664e,"Enter_Dungeon051",
0xa4c337f4,"Enter_Dungeon052",
0xd3c40762,"Enter_Dungeon053",
0x4da092c1,"Enter_Dungeon054",
0x3aa7a257,"Enter_Dungeon055",
0xa3aef3ed,"Enter_Dungeon056",
0xd4a9c37b,"Enter_Dungeon057",
0x4416deea,"Enter_Dungeon058",
0x3311ee7c,"Enter_Dungeon059",
0x61e0051b,"Enter_Dungeon060",
0x16e7358d,"Enter_Dungeon061",
0x8fee6437,"Enter_Dungeon062",
0xf8e954a1,"Enter_Dungeon063",
0x668dc102,"Enter_Dungeon064",
0x118af194,"Enter_Dungeon065",
0x8883a02e,"Enter_Dungeon066",
0xff8490b8,"Enter_Dungeon067",
0x6f3b8d29,"Enter_Dungeon068",
0x183cbdbf,"Enter_Dungeon069",
0x78fb345a,"Enter_Dungeon070",
0x0ffc04cc,"Enter_Dungeon071",
0x96f55576,"Enter_Dungeon072",
0xe1f265e0,"Enter_Dungeon073",
0x7f96f043,"Enter_Dungeon074",
0x0891c0d5,"Enter_Dungeon075",
0x9198916f,"Enter_Dungeon076",
0xe69fa1f9,"Enter_Dungeon077",
0x7620bc68,"Enter_Dungeon078",
0x01278cfe,"Enter_Dungeon079",
0xff632895,"Enter_Dungeon080",
0x88641803,"Enter_Dungeon081",
0x116d49b9,"Enter_Dungeon082",
0x666a792f,"Enter_Dungeon083",
0xf80eec8c,"Enter_Dungeon084",
0x8f09dc1a,"Enter_Dungeon085",
0x16008da0,"Enter_Dungeon086",
0x6107bd36,"Enter_Dungeon087",
0xf1b8a0a7,"Enter_Dungeon088",
0x86bf9031,"Enter_Dungeon089",
0xe67819d4,"Enter_Dungeon090",
0x917f2942,"Enter_Dungeon091",
0x087678f8,"Enter_Dungeon092",
0x7f71486e,"Enter_Dungeon093",
0xe115ddcd,"Enter_Dungeon094",
0x9612ed5b,"Enter_Dungeon095",
0x0f1bbce1,"Enter_Dungeon096",
0x781c8c77,"Enter_Dungeon097",
0xe8a391e6,"Enter_Dungeon098",
0x9fa4a170,"Enter_Dungeon099",
0x3678c8aa,"Enter_Dungeon100",
0x417ff83c,"Enter_Dungeon101",
0xd876a986,"Enter_Dungeon102",
0xaf719910,"Enter_Dungeon103",
0x31150cb3,"Enter_Dungeon104",
0x46123c25,"Enter_Dungeon105",
0xdf1b6d9f,"Enter_Dungeon106",
0xa81c5d09,"Enter_Dungeon107",
0x38a34098,"Enter_Dungeon108",
0x4fa4700e,"Enter_Dungeon109",
0x2f63f9eb,"Enter_Dungeon110",
0x5864c97d,"Enter_Dungeon111",
0xc16d98c7,"Enter_Dungeon112",
0xb66aa851,"Enter_Dungeon113",
0x280e3df2,"Enter_Dungeon114",
0x5f090d64,"Enter_Dungeon115",
0xc6005cde,"Enter_Dungeon116",
0xb1076c48,"Enter_Dungeon117",
0x21b871d9,"Enter_Dungeon118",
0x56bf414f,"Enter_Dungeon119",
0xeb2a1ff7,"Enter_RemainsElectric",
0x84a6b839,"Enter_RemainsFire",
0xaf313b42,"Enter_RemainsWater",
0xfd73e508,"Enter_RemainsWind",
0x6c63e766,"Event_LookMasterSword",
0xf676b558,"F-6_Animal_RupeeRabbit_A_1397879052",
0xb66b8d24,"F-6_Animal_RupeeRabbit_A_558955435",
0x88c800db,"FairyCountCheck",
0x35299e84,"FairyFountain_Activated",
0xfd8876af,"FairyFountain_Finish",
0x6474fbc0,"FairyFountain_Ready",
0xf986ef5d,"FindDungeon_1stClear",
0x1d8dc8bd,"FindDungeon_Activated",
0x2bc5121c,"FindDungeon_AllClear",
0x6535e51c,"FindDungeon_Finish",
0x0bc7f482,"FindDungeon_Ready",
0x57e7415b,"Find_4Relic_1stClear",
0xdcb7b92d,"Find_4Relic_2ndClear",
0xb0634f09,"Find_4Relic_3rdClear",
0xaeb4b351,"Find_4Relic_4thClear",
0xf4400c26,"Find_4Relic_Activated",
0x71e12f66,"Find_4Relic_Finish",
0xde51c19e,"Find_4Relic_Ready",
0x1f7f7516,"Find_Impa_Activated",
0x443f3452,"Find_Impa_Finish",
0xb5e20fb6,"Find_Impa_Ready",
0xe658a31b,"Find_Impa_Sayonara00",
0xce3eb5ee,"FireWood_First",
0xdc8b830f,"Fire_Relic_Activated",
0xbb2f686a,"Fire_Relic_Battle",
0x97ae312d,"Fire_Relic_Battle1stAttack",
0x2e43b5dc,"Fire_Relic_Battle2ndAttack",
0x122c85fb,"Fire_Relic_Battle3rdAttack",
0x5ec7e5ac,"Fire_Relic_BattlePlaying",
0x1144f688,"Fire_Relic_BattlePlaying_AreaInAppear",
0x61a03998,"Fire_Relic_BattlePlaying_ForceSavePos_YunBo_Bridge00",
0xc74b64b5,"Fire_Relic_BattlePlaying_ForceSavePos_YunBo_Cannon1st",
0x27d6a694,"Fire_Relic_BattlePlaying_ForceSavePos_YunBo_Cannon2nd",
0x2614cca3,"Fire_Relic_BattlePlaying_ForceSavePos_YunBo_Cannon3nd",
0xa63e71d2,"Fire_Relic_BattlePlaying_HintArea1_OFF",
0xc1ce70c1,"Fire_Relic_Bridge",
0xe70a5445,"Fire_Relic_Bridge_AreaInAppear",
0x4d2c7b15,"Fire_Relic_Bridge_Cannon",
0x70593d9a,"Fire_Relic_Bridge_Enemy_A_Die",
0x37f9474a,"Fire_Relic_Bridge_Enemy_B_Die",
0x9f713808,"Fire_Relic_Bridge_ExplanationAncestor",
0x0e1dd6da,"Fire_Relic_Bridge_ExplanationOrdinBridge",
0xcff99366,"Fire_Relic_Bridge_ExplanationRudania",
0x7d73a020,"Fire_Relic_Bridge_NPC006_First",
0x5c869267,"Fire_Relic_Bridge_NPC020_First",
0x4eca5cbc,"Fire_Relic_BurudoThankYouGoods",
0x49b09116,"Fire_Relic_CraterFireRelicApper",
0xf44a4daa,"Fire_Relic_DeathBridgeONOFF",
0x1abcb1c6,"Fire_Relic_DeathMT_Cannon1",
0x83b5e07c,"Fire_Relic_DeathMT_Cannon2",
0xf4b2d0ea,"Fire_Relic_DeathMT_Cannon3",
0x5889819c,"Fire_Relic_DeathMt_Wind_OFF",
0x25700b4d,"Fire_Relic_Demo346_4",
0xc5e912ef,"Fire_Relic_Demo346_4_1st",
0x8c2f6b2b,"Fire_Relic_DemoArea_Demo346_4",
0x6dfb748f,"Fire_Relic_DroneAlert1st",
0x294d8b3e,"Fire_Relic_DroneONOFF_Demo346_4",
0xb381d51c,"Fire_Relic_Dungeon",
0xe5139bfb,"Fire_Relic_Dungeon_NPC020_First",
0x7c792f35,"Fire_Relic_EnemyOFF_1",
0x36a60496,"Fire_Relic_Finish_NPC020_First",
0xbf78bad0,"Fire_Relic_Finish_NPC020_Stop",
0x42cd1ffd,"Fire_Relic_Finished",
0x4c555813,"Fire_Relic_FireRelicONOFF",
0xc74e8206,"Fire_Relic_GoDeathMt",
0x42323615,"Fire_Relic_GoDeathMt_Enemy",
0x5a13181b,"Fire_Relic_GoDeathMt_EnemyDie",
0xee5b15d0,"Fire_Relic_GoDeathMt_NPC020_First",
0x9ab403e7,"Fire_Relic_GoGoronCity",
0x83e4c65d,"Fire_Relic_Meet_StrageYunBo",
0xf0481484,"Fire_Relic_NPC020_Follow",
0x634118f5,"Fire_Relic_NPC020_StorageFirst",
0x3d3c9df5,"Fire_Relic_NorthMine",
0xf9358e02,"Fire_Relic_Ready",
0xa12467bc,"Fire_Relic_Ready_NPC006_First",
0xfdbfdb12,"Fire_Relic_Rready_NPC020_First",
0x60252c12,"Fire_Relic_Rready_NPC020_GranPa",
0x6c3101bf,"Fire_Relic_Rready_NPC020_Telepathy",
0xcff6edb9,"Fire_Relic_Rready_NPC020_Wink",
0x48e010ae,"Fire_Relic_StopForever_VolcanicPlume",
0xd801d1b8,"Fire_Relic_Storage",
0x8c6404da,"Fire_Relic_Storage_AscendingCurrent",
0xd72338eb,"Fire_Relic_Storage_ScaffoldIronBroken",
0x52904387,"Fire_Relic_Storage_SkullRockBroken",
0x5fabb58a,"Fire_Relic_TBox_Appear",
0xeafaabf4,"Fire_Relic_VolcanicBomb_OFF_1",
0x73f3fa4e,"Fire_Relic_VolcanicBomb_OFF_2",
0x04f4cad8,"Fire_Relic_VolcanicBomb_OFF_3",
0x811c61a1,"Fire_Relic_VolcanicBomb_ONOFF_1",
0x1815301b,"Fire_Relic_VolcanicBomb_ONOFF_2",
0x6f12008d,"Fire_Relic_VolcanicBomb_ONOFF_3",
0x37084103,"Fire_Relic_WhistleMessage",
0x45a231c1,"Fire_Relic_YunboStopGo",
0x653ed58f,"FironeMini_GiantHorse_Activated",
0x2ba5dab0,"FironeMini_GiantHorse_Finish",
0x146cd13d,"FironeMini_GiantHorse_IsTalked",
0x25bae9fc,"FironeMini_GiantHorse_Present",
0x9f7b0c60,"FironeMini_GiantHorse_Ready",
0xe8cd28b3,"FironeMini_HeartPond_Activated",
0x1a7c2343,"FironeMini_HeartPond_Finish",
0x975f1c45,"FironeMini_HeartPond_GetFlower",
0x55500096,"FironeMini_HeartPond_GetRupee",
0xff7dd4af,"FironeMini_HeartPond_ManIsTalked",
0x00df78ac,"FironeMini_HeartPond_PresentFlower",
0x280c10ce,"FironeMini_HeartPond_Ready",
0x8711cee0,"FironeMini_HeartPond_WomanIsTalked",
0xe448e1b1,"FironeMini_HorseEnemy_Activated",
0xef8e7ce6,"FironeMini_HorseEnemy_Exterminate",
0x8635bfe7,"FironeMini_HorseEnemy_Finish",
0x72dc66ce,"FironeMini_HorseEnemy_GetReward",
0xdc4e22e9,"FironeMini_HorseEnemy_IsTalked",
0x68961d0a,"FironeMini_HorseEnemy_Ready",
0x7df20acc,"FironeMini_HorseEnemy_TalkGiantHorse",
0x918566e1,"FironeMini_HorseEnemy_TalkMaron",
0x27180f3d,"FironeMini_TerribleThunder_Activated",
0x1738ff80,"FironeMini_TerribleThunder_Finish",
0xb18c8842,"FironeMini_TerribleThunder_GetReward",
0x82ec4d3a,"FironeMini_TerribleThunder_Ono",
0x01baed44,"FironeMini_TerribleThunder_Ready",
0x5f7109a8,"FironeMini_TerribleThunder_Remove",
0xe84e9cfa,"FironeMini_TerribleThuner_IsTalked",
0x56704565,"FirstInHyruleCastleBossRoom",
0x580d4235,"FirstMessage_RemainsElectric",
0xa4401d9a,"FirstMessage_RemainsFire",
0xe0ce2fb5,"FirstMessage_RemainsWater",
0xdd9540ab,"FirstMessage_RemainsWind",
0x6ab67dca,"FirstOhenro_Activated",
0xb33715cc,"FirstOhenro_Finish",
0xdf94cc22,"FirstOhenro_Ready",
0xc2f832f9,"FirstOhenro_Step2",
0xa7006307,"FirstTerminal_On",
0x9b6c1cfb,"FirstTips",
0x330948f3,"FirstTouchdown",
0xe87bbc20,"FirstTower_Activated",
0xc6415210,"FirstTower_Finish",
0xc3708ce0,"FirstTower_Ready",
0xd667a996,"First_CDungeon",
0x299b17b0,"First_CDungeon_Enemy",
0x83ccc6d8,"First_CDungeon_Prize",
0x08b9060c,"First_CDungeon_Riddle",
0x1eca5627,"FldObj_Mound_Off",
0x476dcca8,"FourJewel_Activated",
0x3662d44a,"FourJewel_Dungeon",
0x0e4e1a14,"FourJewel_Finish",
0x1b6908d2,"FourJewel_Ready",
0xa3ca8c3c,"FourJewel_SetA",
0x3ac3dd86,"FourJewel_SetB",
0x4dc4ed10,"FourJewel_SetC",
0xd3a078b3,"FourJewel_SetD",
0x5ce2c4ff,"FraratTalk_2nd",
0x84f12a10,"G-5_Animal_RupeeRabbit_A_2088165780",
0xe92ac616,"G-5_Animal_RupeeRabbit_A_3153871387",
0x93bc709d,"G-5_Animal_RupeeRabbit_A_4226568064",
0xac69182b,"G-6_Animal_RupeeRabbit_A_750137237",
0x0998a9cf,"G-7_Animal_RupeeRabbit_A_1416749828",
0x213d8296,"G-7_Animal_RupeeRabbit_A_1860780134",
0xdcd8a663,"G-7_Animal_RupeeRabbit_A_2156716660",
0x09b30dd6,"G-7_Animal_RupeeRabbit_A_216844463",
0xfb9df226,"G-7_Animal_RupeeRabbit_A_3477513170",
0x59d47701,"G-7_Animal_RupeeRabbit_A_3557966274",
0x9dc94f48,"G-7_Animal_RupeeRabbit_A_4139197005",
0x739a86f3,"Gaman_Activated",
0x7bf54257,"Gaman_DungeonArrival",
0x472eaf95,"Gaman_Finish",
0x0d1f759f,"Gaman_Gaman_Step1_2_FailedAir",
0x59b4674a,"Gaman_Gaman_Step2_2",
0xbae969da,"Gaman_Gaman_Step2_Failed",
0x50578e50,"Gaman_Gaman_Step2_FailedAir_Npc_Gaman01_StepStart",
0x129bc434,"Gaman_Ready",
0xfb358269,"Gaman_Start01",
0x96fe6b55,"Gaman_Step1",
0xa98185db,"Gaman_Step1_2",
0xd23a91a4,"Gaman_Step1_2_1",
0x25b28241,"Gaman_Step1_2_10",
0x52b5b2d7,"Gaman_Step1_2_11",
0xcbbce36d,"Gaman_Step1_2_12",
0xbcbbd3fb,"Gaman_Step1_2_13",
0x4b33c01e,"Gaman_Step1_2_2",
0x3c34f088,"Gaman_Step1_2_3",
0xa250652b,"Gaman_Step1_2_4",
0xd55755bd,"Gaman_Step1_2_5",
0x4c5e0407,"Gaman_Step1_2_6",
0x3b593491,"Gaman_Step1_2_7",
0xabe62900,"Gaman_Step1_2_8",
0xdce11996,"Gaman_Step1_2_9",
0x3b485cbb,"Gaman_Step1_2_Failed",
0xdaef45da,"Gaman_Step1_2_Successed",
0x0ff73aef,"Gaman_Step2",
0xdceba9d5,"Gaman_Step2_10",
0xabec9943,"Gaman_Step2_11",
0x32e5c8f9,"Gaman_Step2_12",
0x45e2f86f,"Gaman_Step2_13",
0xdb866dcc,"Gaman_Step2_14",
0xac815d5a,"Gaman_Step2_15",
0x35880ce0,"Gaman_Step2_16",
0x428f3c76,"Gaman_Step2_17",
0xd23021e7,"Gaman_Step2_18",
0xa5371171,"Gaman_Step2_19",
0xf7c6fa16,"Gaman_Step2_20",
0x80c1ca80,"Gaman_Step2_21",
0xdcc00b14,"Gaman_Step2_3",
0x42a49eb7,"Gaman_Step2_4",
0x35a3ae21,"Gaman_Step2_5",
0xacaaff9b,"Gaman_Step2_6",
0xdbadcf0d,"Gaman_Step2_7",
0x4b12d29c,"Gaman_Step2_8",
0x3c15e20a,"Gaman_Step2_9",
0x13fee82d,"Gaman_Step2_AreaMake",
0x7977f228,"Gaman_Step2_Retry",
0x7ef303dd,"GanonQuest_Activated",
0x5d3cbfde,"GanonQuest_Finished",
0x3cadbc09,"GanonQuest_Ready",
0x9bfc7146,"GemShop_Open",
0x80b9c8af,"Gerud_Meat_SoldOut",
0xe27d8c65,"GerudoMiniJewel_Activated",
0x2a3be36d,"GerudoMiniJewel_Display",
0xdab5ae1a,"GerudoMiniJewel_Finished",
0x300e3b3a,"GerudoMiniJewel_First",
0xba6655f3,"GerudoMiniJewel_Poach",
0x8aea50c2,"GerudoMiniJewel_Ready",
0xd1a58460,"GerudoMiniJewel_Talked",
0x2c336caa,"GerudoRoad_ClosedRock_01",
0xb53a3d10,"GerudoRoad_ClosedRock_02",
0xc23d0d86,"GerudoRoad_ClosedRock_03",
0x13638585,"Gerudo_Arrow_SoldOut",
0x96b0503a,"Gerudo_CarryIce_Activated",
0x3ddcf360,"Gerudo_CarryIce_Bar_First",
0xc59625be,"Gerudo_CarryIce_Delivered",
0xa6bb6b99,"Gerudo_CarryIce_Finished",
0xd234522a,"Gerudo_CarryIce_FirstVoy",
0xbfe2d5a8,"Gerudo_CarryIce_Gerudo_CarryIce_bar",
0x1960cdbb,"Gerudo_CarryIce_Get",
0x62f15c38,"Gerudo_CarryIce_IceExist",
0x3998dcb3,"Gerudo_CarryIce_Ready",
0x93769560,"Gerudo_CarryIce_Talk",
0x1c4c125f,"Gerudo_CarryIce_fireworks_end",
0xd7ffa5e0,"Gerudo_CarryIce_fireworks_ice",
0x39fb055a,"Gerudo_CarryIce_mini_Ready",
0x397da553,"Gerudo_Ch_FindingValetta_Activated",
0x499ce345,"Gerudo_Ch_FindingValetta_Find",
0xba7703ed,"Gerudo_Ch_FindingValetta_Finish",
0x29cec62e,"Gerudo_Ch_FindingValetta_Ready",
0xadab694f,"Gerudo_Ch_FindingValetta_Step1",
0x34a238f5,"Gerudo_Ch_FindingValetta_Step2",
0x2e960bec,"Gerudo_Ch_Helmet_Activated",
0x90edcc8f,"Gerudo_Ch_Helmet_Finish",
0x9641cc20,"Gerudo_Ch_Helmet_Get",
0xdc2a30d5,"Gerudo_Ch_Helmet_Ready",
0xa70f7429,"Gerudo_Ch_Helmet_first",
0xb0816702,"Gerudo_Ch_Poison_Activated",
0xa4bbeb3d,"Gerudo_Ch_Poison_Finish",
0x5e301544,"Gerudo_Ch_Poison_First",
0x8afc5c63,"Gerudo_Ch_Poison_Ichigo",
0xe4d47ebc,"Gerudo_Ch_Poison_Ready",
0x60b1d1dd,"Gerudo_Ch_Poison_Step1",
0xf9b88067,"Gerudo_Ch_Poison_Step2",
0xdb520421,"Gerudo_Ch_SandWarm_Activated",
0xb37a23e1,"Gerudo_Ch_SandWarm_Finish",
0x28468ada,"Gerudo_Ch_SandWarm_First",
0x92a2e122,"Gerudo_Ch_SandWarm_Ready",
0x20532624,"Gerudo_Ch_SecretClub_Activated",
0xab609f5e,"Gerudo_Ch_SecretClub_CodeA",
0x3269cee4,"Gerudo_Ch_SecretClub_CodeB",
0x456efe72,"Gerudo_Ch_SecretClub_CodeC",
0xdb0a6bd1,"Gerudo_Ch_SecretClub_CodeD",
0x4e3e11e5,"Gerudo_Ch_SecretClub_CodeGet",
0x494933e9,"Gerudo_Ch_SecretClub_Finish",
0xae8f8a3c,"Gerudo_Ch_SecretClub_Ready",
0x708b5166,"Gerudo_Ch_SecretClub_Stop",
0x9e54ef79,"Gerudo_Ch_SnowBoots_Activated",
0x5f2dfcd0,"Gerudo_Ch_SnowBoots_Boots",
0xaf74193f,"Gerudo_Ch_SnowBoots_Finish",
0x411e0ddc,"Gerudo_Ch_SnowBoots_First_boy",
0x61d290c2,"Gerudo_Ch_SnowBoots_First_lady",
0x818cd208,"Gerudo_Ch_SnowBoots_Full",
0xf0f95d73,"Gerudo_Ch_SnowBoots_Ready",
0x4e6bc7b0,"Gerudo_Ch_SnowBoots_sand_first",
0x26067fc7,"Gerudo_Ch_SnowMT_Activated",
0x1aa26c53,"Gerudo_Ch_SnowMT_Boots",
0xdb804435,"Gerudo_Ch_SnowMT_Finish",
0x0f92a608,"Gerudo_Ch_SnowMT_First",
0xb576cdf0,"Gerudo_Ch_SnowMT_Ready",
0xcb870030,"Gerudo_Ch_SnowMT_boots_get",
0x71728b1b,"Gerudo_Ch_SnowMT_full",
0x55828721,"Gerudo_Ch_SnowMt_First_lady",
0xb8e74df8,"Gerudo_Ch_SnowMt_First_lady0",
0x2324cc85,"Gerudo_Clothes_SoldOut",
0xb242e26a,"Gerudo_FirstEnt",
0x3ba1961c,"Gerudo_Food_SoldOut",
0xa7e59704,"Gerudo_Fruits_SoldOut",
0x9f459feb,"Gerudo_GSC_SoldOut",
0x05ff635b,"Gerudo_Hint_01",
0x9cf632e1,"Gerudo_Hint_02",
0x09aca807,"Gerudo_HorseBuyer_Activated",
0xa609a0ee,"Gerudo_HorseBuyer_Finish",
0x3b0ca77f,"Gerudo_HorseBuyer_Ready",
0xddcc3c2f,"Gerudo_Mush_SoldOut",
0x5ceca29c,"Gerudo_oasis005_ArmerFirst",
0xaa0b33ed,"Gerudo_tsukamidake_Activated",
0x22a3436f,"Gerudo_tsukamidake_Finish",
0x2faef86c,"Gerudo_tsukamidake_Ready",
0x484e46ac,"Get_GerudoWeapon",
0xb6207bb5,"Get_GoronWeapon",
0xf14dc15d,"Get_GreatHeroAmor",
0xa594fa9a,"Get_HeartUtuwa_Electric",
0x48ad79aa,"Get_HeartUtuwa_Fire",
0xc6fbf27d,"Get_HeartUtuwa_Water",
0x3178249b,"Get_HeartUtuwa_Wind",
0xc86d4e29,"Get_MasterSword_Activated",
0x87c058f8,"Get_MasterSword_Deku",
0x15ad023f,"Get_MasterSword_Finish",
0x5e68a0c9,"Get_MasterSword_FirstFailure",
0xf9d45887,"Get_MasterSword_Ready",
0xed811f4d,"Get_RitoWeapon",
0xe9c9b65c,"Get_Vagrant_Axe",
0x1d50d8f6,"Get_Vagrant_BOKANGI",
0x1db5fddb,"Get_Vagrant_Bow",
0x1b1c54e9,"Get_ZoraWeapon",
0x48fe86d7,"Giant_ZoraMini_Activated",
0x6aac92e0,"Giant_ZoraMini_Finish",
0xc3a4aee8,"Giant_ZoraMini_Punitive",
0x92493ccb,"Giant_ZoraMini_Ready",
0x162c93aa,"Giant_ZoraMini_Step1",
0xa2443641,"GodTree_Activated",
0xb45b3659,"GodTree_Dis_Tree",
0x08d60c6b,"GodTree_Dis_Tree2",
0x73406d78,"GodTree_Enemy_Pop",
0x00063b0e,"GodTree_Finish",
0x88c020da,"GodTree_Ready",
0xdaa9aab1,"GodTree_Step010",
0xde6d7d1a,"GoronCamp_Activated",
0xf9a23521,"GoronCamp_Finish",
0xb105381b,"GoronCamp_GameReady",
0x95116046,"GoronCamp_Ready",
0x65808c5b,"GoronCamp_ResetGame",
0xdf2753da,"GoronCamp_mini_Activated",
0x28058ae1,"GoronCamp_mini_ChoiceExclude",
0x3d54b301,"GoronCamp_mini_Finish",
0x0dbeaaac,"GoronCamp_mini_FirstTalk",
0xb04f16db,"GoronCamp_mini_GameReady",
0x02ddf8bf,"GoronCamp_mini_Ready",
0x64caa29b,"GoronCamp_mini_ResetGame",
0x77e8c612,"GoronCityMini_BeatGolem_Activated",
0x7a91e560,"GoronCityMini_BeatGolem_Beated",
0x7d4393fe,"GoronCityMini_BeatGolem_Finish",
0xcaf66155,"GoronCityMini_BeatGolem_Ready",
0x4da8f95f,"GoronMini_ImportGem_Activated",
0x518a1720,"GoronMini_ImportGem_Finish",
0xdaec34eb,"GoronMini_ImportGem_Ready",
0xd722240b,"GoronMini_WallCrackTBox_Activated",
0xa038e90a,"GoronMini_WallCrackTBox_Finish",
0xc45ba131,"GoronMini_WallCrackTBox_Get",
0xfbbe5dfd,"GoronMini_WallCrackTBox_GetOn",
0xbcf05c3a,"GoronMini_WallCrackTBox_Ready",
0xa10794b6,"GoronRock_Father_Cave",
0x6a4ea904,"Goron_Steakrock_Activated",
0xb573f5f3,"Goron_Steakrock_Finish",
0x7d5d1eb2,"Goron_Steakrock_Goron_get_Bigbomb",
0x2dc1ec67,"Goron_Steakrock_Ready",
0xa9a44306,"Goron_Steakrock_Step1",
0x10c5dcf4,"Goron_Steakrock_get_bigbomb",
0xed049cbb,"Goron_Steakrock_get_bigbomb_carry",
0x5b77d699,"GoronsRock_Activated",
0xb202a565,"GoronsRock_Break",
0x46f1f18b,"GoronsRock_Carry",
0xe8f1ecf0,"GoronsRock_Cooking",
0xe09823ca,"GoronsRock_Dish",
0xf5af32cb,"GoronsRock_DungeonAppear",
0x4eaa2268,"GoronsRock_Father_Disp",
0x7ba93bf0,"GoronsRock_Finish",
0x96eb7010,"GoronsRock_Ready",
0xe9a61f64,"GoronsRock_Step005",
0x80d7daaa,"GoronsRock_Step010",
0xabfa8969,"GoronsRock_Step020",
0xb2e1b828,"GoronsRock_Step030",
0xfda02eef,"GoronsRock_Step040",
0x0c91cd1f,"GotoZoraVillage_Activated",
0xed5efc6a,"GotoZoraVillage_Finish",
0x6884645a,"GotoZoraVillage_Ready",
0xece1cb3b,"GotoZoraVillage_Step1",
0x421063a0,"GreatFairy1_AllRevival",
0xd8ba3e28,"GreatFairy1_ChoiceExclude1",
0x41b36f92,"GreatFairy1_ChoiceExclude2",
0x611bd770,"GreatFairy1_Talk",
0x358eb150,"GreatFairy2_AllRevival",
0xf1728ada,"GreatFairy2_ChoiceExclude1",
0x687bdb60,"GreatFairy2_ChoiceExclude2",
0xe78fa5de,"GreatFairy2_Talk",
0xae2bfd3f,"GreatFairy3_AllRevival",
0x5f1a1b4b,"GreatFairy3_ChoiceExclude1",
0xc6134af1,"GreatFairy3_ChoiceExclude2",
0x2cd3767b,"GreatFairy3_Talk",
0xdab314b0,"GreatFairy4_AllRevival",
0xa2e3e33e,"GreatFairy4_ChoiceExclude1",
0x3beab284,"GreatFairy4_ChoiceExclude2",
0x31d646c3,"GreatFairy4_Talk",
0x0c8b72af,"GreatFairy5_ChoiceExclude1",
0x95822315,"GreatFairy5_ChoiceExclude2",
0x063f4207,"GreatFairy5_HorseRLife",
0x18691dbd,"GreatFairy5_HorseRLife2nd",
0xfa8a9566,"GreatFairy5_Talk",
0x1bf814fa,"GreatFairyBeforeRevival_DispNameFlag",
0x9b6c7627,"Guardian_BootUp",
0xcad46359,"GuideP_ChallengePoint",
0x7cd7c0e6,"GuideSPlus_GurdJust",
0x9853c5e9,"GuideSPlus_ItemShortCut",
0xa54459f9,"GuideSPlus_KaihiJust",
0xa5a46d63,"GuideSPlus_Squat",
0xb43ab694,"GuideSPlus_Surfing",
0x58b5539d,"GuideSPlus_WeaponsShortCut",
0xaa3614d7,"GuideS_ClimbSlip",
0x0612bfb1,"GuideS_ElectricDamage",
0xcbd6bdde,"GuideS_HeadShoot",
0xf4d09710,"GuideS_Recipe",
0x2010fe03,"GuideS_RelicControl",
0x296fedd1,"GuideS_SandSealError",
0x787b62bf,"GuideS_StealthAttack",
0xd57914bd,"GuideS_WarningBurn",
0xfd82511d,"GuideS_WarningBurnLv2",
0x48e083bf,"GuideS_WarningCold",
0x49d5b2d3,"GuideS_WarningColdLv2",
0x707b63c7,"GuideS_WarningGanbari",
0x42e6f352,"GuideS_WarningHot",
0x2eb32de9,"GuideS_WarningHotLv2",
0x75f52324,"GuideS_WeaponBurning",
0x5adc9d73,"GuideS_WeaponDestruction",
0xfaa12083,"GuideS_WeaponThrowAttack",
0x76223e09,"GuideS_WolfLead",
0x201e279c,"Guide_Attack",
0x4f901d56,"Guide_Attention",
0x79f32ea6,"Guide_BackStep",
0xa8a78c7e,"Guide_Bow",
0xdf7edd1c,"Guide_GurdJust",
0x8bd0100f,"Guide_ItemBomb",
0xcac28442,"Guide_ItemIceMaker",
0x8006b002,"Guide_ItemMagnetglove",
0x6f4d8c16,"Guide_ItemStopTimer",
0x9007f584,"Guide_ItemUse",
0x513a9194,"Guide_Jump",
0x316f9bce,"Guide_JumpAtack",
0x6ab8a0d7,"Guide_PadMenu",
0xd88810de,"Guide_Parashawl",
0xd4daf976,"Guide_Pause",
0x053d14d9,"Guide_PouchCarringMode",
0x4057938c,"Guide_Quest",
0x9aa5cb10,"Guide_SandSeal",
0x03ef3e48,"Guide_Scope",
0x794dd84f,"Guide_Shield",
0x5f4d6aca,"Guide_ShortCutArrow",
0xb857416d,"Guide_ShortCutBow",
0x6d3c038f,"Guide_ShortCutItem",
0xa4c49e69,"Guide_ShortCutShield",
0x1d9f43fb,"Guide_ShortCutSword",
0x5838dfad,"Guide_SideStep",
0xb47f2678,"Guide_SpinAtack",
0xd1278418,"Guide_Squat",
0xa12738f6,"Guide_Surfing",
0x4ca29e8c,"Guide_SystemMenu",
0x4cd32aac,"Guide_Throw",
0x52113f13,"Guide_Whistle",
0xf854fcfb,"Guide_amiibo",
0x9b68e91c,"Guide_amiibo_f",
0x61860601,"H-5_Animal_RupeeRabbit_A_1206289926",
0xa3a10b18,"H-7_Animal_RupeeRabbit_A_1918020349",
0x33f1156c,"H-7_Animal_RupeeRabbit_A_2052852149",
0xbd2b3fd0,"H-7_Animal_RupeeRabbit_A_4222375871",
0xfa297f96,"HateeluMini_Treasure_Activated",
0x917fefc9,"HateeluMini_Treasure_Finish",
0x95787094,"HateeluMini_Treasure_Ready",
0x4fa606f4,"Hateno013_Horse01",
0xb2b1155d,"HatenoDyeReset_End",
0xbfbfdedf,"HatenoDyeReset_Start",
0x267764ae,"HatenoDye_End",
0xa5bbd6db,"HatenoHarborNPC001_ship",
0x1932428f,"HatenoMini_BlueFire_Activated",
0x5d3c1f24,"HatenoMini_BlueFire_Carry",
0x3ddb4867,"HatenoMini_BlueFire_Finish",
0x5058a8e9,"HatenoMini_BlueFire_Meet",
0x8d269ebf,"HatenoMini_BlueFire_Ready",
0x60419f04,"HatenoMini_CameraBoy_Activated",
0x9d89dd46,"HatenoMini_CameraBoy_Bought",
0xf23a7ffd,"HatenoMini_CameraBoy_BuyPicture",
0x31db5775,"HatenoMini_CameraBoy_Buy_First",
0xa23c05ac,"HatenoMini_CameraBoy_Camera",
0x064769d3,"HatenoMini_CameraBoy_Chat",
0xb9e900b1,"HatenoMini_CameraBoy_Finish",
0xed231e29,"HatenoMini_CameraBoy_FinishFirst",
0xec6366dc,"HatenoMini_CameraBoy_FullComp",
0x21cc8e6d,"HatenoMini_CameraBoy_Get",
0xf6d5187b,"HatenoMini_CameraBoy_GetReward",
0x012f96ac,"HatenoMini_CameraBoy_Ready",
0xc8dd13c2,"HatenoMini_CameraBoy_Shot",
0x32e08f56,"HatenoMini_DeathDevil_Activated",
0x29c70529,"HatenoMini_DeathDevil_Finish",
0x90379fa3,"HatenoMini_DeathDevil_Ready",
0x5c06fd0b,"HatenoMini_DeathDevil_Step",
0x3f1a6cff,"HatenoMini_DevilSeal_Activated",
0xb96b1396,"HatenoMini_DevilSeal_Finish",
0x11196943,"HatenoMini_DevilSeal_GiveUtsuwa",
0x833cb1ac,"HatenoMini_DevilSeal_Ready",
0x3bcb6f6a,"HatenoMini_Fish",
0xcab47e27,"HatenoMini_GoatThief_Activated",
0x5b80b494,"HatenoMini_GoatThief_Extermination",
0x11ff7222,"HatenoMini_GoatThief_Finished",
0xdf013ce8,"HatenoMini_GoatThief_Ready",
0x388018b3,"HatenoMini_Liver",
0xdced5d91,"HatenoMini_LoveInsects_Activated",
0xda8a35d3,"HatenoMini_LoveInsects_Answer",
0x209e94ee,"HatenoMini_LoveInsects_Finish",
0x7fea73af,"HatenoMini_LoveInsects_Ready",
0xc4780872,"HatenoMini_LoveInsects_Report",
0x5a16a99b,"HatenoMini_MyHome_Activated",
0x1ab14a89,"HatenoMini_MyHome_EnokidaHummer",
0xf2730a1d,"HatenoMini_MyHome_Finish",
0x0c4780b5,"HatenoMini_MyHome_Furniture",
0x1f06cecb,"HatenoMini_MyHome_KatsuradaHummer",
0x9ff17d89,"HatenoMini_MyHome_Ready",
0x8735030c,"HatenoMini_MyHome_Ready_Toryo_First",
0xeafe4805,"HatenoMini_MyHome_Repurchase",
0x86ba674c,"HatenoMini_MyHome_StopNow",
0xe5390473,"HatenoMini_MyHome_Wood",
0x2ec1e622,"HatenoMini_Net_Activated",
0xa8402e0c,"HatenoMini_Net_Findbro",
0x0893709e,"HatenoMini_Net_Findsis",
0x6aa87cae,"HatenoMini_Net_Finished",
0xc8740a47,"HatenoMini_Net_Ready",
0xc5979c2a,"HatenoMini_RoastBoar",
0xa95974ce,"HatenoMini_ThreeTree_Activated",
0xfa99423f,"HatenoMini_ThreeTree_Finished",
0xf5f538c2,"HatenoMini_ThreeTree_Ready",
0x27e8f538,"HatenoMini_Treasure",
0x9f3bab43,"HatenoMini_WeaponMania_Activated",
0x43753ed8,"HatenoMini_WeaponMania_Finish",
0x922c9cb2,"HatenoMini_WeaponMania_Ready",
0xb294bd8d,"HatenoMini_WeaponMania_Weapon01",
0x2b9dec37,"HatenoMini_WeaponMania_Weapon02",
0x5c9adca1,"HatenoMini_WeaponMania_Weapon03",
0xc2fe4902,"HatenoMini_WeaponMania_Weapon04",
0xb5f97994,"HatenoMini_WeaponMania_Weapon05",
0x2cf0282e,"HatenoMini_WeaponMania_Weapon06",
0x5bf718b8,"HatenoMini_WeaponMania_Weapon07",
0xcb480529,"HatenoMini_WeaponMania_Weapon08",
0xbc4f35bf,"HatenoMini_WeaponMania_Weapon09",
0xdc88bc5a,"HatenoMini_WeaponMania_Weapon10",
0xab8f8ccc,"HatenoMini_WeaponMania_Weapon11",
0x3286dd76,"HatenoMini_WeaponMania_Weapon12",
0x4581ede0,"HatenoMini_WeaponMania_Weapon13",
0xdbe57843,"HatenoMini_WeaponMania_Weapon14",
0xace248d5,"HatenoMini_WeaponMania_Weapon15",
0x35eb196f,"HatenoMini_WeaponMania_Weapon16",
0x42ec29f9,"HatenoMini_WeaponMania_Weapon17",
0xd2533468,"HatenoMini_WeaponMania_Weapon18",
0xa55404fe,"HatenoMini_WeaponMania_Weapon19",
0x99bc40e7,"HatenoMini_WhiteDragon_Activated",
0xc3207937,"HatenoMini_WhiteDragon_Finish",
0xa27dbaa1,"HatenoMini_WhiteDragon_Play",
0xa20cb827,"HatenoMini_WhiteDragon_Ready",
0x26691746,"HatenoMini_WhiteDragon_Step1",
0x951be3a0,"HatenoNPC000_First",
0x94aead58,"HatenoNPC000_Labo",
0x66c01d17,"HatenoNPC001_Dinner",
0x336ce814,"HatenoNPC001_First",
0xb65701f6,"HatenoNPC001_First_test",
0x9fdb5521,"HatenoNPC001_Work",
0x7517358a,"HatenoNPC002_talk",
0x6154c7b3,"HatenoNPC004_First",
0x839c68db,"HatenoNPC004_Fukuya",
0x360903be,"HatenoNPC004_OK",
0x1d051a2a,"HatenoNPC004_Somemonoya",
0xadaedf27,"HatenoNPC004_Yadoya",
0x561beebb,"HatenoNPC004_Yorozu",
0x0083a673,"HatenoNPC009_First",
0xffc6e88a,"HatenoNPC011_First",
0xce2ef217,"HatenoNPC012_First",
0x6859f9a3,"HatenoNPC013_First",
0xa7ed76c4,"HatenoNPC013_FirstGanon",
0xc8693fc9,"HatenoNPC013_GanonTalk",
0x08e2623a,"HatenoNPC014_Favorite",
0x45bc3656,"HatenoNPC014_Ride_Favorite",
0x3a61d604,"HatenoNPC016_First",
0x9c16ddb0,"HatenoNPC017_First",
0xaec87248,"HatenoNPC018_First_Ready",
0xb03294fb,"HatenoNPC019_Npc_SoldOut",
0x40a1f5f4,"HatenoNPC022_First",
0xb65c0a98,"HatenoNPC022_Ganon",
0xaf8bc821,"HatenoNPC022_Heard",
0x6c85d9be,"HatenoNPC022_Here",
0xd7ff5522,"HatenoNPC022_Maji",
0x01dc501a,"HatenoNPC022_Reward",
0xd181dec8,"HatenoNPC022_Zelda",
0x7358638d,"HatenoNPC023_Traveler",
0x8506cb7a,"HatenoNPC025_First",
0xd18c9ff4,"HatenoNPC025_FirstFinish",
0xd1680018,"HatenoNPC025_GetMilk",
0xe56b66ad,"HatenoNPC025_NotMilk",
0xb4eed1e7,"HatenoNPC026_First",
0x4565a199,"HatenoNPC028_DemonStatue_FinishFirst",
0x57d640f5,"HatenoNPC028_DemonStatue_NormalFirst",
0x239b5ad7,"HatenoNPC028_Produce",
0xe00947ec,"HatenoNPC028_Rail1Goal",
0xa6dc602c,"HatenoNPC030_Repurchase_First",
0xcb35d549,"HatenoNPC030_Rest",
0xe587021b,"HatenoNPC030_ReturnHatenoFirst",
0xc8cf2891,"HatenoNPC030_Surprise",
0x5c0a82b1,"HatenoNPC032_ArrowStand1",
0xc503d30b,"HatenoNPC032_ArrowStand2",
0xb204e39d,"HatenoNPC032_ArrowStand3",
0xbe39c035,"HatenoNPC032_Bed",
0x29b25117,"HatenoNPC032_CantGo",
0xab4e3446,"HatenoNPC032_CompInDoor",
0xd0a26ecb,"HatenoNPC032_CompOutDoor",
0x35d83b61,"HatenoNPC032_CompStand",
0xf10c01bf,"HatenoNPC032_Door",
0x23ff85d7,"HatenoNPC032_Farm",
0xfff96d38,"HatenoNPC032_IchikaraFirst",
0xacc35e02,"HatenoNPC032_Ikuwayo",
0xf4771c7f,"HatenoNPC032_Ikuwayo2",
0x83702ce9,"HatenoNPC032_Ikuwayo3",
0x1d14b94a,"HatenoNPC032_Ikuwayo4",
0x6a1389dc,"HatenoNPC032_Ikuwayo5",
0xf31ad866,"HatenoNPC032_Ikuwayo6",
0x841de8f0,"HatenoNPC032_Ikuwayo7",
0x14a2f561,"HatenoNPC032_Ikuwayo8",
0x756047ca,"HatenoNPC032_Light",
0xc347115f,"HatenoNPC032_MarriedFirst",
0x56e0bdc5,"HatenoNPC032_Nameplate",
0xa3080baf,"HatenoNPC032_Repurchase_First",
0x913304ea,"HatenoNPC032_ReturnHatenoFirst",
0xb578dece,"HatenoNPC032_ShieldStand1",
0x2c718f74,"HatenoNPC032_ShieldStand2",
0x5b76bfe2,"HatenoNPC032_ShieldStand3",
0xcc3af8ac,"HatenoNPC032_Surprise",
0xccd70b4e,"HatenoNPC032_Tree",
0xa295b7d1,"HatenoNPC032_WeaponStand1",
0x3b9ce66b,"HatenoNPC032_WeaponStand2",
0x4c9bd6fd,"HatenoNPC032_WeaponStand3",
0xa6723270,"HatenoNPC032_Whats",
0x67c26668,"HatenoNPC032_dAnce",
0x8fe7d0a9,"HatenoNPC033_DontCome",
0x2a7cfede,"HatenoNPC033_First",
0x0a5b2193,"HatenoNPC033_GetEgg",
0x73d927d2,"HatenoNPC033_GetEgg2",
0x04de1744,"HatenoNPC033_GetEgg3",
0x9aba82e7,"HatenoNPC033_GetEgg4",
0x662285dd,"HatenoNPC033_MaigoKokkoON",
0x5fad0843,"HatenoNPC_018_First",
0xa84eea14,"HatenoVillage001_talk",
0x39ac0624,"Hateno_BlackSmith",
0x85db2043,"Hateno_SheikPad_PowerUp_Activated",
0x0d5699d4,"Hateno_SheikPad_PowerUp_Eemon",
0x0dba0246,"Hateno_SheikPad_PowerUp_Explain",
0xbae8665c,"Hateno_SheikPad_PowerUp_Finish",
0x0c4f1cd6,"Hateno_SheikPad_PowerUp_First",
0xb6ab772e,"Hateno_SheikPad_PowerUp_Ready",
0x64622a86,"HiddenKorok_Complete",
0xfaefa206,"HiddenKorok_First",
0x737adc41,"HigakkareMini_RedDragon_Activated",
0x79ae5a49,"HigakkareMini_RedDragon_Finish",
0xfc19c3d3,"HigakkareMini_RedDragon_Ready",
0x787c6cb2,"HigakkareMini_RedDragon_Step1",
0xaecc8dc6,"HigakkareMini_StrangeMan_Activated",
0xa9f24379,"HigakkareMini_StrangeMan_Finish",
0xc0160436,"HigakkareMini_StrangeMan_Ready",
0xaacc2241,"Higakkare_NPC001_First",
0x8f62dff2,"Higakkare_NPC001_Money",
0x42f319f4,"Higakkare_NPC001_Second",
0x9b2438dc,"Higakkare_NPC002_First",
0x3d533368,"Higakkare_NPC003_First",
0xddf05854,"HorseAssociationCustum001_DispNameFlag",
0xa1917d8f,"HorseAssociationCustum002_DispNameFlag",
0x3c9e9cf9,"HorseAssociationCustum003_DispNameFlag",
0x59533639,"HorseAssociationCustum004_DispNameFlag",
0xd62ac137,"HorseGod001_DispNameFlag",
0xb4575e8f,"Horse_IsInHyruleCastleArea",
0x74fc35e2,"Horse_IsRide",
0x3975f71a,"Horse_Is_LastRidden_Owned",
0x28d159f9,"Horse_Is_Registration",
0x666aa36a,"HutagoHatago_Ch_001_Activated",
0x61ad980e,"HutagoHatago_Ch_001_Finish",
0xf27898fc,"HutagoHatago_Ch_001_First",
0x6391b806,"HutagoHatago_Ch_001_Get",
0x489cf304,"HutagoHatago_Ch_001_Ready",
0x4ee92748,"HutagoHatago_Ch_001_Retry",
0xbbf8ac62,"HutagoYama_Osonaemono",
0xd48d5b2b,"Hutago_Hatago_Ch_001_Tutorial",
0xfaf4414f,"HyperGoddess_Camera_First",
0x07a9bfba,"HyruleDepthMini_WhiteHorse_Activated",
0x0367eb1d,"HyruleDepthMini_WhiteHorse_Finish",
0x5ff23f58,"HyruleDepthMini_WhiteHorse_Ready",
0xe8addb44,"HyrulePlainMini_Balloon_Activated",
0x0a40d62b,"HyrulePlainMini_Balloon_Finish",
0xce832254,"HyrulePlainMini_Balloon_Ready",
0x452806b4,"I-1_Animal_RupeeRabbit_A_2653143848",
0x18ff45b9,"I-1_Animal_RupeeRabbit_A_3694619243",
0xa5185d7d,"I-1_Animal_RupeeRabbit_A_3999689016",
0x94b15fe8,"I-1_Animal_RupeeRabbit_A_4164653337",
0x76a6d4c1,"Ice_Close_Challenge_01",
0x86fedbac,"Ichikara_NPC010_SoldOut",
0x275e25e0,"Ichikara_NPC025_SoldOut",
0xc565e1f9,"Ichikara_NPC050_First",
0x8f157bd6,"Ichikara_NPC050_SoldOut",
0xb9f2ff0d,"IkkakuTest_Medori",
0x8efe0412,"Ikkaku_Test_Zulee",
0x2e76f2ce,"Imagawa_test05_Activated",
0x826d4d31,"Imagawa_test05_Finish",
0xeb42de8c,"Imagawa_test05_Ready",
0x986c7cc1,"Imagawa_test05_Step",
0x30c83360,"Imagawa_test6_Activated",
0x46e26cb9,"Imagawa_test6_Finish",
0x0529ee75,"Imagawa_test6_Ready",
0xfd3da9c3,"Imagawa_test6_Step",
0xa6f15bb2,"Imagawa_tunobue_Activated",
0x8fe21f7b,"Imagawa_tunobue_Finish",
0x097aaa06,"Imagawa_tunobue_Ready",
0x8d1f0567,"Imagawa_tunobue_Step1",
0x6799c962,"InSnatchArea",
0x8d87b86a,"IsBreak_Tower_02",
0xe6ad6158,"IsEnable_SeakSensor_Daichi",
0x85439bad,"IsEnable_SeakSensor_Slot0",
0xf244ab3b,"IsEnable_SeakSensor_Slot1",
0x6b4dfa81,"IsEnable_SeakSensor_Slot2",
0xd713ec68,"IsFirstFairyEqualsGreatFairy02",
0x2c4f5068,"IsGanonLiving",
0x022d3770,"IsGet_AncientArrow",
0xb1b5b94b,"IsGet_Animal_Insect_A",
0x3266b32a,"IsGet_Animal_Insect_AA",
0xab6fe290,"IsGet_Animal_Insect_AB",
0x28bce8f1,"IsGet_Animal_Insect_B",
0x5fbbd867,"IsGet_Animal_Insect_C",
0xb6d87d52,"IsGet_Animal_Insect_E",
0x2fd12ce8,"IsGet_Animal_Insect_F",
0x58d61c7e,"IsGet_Animal_Insect_G",
0xc86901ef,"IsGet_Animal_Insect_H",
0xbf6e3179,"IsGet_Animal_Insect_I",
0xb803f560,"IsGet_Animal_Insect_M",
0x210aa4da,"IsGet_Animal_Insect_N",
0xdb0599b9,"IsGet_Animal_Insect_P",
0xac02a92f,"IsGet_Animal_Insect_Q",
0x350bf895,"IsGet_Animal_Insect_R",
0x420cc803,"IsGet_Animal_Insect_S",
0xdc685da0,"IsGet_Animal_Insect_T",
0xd5de118b,"IsGet_Animal_Insect_X",
0x0c7b0a99,"IsGet_App_ElectricRelic",
0x7532e9f2,"IsGet_App_FireRelic",
0x75253bb4,"IsGet_App_WaterRelic",
0xa5ab9adf,"IsGet_App_WindRelic",
0x46711f96,"IsGet_Armor_001_Head",
0xeeae10d4,"IsGet_Armor_001_Lower",
0x8e6bb46b,"IsGet_Armor_001_Upper",
0xc0e56d38,"IsGet_Armor_002_Head",
0xdf460a49,"IsGet_Armor_002_Lower",
0xbf83aef6,"IsGet_Armor_002_Upper",
0x0bb9be9d,"IsGet_Armor_003_Head",
0x793101fd,"IsGet_Armor_003_Lower",
0x19f4a542,"IsGet_Armor_003_Upper",
0x16bc8e25,"IsGet_Armor_004_Head",
0xbc963f73,"IsGet_Armor_004_Lower",
0xdc539bcc,"IsGet_Armor_004_Upper",
0xdde05d80,"IsGet_Armor_005_Head",
0x1ae134c7,"IsGet_Armor_005_Lower",
0x7a249078,"IsGet_Armor_005_Upper",
0x5b742f2e,"IsGet_Armor_006_Head",
0x2b092e5a,"IsGet_Armor_006_Lower",
0x4bcc8ae5,"IsGet_Armor_006_Upper",
0x9028fc8b,"IsGet_Armor_007_Head",
0x8d7e25ee,"IsGet_Armor_007_Lower",
0xedbb8151,"IsGet_Armor_007_Upper",
0x617e4e5e,"IsGet_Armor_008_Head",
0x7b365507,"IsGet_Armor_008_Lower",
0x1bf3f1b8,"IsGet_Armor_008_Upper",
0xaa229dfb,"IsGet_Armor_009_Head",
0xdd415eb3,"IsGet_Armor_009_Lower",
0xbd84fa0c,"IsGet_Armor_009_Upper",
0xe0061422,"IsGet_Armor_011_Head",
0x2204104a,"IsGet_Armor_011_Lower",
0x42c1b4f5,"IsGet_Armor_011_Upper",
0x6692668c,"IsGet_Armor_012_Head",
0x13ec0ad7,"IsGet_Armor_012_Lower",
0x7329ae68,"IsGet_Armor_012_Upper",
0xb0cb8591,"IsGet_Armor_014_Head",
0x703c3fed,"IsGet_Armor_014_Lower",
0x10f99b52,"IsGet_Armor_014_Upper",
0x7b975634,"IsGet_Armor_015_Head",
0xd64b3459,"IsGet_Armor_015_Lower",
0xb68e90e6,"IsGet_Armor_015_Upper",
0x365ff73f,"IsGet_Armor_017_Head",
0x41d42570,"IsGet_Armor_017_Lower",
0x211181cf,"IsGet_Armor_017_Upper",
0x1ab2dd1a,"IsGet_Armor_020_Head",
0x0afc1c1d,"IsGet_Armor_020_Lower",
0x6a39b8a2,"IsGet_Armor_020_Upper",
0xd1ee0ebf,"IsGet_Armor_021_Head",
0xac8b17a9,"IsGet_Armor_021_Lower",
0xcc4eb316,"IsGet_Armor_021_Upper",
0x577a7c11,"IsGet_Armor_022_Head",
0x81239f0c,"IsGet_Armor_024_Head",
0x4a7f4ca9,"IsGet_Armor_025_Head",
0xcceb3e07,"IsGet_Armor_026_Head",
0x07b7eda2,"IsGet_Armor_027_Head",
0xf6e15f77,"IsGet_Armor_028_Head",
0x3dbd8cd2,"IsGet_Armor_029_Head",
0xec08471d,"IsGet_Armor_035_Head",
0x946e3324,"IsGet_Armor_035_Lower",
0xf4ab979b,"IsGet_Armor_035_Upper",
0x6a9c35b3,"IsGet_Armor_036_Head",
0xa58629b9,"IsGet_Armor_036_Lower",
0xc5438d06,"IsGet_Armor_036_Upper",
0xa1c0e616,"IsGet_Armor_037_Head",
0x03f1220d,"IsGet_Armor_037_Lower",
0x633486b2,"IsGet_Armor_037_Upper",
0x9bca8766,"IsGet_Armor_039_Head",
0x53ce5950,"IsGet_Armor_039_Lower",
0x330bfdef,"IsGet_Armor_039_Upper",
0x7962e820,"IsGet_Armor_040_Head",
0xcc93159a,"IsGet_Armor_040_Lower",
0xac56b125,"IsGet_Armor_040_Upper",
0x34aa492b,"IsGet_Armor_042_Head",
0x5b0c04b3,"IsGet_Armor_042_Lower",
0x3bc9a00c,"IsGet_Armor_042_Upper",
0xfd7b0f07,"IsGet_Armor_043_Lower",
0x9dbeabb8,"IsGet_Armor_043_Upper",
0x58199536,"IsGet_Armor_044_Upper",
0x29af7993,"IsGet_Armor_045_Head",
0xaf3b0b3d,"IsGet_Armor_046_Head",
0xaf4320a0,"IsGet_Armor_046_Lower",
0xcf86841f,"IsGet_Armor_046_Upper",
0x95316a4d,"IsGet_Armor_048_Head",
0xff7c5bfd,"IsGet_Armor_048_Lower",
0x9fb9ff42,"IsGet_Armor_048_Upper",
0x590b5049,"IsGet_Armor_049_Lower",
0x5981913a,"IsGet_Armor_053_Head",
0x31d10f99,"IsGet_Armor_053_Lower",
0x5114ab26,"IsGet_Armor_053_Upper",
0x8fd87227,"IsGet_Armor_055_Head",
0x094c0089,"IsGet_Armor_056_Head",
0xeefdf909,"IsGet_Armor_060_Head",
0x8eb612e7,"IsGet_Armor_060_Lower",
0xee73b658,"IsGet_Armor_060_Upper",
0x25a12aac,"IsGet_Armor_061_Head",
0x28c11953,"IsGet_Armor_061_Lower",
0x4804bdec,"IsGet_Armor_061_Upper",
0xa3355802,"IsGet_Armor_062_Head",
0x192903ce,"IsGet_Armor_062_Lower",
0x79eca771,"IsGet_Armor_062_Upper",
0x68698ba7,"IsGet_Armor_063_Head",
0xbf5e087a,"IsGet_Armor_063_Lower",
0xdf9bacc5,"IsGet_Armor_063_Upper",
0x756cbb1f,"IsGet_Armor_064_Head",
0x7af936f4,"IsGet_Armor_064_Lower",
0x1a3c924b,"IsGet_Armor_064_Upper",
0xbe3068ba,"IsGet_Armor_065_Head",
0xdc8e3d40,"IsGet_Armor_065_Lower",
0xbc4b99ff,"IsGet_Armor_065_Upper",
0x38a41a14,"IsGet_Armor_066_Head",
0xed6627dd,"IsGet_Armor_066_Lower",
0x8da38362,"IsGet_Armor_066_Upper",
0xf3f8c9b1,"IsGet_Armor_067_Head",
0x4b112c69,"IsGet_Armor_067_Lower",
0x2bd488d6,"IsGet_Armor_067_Upper",
0x83d62118,"IsGet_Armor_071_Head",
0xe46b19cd,"IsGet_Armor_071_Lower",
0x84aebd72,"IsGet_Armor_071_Upper",
0x054253b6,"IsGet_Armor_072_Head",
0xd5830350,"IsGet_Armor_072_Lower",
0xb546a7ef,"IsGet_Armor_072_Upper",
0xce1e8013,"IsGet_Armor_073_Head",
0x73f408e4,"IsGet_Armor_073_Lower",
0x1331ac5b,"IsGet_Armor_073_Upper",
0xd31bb0ab,"IsGet_Armor_074_Head",
0xb653366a,"IsGet_Armor_074_Lower",
0xd69692d5,"IsGet_Armor_074_Upper",
0x1847630e,"IsGet_Armor_075_Head",
0x10243dde,"IsGet_Armor_075_Lower",
0x70e19961,"IsGet_Armor_075_Upper",
0x9ed311a0,"IsGet_Armor_076_Head",
0x21cc2743,"IsGet_Armor_076_Lower",
0x410983fc,"IsGet_Armor_076_Upper",
0x558fc205,"IsGet_Armor_077_Head",
0x87bb2cf7,"IsGet_Armor_077_Lower",
0xe77e8848,"IsGet_Armor_077_Upper",
0xa4d970d0,"IsGet_Armor_078_Head",
0x71f35c1e,"IsGet_Armor_078_Lower",
0x1136f8a1,"IsGet_Armor_078_Upper",
0x6f85a375,"IsGet_Armor_079_Head",
0xd78457aa,"IsGet_Armor_079_Lower",
0xb741f315,"IsGet_Armor_079_Upper",
0x3856f0fa,"IsGet_Armor_083_Head",
0xaad41a48,"IsGet_Armor_083_Lower",
0xca11bef7,"IsGet_Armor_083_Upper",
0x2553c042,"IsGet_Armor_084_Head",
0x6f7324c6,"IsGet_Armor_084_Lower",
0x0fb68079,"IsGet_Armor_084_Upper",
0xee0f13e7,"IsGet_Armor_085_Head",
0xc9042f72,"IsGet_Armor_085_Lower",
0xa9c18bcd,"IsGet_Armor_085_Upper",
0x689b6149,"IsGet_Armor_086_Head",
0xf8ec35ef,"IsGet_Armor_086_Lower",
0x98299150,"IsGet_Armor_086_Upper",
0xa3c7b2ec,"IsGet_Armor_087_Head",
0x5e9b3e5b,"IsGet_Armor_087_Lower",
0x3e5e9ae4,"IsGet_Armor_087_Upper",
0x52910039,"IsGet_Armor_088_Head",
0xa8d34eb2,"IsGet_Armor_088_Lower",
0xc816ea0d,"IsGet_Armor_088_Upper",
0x99cdd39c,"IsGet_Armor_089_Head",
0x0ea44506,"IsGet_Armor_089_Lower",
0x6e61e1b9,"IsGet_Armor_089_Upper",
0x18b589e0,"IsGet_Armor_090_Head",
0x5796004b,"IsGet_Armor_090_Lower",
0x3753a4f4,"IsGet_Armor_090_Upper",
0x48781853,"IsGet_Armor_095_Head",
0x05ae2fec,"IsGet_Armor_095_Lower",
0x656b8b53,"IsGet_Armor_095_Upper",
0xceec6afd,"IsGet_Armor_096_Head",
0x34463571,"IsGet_Armor_096_Lower",
0x548391ce,"IsGet_Armor_096_Upper",
0x05b0b958,"IsGet_Armor_097_Head",
0x92313ec5,"IsGet_Armor_097_Lower",
0xf2f49a7a,"IsGet_Armor_097_Upper",
0xf4e60b8d,"IsGet_Armor_098_Head",
0x64794e2c,"IsGet_Armor_098_Lower",
0x04bcea93,"IsGet_Armor_098_Upper",
0x3fbad828,"IsGet_Armor_099_Head",
0xc20e4598,"IsGet_Armor_099_Lower",
0xa2cbe127,"IsGet_Armor_099_Upper",
0x4187ccad,"IsGet_Armor_100_Head",
0x5fa20f23,"IsGet_Armor_100_Lower",
0x3f67ab9c,"IsGet_Armor_100_Upper",
0x8adb1f08,"IsGet_Armor_101_Head",
0xf9d50497,"IsGet_Armor_101_Lower",
0x9910a028,"IsGet_Armor_101_Upper",
0x0c4f6da6,"IsGet_Armor_102_Head",
0xc83d1e0a,"IsGet_Armor_102_Lower",
0xa8f8bab5,"IsGet_Armor_102_Upper",
0xc713be03,"IsGet_Armor_103_Head",
0x6e4a15be,"IsGet_Armor_103_Lower",
0x0e8fb101,"IsGet_Armor_103_Upper",
0xda168ebb,"IsGet_Armor_104_Head",
0xabed2b30,"IsGet_Armor_104_Lower",
0xcb288f8f,"IsGet_Armor_104_Upper",
0x114a5d1e,"IsGet_Armor_105_Head",
0x0d9a2084,"IsGet_Armor_105_Lower",
0x6d5f843b,"IsGet_Armor_105_Upper",
0x97de2fb0,"IsGet_Armor_106_Head",
0x3c723a19,"IsGet_Armor_106_Lower",
0x5cb79ea6,"IsGet_Armor_106_Upper",
0x2cac14bc,"IsGet_Armor_111_Head",
0x357f0409,"IsGet_Armor_111_Lower",
0x55baa0b6,"IsGet_Armor_111_Upper",
0xaa386612,"IsGet_Armor_112_Head",
0x04971e94,"IsGet_Armor_112_Lower",
0x6452ba2b,"IsGet_Armor_112_Upper",
0x6164b5b7,"IsGet_Armor_113_Head",
0xa2e01520,"IsGet_Armor_113_Lower",
0xc225b19f,"IsGet_Armor_113_Upper",
0x7c61850f,"IsGet_Armor_114_Head",
0x67472bae,"IsGet_Armor_114_Lower",
0x07828f11,"IsGet_Armor_114_Upper",
0xb73d56aa,"IsGet_Armor_115_Head",
0x901d9e38,"IsGet_Armor_116_Upper",
0xfaf5f7a1,"IsGet_Armor_117_Head",
0x0ba34574,"IsGet_Armor_118_Head",
0xc0ff96d1,"IsGet_Armor_119_Head",
0xd618dd84,"IsGet_Armor_120_Head",
0x1d440e21,"IsGet_Armor_121_Head",
0x9bd07c8f,"IsGet_Armor_122_Head",
0x508caf2a,"IsGet_Armor_123_Head",
0x4d899f92,"IsGet_Armor_124_Head",
0x86d54c37,"IsGet_Armor_125_Head",
0x00413e99,"IsGet_Armor_126_Head",
0xcb1ded3c,"IsGet_Armor_127_Head",
0x3a4b5fe9,"IsGet_Armor_128_Head",
0xf1178c4c,"IsGet_Armor_129_Head",
0x706fd630,"IsGet_Armor_130_Head",
0xbb330595,"IsGet_Armor_131_Head",
0x3da7773b,"IsGet_Armor_132_Head",
0xf6fba49e,"IsGet_Armor_133_Head",
0xebfe9426,"IsGet_Armor_134_Head",
0x20a24783,"IsGet_Armor_135_Head",
0xa636352d,"IsGet_Armor_136_Head",
0x6d6ae688,"IsGet_Armor_137_Head",
0x9c3c545d,"IsGet_Armor_138_Head",
0x576087f8,"IsGet_Armor_139_Head",
0xb5c8e8be,"IsGet_Armor_140_Head",
0xdbe801d9,"IsGet_Armor_140_Lower",
0x7d9f0a6d,"IsGet_Armor_141_Lower",
0x88c2eb01,"IsGet_Armor_148_Upper",
0x2eb5e0b5,"IsGet_Armor_149_Upper",
0x7787a5f8,"IsGet_Armor_150_Upper",
0xd1f0ae4c,"IsGet_Armor_151_Upper",
0x80dd106e,"IsGet_Armor_152_Lower",
0x26aa1bda,"IsGet_Armor_153_Lower",
0xe30d2554,"IsGet_Armor_154_Lower",
0x457a2ee0,"IsGet_Armor_155_Lower",
0x7492347d,"IsGet_Armor_156_Lower",
0xd2e53fc9,"IsGet_Armor_157_Lower",
0x24ad4f20,"IsGet_Armor_158_Lower",
0x82da4494,"IsGet_Armor_159_Lower",
0x2257f997,"IsGet_Armor_160_Head",
0x99cd06a4,"IsGet_Armor_160_Lower",
0xf908a21b,"IsGet_Armor_160_Upper",
0x35a2a285,"IsGet_Armor_170_Upper",
0xcf08cb4e,"IsGet_Armor_200_Head",
0x662f33e6,"IsGet_Armor_200_Lower",
0x06ea9759,"IsGet_Armor_200_Upper",
0x045418eb,"IsGet_Armor_201_Head",
0xc0583852,"IsGet_Armor_201_Lower",
0xa09d9ced,"IsGet_Armor_201_Upper",
0x82c06a45,"IsGet_Armor_202_Head",
0xf1b022cf,"IsGet_Armor_202_Lower",
0x91758670,"IsGet_Armor_202_Upper",
0x499cb9e0,"IsGet_Armor_203_Head",
0x57c7297b,"IsGet_Armor_203_Lower",
0x37028dc4,"IsGet_Armor_203_Upper",
0x54998958,"IsGet_Armor_204_Head",
0x926017f5,"IsGet_Armor_204_Lower",
0xf2a5b34a,"IsGet_Armor_204_Upper",
0x9fc55afd,"IsGet_Armor_205_Head",
0x34171c41,"IsGet_Armor_205_Lower",
0x54d2b8fe,"IsGet_Armor_205_Upper",
0x19512853,"IsGet_Armor_206_Head",
0x05ff06dc,"IsGet_Armor_206_Lower",
0x653aa263,"IsGet_Armor_206_Upper",
0xd20dfbf6,"IsGet_Armor_207_Head",
0xa3880d68,"IsGet_Armor_207_Lower",
0xc34da9d7,"IsGet_Armor_207_Upper",
0x235b4923,"IsGet_Armor_208_Head",
0x55c07d81,"IsGet_Armor_208_Lower",
0x3505d93e,"IsGet_Armor_208_Upper",
0xe8079a86,"IsGet_Armor_209_Head",
0xf3b77635,"IsGet_Armor_209_Lower",
0x9372d28a,"IsGet_Armor_209_Upper",
0x697fc0fa,"IsGet_Armor_210_Head",
0xaa853378,"IsGet_Armor_210_Lower",
0xca4097c7,"IsGet_Armor_210_Upper",
0xa223135f,"IsGet_Armor_211_Head",
0x0cf238cc,"IsGet_Armor_211_Lower",
0x6c379c73,"IsGet_Armor_211_Upper",
0x24b761f1,"IsGet_Armor_212_Head",
0x3d1a2251,"IsGet_Armor_212_Lower",
0x5ddf86ee,"IsGet_Armor_212_Upper",
0xefebb254,"IsGet_Armor_213_Head",
0x9b6d29e5,"IsGet_Armor_213_Lower",
0xfba88d5a,"IsGet_Armor_213_Upper",
0xf2ee82ec,"IsGet_Armor_214_Head",
0x5eca176b,"IsGet_Armor_214_Lower",
0x3e0fb3d4,"IsGet_Armor_214_Upper",
0x39b25149,"IsGet_Armor_215_Head",
0xf8bd1cdf,"IsGet_Armor_215_Lower",
0x9878b860,"IsGet_Armor_215_Upper",
0xbf2623e7,"IsGet_Armor_216_Head",
0xc9550642,"IsGet_Armor_216_Lower",
0xa990a2fd,"IsGet_Armor_216_Upper",
0x747af042,"IsGet_Armor_217_Head",
0x6f220df6,"IsGet_Armor_217_Lower",
0x0fe7a949,"IsGet_Armor_217_Upper",
0x852c4297,"IsGet_Armor_218_Head",
0x996a7d1f,"IsGet_Armor_218_Lower",
0xf9afd9a0,"IsGet_Armor_218_Upper",
0x4e709132,"IsGet_Armor_219_Head",
0x3f1d76ab,"IsGet_Armor_219_Lower",
0x5fd8d214,"IsGet_Armor_219_Upper",
0x5897da67,"IsGet_Armor_220_Head",
0x93cb09c2,"IsGet_Armor_221_Head",
0x155f7b6c,"IsGet_Armor_222_Head",
0xde03a8c9,"IsGet_Armor_223_Head",
0xc3069871,"IsGet_Armor_224_Head",
0x085a4bd4,"IsGet_Armor_225_Head",
0x76321b3c,"IsGet_Armor_225_Lower",
0x16f7bf83,"IsGet_Armor_225_Upper",
0x8ece397a,"IsGet_Armor_226_Head",
0x47da01a1,"IsGet_Armor_226_Lower",
0x271fa51e,"IsGet_Armor_226_Upper",
0x4592eadf,"IsGet_Armor_227_Head",
0xe1ad0a15,"IsGet_Armor_227_Lower",
0x8168aeaa,"IsGet_Armor_227_Upper",
0xb4c4580a,"IsGet_Armor_228_Head",
0x17e57afc,"IsGet_Armor_228_Lower",
0x7720de43,"IsGet_Armor_228_Upper",
0x7f988baf,"IsGet_Armor_229_Head",
0xb1927148,"IsGet_Armor_229_Lower",
0xd157d5f7,"IsGet_Armor_229_Upper",
0xfee0d1d3,"IsGet_Armor_230_Head",
0xe8a03405,"IsGet_Armor_230_Lower",
0x886590ba,"IsGet_Armor_230_Upper",
0x35bc0276,"IsGet_Armor_231_Head",
0x4ed73fb1,"IsGet_Armor_231_Lower",
0x2e129b0e,"IsGet_Armor_231_Upper",
0xb32870d8,"IsGet_Armor_232_Head",
0x7f3f252c,"IsGet_Armor_232_Lower",
0x1ffa8193,"IsGet_Armor_232_Upper",
0x7874a37d,"IsGet_Armor_233_Head",
0xd9482e98,"IsGet_Armor_233_Lower",
0xb98d8a27,"IsGet_Armor_233_Upper",
0x657193c5,"IsGet_Armor_234_Head",
0x1cef1016,"IsGet_Armor_234_Lower",
0x7c2ab4a9,"IsGet_Armor_234_Upper",
0xc5cdc257,"IsGet_Armor_500_Head",
0x024e5e2f,"IsGet_Armor_500_Lower",
0x628bfa90,"IsGet_Armor_500_Upper",
0x0e9111f2,"IsGet_Armor_501_Head",
0xa439559b,"IsGet_Armor_501_Lower",
0xc4fcf124,"IsGet_Armor_501_Upper",
0xf514ebb9,"IsGet_Armor_502_Upper",
0x4359b0f9,"IsGet_Armor_503_Head",
0xbc2ffbd9,"IsGet_Armor_Default_Head",
0x68f2b68e,"IsGet_Armor_Default_Upper",
0xa5934058,"IsGet_BeeHome",
0xd63aa760,"IsGet_BombArrow_A",
0x33d7b8d3,"IsGet_ElectricArrow",
0x158921e5,"IsGet_FireArrow",
0xae33f926,"IsGet_GameRomHorseReins_01",
0x373aa89c,"IsGet_GameRomHorseReins_02",
0x403d980a,"IsGet_GameRomHorseReins_03",
0xde590da9,"IsGet_GameRomHorseReins_04",
0xa95e3d3f,"IsGet_GameRomHorseReins_05",
0x25b4a466,"IsGet_GameRomHorseSaddle_01",
0xbcbdf5dc,"IsGet_GameRomHorseSaddle_02",
0xcbbac54a,"IsGet_GameRomHorseSaddle_03",
0x55de50e9,"IsGet_GameRomHorseSaddle_04",
0x22d9607f,"IsGet_GameRomHorseSaddle_05",
0x053a890c,"IsGet_IceArrow",
0x76f273aa,"IsGet_Item_Boiled_01",
0x5f52e426,"IsGet_Item_ChilledFish_01",
0xc65bb59c,"IsGet_Item_ChilledFish_02",
0xb15c850a,"IsGet_Item_ChilledFish_03",
0x2f3810a9,"IsGet_Item_ChilledFish_04",
0x583f203f,"IsGet_Item_ChilledFish_05",
0xc1367185,"IsGet_Item_ChilledFish_06",
0xb6314113,"IsGet_Item_ChilledFish_07",
0x268e5c82,"IsGet_Item_ChilledFish_08",
0x51896c14,"IsGet_Item_ChilledFish_09",
0x59dfeb41,"IsGet_Item_Chilled_01",
0xc0d6bafb,"IsGet_Item_Chilled_02",
0xb7d18a6d,"IsGet_Item_Chilled_03",
0x29b51fce,"IsGet_Item_Chilled_04",
0x5eb22f58,"IsGet_Item_Chilled_05",
0xc7bb7ee2,"IsGet_Item_Chilled_06",
0x63219d40,"IsGet_Item_Cook_A_01",
0xfa28ccfa,"IsGet_Item_Cook_A_02",
0x8d2ffc6c,"IsGet_Item_Cook_A_03",
0x134b69cf,"IsGet_Item_Cook_A_04",
0x644c5959,"IsGet_Item_Cook_A_05",
0x8a423875,"IsGet_Item_Cook_A_07",
0x1afd25e4,"IsGet_Item_Cook_A_08",
0x6dfa1572,"IsGet_Item_Cook_A_09",
0x0d3d9c97,"IsGet_Item_Cook_A_10",
0x7a3aac01,"IsGet_Item_Cook_A_11",
0xe333fdbb,"IsGet_Item_Cook_A_12",
0x9434cd2d,"IsGet_Item_Cook_A_13",
0x0a50588e,"IsGet_Item_Cook_A_14",
0x719432ae,"IsGet_Item_Cook_B_01",
0xe89d6314,"IsGet_Item_Cook_B_02",
0x9f9a5382,"IsGet_Item_Cook_B_03",
0x01fec621,"IsGet_Item_Cook_B_04",
0x76f9f6b7,"IsGet_Item_Cook_B_05",
0xeff0a70d,"IsGet_Item_Cook_B_06",
0x688f03ef,"IsGet_Item_Cook_B_11",
0xf1865255,"IsGet_Item_Cook_B_12",
0x868162c3,"IsGet_Item_Cook_B_13",
0x18e5f760,"IsGet_Item_Cook_B_14",
0x6fe2c7f6,"IsGet_Item_Cook_B_15",
0xf6eb964c,"IsGet_Item_Cook_B_16",
0x81eca6da,"IsGet_Item_Cook_B_17",
0x1153bb4b,"IsGet_Item_Cook_B_18",
0x66548bdd,"IsGet_Item_Cook_B_19",
0x34a560ba,"IsGet_Item_Cook_B_20",
0x43a2502c,"IsGet_Item_Cook_B_21",
0xdaab0196,"IsGet_Item_Cook_B_22",
0xadac3100,"IsGet_Item_Cook_B_23",
0xc92855cb,"IsGet_Item_Cook_C_01",
0x50210471,"IsGet_Item_Cook_C_02",
0xce4591d2,"IsGet_Item_Cook_C_05",
0x574cc068,"IsGet_Item_Cook_C_06",
0x204bf0fe,"IsGet_Item_Cook_C_07",
0xb0f4ed6f,"IsGet_Item_Cook_C_08",
0xd033648a,"IsGet_Item_Cook_C_11",
0x493a3530,"IsGet_Item_Cook_C_12",
0x3e3d05a6,"IsGet_Item_Cook_C_13",
0xa0599005,"IsGet_Item_Cook_C_14",
0xd75ea093,"IsGet_Item_Cook_C_15",
0x4e57f129,"IsGet_Item_Cook_C_16",
0x3950c1bf,"IsGet_Item_Cook_C_17",
0x16b6b82f,"IsGet_Item_Cook_C_17_00",
0x54ff6d72,"IsGet_Item_Cook_D_01",
0xcdf63cc8,"IsGet_Item_Cook_D_02",
0xbaf10c5e,"IsGet_Item_Cook_D_03",
0x249599fd,"IsGet_Item_Cook_D_04",
0x5392a96b,"IsGet_Item_Cook_D_05",
0xca9bf8d1,"IsGet_Item_Cook_D_06",
0xbd9cc847,"IsGet_Item_Cook_D_07",
0x2d23d5d6,"IsGet_Item_Cook_D_08",
0x5a24e540,"IsGet_Item_Cook_D_09",
0x3ae36ca5,"IsGet_Item_Cook_D_10",
0xec430a17,"IsGet_Item_Cook_E_01",
0x754a5bad,"IsGet_Item_Cook_E_02",
0x024d6b3b,"IsGet_Item_Cook_E_03",
0x9c29fe98,"IsGet_Item_Cook_E_04",
0xfef6a5f9,"IsGet_Item_Cook_F_01",
0x67fff443,"IsGet_Item_Cook_F_02",
0x10f8c4d5,"IsGet_Item_Cook_F_03",
0x8e9c5176,"IsGet_Item_Cook_F_04",
0xdf439326,"IsGet_Item_Cook_G_02",
0xa844a3b0,"IsGet_Item_Cook_G_03",
0x36203613,"IsGet_Item_Cook_G_04",
0x41270685,"IsGet_Item_Cook_G_05",
0xd82e573f,"IsGet_Item_Cook_G_06",
0xaf2967a9,"IsGet_Item_Cook_G_07",
0x3f967a38,"IsGet_Item_Cook_G_08",
0x48914aae,"IsGet_Item_Cook_G_09",
0x2856c34b,"IsGet_Item_Cook_G_10",
0x5f51f3dd,"IsGet_Item_Cook_G_11",
0xc658a267,"IsGet_Item_Cook_G_12",
0xb15f92f1,"IsGet_Item_Cook_G_13",
0x2f3b0752,"IsGet_Item_Cook_G_14",
0x583c37c4,"IsGet_Item_Cook_G_15",
0xc135667e,"IsGet_Item_Cook_G_16",
0xb63256e8,"IsGet_Item_Cook_G_17",
0x1e29d2ca,"IsGet_Item_Cook_H_01",
0x87208370,"IsGet_Item_Cook_H_02",
0xf027b3e6,"IsGet_Item_Cook_H_03",
0xa695b5af,"IsGet_Item_Cook_I_01",
0x3f9ce415,"IsGet_Item_Cook_I_02",
0x489bd483,"IsGet_Item_Cook_I_03",
0xd6ff4120,"IsGet_Item_Cook_I_04",
0xa1f871b6,"IsGet_Item_Cook_I_05",
0x38f1200c,"IsGet_Item_Cook_I_06",
0x4ff6109a,"IsGet_Item_Cook_I_07",
0xdf490d0b,"IsGet_Item_Cook_I_08",
0xa84e3d9d,"IsGet_Item_Cook_I_09",
0xc889b478,"IsGet_Item_Cook_I_10",
0xbf8e84ee,"IsGet_Item_Cook_I_11",
0x2687d554,"IsGet_Item_Cook_I_12",
0x5180e5c2,"IsGet_Item_Cook_I_13",
0xcfe47061,"IsGet_Item_Cook_I_14",
0xb8e340f7,"IsGet_Item_Cook_I_15",
0x21ea114d,"IsGet_Item_Cook_I_16",
0x56ed21db,"IsGet_Item_Cook_I_17",
0xb4201a41,"IsGet_Item_Cook_J_01",
0x2d294bfb,"IsGet_Item_Cook_J_02",
0x5a2e7b6d,"IsGet_Item_Cook_J_03",
0xc44aeece,"IsGet_Item_Cook_J_04",
0xb34dde58,"IsGet_Item_Cook_J_05",
0x2a448fe2,"IsGet_Item_Cook_J_06",
0x5d43bf74,"IsGet_Item_Cook_J_07",
0xcdfca2e5,"IsGet_Item_Cook_J_08",
0xbafb9273,"IsGet_Item_Cook_J_09",
0x0c9c7d24,"IsGet_Item_Cook_K_01",
0x95952c9e,"IsGet_Item_Cook_K_02",
0xe2921c08,"IsGet_Item_Cook_K_03",
0x7cf689ab,"IsGet_Item_Cook_K_04",
0x0bf1b93d,"IsGet_Item_Cook_K_05",
0x92f8e887,"IsGet_Item_Cook_K_06",
0xe5ffd811,"IsGet_Item_Cook_K_07",
0x7540c580,"IsGet_Item_Cook_K_08",
0x0247f516,"IsGet_Item_Cook_K_09",
0x914b459d,"IsGet_Item_Cook_L_01",
0x08421427,"IsGet_Item_Cook_L_02",
0x7f4524b1,"IsGet_Item_Cook_L_03",
0xe121b112,"IsGet_Item_Cook_L_04",
0x96268184,"IsGet_Item_Cook_L_05",
0x29f722f8,"IsGet_Item_Cook_M_01",
0x3b428d16,"IsGet_Item_Cook_N_01",
0xa24bdcac,"IsGet_Item_Cook_N_02",
0xd54cec3a,"IsGet_Item_Cook_N_03",
0x4b287999,"IsGet_Item_Cook_N_04",
0x83feea73,"IsGet_Item_Cook_O_01",
0x1af7bbc9,"IsGet_Item_Cook_O_02",
0x8b84adba,"IsGet_Item_Cook_P_01",
0x128dfc00,"IsGet_Item_Cook_P_02",
0x658acc96,"IsGet_Item_Cook_P_03",
0xfbee5935,"IsGet_Item_Cook_P_04",
0x8ce969a3,"IsGet_Item_Cook_P_05",
0x17dc340f,"IsGet_Item_Enemy_00",
0x60db0499,"IsGet_Item_Enemy_01",
0xf9d25523,"IsGet_Item_Enemy_02",
0x8ed565b5,"IsGet_Item_Enemy_03",
0x10b1f016,"IsGet_Item_Enemy_04",
0x67b6c080,"IsGet_Item_Enemy_05",
0xfebf913a,"IsGet_Item_Enemy_06",
0x89b8a1ac,"IsGet_Item_Enemy_07",
0x1907bc3d,"IsGet_Item_Enemy_08",
0xe0c96462,"IsGet_Item_Enemy_12",
0x97ce54f4,"IsGet_Item_Enemy_13",
0x09aac157,"IsGet_Item_Enemy_14",
0x7eadf1c1,"IsGet_Item_Enemy_15",
0xe7a4a07b,"IsGet_Item_Enemy_16",
0x90a390ed,"IsGet_Item_Enemy_17",
0x001c8d7c,"IsGet_Item_Enemy_18",
0x771bbdea,"IsGet_Item_Enemy_19",
0x25ea568d,"IsGet_Item_Enemy_20",
0x52ed661b,"IsGet_Item_Enemy_21",
0x22879294,"IsGet_Item_Enemy_24",
0x5580a202,"IsGet_Item_Enemy_25",
0xcc89f3b8,"IsGet_Item_Enemy_26",
0xbb8ec32e,"IsGet_Item_Enemy_27",
0x2b31debf,"IsGet_Item_Enemy_28",
0x5c36ee29,"IsGet_Item_Enemy_29",
0x3cf167cc,"IsGet_Item_Enemy_30",
0x4bf6575a,"IsGet_Item_Enemy_31",
0xd2ff06e0,"IsGet_Item_Enemy_32",
0xa5f83676,"IsGet_Item_Enemy_33",
0x3b9ca3d5,"IsGet_Item_Enemy_34",
0x322aeffe,"IsGet_Item_Enemy_38",
0x452ddf68,"IsGet_Item_Enemy_39",
0x73b0f10b,"IsGet_Item_Enemy_40",
0x04b7c19d,"IsGet_Item_Enemy_41",
0x9dbe9027,"IsGet_Item_Enemy_42",
0xeab9a0b1,"IsGet_Item_Enemy_43",
0x74dd3512,"IsGet_Item_Enemy_44",
0x03da0584,"IsGet_Item_Enemy_45",
0x9ad3543e,"IsGet_Item_Enemy_46",
0xedd464a8,"IsGet_Item_Enemy_47",
0x7d6b7939,"IsGet_Item_Enemy_48",
0x0a6c49af,"IsGet_Item_Enemy_49",
0x6aabc04a,"IsGet_Item_Enemy_50",
0x1dacf0dc,"IsGet_Item_Enemy_51",
0x84a5a166,"IsGet_Item_Enemy_52",
0xf3a291f0,"IsGet_Item_Enemy_53",
0x6dc60453,"IsGet_Item_Enemy_54",
0x1ac134c5,"IsGet_Item_Enemy_55",
0x83c8657f,"IsGet_Item_Enemy_56",
0xf4cf55e9,"IsGet_Item_Enemy_57",
0x0d444258,"IsGet_Item_Enemy_Living_Template",
0x35db04c4,"IsGet_Item_Enemy_Nonburnable_Template",
0xf5a7e6a7,"IsGet_Item_Enemy_Normal_Template",
0x308b1ee3,"IsGet_Item_FishGet_A",
0xa9824f59,"IsGet_Item_FishGet_B",
0xde857fcf,"IsGet_Item_FishGet_C",
0x40e1ea6c,"IsGet_Item_FishGet_D",
0x37e6dafa,"IsGet_Item_FishGet_E",
0xaeef8b40,"IsGet_Item_FishGet_F",
0xd9e8bbd6,"IsGet_Item_FishGet_G",
0x4957a647,"IsGet_Item_FishGet_H",
0x3e5096d1,"IsGet_Item_FishGet_I",
0xa759c76b,"IsGet_Item_FishGet_J",
0xd05ef7fd,"IsGet_Item_FishGet_K",
0x4e3a625e,"IsGet_Item_FishGet_L",
0x96db65a3,"IsGet_Item_FishGet_L_00",
0x393d52c8,"IsGet_Item_FishGet_M",
0x54e0b623,"IsGet_Item_FishGet_X",
0xbaeed70f,"IsGet_Item_FishGet_Z",
0xe14436d9,"IsGet_Item_Fruit_A",
0x784d6763,"IsGet_Item_Fruit_B",
0x0f4a57f5,"IsGet_Item_Fruit_C",
0x912ec256,"IsGet_Item_Fruit_D",
0xe629f2c0,"IsGet_Item_Fruit_E",
0x94a93b35,"IsGet_Item_Fruit_E_00",
0x7f20a37a,"IsGet_Item_Fruit_F",
0x082793ec,"IsGet_Item_Fruit_G",
0x98988e7d,"IsGet_Item_Fruit_H",
0xef9fbeeb,"IsGet_Item_Fruit_I",
0x7696ef51,"IsGet_Item_Fruit_J",
0x0191dfc7,"IsGet_Item_Fruit_K",
0x9ff54a64,"IsGet_Item_Fruit_L",
0x5ca7c543,"IsGet_Item_Fruit_Template",
0x5046ad8c,"IsGet_Item_InsectGet_K",
0x572b6995,"IsGet_Item_InsectGet_O",
0x3af68d7e,"IsGet_Item_InsectGet_Z",
0x5e30bd10,"IsGet_Item_Material_01",
0xc739ecaa,"IsGet_Item_Material_02",
0xb03edc3c,"IsGet_Item_Material_03",
0x2e5a499f,"IsGet_Item_Material_04",
0x595d7909,"IsGet_Item_Material_05",
0x85c862e0,"IsGet_Item_Material_05_00",
0xc05428b3,"IsGet_Item_Material_06",
0xb7531825,"IsGet_Item_Material_07",
0x27ec05b4,"IsGet_Item_Material_08",
0x4ca7cba1,"IsGet_Item_Material_Template",
0x4cfb4155,"IsGet_Item_Meat_01",
0xd5f210ef,"IsGet_Item_Meat_02",
0xd29fd4f6,"IsGet_Item_Meat_06",
0xa598e460,"IsGet_Item_Meat_07",
0x55e07014,"IsGet_Item_Meat_11",
0xcce921ae,"IsGet_Item_Meat_12",
0x2018f33b,"IsGet_Item_Meat_Template",
0xa69cd08a,"IsGet_Item_MushroomGet_D",
0x324fbcc5,"IsGet_Item_Mushroom_A",
0xab46ed7f,"IsGet_Item_Mushroom_B",
0xdc41dde9,"IsGet_Item_Mushroom_C",
0x352278dc,"IsGet_Item_Mushroom_E",
0xac2b2966,"IsGet_Item_Mushroom_F",
0xc59e6755,"IsGet_Item_Mushroom_F_00",
0x4b930461,"IsGet_Item_Mushroom_H",
0xa59d654d,"IsGet_Item_Mushroom_J",
0x4cfec078,"IsGet_Item_Mushroom_L",
0x3bf9f0ee,"IsGet_Item_Mushroom_M",
0xa2f0a154,"IsGet_Item_Mushroom_N",
0x002a4fba,"IsGet_Item_Mushroom_N_00",
0xd5f791c2,"IsGet_Item_Mushroom_O",
0x00c32321,"IsGet_Item_Mushroom_Template",
0xda1ebd03,"IsGet_Item_Ore_A",
0xc81ce5d5,"IsGet_Item_Ore_A_00",
0x4317ecb9,"IsGet_Item_Ore_B",
0x3410dc2f,"IsGet_Item_Ore_C",
0xaa74498c,"IsGet_Item_Ore_D",
0xdd73791a,"IsGet_Item_Ore_E",
0x447a28a0,"IsGet_Item_Ore_F",
0x337d1836,"IsGet_Item_Ore_G",
0xa3c205a7,"IsGet_Item_Ore_H",
0xd4c53531,"IsGet_Item_Ore_I",
0x4dcc648b,"IsGet_Item_Ore_J",
0x72ead599,"IsGet_Item_Ore_Template",
0xe690be98,"IsGet_Item_PlantGet_A",
0x7f99ef22,"IsGet_Item_PlantGet_B",
0x089edfb4,"IsGet_Item_PlantGet_C",
0xe1fd7a81,"IsGet_Item_PlantGet_E",
0x78f42b3b,"IsGet_Item_PlantGet_F",
0x0ff31bad,"IsGet_Item_PlantGet_G",
0x9f4c063c,"IsGet_Item_PlantGet_H",
0xe84b36aa,"IsGet_Item_PlantGet_I",
0x71426710,"IsGet_Item_PlantGet_J",
0x9821c225,"IsGet_Item_PlantGet_L",
0xef26f2b3,"IsGet_Item_PlantGet_M",
0x0128939f,"IsGet_Item_PlantGet_O",
0xfb27aefc,"IsGet_Item_PlantGet_Q",
0xfe02884f,"IsGet_Item_Plant_Get_Template",
0x804dff9d,"IsGet_Item_Plant_RootVegetable_Template",
0xaa13dbf3,"IsGet_Item_Plant_Template",
0x6a87583a,"IsGet_Item_RoastFish_01",
0xf38e0980,"IsGet_Item_RoastFish_02",
0x84893916,"IsGet_Item_RoastFish_03",
0x1aedacb5,"IsGet_Item_RoastFish_04",
0x83e4fd0f,"IsGet_Item_RoastFish_07",
0x645cd008,"IsGet_Item_RoastFish_09",
0x739c697b,"IsGet_Item_RoastFish_11",
0x9d920857,"IsGet_Item_RoastFish_13",
0x74f1ad62,"IsGet_Item_RoastFish_15",
0xa9639e31,"IsGet_Item_RoastFish_Template",
0x06f98437,"IsGet_Item_Roast_01",
0x9ff0d58d,"IsGet_Item_Roast_02",
0xe8f7e51b,"IsGet_Item_Roast_03",
0x769370b8,"IsGet_Item_Roast_04",
0x0194402e,"IsGet_Item_Roast_05",
0x989d1194,"IsGet_Item_Roast_06",
0xef9a2102,"IsGet_Item_Roast_07",
0x7f253c93,"IsGet_Item_Roast_08",
0x08220c05,"IsGet_Item_Roast_09",
0x68e585e0,"IsGet_Item_Roast_10",
0x1fe2b576,"IsGet_Item_Roast_11",
0x86ebe4cc,"IsGet_Item_Roast_12",
0xf1ecd45a,"IsGet_Item_Roast_13",
0x188f716f,"IsGet_Item_Roast_15",
0x818620d5,"IsGet_Item_Roast_16",
0x663e0dd2,"IsGet_Item_Roast_18",
0x11393d44,"IsGet_Item_Roast_19",
0x44a5123a,"IsGet_Item_Roast_24",
0xddac4380,"IsGet_Item_Roast_27",
0x4d135e11,"IsGet_Item_Roast_28",
0x2dd4d7f4,"IsGet_Item_Roast_31",
0xb4dd864e,"IsGet_Item_Roast_32",
0xc3dab6d8,"IsGet_Item_Roast_33",
0xb3b04257,"IsGet_Item_Roast_36",
0xc4b772c1,"IsGet_Item_Roast_37",
0x54086f50,"IsGet_Item_Roast_38",
0x230f5fc6,"IsGet_Item_Roast_39",
0x159271a5,"IsGet_Item_Roast_40",
0x62954133,"IsGet_Item_Roast_41",
0x65f8852a,"IsGet_Item_Roast_45",
0xfcf1d490,"IsGet_Item_Roast_46",
0x1b49f997,"IsGet_Item_Roast_48",
0x6c4ec901,"IsGet_Item_Roast_49",
0x0c8940e4,"IsGet_Item_Roast_50",
0x7b8e7072,"IsGet_Item_Roast_51",
0xe28721c8,"IsGet_Item_Roast_52",
0x9580115e,"IsGet_Item_Roast_53",
0xfda97d86,"IsGet_KeySmall",
0x72c7ed5e,"IsGet_MemoryPhoto_000",
0x05c0ddc8,"IsGet_MemoryPhoto_001",
0x9cc98c72,"IsGet_MemoryPhoto_002",
0xebcebce4,"IsGet_MemoryPhoto_003",
0x75aa2947,"IsGet_MemoryPhoto_004",
0x02ad19d1,"IsGet_MemoryPhoto_005",
0x9ba4486b,"IsGet_MemoryPhoto_006",
0xeca378fd,"IsGet_MemoryPhoto_007",
0x7c1c656c,"IsGet_MemoryPhoto_008",
0x0b1b55fa,"IsGet_MemoryPhoto_009",
0x6bdcdc1f,"IsGet_MemoryPhoto_010",
0x1cdbec89,"IsGet_MemoryPhoto_011",
0x85d2bd33,"IsGet_MemoryPhoto_012",
0xf2d58da5,"IsGet_MemoryPhoto_013",
0xd2dddc49,"IsGet_NormalArrow",
0xf43ffdf7,"IsGet_Obj_Album",
0x24b87dfb,"IsGet_Obj_Armor_115_Head",
0x843d94ba,"IsGet_Obj_BombArrow_A_10",
0xf7dd3e03,"IsGet_Obj_Camera",
0x7c3f032c,"IsGet_Obj_DRStone_A_01",
0xbc13cc9b,"IsGet_Obj_DRStone_Get",
0x4ea47ee6,"IsGet_Obj_DungeonClearSeal",
0xe3f8153b,"IsGet_Obj_FireWoodBundle",
0xf284591d,"IsGet_Obj_HeartUtuwa_A_01",
0x8e7188d0,"IsGet_Obj_HeroSoul_Gerudo",
0xff48aa75,"IsGet_Obj_HeroSoul_Goron",
0x7dba0908,"IsGet_Obj_HeroSoul_Rito",
0x0d61d7d4,"IsGet_Obj_HeroSoul_Zora",
0x5992b256,"IsGet_Obj_IceMaker",
0x9f3bd197,"IsGet_Obj_KorokNuts",
0x795e7bbc,"IsGet_Obj_Magnetglove",
0xa2f96691,"IsGet_Obj_Maracas",
0x31502b02,"IsGet_Obj_Photo_Animal",
0xc755a916,"IsGet_Obj_Photo_BossEnemy",
0xde395a16,"IsGet_Obj_Photo_Enemy",
0x8d09ca40,"IsGet_Obj_Photo_Material",
0xfcfe359f,"IsGet_Obj_Photo_Other",
0x32c8affb,"IsGet_Obj_Photo_Weapon",
0x486ebeca,"IsGet_Obj_PictureBook",
0xe6167bfc,"IsGet_Obj_Porch_Bow",
0x7c67e38f,"IsGet_Obj_Porch_Shield",
0x0bc79681,"IsGet_Obj_Porch_Weapon",
0x49b4daac,"IsGet_Obj_ProofBook",
0x2ed4071b,"IsGet_Obj_ProofGiantKiller",
0x7da95540,"IsGet_Obj_ProofGolemKiller",
0xaac99b12,"IsGet_Obj_ProofKorok",
0xafb4b9b8,"IsGet_Obj_ProofSandwormKiller",
0x676fbac5,"IsGet_Obj_RemainInsideMap",
0x191bccf9,"IsGet_Obj_RemoteBomb",
0x7a86b236,"IsGet_Obj_RemoteBombBall",
0xd515ad26,"IsGet_Obj_RemoteBombBall2",
0x3e4a4bd5,"IsGet_Obj_RemoteBombCube",
0xec52d21d,"IsGet_Obj_RemoteBombCube2",
0x604c3bae,"IsGet_Obj_RemoteBombLv2",
0x140a43f8,"IsGet_Obj_SheikPadLv2",
0xca483dd7,"IsGet_Obj_SheikSensor",
0xad5a8025,"IsGet_Obj_SheikSensorLv2",
0x550c96f1,"IsGet_Obj_StaminaUtuwa_A_01",
0x7504085d,"IsGet_Obj_StopTimer",
0x9ca3d1c1,"IsGet_Obj_StopTimerLv2",
0xfe4d1501,"IsGet_PlayerStole2",
0xd820eb59,"IsGet_PlayerStole2_Vagrant",
0xbe5950c0,"IsGet_PortableUnit",
0x1a0f4381,"IsGet_PutRupee",
0xdd77c686,"IsGet_PutRupee_Blue",
0x04e81163,"IsGet_PutRupee_Gold",
0x0e4329fd,"IsGet_PutRupee_Purple",
0xd28407e7,"IsGet_PutRupee_Red",
0xcb9d4340,"IsGet_PutRupee_Silver",
0x736a8876,"IsGet_Rupee",
0x06773e58,"IsGet_TestSakookaBow",
0x620d8953,"IsGet_TestSakookaWeapon",
0x7ed49272,"IsGet_TestSakooka_Head",
0x12c0f075,"IsGet_Test_Sakooka_Shield",
0x986c7359,"IsGet_Test_Zero_Shield",
0x687a5fcf,"IsGet_Weapon_Bow_001",
0xf1730e75,"IsGet_Weapon_Bow_002",
0x86743ee3,"IsGet_Weapon_Bow_003",
0x1810ab40,"IsGet_Weapon_Bow_004",
0xf61eca6c,"IsGet_Weapon_Bow_006",
0x66a1d7fd,"IsGet_Weapon_Bow_009",
0x71616e8e,"IsGet_Weapon_Bow_011",
0x9f6f0fa2,"IsGet_Weapon_Bow_013",
0x010b9a01,"IsGet_Weapon_Bow_014",
0x760caa97,"IsGet_Weapon_Bow_015",
0xef05fb2d,"IsGet_Weapon_Bow_016",
0x9802cbbb,"IsGet_Weapon_Bow_017",
0xb4425c61,"IsGet_Weapon_Bow_023",
0xc428a8ee,"IsGet_Weapon_Bow_026",
0xb32f9878,"IsGet_Weapon_Bow_027",
0x239085e9,"IsGet_Weapon_Bow_028",
0x5497b57f,"IsGet_Weapon_Bow_029",
0x34503c9a,"IsGet_Weapon_Bow_030",
0xda5e5db6,"IsGet_Weapon_Bow_032",
0xad596d20,"IsGet_Weapon_Bow_033",
0x443ac815,"IsGet_Weapon_Bow_035",
0xdd3399af,"IsGet_Weapon_Bow_036",
0x3a8bb4a8,"IsGet_Weapon_Bow_038",
0x7b11aa5d,"IsGet_Weapon_Bow_040",
0x273bc908,"IsGet_Weapon_Bow_071",
0xbe3298b2,"IsGet_Weapon_Bow_072",
0xea16643f,"IsGet_Weapon_Lsword_001",
0x731f3585,"IsGet_Weapon_Lsword_002",
0x04180513,"IsGet_Weapon_Lsword_003",
0x9a7c90b0,"IsGet_Weapon_Lsword_004",
0xed7ba026,"IsGet_Weapon_Lsword_005",
0x7472f19c,"IsGet_Weapon_Lsword_006",
0x840a65e8,"IsGet_Weapon_Lsword_010",
0xf30d557e,"IsGet_Weapon_Lsword_011",
0x6a0404c4,"IsGet_Weapon_Lsword_012",
0x1d033452,"IsGet_Weapon_Lsword_013",
0x8367a1f1,"IsGet_Weapon_Lsword_014",
0xf4609167,"IsGet_Weapon_Lsword_015",
0x6d69c0dd,"IsGet_Weapon_Lsword_016",
0x1a6ef04b,"IsGet_Weapon_Lsword_017",
0x8ad1edda,"IsGet_Weapon_Lsword_018",
0xfdd6dd4c,"IsGet_Weapon_Lsword_019",
0xaf27362b,"IsGet_Weapon_Lsword_020",
0x362e6791,"IsGet_Weapon_Lsword_023",
0xa84af232,"IsGet_Weapon_Lsword_024",
0x3143a388,"IsGet_Weapon_Lsword_027",
0xd6fb8e8f,"IsGet_Weapon_Lsword_029",
0xb63c076a,"IsGet_Weapon_Lsword_030",
0xc13b37fc,"IsGet_Weapon_Lsword_031",
0x58326646,"IsGet_Weapon_Lsword_032",
0x2f3556d0,"IsGet_Weapon_Lsword_033",
0xb151c373,"IsGet_Weapon_Lsword_034",
0xc656f3e5,"IsGet_Weapon_Lsword_035",
0x5f5fa25f,"IsGet_Weapon_Lsword_036",
0x285892c9,"IsGet_Weapon_Lsword_037",
0xb8e78f58,"IsGet_Weapon_Lsword_038",
0x8e7aa13b,"IsGet_Weapon_Lsword_041",
0x89176522,"IsGet_Weapon_Lsword_045",
0x6719040e,"IsGet_Weapon_Lsword_047",
0x9761907a,"IsGet_Weapon_Lsword_051",
0xe70b64f5,"IsGet_Weapon_Lsword_054",
0x900c5463,"IsGet_Weapon_Lsword_055",
0x090505d9,"IsGet_Weapon_Lsword_056",
0x7e02354f,"IsGet_Weapon_Lsword_057",
0x99ba1848,"IsGet_Weapon_Lsword_059",
0xcb4bf32f,"IsGet_Weapon_Lsword_060",
0xd53d0677,"IsGet_Weapon_Lsword_074",
0xe6bb7d4a,"IsGet_Weapon_Shield_001",
0x7fb22cf0,"IsGet_Weapon_Shield_002",
0x08b51c66,"IsGet_Weapon_Shield_003",
0x96d189c5,"IsGet_Weapon_Shield_004",
0xe1d6b953,"IsGet_Weapon_Shield_005",
0x78dfe8e9,"IsGet_Weapon_Shield_006",
0x0fd8d87f,"IsGet_Weapon_Shield_007",
0x9f67c5ee,"IsGet_Weapon_Shield_008",
0xe860f578,"IsGet_Weapon_Shield_009",
0x11ae2d27,"IsGet_Weapon_Shield_013",
0x8fcab884,"IsGet_Weapon_Shield_014",
0xf8cd8812,"IsGet_Weapon_Shield_015",
0x61c4d9a8,"IsGet_Weapon_Shield_016",
0x16c3e93e,"IsGet_Weapon_Shield_017",
0x867cf4af,"IsGet_Weapon_Shield_018",
0xd48d1fc8,"IsGet_Weapon_Shield_021",
0x4d844e72,"IsGet_Weapon_Shield_022",
0x3a837ee4,"IsGet_Weapon_Shield_023",
0xd3e0dbd1,"IsGet_Weapon_Shield_025",
0x4ae98a6b,"IsGet_Weapon_Shield_026",
0xba911e1f,"IsGet_Weapon_Shield_030",
0xcd962e89,"IsGet_Weapon_Shield_031",
0x549f7f33,"IsGet_Weapon_Shield_032",
0x23984fa5,"IsGet_Weapon_Shield_033",
0xbdfcda06,"IsGet_Weapon_Shield_034",
0xcafbea90,"IsGet_Weapon_Shield_035",
0x53f2bb2a,"IsGet_Weapon_Shield_036",
0x24f58bbc,"IsGet_Weapon_Shield_037",
0xb44a962d,"IsGet_Weapon_Shield_038",
0xf5d088d8,"IsGet_Weapon_Shield_040",
0x82d7b84e,"IsGet_Weapon_Shield_041",
0x1bdee9f4,"IsGet_Weapon_Shield_042",
0x72af2c3a,"IsGet_Weapon_Shield_057",
0xe852d36c,"IsGet_Weapon_Spear_001",
0x715b82d6,"IsGet_Weapon_Spear_002",
0x065cb240,"IsGet_Weapon_Spear_003",
0x983827e3,"IsGet_Weapon_Spear_004",
0xef3f1775,"IsGet_Weapon_Spear_005",
0x763646cf,"IsGet_Weapon_Spear_006",
0x01317659,"IsGet_Weapon_Spear_007",
0x918e6bc8,"IsGet_Weapon_Spear_008",
0xe6895b5e,"IsGet_Weapon_Spear_009",
0x864ed2bb,"IsGet_Weapon_Spear_010",
0xf149e22d,"IsGet_Weapon_Spear_011",
0x6840b397,"IsGet_Weapon_Spear_012",
0x1f478301,"IsGet_Weapon_Spear_013",
0x812316a2,"IsGet_Weapon_Spear_014",
0xf6242634,"IsGet_Weapon_Spear_015",
0x6f2d778e,"IsGet_Weapon_Spear_016",
0x182a4718,"IsGet_Weapon_Spear_017",
0x88955a89,"IsGet_Weapon_Spear_018",
0xda64b1ee,"IsGet_Weapon_Spear_021",
0x436de054,"IsGet_Weapon_Spear_022",
0x346ad0c2,"IsGet_Weapon_Spear_023",
0xaa0e4561,"IsGet_Weapon_Spear_024",
0xdd0975f7,"IsGet_Weapon_Spear_025",
0x330714db,"IsGet_Weapon_Spear_027",
0xa3b8094a,"IsGet_Weapon_Spear_028",
0xd4bf39dc,"IsGet_Weapon_Spear_029",
0xb478b039,"IsGet_Weapon_Spear_030",
0xc37f80af,"IsGet_Weapon_Spear_031",
0x5a76d115,"IsGet_Weapon_Spear_032",
0x2d71e183,"IsGet_Weapon_Spear_033",
0xb3157420,"IsGet_Weapon_Spear_034",
0xc41244b6,"IsGet_Weapon_Spear_035",
0x5d1b150c,"IsGet_Weapon_Spear_036",
0x2a1c259a,"IsGet_Weapon_Spear_037",
0xbaa3380b,"IsGet_Weapon_Spear_038",
0x655db35d,"IsGet_Weapon_Spear_047",
0x82e59e5a,"IsGet_Weapon_Spear_049",
0xe22217bf,"IsGet_Weapon_Spear_050",
0x2c05972c,"IsGet_Weapon_Sword_001",
0xb50cc696,"IsGet_Weapon_Sword_002",
0xc20bf600,"IsGet_Weapon_Sword_003",
0x5c6f63a3,"IsGet_Weapon_Sword_004",
0x2b685335,"IsGet_Weapon_Sword_005",
0xb261028f,"IsGet_Weapon_Sword_006",
0xc5663219,"IsGet_Weapon_Sword_007",
0x55d92f88,"IsGet_Weapon_Sword_008",
0x22de1f1e,"IsGet_Weapon_Sword_009",
0xdb10c741,"IsGet_Weapon_Sword_013",
0x457452e2,"IsGet_Weapon_Sword_014",
0x32736274,"IsGet_Weapon_Sword_015",
0xab7a33ce,"IsGet_Weapon_Sword_016",
0xdc7d0358,"IsGet_Weapon_Sword_017",
0x4cc21ec9,"IsGet_Weapon_Sword_018",
0x3bc52e5f,"IsGet_Weapon_Sword_019",
0x6934c538,"IsGet_Weapon_Sword_020",
0x1e33f5ae,"IsGet_Weapon_Sword_021",
0x873aa414,"IsGet_Weapon_Sword_022",
0xf03d9482,"IsGet_Weapon_Sword_023",
0x6e590121,"IsGet_Weapon_Sword_024",
0x195e31b7,"IsGet_Weapon_Sword_025",
0xf750509b,"IsGet_Weapon_Sword_027",
0x10e87d9c,"IsGet_Weapon_Sword_029",
0x702ff479,"IsGet_Weapon_Sword_030",
0x0728c4ef,"IsGet_Weapon_Sword_031",
0xe926a5c3,"IsGet_Weapon_Sword_033",
0x77423060,"IsGet_Weapon_Sword_034",
0x004500f6,"IsGet_Weapon_Sword_035",
0x3f6e62be,"IsGet_Weapon_Sword_040",
0x48695228,"IsGet_Weapon_Sword_041",
0xa6673304,"IsGet_Weapon_Sword_043",
0x3803a6a7,"IsGet_Weapon_Sword_044",
0xa10af71d,"IsGet_Weapon_Sword_047",
0x31b5ea8c,"IsGet_Weapon_Sword_048",
0x46b2da1a,"IsGet_Weapon_Sword_049",
0x267553ff,"IsGet_Weapon_Sword_050",
0x51726369,"IsGet_Weapon_Sword_051",
0xc87b32d3,"IsGet_Weapon_Sword_052",
0xbf7c0245,"IsGet_Weapon_Sword_053",
0xb811c65c,"IsGet_Weapon_Sword_057",
0x28aedbcd,"IsGet_Weapon_Sword_058",
0x5fa9eb5b,"IsGet_Weapon_Sword_059",
0x0d58003c,"IsGet_Weapon_Sword_060",
0x7a5f30aa,"IsGet_Weapon_Sword_061",
0xe3566110,"IsGet_Weapon_Sword_062",
0x1443317d,"IsGet_Weapon_Sword_070",
0x8d4a60c7,"IsGet_Weapon_Sword_073",
0x3230f565,"IsGet_Weapon_Sword_DemoCheck",
0xb8b47716,"IsHelp_App_Album",
0xc5969a1a,"IsHelp_App_AllRadar",
0x209c6767,"IsHelp_App_Camera",
0x85ddcabb,"IsHelp_App_ElectricRelic",
0x06d2fe5e,"IsHelp_App_FireRelic",
0x4ab422e4,"IsHelp_App_IceMaker",
0x1cc85e36,"IsHelp_App_Magnetglove",
0xf11c5b24,"IsHelp_App_ReferenceBook",
0x3fe780a1,"IsHelp_App_RemoteBomb",
0x91b9d369,"IsHelp_App_Stoptimer",
0xaa363460,"IsHelp_App_WaterRelic",
0xd64b8d73,"IsHelp_App_WindRelic",
0x1feddb13,"IsHyruleCastlePGanonElectricDead",
0x6624f816,"IsHyruleCastlePGanonFireDead",
0x2b707a3c,"IsHyruleCastlePGanonWaterDead",
0x9f4122f3,"IsHyruleCastlePGanonWindDead",
0xf0e02375,"IsNewPictureBook_AncientArrow",
0x952a6736,"IsNewPictureBook_Animal_Bear_A",
0x0c23368c,"IsNewPictureBook_Animal_Bear_B",
0xdf9a7f97,"IsNewPictureBook_Animal_Boar_A",
0x46932e2d,"IsNewPictureBook_Animal_Boar_B",
0x1118a613,"IsNewPictureBook_Animal_Bull_A",
0x895bc971,"IsNewPictureBook_Animal_Cassowary_A",
0x0a14b535,"IsNewPictureBook_Animal_Cow_A",
0x6aa24ad9,"IsNewPictureBook_Animal_Crow_A",
0xcc11137c,"IsNewPictureBook_Animal_Deer_A",
0xa7961e3b,"IsNewPictureBook_Animal_Doe_A",
0xa412ca55,"IsNewPictureBook_Animal_Dog_A",
0x82ddb56f,"IsNewPictureBook_Animal_Elk_A",
0xc9a87d78,"IsNewPictureBook_Animal_Fox_A",
0x50a12cc2,"IsNewPictureBook_Animal_Fox_B",
0x8bda9296,"IsNewPictureBook_Animal_Goat_A",
0x41d537a0,"IsNewPictureBook_Animal_Gull_A",
0xa7b4722c,"IsNewPictureBook_Animal_Hawk_A",
0xcc5dac53,"IsNewPictureBook_Animal_Heron_A",
0x5554fde9,"IsNewPictureBook_Animal_Heron_B",
0x7c05120a,"IsNewPictureBook_Animal_Insect_A",
0x33707287,"IsNewPictureBook_Animal_Insect_AA",
0xaa79233d,"IsNewPictureBook_Animal_Insect_AB",
0xe50c43b0,"IsNewPictureBook_Animal_Insect_B",
0x920b7326,"IsNewPictureBook_Animal_Insect_C",
0x7b68d613,"IsNewPictureBook_Animal_Insect_E",
0xe26187a9,"IsNewPictureBook_Animal_Insect_F",
0x9566b73f,"IsNewPictureBook_Animal_Insect_G",
0x05d9aaae,"IsNewPictureBook_Animal_Insect_H",
0x72de9a38,"IsNewPictureBook_Animal_Insect_I",
0x75b35e21,"IsNewPictureBook_Animal_Insect_M",
0xecba0f9b,"IsNewPictureBook_Animal_Insect_N",
0x16b532f8,"IsNewPictureBook_Animal_Insect_P",
0x61b2026e,"IsNewPictureBook_Animal_Insect_Q",
0xf8bb53d4,"IsNewPictureBook_Animal_Insect_R",
0x8fbc6342,"IsNewPictureBook_Animal_Insect_S",
0x11d8f6e1,"IsNewPictureBook_Animal_Insect_T",
0x186ebaca,"IsNewPictureBook_Animal_Insect_X",
0xbf7e9d3b,"IsNewPictureBook_Animal_Kokko_A",
0x790065cc,"IsNewPictureBook_Animal_LittleBird_A",
0xe0093476,"IsNewPictureBook_Animal_LittleBird_B",
0x970e04e0,"IsNewPictureBook_Animal_LittleBird_C",
0x096a9143,"IsNewPictureBook_Animal_LittleBird_D",
0x7e6da1d5,"IsNewPictureBook_Animal_LittleBird_E",
0xe764f06f,"IsNewPictureBook_Animal_LittleBird_F",
0x1976df17,"IsNewPictureBook_Animal_Pigeon_A",
0x807f8ead,"IsNewPictureBook_Animal_Pigeon_B",
0x4b928198,"IsNewPictureBook_Animal_Ptarmigan_A",
0xd29bd022,"IsNewPictureBook_Animal_Ptarmigan_B",
0x1d23d7cc,"IsNewPictureBook_Animal_Rhino_A",
0xbbde654a,"IsNewPictureBook_Animal_RupeeRabbit_A",
0xbedf5235,"IsNewPictureBook_Animal_Sheep_A",
0xde84d13f,"IsNewPictureBook_Animal_Squirrel_A",
0x30c29147,"IsNewPictureBook_Animal_SunazarashiSP_C",
0xca7837cf,"IsNewPictureBook_Animal_Sunazarashi_A",
0x8273ab15,"IsNewPictureBook_Animal_WildDuck_A",
0x93c47a5e,"IsNewPictureBook_Animal_WildGoat_A",
0x65c43fce,"IsNewPictureBook_Animal_Wolf_A",
0xfccd6e74,"IsNewPictureBook_Animal_Wolf_B",
0x8bca5ee2,"IsNewPictureBook_Animal_Wolf_C",
0x67269435,"IsNewPictureBook_BeeHome",
0x792eec33,"IsNewPictureBook_BombArrow_A",
0x434f8148,"IsNewPictureBook_ElectricArrow",
0x659177b9,"IsNewPictureBook_Enemy_Assassin_Junior",
0x121f98b1,"IsNewPictureBook_Enemy_Assassin_Middle",
0x21a8b368,"IsNewPictureBook_Enemy_Assassin_Senior",
0xd91e6a61,"IsNewPictureBook_Enemy_Bokoblin_Bone_Junior",
0x2ce511e1,"IsNewPictureBook_Enemy_Bokoblin_Dark",
0x8643d746,"IsNewPictureBook_Enemy_Bokoblin_Junior",
0xf1cd384e,"IsNewPictureBook_Enemy_Bokoblin_Middle",
0xc27a1397,"IsNewPictureBook_Enemy_Bokoblin_Senior",
0xef72d7f1,"IsNewPictureBook_Enemy_Chuchu_Electric_Junior",
0xe8e0c383,"IsNewPictureBook_Enemy_Chuchu_Fire_Junior",
0x8e546215,"IsNewPictureBook_Enemy_Chuchu_Ice_Junior",
0x13e9b4e5,"IsNewPictureBook_Enemy_Chuchu_Junior",
0xec181329,"IsNewPictureBook_Enemy_Dragon_Electric",
0x5c2cf17b,"IsNewPictureBook_Enemy_Dragon_Fire",
0xe2912588,"IsNewPictureBook_Enemy_Dragon_Ice",
0x13ef0c77,"IsNewPictureBook_Enemy_Ganon",
0x30f3906a,"IsNewPictureBook_Enemy_GanonBeast",
0x677def02,"IsNewPictureBook_Enemy_GanonGrudge",
0xe2efb0fd,"IsNewPictureBook_Enemy_GanonGrudge_01",
0x7be6e147,"IsNewPictureBook_Enemy_GanonGrudge_02",
0x6ce6afc7,"IsNewPictureBook_Enemy_Giant_Bone",
0x107c12bb,"IsNewPictureBook_Enemy_Giant_Junior",
0x67f2fdb3,"IsNewPictureBook_Enemy_Giant_Middle",
0x5445d66a,"IsNewPictureBook_Enemy_Giant_Senior",
0xcfbf728d,"IsNewPictureBook_Enemy_Golem_Fire",
0x78ed4e05,"IsNewPictureBook_Enemy_Golem_Ice",
0xec96e795,"IsNewPictureBook_Enemy_Golem_Junior",
0x1c7b2fd7,"IsNewPictureBook_Enemy_Golem_Little",
0xd89e1027,"IsNewPictureBook_Enemy_Golem_Little_Fire",
0xee31a79b,"IsNewPictureBook_Enemy_Golem_Little_Ice",
0x9b18089d,"IsNewPictureBook_Enemy_Golem_Middle",
0xa8af2344,"IsNewPictureBook_Enemy_Golem_Senior",
0xe2c02e21,"IsNewPictureBook_Enemy_Guardian_A",
0xe4e76049,"IsNewPictureBook_Enemy_Guardian_A_Fixed_Moss",
0x7bc97f9b,"IsNewPictureBook_Enemy_Guardian_B",
0x0cce4f0d,"IsNewPictureBook_Enemy_Guardian_C",
0xed3bfdc3,"IsNewPictureBook_Enemy_Guardian_Mini_Baby",
0x11af3d77,"IsNewPictureBook_Enemy_Guardian_Mini_Junior",
0x6621d27f,"IsNewPictureBook_Enemy_Guardian_Mini_Middle",
0x5596f9a6,"IsNewPictureBook_Enemy_Guardian_Mini_Senior",
0x248d1d15,"IsNewPictureBook_Enemy_Keese",
0x479b674a,"IsNewPictureBook_Enemy_Keese_Electric",
0xb7bf282f,"IsNewPictureBook_Enemy_Keese_Fire",
0x7f7e4e4d,"IsNewPictureBook_Enemy_Keese_Ice",
0x918b3ed5,"IsNewPictureBook_Enemy_Lizalfos_Bone_Junior",
0xcd15702c,"IsNewPictureBook_Enemy_Lizalfos_Dark",
0x1d438c96,"IsNewPictureBook_Enemy_Lizalfos_Electric",
0x8eb7c418,"IsNewPictureBook_Enemy_Lizalfos_Fire",
0x7421bbae,"IsNewPictureBook_Enemy_Lizalfos_Ice",
0xc2a2ce7b,"IsNewPictureBook_Enemy_Lizalfos_Junior",
0xb52c2173,"IsNewPictureBook_Enemy_Lizalfos_Middle",
0x869b0aaa,"IsNewPictureBook_Enemy_Lizalfos_Senior",
0x225ed26d,"IsNewPictureBook_Enemy_Lynel_Dark",
0x13189db0,"IsNewPictureBook_Enemy_Lynel_Junior",
0x649672b8,"IsNewPictureBook_Enemy_Lynel_Middle",
0x57215961,"IsNewPictureBook_Enemy_Lynel_Senior",
0xd415e9bd,"IsNewPictureBook_Enemy_Moriblin_Bone_Junior",
0xd3304edd,"IsNewPictureBook_Enemy_Moriblin_Dark",
0x0e641d09,"IsNewPictureBook_Enemy_Moriblin_Junior",
0x79eaf201,"IsNewPictureBook_Enemy_Moriblin_Middle",
0x4a5dd9d8,"IsNewPictureBook_Enemy_Moriblin_Senior",
0xa60c66bf,"IsNewPictureBook_Enemy_Octarock",
0x557d2659,"IsNewPictureBook_Enemy_Octarock_Desert",
0xf3a2c9d7,"IsNewPictureBook_Enemy_Octarock_Forest",
0xb609af87,"IsNewPictureBook_Enemy_Octarock_Snow",
0x38147b95,"IsNewPictureBook_Enemy_Octarock_Stone",
0xfd4e8b68,"IsNewPictureBook_Enemy_Sandworm",
0x17a18e6e,"IsNewPictureBook_Enemy_SiteBoss_Bow",
0x329e304f,"IsNewPictureBook_Enemy_SiteBoss_Lsword",
0x38122943,"IsNewPictureBook_Enemy_SiteBoss_Spear",
0x3d6960ef,"IsNewPictureBook_Enemy_SiteBoss_Sword",
0xa47e2a23,"IsNewPictureBook_Enemy_Wizzrobe_Electric",
0x8f21060e,"IsNewPictureBook_Enemy_Wizzrobe_Electric_Senior",
0xcd22a5f0,"IsNewPictureBook_Enemy_Wizzrobe_Fire",
0x41212398,"IsNewPictureBook_Enemy_Wizzrobe_Fire_Senior",
0x88a939c6,"IsNewPictureBook_Enemy_Wizzrobe_Ice",
0x8aca4c44,"IsNewPictureBook_Enemy_Wizzrobe_Ice_Senior",
0x4464e077,"IsNewPictureBook_FireArrow",
0xc3d37251,"IsNewPictureBook_GameRomHorse",
0x14abdb94,"IsNewPictureBook_GameRomHorse00L",
0x99a3d661,"IsNewPictureBook_GameRomHorse00S",
0x19c8c166,"IsNewPictureBook_GameRomHorseBone",
0x19a02e49,"IsNewPictureBook_GameRomHorseNushi",
0x69cd6be7,"IsNewPictureBook_GameRomHorseZelda",
0x36fb213d,"IsNewPictureBook_IceArrow",
0x5726cc16,"IsNewPictureBook_Item_FishGet_A",
0xce2f9dac,"IsNewPictureBook_Item_FishGet_B",
0xb928ad3a,"IsNewPictureBook_Item_FishGet_C",
0x274c3899,"IsNewPictureBook_Item_FishGet_D",
0x504b080f,"IsNewPictureBook_Item_FishGet_E",
0xc94259b5,"IsNewPictureBook_Item_FishGet_F",
0xbe456923,"IsNewPictureBook_Item_FishGet_G",
0x2efa74b2,"IsNewPictureBook_Item_FishGet_H",
0x59fd4424,"IsNewPictureBook_Item_FishGet_I",
0xc0f4159e,"IsNewPictureBook_Item_FishGet_J",
0xb7f32508,"IsNewPictureBook_Item_FishGet_K",
0x2997b0ab,"IsNewPictureBook_Item_FishGet_L",
0x5e90803d,"IsNewPictureBook_Item_FishGet_M",
0x334d64d6,"IsNewPictureBook_Item_FishGet_X",
0xdd4305fa,"IsNewPictureBook_Item_FishGet_Z",
0x138922dc,"IsNewPictureBook_Item_Fruit_A",
0x8a807366,"IsNewPictureBook_Item_Fruit_B",
0xfd8743f0,"IsNewPictureBook_Item_Fruit_C",
0x63e3d653,"IsNewPictureBook_Item_Fruit_D",
0x14e4e6c5,"IsNewPictureBook_Item_Fruit_E",
0x8dedb77f,"IsNewPictureBook_Item_Fruit_F",
0xfaea87e9,"IsNewPictureBook_Item_Fruit_G",
0x6a559a78,"IsNewPictureBook_Item_Fruit_H",
0x1d52aaee,"IsNewPictureBook_Item_Fruit_I",
0x845bfb54,"IsNewPictureBook_Item_Fruit_J",
0x51506c21,"IsNewPictureBook_Item_InsectGet_K",
0x563da838,"IsNewPictureBook_Item_InsectGet_O",
0x3be04cd3,"IsNewPictureBook_Item_InsectGet_Z",
0x5156e11e,"IsNewPictureBook_Item_MushroomGet_D",
0xffff1784,"IsNewPictureBook_Item_Mushroom_A",
0x66f6463e,"IsNewPictureBook_Item_Mushroom_B",
0x11f176a8,"IsNewPictureBook_Item_Mushroom_C",
0xf892d39d,"IsNewPictureBook_Item_Mushroom_E",
0x619b8227,"IsNewPictureBook_Item_Mushroom_F",
0x8623af20,"IsNewPictureBook_Item_Mushroom_H",
0x682dce0c,"IsNewPictureBook_Item_Mushroom_J",
0x814e6b39,"IsNewPictureBook_Item_Mushroom_L",
0xf6495baf,"IsNewPictureBook_Item_Mushroom_M",
0x6f400a15,"IsNewPictureBook_Item_Mushroom_N",
0x18473a83,"IsNewPictureBook_Item_Mushroom_O",
0x2b2015d9,"IsNewPictureBook_Item_PlantGet_A",
0xb2294463,"IsNewPictureBook_Item_PlantGet_B",
0xc52e74f5,"IsNewPictureBook_Item_PlantGet_C",
0x2c4dd1c0,"IsNewPictureBook_Item_PlantGet_E",
0xb544807a,"IsNewPictureBook_Item_PlantGet_F",
0xc243b0ec,"IsNewPictureBook_Item_PlantGet_G",
0x52fcad7d,"IsNewPictureBook_Item_PlantGet_H",
0x25fb9deb,"IsNewPictureBook_Item_PlantGet_I",
0xbcf2cc51,"IsNewPictureBook_Item_PlantGet_J",
0x55916964,"IsNewPictureBook_Item_PlantGet_L",
0x229659f2,"IsNewPictureBook_Item_PlantGet_M",
0xcc9838de,"IsNewPictureBook_Item_PlantGet_O",
0x369705bd,"IsNewPictureBook_Item_PlantGet_Q",
0x7dc9971a,"IsNewPictureBook_NormalArrow",
0xfb846e29,"IsNewPictureBook_Obj_Mineral_A_01",
0xe931c1c7,"IsNewPictureBook_Obj_Mineral_B_01",
0x518da6a2,"IsNewPictureBook_Obj_Mineral_C_01",
0x470ddddc,"IsNewPictureBook_RemainsFire_Drone_A_01",
0xd4f61927,"IsNewPictureBook_TBox_Field_Iron",
0x0fd78d3a,"IsNewPictureBook_Weapon_Bow_001",
0x96dedc80,"IsNewPictureBook_Weapon_Bow_002",
0xe1d9ec16,"IsNewPictureBook_Weapon_Bow_003",
0x7fbd79b5,"IsNewPictureBook_Weapon_Bow_004",
0x91b31899,"IsNewPictureBook_Weapon_Bow_006",
0x010c0508,"IsNewPictureBook_Weapon_Bow_009",
0x16ccbc7b,"IsNewPictureBook_Weapon_Bow_011",
0xf8c2dd57,"IsNewPictureBook_Weapon_Bow_013",
0x66a648f4,"IsNewPictureBook_Weapon_Bow_014",
0x11a17862,"IsNewPictureBook_Weapon_Bow_015",
0x88a829d8,"IsNewPictureBook_Weapon_Bow_016",
0xffaf194e,"IsNewPictureBook_Weapon_Bow_017",
0xd3ef8e94,"IsNewPictureBook_Weapon_Bow_023",
0xa3857a1b,"IsNewPictureBook_Weapon_Bow_026",
0xd4824a8d,"IsNewPictureBook_Weapon_Bow_027",
0x443d571c,"IsNewPictureBook_Weapon_Bow_028",
0x333a678a,"IsNewPictureBook_Weapon_Bow_029",
0x53fdee6f,"IsNewPictureBook_Weapon_Bow_030",
0xbdf38f43,"IsNewPictureBook_Weapon_Bow_032",
0xcaf4bfd5,"IsNewPictureBook_Weapon_Bow_033",
0x23971ae0,"IsNewPictureBook_Weapon_Bow_035",
0xba9e4b5a,"IsNewPictureBook_Weapon_Bow_036",
0x5d26665d,"IsNewPictureBook_Weapon_Bow_038",
0x1cbc78a8,"IsNewPictureBook_Weapon_Bow_040",
0x40961bfd,"IsNewPictureBook_Weapon_Bow_071",
0x4270adab,"IsNewPictureBook_Weapon_Lsword_001",
0xdb79fc11,"IsNewPictureBook_Weapon_Lsword_002",
0xac7ecc87,"IsNewPictureBook_Weapon_Lsword_003",
0x321a5924,"IsNewPictureBook_Weapon_Lsword_004",
0x451d69b2,"IsNewPictureBook_Weapon_Lsword_005",
0xdc143808,"IsNewPictureBook_Weapon_Lsword_006",
0x2c6cac7c,"IsNewPictureBook_Weapon_Lsword_010",
0x5b6b9cea,"IsNewPictureBook_Weapon_Lsword_011",
0xc262cd50,"IsNewPictureBook_Weapon_Lsword_012",
0xb565fdc6,"IsNewPictureBook_Weapon_Lsword_013",
0x2b016865,"IsNewPictureBook_Weapon_Lsword_014",
0x5c0658f3,"IsNewPictureBook_Weapon_Lsword_015",
0xc50f0949,"IsNewPictureBook_Weapon_Lsword_016",
0xb20839df,"IsNewPictureBook_Weapon_Lsword_017",
0x22b7244e,"IsNewPictureBook_Weapon_Lsword_018",
0x55b014d8,"IsNewPictureBook_Weapon_Lsword_019",
0x0741ffbf,"IsNewPictureBook_Weapon_Lsword_020",
0x9e48ae05,"IsNewPictureBook_Weapon_Lsword_023",
0x002c3ba6,"IsNewPictureBook_Weapon_Lsword_024",
0x99256a1c,"IsNewPictureBook_Weapon_Lsword_027",
0x7e9d471b,"IsNewPictureBook_Weapon_Lsword_029",
0x1e5acefe,"IsNewPictureBook_Weapon_Lsword_030",
0x695dfe68,"IsNewPictureBook_Weapon_Lsword_031",
0xf054afd2,"IsNewPictureBook_Weapon_Lsword_032",
0x87539f44,"IsNewPictureBook_Weapon_Lsword_033",
0x19370ae7,"IsNewPictureBook_Weapon_Lsword_034",
0x6e303a71,"IsNewPictureBook_Weapon_Lsword_035",
0xf7396bcb,"IsNewPictureBook_Weapon_Lsword_036",
0x803e5b5d,"IsNewPictureBook_Weapon_Lsword_037",
0x108146cc,"IsNewPictureBook_Weapon_Lsword_038",
0x261c68af,"IsNewPictureBook_Weapon_Lsword_041",
0x2171acb6,"IsNewPictureBook_Weapon_Lsword_045",
0xcf7fcd9a,"IsNewPictureBook_Weapon_Lsword_047",
0x3f0759ee,"IsNewPictureBook_Weapon_Lsword_051",
0x4f6dad61,"IsNewPictureBook_Weapon_Lsword_054",
0x386a9df7,"IsNewPictureBook_Weapon_Lsword_055",
0xa163cc4d,"IsNewPictureBook_Weapon_Lsword_056",
0x7d5bcfe3,"IsNewPictureBook_Weapon_Lsword_074",
0x4eddb4de,"IsNewPictureBook_Weapon_Shield_001",
0xd7d4e564,"IsNewPictureBook_Weapon_Shield_002",
0xa0d3d5f2,"IsNewPictureBook_Weapon_Shield_003",
0x3eb74051,"IsNewPictureBook_Weapon_Shield_004",
0x49b070c7,"IsNewPictureBook_Weapon_Shield_005",
0xd0b9217d,"IsNewPictureBook_Weapon_Shield_006",
0xa7be11eb,"IsNewPictureBook_Weapon_Shield_007",
0x37010c7a,"IsNewPictureBook_Weapon_Shield_008",
0x40063cec,"IsNewPictureBook_Weapon_Shield_009",
0xb9c8e4b3,"IsNewPictureBook_Weapon_Shield_013",
0x27ac7110,"IsNewPictureBook_Weapon_Shield_014",
0x50ab4186,"IsNewPictureBook_Weapon_Shield_015",
0xc9a2103c,"IsNewPictureBook_Weapon_Shield_016",
0xbea520aa,"IsNewPictureBook_Weapon_Shield_017",
0x2e1a3d3b,"IsNewPictureBook_Weapon_Shield_018",
0x7cebd65c,"IsNewPictureBook_Weapon_Shield_021",
0xe5e287e6,"IsNewPictureBook_Weapon_Shield_022",
0x92e5b770,"IsNewPictureBook_Weapon_Shield_023",
0x7b861245,"IsNewPictureBook_Weapon_Shield_025",
0xe28f43ff,"IsNewPictureBook_Weapon_Shield_026",
0x12f7d78b,"IsNewPictureBook_Weapon_Shield_030",
0x65f0e71d,"IsNewPictureBook_Weapon_Shield_031",
0xfcf9b6a7,"IsNewPictureBook_Weapon_Shield_032",
0x8bfe8631,"IsNewPictureBook_Weapon_Shield_033",
0x159a1392,"IsNewPictureBook_Weapon_Shield_034",
0x629d2304,"IsNewPictureBook_Weapon_Shield_035",
0xfb9472be,"IsNewPictureBook_Weapon_Shield_036",
0x8c934228,"IsNewPictureBook_Weapon_Shield_037",
0x1c2c5fb9,"IsNewPictureBook_Weapon_Shield_038",
0x5db6414c,"IsNewPictureBook_Weapon_Shield_040",
0x2ab171da,"IsNewPictureBook_Weapon_Shield_041",
0xb3b82060,"IsNewPictureBook_Weapon_Shield_042",
0xe94412c1,"IsNewPictureBook_Weapon_Spear_001",
0x704d437b,"IsNewPictureBook_Weapon_Spear_002",
0x074a73ed,"IsNewPictureBook_Weapon_Spear_003",
0x992ee64e,"IsNewPictureBook_Weapon_Spear_004",
0xee29d6d8,"IsNewPictureBook_Weapon_Spear_005",
0x77208762,"IsNewPictureBook_Weapon_Spear_006",
0x0027b7f4,"IsNewPictureBook_Weapon_Spear_007",
0x9098aa65,"IsNewPictureBook_Weapon_Spear_008",
0xe79f9af3,"IsNewPictureBook_Weapon_Spear_009",
0x87581316,"IsNewPictureBook_Weapon_Spear_010",
0xf05f2380,"IsNewPictureBook_Weapon_Spear_011",
0x6956723a,"IsNewPictureBook_Weapon_Spear_012",
0x1e5142ac,"IsNewPictureBook_Weapon_Spear_013",
0x8035d70f,"IsNewPictureBook_Weapon_Spear_014",
0xf732e799,"IsNewPictureBook_Weapon_Spear_015",
0x6e3bb623,"IsNewPictureBook_Weapon_Spear_016",
0x193c86b5,"IsNewPictureBook_Weapon_Spear_017",
0x89839b24,"IsNewPictureBook_Weapon_Spear_018",
0xdb727043,"IsNewPictureBook_Weapon_Spear_021",
0x427b21f9,"IsNewPictureBook_Weapon_Spear_022",
0x357c116f,"IsNewPictureBook_Weapon_Spear_023",
0xab1884cc,"IsNewPictureBook_Weapon_Spear_024",
0xdc1fb45a,"IsNewPictureBook_Weapon_Spear_025",
0x3211d576,"IsNewPictureBook_Weapon_Spear_027",
0xa2aec8e7,"IsNewPictureBook_Weapon_Spear_028",
0xd5a9f871,"IsNewPictureBook_Weapon_Spear_029",
0xb56e7194,"IsNewPictureBook_Weapon_Spear_030",
0xc2694102,"IsNewPictureBook_Weapon_Spear_031",
0x5b6010b8,"IsNewPictureBook_Weapon_Spear_032",
0x2c67202e,"IsNewPictureBook_Weapon_Spear_033",
0xb203b58d,"IsNewPictureBook_Weapon_Spear_034",
0xc504851b,"IsNewPictureBook_Weapon_Spear_035",
0x5c0dd4a1,"IsNewPictureBook_Weapon_Spear_036",
0x2b0ae437,"IsNewPictureBook_Weapon_Spear_037",
0xbbb5f9a6,"IsNewPictureBook_Weapon_Spear_038",
0x644b72f0,"IsNewPictureBook_Weapon_Spear_047",
0x83f35ff7,"IsNewPictureBook_Weapon_Spear_049",
0xe334d612,"IsNewPictureBook_Weapon_Spear_050",
0x2d135681,"IsNewPictureBook_Weapon_Sword_001",
0xb41a073b,"IsNewPictureBook_Weapon_Sword_002",
0xc31d37ad,"IsNewPictureBook_Weapon_Sword_003",
0x5d79a20e,"IsNewPictureBook_Weapon_Sword_004",
0x2a7e9298,"IsNewPictureBook_Weapon_Sword_005",
0xb377c322,"IsNewPictureBook_Weapon_Sword_006",
0xc470f3b4,"IsNewPictureBook_Weapon_Sword_007",
0x54cfee25,"IsNewPictureBook_Weapon_Sword_008",
0x23c8deb3,"IsNewPictureBook_Weapon_Sword_009",
0xda0606ec,"IsNewPictureBook_Weapon_Sword_013",
0x4462934f,"IsNewPictureBook_Weapon_Sword_014",
0x3365a3d9,"IsNewPictureBook_Weapon_Sword_015",
0xaa6cf263,"IsNewPictureBook_Weapon_Sword_016",
0xdd6bc2f5,"IsNewPictureBook_Weapon_Sword_017",
0x4dd4df64,"IsNewPictureBook_Weapon_Sword_018",
0x3ad3eff2,"IsNewPictureBook_Weapon_Sword_019",
0x68220495,"IsNewPictureBook_Weapon_Sword_020",
0x1f253403,"IsNewPictureBook_Weapon_Sword_021",
0x862c65b9,"IsNewPictureBook_Weapon_Sword_022",
0xf12b552f,"IsNewPictureBook_Weapon_Sword_023",
0x6f4fc08c,"IsNewPictureBook_Weapon_Sword_024",
0x1848f01a,"IsNewPictureBook_Weapon_Sword_025",
0xf6469136,"IsNewPictureBook_Weapon_Sword_027",
0x11febc31,"IsNewPictureBook_Weapon_Sword_029",
0x713935d4,"IsNewPictureBook_Weapon_Sword_030",
0x063e0542,"IsNewPictureBook_Weapon_Sword_031",
0xe830646e,"IsNewPictureBook_Weapon_Sword_033",
0x7654f1cd,"IsNewPictureBook_Weapon_Sword_034",
0x0153c15b,"IsNewPictureBook_Weapon_Sword_035",
0x3e78a313,"IsNewPictureBook_Weapon_Sword_040",
0x497f9385,"IsNewPictureBook_Weapon_Sword_041",
0xa771f2a9,"IsNewPictureBook_Weapon_Sword_043",
0x3915670a,"IsNewPictureBook_Weapon_Sword_044",
0xa01c36b0,"IsNewPictureBook_Weapon_Sword_047",
0x30a32b21,"IsNewPictureBook_Weapon_Sword_048",
0x47a41bb7,"IsNewPictureBook_Weapon_Sword_049",
0x27639252,"IsNewPictureBook_Weapon_Sword_050",
0x5064a2c4,"IsNewPictureBook_Weapon_Sword_051",
0xc96df37e,"IsNewPictureBook_Weapon_Sword_052",
0xbe6ac3e8,"IsNewPictureBook_Weapon_Sword_053",
0x0c4ec191,"IsNewPictureBook_Weapon_Sword_060",
0x7b49f107,"IsNewPictureBook_Weapon_Sword_061",
0xe240a0bd,"IsNewPictureBook_Weapon_Sword_062",
0x1555f0d0,"IsNewPictureBook_Weapon_Sword_070",
0x8c5ca16a,"IsNewPictureBook_Weapon_Sword_073",
0x8ffb5b31,"IsOtherFairyRevivalForGreatFairy1",
0x16f20a8b,"IsOtherFairyRevivalForGreatFairy2",
0x61f53a1d,"IsOtherFairyRevivalForGreatFairy3",
0xff91afbe,"IsOtherFairyRevivalForGreatFairy4",
0x8f5495fe,"IsPlayed_Demo000_0",
0x615af4d2,"IsPlayed_Demo000_2",
0x8e96ffc9,"IsPlayed_Demo001_0",
0x8cd04190,"IsPlayed_Demo002_0",
0x8d122ba7,"IsPlayed_Demo003_0",
0x885d3d22,"IsPlayed_Demo004_0",
0x899f5715,"IsPlayed_Demo005_0",
0xfe986783,"IsPlayed_Demo005_1",
0x8bd9e94c,"IsPlayed_Demo006_0",
0x8a1b837b,"IsPlayed_Demo007_0",
0xfd1cb3ed,"IsPlayed_Demo007_1",
0x6415e257,"IsPlayed_Demo007_2",
0x8147c446,"IsPlayed_Demo008_0",
0xf640f4d0,"IsPlayed_Demo008_1",
0x6f49a56a,"IsPlayed_Demo008_2",
0x184e95fc,"IsPlayed_Demo008_3",
0x862a005f,"IsPlayed_Demo008_4",
0xf89b7ce2,"IsPlayed_Demo008_9",
0x37e8f29b,"IsPlayed_Demo010_0",
0x40efc20d,"IsPlayed_Demo010_1",
0x362a98ac,"IsPlayed_Demo011_0",
0x346c26f5,"IsPlayed_Demo012_0",
0x35ae4cc2,"IsPlayed_Demo013_0",
0x30e15a47,"IsPlayed_Demo014_0",
0x33658e29,"IsPlayed_Demo016_0",
0x32a7e41e,"IsPlayed_Demo017_0",
0x39fba323,"IsPlayed_Demo018_0",
0x3839c914,"IsPlayed_Demo019_0",
0x255d5d75,"IsPlayed_Demo020_0",
0x249f3742,"IsPlayed_Demo021_0",
0x26d9891b,"IsPlayed_Demo022_0",
0x271be32c,"IsPlayed_Demo023_0",
0x2254f5a9,"IsPlayed_Demo024_0",
0x5553c53f,"IsPlayed_Demo024_1",
0xcc5a9485,"IsPlayed_Demo024_2",
0x23969f9e,"IsPlayed_Demo025_0",
0x5491af08,"IsPlayed_Demo025_1",
0xcd98feb2,"IsPlayed_Demo025_2",
0x21d021c7,"IsPlayed_Demo026_0",
0x20124bf0,"IsPlayed_Demo027_0",
0xce1c2adc,"IsPlayed_Demo027_2",
0x2b4e0ccd,"IsPlayed_Demo028_0",
0x2a8c66fa,"IsPlayed_Demo029_0",
0x9de13a10,"IsPlayed_Demo030_0",
0x9e65ee7e,"IsPlayed_Demo032_0",
0x9fa78449,"IsPlayed_Demo033_0",
0xe8a0b4df,"IsPlayed_Demo033_1",
0x9ae892cc,"IsPlayed_Demo034_0",
0x9b2af8fb,"IsPlayed_Demo035_0",
0x98ae2c95,"IsPlayed_Demo037_0",
0x93f26ba8,"IsPlayed_Demo038_0",
0x9230019f,"IsPlayed_Demo039_0",
0x003602a9,"IsPlayed_Demo040_0",
0x01f4689e,"IsPlayed_Demo041_0",
0x03b2d6c7,"IsPlayed_Demo042_0",
0x74b5e651,"IsPlayed_Demo042_1",
0x0270bcf0,"IsPlayed_Demo043_0",
0x073faa75,"IsPlayed_Demo044_0",
0x06fdc042,"IsPlayed_Demo045_0",
0x04bb7e1b,"IsPlayed_Demo046_0",
0x0579142c,"IsPlayed_Demo047_0",
0x0e255311,"IsPlayed_Demo048_0",
0x79226387,"IsPlayed_Demo048_1",
0x0fe73926,"IsPlayed_Demo049_0",
0xb88a65cc,"IsPlayed_Demo050_0",
0xb9480ffb,"IsPlayed_Demo051_0",
0xb234bc4e,"IsPlayed_Demo100_0",
0xb3f6d679,"IsPlayed_Demo101_0",
0xc4f1e6ef,"IsPlayed_Demo101_1",
0x5df8b755,"IsPlayed_Demo101_2",
0x2aff87c3,"IsPlayed_Demo101_3",
0xb49b1260,"IsPlayed_Demo101_4",
0xc39c22f6,"IsPlayed_Demo101_5",
0x5a95734c,"IsPlayed_Demo101_6",
0x2d9243da,"IsPlayed_Demo101_7",
0xbd2d5e4b,"IsPlayed_Demo101_8",
0xca2a6edd,"IsPlayed_Demo101_9",
0xb1b06820,"IsPlayed_Demo102_0",
0xb0720217,"IsPlayed_Demo103_0",
0xb53d1492,"IsPlayed_Demo104_0",
0xc23a2404,"IsPlayed_Demo104_1",
0xb4ff7ea5,"IsPlayed_Demo105_0",
0xc3f84e33,"IsPlayed_Demo105_1",
0xbde587c1,"IsPlayed_Demo109_0",
0xcae2b757,"IsPlayed_Demo109_1",
0x53ebe6ed,"IsPlayed_Demo109_2",
0x0a88db2b,"IsPlayed_Demo110_0",
0x0b4ab11c,"IsPlayed_Demo111_0",
0x090c0f45,"IsPlayed_Demo112_0",
0x08ce6572,"IsPlayed_Demo113_0",
0x0d8173f7,"IsPlayed_Demo114_0",
0x0c4319c0,"IsPlayed_Demo115_0",
0x0e05a799,"IsPlayed_Demo116_0",
0x0fc7cdae,"IsPlayed_Demo117_0",
0x049b8a93,"IsPlayed_Demo118_0",
0x0559e0a4,"IsPlayed_Demo119_0",
0x183d74c5,"IsPlayed_Demo120_0",
0x19ff1ef2,"IsPlayed_Demo121_0",
0x1bb9a0ab,"IsPlayed_Demo122_0",
0x1a7bca9c,"IsPlayed_Demo123_0",
0x1f34dc19,"IsPlayed_Demo124_0",
0x1ef6b62e,"IsPlayed_Demo125_0",
0x1cb00877,"IsPlayed_Demo126_0",
0x1d726240,"IsPlayed_Demo127_0",
0x162e257d,"IsPlayed_Demo128_0",
0x17ec4f4a,"IsPlayed_Demo129_0",
0xa08113a0,"IsPlayed_Demo130_0",
0xa1437997,"IsPlayed_Demo131_0",
0xa305c7ce,"IsPlayed_Demo132_0",
0xa2c7adf9,"IsPlayed_Demo133_0",
0xa788bb7c,"IsPlayed_Demo134_0",
0xa64ad14b,"IsPlayed_Demo135_0",
0xa40c6f12,"IsPlayed_Demo136_0",
0xa5ce0525,"IsPlayed_Demo137_0",
0xae924218,"IsPlayed_Demo138_0",
0xaf50282f,"IsPlayed_Demo139_0",
0x3d562b19,"IsPlayed_Demo140_0",
0x3c94412e,"IsPlayed_Demo141_0",
0x4b9371b8,"IsPlayed_Demo141_1",
0xd29a2002,"IsPlayed_Demo141_2",
0xa59d1094,"IsPlayed_Demo141_3",
0x3bf98537,"IsPlayed_Demo141_4",
0x3ed2ff77,"IsPlayed_Demo142_0",
0x3f109540,"IsPlayed_Demo143_0",
0x4817a5d6,"IsPlayed_Demo143_1",
0xd11ef46c,"IsPlayed_Demo143_2",
0xa619c4fa,"IsPlayed_Demo143_3",
0x387d5159,"IsPlayed_Demo143_4",
0x3b9de9f2,"IsPlayed_Demo145_0",
0x39db57ab,"IsPlayed_Demo146_0",
0x4edc673d,"IsPlayed_Demo146_1",
0x38193d9c,"IsPlayed_Demo147_0",
0x33457aa1,"IsPlayed_Demo148_0",
0x45802000,"IsPlayed_Demo149_1",
0x85ea4c7c,"IsPlayed_Demo150_0",
0x8428264b,"IsPlayed_Demo151_0",
0x866e9812,"IsPlayed_Demo152_0",
0x87acf225,"IsPlayed_Demo153_0",
0x82e3e4a0,"IsPlayed_Demo154_0",
0xf5e4d436,"IsPlayed_Demo154_1",
0x6ced858c,"IsPlayed_Demo154_2",
0x83218e97,"IsPlayed_Demo155_0",
0x816730ce,"IsPlayed_Demo156_0",
0x8bf91dc4,"IsPlayed_Demo158_0",
0x8a3b77f3,"IsPlayed_Demo159_0",
0x975fe392,"IsPlayed_Demo160_0",
0x969d89a5,"IsPlayed_Demo161_0",
0x94db37fc,"IsPlayed_Demo162_0",
0x95195dcb,"IsPlayed_Demo163_0",
0x90564b4e,"IsPlayed_Demo164_0",
0x91942179,"IsPlayed_Demo165_0",
0xe69311ef,"IsPlayed_Demo165_1",
0x7f9a4055,"IsPlayed_Demo165_2",
0x089d70c3,"IsPlayed_Demo165_3",
0x96f9e560,"IsPlayed_Demo165_4",
0xe1fed5f6,"IsPlayed_Demo165_5",
0x93d29f20,"IsPlayed_Demo166_0",
0x9210f517,"IsPlayed_Demo167_0",
0x988ed81d,"IsPlayed_Demo169_0",
0x2da53aae,"IsPlayed_Demo173_0",
0x5aa20a38,"IsPlayed_Demo173_1",
0xc3ab5b82,"IsPlayed_Demo173_2",
0xb4ac6b14,"IsPlayed_Demo173_3",
0x2ac8feb7,"IsPlayed_Demo173_4",
0x5dcfce21,"IsPlayed_Demo173_5",
0xc4c69f9b,"IsPlayed_Demo173_6",
0xb3c1af0d,"IsPlayed_Demo173_7",
0x237eb29c,"IsPlayed_Demo173_8",
0x5479820a,"IsPlayed_Demo173_9",
0x28ea2c2b,"IsPlayed_Demo174_0",
0x2928461c,"IsPlayed_Demo175_0",
0x2b6ef845,"IsPlayed_Demo176_0",
0x2aac9272,"IsPlayed_Demo177_0",
0x5daba2e4,"IsPlayed_Demo177_1",
0xc8f4ef2e,"IsPlayed_Demo300_0",
0xce3f2dc5,"IsPlayed_Demo305_0",
0xc725d4a1,"IsPlayed_Demo309_0",
0xb022e437,"IsPlayed_Demo309_1",
0x292bb58d,"IsPlayed_Demo309_2",
0x7e5bd9f3,"IsPlayed_Demo318_0",
0x67b23120,"IsPlayed_Demo327_0",
0xda4140c0,"IsPlayed_Demo330_0",
0xad467056,"IsPlayed_Demo330_1",
0xdb832af7,"IsPlayed_Demo331_0",
0xd807fe99,"IsPlayed_Demo333_0",
0x409fd0a5,"IsPlayed_Demo344_0",
0x3798e033,"IsPlayed_Demo344_1",
0xae91b189,"IsPlayed_Demo344_2",
0x47f214bc,"IsPlayed_Demo344_4",
0x431b04cb,"IsPlayed_Demo346_0",
0x341c345d,"IsPlayed_Demo346_1",
0xda125571,"IsPlayed_Demo346_3",
0x4476c0d2,"IsPlayed_Demo346_4",
0x3371f044,"IsPlayed_Demo346_5",
0xaa78a1fe,"IsPlayed_Demo346_6",
0x3ac7bc6f,"IsPlayed_Demo346_9",
0x42d96efc,"IsPlayed_Demo347_0",
0x35de5e6a,"IsPlayed_Demo347_1",
0xacd70fd0,"IsPlayed_Demo347_2",
0xdbd03f46,"IsPlayed_Demo347_3",
0x45b4aae5,"IsPlayed_Demo347_4",
0x32b39a73,"IsPlayed_Demo347_5",
0xabbacbc9,"IsPlayed_Demo347_6",
0x498529c1,"IsPlayed_Demo348_0",
0x3e821957,"IsPlayed_Demo348_1",
0xefd90eab,"IsPlayed_Demo363_0",
0x5523d797,"IsPlayed_Demo370_0",
0x5b30862f,"IsPlayed_Demo378_0",
0x2c37b6b9,"IsPlayed_Demo378_1",
0x0d40c7c1,"IsPlayed_Demo380_0",
0x7b165909,"IsPlayed_Demo401_0",
0x0c11699f,"IsPlayed_Demo401_1",
0xbf601bd1,"IsPlayed_Demo480_0",
0xc8672b47,"IsPlayed_Demo480_1",
0x516e7afd,"IsPlayed_Demo480_2",
0x26694a6b,"IsPlayed_Demo480_3",
0x3d7449ee,"IsPlayed_Demo700_0",
0x3cb623d9,"IsPlayed_Demo701_0",
0x3ef09d80,"IsPlayed_Demo702_0",
0x3a7de132,"IsPlayed_Demo704_0",
0x3bbf8b05,"IsPlayed_Demo705_0",
0x39f9355c,"IsPlayed_Demo706_0",
0x383b5f6b,"IsPlayed_Demo707_0",
0x33671856,"IsPlayed_Demo708_0",
0x32a57261,"IsPlayed_Demo709_0",
0x85c82e8b,"IsPlayed_Demo710_0",
0x864cfae5,"IsPlayed_Demo712_0",
0xf14bca73,"IsPlayed_Demo712_1",
0x68429bc9,"IsPlayed_Demo712_2",
0x1f45ab5f,"IsPlayed_Demo712_3",
0x8087380e,"IsPlayed_Demo717_0",
0x8bdb7f33,"IsPlayed_Demo718_0",
0x8a191504,"IsPlayed_Demo719_0",
0x977d8165,"IsPlayed_Demo720_0",
0x96bfeb52,"IsPlayed_Demo721_0",
0x4fad3deb,"IsPlayed_Demo799_0",
0xbf24de3f,"IsPlayed_Demo800_0",
0xbee6b408,"IsPlayed_Demo801_0",
0xbca00a51,"IsPlayed_Demo802_0",
0xbd626066,"IsPlayed_Demo803_0",
0xb82d76e3,"IsPlayed_Demo804_0",
0xb9ef1cd4,"IsPlayed_Demo805_0",
0xbba9a28d,"IsPlayed_Demo806_0",
0xba6bc8ba,"IsPlayed_Demo807_0",
0xb1378f87,"IsPlayed_Demo808_0",
0xb0f5e5b0,"IsPlayed_Demo809_0",
0xc7f2d526,"IsPlayed_Demo809_1",
0x0798b95a,"IsPlayed_Demo810_0",
0x065ad36d,"IsPlayed_Demo811_0",
0x041c6d34,"IsPlayed_Demo812_0",
0x05de0703,"IsPlayed_Demo813_0",
0x00911186,"IsPlayed_Demo814_0",
0x01537bb1,"IsPlayed_Demo815_0",
0x0315c5e8,"IsPlayed_Demo816_0",
0x02d7afdf,"IsPlayed_Demo817_0",
0x098be8e2,"IsPlayed_Demo818_0",
0x084982d5,"IsPlayed_Demo819_0",
0x152d16b4,"IsPlayed_Demo820_0",
0x14ef7c83,"IsPlayed_Demo821_0",
0x16a9c2da,"IsPlayed_Demo822_0",
0x176ba8ed,"IsPlayed_Demo823_0",
0x1224be68,"IsPlayed_Demo824_0",
0x13e6d45f,"IsPlayed_Demo825_0",
0x11a06a06,"IsPlayed_Demo826_0",
0x10620031,"IsPlayed_Demo827_0",
0x1b3e470c,"IsPlayed_Demo828_0",
0x1afc2d3b,"IsPlayed_Demo829_0",
0xad9171d1,"IsPlayed_Demo830_0",
0xac531be6,"IsPlayed_Demo831_0",
0xae15a5bf,"IsPlayed_Demo832_0",
0xafd7cf88,"IsPlayed_Demo833_0",
0x9c844308,"IsPlayed_Demo865_0",
0x9ec2fd51,"IsPlayed_Demo866_0",
0x9f009766,"IsPlayed_Demo867_0",
0x945cd05b,"IsPlayed_Demo868_0",
0x959eba6c,"IsPlayed_Demo869_0",
0x22f3e686,"IsPlayed_Demo870_0",
0x23318cb1,"IsPlayed_Demo871_0",
0x217732e8,"IsPlayed_Demo872_0",
0x20b558df,"IsPlayed_Demo873_0",
0x25fa4e5a,"IsPlayed_Demo874_0",
0x2438246d,"IsPlayed_Demo875_0",
0x267e9a34,"IsPlayed_Demo876_0",
0x27bcf003,"IsPlayed_Demo877_0",
0x2ce0b73e,"IsPlayed_Demo878_0",
0x2d22dd09,"IsPlayed_Demo879_0",
0x7a90f6d0,"IsPlayed_Demo880_0",
0x7b529ce7,"IsPlayed_Demo881_0",
0x791422be,"IsPlayed_Demo882_0",
0x78d64889,"IsPlayed_Demo883_0",
0x7d995e0c,"IsPlayed_Demo884_0",
0x7c5b343b,"IsPlayed_Demo885_0",
0x7e1d8a62,"IsPlayed_Demo886_0",
0x7fdfe055,"IsPlayed_Demo887_0",
0x7483a768,"IsPlayed_Demo888_0",
0x7541cd5f,"IsPlayed_Demo889_0",
0xc22c91b5,"IsPlayed_Demo890_0",
0xc3eefb82,"IsPlayed_Demo891_0",
0xc1a845db,"IsPlayed_Demo892_0",
0xc06a2fec,"IsPlayed_Demo893_0",
0xc5253969,"IsPlayed_Demo894_0",
0xc4e7535e,"IsPlayed_Demo895_0",
0xc6a1ed07,"IsPlayed_Demo896_0",
0xc7638730,"IsPlayed_Demo897_0",
0xcc3fc00d,"IsPlayed_Demo898_0",
0xcdfdaa3a,"IsPlayed_Demo899_0",
0x8244f78f,"IsPlayed_Demo900_0",
0x0d2660d8,"IsPlayed_Demo940_0",
0x0ce40aef,"IsPlayed_Demo941_0",
0x0ea2b4b6,"IsPlayed_Demo942_0",
0x0f60de81,"IsPlayed_Demo943_0",
0x0a2fc804,"IsPlayed_Demo944_0",
0x0beda233,"IsPlayed_Demo945_0",
0x09ab1c6a,"IsPlayed_Demo946_0",
0x0869765d,"IsPlayed_Demo947_0",
0x03353160,"IsPlayed_Demo948_0",
0x02f75b57,"IsPlayed_Demo949_0",
0xb59a07bd,"IsPlayed_Demo950_0",
0x1f93cf36,"IsPlayed_Demo970_0",
0x4632b557,"IsPlayed_Demo981_0",
0x44740b0e,"IsPlayed_Demo982_0",
0x33733b98,"IsPlayed_Demo982_1",
0xaa7a6a22,"IsPlayed_Demo982_2",
0xdd7d5ab4,"IsPlayed_Demo982_3",
0x4319cf17,"IsPlayed_Demo982_4",
0x341eff81,"IsPlayed_Demo982_5",
0x45b66139,"IsPlayed_Demo983_0",
0x32b151af,"IsPlayed_Demo983_1",
0xabb80015,"IsPlayed_Demo983_2",
0xdcbf3083,"IsPlayed_Demo983_3",
0x42dba520,"IsPlayed_Demo983_4",
0x35dc95b6,"IsPlayed_Demo983_5",
0xacd5c40c,"IsPlayed_Demo983_6",
0xdbd2f49a,"IsPlayed_Demo983_7",
0x4b6de90b,"IsPlayed_Demo983_8",
0x3c6ad99d,"IsPlayed_Demo983_9",
0x40f977bc,"IsPlayed_Demo984_0",
0x37fe472a,"IsPlayed_Demo984_1",
0xd9f02606,"IsPlayed_Demo984_3",
0x4794b3a5,"IsPlayed_Demo984_4",
0xa99ad289,"IsPlayed_Demo984_6",
0xde9de21f,"IsPlayed_Demo984_7",
0x4e22ff8e,"IsPlayed_Demo984_8",
0x3925cf18,"IsPlayed_Demo984_9",
0x413b1d8b,"IsPlayed_Demo985_0",
0x437da3d2,"IsPlayed_Demo986_0",
0x42bfc9e5,"IsPlayed_Demo987_0",
0x49e38ed8,"IsPlayed_Demo988_0",
0x3ee4be4e,"IsPlayed_Demo988_1",
0x4821e4ef,"IsPlayed_Demo989_0",
0x3f26d479,"IsPlayed_Demo989_1",
0xff4cb805,"IsPlayed_Demo990_0",
0x884b8893,"IsPlayed_Demo990_1",
0x1142d929,"IsPlayed_Demo990_2",
0xfe8ed232,"IsPlayed_Demo991_0",
0xfcc86c6b,"IsPlayed_Demo992_0",
0x8bcf5cfd,"IsPlayed_Demo992_1",
0x12c60d47,"IsPlayed_Demo992_2",
0x65c13dd1,"IsPlayed_Demo992_3",
0xf84510d9,"IsPlayed_Demo994_0",
0x8f42204f,"IsPlayed_Demo994_1",
0x164b71f5,"IsPlayed_Demo994_2",
0x614c4163,"IsPlayed_Demo994_3",
0xff28d4c0,"IsPlayed_Demo994_4",
0x882fe456,"IsPlayed_Demo994_5",
0x1126b5ec,"IsPlayed_Demo994_6",
0x6621857a,"IsPlayed_Demo994_7",
0xf69e98eb,"IsPlayed_Demo994_8",
0x8199a87d,"IsPlayed_Demo994_9",
0xf9877aee,"IsPlayed_Demo995_0",
0xfbc1c4b7,"IsPlayed_Demo996_0",
0x8cc6f421,"IsPlayed_Demo996_1",
0x15cfa59b,"IsPlayed_Demo996_2",
0xf15fe9bd,"IsPlayed_Demo998_0",
0x8658d92b,"IsPlayed_Demo998_1",
0x1f518891,"IsPlayed_Demo998_2",
0x719ff43a,"IsPrizonStalHinoxDead",
0xe58bd4b1,"IsRegisteredPictureBook_AncientArrow",
0x09360a68,"IsRegisteredPictureBook_Animal_Bear_A",
0x903f5bd2,"IsRegisteredPictureBook_Animal_Bear_B",
0x438612c9,"IsRegisteredPictureBook_Animal_Boar_A",
0xda8f4373,"IsRegisteredPictureBook_Animal_Boar_B",
0x8d04cb4d,"IsRegisteredPictureBook_Animal_Bull_A",
0x0ffc0807,"IsRegisteredPictureBook_Animal_Cassowary_A",
0x1f7f42f1,"IsRegisteredPictureBook_Animal_Cow_A",
0xf6be2787,"IsRegisteredPictureBook_Animal_Crow_A",
0x500d7e22,"IsRegisteredPictureBook_Animal_Deer_A",
0xb2fde9ff,"IsRegisteredPictureBook_Animal_Doe_A",
0xb1793d91,"IsRegisteredPictureBook_Animal_Dog_A",
0x97b642ab,"IsRegisteredPictureBook_Animal_Elk_A",
0xdcc38abc,"IsRegisteredPictureBook_Animal_Fox_A",
0x45cadb06,"IsRegisteredPictureBook_Animal_Fox_B",
0x17c6ffc8,"IsRegisteredPictureBook_Animal_Goat_A",
0xddc95afe,"IsRegisteredPictureBook_Animal_Gull_A",
0x3ba81f72,"IsRegisteredPictureBook_Animal_Hawk_A",
0x4012cccd,"IsRegisteredPictureBook_Animal_Heron_A",
0xd91b9d77,"IsRegisteredPictureBook_Animal_Heron_B",
0x6b3ee329,"IsRegisteredPictureBook_Animal_Insect_A",
0x91003804,"IsRegisteredPictureBook_Animal_Insect_AA",
0x080969be,"IsRegisteredPictureBook_Animal_Insect_AB",
0xf237b293,"IsRegisteredPictureBook_Animal_Insect_B",
0x85308205,"IsRegisteredPictureBook_Animal_Insect_C",
0x6c532730,"IsRegisteredPictureBook_Animal_Insect_E",
0xf55a768a,"IsRegisteredPictureBook_Animal_Insect_F",
0x825d461c,"IsRegisteredPictureBook_Animal_Insect_G",
0x12e25b8d,"IsRegisteredPictureBook_Animal_Insect_H",
0x65e56b1b,"IsRegisteredPictureBook_Animal_Insect_I",
0x6288af02,"IsRegisteredPictureBook_Animal_Insect_M",
0xfb81feb8,"IsRegisteredPictureBook_Animal_Insect_N",
0x018ec3db,"IsRegisteredPictureBook_Animal_Insect_P",
0x7689f34d,"IsRegisteredPictureBook_Animal_Insect_Q",
0xef80a2f7,"IsRegisteredPictureBook_Animal_Insect_R",
0x98879261,"IsRegisteredPictureBook_Animal_Insect_S",
0x06e307c2,"IsRegisteredPictureBook_Animal_Insect_T",
0x0f554be9,"IsRegisteredPictureBook_Animal_Insect_X",
0x3331fda5,"IsRegisteredPictureBook_Animal_Kokko_A",
0xc0e01604,"IsRegisteredPictureBook_Animal_LittleBird_A",
0x59e947be,"IsRegisteredPictureBook_Animal_LittleBird_B",
0x2eee7728,"IsRegisteredPictureBook_Animal_LittleBird_C",
0xb08ae28b,"IsRegisteredPictureBook_Animal_LittleBird_D",
0xc78dd21d,"IsRegisteredPictureBook_Animal_LittleBird_E",
0x5e8483a7,"IsRegisteredPictureBook_Animal_LittleBird_F",
0x0e4d2e34,"IsRegisteredPictureBook_Animal_Pigeon_A",
0x97447f8e,"IsRegisteredPictureBook_Animal_Pigeon_B",
0xcd3540ee,"IsRegisteredPictureBook_Animal_Ptarmigan_A",
0x543c1154,"IsRegisteredPictureBook_Animal_Ptarmigan_B",
0x916cb752,"IsRegisteredPictureBook_Animal_Rhino_A",
0x2ed8cfbb,"IsRegisteredPictureBook_Animal_RupeeRabbit_A",
0x329032ab,"IsRegisteredPictureBook_Animal_Sheep_A",
0xaa9773ef,"IsRegisteredPictureBook_Animal_Squirrel_A",
0x0b669e4b,"IsRegisteredPictureBook_Animal_SunazarashiSP_C",
0x5f7e9d3e,"IsRegisteredPictureBook_Animal_Sunazarashi_A",
0xf66009c5,"IsRegisteredPictureBook_Animal_WildDuck_A",
0xe7d7d88e,"IsRegisteredPictureBook_Animal_WildGoat_A",
0xf9d85290,"IsRegisteredPictureBook_Animal_Wolf_A",
0x60d1032a,"IsRegisteredPictureBook_Animal_Wolf_B",
0x17d633bc,"IsRegisteredPictureBook_Animal_Wolf_C",
0xf33d8538,"IsRegisteredPictureBook_BeeHome",
0xc8f4616e,"IsRegisteredPictureBook_BombArrow_A",
0xdf53ec16,"IsRegisteredPictureBook_ElectricArrow",
0xafbeb399,"IsRegisteredPictureBook_Enemy_Assassin_Junior",
0xd8305c91,"IsRegisteredPictureBook_Enemy_Assassin_Middle",
0xeb877748,"IsRegisteredPictureBook_Enemy_Assassin_Senior",
0xe3f4022d,"IsRegisteredPictureBook_Enemy_Bokoblin_Bone_Junior",
0x95056229,"IsRegisteredPictureBook_Enemy_Bokoblin_Dark",
0x4c6c1366,"IsRegisteredPictureBook_Enemy_Bokoblin_Junior",
0x3be2fc6e,"IsRegisteredPictureBook_Enemy_Bokoblin_Middle",
0x0855d7b7,"IsRegisteredPictureBook_Enemy_Bokoblin_Senior",
0xf0d70478,"IsRegisteredPictureBook_Enemy_Chuchu_Electric_Junior",
0xd4eaaaba,"IsRegisteredPictureBook_Enemy_Chuchu_Fire_Junior",
0x87d98a31,"IsRegisteredPictureBook_Enemy_Chuchu_Ice_Junior",
0xaa09c72d,"IsRegisteredPictureBook_Enemy_Chuchu_Junior",
0x2637d709,"IsRegisteredPictureBook_Enemy_Dragon_Electric",
0x283f53ab,"IsRegisteredPictureBook_Enemy_Dragon_Fire",
0x40e16f0b,"IsRegisteredPictureBook_Enemy_Dragon_Ice",
0xa235812a,"IsRegisteredPictureBook_Enemy_Ganon",
0x9283dae9,"IsRegisteredPictureBook_Enemy_GanonBeast",
0x136e4dd2,"IsRegisteredPictureBook_Enemy_GanonGrudge",
0x77e91a0c,"IsRegisteredPictureBook_Enemy_GanonGrudge_01",
0xeee04bb6,"IsRegisteredPictureBook_Enemy_GanonGrudge_02",
0xce96e544,"IsRegisteredPictureBook_Enemy_Giant_Bone",
0x96dbd3cd,"IsRegisteredPictureBook_Enemy_Giant_Junior",
0xe1553cc5,"IsRegisteredPictureBook_Enemy_Giant_Middle",
0xd2e2171c,"IsRegisteredPictureBook_Enemy_Giant_Senior",
0x6dcf380e,"IsRegisteredPictureBook_Enemy_Golem_Fire",
0x6fd6bf26,"IsRegisteredPictureBook_Enemy_Golem_Ice",
0x6a3126e3,"IsRegisteredPictureBook_Enemy_Golem_Junior",
0x9adceea1,"IsRegisteredPictureBook_Enemy_Golem_Little",
0xd113f803,"IsRegisteredPictureBook_Enemy_Golem_Little_Fire",
0xd595a897,"IsRegisteredPictureBook_Enemy_Golem_Little_Ice",
0x1dbfc9eb,"IsRegisteredPictureBook_Enemy_Golem_Middle",
0x2e08e232,"IsRegisteredPictureBook_Enemy_Golem_Senior",
0x40b064a2,"IsRegisteredPictureBook_Enemy_Guardian_A",
0x9bb7879a,"IsRegisteredPictureBook_Enemy_Guardian_A_Fixed_Moss",
0xd9b93518,"IsRegisteredPictureBook_Enemy_Guardian_B",
0xaebe058e,"IsRegisteredPictureBook_Enemy_Guardian_C",
0xd13194fa,"IsRegisteredPictureBook_Enemy_Guardian_Mini_Baby",
0x2b45553b,"IsRegisteredPictureBook_Enemy_Guardian_Mini_Junior",
0x5ccbba33,"IsRegisteredPictureBook_Enemy_Guardian_Mini_Middle",
0x6f7c91ea,"IsRegisteredPictureBook_Enemy_Guardian_Mini_Senior",
0x95579048,"IsRegisteredPictureBook_Enemy_Keese",
0xd29dcdbb,"IsRegisteredPictureBook_Enemy_Keese_Electric",
0x15cf62ac,"IsRegisteredPictureBook_Enemy_Keese_Fire",
0x6845bf6e,"IsRegisteredPictureBook_Enemy_Keese_Ice",
0xab615699,"IsRegisteredPictureBook_Enemy_Lizalfos_Bone_Junior",
0x74f503e4,"IsRegisteredPictureBook_Enemy_Lizalfos_Dark",
0x14ce64b2,"IsRegisteredPictureBook_Enemy_Lizalfos_Electric",
0x3757b7d0,"IsRegisteredPictureBook_Enemy_Lizalfos_Fire",
0xf2867ad8,"IsRegisteredPictureBook_Enemy_Lizalfos_Ice",
0x088d0a5b,"IsRegisteredPictureBook_Enemy_Lizalfos_Junior",
0x7f03e553,"IsRegisteredPictureBook_Enemy_Lizalfos_Middle",
0x4cb4ce8a,"IsRegisteredPictureBook_Enemy_Lizalfos_Senior",
0x802e98ee,"IsRegisteredPictureBook_Enemy_Lynel_Dark",
0x95bf5cc6,"IsRegisteredPictureBook_Enemy_Lynel_Junior",
0xe231b3ce,"IsRegisteredPictureBook_Enemy_Lynel_Middle",
0xd1869817,"IsRegisteredPictureBook_Enemy_Lynel_Senior",
0xeeff81f1,"IsRegisteredPictureBook_Enemy_Moriblin_Bone_Junior",
0x6ad03d15,"IsRegisteredPictureBook_Enemy_Moriblin_Dark",
0xc44bd929,"IsRegisteredPictureBook_Enemy_Moriblin_Junior",
0xb3c53621,"IsRegisteredPictureBook_Enemy_Moriblin_Middle",
0x80721df8,"IsRegisteredPictureBook_Enemy_Moriblin_Senior",
0x2a430621,"IsRegisteredPictureBook_Enemy_Octarock",
0x9f52e279,"IsRegisteredPictureBook_Enemy_Octarock_Desert",
0x398d0df7,"IsRegisteredPictureBook_Enemy_Octarock_Forest",
0x0fe9dc4f,"IsRegisteredPictureBook_Enemy_Octarock_Snow",
0xad12d164,"IsRegisteredPictureBook_Enemy_Octarock_Stone",
0x7101ebf6,"IsRegisteredPictureBook_Enemy_Sandworm",
0x91064f18,"IsRegisteredPictureBook_Enemy_SiteBoss_Bow",
0xf8b1f46f,"IsRegisteredPictureBook_Enemy_SiteBoss_Lsword",
0xad1483b2,"IsRegisteredPictureBook_Enemy_SiteBoss_Spear",
0xa86fca1e,"IsRegisteredPictureBook_Enemy_SiteBoss_Sword",
0xadf3c207,"IsRegisteredPictureBook_Enemy_Wizzrobe_Electric",
0x7abab9c7,"IsRegisteredPictureBook_Enemy_Wizzrobe_Electric_Senior",
0x74c2d638,"IsRegisteredPictureBook_Enemy_Wizzrobe_Fire",
0x7bcb4bd4,"IsRegisteredPictureBook_Enemy_Wizzrobe_Fire_Senior",
0x0e0ef8b0,"IsRegisteredPictureBook_Enemy_Wizzrobe_Ice",
0xd5f3ce25,"IsRegisteredPictureBook_Enemy_Wizzrobe_Ice_Senior",
0x9b7a2ad3,"IsRegisteredPictureBook_FireArrow",
0xd6b88595,"IsRegisteredPictureBook_GameRomHorse",
0x03902ab7,"IsRegisteredPictureBook_GameRomHorse00L",
0x8e982742,"IsRegisteredPictureBook_GameRomHorse00S",
0xbbb88be5,"IsRegisteredPictureBook_GameRomHorseBone",
0x6db38c99,"IsRegisteredPictureBook_GameRomHorseNushi",
0x1ddec937,"IsRegisteredPictureBook_GameRomHorseZelda",
0x48de4691,"IsRegisteredPictureBook_IceArrow",
0xdb69ac88,"IsRegisteredPictureBook_Item_FishGet_A",
0x4260fd32,"IsRegisteredPictureBook_Item_FishGet_B",
0x3567cda4,"IsRegisteredPictureBook_Item_FishGet_C",
0xab035807,"IsRegisteredPictureBook_Item_FishGet_D",
0xdc046891,"IsRegisteredPictureBook_Item_FishGet_E",
0x450d392b,"IsRegisteredPictureBook_Item_FishGet_F",
0x320a09bd,"IsRegisteredPictureBook_Item_FishGet_G",
0xa2b5142c,"IsRegisteredPictureBook_Item_FishGet_H",
0xd5b224ba,"IsRegisteredPictureBook_Item_FishGet_I",
0x4cbb7500,"IsRegisteredPictureBook_Item_FishGet_J",
0x3bbc4596,"IsRegisteredPictureBook_Item_FishGet_K",
0xa5d8d035,"IsRegisteredPictureBook_Item_FishGet_L",
0xd2dfe0a3,"IsRegisteredPictureBook_Item_FishGet_M",
0xbf020448,"IsRegisteredPictureBook_Item_FishGet_X",
0x510c6564,"IsRegisteredPictureBook_Item_FishGet_Z",
0x06e2d518,"IsRegisteredPictureBook_Item_Fruit_A",
0x9feb84a2,"IsRegisteredPictureBook_Item_Fruit_B",
0xe8ecb434,"IsRegisteredPictureBook_Item_Fruit_C",
0x76882197,"IsRegisteredPictureBook_Item_Fruit_D",
0x018f1101,"IsRegisteredPictureBook_Item_Fruit_E",
0x988640bb,"IsRegisteredPictureBook_Item_Fruit_F",
0xef81702d,"IsRegisteredPictureBook_Item_Fruit_G",
0x7f3e6dbc,"IsRegisteredPictureBook_Item_Fruit_H",
0x08395d2a,"IsRegisteredPictureBook_Item_Fruit_I",
0x91300c90,"IsRegisteredPictureBook_Item_Fruit_J",
0xf32026a2,"IsRegisteredPictureBook_Item_InsectGet_K",
0xf44de2bb,"IsRegisteredPictureBook_Item_InsectGet_O",
0x99900650,"IsRegisteredPictureBook_Item_InsectGet_Z",
0xd7f12068,"IsRegisteredPictureBook_Item_MushroomGet_D",
0xe8c4e6a7,"IsRegisteredPictureBook_Item_Mushroom_A",
0x71cdb71d,"IsRegisteredPictureBook_Item_Mushroom_B",
0x06ca878b,"IsRegisteredPictureBook_Item_Mushroom_C",
0xefa922be,"IsRegisteredPictureBook_Item_Mushroom_E",
0x76a07304,"IsRegisteredPictureBook_Item_Mushroom_F",
0x91185e03,"IsRegisteredPictureBook_Item_Mushroom_H",
0x7f163f2f,"IsRegisteredPictureBook_Item_Mushroom_J",
0x96759a1a,"IsRegisteredPictureBook_Item_Mushroom_L",
0xe172aa8c,"IsRegisteredPictureBook_Item_Mushroom_M",
0x787bfb36,"IsRegisteredPictureBook_Item_Mushroom_N",
0x0f7ccba0,"IsRegisteredPictureBook_Item_Mushroom_O",
0x3c1be4fa,"IsRegisteredPictureBook_Item_PlantGet_A",
0xa512b540,"IsRegisteredPictureBook_Item_PlantGet_B",
0xd21585d6,"IsRegisteredPictureBook_Item_PlantGet_C",
0x3b7620e3,"IsRegisteredPictureBook_Item_PlantGet_E",
0xa27f7159,"IsRegisteredPictureBook_Item_PlantGet_F",
0xd57841cf,"IsRegisteredPictureBook_Item_PlantGet_G",
0x45c75c5e,"IsRegisteredPictureBook_Item_PlantGet_H",
0x32c06cc8,"IsRegisteredPictureBook_Item_PlantGet_I",
0xabc93d72,"IsRegisteredPictureBook_Item_PlantGet_J",
0x42aa9847,"IsRegisteredPictureBook_Item_PlantGet_L",
0x35ada8d1,"IsRegisteredPictureBook_Item_PlantGet_M",
0xdba3c9fd,"IsRegisteredPictureBook_Item_PlantGet_O",
0x21acf49e,"IsRegisteredPictureBook_Item_PlantGet_Q",
0xcc131a47,"IsRegisteredPictureBook_NormalArrow",
0x59f424aa,"IsRegisteredPictureBook_Obj_Mineral_A_01",
0x4b418b44,"IsRegisteredPictureBook_Obj_Mineral_B_01",
0xf3fdec21,"IsRegisteredPictureBook_Obj_Mineral_C_01",
0x7ca9d2d0,"IsRegisteredPictureBook_RemainsFire_Drone_A_01",
0xc3cde804,"IsRegisteredPictureBook_TBox_Field_Iron",
0x8398eda4,"IsRegisteredPictureBook_Weapon_Bow_001",
0x1a91bc1e,"IsRegisteredPictureBook_Weapon_Bow_002",
0x6d968c88,"IsRegisteredPictureBook_Weapon_Bow_003",
0xf3f2192b,"IsRegisteredPictureBook_Weapon_Bow_004",
0x1dfc7807,"IsRegisteredPictureBook_Weapon_Bow_006",
0x8d436596,"IsRegisteredPictureBook_Weapon_Bow_009",
0x9a83dce5,"IsRegisteredPictureBook_Weapon_Bow_011",
0x748dbdc9,"IsRegisteredPictureBook_Weapon_Bow_013",
0xeae9286a,"IsRegisteredPictureBook_Weapon_Bow_014",
0x9dee18fc,"IsRegisteredPictureBook_Weapon_Bow_015",
0x04e74946,"IsRegisteredPictureBook_Weapon_Bow_016",
0x73e079d0,"IsRegisteredPictureBook_Weapon_Bow_017",
0x5fa0ee0a,"IsRegisteredPictureBook_Weapon_Bow_023",
0x2fca1a85,"IsRegisteredPictureBook_Weapon_Bow_026",
0x58cd2a13,"IsRegisteredPictureBook_Weapon_Bow_027",
0xc8723782,"IsRegisteredPictureBook_Weapon_Bow_028",
0xbf750714,"IsRegisteredPictureBook_Weapon_Bow_029",
0xdfb28ef1,"IsRegisteredPictureBook_Weapon_Bow_030",
0x31bcefdd,"IsRegisteredPictureBook_Weapon_Bow_032",
0x46bbdf4b,"IsRegisteredPictureBook_Weapon_Bow_033",
0xafd87a7e,"IsRegisteredPictureBook_Weapon_Bow_035",
0x36d12bc4,"IsRegisteredPictureBook_Weapon_Bow_036",
0xd16906c3,"IsRegisteredPictureBook_Weapon_Bow_038",
0x90f31836,"IsRegisteredPictureBook_Weapon_Bow_040",
0xccd97b63,"IsRegisteredPictureBook_Weapon_Bow_071",
0x36630f7b,"IsRegisteredPictureBook_Weapon_Lsword_001",
0xaf6a5ec1,"IsRegisteredPictureBook_Weapon_Lsword_002",
0xd86d6e57,"IsRegisteredPictureBook_Weapon_Lsword_003",
0x4609fbf4,"IsRegisteredPictureBook_Weapon_Lsword_004",
0x310ecb62,"IsRegisteredPictureBook_Weapon_Lsword_005",
0xa8079ad8,"IsRegisteredPictureBook_Weapon_Lsword_006",
0x587f0eac,"IsRegisteredPictureBook_Weapon_Lsword_010",
0x2f783e3a,"IsRegisteredPictureBook_Weapon_Lsword_011",
0xb6716f80,"IsRegisteredPictureBook_Weapon_Lsword_012",
0xc1765f16,"IsRegisteredPictureBook_Weapon_Lsword_013",
0x5f12cab5,"IsRegisteredPictureBook_Weapon_Lsword_014",
0x2815fa23,"IsRegisteredPictureBook_Weapon_Lsword_015",
0xb11cab99,"IsRegisteredPictureBook_Weapon_Lsword_016",
0xc61b9b0f,"IsRegisteredPictureBook_Weapon_Lsword_017",
0x56a4869e,"IsRegisteredPictureBook_Weapon_Lsword_018",
0x21a3b608,"IsRegisteredPictureBook_Weapon_Lsword_019",
0x73525d6f,"IsRegisteredPictureBook_Weapon_Lsword_020",
0xea5b0cd5,"IsRegisteredPictureBook_Weapon_Lsword_023",
0x743f9976,"IsRegisteredPictureBook_Weapon_Lsword_024",
0xed36c8cc,"IsRegisteredPictureBook_Weapon_Lsword_027",
0x0a8ee5cb,"IsRegisteredPictureBook_Weapon_Lsword_029",
0x6a496c2e,"IsRegisteredPictureBook_Weapon_Lsword_030",
0x1d4e5cb8,"IsRegisteredPictureBook_Weapon_Lsword_031",
0x84470d02,"IsRegisteredPictureBook_Weapon_Lsword_032",
0xf3403d94,"IsRegisteredPictureBook_Weapon_Lsword_033",
0x6d24a837,"IsRegisteredPictureBook_Weapon_Lsword_034",
0x1a2398a1,"IsRegisteredPictureBook_Weapon_Lsword_035",
0x832ac91b,"IsRegisteredPictureBook_Weapon_Lsword_036",
0xf42df98d,"IsRegisteredPictureBook_Weapon_Lsword_037",
0x6492e41c,"IsRegisteredPictureBook_Weapon_Lsword_038",
0x520fca7f,"IsRegisteredPictureBook_Weapon_Lsword_041",
0x55620e66,"IsRegisteredPictureBook_Weapon_Lsword_045",
0xbb6c6f4a,"IsRegisteredPictureBook_Weapon_Lsword_047",
0x4b14fb3e,"IsRegisteredPictureBook_Weapon_Lsword_051",
0x3b7e0fb1,"IsRegisteredPictureBook_Weapon_Lsword_054",
0x4c793f27,"IsRegisteredPictureBook_Weapon_Lsword_055",
0xd5706e9d,"IsRegisteredPictureBook_Weapon_Lsword_056",
0x09486d33,"IsRegisteredPictureBook_Weapon_Lsword_074",
0x3ace160e,"IsRegisteredPictureBook_Weapon_Shield_001",
0xa3c747b4,"IsRegisteredPictureBook_Weapon_Shield_002",
0xd4c07722,"IsRegisteredPictureBook_Weapon_Shield_003",
0x4aa4e281,"IsRegisteredPictureBook_Weapon_Shield_004",
0x3da3d217,"IsRegisteredPictureBook_Weapon_Shield_005",
0xa4aa83ad,"IsRegisteredPictureBook_Weapon_Shield_006",
0xd3adb33b,"IsRegisteredPictureBook_Weapon_Shield_007",
0x4312aeaa,"IsRegisteredPictureBook_Weapon_Shield_008",
0x34159e3c,"IsRegisteredPictureBook_Weapon_Shield_009",
0xcddb4663,"IsRegisteredPictureBook_Weapon_Shield_013",
0x53bfd3c0,"IsRegisteredPictureBook_Weapon_Shield_014",
0x24b8e356,"IsRegisteredPictureBook_Weapon_Shield_015",
0xbdb1b2ec,"IsRegisteredPictureBook_Weapon_Shield_016",
0xcab6827a,"IsRegisteredPictureBook_Weapon_Shield_017",
0x5a099feb,"IsRegisteredPictureBook_Weapon_Shield_018",
0x08f8748c,"IsRegisteredPictureBook_Weapon_Shield_021",
0x91f12536,"IsRegisteredPictureBook_Weapon_Shield_022",
0xe6f615a0,"IsRegisteredPictureBook_Weapon_Shield_023",
0x0f95b095,"IsRegisteredPictureBook_Weapon_Shield_025",
0x969ce12f,"IsRegisteredPictureBook_Weapon_Shield_026",
0x66e4755b,"IsRegisteredPictureBook_Weapon_Shield_030",
0x11e345cd,"IsRegisteredPictureBook_Weapon_Shield_031",
0x88ea1477,"IsRegisteredPictureBook_Weapon_Shield_032",
0xffed24e1,"IsRegisteredPictureBook_Weapon_Shield_033",
0x6189b142,"IsRegisteredPictureBook_Weapon_Shield_034",
0x168e81d4,"IsRegisteredPictureBook_Weapon_Shield_035",
0x8f87d06e,"IsRegisteredPictureBook_Weapon_Shield_036",
0xf880e0f8,"IsRegisteredPictureBook_Weapon_Shield_037",
0x683ffd69,"IsRegisteredPictureBook_Weapon_Shield_038",
0x29a5e39c,"IsRegisteredPictureBook_Weapon_Shield_040",
0x5ea2d30a,"IsRegisteredPictureBook_Weapon_Shield_041",
0xc7ab82b0,"IsRegisteredPictureBook_Weapon_Shield_042",
0x4b345842,"IsRegisteredPictureBook_Weapon_Spear_001",
0xd23d09f8,"IsRegisteredPictureBook_Weapon_Spear_002",
0xa53a396e,"IsRegisteredPictureBook_Weapon_Spear_003",
0x3b5eaccd,"IsRegisteredPictureBook_Weapon_Spear_004",
0x4c599c5b,"IsRegisteredPictureBook_Weapon_Spear_005",
0xd550cde1,"IsRegisteredPictureBook_Weapon_Spear_006",
0xa257fd77,"IsRegisteredPictureBook_Weapon_Spear_007",
0x32e8e0e6,"IsRegisteredPictureBook_Weapon_Spear_008",
0x45efd070,"IsRegisteredPictureBook_Weapon_Spear_009",
0x25285995,"IsRegisteredPictureBook_Weapon_Spear_010",
0x522f6903,"IsRegisteredPictureBook_Weapon_Spear_011",
0xcb2638b9,"IsRegisteredPictureBook_Weapon_Spear_012",
0xbc21082f,"IsRegisteredPictureBook_Weapon_Spear_013",
0x22459d8c,"IsRegisteredPictureBook_Weapon_Spear_014",
0x5542ad1a,"IsRegisteredPictureBook_Weapon_Spear_015",
0xcc4bfca0,"IsRegisteredPictureBook_Weapon_Spear_016",
0xbb4ccc36,"IsRegisteredPictureBook_Weapon_Spear_017",
0x2bf3d1a7,"IsRegisteredPictureBook_Weapon_Spear_018",
0x79023ac0,"IsRegisteredPictureBook_Weapon_Spear_021",
0xe00b6b7a,"IsRegisteredPictureBook_Weapon_Spear_022",
0x970c5bec,"IsRegisteredPictureBook_Weapon_Spear_023",
0x0968ce4f,"IsRegisteredPictureBook_Weapon_Spear_024",
0x7e6ffed9,"IsRegisteredPictureBook_Weapon_Spear_025",
0x90619ff5,"IsRegisteredPictureBook_Weapon_Spear_027",
0x00de8264,"IsRegisteredPictureBook_Weapon_Spear_028",
0x77d9b2f2,"IsRegisteredPictureBook_Weapon_Spear_029",
0x171e3b17,"IsRegisteredPictureBook_Weapon_Spear_030",
0x60190b81,"IsRegisteredPictureBook_Weapon_Spear_031",
0xf9105a3b,"IsRegisteredPictureBook_Weapon_Spear_032",
0x8e176aad,"IsRegisteredPictureBook_Weapon_Spear_033",
0x1073ff0e,"IsRegisteredPictureBook_Weapon_Spear_034",
0x6774cf98,"IsRegisteredPictureBook_Weapon_Spear_035",
0xfe7d9e22,"IsRegisteredPictureBook_Weapon_Spear_036",
0x897aaeb4,"IsRegisteredPictureBook_Weapon_Spear_037",
0x19c5b325,"IsRegisteredPictureBook_Weapon_Spear_038",
0xc63b3873,"IsRegisteredPictureBook_Weapon_Spear_047",
0x21831574,"IsRegisteredPictureBook_Weapon_Spear_049",
0x41449c91,"IsRegisteredPictureBook_Weapon_Spear_050",
0x8f631c02,"IsRegisteredPictureBook_Weapon_Sword_001",
0x166a4db8,"IsRegisteredPictureBook_Weapon_Sword_002",
0x616d7d2e,"IsRegisteredPictureBook_Weapon_Sword_003",
0xff09e88d,"IsRegisteredPictureBook_Weapon_Sword_004",
0x880ed81b,"IsRegisteredPictureBook_Weapon_Sword_005",
0x110789a1,"IsRegisteredPictureBook_Weapon_Sword_006",
0x6600b937,"IsRegisteredPictureBook_Weapon_Sword_007",
0xf6bfa4a6,"IsRegisteredPictureBook_Weapon_Sword_008",
0x81b89430,"IsRegisteredPictureBook_Weapon_Sword_009",
0x78764c6f,"IsRegisteredPictureBook_Weapon_Sword_013",
0xe612d9cc,"IsRegisteredPictureBook_Weapon_Sword_014",
0x9115e95a,"IsRegisteredPictureBook_Weapon_Sword_015",
0x081cb8e0,"IsRegisteredPictureBook_Weapon_Sword_016",
0x7f1b8876,"IsRegisteredPictureBook_Weapon_Sword_017",
0xefa495e7,"IsRegisteredPictureBook_Weapon_Sword_018",
0x98a3a571,"IsRegisteredPictureBook_Weapon_Sword_019",
0xca524e16,"IsRegisteredPictureBook_Weapon_Sword_020",
0xbd557e80,"IsRegisteredPictureBook_Weapon_Sword_021",
0x245c2f3a,"IsRegisteredPictureBook_Weapon_Sword_022",
0x535b1fac,"IsRegisteredPictureBook_Weapon_Sword_023",
0xcd3f8a0f,"IsRegisteredPictureBook_Weapon_Sword_024",
0xba38ba99,"IsRegisteredPictureBook_Weapon_Sword_025",
0x5436dbb5,"IsRegisteredPictureBook_Weapon_Sword_027",
0xb38ef6b2,"IsRegisteredPictureBook_Weapon_Sword_029",
0xd3497f57,"IsRegisteredPictureBook_Weapon_Sword_030",
0xa44e4fc1,"IsRegisteredPictureBook_Weapon_Sword_031",
0x4a402eed,"IsRegisteredPictureBook_Weapon_Sword_033",
0xd424bb4e,"IsRegisteredPictureBook_Weapon_Sword_034",
0xa3238bd8,"IsRegisteredPictureBook_Weapon_Sword_035",
0x9c08e990,"IsRegisteredPictureBook_Weapon_Sword_040",
0xeb0fd906,"IsRegisteredPictureBook_Weapon_Sword_041",
0x0501b82a,"IsRegisteredPictureBook_Weapon_Sword_043",
0x9b652d89,"IsRegisteredPictureBook_Weapon_Sword_044",
0x026c7c33,"IsRegisteredPictureBook_Weapon_Sword_047",
0x92d361a2,"IsRegisteredPictureBook_Weapon_Sword_048",
0xe5d45134,"IsRegisteredPictureBook_Weapon_Sword_049",
0x8513d8d1,"IsRegisteredPictureBook_Weapon_Sword_050",
0xf214e847,"IsRegisteredPictureBook_Weapon_Sword_051",
0x6b1db9fd,"IsRegisteredPictureBook_Weapon_Sword_052",
0x1c1a896b,"IsRegisteredPictureBook_Weapon_Sword_053",
0xae3e8b12,"IsRegisteredPictureBook_Weapon_Sword_060",
0xd939bb84,"IsRegisteredPictureBook_Weapon_Sword_061",
0x4030ea3e,"IsRegisteredPictureBook_Weapon_Sword_062",
0xb725ba53,"IsRegisteredPictureBook_Weapon_Sword_070",
0x2e2cebe9,"IsRegisteredPictureBook_Weapon_Sword_073",
0x574afd27,"IsRepair_MemoryPhoto_013",
0xaaf1add0,"IsStart_Obj_SheikSensor",
0x61dd4c84,"IsTalkAboutRupeeWithGreatFairy1",
0xf8d41d3e,"IsTalkAboutRupeeWithGreatFairy2",
0x8fd32da8,"IsTalkAboutRupeeWithGreatFairy3",
0x11b7b80b,"IsTalkAboutRupeeWithGreatFairy4",
0xa84e69fd,"IsTempAddBowPouch",
0x5aeab776,"IslandMini_Battle_Activated",
0xf02bb22d,"IslandMini_Battle_Finished",
0xa858934a,"IslandMini_Battle_Ready",
0xb8dded87,"IslandMini_Battle_Success",
0xcb99384b,"IslandMini_Battle_playing",
0xe0e74eaf,"Johnny_First",
0x8e4cd279,"Johnny_UnburnedMedicineBuy",
0xbd4b4546,"Johnny_UnburnedMedicineExplain",
0x09195b1b,"Kaka_Kokko_01",
0x90100aa1,"Kaka_Kokko_02",
0xe7173a37,"Kaka_Kokko_03",
0x7973af94,"Kaka_Kokko_04",
0x0e749f02,"Kaka_Kokko_05",
0x977dceb8,"Kaka_Kokko_06",
0xe07afe2e,"Kaka_Kokko_07",
0x70c5e3bf,"Kaka_Kokko_08",
0x07c2d329,"Kaka_Kokko_09",
0x67055acc,"Kaka_Kokko_10",
0xfd86f659,"Kakariko001_DirectComp_Find_4Relic",
0x4f3f28b8,"Kakariko001_GaveChampionShirt",
0xc6e2ee8c,"Kakariko001_NoCont",
0x1ddc40cf,"Kakariko_Ch_004_Activated",
0xa4cbf729,"Kakariko_Ch_004_Finish",
0x2e2c0144,"Kakariko_Ch_004_First",
0x94c86abc,"Kakariko_Ch_004_Ready",
0x73ab9a83,"Kakariko_Ch_005_1stReward",
0xdc529f0f,"Kakariko_Ch_005_Activated",
0x6861f7b7,"Kakariko_Ch_005_Finish",
0x32bf6108,"Kakariko_Ch_005_Ready",
0xb07da18a,"Kakariko_Ch_005_full",
0x45b0f90e,"Kakariko_Ch_006_Activated",
0xf3f51004,"Kakariko_Ch_006_Arrow",
0xe6eef054,"Kakariko_Ch_006_Finish",
0x40d8a2cd,"Kakariko_Ch_006_Light",
0x03577b95,"Kakariko_Ch_006_Ready",
0x78723f69,"Kakariko_Ch_006_first",
0x296e5bb0,"Kakariko_Ch_006_hobi",
0x61f3db8a,"Kakariko_Ch_Cooking2_Activated",
0xf04b76dc,"Kakariko_Ch_Cooking2_Finish",
0xca1eb64a,"Kakariko_Ch_Cooking2_Ready",
0x9e53ef97,"Kakariko_Ch_Cooking2_Reward",
0xa07d044a,"Kakariko_Ch_Cooking3_Activated",
0x3ce17642,"Kakariko_Ch_Cooking3_Finish",
0x6c69bdfe,"Kakariko_Ch_Cooking3_Ready",
0x52f9ef09,"Kakariko_Ch_Cooking3_Reward",
0x894611c9,"Kakariko_Ch_Cooking4_Activated",
0x36247f5b,"Kakariko_Ch_Cooking4_Finish",
0xa9ce8370,"Kakariko_Ch_Cooking4_Ready",
0x583ce610,"Kakariko_Ch_Cooking4_Reward",
0xcf4d37a2,"Kakariko_Cha_001_Activated",
0xe6085676,"Kakariko_Cha_001_Finish",
0xe5f15995,"Kakariko_Cha_001_Ready",
0x8810cf3d,"Kakariko_Cha_001_Reward",
0x56af51a3,"Kakariko_Cha_002_Activated",
0x5bd7adb9,"Kakariko_Cha_002_Battle",
0x68875195,"Kakariko_Cha_002_Finish",
0xd4194308,"Kakariko_Cha_002_Ready",
0xaf3c07f4,"Kakariko_Cha_002_first",
0x4ea4a1a0,"Kakariko_Cha_003_10kokko",
0x97218e63,"Kakariko_Cha_003_Activated",
0xa42d510b,"Kakariko_Cha_003_Finish",
0x726e48bc,"Kakariko_Cha_003_Ready",
0xfad0a099,"Kakariko_Osonaemono",
0x74f96fa2,"King_Vagrant_FirstTalk",
0x71899693,"KirutonDokuro_Delete",
0x0c4a001e,"KirutonDokuro_Picture",
0x0377eab8,"KirutonDokuro_PictureOK",
0x67889014,"KnightDoll_Activated",
0x8aa44d51,"KnightDoll_Dungeon",
0xae541ba5,"KnightDoll_Finish",
0x0b8ac132,"KnightDoll_Ready",
0x9a7936a6,"KnightDoll_SetA",
0x0370671c,"KnightDoll_SetB",
0x7477578a,"KnightDoll_SetC",
0xea13c229,"KnightDoll_SetD",
0x9d14f2bf,"KnightDoll_SetE",
0x041da305,"KnightDoll_SetF",
0x731a9393,"KnightDoll_SetG",
0x70af85ce,"KnightDoll_first",
0xfe2f84ec,"KokiriNPC002_OtherSageQuest",
0xab86ef8d,"KokiriShop_000",
0x1dc197e7,"KorinBox_Appear",
0x730d0018,"KoroguArmor_1_3",
0xf468a1e2,"KorokMini_KorokShiren_Activated",
0x07645bcf,"KorokMini_KorokShiren_Finish",
0x8764f8de,"KorokMini_KorokShiren_Ready",
0x273b20ee,"KorokMini_KorokShiren_RewardNotYet",
0x36cd3af7,"KorokMini_KorokShiren_Step010",
0x5aa94f1b,"KorokMini_RiddleShiren_Activated",
0x102fcfb7,"KorokMini_RiddleShiren_Finish",
0x7b4ec340,"KorokMini_RiddleShiren_Ready",
0xb1b0445b,"KorokMini_RiddleShiren_RewardNotYet",
0x6804886d,"KorokMini_RiddleShiren_Step010",
0x4329dbae,"KorokMini_RiddleShiren_Step020",
0x5a32eaef,"KorokMini_RiddleShiren_Step030",
0x15737c28,"KorokMini_RiddleShiren_Step040",
0x15bff1b6,"KorokMini_RodShiren_Activated",
0x1001da5d,"KorokMini_RodShiren_Finish",
0x555b2940,"KorokMini_RodShiren_Ready",
0xdd1e7738,"KorokMini_UMAShiren_Activated",
0x847cbb92,"KorokMini_UMAShiren_Finish",
0x4c0162c9,"KorokMini_UMAShiren_Ready",
0xaef8bab3,"LanayruMini_ZoraRelief_Activated",
0xb6333d05,"LanayruMini_ZoraRelief_Finish",
0xdbeb16e5,"LanayruMini_ZoraRelief_Ready",
0x462b092f,"LastBossGanonBeastGenerateFlag",
0xf32b647f,"LastBossIncompleteGanonGenerateFlag",
0xd5a3f586,"LetterErrand_Activated",
0xdcf7bdf1,"LetterErrand_Finished",
0xcc99c304,"LetterErrand_Ready",
0x2547095d,"Lizarfos_Complete",
0x91c0aa01,"MacuseIseki_Activated",
0xd5924a97,"MacuseIseki_Finish",
0x7cf05df8,"MacuseIseki_Ready",
0x0c3988ae,"MagneticFld_Activated",
0x811de8dc,"MagneticFld_Finish",
0x2d053f0e,"MagneticFld_Ready",
0x73e651d0,"MagneticFld_assassin",
0xd6d0dfc8,"MagneticFld_assassin_GJ",
0xaaa73072,"MagneticFld_assassin_first",
0x31202b63,"MainFieldDungeon_Barrel_3232809450",
0xf65ccab7,"MainFieldDungeon_Barrel_75890609",
0xe2646d35,"MainFieldDungeon_Barrel_91216696",
0x1c4a8bfc,"MainFieldDungeon_DgnObj_AncientReactor_B_01_3183677059",
0x3328e108,"MainFieldDungeon_DgnObj_AncientReactor_B_01_3663222571",
0x2dccf457,"MainFieldDungeon_DgnObj_AncientReactor_B_01_4028066808",
0xe3100f86,"MainFieldDungeon_DgnObj_AncientReactor_B_01_897080685",
0x97c6988d,"MainFieldDungeon_DgnObj_AncientReactor_B_01_on_2216941002",
0x0b23bea6,"MainFieldDungeon_DgnObj_AncientReactor_B_01_on_2818121394",
0x86ed4fbc,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_1710346876",
0x05dc4a3e,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_1863979501",
0xb86d44c0,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_1871746289",
0x499f908d,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_2270797422",
0xf027f9eb,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_2500272373",
0x2b904d16,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_2572702401",
0x7d7476b8,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_2608135785",
0x1951b40e,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_2696982252",
0x09b1bd5f,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_297229595",
0x1064dd9e,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_3001593530",
0x48ca6686,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_3489382707",
0x5b73fb3f,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_3747003481",
0x19cbbca0,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_3926727362",
0xe7850d13,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_3941245679",
0x185bf038,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_4032737016",
0x38655b19,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_4117299134",
0x97c47b4d,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_418382979",
0x5cff608c,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_637989014",
0x8541eb7d,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_827907445",
0x3e0705af,"MainFieldDungeon_DgnObj_RemainsTerminalBody_A_838829532",
0xdeae48be,"MainFieldDungeon_DgnObj_WallCrack_C_01_2773947421",
0x9b04e259,"MainFieldDungeon_DgnObj_WallCrack_C_01_3598574216",
0x6d4a2756,"MainFieldDungeon_DgnObj_WallCrack_Wind_A_02_4253555207",
0xd2882c9d,"MainFieldDungeon_Enemy_Guardian_Mini_Baby_Dark_1532176960",
0xb91dc3d3,"MainFieldDungeon_Enemy_Guardian_Mini_Baby_Dark_2110680702",
0x4ee99fc8,"MainFieldDungeon_Enemy_Guardian_Mini_Junior_Dark_1804304434",
0xce688e88,"MainFieldDungeon_Enemy_Guardian_Mini_Junior_Dark_183090620",
0x56a3d717,"MainFieldDungeon_Enemy_Guardian_Mini_Junior_Dark_2612203402",
0x9dda3315,"MainFieldDungeon_Enemy_Guardian_Mini_Junior_Dark_3065994546",
0x4173739c,"MainFieldDungeon_Enemy_Guardian_Mini_Junior_Dark_4169926085",
0xc27c0da3,"MainFieldDungeon_Enemy_Guardian_Mini_Junior_Dark_638100967",
0xc9719d19,"MainFieldDungeon_Enemy_Guardian_Mini_Junior_Dark_791680943",
0x864e0cda,"MainFieldDungeon_Enemy_Guardian_Mini_Junior_Dark_810427046",
0x31ccf179,"MainFieldDungeon_Enemy_Guardian_Mini_Junior_Dark_996206242",
0x5b83276c,"MainFieldDungeon_Enemy_SiteBoss_Bow_2684730666",
0xbbdc2694,"MainFieldDungeon_Enemy_SiteBoss_Lsword_674596187",
0xadddcda4,"MainFieldDungeon_Enemy_SiteBoss_Spear_781518966",
0x45c53274,"MainFieldDungeon_Enemy_SiteBoss_Sword_3700074540",
0x9cf0166c,"MainFieldDungeon_FldObj_DownloadTerminal_A_01_2498109624",
0xf631e6ab,"MainFieldDungeon_FldObj_DownloadTerminal_A_01_3698093779",
0xe14b2e39,"MainFieldDungeon_FldObj_DownloadTerminal_A_01_3732591143",
0x32885bd9,"MainFieldDungeon_FldObj_DownloadTerminal_A_01_951123958",
0x7107eb73,"MainFieldDungeon_FldObj_Terminal_A_01_114051416",
0xdfd13e6a,"MainFieldDungeon_FldObj_Terminal_A_01_1283068278",
0x8b945625,"MainFieldDungeon_FldObj_Terminal_A_01_1285400923",
0x15da6f8c,"MainFieldDungeon_FldObj_Terminal_A_01_1474232055",
0x9f00fbe4,"MainFieldDungeon_FldObj_Terminal_A_01_1770834567",
0x4beb4635,"MainFieldDungeon_FldObj_Terminal_A_01_1890065297",
0x3741659e,"MainFieldDungeon_FldObj_Terminal_A_01_2019624366",
0xf23aaae0,"MainFieldDungeon_FldObj_Terminal_A_01_2028699826",
0xb0cbc802,"MainFieldDungeon_FldObj_Terminal_A_01_2032252909",
0x38cf0d9d,"MainFieldDungeon_FldObj_Terminal_A_01_2189178038",
0x8e1c9b59,"MainFieldDungeon_FldObj_Terminal_A_01_2384676482",
0x7133d21a,"MainFieldDungeon_FldObj_Terminal_A_01_2418182701",
0x1c18a5e1,"MainFieldDungeon_FldObj_Terminal_A_01_2479570667",
0xf32013b7,"MainFieldDungeon_FldObj_Terminal_A_01_2536494785",
0xf12238cd,"MainFieldDungeon_FldObj_Terminal_A_01_260772032",
0x41ac752f,"MainFieldDungeon_FldObj_Terminal_A_01_2841367266",
0xec784695,"MainFieldDungeon_FldObj_Terminal_A_01_2874875320",
0x522c9804,"MainFieldDungeon_FldObj_Terminal_A_01_3082533551",
0xf29845a5,"MainFieldDungeon_FldObj_Terminal_A_01_3439469688",
0xf89e7227,"MainFieldDungeon_FldObj_Terminal_A_01_346006798",
0xba2edb7f,"MainFieldDungeon_FldObj_Terminal_A_01_3878009531",
0x814048a3,"MainFieldDungeon_FldObj_Terminal_A_01_646222239",
0x85470cec,"MainFieldDungeon_FldObj_Terminal_A_01_830334165",
0x1cc64a4e,"MainFieldDungeon_FldObj_Terminal_A_01_969080608",
0x84c09fa5,"MainFieldDungeon_Obj_HeartUtuwa_A_01_2224435968",
0x5c811ba5,"MainFieldDungeon_Obj_HeartUtuwa_A_01_2805247992",
0x2f726f50,"MainFieldDungeon_Obj_HeartUtuwa_A_01_3046120871",
0x09c30c55,"MainFieldDungeon_Obj_HeartUtuwa_A_01_3159171529",
0x22ba4365,"MainFieldDungeon_TBox_Dungeon_Iron_1107920290",
0x263d71be,"MainFieldDungeon_TBox_Dungeon_Iron_1439251665",
0x4967bbd1,"MainFieldDungeon_TBox_Dungeon_Iron_2574653140",
0xb28c67cc,"MainFieldDungeon_TBox_Dungeon_Iron_703988891",
0xbdb85789,"MainFieldDungeon_TBox_Dungeon_Iron_Hanged_3029802873",
0x4a73ee01,"MainFieldDungeon_TBox_Dungeon_Stone_1261793201",
0x462b5a78,"MainFieldDungeon_TBox_Dungeon_Stone_1407963741",
0xb819b558,"MainFieldDungeon_TBox_Dungeon_Stone_1515121245",
0xa9962652,"MainFieldDungeon_TBox_Dungeon_Stone_1716309953",
0xdd6a4b1a,"MainFieldDungeon_TBox_Dungeon_Stone_2024040400",
0xa17873fe,"MainFieldDungeon_TBox_Dungeon_Stone_2502766660",
0x73f70167,"MainFieldDungeon_TBox_Dungeon_Stone_3527163403",
0x92c3acac,"MainFieldDungeon_TBox_Dungeon_Stone_3535181875",
0xe86ce3de,"MainFieldDungeon_TBox_Dungeon_Stone_3777321501",
0x7ad4024c,"MainFieldDungeon_TBox_Dungeon_Stone_3875439170",
0xf542d646,"MainFieldDungeon_TBox_Dungeon_Stone_388540865",
0xa515e29b,"MainFieldDungeon_TBox_Dungeon_Stone_3937043794",
0x9a8a2d1b,"MainFieldDungeon_TBox_Dungeon_Stone_3992374850",
0x931085c0,"MainFieldDungeon_TBox_Dungeon_Stone_404901059",
0x46a6e43b,"MainFieldDungeon_TBox_Dungeon_Stone_4082461528",
0xaf9c0e48,"MainFieldDungeon_TBox_Dungeon_Stone_4271575548",
0xe0c02111,"MainFieldDungeon_TBox_Dungeon_Stone_456917981",
0x5b388ea5,"MainFieldDungeon_TBox_Dungeon_Stone_543487524",
0xee13e398,"MainFieldDungeon_TBox_Dungeon_Stone_572020553",
0x07928e86,"MainFieldDungeon_TBox_Dungeon_Stone_583928903",
0xac9b68c4,"MainFieldDungeon_TBox_Dungeon_Stone_654620011",
0x969508b1,"MainFieldDungeon_TBox_Dungeon_Stone_80984099",
0x45c14bf4,"MainFieldDungeon_TBox_Dungeon_Stone_890156340",
0x581e746e,"MainFieldDungeon_TBox_Dungeon_Wood_1346994567",
0x19ef63e6,"MainFieldDungeon_TBox_Dungeon_Wood_142838264",
0xb2508c04,"MainFieldDungeon_TBox_Dungeon_Wood_1739313242",
0x1a8304e8,"MainFieldDungeon_TBox_Dungeon_Wood_3877021021",
0x67ffee6a,"MainFieldDungeon_TBox_Dungeon_Wood_421030062",
0xf6e0a1e9,"MainFieldDungeon_TBox_Dungeon_Wood_551066994",
0x0b762cee,"MainFieldDungeon_TBox_Dungeon_Wood_935051534",
0x25e9d18f,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_1366728947",
0x5dc5c600,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_1408146088",
0x66b16a45,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_1411065004",
0x83dd7d08,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_1521613396",
0xcef66541,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_1896743214",
0xfec7b032,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_1994238103",
0xfdc3c569,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_2097914985",
0x066bde4b,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_2337663528",
0xf57d0207,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_243539911",
0xf9728d14,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_2817596871",
0x21b55cba,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_282526482",
0xe1ae028c,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_2863908770",
0xf8eb39dc,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3034529496",
0x1effb1fe,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3151147972",
0x8b5882eb,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3353799693",
0x32958bfb,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3356904144",
0x022249b2,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3654132759",
0x78a45314,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3827550871",
0xb07251c0,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3849979110",
0x200da9d5,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3984230029",
0x7e560512,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3986567763",
0xb71a4b58,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3992981477",
0x11fb1cfa,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_4030089798",
0xdb44eebd,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_4043571028",
0x91144ef8,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_4235580151",
0x341d472d,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_477411398",
0xfc55aea3,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_493726876",
0x2390c75b,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_548815768",
0xda2977cc,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_556316970",
0x5ed848d9,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_563611273",
0x01f772fb,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_589237343",
0xb119f2ba,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_891775170",
0x95fa3896,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_897543486",
0xd682dc96,"MainFieldDungeon_TwnObj_GanonGrudgeSolid_Eyeball_A_01_962283061",
0x6e7c304a,"MainFieldDungeon_WallCrack_Wind_2324638353",
0x81aeaf67,"MainField_Animal_Bear_A_1715921240",
0x142a6c37,"MainField_Animal_Bear_A_1737190204",
0xa3111843,"MainField_Animal_Bear_A_1860392359",
0x44a81547,"MainField_Animal_Bear_A_1970615967",
0xdcfa0d38,"MainField_Animal_Bear_B_1726629232",
0x70226ded,"MainField_Animal_Bear_B_1843168105",
0x0d7654a9,"MainField_Animal_Bear_B_2210201350",
0x0ccadfc1,"MainField_Animal_Bear_B_3236299109",
0x3b7dbd85,"MainField_Animal_Bear_B_3545633995",
0x88f13ea6,"MainField_Animal_Bear_B_3762742301",
0x7735c7fc,"MainField_Animal_Bear_B_4234308755",
0xa92497fc,"MainField_Animal_Bear_B_516446812",
0x95eafd3d,"MainField_Animal_Bear_B_834378035",
0x7203b721,"MainField_Animal_Bear_B_985371369",
0x3b49cc7f,"MainField_Animal_Boar_A_1091143776",
0x60102094,"MainField_Animal_Boar_A_1211555117",
0x0f7e893c,"MainField_Animal_Boar_A_1259031255",
0xfa8a43f1,"MainField_Animal_Boar_A_126645873",
0x31cd545f,"MainField_Animal_Boar_A_1612874288",
0x0f96e8ed,"MainField_Animal_Boar_A_1703901080",
0xf5be670f,"MainField_Animal_Boar_A_1777664942",
0x29fbcaea,"MainField_Animal_Boar_A_2044425248",
0x93d9e947,"MainField_Animal_Boar_A_2095123956",
0x141c645f,"MainField_Animal_Boar_A_2154248365",
0x47b065b5,"MainField_Animal_Boar_A_2277548161",
0x84066c87,"MainField_Animal_Boar_A_2323530228",
0xfef94e7f,"MainField_Animal_Boar_A_2560077666",
0x07b999e0,"MainField_Animal_Boar_A_2646269784",
0xff05b7e8,"MainField_Animal_Boar_A_2842755364",
0x389fd820,"MainField_Animal_Boar_A_2885165803",
0xfd2bdbdd,"MainField_Animal_Boar_A_2968804638",
0x080a0df2,"MainField_Animal_Boar_A_3091989673",
0xbf74dba5,"MainField_Animal_Boar_A_3206660272",
0x39108f69,"MainField_Animal_Boar_A_3220338235",
0xf2055d73,"MainField_Animal_Boar_A_3400499397",
0xb50d3d29,"MainField_Animal_Boar_A_3477812287",
0xd5c333a9,"MainField_Animal_Boar_A_3580840839",
0x582354a2,"MainField_Animal_Boar_A_3763127002",
0x7e3cc120,"MainField_Animal_Boar_A_381015492",
0xa0ae07a3,"MainField_Animal_Boar_A_3938316238",
0xf8e5d977,"MainField_Animal_Boar_A_4032722332",
0x4f441ea1,"MainField_Animal_Boar_A_4147254912",
0x20c6b729,"MainField_Animal_Boar_A_4232584849",
0x60caeb53,"MainField_Animal_Boar_A_4234362958",
0x7e4e98d8,"MainField_Animal_Boar_A_689366419",
0x776d158f,"MainField_Animal_Boar_A_689792347",
0xee8c863e,"MainField_Animal_Boar_A_741121095",
0x3b589682,"MainField_Animal_Boar_A_76631430",
0x739a6fbd,"MainField_Animal_Boar_A_879964005",
0x4feab09c,"MainField_Animal_Boar_A_899462984",
0x4073f29d,"MainField_Animal_Boar_A_912393961",
0x05b6b63c,"MainField_Animal_Boar_A_921067468",
0x3df16831,"MainField_Animal_Boar_B_1220138319",
0x12220b06,"MainField_Animal_Boar_B_1476016929",
0x60f77a43,"MainField_Animal_Boar_B_1612812193",
0x9e466387,"MainField_Animal_Boar_B_1971521909",
0x05e189b2,"MainField_Animal_Boar_B_2938425364",
0x9994d6a8,"MainField_Animal_Boar_B_3032556378",
0x3a0655ae,"MainField_Animal_Boar_B_3188353506",
0xf5ecae6b,"MainField_Animal_Boar_B_3861425117",
0x08b486c7,"MainField_Animal_Boar_B_486374736",
0xa7c78b76,"MainField_Animal_Boar_B_612054083",
0xa1cd4310,"MainField_Animal_Boar_B_856488880",
0xb9d5c1ec,"MainField_Animal_Bull_A_1059023278",
0x05ac17b4,"MainField_Animal_Bull_A_1158709725",
0xf1325486,"MainField_Animal_Bull_A_1228739332",
0xd735e519,"MainField_Animal_Bull_A_1296498507",
0x7af8815b,"MainField_Animal_Bull_A_1558280109",
0xc729c33b,"MainField_Animal_Bull_A_1812144894",
0x3890360b,"MainField_Animal_Bull_A_1839714254",
0xfa28ee06,"MainField_Animal_Bull_A_1845302947",
0x2ef68fb2,"MainField_Animal_Bull_A_1855183230",
0xf294b042,"MainField_Animal_Bull_A_2043810120",
0xb21137e3,"MainField_Animal_Bull_A_2175301744",
0x8285fff4,"MainField_Animal_Bull_A_2232025535",
0x5bed0d91,"MainField_Animal_Bull_A_2312353984",
0x35eabc5d,"MainField_Animal_Bull_A_3312947831",
0x30743445,"MainField_Animal_Bull_A_3616462749",
0xbe77af59,"MainField_Animal_Bull_A_3642453690",
0x90475adb,"MainField_Animal_Bull_A_3644857811",
0x0f8b6a03,"MainField_Animal_Bull_A_532604098",
0xb25a534e,"MainField_Animal_Bull_A_567102721",
0xa0f460b6,"MainField_Animal_Bull_A_616956715",
0x4ec5fea7,"MainField_Animal_Bull_A_6465727",
0x63de0ad2,"MainField_Animal_Bull_A_718603353",
0x77cb2fee,"MainField_Animal_Cassowary_A_1800548852",
0x480cf97f,"MainField_Animal_Cassowary_A_3780314816",
0x9664093e,"MainField_Animal_Cassowary_A_553042551",
0x8a51823c,"MainField_Animal_Crow_A_1060599244",
0x7e439d55,"MainField_Animal_Crow_A_1066073001",
0xea6a1948,"MainField_Animal_Crow_A_1066521898",
0xa37fe629,"MainField_Animal_Crow_A_1086539301",
0x0d8277eb,"MainField_Animal_Crow_A_1101486596",
0x3f983fa7,"MainField_Animal_Crow_A_1124313675",
0x775aee55,"MainField_Animal_Crow_A_1161304951",
0xe6a361fb,"MainField_Animal_Crow_A_1163346435",
0xc9e7d408,"MainField_Animal_Crow_A_1174500150",
0x9ee0f30a,"MainField_Animal_Crow_A_1192580211",
0x9199e432,"MainField_Animal_Crow_A_1201675625",
0x1c4ccc64,"MainField_Animal_Crow_A_1255072820",
0x87bca02e,"MainField_Animal_Crow_A_1264661505",
0x5340b6c6,"MainField_Animal_Crow_A_1274162344",
0xa91ef04c,"MainField_Animal_Crow_A_1322554735",
0x6a14dcc9,"MainField_Animal_Crow_A_1370246888",
0x912df578,"MainField_Animal_Crow_A_1370926094",
0x13e49ddf,"MainField_Animal_Crow_A_1372644641",
0x16844f82,"MainField_Animal_Crow_A_1374179973",
0x79bc44a4,"MainField_Animal_Crow_A_1403569681",
0x1b4c42d9,"MainField_Animal_Crow_A_140382764",
0xbafc7d03,"MainField_Animal_Crow_A_1445783955",
0xa315269c,"MainField_Animal_Crow_A_1456067832",
0x8b33bb02,"MainField_Animal_Crow_A_1488245433",
0x3a3942ce,"MainField_Animal_Crow_A_1566550870",
0x123fbf14,"MainField_Animal_Crow_A_1657600871",
0xefb5ca00,"MainField_Animal_Crow_A_1679374789",
0xf64b1c81,"MainField_Animal_Crow_A_1708369722",
0x01f3b62e,"MainField_Animal_Crow_A_1714036779",
0x92a75990,"MainField_Animal_Crow_A_1723586667",
0x84be0a33,"MainField_Animal_Crow_A_1761613705",
0x83950838,"MainField_Animal_Crow_A_1775252703",
0x3f8d9a2b,"MainField_Animal_Crow_A_1779672104",
0x462c77f7,"MainField_Animal_Crow_A_1781548824",
0x2ddaa443,"MainField_Animal_Crow_A_1799076860",
0x4edcd333,"MainField_Animal_Crow_A_1812514970",
0x3d9b52d5,"MainField_Animal_Crow_A_1822870759",
0x99a3db50,"MainField_Animal_Crow_A_190326746",
0x5c5c5167,"MainField_Animal_Crow_A_1964841111",
0x9bdf4479,"MainField_Animal_Crow_A_2025381144",
0x9d5044ee,"MainField_Animal_Crow_A_2031687734",
0x98db108d,"MainField_Animal_Crow_A_2094135630",
0xdd81b284,"MainField_Animal_Crow_A_2098628754",
0x0352c318,"MainField_Animal_Crow_A_2102395622",
0x7f094312,"MainField_Animal_Crow_A_2105043528",
0x78e5d2b7,"MainField_Animal_Crow_A_2109618779",
0x0edcb234,"MainField_Animal_Crow_A_2113628611",
0xf9583ad3,"MainField_Animal_Crow_A_2121260630",
0x0a54db30,"MainField_Animal_Crow_A_2147778898",
0xaccd4328,"MainField_Animal_Crow_A_2149604045",
0x2c00e26e,"MainField_Animal_Crow_A_2158159973",
0xff5cf1ed,"MainField_Animal_Crow_A_2161117600",
0xbe6748bf,"MainField_Animal_Crow_A_2163536546",
0x39f3afb9,"MainField_Animal_Crow_A_2237544756",
0xd57126dc,"MainField_Animal_Crow_A_2241521027",
0x66e1c0e7,"MainField_Animal_Crow_A_2280144363",
0xd242a499,"MainField_Animal_Crow_A_2292977956",
0xbeaafa39,"MainField_Animal_Crow_A_2346847604",
0x4df864d9,"MainField_Animal_Crow_A_2353011748",
0xe94355a6,"MainField_Animal_Crow_A_2370953336",
0x4c619f77,"MainField_Animal_Crow_A_2391402845",
0x6dc50a6b,"MainField_Animal_Crow_A_2392782266",
0x6e410292,"MainField_Animal_Crow_A_2394974225",
0xd6b71cce,"MainField_Animal_Crow_A_2408129052",
0x556eafbb,"MainField_Animal_Crow_A_241095927",
0xd20fedde,"MainField_Animal_Crow_A_2428157413",
0x5174edf1,"MainField_Animal_Crow_A_2451272339",
0x1baf0505,"MainField_Animal_Crow_A_2458678154",
0xefb5bf1d,"MainField_Animal_Crow_A_2484892575",
0x78c8312d,"MainField_Animal_Crow_A_2515842347",
0xec63eb95,"MainField_Animal_Crow_A_2518504344",
0x324455e6,"MainField_Animal_Crow_A_2594718756",
0xcc946d9c,"MainField_Animal_Crow_A_2657959321",
0x8be58f6d,"MainField_Animal_Crow_A_2669514972",
0x7e0e905a,"MainField_Animal_Crow_A_2717593681",
0x0f119463,"MainField_Animal_Crow_A_2718180746",
0x3ad3aa35,"MainField_Animal_Crow_A_2722818901",
0x40959adb,"MainField_Animal_Crow_A_2726634547",
0x1feae1e0,"MainField_Animal_Crow_A_2748561693",
0xc1ccbfc8,"MainField_Animal_Crow_A_2749289383",
0xd5a07aec,"MainField_Animal_Crow_A_2787149855",
0x5e8cce53,"MainField_Animal_Crow_A_2804623229",
0xca50b79a,"MainField_Animal_Crow_A_2807531143",
0xd8944c2c,"MainField_Animal_Crow_A_2897880946",
0xd66d2c0c,"MainField_Animal_Crow_A_291575764",
0xae1643d0,"MainField_Animal_Crow_A_2922419953",
0xc6631a39,"MainField_Animal_Crow_A_2936618978",
0x78b27c1a,"MainField_Animal_Crow_A_2940440326",
0x80cb955c,"MainField_Animal_Crow_A_2987873907",
0x67704a87,"MainField_Animal_Crow_A_2993587747",
0xc6cb2592,"MainField_Animal_Crow_A_3029306362",
0xf520a6f5,"MainField_Animal_Crow_A_3029619833",
0xcc7c0a07,"MainField_Animal_Crow_A_3098180041",
0xd4c920fb,"MainField_Animal_Crow_A_3144627042",
0xcabfdade,"MainField_Animal_Crow_A_3145211684",
0xa4f4e739,"MainField_Animal_Crow_A_3156649567",
0xfa2f6a4b,"MainField_Animal_Crow_A_317742093",
0x1a4e490c,"MainField_Animal_Crow_A_3180873203",
0x043d738e,"MainField_Animal_Crow_A_3193336077",
0x09d96da1,"MainField_Animal_Crow_A_3307039188",
0xd42d44ab,"MainField_Animal_Crow_A_331125617",
0x8dbff105,"MainField_Animal_Crow_A_3313077011",
0xcbacc4ec,"MainField_Animal_Crow_A_3390720717",
0xd9e8e039,"MainField_Animal_Crow_A_3393811227",
0x65ee3ee7,"MainField_Animal_Crow_A_3394751277",
0x419be0f5,"MainField_Animal_Crow_A_3397894388",
0x07d25456,"MainField_Animal_Crow_A_3405442536",
0x4e780ce9,"MainField_Animal_Crow_A_3425806432",
0x6a297a93,"MainField_Animal_Crow_A_3465401342",
0xd7037cdf,"MainField_Animal_Crow_A_346731838",
0xb31d77e8,"MainField_Animal_Crow_A_3525874992",
0x397b0dad,"MainField_Animal_Crow_A_3535724629",
0x49e00256,"MainField_Animal_Crow_A_3536698347",
0x50715370,"MainField_Animal_Crow_A_3542229274",
0xfdcf5ee7,"MainField_Animal_Crow_A_3604065380",
0x60cd65e2,"MainField_Animal_Crow_A_3684058220",
0x6bb9efa9,"MainField_Animal_Crow_A_3691055464",
0x69a78b19,"MainField_Animal_Crow_A_3761668643",
0xd94cd320,"MainField_Animal_Crow_A_3817998542",
0xbef4dbe3,"MainField_Animal_Crow_A_3823027407",
0xcc557209,"MainField_Animal_Crow_A_3828525445",
0xb17ab44e,"MainField_Animal_Crow_A_3842506619",
0x2917edb0,"MainField_Animal_Crow_A_3851468276",
0x27533269,"MainField_Animal_Crow_A_3946846860",
0x1e121077,"MainField_Animal_Crow_A_3949478974",
0x487f90bb,"MainField_Animal_Crow_A_4008256479",
0xa6ed01d0,"MainField_Animal_Crow_A_402831066",
0xe3271afb,"MainField_Animal_Crow_A_4068621502",
0x96c528c3,"MainField_Animal_Crow_A_4103661666",
0x6e53a2bf,"MainField_Animal_Crow_A_4108768416",
0x5dbea921,"MainField_Animal_Crow_A_4143290439",
0x4728d6a4,"MainField_Animal_Crow_A_4186510644",
0x23923bca,"MainField_Animal_Crow_A_4196471928",
0xc1b80cd6,"MainField_Animal_Crow_A_420767518",
0x5bc5a04e,"MainField_Animal_Crow_A_4218087780",
0x75967a46,"MainField_Animal_Crow_A_4244963154",
0xd2cd6682,"MainField_Animal_Crow_A_4249005320",
0xd4cefead,"MainField_Animal_Crow_A_4285144582",
0x607f293b,"MainField_Animal_Crow_A_442777426",
0x69851e64,"MainField_Animal_Crow_A_472997990",
0x48ee2b43,"MainField_Animal_Crow_A_475501948",
0x2e16fc15,"MainField_Animal_Crow_A_481934569",
0xa2872e68,"MainField_Animal_Crow_A_485094696",
0x15e73993,"MainField_Animal_Crow_A_491233982",
0xb478ecbc,"MainField_Animal_Crow_A_501245641",
0xdc53180e,"MainField_Animal_Crow_A_510769954",
0xd09730d1,"MainField_Animal_Crow_A_566760844",
0x8cd361c4,"MainField_Animal_Crow_A_63627018",
0xd6b49857,"MainField_Animal_Crow_A_638193155",
0x80f576d4,"MainField_Animal_Crow_A_642513635",
0x76fb0364,"MainField_Animal_Crow_A_740983787",
0x50923efe,"MainField_Animal_Crow_A_779188260",
0x3df4254c,"MainField_Animal_Crow_A_784609318",
0x453c136a,"MainField_Animal_Crow_A_813290213",
0x439ca379,"MainField_Animal_Crow_A_818963454",
0xbcf5041c,"MainField_Animal_Crow_A_830579094",
0x50b13b21,"MainField_Animal_Crow_A_835806192",
0x36c0bb9b,"MainField_Animal_Crow_A_838779665",
0xa1aa9d68,"MainField_Animal_Crow_A_842946328",
0x06731dc8,"MainField_Animal_Crow_A_866488633",
0xfbbcf507,"MainField_Animal_Crow_A_879678181",
0x32909322,"MainField_Animal_Crow_A_935580157",
0x35229e03,"MainField_Animal_Crow_A_95491151",
0x0ed1cb26,"MainField_Animal_Crow_A_964469196",
0xf721099d,"MainField_Animal_Crow_A_968178694",
0x09828130,"MainField_Animal_Crow_Swarm_1056673066",
0x97932b55,"MainField_Animal_Crow_Swarm_2954255279",
0xe5b907a9,"MainField_Animal_Crow_Swarm_3231942201",
0x573ae575,"MainField_Animal_Crow_Swarm_3363999869",
0x7cf3992d,"MainField_Animal_Crow_Swarm_4135261984",
0x868cc261,"MainField_Animal_Crow_Swarm_422522890",
0xe8c08129,"MainField_Animal_Crow_Swarm_52368661",
0x519c85da,"MainField_Animal_Deer_A_1044123464",
0x9a7988d9,"MainField_Animal_Deer_A_1100294752",
0x7509f503,"MainField_Animal_Deer_A_1226452564",
0x870cab10,"MainField_Animal_Deer_A_1262534137",
0x369d9b81,"MainField_Animal_Deer_A_1274020497",
0x1a5e919b,"MainField_Animal_Deer_A_1297977020",
0xaafa1df7,"MainField_Animal_Deer_A_1321473507",
0x940c09eb,"MainField_Animal_Deer_A_1343514730",
0xae38a27c,"MainField_Animal_Deer_A_1896516716",
0xe0655228,"MainField_Animal_Deer_A_1966475782",
0xfc912196,"MainField_Animal_Deer_A_2048712243",
0x33bdfb33,"MainField_Animal_Deer_A_2152903116",
0xda1fd772,"MainField_Animal_Deer_A_2529581172",
0x792a9276,"MainField_Animal_Deer_A_2592029846",
0x9b00f09d,"MainField_Animal_Deer_A_2819698887",
0x0835c42f,"MainField_Animal_Deer_A_2843019254",
0xbd0b8fae,"MainField_Animal_Deer_A_3006839873",
0x267a5b99,"MainField_Animal_Deer_A_3158414133",
0x8dcaf061,"MainField_Animal_Deer_A_316993651",
0xe42b4b07,"MainField_Animal_Deer_A_3270888251",
0xa3d81947,"MainField_Animal_Deer_A_3287303877",
0xe0d36dce,"MainField_Animal_Deer_A_3307225680",
0xbcb12f25,"MainField_Animal_Deer_A_3761512499",
0xe3f13f34,"MainField_Animal_Deer_A_3799897095",
0xed703d1c,"MainField_Animal_Deer_A_3809841033",
0x8db1e5bc,"MainField_Animal_Deer_A_3815438119",
0xc61b6e4f,"MainField_Animal_Deer_A_3837152474",
0x76e33685,"MainField_Animal_Deer_A_4025023696",
0x7c08a401,"MainField_Animal_Deer_A_4114277855",
0x859ba7c7,"MainField_Animal_Deer_A_4158646122",
0x43baeaa3,"MainField_Animal_Deer_A_4211962996",
0xf9fbce28,"MainField_Animal_Deer_A_556459303",
0xf00bf1d5,"MainField_Animal_Deer_A_611635319",
0x70074dc5,"MainField_Animal_Deer_A_762582691",
0xe00c3c07,"MainField_Animal_Deer_A_773552591",
0x48ac48a1,"MainField_Animal_Deer_A_883344984",
0x02a94b1f,"MainField_Animal_Deer_A_89904",
0xa416e3d7,"MainField_Animal_Deer_A_995526419",
0xaf1d8f62,"MainField_Animal_Doe_A_1021998680",
0xf041ced7,"MainField_Animal_Doe_A_1070797349",
0xe0bc34a5,"MainField_Animal_Doe_A_112133321",
0x231781be,"MainField_Animal_Doe_A_1224255911",
0xd3c2ec22,"MainField_Animal_Doe_A_1343390662",
0x0bfb1019,"MainField_Animal_Doe_A_1385211445",
0xbd3553ec,"MainField_Animal_Doe_A_1430907685",
0x41a34f5e,"MainField_Animal_Doe_A_1447226586",
0x6975c7df,"MainField_Animal_Doe_A_1516943761",
0xe367b03f,"MainField_Animal_Doe_A_1575279310",
0x1eca3a22,"MainField_Animal_Doe_A_1594284686",
0xde1c0f2c,"MainField_Animal_Doe_A_1691197182",
0x5d38d77e,"MainField_Animal_Doe_A_1714947461",
0xb6d5a301,"MainField_Animal_Doe_A_1739313755",
0xc0e135cb,"MainField_Animal_Doe_A_1770379346",
0x6360507f,"MainField_Animal_Doe_A_18275447",
0x9b09c4f7,"MainField_Animal_Doe_A_1897471200",
0x9986e11c,"MainField_Animal_Doe_A_1898586782",
0x6f150cb4,"MainField_Animal_Doe_A_1971561244",
0x86af1381,"MainField_Animal_Doe_A_2084622761",
0x87f2c1b2,"MainField_Animal_Doe_A_2132284912",
0x14894cf4,"MainField_Animal_Doe_A_2194301683",
0xc977a420,"MainField_Animal_Doe_A_2276342373",
0xca336526,"MainField_Animal_Doe_A_2288457174",
0x448abb44,"MainField_Animal_Doe_A_2320674739",
0x4b8ed956,"MainField_Animal_Doe_A_2439163873",
0x89fb8aed,"MainField_Animal_Doe_A_2475858406",
0x572cd705,"MainField_Animal_Doe_A_2588132235",
0x69e38908,"MainField_Animal_Doe_A_2887259081",
0x5f3466ea,"MainField_Animal_Doe_A_2909196104",
0x08c82495,"MainField_Animal_Doe_A_2914007946",
0x6afc7975,"MainField_Animal_Doe_A_2930634273",
0xb6023513,"MainField_Animal_Doe_A_2940553652",
0x4c65d578,"MainField_Animal_Doe_A_2956870118",
0x389431c6,"MainField_Animal_Doe_A_2991287574",
0xa4db98f3,"MainField_Animal_Doe_A_3015021452",
0x00e28e72,"MainField_Animal_Doe_A_304861219",
0xde35a859,"MainField_Animal_Doe_A_3075679216",
0x5be79ee7,"MainField_Animal_Doe_A_3083772573",
0x9818d9dd,"MainField_Animal_Doe_A_3095292931",
0x98573db9,"MainField_Animal_Doe_A_3099642012",
0x5281c1d4,"MainField_Animal_Doe_A_3140215602",
0xd5685230,"MainField_Animal_Doe_A_3260060977",
0xd944dadb,"MainField_Animal_Doe_A_3273732587",
0xf9cef465,"MainField_Animal_Doe_A_329474132",
0x6b13b893,"MainField_Animal_Doe_A_3332400281",
0x738ed621,"MainField_Animal_Doe_A_3339564448",
0x1334c7b7,"MainField_Animal_Doe_A_3384013170",
0xc571a1d0,"MainField_Animal_Doe_A_3460898345",
0x957a1941,"MainField_Animal_Doe_A_3487344574",
0xbe475a8c,"MainField_Animal_Doe_A_3512568973",
0x9abb6eb8,"MainField_Animal_Doe_A_3526131199",
0x959ddd77,"MainField_Animal_Doe_A_3603764899",
0xc042cbb6,"MainField_Animal_Doe_A_3731869647",
0xe45e539f,"MainField_Animal_Doe_A_3794066799",
0x28451bd4,"MainField_Animal_Doe_A_3803489110",
0xdc5864b5,"MainField_Animal_Doe_A_380855560",
0x2fb1de99,"MainField_Animal_Doe_A_3847376099",
0xc16b06cd,"MainField_Animal_Doe_A_3884060181",
0x0bcbf6a2,"MainField_Animal_Doe_A_3912696648",
0x640dacf8,"MainField_Animal_Doe_A_3930036497",
0x488a7e49,"MainField_Animal_Doe_A_3975341714",
0x527fa255,"MainField_Animal_Doe_A_4081596090",
0x7414fdf8,"MainField_Animal_Doe_A_4150289276",
0x210102ca,"MainField_Animal_Doe_A_4254840761",
0x1ec9b016,"MainField_Animal_Doe_A_4276378486",
0xb2d77581,"MainField_Animal_Doe_A_567413371",
0xb79c5a4d,"MainField_Animal_Doe_A_599828064",
0x01ad9580,"MainField_Animal_Doe_A_606236274",
0x93ad5463,"MainField_Animal_Doe_A_62003650",
0x3dddd1a5,"MainField_Animal_Doe_A_625652107",
0xdb1c80b4,"MainField_Animal_Doe_A_759301965",
0xa11c4b0b,"MainField_Animal_Doe_A_806803316",
0x9d5a3867,"MainField_Animal_Elk_A_1207787126",
0xbc5016bb,"MainField_Animal_Elk_A_1320933199",
0x5f0450be,"MainField_Animal_Elk_A_1363167387",
0x3fc1f05b,"MainField_Animal_Elk_A_1568468925",
0xe408a904,"MainField_Animal_Elk_A_1571934512",
0xf6de24c6,"MainField_Animal_Elk_A_160688574",
0xb1497659,"MainField_Animal_Elk_A_1625222364",
0x2a654e81,"MainField_Animal_Elk_A_1695778540",
0xff375f6a,"MainField_Animal_Elk_A_1695861500",
0x1caa5fe1,"MainField_Animal_Elk_A_1700671466",
0x94c565e1,"MainField_Animal_Elk_A_1765552797",
0x22bacaf1,"MainField_Animal_Elk_A_1794766500",
0xa1c163f8,"MainField_Animal_Elk_A_1862247701",
0x7c570759,"MainField_Animal_Elk_A_1901326537",
0x2262453a,"MainField_Animal_Elk_A_2061684022",
0xf3498181,"MainField_Animal_Elk_A_2135150284",
0x65ce9b69,"MainField_Animal_Elk_A_2367715066",
0x0370e0e4,"MainField_Animal_Elk_A_2391697424",
0x8cdff980,"MainField_Animal_Elk_A_2393640474",
0x134bde4f,"MainField_Animal_Elk_A_242905854",
0xd29dbba1,"MainField_Animal_Elk_A_2439284907",
0xf6deb7d6,"MainField_Animal_Elk_A_2521327163",
0x665febdb,"MainField_Animal_Elk_A_2526733033",
0xf35f6776,"MainField_Animal_Elk_A_2564932862",
0xce2cc35f,"MainField_Animal_Elk_A_260202546",
0x33b3de52,"MainField_Animal_Elk_A_2604746813",
0x55901ec1,"MainField_Animal_Elk_A_2680498460",
0x7bad7a57,"MainField_Animal_Elk_A_2690408989",
0xd5bacacd,"MainField_Animal_Elk_A_2726493057",
0x075e5659,"MainField_Animal_Elk_A_2826110703",
0x4ce1c53e,"MainField_Animal_Elk_A_2832923060",
0x2bd48bce,"MainField_Animal_Elk_A_2856130588",
0x5fcf26be,"MainField_Animal_Elk_A_2938025847",
0x920b8f7f,"MainField_Animal_Elk_A_2948871103",
0xe1414b7c,"MainField_Animal_Elk_A_3062479547",
0x48abb3c1,"MainField_Animal_Elk_A_307312102",
0x04090a02,"MainField_Animal_Elk_A_322897726",
0x8a85add4,"MainField_Animal_Elk_A_3293281583",
0x8efee6bf,"MainField_Animal_Elk_A_3309360120",
0xa3b4d47c,"MainField_Animal_Elk_A_3336846760",
0x7b4d731a,"MainField_Animal_Elk_A_3363610221",
0xe4ace137,"MainField_Animal_Elk_A_3530400395",
0xf5fad9c1,"MainField_Animal_Elk_A_3600004745",
0x2208c1a3,"MainField_Animal_Elk_A_3638598944",
0xc8ed9215,"MainField_Animal_Elk_A_3725958813",
0xa6805ca3,"MainField_Animal_Elk_A_3734027974",
0x2756ce62,"MainField_Animal_Elk_A_373464616",
0xe2bff653,"MainField_Animal_Elk_A_3750548489",
0xad1e1b39,"MainField_Animal_Elk_A_379640540",
0x79824de0,"MainField_Animal_Elk_A_3872167403",
0x28092e41,"MainField_Animal_Elk_A_4016268209",
0x7ac244bd,"MainField_Animal_Elk_A_4070287840",
0xa072f9c7,"MainField_Animal_Elk_A_4108153748",
0xe75d7c42,"MainField_Animal_Elk_A_4113339293",
0x024c85a0,"MainField_Animal_Elk_A_4184833501",
0xd961f0ff,"MainField_Animal_Elk_A_430830702",
0x8ff70bb2,"MainField_Animal_Elk_A_437908875",
0x4ae0458e,"MainField_Animal_Elk_A_505607364",
0xd9d56888,"MainField_Animal_Elk_A_534491046",
0xcddb7b79,"MainField_Animal_Elk_A_54994533",
0x2a63c94d,"MainField_Animal_Elk_A_747887059",
0x3afcda34,"MainField_Animal_Fish_A_1015023162",
0xf9fea06e,"MainField_Animal_Fish_A_1023826693",
0x6a5ed33a,"MainField_Animal_Fish_A_1038857945",
0xfbd3cd15,"MainField_Animal_Fish_A_1046927199",
0xed8c0a16,"MainField_Animal_Fish_A_1055551691",
0xb5411eef,"MainField_Animal_Fish_A_1055592369",
0xb79c2e40,"MainField_Animal_Fish_A_1130023524",
0xbd83ba81,"MainField_Animal_Fish_A_1131791864",
0x4c38361f,"MainField_Animal_Fish_A_1134217185",
0xeb64c7a0,"MainField_Animal_Fish_A_1136489033",
0x8770e8b7,"MainField_Animal_Fish_A_1141657778",
0x5b002ce1,"MainField_Animal_Fish_A_1144458877",
0xfceebf6e,"MainField_Animal_Fish_A_1146610152",
0x55446d98,"MainField_Animal_Fish_A_1154528848",
0x177ec3f3,"MainField_Animal_Fish_A_1163794841",
0x5266c6a0,"MainField_Animal_Fish_A_117137295",
0x1429799a,"MainField_Animal_Fish_A_1195792517",
0xa57ea411,"MainField_Animal_Fish_A_1257014863",
0xdef781d0,"MainField_Animal_Fish_A_1325633504",
0xc6b9ad02,"MainField_Animal_Fish_A_1342773930",
0x4bc56322,"MainField_Animal_Fish_A_1395885072",
0xc6ab02f0,"MainField_Animal_Fish_A_1408743182",
0xae50de7e,"MainField_Animal_Fish_A_1417366927",
0x434d62e6,"MainField_Animal_Fish_A_1474103825",
0x3a2113bf,"MainField_Animal_Fish_A_1514273573",
0xf3dbc2aa,"MainField_Animal_Fish_A_1520845576",
0xddb1391a,"MainField_Animal_Fish_A_1554924542",
0x43471953,"MainField_Animal_Fish_A_1573619473",
0x285c4bf9,"MainField_Animal_Fish_A_1650486106",
0x2c17bce9,"MainField_Animal_Fish_A_1652916784",
0x112eed72,"MainField_Animal_Fish_A_1656014246",
0x76ea1243,"MainField_Animal_Fish_A_1664406143",
0x895cae58,"MainField_Animal_Fish_A_1673869123",
0x0b3ba603,"MainField_Animal_Fish_A_1715388012",
0x8169e5d6,"MainField_Animal_Fish_A_1744990380",
0xc3db605b,"MainField_Animal_Fish_A_1754037533",
0x63a65628,"MainField_Animal_Fish_A_1779594496",
0x472d1b32,"MainField_Animal_Fish_A_1818701390",
0x8ed30261,"MainField_Animal_Fish_A_1820147095",
0xf5815a88,"MainField_Animal_Fish_A_1861969246",
0x86f122b9,"MainField_Animal_Fish_A_1876232113",
0xe0068772,"MainField_Animal_Fish_A_1943621507",
0x968029d1,"MainField_Animal_Fish_A_1951102732",
0xe8915610,"MainField_Animal_Fish_A_1984904585",
0xa97d296b,"MainField_Animal_Fish_A_2023681210",
0x5439f30a,"MainField_Animal_Fish_A_2041781050",
0x2851e6ba,"MainField_Animal_Fish_A_2067342715",
0x2d7cd39c,"MainField_Animal_Fish_A_2111166499",
0x6eebd75b,"MainField_Animal_Fish_A_2122568230",
0x8d6f8d2d,"MainField_Animal_Fish_A_213060961",
0x05f1f94d,"MainField_Animal_Fish_A_2217972213",
0xc73c0c1c,"MainField_Animal_Fish_A_2263898772",
0x40d8f6b9,"MainField_Animal_Fish_A_2294816102",
0x2b71eefa,"MainField_Animal_Fish_A_2304218363",
0x55be4ba4,"MainField_Animal_Fish_A_2307121192",
0x730432a0,"MainField_Animal_Fish_A_2312683309",
0x41d65972,"MainField_Animal_Fish_A_2323068626",
0x7f419bda,"MainField_Animal_Fish_A_2403661311",
0x6544dc06,"MainField_Animal_Fish_A_2405012219",
0x51aca3a2,"MainField_Animal_Fish_A_2436278829",
0x6fb7a62a,"MainField_Animal_Fish_A_2436304470",
0x726e73f1,"MainField_Animal_Fish_A_2448471256",
0xfba397e0,"MainField_Animal_Fish_A_2499841618",
0x0f866263,"MainField_Animal_Fish_A_2501921507",
0xd8b987f8,"MainField_Animal_Fish_A_252845743",
0x24e110e6,"MainField_Animal_Fish_A_2594321864",
0x5e34a091,"MainField_Animal_Fish_A_2597735392",
0xc50e3fe3,"MainField_Animal_Fish_A_2634511017",
0xfe9855f1,"MainField_Animal_Fish_A_2665415345",
0x6f4bc486,"MainField_Animal_Fish_A_2696207986",
0x999d17e3,"MainField_Animal_Fish_A_2722628890",
0xa09fb9ec,"MainField_Animal_Fish_A_2741541719",
0x3699b870,"MainField_Animal_Fish_A_2755016163",
0xf430747d,"MainField_Animal_Fish_A_2855890769",
0x7249a89a,"MainField_Animal_Fish_A_2866730876",
0x4e7894ea,"MainField_Animal_Fish_A_2880650472",
0x762e3d69,"MainField_Animal_Fish_A_2905454763",
0xfbc9cff2,"MainField_Animal_Fish_A_2908007240",
0x91737d42,"MainField_Animal_Fish_A_2914194277",
0xfda20aad,"MainField_Animal_Fish_A_2942471774",
0x7777c9ca,"MainField_Animal_Fish_A_2948765968",
0x4c83ebe3,"MainField_Animal_Fish_A_2960850703",
0x5f59c6f8,"MainField_Animal_Fish_A_2998313106",
0x25ca014e,"MainField_Animal_Fish_A_3016091197",
0x47194a8a,"MainField_Animal_Fish_A_3025115657",
0x24436107,"MainField_Animal_Fish_A_3030917668",
0x38d4f776,"MainField_Animal_Fish_A_303738339",
0xf6442e94,"MainField_Animal_Fish_A_3043039351",
0xdda80f1d,"MainField_Animal_Fish_A_31027951",
0x696baa4d,"MainField_Animal_Fish_A_3103844876",
0x67207167,"MainField_Animal_Fish_A_3103959295",
0xf2f657ba,"MainField_Animal_Fish_A_3128142358",
0x07d3ca6a,"MainField_Animal_Fish_A_318390689",
0x8ed79b4b,"MainField_Animal_Fish_A_3213461417",
0x8141eedc,"MainField_Animal_Fish_A_3220049441",
0x70b9e34c,"MainField_Animal_Fish_A_3229886100",
0x54e44094,"MainField_Animal_Fish_A_3276319391",
0x160f95ed,"MainField_Animal_Fish_A_3283030706",
0x4f3e6b38,"MainField_Animal_Fish_A_3290787462",
0x0bc4883f,"MainField_Animal_Fish_A_3301103275",
0x7908fea6,"MainField_Animal_Fish_A_3313763825",
0xa535da68,"MainField_Animal_Fish_A_3331069996",
0x1e224095,"MainField_Animal_Fish_A_335180343",
0x7d4173d3,"MainField_Animal_Fish_A_3352689358",
0x8ec69c09,"MainField_Animal_Fish_A_3366803852",
0x2024da15,"MainField_Animal_Fish_A_3397646137",
0xe765a9cc,"MainField_Animal_Fish_A_340994808",
0x0c5a2a4e,"MainField_Animal_Fish_A_3426549994",
0xe8467d60,"MainField_Animal_Fish_A_3446971008",
0x75a9ae5d,"MainField_Animal_Fish_A_3456473193",
0xf255e41a,"MainField_Animal_Fish_A_3463987395",
0x258e06f8,"MainField_Animal_Fish_A_3478754048",
0xc82dc899,"MainField_Animal_Fish_A_348260309",
0x05ac8678,"MainField_Animal_Fish_A_3504927455",
0x65144b8a,"MainField_Animal_Fish_A_3536257427",
0x1c9ccb5e,"MainField_Animal_Fish_A_3564088257",
0x586bb7f3,"MainField_Animal_Fish_A_3572167182",
0x7c3b7022,"MainField_Animal_Fish_A_3578972225",
0x1f81cab5,"MainField_Animal_Fish_A_3607890592",
0x24a10407,"MainField_Animal_Fish_A_3632213768",
0x14a817b6,"MainField_Animal_Fish_A_3647062158",
0x760561cb,"MainField_Animal_Fish_A_3648211555",
0xa56712dd,"MainField_Animal_Fish_A_3662912478",
0xa340031d,"MainField_Animal_Fish_A_3669238771",
0x31d2bd82,"MainField_Animal_Fish_A_3683034414",
0xceae801d,"MainField_Animal_Fish_A_3704208572",
0xa0ca6121,"MainField_Animal_Fish_A_3711852487",
0x123449e3,"MainField_Animal_Fish_A_3722045418",
0x7d1f265a,"MainField_Animal_Fish_A_3731609953",
0x8c3a738a,"MainField_Animal_Fish_A_3756649957",
0x1dff6782,"MainField_Animal_Fish_A_3803999233",
0x729912e3,"MainField_Animal_Fish_A_3824226314",
0x7b563958,"MainField_Animal_Fish_A_382604172",
0xacd46456,"MainField_Animal_Fish_A_3826443296",
0x640ebdf2,"MainField_Animal_Fish_A_3834516952",
0x8e638c96,"MainField_Animal_Fish_A_3842912806",
0x8b8e6631,"MainField_Animal_Fish_A_3870261892",
0xb2d692eb,"MainField_Animal_Fish_A_3912850645",
0x5d41fafa,"MainField_Animal_Fish_A_3927614219",
0x2369c54c,"MainField_Animal_Fish_A_3939025223",
0x7000a715,"MainField_Animal_Fish_A_3943534417",
0x6d85f43d,"MainField_Animal_Fish_A_3947902150",
0x1046c0be,"MainField_Animal_Fish_A_3954173614",
0x28449af4,"MainField_Animal_Fish_A_3987015006",
0xb895b50e,"MainField_Animal_Fish_A_3993951941",
0x691d638c,"MainField_Animal_Fish_A_4071638300",
0xf97d7f26,"MainField_Animal_Fish_A_4075341247",
0x0aa45f5a,"MainField_Animal_Fish_A_4078583694",
0x1f221041,"MainField_Animal_Fish_A_4125038251",
0x281a8c65,"MainField_Animal_Fish_A_4139511144",
0x0fec3e85,"MainField_Animal_Fish_A_4159614679",
0x5f4eb1f3,"MainField_Animal_Fish_A_417300305",
0x3cf71638,"MainField_Animal_Fish_A_4191208817",
0x03c4205d,"MainField_Animal_Fish_A_4210051559",
0xcea34617,"MainField_Animal_Fish_A_4273500918",
0x3faa9c4b,"MainField_Animal_Fish_A_4275847983",
0x418b6b4c,"MainField_Animal_Fish_A_432285737",
0xa3c09331,"MainField_Animal_Fish_A_438295280",
0x7af4f3f1,"MainField_Animal_Fish_A_46830638",
0x76cbdbdc,"MainField_Animal_Fish_A_469739195",
0xc9383ccd,"MainField_Animal_Fish_A_478985701",
0x149cfef4,"MainField_Animal_Fish_A_485616643",
0x6a947c1b,"MainField_Animal_Fish_A_53636966",
0x9673284c,"MainField_Animal_Fish_A_537655621",
0x5e8bda2f,"MainField_Animal_Fish_A_576194832",
0x30d1dfcb,"MainField_Animal_Fish_A_61564018",
0x12206341,"MainField_Animal_Fish_A_619743128",
0x9dadfcc0,"MainField_Animal_Fish_A_621489361",
0x9dbdeb10,"MainField_Animal_Fish_A_717144585",
0x7b83b8e9,"MainField_Animal_Fish_A_718059399",
0x1bda908e,"MainField_Animal_Fish_A_730345797",
0x76ae3ce1,"MainField_Animal_Fish_A_759614387",
0xba29ca65,"MainField_Animal_Fish_A_765555614",
0x545f1f82,"MainField_Animal_Fish_A_80126059",
0xc7079a8f,"MainField_Animal_Fish_A_831507340",
0x8529cce9,"MainField_Animal_Fish_A_838571082",
0x504c9fd2,"MainField_Animal_Fish_A_85093598",
0x56be8ca4,"MainField_Animal_Fish_A_858799851",
0x59976fba,"MainField_Animal_Fish_A_869762758",
0xce1464f2,"MainField_Animal_Fish_A_878574322",
0x9ffd5378,"MainField_Animal_Fish_A_884622047",
0x6b5e80d1,"MainField_Animal_Fish_A_913969241",
0xbcef1c51,"MainField_Animal_Fish_A_950865843",
0x241d290d,"MainField_Animal_Fish_A_975611554",
0x7639445f,"MainField_Animal_Fish_A_992635959",
0x31d0360d,"MainField_Animal_Fish_B_1037922742",
0xed83f5f4,"MainField_Animal_Fish_B_1043196581",
0x76fc565a,"MainField_Animal_Fish_B_1064826716",
0x24f60cc2,"MainField_Animal_Fish_B_1145890104",
0x9e9f6446,"MainField_Animal_Fish_B_127123267",
0x57fb8056,"MainField_Animal_Fish_B_1288399211",
0x06b91141,"MainField_Animal_Fish_B_1301273909",
0x37272ff3,"MainField_Animal_Fish_B_1457271272",
0x7329647a,"MainField_Animal_Fish_B_1570829522",
0x799efa1f,"MainField_Animal_Fish_B_1596180633",
0x82f784ea,"MainField_Animal_Fish_B_1615611175",
0x8396a85c,"MainField_Animal_Fish_B_1870744187",
0xba8c24d1,"MainField_Animal_Fish_B_1886959869",
0xfb162aad,"MainField_Animal_Fish_B_2017724612",
0x03fa2122,"MainField_Animal_Fish_B_2136220165",
0x7572efbb,"MainField_Animal_Fish_B_2231527134",
0x0366a3a9,"MainField_Animal_Fish_B_2338623573",
0x9db81f4b,"MainField_Animal_Fish_B_2419181779",
0x38a15ce0,"MainField_Animal_Fish_B_2422584489",
0x9110a311,"MainField_Animal_Fish_B_2533270305",
0x4b183fd9,"MainField_Animal_Fish_B_264047987",
0x4bd7dd0c,"MainField_Animal_Fish_B_2669160088",
0x4cdcf06f,"MainField_Animal_Fish_B_282426613",
0x1a004f88,"MainField_Animal_Fish_B_2872863772",
0x802c3ccc,"MainField_Animal_Fish_B_2927847867",
0x4b6cb0b4,"MainField_Animal_Fish_B_2994234640",
0x59eb5d0c,"MainField_Animal_Fish_B_3036283398",
0x16bed84b,"MainField_Animal_Fish_B_3045407732",
0x0aa3193c,"MainField_Animal_Fish_B_3145532258",
0x10da07d9,"MainField_Animal_Fish_B_3175984884",
0xd6c3aa6a,"MainField_Animal_Fish_B_3379775002",
0x965ab47b,"MainField_Animal_Fish_B_3431704809",
0xb20d44d2,"MainField_Animal_Fish_B_3664100571",
0x9279799f,"MainField_Animal_Fish_B_3696218334",
0x35bcf40b,"MainField_Animal_Fish_B_3818909369",
0x02cd614a,"MainField_Animal_Fish_B_3841940070",
0x67e6506b,"MainField_Animal_Fish_B_4048988906",
0xb3b8e558,"MainField_Animal_Fish_B_4053329227",
0x2075a29a,"MainField_Animal_Fish_B_4147612922",
0xb5296fa2,"MainField_Animal_Fish_B_4275338782",
0xdbd5364b,"MainField_Animal_Fish_B_445353840",
0x4a42db17,"MainField_Animal_Fish_B_463026722",
0xb440e3a5,"MainField_Animal_Fish_B_513380390",
0x2e556b30,"MainField_Animal_Fish_B_527855903",
0x2cc275bf,"MainField_Animal_Fish_B_697278973",
0x38e2ea66,"MainField_Animal_Fish_B_805781286",
0xdafd4509,"MainField_Animal_Fish_C_1331679293",
0x0b6d161e,"MainField_Animal_Fish_C_1610650779",
0x484eda4b,"MainField_Animal_Fish_C_188092462",
0x59d75df6,"MainField_Animal_Fish_C_2512002252",
0xd8ba2ca6,"MainField_Animal_Fish_C_263423835",
0xf9aede57,"MainField_Animal_Fish_C_2690417426",
0xda89685a,"MainField_Animal_Fish_C_2748626316",
0xae8ae4e1,"MainField_Animal_Fish_C_2861640761",
0x035dc8e8,"MainField_Animal_Fish_C_2940355161",
0x455ffb7e,"MainField_Animal_Fish_C_2949872409",
0xb53c5df2,"MainField_Animal_Fish_C_3080484098",
0x9dd5f45e,"MainField_Animal_Fish_C_3173925625",
0x78e9f939,"MainField_Animal_Fish_C_3337690693",
0xb15f43d4,"MainField_Animal_Fish_C_3454194842",
0xe13f1436,"MainField_Animal_Fish_C_3477089791",
0x0dd8488f,"MainField_Animal_Fish_C_4105475330",
0x3ce67918,"MainField_Animal_Fish_C_4231227506",
0x95c3c3f5,"MainField_Animal_Fish_C_4286610107",
0xb67979f1,"MainField_Animal_Fish_C_487821201",
0x4f96cbc2,"MainField_Animal_Fish_C_52180901",
0xac2b8a6a,"MainField_Animal_Fish_C_612506483",
0x77cf5cae,"MainField_Animal_Fish_C_679012299",
0x9ffceb97,"MainField_Animal_Fish_C_783415234",
0x4f77891b,"MainField_Animal_Fish_C_90642784",
0x9b6381b6,"MainField_Animal_Fish_D_1282474362",
0x09b9e7a5,"MainField_Animal_Fish_D_2427304022",
0xe2090cc7,"MainField_Animal_Fish_D_3447733883",
0xff8f7c6c,"MainField_Animal_Fish_D_3806641287",
0x6b825e86,"MainField_Animal_Fish_D_3880854897",
0xf1e69f13,"MainField_Animal_Fish_E_1074216430",
0xf0501430,"MainField_Animal_Fish_E_1084437257",
0x4b949e37,"MainField_Animal_Fish_E_1157304599",
0x42ac1632,"MainField_Animal_Fish_E_1183722308",
0xaefc4db0,"MainField_Animal_Fish_E_1197498625",
0x072d2039,"MainField_Animal_Fish_E_1342948270",
0xc5bd6f95,"MainField_Animal_Fish_E_1359461543",
0x745412f0,"MainField_Animal_Fish_E_1371309548",
0xace3a8d1,"MainField_Animal_Fish_E_1624049256",
0xa66c585a,"MainField_Animal_Fish_E_194103284",
0xe5beb7c8,"MainField_Animal_Fish_E_2042322392",
0x13e1903e,"MainField_Animal_Fish_E_210468919",
0xa35f11b4,"MainField_Animal_Fish_E_2143230356",
0x97f8d41e,"MainField_Animal_Fish_E_2174568247",
0xce17db87,"MainField_Animal_Fish_E_2233206527",
0xcc3dc0cb,"MainField_Animal_Fish_E_2329878365",
0x69d8fe94,"MainField_Animal_Fish_E_2456421026",
0x1e308663,"MainField_Animal_Fish_E_2477361195",
0xf1e66c36,"MainField_Animal_Fish_E_2492231151",
0x4233f8a9,"MainField_Animal_Fish_E_2608389935",
0x199c76ec,"MainField_Animal_Fish_E_2618720515",
0xd4e0a5df,"MainField_Animal_Fish_E_2675155483",
0xe0ff80d3,"MainField_Animal_Fish_E_2680409296",
0x3fb8dd68,"MainField_Animal_Fish_E_2951065887",
0x69066aea,"MainField_Animal_Fish_E_3051448568",
0x3cf0effb,"MainField_Animal_Fish_E_320164532",
0x79454e8f,"MainField_Animal_Fish_E_3215668165",
0x5ed3dd60,"MainField_Animal_Fish_E_3310062547",
0x223e0177,"MainField_Animal_Fish_E_3519287791",
0xef3a6880,"MainField_Animal_Fish_E_3621595811",
0xb8a92e7f,"MainField_Animal_Fish_E_3709993200",
0x5a3180b5,"MainField_Animal_Fish_E_372384917",
0xa2028216,"MainField_Animal_Fish_E_3928261658",
0x7d27183e,"MainField_Animal_Fish_E_3941282",
0x383ed953,"MainField_Animal_Fish_E_4001890506",
0x87a79726,"MainField_Animal_Fish_E_4033905085",
0x549a42e9,"MainField_Animal_Fish_E_4062144365",
0x45a79f8a,"MainField_Animal_Fish_E_4147635265",
0x0021a57c,"MainField_Animal_Fish_E_510599402",
0xf45cd3ce,"MainField_Animal_Fish_E_535536114",
0xabead9a1,"MainField_Animal_Fish_E_601003386",
0xd05cd999,"MainField_Animal_Fish_E_658245918",
0xf3075147,"MainField_Animal_Fish_E_731200381",
0x3de319bc,"MainField_Animal_Fish_E_769060784",
0x1fd212da,"MainField_Animal_Fish_F_1050475470",
0x18983c0c,"MainField_Animal_Fish_F_1073340487",
0xed39aa25,"MainField_Animal_Fish_F_1104678599",
0x6df85314,"MainField_Animal_Fish_F_1113059928",
0x4994a679,"MainField_Animal_Fish_F_1130283585",
0x7cd70f69,"MainField_Animal_Fish_F_1226762393",
0xaa7b10a5,"MainField_Animal_Fish_F_1234946904",
0x07fd00ee,"MainField_Animal_Fish_F_1256692158",
0xc53791e5,"MainField_Animal_Fish_F_1355342596",
0x72915fed,"MainField_Animal_Fish_F_1361098553",
0x171b4259,"MainField_Animal_Fish_F_1371005533",
0x6d702e69,"MainField_Animal_Fish_F_1372779210",
0xcd45e0dd,"MainField_Animal_Fish_F_1405036072",
0x5eb6daf6,"MainField_Animal_Fish_F_1440875506",
0x7b290db4,"MainField_Animal_Fish_F_1453515059",
0x4ce6369f,"MainField_Animal_Fish_F_1470877994",
0xb29ec4f9,"MainField_Animal_Fish_F_1550313514",
0x2cdbdf4b,"MainField_Animal_Fish_F_1580938914",
0xd3f85b59,"MainField_Animal_Fish_F_1617918658",
0xf17d7003,"MainField_Animal_Fish_F_1626545724",
0x0941c24c,"MainField_Animal_Fish_F_1654955510",
0x3ad69973,"MainField_Animal_Fish_F_1669387992",
0xe644579e,"MainField_Animal_Fish_F_1709888751",
0x05b7642b,"MainField_Animal_Fish_F_1738942695",
0x35c9d5eb,"MainField_Animal_Fish_F_1748688536",
0x84518035,"MainField_Animal_Fish_F_1749872600",
0x6d3bce90,"MainField_Animal_Fish_F_1762633522",
0xddae85f7,"MainField_Animal_Fish_F_1807587231",
0xf6cf32c0,"MainField_Animal_Fish_F_1865407990",
0x783b6ee3,"MainField_Animal_Fish_F_1883450601",
0x188a7ba9,"MainField_Animal_Fish_F_1987900842",
0xe9addda2,"MainField_Animal_Fish_F_203613326",
0x95791003,"MainField_Animal_Fish_F_2045894980",
0x632f81ff,"MainField_Animal_Fish_F_2057686980",
0xa3a2d559,"MainField_Animal_Fish_F_2077937309",
0x214247d0,"MainField_Animal_Fish_F_2121666929",
0xdb1b91e3,"MainField_Animal_Fish_F_2127540600",
0x5e358156,"MainField_Animal_Fish_F_215875083",
0x3e38d7a2,"MainField_Animal_Fish_F_2172843683",
0x894aea2a,"MainField_Animal_Fish_F_2176753798",
0x4f6d84b6,"MainField_Animal_Fish_F_2186476008",
0xc4761d39,"MainField_Animal_Fish_F_2218327007",
0x6fbfa8f1,"MainField_Animal_Fish_F_226864531",
0x6f00d701,"MainField_Animal_Fish_F_228786002",
0xe1fb5a75,"MainField_Animal_Fish_F_2291658993",
0xf807f59b,"MainField_Animal_Fish_F_2386974113",
0x538fc8e5,"MainField_Animal_Fish_F_2407723935",
0x7a195742,"MainField_Animal_Fish_F_2415182559",
0xd4563d41,"MainField_Animal_Fish_F_2415252822",
0x031f4bee,"MainField_Animal_Fish_F_2445596845",
0xc6093cd7,"MainField_Animal_Fish_F_2476845224",
0x9c04c4df,"MainField_Animal_Fish_F_24888636",
0xe3a36305,"MainField_Animal_Fish_F_2503288932",
0x3a02547b,"MainField_Animal_Fish_F_2506190075",
0xe697eaf0,"MainField_Animal_Fish_F_2530350513",
0x4e88698c,"MainField_Animal_Fish_F_2554897334",
0x096f1bdd,"MainField_Animal_Fish_F_257184904",
0x5c1fe850,"MainField_Animal_Fish_F_2573688457",
0x0702a389,"MainField_Animal_Fish_F_2622214789",
0x7e90bc69,"MainField_Animal_Fish_F_2623726490",
0x72e38c78,"MainField_Animal_Fish_F_2635981507",
0x63e53a56,"MainField_Animal_Fish_F_2639269568",
0x258a7038,"MainField_Animal_Fish_F_2665181148",
0xde79c84b,"MainField_Animal_Fish_F_2687829937",
0xb379005f,"MainField_Animal_Fish_F_2710569760",
0xb62f38b7,"MainField_Animal_Fish_F_2733891977",
0x978b2a65,"MainField_Animal_Fish_F_2766597958",
0x6782f3b5,"MainField_Animal_Fish_F_2783960927",
0x70fea5d5,"MainField_Animal_Fish_F_2799229596",
0x655b8821,"MainField_Animal_Fish_F_2921588319",
0x6918ff2a,"MainField_Animal_Fish_F_2941585368",
0x6c6a3126,"MainField_Animal_Fish_F_2954099757",
0xc08a19da,"MainField_Animal_Fish_F_295517713",
0x60da6485,"MainField_Animal_Fish_F_2971323444",
0x9c4814bd,"MainField_Animal_Fish_F_2977147885",
0x88da631f,"MainField_Animal_Fish_F_3037107656",
0xb48f25c3,"MainField_Animal_Fish_F_30489818",
0x44efcf80,"MainField_Animal_Fish_F_3054489434",
0x6429352c,"MainField_Animal_Fish_F_3061154046",
0xdf34632b,"MainField_Animal_Fish_F_3077623299",
0xcf60ce9b,"MainField_Animal_Fish_F_3092930438",
0xbdffa5d6,"MainField_Animal_Fish_F_3101395037",
0x357f8e82,"MainField_Animal_Fish_F_31240164",
0x0f669046,"MainField_Animal_Fish_F_3152004397",
0xf8c43bcb,"MainField_Animal_Fish_F_317580409",
0xfd308035,"MainField_Animal_Fish_F_3229089429",
0x30f1c9bd,"MainField_Animal_Fish_F_3238592460",
0x7c91f77b,"MainField_Animal_Fish_F_3252626908",
0x57cd0bc8,"MainField_Animal_Fish_F_3323853986",
0xd75c1d80,"MainField_Animal_Fish_F_3340029115",
0x880d1db4,"MainField_Animal_Fish_F_3362521658",
0x720aa666,"MainField_Animal_Fish_F_3385057203",
0xe53155f3,"MainField_Animal_Fish_F_3436770747",
0xedae8f1d,"MainField_Animal_Fish_F_344450638",
0xf66fd7be,"MainField_Animal_Fish_F_3447257416",
0xc5b6bcc3,"MainField_Animal_Fish_F_3479708798",
0x0f2faeb8,"MainField_Animal_Fish_F_3504172815",
0xaeecd03f,"MainField_Animal_Fish_F_3538481097",
0x0a8913e2,"MainField_Animal_Fish_F_3541084226",
0x8be54a5c,"MainField_Animal_Fish_F_3545851642",
0x72550e58,"MainField_Animal_Fish_F_3554795472",
0x94c4e299,"MainField_Animal_Fish_F_3643070153",
0x1acfb32a,"MainField_Animal_Fish_F_3660563815",
0xc1213ada,"MainField_Animal_Fish_F_371339192",
0x16e05b9f,"MainField_Animal_Fish_F_3752820571",
0xc34a8ab2,"MainField_Animal_Fish_F_3764775508",
0x466269ff,"MainField_Animal_Fish_F_3778822834",
0x841e5f55,"MainField_Animal_Fish_F_3871598651",
0x90c88e21,"MainField_Animal_Fish_F_3918311288",
0xe729d4f1,"MainField_Animal_Fish_F_3927626325",
0x95901966,"MainField_Animal_Fish_F_3928387340",
0xfc5b8f3b,"MainField_Animal_Fish_F_3949318924",
0x09ade196,"MainField_Animal_Fish_F_3975460957",
0xccc836c7,"MainField_Animal_Fish_F_3990859765",
0x129811cf,"MainField_Animal_Fish_F_405335392",
0xa46e5cc7,"MainField_Animal_Fish_F_4081044297",
0x91394e41,"MainField_Animal_Fish_F_4082845149",
0xc1dea202,"MainField_Animal_Fish_F_409135848",
0x12e9c658,"MainField_Animal_Fish_F_4116040062",
0x443ffff1,"MainField_Animal_Fish_F_4169515785",
0xfbe41e2b,"MainField_Animal_Fish_F_4176736841",
0x5acc6bec,"MainField_Animal_Fish_F_4192056480",
0x403fbe89,"MainField_Animal_Fish_F_4242896735",
0xf896a1b7,"MainField_Animal_Fish_F_4287676519",
0x68ad573f,"MainField_Animal_Fish_F_443024776",
0xe1eb0f42,"MainField_Animal_Fish_F_479867151",
0xb94ac2fa,"MainField_Animal_Fish_F_503460808",
0xad6228e8,"MainField_Animal_Fish_F_524152334",
0x9ed9f257,"MainField_Animal_Fish_F_525066062",
0x10b035a4,"MainField_Animal_Fish_F_531937960",
0x5709dc10,"MainField_Animal_Fish_F_548200892",
0x4225c1f0,"MainField_Animal_Fish_F_564515237",
0x0efe6d0c,"MainField_Animal_Fish_F_598826055",
0x1960e728,"MainField_Animal_Fish_F_619780888",
0x57ab8010,"MainField_Animal_Fish_F_636484692",
0x37dedeb1,"MainField_Animal_Fish_F_649122507",
0x5dd0e8f0,"MainField_Animal_Fish_F_667931538",
0x0b51cdbe,"MainField_Animal_Fish_F_688237598",
0xc7d82410,"MainField_Animal_Fish_F_711277050",
0x95d09602,"MainField_Animal_Fish_F_713259438",
0x74847469,"MainField_Animal_Fish_F_723513763",
0x05346451,"MainField_Animal_Fish_F_727767228",
0x33a0a820,"MainField_Animal_Fish_F_747036272",
0xb835d931,"MainField_Animal_Fish_F_802583579",
0x77e176e1,"MainField_Animal_Fish_F_878310103",
0xc410ce9d,"MainField_Animal_Fish_F_894485198",
0xc8e87806,"MainField_Animal_Fish_F_931552223",
0x5159148d,"MainField_Animal_Fish_F_947612669",
0xdd43a8b4,"MainField_Animal_Fish_F_983231012",
0xa845bc89,"MainField_Animal_Fish_F_988985949",
0x14812ef0,"MainField_Animal_Fish_G_1008454911",
0xb4843cd3,"MainField_Animal_Fish_G_102299410",
0xf85f65ff,"MainField_Animal_Fish_G_1046115525",
0x32978f0c,"MainField_Animal_Fish_G_106347462",
0xa3565c27,"MainField_Animal_Fish_G_1066781655",
0x5cfdcd54,"MainField_Animal_Fish_G_1088397724",
0x453bbbb0,"MainField_Animal_Fish_G_1109508888",
0xf35fbcd5,"MainField_Animal_Fish_G_1144909935",
0x00b79d30,"MainField_Animal_Fish_G_1217592129",
0xd3486e01,"MainField_Animal_Fish_G_1227880275",
0xf82a579e,"MainField_Animal_Fish_G_1260190825",
0x5a992cd6,"MainField_Animal_Fish_G_1275698627",
0x02205cc6,"MainField_Animal_Fish_G_1302460834",
0x298bdc84,"MainField_Animal_Fish_G_1324939638",
0x595a5cce,"MainField_Animal_Fish_G_1368051905",
0x572667ea,"MainField_Animal_Fish_G_1371863734",
0xcde30371,"MainField_Animal_Fish_G_1413779661",
0xbe54db04,"MainField_Animal_Fish_G_1429489354",
0xcad04945,"MainField_Animal_Fish_G_1470754768",
0xd60f1e4c,"MainField_Animal_Fish_G_1510861341",
0x33ad9e81,"MainField_Animal_Fish_G_1519095417",
0x17c51a95,"MainField_Animal_Fish_G_1525736876",
0xaa5cec87,"MainField_Animal_Fish_G_1565070668",
0xe2924466,"MainField_Animal_Fish_G_1567545395",
0x80cde01e,"MainField_Animal_Fish_G_1653204673",
0x115541ec,"MainField_Animal_Fish_G_1660055632",
0x33d16ee6,"MainField_Animal_Fish_G_1753586512",
0x3912c2bf,"MainField_Animal_Fish_G_1765011073",
0x9efe9c36,"MainField_Animal_Fish_G_1766186945",
0x483ad9e0,"MainField_Animal_Fish_G_179829820",
0xfcd2ec88,"MainField_Animal_Fish_G_1843650505",
0x8a9623a8,"MainField_Animal_Fish_G_184443473",
0x18f99453,"MainField_Animal_Fish_G_1897908100",
0xc9bda52e,"MainField_Animal_Fish_G_1904013138",
0xd6489614,"MainField_Animal_Fish_G_197053477",
0x03ca6b66,"MainField_Animal_Fish_G_1977864910",
0xf985f17d,"MainField_Animal_Fish_G_2004207027",
0x1ee7ee19,"MainField_Animal_Fish_G_2008444785",
0x7cff5249,"MainField_Animal_Fish_G_2086759541",
0x48982d2c,"MainField_Animal_Fish_G_2089087673",
0x6ba5a1fc,"MainField_Animal_Fish_G_2092563635",
0xe460c618,"MainField_Animal_Fish_G_2108738730",
0x0830c50c,"MainField_Animal_Fish_G_2219819691",
0x9ff25c9c,"MainField_Animal_Fish_G_2229212936",
0xd49390e8,"MainField_Animal_Fish_G_2234633158",
0x4fdd760d,"MainField_Animal_Fish_G_226689836",
0x428e852d,"MainField_Animal_Fish_G_2330846085",
0x7cf4458e,"MainField_Animal_Fish_G_2391524230",
0x8821995b,"MainField_Animal_Fish_G_2397964796",
0x732e1a66,"MainField_Animal_Fish_G_2399007430",
0x573a9700,"MainField_Animal_Fish_G_2412316166",
0xdd4dff8c,"MainField_Animal_Fish_G_2429413556",
0x0c9d29c7,"MainField_Animal_Fish_G_2527586947",
0x56176265,"MainField_Animal_Fish_G_253845071",
0x5b2cefe1,"MainField_Animal_Fish_G_2546125759",
0xb1fd3bb5,"MainField_Animal_Fish_G_2584273325",
0xfab83639,"MainField_Animal_Fish_G_2600587700",
0x2329dd86,"MainField_Animal_Fish_G_2626541049",
0x29dd197a,"MainField_Animal_Fish_G_2743459176",
0x4594cef2,"MainField_Animal_Fish_G_2743625654",
0xa2b02b0e,"MainField_Animal_Fish_G_2767234479",
0x8fc5717a,"MainField_Animal_Fish_G_2778328607",
0x519f15e2,"MainField_Animal_Fish_G_2797484362",
0x0028003c,"MainField_Animal_Fish_G_2839598193",
0x825bb85b,"MainField_Animal_Fish_G_2885555340",
0xfce41cdd,"MainField_Animal_Fish_G_2896482670",
0x72ea83a9,"MainField_Animal_Fish_G_2908918521",
0x482cfdab,"MainField_Animal_Fish_G_2938820166",
0x794d0571,"MainField_Animal_Fish_G_2992251341",
0x4f60739a,"MainField_Animal_Fish_G_3106598920",
0x123e83c3,"MainField_Animal_Fish_G_3121803517",
0x3adb04fc,"MainField_Animal_Fish_G_3134649652",
0x8d87a14d,"MainField_Animal_Fish_G_3181982836",
0x13747354,"MainField_Animal_Fish_G_3182806810",
0xe07ad39d,"MainField_Animal_Fish_G_3276733501",
0xca8c517d,"MainField_Animal_Fish_G_3287295549",
0x67b1865d,"MainField_Animal_Fish_G_3361878675",
0x2d50a3d6,"MainField_Animal_Fish_G_3372390703",
0x4b58378c,"MainField_Animal_Fish_G_3399718213",
0x711b457a,"MainField_Animal_Fish_G_3400726412",
0xfce07439,"MainField_Animal_Fish_G_3453076898",
0x43b9c0d2,"MainField_Animal_Fish_G_3484372592",
0xdcc94ac7,"MainField_Animal_Fish_G_3491179295",
0xdd84a4d4,"MainField_Animal_Fish_G_3533627017",
0xe3b7502f,"MainField_Animal_Fish_G_3539943302",
0xd3aec350,"MainField_Animal_Fish_G_3565350398",
0x32d9ed5d,"MainField_Animal_Fish_G_3569865753",
0x641b397b,"MainField_Animal_Fish_G_361682519",
0x6ac63f59,"MainField_Animal_Fish_G_3626886864",
0x4f76f674,"MainField_Animal_Fish_G_3650940071",
0x11e7e83e,"MainField_Animal_Fish_G_3684950520",
0x42e6e452,"MainField_Animal_Fish_G_3686442698",
0x10b87b96,"MainField_Animal_Fish_G_3689022711",
0x20ed9720,"MainField_Animal_Fish_G_3729253607",
0x5a114a17,"MainField_Animal_Fish_G_3785415189",
0xdb4dc99d,"MainField_Animal_Fish_G_3804882150",
0xc7ac5e4d,"MainField_Animal_Fish_G_381915233",
0xb6744413,"MainField_Animal_Fish_G_3888830498",
0x08ef9555,"MainField_Animal_Fish_G_3926562415",
0x761e4ab3,"MainField_Animal_Fish_G_3959873826",
0x76608c4b,"MainField_Animal_Fish_G_3977236795",
0xd77d2f8d,"MainField_Animal_Fish_G_3987346276",
0xb47f06ff,"MainField_Animal_Fish_G_4017817046",
0x858d6874,"MainField_Animal_Fish_G_4063689552",
0x130c45f6,"MainField_Animal_Fish_G_4067619443",
0xbc1e5fbd,"MainField_Animal_Fish_G_4141073835",
0x394d8cdc,"MainField_Animal_Fish_G_4164762235",
0xad4cd105,"MainField_Animal_Fish_G_4179565233",
0xac47ee0f,"MainField_Animal_Fish_G_4186764048",
0xd72c527c,"MainField_Animal_Fish_G_4193960528",
0x016cc667,"MainField_Animal_Fish_G_4196268341",
0xf1e31fcc,"MainField_Animal_Fish_G_436434951",
0x385139ac,"MainField_Animal_Fish_G_459815570",
0x36cc0170,"MainField_Animal_Fish_G_506781207",
0x3ab25555,"MainField_Animal_Fish_G_507703127",
0x74e9a84e,"MainField_Animal_Fish_G_662790487",
0x7bfe6ae4,"MainField_Animal_Fish_G_672210943",
0x1d1bbc80,"MainField_Animal_Fish_G_711583909",
0x4a5320a9,"MainField_Animal_Fish_G_711733310",
0x083faab3,"MainField_Animal_Fish_G_730061688",
0xc26839a9,"MainField_Animal_Fish_G_7533861",
0x060b60ff,"MainField_Animal_Fish_G_755686027",
0x4342cd52,"MainField_Animal_Fish_G_779259564",
0x7af6a1de,"MainField_Animal_Fish_G_833927976",
0x69f6d227,"MainField_Animal_Fish_G_859488505",
0x74e2909d,"MainField_Animal_Fish_G_891745166",
0x09f28a19,"MainField_Animal_Fish_G_902774304",
0x3a246e5f,"MainField_Animal_Fish_G_91945150",
0xc82006bb,"MainField_Animal_Fish_G_969851662",
0x25cc60a5,"MainField_Animal_Fish_G_973064672",
0xc77bd33f,"MainField_Animal_Fish_H_1154465572",
0x960f6472,"MainField_Animal_Fish_H_1324860556",
0xd7c13406,"MainField_Animal_Fish_H_1415377439",
0x7ffe0ade,"MainField_Animal_Fish_H_1499837101",
0x210e71bc,"MainField_Animal_Fish_H_1759203861",
0x00f802be,"MainField_Animal_Fish_H_1819059081",
0xcfc4e869,"MainField_Animal_Fish_H_1877697145",
0x82e399cf,"MainField_Animal_Fish_H_1921773107",
0x5f3896a9,"MainField_Animal_Fish_H_1965227686",
0xfd42369d,"MainField_Animal_Fish_H_2056281814",
0xbe58367e,"MainField_Animal_Fish_H_2165755426",
0xff0de76b,"MainField_Animal_Fish_H_2174152281",
0x2210e430,"MainField_Animal_Fish_H_2179313404",
0x02060f4d,"MainField_Animal_Fish_H_2183376153",
0x78a31988,"MainField_Animal_Fish_H_2244428504",
0xf5c42cac,"MainField_Animal_Fish_H_2257937099",
0xca7dd4a0,"MainField_Animal_Fish_H_2354945727",
0xaba93094,"MainField_Animal_Fish_H_2658906272",
0x591edae5,"MainField_Animal_Fish_H_2764428230",
0xec820a0f,"MainField_Animal_Fish_H_2839657284",
0xf10b5cce,"MainField_Animal_Fish_H_2976460765",
0x6b4ff1c6,"MainField_Animal_Fish_H_3103094765",
0x5033c2ba,"MainField_Animal_Fish_H_3156695715",
0xaadf727b,"MainField_Animal_Fish_H_3239787788",
0x56e1c00e,"MainField_Animal_Fish_H_3310202891",
0x966e5263,"MainField_Animal_Fish_H_3579672119",
0x80a19281,"MainField_Animal_Fish_H_3642305933",
0x22b2cd5f,"MainField_Animal_Fish_H_3667816797",
0xc2cc5ccb,"MainField_Animal_Fish_H_3718790823",
0xbf0a10ec,"MainField_Animal_Fish_H_3752395783",
0xf0161c49,"MainField_Animal_Fish_H_385508731",
0xdc86f037,"MainField_Animal_Fish_H_3952220201",
0x59f39b69,"MainField_Animal_Fish_H_4041989114",
0x9d1a9488,"MainField_Animal_Fish_H_4048950280",
0xd08ff20a,"MainField_Animal_Fish_H_4112317533",
0x0916c9cc,"MainField_Animal_Fish_H_4142723101",
0x3982f23a,"MainField_Animal_Fish_H_4150085515",
0xf65f9775,"MainField_Animal_Fish_H_62638918",
0x846ba147,"MainField_Animal_Fish_H_831509989",
0x997eeabb,"MainField_Animal_Fish_I_1277938723",
0xb2f26a4e,"MainField_Animal_Fish_I_1735202007",
0x81bb2cff,"MainField_Animal_Fish_I_1767336501",
0x1ae6de9d,"MainField_Animal_Fish_I_177293826",
0x99908d95,"MainField_Animal_Fish_I_2157723406",
0x0169057a,"MainField_Animal_Fish_I_2179083928",
0x546b8c58,"MainField_Animal_Fish_I_3014468380",
0x0fb016f5,"MainField_Animal_Fish_I_4154134424",
0xd5822ea6,"MainField_Animal_Fish_I_4242611961",
0xa6d3528e,"MainField_Animal_Fish_I_565666897",
0xe5ae765f,"MainField_Animal_Fish_I_696478521",
0x1fbcf08b,"MainField_Animal_Fish_K_1083572130",
0xb565171b,"MainField_Animal_Fish_K_1219533235",
0x34b4c235,"MainField_Animal_Fish_K_1369942566",
0x9d977d91,"MainField_Animal_Fish_K_1446750448",
0xcf939351,"MainField_Animal_Fish_K_1488175776",
0xc1d829cb,"MainField_Animal_Fish_K_157214162",
0x5ad6a531,"MainField_Animal_Fish_K_1727727298",
0x869c8348,"MainField_Animal_Fish_K_2218948124",
0x437c60e6,"MainField_Animal_Fish_K_24127973",
0xc797a4dc,"MainField_Animal_Fish_K_2549448521",
0xe1cd817e,"MainField_Animal_Fish_K_2613280915",
0x34baa278,"MainField_Animal_Fish_K_2706079917",
0xb116382d,"MainField_Animal_Fish_K_286749176",
0x46fc0e42,"MainField_Animal_Fish_K_3318976023",
0xf2ac7d8c,"MainField_Animal_Fish_K_3518778885",
0x20295270,"MainField_Animal_Fish_K_3791901247",
0xea4f485c,"MainField_Animal_Fish_K_3950474524",
0x8d368beb,"MainField_Animal_Fish_K_639540768",
0x55aafaee,"MainField_Animal_Fish_K_883603294",
0xdd228da1,"MainField_Animal_Fish_L_1068919795",
0x97d40519,"MainField_Animal_Fish_L_1147540742",
0x87d7aa53,"MainField_Animal_Fish_L_1346148570",
0xac995424,"MainField_Animal_Fish_L_1386595772",
0x1ceec4b9,"MainField_Animal_Fish_L_1397555837",
0x89803ff3,"MainField_Animal_Fish_L_1510222254",
0x5f0270b6,"MainField_Animal_Fish_L_1663026919",
0x4b84112f,"MainField_Animal_Fish_L_1712522349",
0x9bbf7498,"MainField_Animal_Fish_L_1714758292",
0xc0e03396,"MainField_Animal_Fish_L_1716866495",
0x5fa30b7c,"MainField_Animal_Fish_L_1717805559",
0x01170aed,"MainField_Animal_Fish_L_1755034435",
0x57d664c6,"MainField_Animal_Fish_L_1888626819",
0x18aa56f9,"MainField_Animal_Fish_L_1900587827",
0x3783a068,"MainField_Animal_Fish_L_2086103568",
0x59980398,"MainField_Animal_Fish_L_2107946150",
0xb28d2ebc,"MainField_Animal_Fish_L_2172219279",
0xdd72cac3,"MainField_Animal_Fish_L_2180170244",
0x410f0a00,"MainField_Animal_Fish_L_2187715160",
0x55df5b9b,"MainField_Animal_Fish_L_2225890042",
0x318e5d26,"MainField_Animal_Fish_L_2247577086",
0xe8642b38,"MainField_Animal_Fish_L_2258543051",
0xe9be54b2,"MainField_Animal_Fish_L_2274087452",
0x89154fca,"MainField_Animal_Fish_L_2315636203",
0x337a015b,"MainField_Animal_Fish_L_237654216",
0xfd18522c,"MainField_Animal_Fish_L_2413381931",
0x8742c76e,"MainField_Animal_Fish_L_242683450",
0x5f2e902e,"MainField_Animal_Fish_L_2468301584",
0x9b4f9cce,"MainField_Animal_Fish_L_2549529803",
0x2c6e05b4,"MainField_Animal_Fish_L_2571613779",
0x3ccf97f5,"MainField_Animal_Fish_L_2621174535",
0x4aa57c43,"MainField_Animal_Fish_L_2766606354",
0x5d5a4dd3,"MainField_Animal_Fish_L_2768320975",
0x08bdd0f8,"MainField_Animal_Fish_L_2881280837",
0xf9b2941a,"MainField_Animal_Fish_L_2894349595",
0x755d6876,"MainField_Animal_Fish_L_3008362859",
0x1e4ccd45,"MainField_Animal_Fish_L_3009147980",
0xc5e94e20,"MainField_Animal_Fish_L_3103449114",
0x225af676,"MainField_Animal_Fish_L_3109916005",
0x7788acc9,"MainField_Animal_Fish_L_3308812402",
0xbd35449b,"MainField_Animal_Fish_L_3496176069",
0x7c8d521e,"MainField_Animal_Fish_L_3672395863",
0xc9c22ecd,"MainField_Animal_Fish_L_3835897796",
0x5df6945b,"MainField_Animal_Fish_L_3844903693",
0x40fc4b2e,"MainField_Animal_Fish_L_3861496736",
0xb946dff5,"MainField_Animal_Fish_L_3899297781",
0x6488968e,"MainField_Animal_Fish_L_3932265267",
0xba4c3b1e,"MainField_Animal_Fish_L_3960423667",
0x230ad639,"MainField_Animal_Fish_L_4086785613",
0xe09e283f,"MainField_Animal_Fish_L_450570386",
0x2dcea430,"MainField_Animal_Fish_L_469121014",
0x5a3d92e3,"MainField_Animal_Fish_L_542222123",
0x068a3d4a,"MainField_Animal_Fish_L_630280033",
0x07fe36df,"MainField_Animal_Fish_L_679476794",
0x80205f6a,"MainField_Animal_Fish_L_686773763",
0x97f9aa91,"MainField_Animal_Fish_L_726846798",
0x15dbeb01,"MainField_Animal_Fish_L_780516726",
0x8fa4d16e,"MainField_Animal_Fish_L_798062094",
0x428e0bef,"MainField_Animal_Fish_L_844945068",
0xa9cacae2,"MainField_Animal_Fish_L_963676017",
0xcab2f39a,"MainField_Animal_Fish_L_972847670",
0xb57037ef,"MainField_Animal_Fish_M_1006979593",
0xe6823698,"MainField_Animal_Fish_M_1062208878",
0x5c0e999b,"MainField_Animal_Fish_M_1133689114",
0x1b05ba66,"MainField_Animal_Fish_M_1196147776",
0x37b0b39b,"MainField_Animal_Fish_M_1560178001",
0x3f67c5af,"MainField_Animal_Fish_M_1723476935",
0xd7af5256,"MainField_Animal_Fish_M_1725219977",
0xe26b03f7,"MainField_Animal_Fish_M_1913583089",
0xf40b344c,"MainField_Animal_Fish_M_1975710339",
0x387a38a4,"MainField_Animal_Fish_M_2122023804",
0xe7daddb6,"MainField_Animal_Fish_M_2136458645",
0x13d97dba,"MainField_Animal_Fish_M_2140026121",
0x9b4eb4fb,"MainField_Animal_Fish_M_218969048",
0x7f0b32a5,"MainField_Animal_Fish_M_2265162981",
0x17b28836,"MainField_Animal_Fish_M_2377543363",
0xaeece6ae,"MainField_Animal_Fish_M_2446803501",
0xcef5fb8b,"MainField_Animal_Fish_M_2652731714",
0x3e114a11,"MainField_Animal_Fish_M_2865342139",
0x4a9edbc1,"MainField_Animal_Fish_M_2907960814",
0x019f1345,"MainField_Animal_Fish_M_3084559985",
0x75f89ffb,"MainField_Animal_Fish_M_3289828528",
0x60a4090e,"MainField_Animal_Fish_M_3337526411",
0x1dfe8e03,"MainField_Animal_Fish_M_3343112162",
0x71fd414c,"MainField_Animal_Fish_M_3353296738",
0x64eed257,"MainField_Animal_Fish_M_3357309635",
0x3ec79f08,"MainField_Animal_Fish_M_3715740235",
0x037e4ebb,"MainField_Animal_Fish_M_3778502573",
0x95bb8248,"MainField_Animal_Fish_M_3903244274",
0xb2e7dc1b,"MainField_Animal_Fish_M_393753256",
0xf3cd7fe6,"MainField_Animal_Fish_M_3979907255",
0xbd40b080,"MainField_Animal_Fish_M_4038103629",
0xff465e97,"MainField_Animal_Fish_M_404425558",
0xe5affe44,"MainField_Animal_Fish_M_4242409223",
0xc8d1ed92,"MainField_Animal_Fish_M_463492023",
0xb70f5658,"MainField_Animal_Fish_M_631948008",
0xab1197f2,"MainField_Animal_Fish_M_740311366",
0xfe64da09,"MainField_Animal_Fish_M_812059331",
0xce40c6cd,"MainField_Animal_Fish_M_836481654",
0x67b5d486,"MainField_Animal_Fish_M_843955121",
0xaa087b9e,"MainField_Animal_Fish_M_863066185",
0x0896e9ae,"MainField_Animal_Fish_M_94444751",
0x299fb28e,"MainField_Animal_Fish_M_981734594",
0x0d356b78,"MainField_Animal_Fish_Z_1260525016",
0x80ea3919,"MainField_Animal_Fish_Z_1427926938",
0x9bbbe799,"MainField_Animal_Fish_Z_1530842053",
0x9905263a,"MainField_Animal_Fish_Z_2216218729",
0x78b04e14,"MainField_Animal_Fish_Z_3717747548",
0x1a27e7e2,"MainField_Animal_Fish_Z_524712014",
0x1f355974,"MainField_Animal_Fox_A_1030849684",
0xb0938c36,"MainField_Animal_Fox_A_1071327521",
0x6110d0c3,"MainField_Animal_Fox_A_122564398",
0x531ea340,"MainField_Animal_Fox_A_1266965254",
0x90b8d846,"MainField_Animal_Fox_A_1310352485",
0x9c430343,"MainField_Animal_Fox_A_1379879338",
0x1ce27264,"MainField_Animal_Fox_A_1403335062",
0x640f0e38,"MainField_Animal_Fox_A_1435020491",
0xb4aee307,"MainField_Animal_Fox_A_1534030447",
0x390bae08,"MainField_Animal_Fox_A_1839145972",
0xdaf80fad,"MainField_Animal_Fox_A_204820085",
0x954d197a,"MainField_Animal_Fox_A_2126844802",
0xd4eb76e9,"MainField_Animal_Fox_A_2516124126",
0xb054a05b,"MainField_Animal_Fox_A_2613454996",
0x4842d297,"MainField_Animal_Fox_A_2733340420",
0x82e2b04c,"MainField_Animal_Fox_A_2920307691",
0xf33d7022,"MainField_Animal_Fox_A_2975896741",
0xb6dbb84a,"MainField_Animal_Fox_A_3204542960",
0x53a72ecd,"MainField_Animal_Fox_A_3306799568",
0xc300e547,"MainField_Animal_Fox_A_337081810",
0x95790eb4,"MainField_Animal_Fox_A_3518671636",
0xce4e1622,"MainField_Animal_Fox_A_3566565612",
0x2f52eaf0,"MainField_Animal_Fox_A_3723135185",
0xd8098fb9,"MainField_Animal_Fox_A_3775409096",
0x89bd0efe,"MainField_Animal_Fox_A_3831867977",
0x1a2e51da,"MainField_Animal_Fox_A_386825941",
0x426848c5,"MainField_Animal_Fox_A_3887193504",
0x3e7bed75,"MainField_Animal_Fox_A_3993127527",
0x12ee6af0,"MainField_Animal_Fox_A_4077959803",
0x5b2330dd,"MainField_Animal_Fox_A_4131466442",
0xa0b762bd,"MainField_Animal_Fox_A_545415846",
0x52455e93,"MainField_Animal_Fox_A_847094851",
0x414ff5cd,"MainField_Animal_Fox_A_859442780",
0xcb224c86,"MainField_Animal_Fox_A_964198750",
0x81e53e04,"MainField_Animal_Fox_B_1068612709",
0x85410948,"MainField_Animal_Fox_B_1097667277",
0x5645b99f,"MainField_Animal_Fox_B_1148419848",
0x04cdfbee,"MainField_Animal_Fox_B_1191461474",
0x5363476b,"MainField_Animal_Fox_B_1430447371",
0xf6b19091,"MainField_Animal_Fox_B_1460228073",
0xb78af53d,"MainField_Animal_Fox_B_1498085917",
0x82136fc1,"MainField_Animal_Fox_B_1607079437",
0x9acd50bd,"MainField_Animal_Fox_B_1673601947",
0xb550d807,"MainField_Animal_Fox_B_1735839152",
0x626d394e,"MainField_Animal_Fox_B_1939176336",
0x557d396e,"MainField_Animal_Fox_B_20852099",
0xe8dd631e,"MainField_Animal_Fox_B_2504645653",
0xa8f6cac1,"MainField_Animal_Fox_B_2540937728",
0x8f4fded5,"MainField_Animal_Fox_B_2847281718",
0x10a79166,"MainField_Animal_Fox_B_2940021235",
0xa74f351d,"MainField_Animal_Fox_B_2963965488",
0x22a109d3,"MainField_Animal_Fox_B_3258744535",
0xfe66ab61,"MainField_Animal_Fox_B_3331347429",
0x8ad9103f,"MainField_Animal_Fox_B_3428935266",
0x1fd8e799,"MainField_Animal_Fox_B_3481892892",
0xbd533886,"MainField_Animal_Fox_B_3590179491",
0x884766a8,"MainField_Animal_Fox_B_3753502261",
0x6bcc0503,"MainField_Animal_Fox_B_3793161656",
0xe3025b59,"MainField_Animal_Fox_B_3793634443",
0xe46bb079,"MainField_Animal_Fox_B_3880796847",
0x0b99f99c,"MainField_Animal_Fox_B_4015654023",
0x4a339516,"MainField_Animal_Fox_B_4155810979",
0x368ca65a,"MainField_Animal_Fox_B_588994642",
0x9dcaa76e,"MainField_Animal_Fox_B_664418351",
0x2d6ae884,"MainField_Animal_Fox_B_707332183",
0xb6b9ae1e,"MainField_Animal_Gull_A_1000101350",
0x1c71020e,"MainField_Animal_Gull_A_1026078016",
0xe1dbce3a,"MainField_Animal_Gull_A_1071389028",
0x3e6d8f1d,"MainField_Animal_Gull_A_1096077194",
0x9714ba5f,"MainField_Animal_Gull_A_1121819158",
0xa3fa1bd2,"MainField_Animal_Gull_A_1152748143",
0x07984218,"MainField_Animal_Gull_A_1216203044",
0xbad4f155,"MainField_Animal_Gull_A_1245297686",
0xcaf36e01,"MainField_Animal_Gull_A_1274877219",
0x14af0558,"MainField_Animal_Gull_A_132454915",
0x1422105e,"MainField_Animal_Gull_A_1379731602",
0xe50fe197,"MainField_Animal_Gull_A_1380912159",
0xc69f2b36,"MainField_Animal_Gull_A_1428831125",
0x896cde77,"MainField_Animal_Gull_A_1449364247",
0xb8740ee6,"MainField_Animal_Gull_A_1462584902",
0x172ebf99,"MainField_Animal_Gull_A_1476120865",
0xbe859d76,"MainField_Animal_Gull_A_1484658276",
0x5107a147,"MainField_Animal_Gull_A_1541457774",
0x163def34,"MainField_Animal_Gull_A_1604038991",
0x659c1bf9,"MainField_Animal_Gull_A_162244789",
0xefd536fd,"MainField_Animal_Gull_A_1654478909",
0x90eb64fd,"MainField_Animal_Gull_A_1689410581",
0xaeb1d738,"MainField_Animal_Gull_A_1696862290",
0xa6c81d87,"MainField_Animal_Gull_A_1697009463",
0x23c30799,"MainField_Animal_Gull_A_1699768971",
0xaf09a172,"MainField_Animal_Gull_A_1772633919",
0x5f967090,"MainField_Animal_Gull_A_1789371029",
0xa3670821,"MainField_Animal_Gull_A_1802140708",
0x2fa9e116,"MainField_Animal_Gull_A_1802537928",
0x0a45653d,"MainField_Animal_Gull_A_1810097160",
0x04d5e5e9,"MainField_Animal_Gull_A_1833927054",
0x5d3b765c,"MainField_Animal_Gull_A_1894863867",
0xd10ecba6,"MainField_Animal_Gull_A_1977552792",
0x80c34a95,"MainField_Animal_Gull_A_2017038135",
0x079a19be,"MainField_Animal_Gull_A_2086122043",
0xb9ef9e91,"MainField_Animal_Gull_A_2195791845",
0x25b4bf6e,"MainField_Animal_Gull_A_2198396902",
0xc54febb7,"MainField_Animal_Gull_A_2214362482",
0xe21310b9,"MainField_Animal_Gull_A_2236668302",
0x4451ca78,"MainField_Animal_Gull_A_2251997540",
0x7de693b9,"MainField_Animal_Gull_A_2278616716",
0x1602d9f5,"MainField_Animal_Gull_A_2351623986",
0x78af8a46,"MainField_Animal_Gull_A_2375444976",
0x10b50139,"MainField_Animal_Gull_A_2404242047",
0x240bfd0a,"MainField_Animal_Gull_A_2433683219",
0x88096828,"MainField_Animal_Gull_A_2437301116",
0xbccbada4,"MainField_Animal_Gull_A_2468519683",
0xaa401d0c,"MainField_Animal_Gull_A_249776462",
0xd77d70be,"MainField_Animal_Gull_A_254160072",
0xa85fca31,"MainField_Animal_Gull_A_2581897781",
0xa7263856,"MainField_Animal_Gull_A_2685995414",
0xc31d594c,"MainField_Animal_Gull_A_2698867928",
0x4f0eb6b2,"MainField_Animal_Gull_A_2699232974",
0x7ded07ae,"MainField_Animal_Gull_A_2713953093",
0xc28b4629,"MainField_Animal_Gull_A_2727112269",
0x5e6f9fc9,"MainField_Animal_Gull_A_2729504219",
0xc8fdd61b,"MainField_Animal_Gull_A_2749705880",
0x2eb0e11f,"MainField_Animal_Gull_A_2787475849",
0x279a71a6,"MainField_Animal_Gull_A_2794057699",
0x440b1763,"MainField_Animal_Gull_A_2838126413",
0xdac18673,"MainField_Animal_Gull_A_2883788924",
0x5c67237b,"MainField_Animal_Gull_A_2941623392",
0x47f34733,"MainField_Animal_Gull_A_3010752129",
0x1c06bb35,"MainField_Animal_Gull_A_3042094928",
0xc2855378,"MainField_Animal_Gull_A_3042397254",
0xcd8ce16f,"MainField_Animal_Gull_A_3115990177",
0x775e1469,"MainField_Animal_Gull_A_3137070338",
0xd7af234a,"MainField_Animal_Gull_A_3149473783",
0x5a59f3fb,"MainField_Animal_Gull_A_3223735926",
0x29852d42,"MainField_Animal_Gull_A_3244072090",
0x900fc4cc,"MainField_Animal_Gull_A_3246576186",
0xd99d82ea,"MainField_Animal_Gull_A_3297191606",
0x1f261d61,"MainField_Animal_Gull_A_3325808491",
0x8891f4cd,"MainField_Animal_Gull_A_3330556292",
0x57b68561,"MainField_Animal_Gull_A_3482434395",
0x5df2cb1b,"MainField_Animal_Gull_A_3497883123",
0xa5d5648a,"MainField_Animal_Gull_A_3506192568",
0xbcc79616,"MainField_Animal_Gull_A_3545665016",
0xf5783168,"MainField_Animal_Gull_A_3553976067",
0x32b822f7,"MainField_Animal_Gull_A_3582932229",
0x79525e7e,"MainField_Animal_Gull_A_3594884279",
0x68fcb644,"MainField_Animal_Gull_A_3643292047",
0xc4ee8380,"MainField_Animal_Gull_A_3688809750",
0x4782f725,"MainField_Animal_Gull_A_3694962219",
0x6a5c7aef,"MainField_Animal_Gull_A_3713252144",
0x39ed5d50,"MainField_Animal_Gull_A_3723214919",
0x4de01d8b,"MainField_Animal_Gull_A_3723555821",
0x0a23066b,"MainField_Animal_Gull_A_3737041490",
0xbae1cbb5,"MainField_Animal_Gull_A_3748764380",
0x791de729,"MainField_Animal_Gull_A_3753658759",
0xdbe5b005,"MainField_Animal_Gull_A_3806380708",
0xcb5d093c,"MainField_Animal_Gull_A_3821687487",
0x78b04fd0,"MainField_Animal_Gull_A_3935960014",
0xfdd8ebb9,"MainField_Animal_Gull_A_3939358350",
0x247df57f,"MainField_Animal_Gull_A_3966075118",
0xee36d4e7,"MainField_Animal_Gull_A_4011522887",
0xf22a50e4,"MainField_Animal_Gull_A_4082279348",
0x9a7d3ee2,"MainField_Animal_Gull_A_4129006934",
0x51c3b20e,"MainField_Animal_Gull_A_415956347",
0xdf0ee7c9,"MainField_Animal_Gull_A_4182190118",
0x5993cbb8,"MainField_Animal_Gull_A_4270187993",
0x8c231226,"MainField_Animal_Gull_A_440844219",
0x39bd953e,"MainField_Animal_Gull_A_447928049",
0x6b68280b,"MainField_Animal_Gull_A_450798151",
0xf448a552,"MainField_Animal_Gull_A_481539741",
0x2cacf29f,"MainField_Animal_Gull_A_528588718",
0x0df5970e,"MainField_Animal_Gull_A_533764425",
0xc1ad43b3,"MainField_Animal_Gull_A_582057371",
0x5b073aa5,"MainField_Animal_Gull_A_608615815",
0x83c48817,"MainField_Animal_Gull_A_852249313",
0x7ada897f,"MainField_Animal_Gull_A_908279736",
0x5c6e6a27,"MainField_Animal_Gull_A_917303780",
0xfcf0a1cf,"MainField_Animal_Gull_A_964261466",
0x41ea7d0f,"MainField_Animal_Gull_A_970855935",
0x5297ba9f,"MainField_Animal_Gull_Swarm_1126757121",
0xd3058afb,"MainField_Animal_Gull_Swarm_1452133191",
0xb462c5c3,"MainField_Animal_Gull_Swarm_2283885645",
0x1d787f8d,"MainField_Animal_Gull_Swarm_2587655306",
0xd9dfd268,"MainField_Animal_Gull_Swarm_2760957446",
0x5190c9ea,"MainField_Animal_Gull_Swarm_3549949584",
0x5535e0f6,"MainField_Animal_Gull_Swarm_3925726980",
0xca96f17a,"MainField_Animal_Gull_Swarm_875545495",
0x94bbf824,"MainField_Animal_Gull_Swarm_899872825",
0x37a98225,"MainField_Animal_Hawk_A_1048272619",
0x5395ad3c,"MainField_Animal_Hawk_A_1230086968",
0x28f3e360,"MainField_Animal_Hawk_A_1233163331",
0x9739cb1e,"MainField_Animal_Hawk_A_1241044339",
0x0675f983,"MainField_Animal_Hawk_A_1242401199",
0x30aa1866,"MainField_Animal_Hawk_A_1243473061",
0x52757597,"MainField_Animal_Hawk_A_1298212296",
0x9ac30f26,"MainField_Animal_Hawk_A_1306532836",
0x95d81d19,"MainField_Animal_Hawk_A_1341998594",
0x42b9f7da,"MainField_Animal_Hawk_A_1525998874",
0x74a7c6f5,"MainField_Animal_Hawk_A_1584806119",
0x3c0c5b72,"MainField_Animal_Hawk_A_1587622577",
0xc7c31071,"MainField_Animal_Hawk_A_1601328607",
0x69281a6f,"MainField_Animal_Hawk_A_1732288214",
0x690adcc8,"MainField_Animal_Hawk_A_1883434822",
0x216d10ac,"MainField_Animal_Hawk_A_196341024",
0x05c9879f,"MainField_Animal_Hawk_A_200735316",
0xa32d9bd9,"MainField_Animal_Hawk_A_2047879707",
0x64c334a6,"MainField_Animal_Hawk_A_2118588850",
0x77832e7a,"MainField_Animal_Hawk_A_2192287660",
0xef29eeb9,"MainField_Animal_Hawk_A_2200450067",
0xf39b99f9,"MainField_Animal_Hawk_A_2232270119",
0xc061ddf1,"MainField_Animal_Hawk_A_2392769132",
0xa176066c,"MainField_Animal_Hawk_A_2397198549",
0x898281c6,"MainField_Animal_Hawk_A_2578986152",
0xb33507f4,"MainField_Animal_Hawk_A_2650891142",
0x551c5c41,"MainField_Animal_Hawk_A_2765900622",
0x6821a7c1,"MainField_Animal_Hawk_A_2877546867",
0xc5c57638,"MainField_Animal_Hawk_A_3144808192",
0x5fccf135,"MainField_Animal_Hawk_A_3222694829",
0x2e932a06,"MainField_Animal_Hawk_A_3233242918",
0xfced1ede,"MainField_Animal_Hawk_A_3257644221",
0x74b615fc,"MainField_Animal_Hawk_A_3442387923",
0x367084b5,"MainField_Animal_Hawk_A_347719191",
0xadcd20c7,"MainField_Animal_Hawk_A_3578433276",
0x09002a5f,"MainField_Animal_Hawk_A_3583445370",
0x9c3550c3,"MainField_Animal_Hawk_A_3583488506",
0x842a0a1b,"MainField_Animal_Hawk_A_3677587005",
0x7f77cbea,"MainField_Animal_Hawk_A_370127373",
0xd283438c,"MainField_Animal_Hawk_A_3712434555",
0xf9fdc71b,"MainField_Animal_Hawk_A_3777471505",
0x3152d020,"MainField_Animal_Hawk_A_3802771081",
0xe81c344d,"MainField_Animal_Hawk_A_389101587",
0xa44d448d,"MainField_Animal_Hawk_A_4128281587",
0xa110bac9,"MainField_Animal_Hawk_A_4206046711",
0xa37f7fc6,"MainField_Animal_Hawk_A_4255623250",
0x30178c73,"MainField_Animal_Hawk_A_4273047845",
0xc75e1eb6,"MainField_Animal_Hawk_A_4277359421",
0x00220845,"MainField_Animal_Hawk_A_438361153",
0xadef4738,"MainField_Animal_Hawk_A_576939338",
0xbdf8ac1a,"MainField_Animal_Hawk_A_648148871",
0x54054201,"MainField_Animal_Hawk_A_666265136",
0xb783449f,"MainField_Animal_Hawk_A_706735248",
0x113bdf49,"MainField_Animal_Hawk_A_922233536",
0xf3f0a89e,"MainField_Animal_Heron_A_1115562888",
0x406c6fb2,"MainField_Animal_Heron_A_1204471680",
0xab0dc902,"MainField_Animal_Heron_A_1283700366",
0x537e92bc,"MainField_Animal_Heron_A_1312601507",
0x638a8765,"MainField_Animal_Heron_A_131924767",
0x0a7a5857,"MainField_Animal_Heron_A_1476745461",
0x025628c1,"MainField_Animal_Heron_A_1547442859",
0xd6fbe595,"MainField_Animal_Heron_A_1579179588",
0x973653d1,"MainField_Animal_Heron_A_1623627740",
0x757a0820,"MainField_Animal_Heron_A_1677737167",
0xd4ecdf16,"MainField_Animal_Heron_A_1853870913",
0xa26660de,"MainField_Animal_Heron_A_185785448",
0xa9617a64,"MainField_Animal_Heron_A_1967356527",
0x9e05c53b,"MainField_Animal_Heron_A_2111074712",
0x8550784c,"MainField_Animal_Heron_A_2162889978",
0x823ed888,"MainField_Animal_Heron_A_2228352145",
0xfd44f909,"MainField_Animal_Heron_A_2427353968",
0x46524036,"MainField_Animal_Heron_A_2436238318",
0xfe3410d8,"MainField_Animal_Heron_A_2446466168",
0x26b7e536,"MainField_Animal_Heron_A_2652430209",
0xcaee1000,"MainField_Animal_Heron_A_2805889972",
0xbdc4c526,"MainField_Animal_Heron_A_2918839441",
0xa37eefc3,"MainField_Animal_Heron_A_2932439303",
0x3e81b56e,"MainField_Animal_Heron_A_3146576380",
0xc6810eef,"MainField_Animal_Heron_A_3338603829",
0xb204f629,"MainField_Animal_Heron_A_3550417074",
0x00e78a1d,"MainField_Animal_Heron_A_3681510223",
0x935a72b2,"MainField_Animal_Heron_A_3729740959",
0x1a8dfb99,"MainField_Animal_Heron_A_3995583129",
0x5194004c,"MainField_Animal_Heron_A_4004587801",
0xbfd4b7e5,"MainField_Animal_Heron_A_4027779663",
0x352b265d,"MainField_Animal_Heron_A_4074489028",
0xcc0fbe4e,"MainField_Animal_Heron_A_4082291487",
0x2e03e7b7,"MainField_Animal_Heron_A_4214907776",
0xf319c20e,"MainField_Animal_Heron_A_4257237906",
0x5f006d5f,"MainField_Animal_Heron_A_514086550",
0x85319b9f,"MainField_Animal_Heron_A_541501874",
0xab5da25e,"MainField_Animal_Heron_A_569495339",
0x0897da2c,"MainField_Animal_Heron_A_755258558",
0xddfcf26b,"MainField_Animal_Heron_A_820288626",
0x94c0cc72,"MainField_Animal_Heron_A_87367823",
0x50150d8d,"MainField_Animal_Heron_B_118298806",
0x73d618b3,"MainField_Animal_Heron_B_1247644404",
0xadcea445,"MainField_Animal_Heron_B_1293321731",
0xdea33eca,"MainField_Animal_Heron_B_1504338394",
0xaca454a8,"MainField_Animal_Heron_B_156832821",
0x9d19d1e3,"MainField_Animal_Heron_B_1622734331",
0xb885fae2,"MainField_Animal_Heron_B_1756177776",
0x44723cd9,"MainField_Animal_Heron_B_1831469255",
0xef26114a,"MainField_Animal_Heron_B_2040748841",
0x6c01f376,"MainField_Animal_Heron_B_2058889857",
0x156335cb,"MainField_Animal_Heron_B_2197042426",
0x4e355f0b,"MainField_Animal_Heron_B_2245826331",
0x9a34623a,"MainField_Animal_Heron_B_2259797462",
0xbceb770e,"MainField_Animal_Heron_B_2326474731",
0x78cc50be,"MainField_Animal_Heron_B_2465622957",
0x0eb76e81,"MainField_Animal_Heron_B_2514182101",
0x05b12f7f,"MainField_Animal_Heron_B_2627331390",
0xc489b5d6,"MainField_Animal_Heron_B_2639694791",
0xaec356b4,"MainField_Animal_Heron_B_2642118495",
0xa24b9da7,"MainField_Animal_Heron_B_2725103387",
0x5b322cd6,"MainField_Animal_Heron_B_2785549796",
0xdad78539,"MainField_Animal_Heron_B_2839992476",
0x32f60844,"MainField_Animal_Heron_B_2854739193",
0xabf3d20f,"MainField_Animal_Heron_B_2905726429",
0x6fc632df,"MainField_Animal_Heron_B_2929811238",
0x350a5d04,"MainField_Animal_Heron_B_2946624359",
0xb46b7297,"MainField_Animal_Heron_B_298102208",
0x5a250e20,"MainField_Animal_Heron_B_303581587",
0x7ba451f4,"MainField_Animal_Heron_B_3060319480",
0x4f75286b,"MainField_Animal_Heron_B_3079374098",
0x59e95baf,"MainField_Animal_Heron_B_3203599673",
0xa0d0cc34,"MainField_Animal_Heron_B_3241640296",
0xb61a211c,"MainField_Animal_Heron_B_3293311759",
0xb5ed78f6,"MainField_Animal_Heron_B_3351842544",
0xa9499563,"MainField_Animal_Heron_B_3449818678",
0x5f037ed6,"MainField_Animal_Heron_B_3475396325",
0x777d6f97,"MainField_Animal_Heron_B_3480826816",
0xba6c278c,"MainField_Animal_Heron_B_3585763021",
0xc895e68d,"MainField_Animal_Heron_B_3688065564",
0x93986d7a,"MainField_Animal_Heron_B_3704975541",
0xad544f29,"MainField_Animal_Heron_B_3780327084",
0x1242102b,"MainField_Animal_Heron_B_3964233565",
0xf820ddbf,"MainField_Animal_Heron_B_3981418516",
0xc68335e2,"MainField_Animal_Heron_B_4032291501",
0xca2758bb,"MainField_Animal_Heron_B_4070618062",
0x8131be04,"MainField_Animal_Heron_B_428275889",
0x1e56bdff,"MainField_Animal_Heron_B_435364136",
0x9eca4e51,"MainField_Animal_Heron_B_597963415",
0xca28c995,"MainField_Animal_Heron_B_624143812",
0x9e74dafa,"MainField_Animal_Heron_B_711952108",
0xbf399a32,"MainField_Animal_Heron_B_714619895",
0x9e6d70dc,"MainField_Animal_Heron_B_762161442",
0x3cf4601f,"MainField_Animal_Heron_B_873961101",
0x93649184,"MainField_Animal_Heron_B_936221413",
0xcafca5f7,"MainField_Animal_Heron_B_947467201",
0xd8c492cc,"MainField_Animal_Insect_AA_1010898802",
0xf3449a9c,"MainField_Animal_Insect_AA_1051613379",
0x1dedd6fd,"MainField_Animal_Insect_AA_1097998743",
0x4c213cda,"MainField_Animal_Insect_AA_1114823324",
0x789be6cc,"MainField_Animal_Insect_AA_1869291062",
0x5573b051,"MainField_Animal_Insect_AA_189282886",
0x46fec3a4,"MainField_Animal_Insect_AA_347372435",
0xb65f95cf,"MainField_Animal_Insect_AA_3607307480",
0xae76abbd,"MainField_Animal_Insect_AA_385499100",
0xed6d58d0,"MainField_Animal_Insect_AA_3906282482",
0xfad18171,"MainField_Animal_Insect_AA_4254133732",
0xfcaf17c8,"MainField_Animal_Insect_AA_525703439",
0xd30008ea,"MainField_Animal_Insect_AA_664074760",
0x705c6da3,"MainField_Animal_Insect_A_1023997309",
0x2b0741b1,"MainField_Animal_Insect_A_1054502676",
0xbc8ca6bc,"MainField_Animal_Insect_A_1065999981",
0xc874a506,"MainField_Animal_Insect_A_1073140300",
0x3b0e43e0,"MainField_Animal_Insect_A_1113544379",
0x09e96960,"MainField_Animal_Insect_A_1139423728",
0x79fe8634,"MainField_Animal_Insect_A_1197069242",
0x5539e53f,"MainField_Animal_Insect_A_1377741696",
0xda63df5f,"MainField_Animal_Insect_A_144828913",
0x872f5c36,"MainField_Animal_Insect_A_1591794138",
0x42793532,"MainField_Animal_Insect_A_1626407326",
0xf00ec10a,"MainField_Animal_Insect_A_1655124348",
0xaac581c5,"MainField_Animal_Insect_A_1669526393",
0x59695c93,"MainField_Animal_Insect_A_1675021350",
0x1ec1af21,"MainField_Animal_Insect_A_1678780862",
0xb698298b,"MainField_Animal_Insect_A_1684683722",
0xe6606939,"MainField_Animal_Insect_A_1702878078",
0x5d35cee4,"MainField_Animal_Insect_A_1712296888",
0xbe2b2a62,"MainField_Animal_Insect_A_1720040295",
0x922c864b,"MainField_Animal_Insect_A_1761088027",
0x4eab24a7,"MainField_Animal_Insect_A_179990442",
0xa6b716e9,"MainField_Animal_Insect_A_1808079986",
0xc0c23a33,"MainField_Animal_Insect_A_1887196621",
0x37ba96cd,"MainField_Animal_Insect_A_1975414265",
0xb58a74d0,"MainField_Animal_Insect_A_1980390246",
0x4dca6300,"MainField_Animal_Insect_A_1985721401",
0x579ed59d,"MainField_Animal_Insect_A_2054063687",
0x8420d364,"MainField_Animal_Insect_A_2110591007",
0x5318cada,"MainField_Animal_Insect_A_2147281053",
0xab3871f1,"MainField_Animal_Insect_A_2171044574",
0xab028bc1,"MainField_Animal_Insect_A_2194260089",
0xd57a94a2,"MainField_Animal_Insect_A_2213992295",
0xb8ce2ec6,"MainField_Animal_Insect_A_2246602982",
0x9ba174b7,"MainField_Animal_Insect_A_2256602636",
0x7392e76f,"MainField_Animal_Insect_A_2272312716",
0x1e317c32,"MainField_Animal_Insect_A_2290821282",
0x651b979b,"MainField_Animal_Insect_A_2386146794",
0x83f772ae,"MainField_Animal_Insect_A_2442176433",
0x73d812cb,"MainField_Animal_Insect_A_2454545930",
0xc25cc914,"MainField_Animal_Insect_A_250377944",
0xc6dd79dd,"MainField_Animal_Insect_A_2510376954",
0x5dec000e,"MainField_Animal_Insect_A_2513743520",
0x5d31ee14,"MainField_Animal_Insect_A_2566109738",
0x30f1fb76,"MainField_Animal_Insect_A_2611086110",
0x2bea64d1,"MainField_Animal_Insect_A_2617326423",
0x84c11182,"MainField_Animal_Insect_A_266060753",
0x64ebad08,"MainField_Animal_Insect_A_2741913255",
0x04cf7598,"MainField_Animal_Insect_A_2804449150",
0x9847302a,"MainField_Animal_Insect_A_2814074955",
0x799a1548,"MainField_Animal_Insect_A_2825660623",
0xf6a3151d,"MainField_Animal_Insect_A_284868096",
0x06dffe5c,"MainField_Animal_Insect_A_2903880219",
0x3e5ba9a4,"MainField_Animal_Insect_A_2904926396",
0x40b0c095,"MainField_Animal_Insect_A_2928615544",
0x9a422821,"MainField_Animal_Insect_A_2998743536",
0xfea9dea9,"MainField_Animal_Insect_A_3003553642",
0xbddf9577,"MainField_Animal_Insect_A_3004097949",
0x42b3ffae,"MainField_Animal_Insect_A_3055825679",
0x77727826,"MainField_Animal_Insect_A_3151836832",
0xe35bf474,"MainField_Animal_Insect_A_3448148979",
0x5c5ae1c0,"MainField_Animal_Insect_A_3449699454",
0xd5fa647b,"MainField_Animal_Insect_A_3469307708",
0x25c67765,"MainField_Animal_Insect_A_3503765724",
0x24aa34ca,"MainField_Animal_Insect_A_3527874612",
0xc1a60838,"MainField_Animal_Insect_A_3544172453",
0x8962f09d,"MainField_Animal_Insect_A_357701643",
0x53cb4f44,"MainField_Animal_Insect_A_3631193641",
0xb40de5ea,"MainField_Animal_Insect_A_3670673258",
0xaf33896e,"MainField_Animal_Insect_A_3677840632",
0x2cf2e2e5,"MainField_Animal_Insect_A_3726736526",
0x45fc4561,"MainField_Animal_Insect_A_3781396624",
0xcd13ddd8,"MainField_Animal_Insect_A_3797775969",
0x4f34b35a,"MainField_Animal_Insect_A_3862729605",
0x6361f846,"MainField_Animal_Insect_A_3865246614",
0x26a30a89,"MainField_Animal_Insect_A_3908647263",
0xa69c3881,"MainField_Animal_Insect_A_3909883856",
0x67e258e4,"MainField_Animal_Insect_A_3923019779",
0x357eb201,"MainField_Animal_Insect_A_3956260681",
0x6a50bb04,"MainField_Animal_Insect_A_4043883879",
0xb54b1fde,"MainField_Animal_Insect_A_4076718814",
0x91a29f79,"MainField_Animal_Insect_A_408120234",
0x9654d1d1,"MainField_Animal_Insect_A_4090306855",
0x2a52c0b8,"MainField_Animal_Insect_A_4120891108",
0x7008f520,"MainField_Animal_Insect_A_4122022029",
0xe8e15c18,"MainField_Animal_Insect_A_4166766132",
0x561ee1f7,"MainField_Animal_Insect_A_4198197114",
0x730d7584,"MainField_Animal_Insect_A_4287169035",
0xee34075a,"MainField_Animal_Insect_A_436939261",
0x08d64da0,"MainField_Animal_Insect_A_488997044",
0xba71d374,"MainField_Animal_Insect_A_497290710",
0x44ed33b2,"MainField_Animal_Insect_A_622992459",
0xe5a0d22a,"MainField_Animal_Insect_A_677403089",
0x2562e576,"MainField_Animal_Insect_A_715543655",
0x67bb186c,"MainField_Animal_Insect_A_717523383",
0xfaf699e9,"MainField_Animal_Insect_A_718039792",
0x376b00d1,"MainField_Animal_Insect_A_759184956",
0xc425b01f,"MainField_Animal_Insect_A_872916281",
0x939fd086,"MainField_Animal_Insect_A_885926189",
0x0a9a16b4,"MainField_Animal_Insect_A_887168578",
0xfef4cb4b,"MainField_Animal_Insect_A_902737219",
0x3c921d19,"MainField_Animal_Insect_A_93730826",
0x035a77a9,"MainField_Animal_Insect_A_991330489",
0xfe52300a,"MainField_Animal_Insect_B_280611627",
0xaabfd51d,"MainField_Animal_Insect_B_3134481348",
0x397b31a3,"MainField_Animal_Insect_B_4150539039",
0xaf884d92,"MainField_Animal_Insect_C_104999286",
0x904f6405,"MainField_Animal_Insect_C_1077434069",
0xf513b9cb,"MainField_Animal_Insect_C_1183763776",
0xf3736026,"MainField_Animal_Insect_C_1193758812",
0xfb47605c,"MainField_Animal_Insect_C_1215892934",
0xca8dacf0,"MainField_Animal_Insect_C_1218925660",
0x87741ead,"MainField_Animal_Insect_C_1600316193",
0xde09ae6d,"MainField_Animal_Insect_C_1611165652",
0x21342e13,"MainField_Animal_Insect_C_1714740237",
0x54ef2f4d,"MainField_Animal_Insect_C_1782990454",
0xc363b6da,"MainField_Animal_Insect_C_1930681307",
0xbaacfc71,"MainField_Animal_Insect_C_2085014621",
0x026fbce6,"MainField_Animal_Insect_C_2089415214",
0x8c33582d,"MainField_Animal_Insect_C_2126003019",
0xd6be9dd6,"MainField_Animal_Insect_C_212934705",
0xca035ac6,"MainField_Animal_Insect_C_2162674872",
0x678db1d4,"MainField_Animal_Insect_C_2505520207",
0x76c6df38,"MainField_Animal_Insect_C_2584254994",
0x068773d5,"MainField_Animal_Insect_C_2711439911",
0xb48c20e5,"MainField_Animal_Insect_C_2844404776",
0x3da4077a,"MainField_Animal_Insect_C_2853403562",
0x89583f85,"MainField_Animal_Insect_C_2934094318",
0xd3d0142c,"MainField_Animal_Insect_C_3002524867",
0xaa488b55,"MainField_Animal_Insect_C_3057153518",
0x91ffcccf,"MainField_Animal_Insect_C_3099136552",
0xcd243df5,"MainField_Animal_Insect_C_3267351892",
0xb07b8211,"MainField_Animal_Insect_C_3295339093",
0x7988d1a1,"MainField_Animal_Insect_C_3446706298",
0x8cfb4902,"MainField_Animal_Insect_C_3588554741",
0xf25c5029,"MainField_Animal_Insect_C_3597112010",
0xb6ec211f,"MainField_Animal_Insect_C_3629458927",
0x41fb060c,"MainField_Animal_Insect_C_3715713599",
0x26fe3274,"MainField_Animal_Insect_C_3722149216",
0xe9996aae,"MainField_Animal_Insect_C_3785170283",
0xa722ba73,"MainField_Animal_Insect_C_3803907189",
0x23fc7def,"MainField_Animal_Insect_C_3985417141",
0x278c94eb,"MainField_Animal_Insect_C_4045446754",
0x153af68e,"MainField_Animal_Insect_C_4054092280",
0x302c175d,"MainField_Animal_Insect_C_4128949671",
0xfab50740,"MainField_Animal_Insect_C_4192869989",
0xe7038fb4,"MainField_Animal_Insect_C_4233122891",
0xc4359729,"MainField_Animal_Insect_C_495088749",
0xc1e5f63c,"MainField_Animal_Insect_C_782378330",
0x802080b5,"MainField_Animal_Insect_C_816010807",
0x29aa4044,"MainField_Animal_Insect_C_935913348",
0x8e5fe96b,"MainField_Animal_Insect_E_1015884346",
0x070a3467,"MainField_Animal_Insect_E_1032658408",
0x16fe874c,"MainField_Animal_Insect_E_1038345934",
0xa45eb94a,"MainField_Animal_Insect_E_1040564685",
0xf1104b1c,"MainField_Animal_Insect_E_1042724232",
0x2ea86cc0,"MainField_Animal_Insect_E_1046875311",
0xfa8e5589,"MainField_Animal_Insect_E_107523236",
0x869092fe,"MainField_Animal_Insect_E_1089543262",
0x2a8959d2,"MainField_Animal_Insect_E_1107902162",
0xab6c6b98,"MainField_Animal_Insect_E_1154841844",
0xe822b2b4,"MainField_Animal_Insect_E_116653046",
0x7656263f,"MainField_Animal_Insect_E_1179395642",
0xd470227f,"MainField_Animal_Insect_E_1219931985",
0x12172296,"MainField_Animal_Insect_E_1221340815",
0x9b578dc7,"MainField_Animal_Insect_E_1226933534",
0x444a422d,"MainField_Animal_Insect_E_1262701687",
0x50acc016,"MainField_Animal_Insect_E_1302015492",
0xe917e88a,"MainField_Animal_Insect_E_130242537",
0xe821ab40,"MainField_Animal_Insect_E_1311460440",
0x51e0f1c2,"MainField_Animal_Insect_E_1322932666",
0xf5c24757,"MainField_Animal_Insect_E_1345975854",
0xa9645d9b,"MainField_Animal_Insect_E_1346384717",
0x463dc028,"MainField_Animal_Insect_E_1356663922",
0xecac8587,"MainField_Animal_Insect_E_1372473485",
0x94c6c4d2,"MainField_Animal_Insect_E_1386784058",
0x080885bf,"MainField_Animal_Insect_E_1429345608",
0xd978c9bf,"MainField_Animal_Insect_E_1442390190",
0x38d03914,"MainField_Animal_Insect_E_1469061384",
0x3ea81415,"MainField_Animal_Insect_E_1471203435",
0xb13fe801,"MainField_Animal_Insect_E_150403885",
0x3e8b7bcc,"MainField_Animal_Insect_E_152400685",
0xf87aa59d,"MainField_Animal_Insect_E_1528752401",
0x121ad341,"MainField_Animal_Insect_E_1550288848",
0x0dfb32c6,"MainField_Animal_Insect_E_1564701314",
0x36ed3500,"MainField_Animal_Insect_E_1569865962",
0xb4666be3,"MainField_Animal_Insect_E_160067091",
0x6b55aa84,"MainField_Animal_Insect_E_1617319549",
0xefc82a4e,"MainField_Animal_Insect_E_1634580374",
0x67a4f413,"MainField_Animal_Insect_E_1688279781",
0xbcffbd56,"MainField_Animal_Insect_E_1725171288",
0xe51540a1,"MainField_Animal_Insect_E_1743556425",
0x5fc8cef0,"MainField_Animal_Insect_E_1743680795",
0xd709d1ad,"MainField_Animal_Insect_E_1747866750",
0xab032928,"MainField_Animal_Insect_E_1749491292",
0x4fe1e29a,"MainField_Animal_Insect_E_1763682839",
0x00345ad9,"MainField_Animal_Insect_E_1773509585",
0x86584646,"MainField_Animal_Insect_E_1784274009",
0x8b21a38e,"MainField_Animal_Insect_E_1795798681",
0x754a3961,"MainField_Animal_Insect_E_1799489113",
0xa62bccc4,"MainField_Animal_Insect_E_1800383558",
0xa310f198,"MainField_Animal_Insect_E_1804511312",
0xdbd4c308,"MainField_Animal_Insect_E_1826915172",
0x51442781,"MainField_Animal_Insect_E_1830122389",
0x0dd17dae,"MainField_Animal_Insect_E_1851152796",
0x5edb6f71,"MainField_Animal_Insect_E_186961071",
0x3c4ac5ae,"MainField_Animal_Insect_E_1876131118",
0xf2356b45,"MainField_Animal_Insect_E_1895950338",
0xf3a02fe9,"MainField_Animal_Insect_E_1914505284",
0x844a8307,"MainField_Animal_Insect_E_1926790273",
0x0880fd4d,"MainField_Animal_Insect_E_1934999021",
0x9af33161,"MainField_Animal_Insect_E_1935020839",
0x35383378,"MainField_Animal_Insect_E_1945480634",
0x2cdb2ed8,"MainField_Animal_Insect_E_1972568663",
0xcb0f9b9e,"MainField_Animal_Insect_E_1975205646",
0x41bc5ef9,"MainField_Animal_Insect_E_1985566531",
0x282b870f,"MainField_Animal_Insect_E_2002551481",
0x1e2de89b,"MainField_Animal_Insect_E_2003406237",
0x59846053,"MainField_Animal_Insect_E_2010781182",
0xb791f1f3,"MainField_Animal_Insect_E_2013664837",
0x18cdb2b2,"MainField_Animal_Insect_E_2039017613",
0x808b0d98,"MainField_Animal_Insect_E_2040624808",
0x88df31fb,"MainField_Animal_Insect_E_2041414391",
0xd4bd7357,"MainField_Animal_Insect_E_2048272518",
0xfa003685,"MainField_Animal_Insect_E_2097584502",
0x974975a7,"MainField_Animal_Insect_E_210303724",
0x1420766d,"MainField_Animal_Insect_E_2106863224",
0xef6db309,"MainField_Animal_Insect_E_2111764099",
0x2dd8ddab,"MainField_Animal_Insect_E_2121540435",
0x66c803d2,"MainField_Animal_Insect_E_2155757059",
0x88f9435d,"MainField_Animal_Insect_E_2167763824",
0x3d7666ce,"MainField_Animal_Insect_E_2176333733",
0x87486f25,"MainField_Animal_Insect_E_2182668919",
0x29590a88,"MainField_Animal_Insect_E_2187579807",
0x39d95c4d,"MainField_Animal_Insect_E_2203656396",
0x2ac7d1d7,"MainField_Animal_Insect_E_2217175040",
0x0f3acf7d,"MainField_Animal_Insect_E_2217285425",
0xb7210f81,"MainField_Animal_Insect_E_2244041846",
0xd5367e90,"MainField_Animal_Insect_E_2244180583",
0x65b8464e,"MainField_Animal_Insect_E_2255912088",
0xfc0a53f2,"MainField_Animal_Insect_E_227424853",
0xedaca0ba,"MainField_Animal_Insect_E_2276846089",
0x6705db4a,"MainField_Animal_Insect_E_2304975234",
0xa7aeae70,"MainField_Animal_Insect_E_23360570",
0x0c0c4f3c,"MainField_Animal_Insect_E_2342080624",
0x0626cee3,"MainField_Animal_Insect_E_2346274346",
0x2768a3e5,"MainField_Animal_Insect_E_2357570087",
0xc01d6e84,"MainField_Animal_Insect_E_2379315363",
0xe2b36654,"MainField_Animal_Insect_E_2435175859",
0xc1c73b45,"MainField_Animal_Insect_E_2487127113",
0x6ee0f4c7,"MainField_Animal_Insect_E_2515871154",
0x51374725,"MainField_Animal_Insect_E_2543761372",
0xa30d3b7d,"MainField_Animal_Insect_E_2566679950",
0x1eea335f,"MainField_Animal_Insect_E_2589271950",
0x50c70ecb,"MainField_Animal_Insect_E_263648166",
0xdf93fd32,"MainField_Animal_Insect_E_2809768113",
0xa69bf36c,"MainField_Animal_Insect_E_2817165721",
0x743d565c,"MainField_Animal_Insect_E_281880682",
0x51d3932c,"MainField_Animal_Insect_E_2844395504",
0x18e42257,"MainField_Animal_Insect_E_2908449389",
0x6594aef6,"MainField_Animal_Insect_E_2911576125",
0xab776114,"MainField_Animal_Insect_E_2929305625",
0x7e1b0f96,"MainField_Animal_Insect_E_2960642385",
0x37c34e02,"MainField_Animal_Insect_E_298033233",
0x0cf80ae2,"MainField_Animal_Insect_E_3006207001",
0x216d06f2,"MainField_Animal_Insect_E_3070969224",
0x2b9862b7,"MainField_Animal_Insect_E_3083027906",
0xaa74ae20,"MainField_Animal_Insect_E_3086007097",
0xaef71f8d,"MainField_Animal_Insect_E_3100262649",
0x932652bc,"MainField_Animal_Insect_E_3101432307",
0xac06704a,"MainField_Animal_Insect_E_3122895005",
0xcc327748,"MainField_Animal_Insect_E_3150643215",
0xaada0fcc,"MainField_Animal_Insect_E_3165045782",
0x15c6127f,"MainField_Animal_Insect_E_3175352909",
0xdf153fd2,"MainField_Animal_Insect_E_3187092089",
0xdcb887ad,"MainField_Animal_Insect_E_3187971475",
0x8f7275ef,"MainField_Animal_Insect_E_3189576090",
0xdae9b249,"MainField_Animal_Insect_E_3201527134",
0xa437d421,"MainField_Animal_Insect_E_322821337",
0x14e57b25,"MainField_Animal_Insect_E_3240916597",
0xcda00cdd,"MainField_Animal_Insect_E_3241634018",
0xa4bb49d8,"MainField_Animal_Insect_E_3277045070",
0xb90c1245,"MainField_Animal_Insect_E_3302595433",
0x7f167ae8,"MainField_Animal_Insect_E_3326913572",
0x1970b658,"MainField_Animal_Insect_E_3368426722",
0x9cd4545c,"MainField_Animal_Insect_E_3386138056",
0x2487d65b,"MainField_Animal_Insect_E_3395285822",
0x7669e570,"MainField_Animal_Insect_E_3405408506",
0x8ef703a7,"MainField_Animal_Insect_E_3416388736",
0x28d2475a,"MainField_Animal_Insect_E_3435933849",
0x13a0e753,"MainField_Animal_Insect_E_3443402300",
0x90b7960c,"MainField_Animal_Insect_E_3496274804",
0x62c86470,"MainField_Animal_Insect_E_3507067179",
0xdcd03743,"MainField_Animal_Insect_E_3510662911",
0x3b5fe81e,"MainField_Animal_Insect_E_3553287161",
0x9fb0e1f6,"MainField_Animal_Insect_E_3555175597",
0x6fd3d93f,"MainField_Animal_Insect_E_3576330425",
0x6627c777,"MainField_Animal_Insect_E_3608390948",
0x4005fe7d,"MainField_Animal_Insect_E_3627118474",
0x51f62e86,"MainField_Animal_Insect_E_3648404570",
0xb0c9ba90,"MainField_Animal_Insect_E_3651049615",
0x52120f30,"MainField_Animal_Insect_E_3658137846",
0x0692125b,"MainField_Animal_Insect_E_366101628",
0xa05f1919,"MainField_Animal_Insect_E_3732860909",
0x9b6ff7f6,"MainField_Animal_Insect_E_3744280792",
0x840c5456,"MainField_Animal_Insect_E_3754713076",
0x3ad7614f,"MainField_Animal_Insect_E_3765937916",
0x03917f40,"MainField_Animal_Insect_E_3775518864",
0x9e4e8cac,"MainField_Animal_Insect_E_3776329798",
0xe9231d06,"MainField_Animal_Insect_E_3782987142",
0x2141e8c8,"MainField_Animal_Insect_E_3783898029",
0x6c137981,"MainField_Animal_Insect_E_3801134362",
0xe7469c24,"MainField_Animal_Insect_E_3826404095",
0xfc095f99,"MainField_Animal_Insect_E_3850751399",
0xa4e7f864,"MainField_Animal_Insect_E_3861118613",
0x683ba25d,"MainField_Animal_Insect_E_3869698483",
0x044b07df,"MainField_Animal_Insect_E_3900623977",
0x287b2489,"MainField_Animal_Insect_E_3925086174",
0x37676b33,"MainField_Animal_Insect_E_3934041444",
0xa24c40d9,"MainField_Animal_Insect_E_3954467843",
0xb06e0e94,"MainField_Animal_Insect_E_3989282021",
0xa293997e,"MainField_Animal_Insect_E_4008517029",
0x21b298e1,"MainField_Animal_Insect_E_4067211963",
0x3b14590c,"MainField_Animal_Insect_E_4117815006",
0x77beedb6,"MainField_Animal_Insect_E_4178762293",
0x5425c783,"MainField_Animal_Insect_E_4220030664",
0x98f35070,"MainField_Animal_Insect_E_4270235299",
0x2132d25b,"MainField_Animal_Insect_E_432056731",
0x1add63a0,"MainField_Animal_Insect_E_46845321",
0x46fd3ef8,"MainField_Animal_Insect_E_47748181",
0xc109e5ef,"MainField_Animal_Insect_E_485294459",
0xf9112483,"MainField_Animal_Insect_E_502518939",
0x52be3e0d,"MainField_Animal_Insect_E_532212859",
0xc7819c3a,"MainField_Animal_Insect_E_53838121",
0xfa2eec35,"MainField_Animal_Insect_E_548927741",
0x2a24b766,"MainField_Animal_Insect_E_583160245",
0x3597e656,"MainField_Animal_Insect_E_609199185",
0xf9d97e58,"MainField_Animal_Insect_E_632264108",
0x40902a6e,"MainField_Animal_Insect_E_635960356",
0x2da3abf8,"MainField_Animal_Insect_E_641849584",
0xf67f9df6,"MainField_Animal_Insect_E_668613860",
0xc71c442d,"MainField_Animal_Insect_E_725395726",
0x4c18f3ba,"MainField_Animal_Insect_E_729688451",
0x7255778f,"MainField_Animal_Insect_E_752225514",
0xf8250968,"MainField_Animal_Insect_E_769769512",
0xcf26c8b0,"MainField_Animal_Insect_E_771179277",
0xf9b7f7e4,"MainField_Animal_Insect_E_779566569",
0x3e4bb323,"MainField_Animal_Insect_E_781485540",
0x70676e25,"MainField_Animal_Insect_E_783674284",
0x485e4f95,"MainField_Animal_Insect_E_830542776",
0x44e3f6ad,"MainField_Animal_Insect_E_857852530",
0x7048642d,"MainField_Animal_Insect_E_86984836",
0xc42cd5dc,"MainField_Animal_Insect_E_893265705",
0x61340b58,"MainField_Animal_Insect_E_910710016",
0x7f079b07,"MainField_Animal_Insect_E_927774291",
0x98dfdbee,"MainField_Animal_Insect_E_930417164",
0xb3151991,"MainField_Animal_Insect_F_114635158",
0xda7a4f21,"MainField_Animal_Insect_F_1350143820",
0x5a3e4a5b,"MainField_Animal_Insect_F_1634092097",
0xe3bba1cb,"MainField_Animal_Insect_F_1923474061",
0xa58bd344,"MainField_Animal_Insect_F_199190633",
0x39a2b304,"MainField_Animal_Insect_F_202087140",
0x52db0a32,"MainField_Animal_Insect_F_2086105593",
0x7dba64db,"MainField_Animal_Insect_F_2103187448",
0x94ba6d7f,"MainField_Animal_Insect_F_2121258405",
0x66b22dea,"MainField_Animal_Insect_F_2140133675",
0xc450af51,"MainField_Animal_Insect_F_2144263765",
0xc725019c,"MainField_Animal_Insect_F_2184003433",
0x0480e872,"MainField_Animal_Insect_F_256706358",
0xd368ad7b,"MainField_Animal_Insect_F_2684279835",
0xe2c1a480,"MainField_Animal_Insect_F_2734975651",
0x02b702ec,"MainField_Animal_Insect_F_2842207769",
0xcd1233e0,"MainField_Animal_Insect_F_2853889647",
0x81c8a749,"MainField_Animal_Insect_F_2910899530",
0x43a1e496,"MainField_Animal_Insect_F_3121636666",
0x241c0832,"MainField_Animal_Insect_F_3196472205",
0xeb803c60,"MainField_Animal_Insect_F_3290643387",
0x5d2805b3,"MainField_Animal_Insect_F_3702648012",
0xdbedc580,"MainField_Animal_Insect_F_3772829758",
0xce7b6914,"MainField_Animal_Insect_F_3915514476",
0xc857af25,"MainField_Animal_Insect_F_3992382629",
0xb2c47b3e,"MainField_Animal_Insect_F_4041103441",
0xdb12b8c5,"MainField_Animal_Insect_F_4268399299",
0x95bbe957,"MainField_Animal_Insect_F_566239021",
0x7548fbf0,"MainField_Animal_Insect_F_729396738",
0x7a76473f,"MainField_Animal_Insect_F_729704445",
0xb93921fe,"MainField_Animal_Insect_F_951750340",
0x7b81f90f,"MainField_Animal_Insect_G_1121508128",
0x738fde63,"MainField_Animal_Insect_G_1169009215",
0x49f77f45,"MainField_Animal_Insect_G_1180968732",
0xb4fbf488,"MainField_Animal_Insect_G_1302939338",
0x3103ed98,"MainField_Animal_Insect_G_1396533064",
0xe45a2f44,"MainField_Animal_Insect_G_1747113951",
0x4d03e166,"MainField_Animal_Insect_G_183618114",
0x90b0c3dc,"MainField_Animal_Insect_G_184502922",
0x459d96bf,"MainField_Animal_Insect_G_1928097634",
0xb73c7c66,"MainField_Animal_Insect_G_1931558593",
0x66593d0c,"MainField_Animal_Insect_G_1961611973",
0x5a741fb7,"MainField_Animal_Insect_G_2375865772",
0x65e47eb3,"MainField_Animal_Insect_G_240677485",
0x171208ad,"MainField_Animal_Insect_G_2434526212",
0x32a0e784,"MainField_Animal_Insect_G_2522978224",
0xc88cb3c6,"MainField_Animal_Insect_G_2571704001",
0xfd6cd060,"MainField_Animal_Insect_G_3313674516",
0x0e4a1419,"MainField_Animal_Insect_G_3763737409",
0xddbcec33,"MainField_Animal_Insect_G_3779333898",
0x6382d042,"MainField_Animal_Insect_G_3874601715",
0x6ce2f3b6,"MainField_Animal_Insect_G_4037521058",
0x7a8a8356,"MainField_Animal_Insect_G_4064501519",
0x33a551db,"MainField_Animal_Insect_G_4120004861",
0x425ac062,"MainField_Animal_Insect_G_4139986385",
0x8cc9bb9f,"MainField_Animal_Insect_G_43263254",
0xa1058b35,"MainField_Animal_Insect_G_542431318",
0x0a80003f,"MainField_Animal_Insect_H_1091516991",
0x64f1665b,"MainField_Animal_Insect_H_1158955946",
0x4d34be53,"MainField_Animal_Insect_H_1221933176",
0x8316db9d,"MainField_Animal_Insect_H_1235751197",
0x33263e38,"MainField_Animal_Insect_H_1267040566",
0x1db662ac,"MainField_Animal_Insect_H_1351972176",
0x38636aaf,"MainField_Animal_Insect_H_1363477814",
0xa1b0076e,"MainField_Animal_Insect_H_1416596650",
0x18fa0452,"MainField_Animal_Insect_H_1623058725",
0x0213cb88,"MainField_Animal_Insect_H_1628727223",
0x51aae02c,"MainField_Animal_Insect_H_1943017766",
0x811cd375,"MainField_Animal_Insect_H_2183434007",
0x4f7c27b7,"MainField_Animal_Insect_H_2308284255",
0xecd1faf4,"MainField_Animal_Insect_H_2676138500",
0x8fa20729,"MainField_Animal_Insect_H_2717427182",
0xf2ef3bc6,"MainField_Animal_Insect_H_2728093621",
0x8bc5d9d0,"MainField_Animal_Insect_H_3028204648",
0x48345dc6,"MainField_Animal_Insect_H_3248683850",
0x42d74f9f,"MainField_Animal_Insect_H_3269313336",
0xd931ec96,"MainField_Animal_Insect_H_3446375896",
0xd880186a,"MainField_Animal_Insect_H_3680343070",
0x21bb93b2,"MainField_Animal_Insect_H_3974843986",
0xa0dab329,"MainField_Animal_Insect_H_4088390509",
0xf9212923,"MainField_Animal_Insect_H_4098349111",
0xcd02d1eb,"MainField_Animal_Insect_H_4162719540",
0xc6cfcb32,"MainField_Animal_Insect_H_614465137",
0xde4a7c47,"MainField_Animal_Insect_H_693802098",
0x066cebc3,"MainField_Animal_Insect_H_726300626",
0x76466989,"MainField_Animal_Insect_H_785469257",
0x2f3844ea,"MainField_Animal_Insect_H_801136517",
0x7bf8a739,"MainField_Animal_Insect_H_804269756",
0xae6e928b,"MainField_Animal_Insect_H_882007094",
0x14b31bdf,"MainField_Animal_Insect_H_911554877",
0x404ab25d,"MainField_Animal_Insect_H_970749411",
0xbc248088,"MainField_Animal_Insect_I_1719910660",
0xf6d9554f,"MainField_Animal_Insect_I_1976311643",
0x0aba0d7d,"MainField_Animal_Insect_I_3413598747",
0x770cfdd8,"MainField_Animal_Insect_I_3657791957",
0x1ec86899,"MainField_Animal_Insect_I_738488950",
0xd0c1f693,"MainField_Animal_Insect_I_754021745",
0xc42f0f13,"MainField_Animal_Insect_K_1035492759",
0x35094c10,"MainField_Animal_Insect_K_1160621273",
0xa24f6ba2,"MainField_Animal_Insect_K_1493496993",
0x611d027f,"MainField_Animal_Insect_K_1990018900",
0xb1a9e868,"MainField_Animal_Insect_K_2029248294",
0x4dd75eba,"MainField_Animal_Insect_K_2229897500",
0xd947d523,"MainField_Animal_Insect_K_2277123287",
0x69ef74ba,"MainField_Animal_Insect_K_2352724323",
0xd3c7b5bf,"MainField_Animal_Insect_K_2601427533",
0xc725281a,"MainField_Animal_Insect_K_2733555454",
0xeb68e0b7,"MainField_Animal_Insect_K_3331987487",
0x15011f2d,"MainField_Animal_Insect_K_3626059773",
0xe591f084,"MainField_Animal_Insect_K_3876216125",
0x214a0c4f,"MainField_Animal_Insect_K_636093386",
0x3b25f878,"MainField_Animal_Insect_K_81598031",
0xe42c907a,"MainField_Animal_Insect_M_1056947393",
0x301a89c9,"MainField_Animal_Insect_M_2495244871",
0xca096ec3,"MainField_Animal_Insect_M_2526576725",
0x3799c4c6,"MainField_Animal_Insect_M_2572516817",
0xd9612211,"MainField_Animal_Insect_M_2934553376",
0xcff98db2,"MainField_Animal_Insect_M_330737777",
0x8f19cbf1,"MainField_Animal_Insect_M_3589499472",
0xc8ddf481,"MainField_Animal_Insect_M_3901385046",
0xc62d3dc7,"MainField_Animal_Insect_M_403751699",
0x1582ba09,"MainField_Animal_Insect_M_908453576",
0x11295318,"MainField_Animal_Insect_M_994735183",
0x23580e24,"MainField_Animal_Insect_N_1036263776",
0x7e3670d4,"MainField_Animal_Insect_N_1042244657",
0x2d45f647,"MainField_Animal_Insect_N_1153702597",
0x0f6d190a,"MainField_Animal_Insect_N_1161594641",
0xb5cecd5a,"MainField_Animal_Insect_N_1280958890",
0xb15ec599,"MainField_Animal_Insect_N_1381363",
0xb8f56e53,"MainField_Animal_Insect_N_165597408",
0x25203bed,"MainField_Animal_Insect_N_1688881748",
0x7ece324d,"MainField_Animal_Insect_N_1699884411",
0x9a5ed60b,"MainField_Animal_Insect_N_1752510116",
0xf48215f5,"MainField_Animal_Insect_N_1824675018",
0xb61302e6,"MainField_Animal_Insect_N_1889865295",
0xa3979b58,"MainField_Animal_Insect_N_1897029750",
0xae665a7a,"MainField_Animal_Insect_N_1941582583",
0xbda94dfe,"MainField_Animal_Insect_N_2060038623",
0xdaf6f7a5,"MainField_Animal_Insect_N_2063896066",
0x26007fe3,"MainField_Animal_Insect_N_2080779105",
0x1790e9ca,"MainField_Animal_Insect_N_2139386990",
0xa1cc2ad4,"MainField_Animal_Insect_N_236432757",
0xf3658ce5,"MainField_Animal_Insect_N_2419594850",
0xd3d42097,"MainField_Animal_Insect_N_2514040300",
0xa786ef22,"MainField_Animal_Insect_N_2568804369",
0x7692e658,"MainField_Animal_Insect_N_2907660241",
0x4e44c216,"MainField_Animal_Insect_N_2959576995",
0xe1d15999,"MainField_Animal_Insect_N_3041293865",
0x73d6551e,"MainField_Animal_Insect_N_3077263294",
0x5cb367ef,"MainField_Animal_Insect_N_3283339156",
0x10aaf460,"MainField_Animal_Insect_N_3285779373",
0x823eb539,"MainField_Animal_Insect_N_3352939588",
0xec12d797,"MainField_Animal_Insect_N_3453627325",
0xf81d5a9c,"MainField_Animal_Insect_N_3606094773",
0x94d18663,"MainField_Animal_Insect_N_3627050518",
0x43a794d7,"MainField_Animal_Insect_N_3758952172",
0x99ae81e1,"MainField_Animal_Insect_N_4139370047",
0xea651ec0,"MainField_Animal_Insect_N_4208874454",
0x062379f6,"MainField_Animal_Insect_N_4231881092",
0x201afbc4,"MainField_Animal_Insect_N_4249715931",
0x9a45ca7d,"MainField_Animal_Insect_N_4279790244",
0x4b7d5c9d,"MainField_Animal_Insect_N_568534007",
0xde007ce0,"MainField_Animal_Insect_N_589656714",
0xf024ef7e,"MainField_Animal_Insect_N_790030328",
0xd43c497d,"MainField_Animal_Insect_N_846920338",
0xaa3fd39e,"MainField_Animal_Insect_N_949282828",
0xb6ac7a4a,"MainField_Animal_Insect_N_982820396",
0xe1b9d3bd,"MainField_Animal_Insect_O_1129037122",
0x95699756,"MainField_Animal_Insect_O_1741436308",
0x923f536a,"MainField_Animal_Insect_O_1856443068",
0x6dc9fdf3,"MainField_Animal_Insect_O_2041862697",
0xe984d0eb,"MainField_Animal_Insect_O_2521989653",
0xcf6b7dfb,"MainField_Animal_Insect_O_3495581038",
0x7ca555ab,"MainField_Animal_Insect_P_1328473160",
0x633cec67,"MainField_Animal_Insect_P_1406723401",
0x5cacf0e8,"MainField_Animal_Insect_P_1509577795",
0x481b1655,"MainField_Animal_Insect_P_1575065781",
0x0aaf5e29,"MainField_Animal_Insect_P_1610285191",
0xab7b252c,"MainField_Animal_Insect_P_1913667340",
0x0ac52ab6,"MainField_Animal_Insect_P_2305800596",
0x2d850f2c,"MainField_Animal_Insect_P_2545364613",
0x64680b00,"MainField_Animal_Insect_P_2581431542",
0x7dd6f2da,"MainField_Animal_Insect_P_3092154282",
0xa8c8b7c9,"MainField_Animal_Insect_P_3238470197",
0xf602aca0,"MainField_Animal_Insect_P_3377101069",
0xbd9e7315,"MainField_Animal_Insect_P_365398164",
0xcff4156b,"MainField_Animal_Insect_P_4110482299",
0xb064e2b5,"MainField_Animal_Insect_P_4178671143",
0x74a1e1b4,"MainField_Animal_Insect_P_4279799345",
0xce0d77d0,"MainField_Animal_Insect_P_747947399",
0x4444ba3f,"MainField_Animal_Insect_P_878142839",
0xfa644f17,"MainField_Animal_Insect_P_94370005",
0x91d38939,"MainField_Animal_Insect_Q_1306619007",
0x66b88c0a,"MainField_Animal_Insect_Q_1343042101",
0x4c7c2016,"MainField_Animal_Insect_Q_1572291591",
0x3dbf3de2,"MainField_Animal_Insect_Q_166535019",
0xd14d2ea0,"MainField_Animal_Insect_Q_1928381884",
0x41895b7d,"MainField_Animal_Insect_Q_2213967346",
0x6fb3f59d,"MainField_Animal_Insect_Q_2391872524",
0x7e9ad9ba,"MainField_Animal_Insect_Q_2467566943",
0xaf076605,"MainField_Animal_Insect_Q_2526607191",
0xfc7a0c3e,"MainField_Animal_Insect_Q_2861850838",
0x1cea2510,"MainField_Animal_Insect_Q_2941854745",
0x4bde5bc9,"MainField_Animal_Insect_Q_3106188636",
0xadd3715d,"MainField_Animal_Insect_Q_310835555",
0x20f457f9,"MainField_Animal_Insect_Q_3126981654",
0x387b5acb,"MainField_Animal_Insect_Q_3158733422",
0x18ec0f02,"MainField_Animal_Insect_Q_324917222",
0xaac8e010,"MainField_Animal_Insect_Q_3402443422",
0x6ed756f3,"MainField_Animal_Insect_Q_3410743688",
0x24377a16,"MainField_Animal_Insect_Q_3435441533",
0x227e1b69,"MainField_Animal_Insect_Q_3458622032",
0xc70c428e,"MainField_Animal_Insect_Q_3518097686",
0xa6a624bc,"MainField_Animal_Insect_Q_364314868",
0x07ae313f,"MainField_Animal_Insect_Q_3659481858",
0x977963b2,"MainField_Animal_Insect_Q_3664005486",
0x92e27e31,"MainField_Animal_Insect_Q_517787599",
0x4cdbbf81,"MainField_Animal_Insect_Q_781890850",
0xdeaf1b04,"MainField_Animal_Insect_Q_802693847",
0x0cfee97f,"MainField_Animal_Insect_Q_819434701",
0xf38a1d10,"MainField_Animal_Insect_Q_82474427",
0x5898fedc,"MainField_Animal_Insect_Q_908634345",
0xb60af270,"MainField_Animal_Insect_Q_983665987",
0x6787f459,"MainField_Animal_Insect_R_127784200",
0xde1e27c5,"MainField_Animal_Insect_R_1490638150",
0xd0634f0a,"MainField_Animal_Insect_R_1735136558",
0x1aad7ad4,"MainField_Animal_Insect_R_1910035562",
0x687c8771,"MainField_Animal_Insect_R_1921197063",
0x01058ac8,"MainField_Animal_Insect_R_2125031899",
0xb257719e,"MainField_Animal_Insect_R_2185319366",
0x290ace82,"MainField_Animal_Insect_R_2314124174",
0x172c7f8c,"MainField_Animal_Insect_R_2888029218",
0x967a2a44,"MainField_Animal_Insect_R_2906664190",
0xfaa3e74d,"MainField_Animal_Insect_R_3000438383",
0x6206e014,"MainField_Animal_Insect_R_3080627992",
0x73727e61,"MainField_Animal_Insect_R_3182674154",
0xd612a121,"MainField_Animal_Insect_R_335394740",
0x47f4141f,"MainField_Animal_Insect_R_3422026738",
0xd5ea1e46,"MainField_Animal_Insect_R_3586511797",
0xa8f50cf3,"MainField_Animal_Insect_R_3704152482",
0x311bf949,"MainField_Animal_Insect_R_3744461728",
0x9507cfa5,"MainField_Animal_Insect_R_3888981494",
0xfdc5c66a,"MainField_Animal_Insect_R_4292946816",
0x38831151,"MainField_Animal_Insect_S_110832954",
0x2189dbd1,"MainField_Animal_Insect_S_1186571843",
0x1484ea76,"MainField_Animal_Insect_S_1192336970",
0xa1cfa9b4,"MainField_Animal_Insect_S_12101971",
0x98cd4f71,"MainField_Animal_Insect_S_1240260098",
0x2dd47a41,"MainField_Animal_Insect_S_1264628809",
0x3395f5ec,"MainField_Animal_Insect_S_1272242381",
0x19e78b47,"MainField_Animal_Insect_S_1380447886",
0x4127c516,"MainField_Animal_Insect_S_1387629656",
0xe81f960b,"MainField_Animal_Insect_S_1554145492",
0x6824357f,"MainField_Animal_Insect_S_1562702953",
0xb18c29ec,"MainField_Animal_Insect_S_1595672564",
0x048167bd,"MainField_Animal_Insect_S_1750294463",
0xe8ab35b1,"MainField_Animal_Insect_S_177852330",
0xb31d594e,"MainField_Animal_Insect_S_1817472127",
0xaefd864a,"MainField_Animal_Insect_S_1846444696",
0x7ed81a2d,"MainField_Animal_Insect_S_1871809590",
0xb88dfb83,"MainField_Animal_Insect_S_1907789327",
0x5f3b6c42,"MainField_Animal_Insect_S_1991566107",
0xc6f969b0,"MainField_Animal_Insect_S_2000783796",
0x73db0800,"MainField_Animal_Insect_S_2004639623",
0x376bb1bb,"MainField_Animal_Insect_S_2028227230",
0x01308e32,"MainField_Animal_Insect_S_2055315760",
0x076a8d8e,"MainField_Animal_Insect_S_2146562893",
0xbebed406,"MainField_Animal_Insect_S_2157204438",
0x72739ae9,"MainField_Animal_Insect_S_2205102023",
0x4f6c0764,"MainField_Animal_Insect_S_2285793065",
0x1cf72a6a,"MainField_Animal_Insect_S_2376112790",
0x7cb6693c,"MainField_Animal_Insect_S_2496335748",
0x4f7f35a0,"MainField_Animal_Insect_S_2560368832",
0x7fd18dbe,"MainField_Animal_Insect_S_2861544797",
0xf1e052ea,"MainField_Animal_Insect_S_2999877177",
0xcb0ff1d9,"MainField_Animal_Insect_S_307047888",
0x97006eb7,"MainField_Animal_Insect_S_3104763890",
0xa0b0c3c9,"MainField_Animal_Insect_S_3328813404",
0x03d264eb,"MainField_Animal_Insect_S_3380672155",
0x8c4513ae,"MainField_Animal_Insect_S_3430684841",
0xba83c684,"MainField_Animal_Insect_S_3445088102",
0xa9c8ee5d,"MainField_Animal_Insect_S_3500266886",
0xfd74a298,"MainField_Animal_Insect_S_3516168033",
0x081ec4f9,"MainField_Animal_Insect_S_3729139740",
0xe22ecbda,"MainField_Animal_Insect_S_3840082944",
0x06814496,"MainField_Animal_Insect_S_3843217163",
0x71405a53,"MainField_Animal_Insect_S_3961018648",
0xf7b3ef4d,"MainField_Animal_Insect_S_4021665533",
0x2af83058,"MainField_Animal_Insect_S_4040780770",
0x6ec88950,"MainField_Animal_Insect_S_4061827836",
0xca684961,"MainField_Animal_Insect_S_4076555436",
0xe797f432,"MainField_Animal_Insect_S_4079236403",
0x87531c75,"MainField_Animal_Insect_S_571931334",
0x306eacb1,"MainField_Animal_Insect_S_59434808",
0x8c7925eb,"MainField_Animal_Insect_S_734960523",
0xdad75d6e,"MainField_Animal_Insect_S_738409305",
0x39fe2446,"MainField_Animal_Insect_S_842399412",
0x84c97675,"MainField_Animal_Insect_S_860486257",
0xe54e8085,"MainField_Animal_Insect_T_1003770565",
0x1d531950,"MainField_Animal_Insect_T_109078639",
0x53c7a459,"MainField_Animal_Insect_T_1199132133",
0x8608d219,"MainField_Animal_Insect_T_1253913313",
0xcb448a9f,"MainField_Animal_Insect_T_1612930894",
0xb187ab99,"MainField_Animal_Insect_T_1639568103",
0x140ba3d6,"MainField_Animal_Insect_T_1682440118",
0x028a95d2,"MainField_Animal_Insect_T_1731555937",
0x49f10e8a,"MainField_Animal_Insect_T_1791820168",
0x21fd469a,"MainField_Animal_Insect_T_1824398592",
0x472f861d,"MainField_Animal_Insect_T_1853560908",
0x36538af6,"MainField_Animal_Insect_T_1882527282",
0x54428378,"MainField_Animal_Insect_T_1961768452",
0x32756845,"MainField_Animal_Insect_T_1971718146",
0x74e0932c,"MainField_Animal_Insect_T_2000886920",
0x24548644,"MainField_Animal_Insect_T_2020860433",
0xca12bf9b,"MainField_Animal_Insect_T_2044229178",
0xc73c6271,"MainField_Animal_Insect_T_2055670408",
0xddc0a859,"MainField_Animal_Insect_T_219674747",
0xa0556b14,"MainField_Animal_Insect_T_2261711867",
0x17ce65c7,"MainField_Animal_Insect_T_2349801422",
0x54e07115,"MainField_Animal_Insect_T_2389045335",
0x519f5166,"MainField_Animal_Insect_T_2409057709",
0xe4699106,"MainField_Animal_Insect_T_2415361065",
0x8f3c39ab,"MainField_Animal_Insect_T_2417301664",
0x2b56405c,"MainField_Animal_Insect_T_2485411731",
0xe8595143,"MainField_Animal_Insect_T_252110487",
0x13082eb1,"MainField_Animal_Insect_T_2590171179",
0x3bfa58b2,"MainField_Animal_Insect_T_2688230545",
0x7790936b,"MainField_Animal_Insect_T_2727174694",
0x90680483,"MainField_Animal_Insect_T_2733872355",
0xe7b7a47a,"MainField_Animal_Insect_T_2955199368",
0x1833d85d,"MainField_Animal_Insect_T_3066312216",
0x30c83d96,"MainField_Animal_Insect_T_3166781470",
0x5f9c005c,"MainField_Animal_Insect_T_321007491",
0x30734011,"MainField_Animal_Insect_T_3210337473",
0x2da6796f,"MainField_Animal_Insect_T_3456144037",
0x11c0cdc0,"MainField_Animal_Insect_T_355243065",
0x11c435a9,"MainField_Animal_Insect_T_3561256536",
0x546ae8d3,"MainField_Animal_Insect_T_3577189351",
0x57cc8e52,"MainField_Animal_Insect_T_3768866626",
0xef6fbdd3,"MainField_Animal_Insect_T_3790860627",
0x1a035a07,"MainField_Animal_Insect_T_3939677708",
0x263876c9,"MainField_Animal_Insect_T_3954499717",
0x52f57dea,"MainField_Animal_Insect_T_3964989247",
0xbb8e6588,"MainField_Animal_Insect_T_4042445270",
0xbacff848,"MainField_Animal_Insect_T_4061762635",
0xd0e556d6,"MainField_Animal_Insect_T_4248874038",
0x7423c75a,"MainField_Animal_Insect_T_4249818157",
0x20e54579,"MainField_Animal_Insect_T_4256302533",
0x506004d4,"MainField_Animal_Insect_T_53538176",
0xa30285ab,"MainField_Animal_Insect_T_551992979",
0x49012066,"MainField_Animal_Insect_T_570663143",
0xd2f01a44,"MainField_Animal_Insect_T_589983719",
0xb1a1c7d1,"MainField_Animal_Insect_T_624474311",
0x383779e0,"MainField_Animal_Insect_T_627900439",
0x2e1b4bda,"MainField_Animal_Insect_T_646170858",
0x8120aff8,"MainField_Animal_Insect_T_787150574",
0xe94c2ac4,"MainField_Animal_Insect_T_852589334",
0xe676978a,"MainField_Animal_Insect_T_922624955",
0x948c319b,"MainField_Animal_Insect_T_966203150",
0xa985295b,"MainField_Animal_Insect_X_1050275658",
0x2d21814d,"MainField_Animal_Insect_X_1076958992",
0x81ddf06e,"MainField_Animal_Insect_X_1160227376",
0x9e00792e,"MainField_Animal_Insect_X_1221931673",
0x170ac71a,"MainField_Animal_Insect_X_1295429885",
0x50ceb383,"MainField_Animal_Insect_X_1348416855",
0x74d1df37,"MainField_Animal_Insect_X_1446786103",
0x38935eaa,"MainField_Animal_Insect_X_1693183673",
0x9a81107f,"MainField_Animal_Insect_X_1933853753",
0x0cee258b,"MainField_Animal_Insect_X_2325578985",
0x0e3ab541,"MainField_Animal_Insect_X_2516811324",
0x0dd8aa4d,"MainField_Animal_Insect_X_2555251826",
0x98878019,"MainField_Animal_Insect_X_2584091312",
0x1d3f7330,"MainField_Animal_Insect_X_2666326176",
0x95d1b224,"MainField_Animal_Insect_X_3355237550",
0x08d12739,"MainField_Animal_Insect_X_3803922630",
0x4a14c567,"MainField_Animal_Insect_X_387760378",
0x1cdefa08,"MainField_Animal_Insect_X_389678505",
0x8d9c50d0,"MainField_Animal_Insect_X_3976663335",
0x5ed53d9e,"MainField_Animal_Insect_X_4069536017",
0xf1ddefb9,"MainField_Animal_Insect_X_420711688",
0xfd1642d1,"MainField_Animal_Insect_X_4246993137",
0x0d09ba0f,"MainField_Animal_Insect_X_4256756325",
0x6f646ff2,"MainField_Animal_Insect_X_4264501782",
0x3e5d10de,"MainField_Animal_Insect_X_4273920580",
0xeda8679f,"MainField_Animal_Insect_X_439820646",
0x1cdf766e,"MainField_Animal_Insect_X_780022480",
0xd1bc3326,"MainField_Animal_Insect_X_977169075",
0x4d7d7bec,"MainField_Animal_Insect_Z_1020813654",
0x04490456,"MainField_Animal_Insect_Z_1175931869",
0xf8e00de3,"MainField_Animal_Insect_Z_1376740943",
0x13cd2997,"MainField_Animal_Insect_Z_1388197782",
0x70455c45,"MainField_Animal_Insect_Z_150118016",
0xf2bb9cd5,"MainField_Animal_Insect_Z_2005188332",
0xd51cbb98,"MainField_Animal_Insect_Z_2067489168",
0x92de119f,"MainField_Animal_Insect_Z_2095545218",
0x54198e9d,"MainField_Animal_Insect_Z_2108182901",
0x69ecb11b,"MainField_Animal_Insect_Z_2166907879",
0xd79793bf,"MainField_Animal_Insect_Z_2232109355",
0x5adba7e5,"MainField_Animal_Insect_Z_2383082025",
0x639d4e16,"MainField_Animal_Insect_Z_2406379774",
0x19dd3cd4,"MainField_Animal_Insect_Z_2416443163",
0xb918ae67,"MainField_Animal_Insect_Z_2573844004",
0xe2ec0816,"MainField_Animal_Insect_Z_2599165508",
0xa904e922,"MainField_Animal_Insect_Z_2714607806",
0x9e4e3076,"MainField_Animal_Insect_Z_2756531338",
0xba3b82e4,"MainField_Animal_Insect_Z_2763248208",
0x892c46c7,"MainField_Animal_Insect_Z_2804214811",
0x9b2e80b1,"MainField_Animal_Insect_Z_2963286884",
0xede11b7e,"MainField_Animal_Insect_Z_302913191",
0x2c7d5cf0,"MainField_Animal_Insect_Z_3132785813",
0x784947a6,"MainField_Animal_Insect_Z_3181402603",
0x778228ea,"MainField_Animal_Insect_Z_3223486718",
0x8141f6b1,"MainField_Animal_Insect_Z_3309005423",
0xb1ee2f80,"MainField_Animal_Insect_Z_3328400926",
0xde8aaddb,"MainField_Animal_Insect_Z_3377649336",
0x07e25889,"MainField_Animal_Insect_Z_3641444908",
0x69d5b38f,"MainField_Animal_Insect_Z_375552094",
0x0c8a3567,"MainField_Animal_Insect_Z_3803355329",
0x84b67884,"MainField_Animal_Insect_Z_3891891411",
0xe1474ecf,"MainField_Animal_Insect_Z_3905749028",
0xc25733fe,"MainField_Animal_Insect_Z_3963005918",
0x122a4790,"MainField_Animal_Insect_Z_39850919",
0x5768d559,"MainField_Animal_Insect_Z_400386705",
0x16d1c118,"MainField_Animal_Insect_Z_401403746",
0xbaba6b5f,"MainField_Animal_Insect_Z_4071829976",
0x6bfa2bf0,"MainField_Animal_Insect_Z_4184656124",
0xf91ebe99,"MainField_Animal_Insect_Z_4232670243",
0x5d978e15,"MainField_Animal_Insect_Z_450557072",
0x5466f37d,"MainField_Animal_Insect_Z_485558575",
0x69df6769,"MainField_Animal_Insect_Z_612930767",
0xbca40122,"MainField_Animal_Insect_Z_918015408",
0xf325140b,"MainField_Animal_LittleBird_A_104202019",
0x29556385,"MainField_Animal_LittleBird_A_106162930",
0x4e120fb4,"MainField_Animal_LittleBird_A_1087919557",
0x6778bd4f,"MainField_Animal_LittleBird_A_1095464711",
0x72688098,"MainField_Animal_LittleBird_A_1148488519",
0x5b6b78a5,"MainField_Animal_LittleBird_A_1158092797",
0x36de7e04,"MainField_Animal_LittleBird_A_1170836509",
0x7b5a0d1f,"MainField_Animal_LittleBird_A_1184167497",
0x5d7895be,"MainField_Animal_LittleBird_A_1192825185",
0x261f9976,"MainField_Animal_LittleBird_A_1209908695",
0x592e76d2,"MainField_Animal_LittleBird_A_1232361700",
0xa7a3ed96,"MainField_Animal_LittleBird_A_1246545486",
0xbbe30a8f,"MainField_Animal_LittleBird_A_1249123447",
0xcc3c9e54,"MainField_Animal_LittleBird_A_1258312660",
0x56ca6cdc,"MainField_Animal_LittleBird_A_1261137794",
0x3001e88b,"MainField_Animal_LittleBird_A_1275997030",
0xe21ff732,"MainField_Animal_LittleBird_A_1367801060",
0x049ab0e9,"MainField_Animal_LittleBird_A_1419733152",
0x144c2f0d,"MainField_Animal_LittleBird_A_1432308584",
0x3361212f,"MainField_Animal_LittleBird_A_1434289580",
0xc24700cc,"MainField_Animal_LittleBird_A_1441321934",
0x779b84eb,"MainField_Animal_LittleBird_A_1468234013",
0x4a3f2a34,"MainField_Animal_LittleBird_A_1471579816",
0xb9190d39,"MainField_Animal_LittleBird_A_1492883354",
0x37da404f,"MainField_Animal_LittleBird_A_1498838316",
0xe29033e4,"MainField_Animal_LittleBird_A_1547429872",
0x3cb5e95f,"MainField_Animal_LittleBird_A_1566608210",
0xa3f137f5,"MainField_Animal_LittleBird_A_1566725867",
0x1b21ca3d,"MainField_Animal_LittleBird_A_1590091086",
0xf7e96ad4,"MainField_Animal_LittleBird_A_1594904930",
0xb434ed40,"MainField_Animal_LittleBird_A_1609113944",
0x9a0a7d84,"MainField_Animal_LittleBird_A_1610069055",
0x67bfc0a9,"MainField_Animal_LittleBird_A_1667957485",
0x4708e3f9,"MainField_Animal_LittleBird_A_1697001542",
0x452b59f8,"MainField_Animal_LittleBird_A_1705352397",
0x47b916e9,"MainField_Animal_LittleBird_A_1727906554",
0x45923bb4,"MainField_Animal_LittleBird_A_1763750994",
0xc4d38dbc,"MainField_Animal_LittleBird_A_1790774425",
0x45ad915b,"MainField_Animal_LittleBird_A_1813508674",
0x252db187,"MainField_Animal_LittleBird_A_1876578608",
0xa32658f8,"MainField_Animal_LittleBird_A_2001871290",
0xb288c243,"MainField_Animal_LittleBird_A_2082146298",
0x78a52478,"MainField_Animal_LittleBird_A_2100551654",
0x98fe4b7c,"MainField_Animal_LittleBird_A_211863775",
0x5d9d1e7a,"MainField_Animal_LittleBird_A_2151800650",
0x11668154,"MainField_Animal_LittleBird_A_2256617383",
0xb510f6db,"MainField_Animal_LittleBird_A_2310806481",
0x6cca8a1f,"MainField_Animal_LittleBird_A_2354061102",
0x22edc229,"MainField_Animal_LittleBird_A_2424671365",
0xe96096b6,"MainField_Animal_LittleBird_A_2437270479",
0x202b08d2,"MainField_Animal_LittleBird_A_2496661077",
0x3651aaf4,"MainField_Animal_LittleBird_A_2520663525",
0xc239a7bd,"MainField_Animal_LittleBird_A_2534168278",
0x1932c3d7,"MainField_Animal_LittleBird_A_2555529630",
0xc3c7ce2d,"MainField_Animal_LittleBird_A_2568507013",
0x3c3251b9,"MainField_Animal_LittleBird_A_2587323081",
0x1a8f6f91,"MainField_Animal_LittleBird_A_2597351397",
0x62a01525,"MainField_Animal_LittleBird_A_2638979705",
0xebab91dc,"MainField_Animal_LittleBird_A_2680726475",
0xad5c7a85,"MainField_Animal_LittleBird_A_2696610750",
0x59cab783,"MainField_Animal_LittleBird_A_2733726116",
0x02a283db,"MainField_Animal_LittleBird_A_2737417182",
0xdce79ee5,"MainField_Animal_LittleBird_A_2743279181",
0x3304dd60,"MainField_Animal_LittleBird_A_2776276294",
0x70b5397c,"MainField_Animal_LittleBird_A_2791522314",
0x51abd1e5,"MainField_Animal_LittleBird_A_2845164232",
0xd972a704,"MainField_Animal_LittleBird_A_2914335446",
0xe8373ca4,"MainField_Animal_LittleBird_A_2936774062",
0x0418564c,"MainField_Animal_LittleBird_A_2942209766",
0xeec177f1,"MainField_Animal_LittleBird_A_2960287664",
0x9282ce24,"MainField_Animal_LittleBird_A_2969384719",
0xe721a9bc,"MainField_Animal_LittleBird_A_2972790012",
0x07873167,"MainField_Animal_LittleBird_A_2992222742",
0x827e88f1,"MainField_Animal_LittleBird_A_3048053479",
0x94aea415,"MainField_Animal_LittleBird_A_3048927771",
0x33354407,"MainField_Animal_LittleBird_A_3127986493",
0x4ee9a9de,"MainField_Animal_LittleBird_A_315785619",
0x3c041e81,"MainField_Animal_LittleBird_A_3214345189",
0x600ca9ba,"MainField_Animal_LittleBird_A_3239975982",
0x02f439b7,"MainField_Animal_LittleBird_A_3266749432",
0x5cdd515c,"MainField_Animal_LittleBird_A_3268843660",
0x605dbb06,"MainField_Animal_LittleBird_A_328546075",
0xb1835a7b,"MainField_Animal_LittleBird_A_3287078738",
0x097292c7,"MainField_Animal_LittleBird_A_3290407257",
0x2c4557e0,"MainField_Animal_LittleBird_A_329266289",
0xd3caeef2,"MainField_Animal_LittleBird_A_3317771769",
0x61464ee1,"MainField_Animal_LittleBird_A_332792944",
0x23a4da2a,"MainField_Animal_LittleBird_A_3366887186",
0x50d3c8e7,"MainField_Animal_LittleBird_A_3385906520",
0x0f90d2d6,"MainField_Animal_LittleBird_A_3387114413",
0x26e76d34,"MainField_Animal_LittleBird_A_3408049014",
0x8a1646ee,"MainField_Animal_LittleBird_A_3410972443",
0x7f776181,"MainField_Animal_LittleBird_A_3424339112",
0x31bc5910,"MainField_Animal_LittleBird_A_3425044622",
0x72e0c793,"MainField_Animal_LittleBird_A_3438775790",
0x1ee1b472,"MainField_Animal_LittleBird_A_3455220189",
0x6701d13a,"MainField_Animal_LittleBird_A_347774876",
0x3d8aacac,"MainField_Animal_LittleBird_A_3485854452",
0x59c17b61,"MainField_Animal_LittleBird_A_3523313018",
0xf7c0608b,"MainField_Animal_LittleBird_A_3530210852",
0x87d69f6d,"MainField_Animal_LittleBird_A_3535076631",
0x2e9be47f,"MainField_Animal_LittleBird_A_3572633772",
0x810b58ff,"MainField_Animal_LittleBird_A_3588456213",
0x37ffd5ed,"MainField_Animal_LittleBird_A_358964436",
0xdb2cb623,"MainField_Animal_LittleBird_A_3590162166",
0xc14ca164,"MainField_Animal_LittleBird_A_3594639687",
0xd3eb014e,"MainField_Animal_LittleBird_A_3614006440",
0xcd5fd6df,"MainField_Animal_LittleBird_A_3636808795",
0x7163ad05,"MainField_Animal_LittleBird_A_373123974",
0xf0cf528c,"MainField_Animal_LittleBird_A_3735590663",
0x988d20f6,"MainField_Animal_LittleBird_A_3769717667",
0xc579e459,"MainField_Animal_LittleBird_A_3772541916",
0x9f8ec668,"MainField_Animal_LittleBird_A_3773071218",
0x6c14046c,"MainField_Animal_LittleBird_A_3776056039",
0x58bc8f16,"MainField_Animal_LittleBird_A_3795085099",
0xeae43e9c,"MainField_Animal_LittleBird_A_3816967145",
0x0efc7b2b,"MainField_Animal_LittleBird_A_3856438960",
0x1aa534c0,"MainField_Animal_LittleBird_A_3870365274",
0xfba33164,"MainField_Animal_LittleBird_A_3879173215",
0x8a4955d8,"MainField_Animal_LittleBird_A_3894137541",
0xa16f97ce,"MainField_Animal_LittleBird_A_3900865413",
0x11cdb709,"MainField_Animal_LittleBird_A_3953576122",
0x304bf736,"MainField_Animal_LittleBird_A_3969115177",
0x5c8644fe,"MainField_Animal_LittleBird_A_3999567906",
0xd122538e,"MainField_Animal_LittleBird_A_3999651346",
0x758f0af7,"MainField_Animal_LittleBird_A_4011353754",
0x5f3998c2,"MainField_Animal_LittleBird_A_4012081737",
0xd4eab2ef,"MainField_Animal_LittleBird_A_4012125024",
0xee0eff0d,"MainField_Animal_LittleBird_A_4030348206",
0x45caf8c9,"MainField_Animal_LittleBird_A_4040462125",
0x443dcd68,"MainField_Animal_LittleBird_A_4052136800",
0x792f73e8,"MainField_Animal_LittleBird_A_4068845007",
0xadcd070e,"MainField_Animal_LittleBird_A_4109381049",
0x7f1bca8d,"MainField_Animal_LittleBird_A_4129988083",
0xdcc8b2f9,"MainField_Animal_LittleBird_A_4172758450",
0x48ccbed6,"MainField_Animal_LittleBird_A_4210912065",
0xf6fc57d9,"MainField_Animal_LittleBird_A_4237943111",
0xc1f83b62,"MainField_Animal_LittleBird_A_4250918121",
0x13107770,"MainField_Animal_LittleBird_A_4251207496",
0x56f6378b,"MainField_Animal_LittleBird_A_4253573780",
0xa959d2fe,"MainField_Animal_LittleBird_A_4256544766",
0x4766583f,"MainField_Animal_LittleBird_A_4276818882",
0x80124fb5,"MainField_Animal_LittleBird_A_471531570",
0x3fbb1dba,"MainField_Animal_LittleBird_A_495249050",
0x14e49011,"MainField_Animal_LittleBird_A_505504121",
0x87645857,"MainField_Animal_LittleBird_A_511370600",
0x33e9dbad,"MainField_Animal_LittleBird_A_590926646",
0x8684715d,"MainField_Animal_LittleBird_A_591264227",
0x3d6b6090,"MainField_Animal_LittleBird_A_604413407",
0x52463e58,"MainField_Animal_LittleBird_A_634393664",
0xe492692d,"MainField_Animal_LittleBird_A_687681470",
0xa7d28d19,"MainField_Animal_LittleBird_A_776340729",
0x3279649d,"MainField_Animal_LittleBird_A_784601746",
0xaa65ad70,"MainField_Animal_LittleBird_A_797902651",
0x146a6e5c,"MainField_Animal_LittleBird_A_81571512",
0x3be0c2bd,"MainField_Animal_LittleBird_A_83429913",
0x12491b24,"MainField_Animal_LittleBird_A_850750927",
0xb2048b39,"MainField_Animal_LittleBird_A_864591570",
0x0bc6aca1,"MainField_Animal_LittleBird_A_877226047",
0x500c8edc,"MainField_Animal_LittleBird_A_887415543",
0xc623cff0,"MainField_Animal_LittleBird_A_931573686",
0x07d5acd8,"MainField_Animal_LittleBird_A_932365549",
0x3a581197,"MainField_Animal_LittleBird_A_941071569",
0xc795875b,"MainField_Animal_LittleBird_A_958140566",
0x6b542e2e,"MainField_Animal_LittleBird_A_960382042",
0x3fc5f5a1,"MainField_Animal_LittleBird_B_1068784032",
0x6d437ad5,"MainField_Animal_LittleBird_B_1268272729",
0x126372c8,"MainField_Animal_LittleBird_B_1361658392",
0x010d8d2c,"MainField_Animal_LittleBird_B_1802095385",
0x3298ff56,"MainField_Animal_LittleBird_B_1802807432",
0x6d917303,"MainField_Animal_LittleBird_B_190813891",
0xc9b91dbb,"MainField_Animal_LittleBird_B_2038347426",
0xdf1eea75,"MainField_Animal_LittleBird_B_2056941873",
0x8a4725f1,"MainField_Animal_LittleBird_B_2180184554",
0xf79dc4a2,"MainField_Animal_LittleBird_B_2280090225",
0x1ff96e93,"MainField_Animal_LittleBird_B_2412080919",
0xe2968f76,"MainField_Animal_LittleBird_B_2431075539",
0x38d8aa16,"MainField_Animal_LittleBird_B_2666521924",
0xa9b14ae7,"MainField_Animal_LittleBird_B_283359235",
0x86983b1a,"MainField_Animal_LittleBird_B_2859832572",
0x61ccd923,"MainField_Animal_LittleBird_B_2945400880",
0x881298f3,"MainField_Animal_LittleBird_B_2977170394",
0x2193b484,"MainField_Animal_LittleBird_B_3024072461",
0xe6eb8282,"MainField_Animal_LittleBird_B_3043409710",
0xa2c4f589,"MainField_Animal_LittleBird_B_3121281375",
0xbc47c2bc,"MainField_Animal_LittleBird_B_3169316480",
0x349ca585,"MainField_Animal_LittleBird_B_3186320088",
0xe22f8a3a,"MainField_Animal_LittleBird_B_3372447941",
0xf9447a4d,"MainField_Animal_LittleBird_B_3478505760",
0x3b2bb9ca,"MainField_Animal_LittleBird_B_3535517616",
0xe5e54b9a,"MainField_Animal_LittleBird_B_3671447811",
0xbe647d09,"MainField_Animal_LittleBird_B_3704815740",
0x827a2b5c,"MainField_Animal_LittleBird_B_3964874040",
0x6eb306d4,"MainField_Animal_LittleBird_B_41197871",
0xe7d2f292,"MainField_Animal_LittleBird_B_4286800726",
0x6d06e58b,"MainField_Animal_LittleBird_B_610112417",
0xd9a0357d,"MainField_Animal_LittleBird_B_84629008",
0x6fe57894,"MainField_Animal_LittleBird_B_895902449",
0x8717e183,"MainField_Animal_LittleBird_C_1030239074",
0x7537343a,"MainField_Animal_LittleBird_C_111783743",
0x4c72ac5a,"MainField_Animal_LittleBird_C_1145459156",
0xc74fd8ca,"MainField_Animal_LittleBird_C_1250527969",
0xc99298cd,"MainField_Animal_LittleBird_C_126124255",
0x1fae9499,"MainField_Animal_LittleBird_C_1308284090",
0xbb9fbd8b,"MainField_Animal_LittleBird_C_1322652273",
0xf77c2c81,"MainField_Animal_LittleBird_C_1351376004",
0xe6e29045,"MainField_Animal_LittleBird_C_1396857034",
0x2fe0c1f6,"MainField_Animal_LittleBird_C_1484594881",
0xf11b5e56,"MainField_Animal_LittleBird_C_1587298014",
0xa148acf2,"MainField_Animal_LittleBird_C_1597755557",
0x4c4b4c25,"MainField_Animal_LittleBird_C_159948735",
0x366a39f3,"MainField_Animal_LittleBird_C_1619369568",
0xc810e668,"MainField_Animal_LittleBird_C_168326275",
0x2e09fc21,"MainField_Animal_LittleBird_C_16843525",
0xaafd4b7d,"MainField_Animal_LittleBird_C_1696369806",
0x116f71c0,"MainField_Animal_LittleBird_C_1749514189",
0x33c58289,"MainField_Animal_LittleBird_C_1758766799",
0x20a6c9f1,"MainField_Animal_LittleBird_C_1797065814",
0x52fafd6d,"MainField_Animal_LittleBird_C_1916030969",
0x21016199,"MainField_Animal_LittleBird_C_1942660116",
0x29284f09,"MainField_Animal_LittleBird_C_1977304954",
0x0c09ce7e,"MainField_Animal_LittleBird_C_2057683876",
0xa8485628,"MainField_Animal_LittleBird_C_2075732122",
0x9b27c8ae,"MainField_Animal_LittleBird_C_2127443527",
0x47929e0b,"MainField_Animal_LittleBird_C_2162483140",
0xe9bb48de,"MainField_Animal_LittleBird_C_2170274987",
0x970e4e01,"MainField_Animal_LittleBird_C_2189638097",
0xf15fa84c,"MainField_Animal_LittleBird_C_2234984257",
0x974de1f2,"MainField_Animal_LittleBird_C_2257991550",
0x113f3cff,"MainField_Animal_LittleBird_C_2290000043",
0x472294df,"MainField_Animal_LittleBird_C_2430229459",
0xcee439ea,"MainField_Animal_LittleBird_C_2480854224",
0xb66ccb3e,"MainField_Animal_LittleBird_C_2518621367",
0xcef95458,"MainField_Animal_LittleBird_C_2598713974",
0x484a0a99,"MainField_Animal_LittleBird_C_2634303755",
0x0b360d36,"MainField_Animal_LittleBird_C_2741760955",
0x780f97be,"MainField_Animal_LittleBird_C_2743916697",
0xdda256b1,"MainField_Animal_LittleBird_C_2773903048",
0x7245fc3b,"MainField_Animal_LittleBird_C_2815596712",
0xcfff93dc,"MainField_Animal_LittleBird_C_2848124161",
0x864e518d,"MainField_Animal_LittleBird_C_2910384704",
0xf1183aff,"MainField_Animal_LittleBird_C_2977331896",
0x3682c06f,"MainField_Animal_LittleBird_C_2980077262",
0x540a46ac,"MainField_Animal_LittleBird_C_2981720324",
0x069441d7,"MainField_Animal_LittleBird_C_3045761067",
0xefc9397a,"MainField_Animal_LittleBird_C_3108119701",
0xd50b74a5,"MainField_Animal_LittleBird_C_3113023948",
0x197edb61,"MainField_Animal_LittleBird_C_3144284166",
0x93216970,"MainField_Animal_LittleBird_C_3303382689",
0xe246fc03,"MainField_Animal_LittleBird_C_3449112732",
0x40978a78,"MainField_Animal_LittleBird_C_3614184370",
0xb2337d03,"MainField_Animal_LittleBird_C_3622056684",
0xd1526c0e,"MainField_Animal_LittleBird_C_3636033865",
0x3dcb8665,"MainField_Animal_LittleBird_C_3669153192",
0xab179878,"MainField_Animal_LittleBird_C_3820139352",
0x6cd2d6a7,"MainField_Animal_LittleBird_C_3873690700",
0x2ea2f00b,"MainField_Animal_LittleBird_C_390559286",
0x6b044791,"MainField_Animal_LittleBird_C_3958304017",
0xb141f824,"MainField_Animal_LittleBird_C_4066534668",
0x36287158,"MainField_Animal_LittleBird_C_4079449611",
0x33399eb5,"MainField_Animal_LittleBird_C_409270663",
0xebef120f,"MainField_Animal_LittleBird_C_4250359415",
0x1447467c,"MainField_Animal_LittleBird_C_4274544175",
0x6f3ecdf6,"MainField_Animal_LittleBird_C_532224274",
0x9cb86954,"MainField_Animal_LittleBird_C_565496175",
0x849c07b7,"MainField_Animal_LittleBird_C_614985738",
0x97e051fd,"MainField_Animal_LittleBird_C_616450173",
0xabaf9a52,"MainField_Animal_LittleBird_C_641469031",
0x8b29bcab,"MainField_Animal_LittleBird_C_642919521",
0x346e7a08,"MainField_Animal_LittleBird_C_720413411",
0x54418f02,"MainField_Animal_LittleBird_C_756129749",
0x12833a1d,"MainField_Animal_LittleBird_C_839583751",
0x13e78e36,"MainField_Animal_LittleBird_D_1604452069",
0xfa1b8e85,"MainField_Animal_LittleBird_D_1626547694",
0xba86061b,"MainField_Animal_LittleBird_D_1798972742",
0xc313d917,"MainField_Animal_LittleBird_D_1922466167",
0x89422c7b,"MainField_Animal_LittleBird_D_1956570723",
0x0628d485,"MainField_Animal_LittleBird_D_2103156499",
0x1c4d01fb,"MainField_Animal_LittleBird_D_2256279679",
0x539ac155,"MainField_Animal_LittleBird_D_2420737682",
0xee50266b,"MainField_Animal_LittleBird_D_2544977301",
0xde3e4055,"MainField_Animal_LittleBird_D_2626119972",
0x488825e6,"MainField_Animal_LittleBird_D_2905041283",
0x6c47663f,"MainField_Animal_LittleBird_D_290634695",
0x1b2c6785,"MainField_Animal_LittleBird_D_4074633688",
0x9103f620,"MainField_Animal_LittleBird_F_10412650",
0x31d758a5,"MainField_Animal_LittleBird_F_1160087057",
0x3270a92c,"MainField_Animal_LittleBird_F_1279323788",
0x8f568312,"MainField_Animal_LittleBird_F_1455167081",
0x895c8d1b,"MainField_Animal_LittleBird_F_1519661294",
0x4cf46bfa,"MainField_Animal_LittleBird_F_167087670",
0x3766b4b7,"MainField_Animal_LittleBird_F_1834907157",
0xb69758e0,"MainField_Animal_LittleBird_F_1965530720",
0xd42fa587,"MainField_Animal_LittleBird_F_2001635352",
0xe1609323,"MainField_Animal_LittleBird_F_2402090219",
0x859bcdc8,"MainField_Animal_LittleBird_F_2499360249",
0x6e1a2e09,"MainField_Animal_LittleBird_F_2831149543",
0xc786280c,"MainField_Animal_LittleBird_F_2838719326",
0xb50417fb,"MainField_Animal_LittleBird_F_3084342753",
0xdc0fd3e4,"MainField_Animal_LittleBird_F_3568384412",
0x536474d9,"MainField_Animal_LittleBird_F_3596131543",
0x1b718387,"MainField_Animal_LittleBird_F_3759704203",
0x336a46c6,"MainField_Animal_LittleBird_F_3928522536",
0xca3594e2,"MainField_Animal_LittleBird_F_4202149901",
0xd2ff879f,"MainField_Animal_LittleBird_F_554038749",
0x4b6ca49b,"MainField_Animal_LittleBird_F_654009974",
0x224bf1a5,"MainField_Animal_LittleBird_F_812805191",
0xd3d3e2d6,"MainField_Animal_LittleBird_F_895186678",
0x64cb5574,"MainField_Animal_LittleBird_F_998271049",
0xcf27b4a0,"MainField_Animal_Pigeon_A_1057288542",
0xd59a0247,"MainField_Animal_Pigeon_A_1078657010",
0xb1eac3f2,"MainField_Animal_Pigeon_A_125320274",
0xe1509347,"MainField_Animal_Pigeon_A_135479754",
0xc95a810f,"MainField_Animal_Pigeon_A_1395086322",
0xdb01a3d0,"MainField_Animal_Pigeon_A_1411003746",
0x9d1befb9,"MainField_Animal_Pigeon_A_1411030303",
0xb64cabda,"MainField_Animal_Pigeon_A_1430441436",
0x77984928,"MainField_Animal_Pigeon_A_1636942657",
0xd6dec9c5,"MainField_Animal_Pigeon_A_1767318566",
0x324f9cac,"MainField_Animal_Pigeon_A_1981031540",
0x3366e983,"MainField_Animal_Pigeon_A_2040514888",
0x4ca24701,"MainField_Animal_Pigeon_A_2047898789",
0xf9095add,"MainField_Animal_Pigeon_A_2167576589",
0x5471998a,"MainField_Animal_Pigeon_A_2204397159",
0x6ab618aa,"MainField_Animal_Pigeon_A_2230751775",
0x9a958f9c,"MainField_Animal_Pigeon_A_2445785356",
0x9ac1e982,"MainField_Animal_Pigeon_A_2500997443",
0x39e8fae3,"MainField_Animal_Pigeon_A_2742866156",
0x67354b63,"MainField_Animal_Pigeon_A_2782667410",
0xa8e4c637,"MainField_Animal_Pigeon_A_3062342831",
0x04032090,"MainField_Animal_Pigeon_A_3089067261",
0x3ffdebda,"MainField_Animal_Pigeon_A_3094625392",
0xad4dd2ea,"MainField_Animal_Pigeon_A_3213172222",
0x139da458,"MainField_Animal_Pigeon_A_322197291",
0xb498df10,"MainField_Animal_Pigeon_A_3260971459",
0xb3f96a6a,"MainField_Animal_Pigeon_A_3587176488",
0x5331aa7b,"MainField_Animal_Pigeon_A_3588842071",
0x31d5750d,"MainField_Animal_Pigeon_A_3639489447",
0xc277fd4f,"MainField_Animal_Pigeon_A_3650646046",
0x23543025,"MainField_Animal_Pigeon_A_3940189916",
0x3d2bf99e,"MainField_Animal_Pigeon_A_3957126047",
0x85cfd126,"MainField_Animal_Pigeon_A_4092169795",
0xb1d1a9bb,"MainField_Animal_Pigeon_A_4192745911",
0x59ed46ba,"MainField_Animal_Pigeon_A_4199177737",
0x40f21968,"MainField_Animal_Pigeon_A_492821781",
0xb87e8dbb,"MainField_Animal_Pigeon_A_508564656",
0x9713e2e1,"MainField_Animal_Pigeon_A_572705148",
0xf25487e1,"MainField_Animal_Pigeon_A_660978910",
0xf85d4545,"MainField_Animal_Pigeon_A_747363188",
0xa488fc6b,"MainField_Animal_Pigeon_A_862404431",
0xd15c6e6a,"MainField_Animal_Pigeon_A_886904058",
0xc7d8c01a,"MainField_Animal_Pigeon_B_1159527837",
0xad72e5aa,"MainField_Animal_Pigeon_B_1355268746",
0xbc8d97b7,"MainField_Animal_Pigeon_B_1674212206",
0xee1f9b2a,"MainField_Animal_Pigeon_B_1722770116",
0x52e6c06b,"MainField_Animal_Pigeon_B_1831112952",
0x79d7d035,"MainField_Animal_Pigeon_B_1897455454",
0x9dcf50ec,"MainField_Animal_Pigeon_B_1913385825",
0x05a972d0,"MainField_Animal_Pigeon_B_2127338550",
0x023bc031,"MainField_Animal_Pigeon_B_2157026369",
0x9eca82bd,"MainField_Animal_Pigeon_B_2170819032",
0x8daf3a3f,"MainField_Animal_Pigeon_B_2252991204",
0x988c6da3,"MainField_Animal_Pigeon_B_2458345418",
0xd8ca0774,"MainField_Animal_Pigeon_B_2600315780",
0x5c90b3ea,"MainField_Animal_Pigeon_B_2689454468",
0x6c2992fa,"MainField_Animal_Pigeon_B_2786473411",
0x3d8e7e03,"MainField_Animal_Pigeon_B_2810088835",
0xe46931c7,"MainField_Animal_Pigeon_B_2908277843",
0xf5f63054,"MainField_Animal_Pigeon_B_3245469119",
0xb9ff0b18,"MainField_Animal_Pigeon_B_3521936968",
0x328affc2,"MainField_Animal_Pigeon_B_3702411126",
0x090513f9,"MainField_Animal_Pigeon_B_3757130015",
0xed9af763,"MainField_Animal_Pigeon_B_3763588198",
0x7643ba3d,"MainField_Animal_Pigeon_B_759182359",
0xef086e44,"MainField_Animal_Pigeon_B_991403306",
0xd95b5793,"MainField_Animal_Pigeon_B_Swarm_2404846667",
0x8ac3cd6d,"MainField_Animal_Pigeon_B_Swarm_2793662133",
0x1accf5c9,"MainField_Animal_Pigeon_B_Swarm_3151759930",
0x9ff7f716,"MainField_Animal_Pigeon_B_Swarm_3932606862",
0x23759175,"MainField_Animal_Pigeon_B_Swarm_4222332175",
0x82cbb008,"MainField_Animal_Pigeon_B_Swarm_537412628",
0x70b794dd,"MainField_Animal_Pigeon_Swarm_1489483662",
0x2f0b7b93,"MainField_Animal_Ptarmigan_A_1070695304",
0x07e7abf3,"MainField_Animal_Ptarmigan_A_2458982285",
0x15c41202,"MainField_Animal_Ptarmigan_A_3107285829",
0x7b994c3a,"MainField_Animal_Ptarmigan_A_3189969179",
0x23126847,"MainField_Animal_Ptarmigan_A_3415142685",
0x0d9f52b5,"MainField_Animal_Ptarmigan_A_3841318665",
0x2145e0ab,"MainField_Animal_Ptarmigan_A_486208438",
0x91f2dc58,"MainField_Animal_Ptarmigan_A_491206309",
0xb08c4db6,"MainField_Animal_Ptarmigan_A_540103884",
0xe7e93244,"MainField_Animal_Ptarmigan_A_675789443",
0x1b3baa54,"MainField_Animal_Rhino_A_1025997384",
0x8350157e,"MainField_Animal_Rhino_A_115834956",
0x2959698d,"MainField_Animal_Rhino_A_1167289693",
0x3ba3a09f,"MainField_Animal_Rhino_A_126549244",
0x6afbc233,"MainField_Animal_Rhino_A_1284182125",
0xbcd7247b,"MainField_Animal_Rhino_A_1461271733",
0xcc2a19f8,"MainField_Animal_Rhino_A_1514072194",
0xc185bf90,"MainField_Animal_Rhino_A_164420333",
0x10bcdea3,"MainField_Animal_Rhino_A_1766881132",
0x2e6ec71f,"MainField_Animal_Rhino_A_1783966539",
0x4b97beaf,"MainField_Animal_Rhino_A_2001824648",
0x9e9d5b82,"MainField_Animal_Rhino_A_2589101550",
0xde35f602,"MainField_Animal_Rhino_A_3646836637",
0x2d992f32,"MainField_Animal_Rhino_A_3710085012",
0xdebf609e,"MainField_Animal_Rhino_A_4029457560",
0x3691239a,"MainField_Animal_Rhino_A_411897080",
0x293fabcd,"MainField_Animal_Rhino_A_4233296129",
0x5aab2ad3,"MainField_Animal_Rhino_A_52852575",
0xc0173efb,"MainField_Animal_Rhino_A_588109248",
0x38c43dc8,"MainField_Animal_Rhino_A_700084060",
0xd16083a9,"MainField_Animal_Rhino_A_740932593",
0x56e4a29d,"MainField_Animal_Rhino_A_749612173",
0x554c272b,"MainField_Animal_Rhino_A_863731574",
0x600430cd,"MainField_Animal_Rhino_A_906688592",
0x28702c35,"MainField_Animal_RupeeRabbit_A_1158573626",
0x81a478e8,"MainField_Animal_RupeeRabbit_A_119676254",
0xf29665f9,"MainField_Animal_RupeeRabbit_A_1206289926",
0x47c9efa4,"MainField_Animal_RupeeRabbit_A_1207652058",
0x6536fadb,"MainField_Animal_RupeeRabbit_A_125156133",
0x7cf8bd50,"MainField_Animal_RupeeRabbit_A_1326369154",
0x5cb0e17f,"MainField_Animal_RupeeRabbit_A_1355702776",
0x38e3bcc3,"MainField_Animal_RupeeRabbit_A_1397879052",
0x3e61276f,"MainField_Animal_RupeeRabbit_A_1416749828",
0x870649b0,"MainField_Animal_RupeeRabbit_A_1452972064",
0x16c40c36,"MainField_Animal_RupeeRabbit_A_1860780134",
0x5d544906,"MainField_Animal_RupeeRabbit_A_1918020349",
0xcd045772,"MainField_Animal_RupeeRabbit_A_2052852149",
0xdeed8556,"MainField_Animal_RupeeRabbit_A_2088165780",
0x933ca0bc,"MainField_Animal_RupeeRabbit_A_2133969819",
0xeb2128c3,"MainField_Animal_RupeeRabbit_A_2156716660",
0x28065d3d,"MainField_Animal_RupeeRabbit_A_216844463",
0x6a768550,"MainField_Animal_RupeeRabbit_A_2206925384",
0x362e835c,"MainField_Animal_RupeeRabbit_A_2216052439",
0x2215de27,"MainField_Animal_RupeeRabbit_A_2444329800",
0x0031c58c,"MainField_Animal_RupeeRabbit_A_2462742580",
0x2ca38b6d,"MainField_Animal_RupeeRabbit_A_2479244960",
0x4f9a9ed3,"MainField_Animal_RupeeRabbit_A_2487515977",
0xc26c3148,"MainField_Animal_RupeeRabbit_A_2653143848",
0x0632e36e,"MainField_Animal_RupeeRabbit_A_2847406036",
0x7fa5cac0,"MainField_Animal_RupeeRabbit_A_2949923055",
0x4ba1cc71,"MainField_Animal_RupeeRabbit_A_2987031642",
0x96f4a20c,"MainField_Animal_RupeeRabbit_A_3038019600",
0xb33f24e5,"MainField_Animal_RupeeRabbit_A_3058004865",
0xb3366950,"MainField_Animal_RupeeRabbit_A_3153871387",
0x761b6dc1,"MainField_Animal_RupeeRabbit_A_3182492758",
0x04695db3,"MainField_Animal_RupeeRabbit_A_3266955628",
0x60937c9b,"MainField_Animal_RupeeRabbit_A_3289973653",
0x6bc0b76c,"MainField_Animal_RupeeRabbit_A_3349720697",
0xd4aa8d79,"MainField_Animal_RupeeRabbit_A_3385779494",
0x4e9da14e,"MainField_Animal_RupeeRabbit_A_3430815441",
0x580eef00,"MainField_Animal_RupeeRabbit_A_3453661316",
0xcc647c86,"MainField_Animal_RupeeRabbit_A_3477513170",
0x83c0a938,"MainField_Animal_RupeeRabbit_A_3491694435",
0x910f6b91,"MainField_Animal_RupeeRabbit_A_3494308151",
0x1034ed91,"MainField_Animal_RupeeRabbit_A_352056706",
0x6e2df9a1,"MainField_Animal_RupeeRabbit_A_3557966274",
0x9bfa5631,"MainField_Animal_RupeeRabbit_A_3567214278",
0x9fbb7245,"MainField_Animal_RupeeRabbit_A_3694619243",
0x3f4d5076,"MainField_Animal_RupeeRabbit_A_3698220879",
0x326d6d9b,"MainField_Animal_RupeeRabbit_A_3835373149",
0xcdfdca06,"MainField_Animal_RupeeRabbit_A_3844573003",
0xb9837e90,"MainField_Animal_RupeeRabbit_A_3870968455",
0x225c6a81,"MainField_Animal_RupeeRabbit_A_3999689016",
0xe712849c,"MainField_Animal_RupeeRabbit_A_4061398525",
0x2eefb3bc,"MainField_Animal_RupeeRabbit_A_4107212023",
0xaa30c1e8,"MainField_Animal_RupeeRabbit_A_4139197005",
0x13f56814,"MainField_Animal_RupeeRabbit_A_4164653337",
0x43de7dce,"MainField_Animal_RupeeRabbit_A_4222375871",
0xc9a0dfdb,"MainField_Animal_RupeeRabbit_A_4226568064",
0x42868953,"MainField_Animal_RupeeRabbit_A_558955435",
0x57e3e2f9,"MainField_Animal_RupeeRabbit_A_581551889",
0xea0dc33b,"MainField_Animal_RupeeRabbit_A_647652707",
0xff884da1,"MainField_Animal_RupeeRabbit_A_654864030",
0xad2f3e3e,"MainField_Animal_RupeeRabbit_A_67622795",
0x7c064d6a,"MainField_Animal_RupeeRabbit_A_750137237",
0x46824154,"MainField_Animal_RupeeRabbit_A_750180744",
0x055d13da,"MainField_Animal_RupeeRabbit_A_789917160",
0x5ad791e0,"MainField_Animal_RupeeRabbit_A_8112710",
0x65534db9,"MainField_Animal_RupeeRabbit_A_823897486",
0xf4ac887f,"MainField_Animal_RupeeRabbit_A_956029213",
0x34fc3ea1,"MainField_Animal_Squirrel_A_1083286102",
0xbab014ad,"MainField_Animal_Squirrel_A_1120934911",
0xece1896a,"MainField_Animal_Squirrel_A_1135162891",
0x40fe4db3,"MainField_Animal_Squirrel_A_1195777174",
0x8cc9489c,"MainField_Animal_Squirrel_A_1204745502",
0x96eb6adf,"MainField_Animal_Squirrel_A_1235845434",
0xa939e7a5,"MainField_Animal_Squirrel_A_1286482564",
0xd9c69ee2,"MainField_Animal_Squirrel_A_1340164163",
0x3b5346bf,"MainField_Animal_Squirrel_A_1348575504",
0x18f661f0,"MainField_Animal_Squirrel_A_1384172789",
0x25ea7b3a,"MainField_Animal_Squirrel_A_1386139292",
0x3b183d87,"MainField_Animal_Squirrel_A_1449845492",
0x7a64f4b2,"MainField_Animal_Squirrel_A_1500184282",
0x5c54bc88,"MainField_Animal_Squirrel_A_1630920207",
0x3f1d47da,"MainField_Animal_Squirrel_A_1754796389",
0xcf273061,"MainField_Animal_Squirrel_A_1862182145",
0xb3af9599,"MainField_Animal_Squirrel_A_2116005633",
0xfe506c7c,"MainField_Animal_Squirrel_A_2138987539",
0x6694244f,"MainField_Animal_Squirrel_A_2249978617",
0x1987d500,"MainField_Animal_Squirrel_A_2303292358",
0xbe626bf8,"MainField_Animal_Squirrel_A_2310632443",
0x78a911e9,"MainField_Animal_Squirrel_A_2332023937",
0xe80a1256,"MainField_Animal_Squirrel_A_2419787094",
0x1795badc,"MainField_Animal_Squirrel_A_2451846718",
0x94e77d4c,"MainField_Animal_Squirrel_A_2487052720",
0x748fcaf5,"MainField_Animal_Squirrel_A_2792569333",
0x47bf3277,"MainField_Animal_Squirrel_A_290712119",
0x0a7ce801,"MainField_Animal_Squirrel_A_2970533361",
0xe6d65bd8,"MainField_Animal_Squirrel_A_3169390270",
0xfb639c1e,"MainField_Animal_Squirrel_A_3253307673",
0x3efd9c81,"MainField_Animal_Squirrel_A_3386905688",
0x45216df7,"MainField_Animal_Squirrel_A_3441148391",
0x3fb4db0a,"MainField_Animal_Squirrel_A_3443603909",
0xdd145ecb,"MainField_Animal_Squirrel_A_3563308874",
0x67ba5340,"MainField_Animal_Squirrel_A_3610762713",
0xef70a3fb,"MainField_Animal_Squirrel_A_3739777967",
0xc5b2c5ed,"MainField_Animal_Squirrel_A_3909680905",
0xf40fe185,"MainField_Animal_Squirrel_A_3954731044",
0x99db19f5,"MainField_Animal_Squirrel_A_3983357911",
0x2df90082,"MainField_Animal_Squirrel_A_4224072534",
0xefed8b3b,"MainField_Animal_Squirrel_A_4230613892",
0x7a17445a,"MainField_Animal_Squirrel_A_4261729976",
0x7e2cbc49,"MainField_Animal_Squirrel_A_640901142",
0xde633d06,"MainField_Animal_Squirrel_A_645278022",
0x5c145371,"MainField_Animal_Squirrel_A_675637906",
0xaef71d33,"MainField_Animal_Squirrel_A_900594793",
0x16cdaf59,"MainField_Animal_WildDuck_A_1004062377",
0x683e2261,"MainField_Animal_WildDuck_A_110773066",
0xe1939c3e,"MainField_Animal_WildDuck_A_1165974311",
0x099f90b9,"MainField_Animal_WildDuck_A_1169494178",
0x01640bdf,"MainField_Animal_WildDuck_A_1175493322",
0xd0c54400,"MainField_Animal_WildDuck_A_1187747737",
0xacdf74c9,"MainField_Animal_WildDuck_A_1199364776",
0x710193f4,"MainField_Animal_WildDuck_A_1281503150",
0xbf583b8d,"MainField_Animal_WildDuck_A_1312151701",
0xa34d4530,"MainField_Animal_WildDuck_A_1331904651",
0x0de09a06,"MainField_Animal_WildDuck_A_15236647",
0x02654398,"MainField_Animal_WildDuck_A_1667140235",
0xc2d19c4d,"MainField_Animal_WildDuck_A_1804506302",
0xb7ad66d3,"MainField_Animal_WildDuck_A_1817062124",
0x198294d8,"MainField_Animal_WildDuck_A_1894727176",
0x21a76e67,"MainField_Animal_WildDuck_A_1906025985",
0x16d92e75,"MainField_Animal_WildDuck_A_2103763474",
0x87036132,"MainField_Animal_WildDuck_A_2289494251",
0x39cf018c,"MainField_Animal_WildDuck_A_2387956361",
0x41fcf1ea,"MainField_Animal_WildDuck_A_2389800369",
0xd6d59bc3,"MainField_Animal_WildDuck_A_2395570462",
0xaea974d9,"MainField_Animal_WildDuck_A_2508869871",
0x66013141,"MainField_Animal_WildDuck_A_271659896",
0xe0d00dba,"MainField_Animal_WildDuck_A_2747455331",
0x374ed1bb,"MainField_Animal_WildDuck_A_2815457538",
0x8570b245,"MainField_Animal_WildDuck_A_289055927",
0x5d27062f,"MainField_Animal_WildDuck_A_2955039986",
0xca869176,"MainField_Animal_WildDuck_A_3019009243",
0x2811f267,"MainField_Animal_WildDuck_A_3025494220",
0x5712e18d,"MainField_Animal_WildDuck_A_3055321414",
0x5ebb3166,"MainField_Animal_WildDuck_A_3104094630",
0x54ad217f,"MainField_Animal_WildDuck_A_328076396",
0x32713257,"MainField_Animal_WildDuck_A_330237791",
0xdfbd9772,"MainField_Animal_WildDuck_A_3368070881",
0x67231f11,"MainField_Animal_WildDuck_A_3389652685",
0xd8ef5119,"MainField_Animal_WildDuck_A_3473105039",
0x34647580,"MainField_Animal_WildDuck_A_3533878770",
0x06cca13d,"MainField_Animal_WildDuck_A_3537419263",
0x0794d99e,"MainField_Animal_WildDuck_A_3547156775",
0x5a4c8f28,"MainField_Animal_WildDuck_A_355026652",
0xd79e5998,"MainField_Animal_WildDuck_A_3592264229",
0x19e18bb5,"MainField_Animal_WildDuck_A_3598695846",
0xac93c516,"MainField_Animal_WildDuck_A_360678521",
0xd0f56c70,"MainField_Animal_WildDuck_A_3640005659",
0x2ea1c8ad,"MainField_Animal_WildDuck_A_3671934255",
0x2900160b,"MainField_Animal_WildDuck_A_3676845208",
0x69300ac4,"MainField_Animal_WildDuck_A_3679034131",
0x0c7c71c9,"MainField_Animal_WildDuck_A_3726279026",
0x0ee6e006,"MainField_Animal_WildDuck_A_3756823273",
0xd611cbf5,"MainField_Animal_WildDuck_A_3778669889",
0xe8317674,"MainField_Animal_WildDuck_A_3783088546",
0xe73981e0,"MainField_Animal_WildDuck_A_380686754",
0x78477001,"MainField_Animal_WildDuck_A_3839668764",
0x0e740eca,"MainField_Animal_WildDuck_A_3891529670",
0xa0767f15,"MainField_Animal_WildDuck_A_3908997507",
0xefb67e8a,"MainField_Animal_WildDuck_A_4025366869",
0x41de417d,"MainField_Animal_WildDuck_A_4031719880",
0x33816af7,"MainField_Animal_WildDuck_A_4098291145",
0xc35bd002,"MainField_Animal_WildDuck_A_4104966635",
0xc87a0c5a,"MainField_Animal_WildDuck_A_4185380449",
0xe86bbfa3,"MainField_Animal_WildDuck_A_4185616278",
0xf93198f4,"MainField_Animal_WildDuck_A_419645887",
0x984da33f,"MainField_Animal_WildDuck_A_4207835049",
0x897d30e8,"MainField_Animal_WildDuck_A_4241728826",
0x1ad5f6a3,"MainField_Animal_WildDuck_A_571527443",
0x3555a177,"MainField_Animal_WildDuck_A_651236079",
0xa81e8988,"MainField_Animal_WildDuck_A_65930410",
0xe858b173,"MainField_Animal_WildDuck_A_716619",
0xb4f6b3e8,"MainField_Animal_WildDuck_A_739848495",
0xf22744f5,"MainField_Animal_WildDuck_A_751692481",
0x016a2578,"MainField_Animal_WildDuck_A_783901976",
0x8bb58582,"MainField_Animal_WildDuck_A_894748993",
0xc8ea8189,"MainField_Animal_WildDuck_A_922794503",
0x15c4e15a,"MainField_Animal_WildDuck_A_938625334",
0xb1142ca9,"MainField_Animal_WildDuck_A_955326919",
0x7a1ec9d6,"MainField_Animal_WildDuck_A_967197496",
0xdfa959a5,"MainField_Animal_WildDuck_A_970490735",
0x1442959e,"MainField_Animal_WildDuck_A_981497405",
0xd5caa3c8,"MainField_Animal_WildDuck_A_996704082",
0xa1800c84,"MainField_Animal_WildGoat_A_1001477654",
0x4e08683b,"MainField_Animal_WildGoat_A_1003403271",
0xa37135c3,"MainField_Animal_WildGoat_A_1024311026",
0xd0c05fdc,"MainField_Animal_WildGoat_A_1045712225",
0x1e472a95,"MainField_Animal_WildGoat_A_1059211867",
0x3be60d2d,"MainField_Animal_WildGoat_A_1083540233",
0xdfdb4702,"MainField_Animal_WildGoat_A_1090999135",
0x537f61f7,"MainField_Animal_WildGoat_A_1093953793",
0x51ac24a8,"MainField_Animal_WildGoat_A_114835838",
0x4081b4b7,"MainField_Animal_WildGoat_A_1164918001",
0xeb9fad1f,"MainField_Animal_WildGoat_A_1174282110",
0x48d76dac,"MainField_Animal_WildGoat_A_118855450",
0x2c030c10,"MainField_Animal_WildGoat_A_1209902194",
0xcac38269,"MainField_Animal_WildGoat_A_1227348918",
0x84a4e33b,"MainField_Animal_WildGoat_A_1245569849",
0xdc360e00,"MainField_Animal_WildGoat_A_1271338652",
0xca73742c,"MainField_Animal_WildGoat_A_1306544001",
0xbd343a43,"MainField_Animal_WildGoat_A_132383977",
0x8f82fe37,"MainField_Animal_WildGoat_A_1340708165",
0x6514f756,"MainField_Animal_WildGoat_A_1350713953",
0x129eb21d,"MainField_Animal_WildGoat_A_1387996343",
0x4861b937,"MainField_Animal_WildGoat_A_1408716406",
0xe5c55de1,"MainField_Animal_WildGoat_A_1485407943",
0xc00e383e,"MainField_Animal_WildGoat_A_1517093064",
0x90db10a0,"MainField_Animal_WildGoat_A_1538398457",
0x4dfa06ad,"MainField_Animal_WildGoat_A_1540356830",
0xc250c150,"MainField_Animal_WildGoat_A_1584469151",
0xd29ee3e7,"MainField_Animal_WildGoat_A_158456908",
0xc0ebc32b,"MainField_Animal_WildGoat_A_1600358716",
0x653f09fb,"MainField_Animal_WildGoat_A_1603165003",
0x0e60b43d,"MainField_Animal_WildGoat_A_1616580905",
0x10020510,"MainField_Animal_WildGoat_A_1635587913",
0x43b5fa61,"MainField_Animal_WildGoat_A_1709429057",
0xb0e65744,"MainField_Animal_WildGoat_A_1779540934",
0x2e9eeb7b,"MainField_Animal_WildGoat_A_1819571431",
0xcd704f8f,"MainField_Animal_WildGoat_A_1847607881",
0x8f1f2ed9,"MainField_Animal_WildGoat_A_1858448330",
0x80c3f59e,"MainField_Animal_WildGoat_A_1864784340",
0x9cad9cc0,"MainField_Animal_WildGoat_A_1869182106",
0x36254683,"MainField_Animal_WildGoat_A_1875298852",
0xa543d45e,"MainField_Animal_WildGoat_A_1903781019",
0xc5db286f,"MainField_Animal_WildGoat_A_1913089218",
0x3ae20bda,"MainField_Animal_WildGoat_A_191400880",
0xee22026b,"MainField_Animal_WildGoat_A_1997334714",
0xfac719e3,"MainField_Animal_WildGoat_A_2057703983",
0x36dee758,"MainField_Animal_WildGoat_A_205827917",
0x166e9fb8,"MainField_Animal_WildGoat_A_2066933253",
0x04e2f0a6,"MainField_Animal_WildGoat_A_2069779372",
0x04fa7a87,"MainField_Animal_WildGoat_A_2101308177",
0x8f139060,"MainField_Animal_WildGoat_A_2113903505",
0x9485eafe,"MainField_Animal_WildGoat_A_2130499588",
0x82943694,"MainField_Animal_WildGoat_A_2132433619",
0x396b350e,"MainField_Animal_WildGoat_A_2143034301",
0x183f249c,"MainField_Animal_WildGoat_A_2165351029",
0x798d068a,"MainField_Animal_WildGoat_A_2181150175",
0x5e8c827d,"MainField_Animal_WildGoat_A_2205461099",
0xa9f08ca2,"MainField_Animal_WildGoat_A_2223344961",
0xa225e626,"MainField_Animal_WildGoat_A_2226814826",
0xe273f8db,"MainField_Animal_WildGoat_A_2233899563",
0x92ea03ce,"MainField_Animal_WildGoat_A_2248978454",
0x1e245a86,"MainField_Animal_WildGoat_A_2323923128",
0xb4565db4,"MainField_Animal_WildGoat_A_2337422299",
0xc613caf3,"MainField_Animal_WildGoat_A_2388941383",
0x3a846473,"MainField_Animal_WildGoat_A_241857625",
0x1b15b88b,"MainField_Animal_WildGoat_A_2425733492",
0x42cf8bf4,"MainField_Animal_WildGoat_A_2429371540",
0x139127cf,"MainField_Animal_WildGoat_A_244531434",
0x05d4a49c,"MainField_Animal_WildGoat_A_2591174461",
0xc46b794a,"MainField_Animal_WildGoat_A_2605510599",
0xc6a6522f,"MainField_Animal_WildGoat_A_2627916669",
0x4113d020,"MainField_Animal_WildGoat_A_2630254252",
0x4cf59331,"MainField_Animal_WildGoat_A_2630820100",
0xe9b54684,"MainField_Animal_WildGoat_A_2632128494",
0x20e16a81,"MainField_Animal_WildGoat_A_263377003",
0x816c15b3,"MainField_Animal_WildGoat_A_2662305520",
0xf0408962,"MainField_Animal_WildGoat_A_2691699362",
0x8d9ec779,"MainField_Animal_WildGoat_A_2693721330",
0x2b208ab9,"MainField_Animal_WildGoat_A_2696929834",
0xffca4235,"MainField_Animal_WildGoat_A_2734622236",
0xe5a39775,"MainField_Animal_WildGoat_A_2756644186",
0x210a09de,"MainField_Animal_WildGoat_A_2770667799",
0x91199dd2,"MainField_Animal_WildGoat_A_2773409651",
0x263ddfce,"MainField_Animal_WildGoat_A_279869997",
0xf51c401a,"MainField_Animal_WildGoat_A_2800582827",
0xcb733f11,"MainField_Animal_WildGoat_A_2812984552",
0xc4f804f1,"MainField_Animal_WildGoat_A_2820603626",
0x649772aa,"MainField_Animal_WildGoat_A_2820773500",
0x27376840,"MainField_Animal_WildGoat_A_2846094877",
0x8498df0a,"MainField_Animal_WildGoat_A_2891327236",
0xdad2ef01,"MainField_Animal_WildGoat_A_2907218565",
0xfe43b9ac,"MainField_Animal_WildGoat_A_2917493643",
0x696a7236,"MainField_Animal_WildGoat_A_2920736990",
0x28b6df8d,"MainField_Animal_WildGoat_A_2926950715",
0xd50c9bc8,"MainField_Animal_WildGoat_A_2941376548",
0xf88b13ca,"MainField_Animal_WildGoat_A_2964271004",
0xfa0e93f4,"MainField_Animal_WildGoat_A_3019684200",
0xb283cf85,"MainField_Animal_WildGoat_A_3022400024",
0x5f19bb64,"MainField_Animal_WildGoat_A_3037826648",
0x350d5bc2,"MainField_Animal_WildGoat_A_3171468356",
0x63f51ddc,"MainField_Animal_WildGoat_A_3180883591",
0x0258f75a,"MainField_Animal_WildGoat_A_3184975960",
0x29c70c32,"MainField_Animal_WildGoat_A_3221089240",
0xc974251d,"MainField_Animal_WildGoat_A_3238416645",
0xbadbd175,"MainField_Animal_WildGoat_A_3252217111",
0x5a3720f9,"MainField_Animal_WildGoat_A_326664768",
0x297dfa70,"MainField_Animal_WildGoat_A_3287205295",
0x6c0e699e,"MainField_Animal_WildGoat_A_3330957019",
0x3aa200b2,"MainField_Animal_WildGoat_A_3363933070",
0x0e85f820,"MainField_Animal_WildGoat_A_3364285117",
0x18e12d40,"MainField_Animal_WildGoat_A_3366651306",
0x7eb3b6f3,"MainField_Animal_WildGoat_A_3372765489",
0x0253c706,"MainField_Animal_WildGoat_A_338763084",
0x4ef785da,"MainField_Animal_WildGoat_A_3421501490",
0xacea3659,"MainField_Animal_WildGoat_A_3434683633",
0x43db7bd9,"MainField_Animal_WildGoat_A_3439358725",
0x69e67875,"MainField_Animal_WildGoat_A_3470143026",
0xb7ba8e74,"MainField_Animal_WildGoat_A_3512303147",
0x32fc4670,"MainField_Animal_WildGoat_A_3551850550",
0x884e181a,"MainField_Animal_WildGoat_A_3555964224",
0x9598cec9,"MainField_Animal_WildGoat_A_3579621603",
0x88ef8ba8,"MainField_Animal_WildGoat_A_3581385343",
0xed774ec4,"MainField_Animal_WildGoat_A_3597881594",
0xea8578e5,"MainField_Animal_WildGoat_A_3605647281",
0xdbb70eb2,"MainField_Animal_WildGoat_A_3634577848",
0x19191f5e,"MainField_Animal_WildGoat_A_3638862173",
0xebb930e6,"MainField_Animal_WildGoat_A_3652920794",
0x5c102ddc,"MainField_Animal_WildGoat_A_3681439424",
0x8e8987c8,"MainField_Animal_WildGoat_A_368345084",
0x5e86352b,"MainField_Animal_WildGoat_A_3701614875",
0xb8f0a990,"MainField_Animal_WildGoat_A_3708938213",
0xa0976b18,"MainField_Animal_WildGoat_A_3710406754",
0x7603324e,"MainField_Animal_WildGoat_A_3791712502",
0xa2db086d,"MainField_Animal_WildGoat_A_3813946798",
0x13cdeed2,"MainField_Animal_WildGoat_A_3840415201",
0xdaf5f36e,"MainField_Animal_WildGoat_A_3866911623",
0xfed70abe,"MainField_Animal_WildGoat_A_3884010752",
0x2161d79d,"MainField_Animal_WildGoat_A_3884935934",
0x422821d0,"MainField_Animal_WildGoat_A_3894177496",
0xf4e22c13,"MainField_Animal_WildGoat_A_3928882356",
0x69c84bd1,"MainField_Animal_WildGoat_A_3952624412",
0x20056074,"MainField_Animal_WildGoat_A_4078187387",
0x047a70a7,"MainField_Animal_WildGoat_A_4103828531",
0x792df69c,"MainField_Animal_WildGoat_A_4106570388",
0x2738b3d5,"MainField_Animal_WildGoat_A_4116442401",
0x4757cbed,"MainField_Animal_WildGoat_A_4119977769",
0xd3811d66,"MainField_Animal_WildGoat_A_4122878095",
0x4fff9318,"MainField_Animal_WildGoat_A_4134690694",
0xa342a915,"MainField_Animal_WildGoat_A_4145237198",
0x44756dc2,"MainField_Animal_WildGoat_A_4198086078",
0x2bfd45ce,"MainField_Animal_WildGoat_A_4203135923",
0x05cc797f,"MainField_Animal_WildGoat_A_4219094026",
0x0ff57948,"MainField_Animal_WildGoat_A_4220634868",
0x723425e7,"MainField_Animal_WildGoat_A_4222761917",
0x9405b1f7,"MainField_Animal_WildGoat_A_4225178771",
0x1ac3228e,"MainField_Animal_WildGoat_A_4270321064",
0x3618c3b6,"MainField_Animal_WildGoat_A_436045046",
0xc8204b99,"MainField_Animal_WildGoat_A_452737929",
0xda82e6ff,"MainField_Animal_WildGoat_A_513174264",
0xb91cb69b,"MainField_Animal_WildGoat_A_514974110",
0xf36f5a3b,"MainField_Animal_WildGoat_A_52000273",
0x4868a8d5,"MainField_Animal_WildGoat_A_521811893",
0x0a8c8227,"MainField_Animal_WildGoat_A_535364794",
0xdece094e,"MainField_Animal_WildGoat_A_560156762",
0x4e80d7c9,"MainField_Animal_WildGoat_A_591957359",
0x36927ab2,"MainField_Animal_WildGoat_A_597896023",
0xddb59d76,"MainField_Animal_WildGoat_A_677587683",
0x8279a909,"MainField_Animal_WildGoat_A_726311208",
0xa6c5f663,"MainField_Animal_WildGoat_A_732029318",
0xdfc4f423,"MainField_Animal_WildGoat_A_738100851",
0xa02b3aea,"MainField_Animal_WildGoat_A_74442440",
0x6201a4a8,"MainField_Animal_WildGoat_A_816596967",
0x3dccd18c,"MainField_Animal_WildGoat_A_827794583",
0x8dcdbb96,"MainField_Animal_WildGoat_A_834932878",
0xf1bfc2f4,"MainField_Animal_WildGoat_A_858704215",
0x3454a386,"MainField_Animal_WildGoat_A_859463852",
0x5f7401cc,"MainField_Animal_WildGoat_A_906956052",
0x20d0007c,"MainField_Animal_WildGoat_A_921574680",
0x231b209a,"MainField_Animal_WildGoat_A_938963225",
0xafb6d317,"MainField_Animal_WildGoat_A_956136470",
0x4374c5a7,"MainField_Animal_WildGoat_A_973571578",
0xb37ff7eb,"MainField_Animal_WildGoat_A_986480384",
0xec0688f1,"MainField_Animal_Wolf_A_1193389058",
0x90f2a328,"MainField_Animal_Wolf_A_126756655",
0xa176d2c1,"MainField_Animal_Wolf_A_1316055695",
0xe38ce7d6,"MainField_Animal_Wolf_A_1451506093",
0x3df5ff4f,"MainField_Animal_Wolf_A_1456751029",
0x93a980d4,"MainField_Animal_Wolf_A_1802764063",
0x2c15a5b8,"MainField_Animal_Wolf_A_2055517325",
0x7d6890ea,"MainField_Animal_Wolf_A_2228179172",
0x70725a52,"MainField_Animal_Wolf_A_2313381292",
0x8e6e3cac,"MainField_Animal_Wolf_A_2779541002",
0x963bc138,"MainField_Animal_Wolf_A_2786271213",
0x8c164183,"MainField_Animal_Wolf_A_2980885403",
0xcfe004e5,"MainField_Animal_Wolf_A_3157610173",
0x438ddfa9,"MainField_Animal_Wolf_A_3244595410",
0xa8526b8d,"MainField_Animal_Wolf_A_3406064343",
0xb0556132,"MainField_Animal_Wolf_A_351137982",
0x9aa967ca,"MainField_Animal_Wolf_A_4000742383",
0x0ef4bb56,"MainField_Animal_Wolf_A_4184964152",
0x767bf122,"MainField_Animal_Wolf_A_59927604",
0x6eabeb2a,"MainField_Animal_Wolf_A_666726672",
0xd5066edb,"MainField_Animal_Wolf_B_1058924915",
0x5bc7b788,"MainField_Animal_Wolf_B_1070907512",
0xd50e0f6c,"MainField_Animal_Wolf_B_1273906622",
0x94b8af2f,"MainField_Animal_Wolf_B_1285789358",
0x35ca580e,"MainField_Animal_Wolf_B_1304823340",
0x6e094f53,"MainField_Animal_Wolf_B_1438192912",
0x984de5ec,"MainField_Animal_Wolf_B_1643304370",
0x66fe095f,"MainField_Animal_Wolf_B_1791709632",
0xb7a23f4f,"MainField_Animal_Wolf_B_1807859042",
0x9da024dc,"MainField_Animal_Wolf_B_1848547389",
0x38c82c35,"MainField_Animal_Wolf_B_1972975637",
0xcf26d312,"MainField_Animal_Wolf_B_2657455795",
0x983e70c5,"MainField_Animal_Wolf_B_275823699",
0x4b47b620,"MainField_Animal_Wolf_B_2813370463",
0x132971f0,"MainField_Animal_Wolf_B_2863383455",
0xf0530d55,"MainField_Animal_Wolf_B_2877911590",
0x0f8fe809,"MainField_Animal_Wolf_B_3505508952",
0x17976d78,"MainField_Animal_Wolf_B_3698285941",
0x9f786dba,"MainField_Animal_Wolf_B_3720058527",
0x15e489f0,"MainField_Animal_Wolf_B_3732247931",
0x5277db6c,"MainField_Animal_Wolf_B_3737808129",
0xd49d94e6,"MainField_Animal_Wolf_B_4097310329",
0x93f4c972,"MainField_Animal_Wolf_B_4101200456",
0x2955240e,"MainField_Animal_Wolf_B_453969014",
0x12d6c5d8,"MainField_Animal_Wolf_B_833407094",
0x4f416685,"MainField_Animal_Wolf_B_896944041",
0x418333c7,"MainField_Animal_Wolf_B_974343137",
0x652367ab,"MainField_Animal_Wolf_C_1017950559",
0x821d27d0,"MainField_Animal_Wolf_C_1197034359",
0x61b5bf6c,"MainField_Animal_Wolf_C_1228443136",
0xc39ff4c8,"MainField_Animal_Wolf_C_1548590157",
0x835f775d,"MainField_Animal_Wolf_C_1731925365",
0x46c2f2cc,"MainField_Animal_Wolf_C_1806648812",
0xb054eba6,"MainField_Animal_Wolf_C_1862065219",
0xc309f842,"MainField_Animal_Wolf_C_2052867342",
0x77899a6b,"MainField_Animal_Wolf_C_2207900473",
0x0b0a687e,"MainField_Animal_Wolf_C_2266276579",
0xe0aba110,"MainField_Animal_Wolf_C_2332715762",
0x9b0a39ed,"MainField_Animal_Wolf_C_2340035820",
0xa6032dde,"MainField_Animal_Wolf_C_2619917119",
0x5e902fd7,"MainField_Animal_Wolf_C_2635796599",
0xe5e9f0f3,"MainField_Animal_Wolf_C_2991516694",
0x95e8c528,"MainField_Animal_Wolf_C_3165181190",
0x06a7fc14,"MainField_Animal_Wolf_C_3339444325",
0xf21f1d7e,"MainField_Animal_Wolf_C_3985571404",
0x31fe5aae,"MainField_Animal_Wolf_C_4131699084",
0x217156ce,"MainField_Animal_Wolf_C_4254831967",
0x1b703403,"MainField_Animal_Wolf_C_488273270",
0xd3ca5e2d,"MainField_Animal_Wolf_C_630604005",
0x90bf17db,"MainField_Animal_Wolf_C_702777087",
0xe3bb0c64,"MainField_Animal_Wolf_C_849241978",
0x2c05e8a3,"MainField_BM_Relief_3124109991",
0xe7126a19,"MainField_Barrel_1024450405",
0xc3d4618c,"MainField_Barrel_1040138550",
0x4bb7f750,"MainField_Barrel_105398887",
0xa829be42,"MainField_Barrel_1056472954",
0xd3e1463b,"MainField_Barrel_1061849691",
0x4e358d0a,"MainField_Barrel_1062938084",
0x43da1b84,"MainField_Barrel_1063559761",
0xf1567518,"MainField_Barrel_1071201122",
0xc126aa43,"MainField_Barrel_1076222178",
0x35378097,"MainField_Barrel_1080976386",
0xf364ce0e,"MainField_Barrel_1102817770",
0xfac8e4a4,"MainField_Barrel_1102937316",
0x53e0f702,"MainField_Barrel_1108363414",
0xdefa1fe8,"MainField_Barrel_1117620414",
0x36cb1e9b,"MainField_Barrel_1117953172",
0xa1e71435,"MainField_Barrel_1118453050",
0x2603f945,"MainField_Barrel_1135171731",
0x9d1dfd15,"MainField_Barrel_1154009328",
0x2a6f4b38,"MainField_Barrel_1182665144",
0x2683b237,"MainField_Barrel_1192474611",
0x07c173ca,"MainField_Barrel_1203488604",
0x5fa57627,"MainField_Barrel_1213045296",
0xa1850c06,"MainField_Barrel_1226639324",
0x509ec6a7,"MainField_Barrel_1228949690",
0xb7bc3683,"MainField_Barrel_1242066260",
0x40bf9ac7,"MainField_Barrel_124620683",
0x76bcd3a3,"MainField_Barrel_1261399666",
0x52dd5e31,"MainField_Barrel_1268091597",
0x0bcf6b9f,"MainField_Barrel_1270691196",
0x2d82399f,"MainField_Barrel_1276076746",
0x64421ae2,"MainField_Barrel_1277046969",
0x6c659797,"MainField_Barrel_1323674062",
0x1a01c87c,"MainField_Barrel_1328643345",
0x1adbd2ad,"MainField_Barrel_1332223611",
0xa4633ed3,"MainField_Barrel_1340549137",
0xe8f09e1f,"MainField_Barrel_1347247984",
0x96385717,"MainField_Barrel_1355719852",
0xac866a41,"MainField_Barrel_137959209",
0xdc964c52,"MainField_Barrel_1383196756",
0x553d6167,"MainField_Barrel_1393788222",
0x52a97097,"MainField_Barrel_1397554922",
0xf2d854cc,"MainField_Barrel_1400816334",
0xdeba8f94,"MainField_Barrel_1404753032",
0xd0ef52b6,"MainField_Barrel_1407300837",
0x819abede,"MainField_Barrel_1428803065",
0x094176cf,"MainField_Barrel_1439145779",
0x31ab0f4d,"MainField_Barrel_1439575590",
0x9b2d2726,"MainField_Barrel_1477722089",
0xa938a284,"MainField_Barrel_1503104057",
0x5e891ab9,"MainField_Barrel_1507410704",
0x6bd298b9,"MainField_Barrel_152323683",
0xe86b2879,"MainField_Barrel_1530991267",
0x04ca73d7,"MainField_Barrel_1536374448",
0x64a87d49,"MainField_Barrel_1543251377",
0x29ffcc07,"MainField_Barrel_1565816909",
0x6cd17f7c,"MainField_Barrel_1587411431",
0x7e35765a,"MainField_Barrel_1588644451",
0x66724784,"MainField_Barrel_1591236084",
0xda3c4c86,"MainField_Barrel_1594120462",
0x6ba1bac5,"MainField_Barrel_1596792556",
0xefff70a6,"MainField_Barrel_1639138262",
0xee585c93,"MainField_Barrel_1646699976",
0xfba7c8c2,"MainField_Barrel_1659273384",
0x594e00e1,"MainField_Barrel_1673849808",
0x6da4f20e,"MainField_Barrel_1685731508",
0xc772053e,"MainField_Barrel_1688218056",
0x739fb9f5,"MainField_Barrel_1701987978",
0xbc2ba8da,"MainField_Barrel_1705202110",
0x7e7406d6,"MainField_Barrel_170659918",
0x77e5233d,"MainField_Barrel_1710097792",
0xac1c0289,"MainField_Barrel_1731701579",
0x0bb14a71,"MainField_Barrel_1735841815",
0xa1e98a77,"MainField_Barrel_1738135885",
0x1c60f2d3,"MainField_Barrel_1741443787",
0x9380b7a0,"MainField_Barrel_1742635720",
0xe1dfb9a3,"MainField_Barrel_1765323042",
0xb60a0aa5,"MainField_Barrel_1772794823",
0x9faade55,"MainField_Barrel_1773126638",
0xc1816fd8,"MainField_Barrel_178290985",
0xe4abe034,"MainField_Barrel_178367478",
0x5d4984d9,"MainField_Barrel_1789391592",
0xcd53ac68,"MainField_Barrel_1800492228",
0x6b3fdd43,"MainField_Barrel_1813930628",
0x7bd0cacc,"MainField_Barrel_1816440854",
0x347228fc,"MainField_Barrel_1820770018",
0x40948242,"MainField_Barrel_1833334509",
0x92a9281d,"MainField_Barrel_1846570987",
0x246adb73,"MainField_Barrel_1847027297",
0x615177f0,"MainField_Barrel_1851731776",
0x9a72a51e,"MainField_Barrel_1888185750",
0x2ada67ce,"MainField_Barrel_188892246",
0x609247fb,"MainField_Barrel_191533377",
0xd7ecbb45,"MainField_Barrel_1919353555",
0xd11b2d27,"MainField_Barrel_1931031877",
0xdcdc49e8,"MainField_Barrel_1947127115",
0x99e0c56e,"MainField_Barrel_1947779922",
0x0dd1ece9,"MainField_Barrel_1961816178",
0xac69a22e,"MainField_Barrel_1982052139",
0x979ffeda,"MainField_Barrel_1989646255",
0x74c6394b,"MainField_Barrel_201457534",
0xc8c1a1c7,"MainField_Barrel_2015273282",
0xf3e0fdb4,"MainField_Barrel_2015738715",
0x92d65c39,"MainField_Barrel_2033171305",
0x4218601c,"MainField_Barrel_2069316341",
0x12e6712d,"MainField_Barrel_2072039434",
0x5cf78608,"MainField_Barrel_2079351023",
0x86896eed,"MainField_Barrel_2080406954",
0x7d6b56f1,"MainField_Barrel_2090657509",
0x8a55c5b4,"MainField_Barrel_2091534420",
0xf8f1c56f,"MainField_Barrel_2100089275",
0x41bfaed0,"MainField_Barrel_2116845174",
0x5dbd33b8,"MainField_Barrel_212000724",
0x45caea2c,"MainField_Barrel_2121661180",
0x41089369,"MainField_Barrel_2135316446",
0xf2df2449,"MainField_Barrel_2141365419",
0x7a0c8bd4,"MainField_Barrel_2144823400",
0x6e9705fd,"MainField_Barrel_2146004346",
0xbbf23184,"MainField_Barrel_214813909",
0xc2a8fe41,"MainField_Barrel_2148686228",
0xc836c722,"MainField_Barrel_2164050262",
0x88c23fbc,"MainField_Barrel_2168182291",
0x079a3d39,"MainField_Barrel_2168284242",
0x8b9567e6,"MainField_Barrel_2172966545",
0xffb93d48,"MainField_Barrel_2173729187",
0x778261b5,"MainField_Barrel_2182305708",
0x2e77fb01,"MainField_Barrel_2194910464",
0xaba0ac3e,"MainField_Barrel_2227650717",
0xbe0ce262,"MainField_Barrel_223485815",
0x85c1706d,"MainField_Barrel_2235758247",
0x12c7b238,"MainField_Barrel_2258957427",
0xd3d59692,"MainField_Barrel_2270907377",
0xeec62cfb,"MainField_Barrel_227404697",
0x779a74b9,"MainField_Barrel_2288337154",
0xe2adc94c,"MainField_Barrel_2290211713",
0xadac7462,"MainField_Barrel_2304949487",
0x350f4145,"MainField_Barrel_2321807400",
0x5359901d,"MainField_Barrel_2382647651",
0x9d63e1e5,"MainField_Barrel_2382928906",
0x92c6944e,"MainField_Barrel_2385583636",
0xb406e4b9,"MainField_Barrel_2389991980",
0x906f16ec,"MainField_Barrel_2391211478",
0x04fc3c1e,"MainField_Barrel_2394572808",
0x4630a80c,"MainField_Barrel_2410385888",
0x9b373f35,"MainField_Barrel_2415398342",
0x921518e4,"MainField_Barrel_2420311418",
0x7b1ae7c7,"MainField_Barrel_2423197252",
0xd95b9129,"MainField_Barrel_2430505843",
0x579aaeed,"MainField_Barrel_2434818443",
0x44adaf46,"MainField_Barrel_2443577793",
0xac0a04ee,"MainField_Barrel_2444887568",
0xd2559be1,"MainField_Barrel_2456376669",
0x1c317779,"MainField_Barrel_2467340872",
0xb3b7e83e,"MainField_Barrel_2470722337",
0xba995f7f,"MainField_Barrel_2471943541",
0x59864de5,"MainField_Barrel_2496208819",
0xb20df15c,"MainField_Barrel_2508093393",
0xaf0b0a7a,"MainField_Barrel_2516953965",
0xe7cfe987,"MainField_Barrel_2518770440",
0x2d21d531,"MainField_Barrel_2523977946",
0xe9a3c56e,"MainField_Barrel_2530014759",
0x055ef622,"MainField_Barrel_2530953827",
0x4c4c3573,"MainField_Barrel_2536297891",
0x90e635e6,"MainField_Barrel_2539449297",
0x074426e9,"MainField_Barrel_2544934693",
0xa92c5ca6,"MainField_Barrel_2559753419",
0x9690b4ec,"MainField_Barrel_2574798015",
0x3f27bdea,"MainField_Barrel_2578232950",
0x57678dff,"MainField_Barrel_2578946774",
0x94a676ff,"MainField_Barrel_2581381675",
0x3ba4ba4e,"MainField_Barrel_2603637306",
0xffb7025e,"MainField_Barrel_260781203",
0xe85a07be,"MainField_Barrel_2631702607",
0x2e62dd01,"MainField_Barrel_2636905328",
0x68878c40,"MainField_Barrel_2643785769",
0xae29917b,"MainField_Barrel_2651819100",
0xfd5379a0,"MainField_Barrel_265292037",
0x05495f05,"MainField_Barrel_2656231358",
0x949fa2a6,"MainField_Barrel_2671568063",
0x362b4743,"MainField_Barrel_2674481620",
0xbc9836fb,"MainField_Barrel_2689360014",
0xbbf61556,"MainField_Barrel_269420092",
0xb5f05306,"MainField_Barrel_2696427887",
0xfb6601e3,"MainField_Barrel_2729867865",
0xfb85c54e,"MainField_Barrel_2730321663",
0x4a261c1e,"MainField_Barrel_2737414277",
0x286b524e,"MainField_Barrel_2764679812",
0x8829cd09,"MainField_Barrel_2776345635",
0x4e6c3151,"MainField_Barrel_2782098879",
0x99e7da34,"MainField_Barrel_2784909014",
0xf13059a2,"MainField_Barrel_2795513196",
0x207438af,"MainField_Barrel_2810339558",
0xe059f3fe,"MainField_Barrel_2821653134",
0x61367f9e,"MainField_Barrel_283718587",
0x8a8d10fc,"MainField_Barrel_2844815575",
0x3569ec0b,"MainField_Barrel_2845331889",
0x53a279a8,"MainField_Barrel_2846343412",
0xb141b82c,"MainField_Barrel_2847563918",
0x1cc6e1ce,"MainField_Barrel_2857540745",
0xc1b03768,"MainField_Barrel_2886240302",
0x1711978c,"MainField_Barrel_2892828006",
0xf6b9c445,"MainField_Barrel_2898426675",
0x12254518,"MainField_Barrel_2904113992",
0x45dd008b,"MainField_Barrel_2907806550",
0x2f3ce7d4,"MainField_Barrel_2910513723",
0x344e065e,"MainField_Barrel_2914292926",
0x6afaac63,"MainField_Barrel_2922334532",
0xdfa21b63,"MainField_Barrel_2922886632",
0x2321a2e2,"MainField_Barrel_2941172257",
0x5f1e8ba1,"MainField_Barrel_2942578987",
0x03cd78de,"MainField_Barrel_294745908",
0xccb52373,"MainField_Barrel_2953570140",
0x447ea0d3,"MainField_Barrel_2957819564",
0x23e5d126,"MainField_Barrel_2961057138",
0x3deeb5af,"MainField_Barrel_2964556417",
0x0c522e10,"MainField_Barrel_2968362348",
0x1b2bef26,"MainField_Barrel_2982443518",
0xd98266e8,"MainField_Barrel_2989102680",
0xc833bf2a,"MainField_Barrel_2994274344",
0x73f8416e,"MainField_Barrel_2997132762",
0x49fe0642,"MainField_Barrel_2998648881",
0x1edb7231,"MainField_Barrel_300086938",
0xf7b1ad9b,"MainField_Barrel_3001023074",
0x942cae4f,"MainField_Barrel_3016349778",
0xeb2ce0de,"MainField_Barrel_3016423867",
0xe9f859b3,"MainField_Barrel_3019961494",
0x19f7d03e,"MainField_Barrel_3040342420",
0x6d28a0ea,"MainField_Barrel_3048408716",
0x00b25078,"MainField_Barrel_3049681596",
0x659a7467,"MainField_Barrel_3075738922",
0x7e29197f,"MainField_Barrel_3102880364",
0x8040d6f4,"MainField_Barrel_3114173689",
0x3e1a88bd,"MainField_Barrel_3164617314",
0x404197e4,"MainField_Barrel_3173757664",
0xf0c9899a,"MainField_Barrel_3190260029",
0xf07003b4,"MainField_Barrel_3206144050",
0x1d2b09c6,"MainField_Barrel_3211729424",
0x2fbad6b9,"MainField_Barrel_3239770132",
0xddf5e5bf,"MainField_Barrel_3256764137",
0x75c2939e,"MainField_Barrel_3263852383",
0x3e8fbb1d,"MainField_Barrel_3278821758",
0x0ef9b78e,"MainField_Barrel_3292244406",
0xe66e3318,"MainField_Barrel_3301894340",
0xe2f688f6,"MainField_Barrel_3319353499",
0xe2e8f8fd,"MainField_Barrel_3320452914",
0xa8902d65,"MainField_Barrel_3322101893",
0x10ea6efb,"MainField_Barrel_3330885392",
0x3a76631f,"MainField_Barrel_3335075030",
0x1b1042bc,"MainField_Barrel_3340138403",
0x8ec0db41,"MainField_Barrel_3346108630",
0x78157158,"MainField_Barrel_3383493464",
0x54d2fa3d,"MainField_Barrel_3408876579",
0x65af5930,"MainField_Barrel_3410986985",
0xe498f39f,"MainField_Barrel_3414860456",
0x27a2369b,"MainField_Barrel_3417018047",
0x3d7f9070,"MainField_Barrel_3435878639",
0xa6f34f4c,"MainField_Barrel_3448535610",
0x5d0148f5,"MainField_Barrel_3451934258",
0x3ca3a112,"MainField_Barrel_3459230188",
0xf81c9413,"MainField_Barrel_3459819823",
0x6b199037,"MainField_Barrel_3467994902",
0x15b29ea5,"MainField_Barrel_3476336137",
0xb8bc4943,"MainField_Barrel_3476914511",
0x9b934616,"MainField_Barrel_3489281124",
0x17ba851d,"MainField_Barrel_3492125024",
0xe68ebab9,"MainField_Barrel_3493867",
0xe78b32cb,"MainField_Barrel_35074394",
0x0b30e8f7,"MainField_Barrel_3516931578",
0x38b754b8,"MainField_Barrel_3520000078",
0x68ecf432,"MainField_Barrel_3520878446",
0xaed0c413,"MainField_Barrel_3525176598",
0xd177bd48,"MainField_Barrel_3546019925",
0xee45c795,"MainField_Barrel_3548152280",
0xa8bc7184,"MainField_Barrel_3555156895",
0xcd488c4a,"MainField_Barrel_3568048120",
0xb8f0978a,"MainField_Barrel_3568219154",
0xd70d60d9,"MainField_Barrel_3571882079",
0x55f1de09,"MainField_Barrel_358687605",
0x87a9d507,"MainField_Barrel_3592730819",
0x14d5aeed,"MainField_Barrel_3599361441",
0x316ab8d3,"MainField_Barrel_359955145",
0x20e7e11a,"MainField_Barrel_3624961217",
0x6d7527dd,"MainField_Barrel_3627116296",
0xa3e43c6d,"MainField_Barrel_3636188950",
0x34498dac,"MainField_Barrel_3675408724",
0x77426989,"MainField_Barrel_3702626554",
0xd7b730bc,"MainField_Barrel_3703293491",
0x965bac3f,"MainField_Barrel_3709780824",
0x1718525b,"MainField_Barrel_372448000",
0x17bf0fa6,"MainField_Barrel_373285725",
0xc3ff0ddb,"MainField_Barrel_3766116576",
0x9c88b5a1,"MainField_Barrel_3770138784",
0x9e58e963,"MainField_Barrel_3772537703",
0xbb0fabe5,"MainField_Barrel_3775109178",
0x4c069d0e,"MainField_Barrel_3776442616",
0x255d3265,"MainField_Barrel_3776459809",
0x7f6845f6,"MainField_Barrel_3792216962",
0xe21a4c95,"MainField_Barrel_3792944261",
0x69a5591e,"MainField_Barrel_3796886247",
0xffec85f0,"MainField_Barrel_3798558707",
0x3bf83e9c,"MainField_Barrel_3806209676",
0x8ba75450,"MainField_Barrel_3810336411",
0x87be8f5b,"MainField_Barrel_3824203923",
0xe0df43a7,"MainField_Barrel_3830574563",
0x5b3d753a,"MainField_Barrel_3845321604",
0x82a114d4,"MainField_Barrel_3845990137",
0x5a649651,"MainField_Barrel_3863175982",
0x5861c14e,"MainField_Barrel_3864248122",
0xdbe27305,"MainField_Barrel_3869181100",
0x111b49f2,"MainField_Barrel_3889512135",
0x267f5204,"MainField_Barrel_3912155056",
0xfe869ab9,"MainField_Barrel_3952612933",
0xe9196959,"MainField_Barrel_3966916972",
0xd6cc05c9,"MainField_Barrel_3983790643",
0x66da2577,"MainField_Barrel_3984259040",
0x287a4479,"MainField_Barrel_3989511340",
0xa1f34926,"MainField_Barrel_4013064480",
0xebe44b0f,"MainField_Barrel_4013704245",
0x32893b3c,"MainField_Barrel_4017756860",
0xe79f61b2,"MainField_Barrel_4037245333",
0x2729734a,"MainField_Barrel_4054041829",
0xa62b5c3a,"MainField_Barrel_4055656441",
0x996876a8,"MainField_Barrel_4060871534",
0x60189819,"MainField_Barrel_4067046186",
0x52619237,"MainField_Barrel_4071909971",
0x488cce9f,"MainField_Barrel_4073971767",
0x2e692137,"MainField_Barrel_4085929314",
0x81d785d3,"MainField_Barrel_4086274822",
0x6cbd8f3f,"MainField_Barrel_4088435847",
0xb6b9d81a,"MainField_Barrel_4094888194",
0x06607b13,"MainField_Barrel_4095349073",
0xb2608b10,"MainField_Barrel_4100385985",
0x2c197597,"MainField_Barrel_4103129683",
0x3438578f,"MainField_Barrel_4117575715",
0x0996dc89,"MainField_Barrel_413108485",
0x091dca57,"MainField_Barrel_4131186309",
0xfca06a99,"MainField_Barrel_413770702",
0x87eb6b01,"MainField_Barrel_4145375490",
0xc6553fb6,"MainField_Barrel_4150559506",
0x53cb8e5a,"MainField_Barrel_4159396376",
0xe274819b,"MainField_Barrel_4176147027",
0xb17db2eb,"MainField_Barrel_4181347327",
0xa90e4f7c,"MainField_Barrel_419626292",
0xdee32af1,"MainField_Barrel_4196401159",
0x610a9f3f,"MainField_Barrel_4208050338",
0xb80f99c0,"MainField_Barrel_4210720599",
0xbf8a0c24,"MainField_Barrel_4212577065",
0x3af36ad2,"MainField_Barrel_4266418621",
0x742e1de6,"MainField_Barrel_4268067961",
0xa30476f6,"MainField_Barrel_4273356170",
0xd1f92b97,"MainField_Barrel_4275877012",
0xd42e3a8e,"MainField_Barrel_4280862850",
0x96680b81,"MainField_Barrel_4284502516",
0xda2b5d0b,"MainField_Barrel_430593568",
0xb542fbef,"MainField_Barrel_432490754",
0x7d6baca8,"MainField_Barrel_442433153",
0x74b66d61,"MainField_Barrel_447283510",
0x152a37e5,"MainField_Barrel_459235333",
0x69827b07,"MainField_Barrel_461853054",
0xae34614a,"MainField_Barrel_479036741",
0x6fdbd03b,"MainField_Barrel_491396392",
0x41c8a38c,"MainField_Barrel_498271515",
0x0ce3b948,"MainField_Barrel_521975514",
0x3eaf6b1e,"MainField_Barrel_522836996",
0xb2c6b51c,"MainField_Barrel_524734213",
0x6922c0d6,"MainField_Barrel_531473865",
0xad957e28,"MainField_Barrel_538293814",
0xd3db2b56,"MainField_Barrel_556783156",
0x9204f643,"MainField_Barrel_569303259",
0x8ef19563,"MainField_Barrel_573504879",
0x71b98a60,"MainField_Barrel_57973976",
0x413883ee,"MainField_Barrel_585095891",
0x625473bf,"MainField_Barrel_586582129",
0x14cf7c81,"MainField_Barrel_597050539",
0x4aa241bd,"MainField_Barrel_612377390",
0xc01d5506,"MainField_Barrel_613575846",
0x0f7e18cb,"MainField_Barrel_620162067",
0x244ac315,"MainField_Barrel_621382962",
0x67d14c23,"MainField_Barrel_657379463",
0x962747d7,"MainField_Barrel_659209241",
0xfb5c5ae1,"MainField_Barrel_669511910",
0xbc4e6ae3,"MainField_Barrel_673399210",
0x994a99de,"MainField_Barrel_687137312",
0x3562f392,"MainField_Barrel_689504188",
0xe4847e7e,"MainField_Barrel_697290145",
0x31dd6bda,"MainField_Barrel_714172472",
0xcab40cd4,"MainField_Barrel_717261288",
0x29e6d6f0,"MainField_Barrel_71953527",
0x2c401ce7,"MainField_Barrel_722122007",
0x0f556b67,"MainField_Barrel_724298178",
0x676b3430,"MainField_Barrel_749640330",
0xbbeebf6c,"MainField_Barrel_767070909",
0x360307a3,"MainField_Barrel_773988395",
0xe3aef774,"MainField_Barrel_784557933",
0x5b2f12a8,"MainField_Barrel_794117999",
0x27427b62,"MainField_Barrel_823908318",
0x949b3af6,"MainField_Barrel_824193664",
0x844ebf71,"MainField_Barrel_830998893",
0xe3b53729,"MainField_Barrel_8320185",
0xa1af66cb,"MainField_Barrel_848946025",
0x56cc75dc,"MainField_Barrel_865813676",
0x32a8e051,"MainField_Barrel_8887273",
0x309a4cc1,"MainField_Barrel_889189364",
0xb959a81f,"MainField_Barrel_906442399",
0x21713876,"MainField_Barrel_921687455",
0xf424e047,"MainField_Barrel_92691780",
0xc4a345e0,"MainField_Barrel_929649812",
0xcdd57022,"MainField_Barrel_952716776",
0xa159e74d,"MainField_Barrel_971151704",
0xbf2ba122,"MainField_Barrel_972677919",
0x80314686,"MainField_Barrel_979415924",
0x8426d5f9,"MainField_Barrel_984719129",
0x7911ccd7,"MainField_Barrel_990401116",
0x5707e1a5,"MainField_BeeHome_1076194132",
0xd544b785,"MainField_BeeHome_1088275729",
0x9a55325b,"MainField_BeeHome_1091750425",
0x031ca7b7,"MainField_BeeHome_1111890430",
0x4ddb41bd,"MainField_BeeHome_129653261",
0xe6d2f854,"MainField_BeeHome_1355505745",
0xfb477774,"MainField_BeeHome_1400218195",
0x08384f81,"MainField_BeeHome_1490676062",
0xb09d7dff,"MainField_BeeHome_1646711108",
0xd47c66d5,"MainField_BeeHome_1810570316",
0x6cfeae01,"MainField_BeeHome_1838065119",
0xcc8a204c,"MainField_BeeHome_184849824",
0x0f496ac0,"MainField_BeeHome_1962400798",
0x1550ae57,"MainField_BeeHome_1962900554",
0x7cf965bf,"MainField_BeeHome_1988040909",
0x6a100b7f,"MainField_BeeHome_2118244606",
0xcace878a,"MainField_BeeHome_2132223566",
0x7a316898,"MainField_BeeHome_2151567407",
0x1d31e6ee,"MainField_BeeHome_2213526003",
0x3afd9901,"MainField_BeeHome_2382400842",
0xc32a750a,"MainField_BeeHome_2448766232",
0x1824c3de,"MainField_BeeHome_2465947867",
0xbab85ca4,"MainField_BeeHome_2501084233",
0x629dfe84,"MainField_BeeHome_2553083030",
0xa5cf76f3,"MainField_BeeHome_2706480239",
0x114ac44b,"MainField_BeeHome_274404909",
0x8fefd8b0,"MainField_BeeHome_2769879532",
0x8bc7f8d8,"MainField_BeeHome_2881748579",
0xca26a97f,"MainField_BeeHome_3274258606",
0x091b94c2,"MainField_BeeHome_3284844482",
0x2676083b,"MainField_BeeHome_336319256",
0x80e5fa5f,"MainField_BeeHome_3499783877",
0x319adbf4,"MainField_BeeHome_3724094490",
0x861da7c9,"MainField_BeeHome_3784605052",
0x6d6077fe,"MainField_BeeHome_4179833251",
0x5c09663e,"MainField_BeeHome_478053870",
0x6e19b017,"MainField_BeeHome_557670415",
0xb08b2201,"MainField_BeeHome_559077427",
0x372dc916,"MainField_BeeHome_611488453",
0xa5983b47,"MainField_BeeHome_700726161",
0xe0bde37c,"MainField_BeeHome_810626933",
0x407e18bc,"MainField_BeeHome_893462888",
0xde31a71b,"MainField_BeeHome_893713798",
0xc1123155,"MainField_BeeHome_933974711",
0xcc8a00d0,"MainField_BeeHome_960820456",
0xa7110dc1,"MainField_BoarMeat_1446121669",
0xf1d67065,"MainField_BoarMeat_1818934544",
0xcb813873,"MainField_BoarMeat_1951849694",
0x1e05528e,"MainField_BoarMeat_2150063508",
0x39519a46,"MainField_BoarMeat_2225877814",
0x54cafc36,"MainField_BoarMeat_2576928824",
0x9c4abb7a,"MainField_BoarMeat_2757605970",
0x22bc5d85,"MainField_BoarMeat_2798303938",
0x52ae4f44,"MainField_BoarMeat_2885430640",
0xf149a6d8,"MainField_BoarMeat_3228208049",
0x32811c46,"MainField_BoarMeat_332021479",
0x9bd95880,"MainField_BoarMeat_3595233462",
0x6b8efc01,"MainField_BoarMeat_3922195894",
0x71c6306b,"MainField_BoarMeat_4191084566",
0x97baeacb,"MainField_BoarMeat_460164486",
0xb05ac365,"MainField_BoarMeat_484409767",
0xd2bcfff0,"MainField_BoarMeat_559455336",
0x7efe1a41,"MainField_BoarMeat_736523961",
0x1c5f1efc,"MainField_BoarMeat_853922484",
0x847b3d41,"MainField_BreakableRock_1968056599",
0x3edf834a,"MainField_DgnObj_DungeonEntranceSP_1529263976",
0xd1f0f4e1,"MainField_DgnObj_DungeonEntranceSP_1720603006",
0x7c55d7a6,"MainField_DgnObj_DungeonEntranceSP_184056157",
0x87343482,"MainField_DgnObj_DungeonEntranceSP_213340619",
0x915fa8cb,"MainField_DgnObj_DungeonEntranceSP_2214935753",
0xf7748cb7,"MainField_DgnObj_DungeonEntranceSP_2488434634",
0xc3870d96,"MainField_DgnObj_DungeonEntranceSP_2549283636",
0x8daa9fc0,"MainField_DgnObj_DungeonEntranceSP_2635992456",
0x9cd9a25d,"MainField_DgnObj_DungeonEntranceSP_270891272",
0x1cc20370,"MainField_DgnObj_DungeonEntranceSP_303983264",
0x84b65e2e,"MainField_DgnObj_DungeonEntranceSP_3073341123",
0xa9084939,"MainField_DgnObj_DungeonEntranceSP_3079604233",
0x850031a9,"MainField_DgnObj_DungeonEntranceSP_3165427376",
0xd67eb5e2,"MainField_DgnObj_DungeonEntranceSP_3310376302",
0x22ebcc97,"MainField_DgnObj_DungeonEntranceSP_3333021085",
0x00e721a8,"MainField_DgnObj_DungeonEntranceSP_3357761325",
0x30bcb3d3,"MainField_DgnObj_DungeonEntranceSP_3570653295",
0x36d76609,"MainField_DgnObj_DungeonEntranceSP_3571820507",
0x41a98480,"MainField_DgnObj_DungeonEntranceSP_4015690628",
0x8c1df5ff,"MainField_DgnObj_DungeonEntranceSP_4017574790",
0x67142c68,"MainField_DgnObj_DungeonEntranceSP_4134242930",
0xaacbd938,"MainField_DgnObj_DungeonEntranceSP_4288876538",
0x8a02d8eb,"MainField_DgnObj_DungeonEntranceSP_4290066134",
0xf9e239dc,"MainField_DgnObj_DungeonEntranceSP_593331572",
0x463ede30,"MainField_DgnObj_DungeonEntranceSP_709038909",
0x35696a85,"MainField_DgnObj_DungeonEntranceSP_846303989",
0x3ddd8183,"MainField_DgnObj_DungeonEntranceSP_Far_1029845736",
0x9e6df795,"MainField_DgnObj_DungeonEntranceSP_Far_1316997685",
0x65758898,"MainField_DgnObj_DungeonEntranceSP_Far_1512537447",
0x000810dd,"MainField_DgnObj_DungeonEntranceSP_Far_1792025272",
0x1d261633,"MainField_DgnObj_DungeonEntranceSP_Far_1862320351",
0x74f0f6e5,"MainField_DgnObj_DungeonEntranceSP_Far_2001547300",
0xd8784cc0,"MainField_DgnObj_DungeonEntranceSP_Far_2115675853",
0x5b6f7bed,"MainField_DgnObj_DungeonEntranceSP_Far_2330529139",
0xe2fabe4f,"MainField_DgnObj_DungeonEntranceSP_Far_2332368757",
0x9582a538,"MainField_DgnObj_DungeonEntranceSP_Far_2478420347",
0xc7e8655a,"MainField_DgnObj_DungeonEntranceSP_Far_2664810485",
0xec79d5ce,"MainField_DgnObj_DungeonEntranceSP_Far_2852243367",
0xccb514c9,"MainField_DgnObj_DungeonEntranceSP_Far_3037522000",
0xd20bc57f,"MainField_DgnObj_DungeonEntranceSP_Far_3041293672",
0xaf52a8af,"MainField_DgnObj_DungeonEntranceSP_Far_3278472275",
0x74c01f4e,"MainField_DgnObj_DungeonEntranceSP_Far_3483662496",
0x5027e13c,"MainField_DgnObj_DungeonEntranceSP_Far_3818887660",
0x3a1c9dba,"MainField_DgnObj_DungeonEntranceSP_Far_3840638060",
0xfebfeb4f,"MainField_DgnObj_DungeonEntranceSP_Far_3915323038",
0xeaed1296,"MainField_DgnObj_DungeonEntranceSP_Far_404656937",
0x0099b46e,"MainField_DgnObj_DungeonEntranceSP_Far_4073919162",
0x1eae0210,"MainField_DgnObj_DungeonEntranceSP_Far_4220741736",
0xa6602c5c,"MainField_DgnObj_DungeonEntranceSP_Far_444487905",
0xe31731cd,"MainField_DgnObj_DungeonEntranceSP_Far_542768630",
0xe94f0b85,"MainField_DgnObj_DungeonEntranceSP_Far_721641992",
0xa86d96e5,"MainField_DgnObj_DungeonEntranceSP_Far_87312205",
0xac562426,"MainField_DgnObj_EntranceTerminalSP_1210505854",
0x52083d0e,"MainField_DgnObj_EntranceTerminalSP_1330046348",
0x0f8b47d5,"MainField_DgnObj_EntranceTerminalSP_1516035980",
0xf2693607,"MainField_DgnObj_EntranceTerminalSP_1643664485",
0x468f0ca7,"MainField_DgnObj_EntranceTerminalSP_176831274",
0x1bb5d727,"MainField_DgnObj_EntranceTerminalSP_2032223352",
0x4ee8298b,"MainField_DgnObj_EntranceTerminalSP_2184730152",
0xfe0d3537,"MainField_DgnObj_EntranceTerminalSP_2370079789",
0xdc1e12ab,"MainField_DgnObj_EntranceTerminalSP_2393000454",
0x3a7dee13,"MainField_DgnObj_EntranceTerminalSP_2626776387",
0x42a0a993,"MainField_DgnObj_EntranceTerminalSP_2780456279",
0x88565ea5,"MainField_DgnObj_EntranceTerminalSP_2899796371",
0x74e4b209,"MainField_DgnObj_EntranceTerminalSP_291932570",
0xf2fb97d4,"MainField_DgnObj_EntranceTerminalSP_313892841",
0xdc6cdee3,"MainField_DgnObj_EntranceTerminalSP_328272491",
0x02c6fd11,"MainField_DgnObj_EntranceTerminalSP_3347288303",
0xbdefecff,"MainField_DgnObj_EntranceTerminalSP_3461027175",
0xa3e481a5,"MainField_DgnObj_EntranceTerminalSP_3584667787",
0xb965676c,"MainField_DgnObj_EntranceTerminalSP_3604459329",
0xa86b1e0f,"MainField_DgnObj_EntranceTerminalSP_3842882886",
0x07f0f33a,"MainField_DgnObj_EntranceTerminalSP_4105719795",
0xad405e06,"MainField_DgnObj_EntranceTerminalSP_4190848432",
0x466e4d26,"MainField_DgnObj_EntranceTerminalSP_60393576",
0x1f4d8f63,"MainField_DgnObj_EntranceTerminalSP_698893773",
0xa772c17b,"MainField_DgnObj_EntranceTerminalSP_863323544",
0x7523a352,"MainField_DgnObj_EntranceTerminalSP_956305386",
0xb9047c93,"MainField_DgnObj_EntranceTerminal_A_01_1087310173",
0x5c550f95,"MainField_DgnObj_EntranceTerminal_A_01_110215957",
0xc186c383,"MainField_DgnObj_EntranceTerminal_A_01_1244111545",
0x116579f3,"MainField_DgnObj_EntranceTerminal_A_01_1290401588",
0xf3bddc99,"MainField_DgnObj_EntranceTerminal_A_01_1293261921",
0xb7e0b14b,"MainField_DgnObj_EntranceTerminal_A_01_1295665254",
0x6b43263b,"MainField_DgnObj_EntranceTerminal_A_01_1327182578",
0x67215a3a,"MainField_DgnObj_EntranceTerminal_A_01_1379190536",
0xe7a577a5,"MainField_DgnObj_EntranceTerminal_A_01_1435385930",
0x713cc993,"MainField_DgnObj_EntranceTerminal_A_01_1466009179",
0x692a756a,"MainField_DgnObj_EntranceTerminal_A_01_146852692",
0xb247f3a7,"MainField_DgnObj_EntranceTerminal_A_01_1522959150",
0x1f485df7,"MainField_DgnObj_EntranceTerminal_A_01_1525806493",
0x06da9874,"MainField_DgnObj_EntranceTerminal_A_01_1551479024",
0xd3017a36,"MainField_DgnObj_EntranceTerminal_A_01_1609275571",
0xee5a8b31,"MainField_DgnObj_EntranceTerminal_A_01_1759577702",
0xc674fe77,"MainField_DgnObj_EntranceTerminal_A_01_1797433301",
0x8e2c2cd6,"MainField_DgnObj_EntranceTerminal_A_01_1810769294",
0xc6be180e,"MainField_DgnObj_EntranceTerminal_A_01_1870415265",
0x1a2fa3d5,"MainField_DgnObj_EntranceTerminal_A_01_1913096172",
0x74653c02,"MainField_DgnObj_EntranceTerminal_A_01_2041042482",
0x8b8cb961,"MainField_DgnObj_EntranceTerminal_A_01_211249082",
0x0f8b012d,"MainField_DgnObj_EntranceTerminal_A_01_2128477312",
0x1abfc4e9,"MainField_DgnObj_EntranceTerminal_A_01_2146079451",
0x3509fb06,"MainField_DgnObj_EntranceTerminal_A_01_2157175563",
0x57373442,"MainField_DgnObj_EntranceTerminal_A_01_219837396",
0xcc4ccfae,"MainField_DgnObj_EntranceTerminal_A_01_2263739210",
0x46dbd6b5,"MainField_DgnObj_EntranceTerminal_A_01_2361150520",
0x707af8c8,"MainField_DgnObj_EntranceTerminal_A_01_2368793903",
0x9fdf1e18,"MainField_DgnObj_EntranceTerminal_A_01_2439182591",
0x20a5295e,"MainField_DgnObj_EntranceTerminal_A_01_2462652737",
0xae24a0b4,"MainField_DgnObj_EntranceTerminal_A_01_2485505771",
0xfaa76f72,"MainField_DgnObj_EntranceTerminal_A_01_2501247945",
0x91c02c9f,"MainField_DgnObj_EntranceTerminal_A_01_2514161324",
0x55eaa4db,"MainField_DgnObj_EntranceTerminal_A_01_2521201461",
0xb6af1ccf,"MainField_DgnObj_EntranceTerminal_A_01_2536831938",
0xcdc9f6c9,"MainField_DgnObj_EntranceTerminal_A_01_2573353484",
0xd84ce010,"MainField_DgnObj_EntranceTerminal_A_01_2599970823",
0x7e74df73,"MainField_DgnObj_EntranceTerminal_A_01_2644864581",
0x0794d692,"MainField_DgnObj_EntranceTerminal_A_01_2663795146",
0x16c8db18,"MainField_DgnObj_EntranceTerminal_A_01_2700783948",
0xa064ddf5,"MainField_DgnObj_EntranceTerminal_A_01_273635496",
0xcccafcb7,"MainField_DgnObj_EntranceTerminal_A_01_2738104650",
0x3450e64c,"MainField_DgnObj_EntranceTerminal_A_01_2841673843",
0x06f2047b,"MainField_DgnObj_EntranceTerminal_A_01_2911064106",
0x9146f13c,"MainField_DgnObj_EntranceTerminal_A_01_3001101846",
0xb160be05,"MainField_DgnObj_EntranceTerminal_A_01_3024921265",
0xbc37d69c,"MainField_DgnObj_EntranceTerminal_A_01_3104333293",
0x301dd9be,"MainField_DgnObj_EntranceTerminal_A_01_3145329962",
0x6486fcc3,"MainField_DgnObj_EntranceTerminal_A_01_3157881131",
0x8f30c9e4,"MainField_DgnObj_EntranceTerminal_A_01_3163480604",
0xe4caf29e,"MainField_DgnObj_EntranceTerminal_A_01_3197543987",
0xc11cb014,"MainField_DgnObj_EntranceTerminal_A_01_3209194212",
0xf3f982bb,"MainField_DgnObj_EntranceTerminal_A_01_3285176980",
0xec9b279f,"MainField_DgnObj_EntranceTerminal_A_01_3293520908",
0xdf6870de,"MainField_DgnObj_EntranceTerminal_A_01_3304413740",
0x5583452b,"MainField_DgnObj_EntranceTerminal_A_01_3305154705",
0x01ffb737,"MainField_DgnObj_EntranceTerminal_A_01_3328811253",
0xd26dc112,"MainField_DgnObj_EntranceTerminal_A_01_335376859",
0xa0f0b664,"MainField_DgnObj_EntranceTerminal_A_01_3392545121",
0x039d6727,"MainField_DgnObj_EntranceTerminal_A_01_3405905026",
0x44186eac,"MainField_DgnObj_EntranceTerminal_A_01_3407310195",
0xd6f5c693,"MainField_DgnObj_EntranceTerminal_A_01_3439018499",
0x18f513ea,"MainField_DgnObj_EntranceTerminal_A_01_3449162454",
0x410edae4,"MainField_DgnObj_EntranceTerminal_A_01_3572795101",
0x56114d4d,"MainField_DgnObj_EntranceTerminal_A_01_3614581704",
0x948fdfbd,"MainField_DgnObj_EntranceTerminal_A_01_3643165353",
0x3cd8455a,"MainField_DgnObj_EntranceTerminal_A_01_3665924567",
0xc2859218,"MainField_DgnObj_EntranceTerminal_A_01_3688033788",
0xb1eb879d,"MainField_DgnObj_EntranceTerminal_A_01_3770475359",
0x3be967b9,"MainField_DgnObj_EntranceTerminal_A_01_3776697589",
0x7114c5a2,"MainField_DgnObj_EntranceTerminal_A_01_3808772271",
0x12bfadf3,"MainField_DgnObj_EntranceTerminal_A_01_3837933160",
0xa38d0ff9,"MainField_DgnObj_EntranceTerminal_A_01_3842018124",
0xad80c977,"MainField_DgnObj_EntranceTerminal_A_01_3896523552",
0x6095fc9d,"MainField_DgnObj_EntranceTerminal_A_01_4135503284",
0xa72a0de1,"MainField_DgnObj_EntranceTerminal_A_01_415403863",
0xf90179fc,"MainField_DgnObj_EntranceTerminal_A_01_4206060014",
0xee8044bb,"MainField_DgnObj_EntranceTerminal_A_01_4206609530",
0xda7c9b31,"MainField_DgnObj_EntranceTerminal_A_01_4223601667",
0x28ecc532,"MainField_DgnObj_EntranceTerminal_A_01_428694554",
0x335ef5e2,"MainField_DgnObj_EntranceTerminal_A_01_4290992176",
0x8e3966e1,"MainField_DgnObj_EntranceTerminal_A_01_4291616943",
0x0110118a,"MainField_DgnObj_EntranceTerminal_A_01_449814471",
0x1650975c,"MainField_DgnObj_EntranceTerminal_A_01_466947442",
0x3be7a973,"MainField_DgnObj_EntranceTerminal_A_01_484060157",
0x04250e4b,"MainField_DgnObj_EntranceTerminal_A_01_524989562",
0xbcf550e6,"MainField_DgnObj_EntranceTerminal_A_01_606405034",
0xa1920e6c,"MainField_DgnObj_EntranceTerminal_A_01_654731591",
0x3af7b275,"MainField_DgnObj_EntranceTerminal_A_01_742860559",
0xbd3f07de,"MainField_DgnObj_EntranceTerminal_A_01_773255150",
0x57d50ab3,"MainField_DgnObj_EntranceTerminal_A_01_775101124",
0x3abc175c,"MainField_DgnObj_EntranceTerminal_A_01_888069889",
0xf2ca584f,"MainField_DgnObj_EntranceTerminal_A_01_978426346",
0xaee128e6,"MainField_DgnObj_RemainsLithogragh_A_02_1301347618",
0x6db9268f,"MainField_DgnObj_RemainsLithogragh_A_02_1822262999",
0xc910bd8c,"MainField_DgnObj_RemainsLithogragh_A_02_2456751716",
0x87a71071,"MainField_DgnObj_RemainsLithogragh_A_02_260146947",
0x4ae69acb,"MainField_DgnObj_RemainsLithogragh_A_02_2753271720",
0xac02cec0,"MainField_DgnObj_RemainsLithogragh_A_02_2848817684",
0xb9eceb8c,"MainField_DgnObj_RemainsLithogragh_A_02_2938534245",
0xb2821a19,"MainField_DgnObj_RemainsLithogragh_A_02_3017486043",
0x74256eb3,"MainField_DgnObj_RemainsLithogragh_A_02_3302760013",
0xe0f0977d,"MainField_DgnObj_RemainsLithogragh_A_02_528063980",
0xa720d8bc,"MainField_DgnObj_RemainsLithogragh_A_02_789666109",
0xb78411a2,"MainField_DgnObj_RemainsLithograph_A_01_1075362475",
0xe214dfdf,"MainField_DgnObj_RemainsLithograph_A_01_1380247731",
0xde1e53b2,"MainField_DgnObj_RemainsLithograph_A_01_175874748",
0x594f5c2f,"MainField_DgnObj_RemainsLithograph_A_01_2533103206",
0xb4ed69cb,"MainField_DgnObj_RemainsLithograph_A_01_2815211088",
0xe4b0dbf5,"MainField_DgnObj_RemainsLithograph_A_01_3870706049",
0xecd6e492,"MainField_DgnObj_RemainsLithograph_A_01_418408075",
0x3075a2de,"MainField_DgnObj_RemainsLithograph_A_01_4233062118",
0x14e1da2f,"MainField_DgnObj_RemainsLithograph_A_01_44370897",
0x6b404d1d,"MainField_DgnObj_RemainsLithograph_A_01_557836451",
0xb5ed734c,"MainField_DgnObj_RemainsLithograph_A_01_923898119",
0x2d485138,"MainField_Enemy_Assassin_Middle_Quest_671216765",
0xf8cce5ff,"MainField_Enemy_Assassin_Senior_2126854204",
0x62f2486f,"MainField_Enemy_Assassin_Shooter_Junior_1108646018",
0x14b51b91,"MainField_Enemy_Assassin_Shooter_Junior_1430838647",
0xf1f24083,"MainField_Enemy_Assassin_Shooter_Junior_164050354",
0x4ec68b95,"MainField_Enemy_Assassin_Shooter_Junior_2276697612",
0xef3c1406,"MainField_Enemy_Assassin_Shooter_Junior_2656576667",
0x8223b2b0,"MainField_Enemy_Assassin_Shooter_Junior_2947563014",
0x8e8d447e,"MainField_Enemy_Assassin_Shooter_Junior_3059465361",
0xe18dd467,"MainField_Enemy_Assassin_Shooter_Junior_3321348326",
0x39356a30,"MainField_Enemy_Bokoblin_Bone_Junior_1058302183",
0xaf641b90,"MainField_Enemy_Bokoblin_Bone_Junior_110671315",
0x467f4784,"MainField_Enemy_Bokoblin_Bone_Junior_1172815045",
0xce4f1345,"MainField_Enemy_Bokoblin_Bone_Junior_1199028694",
0x4cad059c,"MainField_Enemy_Bokoblin_Bone_Junior_125132995",
0x06631c06,"MainField_Enemy_Bokoblin_Bone_Junior_1280645090",
0xf713b1c8,"MainField_Enemy_Bokoblin_Bone_Junior_1360093965",
0x04abcf09,"MainField_Enemy_Bokoblin_Bone_Junior_1386907804",
0xc377375f,"MainField_Enemy_Bokoblin_Bone_Junior_1440102021",
0x9e5433e9,"MainField_Enemy_Bokoblin_Bone_Junior_1457730130",
0x539d3f83,"MainField_Enemy_Bokoblin_Bone_Junior_1539531052",
0xb64ae309,"MainField_Enemy_Bokoblin_Bone_Junior_1596668682",
0xa664d514,"MainField_Enemy_Bokoblin_Bone_Junior_1681368831",
0xea355158,"MainField_Enemy_Bokoblin_Bone_Junior_1729595208",
0xdd7ab4f2,"MainField_Enemy_Bokoblin_Bone_Junior_1746021691",
0x65225500,"MainField_Enemy_Bokoblin_Bone_Junior_1749371295",
0x0e354760,"MainField_Enemy_Bokoblin_Bone_Junior_1805737410",
0xba819799,"MainField_Enemy_Bokoblin_Bone_Junior_1854848864",
0x04db6592,"MainField_Enemy_Bokoblin_Bone_Junior_186670059",
0x36056cb0,"MainField_Enemy_Bokoblin_Bone_Junior_1923775474",
0x6a60034a,"MainField_Enemy_Bokoblin_Bone_Junior_1952837341",
0x4a79ca8f,"MainField_Enemy_Bokoblin_Bone_Junior_2219770993",
0x0c3395b6,"MainField_Enemy_Bokoblin_Bone_Junior_2300175972",
0xf1f524a1,"MainField_Enemy_Bokoblin_Bone_Junior_2327009806",
0x4294078d,"MainField_Enemy_Bokoblin_Bone_Junior_2329875645",
0xcdb66d35,"MainField_Enemy_Bokoblin_Bone_Junior_2360887076",
0x1613c8f7,"MainField_Enemy_Bokoblin_Bone_Junior_2394648416",
0xe40492f2,"MainField_Enemy_Bokoblin_Bone_Junior_2452163153",
0xdf1d1aa1,"MainField_Enemy_Bokoblin_Bone_Junior_2515284034",
0x6501f06a,"MainField_Enemy_Bokoblin_Bone_Junior_2539282896",
0x7805c7e4,"MainField_Enemy_Bokoblin_Bone_Junior_2555201904",
0x6fccc794,"MainField_Enemy_Bokoblin_Bone_Junior_2560697258",
0xd0cb4cf6,"MainField_Enemy_Bokoblin_Bone_Junior_2584719279",
0x2bad0b10,"MainField_Enemy_Bokoblin_Bone_Junior_2588488396",
0xfe5bcf7d,"MainField_Enemy_Bokoblin_Bone_Junior_269531102",
0x40b32606,"MainField_Enemy_Bokoblin_Bone_Junior_2772297089",
0xd4dcb5fe,"MainField_Enemy_Bokoblin_Bone_Junior_2812793057",
0xe353274b,"MainField_Enemy_Bokoblin_Bone_Junior_2824562574",
0xebb7e558,"MainField_Enemy_Bokoblin_Bone_Junior_2924566726",
0x5cdc322b,"MainField_Enemy_Bokoblin_Bone_Junior_3126798942",
0x95317241,"MainField_Enemy_Bokoblin_Bone_Junior_3131719936",
0xb7597b36,"MainField_Enemy_Bokoblin_Bone_Junior_3207719296",
0x20c57ac8,"MainField_Enemy_Bokoblin_Bone_Junior_3207893704",
0x920e06eb,"MainField_Enemy_Bokoblin_Bone_Junior_3226114990",
0xd6a0ecce,"MainField_Enemy_Bokoblin_Bone_Junior_3232244706",
0x2c3a5cba,"MainField_Enemy_Bokoblin_Bone_Junior_3247244978",
0x0f97f77a,"MainField_Enemy_Bokoblin_Bone_Junior_3259018571",
0xd82a0bf6,"MainField_Enemy_Bokoblin_Bone_Junior_3340033859",
0x5568f784,"MainField_Enemy_Bokoblin_Bone_Junior_3386132984",
0x1b3d4e70,"MainField_Enemy_Bokoblin_Bone_Junior_3452000395",
0x1a33dae2,"MainField_Enemy_Bokoblin_Bone_Junior_348534063",
0xd2510171,"MainField_Enemy_Bokoblin_Bone_Junior_3499550191",
0x6c6b95b9,"MainField_Enemy_Bokoblin_Bone_Junior_3624992618",
0xe07be4dd,"MainField_Enemy_Bokoblin_Bone_Junior_3630970289",
0x138e6030,"MainField_Enemy_Bokoblin_Bone_Junior_363927198",
0xbc86b130,"MainField_Enemy_Bokoblin_Bone_Junior_3692795368",
0x774b41e2,"MainField_Enemy_Bokoblin_Bone_Junior_3820953329",
0xdfffca42,"MainField_Enemy_Bokoblin_Bone_Junior_3923428249",
0xecbfc569,"MainField_Enemy_Bokoblin_Bone_Junior_3924724382",
0xbbc4e62b,"MainField_Enemy_Bokoblin_Bone_Junior_3978421397",
0x1f378097,"MainField_Enemy_Bokoblin_Bone_Junior_3983798585",
0x3657a429,"MainField_Enemy_Bokoblin_Bone_Junior_408514009",
0xa56234ec,"MainField_Enemy_Bokoblin_Bone_Junior_4108157736",
0x4b9dde51,"MainField_Enemy_Bokoblin_Bone_Junior_4163836043",
0xffe6b2aa,"MainField_Enemy_Bokoblin_Bone_Junior_423143175",
0x0452474b,"MainField_Enemy_Bokoblin_Bone_Junior_4256459416",
0x8ff9eb2b,"MainField_Enemy_Bokoblin_Bone_Junior_4263433970",
0x0d116c8d,"MainField_Enemy_Bokoblin_Bone_Junior_4266966794",
0x42a5c34d,"MainField_Enemy_Bokoblin_Bone_Junior_466237002",
0xcc8b17e6,"MainField_Enemy_Bokoblin_Bone_Junior_484264628",
0xbf443a9b,"MainField_Enemy_Bokoblin_Bone_Junior_503528502",
0x4aa2f9bf,"MainField_Enemy_Bokoblin_Bone_Junior_574099397",
0xde553ff3,"MainField_Enemy_Bokoblin_Bone_Junior_635013852",
0xf59a5d1d,"MainField_Enemy_Bokoblin_Bone_Junior_751063482",
0xfe510621,"MainField_Enemy_Bokoblin_Bone_Junior_774296036",
0x34328e51,"MainField_Enemy_Bokoblin_Bone_Junior_840299716",
0x0f41acdf,"MainField_Enemy_Bokoblin_Bone_Junior_865811446",
0x0be6a228,"MainField_Enemy_Bokoblin_Bone_Junior_916290142",
0xc359a1c3,"MainField_Enemy_Bokoblin_Bone_Junior_988946681",
0x4c8e9bb2,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_1386921869",
0x49461143,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_2015083349",
0x0df4c58b,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_2034362192",
0xf0c80a25,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_2146143232",
0xcc9aa9f7,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_2373280731",
0x9ac64d4c,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_2449254871",
0xcdcbef89,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_2540756414",
0x312efd5d,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_2618908433",
0x1929141c,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_2723155190",
0xb91e4525,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_3009049476",
0x88d1de68,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_3623813729",
0x5e747032,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_3816422059",
0x15f26aac,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_3903428323",
0x5df5097e,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_3937963951",
0x6f804fac,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_416895860",
0x12f34664,"MainField_Enemy_Bokoblin_Bone_Junior_AllDay_440992068",
0x1825e72c,"MainField_Enemy_Bokoblin_Guard_Junior_1023880069",
0xe4cd1fe4,"MainField_Enemy_Bokoblin_Guard_Junior_107840627",
0xd054fe8e,"MainField_Enemy_Bokoblin_Guard_Junior_1090300090",
0xf956b8f0,"MainField_Enemy_Bokoblin_Guard_Junior_1201681337",
0xe7420c45,"MainField_Enemy_Bokoblin_Guard_Junior_1234828109",
0xdf6ba3b7,"MainField_Enemy_Bokoblin_Guard_Junior_1284012326",
0x818c71c7,"MainField_Enemy_Bokoblin_Guard_Junior_1449598780",
0x5c163352,"MainField_Enemy_Bokoblin_Guard_Junior_1465900130",
0x57b24e63,"MainField_Enemy_Bokoblin_Guard_Junior_1576638466",
0xdd6157dd,"MainField_Enemy_Bokoblin_Guard_Junior_159909367",
0x37cbdf57,"MainField_Enemy_Bokoblin_Guard_Junior_1626688977",
0xb02ae2b7,"MainField_Enemy_Bokoblin_Guard_Junior_1633375806",
0xd8ed37a0,"MainField_Enemy_Bokoblin_Guard_Junior_1674508860",
0x418efdfc,"MainField_Enemy_Bokoblin_Guard_Junior_1732119069",
0x99883052,"MainField_Enemy_Bokoblin_Guard_Junior_1744713586",
0x00b2cf82,"MainField_Enemy_Bokoblin_Guard_Junior_1824306896",
0xbcfadee3,"MainField_Enemy_Bokoblin_Guard_Junior_1842002486",
0x1a1f3609,"MainField_Enemy_Bokoblin_Guard_Junior_185019670",
0xef6f07e5,"MainField_Enemy_Bokoblin_Guard_Junior_1892649614",
0x3517f332,"MainField_Enemy_Bokoblin_Guard_Junior_1935756369",
0xbd50c9cd,"MainField_Enemy_Bokoblin_Guard_Junior_198804540",
0x8794ea62,"MainField_Enemy_Bokoblin_Guard_Junior_209694102",
0xea7a721c,"MainField_Enemy_Bokoblin_Guard_Junior_2099885743",
0x94e3fd77,"MainField_Enemy_Bokoblin_Guard_Junior_2162556710",
0xab081f11,"MainField_Enemy_Bokoblin_Guard_Junior_2241546783",
0xae9555ad,"MainField_Enemy_Bokoblin_Guard_Junior_2301577128",
0x09744ee3,"MainField_Enemy_Bokoblin_Guard_Junior_2365382684",
0xed72584c,"MainField_Enemy_Bokoblin_Guard_Junior_2412842669",
0x27733c73,"MainField_Enemy_Bokoblin_Guard_Junior_2519208480",
0x62343f4d,"MainField_Enemy_Bokoblin_Guard_Junior_2565752333",
0x20d57013,"MainField_Enemy_Bokoblin_Guard_Junior_2591487752",
0x14b447ed,"MainField_Enemy_Bokoblin_Guard_Junior_2652312798",
0xc52396e7,"MainField_Enemy_Bokoblin_Guard_Junior_2676677439",
0x4919e083,"MainField_Enemy_Bokoblin_Guard_Junior_2700762744",
0x002c7a8f,"MainField_Enemy_Bokoblin_Guard_Junior_2708071169",
0x250b3ca0,"MainField_Enemy_Bokoblin_Guard_Junior_2817263124",
0xca84a19e,"MainField_Enemy_Bokoblin_Guard_Junior_2817900430",
0x4b61c737,"MainField_Enemy_Bokoblin_Guard_Junior_2828151193",
0x22f056c2,"MainField_Enemy_Bokoblin_Guard_Junior_2879641406",
0xe3a1204e,"MainField_Enemy_Bokoblin_Guard_Junior_2972348245",
0x21870cd8,"MainField_Enemy_Bokoblin_Guard_Junior_2987219153",
0x4945cc47,"MainField_Enemy_Bokoblin_Guard_Junior_3052558583",
0xab31cc70,"MainField_Enemy_Bokoblin_Guard_Junior_3092879307",
0xd2fcea43,"MainField_Enemy_Bokoblin_Guard_Junior_3098201256",
0x3d961c8f,"MainField_Enemy_Bokoblin_Guard_Junior_3135846060",
0x0d31be4e,"MainField_Enemy_Bokoblin_Guard_Junior_3146996011",
0x5c1a3dbc,"MainField_Enemy_Bokoblin_Guard_Junior_3148904888",
0xe0b7d46b,"MainField_Enemy_Bokoblin_Guard_Junior_3191634263",
0x5a37c0d5,"MainField_Enemy_Bokoblin_Guard_Junior_3319860032",
0x7a42b695,"MainField_Enemy_Bokoblin_Guard_Junior_3376105921",
0x97dda6d9,"MainField_Enemy_Bokoblin_Guard_Junior_3399222830",
0xf56d9e50,"MainField_Enemy_Bokoblin_Guard_Junior_3499141879",
0xee28fcf4,"MainField_Enemy_Bokoblin_Guard_Junior_3667788602",
0x511d12ef,"MainField_Enemy_Bokoblin_Guard_Junior_3677093415",
0x129fa99c,"MainField_Enemy_Bokoblin_Guard_Junior_3743194910",
0x12ab92f9,"MainField_Enemy_Bokoblin_Guard_Junior_3749908010",
0xc6db6b48,"MainField_Enemy_Bokoblin_Guard_Junior_3764255989",
0x1bbe324d,"MainField_Enemy_Bokoblin_Guard_Junior_3826366479",
0xdf08d688,"MainField_Enemy_Bokoblin_Guard_Junior_387639707",
0xcdf4dfed,"MainField_Enemy_Bokoblin_Guard_Junior_3901615017",
0x9279bb70,"MainField_Enemy_Bokoblin_Guard_Junior_3996820229",
0xae1addab,"MainField_Enemy_Bokoblin_Guard_Junior_4013942664",
0x94ea53cb,"MainField_Enemy_Bokoblin_Guard_Junior_4016721203",
0x0b477fec,"MainField_Enemy_Bokoblin_Guard_Junior_4027585672",
0xf7a5df42,"MainField_Enemy_Bokoblin_Guard_Junior_4060618740",
0xd6315514,"MainField_Enemy_Bokoblin_Guard_Junior_4291604915",
0x2f99c1bc,"MainField_Enemy_Bokoblin_Guard_Junior_514872961",
0x2a63bbdb,"MainField_Enemy_Bokoblin_Guard_Junior_55249506",
0x7d5b1bab,"MainField_Enemy_Bokoblin_Guard_Junior_615786510",
0xb92c29f0,"MainField_Enemy_Bokoblin_Guard_Junior_680779429",
0x630092af,"MainField_Enemy_Bokoblin_Guard_Junior_731749771",
0x7b4c14e5,"MainField_Enemy_Bokoblin_Guard_Junior_786731763",
0x8d28f66b,"MainField_Enemy_Bokoblin_Guard_Junior_794315804",
0xd8e2e2e6,"MainField_Enemy_Bokoblin_Guard_Junior_862704338",
0x29062afb,"MainField_Enemy_Bokoblin_Guard_Junior_903357653",
0x1a1157bd,"MainField_Enemy_Bokoblin_Guard_Junior_Ambush_1127667894",
0x2c2ff0d5,"MainField_Enemy_Bokoblin_Guard_Junior_Ambush_1424992077",
0x739a170c,"MainField_Enemy_Bokoblin_Guard_Junior_Ambush_2892933474",
0xf6f7a46d,"MainField_Enemy_Bokoblin_Guard_Junior_Ambush_3014511817",
0xb66e03a3,"MainField_Enemy_Bokoblin_Guard_Junior_Ambush_3560976514",
0x51a3b7cf,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1080209254",
0xb8128149,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1183512225",
0x93c96e29,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_118954450",
0xa2ae40a4,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1263751526",
0x36731384,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1635694359",
0xd2ab3151,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2125177745",
0xd65529ad,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2136845834",
0xdd925fdd,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2522581781",
0x0296f04b,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2572492167",
0xd1b51610,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2683160382",
0x3b0804e0,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2779855843",
0xacd4dc53,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2819972064",
0x52b5afdb,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2889069050",
0x8440fc15,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2915271898",
0xb1355fb9,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_328847149",
0x9d78fcbc,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3475003021",
0x1bc5fc9b,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_4080357896",
0x3db13c70,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_4255805343",
0x5b61808b,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_4256565956",
0x94617ecc,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_470788393",
0x7b7d9f12,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_78639580",
0x6aa8ef78,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_788755615",
0x5811c360,"MainField_Enemy_Bokoblin_Guard_Middle_1013811738",
0xe0416c3f,"MainField_Enemy_Bokoblin_Guard_Middle_1199203567",
0xcd8d5d50,"MainField_Enemy_Bokoblin_Guard_Middle_1442779785",
0xa51b609e,"MainField_Enemy_Bokoblin_Guard_Middle_1532384765",
0x1dc80da8,"MainField_Enemy_Bokoblin_Guard_Middle_1561452682",
0x05fde170,"MainField_Enemy_Bokoblin_Guard_Middle_1828321815",
0xb357e814,"MainField_Enemy_Bokoblin_Guard_Middle_1886488924",
0xa59cc75c,"MainField_Enemy_Bokoblin_Guard_Middle_1906649873",
0x3f7bc92d,"MainField_Enemy_Bokoblin_Guard_Middle_1928348481",
0x538544cb,"MainField_Enemy_Bokoblin_Guard_Middle_1977324499",
0xd5bd2b0b,"MainField_Enemy_Bokoblin_Guard_Middle_2062111936",
0xbaa4392c,"MainField_Enemy_Bokoblin_Guard_Middle_241160302",
0x4a8d0cf0,"MainField_Enemy_Bokoblin_Guard_Middle_2455660787",
0xd240b8f2,"MainField_Enemy_Bokoblin_Guard_Middle_2635976784",
0x951206ed,"MainField_Enemy_Bokoblin_Guard_Middle_2795258147",
0x991d1fd0,"MainField_Enemy_Bokoblin_Guard_Middle_2855243552",
0x66c1a2ce,"MainField_Enemy_Bokoblin_Guard_Middle_2919267280",
0x0daef25c,"MainField_Enemy_Bokoblin_Guard_Middle_2973151238",
0x9efcd995,"MainField_Enemy_Bokoblin_Guard_Middle_3010857459",
0x095c8b6d,"MainField_Enemy_Bokoblin_Guard_Middle_3189152414",
0xf6d225bb,"MainField_Enemy_Bokoblin_Guard_Middle_3246787644",
0x484ab107,"MainField_Enemy_Bokoblin_Guard_Middle_3531396311",
0x438e239f,"MainField_Enemy_Bokoblin_Guard_Middle_3956589875",
0xe0a47e9b,"MainField_Enemy_Bokoblin_Guard_Middle_4027168957",
0xe48a413e,"MainField_Enemy_Bokoblin_Guard_Middle_4090427046",
0x9477803f,"MainField_Enemy_Bokoblin_Guard_Middle_4147206591",
0x5b359c36,"MainField_Enemy_Bokoblin_Guard_Middle_4182230508",
0xb46e9a9d,"MainField_Enemy_Bokoblin_Guard_Middle_449372901",
0xdb5ff5c0,"MainField_Enemy_Bokoblin_Guard_Middle_543914872",
0x3e28f6cd,"MainField_Enemy_Bokoblin_Guard_Middle_560511650",
0x3dd6af20,"MainField_Enemy_Bokoblin_Guard_Middle_780543934",
0xd497bce0,"MainField_Enemy_Bokoblin_Guard_Middle_822300682",
0x71e558ea,"MainField_Enemy_Bokoblin_Guard_Middle_851162926",
0xea60ff26,"MainField_Enemy_Bokoblin_Guard_Middle_954380264",
0xaaa8532b,"MainField_Enemy_Bokoblin_Guard_Middle_Ambush_1127407760",
0xef76e07a,"MainField_Enemy_Bokoblin_Guard_Middle_Ambush_2228793224",
0xe301e50d,"MainField_Enemy_Bokoblin_Guard_Middle_Ambush_2478711707",
0x538b7cdb,"MainField_Enemy_Bokoblin_Guard_Middle_Ambush_2723559686",
0x8306385c,"MainField_Enemy_Bokoblin_Guard_Middle_Ambush_3661345066",
0x95760e4a,"MainField_Enemy_Bokoblin_Guard_Middle_Ambush_609705216",
0x87f3c702,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1069458457",
0x390733f5,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1113357472",
0xb5dd946b,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1142211222",
0x846a6929,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1215521404",
0x3b02ed85,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1220121743",
0x0dedb461,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1253011801",
0xc57ae12f,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1406575917",
0x4b91a9d5,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1438426167",
0x3dde78be,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1450707008",
0xca6c7881,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1609063018",
0x7539b0e0,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1704933624",
0x528a29d0,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1753774584",
0xa6a1abac,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1915475430",
0x5a3ddaf4,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1920337170",
0x47566b06,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1972193600",
0xb2fb3154,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_1992271001",
0xba82bea0,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2093758563",
0xe968ce06,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2212841258",
0xd83b3d2b,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2350777073",
0x475839df,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2396315136",
0xfbc3c8b0,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2480879310",
0xeb7a8a5b,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_262021965",
0x3fee03f1,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2705502562",
0xa6091bde,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2740548315",
0xc3732302,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2748623453",
0x97cc0429,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2756246755",
0x401f2774,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2821676041",
0xde88a44b,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2850629881",
0x967b0133,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2904887688",
0xc85f2c57,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2968358968",
0xe6cd9d78,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_3012855551",
0x5251d003,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_3185801020",
0xaed99c22,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_3201407717",
0x7bf942da,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_3208430749",
0xedaa4e25,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_3408942335",
0xd6e7a2f6,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_3752955641",
0x5449afb0,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_3868394662",
0x9b4d054f,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_3976522554",
0x5d4b8560,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_4108482492",
0xdc7d2f57,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_4185470782",
0xd20c727b,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_582718625",
0x65848acc,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_591110349",
0x2d132cb5,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_681825716",
0x4489402c,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_696206435",
0xd2e4a8ec,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_71313383",
0xe288e3c0,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_728472959",
0x124e1f6f,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_864686182",
0x499ab34b,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_900218234",
0xe4025a36,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_91305348",
0x9b103a4c,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_914382311",
0xa306cb99,"MainField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_944105547",
0xdd5da33a,"MainField_Enemy_Bokoblin_Junior_1031691508",
0xa9f81081,"MainField_Enemy_Bokoblin_Junior_1070286140",
0x08f09214,"MainField_Enemy_Bokoblin_Junior_109283973",
0x65cf92c8,"MainField_Enemy_Bokoblin_Junior_1092895672",
0xaeb058ac,"MainField_Enemy_Bokoblin_Junior_1099140217",
0x9004f1c1,"MainField_Enemy_Bokoblin_Junior_1103584980",
0x19080168,"MainField_Enemy_Bokoblin_Junior_1109555399",
0x1057b125,"MainField_Enemy_Bokoblin_Junior_1112852429",
0x2ee0e5a5,"MainField_Enemy_Bokoblin_Junior_1125174225",
0xd9009fe1,"MainField_Enemy_Bokoblin_Junior_1133377874",
0xbba7b69d,"MainField_Enemy_Bokoblin_Junior_1151077216",
0x08ce0796,"MainField_Enemy_Bokoblin_Junior_1162979475",
0xde251098,"MainField_Enemy_Bokoblin_Junior_1182612283",
0x2645de0a,"MainField_Enemy_Bokoblin_Junior_1229857743",
0x61689794,"MainField_Enemy_Bokoblin_Junior_1230299319",
0x38313286,"MainField_Enemy_Bokoblin_Junior_1248430057",
0xdab36c8b,"MainField_Enemy_Bokoblin_Junior_1264579188",
0x649302e0,"MainField_Enemy_Bokoblin_Junior_1284218478",
0xc62b4467,"MainField_Enemy_Bokoblin_Junior_1289452563",
0x360a70ed,"MainField_Enemy_Bokoblin_Junior_1294616395",
0x20bf670a,"MainField_Enemy_Bokoblin_Junior_1296050365",
0x2eec81bc,"MainField_Enemy_Bokoblin_Junior_1296276071",
0x8dce3a78,"MainField_Enemy_Bokoblin_Junior_132089712",
0x6d99a208,"MainField_Enemy_Bokoblin_Junior_1338188329",
0x74422cad,"MainField_Enemy_Bokoblin_Junior_1351366914",
0xe77b96e5,"MainField_Enemy_Bokoblin_Junior_1355641488",
0x75233902,"MainField_Enemy_Bokoblin_Junior_1362709057",
0x09ac3cb2,"MainField_Enemy_Bokoblin_Junior_1384488657",
0x59880b0e,"MainField_Enemy_Bokoblin_Junior_1403815529",
0x77257cb8,"MainField_Enemy_Bokoblin_Junior_1429510255",
0x39bbee7f,"MainField_Enemy_Bokoblin_Junior_1438315280",
0xdd0c1209,"MainField_Enemy_Bokoblin_Junior_1477840888",
0x4f12fe50,"MainField_Enemy_Bokoblin_Junior_1478302003",
0x57490480,"MainField_Enemy_Bokoblin_Junior_1478704496",
0x186ee8d5,"MainField_Enemy_Bokoblin_Junior_1563464172",
0x8177aa03,"MainField_Enemy_Bokoblin_Junior_1588365142",
0xcfec3905,"MainField_Enemy_Bokoblin_Junior_1617670493",
0x1676a1a3,"MainField_Enemy_Bokoblin_Junior_1623879299",
0x73b60768,"MainField_Enemy_Bokoblin_Junior_1637323296",
0x837da1ce,"MainField_Enemy_Bokoblin_Junior_1650780151",
0xb3559537,"MainField_Enemy_Bokoblin_Junior_1668121676",
0xd2b5db4d,"MainField_Enemy_Bokoblin_Junior_171210282",
0x478c29f3,"MainField_Enemy_Bokoblin_Junior_1727237127",
0xae287c38,"MainField_Enemy_Bokoblin_Junior_173740639",
0x80e84589,"MainField_Enemy_Bokoblin_Junior_1739062525",
0xbf777815,"MainField_Enemy_Bokoblin_Junior_1748104237",
0xf8e6e08e,"MainField_Enemy_Bokoblin_Junior_1762986740",
0x48e7ed6c,"MainField_Enemy_Bokoblin_Junior_1808034356",
0xb427187a,"MainField_Enemy_Bokoblin_Junior_1809256376",
0x541c113b,"MainField_Enemy_Bokoblin_Junior_1849283355",
0x9e93913a,"MainField_Enemy_Bokoblin_Junior_1885307869",
0x1ad4e089,"MainField_Enemy_Bokoblin_Junior_1917352436",
0xf8a71da7,"MainField_Enemy_Bokoblin_Junior_1946959066",
0xa6c5f13b,"MainField_Enemy_Bokoblin_Junior_2010375149",
0x54874226,"MainField_Enemy_Bokoblin_Junior_2010392263",
0x5d2de13c,"MainField_Enemy_Bokoblin_Junior_2044643248",
0xfbdfbf1f,"MainField_Enemy_Bokoblin_Junior_2055094118",
0x6b4ab1da,"MainField_Enemy_Bokoblin_Junior_2056815794",
0x3e69d8f1,"MainField_Enemy_Bokoblin_Junior_2087614592",
0xc1a075b4,"MainField_Enemy_Bokoblin_Junior_2100512444",
0x9f08178c,"MainField_Enemy_Bokoblin_Junior_2100520590",
0x534e3ef3,"MainField_Enemy_Bokoblin_Junior_2131371552",
0x6c869b20,"MainField_Enemy_Bokoblin_Junior_2145478520",
0x0222e119,"MainField_Enemy_Bokoblin_Junior_2151389425",
0x1e762984,"MainField_Enemy_Bokoblin_Junior_2181188996",
0x4bf5ecb4,"MainField_Enemy_Bokoblin_Junior_2229237587",
0x00518f0b,"MainField_Enemy_Bokoblin_Junior_2239269747",
0xa70bf74c,"MainField_Enemy_Bokoblin_Junior_2271554321",
0xcdb34d03,"MainField_Enemy_Bokoblin_Junior_2296281886",
0x5d57d3d4,"MainField_Enemy_Bokoblin_Junior_2329894876",
0x4f6d8008,"MainField_Enemy_Bokoblin_Junior_2342281826",
0x99831957,"MainField_Enemy_Bokoblin_Junior_2349472725",
0x3771e944,"MainField_Enemy_Bokoblin_Junior_2357034192",
0x17a58e90,"MainField_Enemy_Bokoblin_Junior_2364792377",
0xfe6dee9c,"MainField_Enemy_Bokoblin_Junior_2409231116",
0x58d4c01b,"MainField_Enemy_Bokoblin_Junior_2458121559",
0xca794150,"MainField_Enemy_Bokoblin_Junior_2554891418",
0x6d1a07c5,"MainField_Enemy_Bokoblin_Junior_2555988607",
0x7bc1c4e7,"MainField_Enemy_Bokoblin_Junior_2584566407",
0x123d1d12,"MainField_Enemy_Bokoblin_Junior_2592356580",
0x8836c0dd,"MainField_Enemy_Bokoblin_Junior_2669828246",
0xdcfd69b7,"MainField_Enemy_Bokoblin_Junior_2716167837",
0x4e921072,"MainField_Enemy_Bokoblin_Junior_274706856",
0x9002cd27,"MainField_Enemy_Bokoblin_Junior_2750036023",
0x69cbdca5,"MainField_Enemy_Bokoblin_Junior_277007734",
0xdfe9b2ed,"MainField_Enemy_Bokoblin_Junior_2787713853",
0x6d50e2b1,"MainField_Enemy_Bokoblin_Junior_2810773673",
0x0245df27,"MainField_Enemy_Bokoblin_Junior_2839584475",
0xaeaaa70a,"MainField_Enemy_Bokoblin_Junior_2847933787",
0x0368835f,"MainField_Enemy_Bokoblin_Junior_2904546599",
0xa90a77e2,"MainField_Enemy_Bokoblin_Junior_2917425872",
0x8fe22178,"MainField_Enemy_Bokoblin_Junior_2963523003",
0x72886b31,"MainField_Enemy_Bokoblin_Junior_2973686139",
0x449cbcb7,"MainField_Enemy_Bokoblin_Junior_298823816",
0x6d39e81c,"MainField_Enemy_Bokoblin_Junior_3016140340",
0xc1b359ba,"MainField_Enemy_Bokoblin_Junior_301819567",
0xc722802d,"MainField_Enemy_Bokoblin_Junior_3022572588",
0x38df8bf3,"MainField_Enemy_Bokoblin_Junior_3033528943",
0x90f700b5,"MainField_Enemy_Bokoblin_Junior_3056122396",
0x5413f491,"MainField_Enemy_Bokoblin_Junior_3056763948",
0x4dadec40,"MainField_Enemy_Bokoblin_Junior_3059415303",
0xf3dea67e,"MainField_Enemy_Bokoblin_Junior_3104814373",
0x38050bc4,"MainField_Enemy_Bokoblin_Junior_3121696895",
0x3e85e7c0,"MainField_Enemy_Bokoblin_Junior_3129523435",
0xc4ff1aa9,"MainField_Enemy_Bokoblin_Junior_3156053062",
0xe487e351,"MainField_Enemy_Bokoblin_Junior_3172713636",
0xe05efbea,"MainField_Enemy_Bokoblin_Junior_31741041",
0x4aad57f1,"MainField_Enemy_Bokoblin_Junior_3182476913",
0x1246f730,"MainField_Enemy_Bokoblin_Junior_3198445206",
0xefbafb45,"MainField_Enemy_Bokoblin_Junior_322128821",
0x07d9f9f7,"MainField_Enemy_Bokoblin_Junior_323241358",
0x2fa8e9b4,"MainField_Enemy_Bokoblin_Junior_3277044986",
0x81d2c778,"MainField_Enemy_Bokoblin_Junior_328185575",
0x2f15a0ad,"MainField_Enemy_Bokoblin_Junior_3306852344",
0x68dba587,"MainField_Enemy_Bokoblin_Junior_3339539781",
0x83478007,"MainField_Enemy_Bokoblin_Junior_33426205",
0x8af83f2a,"MainField_Enemy_Bokoblin_Junior_334353976",
0xbb237512,"MainField_Enemy_Bokoblin_Junior_3343835479",
0x4441884f,"MainField_Enemy_Bokoblin_Junior_335083066",
0xed7f9f1b,"MainField_Enemy_Bokoblin_Junior_3368910095",
0x37040daa,"MainField_Enemy_Bokoblin_Junior_3382489104",
0x50bcc84e,"MainField_Enemy_Bokoblin_Junior_3436641902",
0x84f10e56,"MainField_Enemy_Bokoblin_Junior_3449816076",
0xcb6f0b5c,"MainField_Enemy_Bokoblin_Junior_3535639669",
0xa2a6bcae,"MainField_Enemy_Bokoblin_Junior_3566895504",
0x7d884aa9,"MainField_Enemy_Bokoblin_Junior_3584374624",
0x798eca70,"MainField_Enemy_Bokoblin_Junior_3641231808",
0x42e14014,"MainField_Enemy_Bokoblin_Junior_3649516712",
0xba5455d4,"MainField_Enemy_Bokoblin_Junior_3664044714",
0x713ec096,"MainField_Enemy_Bokoblin_Junior_3665891287",
0x67eae852,"MainField_Enemy_Bokoblin_Junior_3672330822",
0xf1d0c93e,"MainField_Enemy_Bokoblin_Junior_3713097319",
0x1b8facdb,"MainField_Enemy_Bokoblin_Junior_3721951060",
0x66d3d884,"MainField_Enemy_Bokoblin_Junior_3724314614",
0x98f81541,"MainField_Enemy_Bokoblin_Junior_3740849117",
0xb539b9cc,"MainField_Enemy_Bokoblin_Junior_3747626617",
0xadd9ab89,"MainField_Enemy_Bokoblin_Junior_3752446980",
0xcfc7c7c0,"MainField_Enemy_Bokoblin_Junior_3792560744",
0xc5da6274,"MainField_Enemy_Bokoblin_Junior_3810198838",
0x8069dce4,"MainField_Enemy_Bokoblin_Junior_3850683841",
0x41de6143,"MainField_Enemy_Bokoblin_Junior_3865087168",
0xe904d5c6,"MainField_Enemy_Bokoblin_Junior_3914440242",
0x07d193bf,"MainField_Enemy_Bokoblin_Junior_3932727673",
0x7177d7e2,"MainField_Enemy_Bokoblin_Junior_3948760319",
0x498844c9,"MainField_Enemy_Bokoblin_Junior_3953795061",
0xc1c64ac9,"MainField_Enemy_Bokoblin_Junior_3991821019",
0xd47cc477,"MainField_Enemy_Bokoblin_Junior_3996532934",
0xc05e20b5,"MainField_Enemy_Bokoblin_Junior_4015946473",
0x4b73aea1,"MainField_Enemy_Bokoblin_Junior_4032898651",
0x93fb105e,"MainField_Enemy_Bokoblin_Junior_4069373007",
0xbe862451,"MainField_Enemy_Bokoblin_Junior_4075182467",
0x61fc9f12,"MainField_Enemy_Bokoblin_Junior_4098417800",
0xf94616c2,"MainField_Enemy_Bokoblin_Junior_4100606948",
0xf74554b9,"MainField_Enemy_Bokoblin_Junior_4120682356",
0x6615f80f,"MainField_Enemy_Bokoblin_Junior_4171154330",
0xde24edab,"MainField_Enemy_Bokoblin_Junior_4180656335",
0x3bdfb5c4,"MainField_Enemy_Bokoblin_Junior_4201407767",
0x12ab41b7,"MainField_Enemy_Bokoblin_Junior_420427774",
0x66801385,"MainField_Enemy_Bokoblin_Junior_4211944259",
0xbf66d494,"MainField_Enemy_Bokoblin_Junior_4215773784",
0xfde5dffd,"MainField_Enemy_Bokoblin_Junior_422758731",
0x7d69c9e1,"MainField_Enemy_Bokoblin_Junior_4244742414",
0xeb15b8a0,"MainField_Enemy_Bokoblin_Junior_4265974524",
0x899f2c22,"MainField_Enemy_Bokoblin_Junior_4268364423",
0x2faa4ca3,"MainField_Enemy_Bokoblin_Junior_439909900",
0x5f44d4cb,"MainField_Enemy_Bokoblin_Junior_460115728",
0x798dc530,"MainField_Enemy_Bokoblin_Junior_475707588",
0x73e286c6,"MainField_Enemy_Bokoblin_Junior_48063290",
0x4f82b4a6,"MainField_Enemy_Bokoblin_Junior_486354635",
0xc7981de1,"MainField_Enemy_Bokoblin_Junior_486520671",
0x5aa4bfad,"MainField_Enemy_Bokoblin_Junior_517850908",
0xb0191a1b,"MainField_Enemy_Bokoblin_Junior_549822702",
0xb35f0392,"MainField_Enemy_Bokoblin_Junior_613479287",
0xf6ad2591,"MainField_Enemy_Bokoblin_Junior_631350222",
0x8ac1a3e0,"MainField_Enemy_Bokoblin_Junior_675796627",
0xd4f2a057,"MainField_Enemy_Bokoblin_Junior_685720050",
0x95c4f980,"MainField_Enemy_Bokoblin_Junior_709646735",
0x3da70336,"MainField_Enemy_Bokoblin_Junior_782754550",
0x7e2ed4fc,"MainField_Enemy_Bokoblin_Junior_80692187",
0x59e8d8b1,"MainField_Enemy_Bokoblin_Junior_844535813",
0x790c67c7,"MainField_Enemy_Bokoblin_Junior_904172580",
0xdb556c4a,"MainField_Enemy_Bokoblin_Junior_910707400",
0x5924fb1d,"MainField_Enemy_Bokoblin_Junior_940799969",
0x3f145d0e,"MainField_Enemy_Bokoblin_Middle_1011464761",
0x41769c2f,"MainField_Enemy_Bokoblin_Middle_1015522689",
0x6b5bbd0f,"MainField_Enemy_Bokoblin_Middle_1015670154",
0xc0a57a24,"MainField_Enemy_Bokoblin_Middle_1027245261",
0x28275d2a,"MainField_Enemy_Bokoblin_Middle_1040659265",
0x8297ffb1,"MainField_Enemy_Bokoblin_Middle_1043246721",
0x437d9740,"MainField_Enemy_Bokoblin_Middle_1051203749",
0xc82acf76,"MainField_Enemy_Bokoblin_Middle_1059947333",
0xb0942dd8,"MainField_Enemy_Bokoblin_Middle_1062355950",
0x2a0cd90b,"MainField_Enemy_Bokoblin_Middle_1070219736",
0x4417f420,"MainField_Enemy_Bokoblin_Middle_1070864093",
0x260f9beb,"MainField_Enemy_Bokoblin_Middle_1103836040",
0x2d5d13c3,"MainField_Enemy_Bokoblin_Middle_1106240555",
0xff1943c2,"MainField_Enemy_Bokoblin_Middle_1139672194",
0x8213cdc3,"MainField_Enemy_Bokoblin_Middle_1163222318",
0x794a8998,"MainField_Enemy_Bokoblin_Middle_1164501735",
0x0af285e7,"MainField_Enemy_Bokoblin_Middle_1167117503",
0xdfa08d33,"MainField_Enemy_Bokoblin_Middle_1186162709",
0x7f3915bf,"MainField_Enemy_Bokoblin_Middle_1187073520",
0xc8d4923a,"MainField_Enemy_Bokoblin_Middle_1253931799",
0x37c19ddb,"MainField_Enemy_Bokoblin_Middle_1267651868",
0x22ec8aed,"MainField_Enemy_Bokoblin_Middle_1280311224",
0x40a3dc9a,"MainField_Enemy_Bokoblin_Middle_131050949",
0x79294b43,"MainField_Enemy_Bokoblin_Middle_1320807407",
0xd562a8bc,"MainField_Enemy_Bokoblin_Middle_1342135723",
0x72831cdf,"MainField_Enemy_Bokoblin_Middle_1360983144",
0x4882d716,"MainField_Enemy_Bokoblin_Middle_1385555037",
0xd7bdb4c2,"MainField_Enemy_Bokoblin_Middle_1402318242",
0xed1c08f5,"MainField_Enemy_Bokoblin_Middle_1415205181",
0xfcfde924,"MainField_Enemy_Bokoblin_Middle_1443097011",
0x27c2e242,"MainField_Enemy_Bokoblin_Middle_1466630772",
0x3e6409d4,"MainField_Enemy_Bokoblin_Middle_1528251673",
0xcf64c002,"MainField_Enemy_Bokoblin_Middle_1534948559",
0xc815cc72,"MainField_Enemy_Bokoblin_Middle_1546795223",
0x4421f5d4,"MainField_Enemy_Bokoblin_Middle_1550472069",
0x13cc0bfd,"MainField_Enemy_Bokoblin_Middle_1551356620",
0xd9d581b1,"MainField_Enemy_Bokoblin_Middle_1553986020",
0x744f3542,"MainField_Enemy_Bokoblin_Middle_1568430447",
0x48c7aa3b,"MainField_Enemy_Bokoblin_Middle_1579855444",
0x3e009526,"MainField_Enemy_Bokoblin_Middle_1592734532",
0x70abcb7e,"MainField_Enemy_Bokoblin_Middle_1593546409",
0x8f080290,"MainField_Enemy_Bokoblin_Middle_1607020727",
0x378ec8ef,"MainField_Enemy_Bokoblin_Middle_1624090318",
0x4e79e05f,"MainField_Enemy_Bokoblin_Middle_1642113969",
0xf90eb90f,"MainField_Enemy_Bokoblin_Middle_1642347852",
0x7991fe44,"MainField_Enemy_Bokoblin_Middle_1646276122",
0x1166ad9b,"MainField_Enemy_Bokoblin_Middle_1675407094",
0x1ca069f1,"MainField_Enemy_Bokoblin_Middle_1698577073",
0x3b864da4,"MainField_Enemy_Bokoblin_Middle_1706910284",
0x5227665a,"MainField_Enemy_Bokoblin_Middle_1710903830",
0x403985f1,"MainField_Enemy_Bokoblin_Middle_1721531942",
0x3bec4900,"MainField_Enemy_Bokoblin_Middle_1721654732",
0x00863d47,"MainField_Enemy_Bokoblin_Middle_1722172746",
0x560fcb32,"MainField_Enemy_Bokoblin_Middle_1722227595",
0xcc0542bd,"MainField_Enemy_Bokoblin_Middle_1723124751",
0x90d7eb83,"MainField_Enemy_Bokoblin_Middle_1730566048",
0x87b03d8e,"MainField_Enemy_Bokoblin_Middle_1751837898",
0xcd8bea73,"MainField_Enemy_Bokoblin_Middle_1761364823",
0xbf052ec3,"MainField_Enemy_Bokoblin_Middle_1777496793",
0xfa5497be,"MainField_Enemy_Bokoblin_Middle_1790138682",
0x4dd53ef9,"MainField_Enemy_Bokoblin_Middle_1793810292",
0x03399efa,"MainField_Enemy_Bokoblin_Middle_1796188467",
0x507a1c1e,"MainField_Enemy_Bokoblin_Middle_1800631469",
0xe56f0d2d,"MainField_Enemy_Bokoblin_Middle_1808084475",
0xf8f49ca5,"MainField_Enemy_Bokoblin_Middle_1841131066",
0xf56beb65,"MainField_Enemy_Bokoblin_Middle_1842698976",
0xde5948db,"MainField_Enemy_Bokoblin_Middle_1868231484",
0x198d71be,"MainField_Enemy_Bokoblin_Middle_1883104002",
0xfbae08e6,"MainField_Enemy_Bokoblin_Middle_1897657612",
0x74fb37e2,"MainField_Enemy_Bokoblin_Middle_1898781307",
0xa7b610ec,"MainField_Enemy_Bokoblin_Middle_1913314230",
0xbbb5d897,"MainField_Enemy_Bokoblin_Middle_191510122",
0xe1e87e3a,"MainField_Enemy_Bokoblin_Middle_1918424871",
0x06a74a15,"MainField_Enemy_Bokoblin_Middle_1919525600",
0x280a63d0,"MainField_Enemy_Bokoblin_Middle_1938893969",
0x2ebf1023,"MainField_Enemy_Bokoblin_Middle_1940439370",
0x5943e83c,"MainField_Enemy_Bokoblin_Middle_196362379",
0xe8ee9516,"MainField_Enemy_Bokoblin_Middle_1990468811",
0x1dd1047a,"MainField_Enemy_Bokoblin_Middle_1999779604",
0x14282597,"MainField_Enemy_Bokoblin_Middle_2014415266",
0x2b442a03,"MainField_Enemy_Bokoblin_Middle_2020294572",
0x6de6c369,"MainField_Enemy_Bokoblin_Middle_2041897679",
0xd4cb92cf,"MainField_Enemy_Bokoblin_Middle_2055079404",
0xab835a11,"MainField_Enemy_Bokoblin_Middle_2059498382",
0xf1e90024,"MainField_Enemy_Bokoblin_Middle_2083791362",
0x8343093d,"MainField_Enemy_Bokoblin_Middle_2086079447",
0xc6f20a89,"MainField_Enemy_Bokoblin_Middle_2092134429",
0x17c59615,"MainField_Enemy_Bokoblin_Middle_2098083749",
0x2aeb0a21,"MainField_Enemy_Bokoblin_Middle_211559169",
0xd5aaba15,"MainField_Enemy_Bokoblin_Middle_2120870626",
0x5de2df70,"MainField_Enemy_Bokoblin_Middle_2127113634",
0xef1d3564,"MainField_Enemy_Bokoblin_Middle_2138612159",
0x450da42a,"MainField_Enemy_Bokoblin_Middle_2153855979",
0x49b542e9,"MainField_Enemy_Bokoblin_Middle_215581842",
0x40fc3c32,"MainField_Enemy_Bokoblin_Middle_2159613305",
0x41c74028,"MainField_Enemy_Bokoblin_Middle_2168617790",
0xaf4ef403,"MainField_Enemy_Bokoblin_Middle_217659729",
0x5b293bb5,"MainField_Enemy_Bokoblin_Middle_218991381",
0x87a19d5e,"MainField_Enemy_Bokoblin_Middle_219076688",
0x675ad8df,"MainField_Enemy_Bokoblin_Middle_2207749110",
0x353e7ac5,"MainField_Enemy_Bokoblin_Middle_223282845",
0xf2ee7920,"MainField_Enemy_Bokoblin_Middle_2237470081",
0xd6b45eeb,"MainField_Enemy_Bokoblin_Middle_224074490",
0x2dcb1be1,"MainField_Enemy_Bokoblin_Middle_2247680276",
0x293c4eb0,"MainField_Enemy_Bokoblin_Middle_2259632802",
0xbaa9076c,"MainField_Enemy_Bokoblin_Middle_226047354",
0x029e2621,"MainField_Enemy_Bokoblin_Middle_2265198937",
0xe590b1c8,"MainField_Enemy_Bokoblin_Middle_2288117225",
0x39831c9f,"MainField_Enemy_Bokoblin_Middle_2304593766",
0x65cc1eb4,"MainField_Enemy_Bokoblin_Middle_2307325139",
0x2de522c0,"MainField_Enemy_Bokoblin_Middle_2309224492",
0xee88b6ae,"MainField_Enemy_Bokoblin_Middle_2311097034",
0x013e676f,"MainField_Enemy_Bokoblin_Middle_2322393558",
0x4fd9f319,"MainField_Enemy_Bokoblin_Middle_2337498919",
0xaba06cd3,"MainField_Enemy_Bokoblin_Middle_2346821368",
0x1335da8e,"MainField_Enemy_Bokoblin_Middle_2356207998",
0xfa86ede6,"MainField_Enemy_Bokoblin_Middle_2372621697",
0xdede32fd,"MainField_Enemy_Bokoblin_Middle_2382087900",
0x4b1c852c,"MainField_Enemy_Bokoblin_Middle_2383034335",
0xbd974346,"MainField_Enemy_Bokoblin_Middle_2387082804",
0xeb397f6b,"MainField_Enemy_Bokoblin_Middle_2404622425",
0x3babdc35,"MainField_Enemy_Bokoblin_Middle_2407415524",
0xe4f2ccc3,"MainField_Enemy_Bokoblin_Middle_2418124650",
0xb402f386,"MainField_Enemy_Bokoblin_Middle_241891525",
0x5024cd2e,"MainField_Enemy_Bokoblin_Middle_2424261588",
0xc1377fbd,"MainField_Enemy_Bokoblin_Middle_2449366576",
0x6bc295d1,"MainField_Enemy_Bokoblin_Middle_2464594577",
0x25a69679,"MainField_Enemy_Bokoblin_Middle_2465499500",
0x14374c41,"MainField_Enemy_Bokoblin_Middle_2468192473",
0xe04e75a5,"MainField_Enemy_Bokoblin_Middle_2490393792",
0x5d7ae12f,"MainField_Enemy_Bokoblin_Middle_2522624653",
0x57a963f9,"MainField_Enemy_Bokoblin_Middle_2534934817",
0x404bc359,"MainField_Enemy_Bokoblin_Middle_2536925146",
0x17b6bc9e,"MainField_Enemy_Bokoblin_Middle_2544652103",
0x29ba09be,"MainField_Enemy_Bokoblin_Middle_2556820890",
0x7fba9efa,"MainField_Enemy_Bokoblin_Middle_2570684564",
0xad8bb760,"MainField_Enemy_Bokoblin_Middle_2571443871",
0xe1429955,"MainField_Enemy_Bokoblin_Middle_2577879478",
0x120d3d87,"MainField_Enemy_Bokoblin_Middle_2579319404",
0x141b0f1f,"MainField_Enemy_Bokoblin_Middle_2621842624",
0x5b5a1dae,"MainField_Enemy_Bokoblin_Middle_2633026988",
0x788e69a5,"MainField_Enemy_Bokoblin_Middle_2649015914",
0x077edece,"MainField_Enemy_Bokoblin_Middle_2656111645",
0x37508d6d,"MainField_Enemy_Bokoblin_Middle_2691348393",
0x57f333ea,"MainField_Enemy_Bokoblin_Middle_2703454877",
0xf889dd57,"MainField_Enemy_Bokoblin_Middle_2705553678",
0xd7c3e9ed,"MainField_Enemy_Bokoblin_Middle_2724363349",
0x1eae6682,"MainField_Enemy_Bokoblin_Middle_2735075261",
0x88297ea0,"MainField_Enemy_Bokoblin_Middle_2737055335",
0xba564098,"MainField_Enemy_Bokoblin_Middle_2746629006",
0xc2b8a649,"MainField_Enemy_Bokoblin_Middle_2777108519",
0xc8936fe5,"MainField_Enemy_Bokoblin_Middle_2786679204",
0x9e3ae647,"MainField_Enemy_Bokoblin_Middle_2789606978",
0x4c27a9fd,"MainField_Enemy_Bokoblin_Middle_2826006316",
0x716201a1,"MainField_Enemy_Bokoblin_Middle_2830622521",
0x5e809696,"MainField_Enemy_Bokoblin_Middle_2836361466",
0x32593174,"MainField_Enemy_Bokoblin_Middle_2840820856",
0xcc327ca4,"MainField_Enemy_Bokoblin_Middle_2865948947",
0x25542b27,"MainField_Enemy_Bokoblin_Middle_2885168107",
0xf966886c,"MainField_Enemy_Bokoblin_Middle_2889283611",
0xde7f8f4a,"MainField_Enemy_Bokoblin_Middle_2893303322",
0x1784318b,"MainField_Enemy_Bokoblin_Middle_290332106",
0x9a5d0532,"MainField_Enemy_Bokoblin_Middle_2904374750",
0xd2bb4c54,"MainField_Enemy_Bokoblin_Middle_2910364024",
0x5f044df8,"MainField_Enemy_Bokoblin_Middle_2919738379",
0xb9d7b000,"MainField_Enemy_Bokoblin_Middle_2920932952",
0x7a282935,"MainField_Enemy_Bokoblin_Middle_2935212090",
0x15cb11a9,"MainField_Enemy_Bokoblin_Middle_2947262756",
0x4c1f3e29,"MainField_Enemy_Bokoblin_Middle_2949696800",
0x36ccc13c,"MainField_Enemy_Bokoblin_Middle_2981731551",
0xc3c6c5a5,"MainField_Enemy_Bokoblin_Middle_298345565",
0x31c7a36a,"MainField_Enemy_Bokoblin_Middle_3003292851",
0x575cf337,"MainField_Enemy_Bokoblin_Middle_301505455",
0xd00048d3,"MainField_Enemy_Bokoblin_Middle_303637358",
0xfb041ca2,"MainField_Enemy_Bokoblin_Middle_3045332891",
0x3e18d2bb,"MainField_Enemy_Bokoblin_Middle_3046825356",
0x00d2f43f,"MainField_Enemy_Bokoblin_Middle_3057996548",
0xbfc110ad,"MainField_Enemy_Bokoblin_Middle_3058885632",
0xa24c6a48,"MainField_Enemy_Bokoblin_Middle_3073427959",
0xbcd94a48,"MainField_Enemy_Bokoblin_Middle_3077518420",
0x22f8b848,"MainField_Enemy_Bokoblin_Middle_3102841606",
0x5c1db36a,"MainField_Enemy_Bokoblin_Middle_3103473499",
0xb476f082,"MainField_Enemy_Bokoblin_Middle_3112754297",
0x7c4011ed,"MainField_Enemy_Bokoblin_Middle_3113262814",
0xed8e4f27,"MainField_Enemy_Bokoblin_Middle_3131272015",
0x766d94ba,"MainField_Enemy_Bokoblin_Middle_3136737276",
0x25fb9da4,"MainField_Enemy_Bokoblin_Middle_3147193193",
0xc8930d9b,"MainField_Enemy_Bokoblin_Middle_3149150361",
0x6a2e9768,"MainField_Enemy_Bokoblin_Middle_3162464960",
0x0d73f966,"MainField_Enemy_Bokoblin_Middle_3167588283",
0x07f8aa36,"MainField_Enemy_Bokoblin_Middle_3187994529",
0x1e0fed72,"MainField_Enemy_Bokoblin_Middle_3197180867",
0x74f548f1,"MainField_Enemy_Bokoblin_Middle_3199347402",
0x3e1f93d8,"MainField_Enemy_Bokoblin_Middle_3231164214",
0x86686a64,"MainField_Enemy_Bokoblin_Middle_3234954435",
0x8761171d,"MainField_Enemy_Bokoblin_Middle_3237221213",
0x7f41d371,"MainField_Enemy_Bokoblin_Middle_3247444114",
0x3a18e838,"MainField_Enemy_Bokoblin_Middle_3254972650",
0xbc527a9e,"MainField_Enemy_Bokoblin_Middle_3282979594",
0xd815bef8,"MainField_Enemy_Bokoblin_Middle_3294180755",
0x8198e8c1,"MainField_Enemy_Bokoblin_Middle_3308149486",
0x30fbd3a6,"MainField_Enemy_Bokoblin_Middle_3320139539",
0xaf07c953,"MainField_Enemy_Bokoblin_Middle_3321417595",
0x9523b6e8,"MainField_Enemy_Bokoblin_Middle_3327219888",
0xf9c66b9e,"MainField_Enemy_Bokoblin_Middle_3332397869",
0x4a7d695e,"MainField_Enemy_Bokoblin_Middle_3345203441",
0x2a57eaed,"MainField_Enemy_Bokoblin_Middle_3409164188",
0xf926d81a,"MainField_Enemy_Bokoblin_Middle_3428411661",
0xc37d6bff,"MainField_Enemy_Bokoblin_Middle_3434324650",
0x3e1a2aa2,"MainField_Enemy_Bokoblin_Middle_3448300281",
0xf578f212,"MainField_Enemy_Bokoblin_Middle_3449248938",
0xfc6fed5c,"MainField_Enemy_Bokoblin_Middle_3538639251",
0x7db11828,"MainField_Enemy_Bokoblin_Middle_3590131943",
0x06ea8c30,"MainField_Enemy_Bokoblin_Middle_3595838113",
0x3bd3a95f,"MainField_Enemy_Bokoblin_Middle_3614008268",
0x29c34cc1,"MainField_Enemy_Bokoblin_Middle_3624763946",
0x3a9e1e72,"MainField_Enemy_Bokoblin_Middle_3630432231",
0xf6eef3d4,"MainField_Enemy_Bokoblin_Middle_3638033846",
0x7a72b116,"MainField_Enemy_Bokoblin_Middle_3643210438",
0x6cbe7ad6,"MainField_Enemy_Bokoblin_Middle_3666953199",
0x06a773b4,"MainField_Enemy_Bokoblin_Middle_3671763985",
0x60ea7046,"MainField_Enemy_Bokoblin_Middle_3706236091",
0x9f5f0f1b,"MainField_Enemy_Bokoblin_Middle_371124352",
0xac2c7eb4,"MainField_Enemy_Bokoblin_Middle_371671997",
0x5d38e55f,"MainField_Enemy_Bokoblin_Middle_3725167410",
0x2c9d45ae,"MainField_Enemy_Bokoblin_Middle_3728023081",
0x043ea5a3,"MainField_Enemy_Bokoblin_Middle_3738021512",
0x76129c45,"MainField_Enemy_Bokoblin_Middle_3741519184",
0x3d5d717f,"MainField_Enemy_Bokoblin_Middle_3742700185",
0xce0ad179,"MainField_Enemy_Bokoblin_Middle_374760175",
0xe48792a9,"MainField_Enemy_Bokoblin_Middle_3764670475",
0xe9e5828b,"MainField_Enemy_Bokoblin_Middle_3781270006",
0x118f32cc,"MainField_Enemy_Bokoblin_Middle_3793218808",
0x296d6a5b,"MainField_Enemy_Bokoblin_Middle_3807051988",
0xcaa87cb8,"MainField_Enemy_Bokoblin_Middle_3815853720",
0xc77c6a83,"MainField_Enemy_Bokoblin_Middle_3823617803",
0xbd422896,"MainField_Enemy_Bokoblin_Middle_3823719232",
0x2ec482f0,"MainField_Enemy_Bokoblin_Middle_3826946127",
0x81f1c466,"MainField_Enemy_Bokoblin_Middle_3832764573",
0xd01f4535,"MainField_Enemy_Bokoblin_Middle_383402107",
0xecff18f5,"MainField_Enemy_Bokoblin_Middle_3835821821",
0x00112225,"MainField_Enemy_Bokoblin_Middle_3879433600",
0x70af5341,"MainField_Enemy_Bokoblin_Middle_3885067885",
0xb7e1e8ae,"MainField_Enemy_Bokoblin_Middle_3893659623",
0x364b4a3a,"MainField_Enemy_Bokoblin_Middle_3905871963",
0x4852eb71,"MainField_Enemy_Bokoblin_Middle_3927349617",
0x79f9b491,"MainField_Enemy_Bokoblin_Middle_3942743694",
0x58bb2603,"MainField_Enemy_Bokoblin_Middle_3946910806",
0x9b6fe03f,"MainField_Enemy_Bokoblin_Middle_3947980913",
0x672d4b1e,"MainField_Enemy_Bokoblin_Middle_3960364544",
0xcf5a3a3a,"MainField_Enemy_Bokoblin_Middle_3967201154",
0x369bfc10,"MainField_Enemy_Bokoblin_Middle_4027406459",
0x8902d93a,"MainField_Enemy_Bokoblin_Middle_4033038466",
0xfa4a3048,"MainField_Enemy_Bokoblin_Middle_4041456295",
0x69c51fee,"MainField_Enemy_Bokoblin_Middle_4042990435",
0x9115529d,"MainField_Enemy_Bokoblin_Middle_4045146560",
0x7ffdbb27,"MainField_Enemy_Bokoblin_Middle_4048781083",
0xca86437b,"MainField_Enemy_Bokoblin_Middle_4057706205",
0xa30f2b79,"MainField_Enemy_Bokoblin_Middle_4065994007",
0x63febb79,"MainField_Enemy_Bokoblin_Middle_407168222",
0xde2a8799,"MainField_Enemy_Bokoblin_Middle_4076851586",
0x7e00d0d2,"MainField_Enemy_Bokoblin_Middle_4088737287",
0xdecad430,"MainField_Enemy_Bokoblin_Middle_4091037637",
0x37c5c0a7,"MainField_Enemy_Bokoblin_Middle_4096295992",
0x8c316a40,"MainField_Enemy_Bokoblin_Middle_410228374",
0x63baf306,"MainField_Enemy_Bokoblin_Middle_4104304317",
0x8d079b52,"MainField_Enemy_Bokoblin_Middle_4131736488",
0x645dd720,"MainField_Enemy_Bokoblin_Middle_4139538633",
0xd0480eb0,"MainField_Enemy_Bokoblin_Middle_4140485773",
0xa404dfb9,"MainField_Enemy_Bokoblin_Middle_4144892842",
0x489f60c6,"MainField_Enemy_Bokoblin_Middle_4147666648",
0x60b17838,"MainField_Enemy_Bokoblin_Middle_4155008795",
0xfc1cfe2c,"MainField_Enemy_Bokoblin_Middle_415569071",
0x6469a7bc,"MainField_Enemy_Bokoblin_Middle_4163984418",
0xf31be347,"MainField_Enemy_Bokoblin_Middle_4175956209",
0xc7a203da,"MainField_Enemy_Bokoblin_Middle_4191101345",
0xfc4f6d40,"MainField_Enemy_Bokoblin_Middle_4212590498",
0xa4f06107,"MainField_Enemy_Bokoblin_Middle_4219896310",
0x7adc3d4a,"MainField_Enemy_Bokoblin_Middle_4248384073",
0xc5ba0238,"MainField_Enemy_Bokoblin_Middle_4250923257",
0x90b52eb0,"MainField_Enemy_Bokoblin_Middle_4262997539",
0xc40e2058,"MainField_Enemy_Bokoblin_Middle_428039658",
0x3fdaa965,"MainField_Enemy_Bokoblin_Middle_4290512288",
0x2eb23a69,"MainField_Enemy_Bokoblin_Middle_430166592",
0xdb213e16,"MainField_Enemy_Bokoblin_Middle_430307885",
0x6e0c4572,"MainField_Enemy_Bokoblin_Middle_442816501",
0x40f31971,"MainField_Enemy_Bokoblin_Middle_453230663",
0x704048ba,"MainField_Enemy_Bokoblin_Middle_458098675",
0x1b207268,"MainField_Enemy_Bokoblin_Middle_460605536",
0x8dd14850,"MainField_Enemy_Bokoblin_Middle_467647463",
0xb1da95df,"MainField_Enemy_Bokoblin_Middle_476088826",
0x45e17d0c,"MainField_Enemy_Bokoblin_Middle_478266440",
0x79ee0706,"MainField_Enemy_Bokoblin_Middle_479232562",
0xb33a0090,"MainField_Enemy_Bokoblin_Middle_482068829",
0x67d1c1fc,"MainField_Enemy_Bokoblin_Middle_489376067",
0x18b5e45a,"MainField_Enemy_Bokoblin_Middle_48957650",
0xd8f05054,"MainField_Enemy_Bokoblin_Middle_501305565",
0xb592fde2,"MainField_Enemy_Bokoblin_Middle_505888518",
0x478c0ffb,"MainField_Enemy_Bokoblin_Middle_519389775",
0xd66a048a,"MainField_Enemy_Bokoblin_Middle_547171185",
0x09fde758,"MainField_Enemy_Bokoblin_Middle_57543256",
0x2e0efe18,"MainField_Enemy_Bokoblin_Middle_581630282",
0x28608154,"MainField_Enemy_Bokoblin_Middle_614671614",
0xe09d22b6,"MainField_Enemy_Bokoblin_Middle_620111792",
0x18b2658f,"MainField_Enemy_Bokoblin_Middle_651042509",
0x633159b1,"MainField_Enemy_Bokoblin_Middle_663638345",
0xf3b1afe0,"MainField_Enemy_Bokoblin_Middle_664391761",
0x33fe201b,"MainField_Enemy_Bokoblin_Middle_675573397",
0x6ba748d0,"MainField_Enemy_Bokoblin_Middle_692801740",
0x9c3ef7e1,"MainField_Enemy_Bokoblin_Middle_703786444",
0x0bd3eadc,"MainField_Enemy_Bokoblin_Middle_710804719",
0x4a0d7b1a,"MainField_Enemy_Bokoblin_Middle_711518193",
0xc5e0c4b5,"MainField_Enemy_Bokoblin_Middle_713348493",
0x12321718,"MainField_Enemy_Bokoblin_Middle_714828466",
0x27eececc,"MainField_Enemy_Bokoblin_Middle_736408835",
0xcdd9795c,"MainField_Enemy_Bokoblin_Middle_739724916",
0xa4637535,"MainField_Enemy_Bokoblin_Middle_744855500",
0x54d3dbdd,"MainField_Enemy_Bokoblin_Middle_751150750",
0x8d75c021,"MainField_Enemy_Bokoblin_Middle_77426438",
0xd31e0c9d,"MainField_Enemy_Bokoblin_Middle_77458986",
0x073b7050,"MainField_Enemy_Bokoblin_Middle_784788655",
0xe1f1b916,"MainField_Enemy_Bokoblin_Middle_785821913",
0xe3a835ea,"MainField_Enemy_Bokoblin_Middle_788165233",
0xa0d10a88,"MainField_Enemy_Bokoblin_Middle_797513132",
0xcbc2b1be,"MainField_Enemy_Bokoblin_Middle_800539081",
0x82680e49,"MainField_Enemy_Bokoblin_Middle_833902723",
0x6c828dbc,"MainField_Enemy_Bokoblin_Middle_844262840",
0x67f06aae,"MainField_Enemy_Bokoblin_Middle_851152328",
0xc04428fa,"MainField_Enemy_Bokoblin_Middle_854994967",
0x2e884d5e,"MainField_Enemy_Bokoblin_Middle_858828110",
0x12e01e9c,"MainField_Enemy_Bokoblin_Middle_875961525",
0xc45e88d2,"MainField_Enemy_Bokoblin_Middle_877734963",
0xc3e9a83c,"MainField_Enemy_Bokoblin_Middle_882844355",
0x15d67313,"MainField_Enemy_Bokoblin_Middle_89337299",
0xfcdd2547,"MainField_Enemy_Bokoblin_Middle_893857558",
0x44e5b4dd,"MainField_Enemy_Bokoblin_Middle_914412650",
0x46ca02fa,"MainField_Enemy_Bokoblin_Middle_920231066",
0xb1c13a40,"MainField_Enemy_Bokoblin_Middle_936444533",
0x4789ece4,"MainField_Enemy_Bokoblin_Middle_942016425",
0x9c047b26,"MainField_Enemy_Bokoblin_Middle_950083343",
0x16d29fad,"MainField_Enemy_Bokoblin_Middle_953135122",
0x498dccad,"MainField_Enemy_Bokoblin_Middle_961698596",
0x513f39b4,"MainField_Enemy_Bokoblin_Middle_977578737",
0x97518b08,"MainField_Enemy_Bokoblin_Middle_983496084",
0x00289a70,"MainField_Enemy_Bokoblin_Middle_996452146",
0x436cfa7a,"MainField_Enemy_Bokoblin_Senior_1019891529",
0x1142c929,"MainField_Enemy_Bokoblin_Senior_1021674645",
0x5f9e111f,"MainField_Enemy_Bokoblin_Senior_1022923663",
0x4276cfeb,"MainField_Enemy_Bokoblin_Senior_1072541836",
0xafe101ee,"MainField_Enemy_Bokoblin_Senior_1148523592",
0x61495993,"MainField_Enemy_Bokoblin_Senior_118082339",
0x50b8edb3,"MainField_Enemy_Bokoblin_Senior_1186790006",
0xc0e0c3b4,"MainField_Enemy_Bokoblin_Senior_121564161",
0x87fd6732,"MainField_Enemy_Bokoblin_Senior_1273148419",
0xeacf5a78,"MainField_Enemy_Bokoblin_Senior_1279604155",
0x8b2b943a,"MainField_Enemy_Bokoblin_Senior_1285845600",
0xdf471055,"MainField_Enemy_Bokoblin_Senior_1294581284",
0x852d8985,"MainField_Enemy_Bokoblin_Senior_1299978509",
0x8f92968f,"MainField_Enemy_Bokoblin_Senior_1305153543",
0xc1b49149,"MainField_Enemy_Bokoblin_Senior_1308178254",
0x262af7b3,"MainField_Enemy_Bokoblin_Senior_1333480951",
0x6a6b8068,"MainField_Enemy_Bokoblin_Senior_1338552694",
0x35d927bf,"MainField_Enemy_Bokoblin_Senior_1344671327",
0x5936646e,"MainField_Enemy_Bokoblin_Senior_1373499562",
0x537426ce,"MainField_Enemy_Bokoblin_Senior_1390136277",
0x43fde496,"MainField_Enemy_Bokoblin_Senior_1393303184",
0x10acf86f,"MainField_Enemy_Bokoblin_Senior_1400316398",
0x14c63295,"MainField_Enemy_Bokoblin_Senior_1403735714",
0x876b7ef1,"MainField_Enemy_Bokoblin_Senior_141802589",
0x4375a276,"MainField_Enemy_Bokoblin_Senior_1424822607",
0x52f78853,"MainField_Enemy_Bokoblin_Senior_1456360765",
0xdd50dece,"MainField_Enemy_Bokoblin_Senior_1470825265",
0x1dc6eb8b,"MainField_Enemy_Bokoblin_Senior_1499207441",
0xade1abcf,"MainField_Enemy_Bokoblin_Senior_1502448273",
0xaeea85fb,"MainField_Enemy_Bokoblin_Senior_1509884430",
0x2bb8e2b5,"MainField_Enemy_Bokoblin_Senior_1545237730",
0x1116aa16,"MainField_Enemy_Bokoblin_Senior_1545563978",
0xfba7cd08,"MainField_Enemy_Bokoblin_Senior_1548333698",
0x8bb035c4,"MainField_Enemy_Bokoblin_Senior_1581651314",
0x04209c48,"MainField_Enemy_Bokoblin_Senior_1583267043",
0x05e0b7f9,"MainField_Enemy_Bokoblin_Senior_1586975139",
0x7e2ae96c,"MainField_Enemy_Bokoblin_Senior_1602354287",
0xe0ade65d,"MainField_Enemy_Bokoblin_Senior_1605436871",
0x274aa65b,"MainField_Enemy_Bokoblin_Senior_1618234161",
0xb512269c,"MainField_Enemy_Bokoblin_Senior_1636274683",
0xff8f5352,"MainField_Enemy_Bokoblin_Senior_1637592175",
0x026f133b,"MainField_Enemy_Bokoblin_Senior_1677738069",
0xdffd6e70,"MainField_Enemy_Bokoblin_Senior_1692288152",
0xb55846b7,"MainField_Enemy_Bokoblin_Senior_1729262244",
0x3a2f3564,"MainField_Enemy_Bokoblin_Senior_1734137296",
0x2afe8711,"MainField_Enemy_Bokoblin_Senior_1741799751",
0xa4cf29ed,"MainField_Enemy_Bokoblin_Senior_1768713804",
0xeaeb6269,"MainField_Enemy_Bokoblin_Senior_1786502784",
0x635f40e5,"MainField_Enemy_Bokoblin_Senior_1797016280",
0x43fd92c5,"MainField_Enemy_Bokoblin_Senior_1822233271",
0xdce00055,"MainField_Enemy_Bokoblin_Senior_1842154516",
0x69ee3ba3,"MainField_Enemy_Bokoblin_Senior_1845244796",
0xd4684d65,"MainField_Enemy_Bokoblin_Senior_1849304801",
0xe72184f4,"MainField_Enemy_Bokoblin_Senior_1849900848",
0x0a53cb21,"MainField_Enemy_Bokoblin_Senior_1853862108",
0x994b4218,"MainField_Enemy_Bokoblin_Senior_1883353556",
0x8a1fcb01,"MainField_Enemy_Bokoblin_Senior_1899279996",
0xe00dacf8,"MainField_Enemy_Bokoblin_Senior_1908735941",
0x4fbceeb4,"MainField_Enemy_Bokoblin_Senior_1912782624",
0x2c391d5d,"MainField_Enemy_Bokoblin_Senior_1933128425",
0x7dc65419,"MainField_Enemy_Bokoblin_Senior_1951412392",
0xb5282e28,"MainField_Enemy_Bokoblin_Senior_1962291562",
0xd9bfcfde,"MainField_Enemy_Bokoblin_Senior_1982589834",
0x9a2500d6,"MainField_Enemy_Bokoblin_Senior_2000256890",
0x0e9271df,"MainField_Enemy_Bokoblin_Senior_2003312832",
0x2fab64fc,"MainField_Enemy_Bokoblin_Senior_2013579223",
0x6f6bb3ad,"MainField_Enemy_Bokoblin_Senior_2014895848",
0x260432fb,"MainField_Enemy_Bokoblin_Senior_2043571034",
0x6bac4b90,"MainField_Enemy_Bokoblin_Senior_2054599949",
0xf8dec4a9,"MainField_Enemy_Bokoblin_Senior_2057283343",
0x20e0a966,"MainField_Enemy_Bokoblin_Senior_2063793320",
0xae43cd5a,"MainField_Enemy_Bokoblin_Senior_2065166989",
0xf611ef44,"MainField_Enemy_Bokoblin_Senior_2105510824",
0x032a839c,"MainField_Enemy_Bokoblin_Senior_2108589222",
0xc180b05d,"MainField_Enemy_Bokoblin_Senior_2133989757",
0xe565d1c7,"MainField_Enemy_Bokoblin_Senior_2151832912",
0xe25e4772,"MainField_Enemy_Bokoblin_Senior_2158573956",
0x3c6ec077,"MainField_Enemy_Bokoblin_Senior_2176664501",
0xa444c0f1,"MainField_Enemy_Bokoblin_Senior_2203005248",
0x6b5bef51,"MainField_Enemy_Bokoblin_Senior_2220778917",
0x97d93d5e,"MainField_Enemy_Bokoblin_Senior_2241197830",
0xbd8522ee,"MainField_Enemy_Bokoblin_Senior_2274877029",
0x28803ebb,"MainField_Enemy_Bokoblin_Senior_2281631889",
0x3fbf95ed,"MainField_Enemy_Bokoblin_Senior_2305790095",
0xe909fa1c,"MainField_Enemy_Bokoblin_Senior_2321048357",
0x8622b1eb,"MainField_Enemy_Bokoblin_Senior_2328299118",
0xcb00991a,"MainField_Enemy_Bokoblin_Senior_2358441937",
0xdd6b996c,"MainField_Enemy_Bokoblin_Senior_2371846756",
0x5d6521f1,"MainField_Enemy_Bokoblin_Senior_2371913686",
0x6aa764c2,"MainField_Enemy_Bokoblin_Senior_2382722075",
0x374ef481,"MainField_Enemy_Bokoblin_Senior_2386137374",
0x2626eded,"MainField_Enemy_Bokoblin_Senior_2404963998",
0x519e5d4f,"MainField_Enemy_Bokoblin_Senior_2421122118",
0xaf309c52,"MainField_Enemy_Bokoblin_Senior_2429643116",
0x2ae14404,"MainField_Enemy_Bokoblin_Senior_2431283690",
0x21b003f0,"MainField_Enemy_Bokoblin_Senior_2435395812",
0xf86b85a1,"MainField_Enemy_Bokoblin_Senior_2456694075",
0xa0c32a07,"MainField_Enemy_Bokoblin_Senior_248077260",
0x72af4d2c,"MainField_Enemy_Bokoblin_Senior_2494886894",
0x5313cf0c,"MainField_Enemy_Bokoblin_Senior_2510712385",
0xf9aa97f6,"MainField_Enemy_Bokoblin_Senior_2535815944",
0x23ad3039,"MainField_Enemy_Bokoblin_Senior_2540019497",
0x8cdcc04c,"MainField_Enemy_Bokoblin_Senior_2574081664",
0x55780c3f,"MainField_Enemy_Bokoblin_Senior_2588026829",
0x6e1d2b9f,"MainField_Enemy_Bokoblin_Senior_2650759514",
0x6a9a21a7,"MainField_Enemy_Bokoblin_Senior_2672569532",
0x6194cc3f,"MainField_Enemy_Bokoblin_Senior_2701290359",
0x0e660b69,"MainField_Enemy_Bokoblin_Senior_2713400006",
0x05d99b1e,"MainField_Enemy_Bokoblin_Senior_2730661306",
0xb067849d,"MainField_Enemy_Bokoblin_Senior_2782725661",
0x4e4bdb4a,"MainField_Enemy_Bokoblin_Senior_2802164270",
0xda1300f2,"MainField_Enemy_Bokoblin_Senior_2809275598",
0x70e0eb5b,"MainField_Enemy_Bokoblin_Senior_2810539746",
0xd45b8efe,"MainField_Enemy_Bokoblin_Senior_2816103143",
0xed03d622,"MainField_Enemy_Bokoblin_Senior_2843540927",
0x6a22fd0e,"MainField_Enemy_Bokoblin_Senior_2857728549",
0x87cd7a7f,"MainField_Enemy_Bokoblin_Senior_2864246505",
0xcca7a4b4,"MainField_Enemy_Bokoblin_Senior_2874660024",
0xe99ff3a1,"MainField_Enemy_Bokoblin_Senior_2890126292",
0x4aebcb49,"MainField_Enemy_Bokoblin_Senior_2894370832",
0x74ad0af4,"MainField_Enemy_Bokoblin_Senior_2895186566",
0x9c1d5823,"MainField_Enemy_Bokoblin_Senior_2909550894",
0xe57ed0e9,"MainField_Enemy_Bokoblin_Senior_2918382493",
0x730b7e58,"MainField_Enemy_Bokoblin_Senior_2921995136",
0xce2380b1,"MainField_Enemy_Bokoblin_Senior_2932631985",
0x2d246929,"MainField_Enemy_Bokoblin_Senior_2944571197",
0x11a2c400,"MainField_Enemy_Bokoblin_Senior_2966343952",
0xff6ec8a1,"MainField_Enemy_Bokoblin_Senior_2975158469",
0x14ad65b1,"MainField_Enemy_Bokoblin_Senior_3009274147",
0x2b566ffd,"MainField_Enemy_Bokoblin_Senior_3026910449",
0xd57192b0,"MainField_Enemy_Bokoblin_Senior_3034567657",
0x2e7ab357,"MainField_Enemy_Bokoblin_Senior_3036444865",
0x8da58670,"MainField_Enemy_Bokoblin_Senior_3048608782",
0xcd366ef6,"MainField_Enemy_Bokoblin_Senior_3049561233",
0x435703f7,"MainField_Enemy_Bokoblin_Senior_3050080220",
0x5d9a4b91,"MainField_Enemy_Bokoblin_Senior_3085514520",
0x2ee143ab,"MainField_Enemy_Bokoblin_Senior_3101886944",
0x2a1c5b38,"MainField_Enemy_Bokoblin_Senior_3119994280",
0xbfcad650,"MainField_Enemy_Bokoblin_Senior_3145949422",
0x60c710de,"MainField_Enemy_Bokoblin_Senior_3177717108",
0xc5b95305,"MainField_Enemy_Bokoblin_Senior_318255316",
0x07864f5a,"MainField_Enemy_Bokoblin_Senior_3202372938",
0x6a26e996,"MainField_Enemy_Bokoblin_Senior_3228573467",
0x80378e97,"MainField_Enemy_Bokoblin_Senior_3229014488",
0xbc9df626,"MainField_Enemy_Bokoblin_Senior_3235067464",
0x8c287fc0,"MainField_Enemy_Bokoblin_Senior_3241082241",
0x864914f6,"MainField_Enemy_Bokoblin_Senior_3260698209",
0x11023607,"MainField_Enemy_Bokoblin_Senior_3262296100",
0x6cca1177,"MainField_Enemy_Bokoblin_Senior_3262598113",
0x1d3d1bc0,"MainField_Enemy_Bokoblin_Senior_3274665976",
0x0cfd96d0,"MainField_Enemy_Bokoblin_Senior_3279171758",
0xd03d08e5,"MainField_Enemy_Bokoblin_Senior_3282483655",
0xc8b26efc,"MainField_Enemy_Bokoblin_Senior_3297329962",
0xe7105839,"MainField_Enemy_Bokoblin_Senior_333018126",
0x4334ea3a,"MainField_Enemy_Bokoblin_Senior_3350792717",
0x282393b4,"MainField_Enemy_Bokoblin_Senior_338012109",
0xe028b17e,"MainField_Enemy_Bokoblin_Senior_342419687",
0xb8c559ac,"MainField_Enemy_Bokoblin_Senior_3458554846",
0xe63e277c,"MainField_Enemy_Bokoblin_Senior_3495747646",
0xda1cbb5a,"MainField_Enemy_Bokoblin_Senior_3498955801",
0x78dc7336,"MainField_Enemy_Bokoblin_Senior_3517682454",
0x49ab9fea,"MainField_Enemy_Bokoblin_Senior_3524727021",
0xfbb9cdf4,"MainField_Enemy_Bokoblin_Senior_355134103",
0x4453b4d3,"MainField_Enemy_Bokoblin_Senior_3557656033",
0x8fe8961b,"MainField_Enemy_Bokoblin_Senior_3559235234",
0x65275637,"MainField_Enemy_Bokoblin_Senior_3583519194",
0x9049ee6b,"MainField_Enemy_Bokoblin_Senior_3616407325",
0x24c12d4b,"MainField_Enemy_Bokoblin_Senior_3625742751",
0xaa7c5d17,"MainField_Enemy_Bokoblin_Senior_3632637867",
0xc756798e,"MainField_Enemy_Bokoblin_Senior_3633293139",
0x0b486b54,"MainField_Enemy_Bokoblin_Senior_3673541387",
0x65dcc5bd,"MainField_Enemy_Bokoblin_Senior_3687310089",
0xb2ba6c25,"MainField_Enemy_Bokoblin_Senior_3689359989",
0x027ac64d,"MainField_Enemy_Bokoblin_Senior_3691101881",
0xf4e383d2,"MainField_Enemy_Bokoblin_Senior_3695008700",
0x9adfdd19,"MainField_Enemy_Bokoblin_Senior_3714119993",
0x968cfcc2,"MainField_Enemy_Bokoblin_Senior_3724995139",
0xa7475bf9,"MainField_Enemy_Bokoblin_Senior_3728668138",
0xda80d015,"MainField_Enemy_Bokoblin_Senior_3819015096",
0xd6348ae0,"MainField_Enemy_Bokoblin_Senior_3821715452",
0x0a6d57ba,"MainField_Enemy_Bokoblin_Senior_382378032",
0xa51a3019,"MainField_Enemy_Bokoblin_Senior_3838720580",
0xfc78775f,"MainField_Enemy_Bokoblin_Senior_3852013366",
0xba82c32f,"MainField_Enemy_Bokoblin_Senior_3861156397",
0xd8d1ab36,"MainField_Enemy_Bokoblin_Senior_3892529633",
0x0777a9e4,"MainField_Enemy_Bokoblin_Senior_3903312517",
0x1a3f1602,"MainField_Enemy_Bokoblin_Senior_3922677252",
0x1b13c733,"MainField_Enemy_Bokoblin_Senior_3929037981",
0xd574db79,"MainField_Enemy_Bokoblin_Senior_3932271530",
0x2bbcfea8,"MainField_Enemy_Bokoblin_Senior_3952210665",
0xf9dfdadb,"MainField_Enemy_Bokoblin_Senior_3955037107",
0x3cc5a18c,"MainField_Enemy_Bokoblin_Senior_3968394148",
0xdc3b12e4,"MainField_Enemy_Bokoblin_Senior_401250753",
0x7c1b1adb,"MainField_Enemy_Bokoblin_Senior_4012732075",
0x46a4f77c,"MainField_Enemy_Bokoblin_Senior_4018800762",
0x5dc16bff,"MainField_Enemy_Bokoblin_Senior_4038674057",
0xcd616a5c,"MainField_Enemy_Bokoblin_Senior_4047310890",
0xe4a7490c,"MainField_Enemy_Bokoblin_Senior_4133849588",
0xea17844e,"MainField_Enemy_Bokoblin_Senior_4158990778",
0xce057996,"MainField_Enemy_Bokoblin_Senior_4182226419",
0x203c7480,"MainField_Enemy_Bokoblin_Senior_4201101120",
0x91292ddd,"MainField_Enemy_Bokoblin_Senior_4204884016",
0x014f8802,"MainField_Enemy_Bokoblin_Senior_4214846788",
0x3a87affd,"MainField_Enemy_Bokoblin_Senior_4218272639",
0x87d736d6,"MainField_Enemy_Bokoblin_Senior_4224780126",
0xf1e6e0fe,"MainField_Enemy_Bokoblin_Senior_4225465059",
0x575f08ae,"MainField_Enemy_Bokoblin_Senior_4253104423",
0xc25fb798,"MainField_Enemy_Bokoblin_Senior_4268269593",
0xda757c9e,"MainField_Enemy_Bokoblin_Senior_429548850",
0x89eae1b0,"MainField_Enemy_Bokoblin_Senior_442162840",
0xb6ad3669,"MainField_Enemy_Bokoblin_Senior_502557530",
0x6bcda271,"MainField_Enemy_Bokoblin_Senior_515436304",
0xfa47ed61,"MainField_Enemy_Bokoblin_Senior_531777006",
0x1c8636c8,"MainField_Enemy_Bokoblin_Senior_531894364",
0x77d59695,"MainField_Enemy_Bokoblin_Senior_537878327",
0x1cc734b9,"MainField_Enemy_Bokoblin_Senior_544257454",
0x69fafd97,"MainField_Enemy_Bokoblin_Senior_562690853",
0x447070c5,"MainField_Enemy_Bokoblin_Senior_573140497",
0x4c7667c8,"MainField_Enemy_Bokoblin_Senior_576338634",
0x3b872fe6,"MainField_Enemy_Bokoblin_Senior_599496708",
0xf354d865,"MainField_Enemy_Bokoblin_Senior_602078386",
0x962003f0,"MainField_Enemy_Bokoblin_Senior_61476265",
0x123bf570,"MainField_Enemy_Bokoblin_Senior_617704566",
0x5aa9f7ae,"MainField_Enemy_Bokoblin_Senior_635710903",
0x7123e3b8,"MainField_Enemy_Bokoblin_Senior_641933914",
0xad10c301,"MainField_Enemy_Bokoblin_Senior_652096048",
0x92e108cd,"MainField_Enemy_Bokoblin_Senior_658240119",
0x72863025,"MainField_Enemy_Bokoblin_Senior_669219149",
0xa57e1b69,"MainField_Enemy_Bokoblin_Senior_678592431",
0x98260ac2,"MainField_Enemy_Bokoblin_Senior_691002666",
0xead157b8,"MainField_Enemy_Bokoblin_Senior_709379341",
0xad7d232b,"MainField_Enemy_Bokoblin_Senior_72224583",
0x34f7960d,"MainField_Enemy_Bokoblin_Senior_752365712",
0x620638b4,"MainField_Enemy_Bokoblin_Senior_756376350",
0xdf933497,"MainField_Enemy_Bokoblin_Senior_760347177",
0x4dd4ea24,"MainField_Enemy_Bokoblin_Senior_773019745",
0xea982e04,"MainField_Enemy_Bokoblin_Senior_777725831",
0xdcc3ea7b,"MainField_Enemy_Bokoblin_Senior_778162430",
0x4df35f10,"MainField_Enemy_Bokoblin_Senior_79011342",
0x46328996,"MainField_Enemy_Bokoblin_Senior_82403539",
0x97c8d0a9,"MainField_Enemy_Bokoblin_Senior_84385002",
0x01a56840,"MainField_Enemy_Bokoblin_Senior_886079513",
0xcb1c84a9,"MainField_Enemy_Bokoblin_Senior_907082154",
0xbd7cb953,"MainField_Enemy_Bokoblin_Senior_911208189",
0x03295f72,"MainField_Enemy_Bokoblin_Senior_918937002",
0x15589f66,"MainField_Enemy_Bokoblin_Senior_932931692",
0x00f08ca0,"MainField_Enemy_Bokoblin_Senior_933659928",
0x6a64f0b0,"MainField_Enemy_Bokoblin_Senior_938169908",
0x4bdeaaec,"MainField_Enemy_Bokoblin_Senior_973004645",
0x82326aec,"MainField_Enemy_Bokoblin_Senior_975912942",
0x8c8b0ca8,"MainField_Enemy_Bokoblin_Senior_977371565",
0x4733ec64,"MainField_Enemy_Bokoblin_Senior_987980991",
0x9b3fe8af,"MainField_Enemy_Bokoblin_Senior_988282125",
0x13f80bd6,"MainField_Enemy_Chuchu_Electric_Junior_1038203817",
0x57d1b3b2,"MainField_Enemy_Chuchu_Electric_Junior_1078021373",
0x271592c0,"MainField_Enemy_Chuchu_Electric_Junior_1092598474",
0x55aa082d,"MainField_Enemy_Chuchu_Electric_Junior_111610835",
0x0eee66cb,"MainField_Enemy_Chuchu_Electric_Junior_1180433958",
0xa20d2a43,"MainField_Enemy_Chuchu_Electric_Junior_1294518733",
0xb006a1de,"MainField_Enemy_Chuchu_Electric_Junior_1636747782",
0x46daeea0,"MainField_Enemy_Chuchu_Electric_Junior_1652294535",
0xdb772eb5,"MainField_Enemy_Chuchu_Electric_Junior_1718192783",
0x60158862,"MainField_Enemy_Chuchu_Electric_Junior_1723808082",
0x589091da,"MainField_Enemy_Chuchu_Electric_Junior_177227833",
0xbe75899a,"MainField_Enemy_Chuchu_Electric_Junior_1800722966",
0x05eefdc8,"MainField_Enemy_Chuchu_Electric_Junior_1844671840",
0x56d3e5d7,"MainField_Enemy_Chuchu_Electric_Junior_1873456622",
0x562adf7e,"MainField_Enemy_Chuchu_Electric_Junior_188550499",
0xf7ff5ed9,"MainField_Enemy_Chuchu_Electric_Junior_1998519510",
0xe21eaf85,"MainField_Enemy_Chuchu_Electric_Junior_200205161",
0xf8da6014,"MainField_Enemy_Chuchu_Electric_Junior_2152768212",
0x9ccf42bd,"MainField_Enemy_Chuchu_Electric_Junior_2176922513",
0x62352974,"MainField_Enemy_Chuchu_Electric_Junior_2201199036",
0x374cb90a,"MainField_Enemy_Chuchu_Electric_Junior_2324086192",
0xdbf87fa0,"MainField_Enemy_Chuchu_Electric_Junior_2370675230",
0xbabe30bc,"MainField_Enemy_Chuchu_Electric_Junior_2400408769",
0x41be365e,"MainField_Enemy_Chuchu_Electric_Junior_260375696",
0x59a1de90,"MainField_Enemy_Chuchu_Electric_Junior_2815076109",
0x32ef9dd5,"MainField_Enemy_Chuchu_Electric_Junior_2864080110",
0xe2a244b1,"MainField_Enemy_Chuchu_Electric_Junior_2987159037",
0x4cb427fe,"MainField_Enemy_Chuchu_Electric_Junior_301199816",
0x4ed735b5,"MainField_Enemy_Chuchu_Electric_Junior_3053734007",
0x27b9cc15,"MainField_Enemy_Chuchu_Electric_Junior_3087998295",
0xb5d91cf1,"MainField_Enemy_Chuchu_Electric_Junior_3102555906",
0x2d4fd7a7,"MainField_Enemy_Chuchu_Electric_Junior_3119612568",
0x55d1586b,"MainField_Enemy_Chuchu_Electric_Junior_3352554498",
0x2275711c,"MainField_Enemy_Chuchu_Electric_Junior_3375401710",
0x3d8f5913,"MainField_Enemy_Chuchu_Electric_Junior_3680814462",
0xf9ebfe17,"MainField_Enemy_Chuchu_Electric_Junior_3735463296",
0x07380759,"MainField_Enemy_Chuchu_Electric_Junior_3781976522",
0x05b3c0b9,"MainField_Enemy_Chuchu_Electric_Junior_3927556506",
0x7bd1c2ef,"MainField_Enemy_Chuchu_Electric_Junior_4091531612",
0x635f218b,"MainField_Enemy_Chuchu_Electric_Junior_4097085738",
0x3cdd1472,"MainField_Enemy_Chuchu_Electric_Junior_4184990085",
0x6340d1c9,"MainField_Enemy_Chuchu_Electric_Junior_4204525924",
0xa6054037,"MainField_Enemy_Chuchu_Electric_Junior_511014295",
0xe150ccf2,"MainField_Enemy_Chuchu_Electric_Junior_563829086",
0xb418cdb7,"MainField_Enemy_Chuchu_Electric_Junior_808318373",
0x56df91e5,"MainField_Enemy_Chuchu_Electric_Junior_857595307",
0x0388345b,"MainField_Enemy_Chuchu_Electric_Junior_881259927",
0xede40a1f,"MainField_Enemy_Chuchu_Electric_Middle_1173855322",
0xd8f856f0,"MainField_Enemy_Chuchu_Electric_Middle_1184145202",
0x3364d15a,"MainField_Enemy_Chuchu_Electric_Middle_1223335056",
0x1877ffc8,"MainField_Enemy_Chuchu_Electric_Middle_1677063024",
0xaaa6821c,"MainField_Enemy_Chuchu_Electric_Middle_1809772442",
0x878101d0,"MainField_Enemy_Chuchu_Electric_Middle_2089548478",
0x4e72d1ea,"MainField_Enemy_Chuchu_Electric_Middle_211268704",
0x70b6b5d2,"MainField_Enemy_Chuchu_Electric_Middle_2291136323",
0xbaa27853,"MainField_Enemy_Chuchu_Electric_Middle_2377497172",
0xd6579bda,"MainField_Enemy_Chuchu_Electric_Middle_2613589295",
0xe9e68c43,"MainField_Enemy_Chuchu_Electric_Middle_2634917013",
0x76a98923,"MainField_Enemy_Chuchu_Electric_Middle_2692889663",
0x6110921e,"MainField_Enemy_Chuchu_Electric_Middle_2955487500",
0xc0160dbf,"MainField_Enemy_Chuchu_Electric_Middle_3104933987",
0x99620c1c,"MainField_Enemy_Chuchu_Electric_Middle_3116628325",
0x636155f1,"MainField_Enemy_Chuchu_Electric_Middle_3207791634",
0x1823bf69,"MainField_Enemy_Chuchu_Electric_Middle_3247345675",
0x19a637f3,"MainField_Enemy_Chuchu_Electric_Middle_3391240734",
0x941979f1,"MainField_Enemy_Chuchu_Electric_Middle_3440829723",
0x664e7cb3,"MainField_Enemy_Chuchu_Electric_Middle_3552615182",
0x3d3996e6,"MainField_Enemy_Chuchu_Electric_Middle_3562327878",
0xea813abd,"MainField_Enemy_Chuchu_Electric_Middle_3640931780",
0xb7a0677a,"MainField_Enemy_Chuchu_Electric_Middle_4190403022",
0xbf0ecbdb,"MainField_Enemy_Chuchu_Electric_Middle_4217140674",
0xfaf4a616,"MainField_Enemy_Chuchu_Electric_Middle_4249643249",
0xb47ca561,"MainField_Enemy_Chuchu_Electric_Middle_560400027",
0x498003f3,"MainField_Enemy_Chuchu_Electric_Middle_598364472",
0x945a6384,"MainField_Enemy_Chuchu_Electric_Middle_715866561",
0xc201841f,"MainField_Enemy_Chuchu_Electric_Middle_831762340",
0x351ea0a2,"MainField_Enemy_Chuchu_Electric_Middle_925151920",
0xad613e89,"MainField_Enemy_Chuchu_Electric_Senior_2217310679",
0x6118fefa,"MainField_Enemy_Chuchu_Electric_Senior_2395704217",
0x4aed3bf2,"MainField_Enemy_Chuchu_Electric_Senior_2879010587",
0xe4fa8651,"MainField_Enemy_Chuchu_Electric_Senior_3060103018",
0x2f06426a,"MainField_Enemy_Chuchu_Electric_Senior_3107142942",
0x014b31fb,"MainField_Enemy_Chuchu_Electric_Senior_548170311",
0xc14ad6f6,"MainField_Enemy_Chuchu_Fire_Junior_1019922464",
0x5c621d90,"MainField_Enemy_Chuchu_Fire_Junior_1076356240",
0x0aa6aeb1,"MainField_Enemy_Chuchu_Fire_Junior_1085964156",
0x57fa10d5,"MainField_Enemy_Chuchu_Fire_Junior_1091424185",
0x101f94e1,"MainField_Enemy_Chuchu_Fire_Junior_1106197104",
0xb46ed479,"MainField_Enemy_Chuchu_Fire_Junior_1107864389",
0xe563ea9e,"MainField_Enemy_Chuchu_Fire_Junior_1188076662",
0xe2d55d45,"MainField_Enemy_Chuchu_Fire_Junior_1194055498",
0xae43cb89,"MainField_Enemy_Chuchu_Fire_Junior_1269998159",
0x93816ec3,"MainField_Enemy_Chuchu_Fire_Junior_1271789750",
0x9f5d4ba6,"MainField_Enemy_Chuchu_Fire_Junior_1282806159",
0x49e58700,"MainField_Enemy_Chuchu_Fire_Junior_1304399697",
0x403bd1c5,"MainField_Enemy_Chuchu_Fire_Junior_133601214",
0x0d85d6a3,"MainField_Enemy_Chuchu_Fire_Junior_135849728",
0x3393be26,"MainField_Enemy_Chuchu_Fire_Junior_1377557812",
0x1ce0dacc,"MainField_Enemy_Chuchu_Fire_Junior_1421627127",
0xfd975851,"MainField_Enemy_Chuchu_Fire_Junior_1435838669",
0x2ba2b670,"MainField_Enemy_Chuchu_Fire_Junior_1449276613",
0x3f17b7f9,"MainField_Enemy_Chuchu_Fire_Junior_1497909698",
0xc1021cd8,"MainField_Enemy_Chuchu_Fire_Junior_1498877374",
0x6c2cffee,"MainField_Enemy_Chuchu_Fire_Junior_1505233260",
0x3d1fd9b7,"MainField_Enemy_Chuchu_Fire_Junior_1506561797",
0x5c210282,"MainField_Enemy_Chuchu_Fire_Junior_1567040994",
0xe6513036,"MainField_Enemy_Chuchu_Fire_Junior_1630092713",
0x3c085885,"MainField_Enemy_Chuchu_Fire_Junior_1649516069",
0x93535131,"MainField_Enemy_Chuchu_Fire_Junior_1688509045",
0x1c0bd7de,"MainField_Enemy_Chuchu_Fire_Junior_1698454076",
0x506048ef,"MainField_Enemy_Chuchu_Fire_Junior_1751797819",
0x3c8fe82d,"MainField_Enemy_Chuchu_Fire_Junior_1772714139",
0x0fd51a13,"MainField_Enemy_Chuchu_Fire_Junior_183915297",
0x4a69e8df,"MainField_Enemy_Chuchu_Fire_Junior_1840653997",
0x5e162845,"MainField_Enemy_Chuchu_Fire_Junior_1849620119",
0x8eba7648,"MainField_Enemy_Chuchu_Fire_Junior_1889054055",
0xc9d6b5af,"MainField_Enemy_Chuchu_Fire_Junior_189834586",
0xb5e58ee1,"MainField_Enemy_Chuchu_Fire_Junior_1902626328",
0x9b36c5ef,"MainField_Enemy_Chuchu_Fire_Junior_1915990937",
0xe36545e4,"MainField_Enemy_Chuchu_Fire_Junior_1986857509",
0x9606c032,"MainField_Enemy_Chuchu_Fire_Junior_2020369701",
0x85a8cccc,"MainField_Enemy_Chuchu_Fire_Junior_214219304",
0xcdf92816,"MainField_Enemy_Chuchu_Fire_Junior_2175633390",
0x3e25c096,"MainField_Enemy_Chuchu_Fire_Junior_2243607292",
0x3c2a755f,"MainField_Enemy_Chuchu_Fire_Junior_2256856602",
0x54faa7a6,"MainField_Enemy_Chuchu_Fire_Junior_226000588",
0xf5d2b181,"MainField_Enemy_Chuchu_Fire_Junior_2273913775",
0x0a0b1051,"MainField_Enemy_Chuchu_Fire_Junior_2344769805",
0x98bb5562,"MainField_Enemy_Chuchu_Fire_Junior_2355101449",
0xf6bbdd6b,"MainField_Enemy_Chuchu_Fire_Junior_2397205366",
0xf96791f4,"MainField_Enemy_Chuchu_Fire_Junior_2397773705",
0xc4ca8cdc,"MainField_Enemy_Chuchu_Fire_Junior_2407561402",
0x16af1d5b,"MainField_Enemy_Chuchu_Fire_Junior_241518569",
0x47f68435,"MainField_Enemy_Chuchu_Fire_Junior_2425476777",
0x8863dd5f,"MainField_Enemy_Chuchu_Fire_Junior_242791513",
0x9cfefbf6,"MainField_Enemy_Chuchu_Fire_Junior_252691552",
0x09837b93,"MainField_Enemy_Chuchu_Fire_Junior_253689331",
0x7c42549b,"MainField_Enemy_Chuchu_Fire_Junior_253854310",
0x53b0f558,"MainField_Enemy_Chuchu_Fire_Junior_2539070540",
0xdc44cbc5,"MainField_Enemy_Chuchu_Fire_Junior_2655219611",
0x503154db,"MainField_Enemy_Chuchu_Fire_Junior_2745769626",
0x5a7a2824,"MainField_Enemy_Chuchu_Fire_Junior_2757419362",
0x8d75fbc1,"MainField_Enemy_Chuchu_Fire_Junior_2803629851",
0xd28e1710,"MainField_Enemy_Chuchu_Fire_Junior_2833845534",
0x570c51ad,"MainField_Enemy_Chuchu_Fire_Junior_2931039824",
0x6dde32c1,"MainField_Enemy_Chuchu_Fire_Junior_2959265428",
0x0684f2b3,"MainField_Enemy_Chuchu_Fire_Junior_3021926191",
0xfdf49139,"MainField_Enemy_Chuchu_Fire_Junior_3024149420",
0x7dd0fda8,"MainField_Enemy_Chuchu_Fire_Junior_3028505639",
0x27c0e05d,"MainField_Enemy_Chuchu_Fire_Junior_305883818",
0x0b8eafd6,"MainField_Enemy_Chuchu_Fire_Junior_3073158663",
0xee1acb7f,"MainField_Enemy_Chuchu_Fire_Junior_307343106",
0x1db69d25,"MainField_Enemy_Chuchu_Fire_Junior_3081540412",
0x5db1f463,"MainField_Enemy_Chuchu_Fire_Junior_3125194961",
0x754b2144,"MainField_Enemy_Chuchu_Fire_Junior_314086047",
0xf03a002c,"MainField_Enemy_Chuchu_Fire_Junior_3156464048",
0x675cfac2,"MainField_Enemy_Chuchu_Fire_Junior_3183521796",
0x838fa6cb,"MainField_Enemy_Chuchu_Fire_Junior_3206498576",
0x34a341c0,"MainField_Enemy_Chuchu_Fire_Junior_3306861342",
0x0a8c3621,"MainField_Enemy_Chuchu_Fire_Junior_3317875819",
0xd034af3b,"MainField_Enemy_Chuchu_Fire_Junior_3355868115",
0x2de06df6,"MainField_Enemy_Chuchu_Fire_Junior_3378269623",
0xc9e81f4b,"MainField_Enemy_Chuchu_Fire_Junior_3407855905",
0xa0fe75f2,"MainField_Enemy_Chuchu_Fire_Junior_3498944866",
0x9b3e1fdf,"MainField_Enemy_Chuchu_Fire_Junior_3568184844",
0x7ce07768,"MainField_Enemy_Chuchu_Fire_Junior_358002355",
0xb506d37f,"MainField_Enemy_Chuchu_Fire_Junior_3608369281",
0x0badb1dd,"MainField_Enemy_Chuchu_Fire_Junior_3652398790",
0xeb3e1b44,"MainField_Enemy_Chuchu_Fire_Junior_3681740071",
0xc66413c0,"MainField_Enemy_Chuchu_Fire_Junior_371984587",
0x5cafe626,"MainField_Enemy_Chuchu_Fire_Junior_3796932537",
0x4ab1ba20,"MainField_Enemy_Chuchu_Fire_Junior_3815335376",
0xb3e52e5e,"MainField_Enemy_Chuchu_Fire_Junior_3859936523",
0x9b41a396,"MainField_Enemy_Chuchu_Fire_Junior_389726128",
0xe8f43a9f,"MainField_Enemy_Chuchu_Fire_Junior_3926276313",
0x3ebb4598,"MainField_Enemy_Chuchu_Fire_Junior_3938472907",
0xb19f9299,"MainField_Enemy_Chuchu_Fire_Junior_4006690094",
0xebe85194,"MainField_Enemy_Chuchu_Fire_Junior_4010381516",
0x136d1cae,"MainField_Enemy_Chuchu_Fire_Junior_4013355074",
0x79334e63,"MainField_Enemy_Chuchu_Fire_Junior_4035694334",
0xa6bf5eb5,"MainField_Enemy_Chuchu_Fire_Junior_4102134443",
0x4bcb9429,"MainField_Enemy_Chuchu_Fire_Junior_4141977199",
0xb1a84ef8,"MainField_Enemy_Chuchu_Fire_Junior_4216138091",
0x9c1a17b7,"MainField_Enemy_Chuchu_Fire_Junior_4216901535",
0x217cff14,"MainField_Enemy_Chuchu_Fire_Junior_4227186242",
0x98a9ea65,"MainField_Enemy_Chuchu_Fire_Junior_425872362",
0xa8037bf7,"MainField_Enemy_Chuchu_Fire_Junior_597921874",
0x716ae1e9,"MainField_Enemy_Chuchu_Fire_Junior_67254538",
0x71d45805,"MainField_Enemy_Chuchu_Fire_Junior_705507914",
0x1a66ff92,"MainField_Enemy_Chuchu_Fire_Junior_713094856",
0x1ec6e768,"MainField_Enemy_Chuchu_Fire_Junior_865518559",
0xd5e64fa7,"MainField_Enemy_Chuchu_Fire_Junior_875467786",
0x828dadf0,"MainField_Enemy_Chuchu_Fire_Junior_959527222",
0x9b5da8c5,"MainField_Enemy_Chuchu_Fire_Junior_990998031",
0x330bd234,"MainField_Enemy_Chuchu_Fire_Middle_1020478573",
0xf316bd80,"MainField_Enemy_Chuchu_Fire_Middle_104442477",
0xae08b2cf,"MainField_Enemy_Chuchu_Fire_Middle_1279875675",
0xfc1dfcfe,"MainField_Enemy_Chuchu_Fire_Middle_1364609745",
0x97c4a076,"MainField_Enemy_Chuchu_Fire_Middle_1380206631",
0x18164db8,"MainField_Enemy_Chuchu_Fire_Middle_1432065199",
0x9150c4e7,"MainField_Enemy_Chuchu_Fire_Middle_1509092923",
0x8fd61a1f,"MainField_Enemy_Chuchu_Fire_Middle_1694236558",
0xb5a08644,"MainField_Enemy_Chuchu_Fire_Middle_1757191111",
0x3e093ca2,"MainField_Enemy_Chuchu_Fire_Middle_1913317851",
0xc5fcbcdc,"MainField_Enemy_Chuchu_Fire_Middle_1954700713",
0xc3560527,"MainField_Enemy_Chuchu_Fire_Middle_1970436090",
0xe9acf570,"MainField_Enemy_Chuchu_Fire_Middle_2085217934",
0x0f3a6d8a,"MainField_Enemy_Chuchu_Fire_Middle_2185518803",
0x574d321d,"MainField_Enemy_Chuchu_Fire_Middle_2190545148",
0xac84f317,"MainField_Enemy_Chuchu_Fire_Middle_2217612944",
0xe4377616,"MainField_Enemy_Chuchu_Fire_Middle_2219312382",
0x535fda10,"MainField_Enemy_Chuchu_Fire_Middle_2262787775",
0x96fc94c2,"MainField_Enemy_Chuchu_Fire_Middle_2283319775",
0x9d16f487,"MainField_Enemy_Chuchu_Fire_Middle_2304287741",
0x4aaf1a97,"MainField_Enemy_Chuchu_Fire_Middle_239361284",
0x1f296c3a,"MainField_Enemy_Chuchu_Fire_Middle_2536261619",
0xf14326ef,"MainField_Enemy_Chuchu_Fire_Middle_2538270883",
0x6270cd10,"MainField_Enemy_Chuchu_Fire_Middle_255331369",
0xebcb40ab,"MainField_Enemy_Chuchu_Fire_Middle_267602085",
0x630a80eb,"MainField_Enemy_Chuchu_Fire_Middle_2703672331",
0x0f12c1a3,"MainField_Enemy_Chuchu_Fire_Middle_2989416588",
0xe608b7d6,"MainField_Enemy_Chuchu_Fire_Middle_3033752111",
0x52d21279,"MainField_Enemy_Chuchu_Fire_Middle_3069536450",
0x6877c353,"MainField_Enemy_Chuchu_Fire_Middle_3097920899",
0x462277a3,"MainField_Enemy_Chuchu_Fire_Middle_3183645002",
0x95e9bdbd,"MainField_Enemy_Chuchu_Fire_Middle_32123431",
0xc35860d9,"MainField_Enemy_Chuchu_Fire_Middle_3260354791",
0x0973fc3a,"MainField_Enemy_Chuchu_Fire_Middle_3289608824",
0xf9a967e6,"MainField_Enemy_Chuchu_Fire_Middle_3350178609",
0x08cca457,"MainField_Enemy_Chuchu_Fire_Middle_3603889691",
0x67c390f3,"MainField_Enemy_Chuchu_Fire_Middle_3764029509",
0x0d7655b3,"MainField_Enemy_Chuchu_Fire_Middle_3805933890",
0xca2a6581,"MainField_Enemy_Chuchu_Fire_Middle_3934011998",
0x0ebec1fd,"MainField_Enemy_Chuchu_Fire_Middle_4052230258",
0x9c43e20f,"MainField_Enemy_Chuchu_Fire_Middle_4107322389",
0xac254d86,"MainField_Enemy_Chuchu_Fire_Middle_564311273",
0xf5274a6e,"MainField_Enemy_Chuchu_Fire_Middle_637470906",
0x9d06fe20,"MainField_Enemy_Chuchu_Fire_Middle_714198732",
0x5e4d699a,"MainField_Enemy_Chuchu_Fire_Middle_777478310",
0x2a97fccb,"MainField_Enemy_Chuchu_Fire_Middle_781827020",
0x9f78eef8,"MainField_Enemy_Chuchu_Fire_Middle_995612392",
0xe3a6a2e9,"MainField_Enemy_Chuchu_Fire_Senior_1394534404",
0x1ecc37ca,"MainField_Enemy_Chuchu_Fire_Senior_1446798873",
0xa6184e87,"MainField_Enemy_Chuchu_Fire_Senior_1764548575",
0xb1f582f6,"MainField_Enemy_Chuchu_Fire_Senior_2113170019",
0xfe5c915e,"MainField_Enemy_Chuchu_Fire_Senior_2652454836",
0xc893c595,"MainField_Enemy_Chuchu_Fire_Senior_2935044013",
0x79a04992,"MainField_Enemy_Chuchu_Fire_Senior_3198384073",
0x79fe6bed,"MainField_Enemy_Chuchu_Fire_Senior_4202922750",
0x50733f9d,"MainField_Enemy_Chuchu_Fire_Senior_908690016",
0xe9fbd34f,"MainField_Enemy_Chuchu_Ice_Junior_1034701535",
0xa7afed07,"MainField_Enemy_Chuchu_Ice_Junior_1074739318",
0xd3d32e59,"MainField_Enemy_Chuchu_Ice_Junior_1116405699",
0xec0927a7,"MainField_Enemy_Chuchu_Ice_Junior_1252751945",
0x1e456878,"MainField_Enemy_Chuchu_Ice_Junior_1299683038",
0x7316816e,"MainField_Enemy_Chuchu_Ice_Junior_1589504280",
0x99158978,"MainField_Enemy_Chuchu_Ice_Junior_1640385842",
0xfa1951f7,"MainField_Enemy_Chuchu_Ice_Junior_1664662538",
0x2e9d477f,"MainField_Enemy_Chuchu_Ice_Junior_1774886065",
0x99eb1b7e,"MainField_Enemy_Chuchu_Ice_Junior_1778017108",
0x14fa8710,"MainField_Enemy_Chuchu_Ice_Junior_1921736093",
0xadc76250,"MainField_Enemy_Chuchu_Ice_Junior_1931888268",
0xf92d9a38,"MainField_Enemy_Chuchu_Ice_Junior_1980816649",
0x5bab65ce,"MainField_Enemy_Chuchu_Ice_Junior_202618881",
0x7cbad810,"MainField_Enemy_Chuchu_Ice_Junior_2064943255",
0x57959698,"MainField_Enemy_Chuchu_Ice_Junior_2277834141",
0x10005252,"MainField_Enemy_Chuchu_Ice_Junior_235215622",
0x3b0c98a0,"MainField_Enemy_Chuchu_Ice_Junior_2428902845",
0xb5ce931a,"MainField_Enemy_Chuchu_Ice_Junior_2443767822",
0x4c460491,"MainField_Enemy_Chuchu_Ice_Junior_2457284002",
0x02fc3746,"MainField_Enemy_Chuchu_Ice_Junior_2493410187",
0x7c1345a9,"MainField_Enemy_Chuchu_Ice_Junior_2609443519",
0xba561534,"MainField_Enemy_Chuchu_Ice_Junior_2745120575",
0x7e61954a,"MainField_Enemy_Chuchu_Ice_Junior_2781191756",
0xee52cbe7,"MainField_Enemy_Chuchu_Ice_Junior_2894372591",
0xacb61c49,"MainField_Enemy_Chuchu_Ice_Junior_2903725902",
0xf3b407df,"MainField_Enemy_Chuchu_Ice_Junior_305749401",
0x950e9037,"MainField_Enemy_Chuchu_Ice_Junior_3140350338",
0x874ca1f9,"MainField_Enemy_Chuchu_Ice_Junior_3182993846",
0xcd6d7757,"MainField_Enemy_Chuchu_Ice_Junior_324159621",
0xaf8625ca,"MainField_Enemy_Chuchu_Ice_Junior_3243978151",
0x37dd38d9,"MainField_Enemy_Chuchu_Ice_Junior_3264782136",
0x1d8c0a97,"MainField_Enemy_Chuchu_Ice_Junior_3365443372",
0x779aeacf,"MainField_Enemy_Chuchu_Ice_Junior_3420059437",
0x7524bab7,"MainField_Enemy_Chuchu_Ice_Junior_3422943264",
0xc4d9fc8c,"MainField_Enemy_Chuchu_Ice_Junior_3492998907",
0x91d497f9,"MainField_Enemy_Chuchu_Ice_Junior_352843100",
0xf6e5c79d,"MainField_Enemy_Chuchu_Ice_Junior_3556098992",
0x2a7ee617,"MainField_Enemy_Chuchu_Ice_Junior_3699217064",
0x0ab77de1,"MainField_Enemy_Chuchu_Ice_Junior_3706515879",
0x3acd982f,"MainField_Enemy_Chuchu_Ice_Junior_3793606957",
0xd846dd00,"MainField_Enemy_Chuchu_Ice_Junior_4002823893",
0xcf83c476,"MainField_Enemy_Chuchu_Ice_Junior_4014557905",
0x647959bc,"MainField_Enemy_Chuchu_Ice_Junior_4040048939",
0x146bdb54,"MainField_Enemy_Chuchu_Ice_Junior_4171225834",
0x71e51042,"MainField_Enemy_Chuchu_Ice_Junior_4197576112",
0xd934a1e0,"MainField_Enemy_Chuchu_Ice_Junior_4198474180",
0xe4874c9c,"MainField_Enemy_Chuchu_Ice_Junior_516795431",
0xfff9afda,"MainField_Enemy_Chuchu_Ice_Junior_522713069",
0x790e9a97,"MainField_Enemy_Chuchu_Ice_Junior_533817381",
0x53e98db7,"MainField_Enemy_Chuchu_Ice_Junior_582636056",
0x2a3c58ce,"MainField_Enemy_Chuchu_Ice_Junior_63194249",
0xc38ba4be,"MainField_Enemy_Chuchu_Ice_Junior_69301786",
0x24367b12,"MainField_Enemy_Chuchu_Ice_Junior_832605614",
0x8061a536,"MainField_Enemy_Chuchu_Ice_Junior_859285077",
0xf1e0b6ce,"MainField_Enemy_Chuchu_Ice_Junior_969811449",
0x0f4f9335,"MainField_Enemy_Chuchu_Ice_Junior_996254223",
0x0a966777,"MainField_Enemy_Chuchu_Ice_Middle_1432429363",
0x2ba6bf85,"MainField_Enemy_Chuchu_Ice_Middle_2057042451",
0x2022d329,"MainField_Enemy_Chuchu_Ice_Middle_2222086775",
0x1b6df937,"MainField_Enemy_Chuchu_Ice_Middle_2477061770",
0x7b2cbddb,"MainField_Enemy_Chuchu_Ice_Middle_2542178048",
0x04d7b89a,"MainField_Enemy_Chuchu_Ice_Middle_2773144249",
0x91404bd6,"MainField_Enemy_Chuchu_Ice_Middle_2789316598",
0xeebf66c0,"MainField_Enemy_Chuchu_Ice_Middle_2892866111",
0x1d7cac66,"MainField_Enemy_Chuchu_Ice_Middle_2909895500",
0xc12bb541,"MainField_Enemy_Chuchu_Ice_Middle_3602422705",
0x229a0fc5,"MainField_Enemy_Chuchu_Ice_Middle_3604142769",
0xd4382827,"MainField_Enemy_Chuchu_Ice_Middle_3617481495",
0x76c886a6,"MainField_Enemy_Chuchu_Ice_Middle_3681194665",
0x731b9f07,"MainField_Enemy_Chuchu_Ice_Middle_376607542",
0x29204c1e,"MainField_Enemy_Chuchu_Ice_Middle_4069717596",
0x91a1e871,"MainField_Enemy_Chuchu_Ice_Middle_4174919353",
0x15de7f09,"MainField_Enemy_Chuchu_Ice_Middle_4195415165",
0xb652a727,"MainField_Enemy_Chuchu_Ice_Middle_790795800",
0x88a393ef,"MainField_Enemy_Chuchu_Ice_Middle_938864907",
0x7381af2d,"MainField_Enemy_Chuchu_Ice_Senior_1265885326",
0x383e593d,"MainField_Enemy_Chuchu_Ice_Senior_1861487334",
0x10ed7869,"MainField_Enemy_Chuchu_Ice_Senior_2579546439",
0xe5be3c26,"MainField_Enemy_Chuchu_Ice_Senior_4034184112",
0x90f00423,"MainField_Enemy_Chuchu_Ice_Senior_799697775",
0x314da9c0,"MainField_Enemy_Chuchu_Ice_Senior_988365870",
0x72c9908d,"MainField_Enemy_Chuchu_Junior_1014744211",
0x71870b84,"MainField_Enemy_Chuchu_Junior_1024142041",
0xf2cee0ba,"MainField_Enemy_Chuchu_Junior_1057522081",
0xc9254c9d,"MainField_Enemy_Chuchu_Junior_108218263",
0x7715480e,"MainField_Enemy_Chuchu_Junior_1102707996",
0x5eb571dc,"MainField_Enemy_Chuchu_Junior_1108890422",
0x134115ca,"MainField_Enemy_Chuchu_Junior_1184931262",
0xdc9853a9,"MainField_Enemy_Chuchu_Junior_1208987959",
0xd31d1534,"MainField_Enemy_Chuchu_Junior_1266480133",
0x098a8efe,"MainField_Enemy_Chuchu_Junior_1288722797",
0xefc9dc3e,"MainField_Enemy_Chuchu_Junior_1452337358",
0xf07851e3,"MainField_Enemy_Chuchu_Junior_1490446566",
0x2f1cd508,"MainField_Enemy_Chuchu_Junior_1516042294",
0xe10fbf89,"MainField_Enemy_Chuchu_Junior_1634758481",
0x6baf18bb,"MainField_Enemy_Chuchu_Junior_1723125362",
0xd1c91c14,"MainField_Enemy_Chuchu_Junior_1731442010",
0x8ffade69,"MainField_Enemy_Chuchu_Junior_1768310153",
0x039f9d44,"MainField_Enemy_Chuchu_Junior_1807050312",
0xeb3a0c06,"MainField_Enemy_Chuchu_Junior_1819395947",
0x90c0626f,"MainField_Enemy_Chuchu_Junior_1859078884",
0x0f162ead,"MainField_Enemy_Chuchu_Junior_1876641359",
0xd25f26ae,"MainField_Enemy_Chuchu_Junior_1899609755",
0x3d471db8,"MainField_Enemy_Chuchu_Junior_1925755934",
0xd1ec4b9d,"MainField_Enemy_Chuchu_Junior_1953745303",
0x0c6f0a47,"MainField_Enemy_Chuchu_Junior_2017283488",
0x97f726ae,"MainField_Enemy_Chuchu_Junior_2129017839",
0x93157133,"MainField_Enemy_Chuchu_Junior_2129681076",
0x3658642e,"MainField_Enemy_Chuchu_Junior_2156231849",
0xe7eefc8a,"MainField_Enemy_Chuchu_Junior_2178230115",
0x52d5def3,"MainField_Enemy_Chuchu_Junior_2181367355",
0xbfa41b42,"MainField_Enemy_Chuchu_Junior_2188598154",
0xe71f1c80,"MainField_Enemy_Chuchu_Junior_2254514028",
0x528f92f4,"MainField_Enemy_Chuchu_Junior_2306303971",
0x145d5d8b,"MainField_Enemy_Chuchu_Junior_238388640",
0x6c29c48a,"MainField_Enemy_Chuchu_Junior_2392764315",
0xe8ab9f77,"MainField_Enemy_Chuchu_Junior_2453503282",
0x8e03ce41,"MainField_Enemy_Chuchu_Junior_2563642914",
0x8dd92f2d,"MainField_Enemy_Chuchu_Junior_2662829102",
0x0b90ab79,"MainField_Enemy_Chuchu_Junior_2670313005",
0x464ea8b9,"MainField_Enemy_Chuchu_Junior_2775749929",
0x5bf7849d,"MainField_Enemy_Chuchu_Junior_2791887715",
0xc891c9e0,"MainField_Enemy_Chuchu_Junior_2792966559",
0xe960e616,"MainField_Enemy_Chuchu_Junior_2798714672",
0x092434bb,"MainField_Enemy_Chuchu_Junior_2803853014",
0x92233797,"MainField_Enemy_Chuchu_Junior_2816332893",
0x18162384,"MainField_Enemy_Chuchu_Junior_2832881705",
0xfc12c539,"MainField_Enemy_Chuchu_Junior_2875749482",
0xa7962fda,"MainField_Enemy_Chuchu_Junior_2880917453",
0xdd60ddf2,"MainField_Enemy_Chuchu_Junior_2948045872",
0x04560141,"MainField_Enemy_Chuchu_Junior_2983524121",
0x9e13cc53,"MainField_Enemy_Chuchu_Junior_3025312026",
0xc48f97d1,"MainField_Enemy_Chuchu_Junior_3067611594",
0xdae886ad,"MainField_Enemy_Chuchu_Junior_3141600609",
0x6e0e8d65,"MainField_Enemy_Chuchu_Junior_3143066682",
0xff39ce2f,"MainField_Enemy_Chuchu_Junior_3165176848",
0x19637771,"MainField_Enemy_Chuchu_Junior_3202685024",
0x936d9cac,"MainField_Enemy_Chuchu_Junior_3204252834",
0x0c57bda0,"MainField_Enemy_Chuchu_Junior_3222031674",
0x7c392a73,"MainField_Enemy_Chuchu_Junior_3292302033",
0x8821003c,"MainField_Enemy_Chuchu_Junior_3402187811",
0x5c2b3265,"MainField_Enemy_Chuchu_Junior_3427816620",
0x275b60d5,"MainField_Enemy_Chuchu_Junior_3504006347",
0xf5adc5c4,"MainField_Enemy_Chuchu_Junior_3635457190",
0xfc4a11d7,"MainField_Enemy_Chuchu_Junior_3675325068",
0x3b53e1bc,"MainField_Enemy_Chuchu_Junior_3698279676",
0x04bef019,"MainField_Enemy_Chuchu_Junior_3710287904",
0x3b71b67a,"MainField_Enemy_Chuchu_Junior_3710726552",
0xba4873d8,"MainField_Enemy_Chuchu_Junior_3748391410",
0xfd7e4cb8,"MainField_Enemy_Chuchu_Junior_3889305716",
0xd8768ed7,"MainField_Enemy_Chuchu_Junior_3998819966",
0xff0388cb,"MainField_Enemy_Chuchu_Junior_4061769477",
0x9385e72b,"MainField_Enemy_Chuchu_Junior_4072594054",
0x18846c1c,"MainField_Enemy_Chuchu_Junior_411701014",
0xfe5a57ae,"MainField_Enemy_Chuchu_Junior_417101529",
0xb41fa2b7,"MainField_Enemy_Chuchu_Junior_4172026967",
0x0aeb8470,"MainField_Enemy_Chuchu_Junior_4187594509",
0x7e6f181f,"MainField_Enemy_Chuchu_Junior_4284767198",
0xd6c5e193,"MainField_Enemy_Chuchu_Junior_471322153",
0x4ed1a363,"MainField_Enemy_Chuchu_Junior_509703455",
0xb013c1dd,"MainField_Enemy_Chuchu_Junior_551429314",
0xee838889,"MainField_Enemy_Chuchu_Junior_646062832",
0xb7decb91,"MainField_Enemy_Chuchu_Junior_664715705",
0xc604a5ba,"MainField_Enemy_Chuchu_Junior_760989856",
0x324a0d06,"MainField_Enemy_Chuchu_Junior_792872166",
0x064f6425,"MainField_Enemy_Chuchu_Junior_808251561",
0x6169b84c,"MainField_Enemy_Chuchu_Junior_850883484",
0xbcb50cd5,"MainField_Enemy_Chuchu_Junior_918474122",
0xd15221bb,"MainField_Enemy_Chuchu_Middle_1714739403",
0xb3c78821,"MainField_Enemy_Chuchu_Middle_2014166279",
0x977abdad,"MainField_Enemy_Chuchu_Middle_2046826710",
0xcb2f324c,"MainField_Enemy_Chuchu_Middle_218572864",
0x1d10fbef,"MainField_Enemy_Chuchu_Middle_2239721882",
0x1b20567e,"MainField_Enemy_Chuchu_Middle_2294288162",
0xc7dd0286,"MainField_Enemy_Chuchu_Middle_2400525887",
0x6e31ee66,"MainField_Enemy_Chuchu_Middle_2778288539",
0x8fc2c068,"MainField_Enemy_Chuchu_Middle_3032189186",
0xd0314c86,"MainField_Enemy_Chuchu_Middle_321066527",
0xac833f57,"MainField_Enemy_Chuchu_Middle_3283925396",
0x916fb914,"MainField_Enemy_Chuchu_Middle_4066400454",
0x88a00be6,"MainField_Enemy_Chuchu_Middle_697327925",
0xe6ec9c94,"MainField_Enemy_Chuchu_Senior_1575407834",
0x8c4c3d3f,"MainField_Enemy_Chuchu_Senior_2028075378",
0xf04d8dd4,"MainField_Enemy_Chuchu_Senior_3354227555",
0x7a46696d,"MainField_Enemy_Chuchu_Senior_795246686",
0x916598fb,"MainField_Enemy_Ganon_108794243",
0x0b9ddfe3,"MainField_Enemy_Giant_Bone_1287559097",
0xb12e8e42,"MainField_Enemy_Giant_Bone_1698864303",
0xe967ac50,"MainField_Enemy_Giant_Bone_1721655973",
0x2ec14565,"MainField_Enemy_Giant_Bone_2519547405",
0x35d53820,"MainField_Enemy_Giant_Bone_2815854988",
0xdff35c97,"MainField_Enemy_Giant_Bone_2974080467",
0x3233217c,"MainField_Enemy_Giant_Bone_3201775889",
0x83a4f606,"MainField_Enemy_Giant_Bone_3700264996",
0x432b9c75,"MainField_Enemy_Giant_Bone_4237208898",
0xb7add57e,"MainField_Enemy_Giant_Bone_AllDay_73882430",
0xf35dca23,"MainField_Enemy_Giant_Junior_1602515170",
0xa9f9a81f,"MainField_Enemy_Giant_Junior_1833514674",
0x80276ca6,"MainField_Enemy_Giant_Junior_1903632265",
0x015b5950,"MainField_Enemy_Giant_Junior_3337886442",
0x1afb8eb9,"MainField_Enemy_Giant_Junior_3350013125",
0xac816fb4,"MainField_Enemy_Giant_Junior_373348007",
0xb3b14506,"MainField_Enemy_Giant_Junior_4099815924",
0x7da27350,"MainField_Enemy_Giant_Middle_1266282060",
0x670a9150,"MainField_Enemy_Giant_Middle_1384742674",
0xe99ae858,"MainField_Enemy_Giant_Middle_1925665429",
0x1c553c55,"MainField_Enemy_Giant_Middle_1949698080",
0x6c9a04c7,"MainField_Enemy_Giant_Middle_2083136285",
0x6b3d1e6b,"MainField_Enemy_Giant_Middle_2472409077",
0x9fc95a5a,"MainField_Enemy_Giant_Middle_2530315670",
0x27129d20,"MainField_Enemy_Giant_Middle_3272527560",
0xcb485ad9,"MainField_Enemy_Giant_Middle_3572226454",
0x88a9805e,"MainField_Enemy_Giant_Middle_3627901807",
0x86036d5e,"MainField_Enemy_Giant_Middle_3637193649",
0x67c6b12c,"MainField_Enemy_Giant_Middle_3706280648",
0x0cff51df,"MainField_Enemy_Giant_Middle_3744176720",
0x9d87673f,"MainField_Enemy_Giant_Middle_3831867213",
0x2c1d5ffb,"MainField_Enemy_Giant_Middle_4080554254",
0x80cd03a9,"MainField_Enemy_Giant_Middle_936329231",
0xccadd241,"MainField_Enemy_Giant_Senior_1028264440",
0x54e93d33,"MainField_Enemy_Giant_Senior_2009832062",
0x390ac273,"MainField_Enemy_Giant_Senior_2853871544",
0x4874ffbf,"MainField_Enemy_Giant_Senior_3410381997",
0xcd1ff5e2,"MainField_Enemy_Giant_Senior_3741497810",
0xf763f386,"MainField_Enemy_Giant_Senior_3785639248",
0xf551e3d1,"MainField_Enemy_Giant_Senior_851170738",
0x4578791f,"MainField_Enemy_Golem_Fire_1372356658",
0xd6fbf25d,"MainField_Enemy_Golem_Fire_2080410037",
0xffef2766,"MainField_Enemy_Golem_Fire_2112489205",
0xff36e0ac,"MainField_Enemy_Golem_Fire_3180486693",
0x1bb54603,"MainField_Enemy_Golem_Fire_4143112243",
0x53143900,"MainField_Enemy_Golem_Ice_1684711544",
0x83c1a6b2,"MainField_Enemy_Golem_Ice_189892196",
0x372b108a,"MainField_Enemy_Golem_Ice_4007690024",
0x9e884807,"MainField_Enemy_Golem_Ice_4087740554",
0x492a878e,"MainField_Enemy_Golem_Ice_648092042",
0xc52215de,"MainField_Enemy_Golem_Junior_139564765",
0xb3037abf,"MainField_Enemy_Golem_Junior_1450825465",
0x631cc8b0,"MainField_Enemy_Golem_Junior_2009332164",
0x5a034937,"MainField_Enemy_Golem_Junior_3610208070",
0xddede127,"MainField_Enemy_Golem_Junior_3688513041",
0x2974576b,"MainField_Enemy_Golem_Junior_3707687090",
0x87230d3f,"MainField_Enemy_Golem_Junior_3817875409",
0x61edf116,"MainField_Enemy_Golem_Junior_3873221297",
0x445028a9,"MainField_Enemy_Golem_Junior_4097205930",
0xe3fa2d31,"MainField_Enemy_Golem_Junior_4234397911",
0x9269b425,"MainField_Enemy_Golem_Junior_481698249",
0xb922d184,"MainField_Enemy_Golem_Little_1022135161",
0x4ddf7939,"MainField_Enemy_Golem_Little_1079504047",
0xe522e9c5,"MainField_Enemy_Golem_Little_1086580688",
0x1b1d8aac,"MainField_Enemy_Golem_Little_1194419267",
0xad3b4fb1,"MainField_Enemy_Golem_Little_1230577713",
0xa05f4c4e,"MainField_Enemy_Golem_Little_1303923814",
0x94cd7aaf,"MainField_Enemy_Golem_Little_1321039613",
0x203cab3f,"MainField_Enemy_Golem_Little_1375228778",
0x039fa015,"MainField_Enemy_Golem_Little_1398555197",
0x65e37800,"MainField_Enemy_Golem_Little_1439289790",
0x6aca5306,"MainField_Enemy_Golem_Little_1444673880",
0x4fcb0b4c,"MainField_Enemy_Golem_Little_1511851059",
0x0d30c88d,"MainField_Enemy_Golem_Little_1520584841",
0x3ad7ab76,"MainField_Enemy_Golem_Little_1549246708",
0xfe9a6d9e,"MainField_Enemy_Golem_Little_160751214",
0xa9aec92b,"MainField_Enemy_Golem_Little_1629898198",
0xd9945ae1,"MainField_Enemy_Golem_Little_1632290851",
0xf6a2700e,"MainField_Enemy_Golem_Little_1685481802",
0x8844f299,"MainField_Enemy_Golem_Little_1782901588",
0xbc682665,"MainField_Enemy_Golem_Little_1787004489",
0x646c4729,"MainField_Enemy_Golem_Little_1806214137",
0xee9a1301,"MainField_Enemy_Golem_Little_1826879576",
0xfcc1b671,"MainField_Enemy_Golem_Little_1835399856",
0x90116f43,"MainField_Enemy_Golem_Little_1851015492",
0xcbb38b91,"MainField_Enemy_Golem_Little_1862145222",
0x81fc9561,"MainField_Enemy_Golem_Little_1874054854",
0x77ececda,"MainField_Enemy_Golem_Little_1933212329",
0x6cbda7f1,"MainField_Enemy_Golem_Little_2012355687",
0x719ca4c6,"MainField_Enemy_Golem_Little_2057302680",
0xfd43d7ff,"MainField_Enemy_Golem_Little_2066923708",
0x1bd1773d,"MainField_Enemy_Golem_Little_2069877256",
0xa3937612,"MainField_Enemy_Golem_Little_2104316432",
0xa72ac9ae,"MainField_Enemy_Golem_Little_2173448671",
0xd7424cf3,"MainField_Enemy_Golem_Little_2241838199",
0x7fe68ae3,"MainField_Enemy_Golem_Little_2372391847",
0xbf24a8f2,"MainField_Enemy_Golem_Little_2379919319",
0xf3a25a13,"MainField_Enemy_Golem_Little_2489017935",
0x9ffd7243,"MainField_Enemy_Golem_Little_2602909375",
0xcdc40574,"MainField_Enemy_Golem_Little_263628665",
0xbaad61b6,"MainField_Enemy_Golem_Little_2651574087",
0x4cd701ec,"MainField_Enemy_Golem_Little_2683674769",
0xc7b07d8a,"MainField_Enemy_Golem_Little_2707960822",
0x819cf893,"MainField_Enemy_Golem_Little_2734571994",
0x0e445a7f,"MainField_Enemy_Golem_Little_2747236110",
0x791914e5,"MainField_Enemy_Golem_Little_2779600229",
0x3b4ace4c,"MainField_Enemy_Golem_Little_2822726708",
0x24082da8,"MainField_Enemy_Golem_Little_2885410923",
0x1d5fb8d7,"MainField_Enemy_Golem_Little_2927381138",
0xb1f86dce,"MainField_Enemy_Golem_Little_2950944218",
0x4c48ca30,"MainField_Enemy_Golem_Little_2960003865",
0x716e5c16,"MainField_Enemy_Golem_Little_2992868506",
0x1c202957,"MainField_Enemy_Golem_Little_2995429858",
0xcdf4da5d,"MainField_Enemy_Golem_Little_307156074",
0x928c7534,"MainField_Enemy_Golem_Little_3078865551",
0x4ae9d086,"MainField_Enemy_Golem_Little_3090775571",
0x11f1dc11,"MainField_Enemy_Golem_Little_3098858069",
0xbaa57113,"MainField_Enemy_Golem_Little_3205023768",
0xda3c3704,"MainField_Enemy_Golem_Little_3220706609",
0x05fc9bb6,"MainField_Enemy_Golem_Little_3222266602",
0x096ab410,"MainField_Enemy_Golem_Little_326958040",
0x4be5db3d,"MainField_Enemy_Golem_Little_3281607627",
0xb9806981,"MainField_Enemy_Golem_Little_3293431961",
0x43a7b03a,"MainField_Enemy_Golem_Little_3305262389",
0x5c85826d,"MainField_Enemy_Golem_Little_3341417913",
0x9fe01c66,"MainField_Enemy_Golem_Little_3397168651",
0x87b4827d,"MainField_Enemy_Golem_Little_3424903399",
0xbe2267bd,"MainField_Enemy_Golem_Little_3425951968",
0x3e8c9c45,"MainField_Enemy_Golem_Little_3433747135",
0x4ea7c67f,"MainField_Enemy_Golem_Little_3459126991",
0xd325a7e1,"MainField_Enemy_Golem_Little_3461458546",
0x4a9128ce,"MainField_Enemy_Golem_Little_3530599535",
0xef9965d8,"MainField_Enemy_Golem_Little_353716716",
0xbd697e8b,"MainField_Enemy_Golem_Little_3574854239",
0x731c06fa,"MainField_Enemy_Golem_Little_360875766",
0xe39ce59d,"MainField_Enemy_Golem_Little_3635838494",
0xc4d5821d,"MainField_Enemy_Golem_Little_3677746478",
0xbd40e1b9,"MainField_Enemy_Golem_Little_3712100937",
0xdf485b46,"MainField_Enemy_Golem_Little_3723093382",
0xf4814fb0,"MainField_Enemy_Golem_Little_3768812202",
0x385f7f6c,"MainField_Enemy_Golem_Little_3772466855",
0xf40cd429,"MainField_Enemy_Golem_Little_3777908197",
0x9d62184c,"MainField_Enemy_Golem_Little_37816451",
0x92193845,"MainField_Enemy_Golem_Little_3837724254",
0x6042789e,"MainField_Enemy_Golem_Little_3851826705",
0x20ee7168,"MainField_Enemy_Golem_Little_3861499453",
0xee6a1afb,"MainField_Enemy_Golem_Little_3927855038",
0x5c78f20a,"MainField_Enemy_Golem_Little_4015799166",
0x256cf622,"MainField_Enemy_Golem_Little_4078915565",
0xfe9e37c3,"MainField_Enemy_Golem_Little_408190251",
0x1eb93495,"MainField_Enemy_Golem_Little_4082998376",
0xf0d08b17,"MainField_Enemy_Golem_Little_4094607870",
0x2f08299a,"MainField_Enemy_Golem_Little_4101346722",
0xc6845841,"MainField_Enemy_Golem_Little_4156343042",
0x3cb89d38,"MainField_Enemy_Golem_Little_4188379052",
0xc2f63b5b,"MainField_Enemy_Golem_Little_4196261432",
0x41596d52,"MainField_Enemy_Golem_Little_4209594171",
0x65a520cc,"MainField_Enemy_Golem_Little_424589667",
0xe030208d,"MainField_Enemy_Golem_Little_4255560210",
0xb2eb50cb,"MainField_Enemy_Golem_Little_4294216779",
0x9146a3e1,"MainField_Enemy_Golem_Little_431616705",
0x7aed5445,"MainField_Enemy_Golem_Little_456323933",
0x74ebd4b7,"MainField_Enemy_Golem_Little_504281183",
0x1a8db8ad,"MainField_Enemy_Golem_Little_523476258",
0x1f497c2e,"MainField_Enemy_Golem_Little_649350944",
0xf71e5cfa,"MainField_Enemy_Golem_Little_652490556",
0xfdde4eca,"MainField_Enemy_Golem_Little_705317289",
0xed424574,"MainField_Enemy_Golem_Little_729068293",
0x4a19a6b2,"MainField_Enemy_Golem_Little_738152719",
0x35a1757e,"MainField_Enemy_Golem_Little_741009083",
0xe4a36fed,"MainField_Enemy_Golem_Little_745621049",
0x680858a3,"MainField_Enemy_Golem_Little_780723947",
0x6bcb6add,"MainField_Enemy_Golem_Little_781641585",
0xb678c50f,"MainField_Enemy_Golem_Little_808127529",
0x61312966,"MainField_Enemy_Golem_Little_815609981",
0xbbce988f,"MainField_Enemy_Golem_Little_840171449",
0xe52cfd4e,"MainField_Enemy_Golem_Little_922378679",
0x96222aba,"MainField_Enemy_Golem_Little_947700979",
0x000d16bf,"MainField_Enemy_Golem_Little_Fire_1073978275",
0x5e802067,"MainField_Enemy_Golem_Little_Fire_1092594931",
0xdbaa25fb,"MainField_Enemy_Golem_Little_Fire_1209930187",
0x0e345c34,"MainField_Enemy_Golem_Little_Fire_1311778640",
0xf3bf6ae8,"MainField_Enemy_Golem_Little_Fire_1699735992",
0x5d2fefca,"MainField_Enemy_Golem_Little_Fire_1808363407",
0xf9bdf6db,"MainField_Enemy_Golem_Little_Fire_1814761004",
0xcb25850d,"MainField_Enemy_Golem_Little_Fire_1818206981",
0x716b09aa,"MainField_Enemy_Golem_Little_Fire_1935661589",
0x0b7016a1,"MainField_Enemy_Golem_Little_Fire_2017745119",
0xd45b9889,"MainField_Enemy_Golem_Little_Fire_2084113427",
0x2ee4a18b,"MainField_Enemy_Golem_Little_Fire_2103695235",
0x576bef4d,"MainField_Enemy_Golem_Little_Fire_2266037766",
0xf64ff67a,"MainField_Enemy_Golem_Little_Fire_2279879894",
0xdaae6190,"MainField_Enemy_Golem_Little_Fire_2388077252",
0xe7539f1d,"MainField_Enemy_Golem_Little_Fire_2488503923",
0x4138c689,"MainField_Enemy_Golem_Little_Fire_2495332117",
0xb084776d,"MainField_Enemy_Golem_Little_Fire_2683196654",
0xe1cecacd,"MainField_Enemy_Golem_Little_Fire_2772066651",
0x7a803207,"MainField_Enemy_Golem_Little_Fire_277868335",
0xad902177,"MainField_Enemy_Golem_Little_Fire_2810211599",
0xd72d38b5,"MainField_Enemy_Golem_Little_Fire_3069764763",
0x642a4a73,"MainField_Enemy_Golem_Little_Fire_3287583050",
0x39c14125,"MainField_Enemy_Golem_Little_Fire_3309756620",
0xaa573e22,"MainField_Enemy_Golem_Little_Fire_3357701502",
0xf502ed38,"MainField_Enemy_Golem_Little_Fire_3530959412",
0x6cb28f57,"MainField_Enemy_Golem_Little_Fire_3576222113",
0x66d4c0dd,"MainField_Enemy_Golem_Little_Fire_3589838160",
0xae5f244a,"MainField_Enemy_Golem_Little_Fire_3590591370",
0x015ce35c,"MainField_Enemy_Golem_Little_Fire_3728812696",
0xca1372b3,"MainField_Enemy_Golem_Little_Fire_3831630707",
0x76e85929,"MainField_Enemy_Golem_Little_Fire_4048539845",
0x4d4fd9f2,"MainField_Enemy_Golem_Little_Fire_4068095928",
0xcb5fa30b,"MainField_Enemy_Golem_Little_Fire_4190231963",
0x46e66518,"MainField_Enemy_Golem_Little_Fire_4217236343",
0x22747938,"MainField_Enemy_Golem_Little_Fire_4254605257",
0x1219e79a,"MainField_Enemy_Golem_Little_Fire_514378947",
0x6be3b48e,"MainField_Enemy_Golem_Little_Fire_560461234",
0xd24c9b81,"MainField_Enemy_Golem_Little_Fire_567343403",
0x57c586fa,"MainField_Enemy_Golem_Little_Fire_599463312",
0xf19c47d2,"MainField_Enemy_Golem_Little_Fire_60314273",
0xcd957902,"MainField_Enemy_Golem_Little_Fire_667550225",
0x31af8bd3,"MainField_Enemy_Golem_Little_Fire_708216968",
0xa5013ed7,"MainField_Enemy_Golem_Little_Fire_958369144",
0x79339367,"MainField_Enemy_Golem_Little_Ice_104486458",
0x559c4c6c,"MainField_Enemy_Golem_Little_Ice_1080088104",
0x9e76eb27,"MainField_Enemy_Golem_Little_Ice_1113240581",
0xe04c4314,"MainField_Enemy_Golem_Little_Ice_1122249556",
0xc924efe2,"MainField_Enemy_Golem_Little_Ice_1207575313",
0xcc1923b3,"MainField_Enemy_Golem_Little_Ice_1251611130",
0x9c0c65be,"MainField_Enemy_Golem_Little_Ice_1300286746",
0x00da5be7,"MainField_Enemy_Golem_Little_Ice_1314806381",
0x0405e0da,"MainField_Enemy_Golem_Little_Ice_1326615689",
0x3b63a02d,"MainField_Enemy_Golem_Little_Ice_1358568910",
0x43f9934e,"MainField_Enemy_Golem_Little_Ice_1386712420",
0x0ee79997,"MainField_Enemy_Golem_Little_Ice_1413195769",
0xf6d8c1e1,"MainField_Enemy_Golem_Little_Ice_1507185336",
0x2456abdf,"MainField_Enemy_Golem_Little_Ice_1511053427",
0xdedbfa30,"MainField_Enemy_Golem_Little_Ice_1525092060",
0xf1d9d353,"MainField_Enemy_Golem_Little_Ice_1559084423",
0xbf73cf6b,"MainField_Enemy_Golem_Little_Ice_159494551",
0x4eb6510a,"MainField_Enemy_Golem_Little_Ice_1672988259",
0x232588e8,"MainField_Enemy_Golem_Little_Ice_1685231741",
0xc53ed306,"MainField_Enemy_Golem_Little_Ice_1768747783",
0xa74d5cc3,"MainField_Enemy_Golem_Little_Ice_1773155185",
0x5d778b3c,"MainField_Enemy_Golem_Little_Ice_180143677",
0xe8301552,"MainField_Enemy_Golem_Little_Ice_1823487044",
0x2e4d9f92,"MainField_Enemy_Golem_Little_Ice_1839674645",
0x3a7def6c,"MainField_Enemy_Golem_Little_Ice_1895632717",
0xda241715,"MainField_Enemy_Golem_Little_Ice_1941089944",
0xe90e605c,"MainField_Enemy_Golem_Little_Ice_1988051576",
0xc9cc1c54,"MainField_Enemy_Golem_Little_Ice_2036022840",
0xd094128e,"MainField_Enemy_Golem_Little_Ice_2065188053",
0xda5c8ca3,"MainField_Enemy_Golem_Little_Ice_2066871400",
0x9805d136,"MainField_Enemy_Golem_Little_Ice_2071071591",
0x568b49bb,"MainField_Enemy_Golem_Little_Ice_2075408184",
0x3deca333,"MainField_Enemy_Golem_Little_Ice_2103883670",
0x37da2285,"MainField_Enemy_Golem_Little_Ice_2130351636",
0xb00ffc79,"MainField_Enemy_Golem_Little_Ice_2218804952",
0x80e72a3c,"MainField_Enemy_Golem_Little_Ice_2227823524",
0x8272dbb1,"MainField_Enemy_Golem_Little_Ice_2252804642",
0xa4c41ac7,"MainField_Enemy_Golem_Little_Ice_2269320666",
0xb13f786d,"MainField_Enemy_Golem_Little_Ice_2387815353",
0xea33fe14,"MainField_Enemy_Golem_Little_Ice_238837886",
0xf6b1f7da,"MainField_Enemy_Golem_Little_Ice_2397510757",
0x09531b2a,"MainField_Enemy_Golem_Little_Ice_239823894",
0x894d7591,"MainField_Enemy_Golem_Little_Ice_2403151732",
0xbc6ebd56,"MainField_Enemy_Golem_Little_Ice_2465828481",
0xf1df9af8,"MainField_Enemy_Golem_Little_Ice_2484961700",
0x3cb41446,"MainField_Enemy_Golem_Little_Ice_2611812732",
0xb43c376e,"MainField_Enemy_Golem_Little_Ice_2635988187",
0x4c048738,"MainField_Enemy_Golem_Little_Ice_2636371732",
0x19866ab1,"MainField_Enemy_Golem_Little_Ice_274897826",
0xc1a92cba,"MainField_Enemy_Golem_Little_Ice_2773793559",
0x17448b63,"MainField_Enemy_Golem_Little_Ice_2786168298",
0x589d71cb,"MainField_Enemy_Golem_Little_Ice_282854643",
0xa2b35edc,"MainField_Enemy_Golem_Little_Ice_2899037577",
0x70ecc4c6,"MainField_Enemy_Golem_Little_Ice_2908563266",
0x2921f026,"MainField_Enemy_Golem_Little_Ice_2953802786",
0xbe7612e3,"MainField_Enemy_Golem_Little_Ice_2957680657",
0xf623d002,"MainField_Enemy_Golem_Little_Ice_3054866454",
0x30e5a1f8,"MainField_Enemy_Golem_Little_Ice_3064663879",
0xe81cbb8b,"MainField_Enemy_Golem_Little_Ice_3072210854",
0x8e5f258f,"MainField_Enemy_Golem_Little_Ice_3143477402",
0x9460c51b,"MainField_Enemy_Golem_Little_Ice_3184173562",
0x56344690,"MainField_Enemy_Golem_Little_Ice_3252069761",
0x1714d256,"MainField_Enemy_Golem_Little_Ice_3266657819",
0x2bd516a9,"MainField_Enemy_Golem_Little_Ice_3271616741",
0x5f58e1bb,"MainField_Enemy_Golem_Little_Ice_3332615673",
0x831b0661,"MainField_Enemy_Golem_Little_Ice_3352466265",
0x5df62985,"MainField_Enemy_Golem_Little_Ice_3393529715",
0xd3ece55d,"MainField_Enemy_Golem_Little_Ice_3474716851",
0xbdb226d4,"MainField_Enemy_Golem_Little_Ice_3585099001",
0xa16b5160,"MainField_Enemy_Golem_Little_Ice_3586499884",
0x5c1d2456,"MainField_Enemy_Golem_Little_Ice_3598215551",
0x49c92fb2,"MainField_Enemy_Golem_Little_Ice_3631703548",
0x0a51b1b5,"MainField_Enemy_Golem_Little_Ice_3633159584",
0x29af4823,"MainField_Enemy_Golem_Little_Ice_3708092627",
0xcb14be04,"MainField_Enemy_Golem_Little_Ice_3710791781",
0xa69325fa,"MainField_Enemy_Golem_Little_Ice_3717403928",
0x40fd6744,"MainField_Enemy_Golem_Little_Ice_3728917372",
0x05e03fb6,"MainField_Enemy_Golem_Little_Ice_3749341763",
0xc369b3a0,"MainField_Enemy_Golem_Little_Ice_3756287049",
0xf0eaf24d,"MainField_Enemy_Golem_Little_Ice_3828203441",
0xa7e62eb4,"MainField_Enemy_Golem_Little_Ice_3834847456",
0xe7cc38a5,"MainField_Enemy_Golem_Little_Ice_3839704391",
0xd11705b4,"MainField_Enemy_Golem_Little_Ice_3885003746",
0x91c64a3e,"MainField_Enemy_Golem_Little_Ice_3904504815",
0x243ecb4b,"MainField_Enemy_Golem_Little_Ice_3915745085",
0xbc10a015,"MainField_Enemy_Golem_Little_Ice_3974961742",
0xbfe003df,"MainField_Enemy_Golem_Little_Ice_4010862330",
0xc844ad7f,"MainField_Enemy_Golem_Little_Ice_402035949",
0x32047d74,"MainField_Enemy_Golem_Little_Ice_4026272294",
0x30a8c329,"MainField_Enemy_Golem_Little_Ice_4101523938",
0x579e9fc7,"MainField_Enemy_Golem_Little_Ice_410255045",
0x6bda5e43,"MainField_Enemy_Golem_Little_Ice_4114297051",
0x56cf6796,"MainField_Enemy_Golem_Little_Ice_4140585504",
0xd3d91d1a,"MainField_Enemy_Golem_Little_Ice_4294851010",
0x0e815833,"MainField_Enemy_Golem_Little_Ice_442349961",
0xd384bfc3,"MainField_Enemy_Golem_Little_Ice_489333550",
0xed4d1526,"MainField_Enemy_Golem_Little_Ice_508692713",
0x0715fb02,"MainField_Enemy_Golem_Little_Ice_517107305",
0xee80a3b0,"MainField_Enemy_Golem_Little_Ice_557063790",
0x2e15c059,"MainField_Enemy_Golem_Little_Ice_56636238",
0x28eff7fd,"MainField_Enemy_Golem_Little_Ice_572499201",
0x0a5b9792,"MainField_Enemy_Golem_Little_Ice_650894496",
0x407977af,"MainField_Enemy_Golem_Little_Ice_692724928",
0xd08d523d,"MainField_Enemy_Golem_Little_Ice_716252783",
0x82624d6f,"MainField_Enemy_Golem_Little_Ice_723865476",
0x9a0784d4,"MainField_Enemy_Golem_Little_Ice_770511078",
0x4af0062e,"MainField_Enemy_Golem_Little_Ice_785244347",
0x769db952,"MainField_Enemy_Golem_Little_Ice_838608604",
0x159755ce,"MainField_Enemy_Golem_Little_Ice_861157724",
0x1fc7c918,"MainField_Enemy_Golem_Little_Ice_965816389",
0xd3e96907,"MainField_Enemy_Golem_Little_Ice_995417248",
0xa87a30e2,"MainField_Enemy_Golem_Middle_1063952470",
0x3c85c2d7,"MainField_Enemy_Golem_Middle_2367737901",
0xe6978d65,"MainField_Enemy_Golem_Middle_2827830463",
0x251b0d72,"MainField_Enemy_Golem_Middle_3273877672",
0xba83c05e,"MainField_Enemy_Golem_Middle_3529818719",
0x3252e293,"MainField_Enemy_Golem_Middle_3605751469",
0x27d02d09,"MainField_Enemy_Golem_Middle_372172601",
0x80d1f7b4,"MainField_Enemy_Golem_Middle_3812413946",
0xdce8845a,"MainField_Enemy_Golem_Middle_3900382371",
0x2837bf79,"MainField_Enemy_Golem_Middle_491623064",
0x2f6a9d1f,"MainField_Enemy_Golem_Middle_853217538",
0x07975c46,"MainField_Enemy_Golem_Middle_975119268",
0x783001cf,"MainField_Enemy_Golem_Senior_1943962273",
0x3c033f82,"MainField_Enemy_Golem_Senior_2455625785",
0x79152641,"MainField_Enemy_Golem_Senior_2530932938",
0xd42bb083,"MainField_Enemy_Golem_Senior_2763213354",
0x07cb6197,"MainField_Enemy_Golem_Senior_321907823",
0xde6d0fbd,"MainField_Enemy_Golem_Senior_3869714669",
0xc022a633,"MainField_Enemy_Golem_Senior_464316212",
0x50cdda79,"MainField_Enemy_Guardian_A_1026285922",
0x2cbc7c85,"MainField_Enemy_Guardian_A_1190835183",
0x7085bbf1,"MainField_Enemy_Guardian_A_128179700",
0x23835295,"MainField_Enemy_Guardian_A_1347313845",
0x0ee0b229,"MainField_Enemy_Guardian_A_1349980722",
0x7eb3ca60,"MainField_Enemy_Guardian_A_1429678381",
0xf121883e,"MainField_Enemy_Guardian_A_144669291",
0x89b46e93,"MainField_Enemy_Guardian_A_1543401762",
0xdcfe5e63,"MainField_Enemy_Guardian_A_1577003372",
0x53fcc071,"MainField_Enemy_Guardian_A_1736524803",
0xf6bd38df,"MainField_Enemy_Guardian_A_220762433",
0xcad8d940,"MainField_Enemy_Guardian_A_2325055349",
0x1c3dab2d,"MainField_Enemy_Guardian_A_2328048419",
0xed731483,"MainField_Enemy_Guardian_A_238847606",
0x63525fd0,"MainField_Enemy_Guardian_A_2485240059",
0x24874c4f,"MainField_Enemy_Guardian_A_2540234944",
0x1797330b,"MainField_Enemy_Guardian_A_2541954314",
0xf68ac332,"MainField_Enemy_Guardian_A_2700810178",
0xcce64cf8,"MainField_Enemy_Guardian_A_2772275816",
0x7608d3c3,"MainField_Enemy_Guardian_A_3140496781",
0xb052a4fd,"MainField_Enemy_Guardian_A_3231674089",
0x219270a2,"MainField_Enemy_Guardian_A_3348421845",
0x57ea8cd8,"MainField_Enemy_Guardian_A_3458921750",
0x684856d4,"MainField_Enemy_Guardian_A_367948831",
0x36cdefb9,"MainField_Enemy_Guardian_A_3707116099",
0xcc42305f,"MainField_Enemy_Guardian_A_3774681053",
0x56181660,"MainField_Enemy_Guardian_A_3800503521",
0x61763c21,"MainField_Enemy_Guardian_A_3810979949",
0xc903b671,"MainField_Enemy_Guardian_A_3916617587",
0x7a53d368,"MainField_Enemy_Guardian_A_4060806309",
0x63a9953d,"MainField_Enemy_Guardian_A_4213135951",
0xab18d37b,"MainField_Enemy_Guardian_A_4259292662",
0x0087ae7f,"MainField_Enemy_Guardian_A_626309166",
0x5ecf6890,"MainField_Enemy_Guardian_A_744877403",
0x47b2d989,"MainField_Enemy_Guardian_A_82492342",
0x6fbc554a,"MainField_Enemy_Guardian_A_888231907",
0xcd956d57,"MainField_Enemy_Guardian_A_Fixed_Moss_1036545592",
0x5856258f,"MainField_Enemy_Guardian_A_Fixed_Moss_1098862896",
0x1345ba43,"MainField_Enemy_Guardian_A_Fixed_Moss_1116061641",
0xfc04e0ee,"MainField_Enemy_Guardian_A_Fixed_Moss_1129871460",
0xe2d27a77,"MainField_Enemy_Guardian_A_Fixed_Moss_1131939274",
0x75fbf9fa,"MainField_Enemy_Guardian_A_Fixed_Moss_1258322447",
0x8d5f53c9,"MainField_Enemy_Guardian_A_Fixed_Moss_1388604117",
0xba7183d9,"MainField_Enemy_Guardian_A_Fixed_Moss_1514696366",
0x4e365996,"MainField_Enemy_Guardian_A_Fixed_Moss_15757298",
0x71828dec,"MainField_Enemy_Guardian_A_Fixed_Moss_1684975308",
0x4cc0e950,"MainField_Enemy_Guardian_A_Fixed_Moss_1714130641",
0x521ddf18,"MainField_Enemy_Guardian_A_Fixed_Moss_1792291780",
0x9d28a155,"MainField_Enemy_Guardian_A_Fixed_Moss_1795667543",
0xb3c61b1b,"MainField_Enemy_Guardian_A_Fixed_Moss_1831683864",
0x14d12b81,"MainField_Enemy_Guardian_A_Fixed_Moss_1868928329",
0x352061e6,"MainField_Enemy_Guardian_A_Fixed_Moss_1887892887",
0xaabefb7d,"MainField_Enemy_Guardian_A_Fixed_Moss_1901107858",
0x01fe5990,"MainField_Enemy_Guardian_A_Fixed_Moss_2036635130",
0xfc01bbbc,"MainField_Enemy_Guardian_A_Fixed_Moss_2099244869",
0x26c50041,"MainField_Enemy_Guardian_A_Fixed_Moss_2126640931",
0xe9767e4f,"MainField_Enemy_Guardian_A_Fixed_Moss_217044693",
0x45a1b545,"MainField_Enemy_Guardian_A_Fixed_Moss_2278677267",
0xa6081956,"MainField_Enemy_Guardian_A_Fixed_Moss_2329449824",
0x2b17f4be,"MainField_Enemy_Guardian_A_Fixed_Moss_2446549569",
0x88c2a74e,"MainField_Enemy_Guardian_A_Fixed_Moss_2729804071",
0x8fa7c94c,"MainField_Enemy_Guardian_A_Fixed_Moss_2919713469",
0x448de513,"MainField_Enemy_Guardian_A_Fixed_Moss_3061645066",
0xa379c566,"MainField_Enemy_Guardian_A_Fixed_Moss_3071761527",
0x85e41b09,"MainField_Enemy_Guardian_A_Fixed_Moss_323452567",
0xe0880b11,"MainField_Enemy_Guardian_A_Fixed_Moss_332514522",
0x9091c965,"MainField_Enemy_Guardian_A_Fixed_Moss_3418993317",
0x9bcb4876,"MainField_Enemy_Guardian_A_Fixed_Moss_3453452167",
0xc2fcad3f,"MainField_Enemy_Guardian_A_Fixed_Moss_3559982419",
0xf3e47a20,"MainField_Enemy_Guardian_A_Fixed_Moss_3564093310",
0x7f974112,"MainField_Enemy_Guardian_A_Fixed_Moss_3569838413",
0x7351e20f,"MainField_Enemy_Guardian_A_Fixed_Moss_3690642414",
0x06170d8d,"MainField_Enemy_Guardian_A_Fixed_Moss_3747349295",
0x36e3c303,"MainField_Enemy_Guardian_A_Fixed_Moss_3878168362",
0xc4fd0914,"MainField_Enemy_Guardian_A_Fixed_Moss_4088634583",
0x4adaa820,"MainField_Enemy_Guardian_A_Fixed_Moss_4099617698",
0xac292c82,"MainField_Enemy_Guardian_A_Fixed_Moss_418960964",
0x2386e8fa,"MainField_Enemy_Guardian_A_Fixed_Moss_486930483",
0x80d10f74,"MainField_Enemy_Guardian_A_Fixed_Moss_545803614",
0x1699b429,"MainField_Enemy_Guardian_A_Fixed_Moss_731812326",
0xb4445a48,"MainField_Enemy_Guardian_A_Fixed_Moss_900880028",
0x3ec49152,"MainField_Enemy_Guardian_A_Fixed_Moss_924505285",
0xba75f786,"MainField_Enemy_Guardian_A_Fixed_Moss_948025218",
0x968fce21,"MainField_Enemy_Guardian_A_Fixed_Moss_966286228",
0xd354d5a3,"MainField_Enemy_Guardian_A_Fixed_Sand_3131113819",
0xa4822ba3,"MainField_Enemy_Guardian_A_Fixed_Sand_3598885493",
0x057106e4,"MainField_Enemy_Guardian_A_Fixed_Snow_3554407044",
0x68fdee5f,"MainField_Enemy_Guardian_A_Fixed_Snow_3945224292",
0xb3a8bcdc,"MainField_Enemy_Guardian_A_Fixed_Snow_4170457441",
0x08779098,"MainField_Enemy_Guardian_A_Moss_129785380",
0x17900680,"MainField_Enemy_Guardian_A_Moss_2373028017",
0xae0b035b,"MainField_Enemy_Guardian_A_Moss_2450204935",
0xb6db1267,"MainField_Enemy_Guardian_A_Sand_2479341154",
0xc12530c4,"MainField_Enemy_Guardian_A_Sand_3060564958",
0x5f655d8d,"MainField_Enemy_Guardian_A_Sand_3346995733",
0x18e4577a,"MainField_Enemy_Guardian_A_Sand_3559217329",
0xc0a187d8,"MainField_Enemy_Guardian_A_Sand_568632014",
0x9a2b1feb,"MainField_Enemy_Guardian_A_Sand_743552386",
0xbe9ce067,"MainField_Enemy_Guardian_A_Snow_3862084975",
0x7c9286fc,"MainField_Enemy_Guardian_B_1398978899",
0xb0ce95d9,"MainField_Enemy_Guardian_B_1460989990",
0x7ac56c62,"MainField_Enemy_Guardian_B_1820487938",
0x86fd003b,"MainField_Enemy_Guardian_B_1847558585",
0x4e2f7752,"MainField_Enemy_Guardian_B_2073400203",
0x7d119152,"MainField_Enemy_Guardian_B_2604187106",
0x9f228cd7,"MainField_Enemy_Guardian_B_2612981944",
0xdd6b6137,"MainField_Enemy_Guardian_B_2860617295",
0xd9d8ec0c,"MainField_Enemy_Guardian_B_3074396949",
0xe918e91d,"MainField_Enemy_Guardian_B_3187317260",
0x72f53877,"MainField_Enemy_Guardian_B_3288645779",
0x6780101d,"MainField_Enemy_Guardian_B_3443957340",
0xde1addc7,"MainField_Enemy_Guardian_B_3650877381",
0xf64e9be0,"MainField_Enemy_Guardian_B_3792533217",
0x994c4ab3,"MainField_Enemy_Guardian_B_38462232",
0xb2648fe2,"MainField_Enemy_Guardian_B_3903097138",
0x447df0f2,"MainField_Enemy_Guardian_B_4084210840",
0x2f5ed976,"MainField_Enemy_Guardian_B_4112159848",
0xc6541959,"MainField_Enemy_Guardian_B_4292844913",
0xf08e0c69,"MainField_Enemy_Guardian_B_604932524",
0x40a058da,"MainField_Enemy_Guardian_C_1009936452",
0xe251f5f4,"MainField_Enemy_Guardian_C_1165739025",
0xb929bbb2,"MainField_Enemy_Guardian_C_1329210538",
0x6b5646b2,"MainField_Enemy_Guardian_C_1410780922",
0x1d429637,"MainField_Enemy_Guardian_C_1469234556",
0xcd71af71,"MainField_Enemy_Guardian_C_1805862402",
0x8ceba775,"MainField_Enemy_Guardian_C_1981635416",
0x50228d10,"MainField_Enemy_Guardian_C_1983585774",
0x3dda7870,"MainField_Enemy_Guardian_C_2116637744",
0xc2f99ae4,"MainField_Enemy_Guardian_C_2249200231",
0x28ea5ad1,"MainField_Enemy_Guardian_C_2293343495",
0x7713aa45,"MainField_Enemy_Guardian_C_2479542683",
0xce3b8cd8,"MainField_Enemy_Guardian_C_2557205703",
0xba5184d9,"MainField_Enemy_Guardian_C_2643409954",
0x5cfa1ab3,"MainField_Enemy_Guardian_C_2802402972",
0x37c74906,"MainField_Enemy_Guardian_C_2900500761",
0x776fc5b4,"MainField_Enemy_Guardian_C_2994951342",
0x5fa88972,"MainField_Enemy_Guardian_C_3132578758",
0x2c32121e,"MainField_Enemy_Guardian_C_3251855747",
0x66dfd6bc,"MainField_Enemy_Guardian_C_3534493673",
0x4312d606,"MainField_Enemy_Guardian_C_3980550772",
0x571a66b1,"MainField_Enemy_Guardian_C_4006920142",
0x066841e7,"MainField_Enemy_Guardian_C_4117946208",
0x43cd1093,"MainField_Enemy_Guardian_C_510946050",
0x7f2dcce9,"MainField_Enemy_Guardian_C_557056845",
0x5f8bc4f6,"MainField_Enemy_Guardian_C_645337006",
0xe6e73bb4,"MainField_Enemy_Guardian_C_684303411",
0x584b3327,"MainField_Enemy_Guardian_C_856671268",
0x2987089d,"MainField_Enemy_Guardian_C_887589490",
0xcfceb56c,"MainField_Enemy_Keese_1043107442",
0x0159b3cb,"MainField_Enemy_Keese_1043864812",
0x33eb21fe,"MainField_Enemy_Keese_1058993090",
0x781ecece,"MainField_Enemy_Keese_1115663945",
0xb967d603,"MainField_Enemy_Keese_1133351874",
0xb42f45d9,"MainField_Enemy_Keese_1180660342",
0x773324ed,"MainField_Enemy_Keese_1205242466",
0x77dc9fb1,"MainField_Enemy_Keese_1219994204",
0xad4b1475,"MainField_Enemy_Keese_1339947119",
0x593df372,"MainField_Enemy_Keese_1457318060",
0xaa1d848c,"MainField_Enemy_Keese_1470145664",
0xd13f3971,"MainField_Enemy_Keese_1531013771",
0x2adafbd9,"MainField_Enemy_Keese_1609545747",
0x34e7088d,"MainField_Enemy_Keese_1611635534",
0x144da3f2,"MainField_Enemy_Keese_1646003120",
0x887a70f0,"MainField_Enemy_Keese_1712688420",
0x91e42500,"MainField_Enemy_Keese_1735350880",
0x351b1a15,"MainField_Enemy_Keese_1744222214",
0xb76412f9,"MainField_Enemy_Keese_1894349074",
0x18e47479,"MainField_Enemy_Keese_1939322068",
0x6ecc4ed4,"MainField_Enemy_Keese_2003319377",
0x344b960c,"MainField_Enemy_Keese_2046803644",
0x8c15cf6c,"MainField_Enemy_Keese_2098935366",
0x9246c946,"MainField_Enemy_Keese_2129812044",
0x7150ad8f,"MainField_Enemy_Keese_2134978500",
0x699c8048,"MainField_Enemy_Keese_2173104293",
0x58a4eb62,"MainField_Enemy_Keese_2263108288",
0x4d373879,"MainField_Enemy_Keese_2278665654",
0x2bff05d0,"MainField_Enemy_Keese_2318275515",
0x59a4ef28,"MainField_Enemy_Keese_2387814319",
0xadfb8c2b,"MainField_Enemy_Keese_2413062540",
0xe4d7e711,"MainField_Enemy_Keese_2495496055",
0x4f237763,"MainField_Enemy_Keese_2512658467",
0x3a6591f0,"MainField_Enemy_Keese_2519628608",
0x2f384c78,"MainField_Enemy_Keese_2573622141",
0xe30018e8,"MainField_Enemy_Keese_2656454134",
0xae57eeeb,"MainField_Enemy_Keese_2690367941",
0x37e6a712,"MainField_Enemy_Keese_2709603470",
0x298f8bf7,"MainField_Enemy_Keese_275278582",
0x91dcfe1a,"MainField_Enemy_Keese_2763767725",
0x11dd30f2,"MainField_Enemy_Keese_2778550403",
0x415a3510,"MainField_Enemy_Keese_2820358009",
0xc6329d5b,"MainField_Enemy_Keese_282794909",
0x387535a6,"MainField_Enemy_Keese_284264592",
0x1d343f3e,"MainField_Enemy_Keese_2880189162",
0x1149a826,"MainField_Enemy_Keese_2884441341",
0x83a8d8f6,"MainField_Enemy_Keese_3053900901",
0xacea657b,"MainField_Enemy_Keese_3072382342",
0xc5f7de17,"MainField_Enemy_Keese_3100188597",
0x9f9bb7d7,"MainField_Enemy_Keese_3147519554",
0x5628e5d8,"MainField_Enemy_Keese_3163941254",
0xc25dd7a0,"MainField_Enemy_Keese_3212608908",
0x44b831c9,"MainField_Enemy_Keese_3236177636",
0xd2d47a56,"MainField_Enemy_Keese_3243334676",
0x4fe6ba45,"MainField_Enemy_Keese_3247965845",
0x37e6f77a,"MainField_Enemy_Keese_3263906570",
0x6315e1cc,"MainField_Enemy_Keese_329012468",
0xdc77fec1,"MainField_Enemy_Keese_3306807660",
0x493013f6,"MainField_Enemy_Keese_3499254875",
0x83a842ac,"MainField_Enemy_Keese_351107974",
0x6016994b,"MainField_Enemy_Keese_3512058973",
0xe239b596,"MainField_Enemy_Keese_3524430644",
0xe0ba7078,"MainField_Enemy_Keese_3531684574",
0xc96b810d,"MainField_Enemy_Keese_3580077126",
0x1efce369,"MainField_Enemy_Keese_360894784",
0xd13dc8f4,"MainField_Enemy_Keese_3714755784",
0xe7aef5cb,"MainField_Enemy_Keese_3732935511",
0xf326e1b4,"MainField_Enemy_Keese_3753710711",
0x2b362ef1,"MainField_Enemy_Keese_3797611219",
0xda64b3fe,"MainField_Enemy_Keese_3841544263",
0xb5ab3fee,"MainField_Enemy_Keese_3873814184",
0x9bdf04cd,"MainField_Enemy_Keese_3922237190",
0xe1cd70c1,"MainField_Enemy_Keese_3923781800",
0xe4be4496,"MainField_Enemy_Keese_397492654",
0x8c82c6d9,"MainField_Enemy_Keese_3999362027",
0x5d0f6d2f,"MainField_Enemy_Keese_4006047341",
0xf5b8e331,"MainField_Enemy_Keese_4034365112",
0x9fc2d8bc,"MainField_Enemy_Keese_4095766286",
0x45eefc7e,"MainField_Enemy_Keese_4165647228",
0xcb2e076a,"MainField_Enemy_Keese_41995802",
0x80f4c3b7,"MainField_Enemy_Keese_4256219467",
0x75b0a044,"MainField_Enemy_Keese_484733704",
0xe0e992f7,"MainField_Enemy_Keese_493550978",
0x8b099923,"MainField_Enemy_Keese_506710967",
0x33831235,"MainField_Enemy_Keese_520498369",
0xe850138d,"MainField_Enemy_Keese_556497332",
0xd6ccb923,"MainField_Enemy_Keese_628592519",
0xa3c8060c,"MainField_Enemy_Keese_678521571",
0x8c79097a,"MainField_Enemy_Keese_695489465",
0xa04e5168,"MainField_Enemy_Keese_752859259",
0x52cb5dec,"MainField_Enemy_Keese_765696010",
0xa7bd0536,"MainField_Enemy_Keese_793358765",
0x722c2e63,"MainField_Enemy_Keese_810047634",
0xb1e73aaf,"MainField_Enemy_Keese_838424952",
0x168d059b,"MainField_Enemy_Keese_843994347",
0x8d436323,"MainField_Enemy_Keese_900640974",
0xc8cb1b4f,"MainField_Enemy_Keese_997420845",
0x01d6e2c2,"MainField_Enemy_Keese_AllDay_1053216703",
0x269cc0e0,"MainField_Enemy_Keese_AllDay_1054148190",
0xb9b4d14d,"MainField_Enemy_Keese_AllDay_1068689744",
0x12d5404f,"MainField_Enemy_Keese_AllDay_1075948302",
0xaef2e809,"MainField_Enemy_Keese_AllDay_1104857829",
0xdb105773,"MainField_Enemy_Keese_AllDay_1153700649",
0x46ec40b7,"MainField_Enemy_Keese_AllDay_1258242981",
0xb292e88e,"MainField_Enemy_Keese_AllDay_126130721",
0xceac7a92,"MainField_Enemy_Keese_AllDay_1263357705",
0x2734eaa1,"MainField_Enemy_Keese_AllDay_1267769360",
0xf9feabe5,"MainField_Enemy_Keese_AllDay_1294826328",
0x37b2b8c0,"MainField_Enemy_Keese_AllDay_1378881157",
0xed7167a5,"MainField_Enemy_Keese_AllDay_1397860426",
0x02b99c1c,"MainField_Enemy_Keese_AllDay_160198103",
0x5bdc8ffa,"MainField_Enemy_Keese_AllDay_1634164865",
0x8c04c89a,"MainField_Enemy_Keese_AllDay_1706237799",
0x4de2750e,"MainField_Enemy_Keese_AllDay_1794206549",
0xc5ffaf7e,"MainField_Enemy_Keese_AllDay_1819240334",
0x61306597,"MainField_Enemy_Keese_AllDay_1875926029",
0x27d6694f,"MainField_Enemy_Keese_AllDay_1912092088",
0x34847690,"MainField_Enemy_Keese_AllDay_1924732515",
0x3e6986f6,"MainField_Enemy_Keese_AllDay_1937978475",
0x49e3956c,"MainField_Enemy_Keese_AllDay_1943232225",
0x714bdfef,"MainField_Enemy_Keese_AllDay_1943384257",
0xcc32aa3b,"MainField_Enemy_Keese_AllDay_1947791879",
0x34ccd3f9,"MainField_Enemy_Keese_AllDay_1959511061",
0xd41e0678,"MainField_Enemy_Keese_AllDay_1974948969",
0xb670eb4f,"MainField_Enemy_Keese_AllDay_1999133889",
0xa541ebcb,"MainField_Enemy_Keese_AllDay_2072151722",
0x891c35c9,"MainField_Enemy_Keese_AllDay_2087827184",
0x4553ba6b,"MainField_Enemy_Keese_AllDay_211876828",
0xdef9c0e8,"MainField_Enemy_Keese_AllDay_2119228628",
0x02265e4c,"MainField_Enemy_Keese_AllDay_2156798233",
0x03c823d8,"MainField_Enemy_Keese_AllDay_2191726037",
0xe77bf17d,"MainField_Enemy_Keese_AllDay_2211571776",
0xb42ddc5d,"MainField_Enemy_Keese_AllDay_234219695",
0x4bb428c0,"MainField_Enemy_Keese_AllDay_2372371592",
0x534da0cc,"MainField_Enemy_Keese_AllDay_2385266868",
0x4d410713,"MainField_Enemy_Keese_AllDay_2403637001",
0x0233fb96,"MainField_Enemy_Keese_AllDay_2441985259",
0xe2d31033,"MainField_Enemy_Keese_AllDay_2461044916",
0x17d4f154,"MainField_Enemy_Keese_AllDay_2488014025",
0xcb3ad638,"MainField_Enemy_Keese_AllDay_2525011593",
0x200e1f76,"MainField_Enemy_Keese_AllDay_2555782799",
0x9d96e2fe,"MainField_Enemy_Keese_AllDay_2562751637",
0xc7becad3,"MainField_Enemy_Keese_AllDay_2570574981",
0x47d24444,"MainField_Enemy_Keese_AllDay_2644318640",
0xafc5720c,"MainField_Enemy_Keese_AllDay_2696503062",
0xa03d37fb,"MainField_Enemy_Keese_AllDay_2723432956",
0x703a7e40,"MainField_Enemy_Keese_AllDay_2753409494",
0xd1209fc6,"MainField_Enemy_Keese_AllDay_2804447324",
0xac6638c6,"MainField_Enemy_Keese_AllDay_2816801862",
0xce520ebe,"MainField_Enemy_Keese_AllDay_2832028646",
0x58382cc1,"MainField_Enemy_Keese_AllDay_2836043133",
0xf2a81776,"MainField_Enemy_Keese_AllDay_2852501761",
0x02dd7678,"MainField_Enemy_Keese_AllDay_2979690737",
0x4066aad5,"MainField_Enemy_Keese_AllDay_3002990470",
0x625a6f0f,"MainField_Enemy_Keese_AllDay_3044595384",
0x127907a1,"MainField_Enemy_Keese_AllDay_3091504863",
0xfce22899,"MainField_Enemy_Keese_AllDay_3170725204",
0xb547c4ae,"MainField_Enemy_Keese_AllDay_3185103097",
0x56cfa9a5,"MainField_Enemy_Keese_AllDay_3187588578",
0xcd07ef7f,"MainField_Enemy_Keese_AllDay_3188922109",
0xe7c624ea,"MainField_Enemy_Keese_AllDay_3258627224",
0x69634400,"MainField_Enemy_Keese_AllDay_3263320643",
0xbbae990a,"MainField_Enemy_Keese_AllDay_3268536806",
0xed8ab9de,"MainField_Enemy_Keese_AllDay_3353289918",
0x18c44f96,"MainField_Enemy_Keese_AllDay_3366391559",
0x52df7b62,"MainField_Enemy_Keese_AllDay_3390216452",
0xe55dd5d4,"MainField_Enemy_Keese_AllDay_3418834490",
0x6242015a,"MainField_Enemy_Keese_AllDay_3436653711",
0x5e2d546b,"MainField_Enemy_Keese_AllDay_3443127069",
0xb46c14d4,"MainField_Enemy_Keese_AllDay_3455651653",
0x4cd08fd7,"MainField_Enemy_Keese_AllDay_3491255872",
0x69bee60c,"MainField_Enemy_Keese_AllDay_3528705780",
0x8ffcaa37,"MainField_Enemy_Keese_AllDay_3529277436",
0xba0b6142,"MainField_Enemy_Keese_AllDay_3566377174",
0xf9e2d228,"MainField_Enemy_Keese_AllDay_3634153783",
0x97f67220,"MainField_Enemy_Keese_AllDay_363454767",
0xb31422ea,"MainField_Enemy_Keese_AllDay_3720159568",
0x879054be,"MainField_Enemy_Keese_AllDay_3742695727",
0xf2ac15c0,"MainField_Enemy_Keese_AllDay_3801274938",
0xa7f9431f,"MainField_Enemy_Keese_AllDay_3811207618",
0x3d6d2997,"MainField_Enemy_Keese_AllDay_3857616401",
0xb49c0432,"MainField_Enemy_Keese_AllDay_3904615317",
0xf633dfab,"MainField_Enemy_Keese_AllDay_3938239830",
0x17db2665,"MainField_Enemy_Keese_AllDay_3938633975",
0x45e40247,"MainField_Enemy_Keese_AllDay_3940637790",
0xd9be97c7,"MainField_Enemy_Keese_AllDay_3991382948",
0x1b7b8038,"MainField_Enemy_Keese_AllDay_4018075896",
0xe799511f,"MainField_Enemy_Keese_AllDay_4063176549",
0x66bb24f7,"MainField_Enemy_Keese_AllDay_4076192124",
0x7f303ce6,"MainField_Enemy_Keese_AllDay_4082001926",
0x0aab751a,"MainField_Enemy_Keese_AllDay_4164932033",
0xb2863b48,"MainField_Enemy_Keese_AllDay_4167221250",
0x3c9ed41b,"MainField_Enemy_Keese_AllDay_4214147977",
0xe856ac4d,"MainField_Enemy_Keese_AllDay_4231401856",
0xcbe1eeeb,"MainField_Enemy_Keese_AllDay_4240119762",
0x33b2a258,"MainField_Enemy_Keese_AllDay_4248386011",
0x5d06d437,"MainField_Enemy_Keese_AllDay_4278722594",
0x2dcc9a38,"MainField_Enemy_Keese_AllDay_4293459935",
0x36e3b3fb,"MainField_Enemy_Keese_AllDay_433121156",
0xa1dbcadd,"MainField_Enemy_Keese_AllDay_481449694",
0xb192d06c,"MainField_Enemy_Keese_AllDay_514643362",
0x2eb8d4a4,"MainField_Enemy_Keese_AllDay_541382949",
0xd11bba48,"MainField_Enemy_Keese_AllDay_555729773",
0xcffd0862,"MainField_Enemy_Keese_AllDay_625658228",
0x3c32abb5,"MainField_Enemy_Keese_AllDay_668954152",
0xbcfeeb72,"MainField_Enemy_Keese_AllDay_767565309",
0x6c468992,"MainField_Enemy_Keese_AllDay_770019455",
0x76c864e2,"MainField_Enemy_Keese_AllDay_85450413",
0x0ac4ccab,"MainField_Enemy_Keese_AllDay_882233695",
0x77457af9,"MainField_Enemy_Keese_AllDay_897735524",
0xe617c074,"MainField_Enemy_Keese_AllDay_920097998",
0x352c155d,"MainField_Enemy_Keese_AllDay_95161572",
0xc49a5d33,"MainField_Enemy_Keese_AllDay_960373588",
0x2dace3d4,"MainField_Enemy_Keese_AllDay_971772811",
0x773ff277,"MainField_Enemy_Keese_AllDay_993758523",
0x5216edb8,"MainField_Enemy_Keese_AllDay_99415611",
0x5fd5e5a3,"MainField_Enemy_Keese_Electric_14968992",
0x26c990af,"MainField_Enemy_Keese_Electric_2363895513",
0x11f1cf77,"MainField_Enemy_Keese_Electric_3342504389",
0x62f68602,"MainField_Enemy_Keese_Electric_3930646341",
0x9d4ca383,"MainField_Enemy_Keese_Electric_4276001173",
0x8491f2b4,"MainField_Enemy_Keese_Electric_570257980",
0x5bed2db8,"MainField_Enemy_Keese_Electric_57785112",
0xc4c5e18a,"MainField_Enemy_Keese_Electric_AllDay_1165217239",
0x9facb5f6,"MainField_Enemy_Keese_Electric_AllDay_1201046820",
0xe277d795,"MainField_Enemy_Keese_Electric_AllDay_127998033",
0x8bb25176,"MainField_Enemy_Keese_Electric_AllDay_1452814699",
0x202553f3,"MainField_Enemy_Keese_Electric_AllDay_1583393853",
0x724452a3,"MainField_Enemy_Keese_Electric_AllDay_1938033767",
0x62d4aa93,"MainField_Enemy_Keese_Electric_AllDay_1970661977",
0x551a2f16,"MainField_Enemy_Keese_Electric_AllDay_2194509452",
0xe82cdf6a,"MainField_Enemy_Keese_Electric_AllDay_2238321069",
0xaeb01461,"MainField_Enemy_Keese_Electric_AllDay_2272759695",
0x2de8676b,"MainField_Enemy_Keese_Electric_AllDay_2560693750",
0x0c601b30,"MainField_Enemy_Keese_Electric_AllDay_2577887622",
0x735587dc,"MainField_Enemy_Keese_Electric_AllDay_2662748311",
0xe863249b,"MainField_Enemy_Keese_Electric_AllDay_2801940971",
0xe6eef0bb,"MainField_Enemy_Keese_Electric_AllDay_2834749676",
0x945da4aa,"MainField_Enemy_Keese_Electric_AllDay_3207466233",
0x141a6a4d,"MainField_Enemy_Keese_Electric_AllDay_3498190668",
0xeb8d8120,"MainField_Enemy_Keese_Electric_AllDay_3581273636",
0x8c66fe82,"MainField_Enemy_Keese_Electric_AllDay_3978450689",
0xa00059d5,"MainField_Enemy_Keese_Electric_AllDay_4008616997",
0xe9cb5ae0,"MainField_Enemy_Keese_Electric_AllDay_4126301143",
0x0421a312,"MainField_Enemy_Keese_Electric_AllDay_492671261",
0xb8db8b4a,"MainField_Enemy_Keese_Electric_AllDay_505929623",
0x0e490d71,"MainField_Enemy_Keese_Electric_AllDay_606804264",
0x8e758051,"MainField_Enemy_Keese_Fire_1176263695",
0x448815f6,"MainField_Enemy_Keese_Fire_1286343175",
0xb64b69b0,"MainField_Enemy_Keese_Fire_1287285078",
0xb65fed81,"MainField_Enemy_Keese_Fire_1399276135",
0x1be5a3aa,"MainField_Enemy_Keese_Fire_1573377874",
0x1c9714a6,"MainField_Enemy_Keese_Fire_1734022564",
0xd6460d54,"MainField_Enemy_Keese_Fire_1911264226",
0xc9488116,"MainField_Enemy_Keese_Fire_2004731866",
0x1b3d8fb7,"MainField_Enemy_Keese_Fire_2094718154",
0x7a3f5da8,"MainField_Enemy_Keese_Fire_2170931494",
0x77839c40,"MainField_Enemy_Keese_Fire_2244505969",
0x80ae3d66,"MainField_Enemy_Keese_Fire_2256881913",
0x16e17345,"MainField_Enemy_Keese_Fire_2370383102",
0xe5344f3d,"MainField_Enemy_Keese_Fire_2469227234",
0x1fd846b8,"MainField_Enemy_Keese_Fire_2903909885",
0xd803f604,"MainField_Enemy_Keese_Fire_2997871486",
0xf83c59d2,"MainField_Enemy_Keese_Fire_3427969758",
0xca6be333,"MainField_Enemy_Keese_Fire_368192441",
0x8265243a,"MainField_Enemy_Keese_Fire_3743474201",
0xe3375521,"MainField_Enemy_Keese_Fire_3876762746",
0x489b8afe,"MainField_Enemy_Keese_Fire_4072094124",
0x3c01638b,"MainField_Enemy_Keese_Fire_4118134137",
0x5cf8a949,"MainField_Enemy_Keese_Fire_4217072000",
0xb9ba55ef,"MainField_Enemy_Keese_Fire_4234851141",
0xe18275a7,"MainField_Enemy_Keese_Fire_601164113",
0xad789c5b,"MainField_Enemy_Keese_Fire_AllDay_1051376314",
0x263084f9,"MainField_Enemy_Keese_Fire_AllDay_1095957187",
0x5b1e4423,"MainField_Enemy_Keese_Fire_AllDay_1157157745",
0x7657e928,"MainField_Enemy_Keese_Fire_AllDay_1237190956",
0x4cf15cbc,"MainField_Enemy_Keese_Fire_AllDay_1400345945",
0x6e32457a,"MainField_Enemy_Keese_Fire_AllDay_1457374293",
0xac16c2db,"MainField_Enemy_Keese_Fire_AllDay_1504925224",
0xc91c3ba1,"MainField_Enemy_Keese_Fire_AllDay_1699233519",
0x9d8cd6b2,"MainField_Enemy_Keese_Fire_AllDay_2717982106",
0xc5f07db1,"MainField_Enemy_Keese_Fire_AllDay_2807888259",
0x820fe70a,"MainField_Enemy_Keese_Fire_AllDay_2944643932",
0x4a3b9c27,"MainField_Enemy_Keese_Fire_AllDay_3147011390",
0xcb127db5,"MainField_Enemy_Keese_Fire_AllDay_3477252292",
0x3a3ef9ea,"MainField_Enemy_Keese_Fire_AllDay_3640111180",
0x562e3d43,"MainField_Enemy_Keese_Fire_AllDay_3674212029",
0xfb664def,"MainField_Enemy_Keese_Fire_AllDay_3704988843",
0x62747489,"MainField_Enemy_Keese_Fire_AllDay_3905379107",
0x9eb86b64,"MainField_Enemy_Keese_Fire_AllDay_4288655898",
0xecb6ac7e,"MainField_Enemy_Keese_Fire_AllDay_460884492",
0xec00d4e7,"MainField_Enemy_Keese_Fire_AllDay_461263841",
0x2aa223da,"MainField_Enemy_Keese_Fire_AllDay_496979747",
0x9c573cab,"MainField_Enemy_Keese_Fire_AllDay_558859502",
0x84bb4b14,"MainField_Enemy_Keese_Fire_AllDay_630723241",
0x3bbbf752,"MainField_Enemy_Keese_Fire_AllDay_894527336",
0xc111802c,"MainField_Enemy_Keese_Fire_AllDay_976200260",
0x7fb22a4c,"MainField_Enemy_Keese_Ice_1024464392",
0xe8726123,"MainField_Enemy_Keese_Ice_1055128006",
0xa9454a9a,"MainField_Enemy_Keese_Ice_1115872285",
0x6ca62b38,"MainField_Enemy_Keese_Ice_1154195335",
0xd69c9a3b,"MainField_Enemy_Keese_Ice_1392761261",
0x3cacd51e,"MainField_Enemy_Keese_Ice_1458275381",
0x06b96b1f,"MainField_Enemy_Keese_Ice_1832139775",
0xa9d0e186,"MainField_Enemy_Keese_Ice_191848556",
0xfbe53543,"MainField_Enemy_Keese_Ice_2010459207",
0x332a12fc,"MainField_Enemy_Keese_Ice_2011691989",
0x29fd37b6,"MainField_Enemy_Keese_Ice_211323763",
0x7e0129d8,"MainField_Enemy_Keese_Ice_2406259671",
0x3cebdd0d,"MainField_Enemy_Keese_Ice_2437477210",
0x53d279aa,"MainField_Enemy_Keese_Ice_2458715757",
0x177c7a58,"MainField_Enemy_Keese_Ice_2537408284",
0xb58d251c,"MainField_Enemy_Keese_Ice_2639843687",
0x21dd1b2e,"MainField_Enemy_Keese_Ice_2752619342",
0x9107e553,"MainField_Enemy_Keese_Ice_2963464826",
0xb0a015f8,"MainField_Enemy_Keese_Ice_2981110549",
0x8982d759,"MainField_Enemy_Keese_Ice_3506329393",
0xd6002fc4,"MainField_Enemy_Keese_Ice_3748582376",
0x5d91d907,"MainField_Enemy_Keese_Ice_3750575467",
0x0b938603,"MainField_Enemy_Keese_Ice_3768784364",
0xf8a86286,"MainField_Enemy_Keese_Ice_3998003908",
0x391a6a74,"MainField_Enemy_Keese_Ice_4080286428",
0x4120b701,"MainField_Enemy_Keese_Ice_4191682534",
0xbda29b5f,"MainField_Enemy_Keese_Ice_43583179",
0xbbdbb80c,"MainField_Enemy_Keese_Ice_703856495",
0xf93a3d53,"MainField_Enemy_Keese_Ice_861228407",
0xc6956724,"MainField_Enemy_Keese_Ice_AllDay_1135865826",
0x2335974d,"MainField_Enemy_Keese_Ice_AllDay_164189407",
0xe1c568fc,"MainField_Enemy_Keese_Ice_AllDay_1769420262",
0x3ae96080,"MainField_Enemy_Keese_Ice_AllDay_1924641971",
0xb4b29656,"MainField_Enemy_Keese_Ice_AllDay_2102508880",
0xeeb72f8b,"MainField_Enemy_Keese_Ice_AllDay_2119665199",
0x35d0f782,"MainField_Enemy_Keese_Ice_AllDay_2183425501",
0x239daeb8,"MainField_Enemy_Keese_Ice_AllDay_2253033478",
0xac979b18,"MainField_Enemy_Keese_Ice_AllDay_232359866",
0x728daa60,"MainField_Enemy_Keese_Ice_AllDay_2456557744",
0xbeaf3a62,"MainField_Enemy_Keese_Ice_AllDay_2601926201",
0x4104619b,"MainField_Enemy_Keese_Ice_AllDay_2951558979",
0x864c9c24,"MainField_Enemy_Keese_Ice_AllDay_3028504591",
0xe03d1171,"MainField_Enemy_Keese_Ice_AllDay_3451116351",
0xcc544dcd,"MainField_Enemy_Keese_Ice_AllDay_3620972374",
0xcc41e9de,"MainField_Enemy_Keese_Ice_AllDay_3761999810",
0x533c5718,"MainField_Enemy_Keese_Ice_AllDay_3850375434",
0xd48ca727,"MainField_Enemy_Keese_Ice_AllDay_3872427104",
0x8bf1173e,"MainField_Enemy_Keese_Ice_AllDay_3873805887",
0x1ef491fc,"MainField_Enemy_Keese_Ice_AllDay_3874812223",
0x697a89e0,"MainField_Enemy_Keese_Ice_AllDay_4073879945",
0x971e36a1,"MainField_Enemy_Keese_Ice_AllDay_4074285193",
0xc937ea8f,"MainField_Enemy_Keese_Ice_AllDay_502258793",
0x656cff0e,"MainField_Enemy_Keese_Ice_AllDay_518645526",
0xb9c0f2b3,"MainField_Enemy_Keese_Ice_AllDay_820633798",
0x29534553,"MainField_Enemy_Keese_Ice_AllDay_882719152",
0x8e01cf60,"MainField_Enemy_Keese_Swarm_1198221392",
0xbd096b1a,"MainField_Enemy_Keese_Swarm_2890904939",
0x291cad89,"MainField_Enemy_Keese_Swarm_AllDay_1963028406",
0x2b280722,"MainField_Enemy_Keese_Swarm_AllDay_2412278222",
0x44fd6f5e,"MainField_Enemy_Keese_Swarm_AllDay_2824010685",
0xc26ebcc6,"MainField_Enemy_Keese_Swarm_AllDay_3102428621",
0x40e08793,"MainField_Enemy_Lizalfos_Bone_Junior_1327904397",
0x804e68ff,"MainField_Enemy_Lizalfos_Bone_Junior_1363657073",
0x8a225c43,"MainField_Enemy_Lizalfos_Bone_Junior_1481502521",
0xc5f644f4,"MainField_Enemy_Lizalfos_Bone_Junior_1503606483",
0x22af0859,"MainField_Enemy_Lizalfos_Bone_Junior_1504712451",
0xfe2faf9a,"MainField_Enemy_Lizalfos_Bone_Junior_1667370476",
0x8eb3ea8c,"MainField_Enemy_Lizalfos_Bone_Junior_1730908603",
0xfca600f1,"MainField_Enemy_Lizalfos_Bone_Junior_175564507",
0xcfaca1f9,"MainField_Enemy_Lizalfos_Bone_Junior_1866749864",
0x10da06ab,"MainField_Enemy_Lizalfos_Bone_Junior_1907800642",
0xc2543de3,"MainField_Enemy_Lizalfos_Bone_Junior_2083797768",
0xe9ebb4f1,"MainField_Enemy_Lizalfos_Bone_Junior_235966606",
0x849b4dd9,"MainField_Enemy_Lizalfos_Bone_Junior_2885807471",
0x60e4e6e5,"MainField_Enemy_Lizalfos_Bone_Junior_3170101341",
0x767f9d96,"MainField_Enemy_Lizalfos_Bone_Junior_3311262831",
0xb278e135,"MainField_Enemy_Lizalfos_Bone_Junior_3571001843",
0x8b6825d2,"MainField_Enemy_Lizalfos_Bone_Junior_3959459280",
0x378b4afa,"MainField_Enemy_Lizalfos_Bone_Junior_4018050043",
0xad5468e5,"MainField_Enemy_Lizalfos_Bone_Junior_4048065864",
0x61056380,"MainField_Enemy_Lizalfos_Bone_Junior_409995480",
0x382456dc,"MainField_Enemy_Lizalfos_Bone_Junior_4106962537",
0x8f7153e6,"MainField_Enemy_Lizalfos_Bone_Junior_4107394772",
0x4bae2ada,"MainField_Enemy_Lizalfos_Bone_Junior_4131047753",
0xa3ea5a30,"MainField_Enemy_Lizalfos_Bone_Junior_734329321",
0x4a0f00d0,"MainField_Enemy_Lizalfos_Bone_Junior_939468739",
0xb5fd2d96,"MainField_Enemy_Lizalfos_Bone_Junior_955775802",
0x136d3ed0,"MainField_Enemy_Lizalfos_Electric_1043582976",
0xf2f79d18,"MainField_Enemy_Lizalfos_Electric_1171106319",
0xb7aaf5f6,"MainField_Enemy_Lizalfos_Electric_1235901510",
0x43576bc2,"MainField_Enemy_Lizalfos_Electric_1300451783",
0xf1acd4b8,"MainField_Enemy_Lizalfos_Electric_1597399660",
0x41b9ed2e,"MainField_Enemy_Lizalfos_Electric_192559341",
0xa9322e16,"MainField_Enemy_Lizalfos_Electric_1956979855",
0x0173c9af,"MainField_Enemy_Lizalfos_Electric_2087762100",
0x3dffa0ec,"MainField_Enemy_Lizalfos_Electric_2207458819",
0x9044b894,"MainField_Enemy_Lizalfos_Electric_2697466486",
0xd617b730,"MainField_Enemy_Lizalfos_Electric_2780613885",
0x069be914,"MainField_Enemy_Lizalfos_Electric_2837608684",
0xe6b8ef3f,"MainField_Enemy_Lizalfos_Electric_2850062014",
0x26c9caf9,"MainField_Enemy_Lizalfos_Electric_3419296513",
0xbc23b0e3,"MainField_Enemy_Lizalfos_Electric_3551478024",
0x9a5cade7,"MainField_Enemy_Lizalfos_Electric_3649395942",
0x4ecd5e95,"MainField_Enemy_Lizalfos_Electric_3943371449",
0x946bcee3,"MainField_Enemy_Lizalfos_Electric_420780146",
0xe19a7643,"MainField_Enemy_Lizalfos_Electric_698472245",
0x4c315327,"MainField_Enemy_Lizalfos_Electric_961430505",
0xfe9957ee,"MainField_Enemy_Lizalfos_Electric_ForestTower_3973216837",
0xd0cccef9,"MainField_Enemy_Lizalfos_Fire_1201480251",
0x179ba9d4,"MainField_Enemy_Lizalfos_Fire_1232648292",
0x4ed7e0a8,"MainField_Enemy_Lizalfos_Fire_1243926187",
0xe8e67173,"MainField_Enemy_Lizalfos_Fire_1263250574",
0x20e7a535,"MainField_Enemy_Lizalfos_Fire_129667315",
0xae7545b7,"MainField_Enemy_Lizalfos_Fire_1456128483",
0xa9533b00,"MainField_Enemy_Lizalfos_Fire_1466555059",
0x2cfd7c87,"MainField_Enemy_Lizalfos_Fire_1567743544",
0xd6f9333d,"MainField_Enemy_Lizalfos_Fire_1793300653",
0xb0268142,"MainField_Enemy_Lizalfos_Fire_1798092276",
0xdf0c24d0,"MainField_Enemy_Lizalfos_Fire_1857170048",
0xabea567c,"MainField_Enemy_Lizalfos_Fire_1860336463",
0xa75faf1b,"MainField_Enemy_Lizalfos_Fire_1909203692",
0x62654e63,"MainField_Enemy_Lizalfos_Fire_2000417074",
0xb0edba71,"MainField_Enemy_Lizalfos_Fire_2097580945",
0xde852446,"MainField_Enemy_Lizalfos_Fire_212274056",
0x405625ab,"MainField_Enemy_Lizalfos_Fire_2131165841",
0xa3132f47,"MainField_Enemy_Lizalfos_Fire_2202720944",
0xc4baf442,"MainField_Enemy_Lizalfos_Fire_221144016",
0xd1de784d,"MainField_Enemy_Lizalfos_Fire_225233466",
0x955aa2df,"MainField_Enemy_Lizalfos_Fire_2274298077",
0x7871cebe,"MainField_Enemy_Lizalfos_Fire_2302768252",
0x2bf8e26f,"MainField_Enemy_Lizalfos_Fire_2440227174",
0x373c5010,"MainField_Enemy_Lizalfos_Fire_2488340106",
0x365be988,"MainField_Enemy_Lizalfos_Fire_249470919",
0x67b463d8,"MainField_Enemy_Lizalfos_Fire_2547910419",
0xb0d52f14,"MainField_Enemy_Lizalfos_Fire_2622397745",
0xea1cd0c8,"MainField_Enemy_Lizalfos_Fire_2684670267",
0x4c4cdac0,"MainField_Enemy_Lizalfos_Fire_2732482086",
0xb67f14d1,"MainField_Enemy_Lizalfos_Fire_2793393605",
0x5e9752be,"MainField_Enemy_Lizalfos_Fire_2793433267",
0x0a86077e,"MainField_Enemy_Lizalfos_Fire_2844653022",
0xb818f8ca,"MainField_Enemy_Lizalfos_Fire_310454977",
0xc4130145,"MainField_Enemy_Lizalfos_Fire_315072520",
0xae711fc2,"MainField_Enemy_Lizalfos_Fire_3272619301",
0xc0346672,"MainField_Enemy_Lizalfos_Fire_3278579885",
0x4b43e601,"MainField_Enemy_Lizalfos_Fire_3282859513",
0x27ea874a,"MainField_Enemy_Lizalfos_Fire_3499468098",
0xc3ae3fb0,"MainField_Enemy_Lizalfos_Fire_3889065750",
0xa77889bd,"MainField_Enemy_Lizalfos_Fire_3960027373",
0x14f259f2,"MainField_Enemy_Lizalfos_Fire_4022246325",
0x9a232d69,"MainField_Enemy_Lizalfos_Fire_40529870",
0x3bd2a305,"MainField_Enemy_Lizalfos_Fire_4185395208",
0xb82eccb1,"MainField_Enemy_Lizalfos_Fire_4188296079",
0x9002d533,"MainField_Enemy_Lizalfos_Fire_4199558693",
0xfd2e8194,"MainField_Enemy_Lizalfos_Fire_4222903886",
0x958d2a47,"MainField_Enemy_Lizalfos_Fire_4232997408",
0x66dbb911,"MainField_Enemy_Lizalfos_Fire_481900238",
0x85e832ee,"MainField_Enemy_Lizalfos_Fire_528859891",
0x1f4ca15f,"MainField_Enemy_Lizalfos_Fire_536488003",
0xb00b5e21,"MainField_Enemy_Lizalfos_Fire_638321301",
0x4644293a,"MainField_Enemy_Lizalfos_Fire_761657246",
0x091566c4,"MainField_Enemy_Lizalfos_Fire_800790857",
0x719be878,"MainField_Enemy_Lizalfos_Fire_Guard_Ambush_2373123040",
0x0b248121,"MainField_Enemy_Lizalfos_Fire_Guard_Ambush_2877674565",
0xc3318cce,"MainField_Enemy_Lizalfos_Fire_Guard_Ambush_3471356955",
0x4d688bd7,"MainField_Enemy_Lizalfos_Fire_Guard_Ambush_3654331576",
0x135abcab,"MainField_Enemy_Lizalfos_Fire_Guard_Ambush_412761148",
0x5e95ff37,"MainField_Enemy_Lizalfos_Fire_Guard_Ambush_4283589620",
0xc721549b,"MainField_Enemy_Lizalfos_Fire_Guard_Ambush_847467802",
0x0a49ea1f,"MainField_Enemy_Lizalfos_Fire_Guard_Ambush_946245607",
0xf1c4fbdb,"MainField_Enemy_Lizalfos_Guard_Junior_1077238452",
0x27a3d5e2,"MainField_Enemy_Lizalfos_Guard_Junior_114483849",
0x3d594ac9,"MainField_Enemy_Lizalfos_Guard_Junior_1204321088",
0xb5d00918,"MainField_Enemy_Lizalfos_Guard_Junior_1222964551",
0xd50cedb4,"MainField_Enemy_Lizalfos_Guard_Junior_123321160",
0xbed32694,"MainField_Enemy_Lizalfos_Guard_Junior_1373908271",
0x384f887c,"MainField_Enemy_Lizalfos_Guard_Junior_1386088465",
0x88ba8eab,"MainField_Enemy_Lizalfos_Guard_Junior_1523403768",
0xaf2042fa,"MainField_Enemy_Lizalfos_Guard_Junior_161193723",
0xa81e6aea,"MainField_Enemy_Lizalfos_Guard_Junior_1622902906",
0x183b211b,"MainField_Enemy_Lizalfos_Guard_Junior_167828043",
0x0a6f8fb2,"MainField_Enemy_Lizalfos_Guard_Junior_1924033738",
0x18b4d37e,"MainField_Enemy_Lizalfos_Guard_Junior_2074604875",
0x2bd8c714,"MainField_Enemy_Lizalfos_Guard_Junior_2241591825",
0xd4b1b052,"MainField_Enemy_Lizalfos_Guard_Junior_2266274072",
0x80e2b677,"MainField_Enemy_Lizalfos_Guard_Junior_2358552805",
0x18ec50a7,"MainField_Enemy_Lizalfos_Guard_Junior_2462310465",
0x769d045b,"MainField_Enemy_Lizalfos_Guard_Junior_2791380456",
0x4d556e91,"MainField_Enemy_Lizalfos_Guard_Junior_2839887834",
0x14140d9d,"MainField_Enemy_Lizalfos_Guard_Junior_2865036194",
0x6524a093,"MainField_Enemy_Lizalfos_Guard_Junior_3027493004",
0x55657e94,"MainField_Enemy_Lizalfos_Guard_Junior_3101346922",
0xc8eb79fd,"MainField_Enemy_Lizalfos_Guard_Junior_3136716464",
0x66580f5f,"MainField_Enemy_Lizalfos_Guard_Junior_3187723205",
0x548a65e2,"MainField_Enemy_Lizalfos_Guard_Junior_3308363365",
0x3eff82c2,"MainField_Enemy_Lizalfos_Guard_Junior_3322871439",
0x6172da25,"MainField_Enemy_Lizalfos_Guard_Junior_333487850",
0xfe5d7878,"MainField_Enemy_Lizalfos_Guard_Junior_3435919254",
0x57606e7a,"MainField_Enemy_Lizalfos_Guard_Junior_3545253920",
0xd263f072,"MainField_Enemy_Lizalfos_Guard_Junior_3721461503",
0x88df856b,"MainField_Enemy_Lizalfos_Guard_Junior_3894925922",
0xf57e98aa,"MainField_Enemy_Lizalfos_Guard_Junior_3963151458",
0x701e00bf,"MainField_Enemy_Lizalfos_Guard_Junior_4245359492",
0x5be00c58,"MainField_Enemy_Lizalfos_Guard_Junior_440154027",
0x5832f5f8,"MainField_Enemy_Lizalfos_Guard_Junior_6420232",
0x109f90f0,"MainField_Enemy_Lizalfos_Guard_Junior_7007387",
0xd55b6f80,"MainField_Enemy_Lizalfos_Guard_Junior_832637722",
0xd47ad8b3,"MainField_Enemy_Lizalfos_Guard_Junior_853717408",
0xa86e6ef5,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1570675666",
0xafbfce15,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1648230344",
0xf03dfe2a,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1833120021",
0xb88b5197,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1852340499",
0x474b8958,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_2146942138",
0xce168d82,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_2787107504",
0xb1ec67bd,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_2935242096",
0xec6aa951,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3031133983",
0xa0d05167,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3139097096",
0x70c363a8,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3592700427",
0x0273b25c,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3652737448",
0x6d8a9aa1,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3666582301",
0x14ae8bbd,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3956753339",
0x5ce79149,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_751037428",
0x06033a8b,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_842997169",
0x11fae3f7,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_878673290",
0x068d8c93,"MainField_Enemy_Lizalfos_Guard_Middle_LongVisibility_1364851877",
0x3040ff37,"MainField_Enemy_Lizalfos_Guard_Middle_LongVisibility_2197224257",
0x9ae54d5a,"MainField_Enemy_Lizalfos_Guard_Middle_LongVisibility_2851155964",
0x580d0906,"MainField_Enemy_Lizalfos_Guard_Middle_LongVisibility_2890187471",
0x48177215,"MainField_Enemy_Lizalfos_Guard_Middle_LongVisibility_3182919079",
0x64786cb4,"MainField_Enemy_Lizalfos_Guard_Middle_LongVisibility_368255114",
0xaf319020,"MainField_Enemy_Lizalfos_Guard_Middle_LongVisibility_3982947025",
0xd4ee4618,"MainField_Enemy_Lizalfos_Ice_1015583519",
0xf67a3bd7,"MainField_Enemy_Lizalfos_Ice_1232243279",
0x0688fefe,"MainField_Enemy_Lizalfos_Ice_1268927125",
0x67c74dc7,"MainField_Enemy_Lizalfos_Ice_1336315344",
0x0c835e84,"MainField_Enemy_Lizalfos_Ice_1609857055",
0x3b67d7c9,"MainField_Enemy_Lizalfos_Ice_1688324317",
0x897b3413,"MainField_Enemy_Lizalfos_Ice_1729715850",
0x732e4983,"MainField_Enemy_Lizalfos_Ice_1826871487",
0x26ab5811,"MainField_Enemy_Lizalfos_Ice_1845649274",
0xb7f2fff4,"MainField_Enemy_Lizalfos_Ice_1934931253",
0x2cf6b5e3,"MainField_Enemy_Lizalfos_Ice_2155229726",
0xdd2b3bf9,"MainField_Enemy_Lizalfos_Ice_2182987881",
0x969c2c4c,"MainField_Enemy_Lizalfos_Ice_2192906181",
0xb6b8c018,"MainField_Enemy_Lizalfos_Ice_2332108134",
0xd7013dab,"MainField_Enemy_Lizalfos_Ice_2402333550",
0x89d1c08d,"MainField_Enemy_Lizalfos_Ice_2586600814",
0x62541b5b,"MainField_Enemy_Lizalfos_Ice_2689492977",
0x5e6e58bb,"MainField_Enemy_Lizalfos_Ice_2722771774",
0x0e7e06d5,"MainField_Enemy_Lizalfos_Ice_3019340409",
0x6a8c2c9d,"MainField_Enemy_Lizalfos_Ice_3184279717",
0xfdd9df83,"MainField_Enemy_Lizalfos_Ice_3362826554",
0x91ee94be,"MainField_Enemy_Lizalfos_Ice_3428426765",
0x0e41cfe2,"MainField_Enemy_Lizalfos_Ice_3489610376",
0xf5753ad3,"MainField_Enemy_Lizalfos_Ice_3518622941",
0x44d8cd93,"MainField_Enemy_Lizalfos_Ice_3522391437",
0x46f8e985,"MainField_Enemy_Lizalfos_Ice_3666830740",
0x22204c7a,"MainField_Enemy_Lizalfos_Ice_3667133752",
0x5720f77f,"MainField_Enemy_Lizalfos_Ice_3861158814",
0x9e9dfdc9,"MainField_Enemy_Lizalfos_Ice_389756872",
0x075278ee,"MainField_Enemy_Lizalfos_Ice_3990176033",
0x767d5f78,"MainField_Enemy_Lizalfos_Ice_4008555687",
0x2a0fd8ba,"MainField_Enemy_Lizalfos_Ice_4033456431",
0x67d18127,"MainField_Enemy_Lizalfos_Ice_4069859273",
0xe15bbde0,"MainField_Enemy_Lizalfos_Ice_4199159362",
0x714e79e3,"MainField_Enemy_Lizalfos_Ice_690896644",
0x46103ad0,"MainField_Enemy_Lizalfos_Ice_757544675",
0x6607c26a,"MainField_Enemy_Lizalfos_Ice_893201085",
0x5284d0be,"MainField_Enemy_Lizalfos_Junior_1006676483",
0x7ae34645,"MainField_Enemy_Lizalfos_Junior_1064468066",
0x5d2712e3,"MainField_Enemy_Lizalfos_Junior_1070764311",
0x0512e646,"MainField_Enemy_Lizalfos_Junior_1078964392",
0xdbd95913,"MainField_Enemy_Lizalfos_Junior_1084112513",
0x37f691a0,"MainField_Enemy_Lizalfos_Junior_1125247524",
0x995f15e6,"MainField_Enemy_Lizalfos_Junior_1154439433",
0x00080edc,"MainField_Enemy_Lizalfos_Junior_1163152111",
0x0b549756,"MainField_Enemy_Lizalfos_Junior_119465492",
0xffd669a9,"MainField_Enemy_Lizalfos_Junior_120445127",
0x8db22c01,"MainField_Enemy_Lizalfos_Junior_1207153239",
0x36a5fa86,"MainField_Enemy_Lizalfos_Junior_1246630497",
0x225089bd,"MainField_Enemy_Lizalfos_Junior_128866435",
0x91620d14,"MainField_Enemy_Lizalfos_Junior_1294194296",
0x0d25d68e,"MainField_Enemy_Lizalfos_Junior_1305063122",
0xfbbe39a9,"MainField_Enemy_Lizalfos_Junior_1322934198",
0x3a3f60c2,"MainField_Enemy_Lizalfos_Junior_1391151182",
0xe5b3da85,"MainField_Enemy_Lizalfos_Junior_1394329252",
0x58c84252,"MainField_Enemy_Lizalfos_Junior_1401054036",
0xe1e4c19f,"MainField_Enemy_Lizalfos_Junior_1488735641",
0xd6b35f77,"MainField_Enemy_Lizalfos_Junior_1494145493",
0x60e3052b,"MainField_Enemy_Lizalfos_Junior_1497852493",
0x32e2f52f,"MainField_Enemy_Lizalfos_Junior_1503282085",
0xe600a524,"MainField_Enemy_Lizalfos_Junior_155063",
0x5d1ac21c,"MainField_Enemy_Lizalfos_Junior_1597086243",
0x5b998356,"MainField_Enemy_Lizalfos_Junior_1603298937",
0x50151e6f,"MainField_Enemy_Lizalfos_Junior_1621163733",
0x2fda101f,"MainField_Enemy_Lizalfos_Junior_1680616240",
0x158c60ba,"MainField_Enemy_Lizalfos_Junior_1704283284",
0x84e8c884,"MainField_Enemy_Lizalfos_Junior_1708573951",
0x8bed45ed,"MainField_Enemy_Lizalfos_Junior_1714941956",
0x190ab2cd,"MainField_Enemy_Lizalfos_Junior_174628736",
0x4f6fe755,"MainField_Enemy_Lizalfos_Junior_1766637772",
0x1ea8fb8a,"MainField_Enemy_Lizalfos_Junior_1769183353",
0xd4f9b031,"MainField_Enemy_Lizalfos_Junior_1815483415",
0xa1fee9c6,"MainField_Enemy_Lizalfos_Junior_1828415426",
0xa4f56888,"MainField_Enemy_Lizalfos_Junior_1847483488",
0xa11b39b3,"MainField_Enemy_Lizalfos_Junior_1960568416",
0x9d8e8fac,"MainField_Enemy_Lizalfos_Junior_1982843887",
0x57ae8e9c,"MainField_Enemy_Lizalfos_Junior_1983007691",
0x3432cde9,"MainField_Enemy_Lizalfos_Junior_1994007080",
0x276531c6,"MainField_Enemy_Lizalfos_Junior_1996844321",
0xcebc9205,"MainField_Enemy_Lizalfos_Junior_2067739586",
0xaadfbcaa,"MainField_Enemy_Lizalfos_Junior_2108018285",
0x339aad0a,"MainField_Enemy_Lizalfos_Junior_213147214",
0xaa45b9d7,"MainField_Enemy_Lizalfos_Junior_2148547916",
0xaf026e73,"MainField_Enemy_Lizalfos_Junior_2207405265",
0xc16b1545,"MainField_Enemy_Lizalfos_Junior_2241228943",
0x62d33f26,"MainField_Enemy_Lizalfos_Junior_2273179989",
0x9a0f6ae4,"MainField_Enemy_Lizalfos_Junior_2285138637",
0xdd55b327,"MainField_Enemy_Lizalfos_Junior_231871219",
0x4a328f61,"MainField_Enemy_Lizalfos_Junior_2331614943",
0x3eb2e239,"MainField_Enemy_Lizalfos_Junior_2416641607",
0x25b76fab,"MainField_Enemy_Lizalfos_Junior_2423914139",
0x14c60c92,"MainField_Enemy_Lizalfos_Junior_2466735693",
0xae94f50d,"MainField_Enemy_Lizalfos_Junior_251481765",
0x568303c0,"MainField_Enemy_Lizalfos_Junior_2515364563",
0x63a75a40,"MainField_Enemy_Lizalfos_Junior_2548336985",
0x0df29116,"MainField_Enemy_Lizalfos_Junior_2569750155",
0x83cf1644,"MainField_Enemy_Lizalfos_Junior_2576475601",
0xfbc3d696,"MainField_Enemy_Lizalfos_Junior_2598032933",
0xea0b25a4,"MainField_Enemy_Lizalfos_Junior_2703888940",
0xc0199c20,"MainField_Enemy_Lizalfos_Junior_2761426384",
0x61028001,"MainField_Enemy_Lizalfos_Junior_2781029781",
0xf0cd1131,"MainField_Enemy_Lizalfos_Junior_2851504962",
0xb22da647,"MainField_Enemy_Lizalfos_Junior_2875989112",
0x6703b845,"MainField_Enemy_Lizalfos_Junior_2947386777",
0x92e7ca33,"MainField_Enemy_Lizalfos_Junior_2951352812",
0xd587b5d5,"MainField_Enemy_Lizalfos_Junior_2962544063",
0xd4b6f4cd,"MainField_Enemy_Lizalfos_Junior_2976268698",
0xde3b2668,"MainField_Enemy_Lizalfos_Junior_299681209",
0xec72368c,"MainField_Enemy_Lizalfos_Junior_3006446906",
0x4b741fe1,"MainField_Enemy_Lizalfos_Junior_3060085627",
0xb657c0d4,"MainField_Enemy_Lizalfos_Junior_3063852857",
0xa4b961ac,"MainField_Enemy_Lizalfos_Junior_3069779400",
0x2b56d13b,"MainField_Enemy_Lizalfos_Junior_3124135445",
0x08c991f3,"MainField_Enemy_Lizalfos_Junior_3128398704",
0x0930b400,"MainField_Enemy_Lizalfos_Junior_3132222466",
0x71bbd805,"MainField_Enemy_Lizalfos_Junior_317134149",
0xf098b710,"MainField_Enemy_Lizalfos_Junior_3204025425",
0xf67ef8a8,"MainField_Enemy_Lizalfos_Junior_323278622",
0xaa67ae0f,"MainField_Enemy_Lizalfos_Junior_3304266295",
0xb1f512da,"MainField_Enemy_Lizalfos_Junior_334942394",
0x02bcfa45,"MainField_Enemy_Lizalfos_Junior_33672954",
0x9ea45f98,"MainField_Enemy_Lizalfos_Junior_3386845549",
0x4ea8d28f,"MainField_Enemy_Lizalfos_Junior_3407230243",
0x912a0b01,"MainField_Enemy_Lizalfos_Junior_3411566654",
0x713b61f4,"MainField_Enemy_Lizalfos_Junior_3420864576",
0xa6ef8b4c,"MainField_Enemy_Lizalfos_Junior_3422416346",
0x9dceb7d5,"MainField_Enemy_Lizalfos_Junior_3438608270",
0x5a836c59,"MainField_Enemy_Lizalfos_Junior_3442587267",
0x5431a6d5,"MainField_Enemy_Lizalfos_Junior_3464001476",
0xd705c224,"MainField_Enemy_Lizalfos_Junior_3521477102",
0xdd93b913,"MainField_Enemy_Lizalfos_Junior_3533861494",
0x675781de,"MainField_Enemy_Lizalfos_Junior_3593116712",
0x2ed97300,"MainField_Enemy_Lizalfos_Junior_3620236718",
0x7d4e2bb5,"MainField_Enemy_Lizalfos_Junior_3632614239",
0x0ff74bde,"MainField_Enemy_Lizalfos_Junior_3765836139",
0x1511685a,"MainField_Enemy_Lizalfos_Junior_377547942",
0xd8d7802f,"MainField_Enemy_Lizalfos_Junior_3779129363",
0x73639162,"MainField_Enemy_Lizalfos_Junior_3806156227",
0x8bab6c59,"MainField_Enemy_Lizalfos_Junior_3809811002",
0xf9b93a76,"MainField_Enemy_Lizalfos_Junior_3839327210",
0x5852e373,"MainField_Enemy_Lizalfos_Junior_3844280836",
0x61c5b1b2,"MainField_Enemy_Lizalfos_Junior_3853620530",
0xe6e90d04,"MainField_Enemy_Lizalfos_Junior_3857227235",
0xef6d807c,"MainField_Enemy_Lizalfos_Junior_3915511938",
0xfb4622bb,"MainField_Enemy_Lizalfos_Junior_398728545",
0xd845bba2,"MainField_Enemy_Lizalfos_Junior_399543717",
0xdcf9456e,"MainField_Enemy_Lizalfos_Junior_4008315791",
0xc2ecf1c9,"MainField_Enemy_Lizalfos_Junior_4034578883",
0x096ade43,"MainField_Enemy_Lizalfos_Junior_4047901245",
0x0ab737ea,"MainField_Enemy_Lizalfos_Junior_4056566945",
0xbd84b6d7,"MainField_Enemy_Lizalfos_Junior_4065855047",
0x9b206bd0,"MainField_Enemy_Lizalfos_Junior_4108664323",
0x8777bb8b,"MainField_Enemy_Lizalfos_Junior_4119991707",
0x6f7e67d4,"MainField_Enemy_Lizalfos_Junior_4128719056",
0x603abd81,"MainField_Enemy_Lizalfos_Junior_4145483226",
0xec16e663,"MainField_Enemy_Lizalfos_Junior_4193943826",
0x8142b844,"MainField_Enemy_Lizalfos_Junior_4206227899",
0x196d045f,"MainField_Enemy_Lizalfos_Junior_4208032330",
0x1d3090ef,"MainField_Enemy_Lizalfos_Junior_421096694",
0x9f5cc073,"MainField_Enemy_Lizalfos_Junior_4265953918",
0xe887b6cb,"MainField_Enemy_Lizalfos_Junior_439849452",
0xc0ffcf1e,"MainField_Enemy_Lizalfos_Junior_459435312",
0x1c9f7c72,"MainField_Enemy_Lizalfos_Junior_464048049",
0x3e5784a4,"MainField_Enemy_Lizalfos_Junior_491027219",
0x5ec11f8c,"MainField_Enemy_Lizalfos_Junior_510961903",
0x817fa1ee,"MainField_Enemy_Lizalfos_Junior_523907031",
0x790981b3,"MainField_Enemy_Lizalfos_Junior_567757486",
0xaf2a672d,"MainField_Enemy_Lizalfos_Junior_616400489",
0x317d36b9,"MainField_Enemy_Lizalfos_Junior_64172073",
0x65f7e2ed,"MainField_Enemy_Lizalfos_Junior_656370739",
0x7d8b1d04,"MainField_Enemy_Lizalfos_Junior_675322964",
0x2a0e3c16,"MainField_Enemy_Lizalfos_Junior_696249181",
0x1ab11073,"MainField_Enemy_Lizalfos_Junior_716341778",
0x6ce6cedc,"MainField_Enemy_Lizalfos_Junior_783514868",
0xbbdcba35,"MainField_Enemy_Lizalfos_Junior_798905762",
0xb1a8e013,"MainField_Enemy_Lizalfos_Junior_836634595",
0xbf3b0aba,"MainField_Enemy_Lizalfos_Junior_837939551",
0xe886c8e3,"MainField_Enemy_Lizalfos_Junior_936923481",
0x8cb44d2c,"MainField_Enemy_Lizalfos_Junior_959282854",
0x04f2461e,"MainField_Enemy_Lizalfos_Junior_Guard_Ambush_1305543746",
0xa1c42c02,"MainField_Enemy_Lizalfos_Junior_Guard_Ambush_1337787928",
0x4215dfb0,"MainField_Enemy_Lizalfos_Junior_Guard_Ambush_1880999369",
0x30a9f96b,"MainField_Enemy_Lizalfos_Junior_Guard_Ambush_2409215851",
0xa7fe0be0,"MainField_Enemy_Lizalfos_Junior_Guard_Ambush_4079018505",
0x38167ec1,"MainField_Enemy_Lizalfos_Junior_Guard_Ambush_4164707434",
0x4ae8af80,"MainField_Enemy_Lizalfos_Junior_Guard_Ambush_627769133",
0x6f968cde,"MainField_Enemy_Lizalfos_Middle_1002767823",
0x271962c0,"MainField_Enemy_Lizalfos_Middle_101922076",
0x51c8c8f9,"MainField_Enemy_Lizalfos_Middle_1052826748",
0x2559efcc,"MainField_Enemy_Lizalfos_Middle_1068511187",
0xb5c4258e,"MainField_Enemy_Lizalfos_Middle_1114647758",
0x923e2d99,"MainField_Enemy_Lizalfos_Middle_1157166271",
0x59f909f7,"MainField_Enemy_Lizalfos_Middle_123317298",
0x55a15d74,"MainField_Enemy_Lizalfos_Middle_1246540315",
0x35b2499a,"MainField_Enemy_Lizalfos_Middle_1264339967",
0x60e2fe68,"MainField_Enemy_Lizalfos_Middle_126518171",
0x3826bd3c,"MainField_Enemy_Lizalfos_Middle_1268635018",
0x41947423,"MainField_Enemy_Lizalfos_Middle_1301759138",
0xb90fe209,"MainField_Enemy_Lizalfos_Middle_1334139335",
0x407919a7,"MainField_Enemy_Lizalfos_Middle_1340964342",
0x79d29992,"MainField_Enemy_Lizalfos_Middle_1352228075",
0x09a7accc,"MainField_Enemy_Lizalfos_Middle_1376003649",
0xb46a573c,"MainField_Enemy_Lizalfos_Middle_1405792760",
0x1d945ed8,"MainField_Enemy_Lizalfos_Middle_1406526805",
0x4bfbfea2,"MainField_Enemy_Lizalfos_Middle_1428723325",
0x459fa55e,"MainField_Enemy_Lizalfos_Middle_1446498816",
0xfff36638,"MainField_Enemy_Lizalfos_Middle_1451907795",
0xe763fc99,"MainField_Enemy_Lizalfos_Middle_1489268323",
0x5eb6a7cf,"MainField_Enemy_Lizalfos_Middle_1493474973",
0x0b56a3bc,"MainField_Enemy_Lizalfos_Middle_1571914769",
0x4b557367,"MainField_Enemy_Lizalfos_Middle_1609920676",
0x903c31f1,"MainField_Enemy_Lizalfos_Middle_1616574850",
0x1c1cbc45,"MainField_Enemy_Lizalfos_Middle_1637904977",
0xacb8c943,"MainField_Enemy_Lizalfos_Middle_1650231658",
0x84bc43b5,"MainField_Enemy_Lizalfos_Middle_1707676224",
0xa2e168f3,"MainField_Enemy_Lizalfos_Middle_1722574269",
0x50ca0046,"MainField_Enemy_Lizalfos_Middle_1742091984",
0xbb70fc1e,"MainField_Enemy_Lizalfos_Middle_1803827115",
0x39770fa0,"MainField_Enemy_Lizalfos_Middle_181612398",
0x0cc501fa,"MainField_Enemy_Lizalfos_Middle_1817816749",
0xf8d404f4,"MainField_Enemy_Lizalfos_Middle_1825809252",
0x4d25d0d0,"MainField_Enemy_Lizalfos_Middle_1847162151",
0xd13b2394,"MainField_Enemy_Lizalfos_Middle_1904605396",
0xeb6544f6,"MainField_Enemy_Lizalfos_Middle_1929989223",
0x6daef913,"MainField_Enemy_Lizalfos_Middle_1967190464",
0xc46f723a,"MainField_Enemy_Lizalfos_Middle_2000020259",
0xc9e3e3a8,"MainField_Enemy_Lizalfos_Middle_2017199432",
0x2777129d,"MainField_Enemy_Lizalfos_Middle_2034851750",
0x316f9cd7,"MainField_Enemy_Lizalfos_Middle_2036150519",
0xfcb96409,"MainField_Enemy_Lizalfos_Middle_2051771833",
0x2e43edd7,"MainField_Enemy_Lizalfos_Middle_2054541079",
0x5bfc28d6,"MainField_Enemy_Lizalfos_Middle_2062214693",
0xa79fa29b,"MainField_Enemy_Lizalfos_Middle_2066088197",
0x4de738f9,"MainField_Enemy_Lizalfos_Middle_2104838801",
0xc313d437,"MainField_Enemy_Lizalfos_Middle_2114283371",
0xb00ac355,"MainField_Enemy_Lizalfos_Middle_2134643683",
0x8158f60f,"MainField_Enemy_Lizalfos_Middle_2136066511",
0x86d88813,"MainField_Enemy_Lizalfos_Middle_2162445582",
0x53600e72,"MainField_Enemy_Lizalfos_Middle_2227521077",
0x8818e957,"MainField_Enemy_Lizalfos_Middle_2249931094",
0xb2af7dcd,"MainField_Enemy_Lizalfos_Middle_228712591",
0xd07f597b,"MainField_Enemy_Lizalfos_Middle_2317363278",
0x2eaa801f,"MainField_Enemy_Lizalfos_Middle_2347608143",
0x812e09cc,"MainField_Enemy_Lizalfos_Middle_2372794855",
0x4d6ab96b,"MainField_Enemy_Lizalfos_Middle_2377042535",
0x0a6434d2,"MainField_Enemy_Lizalfos_Middle_2386225924",
0x66dfbbb5,"MainField_Enemy_Lizalfos_Middle_2394988098",
0x302cab5a,"MainField_Enemy_Lizalfos_Middle_2423834846",
0x03574c4e,"MainField_Enemy_Lizalfos_Middle_2424153178",
0x8daa4f6b,"MainField_Enemy_Lizalfos_Middle_2452286406",
0x926c02d8,"MainField_Enemy_Lizalfos_Middle_2466875362",
0x431ca9bc,"MainField_Enemy_Lizalfos_Middle_247283217",
0x64fbcd82,"MainField_Enemy_Lizalfos_Middle_2483526615",
0x35168bc2,"MainField_Enemy_Lizalfos_Middle_2504604148",
0xb603db5a,"MainField_Enemy_Lizalfos_Middle_2515490370",
0x1abe1a7d,"MainField_Enemy_Lizalfos_Middle_2528741463",
0x65f9fabc,"MainField_Enemy_Lizalfos_Middle_2537120311",
0x0a700a69,"MainField_Enemy_Lizalfos_Middle_2551172536",
0xe3e2fa3c,"MainField_Enemy_Lizalfos_Middle_2556908994",
0x2115ddd3,"MainField_Enemy_Lizalfos_Middle_2582104290",
0xc171d3f9,"MainField_Enemy_Lizalfos_Middle_2583808247",
0xb333ada1,"MainField_Enemy_Lizalfos_Middle_2620139367",
0x42e676a2,"MainField_Enemy_Lizalfos_Middle_262330296",
0xdb66c7df,"MainField_Enemy_Lizalfos_Middle_265035073",
0xc1373778,"MainField_Enemy_Lizalfos_Middle_2662885233",
0x86d6fd4a,"MainField_Enemy_Lizalfos_Middle_2699268884",
0x3659eda2,"MainField_Enemy_Lizalfos_Middle_2701718099",
0x65351a3e,"MainField_Enemy_Lizalfos_Middle_273212448",
0x50df4cef,"MainField_Enemy_Lizalfos_Middle_273617847",
0x5998583e,"MainField_Enemy_Lizalfos_Middle_2762023785",
0x496c6517,"MainField_Enemy_Lizalfos_Middle_278887622",
0x49c242cb,"MainField_Enemy_Lizalfos_Middle_2804071037",
0x51b26e12,"MainField_Enemy_Lizalfos_Middle_285993100",
0x21cce30a,"MainField_Enemy_Lizalfos_Middle_2867408805",
0x00b3020e,"MainField_Enemy_Lizalfos_Middle_2872737235",
0x9e513eee,"MainField_Enemy_Lizalfos_Middle_2882988859",
0x261d99b8,"MainField_Enemy_Lizalfos_Middle_2909342905",
0xc97a3ffb,"MainField_Enemy_Lizalfos_Middle_2909479841",
0x8da74c11,"MainField_Enemy_Lizalfos_Middle_3023415592",
0xa9dfc3b0,"MainField_Enemy_Lizalfos_Middle_3078415160",
0xfdec8b86,"MainField_Enemy_Lizalfos_Middle_3104076890",
0x4f7b548a,"MainField_Enemy_Lizalfos_Middle_3165552876",
0xc2aa7a0d,"MainField_Enemy_Lizalfos_Middle_3166591788",
0xebbf6f0c,"MainField_Enemy_Lizalfos_Middle_3184271367",
0xf0ad2ebf,"MainField_Enemy_Lizalfos_Middle_3218272665",
0x556a4250,"MainField_Enemy_Lizalfos_Middle_3223711880",
0x84f44ab3,"MainField_Enemy_Lizalfos_Middle_3233752671",
0xf7e11f00,"MainField_Enemy_Lizalfos_Middle_3252648862",
0xd1415a6f,"MainField_Enemy_Lizalfos_Middle_3277457876",
0x8bd1300d,"MainField_Enemy_Lizalfos_Middle_3308112546",
0x7372db2e,"MainField_Enemy_Lizalfos_Middle_3317401079",
0x1aa06f54,"MainField_Enemy_Lizalfos_Middle_3328551138",
0xc71e2c81,"MainField_Enemy_Lizalfos_Middle_3334995480",
0x5351f086,"MainField_Enemy_Lizalfos_Middle_3339079106",
0x68be8855,"MainField_Enemy_Lizalfos_Middle_3358111828",
0x23be86b5,"MainField_Enemy_Lizalfos_Middle_3424594270",
0x7573a873,"MainField_Enemy_Lizalfos_Middle_3432346220",
0x3d651945,"MainField_Enemy_Lizalfos_Middle_3441661312",
0x7c7fadb2,"MainField_Enemy_Lizalfos_Middle_3482369321",
0xf13c38af,"MainField_Enemy_Lizalfos_Middle_353294566",
0xdb4fe0d0,"MainField_Enemy_Lizalfos_Middle_356173544",
0xf2f10659,"MainField_Enemy_Lizalfos_Middle_3595905537",
0x2952a6e6,"MainField_Enemy_Lizalfos_Middle_3597997631",
0x32e60aac,"MainField_Enemy_Lizalfos_Middle_3622599157",
0x624cbbe2,"MainField_Enemy_Lizalfos_Middle_3625653268",
0xb3a0214e,"MainField_Enemy_Lizalfos_Middle_3642276231",
0x3a1e61af,"MainField_Enemy_Lizalfos_Middle_3674929488",
0xe7a446c4,"MainField_Enemy_Lizalfos_Middle_3691083335",
0xe9ab031a,"MainField_Enemy_Lizalfos_Middle_369490277",
0x133a6783,"MainField_Enemy_Lizalfos_Middle_3704018134",
0x535e14bd,"MainField_Enemy_Lizalfos_Middle_3744086851",
0x360670e9,"MainField_Enemy_Lizalfos_Middle_3782413847",
0x7d800327,"MainField_Enemy_Lizalfos_Middle_3791877420",
0xf1147fca,"MainField_Enemy_Lizalfos_Middle_3808424870",
0xe4aa2b42,"MainField_Enemy_Lizalfos_Middle_3818473767",
0xf1439506,"MainField_Enemy_Lizalfos_Middle_3900774838",
0xb7ee0937,"MainField_Enemy_Lizalfos_Middle_3905745732",
0xb28b5dec,"MainField_Enemy_Lizalfos_Middle_3907284509",
0xd0d43417,"MainField_Enemy_Lizalfos_Middle_3919214146",
0xedbb9517,"MainField_Enemy_Lizalfos_Middle_3933164219",
0xf9ae0dfe,"MainField_Enemy_Lizalfos_Middle_3940737997",
0x4ecc780b,"MainField_Enemy_Lizalfos_Middle_3958659699",
0xf88887b4,"MainField_Enemy_Lizalfos_Middle_3961580955",
0xabddbccc,"MainField_Enemy_Lizalfos_Middle_3991592510",
0x43f71027,"MainField_Enemy_Lizalfos_Middle_4005700062",
0xce144364,"MainField_Enemy_Lizalfos_Middle_402377215",
0xfdac3898,"MainField_Enemy_Lizalfos_Middle_4031280149",
0x7715c396,"MainField_Enemy_Lizalfos_Middle_405430044",
0x75b9b8b0,"MainField_Enemy_Lizalfos_Middle_4074608034",
0x6f16b151,"MainField_Enemy_Lizalfos_Middle_408989699",
0xe22dda46,"MainField_Enemy_Lizalfos_Middle_40912747",
0x39cc895f,"MainField_Enemy_Lizalfos_Middle_4110372067",
0x8e1f78cd,"MainField_Enemy_Lizalfos_Middle_41429649",
0xc75ba7af,"MainField_Enemy_Lizalfos_Middle_4157018341",
0xb5107026,"MainField_Enemy_Lizalfos_Middle_4173230338",
0x51e4ddc8,"MainField_Enemy_Lizalfos_Middle_4209505592",
0x244460e0,"MainField_Enemy_Lizalfos_Middle_4235426495",
0xb97bc3fe,"MainField_Enemy_Lizalfos_Middle_4250164172",
0xdb65fa54,"MainField_Enemy_Lizalfos_Middle_4255007088",
0xac4382ae,"MainField_Enemy_Lizalfos_Middle_4273653908",
0x00e1abf3,"MainField_Enemy_Lizalfos_Middle_4274734637",
0xf1aa92f2,"MainField_Enemy_Lizalfos_Middle_440723203",
0x210b4004,"MainField_Enemy_Lizalfos_Middle_458178554",
0x16688808,"MainField_Enemy_Lizalfos_Middle_462089252",
0x42dc776d,"MainField_Enemy_Lizalfos_Middle_474701617",
0x67245ee0,"MainField_Enemy_Lizalfos_Middle_498399182",
0x33a38d49,"MainField_Enemy_Lizalfos_Middle_505543268",
0xe26aedd6,"MainField_Enemy_Lizalfos_Middle_518392637",
0x235c5396,"MainField_Enemy_Lizalfos_Middle_519937051",
0x2a957349,"MainField_Enemy_Lizalfos_Middle_520467944",
0xfd7180cc,"MainField_Enemy_Lizalfos_Middle_543961826",
0xac0dc10c,"MainField_Enemy_Lizalfos_Middle_549618249",
0xa9aabfef,"MainField_Enemy_Lizalfos_Middle_628401097",
0x2fcd7fc1,"MainField_Enemy_Lizalfos_Middle_63255521",
0x73912223,"MainField_Enemy_Lizalfos_Middle_676192432",
0xcd02c747,"MainField_Enemy_Lizalfos_Middle_694858205",
0xe928c850,"MainField_Enemy_Lizalfos_Middle_744366118",
0x667cdee6,"MainField_Enemy_Lizalfos_Middle_818840547",
0xf604f2f6,"MainField_Enemy_Lizalfos_Middle_835876134",
0xf881d94c,"MainField_Enemy_Lizalfos_Middle_859714112",
0x933fa923,"MainField_Enemy_Lizalfos_Middle_869313541",
0x509a424c,"MainField_Enemy_Lizalfos_Middle_884650541",
0xcaf3b775,"MainField_Enemy_Lizalfos_Middle_90345466",
0x280fcd93,"MainField_Enemy_Lizalfos_Middle_913689830",
0x29a0dff6,"MainField_Enemy_Lizalfos_Middle_987142984",
0x80b71a62,"MainField_Enemy_Lizalfos_Senior_100612012",
0xe344f069,"MainField_Enemy_Lizalfos_Senior_1021249854",
0x44438e24,"MainField_Enemy_Lizalfos_Senior_1137173288",
0x94b4154e,"MainField_Enemy_Lizalfos_Senior_1151292624",
0x9988935b,"MainField_Enemy_Lizalfos_Senior_1176469628",
0x31835bd3,"MainField_Enemy_Lizalfos_Senior_1209055136",
0xb9cd16c4,"MainField_Enemy_Lizalfos_Senior_1284892793",
0x10a2ca4e,"MainField_Enemy_Lizalfos_Senior_1316072865",
0x779d98dd,"MainField_Enemy_Lizalfos_Senior_1317079596",
0xcae27b0d,"MainField_Enemy_Lizalfos_Senior_1318239363",
0x1fb0d489,"MainField_Enemy_Lizalfos_Senior_1338138619",
0x14c33b84,"MainField_Enemy_Lizalfos_Senior_1347835625",
0xa6cf608b,"MainField_Enemy_Lizalfos_Senior_1458375279",
0x1f0a7149,"MainField_Enemy_Lizalfos_Senior_1503816238",
0xffdda4ec,"MainField_Enemy_Lizalfos_Senior_1533493071",
0x512ecf46,"MainField_Enemy_Lizalfos_Senior_1537013180",
0xc031e5e7,"MainField_Enemy_Lizalfos_Senior_1564038146",
0xfb8c4f35,"MainField_Enemy_Lizalfos_Senior_1567215521",
0xe293376f,"MainField_Enemy_Lizalfos_Senior_1591834617",
0x082d0d14,"MainField_Enemy_Lizalfos_Senior_1628494291",
0x978664a9,"MainField_Enemy_Lizalfos_Senior_1658616657",
0x428d30fa,"MainField_Enemy_Lizalfos_Senior_1673255800",
0x9139605b,"MainField_Enemy_Lizalfos_Senior_1683018729",
0x5fa3e193,"MainField_Enemy_Lizalfos_Senior_1685682583",
0xa0b6341a,"MainField_Enemy_Lizalfos_Senior_1750285401",
0x62e0c856,"MainField_Enemy_Lizalfos_Senior_1753402347",
0x729bbada,"MainField_Enemy_Lizalfos_Senior_1815268906",
0xb6cee7df,"MainField_Enemy_Lizalfos_Senior_1828639238",
0xe07b89d7,"MainField_Enemy_Lizalfos_Senior_1830839689",
0xe2e8afbd,"MainField_Enemy_Lizalfos_Senior_1840647294",
0xabd3179f,"MainField_Enemy_Lizalfos_Senior_1845356714",
0xfc806bba,"MainField_Enemy_Lizalfos_Senior_186404819",
0xc8b0ad81,"MainField_Enemy_Lizalfos_Senior_191916109",
0x5a08633d,"MainField_Enemy_Lizalfos_Senior_1951747422",
0x0ce1c776,"MainField_Enemy_Lizalfos_Senior_1953264438",
0x4bbc4e48,"MainField_Enemy_Lizalfos_Senior_1992809093",
0x247c7e57,"MainField_Enemy_Lizalfos_Senior_1996412574",
0x8dd33d45,"MainField_Enemy_Lizalfos_Senior_1998405018",
0x2cdd7f73,"MainField_Enemy_Lizalfos_Senior_2057560765",
0x00bed870,"MainField_Enemy_Lizalfos_Senior_2090677285",
0xfdf0c4f8,"MainField_Enemy_Lizalfos_Senior_2100709577",
0x6ee4d7d9,"MainField_Enemy_Lizalfos_Senior_2128233397",
0x6d36b0e4,"MainField_Enemy_Lizalfos_Senior_2166779339",
0x8f345683,"MainField_Enemy_Lizalfos_Senior_2180713761",
0xf51506c7,"MainField_Enemy_Lizalfos_Senior_2203036677",
0x76eeb2a0,"MainField_Enemy_Lizalfos_Senior_2220233468",
0x85230654,"MainField_Enemy_Lizalfos_Senior_2330103885",
0xa266bf69,"MainField_Enemy_Lizalfos_Senior_2380115926",
0xa9ec4eca,"MainField_Enemy_Lizalfos_Senior_2384048126",
0xe7c53a7a,"MainField_Enemy_Lizalfos_Senior_2416373648",
0x45efdced,"MainField_Enemy_Lizalfos_Senior_2483159598",
0x9232b2d8,"MainField_Enemy_Lizalfos_Senior_2493361451",
0xc8fd2389,"MainField_Enemy_Lizalfos_Senior_2583242861",
0xaddb1ee8,"MainField_Enemy_Lizalfos_Senior_2584776018",
0x5ac94035,"MainField_Enemy_Lizalfos_Senior_2645769793",
0xe43c3f75,"MainField_Enemy_Lizalfos_Senior_2649969008",
0x8a9ce1df,"MainField_Enemy_Lizalfos_Senior_2671792437",
0xaeebff2a,"MainField_Enemy_Lizalfos_Senior_2773442511",
0x8fa6546e,"MainField_Enemy_Lizalfos_Senior_2793754688",
0xa49cd1b8,"MainField_Enemy_Lizalfos_Senior_289523826",
0x74720ae6,"MainField_Enemy_Lizalfos_Senior_2895512918",
0xb8abec0d,"MainField_Enemy_Lizalfos_Senior_2919299134",
0xbcb3ab15,"MainField_Enemy_Lizalfos_Senior_2931129991",
0x15a7bc83,"MainField_Enemy_Lizalfos_Senior_2969757916",
0xcf1aec99,"MainField_Enemy_Lizalfos_Senior_2980321199",
0x417390cc,"MainField_Enemy_Lizalfos_Senior_2993701675",
0x54fc2556,"MainField_Enemy_Lizalfos_Senior_3004121714",
0x4050841c,"MainField_Enemy_Lizalfos_Senior_3030989118",
0x0361cce4,"MainField_Enemy_Lizalfos_Senior_3149348100",
0xec135729,"MainField_Enemy_Lizalfos_Senior_3172135515",
0xf254afce,"MainField_Enemy_Lizalfos_Senior_3215812142",
0xe933f9d9,"MainField_Enemy_Lizalfos_Senior_3220077514",
0xdb2a6b29,"MainField_Enemy_Lizalfos_Senior_322985871",
0x47267c37,"MainField_Enemy_Lizalfos_Senior_3244517268",
0x3a169e49,"MainField_Enemy_Lizalfos_Senior_3244772377",
0xa839da64,"MainField_Enemy_Lizalfos_Senior_3319858707",
0x794e65aa,"MainField_Enemy_Lizalfos_Senior_3323495565",
0xdbfdd442,"MainField_Enemy_Lizalfos_Senior_3335291854",
0xb70be042,"MainField_Enemy_Lizalfos_Senior_337345172",
0x8dfa2951,"MainField_Enemy_Lizalfos_Senior_3385217840",
0x14ca94ae,"MainField_Enemy_Lizalfos_Senior_3418425290",
0x18f9b6ea,"MainField_Enemy_Lizalfos_Senior_3449367158",
0xd0afaf62,"MainField_Enemy_Lizalfos_Senior_3472447239",
0x0d4c578d,"MainField_Enemy_Lizalfos_Senior_3495043470",
0x1914f140,"MainField_Enemy_Lizalfos_Senior_3504260577",
0x5018c50e,"MainField_Enemy_Lizalfos_Senior_3543306585",
0x7f651d7f,"MainField_Enemy_Lizalfos_Senior_3587290090",
0x3f7b2ebd,"MainField_Enemy_Lizalfos_Senior_3682015981",
0x5d7eba39,"MainField_Enemy_Lizalfos_Senior_3695400129",
0x5939b940,"MainField_Enemy_Lizalfos_Senior_3716939128",
0x0e4074e2,"MainField_Enemy_Lizalfos_Senior_372186287",
0x4ced7a87,"MainField_Enemy_Lizalfos_Senior_3755408198",
0xff4ba554,"MainField_Enemy_Lizalfos_Senior_3774417706",
0x4c45577a,"MainField_Enemy_Lizalfos_Senior_3785551271",
0x744e4879,"MainField_Enemy_Lizalfos_Senior_3811803402",
0x954f4bd1,"MainField_Enemy_Lizalfos_Senior_3825158129",
0xff5e9bfb,"MainField_Enemy_Lizalfos_Senior_3864588119",
0xb2f1bbda,"MainField_Enemy_Lizalfos_Senior_394593651",
0xc70617d9,"MainField_Enemy_Lizalfos_Senior_3949489249",
0x41764e75,"MainField_Enemy_Lizalfos_Senior_3954973619",
0xc5493d1f,"MainField_Enemy_Lizalfos_Senior_3983767224",
0x9d5d9917,"MainField_Enemy_Lizalfos_Senior_3991036631",
0x5fcb2049,"MainField_Enemy_Lizalfos_Senior_3999620644",
0xcc75e130,"MainField_Enemy_Lizalfos_Senior_4004178926",
0x7e29deba,"MainField_Enemy_Lizalfos_Senior_4086260839",
0x92a60f9e,"MainField_Enemy_Lizalfos_Senior_4108911525",
0xd337961a,"MainField_Enemy_Lizalfos_Senior_4115537013",
0x2e65b7c2,"MainField_Enemy_Lizalfos_Senior_4143726007",
0x91d5e0e3,"MainField_Enemy_Lizalfos_Senior_4160426790",
0x5a8cbfe6,"MainField_Enemy_Lizalfos_Senior_4169217372",
0xa7648a1b,"MainField_Enemy_Lizalfos_Senior_4169505790",
0xa7733792,"MainField_Enemy_Lizalfos_Senior_4205300208",
0xf96009ff,"MainField_Enemy_Lizalfos_Senior_44135244",
0x16df837e,"MainField_Enemy_Lizalfos_Senior_510128032",
0xae6cd96d,"MainField_Enemy_Lizalfos_Senior_524690876",
0xb2498de5,"MainField_Enemy_Lizalfos_Senior_525601999",
0x42da4733,"MainField_Enemy_Lizalfos_Senior_529099813",
0xcdae5ddb,"MainField_Enemy_Lizalfos_Senior_584015546",
0x974bd276,"MainField_Enemy_Lizalfos_Senior_598761443",
0x6b8a5a74,"MainField_Enemy_Lizalfos_Senior_618093973",
0xe7dd0c5d,"MainField_Enemy_Lizalfos_Senior_637156226",
0xb1cfaa0c,"MainField_Enemy_Lizalfos_Senior_66161362",
0x11ded277,"MainField_Enemy_Lizalfos_Senior_720837969",
0x805b3b08,"MainField_Enemy_Lizalfos_Senior_742164841",
0x90248d31,"MainField_Enemy_Lizalfos_Senior_750049145",
0x896e00e0,"MainField_Enemy_Lizalfos_Senior_777962100",
0xbb74de4d,"MainField_Enemy_Lizalfos_Senior_795395900",
0x40f47e17,"MainField_Enemy_Lizalfos_Senior_818738781",
0xda0246e4,"MainField_Enemy_Lizalfos_Senior_909790673",
0x53e759f8,"MainField_Enemy_Lizalfos_Senior_91488073",
0xe42f9939,"MainField_Enemy_Lizalfos_Senior_927336859",
0x523b7283,"MainField_Enemy_Lizalfos_Senior_998011909",
0xe5af102e,"MainField_Enemy_Lynel_Dark_1578404536",
0xeea7a1b1,"MainField_Enemy_Lynel_Junior_1492521341",
0xc305d900,"MainField_Enemy_Lynel_Junior_1819222204",
0xed7ea754,"MainField_Enemy_Lynel_Junior_232962450",
0x6bc4267b,"MainField_Enemy_Lynel_Junior_3391176708",
0x8ad4ab74,"MainField_Enemy_Lynel_Junior_4200912374",
0x5a3c453e,"MainField_Enemy_Lynel_Junior_459804714",
0x9c820609,"MainField_Enemy_Lynel_Junior_Mountain_1396251553",
0xab8b3665,"MainField_Enemy_Lynel_Junior_Mountain_2291678401",
0x5d03cb63,"MainField_Enemy_Lynel_Middle_1423341458",
0xefdb1f57,"MainField_Enemy_Lynel_Middle_1529588945",
0x58fbf605,"MainField_Enemy_Lynel_Middle_1544805954",
0xb2f5bb9c,"MainField_Enemy_Lynel_Middle_2284947922",
0x841abb0a,"MainField_Enemy_Lynel_Middle_3396778219",
0x2b60d3a6,"MainField_Enemy_Lynel_Middle_4192271973",
0x21ab967a,"MainField_Enemy_Lynel_Middle_438554606",
0xa98b7035,"MainField_Enemy_Lynel_Middle_86620264",
0x6eefada7,"MainField_Enemy_Lynel_Senior_1309263371",
0x6a72c7fb,"MainField_Enemy_Lynel_Senior_237760997",
0x677989d5,"MainField_Enemy_Lynel_Senior_2378625228",
0x43ef1faf,"MainField_Enemy_Lynel_Senior_329409903",
0x1ca54b9e,"MainField_Enemy_Lynel_Senior_3371712234",
0xed26d895,"MainField_Enemy_Lynel_Senior_3515533418",
0x724c2b71,"MainField_Enemy_Lynel_Senior_3788100745",
0x92138a89,"MainField_Enemy_Lynel_Senior_4101701470",
0xa8d2d9e2,"MainField_Enemy_Lynel_Senior_4246046905",
0xb5e18c52,"MainField_Enemy_Moriblin_Bone_Junior_1525637263",
0xed1d82aa,"MainField_Enemy_Moriblin_Bone_Junior_1532614312",
0x04bf55b0,"MainField_Enemy_Moriblin_Bone_Junior_1897317715",
0x82360648,"MainField_Enemy_Moriblin_Bone_Junior_2710276379",
0x651b71d1,"MainField_Enemy_Moriblin_Bone_Junior_2717979786",
0xde3511d4,"MainField_Enemy_Moriblin_Bone_Junior_2734324001",
0x39f553f4,"MainField_Enemy_Moriblin_Bone_Junior_2840888026",
0xe84d1686,"MainField_Enemy_Moriblin_Bone_Junior_3261508514",
0x1ac215a3,"MainField_Enemy_Moriblin_Bone_Junior_3356047293",
0xf68e1565,"MainField_Enemy_Moriblin_Bone_Junior_3856977829",
0x7ad87ed4,"MainField_Enemy_Moriblin_Bone_Junior_3948785183",
0xada58f15,"MainField_Enemy_Moriblin_Bone_Junior_4254590299",
0xe573f120,"MainField_Enemy_Moriblin_Bone_Junior_804123957",
0x70f9254e,"MainField_Enemy_Moriblin_Bone_Junior_941383066",
0x9dd4d03b,"MainField_Enemy_Moriblin_Junior_1001365173",
0x34f4b9c2,"MainField_Enemy_Moriblin_Junior_1134039637",
0x0b2a0cd1,"MainField_Enemy_Moriblin_Junior_1432433364",
0xb06bae15,"MainField_Enemy_Moriblin_Junior_1437835106",
0x06210966,"MainField_Enemy_Moriblin_Junior_1529161689",
0x6a25574a,"MainField_Enemy_Moriblin_Junior_1543730610",
0xd10de06f,"MainField_Enemy_Moriblin_Junior_1543885641",
0x45aa1958,"MainField_Enemy_Moriblin_Junior_1736834528",
0xb8fa32c6,"MainField_Enemy_Moriblin_Junior_1767798226",
0x810fb321,"MainField_Enemy_Moriblin_Junior_1770967508",
0xd7692a63,"MainField_Enemy_Moriblin_Junior_1890501329",
0x24e4cf8f,"MainField_Enemy_Moriblin_Junior_192088109",
0x648413b1,"MainField_Enemy_Moriblin_Junior_2050629132",
0x0bf7f974,"MainField_Enemy_Moriblin_Junior_2078192644",
0xeb2cb873,"MainField_Enemy_Moriblin_Junior_2087991483",
0x3f741bd9,"MainField_Enemy_Moriblin_Junior_2317415999",
0x218b0542,"MainField_Enemy_Moriblin_Junior_236961388",
0xc05d7615,"MainField_Enemy_Moriblin_Junior_2407650905",
0x8ab3b466,"MainField_Enemy_Moriblin_Junior_2411315904",
0x06c23b9a,"MainField_Enemy_Moriblin_Junior_2457491863",
0xf96e0a98,"MainField_Enemy_Moriblin_Junior_2912318329",
0x4d67d411,"MainField_Enemy_Moriblin_Junior_2987111013",
0x518ed82e,"MainField_Enemy_Moriblin_Junior_3043391100",
0xc2bb8882,"MainField_Enemy_Moriblin_Junior_33688736",
0xfcb59194,"MainField_Enemy_Moriblin_Junior_339089672",
0xfb3530c2,"MainField_Enemy_Moriblin_Junior_3692540850",
0xba97ed44,"MainField_Enemy_Moriblin_Junior_378046435",
0x13798647,"MainField_Enemy_Moriblin_Junior_3805799870",
0x3f6fd725,"MainField_Enemy_Moriblin_Junior_4177870046",
0xfdd3c064,"MainField_Enemy_Moriblin_Junior_554836133",
0x808e40cf,"MainField_Enemy_Moriblin_Junior_565416702",
0x00a084a8,"MainField_Enemy_Moriblin_Junior_649983064",
0xb8d29e90,"MainField_Enemy_Moriblin_Junior_721609695",
0xa27b6ca8,"MainField_Enemy_Moriblin_Junior_745482182",
0x15b73043,"MainField_Enemy_Moriblin_Middle_1047563228",
0xb5af3ad4,"MainField_Enemy_Moriblin_Middle_1082879458",
0x50bf21bd,"MainField_Enemy_Moriblin_Middle_1098506623",
0x114c7d60,"MainField_Enemy_Moriblin_Middle_1128785280",
0xbea90f23,"MainField_Enemy_Moriblin_Middle_1208096369",
0x4f661aee,"MainField_Enemy_Moriblin_Middle_1226875913",
0x409114e6,"MainField_Enemy_Moriblin_Middle_1306387652",
0x27f7437f,"MainField_Enemy_Moriblin_Middle_1334736366",
0xf8aa5300,"MainField_Enemy_Moriblin_Middle_1462941879",
0xcc5351c7,"MainField_Enemy_Moriblin_Middle_1537958621",
0x992ae428,"MainField_Enemy_Moriblin_Middle_1538394284",
0xeda91e81,"MainField_Enemy_Moriblin_Middle_1656844317",
0x1060b8a6,"MainField_Enemy_Moriblin_Middle_178649063",
0x10729906,"MainField_Enemy_Moriblin_Middle_1820735438",
0x64a452fb,"MainField_Enemy_Moriblin_Middle_1862793938",
0x24224f29,"MainField_Enemy_Moriblin_Middle_1927188256",
0x7891b0e1,"MainField_Enemy_Moriblin_Middle_196781734",
0xbaef3bb4,"MainField_Enemy_Moriblin_Middle_1985877316",
0x5cd9ed86,"MainField_Enemy_Moriblin_Middle_1986366784",
0xcd60624d,"MainField_Enemy_Moriblin_Middle_2018115166",
0xca9295f5,"MainField_Enemy_Moriblin_Middle_2068825377",
0x60434f6c,"MainField_Enemy_Moriblin_Middle_2078958192",
0x1db8dfd0,"MainField_Enemy_Moriblin_Middle_2096714846",
0x9a3666bc,"MainField_Enemy_Moriblin_Middle_2141826487",
0x40bbb0fd,"MainField_Enemy_Moriblin_Middle_2144238695",
0x435602a3,"MainField_Enemy_Moriblin_Middle_2212108301",
0xfca4d63e,"MainField_Enemy_Moriblin_Middle_2272172473",
0xf3b3a3a6,"MainField_Enemy_Moriblin_Middle_2284309825",
0x4777b4ef,"MainField_Enemy_Moriblin_Middle_2412300446",
0xf085682f,"MainField_Enemy_Moriblin_Middle_2457667199",
0x2e5d9d67,"MainField_Enemy_Moriblin_Middle_2488560908",
0x6d6951a4,"MainField_Enemy_Moriblin_Middle_2519986759",
0xb5cfb0c0,"MainField_Enemy_Moriblin_Middle_2598930816",
0x713657bd,"MainField_Enemy_Moriblin_Middle_2655449625",
0x9cb74e8e,"MainField_Enemy_Moriblin_Middle_2840846277",
0xdf0cbf07,"MainField_Enemy_Moriblin_Middle_2866811557",
0x4ab1d564,"MainField_Enemy_Moriblin_Middle_2878454987",
0x73cc6cf3,"MainField_Enemy_Moriblin_Middle_2887317037",
0x21b5420b,"MainField_Enemy_Moriblin_Middle_2925833015",
0xa45786e2,"MainField_Enemy_Moriblin_Middle_2931830636",
0xef782de1,"MainField_Enemy_Moriblin_Middle_293284437",
0x33b60565,"MainField_Enemy_Moriblin_Middle_3026695031",
0x9b7de9c9,"MainField_Enemy_Moriblin_Middle_3030873147",
0x5755bcc9,"MainField_Enemy_Moriblin_Middle_3038834311",
0x164952d3,"MainField_Enemy_Moriblin_Middle_3071197125",
0xbf55a128,"MainField_Enemy_Moriblin_Middle_3135122384",
0x9bd37e6e,"MainField_Enemy_Moriblin_Middle_3137938984",
0x823d66d6,"MainField_Enemy_Moriblin_Middle_3193708632",
0xc0c6e7d8,"MainField_Enemy_Moriblin_Middle_3227528786",
0x7f0e175a,"MainField_Enemy_Moriblin_Middle_3249635121",
0x28e6822c,"MainField_Enemy_Moriblin_Middle_3290220542",
0x7948cd5e,"MainField_Enemy_Moriblin_Middle_3374818351",
0xb1e5df52,"MainField_Enemy_Moriblin_Middle_345383439",
0x1b56f2b4,"MainField_Enemy_Moriblin_Middle_3466703978",
0x1199fa6d,"MainField_Enemy_Moriblin_Middle_3505780117",
0xc7c3c6cb,"MainField_Enemy_Moriblin_Middle_3522501949",
0x8eccf473,"MainField_Enemy_Moriblin_Middle_355969194",
0x8edd090d,"MainField_Enemy_Moriblin_Middle_3612011113",
0x1f1cbb00,"MainField_Enemy_Moriblin_Middle_3692367822",
0x1d949552,"MainField_Enemy_Moriblin_Middle_3705527175",
0xab19f58e,"MainField_Enemy_Moriblin_Middle_3716675756",
0x998c4124,"MainField_Enemy_Moriblin_Middle_3833114478",
0x2ff57051,"MainField_Enemy_Moriblin_Middle_3857498106",
0x3db21d54,"MainField_Enemy_Moriblin_Middle_3896122125",
0xc6b2ab5a,"MainField_Enemy_Moriblin_Middle_3967601672",
0x2b9d311e,"MainField_Enemy_Moriblin_Middle_4029235517",
0x107b764c,"MainField_Enemy_Moriblin_Middle_4035635769",
0x63abcc5c,"MainField_Enemy_Moriblin_Middle_4074970074",
0xacc4a5b2,"MainField_Enemy_Moriblin_Middle_4083019170",
0xc464c4e6,"MainField_Enemy_Moriblin_Middle_4128444200",
0x4acf4593,"MainField_Enemy_Moriblin_Middle_4177964947",
0xe44f4f4f,"MainField_Enemy_Moriblin_Middle_463568543",
0x6bd28f97,"MainField_Enemy_Moriblin_Middle_49008408",
0x44901bcb,"MainField_Enemy_Moriblin_Middle_491049017",
0xfc2ef582,"MainField_Enemy_Moriblin_Middle_514245995",
0xfb9aa825,"MainField_Enemy_Moriblin_Middle_691127437",
0xba756b61,"MainField_Enemy_Moriblin_Middle_702094619",
0x886bf6df,"MainField_Enemy_Moriblin_Middle_814973765",
0xc37e58a1,"MainField_Enemy_Moriblin_Middle_880622559",
0xadb7154c,"MainField_Enemy_Moriblin_Middle_902838187",
0x00d12a18,"MainField_Enemy_Moriblin_Middle_933089512",
0xec58da5e,"MainField_Enemy_Moriblin_Middle_943734742",
0x76360113,"MainField_Enemy_Moriblin_Senior_1017714556",
0xcb842422,"MainField_Enemy_Moriblin_Senior_1034399304",
0x0c8db61f,"MainField_Enemy_Moriblin_Senior_1065058194",
0x596dc678,"MainField_Enemy_Moriblin_Senior_1094548807",
0x3cec221d,"MainField_Enemy_Moriblin_Senior_1115881052",
0x307760f3,"MainField_Enemy_Moriblin_Senior_1131927046",
0xbdca0ea9,"MainField_Enemy_Moriblin_Senior_1138226578",
0xa69b5e34,"MainField_Enemy_Moriblin_Senior_1182860777",
0x24c28de9,"MainField_Enemy_Moriblin_Senior_1270139133",
0x0798fab5,"MainField_Enemy_Moriblin_Senior_1327176595",
0xcf7e8db1,"MainField_Enemy_Moriblin_Senior_1350975983",
0xeceb7007,"MainField_Enemy_Moriblin_Senior_1371283539",
0xaac5b442,"MainField_Enemy_Moriblin_Senior_1372040185",
0xed7b8ebf,"MainField_Enemy_Moriblin_Senior_1377791512",
0x281ebaf1,"MainField_Enemy_Moriblin_Senior_1412727752",
0xc3617099,"MainField_Enemy_Moriblin_Senior_1421307765",
0xa36639ae,"MainField_Enemy_Moriblin_Senior_1459471412",
0x2408f87c,"MainField_Enemy_Moriblin_Senior_1485122072",
0xa5a3a17d,"MainField_Enemy_Moriblin_Senior_1524766699",
0xa23b5d75,"MainField_Enemy_Moriblin_Senior_1537103879",
0x9c7d7765,"MainField_Enemy_Moriblin_Senior_1593838394",
0x60f5e22f,"MainField_Enemy_Moriblin_Senior_160070374",
0x2f67a1ba,"MainField_Enemy_Moriblin_Senior_1649574144",
0x36a567e5,"MainField_Enemy_Moriblin_Senior_165507875",
0x07e87624,"MainField_Enemy_Moriblin_Senior_1671149722",
0x7db4e84b,"MainField_Enemy_Moriblin_Senior_1692306815",
0xdfdefdf8,"MainField_Enemy_Moriblin_Senior_1697632751",
0xf84ccd46,"MainField_Enemy_Moriblin_Senior_1704609113",
0x3b6d31a7,"MainField_Enemy_Moriblin_Senior_1714942535",
0x42aada16,"MainField_Enemy_Moriblin_Senior_1743508121",
0x9e3053d1,"MainField_Enemy_Moriblin_Senior_1755966861",
0xc20aa678,"MainField_Enemy_Moriblin_Senior_1763375814",
0x5ab3b363,"MainField_Enemy_Moriblin_Senior_1825124442",
0xb462b676,"MainField_Enemy_Moriblin_Senior_1847314475",
0xb843a5aa,"MainField_Enemy_Moriblin_Senior_1855094598",
0xc68e18b7,"MainField_Enemy_Moriblin_Senior_1902333944",
0xd52317d1,"MainField_Enemy_Moriblin_Senior_1908293531",
0xd75ac0c1,"MainField_Enemy_Moriblin_Senior_1957278813",
0xcfff00b1,"MainField_Enemy_Moriblin_Senior_1993787485",
0x8b7aa076,"MainField_Enemy_Moriblin_Senior_2053061311",
0x5087301c,"MainField_Enemy_Moriblin_Senior_2137535942",
0x14fccec2,"MainField_Enemy_Moriblin_Senior_2152361388",
0xf29413d9,"MainField_Enemy_Moriblin_Senior_2299809648",
0x27121ddc,"MainField_Enemy_Moriblin_Senior_2311530784",
0x34a633e7,"MainField_Enemy_Moriblin_Senior_2326362082",
0xf8439d58,"MainField_Enemy_Moriblin_Senior_236982467",
0x266612ec,"MainField_Enemy_Moriblin_Senior_2389254047",
0x998e617e,"MainField_Enemy_Moriblin_Senior_2473301115",
0x047e7fb0,"MainField_Enemy_Moriblin_Senior_2475693344",
0xceb71af4,"MainField_Enemy_Moriblin_Senior_2490523841",
0x69af57c7,"MainField_Enemy_Moriblin_Senior_2506784697",
0xda56ca4e,"MainField_Enemy_Moriblin_Senior_253979430",
0x701c835b,"MainField_Enemy_Moriblin_Senior_2560840429",
0x3f223323,"MainField_Enemy_Moriblin_Senior_2563776095",
0x60df89e7,"MainField_Enemy_Moriblin_Senior_2567697880",
0x0c2014c6,"MainField_Enemy_Moriblin_Senior_2576477679",
0x79bc9f5f,"MainField_Enemy_Moriblin_Senior_2635016419",
0x4bb8a343,"MainField_Enemy_Moriblin_Senior_2635473101",
0x5f8f8e2b,"MainField_Enemy_Moriblin_Senior_2660121770",
0x4d0f83a4,"MainField_Enemy_Moriblin_Senior_2731514393",
0xaaa29157,"MainField_Enemy_Moriblin_Senior_2762580060",
0xd5246c6f,"MainField_Enemy_Moriblin_Senior_2779839064",
0x0f36fa82,"MainField_Enemy_Moriblin_Senior_2795938130",
0x65044f5f,"MainField_Enemy_Moriblin_Senior_2807954694",
0x5c6eaffb,"MainField_Enemy_Moriblin_Senior_2832391619",
0x95f535e4,"MainField_Enemy_Moriblin_Senior_2856575650",
0x525f8852,"MainField_Enemy_Moriblin_Senior_2884178422",
0x250b27ec,"MainField_Enemy_Moriblin_Senior_2901002269",
0x6e6eade7,"MainField_Enemy_Moriblin_Senior_2951738210",
0x52d1b038,"MainField_Enemy_Moriblin_Senior_2968634266",
0xf9d9dc0b,"MainField_Enemy_Moriblin_Senior_3022555288",
0xe6c1d210,"MainField_Enemy_Moriblin_Senior_3064935288",
0xdcd5bb14,"MainField_Enemy_Moriblin_Senior_3067709812",
0x320bc28d,"MainField_Enemy_Moriblin_Senior_3085780622",
0x84339a59,"MainField_Enemy_Moriblin_Senior_3091562255",
0x30d672bd,"MainField_Enemy_Moriblin_Senior_3136750335",
0xce761cbb,"MainField_Enemy_Moriblin_Senior_3211522825",
0xe97d6186,"MainField_Enemy_Moriblin_Senior_328804142",
0x4098f544,"MainField_Enemy_Moriblin_Senior_3290052260",
0x10e8d39c,"MainField_Enemy_Moriblin_Senior_3312939965",
0x7d84afb7,"MainField_Enemy_Moriblin_Senior_3314068401",
0x8ca299f0,"MainField_Enemy_Moriblin_Senior_3355909133",
0x05a2adc1,"MainField_Enemy_Moriblin_Senior_3446211624",
0xafc177b0,"MainField_Enemy_Moriblin_Senior_3535260366",
0x88cde360,"MainField_Enemy_Moriblin_Senior_3554029075",
0xeef6641d,"MainField_Enemy_Moriblin_Senior_3631292818",
0x89f549a6,"MainField_Enemy_Moriblin_Senior_3633727247",
0x1cde679d,"MainField_Enemy_Moriblin_Senior_3666777562",
0xb689d588,"MainField_Enemy_Moriblin_Senior_3697862807",
0x34b652a9,"MainField_Enemy_Moriblin_Senior_3699668018",
0x10a47f1a,"MainField_Enemy_Moriblin_Senior_3746799062",
0x549d2325,"MainField_Enemy_Moriblin_Senior_3773466958",
0xa29954c8,"MainField_Enemy_Moriblin_Senior_3776627622",
0x54dcc8ba,"MainField_Enemy_Moriblin_Senior_3799235236",
0x93ccceb3,"MainField_Enemy_Moriblin_Senior_3799907517",
0x7f5f0bec,"MainField_Enemy_Moriblin_Senior_3801856830",
0xdecc5dd8,"MainField_Enemy_Moriblin_Senior_3876811782",
0x47c3a58f,"MainField_Enemy_Moriblin_Senior_3898366362",
0xc3aeb676,"MainField_Enemy_Moriblin_Senior_3904435620",
0x3e95e4ab,"MainField_Enemy_Moriblin_Senior_398151326",
0x3b80ba0e,"MainField_Enemy_Moriblin_Senior_4011777332",
0x113e4ce8,"MainField_Enemy_Moriblin_Senior_4035828775",
0x2ad0833e,"MainField_Enemy_Moriblin_Senior_4052292666",
0x7538250d,"MainField_Enemy_Moriblin_Senior_4086047655",
0x59bd9e6e,"MainField_Enemy_Moriblin_Senior_4100872014",
0x438b689f,"MainField_Enemy_Moriblin_Senior_412873",
0x31e2e380,"MainField_Enemy_Moriblin_Senior_4153956892",
0x7d8faf78,"MainField_Enemy_Moriblin_Senior_4172077709",
0xaa15ffc4,"MainField_Enemy_Moriblin_Senior_4174209577",
0xc029667a,"MainField_Enemy_Moriblin_Senior_4203931395",
0x2f98ab10,"MainField_Enemy_Moriblin_Senior_4240775581",
0x25ae6f3b,"MainField_Enemy_Moriblin_Senior_4293252327",
0xa8531c1c,"MainField_Enemy_Moriblin_Senior_505268816",
0x98ec8260,"MainField_Enemy_Moriblin_Senior_526219962",
0xf4de50d5,"MainField_Enemy_Moriblin_Senior_570094754",
0x4ddd7063,"MainField_Enemy_Moriblin_Senior_577965410",
0x32eaf193,"MainField_Enemy_Moriblin_Senior_687058618",
0x6fa517ce,"MainField_Enemy_Moriblin_Senior_701662810",
0x42c907a0,"MainField_Enemy_Moriblin_Senior_717038457",
0x3dd59b01,"MainField_Enemy_Moriblin_Senior_747059457",
0x69737098,"MainField_Enemy_Moriblin_Senior_807158507",
0x66428e85,"MainField_Enemy_Moriblin_Senior_809779390",
0xea290b34,"MainField_Enemy_Moriblin_Senior_877313805",
0xf6f9cb70,"MainField_Enemy_Moriblin_Senior_879814288",
0x19459f0c,"MainField_Enemy_Moriblin_Senior_89924997",
0x992434ce,"MainField_Enemy_Moriblin_Senior_903085374",
0x928023ec,"MainField_Enemy_Moriblin_Senior_927761522",
0xa254f617,"MainField_Enemy_Moriblin_Senior_947278028",
0xaede741d,"MainField_Enemy_Moriblin_Senior_962915971",
0xa7a0332f,"MainField_Enemy_Moriblin_Senior_Volcano_1017773918",
0x0927c7c0,"MainField_Enemy_Moriblin_Senior_Volcano_102668443",
0xf258af0c,"MainField_Enemy_Moriblin_Senior_Volcano_1198917678",
0xff33e247,"MainField_Enemy_Moriblin_Senior_Volcano_1706018959",
0xb3a89751,"MainField_Enemy_Moriblin_Senior_Volcano_2022306038",
0xd861f236,"MainField_Enemy_Moriblin_Senior_Volcano_2311411558",
0x04714000,"MainField_Enemy_Moriblin_Senior_Volcano_2595221715",
0x2f40cbca,"MainField_Enemy_Moriblin_Senior_Volcano_2720230453",
0xed16b10c,"MainField_Enemy_Moriblin_Senior_Volcano_3006294729",
0x4b7c9c59,"MainField_Enemy_Moriblin_Senior_Volcano_3185518054",
0xe1465a1b,"MainField_Enemy_Moriblin_Senior_Volcano_4109247023",
0x7bf17334,"MainField_Enemy_Moriblin_Senior_Volcano_4210247064",
0x8bcf69a9,"MainField_Enemy_Moriblin_Senior_Volcano_660970525",
0xb12d165c,"MainField_Enemy_Moriblin_Senior_Volcano_714239954",
0x2fb35423,"MainField_Enemy_Moriblin_Senior_Volcano_780471353",
0x30df0c0c,"MainField_Enemy_Moriblin_Senior_Volcano_921461692",
0xd04c0265,"MainField_Enemy_Octarock_1006348772",
0xfd320dc3,"MainField_Enemy_Octarock_1007641955",
0xf38bb609,"MainField_Enemy_Octarock_1104265778",
0xcf1cb079,"MainField_Enemy_Octarock_1177836096",
0x3f9fc995,"MainField_Enemy_Octarock_1233253507",
0x803cd6ac,"MainField_Enemy_Octarock_127865233",
0x73ff8a43,"MainField_Enemy_Octarock_1501359208",
0x03a2df68,"MainField_Enemy_Octarock_1519366603",
0xf2f8b10a,"MainField_Enemy_Octarock_1567912123",
0x061496a3,"MainField_Enemy_Octarock_160585954",
0x3850f1d1,"MainField_Enemy_Octarock_1636855218",
0x374fdccf,"MainField_Enemy_Octarock_1656243848",
0xcb791ca0,"MainField_Enemy_Octarock_1684538706",
0x4b2519ae,"MainField_Enemy_Octarock_1751792522",
0x12fc3067,"MainField_Enemy_Octarock_1917343809",
0xfdeb1275,"MainField_Enemy_Octarock_1960688318",
0x10057442,"MainField_Enemy_Octarock_1964134430",
0x23bb3e6c,"MainField_Enemy_Octarock_1997460802",
0x0bfa0363,"MainField_Enemy_Octarock_207917152",
0xe8430a2c,"MainField_Enemy_Octarock_2290789136",
0xb0a2108b,"MainField_Enemy_Octarock_2291847006",
0x5f7bcb38,"MainField_Enemy_Octarock_2307556360",
0xc2f29601,"MainField_Enemy_Octarock_2363409195",
0x80e017d5,"MainField_Enemy_Octarock_2370419682",
0x17da08db,"MainField_Enemy_Octarock_2411995658",
0xe3afdaab,"MainField_Enemy_Octarock_2495032935",
0x331eaddc,"MainField_Enemy_Octarock_2509580232",
0x2e1abbc8,"MainField_Enemy_Octarock_2550863018",
0x72455cf5,"MainField_Enemy_Octarock_2590863314",
0xb977852e,"MainField_Enemy_Octarock_2667393210",
0x99949ba5,"MainField_Enemy_Octarock_30257176",
0x14d23085,"MainField_Enemy_Octarock_3163581242",
0xdce61c13,"MainField_Enemy_Octarock_362540053",
0x08d110f6,"MainField_Enemy_Octarock_3654284807",
0x04246dbb,"MainField_Enemy_Octarock_369017692",
0x7a176de1,"MainField_Enemy_Octarock_3717726412",
0x28dc0284,"MainField_Enemy_Octarock_3741008651",
0xb231062a,"MainField_Enemy_Octarock_3785990873",
0x74ebbdcc,"MainField_Enemy_Octarock_4003102219",
0xe907ee6d,"MainField_Enemy_Octarock_4033255927",
0x9fbfd174,"MainField_Enemy_Octarock_4033716144",
0xf34fcd9d,"MainField_Enemy_Octarock_4085320988",
0xcf09fbbb,"MainField_Enemy_Octarock_460718121",
0xc0ccf6e3,"MainField_Enemy_Octarock_515355103",
0x0765f0cb,"MainField_Enemy_Octarock_632783191",
0x8d406638,"MainField_Enemy_Octarock_650808215",
0x72a73a87,"MainField_Enemy_Octarock_755168109",
0xadbcd267,"MainField_Enemy_Octarock_780743541",
0xcfc46212,"MainField_Enemy_Octarock_8017975",
0x6cd98760,"MainField_Enemy_Octarock_815333515",
0xf213d0eb,"MainField_Enemy_Octarock_825326301",
0xa87b6957,"MainField_Enemy_Octarock_827692418",
0x6c71e161,"MainField_Enemy_Octarock_939593119",
0x54796fdb,"MainField_Enemy_Octarock_952572607",
0x23fd01b2,"MainField_Enemy_Octarock_953615098",
0x84da2ad9,"MainField_Enemy_Octarock_969174750",
0x8e745726,"MainField_Enemy_Octarock_Desert_1132805248",
0xe2d9d429,"MainField_Enemy_Octarock_Desert_1155619029",
0x937eb301,"MainField_Enemy_Octarock_Desert_1378501193",
0x6f6ab6dc,"MainField_Enemy_Octarock_Desert_1551668948",
0xdc3758c3,"MainField_Enemy_Octarock_Desert_1604675775",
0xf7a9f678,"MainField_Enemy_Octarock_Desert_1671622314",
0x83ae684d,"MainField_Enemy_Octarock_Desert_1793702270",
0xdb9299a0,"MainField_Enemy_Octarock_Desert_1919386742",
0x01962b65,"MainField_Enemy_Octarock_Desert_196352102",
0xaa5e3537,"MainField_Enemy_Octarock_Desert_201544248",
0xc270f9e0,"MainField_Enemy_Octarock_Desert_2048386092",
0xe883aab5,"MainField_Enemy_Octarock_Desert_2078718606",
0x9f6802d4,"MainField_Enemy_Octarock_Desert_2167448643",
0x629a74c3,"MainField_Enemy_Octarock_Desert_2184700903",
0xd776f03a,"MainField_Enemy_Octarock_Desert_2245019489",
0x4798fc1d,"MainField_Enemy_Octarock_Desert_2314194073",
0xa5d4a4e7,"MainField_Enemy_Octarock_Desert_2374781961",
0x9bd7f0cd,"MainField_Enemy_Octarock_Desert_2501865443",
0x277ffa41,"MainField_Enemy_Octarock_Desert_2537013573",
0x72b6d2e0,"MainField_Enemy_Octarock_Desert_2595847882",
0xc940513c,"MainField_Enemy_Octarock_Desert_2783692253",
0xf0dae565,"MainField_Enemy_Octarock_Desert_2790700629",
0x621890b2,"MainField_Enemy_Octarock_Desert_2806735651",
0x91f2afa0,"MainField_Enemy_Octarock_Desert_282775300",
0xb4fd7a0a,"MainField_Enemy_Octarock_Desert_2964438853",
0x7790124a,"MainField_Enemy_Octarock_Desert_3033257757",
0x657e3ca3,"MainField_Enemy_Octarock_Desert_311847922",
0xdff64463,"MainField_Enemy_Octarock_Desert_3434213699",
0xda3d0b2a,"MainField_Enemy_Octarock_Desert_3618267228",
0xbfee0d48,"MainField_Enemy_Octarock_Desert_3620187322",
0x18f86f45,"MainField_Enemy_Octarock_Desert_3894850368",
0x76e10512,"MainField_Enemy_Octarock_Desert_4194134535",
0x1e80e096,"MainField_Enemy_Octarock_Desert_4266584972",
0x6da8df76,"MainField_Enemy_Octarock_Desert_4288665010",
0x6d01fcca,"MainField_Enemy_Octarock_Desert_552589613",
0xe8f11cf4,"MainField_Enemy_Octarock_Desert_721874498",
0x34b98e32,"MainField_Enemy_Octarock_Desert_766915581",
0x1fbe7446,"MainField_Enemy_Octarock_Forest_1006030925",
0x38f571ed,"MainField_Enemy_Octarock_Forest_1117638600",
0xaee6be6e,"MainField_Enemy_Octarock_Forest_1230138479",
0x3de2b54c,"MainField_Enemy_Octarock_Forest_1243860133",
0x9217a829,"MainField_Enemy_Octarock_Forest_1311851669",
0x107226a8,"MainField_Enemy_Octarock_Forest_1362815106",
0x0af2622b,"MainField_Enemy_Octarock_Forest_1376697048",
0x4ecd9f80,"MainField_Enemy_Octarock_Forest_1383055314",
0xca9931bd,"MainField_Enemy_Octarock_Forest_1406500822",
0x166aded9,"MainField_Enemy_Octarock_Forest_150326810",
0xf781abe6,"MainField_Enemy_Octarock_Forest_1528644741",
0x912c47ad,"MainField_Enemy_Octarock_Forest_1547781009",
0x43f5c8a9,"MainField_Enemy_Octarock_Forest_1564418169",
0x7318639b,"MainField_Enemy_Octarock_Forest_159162998",
0xbbf36e56,"MainField_Enemy_Octarock_Forest_162713229",
0xdad58950,"MainField_Enemy_Octarock_Forest_1642070805",
0xd54b1dcc,"MainField_Enemy_Octarock_Forest_1738822478",
0x885f1819,"MainField_Enemy_Octarock_Forest_176202752",
0x5be533a9,"MainField_Enemy_Octarock_Forest_1786691856",
0x6f991ae1,"MainField_Enemy_Octarock_Forest_1860486206",
0x686d1f7f,"MainField_Enemy_Octarock_Forest_1885623856",
0x60f1a2c0,"MainField_Enemy_Octarock_Forest_1946214827",
0x4c2dc53a,"MainField_Enemy_Octarock_Forest_196713556",
0xe637b321,"MainField_Enemy_Octarock_Forest_2065018018",
0x6fd4b1f9,"MainField_Enemy_Octarock_Forest_2091926928",
0x8ee77a81,"MainField_Enemy_Octarock_Forest_2104648910",
0x7ae16a35,"MainField_Enemy_Octarock_Forest_2177563943",
0x122b6812,"MainField_Enemy_Octarock_Forest_2188536129",
0xf6a3aa3d,"MainField_Enemy_Octarock_Forest_2250828964",
0x1a55db6f,"MainField_Enemy_Octarock_Forest_2297187145",
0x7d6a5d5e,"MainField_Enemy_Octarock_Forest_2306510577",
0x67a6fa0d,"MainField_Enemy_Octarock_Forest_2315555316",
0x226620cf,"MainField_Enemy_Octarock_Forest_2344143522",
0xc379e367,"MainField_Enemy_Octarock_Forest_2360589281",
0xf8480f17,"MainField_Enemy_Octarock_Forest_2434473801",
0x509d2cb2,"MainField_Enemy_Octarock_Forest_2465964765",
0x3d603d5f,"MainField_Enemy_Octarock_Forest_2476529532",
0xecda3a5a,"MainField_Enemy_Octarock_Forest_2478361772",
0xf95f6596,"MainField_Enemy_Octarock_Forest_2527950625",
0xee7b7ddb,"MainField_Enemy_Octarock_Forest_258668576",
0xac049c95,"MainField_Enemy_Octarock_Forest_2625164003",
0x28c6b339,"MainField_Enemy_Octarock_Forest_2674000475",
0x767572c0,"MainField_Enemy_Octarock_Forest_2748000492",
0x9860a635,"MainField_Enemy_Octarock_Forest_2860575127",
0x18e87671,"MainField_Enemy_Octarock_Forest_2983364793",
0x846ec2c6,"MainField_Enemy_Octarock_Forest_3021462027",
0x6e584d4b,"MainField_Enemy_Octarock_Forest_3022788869",
0x19c00a97,"MainField_Enemy_Octarock_Forest_3214552070",
0x422d9a02,"MainField_Enemy_Octarock_Forest_3246781715",
0xc397897a,"MainField_Enemy_Octarock_Forest_3273553730",
0x8d91b192,"MainField_Enemy_Octarock_Forest_327849573",
0xa5fda1b3,"MainField_Enemy_Octarock_Forest_331166831",
0xaefd8edc,"MainField_Enemy_Octarock_Forest_3318585397",
0xad66a405,"MainField_Enemy_Octarock_Forest_3358832952",
0x2fb5ce3d,"MainField_Enemy_Octarock_Forest_3397251617",
0x8f47d222,"MainField_Enemy_Octarock_Forest_3490329192",
0x42684a9e,"MainField_Enemy_Octarock_Forest_3544363341",
0x12108f37,"MainField_Enemy_Octarock_Forest_3545918782",
0xf467a7bb,"MainField_Enemy_Octarock_Forest_3595431153",
0xe0209d6f,"MainField_Enemy_Octarock_Forest_36393683",
0xdb23f56e,"MainField_Enemy_Octarock_Forest_3683950194",
0x3099d3ce,"MainField_Enemy_Octarock_Forest_378794469",
0xf87c5832,"MainField_Enemy_Octarock_Forest_3790684507",
0xc5d9803f,"MainField_Enemy_Octarock_Forest_3797936297",
0x5b30eb9e,"MainField_Enemy_Octarock_Forest_3838103112",
0x52266d5e,"MainField_Enemy_Octarock_Forest_3892008477",
0x425e1609,"MainField_Enemy_Octarock_Forest_3892854876",
0xc1a25f02,"MainField_Enemy_Octarock_Forest_3973789605",
0x0039f41e,"MainField_Enemy_Octarock_Forest_3983126891",
0x7860ca59,"MainField_Enemy_Octarock_Forest_4015273956",
0x47657acd,"MainField_Enemy_Octarock_Forest_4044305345",
0x4019b821,"MainField_Enemy_Octarock_Forest_4055040075",
0x457869d9,"MainField_Enemy_Octarock_Forest_408882126",
0xd8c08de5,"MainField_Enemy_Octarock_Forest_4106133691",
0x5f266f03,"MainField_Enemy_Octarock_Forest_4108994729",
0xc68ecafc,"MainField_Enemy_Octarock_Forest_4153350758",
0x78bb7e21,"MainField_Enemy_Octarock_Forest_4164823329",
0xc28be2ef,"MainField_Enemy_Octarock_Forest_4190653483",
0xd14607b7,"MainField_Enemy_Octarock_Forest_4236865073",
0x0bcb4bbb,"MainField_Enemy_Octarock_Forest_4256352054",
0xab745a26,"MainField_Enemy_Octarock_Forest_50843965",
0x8814eb6a,"MainField_Enemy_Octarock_Forest_509168239",
0x6e6cb48e,"MainField_Enemy_Octarock_Forest_578805078",
0xa3dea848,"MainField_Enemy_Octarock_Forest_641929439",
0xf5760e76,"MainField_Enemy_Octarock_Forest_684351998",
0xb47d1688,"MainField_Enemy_Octarock_Forest_687591537",
0x28b0f711,"MainField_Enemy_Octarock_Forest_760802821",
0xa0df7b79,"MainField_Enemy_Octarock_Forest_813327000",
0x252817b9,"MainField_Enemy_Octarock_Forest_872881653",
0x30653705,"MainField_Enemy_Octarock_Forest_880677065",
0xa1018e82,"MainField_Enemy_Octarock_Forest_884769439",
0x17a99d6a,"MainField_Enemy_Octarock_Snow_111223489",
0xd1bb8904,"MainField_Enemy_Octarock_Snow_1981696162",
0xde025155,"MainField_Enemy_Octarock_Snow_2512340514",
0x830d6078,"MainField_Enemy_Octarock_Snow_2741112375",
0x91920a43,"MainField_Enemy_Octarock_Stone_1108753825",
0xa4353969,"MainField_Enemy_Octarock_Stone_111590953",
0xb3a22e99,"MainField_Enemy_Octarock_Stone_1145671807",
0x96de2af3,"MainField_Enemy_Octarock_Stone_1182353191",
0xbab8c8a9,"MainField_Enemy_Octarock_Stone_1221520444",
0x343565d7,"MainField_Enemy_Octarock_Stone_1376124619",
0x939ab3e6,"MainField_Enemy_Octarock_Stone_1447677074",
0x5be70551,"MainField_Enemy_Octarock_Stone_1461174327",
0x5ce0b55c,"MainField_Enemy_Octarock_Stone_1481776588",
0xf9651222,"MainField_Enemy_Octarock_Stone_1504453628",
0x4ace3cf3,"MainField_Enemy_Octarock_Stone_1610447096",
0x51059488,"MainField_Enemy_Octarock_Stone_1754928886",
0xf258df1b,"MainField_Enemy_Octarock_Stone_1793684102",
0x7b27a400,"MainField_Enemy_Octarock_Stone_1972256926",
0x0c4976e5,"MainField_Enemy_Octarock_Stone_2313742864",
0x20b15f82,"MainField_Enemy_Octarock_Stone_2395379080",
0x77293507,"MainField_Enemy_Octarock_Stone_2539637916",
0xdee4212f,"MainField_Enemy_Octarock_Stone_2627063540",
0x448e4991,"MainField_Enemy_Octarock_Stone_2862705460",
0xb32814e4,"MainField_Enemy_Octarock_Stone_3021473868",
0x653b9634,"MainField_Enemy_Octarock_Stone_3047941854",
0x8217ee00,"MainField_Enemy_Octarock_Stone_3074886066",
0xd687960f,"MainField_Enemy_Octarock_Stone_3289459527",
0x77362839,"MainField_Enemy_Octarock_Stone_3386613796",
0xf835f5c3,"MainField_Enemy_Octarock_Stone_3524139480",
0x13bbf0ef,"MainField_Enemy_Octarock_Stone_3589430247",
0x39b59bbb,"MainField_Enemy_Octarock_Stone_3736032419",
0x67e95918,"MainField_Enemy_Octarock_Stone_3753182745",
0x4e9e4f33,"MainField_Enemy_Octarock_Stone_3916122715",
0x9c25d4e7,"MainField_Enemy_Octarock_Stone_4103043942",
0x9ed72a72,"MainField_Enemy_Octarock_Stone_4218806331",
0x957c9a5a,"MainField_Enemy_Octarock_Stone_554150108",
0x7700fdb7,"MainField_Enemy_Octarock_Stone_673950066",
0x55adb799,"MainField_Enemy_Octarock_Stone_770452127",
0x8cb61158,"MainField_Enemy_Octarock_Stone_809304132",
0x78fa9ceb,"MainField_Enemy_Octarock_Stone_819630444",
0x5930ad08,"MainField_Enemy_Octarock_Stone_923739580",
0x3e0c5627,"MainField_Enemy_Octarock_Stone_934839239",
0x9857bd3a,"MainField_Enemy_Octarock_Stone_935511264",
0xb6b38b06,"MainField_Enemy_Octarock_Stone_96332747",
0x34edd474,"MainField_Enemy_Octarock_Stone_966573691",
0x1abaecf3,"MainField_Enemy_Sandworm_3098676163",
0x28fe61d2,"MainField_Enemy_Sandworm_3538536935",
0xfd4e79c0,"MainField_Enemy_Sandworm_3569739054",
0x00333b27,"MainField_Enemy_Sandworm_494020159",
0x9915d0e2,"MainField_Enemy_Wizzrobe_Electric_122372285",
0xa1aa6064,"MainField_Enemy_Wizzrobe_Electric_2328776473",
0x213518b3,"MainField_Enemy_Wizzrobe_Electric_2395398727",
0x87cd7dc2,"MainField_Enemy_Wizzrobe_Electric_2399499615",
0x6475c9b1,"MainField_Enemy_Wizzrobe_Electric_3224520546",
0x0a702eb0,"MainField_Enemy_Wizzrobe_Electric_3400152372",
0xdb39d737,"MainField_Enemy_Wizzrobe_Electric_346741434",
0xb8ab9c51,"MainField_Enemy_Wizzrobe_Electric_3495042527",
0xcaae6f2d,"MainField_Enemy_Wizzrobe_Electric_3516966747",
0x08f3034d,"MainField_Enemy_Wizzrobe_Electric_410515381",
0xaee1216b,"MainField_Enemy_Wizzrobe_Electric_780410236",
0xc34602f3,"MainField_Enemy_Wizzrobe_Electric_817220492",
0xdd40c0b9,"MainField_Enemy_Wizzrobe_Electric_Senior_2924866523",
0x09d7bef5,"MainField_Enemy_Wizzrobe_Electric_Senior_778242681",
0xc1e1f135,"MainField_Enemy_Wizzrobe_Fire_1179191329",
0x2274e962,"MainField_Enemy_Wizzrobe_Fire_1464460165",
0x4687581d,"MainField_Enemy_Wizzrobe_Fire_1467785709",
0x8ebe2ff8,"MainField_Enemy_Wizzrobe_Fire_1587467038",
0x79091405,"MainField_Enemy_Wizzrobe_Fire_1726561686",
0x799099a1,"MainField_Enemy_Wizzrobe_Fire_1799378514",
0xe35d5b31,"MainField_Enemy_Wizzrobe_Fire_2429249575",
0x03e0331a,"MainField_Enemy_Wizzrobe_Fire_2932609917",
0xeece93dd,"MainField_Enemy_Wizzrobe_Fire_3108471465",
0xecb1d3ff,"MainField_Enemy_Wizzrobe_Fire_3460346431",
0xc113040e,"MainField_Enemy_Wizzrobe_Fire_600223672",
0x87001d12,"MainField_Enemy_Wizzrobe_Fire_629321287",
0xd13d5922,"MainField_Enemy_Wizzrobe_Fire_795782529",
0x83b79f28,"MainField_Enemy_Wizzrobe_Fire_Senior_2490774727",
0x5cf21e9c,"MainField_Enemy_Wizzrobe_Fire_Senior_3205650003",
0x9aeefef0,"MainField_Enemy_Wizzrobe_Fire_Senior_3803775886",
0xc646cb0c,"MainField_Enemy_Wizzrobe_Fire_Senior_926370827",
0xdbd2d98b,"MainField_Enemy_Wizzrobe_Ice_1341359998",
0x8cd18479,"MainField_Enemy_Wizzrobe_Ice_1375707729",
0xc877586f,"MainField_Enemy_Wizzrobe_Ice_209009649",
0xcea29bf8,"MainField_Enemy_Wizzrobe_Ice_3408432116",
0x63d1f193,"MainField_Enemy_Wizzrobe_Ice_3731715510",
0x7a6b1303,"MainField_Enemy_Wizzrobe_Ice_3835920240",
0x5dc4ebd8,"MainField_Enemy_Wizzrobe_Ice_4145234022",
0xd4c35af5,"MainField_Enemy_Wizzrobe_Ice_4269998213",
0xc79d3c8d,"MainField_Enemy_Wizzrobe_Ice_586069873",
0xc0992d67,"MainField_Enemy_Wizzrobe_Ice_642600740",
0x060bf269,"MainField_Enemy_Wizzrobe_Ice_Senior_1443478984",
0xc9a6e259,"MainField_Enemy_Wizzrobe_Ice_Senior_1691903847",
0x07ada463,"MainField_Enemy_Wizzrobe_Ice_Senior_1983688223",
0x821930ec,"MainField_Enemy_Wizzrobe_Ice_Senior_2536961314",
0x3ef829ea,"MainField_Enemy_Wizzrobe_Ice_Senior_3452986497",
0x6f2eb7c5,"MainField_Enemy_Wizzrobe_Ice_Senior_3650235623",
0x13774678,"MainField_Enemy_Wizzrobe_Ice_Senior_3928287106",
0x57f6954d,"MainField_Enemy_Wizzrobe_Ice_Senior_410988243",
0xe7edc2b1,"MainField_Enemy_Wizzrobe_Ice_Senior_4165273127",
0x32a25da5,"MainField_Enemy_Wizzrobe_Ice_Senior_913031101",
0x74d363bb,"MainField_Enemy_Wizzrobe_Ice_Senior_920512733",
0xacf80ab9,"MainField_FldObj_AssassinPicture_A_01_1169172528",
0xbb7d8129,"MainField_FldObj_AssassinPicture_A_01_118162960",
0x7cb904fd,"MainField_FldObj_AssassinPicture_A_01_1853539294",
0x5720ba8b,"MainField_FldObj_AssassinPicture_A_01_2002907807",
0x486f7ddd,"MainField_FldObj_AssassinPicture_A_01_2766534543",
0x974259f0,"MainField_FldObj_AssassinPicture_A_01_504379217",
0xcccc4ec2,"MainField_FldObj_AssassinPicture_A_02_1973801801",
0xb88ca75c,"MainField_FldObj_AssassinPicture_A_02_3311960892",
0xc18ec0be,"MainField_FldObj_AssassinPicture_A_02_3562307133",
0xfba37b02,"MainField_FldObj_BreakableRockDeathMT_A_01_895200412",
0x6567d2c5,"MainField_FldObj_BreakableRockDeathMT_A_02_3491196879",
0xd9211b28,"MainField_FldObj_BreakableRockDeathMT_A_02_781087825",
0xb8b2f5a3,"MainField_FldObj_DownloadTerminal_A_01_1285797230",
0x3beeebf9,"MainField_FldObj_DownloadTerminal_A_01_1475180658",
0xe583708d,"MainField_FldObj_DownloadTerminal_A_01_1620889332",
0x0bd12257,"MainField_FldObj_DownloadTerminal_A_01_1669567698",
0xc9f6e36c,"MainField_FldObj_DownloadTerminal_A_01_1943274718",
0x1e77c904,"MainField_FldObj_DownloadTerminal_A_01_219123603",
0x74670519,"MainField_FldObj_DownloadTerminal_A_01_2275687969",
0xe2d863ed,"MainField_FldObj_DownloadTerminal_A_01_2733553602",
0x43716d20,"MainField_FldObj_DownloadTerminal_A_01_2750385337",
0xdc82c2be,"MainField_FldObj_DownloadTerminal_A_01_2751434839",
0x7445a13c,"MainField_FldObj_DownloadTerminal_A_01_2879079867",
0xb667bea9,"MainField_FldObj_DownloadTerminal_A_01_2934598095",
0x51d81fcf,"MainField_FldObj_DownloadTerminal_A_01_3650282708",
0x101821ea,"MainField_FldObj_DownloadTerminal_A_01_3703935795",
0x4fbfcaea,"MainField_FldObj_DownloadTerminal_A_01_4042440915",
0xe2df3083,"MainField_FldObj_DownloadTerminal_A_01_4111125397",
0x94ca077c,"MainField_FldObj_DownloadTerminal_A_01_4138911080",
0x6421606d,"MainField_FldObj_KorokStoneLift_A_01_1132362805",
0xef8a4ce7,"MainField_FldObj_KorokStoneLift_A_01_1196871690",
0x4230d0f2,"MainField_FldObj_KorokStoneLift_A_01_1231138937",
0xcd96226a,"MainField_FldObj_KorokStoneLift_A_01_1247602248",
0xada98379,"MainField_FldObj_KorokStoneLift_A_01_1262969737",
0xed6f9c84,"MainField_FldObj_KorokStoneLift_A_01_1355707812",
0x4cefac97,"MainField_FldObj_KorokStoneLift_A_01_1409011633",
0x6e921934,"MainField_FldObj_KorokStoneLift_A_01_1410024381",
0x70c01fd3,"MainField_FldObj_KorokStoneLift_A_01_1493566170",
0xb0462e63,"MainField_FldObj_KorokStoneLift_A_01_1551371630",
0x468f6020,"MainField_FldObj_KorokStoneLift_A_01_1560778810",
0x66018a3e,"MainField_FldObj_KorokStoneLift_A_01_1587096291",
0x749e8f4e,"MainField_FldObj_KorokStoneLift_A_01_159818902",
0x64f98127,"MainField_FldObj_KorokStoneLift_A_01_1677815646",
0xfdbf8728,"MainField_FldObj_KorokStoneLift_A_01_1917639855",
0xbb5a3cc5,"MainField_FldObj_KorokStoneLift_A_01_1964888142",
0x19ccda08,"MainField_FldObj_KorokStoneLift_A_01_2141715240",
0x7ea3884c,"MainField_FldObj_KorokStoneLift_A_01_217546152",
0xc3d3c118,"MainField_FldObj_KorokStoneLift_A_01_2223800730",
0x91bd0d74,"MainField_FldObj_KorokStoneLift_A_01_2252592684",
0x212ab08c,"MainField_FldObj_KorokStoneLift_A_01_2263100111",
0x7731d8b6,"MainField_FldObj_KorokStoneLift_A_01_2264271005",
0xdbda2254,"MainField_FldObj_KorokStoneLift_A_01_2304200400",
0x5997f348,"MainField_FldObj_KorokStoneLift_A_01_2306561475",
0x6f1a85ea,"MainField_FldObj_KorokStoneLift_A_01_2318570641",
0xc59d1e8a,"MainField_FldObj_KorokStoneLift_A_01_2341327453",
0x40509193,"MainField_FldObj_KorokStoneLift_A_01_2427455691",
0xc8b60535,"MainField_FldObj_KorokStoneLift_A_01_2520736057",
0x43d1878b,"MainField_FldObj_KorokStoneLift_A_01_2530803038",
0x53ba1d6b,"MainField_FldObj_KorokStoneLift_A_01_2621988921",
0xda10c3df,"MainField_FldObj_KorokStoneLift_A_01_2666452991",
0x21e71b49,"MainField_FldObj_KorokStoneLift_A_01_268836092",
0xdcc6b2b5,"MainField_FldObj_KorokStoneLift_A_01_2698266071",
0x9b88156e,"MainField_FldObj_KorokStoneLift_A_01_2767528070",
0x0c7ad501,"MainField_FldObj_KorokStoneLift_A_01_2773224962",
0x5eaf2a86,"MainField_FldObj_KorokStoneLift_A_01_2807537516",
0xb4d15a3a,"MainField_FldObj_KorokStoneLift_A_01_2891997939",
0x1f57adad,"MainField_FldObj_KorokStoneLift_A_01_2959803667",
0x6b245230,"MainField_FldObj_KorokStoneLift_A_01_3047671678",
0xf8cbb19a,"MainField_FldObj_KorokStoneLift_A_01_3065202071",
0xb224a8c6,"MainField_FldObj_KorokStoneLift_A_01_3124075661",
0xc455c714,"MainField_FldObj_KorokStoneLift_A_01_3345757573",
0x99565411,"MainField_FldObj_KorokStoneLift_A_01_3352205633",
0xbf4691d1,"MainField_FldObj_KorokStoneLift_A_01_336686057",
0x987037f1,"MainField_FldObj_KorokStoneLift_A_01_3466794825",
0xe2968893,"MainField_FldObj_KorokStoneLift_A_01_3543257862",
0x3298f608,"MainField_FldObj_KorokStoneLift_A_01_355487715",
0xa17392d5,"MainField_FldObj_KorokStoneLift_A_01_3640274552",
0xbfc09ec3,"MainField_FldObj_KorokStoneLift_A_01_3650611848",
0xf444de09,"MainField_FldObj_KorokStoneLift_A_01_3655219538",
0x3bf0ef7f,"MainField_FldObj_KorokStoneLift_A_01_3659382775",
0x6b29af73,"MainField_FldObj_KorokStoneLift_A_01_3661361209",
0xac0bef3d,"MainField_FldObj_KorokStoneLift_A_01_3740423155",
0xf7dfc962,"MainField_FldObj_KorokStoneLift_A_01_3775879431",
0xab6ec900,"MainField_FldObj_KorokStoneLift_A_01_388502532",
0x5c26acca,"MainField_FldObj_KorokStoneLift_A_01_3949125373",
0x39aa997a,"MainField_FldObj_KorokStoneLift_A_01_402171009",
0xd7022681,"MainField_FldObj_KorokStoneLift_A_01_4059117579",
0xaae9a0b8,"MainField_FldObj_KorokStoneLift_A_01_4088055457",
0x84314ad7,"MainField_FldObj_KorokStoneLift_A_01_4107026068",
0x1e60b277,"MainField_FldObj_KorokStoneLift_A_01_4125776688",
0x70135858,"MainField_FldObj_KorokStoneLift_A_01_4136122984",
0xb4cd23ca,"MainField_FldObj_KorokStoneLift_A_01_4156018869",
0x5dd15fa2,"MainField_FldObj_KorokStoneLift_A_01_4272687388",
0xa2ddaba5,"MainField_FldObj_KorokStoneLift_A_01_839186256",
0x1423ea33,"MainField_FldObj_KorokStoneLift_A_01_88861753",
0x5794d8f9,"MainField_FldObj_RuinGuardianSand_A_Dynamic_3152966927",
0x5bc1321c,"MainField_FldObj_RuinGuardianSand_A_Dynamic_3437839769",
0xd420b5f5,"MainField_FldObj_RuinGuardianSand_A_Dynamic_585573557",
0x32d4c6e2,"MainField_FldObj_RuinGuardianSnow_A_Dynamic_12245108",
0x171955a1,"MainField_FldObj_RuinGuardianSnow_A_Dynamic_4120486809",
0x135f34e6,"MainField_FldObj_RuinGuardian_A_Dynamic_1036349837",
0xbb19c995,"MainField_FldObj_RuinGuardian_A_Dynamic_1195583873",
0x71537f6f,"MainField_FldObj_RuinGuardian_A_Dynamic_1205813713",
0xa39d3c65,"MainField_FldObj_RuinGuardian_A_Dynamic_123879562",
0x9a46423c,"MainField_FldObj_RuinGuardian_A_Dynamic_1476624088",
0xc9a89804,"MainField_FldObj_RuinGuardian_A_Dynamic_1635820262",
0xce21b338,"MainField_FldObj_RuinGuardian_A_Dynamic_1645611601",
0xffbeb2ba,"MainField_FldObj_RuinGuardian_A_Dynamic_1656045961",
0x4305aa7d,"MainField_FldObj_RuinGuardian_A_Dynamic_1738918255",
0xf3602faa,"MainField_FldObj_RuinGuardian_A_Dynamic_1805637131",
0x324ec26a,"MainField_FldObj_RuinGuardian_A_Dynamic_1827803666",
0x3826618f,"MainField_FldObj_RuinGuardian_A_Dynamic_189176748",
0x353bced2,"MainField_FldObj_RuinGuardian_A_Dynamic_1921312586",
0x0af89dac,"MainField_FldObj_RuinGuardian_A_Dynamic_1979180817",
0x52b34def,"MainField_FldObj_RuinGuardian_A_Dynamic_2084678458",
0xd9c1def1,"MainField_FldObj_RuinGuardian_A_Dynamic_2197582654",
0x9b7323fb,"MainField_FldObj_RuinGuardian_A_Dynamic_2339745636",
0x18e4dd5a,"MainField_FldObj_RuinGuardian_A_Dynamic_2344005636",
0x953cd22b,"MainField_FldObj_RuinGuardian_A_Dynamic_2358074393",
0x1dc82479,"MainField_FldObj_RuinGuardian_A_Dynamic_2363192349",
0xecc5bba2,"MainField_FldObj_RuinGuardian_A_Dynamic_2454702614",
0x7cc64206,"MainField_FldObj_RuinGuardian_A_Dynamic_2547866907",
0x0c83f725,"MainField_FldObj_RuinGuardian_A_Dynamic_2615624319",
0x3e2e775b,"MainField_FldObj_RuinGuardian_A_Dynamic_2764841015",
0x83a8241f,"MainField_FldObj_RuinGuardian_A_Dynamic_2914564806",
0x26c35628,"MainField_FldObj_RuinGuardian_A_Dynamic_3188678092",
0x3745da51,"MainField_FldObj_RuinGuardian_A_Dynamic_3231769907",
0xa2a85f3a,"MainField_FldObj_RuinGuardian_A_Dynamic_3303640232",
0xcbcfad4b,"MainField_FldObj_RuinGuardian_A_Dynamic_3445604483",
0xec1ef65e,"MainField_FldObj_RuinGuardian_A_Dynamic_353450695",
0x364f5cf6,"MainField_FldObj_RuinGuardian_A_Dynamic_3624218655",
0xe0a3a388,"MainField_FldObj_RuinGuardian_A_Dynamic_3669985872",
0x7a64740e,"MainField_FldObj_RuinGuardian_A_Dynamic_3772411277",
0x58ec5b0b,"MainField_FldObj_RuinGuardian_A_Dynamic_3833618375",
0xf5fe183e,"MainField_FldObj_RuinGuardian_A_Dynamic_3852661471",
0xd71e24b6,"MainField_FldObj_RuinGuardian_A_Dynamic_4069951409",
0xae06b8cc,"MainField_FldObj_RuinGuardian_A_Dynamic_422655439",
0xa2f781ee,"MainField_FldObj_RuinGuardian_A_Dynamic_4239491218",
0x45f5b8fa,"MainField_FldObj_RuinGuardian_A_Dynamic_584617204",
0x657d2857,"MainField_FldObj_RuinGuardian_A_Dynamic_700661766",
0xe28fd716,"MainField_FldObj_RuinGuardian_A_Dynamic_983186230",
0x3cbe574c,"MainField_FldObj_RuinGuardian_A_Dynamic_984135060",
0xc2aa7d7e,"MainField_FldObj_Terminal_A_01_2355675146",
0x7895f786,"MainField_IceWallRuins_1531469017",
0x8747af32,"MainField_IceWallRuins_2367630524",
0x58f17bb7,"MainField_IceWallRuins_3290808737",
0x2d268bfb,"MainField_IceWallRuins_3363809946",
0x7031631c,"MainField_IceWallRuins_3794545852",
0x5d6c03b1,"MainField_IceWallRuins_4192006947",
0x1fb93dbd,"MainField_IceWallRuins_918148586",
0x4eec31f2,"MainField_IceWallRuins_Challenge_3350870193",
0x329c7013,"MainField_IceWall_115762957",
0x564a79e3,"MainField_IceWall_1187283770",
0x593c17d0,"MainField_IceWall_126851703",
0x483fafe5,"MainField_IceWall_1428445097",
0xb3ee279f,"MainField_IceWall_1438004723",
0xb5ceb8ad,"MainField_IceWall_1552005719",
0x1c346f71,"MainField_IceWall_1557027170",
0xd8db62cd,"MainField_IceWall_1564299102",
0xa632cc4f,"MainField_IceWall_1565981291",
0xc9cd30da,"MainField_IceWall_156984559",
0x425bedb8,"MainField_IceWall_1798096560",
0x2fef61f1,"MainField_IceWall_1823144138",
0x8ad081c3,"MainField_IceWall_1837616088",
0x55aa979e,"MainField_IceWall_1842553700",
0x8133bbdc,"MainField_IceWall_188175583",
0x95aa9c59,"MainField_IceWall_1930177061",
0xcba09ebc,"MainField_IceWall_1932805577",
0x9d699731,"MainField_IceWall_1946573764",
0x4d39d0eb,"MainField_IceWall_2103849752",
0xabbdefc2,"MainField_IceWall_2142802401",
0xf6e67bcd,"MainField_IceWall_2153495172",
0x9798aa44,"MainField_IceWall_221429972",
0x86372a87,"MainField_IceWall_2216089390",
0xa11b7bb4,"MainField_IceWall_2244903760",
0x0172842e,"MainField_IceWall_2353950179",
0x369bfcf0,"MainField_IceWall_2390400718",
0xcd08b985,"MainField_IceWall_2403090024",
0x989529a7,"MainField_IceWall_2461033248",
0xa399346e,"MainField_IceWall_2480150733",
0xd62dbf6d,"MainField_IceWall_256876980",
0x1e731b73,"MainField_IceWall_2583863735",
0x291b0357,"MainField_IceWall_2663208266",
0x810af298,"MainField_IceWall_2688426629",
0x1f404638,"MainField_IceWall_2724495312",
0x5ad68419,"MainField_IceWall_2756084779",
0x8ecb8694,"MainField_IceWall_2769868117",
0x52cd0c90,"MainField_IceWall_2836264177",
0x792a3796,"MainField_IceWall_2844970562",
0x3d4713a9,"MainField_IceWall_2908385766",
0xc5665e34,"MainField_IceWall_2921436202",
0xa0a88677,"MainField_IceWall_2932532875",
0x23c63482,"MainField_IceWall_2974237509",
0x30611570,"MainField_IceWall_2994618336",
0xf7e740f2,"MainField_IceWall_3022886127",
0x8b04eb4c,"MainField_IceWall_3037243710",
0x2855daee,"MainField_IceWall_308807292",
0xe28b6f6a,"MainField_IceWall_3118431514",
0x98ff3d6e,"MainField_IceWall_3154155501",
0xf1edd970,"MainField_IceWall_3219928511",
0x7c348b91,"MainField_IceWall_3257035994",
0x93a0e279,"MainField_IceWall_3356803804",
0x189fbd2f,"MainField_IceWall_3397216894",
0x9fb458ea,"MainField_IceWall_3421601596",
0xd4a416f8,"MainField_IceWall_3426617891",
0xf99115fe,"MainField_IceWall_3504221585",
0x4300e6fc,"MainField_IceWall_3606505025",
0x0c10924b,"MainField_IceWall_3610841619",
0xc269983a,"MainField_IceWall_3629920793",
0xa034d4ad,"MainField_IceWall_3868708178",
0xaa874359,"MainField_IceWall_3881719474",
0xcceff631,"MainField_IceWall_4003572086",
0x63896594,"MainField_IceWall_4012653576",
0x4855578c,"MainField_IceWall_4019322377",
0xf7148a21,"MainField_IceWall_4052238205",
0x08ae2fb6,"MainField_IceWall_4150037010",
0x4d880f9a,"MainField_IceWall_4160163331",
0xed9bb27c,"MainField_IceWall_4293850939",
0x7783f8b1,"MainField_IceWall_502087511",
0x4bc706e6,"MainField_IceWall_620730097",
0x3f169a49,"MainField_IceWall_659813771",
0x6043b7eb,"MainField_IceWall_661122259",
0x2db33565,"MainField_IceWall_731978568",
0x2d81157b,"MainField_IceWall_741706292",
0x360d8664,"MainField_IceWall_761352566",
0x87ecfb5a,"MainField_IceWall_798324794",
0xe93d2387,"MainField_IceWall_811031627",
0x420ef060,"MainField_IceWall_825057773",
0x72b6d646,"MainField_IceWall_852319889",
0xc61f44ca,"MainField_IceWall_Tower_1043548872",
0x837b467a,"MainField_IceWall_Tower_1135985373",
0x32343fd5,"MainField_IceWall_Tower_1270892357",
0xaff60935,"MainField_IceWall_Tower_1285690644",
0x9a8eef9a,"MainField_IceWall_Tower_1403429490",
0xc2668223,"MainField_IceWall_Tower_1455048804",
0x7040199c,"MainField_IceWall_Tower_1797559652",
0xa6732c6c,"MainField_IceWall_Tower_2431318047",
0x1bbb209f,"MainField_IceWall_Tower_2457652469",
0x34a3d116,"MainField_IceWall_Tower_2514653214",
0xccc0fe7f,"MainField_IceWall_Tower_2559456054",
0xc7a29350,"MainField_IceWall_Tower_302611435",
0xe84759a4,"MainField_IceWall_Tower_3081835581",
0x77f31a47,"MainField_IceWall_Tower_3263074030",
0x7838d925,"MainField_IceWall_Tower_3637872308",
0x9ba60e63,"MainField_IceWall_Tower_4007131320",
0xe38fe5fe,"MainField_IceWall_Tower_446717717",
0xac94ad1d,"MainField_IceWall_Tower_861831559",
0x52cb44f3,"MainField_IceWall_Tower_881376306",
0xc87c2c54,"MainField_IceWall_Tower_97393643",
0x3aefa053,"MainField_IsDefeat_Enemy_Giant_Bone_1287559097",
0x805cf1f2,"MainField_IsDefeat_Enemy_Giant_Bone_1698864303",
0xd815d3e0,"MainField_IsDefeat_Enemy_Giant_Bone_1721655973",
0x1fb33ad5,"MainField_IsDefeat_Enemy_Giant_Bone_2519547405",
0x04a74790,"MainField_IsDefeat_Enemy_Giant_Bone_2815854988",
0xee812327,"MainField_IsDefeat_Enemy_Giant_Bone_2974080467",
0x03415ecc,"MainField_IsDefeat_Enemy_Giant_Bone_3201775889",
0xb2d689b6,"MainField_IsDefeat_Enemy_Giant_Bone_3700264996",
0x7259e3c5,"MainField_IsDefeat_Enemy_Giant_Bone_4237208898",
0x051ed62c,"MainField_IsDefeat_Enemy_Giant_Bone_AllDay_73882430",
0xd7223944,"MainField_IsDefeat_Enemy_Giant_Junior_1602515170",
0x8d865b78,"MainField_IsDefeat_Enemy_Giant_Junior_1833514674",
0xa4589fc1,"MainField_IsDefeat_Enemy_Giant_Junior_1903632265",
0x2524aa37,"MainField_IsDefeat_Enemy_Giant_Junior_3337886442",
0x3e847dde,"MainField_IsDefeat_Enemy_Giant_Junior_3350013125",
0x67d1ae47,"MainField_IsDefeat_Enemy_Giant_Junior_373348007",
0x97ceb661,"MainField_IsDefeat_Enemy_Giant_Junior_4099815924",
0x59dd8037,"MainField_IsDefeat_Enemy_Giant_Middle_1266282060",
0x43756237,"MainField_IsDefeat_Enemy_Giant_Middle_1384742674",
0xcde51b3f,"MainField_IsDefeat_Enemy_Giant_Middle_1925665429",
0x382acf32,"MainField_IsDefeat_Enemy_Giant_Middle_1949698080",
0x48e5f7a0,"MainField_IsDefeat_Enemy_Giant_Middle_2083136285",
0x4f42ed0c,"MainField_IsDefeat_Enemy_Giant_Middle_2472409077",
0xbbb6a93d,"MainField_IsDefeat_Enemy_Giant_Middle_2530315670",
0x036d6e47,"MainField_IsDefeat_Enemy_Giant_Middle_3272527560",
0xef37a9be,"MainField_IsDefeat_Enemy_Giant_Middle_3572226454",
0xacd67339,"MainField_IsDefeat_Enemy_Giant_Middle_3627901807",
0xa27c9e39,"MainField_IsDefeat_Enemy_Giant_Middle_3637193649",
0x43b9424b,"MainField_IsDefeat_Enemy_Giant_Middle_3706280648",
0x2880a2b8,"MainField_IsDefeat_Enemy_Giant_Middle_3744176720",
0xb9f89458,"MainField_IsDefeat_Enemy_Giant_Middle_3831867213",
0x0862ac9c,"MainField_IsDefeat_Enemy_Giant_Middle_4080554254",
0x4b9dc25a,"MainField_IsDefeat_Enemy_Giant_Middle_936329231",
0xe8d22126,"MainField_IsDefeat_Enemy_Giant_Senior_1028264440",
0x7096ce54,"MainField_IsDefeat_Enemy_Giant_Senior_2009832062",
0x1d753114,"MainField_IsDefeat_Enemy_Giant_Senior_2853871544",
0x6c0b0cd8,"MainField_IsDefeat_Enemy_Giant_Senior_3410381997",
0xe9600685,"MainField_IsDefeat_Enemy_Giant_Senior_3741497810",
0xd31c00e1,"MainField_IsDefeat_Enemy_Giant_Senior_3785639248",
0x3e012222,"MainField_IsDefeat_Enemy_Giant_Senior_851170738",
0x740a06af,"MainField_IsDefeat_Enemy_Golem_Fire_1372356658",
0xe7898ded,"MainField_IsDefeat_Enemy_Golem_Fire_2080410037",
0xce9d58d6,"MainField_IsDefeat_Enemy_Golem_Fire_2112489205",
0xce449f1c,"MainField_IsDefeat_Enemy_Golem_Fire_3180486693",
0x2ac739b3,"MainField_IsDefeat_Enemy_Golem_Fire_4143112243",
0x4fe566ae,"MainField_IsDefeat_Enemy_Golem_Ice_1684711544",
0x1eff6ae3,"MainField_IsDefeat_Enemy_Golem_Ice_189892196",
0x2bda4f24,"MainField_IsDefeat_Enemy_Golem_Ice_4007690024",
0x827917a9,"MainField_IsDefeat_Enemy_Golem_Ice_4087740554",
0xd4144bdf,"MainField_IsDefeat_Enemy_Golem_Ice_648092042",
0x0e72d42d,"MainField_IsDefeat_Enemy_Golem_Junior_139564765",
0x977c89d8,"MainField_IsDefeat_Enemy_Golem_Junior_1450825465",
0x47633bd7,"MainField_IsDefeat_Enemy_Golem_Junior_2009332164",
0x7e7cba50,"MainField_IsDefeat_Enemy_Golem_Junior_3610208070",
0xf9921240,"MainField_IsDefeat_Enemy_Golem_Junior_3688513041",
0x0d0ba40c,"MainField_IsDefeat_Enemy_Golem_Junior_3707687090",
0xa35cfe58,"MainField_IsDefeat_Enemy_Golem_Junior_3817875409",
0x45920271,"MainField_IsDefeat_Enemy_Golem_Junior_3873221297",
0x602fdbce,"MainField_IsDefeat_Enemy_Golem_Junior_4097205930",
0xc785de56,"MainField_IsDefeat_Enemy_Golem_Junior_4234397911",
0x593975d6,"MainField_IsDefeat_Enemy_Golem_Junior_481698249",
0x8c05c385,"MainField_IsDefeat_Enemy_Golem_Middle_1063952470",
0x18fa31b0,"MainField_IsDefeat_Enemy_Golem_Middle_2367737901",
0xc2e87e02,"MainField_IsDefeat_Enemy_Golem_Middle_2827830463",
0x0164fe15,"MainField_IsDefeat_Enemy_Golem_Middle_3273877672",
0x9efc3339,"MainField_IsDefeat_Enemy_Golem_Middle_3529818719",
0x162d11f4,"MainField_IsDefeat_Enemy_Golem_Middle_3605751469",
0xec80ecfa,"MainField_IsDefeat_Enemy_Golem_Middle_372172601",
0xa4ae04d3,"MainField_IsDefeat_Enemy_Golem_Middle_3812413946",
0xf897773d,"MainField_IsDefeat_Enemy_Golem_Middle_3900382371",
0xe3677e8a,"MainField_IsDefeat_Enemy_Golem_Middle_491623064",
0xe43a5cec,"MainField_IsDefeat_Enemy_Golem_Middle_853217538",
0xccc79db5,"MainField_IsDefeat_Enemy_Golem_Middle_975119268",
0x5c4ff2a8,"MainField_IsDefeat_Enemy_Golem_Senior_1943962273",
0x187ccce5,"MainField_IsDefeat_Enemy_Golem_Senior_2455625785",
0x5d6ad526,"MainField_IsDefeat_Enemy_Golem_Senior_2530932938",
0xf05443e4,"MainField_IsDefeat_Enemy_Golem_Senior_2763213354",
0xcc9ba064,"MainField_IsDefeat_Enemy_Golem_Senior_321907823",
0xfa12fcda,"MainField_IsDefeat_Enemy_Golem_Senior_3869714669",
0x0b7267c0,"MainField_IsDefeat_Enemy_Golem_Senior_464316212",
0x878420a2,"MainField_IsDefeat_Enemy_Sandworm_3098676163",
0xb5c0ad83,"MainField_IsDefeat_Enemy_Sandworm_3538536935",
0x6070b591,"MainField_IsDefeat_Enemy_Sandworm_3569739054",
0xec88c5a2,"MainField_IsDefeat_Enemy_Sandworm_494020159",
0xba5b3e75,"MainField_Item_Boiled_01_1107263928",
0x1c1ec472,"MainField_Item_Boiled_01_3474318978",
0x238c3a44,"MainField_Item_ChilledFish_05_1684124930",
0x1d4e517e,"MainField_Item_ChilledFish_05_2878153904",
0xb6fcefbb,"MainField_Item_ChilledFish_06_1477739414",
0x328387d6,"MainField_Item_Chilled_02_2863424942",
0x42d0b261,"MainField_Item_Chilled_02_3651114298",
0xfaa4a3d3,"MainField_Item_Chilled_03_720004164",
0x6e9bd7c6,"MainField_Item_Chilled_05_3024383950",
0xbee54341,"MainField_Item_Chilled_06_304434283",
0x3caaf378,"MainField_Item_Enemy_02_2298659153",
0xe8a33132,"MainField_Item_Enemy_02_4261802439",
0xa7752675,"MainField_Item_Enemy_02_431873216",
0xaa1ea085,"MainField_Item_Enemy_05_1899376127",
0xaecf7480,"MainField_Item_Enemy_16_233156231",
0x6c0d0214,"MainField_Item_Enemy_16_3477281033",
0xc4cc37ca,"MainField_Item_Enemy_24_1288442718",
0x27c4b655,"MainField_Item_Enemy_26_74247761",
0x9dafc63a,"MainField_Item_Enemy_57_3877416466",
0xacad158e,"MainField_Item_Enemy_Put_57_112779393",
0xe7a46cd4,"MainField_Item_Enemy_Put_57_1482851130",
0x59eebc42,"MainField_Item_Enemy_Put_57_284363524",
0x9dd14dbb,"MainField_Item_Enemy_Put_57_316323603",
0xe94beb71,"MainField_Item_Enemy_Put_57_812143535",
0xaa00a2c7,"MainField_Item_FishGet_A_2069502987",
0x96727bc7,"MainField_Item_FishGet_A_2633410495",
0x2bf02684,"MainField_Item_FishGet_A_3917228584",
0xc7227f1e,"MainField_Item_FishGet_C_1312070797",
0xf286bbc9,"MainField_Item_FishGet_C_507147447",
0xcfe97a11,"MainField_Item_FishGet_E_96601385",
0x98cc85c9,"MainField_Item_FishGet_F_2014295643",
0x168390a2,"MainField_Item_FishGet_F_4048413620",
0x2c55f7ff,"MainField_Item_FishGet_F_852333563",
0x3b06725e,"MainField_Item_FishGet_G_2653214550",
0x9890656c,"MainField_Item_FishGet_G_715018408",
0x13452333,"MainField_Item_FishGet_G_904216046",
0x99d0587c,"MainField_Item_FishGet_J_2337659372",
0xe4adf267,"MainField_Item_FishGet_J_2623284375",
0x06acfa18,"MainField_Item_Fruit_A_1002311999",
0xcd7bb272,"MainField_Item_Fruit_A_1003363290",
0x78d88b86,"MainField_Item_Fruit_A_1007816709",
0x92c49673,"MainField_Item_Fruit_A_1013849175",
0xc71396fb,"MainField_Item_Fruit_A_1013942405",
0xa09240c4,"MainField_Item_Fruit_A_101448776",
0x13bf41ee,"MainField_Item_Fruit_A_101590353",
0xe4a62b52,"MainField_Item_Fruit_A_1021771261",
0x7804c93f,"MainField_Item_Fruit_A_1023422705",
0x740631dd,"MainField_Item_Fruit_A_1030634494",
0x598422c7,"MainField_Item_Fruit_A_1031411135",
0x4f39932b,"MainField_Item_Fruit_A_1033337003",
0xc6eea5cf,"MainField_Item_Fruit_A_1034600883",
0x508c60bf,"MainField_Item_Fruit_A_1049768361",
0x489b2cf7,"MainField_Item_Fruit_A_1050549202",
0xa999e9af,"MainField_Item_Fruit_A_1056711234",
0xd14b5cf6,"MainField_Item_Fruit_A_1061240275",
0xc13beb7b,"MainField_Item_Fruit_A_1065795725",
0x90b7a848,"MainField_Item_Fruit_A_1067487252",
0xa3a704e7,"MainField_Item_Fruit_A_1070169672",
0x2ceb1450,"MainField_Item_Fruit_A_1072448630",
0x994d8a33,"MainField_Item_Fruit_A_1074787157",
0x88625295,"MainField_Item_Fruit_A_1077711853",
0xbd665110,"MainField_Item_Fruit_A_108452009",
0xbd393e92,"MainField_Item_Fruit_A_1085544765",
0x49e9fe3a,"MainField_Item_Fruit_A_1093058231",
0xa4d784db,"MainField_Item_Fruit_A_1093900103",
0xf812308d,"MainField_Item_Fruit_A_1096221571",
0x819ccedf,"MainField_Item_Fruit_A_1110492364",
0x7d4c692f,"MainField_Item_Fruit_A_1113762203",
0xc775205e,"MainField_Item_Fruit_A_1115661203",
0xced6c817,"MainField_Item_Fruit_A_1117358819",
0x3a2796f4,"MainField_Item_Fruit_A_1117499443",
0x0742ebf3,"MainField_Item_Fruit_A_1120831605",
0xa779a5f8,"MainField_Item_Fruit_A_1123840506",
0xea077ebf,"MainField_Item_Fruit_A_1123848815",
0xd8c46ef4,"MainField_Item_Fruit_A_1130897508",
0xe47501c8,"MainField_Item_Fruit_A_1133862189",
0xc772a375,"MainField_Item_Fruit_A_1137441750",
0x300ca4e2,"MainField_Item_Fruit_A_1138457119",
0x86ec74de,"MainField_Item_Fruit_A_1139824091",
0xe981ed6e,"MainField_Item_Fruit_A_1140766743",
0xa67e0e75,"MainField_Item_Fruit_A_1143677484",
0xa7778919,"MainField_Item_Fruit_A_1145832687",
0x8abdb752,"MainField_Item_Fruit_A_1146347476",
0x2623605f,"MainField_Item_Fruit_A_1151522840",
0xb4cca093,"MainField_Item_Fruit_A_1151745999",
0x43c62f30,"MainField_Item_Fruit_A_1158638978",
0x1bb7e031,"MainField_Item_Fruit_A_1159611962",
0x14d687f8,"MainField_Item_Fruit_A_1161557931",
0x4c7847ef,"MainField_Item_Fruit_A_1162793920",
0x52e7ec20,"MainField_Item_Fruit_A_1169258219",
0x13dfd06a,"MainField_Item_Fruit_A_1173779498",
0xa1bb886a,"MainField_Item_Fruit_A_1175759850",
0x6874fd22,"MainField_Item_Fruit_A_1178404815",
0xea846711,"MainField_Item_Fruit_A_1179039448",
0xa3a62eaf,"MainField_Item_Fruit_A_1181679202",
0x69f10e0f,"MainField_Item_Fruit_A_1182542000",
0xc32f695b,"MainField_Item_Fruit_A_1188644799",
0xcaca8f39,"MainField_Item_Fruit_A_1189248390",
0xa20f3a98,"MainField_Item_Fruit_A_1196414273",
0x5c20396b,"MainField_Item_Fruit_A_1197619020",
0x074b79f9,"MainField_Item_Fruit_A_1200839932",
0x15783a3a,"MainField_Item_Fruit_A_120220415",
0x39413453,"MainField_Item_Fruit_A_1211726051",
0x7a8ed406,"MainField_Item_Fruit_A_1218945154",
0x212150a2,"MainField_Item_Fruit_A_1223390432",
0x6ac0ef9e,"MainField_Item_Fruit_A_122344529",
0x478b87d8,"MainField_Item_Fruit_A_1226223068",
0xf750a935,"MainField_Item_Fruit_A_1232590627",
0xbb6b0bc2,"MainField_Item_Fruit_A_1238456711",
0xa335c98a,"MainField_Item_Fruit_A_1250969661",
0x205ea4ff,"MainField_Item_Fruit_A_1261767935",
0x9f13b311,"MainField_Item_Fruit_A_1267862170",
0xa3e93188,"MainField_Item_Fruit_A_1270357911",
0x39ec7193,"MainField_Item_Fruit_A_1273089387",
0xf917cc45,"MainField_Item_Fruit_A_127383394",
0x51b30fc4,"MainField_Item_Fruit_A_1275301430",
0x7b12037c,"MainField_Item_Fruit_A_1277654175",
0x0533af20,"MainField_Item_Fruit_A_1279277057",
0x8cfb09fc,"MainField_Item_Fruit_A_1285339399",
0xb17da43f,"MainField_Item_Fruit_A_1287201193",
0x57c518d4,"MainField_Item_Fruit_A_1291907629",
0xd7182396,"MainField_Item_Fruit_A_1307560488",
0x3c303c6c,"MainField_Item_Fruit_A_1314942005",
0x7107e785,"MainField_Item_Fruit_A_1315426561",
0xd9739f25,"MainField_Item_Fruit_A_1317054956",
0xf1cf43a0,"MainField_Item_Fruit_A_1317863713",
0xd0c83646,"MainField_Item_Fruit_A_1318508237",
0xfb3e584b,"MainField_Item_Fruit_A_1320983966",
0xdeabe785,"MainField_Item_Fruit_A_1324231477",
0x4d73ed59,"MainField_Item_Fruit_A_1330063687",
0x2e7eb6ab,"MainField_Item_Fruit_A_1333129487",
0xc1e7905c,"MainField_Item_Fruit_A_1338664091",
0xc3184942,"MainField_Item_Fruit_A_1345535575",
0xa6038ca6,"MainField_Item_Fruit_A_1352420056",
0x686bce2f,"MainField_Item_Fruit_A_1354807323",
0xf45a84ab,"MainField_Item_Fruit_A_1360390689",
0x06fd7260,"MainField_Item_Fruit_A_1362723169",
0x1642d28d,"MainField_Item_Fruit_A_1363484584",
0x44f3d934,"MainField_Item_Fruit_A_1363891906",
0x209b58ea,"MainField_Item_Fruit_A_1369613513",
0xc13a62c0,"MainField_Item_Fruit_A_1374700961",
0x8bd99d2b,"MainField_Item_Fruit_A_1375721404",
0x934dcb71,"MainField_Item_Fruit_A_1379256195",
0x5f650ba6,"MainField_Item_Fruit_A_1379984360",
0x35370809,"MainField_Item_Fruit_A_1381388092",
0x52399fc0,"MainField_Item_Fruit_A_1383121433",
0xabe92dcd,"MainField_Item_Fruit_A_1383913370",
0xe94435a1,"MainField_Item_Fruit_A_1386571108",
0x161916d3,"MainField_Item_Fruit_A_1387694860",
0x60e48125,"MainField_Item_Fruit_A_1389454516",
0xf9b745b1,"MainField_Item_Fruit_A_1393911650",
0x7ca2df40,"MainField_Item_Fruit_A_1407003532",
0xbccd4c4f,"MainField_Item_Fruit_A_1410544679",
0x542b599b,"MainField_Item_Fruit_A_1423847267",
0x390b6d6b,"MainField_Item_Fruit_A_14251655",
0x7e9799fd,"MainField_Item_Fruit_A_1429021855",
0x7726b2f6,"MainField_Item_Fruit_A_1432029504",
0x8f4e59b4,"MainField_Item_Fruit_A_1432745061",
0xffbe414b,"MainField_Item_Fruit_A_1433598966",
0xd8b1131f,"MainField_Item_Fruit_A_1436684520",
0xdca7fdad,"MainField_Item_Fruit_A_1438587616",
0x7015aa55,"MainField_Item_Fruit_A_1444316112",
0x62587f4d,"MainField_Item_Fruit_A_1445275218",
0xf6a4ba1d,"MainField_Item_Fruit_A_1445518257",
0x8a23331a,"MainField_Item_Fruit_A_145195395",
0x9b50b0d9,"MainField_Item_Fruit_A_1457481844",
0xcac84981,"MainField_Item_Fruit_A_1463618309",
0x86b566f4,"MainField_Item_Fruit_A_1468398683",
0x68bed3c8,"MainField_Item_Fruit_A_1468638476",
0x54884350,"MainField_Item_Fruit_A_1470987266",
0x384b6934,"MainField_Item_Fruit_A_1472225936",
0x6072d68f,"MainField_Item_Fruit_A_1474836750",
0xdfa3c46b,"MainField_Item_Fruit_A_1477371212",
0x35e828e9,"MainField_Item_Fruit_A_1482714237",
0xc585b164,"MainField_Item_Fruit_A_1483054755",
0x93efb344,"MainField_Item_Fruit_A_1483247820",
0xadc5705b,"MainField_Item_Fruit_A_1483963852",
0xa75522d8,"MainField_Item_Fruit_A_1485279284",
0xfb6f505d,"MainField_Item_Fruit_A_1488308992",
0x3ef2af05,"MainField_Item_Fruit_A_1494177335",
0x96f5917e,"MainField_Item_Fruit_A_1494972534",
0x49dc3af6,"MainField_Item_Fruit_A_1497033995",
0xe393ef3f,"MainField_Item_Fruit_A_149945649",
0xa0cedbf9,"MainField_Item_Fruit_A_1499553080",
0xabba91f3,"MainField_Item_Fruit_A_1509299554",
0x94892876,"MainField_Item_Fruit_A_1509586428",
0xccf7f737,"MainField_Item_Fruit_A_1509985359",
0xce9d7cba,"MainField_Item_Fruit_A_1510322434",
0x84681b86,"MainField_Item_Fruit_A_1528844314",
0x1063ac7f,"MainField_Item_Fruit_A_1531786097",
0x5b69048d,"MainField_Item_Fruit_A_1536057307",
0x72b36383,"MainField_Item_Fruit_A_1540733236",
0xe34a7e25,"MainField_Item_Fruit_A_1541716837",
0x35e70a63,"MainField_Item_Fruit_A_1541909828",
0x2b36130f,"MainField_Item_Fruit_A_1545664360",
0xf452e620,"MainField_Item_Fruit_A_1556090799",
0xd83c3e6a,"MainField_Item_Fruit_A_1557738037",
0x43a8dfb4,"MainField_Item_Fruit_A_1559053675",
0x4e4784a3,"MainField_Item_Fruit_A_155991888",
0x85085c07,"MainField_Item_Fruit_A_1565305877",
0x88131218,"MainField_Item_Fruit_A_1567126811",
0x2b628595,"MainField_Item_Fruit_A_1567443030",
0x3e4f9c8a,"MainField_Item_Fruit_A_157073049",
0x5128bc02,"MainField_Item_Fruit_A_1574973583",
0xd31f9d61,"MainField_Item_Fruit_A_1575705726",
0x218088b1,"MainField_Item_Fruit_A_157941080",
0x64455949,"MainField_Item_Fruit_A_1584249247",
0x9bb8cee3,"MainField_Item_Fruit_A_1596706195",
0x9cba0c90,"MainField_Item_Fruit_A_1600592369",
0x7ef43e38,"MainField_Item_Fruit_A_1609051526",
0x738fc698,"MainField_Item_Fruit_A_1612544929",
0x1e50b1c4,"MainField_Item_Fruit_A_1615508760",
0xefe3a600,"MainField_Item_Fruit_A_1618379114",
0xd8c7eab5,"MainField_Item_Fruit_A_1618507423",
0x7eb4aea8,"MainField_Item_Fruit_A_1618684327",
0xc42c4036,"MainField_Item_Fruit_A_1619011331",
0xbe2a03d4,"MainField_Item_Fruit_A_1620460723",
0x67125027,"MainField_Item_Fruit_A_1623640105",
0xc59988ff,"MainField_Item_Fruit_A_1624349692",
0x6d70a778,"MainField_Item_Fruit_A_162991591",
0x0c9fc675,"MainField_Item_Fruit_A_1630848110",
0x0151eef5,"MainField_Item_Fruit_A_1636030857",
0x23d4dbe4,"MainField_Item_Fruit_A_1636056140",
0x98bf1e29,"MainField_Item_Fruit_A_1638957745",
0x041dff8c,"MainField_Item_Fruit_A_1638990273",
0x0002fc04,"MainField_Item_Fruit_A_1641432141",
0x9831e08d,"MainField_Item_Fruit_A_1646769073",
0xa2997ed5,"MainField_Item_Fruit_A_1651151233",
0x3742a82c,"MainField_Item_Fruit_A_1653241508",
0x47cd0f80,"MainField_Item_Fruit_A_1673930531",
0xf11ab4f0,"MainField_Item_Fruit_A_1677122219",
0x50bde607,"MainField_Item_Fruit_A_167732523",
0x5ece51eb,"MainField_Item_Fruit_A_1684431035",
0x48ece9ce,"MainField_Item_Fruit_A_1688986426",
0xd7438377,"MainField_Item_Fruit_A_1691073746",
0x902a52b8,"MainField_Item_Fruit_A_1691672491",
0x9aceb264,"MainField_Item_Fruit_A_169639838",
0xaffcabe1,"MainField_Item_Fruit_A_1699624721",
0x1de4f9dc,"MainField_Item_Fruit_A_1720057431",
0xe456aec9,"MainField_Item_Fruit_A_1721793364",
0x4802965d,"MainField_Item_Fruit_A_1722213723",
0x637f1d72,"MainField_Item_Fruit_A_1725813753",
0xcbaff056,"MainField_Item_Fruit_A_17273845",
0x906c0dc9,"MainField_Item_Fruit_A_1729418301",
0x34c98ebe,"MainField_Item_Fruit_A_1730638081",
0x45206780,"MainField_Item_Fruit_A_1735224258",
0xafa49d0f,"MainField_Item_Fruit_A_1739263024",
0x37ec3c0a,"MainField_Item_Fruit_A_1742767450",
0x9e5bf1c8,"MainField_Item_Fruit_A_1744549419",
0xb51e6443,"MainField_Item_Fruit_A_1749618698",
0x93a2500a,"MainField_Item_Fruit_A_1750040509",
0xb4a4a55e,"MainField_Item_Fruit_A_1754846948",
0x5e8e11ea,"MainField_Item_Fruit_A_1756405342",
0xaf565ca3,"MainField_Item_Fruit_A_1760015424",
0x833843fd,"MainField_Item_Fruit_A_1760057897",
0xdd75002b,"MainField_Item_Fruit_A_176334847",
0xecef41b3,"MainField_Item_Fruit_A_1773593584",
0xcff1586b,"MainField_Item_Fruit_A_1773796807",
0xa08dc05b,"MainField_Item_Fruit_A_1778630960",
0xcd90942c,"MainField_Item_Fruit_A_1782424489",
0xe82e4f7c,"MainField_Item_Fruit_A_1782918682",
0x55e422e6,"MainField_Item_Fruit_A_1783132588",
0x8092650f,"MainField_Item_Fruit_A_1783633737",
0x64e61500,"MainField_Item_Fruit_A_1789806766",
0x336085c0,"MainField_Item_Fruit_A_1790803846",
0x5f840044,"MainField_Item_Fruit_A_1792570040",
0x047bdbdc,"MainField_Item_Fruit_A_179823341",
0x7b8e056e,"MainField_Item_Fruit_A_1798398093",
0x4aa54ed5,"MainField_Item_Fruit_A_1803761626",
0xb9f9afec,"MainField_Item_Fruit_A_1807285062",
0x6fc60457,"MainField_Item_Fruit_A_1808654125",
0x5fc8693a,"MainField_Item_Fruit_A_1813568224",
0x76779908,"MainField_Item_Fruit_A_1816529404",
0x012031c1,"MainField_Item_Fruit_A_1818487874",
0x28484687,"MainField_Item_Fruit_A_1819255397",
0x1565d64b,"MainField_Item_Fruit_A_1820953365",
0x147d82ff,"MainField_Item_Fruit_A_1822391709",
0x1e2fd85c,"MainField_Item_Fruit_A_1823601442",
0x1829480a,"MainField_Item_Fruit_A_1824975946",
0x9efdf40b,"MainField_Item_Fruit_A_1830661905",
0x12195f7b,"MainField_Item_Fruit_A_1831686809",
0x6630ffb5,"MainField_Item_Fruit_A_1835926825",
0xda461b20,"MainField_Item_Fruit_A_1839073506",
0xb60a779d,"MainField_Item_Fruit_A_1858780011",
0x71d77d0f,"MainField_Item_Fruit_A_1859856361",
0x3b67ad01,"MainField_Item_Fruit_A_1875230657",
0xe4e5e2f8,"MainField_Item_Fruit_A_1884153031",
0x109313b4,"MainField_Item_Fruit_A_1884339159",
0xe09c74a3,"MainField_Item_Fruit_A_1889038527",
0xe29ab74d,"MainField_Item_Fruit_A_1889243417",
0x1a837576,"MainField_Item_Fruit_A_1897968386",
0xd02deb65,"MainField_Item_Fruit_A_1901818147",
0x88fdbbcc,"MainField_Item_Fruit_A_1907412622",
0x32a2f08b,"MainField_Item_Fruit_A_1910428521",
0x05392211,"MainField_Item_Fruit_A_1913084322",
0x1a074961,"MainField_Item_Fruit_A_1916797095",
0x78c61789,"MainField_Item_Fruit_A_1918052958",
0xaf3e1851,"MainField_Item_Fruit_A_1926684654",
0x8751e439,"MainField_Item_Fruit_A_1930484789",
0xa120df26,"MainField_Item_Fruit_A_1930827785",
0x34b05ad3,"MainField_Item_Fruit_A_1931310328",
0xf55cf999,"MainField_Item_Fruit_A_1931907816",
0x8ef4e8ef,"MainField_Item_Fruit_A_1932322887",
0x2cad7e16,"MainField_Item_Fruit_A_1935548251",
0x20eb9016,"MainField_Item_Fruit_A_1947177941",
0xdf25413f,"MainField_Item_Fruit_A_195351048",
0x5689a89b,"MainField_Item_Fruit_A_1959004603",
0x86992896,"MainField_Item_Fruit_A_196213752",
0x94fe1455,"MainField_Item_Fruit_A_1963384532",
0xee964ca8,"MainField_Item_Fruit_A_1964082626",
0x09b99157,"MainField_Item_Fruit_A_1965933758",
0x505c50f7,"MainField_Item_Fruit_A_1967837143",
0x2dda118e,"MainField_Item_Fruit_A_197286491",
0xe9131b4b,"MainField_Item_Fruit_A_1975196663",
0xfb3bf6cd,"MainField_Item_Fruit_A_1979770283",
0x33294af6,"MainField_Item_Fruit_A_1981557798",
0x639ba8b2,"MainField_Item_Fruit_A_1987164338",
0x7792ea0e,"MainField_Item_Fruit_A_1988449610",
0x9a265510,"MainField_Item_Fruit_A_1996017098",
0xd474819a,"MainField_Item_Fruit_A_1996052760",
0x565bfe5b,"MainField_Item_Fruit_A_1996504548",
0xa6f328c6,"MainField_Item_Fruit_A_1997121706",
0xeb8d91f2,"MainField_Item_Fruit_A_1999204396",
0x0c87085c,"MainField_Item_Fruit_A_2006760934",
0x5b977d95,"MainField_Item_Fruit_A_2012041456",
0x20c7be8f,"MainField_Item_Fruit_A_2012839601",
0x2cbdeb5f,"MainField_Item_Fruit_A_2021138864",
0x498da215,"MainField_Item_Fruit_A_2021984702",
0x30c036c5,"MainField_Item_Fruit_A_2027097812",
0xb06a09d2,"MainField_Item_Fruit_A_2027989846",
0xece80c59,"MainField_Item_Fruit_A_2030388084",
0x820ead1c,"MainField_Item_Fruit_A_2033601046",
0x23cd6ad1,"MainField_Item_Fruit_A_2040206914",
0x5b794502,"MainField_Item_Fruit_A_2043356035",
0xe603db41,"MainField_Item_Fruit_A_2046535916",
0x4843aa70,"MainField_Item_Fruit_A_2046565003",
0xba59767e,"MainField_Item_Fruit_A_2056630990",
0xf5c7bc72,"MainField_Item_Fruit_A_205706876",
0x5e688f55,"MainField_Item_Fruit_A_2061000653",
0x67e36046,"MainField_Item_Fruit_A_2061835425",
0x0eafd606,"MainField_Item_Fruit_A_2062046064",
0x67f223d3,"MainField_Item_Fruit_A_206439152",
0x2c7f4a90,"MainField_Item_Fruit_A_2068178666",
0xf63765d3,"MainField_Item_Fruit_A_2068394598",
0x6e714f84,"MainField_Item_Fruit_A_2071861967",
0x94743bf0,"MainField_Item_Fruit_A_2073287907",
0x51d58cf6,"MainField_Item_Fruit_A_2078652955",
0x0200c4eb,"MainField_Item_Fruit_A_2081205773",
0xa27b9568,"MainField_Item_Fruit_A_2082950194",
0x404024c5,"MainField_Item_Fruit_A_2083870665",
0x67dbaaf3,"MainField_Item_Fruit_A_2084610954",
0xea27334b,"MainField_Item_Fruit_A_2089484060",
0x8e3a6924,"MainField_Item_Fruit_A_2092354414",
0xf8906f3b,"MainField_Item_Fruit_A_2093215493",
0x8ca367db,"MainField_Item_Fruit_A_2094605631",
0x143d1104,"MainField_Item_Fruit_A_2099970826",
0x55fdac27,"MainField_Item_Fruit_A_2103031039",
0x035dde18,"MainField_Item_Fruit_A_210319328",
0x3ca87b1e,"MainField_Item_Fruit_A_2105907049",
0x71d1c5fc,"MainField_Item_Fruit_A_2108740219",
0x2edc7439,"MainField_Item_Fruit_A_2112049124",
0x888b0f6e,"MainField_Item_Fruit_A_2112649525",
0xe6909f1a,"MainField_Item_Fruit_A_211738930",
0xf0695f77,"MainField_Item_Fruit_A_2121186200",
0x7e678fe9,"MainField_Item_Fruit_A_2140128974",
0x70ecacb0,"MainField_Item_Fruit_A_2145632935",
0xfe53a13a,"MainField_Item_Fruit_A_2145963431",
0x532178bd,"MainField_Item_Fruit_A_214641736",
0x67fea4e1,"MainField_Item_Fruit_A_2148943973",
0x9e6653f6,"MainField_Item_Fruit_A_215085975",
0xa2fbaefc,"MainField_Item_Fruit_A_216057919",
0x00c4cc1b,"MainField_Item_Fruit_A_2161460933",
0x575d6dfa,"MainField_Item_Fruit_A_2162222283",
0xbd1cb9df,"MainField_Item_Fruit_A_2168312656",
0xb52ab0d7,"MainField_Item_Fruit_A_2170383290",
0x967415fe,"MainField_Item_Fruit_A_217692973",
0x049b43f0,"MainField_Item_Fruit_A_2177621741",
0x55d05e62,"MainField_Item_Fruit_A_2181219457",
0x5f3ec627,"MainField_Item_Fruit_A_2182527948",
0x036b161e,"MainField_Item_Fruit_A_2182841069",
0xb41d96dd,"MainField_Item_Fruit_A_2185505707",
0x7ba720b7,"MainField_Item_Fruit_A_2186978039",
0xd26f03a2,"MainField_Item_Fruit_A_2189517127",
0x62d415f5,"MainField_Item_Fruit_A_2196318976",
0xc39ae6ff,"MainField_Item_Fruit_A_2204132357",
0x8abb0e36,"MainField_Item_Fruit_A_2207922497",
0x7dc9a916,"MainField_Item_Fruit_A_2211372009",
0x6cdd0d10,"MainField_Item_Fruit_A_2214242489",
0xad502d19,"MainField_Item_Fruit_A_2215485468",
0xc156af24,"MainField_Item_Fruit_A_2215873453",
0x9d6739f6,"MainField_Item_Fruit_A_2216820387",
0xf793e3a8,"MainField_Item_Fruit_A_2219875720",
0x42925e96,"MainField_Item_Fruit_A_2224257149",
0x1b714023,"MainField_Item_Fruit_A_2225283056",
0xea4e2cb2,"MainField_Item_Fruit_A_2232158032",
0x06048f14,"MainField_Item_Fruit_A_2234118066",
0xf0340715,"MainField_Item_Fruit_A_2235498913",
0x2d683ad9,"MainField_Item_Fruit_A_2235540795",
0xb5ee2e47,"MainField_Item_Fruit_A_2240321317",
0x781255f3,"MainField_Item_Fruit_A_2241246720",
0xb6805f91,"MainField_Item_Fruit_A_2243336810",
0x5f2c5bd2,"MainField_Item_Fruit_A_2248716214",
0x7bc85b07,"MainField_Item_Fruit_A_2252414609",
0xb960f4d2,"MainField_Item_Fruit_A_2260571294",
0x1c6044d2,"MainField_Item_Fruit_A_2268596740",
0x4102403a,"MainField_Item_Fruit_A_2274173138",
0xeb2ca729,"MainField_Item_Fruit_A_2277016284",
0x7ca0c0c6,"MainField_Item_Fruit_A_2277977293",
0xe23bcf45,"MainField_Item_Fruit_A_2278058065",
0x7ab86bf4,"MainField_Item_Fruit_A_2280028757",
0x6ffca460,"MainField_Item_Fruit_A_2280999549",
0xcd431f24,"MainField_Item_Fruit_A_2290164402",
0x69d84778,"MainField_Item_Fruit_A_2294682571",
0x997c6a37,"MainField_Item_Fruit_A_2294795672",
0x8fcb8421,"MainField_Item_Fruit_A_2295575253",
0x5995b250,"MainField_Item_Fruit_A_2296775793",
0x51fdf486,"MainField_Item_Fruit_A_2302648736",
0x7615db73,"MainField_Item_Fruit_A_2303288749",
0x156b8063,"MainField_Item_Fruit_A_2306725632",
0xea8c61b9,"MainField_Item_Fruit_A_2309319964",
0x9796908f,"MainField_Item_Fruit_A_2313527371",
0x52ee7372,"MainField_Item_Fruit_A_2322006021",
0xf45e7ae1,"MainField_Item_Fruit_A_232296462",
0xf09424d3,"MainField_Item_Fruit_A_2322978152",
0x8d415c25,"MainField_Item_Fruit_A_232400628",
0x5af1c243,"MainField_Item_Fruit_A_2325637491",
0x4bcfae5c,"MainField_Item_Fruit_A_2334709489",
0x14689e26,"MainField_Item_Fruit_A_233792486",
0xc9527ce4,"MainField_Item_Fruit_A_2341532089",
0xc1bbecb0,"MainField_Item_Fruit_A_2347289240",
0x25626f8f,"MainField_Item_Fruit_A_2347560668",
0x1b30a735,"MainField_Item_Fruit_A_235041762",
0x98e78145,"MainField_Item_Fruit_A_2351796772",
0xdd2c14a6,"MainField_Item_Fruit_A_2354810739",
0x44dc1aaf,"MainField_Item_Fruit_A_2355172840",
0x1337437f,"MainField_Item_Fruit_A_2357467016",
0x2ae0dc95,"MainField_Item_Fruit_A_2358535356",
0xc0519468,"MainField_Item_Fruit_A_2361019259",
0xdc7c0601,"MainField_Item_Fruit_A_2361058464",
0x439bf467,"MainField_Item_Fruit_A_2361793867",
0xd9b31d9e,"MainField_Item_Fruit_A_2366593777",
0x91b698ea,"MainField_Item_Fruit_A_2369995879",
0x3c214284,"MainField_Item_Fruit_A_2370128672",
0xfd8f374a,"MainField_Item_Fruit_A_2370336847",
0x3a564f3d,"MainField_Item_Fruit_A_2370560508",
0xf446b727,"MainField_Item_Fruit_A_2378712591",
0x47d3c9d0,"MainField_Item_Fruit_A_238123648",
0xf52dc756,"MainField_Item_Fruit_A_2381335756",
0x2f88ac2a,"MainField_Item_Fruit_A_2381888391",
0x0d92aa30,"MainField_Item_Fruit_A_2382168861",
0x031b9e3b,"MainField_Item_Fruit_A_2382979470",
0xbe6083c2,"MainField_Item_Fruit_A_2386006284",
0x5a9e41ef,"MainField_Item_Fruit_A_2386875444",
0x3d22292f,"MainField_Item_Fruit_A_2388029796",
0xa61fdb61,"MainField_Item_Fruit_A_2391945841",
0x1fcdb979,"MainField_Item_Fruit_A_2393776031",
0x741f9e77,"MainField_Item_Fruit_A_2394380567",
0xe2a38bf6,"MainField_Item_Fruit_A_2395515141",
0x0fc01060,"MainField_Item_Fruit_A_2411651997",
0x2b2c8455,"MainField_Item_Fruit_A_2424923460",
0x391b9ba5,"MainField_Item_Fruit_A_2426963638",
0x35d9ab9d,"MainField_Item_Fruit_A_2434371721",
0x76246733,"MainField_Item_Fruit_A_2442483635",
0x884f4c59,"MainField_Item_Fruit_A_2445440389",
0x525d49cc,"MainField_Item_Fruit_A_2445907381",
0x3c0d81b4,"MainField_Item_Fruit_A_2447813771",
0xb457673c,"MainField_Item_Fruit_A_2449335190",
0x89ac3b89,"MainField_Item_Fruit_A_2450880403",
0xe32db282,"MainField_Item_Fruit_A_24545316",
0x866605ae,"MainField_Item_Fruit_A_2459211814",
0x3255f552,"MainField_Item_Fruit_A_2463678806",
0xe0c942ea,"MainField_Item_Fruit_A_2465047107",
0xba317838,"MainField_Item_Fruit_A_2465432422",
0x6fc5fb8f,"MainField_Item_Fruit_A_2467662149",
0xd10265ac,"MainField_Item_Fruit_A_2470587672",
0x5d3713ce,"MainField_Item_Fruit_A_2475423301",
0x08b66f9f,"MainField_Item_Fruit_A_2478748503",
0x91d5fe83,"MainField_Item_Fruit_A_2479692143",
0x18bfdc4a,"MainField_Item_Fruit_A_2488825126",
0xa4ff4b9e,"MainField_Item_Fruit_A_2491830024",
0x6acfc668,"MainField_Item_Fruit_A_2493431791",
0x9979ed7e,"MainField_Item_Fruit_A_2494444814",
0x8941b333,"MainField_Item_Fruit_A_2495466758",
0x0f627cd3,"MainField_Item_Fruit_A_2496385105",
0xeb5664c1,"MainField_Item_Fruit_A_2496886606",
0xb33f921a,"MainField_Item_Fruit_A_2498114140",
0x1a256f61,"MainField_Item_Fruit_A_2508850407",
0x4977f8a1,"MainField_Item_Fruit_A_2511035188",
0xdf59d99b,"MainField_Item_Fruit_A_251142152",
0xf3a5520a,"MainField_Item_Fruit_A_2516069792",
0x2d792635,"MainField_Item_Fruit_A_2525718674",
0xda171b41,"MainField_Item_Fruit_A_2527031079",
0xf128c100,"MainField_Item_Fruit_A_2537040698",
0x54cbff74,"MainField_Item_Fruit_A_2537915472",
0x8f525b3e,"MainField_Item_Fruit_A_2543119661",
0x64e99b2f,"MainField_Item_Fruit_A_2546011152",
0xa6f802da,"MainField_Item_Fruit_A_2546287279",
0x05e96861,"MainField_Item_Fruit_A_2549182898",
0x7356f654,"MainField_Item_Fruit_A_2551336202",
0xa149c018,"MainField_Item_Fruit_A_2555284398",
0xb520b80d,"MainField_Item_Fruit_A_2567105322",
0x28783c9e,"MainField_Item_Fruit_A_2567442550",
0x3d9ec7cf,"MainField_Item_Fruit_A_2567914471",
0x9c27b7ee,"MainField_Item_Fruit_A_2570928203",
0xc9816a3e,"MainField_Item_Fruit_A_2575174843",
0x251d31cd,"MainField_Item_Fruit_A_2575770625",
0x1410bf9c,"MainField_Item_Fruit_A_2580303188",
0x5a4fcfb8,"MainField_Item_Fruit_A_2580578396",
0x4500977e,"MainField_Item_Fruit_A_2582848193",
0xb415064d,"MainField_Item_Fruit_A_2586579900",
0x2eb2fa48,"MainField_Item_Fruit_A_2586899903",
0x8e670e59,"MainField_Item_Fruit_A_2587778915",
0xdd07b7fc,"MainField_Item_Fruit_A_2600733879",
0xfb9fe44b,"MainField_Item_Fruit_A_2600959132",
0xcaca7139,"MainField_Item_Fruit_A_2601444969",
0x0838b9a1,"MainField_Item_Fruit_A_260192552",
0x8dc4b67d,"MainField_Item_Fruit_A_2602291097",
0x86ef92b7,"MainField_Item_Fruit_A_2602804626",
0xf7f0f083,"MainField_Item_Fruit_A_26034030",
0xd64dccaf,"MainField_Item_Fruit_A_2603759157",
0x3952185a,"MainField_Item_Fruit_A_2603799067",
0x48953270,"MainField_Item_Fruit_A_2608525832",
0x5cec0085,"MainField_Item_Fruit_A_2622382475",
0x9097afda,"MainField_Item_Fruit_A_2623359858",
0xe97d6a8f,"MainField_Item_Fruit_A_262976228",
0xb1f7ddc6,"MainField_Item_Fruit_A_263135932",
0x3d35f7fc,"MainField_Item_Fruit_A_2631450306",
0xb3b46ac9,"MainField_Item_Fruit_A_2632705530",
0x7322d1f9,"MainField_Item_Fruit_A_2636286315",
0xcc1b12cd,"MainField_Item_Fruit_A_2639703705",
0x9b20893f,"MainField_Item_Fruit_A_2649062704",
0xf9e6d78f,"MainField_Item_Fruit_A_2652534445",
0x1eee835a,"MainField_Item_Fruit_A_2655225323",
0xdd84a78a,"MainField_Item_Fruit_A_265918095",
0x105755d7,"MainField_Item_Fruit_A_2659209587",
0x6183baf8,"MainField_Item_Fruit_A_2660635027",
0x57586325,"MainField_Item_Fruit_A_2661213912",
0xfa963f67,"MainField_Item_Fruit_A_2671584441",
0x4dae70e0,"MainField_Item_Fruit_A_2679031417",
0xadbefde1,"MainField_Item_Fruit_A_2688643473",
0x75a319e9,"MainField_Item_Fruit_A_2691280941",
0x1935dfe3,"MainField_Item_Fruit_A_2696084658",
0x5039e0aa,"MainField_Item_Fruit_A_2697050506",
0x40e9e3af,"MainField_Item_Fruit_A_2697587932",
0x3467948f,"MainField_Item_Fruit_A_2698648139",
0x10dff759,"MainField_Item_Fruit_A_2712529899",
0xb5171d7e,"MainField_Item_Fruit_A_2713978295",
0xa8cbb442,"MainField_Item_Fruit_A_2721572562",
0x0f4a2cba,"MainField_Item_Fruit_A_2721919079",
0x9ad372b5,"MainField_Item_Fruit_A_2722018208",
0x02f38fa8,"MainField_Item_Fruit_A_2722915325",
0x3c168708,"MainField_Item_Fruit_A_2724902381",
0x23b64867,"MainField_Item_Fruit_A_2732309154",
0x45ce014a,"MainField_Item_Fruit_A_2735105098",
0xeb70bccc,"MainField_Item_Fruit_A_2735641345",
0x25b99467,"MainField_Item_Fruit_A_2746442662",
0xd5583eb7,"MainField_Item_Fruit_A_2746576097",
0x1433a54d,"MainField_Item_Fruit_A_2747124090",
0x36385a8f,"MainField_Item_Fruit_A_275080411",
0x441351d4,"MainField_Item_Fruit_A_2751813794",
0x402040b8,"MainField_Item_Fruit_A_2758296053",
0x272e5df5,"MainField_Item_Fruit_A_2761784883",
0x1c09142a,"MainField_Item_Fruit_A_2762342334",
0x544cb156,"MainField_Item_Fruit_A_2774138987",
0xf094c24f,"MainField_Item_Fruit_A_2774980927",
0x0c059c70,"MainField_Item_Fruit_A_2775471509",
0x421a03eb,"MainField_Item_Fruit_A_2776016242",
0x38544510,"MainField_Item_Fruit_A_2786557720",
0xb3d33144,"MainField_Item_Fruit_A_2786660802",
0x33bf3066,"MainField_Item_Fruit_A_2787234845",
0xbd3e221f,"MainField_Item_Fruit_A_2792518182",
0x362cced6,"MainField_Item_Fruit_A_2796143022",
0xd0d7dfdb,"MainField_Item_Fruit_A_2797590857",
0x13e4dd2c,"MainField_Item_Fruit_A_2797613042",
0x13c51906,"MainField_Item_Fruit_A_2799190153",
0x0bbb9a13,"MainField_Item_Fruit_A_2800388049",
0x4289c1d8,"MainField_Item_Fruit_A_2800692868",
0x20edbc9e,"MainField_Item_Fruit_A_2805416056",
0xc010ece2,"MainField_Item_Fruit_A_2817881080",
0xae1d8083,"MainField_Item_Fruit_A_2822450925",
0xe6043910,"MainField_Item_Fruit_A_2836843948",
0xea2b9113,"MainField_Item_Fruit_A_283698638",
0x6431444e,"MainField_Item_Fruit_A_2842449504",
0xe401adbf,"MainField_Item_Fruit_A_2849515784",
0x4e2eff31,"MainField_Item_Fruit_A_2852971973",
0x6ad7e8a8,"MainField_Item_Fruit_A_2863161059",
0xeebb46e1,"MainField_Item_Fruit_A_2863446324",
0x990b4f03,"MainField_Item_Fruit_A_2867456646",
0xbd725b76,"MainField_Item_Fruit_A_2868972718",
0x25e1ad1c,"MainField_Item_Fruit_A_2869962518",
0x8748a020,"MainField_Item_Fruit_A_2871039854",
0x8cc90698,"MainField_Item_Fruit_A_2873144044",
0x47ed9235,"MainField_Item_Fruit_A_2875235974",
0x8070e8a1,"MainField_Item_Fruit_A_2877210531",
0x91d69458,"MainField_Item_Fruit_A_2882112957",
0x841898f0,"MainField_Item_Fruit_A_2885162246",
0x4507319c,"MainField_Item_Fruit_A_2885300182",
0x878c4dee,"MainField_Item_Fruit_A_2888443805",
0xbdf0d600,"MainField_Item_Fruit_A_2892391219",
0x908ecc08,"MainField_Item_Fruit_A_2892810423",
0xcf1eab6a,"MainField_Item_Fruit_A_2895499551",
0x36e28e46,"MainField_Item_Fruit_A_2897705791",
0x038ba36b,"MainField_Item_Fruit_A_2898929048",
0x9e68f0b8,"MainField_Item_Fruit_A_2899473921",
0x565d2921,"MainField_Item_Fruit_A_2900662194",
0x648ac33c,"MainField_Item_Fruit_A_2905635553",
0x4119f99a,"MainField_Item_Fruit_A_2918336865",
0xac605ffc,"MainField_Item_Fruit_A_2919338617",
0x6ff288ed,"MainField_Item_Fruit_A_2925590183",
0x61e6694b,"MainField_Item_Fruit_A_292612107",
0x254dadb4,"MainField_Item_Fruit_A_2938824216",
0x66a973fc,"MainField_Item_Fruit_A_2940073579",
0x40c20505,"MainField_Item_Fruit_A_2947236668",
0xdb03958d,"MainField_Item_Fruit_A_2951696162",
0x284dbb43,"MainField_Item_Fruit_A_2954704458",
0xe2915a97,"MainField_Item_Fruit_A_2960336289",
0x9e52ba0d,"MainField_Item_Fruit_A_2968280619",
0x929676a8,"MainField_Item_Fruit_A_2970199064",
0x63e9d2fa,"MainField_Item_Fruit_A_297211221",
0x5d450a7c,"MainField_Item_Fruit_A_2972335778",
0xe655f579,"MainField_Item_Fruit_A_2974185899",
0xc19c3d7a,"MainField_Item_Fruit_A_2975395583",
0xb10df396,"MainField_Item_Fruit_A_2979067574",
0xecbea455,"MainField_Item_Fruit_A_2982012925",
0x714f90f6,"MainField_Item_Fruit_A_2988297596",
0x858aa170,"MainField_Item_Fruit_A_2989397572",
0x3effe096,"MainField_Item_Fruit_A_299529790",
0x40a76442,"MainField_Item_Fruit_A_299557743",
0x4bd90a97,"MainField_Item_Fruit_A_2999237382",
0x0433d205,"MainField_Item_Fruit_A_3001230407",
0x62d3a74a,"MainField_Item_Fruit_A_3004388790",
0xead367f7,"MainField_Item_Fruit_A_300710853",
0x870914d3,"MainField_Item_Fruit_A_3009620346",
0x53e6f4d7,"MainField_Item_Fruit_A_301763664",
0x42ea74c3,"MainField_Item_Fruit_A_3022981408",
0xefaa52b4,"MainField_Item_Fruit_A_3029023740",
0xe7a4ec82,"MainField_Item_Fruit_A_3032931641",
0x3fead69f,"MainField_Item_Fruit_A_3033036773",
0x2d31b6d4,"MainField_Item_Fruit_A_3039893212",
0x726f3709,"MainField_Item_Fruit_A_3045080823",
0x7c515804,"MainField_Item_Fruit_A_3045907086",
0x65f12d0c,"MainField_Item_Fruit_A_3046039646",
0x968aec84,"MainField_Item_Fruit_A_3047904978",
0x0c6a5881,"MainField_Item_Fruit_A_3049585309",
0x8e7c48bd,"MainField_Item_Fruit_A_3051307797",
0x8ca7750b,"MainField_Item_Fruit_A_3052067401",
0xe6326556,"MainField_Item_Fruit_A_3052981999",
0x7dfbceec,"MainField_Item_Fruit_A_3053487200",
0xc0de4252,"MainField_Item_Fruit_A_3055374560",
0x0e42a53f,"MainField_Item_Fruit_A_3060531087",
0x2dac7a84,"MainField_Item_Fruit_A_3065553704",
0xbe89151f,"MainField_Item_Fruit_A_3072389994",
0xd46e7ec6,"MainField_Item_Fruit_A_3073674279",
0xb8a63240,"MainField_Item_Fruit_A_3074057682",
0x4caf0a59,"MainField_Item_Fruit_A_3075455621",
0x5ea4b492,"MainField_Item_Fruit_A_3075554969",
0xf3f6059b,"MainField_Item_Fruit_A_3086226640",
0xc6446721,"MainField_Item_Fruit_A_3086271404",
0x530f890d,"MainField_Item_Fruit_A_3089545885",
0x92728837,"MainField_Item_Fruit_A_3097260681",
0xf170eee6,"MainField_Item_Fruit_A_3100707160",
0x15203dd3,"MainField_Item_Fruit_A_3102061052",
0xbe02b1fb,"MainField_Item_Fruit_A_3107154473",
0x6d4d0285,"MainField_Item_Fruit_A_3115028611",
0xb0812ce6,"MainField_Item_Fruit_A_3117286332",
0xdce6598b,"MainField_Item_Fruit_A_311964881",
0x2461319f,"MainField_Item_Fruit_A_312057859",
0x179cb708,"MainField_Item_Fruit_A_3124623575",
0xeb2cb125,"MainField_Item_Fruit_A_3126336904",
0x6d67526c,"MainField_Item_Fruit_A_3131989408",
0x70bb928f,"MainField_Item_Fruit_A_3132501176",
0x78c083e6,"MainField_Item_Fruit_A_3140920017",
0xe8430aa6,"MainField_Item_Fruit_A_3143463858",
0xc0d5dc38,"MainField_Item_Fruit_A_3145477217",
0x4f4bf02d,"MainField_Item_Fruit_A_3145505498",
0x79b7aff6,"MainField_Item_Fruit_A_3145953458",
0x69258cd6,"MainField_Item_Fruit_A_3152251523",
0x3146f64d,"MainField_Item_Fruit_A_3154488288",
0xad76e765,"MainField_Item_Fruit_A_3155921605",
0x84d23c4d,"MainField_Item_Fruit_A_3160184418",
0xbcbe966c,"MainField_Item_Fruit_A_316703275",
0x45fdfdfd,"MainField_Item_Fruit_A_3169065917",
0xd69d62c5,"MainField_Item_Fruit_A_317046630",
0x18ad3315,"MainField_Item_Fruit_A_3179983772",
0x83fdbf35,"MainField_Item_Fruit_A_3193876016",
0x980cfdc6,"MainField_Item_Fruit_A_3196820539",
0x534b2d0c,"MainField_Item_Fruit_A_3196856351",
0x14f1887a,"MainField_Item_Fruit_A_3197550401",
0x48cec22d,"MainField_Item_Fruit_A_3203006521",
0xb4bdfd86,"MainField_Item_Fruit_A_3208625459",
0xc1226a36,"MainField_Item_Fruit_A_3208984558",
0x5b67e208,"MainField_Item_Fruit_A_3209395844",
0xaa0ee26b,"MainField_Item_Fruit_A_3220158479",
0x79de61cb,"MainField_Item_Fruit_A_3220447173",
0xc4501072,"MainField_Item_Fruit_A_3221103757",
0x7ea68bbb,"MainField_Item_Fruit_A_3221826620",
0x13491a76,"MainField_Item_Fruit_A_3223831548",
0xe65fab40,"MainField_Item_Fruit_A_3224616113",
0x52eeedeb,"MainField_Item_Fruit_A_3227593942",
0x5d44dd1d,"MainField_Item_Fruit_A_3229034535",
0x463e9ede,"MainField_Item_Fruit_A_3229590922",
0xf1625805,"MainField_Item_Fruit_A_3238170769",
0xd5bbe32b,"MainField_Item_Fruit_A_3238507766",
0x57c38a42,"MainField_Item_Fruit_A_3242550136",
0x198c184d,"MainField_Item_Fruit_A_3245133593",
0x7e14c960,"MainField_Item_Fruit_A_3248347492",
0x7d044108,"MainField_Item_Fruit_A_3250557886",
0xc0c46ef6,"MainField_Item_Fruit_A_325144621",
0x25d27928,"MainField_Item_Fruit_A_325279652",
0xcd25f4ac,"MainField_Item_Fruit_A_3259021780",
0x96a67e57,"MainField_Item_Fruit_A_325996991",
0x621e9740,"MainField_Item_Fruit_A_3263630945",
0x87bff4a4,"MainField_Item_Fruit_A_3263949336",
0xf98392e0,"MainField_Item_Fruit_A_3266915648",
0x449c8e83,"MainField_Item_Fruit_A_3270372063",
0x6919a0d8,"MainField_Item_Fruit_A_3272401080",
0x089b8b95,"MainField_Item_Fruit_A_3279086852",
0x5c0da010,"MainField_Item_Fruit_A_327957817",
0x7f330b09,"MainField_Item_Fruit_A_3290438744",
0x4b4e4ccf,"MainField_Item_Fruit_A_3292093136",
0x3d126ae0,"MainField_Item_Fruit_A_3294894572",
0xbe488465,"MainField_Item_Fruit_A_3298874435",
0x794ca609,"MainField_Item_Fruit_A_330093484",
0xd51817bc,"MainField_Item_Fruit_A_3307015545",
0x43c03d22,"MainField_Item_Fruit_A_3307841234",
0x60a76e4c,"MainField_Item_Fruit_A_3318308373",
0x4e04f6fb,"MainField_Item_Fruit_A_3320059089",
0x6d354525,"MainField_Item_Fruit_A_3322214878",
0x3c7db19d,"MainField_Item_Fruit_A_3322664996",
0xd72df075,"MainField_Item_Fruit_A_3324553057",
0x2c590706,"MainField_Item_Fruit_A_3324841524",
0x081aa2bf,"MainField_Item_Fruit_A_3326452873",
0x77d97fe3,"MainField_Item_Fruit_A_3327219716",
0xfe17c8a5,"MainField_Item_Fruit_A_3328171625",
0x8e4b094f,"MainField_Item_Fruit_A_3331835424",
0xbeb9f69a,"MainField_Item_Fruit_A_3332804680",
0xa0fc71bb,"MainField_Item_Fruit_A_3333848832",
0x86d45bde,"MainField_Item_Fruit_A_3334133611",
0xee400919,"MainField_Item_Fruit_A_3335309884",
0x06cc6c35,"MainField_Item_Fruit_A_3338886129",
0x243d5e30,"MainField_Item_Fruit_A_334895223",
0xaab528dc,"MainField_Item_Fruit_A_3350856702",
0xf0fed3d9,"MainField_Item_Fruit_A_3352432239",
0x2d0dfa15,"MainField_Item_Fruit_A_3353126067",
0xac8c4b54,"MainField_Item_Fruit_A_3358320361",
0xeb011c9f,"MainField_Item_Fruit_A_335882344",
0xbb9a0e29,"MainField_Item_Fruit_A_3359770488",
0x6cded02f,"MainField_Item_Fruit_A_3370264538",
0x735fcbd6,"MainField_Item_Fruit_A_3371766811",
0x074c086e,"MainField_Item_Fruit_A_3378093734",
0x5ee59eb0,"MainField_Item_Fruit_A_3385452961",
0x160f98df,"MainField_Item_Fruit_A_3396837764",
0x0ad50372,"MainField_Item_Fruit_A_3399755215",
0xe0a18e5b,"MainField_Item_Fruit_A_3401231507",
0x4a77075c,"MainField_Item_Fruit_A_3407521501",
0xba5d4875,"MainField_Item_Fruit_A_3410371356",
0xdf30ea36,"MainField_Item_Fruit_A_3411595389",
0xd2307e8e,"MainField_Item_Fruit_A_3412008582",
0x278cb55e,"MainField_Item_Fruit_A_3414190689",
0x953375f2,"MainField_Item_Fruit_A_3424292695",
0x299426f4,"MainField_Item_Fruit_A_3424620635",
0x45f17530,"MainField_Item_Fruit_A_3425358289",
0xaca4dab5,"MainField_Item_Fruit_A_3428467492",
0x6b3246ef,"MainField_Item_Fruit_A_3430118937",
0x77367a8d,"MainField_Item_Fruit_A_3430588992",
0x19399a9a,"MainField_Item_Fruit_A_3430656588",
0x2a019c72,"MainField_Item_Fruit_A_3433032441",
0x164da7bd,"MainField_Item_Fruit_A_3434137518",
0xd3297499,"MainField_Item_Fruit_A_3435262151",
0xf6f03eb4,"MainField_Item_Fruit_A_3438128760",
0xddc5b6ba,"MainField_Item_Fruit_A_3438637659",
0xb316a5e5,"MainField_Item_Fruit_A_3441059229",
0x3801964b,"MainField_Item_Fruit_A_3445566750",
0xea9d7b8f,"MainField_Item_Fruit_A_3448876393",
0x95ba3772,"MainField_Item_Fruit_A_3459537599",
0x69b750d0,"MainField_Item_Fruit_A_3469203655",
0xfce0c426,"MainField_Item_Fruit_A_3469292330",
0x8fd056ef,"MainField_Item_Fruit_A_3473011535",
0x900dbd96,"MainField_Item_Fruit_A_3474969194",
0x5d7fbbbd,"MainField_Item_Fruit_A_3480969996",
0x2fdcc4c6,"MainField_Item_Fruit_A_348198837",
0x4e03cdb3,"MainField_Item_Fruit_A_3482636290",
0xe67c57ce,"MainField_Item_Fruit_A_3483058719",
0x8e10cce6,"MainField_Item_Fruit_A_3486923214",
0x85391962,"MainField_Item_Fruit_A_3487040608",
0xc1a2accc,"MainField_Item_Fruit_A_3492835566",
0xa26ef644,"MainField_Item_Fruit_A_3500650736",
0x7577c7cf,"MainField_Item_Fruit_A_3500743212",
0x1f745320,"MainField_Item_Fruit_A_3501938419",
0x428bdc84,"MainField_Item_Fruit_A_3505537902",
0x4e3ebee6,"MainField_Item_Fruit_A_3513199205",
0xa8a66de5,"MainField_Item_Fruit_A_351544270",
0xdd33e963,"MainField_Item_Fruit_A_3515648726",
0xb91b036d,"MainField_Item_Fruit_A_3516506918",
0x051e9ff2,"MainField_Item_Fruit_A_3517290324",
0x10f977dd,"MainField_Item_Fruit_A_3517903160",
0x492ca8fb,"MainField_Item_Fruit_A_351870326",
0xdc28b389,"MainField_Item_Fruit_A_3518901980",
0x928ad63b,"MainField_Item_Fruit_A_351993040",
0x56500f99,"MainField_Item_Fruit_A_3521650194",
0xaf7332c1,"MainField_Item_Fruit_A_3522017095",
0xd640f5a8,"MainField_Item_Fruit_A_3522183907",
0x21563064,"MainField_Item_Fruit_A_3522367714",
0xb9df1ca7,"MainField_Item_Fruit_A_3529632169",
0x94aee007,"MainField_Item_Fruit_A_3530659300",
0x2ca4f2ad,"MainField_Item_Fruit_A_3545958427",
0xf19f973d,"MainField_Item_Fruit_A_3546572686",
0x24039c15,"MainField_Item_Fruit_A_3549955752",
0xf8903524,"MainField_Item_Fruit_A_3550083006",
0x46f34252,"MainField_Item_Fruit_A_3552816118",
0x08a7dc4f,"MainField_Item_Fruit_A_3554935507",
0x1e5a8abf,"MainField_Item_Fruit_A_3555160824",
0x7afbe1c9,"MainField_Item_Fruit_A_3556857052",
0x731d88d9,"MainField_Item_Fruit_A_3557401495",
0x8cc4ab7b,"MainField_Item_Fruit_A_3559983630",
0x2d76322e,"MainField_Item_Fruit_A_3568587568",
0x569ed77d,"MainField_Item_Fruit_A_3577316150",
0x3230afca,"MainField_Item_Fruit_A_3578483563",
0x68551288,"MainField_Item_Fruit_A_3584121771",
0x737fd781,"MainField_Item_Fruit_A_3585515773",
0x974119e3,"MainField_Item_Fruit_A_3588000308",
0x2acf3904,"MainField_Item_Fruit_A_3594769835",
0x3cb2e38a,"MainField_Item_Fruit_A_3598491829",
0x43bce7cc,"MainField_Item_Fruit_A_3601626730",
0x0e18117c,"MainField_Item_Fruit_A_3603117345",
0x5d28d35b,"MainField_Item_Fruit_A_361266047",
0x19b0157a,"MainField_Item_Fruit_A_3614587007",
0x06a9b9dd,"MainField_Item_Fruit_A_361580082",
0xe4e058a1,"MainField_Item_Fruit_A_3619015716",
0xed9f073c,"MainField_Item_Fruit_A_3621595869",
0xb7d8bdf4,"MainField_Item_Fruit_A_362213923",
0x818bbbc0,"MainField_Item_Fruit_A_3622569533",
0x7a5ec528,"MainField_Item_Fruit_A_3630482985",
0x21daba75,"MainField_Item_Fruit_A_3631933196",
0xe79ae52c,"MainField_Item_Fruit_A_363605473",
0xf32f2eff,"MainField_Item_Fruit_A_363635313",
0xc1d76337,"MainField_Item_Fruit_A_36369448",
0x45390223,"MainField_Item_Fruit_A_3640263760",
0xc02734c8,"MainField_Item_Fruit_A_3641090799",
0x38f0333a,"MainField_Item_Fruit_A_3642821578",
0x0bf0381a,"MainField_Item_Fruit_A_3642915809",
0xa8938376,"MainField_Item_Fruit_A_3656103918",
0xb408e9a1,"MainField_Item_Fruit_A_365699091",
0xa9bdd87c,"MainField_Item_Fruit_A_3658626010",
0xa3656f8f,"MainField_Item_Fruit_A_3659407371",
0x976f8be7,"MainField_Item_Fruit_A_3661056631",
0xc1a53d4f,"MainField_Item_Fruit_A_3665247020",
0x0bdc882f,"MainField_Item_Fruit_A_3673520631",
0x57822a5e,"MainField_Item_Fruit_A_3677165323",
0xdff59e01,"MainField_Item_Fruit_A_3680424186",
0x30bafb01,"MainField_Item_Fruit_A_3681199833",
0xbffd8adb,"MainField_Item_Fruit_A_3681286177",
0x62ec1a52,"MainField_Item_Fruit_A_3683836297",
0xdadb68b4,"MainField_Item_Fruit_A_3684836220",
0xe7c738f3,"MainField_Item_Fruit_A_3690301217",
0xc737e65d,"MainField_Item_Fruit_A_3691396152",
0x29367fc7,"MainField_Item_Fruit_A_3702136278",
0x6a2efeaa,"MainField_Item_Fruit_A_3707576128",
0xb65db1c9,"MainField_Item_Fruit_A_3707708816",
0xf827151c,"MainField_Item_Fruit_A_3719261813",
0xb5c61ecd,"MainField_Item_Fruit_A_3727329353",
0x52018a85,"MainField_Item_Fruit_A_3730609438",
0xa1bc3ee1,"MainField_Item_Fruit_A_3731580662",
0xfb7c62d5,"MainField_Item_Fruit_A_3743236520",
0x269a1b12,"MainField_Item_Fruit_A_3745259131",
0x7c88038e,"MainField_Item_Fruit_A_3748899300",
0xbe2e6415,"MainField_Item_Fruit_A_375796843",
0x0498905c,"MainField_Item_Fruit_A_3759683354",
0xa45857a3,"MainField_Item_Fruit_A_3761369153",
0x657b7bba,"MainField_Item_Fruit_A_3762232236",
0xf44f28e4,"MainField_Item_Fruit_A_3770839609",
0x6a595341,"MainField_Item_Fruit_A_3774494498",
0xfb409645,"MainField_Item_Fruit_A_3781085426",
0xd89b4bd3,"MainField_Item_Fruit_A_3784071172",
0xac891441,"MainField_Item_Fruit_A_3789027235",
0xefa123b6,"MainField_Item_Fruit_A_3789838152",
0xaa64605b,"MainField_Item_Fruit_A_3790207724",
0x67643567,"MainField_Item_Fruit_A_3791117257",
0x47a249fb,"MainField_Item_Fruit_A_3796678480",
0xf92ff494,"MainField_Item_Fruit_A_3798581887",
0x4c7656dd,"MainField_Item_Fruit_A_3802950562",
0x01493907,"MainField_Item_Fruit_A_3806132567",
0x1d3cea44,"MainField_Item_Fruit_A_3807177633",
0x8a5772bf,"MainField_Item_Fruit_A_3808369974",
0xb963f30e,"MainField_Item_Fruit_A_381137447",
0x711f51e2,"MainField_Item_Fruit_A_3817401494",
0x71289d51,"MainField_Item_Fruit_A_3818491769",
0xde0ff4f7,"MainField_Item_Fruit_A_3819505048",
0xb8230e47,"MainField_Item_Fruit_A_3822340056",
0x2f7856b7,"MainField_Item_Fruit_A_3822939615",
0x8003c667,"MainField_Item_Fruit_A_3823174346",
0xa7e38989,"MainField_Item_Fruit_A_3823914267",
0x29dcf238,"MainField_Item_Fruit_A_3827895712",
0xea6a2d9e,"MainField_Item_Fruit_A_3827969020",
0x1d62ea05,"MainField_Item_Fruit_A_3828512869",
0xe53ce69d,"MainField_Item_Fruit_A_3830936923",
0xd9e53c27,"MainField_Item_Fruit_A_3833694466",
0xc3ab94c2,"MainField_Item_Fruit_A_3834431187",
0xce5e09df,"MainField_Item_Fruit_A_3838454265",
0x16031263,"MainField_Item_Fruit_A_3840223375",
0x8f582f93,"MainField_Item_Fruit_A_3843180293",
0x5f402dd2,"MainField_Item_Fruit_A_3845916275",
0xae8d49ba,"MainField_Item_Fruit_A_3853169444",
0x5868602c,"MainField_Item_Fruit_A_3855367031",
0xfb1ca55f,"MainField_Item_Fruit_A_386409442",
0x76d2d804,"MainField_Item_Fruit_A_3867211971",
0x34a5ac2d,"MainField_Item_Fruit_A_3873545245",
0xce422316,"MainField_Item_Fruit_A_3887052320",
0x4f29e9fc,"MainField_Item_Fruit_A_3895059128",
0x8de2bfa5,"MainField_Item_Fruit_A_3896120367",
0x14296301,"MainField_Item_Fruit_A_3896173242",
0x90392ecb,"MainField_Item_Fruit_A_3903287426",
0x34bfac66,"MainField_Item_Fruit_A_3903452911",
0xbaf7fbba,"MainField_Item_Fruit_A_3908956322",
0xd934ebe8,"MainField_Item_Fruit_A_390956430",
0xd3cbe810,"MainField_Item_Fruit_A_3909925233",
0xe9b75dda,"MainField_Item_Fruit_A_3911330222",
0xb806588d,"MainField_Item_Fruit_A_3911453651",
0x5d9689be,"MainField_Item_Fruit_A_3911460585",
0x632f876f,"MainField_Item_Fruit_A_3911879733",
0x04d2e5d6,"MainField_Item_Fruit_A_3913594237",
0x8c58cfb7,"MainField_Item_Fruit_A_3914196367",
0x6f3b1e5c,"MainField_Item_Fruit_A_3918602501",
0x71c23514,"MainField_Item_Fruit_A_3919173198",
0x21e81d19,"MainField_Item_Fruit_A_3928340989",
0x6e3df84e,"MainField_Item_Fruit_A_393327100",
0xaaa96325,"MainField_Item_Fruit_A_393704896",
0x3ae7c710,"MainField_Item_Fruit_A_3939463766",
0xa70938cb,"MainField_Item_Fruit_A_3939906782",
0xbfa27649,"MainField_Item_Fruit_A_394008881",
0xbc930bf3,"MainField_Item_Fruit_A_3940367875",
0xf5607482,"MainField_Item_Fruit_A_394729365",
0x6f8dad11,"MainField_Item_Fruit_A_3947450653",
0x5da1a0ac,"MainField_Item_Fruit_A_394877260",
0x4502951c,"MainField_Item_Fruit_A_3957296724",
0x5e56902e,"MainField_Item_Fruit_A_3961820420",
0x1b71055c,"MainField_Item_Fruit_A_3964601798",
0x92071a53,"MainField_Item_Fruit_A_3967664798",
0x85c54e32,"MainField_Item_Fruit_A_3971199900",
0x3401cf73,"MainField_Item_Fruit_A_3971387730",
0x3cc407b1,"MainField_Item_Fruit_A_3974517227",
0xf323ba79,"MainField_Item_Fruit_A_3984903554",
0xe15bc1d9,"MainField_Item_Fruit_A_3987057092",
0x819a3658,"MainField_Item_Fruit_A_3993968864",
0x803e4fad,"MainField_Item_Fruit_A_3995083158",
0x3739d527,"MainField_Item_Fruit_A_3997814730",
0x741dba5d,"MainField_Item_Fruit_A_4002577474",
0xadacf90b,"MainField_Item_Fruit_A_4008649445",
0x0f96bb54,"MainField_Item_Fruit_A_4009193852",
0x41d107c6,"MainField_Item_Fruit_A_4009690298",
0x2da7505b,"MainField_Item_Fruit_A_4011163036",
0x62fa73f5,"MainField_Item_Fruit_A_4014586680",
0x7b22eb34,"MainField_Item_Fruit_A_4020953018",
0x022de2a5,"MainField_Item_Fruit_A_4025588848",
0x4d8e2b73,"MainField_Item_Fruit_A_4027566453",
0x8a2b85b7,"MainField_Item_Fruit_A_4030273775",
0x5131e2d8,"MainField_Item_Fruit_A_4035691796",
0x3747beb5,"MainField_Item_Fruit_A_4039094858",
0x8047e15d,"MainField_Item_Fruit_A_4040120519",
0xde4bd581,"MainField_Item_Fruit_A_4040990927",
0x7f4600ca,"MainField_Item_Fruit_A_4041902604",
0x76269f87,"MainField_Item_Fruit_A_4047908359",
0x2df482ba,"MainField_Item_Fruit_A_404868675",
0x8bf115da,"MainField_Item_Fruit_A_405113650",
0xaa30dad9,"MainField_Item_Fruit_A_4060254689",
0xb1db536e,"MainField_Item_Fruit_A_4060821446",
0x999b214a,"MainField_Item_Fruit_A_4065731292",
0x2f176245,"MainField_Item_Fruit_A_4079560281",
0x8888583d,"MainField_Item_Fruit_A_408173056",
0xd42e88d2,"MainField_Item_Fruit_A_4083384022",
0xa66d1b98,"MainField_Item_Fruit_A_4087623526",
0x42ed1522,"MainField_Item_Fruit_A_4106873727",
0x648f87b9,"MainField_Item_Fruit_A_4108847245",
0x77168598,"MainField_Item_Fruit_A_4110808087",
0x004dd41f,"MainField_Item_Fruit_A_4111567706",
0x23685b9a,"MainField_Item_Fruit_A_4117413368",
0x742f65ea,"MainField_Item_Fruit_A_4118347072",
0x216cd0db,"MainField_Item_Fruit_A_4120677983",
0xe006f314,"MainField_Item_Fruit_A_4122518168",
0x3f3f3edc,"MainField_Item_Fruit_A_4127854173",
0xbce2ef32,"MainField_Item_Fruit_A_4131223301",
0x2b12e2d9,"MainField_Item_Fruit_A_413303500",
0x8ed72515,"MainField_Item_Fruit_A_4134973733",
0x3178a702,"MainField_Item_Fruit_A_4136423306",
0x96e9a3b6,"MainField_Item_Fruit_A_4136824495",
0x4c837f6d,"MainField_Item_Fruit_A_4145287585",
0x2c48e88a,"MainField_Item_Fruit_A_4149119701",
0x76b7b03b,"MainField_Item_Fruit_A_4157765203",
0xb563e9db,"MainField_Item_Fruit_A_416786087",
0x5a367cff,"MainField_Item_Fruit_A_4169828403",
0xea3e2f5c,"MainField_Item_Fruit_A_4170394565",
0x91f60441,"MainField_Item_Fruit_A_4172927755",
0xd9e1e1f3,"MainField_Item_Fruit_A_4181976226",
0x868e0a87,"MainField_Item_Fruit_A_4186181881",
0x8695c96e,"MainField_Item_Fruit_A_4188436431",
0x8d412a93,"MainField_Item_Fruit_A_4202371940",
0xde6e5134,"MainField_Item_Fruit_A_4203841537",
0x9d97bd0e,"MainField_Item_Fruit_A_4207900313",
0xd1a87290,"MainField_Item_Fruit_A_4210523226",
0x5bb08125,"MainField_Item_Fruit_A_4211051281",
0x2a58180a,"MainField_Item_Fruit_A_4213875378",
0x833926be,"MainField_Item_Fruit_A_4214153739",
0xfff624e9,"MainField_Item_Fruit_A_4214538106",
0x6f4de19b,"MainField_Item_Fruit_A_4219676446",
0x72ad9b02,"MainField_Item_Fruit_A_4223499245",
0x5ec5e918,"MainField_Item_Fruit_A_4230997607",
0xdc75b3f9,"MainField_Item_Fruit_A_4234082884",
0xed897a84,"MainField_Item_Fruit_A_4237803823",
0x30498f24,"MainField_Item_Fruit_A_4239636540",
0x7be9927d,"MainField_Item_Fruit_A_4241426244",
0xd63ebf67,"MainField_Item_Fruit_A_424308419",
0x9fd5aa97,"MainField_Item_Fruit_A_4245866943",
0x2f2ba91a,"MainField_Item_Fruit_A_4246011737",
0x0a351f0e,"MainField_Item_Fruit_A_4248222122",
0xfc164f90,"MainField_Item_Fruit_A_4250939539",
0x2b153dd0,"MainField_Item_Fruit_A_4251158243",
0x3e00a0e4,"MainField_Item_Fruit_A_4251856430",
0x3a22df60,"MainField_Item_Fruit_A_4254224136",
0x0f63d0cd,"MainField_Item_Fruit_A_4255981725",
0x3a580b1d,"MainField_Item_Fruit_A_4258339000",
0x1aec1f20,"MainField_Item_Fruit_A_4258750597",
0x63f1e0c7,"MainField_Item_Fruit_A_4260858566",
0x09fe0d1a,"MainField_Item_Fruit_A_4266491509",
0xd1cc7a57,"MainField_Item_Fruit_A_4266760310",
0x27f31632,"MainField_Item_Fruit_A_4267651754",
0x103fcfbb,"MainField_Item_Fruit_A_4275877909",
0x7fe3991b,"MainField_Item_Fruit_A_4282910298",
0xd2da4e2c,"MainField_Item_Fruit_A_4286227024",
0x28bc86b6,"MainField_Item_Fruit_A_4289705185",
0x9963939e,"MainField_Item_Fruit_A_4290837774",
0x1afb88a5,"MainField_Item_Fruit_A_4292117059",
0x090d7950,"MainField_Item_Fruit_A_433854335",
0x67f8cf03,"MainField_Item_Fruit_A_43519012",
0xeef7fda9,"MainField_Item_Fruit_A_437952391",
0x0063e875,"MainField_Item_Fruit_A_440756992",
0x1e8e68f4,"MainField_Item_Fruit_A_44328319",
0xc824686e,"MainField_Item_Fruit_A_445697149",
0x81617b65,"MainField_Item_Fruit_A_455076470",
0x57ef5965,"MainField_Item_Fruit_A_459838195",
0x27d636c9,"MainField_Item_Fruit_A_461549422",
0xdda4183c,"MainField_Item_Fruit_A_462194",
0x6bd062de,"MainField_Item_Fruit_A_463268890",
0x1c0996bd,"MainField_Item_Fruit_A_466844940",
0xc7519583,"MainField_Item_Fruit_A_4697830",
0x9468196f,"MainField_Item_Fruit_A_473321499",
0xba2f4f05,"MainField_Item_Fruit_A_4741574",
0x8da97903,"MainField_Item_Fruit_A_481452829",
0xd99e5685,"MainField_Item_Fruit_A_487766292",
0x179ca4de,"MainField_Item_Fruit_A_48919943",
0x50fd2aac,"MainField_Item_Fruit_A_490378047",
0xfedc65f6,"MainField_Item_Fruit_A_490872460",
0x1a75741c,"MainField_Item_Fruit_A_492376188",
0x5f0b9d07,"MainField_Item_Fruit_A_497700426",
0xd19be960,"MainField_Item_Fruit_A_498072095",
0x15066922,"MainField_Item_Fruit_A_499985147",
0x7f230d6f,"MainField_Item_Fruit_A_50441926",
0x64db238c,"MainField_Item_Fruit_A_511996264",
0xb643647c,"MainField_Item_Fruit_A_514970470",
0xe89f1742,"MainField_Item_Fruit_A_515453444",
0x140e158a,"MainField_Item_Fruit_A_518318880",
0xef87224b,"MainField_Item_Fruit_A_524832555",
0x03565655,"MainField_Item_Fruit_A_525169254",
0xb690cb56,"MainField_Item_Fruit_A_527018856",
0xddc6100f,"MainField_Item_Fruit_A_535998762",
0xca6bc76c,"MainField_Item_Fruit_A_541509356",
0x509f71fa,"MainField_Item_Fruit_A_547954286",
0x35643f9e,"MainField_Item_Fruit_A_54926536",
0x7eeeacd8,"MainField_Item_Fruit_A_549933574",
0xd2e85fb5,"MainField_Item_Fruit_A_556776969",
0xf63cd0f4,"MainField_Item_Fruit_A_556788519",
0x060b7834,"MainField_Item_Fruit_A_559585890",
0x8775974e,"MainField_Item_Fruit_A_563786030",
0x94d04a2f,"MainField_Item_Fruit_A_567826658",
0xe3763f09,"MainField_Item_Fruit_A_571446187",
0xb61d7e86,"MainField_Item_Fruit_A_574127324",
0xec50595b,"MainField_Item_Fruit_A_576469462",
0x6dd86980,"MainField_Item_Fruit_A_576549346",
0xf0b10a93,"MainField_Item_Fruit_A_57804030",
0x501d39ae,"MainField_Item_Fruit_A_578145120",
0xe776ccd8,"MainField_Item_Fruit_A_578756454",
0xec00d75a,"MainField_Item_Fruit_A_581247102",
0x5979ed43,"MainField_Item_Fruit_A_58144076",
0x25bdc4d7,"MainField_Item_Fruit_A_582494838",
0x947a10a9,"MainField_Item_Fruit_A_585355092",
0x4ed52c8c,"MainField_Item_Fruit_A_586041802",
0x3e843510,"MainField_Item_Fruit_A_58845331",
0x9c2b1b35,"MainField_Item_Fruit_A_610139684",
0x1840d833,"MainField_Item_Fruit_A_612622886",
0x8ae10612,"MainField_Item_Fruit_A_620129011",
0x31e216b2,"MainField_Item_Fruit_A_625464190",
0x1c2ad8e3,"MainField_Item_Fruit_A_630065997",
0xe477a385,"MainField_Item_Fruit_A_639553114",
0xeba991e6,"MainField_Item_Fruit_A_641871678",
0x359b77b2,"MainField_Item_Fruit_A_650622914",
0xe4f4eab2,"MainField_Item_Fruit_A_653751607",
0x33cceb7e,"MainField_Item_Fruit_A_653846314",
0x8076e803,"MainField_Item_Fruit_A_654921133",
0x8d2d4be4,"MainField_Item_Fruit_A_664786398",
0x58609077,"MainField_Item_Fruit_A_665490975",
0x53f7f27c,"MainField_Item_Fruit_A_673629773",
0xb67ad294,"MainField_Item_Fruit_A_674036997",
0x8b8fdbcb,"MainField_Item_Fruit_A_677390695",
0xc6c228e0,"MainField_Item_Fruit_A_681081083",
0xbc71eb65,"MainField_Item_Fruit_A_68676771",
0x97afb499,"MainField_Item_Fruit_A_687699397",
0xd764f7c8,"MainField_Item_Fruit_A_687723726",
0x7e1fc4f7,"MainField_Item_Fruit_A_68830318",
0x23df165a,"MainField_Item_Fruit_A_689212690",
0x5b1dee68,"MainField_Item_Fruit_A_69982417",
0x18cf7ed6,"MainField_Item_Fruit_A_704709116",
0xb560315b,"MainField_Item_Fruit_A_711620800",
0x92abe530,"MainField_Item_Fruit_A_711726967",
0x4387a8ef,"MainField_Item_Fruit_A_720137448",
0x34b48832,"MainField_Item_Fruit_A_722771142",
0x7c669d48,"MainField_Item_Fruit_A_723928206",
0xffce9613,"MainField_Item_Fruit_A_724027390",
0xa8ffcad6,"MainField_Item_Fruit_A_725680386",
0x64dd05f0,"MainField_Item_Fruit_A_726490972",
0x2b93b0d1,"MainField_Item_Fruit_A_72953805",
0x789188aa,"MainField_Item_Fruit_A_734234443",
0x026dfd4b,"MainField_Item_Fruit_A_736785917",
0x31aee574,"MainField_Item_Fruit_A_747090404",
0x52fcfafa,"MainField_Item_Fruit_A_751995298",
0x6b0a64e2,"MainField_Item_Fruit_A_753110003",
0x51dd8608,"MainField_Item_Fruit_A_75323822",
0xc5f5a54a,"MainField_Item_Fruit_A_762822677",
0x1f3d821c,"MainField_Item_Fruit_A_775193002",
0xc884cdeb,"MainField_Item_Fruit_A_775806365",
0xd0beea13,"MainField_Item_Fruit_A_776399679",
0xd417b1d1,"MainField_Item_Fruit_A_786712052",
0xf678ab76,"MainField_Item_Fruit_A_787555828",
0xbaa6a42b,"MainField_Item_Fruit_A_789377469",
0xd785df3a,"MainField_Item_Fruit_A_793444948",
0x0337b96e,"MainField_Item_Fruit_A_796958882",
0x681114c3,"MainField_Item_Fruit_A_802779742",
0x4bf9234b,"MainField_Item_Fruit_A_807377942",
0x20b5af3b,"MainField_Item_Fruit_A_810490498",
0xb8cc82c6,"MainField_Item_Fruit_A_810554884",
0x10f612ed,"MainField_Item_Fruit_A_811927514",
0xc44e5033,"MainField_Item_Fruit_A_812800494",
0x6f5fd581,"MainField_Item_Fruit_A_829458542",
0xfd9f4cb1,"MainField_Item_Fruit_A_830150694",
0x322753bc,"MainField_Item_Fruit_A_83306874",
0xbba32571,"MainField_Item_Fruit_A_836663081",
0x3332e108,"MainField_Item_Fruit_A_837111056",
0x18159357,"MainField_Item_Fruit_A_839408916",
0xbfd3df25,"MainField_Item_Fruit_A_840529580",
0x915348f6,"MainField_Item_Fruit_A_846720537",
0xd71f0bc1,"MainField_Item_Fruit_A_849869209",
0x488e7322,"MainField_Item_Fruit_A_854176859",
0x9aba0bdb,"MainField_Item_Fruit_A_856011903",
0xf7af4a45,"MainField_Item_Fruit_A_865749387",
0xc327171e,"MainField_Item_Fruit_A_866193241",
0xb48f101d,"MainField_Item_Fruit_A_866510990",
0x086efcfd,"MainField_Item_Fruit_A_870617490",
0x1c62af28,"MainField_Item_Fruit_A_87143849",
0x68f8e32c,"MainField_Item_Fruit_A_876161883",
0x24303f55,"MainField_Item_Fruit_A_88551473",
0x759c4a3d,"MainField_Item_Fruit_A_887838499",
0x859c7e15,"MainField_Item_Fruit_A_890009425",
0x99e967f6,"MainField_Item_Fruit_A_892140442",
0xe8356878,"MainField_Item_Fruit_A_895850765",
0xc2fb985b,"MainField_Item_Fruit_A_896829416",
0xc38b93c1,"MainField_Item_Fruit_A_899448997",
0xdb736092,"MainField_Item_Fruit_A_899594869",
0x79efb153,"MainField_Item_Fruit_A_900265979",
0xa1ef8bf1,"MainField_Item_Fruit_A_901364379",
0x665fd045,"MainField_Item_Fruit_A_907917907",
0xb7b4904e,"MainField_Item_Fruit_A_908168737",
0x73807de3,"MainField_Item_Fruit_A_910716900",
0x5262a4aa,"MainField_Item_Fruit_A_911518282",
0xd99caeba,"MainField_Item_Fruit_A_916591238",
0x85bd82a3,"MainField_Item_Fruit_A_917506429",
0xa5d28902,"MainField_Item_Fruit_A_918310935",
0xd08454b1,"MainField_Item_Fruit_A_923321283",
0x9e409450,"MainField_Item_Fruit_A_930351787",
0xdf282943,"MainField_Item_Fruit_A_935085637",
0xd395ce09,"MainField_Item_Fruit_A_948968685",
0xea852a2c,"MainField_Item_Fruit_A_951104947",
0x02f6fa00,"MainField_Item_Fruit_A_951449393",
0x3237474d,"MainField_Item_Fruit_A_952980493",
0xd3f71d7e,"MainField_Item_Fruit_A_955734727",
0xe458c6d3,"MainField_Item_Fruit_A_95889796",
0x9cb03feb,"MainField_Item_Fruit_A_962650519",
0xf0dc7bfe,"MainField_Item_Fruit_A_965187962",
0x34066694,"MainField_Item_Fruit_A_965358007",
0xfc533067,"MainField_Item_Fruit_A_968600840",
0x96878391,"MainField_Item_Fruit_A_968932710",
0xfc39237a,"MainField_Item_Fruit_A_973919318",
0x6ae58ca5,"MainField_Item_Fruit_A_974075376",
0xfa7a9563,"MainField_Item_Fruit_A_97523369",
0x4a6bab41,"MainField_Item_Fruit_A_97644408",
0x5faf2be0,"MainField_Item_Fruit_A_97863488",
0x85ff7fd7,"MainField_Item_Fruit_A_983568067",
0xb7660985,"MainField_Item_Fruit_A_98743360",
0x1bb643c3,"MainField_Item_Fruit_A_991970313",
0x7516dde5,"MainField_Item_Fruit_A_9927258",
0x9497bf7f,"MainField_Item_Fruit_A_994510999",
0x65a6953c,"MainField_Item_Fruit_A_997951131",
0xcac74099,"MainField_Item_Fruit_B_1014988144",
0x4b587917,"MainField_Item_Fruit_B_1029895755",
0x10f81671,"MainField_Item_Fruit_B_1037349641",
0xbbd8f00a,"MainField_Item_Fruit_B_1051301569",
0x741ec790,"MainField_Item_Fruit_B_1064235728",
0x90176c0c,"MainField_Item_Fruit_B_10711449",
0x6a68a3f8,"MainField_Item_Fruit_B_107227556",
0xd99ca839,"MainField_Item_Fruit_B_1075360380",
0x5436a07e,"MainField_Item_Fruit_B_1075374716",
0xc59087e2,"MainField_Item_Fruit_B_1084442444",
0x3754e2b7,"MainField_Item_Fruit_B_1085895286",
0x90753241,"MainField_Item_Fruit_B_1091546702",
0x75b236a2,"MainField_Item_Fruit_B_110187074",
0x68a1cd48,"MainField_Item_Fruit_B_1137937142",
0x8de8b0d7,"MainField_Item_Fruit_B_1138836702",
0xb5c0b314,"MainField_Item_Fruit_B_1150058695",
0x029d1d94,"MainField_Item_Fruit_B_1152646813",
0x0446eb22,"MainField_Item_Fruit_B_1163231218",
0xf705b25d,"MainField_Item_Fruit_B_116338832",
0x24d2ee43,"MainField_Item_Fruit_B_1165924720",
0xce5aea59,"MainField_Item_Fruit_B_1166795414",
0x5e7b8873,"MainField_Item_Fruit_B_117458639",
0xc04f86fe,"MainField_Item_Fruit_B_1179690898",
0x9ba71991,"MainField_Item_Fruit_B_11883753",
0x44876ac4,"MainField_Item_Fruit_B_1198880357",
0x49930ebd,"MainField_Item_Fruit_B_1204716373",
0x81e03a51,"MainField_Item_Fruit_B_1223637556",
0xbd5ace14,"MainField_Item_Fruit_B_1236182615",
0xe4dd80d6,"MainField_Item_Fruit_B_1248184029",
0x4f0a1edc,"MainField_Item_Fruit_B_1255375775",
0x51f81ec7,"MainField_Item_Fruit_B_1292219062",
0x89b3e980,"MainField_Item_Fruit_B_1297572405",
0x21e9acbd,"MainField_Item_Fruit_B_1312382610",
0x53aef899,"MainField_Item_Fruit_B_1316540286",
0xc9f8712a,"MainField_Item_Fruit_B_1321427534",
0x69552ff2,"MainField_Item_Fruit_B_1351138433",
0xaf02a692,"MainField_Item_Fruit_B_1354077974",
0xd8d64193,"MainField_Item_Fruit_B_1361818577",
0xc0da3562,"MainField_Item_Fruit_B_137683106",
0x7fae9ffb,"MainField_Item_Fruit_B_1383842260",
0x12409309,"MainField_Item_Fruit_B_1402549025",
0x228b06f1,"MainField_Item_Fruit_B_1426071741",
0x21d8d60e,"MainField_Item_Fruit_B_143594898",
0xee225f1f,"MainField_Item_Fruit_B_1456412477",
0xf8a4b650,"MainField_Item_Fruit_B_1457833377",
0x31b1f0ed,"MainField_Item_Fruit_B_1466170943",
0xe9a58823,"MainField_Item_Fruit_B_1473794831",
0xe04b6a9c,"MainField_Item_Fruit_B_1477746447",
0xb1e09010,"MainField_Item_Fruit_B_149161280",
0xb390b12a,"MainField_Item_Fruit_B_1493424957",
0x0f377bdc,"MainField_Item_Fruit_B_1505261069",
0xb5453cdc,"MainField_Item_Fruit_B_1525178341",
0x3e926408,"MainField_Item_Fruit_B_1526363551",
0x50a0e3dc,"MainField_Item_Fruit_B_1553403876",
0xe9016e13,"MainField_Item_Fruit_B_1561927621",
0xd7317575,"MainField_Item_Fruit_B_1570222470",
0x922e2229,"MainField_Item_Fruit_B_1571330012",
0xb436692a,"MainField_Item_Fruit_B_1577619557",
0x7a2a95c0,"MainField_Item_Fruit_B_1591063060",
0x1ad63213,"MainField_Item_Fruit_B_1593699203",
0xada41f92,"MainField_Item_Fruit_B_1601788694",
0x11c8772f,"MainField_Item_Fruit_B_1611247515",
0x0a087b57,"MainField_Item_Fruit_B_1631921516",
0x27242947,"MainField_Item_Fruit_B_1631944044",
0xce9d1b22,"MainField_Item_Fruit_B_1635248964",
0xc90a34da,"MainField_Item_Fruit_B_1644089436",
0x985f41f7,"MainField_Item_Fruit_B_1652815252",
0x672a96b4,"MainField_Item_Fruit_B_1656542158",
0xba51b5c6,"MainField_Item_Fruit_B_1679017137",
0xb3758588,"MainField_Item_Fruit_B_1691980269",
0xb180b879,"MainField_Item_Fruit_B_1698381691",
0x7333d382,"MainField_Item_Fruit_B_1710109069",
0x5af7f2d4,"MainField_Item_Fruit_B_1710927401",
0xb3ff3c03,"MainField_Item_Fruit_B_1713955189",
0xc9cdbfdd,"MainField_Item_Fruit_B_1720915013",
0xbb7e1365,"MainField_Item_Fruit_B_1728203537",
0x5f3c8145,"MainField_Item_Fruit_B_1755913148",
0x6041c1a3,"MainField_Item_Fruit_B_1758205725",
0x19c1fd19,"MainField_Item_Fruit_B_1760768327",
0xcf05606c,"MainField_Item_Fruit_B_1769413651",
0xe41e56c1,"MainField_Item_Fruit_B_1770153973",
0x9627c802,"MainField_Item_Fruit_B_1788078165",
0xc631a43e,"MainField_Item_Fruit_B_1798683071",
0xf70499a7,"MainField_Item_Fruit_B_182061618",
0x061e5ee8,"MainField_Item_Fruit_B_182284586",
0x49076965,"MainField_Item_Fruit_B_1832044773",
0xc40861af,"MainField_Item_Fruit_B_1852932865",
0xa4054b5e,"MainField_Item_Fruit_B_1864718446",
0x8dded8cc,"MainField_Item_Fruit_B_1872670046",
0x01a6b1ea,"MainField_Item_Fruit_B_1872688478",
0x23f8885a,"MainField_Item_Fruit_B_1872876292",
0x17aeab9d,"MainField_Item_Fruit_B_189373802",
0x39ea8875,"MainField_Item_Fruit_B_1911029766",
0x4fd96c14,"MainField_Item_Fruit_B_1912673542",
0x6cfdf322,"MainField_Item_Fruit_B_1917938730",
0x2cab6354,"MainField_Item_Fruit_B_1922377166",
0xb1e5d961,"MainField_Item_Fruit_B_1928621792",
0x390f4c01,"MainField_Item_Fruit_B_1935301279",
0x9733f57e,"MainField_Item_Fruit_B_1979245015",
0xa6ee43b6,"MainField_Item_Fruit_B_1983801647",
0x7fafd365,"MainField_Item_Fruit_B_1988467743",
0xe4e498d3,"MainField_Item_Fruit_B_1988477983",
0x251555ae,"MainField_Item_Fruit_B_1999865318",
0x5c14431a,"MainField_Item_Fruit_B_200641248",
0x37f01b92,"MainField_Item_Fruit_B_2007253263",
0xf61980a3,"MainField_Item_Fruit_B_2008632583",
0xbc2d2bcb,"MainField_Item_Fruit_B_2019431469",
0xd7997245,"MainField_Item_Fruit_B_2019445805",
0xea109e47,"MainField_Item_Fruit_B_2028331293",
0xf4a49ef4,"MainField_Item_Fruit_B_2030737218",
0xda6914fb,"MainField_Item_Fruit_B_2036454421",
0xae7bac8a,"MainField_Item_Fruit_B_2051660966",
0x0c4d0b4a,"MainField_Item_Fruit_B_20535737",
0x9fb03880,"MainField_Item_Fruit_B_2057447074",
0x713de9ed,"MainField_Item_Fruit_B_2058419901",
0x96a556d8,"MainField_Item_Fruit_B_2061342117",
0x43f06617,"MainField_Item_Fruit_B_2085531132",
0x958a72b7,"MainField_Item_Fruit_B_2111650468",
0xc0cbe6ee,"MainField_Item_Fruit_B_2126007688",
0xfd7285f2,"MainField_Item_Fruit_B_2134102068",
0xf0dfa9a7,"MainField_Item_Fruit_B_2137378212",
0xc522dd9b,"MainField_Item_Fruit_B_2139628804",
0x3c8141e2,"MainField_Item_Fruit_B_2147476480",
0x09737d67,"MainField_Item_Fruit_B_2155864621",
0x3af508dc,"MainField_Item_Fruit_B_2159741408",
0x87f260ed,"MainField_Item_Fruit_B_2173736050",
0xdcab8b39,"MainField_Item_Fruit_B_2175775272",
0x30c37f12,"MainField_Item_Fruit_B_2214935894",
0x8d3d7938,"MainField_Item_Fruit_B_2221982787",
0xdc6b7712,"MainField_Item_Fruit_B_2230339248",
0xcfd54dd5,"MainField_Item_Fruit_B_223276080",
0xbd4718f1,"MainField_Item_Fruit_B_2253975357",
0x69fe7ef9,"MainField_Item_Fruit_B_2261763651",
0xcbc9bc0e,"MainField_Item_Fruit_B_2264748139",
0x10cfc6ae,"MainField_Item_Fruit_B_227618245",
0xe60f7a36,"MainField_Item_Fruit_B_2283404803",
0x6ba1b8e3,"MainField_Item_Fruit_B_2284224601",
0xcb834b53,"MainField_Item_Fruit_B_2292020585",
0xaf235f50,"MainField_Item_Fruit_B_2294048216",
0x58bfdb0c,"MainField_Item_Fruit_B_230149675",
0x9e8ed4f3,"MainField_Item_Fruit_B_2304894452",
0x3c4c3099,"MainField_Item_Fruit_B_2305950518",
0x59538f27,"MainField_Item_Fruit_B_2311695878",
0xa2dc241a,"MainField_Item_Fruit_B_2315222892",
0xb8db97eb,"MainField_Item_Fruit_B_2346678011",
0x71bfa8fb,"MainField_Item_Fruit_B_2346733729",
0x71cba6db,"MainField_Item_Fruit_B_235775956",
0xd64acab1,"MainField_Item_Fruit_B_2358088888",
0x49251adb,"MainField_Item_Fruit_B_2360521442",
0xe97b326c,"MainField_Item_Fruit_B_2363080181",
0xac2845e2,"MainField_Item_Fruit_B_2367272611",
0xce70a35d,"MainField_Item_Fruit_B_2376445648",
0x6d6dddee,"MainField_Item_Fruit_B_2404074560",
0xc03d5b80,"MainField_Item_Fruit_B_2407107176",
0x7d0dd5f6,"MainField_Item_Fruit_B_2414915952",
0x291fadef,"MainField_Item_Fruit_B_2433114945",
0xd09989c2,"MainField_Item_Fruit_B_2435054462",
0xaf3ccbdc,"MainField_Item_Fruit_B_2436739352",
0x4e941a42,"MainField_Item_Fruit_B_2441495592",
0x21fb25c6,"MainField_Item_Fruit_B_2451041505",
0x1295fe11,"MainField_Item_Fruit_B_2462756993",
0x98f80275,"MainField_Item_Fruit_B_2473167642",
0x8d1c892a,"MainField_Item_Fruit_B_2480102280",
0x74153cdc,"MainField_Item_Fruit_B_2495520657",
0x084a7af1,"MainField_Item_Fruit_B_2520932219",
0xa515e465,"MainField_Item_Fruit_B_2521855233",
0x3276ca61,"MainField_Item_Fruit_B_2532016177",
0xb6bad0e8,"MainField_Item_Fruit_B_2533684846",
0x23580f5b,"MainField_Item_Fruit_B_25341065",
0xb513db3b,"MainField_Item_Fruit_B_25359497",
0xf74853d6,"MainField_Item_Fruit_B_2540680505",
0x2e21c195,"MainField_Item_Fruit_B_2559288627",
0x8b86b75d,"MainField_Item_Fruit_B_257500347",
0x9239f1b8,"MainField_Item_Fruit_B_257522875",
0xe4bb2f3b,"MainField_Item_Fruit_B_2578063395",
0x6f23b072,"MainField_Item_Fruit_B_2590316451",
0xc40a6bf5,"MainField_Item_Fruit_B_2616623355",
0x2f83535f,"MainField_Item_Fruit_B_2627190789",
0x1a7a292c,"MainField_Item_Fruit_B_2627421410",
0x64084005,"MainField_Item_Fruit_B_2651395957",
0xb25a7668,"MainField_Item_Fruit_B_2662513221",
0x40cc5102,"MainField_Item_Fruit_B_2664546634",
0x5382209f,"MainField_Item_Fruit_B_266523019",
0x63a429f6,"MainField_Item_Fruit_B_2675127363",
0x1f76814b,"MainField_Item_Fruit_B_2676379557",
0x15caa155,"MainField_Item_Fruit_B_2682675498",
0xfbdf9351,"MainField_Item_Fruit_B_2697790306",
0xf60e9535,"MainField_Item_Fruit_B_2735461274",
0x69c54661,"MainField_Item_Fruit_B_2738138560",
0x521ecf6a,"MainField_Item_Fruit_B_2758154115",
0x11819432,"MainField_Item_Fruit_B_2763382971",
0xc19e12fc,"MainField_Item_Fruit_B_276517516",
0x4f3ced89,"MainField_Item_Fruit_B_2778536817",
0x6b0ede8d,"MainField_Item_Fruit_B_2794230993",
0x9675083a,"MainField_Item_Fruit_B_2812331899",
0x388e8176,"MainField_Item_Fruit_B_2819405877",
0xe616a036,"MainField_Item_Fruit_B_2826146163",
0xe5497a31,"MainField_Item_Fruit_B_2834940883",
0xf57fe0ad,"MainField_Item_Fruit_B_2840637111",
0x33ba73a4,"MainField_Item_Fruit_B_2847990393",
0x1e0ef989,"MainField_Item_Fruit_B_2860704235",
0xbe7b84f2,"MainField_Item_Fruit_B_287121355",
0xbf54abb7,"MainField_Item_Fruit_B_288346595",
0x139cbe40,"MainField_Item_Fruit_B_2884127859",
0xbfc7241e,"MainField_Item_Fruit_B_291374041",
0x4b307359,"MainField_Item_Fruit_B_2917002202",
0xeec3f563,"MainField_Item_Fruit_B_2918131186",
0x03c445c7,"MainField_Item_Fruit_B_2920731472",
0x3ab5de3e,"MainField_Item_Fruit_B_2930583136",
0x38a72b22,"MainField_Item_Fruit_B_294756442",
0x0b6afdf2,"MainField_Item_Fruit_B_294921427",
0x4317b6b6,"MainField_Item_Fruit_B_2967196472",
0xafd10659,"MainField_Item_Fruit_B_2977380922",
0xd8edfe8a,"MainField_Item_Fruit_B_3008750750",
0xc51ea09d,"MainField_Item_Fruit_B_3011162400",
0x8d9e252f,"MainField_Item_Fruit_B_3013202090",
0x34cb7f59,"MainField_Item_Fruit_B_3035894963",
0xeb9a0dcf,"MainField_Item_Fruit_B_3052099704",
0x5d24dd3c,"MainField_Item_Fruit_B_3055222307",
0xbedc66bd,"MainField_Item_Fruit_B_3082164001",
0xcb247501,"MainField_Item_Fruit_B_3117822499",
0xdb642eb4,"MainField_Item_Fruit_B_3123682433",
0xbacaa4fb,"MainField_Item_Fruit_B_3138273681",
0x94eeaa81,"MainField_Item_Fruit_B_3140286818",
0x823e2187,"MainField_Item_Fruit_B_3142797925",
0xf407940a,"MainField_Item_Fruit_B_316024641",
0x7617cae3,"MainField_Item_Fruit_B_317260059",
0x5a21e9b1,"MainField_Item_Fruit_B_3199989306",
0xc05e7eb7,"MainField_Item_Fruit_B_3208024648",
0x2bd0c1af,"MainField_Item_Fruit_B_3220199937",
0x85c3181a,"MainField_Item_Fruit_B_3232835511",
0xd5993b2e,"MainField_Item_Fruit_B_3238694387",
0xa5c811a0,"MainField_Item_Fruit_B_3239956149",
0x11ea89fc,"MainField_Item_Fruit_B_3287491621",
0x5960d2f0,"MainField_Item_Fruit_B_329012172",
0xb2831be7,"MainField_Item_Fruit_B_3298746428",
0x0ef1afc1,"MainField_Item_Fruit_B_3304815044",
0xd9d3df34,"MainField_Item_Fruit_B_3329231532",
0x664eac53,"MainField_Item_Fruit_B_3340733826",
0x428fa949,"MainField_Item_Fruit_B_3353142190",
0x0b627f6d,"MainField_Item_Fruit_B_3354064484",
0x6800f882,"MainField_Item_Fruit_B_3371657879",
0x9cf185ec,"MainField_Item_Fruit_B_3379223168",
0xcc2dd4d2,"MainField_Item_Fruit_B_3384592044",
0x04f59474,"MainField_Item_Fruit_B_3397080352",
0x967a495d,"MainField_Item_Fruit_B_3411259114",
0xb4aecc3c,"MainField_Item_Fruit_B_3423170823",
0xd791634b,"MainField_Item_Fruit_B_3424605303",
0xa6176984,"MainField_Item_Fruit_B_3441994775",
0xdb33b088,"MainField_Item_Fruit_B_345652808",
0x537d5d55,"MainField_Item_Fruit_B_3462479370",
0xa8635112,"MainField_Item_Fruit_B_3469804213",
0xe6e815d3,"MainField_Item_Fruit_B_3500668909",
0xaf12cf23,"MainField_Item_Fruit_B_3503348761",
0xe29ed0b3,"MainField_Item_Fruit_B_3507313717",
0xf86abc21,"MainField_Item_Fruit_B_3515897741",
0xce6af85f,"MainField_Item_Fruit_B_3532105338",
0x082d24a8,"MainField_Item_Fruit_B_3546359573",
0x8da536d5,"MainField_Item_Fruit_B_3557614348",
0x8df8389a,"MainField_Item_Fruit_B_3560152406",
0xfaabc013,"MainField_Item_Fruit_B_356244468",
0xf55b57b6,"MainField_Item_Fruit_B_356804851",
0x039fa1a5,"MainField_Item_Fruit_B_3568632884",
0x818d7427,"MainField_Item_Fruit_B_3570054963",
0x98964883,"MainField_Item_Fruit_B_3574454862",
0xd09f604a,"MainField_Item_Fruit_B_361022722",
0xdc34914f,"MainField_Item_Fruit_B_3620353012",
0x0f8f3547,"MainField_Item_Fruit_B_3627040470",
0x91f4c3fd,"MainField_Item_Fruit_B_3641828294",
0xc2d2db8c,"MainField_Item_Fruit_B_364634968",
0xa90ac696,"MainField_Item_Fruit_B_3646624887",
0xdc5a1c26,"MainField_Item_Fruit_B_3651802870",
0xd28c0f6e,"MainField_Item_Fruit_B_3654409366",
0xb51b1644,"MainField_Item_Fruit_B_3671776076",
0xdee6aef2,"MainField_Item_Fruit_B_3672642916",
0x9a01aab2,"MainField_Item_Fruit_B_3697902743",
0x136bb192,"MainField_Item_Fruit_B_3701276529",
0x70a02a1f,"MainField_Item_Fruit_B_3711672516",
0xf53810f1,"MainField_Item_Fruit_B_3716534653",
0xc4029078,"MainField_Item_Fruit_B_3719765845",
0x35d1ce8e,"MainField_Item_Fruit_B_3732678623",
0x61188718,"MainField_Item_Fruit_B_373429754",
0xb2a537f5,"MainField_Item_Fruit_B_373444090",
0x0844a149,"MainField_Item_Fruit_B_3737637615",
0x5fb47007,"MainField_Item_Fruit_B_3742818209",
0xbf783faa,"MainField_Item_Fruit_B_3763457795",
0x02df7cb8,"MainField_Item_Fruit_B_376495058",
0x443b9712,"MainField_Item_Fruit_B_3780593047",
0xde4a65e6,"MainField_Item_Fruit_B_3790385319",
0x908e46b1,"MainField_Item_Fruit_B_3812619172",
0x605bdcfc,"MainField_Item_Fruit_B_3820596999",
0x9eaf0da1,"MainField_Item_Fruit_B_3839126302",
0xfc98a2f0,"MainField_Item_Fruit_B_3847703493",
0x232c01d5,"MainField_Item_Fruit_B_385474762",
0xfb3f7cdb,"MainField_Item_Fruit_B_3861171176",
0xc00ea72a,"MainField_Item_Fruit_B_3862593934",
0x4866f715,"MainField_Item_Fruit_B_3867243710",
0x50799588,"MainField_Item_Fruit_B_3867650981",
0xd968a4b5,"MainField_Item_Fruit_B_3907080636",
0x9bdba9a4,"MainField_Item_Fruit_B_3910019043",
0x2551e42f,"MainField_Item_Fruit_B_3920988883",
0x67662ee3,"MainField_Item_Fruit_B_3921728684",
0xcfd023c8,"MainField_Item_Fruit_B_3931310885",
0x9b4eb2a9,"MainField_Item_Fruit_B_3944089825",
0x6e888368,"MainField_Item_Fruit_B_3952899188",
0x7f246d29,"MainField_Item_Fruit_B_3957551879",
0x5f39fb9b,"MainField_Item_Fruit_B_3975262317",
0xd6be327d,"MainField_Item_Fruit_B_3980668601",
0x73aa7707,"MainField_Item_Fruit_B_3982632757",
0x11ab0e3c,"MainField_Item_Fruit_B_4004319413",
0x0d1b3588,"MainField_Item_Fruit_B_4006732775",
0xb747d846,"MainField_Item_Fruit_B_4018967973",
0x04db964c,"MainField_Item_Fruit_B_4032823796",
0x5d14d6e1,"MainField_Item_Fruit_B_4036327954",
0x00f18222,"MainField_Item_Fruit_B_404908280",
0x1ac6f5de,"MainField_Item_Fruit_B_4049592235",
0xc8858519,"MainField_Item_Fruit_B_4056479445",
0x7269b3a5,"MainField_Item_Fruit_B_4059717885",
0x0a26391f,"MainField_Item_Fruit_B_4084305492",
0x28d77fe5,"MainField_Item_Fruit_B_412720584",
0xe7802d8c,"MainField_Item_Fruit_B_412730824",
0x6d0e791e,"MainField_Item_Fruit_B_413590418",
0xaa294756,"MainField_Item_Fruit_B_4137133284",
0xbf531481,"MainField_Item_Fruit_B_4138238654",
0x10c73fa7,"MainField_Item_Fruit_B_4156115597",
0xa89820d1,"MainField_Item_Fruit_B_4165760214",
0xe0f0dffd,"MainField_Item_Fruit_B_4169054974",
0x52b07212,"MainField_Item_Fruit_B_4176970214",
0x4cff8bb5,"MainField_Item_Fruit_B_4185703358",
0xc0f83fc5,"MainField_Item_Fruit_B_4208985168",
0x0ce9e6ed,"MainField_Item_Fruit_B_4220167214",
0xd76b8c48,"MainField_Item_Fruit_B_4222976628",
0xc244619c,"MainField_Item_Fruit_B_4242032237",
0xc5a581a3,"MainField_Item_Fruit_B_4243021879",
0x7bd171c9,"MainField_Item_Fruit_B_4260471472",
0xfc249331,"MainField_Item_Fruit_B_4269093792",
0xc5a81988,"MainField_Item_Fruit_B_427209623",
0x26792415,"MainField_Item_Fruit_B_4274953872",
0xf1e06d88,"MainField_Item_Fruit_B_4276104464",
0xb2d068d2,"MainField_Item_Fruit_B_4279464671",
0x7dd61aa5,"MainField_Item_Fruit_B_4280209045",
0x4b0a9eca,"MainField_Item_Fruit_B_4280398031",
0x17c5af33,"MainField_Item_Fruit_B_4288300543",
0xbcda6f6a,"MainField_Item_Fruit_B_430430928",
0x097749fc,"MainField_Item_Fruit_B_439744627",
0x8c43b6ec,"MainField_Item_Fruit_B_451557766",
0x686c4d95,"MainField_Item_Fruit_B_473073961",
0x8d5291da,"MainField_Item_Fruit_B_487727281",
0x98223dd2,"MainField_Item_Fruit_B_490362643",
0xeb921b5f,"MainField_Item_Fruit_B_50319681",
0xddcee28a,"MainField_Item_Fruit_B_506308324",
0xdbdd7971,"MainField_Item_Fruit_B_533407627",
0x83b97178,"MainField_Item_Fruit_B_536084945",
0x4b912bd5,"MainField_Item_Fruit_B_543477417",
0xf381d50f,"MainField_Item_Fruit_B_551510937",
0x9cdeefca,"MainField_Item_Fruit_B_566765483",
0xe9fc0c99,"MainField_Item_Fruit_B_569257511",
0x81b20e4b,"MainField_Item_Fruit_B_570888235",
0x2280f240,"MainField_Item_Fruit_B_582036580",
0xb638e4d4,"MainField_Item_Fruit_B_594618162",
0x4e923ed6,"MainField_Item_Fruit_B_627758130",
0x58db317c,"MainField_Item_Fruit_B_651490377",
0xa26d5eda,"MainField_Item_Fruit_B_653855675",
0x8314fe77,"MainField_Item_Fruit_B_654931045",
0xe423b599,"MainField_Item_Fruit_B_666019712",
0x76de86a2,"MainField_Item_Fruit_B_679373696",
0x764151d6,"MainField_Item_Fruit_B_701661826",
0x8ce6af86,"MainField_Item_Fruit_B_711830727",
0x039c06d1,"MainField_Item_Fruit_B_714113296",
0x3c73628f,"MainField_Item_Fruit_B_714659617",
0x1ce50c5a,"MainField_Item_Fruit_B_716154698",
0x02983f82,"MainField_Item_Fruit_B_73230857",
0xcf386dc3,"MainField_Item_Fruit_B_771573001",
0x2dd847c9,"MainField_Item_Fruit_B_772074411",
0x77ebf192,"MainField_Item_Fruit_B_784287387",
0xbc12e146,"MainField_Item_Fruit_B_789819289",
0x18581ad6,"MainField_Item_Fruit_B_812795635",
0x73075773,"MainField_Item_Fruit_B_818525123",
0x2a14a146,"MainField_Item_Fruit_B_823705229",
0x71941874,"MainField_Item_Fruit_B_823895506",
0x2d1bdf78,"MainField_Item_Fruit_B_837738036",
0x3dadd24d,"MainField_Item_Fruit_B_855593364",
0x54ced8e6,"MainField_Item_Fruit_B_864792657",
0xf07334d3,"MainField_Item_Fruit_B_867487243",
0xd5584e10,"MainField_Item_Fruit_B_867791425",
0xe52f7927,"MainField_Item_Fruit_B_876526466",
0x5e491929,"MainField_Item_Fruit_B_884849461",
0x40af0ff0,"MainField_Item_Fruit_B_886356576",
0x4e6a99d5,"MainField_Item_Fruit_B_887518280",
0xb4ec8a51,"MainField_Item_Fruit_B_891191470",
0xf94eb10f,"MainField_Item_Fruit_B_92331490",
0xf85c175e,"MainField_Item_Fruit_B_924812010",
0x9dd133b9,"MainField_Item_Fruit_B_924834538",
0x3693e3d7,"MainField_Item_Fruit_B_933525466",
0xd0101561,"MainField_Item_Fruit_B_93459800",
0xb3a4e321,"MainField_Item_Fruit_B_964689896",
0x06c2c5d6,"MainField_Item_Fruit_B_969167576",
0xfdbfd101,"MainField_Item_Fruit_B_973242912",
0xb6cd1940,"MainField_Item_Fruit_B_991119891",
0x72f2e386,"MainField_Item_Fruit_C_1018304768",
0xacd25a07,"MainField_Item_Fruit_C_10705550",
0xc1931c35,"MainField_Item_Fruit_C_107619388",
0x1cdbd2c6,"MainField_Item_Fruit_C_1088039349",
0xfa91f111,"MainField_Item_Fruit_C_1106602838",
0xf2e97cb9,"MainField_Item_Fruit_C_1152433530",
0x0f62ad95,"MainField_Item_Fruit_C_1175672583",
0x6ecc8e28,"MainField_Item_Fruit_C_1191880032",
0x352ba80a,"MainField_Item_Fruit_C_125665049",
0xb51f175f,"MainField_Item_Fruit_C_1261226992",
0xab6014f7,"MainField_Item_Fruit_C_1311688704",
0x87ffdd86,"MainField_Item_Fruit_C_1369107319",
0x23f7854b,"MainField_Item_Fruit_C_1396244746",
0x14ccc9ef,"MainField_Item_Fruit_C_141096093",
0x83830161,"MainField_Item_Fruit_C_1441824696",
0xc8080d18,"MainField_Item_Fruit_C_1442379118",
0xe3a41917,"MainField_Item_Fruit_C_1455416647",
0xc04e29c6,"MainField_Item_Fruit_C_1495503821",
0xfdce2d85,"MainField_Item_Fruit_C_1523644233",
0xe0815547,"MainField_Item_Fruit_C_1542631193",
0xc9c80dce,"MainField_Item_Fruit_C_1554347904",
0x93c2a500,"MainField_Item_Fruit_C_1615918109",
0x2b06f804,"MainField_Item_Fruit_C_1621577614",
0x6492380f,"MainField_Item_Fruit_C_1645731443",
0x352f01ca,"MainField_Item_Fruit_C_167703919",
0x99181c60,"MainField_Item_Fruit_C_1684260094",
0xac151b03,"MainField_Item_Fruit_C_172129299",
0xd6f94b90,"MainField_Item_Fruit_C_1743158514",
0x11aa9dcc,"MainField_Item_Fruit_C_1748885412",
0xc844b09c,"MainField_Item_Fruit_C_179947593",
0xd1a4c603,"MainField_Item_Fruit_C_1806046808",
0x626ed7f7,"MainField_Item_Fruit_C_1841684293",
0x6e7849b8,"MainField_Item_Fruit_C_1864909444",
0x9f839483,"MainField_Item_Fruit_C_1911879372",
0x236d1f3d,"MainField_Item_Fruit_C_1924885555",
0x93e9d969,"MainField_Item_Fruit_C_1930970043",
0xd53379dc,"MainField_Item_Fruit_C_1935088817",
0x40ebd3d9,"MainField_Item_Fruit_C_1961056062",
0xc0c1db16,"MainField_Item_Fruit_C_1970971152",
0xb9bb2210,"MainField_Item_Fruit_C_2010728557",
0xeef2744c,"MainField_Item_Fruit_C_2027104349",
0x08c6b3bf,"MainField_Item_Fruit_C_2034194655",
0xa4f5b63c,"MainField_Item_Fruit_C_2046956681",
0x4705f709,"MainField_Item_Fruit_C_2083498536",
0x56922e2d,"MainField_Item_Fruit_C_2083537308",
0x653602a6,"MainField_Item_Fruit_C_2093705787",
0x8f422865,"MainField_Item_Fruit_C_2120777798",
0x0d3b1bc8,"MainField_Item_Fruit_C_2130298361",
0x34d9c1d0,"MainField_Item_Fruit_C_2156402151",
0x951cea50,"MainField_Item_Fruit_C_2169828981",
0x77b8887e,"MainField_Item_Fruit_C_2236163190",
0x975ba2d0,"MainField_Item_Fruit_C_2275248327",
0xd8dfcbf0,"MainField_Item_Fruit_C_230985649",
0x4022147d,"MainField_Item_Fruit_C_2409633728",
0x40d5f473,"MainField_Item_Fruit_C_2461523658",
0xdc09db07,"MainField_Item_Fruit_C_2476743186",
0x9e480757,"MainField_Item_Fruit_C_2485578160",
0xfe976aaf,"MainField_Item_Fruit_C_2490772142",
0xc62c79cf,"MainField_Item_Fruit_C_2491326584",
0x91bcfa0d,"MainField_Item_Fruit_C_2531084805",
0x035330e6,"MainField_Item_Fruit_C_2536585863",
0xd4772fbf,"MainField_Item_Fruit_C_2550148510",
0x05139b43,"MainField_Item_Fruit_C_255798276",
0x4a3ee0de,"MainField_Item_Fruit_C_2561494029",
0x0fd7c7d5,"MainField_Item_Fruit_C_2581535261",
0x9f2ca3cb,"MainField_Item_Fruit_C_2584084905",
0xde6c71bf,"MainField_Item_Fruit_C_2606639241",
0x3c719638,"MainField_Item_Fruit_C_2613223823",
0xffcd7bb0,"MainField_Item_Fruit_C_2631227292",
0x3a9fedf3,"MainField_Item_Fruit_C_2636884032",
0xd246edc1,"MainField_Item_Fruit_C_2654763895",
0x90b439cd,"MainField_Item_Fruit_C_2661226776",
0xabc962ca,"MainField_Item_Fruit_C_2676507197",
0x9716515f,"MainField_Item_Fruit_C_2773516033",
0xc590b901,"MainField_Item_Fruit_C_2791875378",
0x7c02d06f,"MainField_Item_Fruit_C_2795048160",
0x020c2f93,"MainField_Item_Fruit_C_2825905142",
0xd1123157,"MainField_Item_Fruit_C_2839585045",
0x8d5d9ea1,"MainField_Item_Fruit_C_2894172292",
0xa06f83b4,"MainField_Item_Fruit_C_2930154820",
0x4599ad85,"MainField_Item_Fruit_C_2940147547",
0x80167868,"MainField_Item_Fruit_C_2960795916",
0x8cfbe33e,"MainField_Item_Fruit_C_2996214907",
0x4c196763,"MainField_Item_Fruit_C_3000615793",
0x76fc3758,"MainField_Item_Fruit_C_3006641139",
0x4f3a6368,"MainField_Item_Fruit_C_3007800666",
0xcb6c6dde,"MainField_Item_Fruit_C_3020996035",
0xf040fe64,"MainField_Item_Fruit_C_3045989352",
0x07fc25ff,"MainField_Item_Fruit_C_3059193773",
0x3301d68b,"MainField_Item_Fruit_C_3065206718",
0x1799c2a6,"MainField_Item_Fruit_C_3075976682",
0xe5ea385d,"MainField_Item_Fruit_C_3098642207",
0x72e398bb,"MainField_Item_Fruit_C_3106873652",
0xd1c7ec21,"MainField_Item_Fruit_C_3109943197",
0x063cda67,"MainField_Item_Fruit_C_3146566473",
0x8fca1dd5,"MainField_Item_Fruit_C_3175462395",
0xbf6ac225,"MainField_Item_Fruit_C_3183386204",
0x7502ea07,"MainField_Item_Fruit_C_3195599277",
0x1546cdcb,"MainField_Item_Fruit_C_3204620181",
0xe63e8bc6,"MainField_Item_Fruit_C_3219545990",
0x90f49fb4,"MainField_Item_Fruit_C_322990641",
0x74f6c198,"MainField_Item_Fruit_C_3240985934",
0x1d1ea744,"MainField_Item_Fruit_C_3274871112",
0x3349e54c,"MainField_Item_Fruit_C_3299565827",
0xdafd92ce,"MainField_Item_Fruit_C_3306262024",
0x2f2760a2,"MainField_Item_Fruit_C_3309769089",
0x4f70c067,"MainField_Item_Fruit_C_3319934841",
0x9f80a7f2,"MainField_Item_Fruit_C_3321889192",
0x56c068b5,"MainField_Item_Fruit_C_3353672700",
0xf7c8785b,"MainField_Item_Fruit_C_33599731",
0x720c67ae,"MainField_Item_Fruit_C_3409385353",
0xbd48bc1f,"MainField_Item_Fruit_C_3415897626",
0x20687565,"MainField_Item_Fruit_C_3423738297",
0x70afcb16,"MainField_Item_Fruit_C_3467768772",
0x0a2df4fb,"MainField_Item_Fruit_C_3516808310",
0x9adb59d2,"MainField_Item_Fruit_C_3536760305",
0x65c47dfd,"MainField_Item_Fruit_C_3557282627",
0xb50a206c,"MainField_Item_Fruit_C_361303171",
0xdce0dd4b,"MainField_Item_Fruit_C_3638864934",
0xcd05de0d,"MainField_Item_Fruit_C_3710229943",
0xeeb0dd44,"MainField_Item_Fruit_C_3721804998",
0x6c329449,"MainField_Item_Fruit_C_3769624695",
0x6dd37bd6,"MainField_Item_Fruit_C_3772595934",
0x996c3e59,"MainField_Item_Fruit_C_3779303669",
0x3b78d857,"MainField_Item_Fruit_C_3783110356",
0xf335b316,"MainField_Item_Fruit_C_3823137416",
0x915d8ad1,"MainField_Item_Fruit_C_3844672570",
0x68f21e7b,"MainField_Item_Fruit_C_3855469609",
0x840995d5,"MainField_Item_Fruit_C_3860086892",
0xca9b7e05,"MainField_Item_Fruit_C_3884639815",
0xf9b87f24,"MainField_Item_Fruit_C_388687614",
0x2b806494,"MainField_Item_Fruit_C_3893792973",
0x475898fa,"MainField_Item_Fruit_C_3937761968",
0x898090dd,"MainField_Item_Fruit_C_3938954265",
0xe7072160,"MainField_Item_Fruit_C_3942262202",
0xdff2fb83,"MainField_Item_Fruit_C_3945520283",
0xfb7d5c0c,"MainField_Item_Fruit_C_3962215307",
0x520c9474,"MainField_Item_Fruit_C_3967550466",
0x669b7d34,"MainField_Item_Fruit_C_3969434641",
0x0ab9b6d6,"MainField_Item_Fruit_C_3973659386",
0x7b916713,"MainField_Item_Fruit_C_3976278335",
0xb770e49f,"MainField_Item_Fruit_C_3991052841",
0xaf03f264,"MainField_Item_Fruit_C_3997288685",
0xe7a590cb,"MainField_Item_Fruit_C_4007652991",
0x7654f5ab,"MainField_Item_Fruit_C_4022908250",
0xecb43dfb,"MainField_Item_Fruit_C_4046198996",
0x7e40451c,"MainField_Item_Fruit_C_4095641652",
0x219d5226,"MainField_Item_Fruit_C_4109183301",
0x8ed3fb74,"MainField_Item_Fruit_C_4140442757",
0xa7ea0ad8,"MainField_Item_Fruit_C_414617716",
0x45083d6a,"MainField_Item_Fruit_C_4194783581",
0x9dc9826c,"MainField_Item_Fruit_C_4227184754",
0x4af74e22,"MainField_Item_Fruit_C_4248618688",
0xb87cb8e6,"MainField_Item_Fruit_C_448982995",
0x9747b346,"MainField_Item_Fruit_C_475587334",
0x6d3131df,"MainField_Item_Fruit_C_511626942",
0x754aa01a,"MainField_Item_Fruit_C_550324021",
0x86a5646c,"MainField_Item_Fruit_C_555825079",
0x30b875ab,"MainField_Item_Fruit_C_577678664",
0x73cac296,"MainField_Item_Fruit_C_5828308",
0xbf590edf,"MainField_Item_Fruit_C_608375785",
0x87f63527,"MainField_Item_Fruit_C_652278164",
0x1e569c0f,"MainField_Item_Fruit_C_65300392",
0x4c37de65,"MainField_Item_Fruit_C_697847565",
0x4f6fa09b,"MainField_Item_Fruit_C_715457371",
0x57ecdbc3,"MainField_Item_Fruit_C_726234073",
0x610a13ae,"MainField_Item_Fruit_C_74248377",
0xc62f2db9,"MainField_Item_Fruit_C_765740107",
0x1b924c8b,"MainField_Item_Fruit_C_770890690",
0xb7fb40c1,"MainField_Item_Fruit_C_783666155",
0xb400813d,"MainField_Item_Fruit_C_793850303",
0x796839da,"MainField_Item_Fruit_C_800453036",
0x7349fcfd,"MainField_Item_Fruit_C_828080888",
0x53cedcd4,"MainField_Item_Fruit_C_831358455",
0x4d117644,"MainField_Item_Fruit_C_86359747",
0xfafeae75,"MainField_Item_Fruit_C_868921241",
0x09877b0f,"MainField_Item_Fruit_C_898652148",
0x93210457,"MainField_Item_Fruit_C_943476188",
0x43f511e5,"MainField_Item_Fruit_D_1004828570",
0xcd8b7508,"MainField_Item_Fruit_D_1025353034",
0x7a62ae95,"MainField_Item_Fruit_D_1039794760",
0x971c0854,"MainField_Item_Fruit_D_1048879599",
0x82bbd41e,"MainField_Item_Fruit_D_1076285155",
0x0d5e68fd,"MainField_Item_Fruit_D_1088814410",
0xdf330646,"MainField_Item_Fruit_D_1147164102",
0xda1e6eb3,"MainField_Item_Fruit_D_1163201319",
0x7b95c8b9,"MainField_Item_Fruit_D_118070570",
0x04a7fe62,"MainField_Item_Fruit_D_1195831653",
0x4e225df7,"MainField_Item_Fruit_D_1233359225",
0x5a304f01,"MainField_Item_Fruit_D_1243256284",
0xb49e924b,"MainField_Item_Fruit_D_1258107614",
0xdac002c2,"MainField_Item_Fruit_D_1289963276",
0x57072cec,"MainField_Item_Fruit_D_1307952378",
0x70e0e7a3,"MainField_Item_Fruit_D_1338526639",
0x9adeedf5,"MainField_Item_Fruit_D_1418446726",
0x65d258c3,"MainField_Item_Fruit_D_1442346573",
0x186d7707,"MainField_Item_Fruit_D_1466149651",
0xf966d4e4,"MainField_Item_Fruit_D_1529085567",
0xbae887a7,"MainField_Item_Fruit_D_1533213013",
0xee7c2b56,"MainField_Item_Fruit_D_1548630630",
0xf4960bd3,"MainField_Item_Fruit_D_1564314541",
0xf16e068b,"MainField_Item_Fruit_D_1580518844",
0xbbf27f27,"MainField_Item_Fruit_D_1659880848",
0xc3e091e5,"MainField_Item_Fruit_D_1668741062",
0xb701c471,"MainField_Item_Fruit_D_1720856392",
0xb076ed8b,"MainField_Item_Fruit_D_1776103167",
0x46e5ee49,"MainField_Item_Fruit_D_1857088230",
0x58c795f9,"MainField_Item_Fruit_D_1880010172",
0x5dae73c4,"MainField_Item_Fruit_D_1976761349",
0x5aff4f6a,"MainField_Item_Fruit_D_1987407293",
0xa1b60cca,"MainField_Item_Fruit_D_1991175160",
0x8312da13,"MainField_Item_Fruit_D_2006787345",
0xea31a996,"MainField_Item_Fruit_D_2017536803",
0xb21921b7,"MainField_Item_Fruit_D_2023896378",
0xf38cf88f,"MainField_Item_Fruit_D_2071589059",
0x3625ef2c,"MainField_Item_Fruit_D_2094544866",
0xea1810c2,"MainField_Item_Fruit_D_2159971274",
0x01d695fc,"MainField_Item_Fruit_D_2354760560",
0x941f9267,"MainField_Item_Fruit_D_2358038141",
0x791bcd88,"MainField_Item_Fruit_D_2463520462",
0x9707b637,"MainField_Item_Fruit_D_2478788939",
0x912f1815,"MainField_Item_Fruit_D_2503227104",
0xd53473a9,"MainField_Item_Fruit_D_2560658132",
0xccc827e3,"MainField_Item_Fruit_D_2650852496",
0x391812cd,"MainField_Item_Fruit_D_2669287485",
0xe0e16858,"MainField_Item_Fruit_D_2681032397",
0x032eeab5,"MainField_Item_Fruit_D_2684078340",
0xaec53534,"MainField_Item_Fruit_D_2751178198",
0x1702ea50,"MainField_Item_Fruit_D_2752836848",
0x4a3754b9,"MainField_Item_Fruit_D_2765892733",
0xf1079e41,"MainField_Item_Fruit_D_27908974",
0x43936276,"MainField_Item_Fruit_D_2800691526",
0x3714ae37,"MainField_Item_Fruit_D_2840105033",
0x67ebc63b,"MainField_Item_Fruit_D_2874894859",
0xd48f7f36,"MainField_Item_Fruit_D_2913640257",
0x5f74e458,"MainField_Item_Fruit_D_294985694",
0x5db24b4a,"MainField_Item_Fruit_D_2990562122",
0x17f36ac0,"MainField_Item_Fruit_D_299139938",
0xd71cbccb,"MainField_Item_Fruit_D_3010790827",
0x37582144,"MainField_Item_Fruit_D_3012731561",
0xefbe7ece,"MainField_Item_Fruit_D_303113711",
0x578e12ee,"MainField_Item_Fruit_D_3039666003",
0x60256007,"MainField_Item_Fruit_D_3110980159",
0x072361b7,"MainField_Item_Fruit_D_3129321130",
0x12f1c756,"MainField_Item_Fruit_D_3202856459",
0xf272ed54,"MainField_Item_Fruit_D_3257577413",
0x49deaf7a,"MainField_Item_Fruit_D_3295819069",
0x545f54b3,"MainField_Item_Fruit_D_3298136639",
0x8e591dbd,"MainField_Item_Fruit_D_3309792220",
0xd32d281d,"MainField_Item_Fruit_D_3386934941",
0x7dabb72f,"MainField_Item_Fruit_D_3404681765",
0xc1281236,"MainField_Item_Fruit_D_3447887420",
0xd0b5dafd,"MainField_Item_Fruit_D_3463117481",
0xc246df36,"MainField_Item_Fruit_D_3483996422",
0xe207fc7e,"MainField_Item_Fruit_D_3489818121",
0x7cc8240a,"MainField_Item_Fruit_D_3498754243",
0x25c17cbb,"MainField_Item_Fruit_D_3551693902",
0xf05ecabe,"MainField_Item_Fruit_D_3656166640",
0xbe7a68cc,"MainField_Item_Fruit_D_3697040029",
0x74991555,"MainField_Item_Fruit_D_3711261976",
0xac93bcd7,"MainField_Item_Fruit_D_3837288925",
0x31d2e559,"MainField_Item_Fruit_D_3894225067",
0xe1c3e90c,"MainField_Item_Fruit_D_3905584731",
0x2d5017e7,"MainField_Item_Fruit_D_3909614598",
0x6506351a,"MainField_Item_Fruit_D_4109807398",
0x996d37a3,"MainField_Item_Fruit_D_4130157273",
0x5606c9f6,"MainField_Item_Fruit_D_4140707008",
0x67e0057d,"MainField_Item_Fruit_D_4156144476",
0xaf31748f,"MainField_Item_Fruit_D_4217240550",
0x799b2e3f,"MainField_Item_Fruit_D_4222063360",
0x6e89ee02,"MainField_Item_Fruit_D_4235145977",
0xbbc6fbbd,"MainField_Item_Fruit_D_4241119001",
0x14d8351c,"MainField_Item_Fruit_D_4262459255",
0x407eb606,"MainField_Item_Fruit_D_431479408",
0x1d5e7e84,"MainField_Item_Fruit_D_474961880",
0x4dea8e4f,"MainField_Item_Fruit_D_47565971",
0xf3c01b44,"MainField_Item_Fruit_D_497009911",
0x495605da,"MainField_Item_Fruit_D_50565528",
0x4c9d43a2,"MainField_Item_Fruit_D_543464325",
0x407bbcb7,"MainField_Item_Fruit_D_545465256",
0xce274aa7,"MainField_Item_Fruit_D_587155163",
0x54bc8939,"MainField_Item_Fruit_D_596440848",
0x75eaa7af,"MainField_Item_Fruit_D_631885523",
0xe23167a4,"MainField_Item_Fruit_D_6576435",
0xc3bd5b50,"MainField_Item_Fruit_D_694187683",
0xa5c25aa2,"MainField_Item_Fruit_D_708492091",
0x71f81b62,"MainField_Item_Fruit_D_714863379",
0x0e769c19,"MainField_Item_Fruit_D_726207216",
0x09b2dc3c,"MainField_Item_Fruit_D_740609769",
0x901eda32,"MainField_Item_Fruit_D_809615859",
0x17b9ddf5,"MainField_Item_Fruit_D_873111881",
0x9e18d900,"MainField_Item_Fruit_D_907006766",
0x4452bac7,"MainField_Item_Fruit_D_971597302",
0x7bd9f3d7,"MainField_Item_Fruit_D_989000812",
0xd3435436,"MainField_Item_Fruit_E_1063179118",
0x660967b3,"MainField_Item_Fruit_E_1115951813",
0x30cd4017,"MainField_Item_Fruit_E_1206318279",
0x35400309,"MainField_Item_Fruit_E_1213850616",
0x63c01ade,"MainField_Item_Fruit_E_1261592302",
0xbde4e65a,"MainField_Item_Fruit_E_1281334007",
0xa2260ab3,"MainField_Item_Fruit_E_1313913068",
0x666355f1,"MainField_Item_Fruit_E_1344687540",
0xa255a942,"MainField_Item_Fruit_E_1363333817",
0x4ab1b0d1,"MainField_Item_Fruit_E_1369795220",
0xc7ca0271,"MainField_Item_Fruit_E_138974990",
0x8fd4386d,"MainField_Item_Fruit_E_1433686507",
0xe1973885,"MainField_Item_Fruit_E_1441064891",
0x95199c1f,"MainField_Item_Fruit_E_1445926560",
0x6b2a6bc9,"MainField_Item_Fruit_E_1484047878",
0xa01373d6,"MainField_Item_Fruit_E_1501557905",
0x644375b6,"MainField_Item_Fruit_E_1578250376",
0xd43b2f69,"MainField_Item_Fruit_E_1626912000",
0x7544408a,"MainField_Item_Fruit_E_1650203586",
0xbed66eec,"MainField_Item_Fruit_E_1652510329",
0x6eef6060,"MainField_Item_Fruit_E_1668591675",
0x3f7bd2b4,"MainField_Item_Fruit_E_1679389730",
0x72c1f949,"MainField_Item_Fruit_E_168130037",
0xaa2e3b90,"MainField_Item_Fruit_E_1771013309",
0x0ca506b3,"MainField_Item_Fruit_E_1772164755",
0x3b3b2c95,"MainField_Item_Fruit_E_177425307",
0x8bb6b3c4,"MainField_Item_Fruit_E_1832453735",
0x9aad63b8,"MainField_Item_Fruit_E_1917921957",
0xaa39e25c,"MainField_Item_Fruit_E_193863670",
0x93caeb7d,"MainField_Item_Fruit_E_1966927548",
0x6fbfd216,"MainField_Item_Fruit_E_1967088459",
0xb50f432e,"MainField_Item_Fruit_E_1978379468",
0x4832f1d6,"MainField_Item_Fruit_E_2012836325",
0x941fb980,"MainField_Item_Fruit_E_2054127994",
0xd4adceab,"MainField_Item_Fruit_E_2097300835",
0xa2b9f89d,"MainField_Item_Fruit_E_2135926680",
0x7ed9a5a9,"MainField_Item_Fruit_E_2160243622",
0x27a92446,"MainField_Item_Fruit_E_2228002108",
0xcc6b9d33,"MainField_Item_Fruit_E_2244436862",
0xaf40f07a,"MainField_Item_Fruit_E_224965100",
0x40229c39,"MainField_Item_Fruit_E_2290630920",
0x2bd03afe,"MainField_Item_Fruit_E_2296685954",
0xbf1c5b87,"MainField_Item_Fruit_E_2308640821",
0x56da5a15,"MainField_Item_Fruit_E_2373636375",
0x29d781bb,"MainField_Item_Fruit_E_2386012602",
0x00f7610b,"MainField_Item_Fruit_E_2398540844",
0xae961726,"MainField_Item_Fruit_E_2458174395",
0x62de76f2,"MainField_Item_Fruit_E_2467046479",
0x5002dc97,"MainField_Item_Fruit_E_2476349985",
0xd1884973,"MainField_Item_Fruit_E_2489442390",
0x366c4c68,"MainField_Item_Fruit_E_2499272248",
0x0d9d2c02,"MainField_Item_Fruit_E_2518223997",
0x399bd841,"MainField_Item_Fruit_E_2583105737",
0x7fb35627,"MainField_Item_Fruit_E_2603680359",
0xec8802e4,"MainField_Item_Fruit_E_2615528928",
0xc4f388d4,"MainField_Item_Fruit_E_2623325822",
0xa07cb529,"MainField_Item_Fruit_E_2623490953",
0x50ee18d8,"MainField_Item_Fruit_E_2723281883",
0x7f60a4da,"MainField_Item_Fruit_E_2772189122",
0x312bbca1,"MainField_Item_Fruit_E_2776353762",
0x1e4fdf72,"MainField_Item_Fruit_E_2790752980",
0x22f1254b,"MainField_Item_Fruit_E_2835833061",
0x5d78e713,"MainField_Item_Fruit_E_2894210172",
0x810bd3da,"MainField_Item_Fruit_E_2912201331",
0xf4ca2e55,"MainField_Item_Fruit_E_2950811391",
0xe304db83,"MainField_Item_Fruit_E_2954645924",
0xd759311b,"MainField_Item_Fruit_E_300807224",
0x35d38c94,"MainField_Item_Fruit_E_3050747528",
0xa472dafc,"MainField_Item_Fruit_E_3061491498",
0xcf2e2cdc,"MainField_Item_Fruit_E_3069902782",
0x5adde95d,"MainField_Item_Fruit_E_3077674429",
0x43006a94,"MainField_Item_Fruit_E_3089201036",
0x58bcefbb,"MainField_Item_Fruit_E_3094084175",
0x2413b9f6,"MainField_Item_Fruit_E_3155686622",
0xa3c2a0e1,"MainField_Item_Fruit_E_320750772",
0x061b08e8,"MainField_Item_Fruit_E_3209474965",
0xf9893ce3,"MainField_Item_Fruit_E_3215657912",
0xeab9589d,"MainField_Item_Fruit_E_3229001003",
0x201e931d,"MainField_Item_Fruit_E_3246233532",
0xdde4fde4,"MainField_Item_Fruit_E_3311139370",
0xa3bd967a,"MainField_Item_Fruit_E_3360391939",
0x787928b9,"MainField_Item_Fruit_E_3366730542",
0xe57f1a96,"MainField_Item_Fruit_E_3406873672",
0x338894c6,"MainField_Item_Fruit_E_3475392282",
0x278276ce,"MainField_Item_Fruit_E_3511726658",
0xc6493191,"MainField_Item_Fruit_E_3579235149",
0xeb472a63,"MainField_Item_Fruit_E_360218351",
0x6d4bbf4d,"MainField_Item_Fruit_E_3612993878",
0x7c19d9dd,"MainField_Item_Fruit_E_3630058562",
0xdccd7dd0,"MainField_Item_Fruit_E_3639017065",
0x184d0602,"MainField_Item_Fruit_E_3667106533",
0x5fea60bf,"MainField_Item_Fruit_E_3682931946",
0x690b7419,"MainField_Item_Fruit_E_3724562547",
0x70118d03,"MainField_Item_Fruit_E_37245917",
0xe7b32b64,"MainField_Item_Fruit_E_37477930",
0x14b99bb8,"MainField_Item_Fruit_E_3815166144",
0x9fe97145,"MainField_Item_Fruit_E_3824217774",
0xf946d097,"MainField_Item_Fruit_E_3825931481",
0x6b3af6a2,"MainField_Item_Fruit_E_3835505335",
0xad1b6e17,"MainField_Item_Fruit_E_3850629933",
0x087b56f6,"MainField_Item_Fruit_E_3962945286",
0x35ca7c5a,"MainField_Item_Fruit_E_3974298998",
0x1572c63e,"MainField_Item_Fruit_E_4008714335",
0xf4c5634b,"MainField_Item_Fruit_E_402621846",
0x6990f16c,"MainField_Item_Fruit_E_4035368199",
0x2ebb1382,"MainField_Item_Fruit_E_4064331689",
0xc3329962,"MainField_Item_Fruit_E_4072690664",
0x7895cd3f,"MainField_Item_Fruit_E_4090211754",
0xe9f23ff1,"MainField_Item_Fruit_E_4156916528",
0xbcc8de96,"MainField_Item_Fruit_E_4180711724",
0xadcf5b32,"MainField_Item_Fruit_E_4202553729",
0x9ad1edc7,"MainField_Item_Fruit_E_4218822595",
0xc7cfb7d4,"MainField_Item_Fruit_E_4231564897",
0x854724c4,"MainField_Item_Fruit_E_4245693848",
0x6b5ddd2a,"MainField_Item_Fruit_E_4293104916",
0xa66d264c,"MainField_Item_Fruit_E_469216539",
0x8a574806,"MainField_Item_Fruit_E_474916485",
0xdf0cb71f,"MainField_Item_Fruit_E_512275499",
0x112cad9f,"MainField_Item_Fruit_E_514205189",
0x5e5b11c5,"MainField_Item_Fruit_E_547572789",
0x35f9c363,"MainField_Item_Fruit_E_556279350",
0xc7a32b64,"MainField_Item_Fruit_E_585357101",
0x5aa867ba,"MainField_Item_Fruit_E_622765412",
0x11b68122,"MainField_Item_Fruit_E_642114095",
0xbdf98f2e,"MainField_Item_Fruit_E_648174082",
0xfb573f47,"MainField_Item_Fruit_E_669557809",
0x515d4c8f,"MainField_Item_Fruit_E_748163858",
0x15dff921,"MainField_Item_Fruit_E_773835350",
0x7b39ee26,"MainField_Item_Fruit_E_779666439",
0x70ff39d7,"MainField_Item_Fruit_E_820102225",
0xca06e2aa,"MainField_Item_Fruit_E_89528883",
0xb77bb93a,"MainField_Item_Fruit_E_942872439",
0xc32abcd1,"MainField_Item_Fruit_F_1041363837",
0x7b8082a9,"MainField_Item_Fruit_F_1073579007",
0x6d83b8eb,"MainField_Item_Fruit_F_1081733341",
0xe72abb10,"MainField_Item_Fruit_F_1090964007",
0xa987be7a,"MainField_Item_Fruit_F_110027823",
0xb96258c5,"MainField_Item_Fruit_F_1117586395",
0x4e07a0ca,"MainField_Item_Fruit_F_1144281799",
0x31562ab5,"MainField_Item_Fruit_F_1153956790",
0xb1f6be7e,"MainField_Item_Fruit_F_1207904133",
0xc193b08a,"MainField_Item_Fruit_F_1231806878",
0xa894cd14,"MainField_Item_Fruit_F_126803560",
0xc3efa8be,"MainField_Item_Fruit_F_1306899778",
0xb41d7afe,"MainField_Item_Fruit_F_1330011967",
0x7a649983,"MainField_Item_Fruit_F_1342459228",
0xd5a36e9c,"MainField_Item_Fruit_F_139820350",
0x936cea50,"MainField_Item_Fruit_F_1427423147",
0xf3e5cd6e,"MainField_Item_Fruit_F_1465095621",
0xff8b65a4,"MainField_Item_Fruit_F_1471632854",
0x4855845e,"MainField_Item_Fruit_F_1490019151",
0x0713d543,"MainField_Item_Fruit_F_1512265272",
0x2724f684,"MainField_Item_Fruit_F_1517029682",
0xf249ab6d,"MainField_Item_Fruit_F_1543522680",
0x004e5b67,"MainField_Item_Fruit_F_1547810707",
0xfb6cb998,"MainField_Item_Fruit_F_1559488521",
0x82494379,"MainField_Item_Fruit_F_1595000536",
0x3e1c216d,"MainField_Item_Fruit_F_1608673193",
0x72087829,"MainField_Item_Fruit_F_167137974",
0x91f4ba99,"MainField_Item_Fruit_F_1677438705",
0xa8e76e5a,"MainField_Item_Fruit_F_1692311745",
0x1abc16e5,"MainField_Item_Fruit_F_1719487164",
0x5ad5c074,"MainField_Item_Fruit_F_1758363349",
0x521afb4e,"MainField_Item_Fruit_F_1776368694",
0x609ad02c,"MainField_Item_Fruit_F_1818744955",
0xd815d2d9,"MainField_Item_Fruit_F_1829719876",
0xddebef43,"MainField_Item_Fruit_F_1837869305",
0x8f3a6212,"MainField_Item_Fruit_F_1845292597",
0x1007c594,"MainField_Item_Fruit_F_1890969565",
0x1f73f2fa,"MainField_Item_Fruit_F_190167243",
0x733d8ade,"MainField_Item_Fruit_F_1916757650",
0x535bdf31,"MainField_Item_Fruit_F_1925823130",
0x8e726788,"MainField_Item_Fruit_F_1944838858",
0x78c1310b,"MainField_Item_Fruit_F_1946207272",
0xfd52b544,"MainField_Item_Fruit_F_1966824699",
0xe844abe2,"MainField_Item_Fruit_F_1973314051",
0x2971a00a,"MainField_Item_Fruit_F_1976106378",
0x9e4ad518,"MainField_Item_Fruit_F_1982518107",
0xebb67a35,"MainField_Item_Fruit_F_1987092985",
0x1a3b1a1e,"MainField_Item_Fruit_F_1993962026",
0xb08e0263,"MainField_Item_Fruit_F_2000521342",
0x674147ae,"MainField_Item_Fruit_F_2023882484",
0x3e3184c3,"MainField_Item_Fruit_F_2048757940",
0x3b60b4cc,"MainField_Item_Fruit_F_2067989912",
0x9e824a0d,"MainField_Item_Fruit_F_2099714745",
0x3822b44a,"MainField_Item_Fruit_F_2123120725",
0x1e9421fe,"MainField_Item_Fruit_F_2134486739",
0x7723be4f,"MainField_Item_Fruit_F_2146838125",
0x1d158f6b,"MainField_Item_Fruit_F_2155590806",
0x46a1f44f,"MainField_Item_Fruit_F_2199116827",
0xe15b1b5b,"MainField_Item_Fruit_F_2235458234",
0x4fb35bd6,"MainField_Item_Fruit_F_2243593479",
0x38727c97,"MainField_Item_Fruit_F_2253958102",
0x7fe13eb2,"MainField_Item_Fruit_F_2255616069",
0x89d307e4,"MainField_Item_Fruit_F_2296725086",
0x40e716f5,"MainField_Item_Fruit_F_2325877808",
0xaef5ad77,"MainField_Item_Fruit_F_2395504895",
0x36864d21,"MainField_Item_Fruit_F_2417243319",
0xecbb66b2,"MainField_Item_Fruit_F_243647247",
0xdb1d0ecd,"MainField_Item_Fruit_F_2441777820",
0x70384db8,"MainField_Item_Fruit_F_2493145195",
0x122072d3,"MainField_Item_Fruit_F_2520353302",
0x7a6dde33,"MainField_Item_Fruit_F_2539976220",
0x6e99182e,"MainField_Item_Fruit_F_2571189391",
0xe6fe4b55,"MainField_Item_Fruit_F_2591846617",
0x2895a497,"MainField_Item_Fruit_F_2611841528",
0x872aaca4,"MainField_Item_Fruit_F_2615334642",
0xc9dde6d8,"MainField_Item_Fruit_F_2642229193",
0x7a86aa7f,"MainField_Item_Fruit_F_2647615571",
0x7f8f443d,"MainField_Item_Fruit_F_2657925225",
0x7eebfa8c,"MainField_Item_Fruit_F_266071063",
0x736a0c38,"MainField_Item_Fruit_F_2715691997",
0x84b32db4,"MainField_Item_Fruit_F_2725605681",
0x8252f403,"MainField_Item_Fruit_F_2744713651",
0xb9a0c383,"MainField_Item_Fruit_F_2775420184",
0x5a5ced85,"MainField_Item_Fruit_F_2816118194",
0xc856765a,"MainField_Item_Fruit_F_2817758588",
0x0c22ebd4,"MainField_Item_Fruit_F_2847080548",
0x3165f9f6,"MainField_Item_Fruit_F_285694047",
0x7d661fcf,"MainField_Item_Fruit_F_2886039353",
0x58a0f9b6,"MainField_Item_Fruit_F_2893165045",
0x85a9b582,"MainField_Item_Fruit_F_2945457956",
0x4ca5f46b,"MainField_Item_Fruit_F_2979606238",
0x7cd02680,"MainField_Item_Fruit_F_2992979210",
0x539a3c50,"MainField_Item_Fruit_F_3031794491",
0xd3413d40,"MainField_Item_Fruit_F_3035921872",
0x9a9f5170,"MainField_Item_Fruit_F_3038252370",
0x39d98010,"MainField_Item_Fruit_F_3080981659",
0x5ecf29c9,"MainField_Item_Fruit_F_311752278",
0xd6230834,"MainField_Item_Fruit_F_3134052952",
0xc5fd0cb7,"MainField_Item_Fruit_F_3165242745",
0x3c74c6ac,"MainField_Item_Fruit_F_3181475304",
0x0b075221,"MainField_Item_Fruit_F_3255879960",
0x88f951bd,"MainField_Item_Fruit_F_327368784",
0x4da61d8f,"MainField_Item_Fruit_F_3280891699",
0x2e392acc,"MainField_Item_Fruit_F_3284217913",
0x2f488d5e,"MainField_Item_Fruit_F_3315847570",
0x6d18aed9,"MainField_Item_Fruit_F_3337855193",
0x31b5251a,"MainField_Item_Fruit_F_3338681327",
0xc37d039a,"MainField_Item_Fruit_F_3363343734",
0x75a49c76,"MainField_Item_Fruit_F_3386614526",
0x290912f2,"MainField_Item_Fruit_F_3387304797",
0x9bec9df6,"MainField_Item_Fruit_F_3403376395",
0x48f11b6f,"MainField_Item_Fruit_F_3418843795",
0x9f2425f6,"MainField_Item_Fruit_F_342195201",
0x0d704db1,"MainField_Item_Fruit_F_3461690327",
0x2e9cd9e7,"MainField_Item_Fruit_F_3473611983",
0xe91354e3,"MainField_Item_Fruit_F_3498674079",
0x102c1234,"MainField_Item_Fruit_F_3524108176",
0x3a25f090,"MainField_Item_Fruit_F_3588989889",
0xa6374d18,"MainField_Item_Fruit_F_3601086782",
0x476a4c9b,"MainField_Item_Fruit_F_3625389399",
0xfdcd4a5d,"MainField_Item_Fruit_F_3636589428",
0x2cf10040,"MainField_Item_Fruit_F_3644346292",
0xede2a85c,"MainField_Item_Fruit_F_3682559450",
0xa4ecb423,"MainField_Item_Fruit_F_3713194875",
0xe6929830,"MainField_Item_Fruit_F_3759347212",
0x14e82fe2,"MainField_Item_Fruit_F_3867301590",
0xb405af9a,"MainField_Item_Fruit_F_3871582269",
0x12aebf90,"MainField_Item_Fruit_F_38793209",
0x044c62e2,"MainField_Item_Fruit_F_3882755668",
0x668edaea,"MainField_Item_Fruit_F_3914826204",
0x3912f20f,"MainField_Item_Fruit_F_3926492363",
0x21699f1c,"MainField_Item_Fruit_F_3960682025",
0x5edd9c82,"MainField_Item_Fruit_F_3968542774",
0x79211c04,"MainField_Item_Fruit_F_3996855916",
0x6ebcad8e,"MainField_Item_Fruit_F_4013137661",
0xbaa0914a,"MainField_Item_Fruit_F_4013626411",
0x1007715c,"MainField_Item_Fruit_F_4026751543",
0xdb932f27,"MainField_Item_Fruit_F_4031761654",
0x7fcfc527,"MainField_Item_Fruit_F_4057971109",
0x910bd2f4,"MainField_Item_Fruit_F_4058983093",
0x77a3d8f3,"MainField_Item_Fruit_F_4064571481",
0x8aa1176f,"MainField_Item_Fruit_F_4100574968",
0x5f4cd2c0,"MainField_Item_Fruit_F_4168907195",
0x8437c057,"MainField_Item_Fruit_F_4224002914",
0xdd27a3fa,"MainField_Item_Fruit_F_426406886",
0x9b8010cc,"MainField_Item_Fruit_F_4288621757",
0x0dac5886,"MainField_Item_Fruit_F_429511319",
0x88ed217e,"MainField_Item_Fruit_F_434893311",
0x113f3bf3,"MainField_Item_Fruit_F_452049434",
0xd2f22cca,"MainField_Item_Fruit_F_45895304",
0xfb4d9c1c,"MainField_Item_Fruit_F_480985719",
0x3ee47601,"MainField_Item_Fruit_F_484181179",
0x8520f105,"MainField_Item_Fruit_F_498624253",
0xfd25b625,"MainField_Item_Fruit_F_509375341",
0x4835bf64,"MainField_Item_Fruit_F_511700678",
0x200b9c0e,"MainField_Item_Fruit_F_559145246",
0x340b39c6,"MainField_Item_Fruit_F_563988940",
0x84d6d149,"MainField_Item_Fruit_F_590330969",
0xae6f03bf,"MainField_Item_Fruit_F_60692184",
0x57d8a979,"MainField_Item_Fruit_F_614978554",
0x574d01c2,"MainField_Item_Fruit_F_637811079",
0x6a444864,"MainField_Item_Fruit_F_659388413",
0xfd36d4c4,"MainField_Item_Fruit_F_664444844",
0xeffd7052,"MainField_Item_Fruit_F_685334044",
0xb092e514,"MainField_Item_Fruit_F_693471825",
0x93f574d2,"MainField_Item_Fruit_F_70061650",
0x46ff2db1,"MainField_Item_Fruit_F_70357497",
0x97fe3f58,"MainField_Item_Fruit_F_725329264",
0xb1db7552,"MainField_Item_Fruit_F_729086586",
0x989731eb,"MainField_Item_Fruit_F_730290955",
0xdfec4fc2,"MainField_Item_Fruit_F_745307625",
0x3a71353a,"MainField_Item_Fruit_F_756423633",
0xd358ea30,"MainField_Item_Fruit_F_760735034",
0xb681e5f2,"MainField_Item_Fruit_F_769942167",
0xb522e836,"MainField_Item_Fruit_F_77550024",
0x99e65c72,"MainField_Item_Fruit_F_776366746",
0xe3c2f40e,"MainField_Item_Fruit_F_784154941",
0x3a8b7b2f,"MainField_Item_Fruit_F_80334401",
0x24b151c0,"MainField_Item_Fruit_F_810640997",
0xeda27215,"MainField_Item_Fruit_F_811726197",
0x5f9f3df8,"MainField_Item_Fruit_F_815974164",
0x7d4e225a,"MainField_Item_Fruit_F_895743621",
0x3ba942a3,"MainField_Item_Fruit_F_904335672",
0x72f631f3,"MainField_Item_Fruit_F_915787336",
0xa4e09909,"MainField_Item_Fruit_F_927267653",
0x30330c72,"MainField_Item_Fruit_F_978056354",
0x66251d51,"MainField_Item_Fruit_F_981166002",
0xbf313d45,"MainField_Item_Fruit_G_1005529244",
0xb3b417be,"MainField_Item_Fruit_G_1010225291",
0x4869fafe,"MainField_Item_Fruit_G_1017955378",
0xa3632d7b,"MainField_Item_Fruit_G_1021570754",
0x8c5a8708,"MainField_Item_Fruit_G_1026935969",
0xb3381f01,"MainField_Item_Fruit_G_1046777662",
0x6a6ee4b8,"MainField_Item_Fruit_G_1070273187",
0xba1f10bc,"MainField_Item_Fruit_G_1113110709",
0x0557fa36,"MainField_Item_Fruit_G_1124363140",
0xdf82d13e,"MainField_Item_Fruit_G_114459069",
0xd755240d,"MainField_Item_Fruit_G_1145310658",
0x7e8b628b,"MainField_Item_Fruit_G_1154057531",
0xa8f6452d,"MainField_Item_Fruit_G_116796777",
0xf3c01f9f,"MainField_Item_Fruit_G_1202302266",
0x63f2dc0f,"MainField_Item_Fruit_G_122796140",
0x0768f328,"MainField_Item_Fruit_G_1231273896",
0xbf12c458,"MainField_Item_Fruit_G_1240173923",
0x7a3ddc13,"MainField_Item_Fruit_G_1248772671",
0xb0b8acc4,"MainField_Item_Fruit_G_1269560484",
0x15e4bac5,"MainField_Item_Fruit_G_1271022207",
0xe45dd13e,"MainField_Item_Fruit_G_1272397707",
0xb897aa72,"MainField_Item_Fruit_G_1273290324",
0x6db5ba41,"MainField_Item_Fruit_G_1278982610",
0xfce020a4,"MainField_Item_Fruit_G_1298085934",
0x3df99cea,"MainField_Item_Fruit_G_1304795451",
0xeabb6a5c,"MainField_Item_Fruit_G_1322942391",
0xd58f1d7a,"MainField_Item_Fruit_G_1328933024",
0xb4cccc86,"MainField_Item_Fruit_G_1334613926",
0xc35bca12,"MainField_Item_Fruit_G_1340558247",
0x79d41691,"MainField_Item_Fruit_G_1343685647",
0x8e12d410,"MainField_Item_Fruit_G_1349063753",
0x40d56197,"MainField_Item_Fruit_G_1349492246",
0x2150ee6e,"MainField_Item_Fruit_G_1360288833",
0xdafc7d45,"MainField_Item_Fruit_G_136514120",
0xc3a15deb,"MainField_Item_Fruit_G_137516082",
0xfa8436c9,"MainField_Item_Fruit_G_1383462425",
0xd5f0680f,"MainField_Item_Fruit_G_1405042864",
0xf397c2f2,"MainField_Item_Fruit_G_1417071405",
0x4c6caad5,"MainField_Item_Fruit_G_1423082665",
0xd93131f7,"MainField_Item_Fruit_G_1470161876",
0x6055e645,"MainField_Item_Fruit_G_1475539677",
0xdf7fd026,"MainField_Item_Fruit_G_1480632981",
0x72149833,"MainField_Item_Fruit_G_1501741218",
0x65faad9f,"MainField_Item_Fruit_G_1506517744",
0x645fe9eb,"MainField_Item_Fruit_G_1526670881",
0x84ad0a72,"MainField_Item_Fruit_G_1529466943",
0x0808f8ed,"MainField_Item_Fruit_G_1530366528",
0x1204fbc4,"MainField_Item_Fruit_G_1545045752",
0x40830f7b,"MainField_Item_Fruit_G_1569089580",
0x09cf7338,"MainField_Item_Fruit_G_1581563638",
0x80924e95,"MainField_Item_Fruit_G_1583076544",
0x45aaa58b,"MainField_Item_Fruit_G_158662030",
0x83415b58,"MainField_Item_Fruit_G_160363481",
0x1b67512f,"MainField_Item_Fruit_G_1644109458",
0x5d087813,"MainField_Item_Fruit_G_1646356109",
0x247b001b,"MainField_Item_Fruit_G_166151426",
0x69676e0c,"MainField_Item_Fruit_G_1664677959",
0x3b5551a3,"MainField_Item_Fruit_G_1680995839",
0x3e31b365,"MainField_Item_Fruit_G_1686793686",
0xf643dea5,"MainField_Item_Fruit_G_1698340404",
0xf49462e4,"MainField_Item_Fruit_G_1699423138",
0x08a4e874,"MainField_Item_Fruit_G_1716058441",
0x7bdbe918,"MainField_Item_Fruit_G_1716774770",
0x2d575ded,"MainField_Item_Fruit_G_1724763695",
0xaf5c973d,"MainField_Item_Fruit_G_1730846927",
0xb27c6b87,"MainField_Item_Fruit_G_1804377813",
0xb7743a2e,"MainField_Item_Fruit_G_1816223839",
0x3a2ed875,"MainField_Item_Fruit_G_1822930406",
0xe579165a,"MainField_Item_Fruit_G_183511520",
0xd2c134f6,"MainField_Item_Fruit_G_1841581329",
0x50400052,"MainField_Item_Fruit_G_1842646365",
0x02c4dfe4,"MainField_Item_Fruit_G_1873522894",
0x0cc63fd6,"MainField_Item_Fruit_G_1880945431",
0x63ca0ff1,"MainField_Item_Fruit_G_1884719354",
0xbda527fb,"MainField_Item_Fruit_G_1886131068",
0x60c8e906,"MainField_Item_Fruit_G_1924476027",
0x3cf5f10e,"MainField_Item_Fruit_G_1991800114",
0xe0d6b472,"MainField_Item_Fruit_G_1999800016",
0x5cc8ae90,"MainField_Item_Fruit_G_2000375011",
0x5daaa109,"MainField_Item_Fruit_G_2001009733",
0xe303edcf,"MainField_Item_Fruit_G_2019074237",
0x72b256f4,"MainField_Item_Fruit_G_2034807470",
0xb8f61f1a,"MainField_Item_Fruit_G_2039918793",
0xffcf04b3,"MainField_Item_Fruit_G_2041233212",
0xc1023fbc,"MainField_Item_Fruit_G_2061814567",
0x48ee5cb4,"MainField_Item_Fruit_G_2106678423",
0x44fd4278,"MainField_Item_Fruit_G_2109031904",
0x7fc239c8,"MainField_Item_Fruit_G_2122957647",
0x6ba146fa,"MainField_Item_Fruit_G_2154063715",
0x9d8fbeab,"MainField_Item_Fruit_G_216037971",
0xabbeab90,"MainField_Item_Fruit_G_2171574693",
0x01cc6e85,"MainField_Item_Fruit_G_2185250386",
0x984c49e7,"MainField_Item_Fruit_G_2188954192",
0xd4090834,"MainField_Item_Fruit_G_2196537204",
0xe68d9a76,"MainField_Item_Fruit_G_2205931205",
0x57472393,"MainField_Item_Fruit_G_224557271",
0xbaa7b9b4,"MainField_Item_Fruit_G_2256884526",
0xbb4dbb54,"MainField_Item_Fruit_G_2260554705",
0xdbf84ef6,"MainField_Item_Fruit_G_2268882993",
0xd9b6c800,"MainField_Item_Fruit_G_2270993212",
0x7a4c7bd7,"MainField_Item_Fruit_G_2287830622",
0xe433cdc8,"MainField_Item_Fruit_G_2300887523",
0x13e68e7f,"MainField_Item_Fruit_G_2309630114",
0x62f1fff2,"MainField_Item_Fruit_G_2317445371",
0x7f708754,"MainField_Item_Fruit_G_2334103777",
0x7795c92a,"MainField_Item_Fruit_G_2348894076",
0x2c36cf23,"MainField_Item_Fruit_G_2358544705",
0x39b89aa8,"MainField_Item_Fruit_G_2370612450",
0xabcf204e,"MainField_Item_Fruit_G_2398234608",
0x39262382,"MainField_Item_Fruit_G_2398271531",
0x134238df,"MainField_Item_Fruit_G_2408928926",
0xfdbc6e9e,"MainField_Item_Fruit_G_2457317924",
0x5ae6e0f6,"MainField_Item_Fruit_G_2498406923",
0x54080b02,"MainField_Item_Fruit_G_2498464346",
0x3e846cd7,"MainField_Item_Fruit_G_2502475836",
0x78083385,"MainField_Item_Fruit_G_2522162879",
0x31697d51,"MainField_Item_Fruit_G_256252853",
0xd772534d,"MainField_Item_Fruit_G_2570399183",
0x5f57c0f0,"MainField_Item_Fruit_G_2570905596",
0x7f1f33ba,"MainField_Item_Fruit_G_257921067",
0xb5f5a7e9,"MainField_Item_Fruit_G_2579241239",
0xb8547441,"MainField_Item_Fruit_G_2593116571",
0x624920c2,"MainField_Item_Fruit_G_2608819066",
0x0d5ca4f9,"MainField_Item_Fruit_G_2619375541",
0x3cd9db26,"MainField_Item_Fruit_G_2625575681",
0x7338f495,"MainField_Item_Fruit_G_2626026493",
0x1609e998,"MainField_Item_Fruit_G_2629520727",
0x8050c7f9,"MainField_Item_Fruit_G_2630196872",
0xd441aaff,"MainField_Item_Fruit_G_2681982983",
0x00682ea8,"MainField_Item_Fruit_G_2684361373",
0x62155b8b,"MainField_Item_Fruit_G_2698256027",
0x0c4ca6b2,"MainField_Item_Fruit_G_2721771399",
0x86a86d0f,"MainField_Item_Fruit_G_2745149625",
0x92b6c9cb,"MainField_Item_Fruit_G_2748053527",
0x33f27537,"MainField_Item_Fruit_G_2755435803",
0x6cb5b847,"MainField_Item_Fruit_G_2774314843",
0x8068b2db,"MainField_Item_Fruit_G_279148824",
0xbf148297,"MainField_Item_Fruit_G_2803826985",
0xf51bd165,"MainField_Item_Fruit_G_2808995460",
0x606ea140,"MainField_Item_Fruit_G_281217561",
0x5fdce95f,"MainField_Item_Fruit_G_2824078218",
0x8053f165,"MainField_Item_Fruit_G_2846346262",
0xdadd2208,"MainField_Item_Fruit_G_2851839511",
0xc33e69f2,"MainField_Item_Fruit_G_2854180008",
0xccd6fad8,"MainField_Item_Fruit_G_2858929841",
0x75ed3dfd,"MainField_Item_Fruit_G_286933511",
0x196a0846,"MainField_Item_Fruit_G_2877095915",
0xfb6b3d37,"MainField_Item_Fruit_G_2911891835",
0xdf3151af,"MainField_Item_Fruit_G_2933625519",
0xbde84764,"MainField_Item_Fruit_G_2953956052",
0xde38bacc,"MainField_Item_Fruit_G_2982262111",
0x1f33fc0d,"MainField_Item_Fruit_G_2998523752",
0x48b8f2a6,"MainField_Item_Fruit_G_3003409897",
0xad1cdaea,"MainField_Item_Fruit_G_3012771092",
0xc17d97c0,"MainField_Item_Fruit_G_3017413691",
0x1df31c1b,"MainField_Item_Fruit_G_3031642540",
0x7c739826,"MainField_Item_Fruit_G_3034959129",
0x61963f29,"MainField_Item_Fruit_G_3055826910",
0x6be80fc4,"MainField_Item_Fruit_G_306975540",
0xd00c6cda,"MainField_Item_Fruit_G_3075623106",
0xd76fc577,"MainField_Item_Fruit_G_3077231458",
0x7c39d734,"MainField_Item_Fruit_G_3087744794",
0xb28fdbe9,"MainField_Item_Fruit_G_3159016166",
0xbc320b02,"MainField_Item_Fruit_G_3162839316",
0xa7cc3fc9,"MainField_Item_Fruit_G_3163531079",
0x4291d520,"MainField_Item_Fruit_G_317725922",
0xc23f0698,"MainField_Item_Fruit_G_3206167917",
0x8bed99c5,"MainField_Item_Fruit_G_3208876524",
0x77fe89aa,"MainField_Item_Fruit_G_3212405401",
0x70023784,"MainField_Item_Fruit_G_3225416445",
0xf3a08d86,"MainField_Item_Fruit_G_3227372845",
0x6d3b93a1,"MainField_Item_Fruit_G_3228427252",
0x3c496ff6,"MainField_Item_Fruit_G_3234256345",
0x5edc9c37,"MainField_Item_Fruit_G_3249674783",
0x2018d04d,"MainField_Item_Fruit_G_3260938133",
0xd4d85e5b,"MainField_Item_Fruit_G_3268078472",
0x16c0c346,"MainField_Item_Fruit_G_3271386722",
0x07e7b7d5,"MainField_Item_Fruit_G_3276447448",
0x32b46255,"MainField_Item_Fruit_G_3298317484",
0x02752486,"MainField_Item_Fruit_G_3310175644",
0x2d0cf76c,"MainField_Item_Fruit_G_3357347323",
0xd594a0fb,"MainField_Item_Fruit_G_3370544718",
0x5cdb676b,"MainField_Item_Fruit_G_3381143157",
0x01f09bbc,"MainField_Item_Fruit_G_3385116137",
0x8406182b,"MainField_Item_Fruit_G_3386617823",
0x23ef99d8,"MainField_Item_Fruit_G_3396895011",
0x1aa320b8,"MainField_Item_Fruit_G_3397165764",
0x848a1766,"MainField_Item_Fruit_G_3414726609",
0xea9ec9c5,"MainField_Item_Fruit_G_3425294633",
0x13a72dad,"MainField_Item_Fruit_G_3433601897",
0x2f04e47a,"MainField_Item_Fruit_G_3436691286",
0x3935d89d,"MainField_Item_Fruit_G_3447954956",
0x59587516,"MainField_Item_Fruit_G_3481855833",
0x2c854e4e,"MainField_Item_Fruit_G_3496935217",
0xeb40af7a,"MainField_Item_Fruit_G_3502172692",
0xac3fa0d1,"MainField_Item_Fruit_G_3506129256",
0x012c88fd,"MainField_Item_Fruit_G_3554352363",
0x24787bdc,"MainField_Item_Fruit_G_3557447580",
0x7a449b46,"MainField_Item_Fruit_G_3597400227",
0xa0bdefc6,"MainField_Item_Fruit_G_359886429",
0xa70a20a8,"MainField_Item_Fruit_G_3646198002",
0x1200c237,"MainField_Item_Fruit_G_3665141799",
0x41ab0dd3,"MainField_Item_Fruit_G_3673803982",
0xf287e5e2,"MainField_Item_Fruit_G_3689305891",
0x78b71251,"MainField_Item_Fruit_G_3695718050",
0x74aa98a5,"MainField_Item_Fruit_G_3699101565",
0xf1f49bbd,"MainField_Item_Fruit_G_3720092599",
0x0fe9ca7d,"MainField_Item_Fruit_G_3730447484",
0x508c5eb2,"MainField_Item_Fruit_G_3735092352",
0x97ddf438,"MainField_Item_Fruit_G_3739261429",
0xa0eb14ea,"MainField_Item_Fruit_G_3741101697",
0x0ef35dc5,"MainField_Item_Fruit_G_374590544",
0x16dd4e01,"MainField_Item_Fruit_G_3755848840",
0x66febee8,"MainField_Item_Fruit_G_3772509548",
0xa68ddbcc,"MainField_Item_Fruit_G_3780243719",
0xf84c9787,"MainField_Item_Fruit_G_3794792618",
0x13109867,"MainField_Item_Fruit_G_3794994806",
0xd0c46dde,"MainField_Item_Fruit_G_38070806",
0x793c7708,"MainField_Item_Fruit_G_3816830918",
0x16462c54,"MainField_Item_Fruit_G_3842218294",
0x703ac202,"MainField_Item_Fruit_G_3883509798",
0x920fccec,"MainField_Item_Fruit_G_3884180030",
0x7d7b59c9,"MainField_Item_Fruit_G_3886638143",
0x0288bd98,"MainField_Item_Fruit_G_3906789521",
0xfa01a37c,"MainField_Item_Fruit_G_3926805950",
0x80234908,"MainField_Item_Fruit_G_3938382364",
0x46220c87,"MainField_Item_Fruit_G_3940177360",
0x8ff7904a,"MainField_Item_Fruit_G_3945221923",
0x1e8d9e59,"MainField_Item_Fruit_G_3951217515",
0x340a75e0,"MainField_Item_Fruit_G_3996261721",
0x1287d62d,"MainField_Item_Fruit_G_3996495381",
0xe1d18169,"MainField_Item_Fruit_G_4005243265",
0x8ac4a194,"MainField_Item_Fruit_G_4020958726",
0x66cf084f,"MainField_Item_Fruit_G_402355786",
0xbf0b995f,"MainField_Item_Fruit_G_4036808725",
0x6c573fa8,"MainField_Item_Fruit_G_4070936349",
0xbaf45eb7,"MainField_Item_Fruit_G_4078271743",
0x63bc19c8,"MainField_Item_Fruit_G_4101826131",
0x2224db86,"MainField_Item_Fruit_G_4118237119",
0x66775d81,"MainField_Item_Fruit_G_4133490619",
0xb3caf271,"MainField_Item_Fruit_G_416948572",
0x7db42db3,"MainField_Item_Fruit_G_418287163",
0xedd9299a,"MainField_Item_Fruit_G_4199045246",
0xc9d1ebc1,"MainField_Item_Fruit_G_4199259252",
0xac865389,"MainField_Item_Fruit_G_4250693092",
0x5f227b5a,"MainField_Item_Fruit_G_4251358865",
0x03f81751,"MainField_Item_Fruit_G_4258884575",
0x58b3ff0f,"MainField_Item_Fruit_G_426587865",
0xde0d5a2c,"MainField_Item_Fruit_G_427116082",
0xc255b5cb,"MainField_Item_Fruit_G_4271906409",
0xd39b5572,"MainField_Item_Fruit_G_4276294815",
0x5a6af8b2,"MainField_Item_Fruit_G_4283502164",
0xc6ae73c1,"MainField_Item_Fruit_G_429644914",
0xfbde287c,"MainField_Item_Fruit_G_442871272",
0xf0672843,"MainField_Item_Fruit_G_450084299",
0xdc04438d,"MainField_Item_Fruit_G_457593033",
0x1211bb20,"MainField_Item_Fruit_G_463504752",
0x2420e71d,"MainField_Item_Fruit_G_463980511",
0x425c6918,"MainField_Item_Fruit_G_4944463",
0x24578706,"MainField_Item_Fruit_G_499022290",
0xdfdfec26,"MainField_Item_Fruit_G_506804619",
0xb6d03d3c,"MainField_Item_Fruit_G_513491198",
0xe3d869e6,"MainField_Item_Fruit_G_552976971",
0x87053735,"MainField_Item_Fruit_G_553807520",
0xdcec7409,"MainField_Item_Fruit_G_555614010",
0xd5f7dce7,"MainField_Item_Fruit_G_576302669",
0x9aefc65b,"MainField_Item_Fruit_G_587265688",
0x0ec66082,"MainField_Item_Fruit_G_590094469",
0xf0ab3524,"MainField_Item_Fruit_G_604777533",
0x149787dd,"MainField_Item_Fruit_G_611770383",
0x5dc91511,"MainField_Item_Fruit_G_619235666",
0x6aecd5f4,"MainField_Item_Fruit_G_628278927",
0x40842240,"MainField_Item_Fruit_G_638807255",
0x9c5a1400,"MainField_Item_Fruit_G_638907010",
0x10c8e85e,"MainField_Item_Fruit_G_661529823",
0x1a4b6f86,"MainField_Item_Fruit_G_698980776",
0xd1fd41f1,"MainField_Item_Fruit_G_711194644",
0xa3b98fca,"MainField_Item_Fruit_G_722515566",
0x11b86cb5,"MainField_Item_Fruit_G_735291576",
0x3313f203,"MainField_Item_Fruit_G_745873820",
0xd54b34d2,"MainField_Item_Fruit_G_750112360",
0x84adb067,"MainField_Item_Fruit_G_784905461",
0x0337cd2c,"MainField_Item_Fruit_G_81016060",
0x0fe240a2,"MainField_Item_Fruit_G_812319498",
0xb44bf03b,"MainField_Item_Fruit_G_821219245",
0x47929b87,"MainField_Item_Fruit_G_835491343",
0xa2fed639,"MainField_Item_Fruit_G_839524087",
0xfc90db8c,"MainField_Item_Fruit_G_843650049",
0x5f73febf,"MainField_Item_Fruit_G_847639062",
0x0d05663e,"MainField_Item_Fruit_G_864784590",
0xaeb85958,"MainField_Item_Fruit_G_872462736",
0xe8596300,"MainField_Item_Fruit_G_873227524",
0xb45b1bfa,"MainField_Item_Fruit_G_880525579",
0x5e22b1a8,"MainField_Item_Fruit_G_88265601",
0x300b4756,"MainField_Item_Fruit_G_888647138",
0x393e5ed1,"MainField_Item_Fruit_G_93104711",
0x9a99b299,"MainField_Item_Fruit_G_933170100",
0x1051f71f,"MainField_Item_Fruit_G_946182543",
0x1fa20d51,"MainField_Item_Fruit_G_960791430",
0xce54272e,"MainField_Item_Fruit_G_970805025",
0x0276d3d9,"MainField_Item_Fruit_H_10052544",
0x01517a6a,"MainField_Item_Fruit_H_1024392328",
0x994eafdd,"MainField_Item_Fruit_H_1075278787",
0xce852b12,"MainField_Item_Fruit_H_1083259150",
0xb6b2a9dd,"MainField_Item_Fruit_H_1099618385",
0x970b16ac,"MainField_Item_Fruit_H_1149739410",
0x3f0fc707,"MainField_Item_Fruit_H_1156384556",
0xef6a3d20,"MainField_Item_Fruit_H_1192440237",
0xa08ae7ed,"MainField_Item_Fruit_H_1197114281",
0x828c2312,"MainField_Item_Fruit_H_1198352716",
0x9179d118,"MainField_Item_Fruit_H_1229773858",
0x0bbd274f,"MainField_Item_Fruit_H_1242156062",
0xdc5f46c5,"MainField_Item_Fruit_H_1298401287",
0x7d0d86b1,"MainField_Item_Fruit_H_1299665349",
0xd707df4b,"MainField_Item_Fruit_H_1300906121",
0x67439e42,"MainField_Item_Fruit_H_1309896595",
0x63e4615d,"MainField_Item_Fruit_H_1352284890",
0xb321cc4e,"MainField_Item_Fruit_H_1356649968",
0xa5b40ac4,"MainField_Item_Fruit_H_1385928134",
0x2d9fbb57,"MainField_Item_Fruit_H_1393753439",
0xc084ee16,"MainField_Item_Fruit_H_1409070545",
0x0d59ea7b,"MainField_Item_Fruit_H_1417623878",
0xf758e9e1,"MainField_Item_Fruit_H_1500019664",
0xc27fa480,"MainField_Item_Fruit_H_1516037439",
0x46232816,"MainField_Item_Fruit_H_1520742772",
0x72f54532,"MainField_Item_Fruit_H_1522002102",
0x7a307f12,"MainField_Item_Fruit_H_1525745564",
0x897230f1,"MainField_Item_Fruit_H_1525765322",
0x021b2511,"MainField_Item_Fruit_H_1557402435",
0x75fd4f08,"MainField_Item_Fruit_H_1583739660",
0x5ee7efe6,"MainField_Item_Fruit_H_1663667633",
0xd543cb66,"MainField_Item_Fruit_H_1670520961",
0x102985b1,"MainField_Item_Fruit_H_1694491800",
0x3aca7cb2,"MainField_Item_Fruit_H_1749183651",
0x7e333ec6,"MainField_Item_Fruit_H_1785825556",
0x58d25a74,"MainField_Item_Fruit_H_1786239931",
0xa5fde07e,"MainField_Item_Fruit_H_1788854682",
0xec44fb81,"MainField_Item_Fruit_H_182707535",
0x91d4c283,"MainField_Item_Fruit_H_1842746031",
0xd04966c5,"MainField_Item_Fruit_H_1844577667",
0x05c4504d,"MainField_Item_Fruit_H_1876111479",
0x17339021,"MainField_Item_Fruit_H_1938092251",
0xaceb20e4,"MainField_Item_Fruit_H_1939100109",
0x654ab344,"MainField_Item_Fruit_H_1946647628",
0x180d3a3e,"MainField_Item_Fruit_H_1951570418",
0x803c34ee,"MainField_Item_Fruit_H_1953896728",
0x70c9d99e,"MainField_Item_Fruit_H_1961440450",
0x4aa13e9c,"MainField_Item_Fruit_H_1966425128",
0x12916a28,"MainField_Item_Fruit_H_2006864726",
0x99bf3613,"MainField_Item_Fruit_H_2055803328",
0x1fa3a8b4,"MainField_Item_Fruit_H_2112148953",
0x2e4b28ef,"MainField_Item_Fruit_H_2147747729",
0x332a323d,"MainField_Item_Fruit_H_2178765147",
0xeb025a21,"MainField_Item_Fruit_H_2199006241",
0xc2813e3a,"MainField_Item_Fruit_H_2214356143",
0x235ddfb4,"MainField_Item_Fruit_H_221613158",
0xdfb41f6d,"MainField_Item_Fruit_H_2222975032",
0x384b7f7c,"MainField_Item_Fruit_H_2224136374",
0xd5394244,"MainField_Item_Fruit_H_227418454",
0x19dc06bd,"MainField_Item_Fruit_H_2331084212",
0x12181454,"MainField_Item_Fruit_H_2347795255",
0xd9f9fe9f,"MainField_Item_Fruit_H_2361226077",
0x9fbeb24a,"MainField_Item_Fruit_H_2375827885",
0x3376f5c1,"MainField_Item_Fruit_H_2415569914",
0x21688c8c,"MainField_Item_Fruit_H_2481632501",
0xce59cb2e,"MainField_Item_Fruit_H_2491904236",
0x5dd8175f,"MainField_Item_Fruit_H_2518542376",
0xea3da5da,"MainField_Item_Fruit_H_2598852078",
0xf61b38f9,"MainField_Item_Fruit_H_2604344580",
0x69519523,"MainField_Item_Fruit_H_2614246143",
0xaac576da,"MainField_Item_Fruit_H_2620406340",
0x3820c36c,"MainField_Item_Fruit_H_2643104247",
0x697e448e,"MainField_Item_Fruit_H_2738872259",
0x931e4ef2,"MainField_Item_Fruit_H_2741659947",
0xe3cb385e,"MainField_Item_Fruit_H_2742628585",
0x5c45caa4,"MainField_Item_Fruit_H_2751966514",
0x8e68866c,"MainField_Item_Fruit_H_2753741013",
0xb6aa2a44,"MainField_Item_Fruit_H_2769208374",
0x6f796ea4,"MainField_Item_Fruit_H_2848764771",
0xc592d29f,"MainField_Item_Fruit_H_2861154494",
0x70ca77d6,"MainField_Item_Fruit_H_2891500911",
0x1448c77f,"MainField_Item_Fruit_H_2892015819",
0x08d8e722,"MainField_Item_Fruit_H_2917401767",
0xf28710f9,"MainField_Item_Fruit_H_2960034849",
0xa86e3a80,"MainField_Item_Fruit_H_2988084749",
0x4535b837,"MainField_Item_Fruit_H_3010967241",
0x9181edc2,"MainField_Item_Fruit_H_3036636848",
0xb7c48864,"MainField_Item_Fruit_H_3064945836",
0xdfc0a7ee,"MainField_Item_Fruit_H_3124277737",
0x55d2bb21,"MainField_Item_Fruit_H_3127990378",
0x371ce3ef,"MainField_Item_Fruit_H_3130986724",
0x2efb6e0a,"MainField_Item_Fruit_H_3172768883",
0xe0f4c4d0,"MainField_Item_Fruit_H_318362253",
0x031c02da,"MainField_Item_Fruit_H_3186808061",
0x7f01ccb5,"MainField_Item_Fruit_H_3201479900",
0xa345dde2,"MainField_Item_Fruit_H_3235813596",
0xdfd540a1,"MainField_Item_Fruit_H_3267408659",
0x1d0109b4,"MainField_Item_Fruit_H_3268908342",
0x94d21b78,"MainField_Item_Fruit_H_3276612175",
0xe6a79057,"MainField_Item_Fruit_H_32770407",
0x8f5ac273,"MainField_Item_Fruit_H_3284042558",
0x9eb6a959,"MainField_Item_Fruit_H_3287811440",
0x7a6a3314,"MainField_Item_Fruit_H_3287832102",
0xafe5d014,"MainField_Item_Fruit_H_335074318",
0x07209ba9,"MainField_Item_Fruit_H_338607399",
0x3d336c02,"MainField_Item_Fruit_H_3386213450",
0x7d3813ad,"MainField_Item_Fruit_H_3386336833",
0xc7d30382,"MainField_Item_Fruit_H_3390811365",
0x5452164c,"MainField_Item_Fruit_H_3405112427",
0xe32cc27b,"MainField_Item_Fruit_H_3413290623",
0x9e2f8a14,"MainField_Item_Fruit_H_342825658",
0xfdf50c72,"MainField_Item_Fruit_H_3447089404",
0xbfea8e73,"MainField_Item_Fruit_H_345313303",
0x6f8f638f,"MainField_Item_Fruit_H_3462356376",
0xb6b3b2fe,"MainField_Item_Fruit_H_3474766800",
0x2f9b51e7,"MainField_Item_Fruit_H_3479727262",
0x456ef50a,"MainField_Item_Fruit_H_3540032932",
0x1944bcfe,"MainField_Item_Fruit_H_3560501077",
0xe4fb7f8e,"MainField_Item_Fruit_H_3563936189",
0x9f2ba524,"MainField_Item_Fruit_H_3565027455",
0x28417db4,"MainField_Item_Fruit_H_3565359411",
0x4b74022e,"MainField_Item_Fruit_H_357877983",
0xcb6601e6,"MainField_Item_Fruit_H_3627011910",
0xf8e0b9ec,"MainField_Item_Fruit_H_364126951",
0x89b4eff1,"MainField_Item_Fruit_H_3642630777",
0x4697e9ad,"MainField_Item_Fruit_H_36691100",
0xe1389099,"MainField_Item_Fruit_H_36723966",
0xe90da67f,"MainField_Item_Fruit_H_36861118",
0xd9da634a,"MainField_Item_Fruit_H_3717132328",
0x1b432f47,"MainField_Item_Fruit_H_3725933881",
0x553bb8c7,"MainField_Item_Fruit_H_3730035219",
0x363326d0,"MainField_Item_Fruit_H_3735843330",
0x1cac2dbd,"MainField_Item_Fruit_H_3769201950",
0xfee861e3,"MainField_Item_Fruit_H_3776633022",
0x2afd9534,"MainField_Item_Fruit_H_3810511365",
0x8dc1cf6d,"MainField_Item_Fruit_H_3816833146",
0xbd67eecd,"MainField_Item_Fruit_H_3840771171",
0x8a0014df,"MainField_Item_Fruit_H_3877719751",
0xd75c0822,"MainField_Item_Fruit_H_3933616076",
0x869f7574,"MainField_Item_Fruit_H_3934623421",
0x6a6d6deb,"MainField_Item_Fruit_H_3935150433",
0x4817cd83,"MainField_Item_Fruit_H_3961634224",
0xbce02571,"MainField_Item_Fruit_H_3977129462",
0xdbc74061,"MainField_Item_Fruit_H_3990906232",
0x7760c900,"MainField_Item_Fruit_H_4024584640",
0x14403f57,"MainField_Item_Fruit_H_4084783278",
0x6e5f228e,"MainField_Item_Fruit_H_4086730784",
0x9fb870b5,"MainField_Item_Fruit_H_4096745701",
0x694e074a,"MainField_Item_Fruit_H_4097078446",
0xa271f67e,"MainField_Item_Fruit_H_4110111801",
0xf359f4ee,"MainField_Item_Fruit_H_4141507021",
0xfb4e406c,"MainField_Item_Fruit_H_4152869440",
0xc7bf5dce,"MainField_Item_Fruit_H_4157759837",
0xbb7c1608,"MainField_Item_Fruit_H_416547041",
0x728e674f,"MainField_Item_Fruit_H_4204491067",
0x79845e33,"MainField_Item_Fruit_H_4217022963",
0x60b90ff5,"MainField_Item_Fruit_H_4223288267",
0xd2a4f09a,"MainField_Item_Fruit_H_4244091809",
0x1e741119,"MainField_Item_Fruit_H_4260803874",
0x84d6af0a,"MainField_Item_Fruit_H_4269913066",
0x420470af,"MainField_Item_Fruit_H_441358373",
0xc620d7fd,"MainField_Item_Fruit_H_452277525",
0x8d276c3c,"MainField_Item_Fruit_H_463503259",
0xc6ada62e,"MainField_Item_Fruit_H_494320002",
0x52e4f2bc,"MainField_Item_Fruit_H_496562444",
0xa9a97e62,"MainField_Item_Fruit_H_50351322",
0x91f244b3,"MainField_Item_Fruit_H_55945572",
0xaa39272d,"MainField_Item_Fruit_H_595069392",
0x9841c252,"MainField_Item_Fruit_H_605408713",
0x0e6325de,"MainField_Item_Fruit_H_620463431",
0x0a1a42b0,"MainField_Item_Fruit_H_65930324",
0x8b45408f,"MainField_Item_Fruit_H_737732620",
0x217d8fa2,"MainField_Item_Fruit_H_752481306",
0x10a84e9f,"MainField_Item_Fruit_H_75743309",
0x4616ab69,"MainField_Item_Fruit_H_764603885",
0x6ed427d5,"MainField_Item_Fruit_H_765690338",
0x3ed8da80,"MainField_Item_Fruit_H_766564384",
0xe7406388,"MainField_Item_Fruit_H_771110666",
0xea585408,"MainField_Item_Fruit_H_771122268",
0xaf288f98,"MainField_Item_Fruit_H_778868224",
0xb03888a0,"MainField_Item_Fruit_H_779045702",
0xd813c628,"MainField_Item_Fruit_H_811447103",
0x19ee0689,"MainField_Item_Fruit_H_864063748",
0x64df3eb1,"MainField_Item_Fruit_H_869616738",
0xd1e78087,"MainField_Item_Fruit_H_884519075",
0x5df14161,"MainField_Item_Fruit_H_888748193",
0x87d8287e,"MainField_Item_Fruit_H_901426030",
0x8a09d654,"MainField_Item_Fruit_H_923812693",
0x163817e7,"MainField_Item_Fruit_H_957636357",
0x5fac0803,"MainField_Item_Fruit_H_979581073",
0x59dba4ba,"MainField_Item_Fruit_H_980443475",
0xf4761cf3,"MainField_Item_Fruit_H_982347807",
0x99a02070,"MainField_Item_Fruit_I_1004369534",
0x161b1971,"MainField_Item_Fruit_I_100629850",
0xb6242d20,"MainField_Item_Fruit_I_100888310",
0x3b58d530,"MainField_Item_Fruit_I_1025583215",
0xb80bf787,"MainField_Item_Fruit_I_1040630403",
0xb4c4ea9b,"MainField_Item_Fruit_I_1047729426",
0x29006e08,"MainField_Item_Fruit_I_1048531955",
0xf530f16e,"MainField_Item_Fruit_I_1054308592",
0x10f88410,"MainField_Item_Fruit_I_1081709224",
0x57553d17,"MainField_Item_Fruit_I_1085137847",
0x50814fdd,"MainField_Item_Fruit_I_1087254350",
0x1ab7e935,"MainField_Item_Fruit_I_1099749871",
0x436f6cb6,"MainField_Item_Fruit_I_1105413113",
0x1dbbf931,"MainField_Item_Fruit_I_1107155484",
0x0447f6af,"MainField_Item_Fruit_I_1112338350",
0x8d9a2342,"MainField_Item_Fruit_I_1138622141",
0x622a545b,"MainField_Item_Fruit_I_1143368075",
0xa2632638,"MainField_Item_Fruit_I_1152400036",
0x11bb01c7,"MainField_Item_Fruit_I_1153780119",
0xf9cbee1d,"MainField_Item_Fruit_I_1159801783",
0xfed8b61d,"MainField_Item_Fruit_I_118254066",
0xb4107f5b,"MainField_Item_Fruit_I_1187204929",
0x7732a2d9,"MainField_Item_Fruit_I_1189158390",
0x22541bf6,"MainField_Item_Fruit_I_1201444109",
0x5a37babe,"MainField_Item_Fruit_I_1203775406",
0x15afa752,"MainField_Item_Fruit_I_1205500455",
0x95f4a5ae,"MainField_Item_Fruit_I_1224864277",
0xde82d4c6,"MainField_Item_Fruit_I_1232348548",
0x394388a9,"MainField_Item_Fruit_I_1232626533",
0x3918b55f,"MainField_Item_Fruit_I_1234632948",
0x84e316ed,"MainField_Item_Fruit_I_1237310015",
0x41629507,"MainField_Item_Fruit_I_1244010745",
0x4b49868e,"MainField_Item_Fruit_I_1244035639",
0x06df3d6e,"MainField_Item_Fruit_I_124904423",
0x30abae7a,"MainField_Item_Fruit_I_1256826433",
0x864e18d9,"MainField_Item_Fruit_I_1262789203",
0x466732f2,"MainField_Item_Fruit_I_1270317809",
0x8a43d2ce,"MainField_Item_Fruit_I_1289373416",
0xe181293e,"MainField_Item_Fruit_I_1296809518",
0x552668fb,"MainField_Item_Fruit_I_1301593489",
0x981b62ec,"MainField_Item_Fruit_I_1307250277",
0x70094596,"MainField_Item_Fruit_I_1310319005",
0x9c243175,"MainField_Item_Fruit_I_1315714572",
0xeeb83a7b,"MainField_Item_Fruit_I_1316369075",
0xe0ab2ea5,"MainField_Item_Fruit_I_1321574219",
0xb6a4a97a,"MainField_Item_Fruit_I_1322674717",
0x21a13d6f,"MainField_Item_Fruit_I_1331612025",
0x865cedd5,"MainField_Item_Fruit_I_1333068650",
0xdddb5b3c,"MainField_Item_Fruit_I_1348686372",
0x3d2a73e5,"MainField_Item_Fruit_I_1367655049",
0x95dd54eb,"MainField_Item_Fruit_I_1369049525",
0xe2ebf523,"MainField_Item_Fruit_I_1371396602",
0x3aa4e16d,"MainField_Item_Fruit_I_139248106",
0x1d836216,"MainField_Item_Fruit_I_1415578763",
0x16d2c033,"MainField_Item_Fruit_I_1416069265",
0xf1c24fb9,"MainField_Item_Fruit_I_1421610252",
0xf79c26ab,"MainField_Item_Fruit_I_1422785483",
0x2f7ac4c1,"MainField_Item_Fruit_I_1438181081",
0x2badd16d,"MainField_Item_Fruit_I_1458112144",
0xb5e7d69c,"MainField_Item_Fruit_I_1459080620",
0xbe3f5473,"MainField_Item_Fruit_I_1460573757",
0x1dd625cd,"MainField_Item_Fruit_I_1467418445",
0xcc829b51,"MainField_Item_Fruit_I_1492762",
0x59653db7,"MainField_Item_Fruit_I_149523132",
0xd9e21932,"MainField_Item_Fruit_I_1495713888",
0xd83ffd1b,"MainField_Item_Fruit_I_1505659656",
0x1bc11b43,"MainField_Item_Fruit_I_1507139087",
0xa2948cd9,"MainField_Item_Fruit_I_1508368138",
0x8c3eb7eb,"MainField_Item_Fruit_I_1508532309",
0x9eba5e64,"MainField_Item_Fruit_I_1522272419",
0xe2952eb7,"MainField_Item_Fruit_I_1532760241",
0xa0abcfc2,"MainField_Item_Fruit_I_1564898063",
0xe17950f3,"MainField_Item_Fruit_I_1564969350",
0x42411477,"MainField_Item_Fruit_I_1566379194",
0x555e2951,"MainField_Item_Fruit_I_1569894914",
0x1af64a06,"MainField_Item_Fruit_I_1574030522",
0xf7d24cc4,"MainField_Item_Fruit_I_1581976697",
0x439b5aff,"MainField_Item_Fruit_I_1585511415",
0x87be9724,"MainField_Item_Fruit_I_1585947724",
0x429606f0,"MainField_Item_Fruit_I_1589839023",
0xa8a3396b,"MainField_Item_Fruit_I_1601459811",
0x3646bc98,"MainField_Item_Fruit_I_1603192434",
0x76cf3de4,"MainField_Item_Fruit_I_160597388",
0x91d0f060,"MainField_Item_Fruit_I_1614002125",
0x0914dbb1,"MainField_Item_Fruit_I_1617617871",
0xa87ef6d6,"MainField_Item_Fruit_I_1618439885",
0x3085b664,"MainField_Item_Fruit_I_1620758834",
0x8143fbc1,"MainField_Item_Fruit_I_1647092182",
0xf80d38b2,"MainField_Item_Fruit_I_1653348001",
0x1ed35ef2,"MainField_Item_Fruit_I_1653773607",
0x6320791e,"MainField_Item_Fruit_I_1661708707",
0xe540e8b2,"MainField_Item_Fruit_I_1662854883",
0xd9b84b12,"MainField_Item_Fruit_I_1667405079",
0xc59239e0,"MainField_Item_Fruit_I_1676237309",
0x08f49388,"MainField_Item_Fruit_I_168330917",
0x586f8c90,"MainField_Item_Fruit_I_1686412772",
0x19976053,"MainField_Item_Fruit_I_1691045834",
0x585214ae,"MainField_Item_Fruit_I_1691365503",
0x8f62bed9,"MainField_Item_Fruit_I_1698231487",
0x5afbe8b5,"MainField_Item_Fruit_I_1711231294",
0xc44e1df6,"MainField_Item_Fruit_I_1721138571",
0xf608d94e,"MainField_Item_Fruit_I_1728956210",
0x1c2c7b22,"MainField_Item_Fruit_I_1734420594",
0x5dec9736,"MainField_Item_Fruit_I_1740539457",
0xe0cd0a53,"MainField_Item_Fruit_I_1746351472",
0x103026de,"MainField_Item_Fruit_I_1748034879",
0x1d55c32b,"MainField_Item_Fruit_I_1772393574",
0xe27929c3,"MainField_Item_Fruit_I_1781752081",
0x7ca1fe0b,"MainField_Item_Fruit_I_1809580759",
0x69ca5205,"MainField_Item_Fruit_I_1816573",
0xf1903be4,"MainField_Item_Fruit_I_1823445787",
0x70dfaabc,"MainField_Item_Fruit_I_1823590094",
0x8c894ad9,"MainField_Item_Fruit_I_1828137444",
0x735450e8,"MainField_Item_Fruit_I_1854913070",
0x16e031e9,"MainField_Item_Fruit_I_186971496",
0x95014ee3,"MainField_Item_Fruit_I_1870328169",
0x110a8ecc,"MainField_Item_Fruit_I_1879791972",
0xe795d951,"MainField_Item_Fruit_I_1880767537",
0x720d2b95,"MainField_Item_Fruit_I_1886471026",
0x8d35c007,"MainField_Item_Fruit_I_1886835569",
0xeb141de9,"MainField_Item_Fruit_I_190669848",
0x96017545,"MainField_Item_Fruit_I_192105087",
0xddce840f,"MainField_Item_Fruit_I_1925657494",
0xd8f44fb0,"MainField_Item_Fruit_I_1928875468",
0x4bfa6769,"MainField_Item_Fruit_I_1935972996",
0x9b78f6c0,"MainField_Item_Fruit_I_1940937609",
0x33548c29,"MainField_Item_Fruit_I_1941681177",
0xda2d476f,"MainField_Item_Fruit_I_1947828564",
0x293ad402,"MainField_Item_Fruit_I_1952854688",
0x7c9daf5a,"MainField_Item_Fruit_I_195335205",
0xaab8a545,"MainField_Item_Fruit_I_1956441067",
0xebf63482,"MainField_Item_Fruit_I_1960210432",
0x9d829b7a,"MainField_Item_Fruit_I_1969459509",
0xe5a8a7bd,"MainField_Item_Fruit_I_1974138767",
0x71521138,"MainField_Item_Fruit_I_1978407077",
0xfa4cf276,"MainField_Item_Fruit_I_1992090257",
0xfe412b44,"MainField_Item_Fruit_I_1997666188",
0x0ca03fca,"MainField_Item_Fruit_I_1998358379",
0xe70aad1f,"MainField_Item_Fruit_I_2010490140",
0x0827743c,"MainField_Item_Fruit_I_2021883443",
0xe740a6e7,"MainField_Item_Fruit_I_203329126",
0x9802e87c,"MainField_Item_Fruit_I_2046078573",
0xe25adbc5,"MainField_Item_Fruit_I_2046324995",
0x1cc74cbe,"MainField_Item_Fruit_I_2053272106",
0x9ef9d892,"MainField_Item_Fruit_I_206123377",
0xdaa665e6,"MainField_Item_Fruit_I_2065284848",
0x8c42bd83,"MainField_Item_Fruit_I_2068709863",
0xfa3cc285,"MainField_Item_Fruit_I_2071348096",
0xff07c141,"MainField_Item_Fruit_I_2075672089",
0x221a184a,"MainField_Item_Fruit_I_2077122730",
0xe931f966,"MainField_Item_Fruit_I_2082719230",
0xecccb202,"MainField_Item_Fruit_I_2086294670",
0xc64b8cb1,"MainField_Item_Fruit_I_208736400",
0xb8646139,"MainField_Item_Fruit_I_2088008425",
0x7a986426,"MainField_Item_Fruit_I_2091099315",
0xe2e22f98,"MainField_Item_Fruit_I_2098181683",
0x9154d7ef,"MainField_Item_Fruit_I_2101900626",
0x6261d4db,"MainField_Item_Fruit_I_2103458859",
0x73af2455,"MainField_Item_Fruit_I_2135683452",
0xa9879728,"MainField_Item_Fruit_I_2146351658",
0x07fe5b5a,"MainField_Item_Fruit_I_2155072784",
0x2694675f,"MainField_Item_Fruit_I_2171899973",
0x9b74a865,"MainField_Item_Fruit_I_2196685643",
0xbdac6647,"MainField_Item_Fruit_I_2201792739",
0x0d3d57e7,"MainField_Item_Fruit_I_2203108388",
0x30c73ce9,"MainField_Item_Fruit_I_2228926480",
0xe1a3c568,"MainField_Item_Fruit_I_2245157883",
0xef612018,"MainField_Item_Fruit_I_224740028",
0xa59fe19f,"MainField_Item_Fruit_I_2276770156",
0x7867650d,"MainField_Item_Fruit_I_2278739656",
0x0fe6dc81,"MainField_Item_Fruit_I_2280398619",
0x10d587c1,"MainField_Item_Fruit_I_2324364488",
0xd1a36476,"MainField_Item_Fruit_I_2335610259",
0x06743df4,"MainField_Item_Fruit_I_2337211619",
0x99bcfe8d,"MainField_Item_Fruit_I_2347839506",
0x13183bb3,"MainField_Item_Fruit_I_2351087866",
0x22dc44b9,"MainField_Item_Fruit_I_2355376080",
0x4a6db506,"MainField_Item_Fruit_I_2359096331",
0x9118550d,"MainField_Item_Fruit_I_2360646189",
0xc22700af,"MainField_Item_Fruit_I_2363499075",
0x17aa0312,"MainField_Item_Fruit_I_2364253760",
0x5aa727ac,"MainField_Item_Fruit_I_2366954654",
0xd22fff56,"MainField_Item_Fruit_I_2371345026",
0x530e6687,"MainField_Item_Fruit_I_2380740817",
0xef9159a3,"MainField_Item_Fruit_I_2392018654",
0xfe3ca700,"MainField_Item_Fruit_I_2392109086",
0xb54af2f5,"MainField_Item_Fruit_I_2425817944",
0x95cb1ea4,"MainField_Item_Fruit_I_2427050635",
0x601c8a79,"MainField_Item_Fruit_I_2429539631",
0xcfa52b60,"MainField_Item_Fruit_I_2447432454",
0xb2cbe988,"MainField_Item_Fruit_I_2452464850",
0x6b9ccdbb,"MainField_Item_Fruit_I_2454992290",
0xb1b74489,"MainField_Item_Fruit_I_2457476037",
0xaa397407,"MainField_Item_Fruit_I_2466371359",
0x8764eee0,"MainField_Item_Fruit_I_2468660972",
0x1e5fa880,"MainField_Item_Fruit_I_2476846163",
0x528d6421,"MainField_Item_Fruit_I_2485899367",
0xb20a43d5,"MainField_Item_Fruit_I_2490110726",
0x7c969790,"MainField_Item_Fruit_I_2501369820",
0xbbeaa3c6,"MainField_Item_Fruit_I_2503602619",
0x584f61f5,"MainField_Item_Fruit_I_2504024267",
0xb7be65ca,"MainField_Item_Fruit_I_2508136081",
0xa4667c7a,"MainField_Item_Fruit_I_2509625096",
0x9ccee04c,"MainField_Item_Fruit_I_2512388486",
0x876794d6,"MainField_Item_Fruit_I_251609493",
0xdb4218cc,"MainField_Item_Fruit_I_2525830943",
0x72096c2a,"MainField_Item_Fruit_I_2549485615",
0x53e3e1de,"MainField_Item_Fruit_I_2556951381",
0x30a89831,"MainField_Item_Fruit_I_2568152647",
0x196f9307,"MainField_Item_Fruit_I_2582141597",
0x11b8766d,"MainField_Item_Fruit_I_2597851436",
0x89cee447,"MainField_Item_Fruit_I_260329125",
0x3edb8271,"MainField_Item_Fruit_I_2611254947",
0xeff5f358,"MainField_Item_Fruit_I_2614019028",
0xf9ec8d2a,"MainField_Item_Fruit_I_2614070894",
0x99b56603,"MainField_Item_Fruit_I_2630439610",
0x1f5764df,"MainField_Item_Fruit_I_2645939509",
0xc70ea62f,"MainField_Item_Fruit_I_2649437275",
0x222ba37b,"MainField_Item_Fruit_I_2651142216",
0x7f2de5ac,"MainField_Item_Fruit_I_2671465268",
0xf85e4bc7,"MainField_Item_Fruit_I_2685894833",
0xe3adbc8a,"MainField_Item_Fruit_I_2686174800",
0x0dcac229,"MainField_Item_Fruit_I_2690258247",
0xe44d390b,"MainField_Item_Fruit_I_2690799392",
0xa6db7c6b,"MainField_Item_Fruit_I_269154212",
0x724e2ddf,"MainField_Item_Fruit_I_2709217862",
0xacbd6f18,"MainField_Item_Fruit_I_2719353541",
0x8b4a4c16,"MainField_Item_Fruit_I_2722280107",
0xe652b5bc,"MainField_Item_Fruit_I_2731812804",
0x22f7607d,"MainField_Item_Fruit_I_2754091803",
0x28158cbf,"MainField_Item_Fruit_I_2758150302",
0xbed7489d,"MainField_Item_Fruit_I_2784281257",
0x6b31ab25,"MainField_Item_Fruit_I_2802815801",
0x66cde022,"MainField_Item_Fruit_I_2809528905",
0xbb44aa98,"MainField_Item_Fruit_I_2809808040",
0xef400b6f,"MainField_Item_Fruit_I_2837067677",
0x109ccfed,"MainField_Item_Fruit_I_2839734762",
0xe4ba05a5,"MainField_Item_Fruit_I_2844980257",
0x6aa6226c,"MainField_Item_Fruit_I_2848867970",
0x045e385d,"MainField_Item_Fruit_I_2855047318",
0x436e145c,"MainField_Item_Fruit_I_2872003227",
0x8236a389,"MainField_Item_Fruit_I_2890043010",
0x4cd49eca,"MainField_Item_Fruit_I_2913397922",
0x388166ff,"MainField_Item_Fruit_I_2916327313",
0xa039ce0f,"MainField_Item_Fruit_I_2929951387",
0xa81e84a8,"MainField_Item_Fruit_I_2944579779",
0x08dc4b56,"MainField_Item_Fruit_I_2951553749",
0xfb83eab3,"MainField_Item_Fruit_I_2957339989",
0xa9401df2,"MainField_Item_Fruit_I_2961818976",
0x89c6e112,"MainField_Item_Fruit_I_2979780493",
0x2d133f72,"MainField_Item_Fruit_I_2986768166",
0x2bd096a7,"MainField_Item_Fruit_I_3005029597",
0x3c9523f3,"MainField_Item_Fruit_I_3017780630",
0x968e6e95,"MainField_Item_Fruit_I_3031591311",
0xc9cc05e5,"MainField_Item_Fruit_I_3033219693",
0x5856adec,"MainField_Item_Fruit_I_3042079133",
0xa7ad9dbb,"MainField_Item_Fruit_I_3043827334",
0x2b26e7f5,"MainField_Item_Fruit_I_3057446798",
0x398a5707,"MainField_Item_Fruit_I_306599257",
0xc4158bdf,"MainField_Item_Fruit_I_3072225819",
0x7017779e,"MainField_Item_Fruit_I_3072895308",
0x5180bfb3,"MainField_Item_Fruit_I_3084665630",
0x5c89d76e,"MainField_Item_Fruit_I_3085173113",
0xf2d18687,"MainField_Item_Fruit_I_3090606965",
0x2dda1299,"MainField_Item_Fruit_I_3102173116",
0x7fad7866,"MainField_Item_Fruit_I_3103370368",
0xb7e127bd,"MainField_Item_Fruit_I_3103848209",
0x735de8c6,"MainField_Item_Fruit_I_3116994987",
0x91572e15,"MainField_Item_Fruit_I_3143892878",
0x1733686f,"MainField_Item_Fruit_I_3149301749",
0x91a37812,"MainField_Item_Fruit_I_3168486380",
0xc4698cda,"MainField_Item_Fruit_I_318267816",
0x010c1c22,"MainField_Item_Fruit_I_3185551417",
0x9f01d470,"MainField_Item_Fruit_I_3189023352",
0x2a460751,"MainField_Item_Fruit_I_3193064786",
0x694707cb,"MainField_Item_Fruit_I_3193958925",
0xfa54a37f,"MainField_Item_Fruit_I_3194827528",
0x59c96495,"MainField_Item_Fruit_I_3205338910",
0xfa4bc98f,"MainField_Item_Fruit_I_3213534117",
0x662f3dbe,"MainField_Item_Fruit_I_3222880909",
0x966c921a,"MainField_Item_Fruit_I_3224345050",
0x2e39f258,"MainField_Item_Fruit_I_3236114927",
0x236478a4,"MainField_Item_Fruit_I_3260367115",
0xbe3e282e,"MainField_Item_Fruit_I_3268510496",
0x7b22299c,"MainField_Item_Fruit_I_3273858085",
0x6a020826,"MainField_Item_Fruit_I_327396722",
0x10ed2a79,"MainField_Item_Fruit_I_3275331865",
0x36cce9f2,"MainField_Item_Fruit_I_3283441945",
0xe4edee69,"MainField_Item_Fruit_I_3290049611",
0x8b19f524,"MainField_Item_Fruit_I_3302530304",
0xf30f8bce,"MainField_Item_Fruit_I_3308815634",
0xed82891c,"MainField_Item_Fruit_I_332857577",
0x7cc9f0ae,"MainField_Item_Fruit_I_3343015363",
0x714f79fb,"MainField_Item_Fruit_I_3348035062",
0xe03bb83a,"MainField_Item_Fruit_I_3364721459",
0x6a6779c1,"MainField_Item_Fruit_I_3373503214",
0xe509a186,"MainField_Item_Fruit_I_3377823172",
0x739d895f,"MainField_Item_Fruit_I_3378037403",
0xa36ee386,"MainField_Item_Fruit_I_3378255238",
0x625b214e,"MainField_Item_Fruit_I_3379168158",
0x6ffa363e,"MainField_Item_Fruit_I_3380961295",
0xd6918ef3,"MainField_Item_Fruit_I_3400101480",
0x91ad9c3f,"MainField_Item_Fruit_I_3403192495",
0x4c5a2826,"MainField_Item_Fruit_I_3434198883",
0x21d7ffb0,"MainField_Item_Fruit_I_3456616327",
0x6ffc611a,"MainField_Item_Fruit_I_3469648189",
0xdb3f8630,"MainField_Item_Fruit_I_3472101864",
0x730ef93f,"MainField_Item_Fruit_I_3476151267",
0xbbc82d94,"MainField_Item_Fruit_I_347784432",
0x598c2384,"MainField_Item_Fruit_I_3480411647",
0xe8f7033b,"MainField_Item_Fruit_I_3487586090",
0x9e3cc9f8,"MainField_Item_Fruit_I_3489569814",
0xbe342acc,"MainField_Item_Fruit_I_3490227119",
0x196de5d6,"MainField_Item_Fruit_I_3497079519",
0xe3546fea,"MainField_Item_Fruit_I_3497882686",
0xc41f5337,"MainField_Item_Fruit_I_351168875",
0x39048038,"MainField_Item_Fruit_I_3512491624",
0x2ebe689e,"MainField_Item_Fruit_I_3539202623",
0x62136381,"MainField_Item_Fruit_I_3543083917",
0x79e57466,"MainField_Item_Fruit_I_3546363912",
0x81192366,"MainField_Item_Fruit_I_355176768",
0x14391b4f,"MainField_Item_Fruit_I_3552584168",
0x51de4e82,"MainField_Item_Fruit_I_3561131354",
0x3ddecf75,"MainField_Item_Fruit_I_3561156500",
0x3f4d1e54,"MainField_Item_Fruit_I_357695536",
0x471d850c,"MainField_Item_Fruit_I_3584726883",
0x2dec254b,"MainField_Item_Fruit_I_3587381074",
0x0866f309,"MainField_Item_Fruit_I_3598471888",
0xe9f45400,"MainField_Item_Fruit_I_3608195111",
0xf81a7d29,"MainField_Item_Fruit_I_3608999622",
0x16d02923,"MainField_Item_Fruit_I_3613747126",
0x36ac2e23,"MainField_Item_Fruit_I_362128817",
0x58aefd1c,"MainField_Item_Fruit_I_3623064919",
0x96c3b96f,"MainField_Item_Fruit_I_3632629845",
0x62321031,"MainField_Item_Fruit_I_3637801728",
0x14baca6c,"MainField_Item_Fruit_I_3638619550",
0xe20ec44a,"MainField_Item_Fruit_I_3639218755",
0x0bafc145,"MainField_Item_Fruit_I_364759959",
0x588c4152,"MainField_Item_Fruit_I_3651449357",
0xca211b49,"MainField_Item_Fruit_I_3656908974",
0x5011d8c5,"MainField_Item_Fruit_I_3661823016",
0xd50c96b4,"MainField_Item_Fruit_I_3668040756",
0xce74cedd,"MainField_Item_Fruit_I_3671371527",
0x356e7adf,"MainField_Item_Fruit_I_3678780948",
0x3ac754eb,"MainField_Item_Fruit_I_3693640205",
0x8876aa61,"MainField_Item_Fruit_I_3713274868",
0x209c41e1,"MainField_Item_Fruit_I_3719916318",
0x3830b267,"MainField_Item_Fruit_I_3726444299",
0x804ab39b,"MainField_Item_Fruit_I_3737744916",
0x09ac7a8e,"MainField_Item_Fruit_I_3760104226",
0x9b007607,"MainField_Item_Fruit_I_3769910517",
0xf66e02ba,"MainField_Item_Fruit_I_3773898937",
0x84c2d1e8,"MainField_Item_Fruit_I_3778435304",
0x21e0b8cd,"MainField_Item_Fruit_I_3783929737",
0x304363cf,"MainField_Item_Fruit_I_3792760650",
0xef678825,"MainField_Item_Fruit_I_3795763501",
0x4f8a25fd,"MainField_Item_Fruit_I_3796324445",
0xb98b55bf,"MainField_Item_Fruit_I_3799526919",
0xf5b12d5a,"MainField_Item_Fruit_I_3799978554",
0x366dca6e,"MainField_Item_Fruit_I_3804705040",
0xe617cca5,"MainField_Item_Fruit_I_3811369201",
0xc015c570,"MainField_Item_Fruit_I_3815449488",
0xafe04718,"MainField_Item_Fruit_I_3825657737",
0xb48af0c3,"MainField_Item_Fruit_I_3837131321",
0x99e130ca,"MainField_Item_Fruit_I_3839438641",
0x334eec59,"MainField_Item_Fruit_I_3844772932",
0x037637f8,"MainField_Item_Fruit_I_3844879298",
0xf2acff60,"MainField_Item_Fruit_I_3847423284",
0x03514fe0,"MainField_Item_Fruit_I_3850185555",
0xa98ee610,"MainField_Item_Fruit_I_3853186313",
0x93f05f98,"MainField_Item_Fruit_I_3873958800",
0xe8a1daf8,"MainField_Item_Fruit_I_3878392709",
0xb829de99,"MainField_Item_Fruit_I_3883092105",
0x12cc84c6,"MainField_Item_Fruit_I_3884989390",
0xd21c243b,"MainField_Item_Fruit_I_3887000093",
0xd38a5c3e,"MainField_Item_Fruit_I_3889646510",
0x03e96569,"MainField_Item_Fruit_I_389318491",
0xf38bd7ec,"MainField_Item_Fruit_I_3896271778",
0xfdd0352a,"MainField_Item_Fruit_I_3909224670",
0x5c859f83,"MainField_Item_Fruit_I_393073497",
0x70ac0076,"MainField_Item_Fruit_I_3937592739",
0x718e6610,"MainField_Item_Fruit_I_3941336269",
0xc8fc2ee3,"MainField_Item_Fruit_I_3962022822",
0xba2160a5,"MainField_Item_Fruit_I_396222884",
0x74f8003e,"MainField_Item_Fruit_I_3970147893",
0x78db147d,"MainField_Item_Fruit_I_3990823354",
0xc955b725,"MainField_Item_Fruit_I_3994408657",
0x31e682d7,"MainField_Item_Fruit_I_4001278354",
0x229602a2,"MainField_Item_Fruit_I_4008667659",
0x88b6a16a,"MainField_Item_Fruit_I_40149411",
0xb5ff4267,"MainField_Item_Fruit_I_4027079882",
0x360ab0f7,"MainField_Item_Fruit_I_4040556126",
0x73d8cb71,"MainField_Item_Fruit_I_4041944973",
0xd3976714,"MainField_Item_Fruit_I_4073391800",
0xcbd9ed8b,"MainField_Item_Fruit_I_4074066797",
0x41b1bee6,"MainField_Item_Fruit_I_4087495538",
0x05fefc51,"MainField_Item_Fruit_I_4099404978",
0xa8e970fe,"MainField_Item_Fruit_I_410894847",
0x48df2a2a,"MainField_Item_Fruit_I_4134772947",
0xe155d859,"MainField_Item_Fruit_I_4172549915",
0xdb21ee86,"MainField_Item_Fruit_I_4177903057",
0xcf815f04,"MainField_Item_Fruit_I_4187193480",
0xf75108a6,"MainField_Item_Fruit_I_4187258440",
0x1b378553,"MainField_Item_Fruit_I_4194866458",
0x2e9a2a4a,"MainField_Item_Fruit_I_4195609608",
0x67df0c4b,"MainField_Item_Fruit_I_4206142940",
0xaa8ec023,"MainField_Item_Fruit_I_4208986183",
0x7a15e02a,"MainField_Item_Fruit_I_4213567596",
0xaf6eca4d,"MainField_Item_Fruit_I_4217577286",
0x50e27509,"MainField_Item_Fruit_I_4221289629",
0x4657b413,"MainField_Item_Fruit_I_4226586326",
0x81a5252e,"MainField_Item_Fruit_I_4231111941",
0x5cdd923f,"MainField_Item_Fruit_I_4232590453",
0x6c85be24,"MainField_Item_Fruit_I_4243980420",
0x96bd2269,"MainField_Item_Fruit_I_4253928542",
0x8bfe1a80,"MainField_Item_Fruit_I_427465898",
0x7290eb5d,"MainField_Item_Fruit_I_4278076561",
0x725bfcdb,"MainField_Item_Fruit_I_442584258",
0xd4f31cac,"MainField_Item_Fruit_I_446944252",
0xdc2afe6a,"MainField_Item_Fruit_I_44943129",
0xb66d6589,"MainField_Item_Fruit_I_464036749",
0x0316b3a0,"MainField_Item_Fruit_I_464975448",
0x64841341,"MainField_Item_Fruit_I_46587860",
0x78ff03e4,"MainField_Item_Fruit_I_468719986",
0xb9ba7b6a,"MainField_Item_Fruit_I_484127297",
0x5b548fb3,"MainField_Item_Fruit_I_48965683",
0xfc79ab8d,"MainField_Item_Fruit_I_497795343",
0x3d78ee11,"MainField_Item_Fruit_I_523752873",
0x7e52ec21,"MainField_Item_Fruit_I_537503403",
0x9f4b3a8e,"MainField_Item_Fruit_I_569259526",
0xbc3c602b,"MainField_Item_Fruit_I_569441594",
0x2e644a64,"MainField_Item_Fruit_I_579287062",
0x4dab0ab2,"MainField_Item_Fruit_I_582997583",
0x9b5d994b,"MainField_Item_Fruit_I_593539079",
0xcfb7c7ef,"MainField_Item_Fruit_I_59744760",
0x5f863787,"MainField_Item_Fruit_I_602697735",
0xfa6f6ddf,"MainField_Item_Fruit_I_60545917",
0x1b4a1bbc,"MainField_Item_Fruit_I_637847204",
0x3ca43649,"MainField_Item_Fruit_I_64045206",
0xede8d70c,"MainField_Item_Fruit_I_646181407",
0xac4a6a38,"MainField_Item_Fruit_I_649783660",
0xc632d73e,"MainField_Item_Fruit_I_656274370",
0x6e0607aa,"MainField_Item_Fruit_I_680691255",
0x36427a1e,"MainField_Item_Fruit_I_692862191",
0x0f4b5f7e,"MainField_Item_Fruit_I_696292570",
0xb5d15b79,"MainField_Item_Fruit_I_696765281",
0x316ca981,"MainField_Item_Fruit_I_708884752",
0x432ff981,"MainField_Item_Fruit_I_710556716",
0x0f787a5d,"MainField_Item_Fruit_I_718732332",
0x06ea4373,"MainField_Item_Fruit_I_729912091",
0xe632baab,"MainField_Item_Fruit_I_744169511",
0x230f14b3,"MainField_Item_Fruit_I_758642741",
0xc6c00296,"MainField_Item_Fruit_I_762939774",
0xe23bd6af,"MainField_Item_Fruit_I_767244341",
0x8ac2c4bb,"MainField_Item_Fruit_I_767947504",
0xdda84e6c,"MainField_Item_Fruit_I_773978358",
0x2dc303a6,"MainField_Item_Fruit_I_783924126",
0x0c1aad3d,"MainField_Item_Fruit_I_785526425",
0xf9a38593,"MainField_Item_Fruit_I_785715876",
0x95dfe0aa,"MainField_Item_Fruit_I_786493340",
0x21a8123f,"MainField_Item_Fruit_I_787304643",
0x71f19828,"MainField_Item_Fruit_I_79463567",
0x000fd170,"MainField_Item_Fruit_I_815506843",
0xbcc9db30,"MainField_Item_Fruit_I_816356640",
0x123ae194,"MainField_Item_Fruit_I_818353976",
0x734f10eb,"MainField_Item_Fruit_I_819808945",
0x30602f93,"MainField_Item_Fruit_I_841358113",
0x70976f9f,"MainField_Item_Fruit_I_862676750",
0x83f017d7,"MainField_Item_Fruit_I_867633714",
0x74350f50,"MainField_Item_Fruit_I_882336836",
0x02b0b7aa,"MainField_Item_Fruit_I_894164792",
0x17959dc2,"MainField_Item_Fruit_I_900092485",
0x994ab1b9,"MainField_Item_Fruit_I_905718869",
0x6274b22f,"MainField_Item_Fruit_I_915138937",
0x8b16ed97,"MainField_Item_Fruit_I_920941423",
0xd567be3e,"MainField_Item_Fruit_I_933942049",
0x4e57ca07,"MainField_Item_Fruit_I_935927768",
0x837fb321,"MainField_Item_Fruit_I_947008508",
0x09f94aec,"MainField_Item_Fruit_I_94945869",
0x19d82e82,"MainField_Item_Fruit_I_958320907",
0x592b4bbb,"MainField_Item_Fruit_I_961758461",
0x0cc7d750,"MainField_Item_Fruit_I_963315354",
0x606e5b5a,"MainField_Item_Fruit_I_96469760",
0x56b8422c,"MainField_Item_Fruit_I_969060317",
0x1b364ee1,"MainField_Item_Fruit_I_973249853",
0x02c165e6,"MainField_Item_Fruit_I_978095800",
0x677e367c,"MainField_Item_Fruit_I_978119798",
0xf85a9909,"MainField_Item_Fruit_I_994006592",
0xd9e587a1,"MainField_Item_Fruit_I_994829073",
0x680ae6ce,"MainField_Item_Fruit_J_1708150109",
0x692a6c6a,"MainField_Item_Fruit_J_1818624374",
0x3b43765c,"MainField_Item_Fruit_J_1913713710",
0xe324a45a,"MainField_Item_Fruit_J_2187878490",
0xaed9d7ca,"MainField_Item_Fruit_J_316112331",
0x2936f666,"MainField_Item_Fruit_J_3944326548",
0x8f6b76e0,"MainField_Item_Fruit_J_4117713100",
0x6dbdc16c,"MainField_Item_Fruit_J_85443768",
0x820ab8ba,"MainField_Item_Fruit_K_1043613914",
0x70c72e59,"MainField_Item_Fruit_K_1136705905",
0xf2dc5b3f,"MainField_Item_Fruit_K_1396947182",
0x3b09c8f4,"MainField_Item_Fruit_K_160816153",
0x498b9900,"MainField_Item_Fruit_K_1615822993",
0x33e95331,"MainField_Item_Fruit_K_1616500019",
0x47110f76,"MainField_Item_Fruit_K_1726627302",
0xfe747f29,"MainField_Item_Fruit_K_1799217286",
0x869b1245,"MainField_Item_Fruit_K_1826269150",
0x7ab86d8a,"MainField_Item_Fruit_K_2034373545",
0x383fc978,"MainField_Item_Fruit_K_204693573",
0xc4326de5,"MainField_Item_Fruit_K_2073600214",
0x8e1cd037,"MainField_Item_Fruit_K_2120511396",
0xb5421989,"MainField_Item_Fruit_K_2321191261",
0xb4e2122b,"MainField_Item_Fruit_K_2347453566",
0x80b66477,"MainField_Item_Fruit_K_2429781299",
0x0fcaf114,"MainField_Item_Fruit_K_300195696",
0xa67b54cc,"MainField_Item_Fruit_K_303287689",
0xa9ed831c,"MainField_Item_Fruit_K_3106981976",
0xe7c5ba8d,"MainField_Item_Fruit_K_3141713296",
0xe853dbdd,"MainField_Item_Fruit_K_3184626446",
0x7d35ff7b,"MainField_Item_Fruit_K_3266913661",
0xd5da2d3a,"MainField_Item_Fruit_K_3420332209",
0x9b0f2f54,"MainField_Item_Fruit_K_3841919166",
0x2ee0f7b8,"MainField_Item_Fruit_K_4105293404",
0x1b5e2c3d,"MainField_Item_Fruit_K_4124574406",
0x8611b388,"MainField_Item_Fruit_K_4245165520",
0xaeddf608,"MainField_Item_Fruit_K_596720204",
0x2d13f789,"MainField_Item_Fruit_K_670409329",
0x4b8e52a1,"MainField_Item_Fruit_K_767775474",
0x3a0d47b9,"MainField_Item_Fruit_K_905048355",
0xafc564d1,"MainField_Item_Fruit_K_968091085",
0x8b2c2e60,"MainField_Item_Fruit_L_1992698449",
0x1467d59f,"MainField_Item_Fruit_L_3744282354",
0xe864afa1,"MainField_Item_Fruit_L_3962620345",
0x1969057c,"MainField_Item_Fruit_L_4218557013",
0xee2e58e4,"MainField_Item_Fruit_L_663867221",
0xfee42cee,"MainField_Item_Fruit_L_873339241",
0xecb81e7a,"MainField_Item_InsectGet_O_3317923458",
0x2b6f5e8c,"MainField_Item_InsectGet_O_3471006514",
0x235cf1c1,"MainField_Item_InsectGet_O_4103452247",
0x9cca7cd5,"MainField_Item_Material_01_1287359318",
0x0cd05bc5,"MainField_Item_Material_01_1456518560",
0xfbbfbefe,"MainField_Item_Material_01_1686888927",
0x5421db35,"MainField_Item_Material_01_1736949412",
0xcaa6b4a8,"MainField_Item_Material_01_1964101291",
0x89f7f848,"MainField_Item_Material_01_3043415669",
0xa70ba0db,"MainField_Item_Material_01_3559016972",
0xbdeb84ca,"MainField_Item_Material_01_4270780190",
0x97c0b115,"MainField_Item_Material_01_732968041",
0x76dde9b2,"MainField_Item_Material_02_1520764356",
0x3f68da03,"MainField_Item_Material_02_705800106",
0xcfaba066,"MainField_Item_Material_02_957690492",
0x039b0852,"MainField_Item_Material_03_1058157746",
0xf70d422c,"MainField_Item_Material_03_1163833336",
0x6a8b7b6e,"MainField_Item_Material_03_1497053731",
0x371f3365,"MainField_Item_Material_03_1748903081",
0x8de95282,"MainField_Item_Material_03_2136474935",
0xdcb3280a,"MainField_Item_Material_03_2178080993",
0x70a34762,"MainField_Item_Material_03_2227244148",
0x887efc89,"MainField_Item_Material_03_2728624675",
0x259e5322,"MainField_Item_Material_03_2735098594",
0x45551b6f,"MainField_Item_Material_03_2816316921",
0xdb2ede13,"MainField_Item_Material_03_360418546",
0xf1b9e91e,"MainField_Item_Material_03_3762182747",
0xdbf4ddf1,"MainField_Item_Material_03_4148147339",
0x22b93a21,"MainField_Item_Material_03_837449055",
0x6e6d817a,"MainField_Item_Material_03_882863430",
0xbdf0629d,"MainField_Item_Material_04_1000244252",
0xf66a4a81,"MainField_Item_Material_04_1002477084",
0x937b988f,"MainField_Item_Material_04_1004080774",
0xfe21c94d,"MainField_Item_Material_04_1023749300",
0xfb33c1b5,"MainField_Item_Material_04_1024228672",
0x23b86ebd,"MainField_Item_Material_04_1027678510",
0x5c0acc49,"MainField_Item_Material_04_1039303180",
0x4d27d73a,"MainField_Item_Material_04_1052805787",
0xb8d1d416,"MainField_Item_Material_04_1059685995",
0x12b52123,"MainField_Item_Material_04_1061864217",
0x952f593d,"MainField_Item_Material_04_1076555174",
0xc7943a14,"MainField_Item_Material_04_1078348831",
0x2b386cc2,"MainField_Item_Material_04_1079533299",
0x6fe87e7d,"MainField_Item_Material_04_1085328384",
0x14a6eb28,"MainField_Item_Material_04_1088868074",
0x617b5588,"MainField_Item_Material_04_1102099142",
0xba648896,"MainField_Item_Material_04_111074349",
0x1cded52e,"MainField_Item_Material_04_1114608924",
0x17860a0e,"MainField_Item_Material_04_1114631959",
0x143eee45,"MainField_Item_Material_04_1129886971",
0x6d8d5981,"MainField_Item_Material_04_1139826067",
0xe66cc277,"MainField_Item_Material_04_1152984027",
0xa8f53b6c,"MainField_Item_Material_04_1154937053",
0x45412975,"MainField_Item_Material_04_1173050262",
0x9d771867,"MainField_Item_Material_04_1174581885",
0x1e142cac,"MainField_Item_Material_04_1180029332",
0x6e1b59fe,"MainField_Item_Material_04_118656068",
0xd7f23263,"MainField_Item_Material_04_1189079931",
0x92c824b1,"MainField_Item_Material_04_1202116157",
0x69ff323f,"MainField_Item_Material_04_1217749632",
0x8db84a63,"MainField_Item_Material_04_121986976",
0x480840b3,"MainField_Item_Material_04_1220188404",
0xed5cbb2c,"MainField_Item_Material_04_1237826061",
0x8398492e,"MainField_Item_Material_04_1238176464",
0xb85fb950,"MainField_Item_Material_04_124220832",
0x5910e93d,"MainField_Item_Material_04_1246106040",
0xd98d6762,"MainField_Item_Material_04_1250605929",
0xac91f9ac,"MainField_Item_Material_04_1257616026",
0xfbc153ac,"MainField_Item_Material_04_1270386060",
0xd7ad6a08,"MainField_Item_Material_04_1270388607",
0x113e81ce,"MainField_Item_Material_04_1279482837",
0xf5e3cc04,"MainField_Item_Material_04_1288412369",
0x19a6535e,"MainField_Item_Material_04_1288589998",
0xbd4e72d9,"MainField_Item_Material_04_128879214",
0x53fae6b7,"MainField_Item_Material_04_1294357468",
0x17c03424,"MainField_Item_Material_04_1294718369",
0x7f790f10,"MainField_Item_Material_04_1303593834",
0xa072814e,"MainField_Item_Material_04_1306433961",
0xf7c53b79,"MainField_Item_Material_04_131910179",
0x7e2ebccf,"MainField_Item_Material_04_1332541731",
0x04294b26,"MainField_Item_Material_04_1338689100",
0xf5331b89,"MainField_Item_Material_04_1347417027",
0xc12adb6e,"MainField_Item_Material_04_1356630860",
0xc23ddacd,"MainField_Item_Material_04_1356832280",
0xce56cade,"MainField_Item_Material_04_1360470704",
0x405debd9,"MainField_Item_Material_04_1394705295",
0xa37231d2,"MainField_Item_Material_04_1397794436",
0xb8c45662,"MainField_Item_Material_04_1397961239",
0x640950c8,"MainField_Item_Material_04_1406625026",
0x1f7132e7,"MainField_Item_Material_04_1408498628",
0xe2a13011,"MainField_Item_Material_04_1417383351",
0xc18f1f7a,"MainField_Item_Material_04_1417889730",
0x3555d381,"MainField_Item_Material_04_142430904",
0x0dfa643d,"MainField_Item_Material_04_1426770132",
0x8eaa1b12,"MainField_Item_Material_04_1454353734",
0xf98a2694,"MainField_Item_Material_04_1455900213",
0x62a3bacc,"MainField_Item_Material_04_1468587134",
0xdc9b67da,"MainField_Item_Material_04_148254052",
0xd8810194,"MainField_Item_Material_04_1488240980",
0x213a6b2a,"MainField_Item_Material_04_1490573242",
0x0a61c5ac,"MainField_Item_Material_04_1491873062",
0x7f71e657,"MainField_Item_Material_04_1500170087",
0xd41e75a8,"MainField_Item_Material_04_1521320952",
0x0453b776,"MainField_Item_Material_04_1521738630",
0x7453b7b0,"MainField_Item_Material_04_1530484642",
0x1bc634b2,"MainField_Item_Material_04_1531265903",
0x8d21788f,"MainField_Item_Material_04_1545510744",
0xef643f77,"MainField_Item_Material_04_1562416448",
0xb4d8605f,"MainField_Item_Material_04_1566242383",
0xde4d856e,"MainField_Item_Material_04_1569367193",
0xcbce7714,"MainField_Item_Material_04_1572882401",
0x1d0e8eae,"MainField_Item_Material_04_1577517950",
0x3a60e56a,"MainField_Item_Material_04_1581600180",
0x2eb9f71f,"MainField_Item_Material_04_1583094014",
0x36b3c641,"MainField_Item_Material_04_1586268898",
0xc726e087,"MainField_Item_Material_04_1586510770",
0x56ab72d6,"MainField_Item_Material_04_1593376196",
0x0221caa5,"MainField_Item_Material_04_1594531345",
0x9697c4e3,"MainField_Item_Material_04_1625542653",
0x9e230a28,"MainField_Item_Material_04_1630835153",
0x1483c4b1,"MainField_Item_Material_04_1654076060",
0x10560891,"MainField_Item_Material_04_1665446472",
0xa3c30af0,"MainField_Item_Material_04_1665677191",
0x520d8f2d,"MainField_Item_Material_04_1674076039",
0x6f71edf2,"MainField_Item_Material_04_1676010831",
0x1532a18b,"MainField_Item_Material_04_1678035024",
0x0c267fb5,"MainField_Item_Material_04_1685354661",
0x8704848b,"MainField_Item_Material_04_1691980721",
0x9ae3acc1,"MainField_Item_Material_04_1704866868",
0x0df38e18,"MainField_Item_Material_04_170783784",
0xd9a47efb,"MainField_Item_Material_04_1718243712",
0xf2dc9d2e,"MainField_Item_Material_04_1732959004",
0xa63ce1a4,"MainField_Item_Material_04_1738515228",
0x36f80146,"MainField_Item_Material_04_1743296507",
0xd90033f4,"MainField_Item_Material_04_1750551467",
0x9e98b21d,"MainField_Item_Material_04_1753245345",
0xf9247ab4,"MainField_Item_Material_04_1753406970",
0x64c5b129,"MainField_Item_Material_04_1756486066",
0x4b33a2bc,"MainField_Item_Material_04_1760100071",
0x3ee1e802,"MainField_Item_Material_04_1772467387",
0xaf7fdf2f,"MainField_Item_Material_04_1775708239",
0xd0200223,"MainField_Item_Material_04_1777651736",
0x689a5313,"MainField_Item_Material_04_1788486354",
0xc935a5e6,"MainField_Item_Material_04_1789694695",
0xfe2a9c19,"MainField_Item_Material_04_1790835855",
0x053d25df,"MainField_Item_Material_04_1803069815",
0xc92f5eaf,"MainField_Item_Material_04_1807798901",
0xcb16b433,"MainField_Item_Material_04_18151782",
0x11ef924b,"MainField_Item_Material_04_1817215816",
0x64405d23,"MainField_Item_Material_04_18196394",
0x6b9aa9bf,"MainField_Item_Material_04_1821298427",
0x06e79282,"MainField_Item_Material_04_1823665362",
0x2d0db4f6,"MainField_Item_Material_04_1825509866",
0x52ae0de3,"MainField_Item_Material_04_1828181488",
0x2a51f9ff,"MainField_Item_Material_04_1839730763",
0x80a039af,"MainField_Item_Material_04_1842811268",
0x4da20402,"MainField_Item_Material_04_185736621",
0xd881c73f,"MainField_Item_Material_04_1858629794",
0xc6ca81b0,"MainField_Item_Material_04_1861435957",
0xe96805ee,"MainField_Item_Material_04_1863478760",
0x4c07fce2,"MainField_Item_Material_04_1869169413",
0xf41e491b,"MainField_Item_Material_04_1870375435",
0x0f06d25c,"MainField_Item_Material_04_1870377208",
0x273952a7,"MainField_Item_Material_04_1877940707",
0x503f31d8,"MainField_Item_Material_04_1886488630",
0xe034f45e,"MainField_Item_Material_04_1893436752",
0xcb1a82e8,"MainField_Item_Material_04_1895409488",
0x382fa419,"MainField_Item_Material_04_1903757416",
0xe1797977,"MainField_Item_Material_04_1920453360",
0x50e999cb,"MainField_Item_Material_04_1932843847",
0x60f7b3a6,"MainField_Item_Material_04_1934137381",
0x67c304fe,"MainField_Item_Material_04_1938475701",
0x2ad5fff0,"MainField_Item_Material_04_1939345136",
0x73fc8c55,"MainField_Item_Material_04_1974536251",
0xc669cd96,"MainField_Item_Material_04_1974789726",
0x88c7f4fb,"MainField_Item_Material_04_1981923059",
0x17d37de9,"MainField_Item_Material_04_1982368569",
0xef3c707a,"MainField_Item_Material_04_19905455",
0x523d2c46,"MainField_Item_Material_04_1991338335",
0x5a89d5e7,"MainField_Item_Material_04_1992764650",
0x86a57c6d,"MainField_Item_Material_04_2016387216",
0x70970223,"MainField_Item_Material_04_2023206607",
0x50ff671d,"MainField_Item_Material_04_2025642475",
0x10734766,"MainField_Item_Material_04_2033763977",
0xd30d4441,"MainField_Item_Material_04_203551251",
0x33fe3eb0,"MainField_Item_Material_04_2037236091",
0x8cf1c5fb,"MainField_Item_Material_04_2040787193",
0xa51fb74e,"MainField_Item_Material_04_2041543236",
0x085c8d0b,"MainField_Item_Material_04_2047484307",
0xdbf1ee30,"MainField_Item_Material_04_2053470209",
0x98d7f924,"MainField_Item_Material_04_2056578140",
0x25493839,"MainField_Item_Material_04_2057429436",
0x824435bc,"MainField_Item_Material_04_2063786146",
0x0c8a900f,"MainField_Item_Material_04_206470442",
0x064d5ab6,"MainField_Item_Material_04_2072565313",
0x7244c784,"MainField_Item_Material_04_2075277162",
0xbd30fede,"MainField_Item_Material_04_2089602909",
0x5ba84b6e,"MainField_Item_Material_04_2092282947",
0xd8c3e83b,"MainField_Item_Material_04_2135048822",
0x31f54e8e,"MainField_Item_Material_04_2139975415",
0xf7727789,"MainField_Item_Material_04_2160464270",
0xc4f97b0b,"MainField_Item_Material_04_2161066106",
0x07c22763,"MainField_Item_Material_04_2179037391",
0x856cc8b6,"MainField_Item_Material_04_2189445150",
0xad5dfc7c,"MainField_Item_Material_04_2189471703",
0xfa7331e9,"MainField_Item_Material_04_2195974821",
0x3c692a8f,"MainField_Item_Material_04_220855495",
0x80fd0468,"MainField_Item_Material_04_2225271755",
0x3ce3589e,"MainField_Item_Material_04_2236696016",
0x36867fbc,"MainField_Item_Material_04_2248866514",
0x3094eee1,"MainField_Item_Material_04_2249417288",
0x77bd1e3b,"MainField_Item_Material_04_2253711570",
0xd17d6264,"MainField_Item_Material_04_2253990535",
0xbf06c739,"MainField_Item_Material_04_2257370326",
0x9c80071a,"MainField_Item_Material_04_2257571452",
0x28e9b428,"MainField_Item_Material_04_2261365727",
0xd201a6f6,"MainField_Item_Material_04_2272597339",
0xc141eb20,"MainField_Item_Material_04_2274536822",
0x8cea4594,"MainField_Item_Material_04_2279164259",
0xfe6b7b44,"MainField_Item_Material_04_228446355",
0xe05625f7,"MainField_Item_Material_04_2302982704",
0x666738f0,"MainField_Item_Material_04_2309164412",
0xfb7ce2e9,"MainField_Item_Material_04_2325586340",
0x8ce890e4,"MainField_Item_Material_04_2330793463",
0xa88ad839,"MainField_Item_Material_04_233298335",
0x1e546b17,"MainField_Item_Material_04_2341882946",
0xa01b200f,"MainField_Item_Material_04_2352561998",
0x73fa3d94,"MainField_Item_Material_04_2359288990",
0x34eee8ce,"MainField_Item_Material_04_2375824366",
0x6ce9bc2a,"MainField_Item_Material_04_238135476",
0x0d26f520,"MainField_Item_Material_04_2394649005",
0xc21493a8,"MainField_Item_Material_04_2422180721",
0xf4bed51e,"MainField_Item_Material_04_2423493895",
0xf3edcb9a,"MainField_Item_Material_04_2444833356",
0xee0b9d33,"MainField_Item_Material_04_2452285577",
0xeda99e16,"MainField_Item_Material_04_2456452456",
0x3caac542,"MainField_Item_Material_04_246646118",
0xb66f09c5,"MainField_Item_Material_04_2467435943",
0xc6858509,"MainField_Item_Material_04_2468652434",
0x57ccf0dc,"MainField_Item_Material_04_2474297622",
0x3dffe4ab,"MainField_Item_Material_04_2480643601",
0x0ff332f6,"MainField_Item_Material_04_2484538639",
0x73991e73,"MainField_Item_Material_04_248586374",
0x23966a0b,"MainField_Item_Material_04_2490415908",
0x07ad8df9,"MainField_Item_Material_04_2491876015",
0x71f98ba1,"MainField_Item_Material_04_2506166080",
0xd640433d,"MainField_Item_Material_04_250811275",
0x16cba147,"MainField_Item_Material_04_2514532634",
0x81c69ce7,"MainField_Item_Material_04_2530493975",
0x8d2469b7,"MainField_Item_Material_04_2539481175",
0x101a0f9d,"MainField_Item_Material_04_2539888727",
0x920c2fdc,"MainField_Item_Material_04_254316550",
0xbea196da,"MainField_Item_Material_04_2547990819",
0x1f869680,"MainField_Item_Material_04_2549760561",
0xc9bf291c,"MainField_Item_Material_04_2566010206",
0xea826f6d,"MainField_Item_Material_04_2574653927",
0x3933eab4,"MainField_Item_Material_04_2579880236",
0xe9c5d0d3,"MainField_Item_Material_04_2613012759",
0x9017f5f1,"MainField_Item_Material_04_2613656790",
0x13594962,"MainField_Item_Material_04_2625626904",
0x6f821d14,"MainField_Item_Material_04_2647965120",
0x3fb8b483,"MainField_Item_Material_04_2653953575",
0x9918a85c,"MainField_Item_Material_04_2668172920",
0xca34adeb,"MainField_Item_Material_04_2677509527",
0xf9d1fa6e,"MainField_Item_Material_04_2681392234",
0xb079603d,"MainField_Item_Material_04_2690526682",
0x6a87b820,"MainField_Item_Material_04_2702020141",
0xbca28c97,"MainField_Item_Material_04_2706338170",
0xb28c60d5,"MainField_Item_Material_04_2722055451",
0x5da76ac0,"MainField_Item_Material_04_2733363944",
0x0aaed572,"MainField_Item_Material_04_2735306257",
0x62eb7bb3,"MainField_Item_Material_04_2737157261",
0x8464be79,"MainField_Item_Material_04_2737790129",
0x1e49dd19,"MainField_Item_Material_04_2744078542",
0x02b47167,"MainField_Item_Material_04_2756210836",
0x07f99430,"MainField_Item_Material_04_2759815727",
0x835dc63e,"MainField_Item_Material_04_276287884",
0xd770321c,"MainField_Item_Material_04_2766573641",
0x13f88077,"MainField_Item_Material_04_2768834160",
0x1c080fd4,"MainField_Item_Material_04_2769025256",
0x15443bbd,"MainField_Item_Material_04_2769371635",
0x73ac4b41,"MainField_Item_Material_04_2778996573",
0x4aa7bd8f,"MainField_Item_Material_04_2792790725",
0x4a9f1c6a,"MainField_Item_Material_04_2805197251",
0xc79bb765,"MainField_Item_Material_04_2806644421",
0x7ebe88bd,"MainField_Item_Material_04_2810629569",
0xa6423695,"MainField_Item_Material_04_2816654822",
0xd95c4afb,"MainField_Item_Material_04_2817309119",
0x1edc1e9f,"MainField_Item_Material_04_2840234835",
0x9210cc43,"MainField_Item_Material_04_2858932015",
0xc6229dda,"MainField_Item_Material_04_28727753",
0x978f2eaf,"MainField_Item_Material_04_2877612760",
0x1cf6f9b2,"MainField_Item_Material_04_2890928898",
0x182f0b9d,"MainField_Item_Material_04_2914342376",
0x4b5abcb8,"MainField_Item_Material_04_2914893122",
0xc445f121,"MainField_Item_Material_04_2927597322",
0x537c0c2b,"MainField_Item_Material_04_2929950079",
0x0609d8f1,"MainField_Item_Material_04_2931731756",
0xf2cc7ec7,"MainField_Item_Material_04_2943607460",
0xcfd48fc6,"MainField_Item_Material_04_2945989730",
0x10e63aec,"MainField_Item_Material_04_295337932",
0x91ef29c2,"MainField_Item_Material_04_2953405010",
0x6d4c142e,"MainField_Item_Material_04_2958539869",
0x765d43fa,"MainField_Item_Material_04_2967703561",
0xae6462b4,"MainField_Item_Material_04_2978222564",
0x3694640b,"MainField_Item_Material_04_2985008861",
0xa548207d,"MainField_Item_Material_04_3004910664",
0xfa87bdce,"MainField_Item_Material_04_3011111336",
0x9f6d7b9a,"MainField_Item_Material_04_3011347951",
0x0a4a6722,"MainField_Item_Material_04_3016632013",
0x25b6e492,"MainField_Item_Material_04_302048964",
0x424d2a85,"MainField_Item_Material_04_3026458953",
0x8e9298b2,"MainField_Item_Material_04_3030639316",
0x7ef4a1cd,"MainField_Item_Material_04_3035138947",
0x41f843a7,"MainField_Item_Material_04_3041305836",
0x316e43a1,"MainField_Item_Material_04_3048685174",
0x181fc18b,"MainField_Item_Material_04_307384492",
0x68acae96,"MainField_Item_Material_04_3076826699",
0x0c34cc41,"MainField_Item_Material_04_3090312710",
0xf968fe4a,"MainField_Item_Material_04_3092629166",
0x0f7a31bb,"MainField_Item_Material_04_3096346824",
0x10401b6a,"MainField_Item_Material_04_309879209",
0x0ca60249,"MainField_Item_Material_04_3099540796",
0x58c9c232,"MainField_Item_Material_04_3101390709",
0x564037a3,"MainField_Item_Material_04_3106690806",
0x7eb0be23,"MainField_Item_Material_04_3122046701",
0x865e13a5,"MainField_Item_Material_04_3127174816",
0x8fe0cbf8,"MainField_Item_Material_04_3128453862",
0xbbcca360,"MainField_Item_Material_04_3129341723",
0xc0db7714,"MainField_Item_Material_04_3134775434",
0x381394a6,"MainField_Item_Material_04_3136433949",
0x080a3c02,"MainField_Item_Material_04_314150506",
0x4f1a2b20,"MainField_Item_Material_04_3150987217",
0x0b5dc42c,"MainField_Item_Material_04_3168703912",
0xc700f478,"MainField_Item_Material_04_3168783204",
0x6726b91e,"MainField_Item_Material_04_3169981152",
0x671b506f,"MainField_Item_Material_04_3175397582",
0x76a94f9e,"MainField_Item_Material_04_3177996889",
0xad871c84,"MainField_Item_Material_04_3195684013",
0x513895f9,"MainField_Item_Material_04_3200805781",
0x96911872,"MainField_Item_Material_04_3211426755",
0x2a9b0edc,"MainField_Item_Material_04_3227752157",
0xcebb69ea,"MainField_Item_Material_04_3238872962",
0x97826aaa,"MainField_Item_Material_04_3246381197",
0x125fc5b6,"MainField_Item_Material_04_3252944576",
0xf795f4cc,"MainField_Item_Material_04_325652913",
0x0485dbc9,"MainField_Item_Material_04_326322227",
0x06d14e4b,"MainField_Item_Material_04_3263704760",
0x28f905fd,"MainField_Item_Material_04_3276906058",
0x8cfa31ed,"MainField_Item_Material_04_3282358850",
0xde47c824,"MainField_Item_Material_04_3282654271",
0xd518382d,"MainField_Item_Material_04_3288710282",
0x554c4869,"MainField_Item_Material_04_3295255607",
0xe415807e,"MainField_Item_Material_04_3296376185",
0xe26b7d81,"MainField_Item_Material_04_3301512795",
0x12d30f5a,"MainField_Item_Material_04_3304237639",
0xcbf1767a,"MainField_Item_Material_04_3311256065",
0xea463442,"MainField_Item_Material_04_3326261903",
0x16d021f9,"MainField_Item_Material_04_3329776548",
0x7b76bfb3,"MainField_Item_Material_04_3337156624",
0xc7dc26c4,"MainField_Item_Material_04_3339604676",
0xa59e8825,"MainField_Item_Material_04_3344861330",
0xd5ee5226,"MainField_Item_Material_04_3345714546",
0x13f9d6ef,"MainField_Item_Material_04_3346848355",
0x3ebd4f1f,"MainField_Item_Material_04_3350140111",
0xe0ad254f,"MainField_Item_Material_04_3350272522",
0xbcbc0079,"MainField_Item_Material_04_3351268701",
0xa249050c,"MainField_Item_Material_04_3363736208",
0xd31e00d8,"MainField_Item_Material_04_3369784951",
0x94112c6f,"MainField_Item_Material_04_3371477299",
0x10090045,"MainField_Item_Material_04_3394893630",
0xd048cfa6,"MainField_Item_Material_04_3406643236",
0x8f04eabb,"MainField_Item_Material_04_3412901904",
0x37bd0fdd,"MainField_Item_Material_04_3434308774",
0x6c49280e,"MainField_Item_Material_04_3441108459",
0x50d2cb8e,"MainField_Item_Material_04_3447675504",
0xa1ea5d53,"MainField_Item_Material_04_3451643128",
0x833e6109,"MainField_Item_Material_04_3460343419",
0x9de7a015,"MainField_Item_Material_04_3461505598",
0x703a00c8,"MainField_Item_Material_04_3467805702",
0xf5a8ddfb,"MainField_Item_Material_04_3472799625",
0x087fd6bb,"MainField_Item_Material_04_3477448069",
0xe420f156,"MainField_Item_Material_04_3484204278",
0x401c5d68,"MainField_Item_Material_04_3489893633",
0x4e21aa21,"MainField_Item_Material_04_3496115333",
0x9a6b9217,"MainField_Item_Material_04_3510483262",
0x5b969cfb,"MainField_Item_Material_04_3510936117",
0x9779b19b,"MainField_Item_Material_04_352968812",
0xdff66462,"MainField_Item_Material_04_3532991000",
0x71a7e8fb,"MainField_Item_Material_04_3533909963",
0xee40e340,"MainField_Item_Material_04_3534866117",
0x25a4e51b,"MainField_Item_Material_04_3534871606",
0x66791cef,"MainField_Item_Material_04_3540130924",
0xb755dbfd,"MainField_Item_Material_04_3545186306",
0x879adf78,"MainField_Item_Material_04_3547873977",
0x3c2a5d99,"MainField_Item_Material_04_3554779359",
0x3b708114,"MainField_Item_Material_04_355495668",
0x23758682,"MainField_Item_Material_04_3559031835",
0x4bca3dd0,"MainField_Item_Material_04_3560881366",
0x64350008,"MainField_Item_Material_04_3564099701",
0x6ebf628e,"MainField_Item_Material_04_3575812649",
0x64c014b8,"MainField_Item_Material_04_3578399343",
0x8aeb6d6a,"MainField_Item_Material_04_3581654396",
0x3b1ce46f,"MainField_Item_Material_04_3591028411",
0x0a8bbaab,"MainField_Item_Material_04_3641966463",
0xf5edc9dc,"MainField_Item_Material_04_3642785756",
0x892ab2da,"MainField_Item_Material_04_3647482795",
0x1697fbe6,"MainField_Item_Material_04_3648563100",
0xfebb52e2,"MainField_Item_Material_04_3654198430",
0x91e98cb4,"MainField_Item_Material_04_3658235937",
0xa117ec4a,"MainField_Item_Material_04_3659725621",
0x206d456a,"MainField_Item_Material_04_3665868754",
0xb4ac11a6,"MainField_Item_Material_04_3673459284",
0x47a19698,"MainField_Item_Material_04_3679636504",
0xa4583188,"MainField_Item_Material_04_3684711758",
0xa43884fe,"MainField_Item_Material_04_3695118337",
0x93e1a2e6,"MainField_Item_Material_04_3710493491",
0x06f04207,"MainField_Item_Material_04_3713557217",
0x19431089,"MainField_Item_Material_04_3722420583",
0xea271554,"MainField_Item_Material_04_3724686153",
0x7d2426f9,"MainField_Item_Material_04_372482375",
0xbd0b1abb,"MainField_Item_Material_04_3729824251",
0x0881fc5e,"MainField_Item_Material_04_3730727104",
0xb57d98e9,"MainField_Item_Material_04_3733085001",
0x0bf70341,"MainField_Item_Material_04_3738458531",
0x9cc3facc,"MainField_Item_Material_04_374246934",
0xaffb2787,"MainField_Item_Material_04_3754726434",
0xd011fc90,"MainField_Item_Material_04_3762341975",
0xe79ba3f1,"MainField_Item_Material_04_3762394067",
0x20064691,"MainField_Item_Material_04_3764687041",
0x6d3cc9b4,"MainField_Item_Material_04_3770923687",
0x32e3cec1,"MainField_Item_Material_04_3773495694",
0x4374667b,"MainField_Item_Material_04_3774706343",
0xa62bbc05,"MainField_Item_Material_04_3788829590",
0xd8f000ef,"MainField_Item_Material_04_3810119775",
0x286bc631,"MainField_Item_Material_04_3811741962",
0x77434a29,"MainField_Item_Material_04_3814348668",
0x35a2ebf4,"MainField_Item_Material_04_3815256995",
0xfb13759a,"MainField_Item_Material_04_3817808250",
0x45a3143a,"MainField_Item_Material_04_3818288176",
0x23c2f1ce,"MainField_Item_Material_04_3826591025",
0x3de4104d,"MainField_Item_Material_04_3827537156",
0xf8aea2dc,"MainField_Item_Material_04_3834843269",
0x33666efe,"MainField_Item_Material_04_3843177332",
0x01c47f0b,"MainField_Item_Material_04_3844489704",
0x25555411,"MainField_Item_Material_04_3858018200",
0x2a36e689,"MainField_Item_Material_04_3866036470",
0x3f19ccb7,"MainField_Item_Material_04_3868143521",
0x17f810c2,"MainField_Item_Material_04_3872934984",
0x232b73a2,"MainField_Item_Material_04_3874700140",
0x980b2c2d,"MainField_Item_Material_04_387550380",
0xe39c3238,"MainField_Item_Material_04_3877552262",
0xc7556465,"MainField_Item_Material_04_3883503432",
0x7125c935,"MainField_Item_Material_04_3897302682",
0xc24805bb,"MainField_Item_Material_04_3901799681",
0x77d9023b,"MainField_Item_Material_04_3904939034",
0x94e4be82,"MainField_Item_Material_04_3920547031",
0xd925b98c,"MainField_Item_Material_04_3930239152",
0xcbcf0d8b,"MainField_Item_Material_04_3943508731",
0xf0a8ef4e,"MainField_Item_Material_04_3952454897",
0x31a3c999,"MainField_Item_Material_04_397261793",
0x2c669ecc,"MainField_Item_Material_04_3973926526",
0xe8247387,"MainField_Item_Material_04_3983205036",
0xf48ec28d,"MainField_Item_Material_04_3989550507",
0x7c6f98d9,"MainField_Item_Material_04_3996751626",
0x2de93e93,"MainField_Item_Material_04_4027350918",
0xdceaa647,"MainField_Item_Material_04_404053374",
0x6678ea6d,"MainField_Item_Material_04_4041112053",
0xbd5936aa,"MainField_Item_Material_04_4041983915",
0xed85cb6d,"MainField_Item_Material_04_4043587168",
0xf111f143,"MainField_Item_Material_04_4044430476",
0xba3affe9,"MainField_Item_Material_04_4052301595",
0xa95cf67d,"MainField_Item_Material_04_4052454464",
0xe4ae8148,"MainField_Item_Material_04_4073690912",
0xac633bc7,"MainField_Item_Material_04_4094131742",
0x2d98f937,"MainField_Item_Material_04_409891731",
0x12c48706,"MainField_Item_Material_04_4116647502",
0x4216e143,"MainField_Item_Material_04_4130594312",
0x02771d7d,"MainField_Item_Material_04_4134723906",
0x19d5be56,"MainField_Item_Material_04_4134730673",
0x1aeb3b2b,"MainField_Item_Material_04_4142286937",
0x94a0e45f,"MainField_Item_Material_04_4143744127",
0x190950c9,"MainField_Item_Material_04_4162242881",
0x0d5276c3,"MainField_Item_Material_04_417017149",
0x36b88966,"MainField_Item_Material_04_4172200208",
0x0b4238e8,"MainField_Item_Material_04_417868149",
0x6366dfac,"MainField_Item_Material_04_4185083669",
0xd7bccc7c,"MainField_Item_Material_04_4194492702",
0x179143ae,"MainField_Item_Material_04_4201429747",
0x36d31d0a,"MainField_Item_Material_04_4212492213",
0xb6a42485,"MainField_Item_Material_04_4215148504",
0x405fdbae,"MainField_Item_Material_04_4224245427",
0x419a1fd5,"MainField_Item_Material_04_4235910664",
0x2dbbd722,"MainField_Item_Material_04_4247560740",
0x794f2d05,"MainField_Item_Material_04_4252223239",
0xb6ce92c6,"MainField_Item_Material_04_4253819089",
0x18ebe858,"MainField_Item_Material_04_4254216765",
0xa79c70ff,"MainField_Item_Material_04_4257718214",
0xd35f734e,"MainField_Item_Material_04_4259348840",
0xc4f69998,"MainField_Item_Material_04_426610885",
0x340ec09b,"MainField_Item_Material_04_4271036469",
0x45513abd,"MainField_Item_Material_04_4276186565",
0x9b3e5fd8,"MainField_Item_Material_04_4289037273",
0xc93739b9,"MainField_Item_Material_04_4289067474",
0x3cc79a35,"MainField_Item_Material_04_438752625",
0x775d7a5c,"MainField_Item_Material_04_438882600",
0x1a0d162b,"MainField_Item_Material_04_441041780",
0x87bcefbb,"MainField_Item_Material_04_441229583",
0x002fc192,"MainField_Item_Material_04_444830219",
0x5f6702f4,"MainField_Item_Material_04_446171884",
0x86912012,"MainField_Item_Material_04_45496493",
0x755960a9,"MainField_Item_Material_04_458343194",
0xb0db1acc,"MainField_Item_Material_04_465368587",
0x31c0f91c,"MainField_Item_Material_04_466506168",
0xb43e6229,"MainField_Item_Material_04_469665479",
0x76320e5d,"MainField_Item_Material_04_478485940",
0x83a80d3a,"MainField_Item_Material_04_482861795",
0x6da264cb,"MainField_Item_Material_04_488844927",
0xc91befa9,"MainField_Item_Material_04_492083698",
0x428fed6a,"MainField_Item_Material_04_496833092",
0xa4711e4c,"MainField_Item_Material_04_497787505",
0xfee8b7f7,"MainField_Item_Material_04_502648565",
0xe21c3f56,"MainField_Item_Material_04_50464603",
0x67083150,"MainField_Item_Material_04_507853824",
0x59400b5f,"MainField_Item_Material_04_516102879",
0x29ba423a,"MainField_Item_Material_04_516954329",
0xe287d480,"MainField_Item_Material_04_518336639",
0x7345086c,"MainField_Item_Material_04_522305062",
0xd4203873,"MainField_Item_Material_04_528870764",
0xec12003c,"MainField_Item_Material_04_538890510",
0xee2166fa,"MainField_Item_Material_04_556892086",
0x0b782dbd,"MainField_Item_Material_04_565546966",
0x7fa066f3,"MainField_Item_Material_04_565682351",
0xed8636c7,"MainField_Item_Material_04_591024651",
0xc38bb608,"MainField_Item_Material_04_596671284",
0xfa80a64a,"MainField_Item_Material_04_605009378",
0xca74e5f8,"MainField_Item_Material_04_611564340",
0x4ba05f9a,"MainField_Item_Material_04_616330537",
0x22bdfbcd,"MainField_Item_Material_04_617997455",
0x1eb2b228,"MainField_Item_Material_04_61876323",
0x806d5bd0,"MainField_Item_Material_04_637751320",
0x8e3e3c6f,"MainField_Item_Material_04_668016947",
0x2a3f3d85,"MainField_Item_Material_04_668711898",
0x4a03eebc,"MainField_Item_Material_04_673895892",
0x0ddab038,"MainField_Item_Material_04_682320782",
0xc196392d,"MainField_Item_Material_04_687879144",
0xa26d1e92,"MainField_Item_Material_04_700110826",
0xfd20921e,"MainField_Item_Material_04_717715319",
0x53863f37,"MainField_Item_Material_04_723397337",
0x6084a15a,"MainField_Item_Material_04_745758338",
0x075b1f62,"MainField_Item_Material_04_764798911",
0x956b4b0e,"MainField_Item_Material_04_766161774",
0x20856cba,"MainField_Item_Material_04_767162825",
0x2e519060,"MainField_Item_Material_04_778958833",
0x2014005c,"MainField_Item_Material_04_791812044",
0x7ca714b8,"MainField_Item_Material_04_792766036",
0xf3262182,"MainField_Item_Material_04_803243079",
0x2661a6f8,"MainField_Item_Material_04_827689040",
0xc34ea94e,"MainField_Item_Material_04_835872433",
0xff7e22ac,"MainField_Item_Material_04_846561638",
0x46d3b9c2,"MainField_Item_Material_04_863051107",
0x5fe8767d,"MainField_Item_Material_04_879321522",
0x25bf85ca,"MainField_Item_Material_04_880617726",
0x85e34b51,"MainField_Item_Material_04_881776382",
0x0fff2c5e,"MainField_Item_Material_04_88402950",
0xe0cb3770,"MainField_Item_Material_04_886816047",
0x9bb2b1d4,"MainField_Item_Material_04_888630533",
0x83aab67f,"MainField_Item_Material_04_912007308",
0x327276f2,"MainField_Item_Material_04_91574770",
0x2598f2fe,"MainField_Item_Material_04_925769456",
0xe02aa470,"MainField_Item_Material_04_930530992",
0x2911d9a8,"MainField_Item_Material_04_9360384",
0x6c8127ac,"MainField_Item_Material_04_945034428",
0xf98a5b03,"MainField_Item_Material_04_976020791",
0x9f3b8e13,"MainField_Item_Material_04_979864672",
0x04d4e724,"MainField_Item_Material_04_990109457",
0x3df46c4a,"MainField_Item_Material_04_995446305",
0x7c44fa7b,"MainField_Item_Material_05_3530986751",
0x01f79493,"MainField_Item_Material_05_3802674378",
0xb2bbd79f,"MainField_Item_Material_05_4020751443",
0x9f8dbf79,"MainField_Item_Material_06_1393483967",
0x08bb94d2,"MainField_Item_Material_06_1798493424",
0x94f6d16f,"MainField_Item_Material_06_1848767119",
0x6afd22dd,"MainField_Item_Material_06_2307514248",
0x33999e36,"MainField_Item_Material_06_2529727570",
0xbdfeeb6f,"MainField_Item_Material_06_270489316",
0xd3ef824a,"MainField_Item_Material_06_3058983883",
0xf57bf4b6,"MainField_Item_Material_06_3175743033",
0xc9d4aa86,"MainField_Item_Material_06_3365461924",
0x61778e2f,"MainField_Item_Material_06_423019033",
0x94df8770,"MainField_Item_Material_07_1171458777",
0x706a5657,"MainField_Item_Material_07_1256391030",
0x055cdc73,"MainField_Item_Material_07_2045856764",
0x198b14fb,"MainField_Item_Material_07_3485876577",
0xbd4e88fb,"MainField_Item_Material_07_3774356038",
0x7c66557a,"MainField_Item_Material_07_4270535843",
0xc2863250,"MainField_Item_Meat_01_3017229350",
0x0d30e0fb,"MainField_Item_Meat_01_3169430315",
0x4e086d02,"MainField_Item_Meat_02_3898697910",
0xb696bfe9,"MainField_Item_Meat_06_235571515",
0x26b17d4b,"MainField_Item_Meat_06_85094595",
0xc814e8a4,"MainField_Item_Meat_06_9319902",
0xa46c13f3,"MainField_Item_Meat_07_1369092958",
0x0715005c,"MainField_Item_Meat_11_3594163773",
0x1744088c,"MainField_Item_Meat_12_1874939349",
0x6b1c9c55,"MainField_Item_MushroomGet_D_1644056108",
0xbd1af161,"MainField_Item_MushroomGet_D_2695243074",
0x602fb47e,"MainField_Item_MushroomGet_D_3793699955",
0x755ffd22,"MainField_Item_MushroomGet_D_3943668654",
0xa4f5110a,"MainField_Item_MushroomGet_D_431184570",
0xe59985ea,"MainField_Item_Mushroom_A_1020421034",
0x988b35d3,"MainField_Item_Mushroom_A_1027972316",
0x29dbbd8c,"MainField_Item_Mushroom_A_1036334235",
0x72e28e92,"MainField_Item_Mushroom_A_1054129998",
0x9be809bf,"MainField_Item_Mushroom_A_1066988571",
0xe0d3d9fa,"MainField_Item_Mushroom_A_1078660039",
0xb9e40480,"MainField_Item_Mushroom_A_1092401336",
0xa8135541,"MainField_Item_Mushroom_A_1101958357",
0x06fa8e25,"MainField_Item_Mushroom_A_1161818337",
0x1cdf844f,"MainField_Item_Mushroom_A_1165339512",
0x69199ebe,"MainField_Item_Mushroom_A_1228678829",
0x728f0c31,"MainField_Item_Mushroom_A_1258355146",
0x5ea0bf80,"MainField_Item_Mushroom_A_1304175900",
0xc96636ce,"MainField_Item_Mushroom_A_1325481086",
0xf8a9d7ae,"MainField_Item_Mushroom_A_1325499106",
0x85efb20d,"MainField_Item_Mushroom_A_133845400",
0x4b90d1ae,"MainField_Item_Mushroom_A_1357767261",
0x70890257,"MainField_Item_Mushroom_A_1407936509",
0x0465abfc,"MainField_Item_Mushroom_A_1409711513",
0x3cd81c8c,"MainField_Item_Mushroom_A_1449570325",
0x95fc821c,"MainField_Item_Mushroom_A_1500239054",
0x08a85e88,"MainField_Item_Mushroom_A_1540374623",
0xd8c6368b,"MainField_Item_Mushroom_A_1554813228",
0xc5804cc9,"MainField_Item_Mushroom_A_1557931755",
0x97099470,"MainField_Item_Mushroom_A_1571705242",
0xabdb4ff8,"MainField_Item_Mushroom_A_157237676",
0x4d6d0bfd,"MainField_Item_Mushroom_A_1597134308",
0xa29d0f28,"MainField_Item_Mushroom_A_1602055315",
0x535b5f7c,"MainField_Item_Mushroom_A_1610140659",
0xfe615c08,"MainField_Item_Mushroom_A_1610234910",
0x66858569,"MainField_Item_Mushroom_A_1664446989",
0xe225a8f9,"MainField_Item_Mushroom_A_1678307344",
0x066e9c06,"MainField_Item_Mushroom_A_1682211669",
0xe0a7ed0c,"MainField_Item_Mushroom_A_1692541790",
0x40a39b64,"MainField_Item_Mushroom_A_1728517405",
0xffafd937,"MainField_Item_Mushroom_A_1742238156",
0xecb6ca49,"MainField_Item_Mushroom_A_1752403308",
0xd2001ff3,"MainField_Item_Mushroom_A_177432901",
0x0c648e5c,"MainField_Item_Mushroom_A_1779221123",
0x732c90e6,"MainField_Item_Mushroom_A_1804357771",
0x55109b99,"MainField_Item_Mushroom_A_1813665872",
0xdd1991be,"MainField_Item_Mushroom_A_1837994135",
0x4fed4e5d,"MainField_Item_Mushroom_A_1839254720",
0x9a4c4bfe,"MainField_Item_Mushroom_A_1843756633",
0x30e71445,"MainField_Item_Mushroom_A_1853155218",
0x8bc9b597,"MainField_Item_Mushroom_A_1868825237",
0x23deb67e,"MainField_Item_Mushroom_A_1902174028",
0xbee967f1,"MainField_Item_Mushroom_A_1902959032",
0xe94907fc,"MainField_Item_Mushroom_A_1907882365",
0xf8709f0d,"MainField_Item_Mushroom_A_1929326746",
0xf204800f,"MainField_Item_Mushroom_A_1959307393",
0xe7667a20,"MainField_Item_Mushroom_A_1987012749",
0xeb99da84,"MainField_Item_Mushroom_A_1993456780",
0xf7a4fd13,"MainField_Item_Mushroom_A_1995352582",
0x091ea9e3,"MainField_Item_Mushroom_A_2014271878",
0x1e790676,"MainField_Item_Mushroom_A_2028578501",
0xaf1ccd97,"MainField_Item_Mushroom_A_205403983",
0xf0cd0a05,"MainField_Item_Mushroom_A_2055922786",
0x9fa74885,"MainField_Item_Mushroom_A_2065229901",
0x6e155aef,"MainField_Item_Mushroom_A_2115707735",
0x9424bb79,"MainField_Item_Mushroom_A_2123867996",
0x9f77376d,"MainField_Item_Mushroom_A_214207535",
0xf6f9bab6,"MainField_Item_Mushroom_A_2158877811",
0x3de8d8c9,"MainField_Item_Mushroom_A_2160460488",
0x8843632c,"MainField_Item_Mushroom_A_2162585172",
0xbaead705,"MainField_Item_Mushroom_A_2193384213",
0xe02d3522,"MainField_Item_Mushroom_A_2195647771",
0x544b288b,"MainField_Item_Mushroom_A_2207441560",
0xa3699ef0,"MainField_Item_Mushroom_A_2223827284",
0xefbe6dcd,"MainField_Item_Mushroom_A_2235105873",
0xd4a16b46,"MainField_Item_Mushroom_A_2242129314",
0x90ce9438,"MainField_Item_Mushroom_A_2243080955",
0xa72de9cf,"MainField_Item_Mushroom_A_2258259779",
0x2e55d89d,"MainField_Item_Mushroom_A_2262863818",
0xa6c68906,"MainField_Item_Mushroom_A_2277132068",
0xf3d01982,"MainField_Item_Mushroom_A_2299463115",
0xc1b4e8c2,"MainField_Item_Mushroom_A_2309443865",
0x9f5fa5a9,"MainField_Item_Mushroom_A_2315707517",
0xa1f9abe9,"MainField_Item_Mushroom_A_2318701543",
0xaf17a8f3,"MainField_Item_Mushroom_A_2335441276",
0x9fe29d6d,"MainField_Item_Mushroom_A_2350608388",
0xd4343006,"MainField_Item_Mushroom_A_2364126701",
0xe0fa5da7,"MainField_Item_Mushroom_A_2370298748",
0x5f804303,"MainField_Item_Mushroom_A_2383894773",
0x10d17206,"MainField_Item_Mushroom_A_2390497851",
0x040f98be,"MainField_Item_Mushroom_A_2395089059",
0xc267a16b,"MainField_Item_Mushroom_A_2402925431",
0x6d954d9b,"MainField_Item_Mushroom_A_2426541024",
0xcec3b43d,"MainField_Item_Mushroom_A_2427902172",
0xc36c4836,"MainField_Item_Mushroom_A_2429063606",
0xeaf38a32,"MainField_Item_Mushroom_A_2455484333",
0x446a8f33,"MainField_Item_Mushroom_A_2457949559",
0x9c18df6e,"MainField_Item_Mushroom_A_2463487196",
0xbd16a200,"MainField_Item_Mushroom_A_246834360",
0x780e8ab6,"MainField_Item_Mushroom_A_2469099075",
0x7144b5c2,"MainField_Item_Mushroom_A_2486438685",
0x1ad2d099,"MainField_Item_Mushroom_A_2490098976",
0x6a315a75,"MainField_Item_Mushroom_A_2494509067",
0x926e89bc,"MainField_Item_Mushroom_A_2498051399",
0x8abf1f10,"MainField_Item_Mushroom_A_2521065672",
0xd103d331,"MainField_Item_Mushroom_A_2527712872",
0xfa1f5812,"MainField_Item_Mushroom_A_2545890562",
0x3fc44b89,"MainField_Item_Mushroom_A_2589453670",
0x2f989251,"MainField_Item_Mushroom_A_2605038689",
0x4d6bed29,"MainField_Item_Mushroom_A_2642624720",
0xc47a3cdb,"MainField_Item_Mushroom_A_264699054",
0x3372d5db,"MainField_Item_Mushroom_A_2654640353",
0xb691b828,"MainField_Item_Mushroom_A_2670822932",
0xe0ca7f23,"MainField_Item_Mushroom_A_2686313537",
0x507baafe,"MainField_Item_Mushroom_A_2724258147",
0xdb8806d9,"MainField_Item_Mushroom_A_2737097939",
0xa7aa429f,"MainField_Item_Mushroom_A_2743712032",
0x92ecf270,"MainField_Item_Mushroom_A_2782396830",
0x951fe0f0,"MainField_Item_Mushroom_A_2789435544",
0x0f5782ae,"MainField_Item_Mushroom_A_2790288312",
0xde0f232e,"MainField_Item_Mushroom_A_2794396232",
0xf163624d,"MainField_Item_Mushroom_A_2808520840",
0x28045aae,"MainField_Item_Mushroom_A_2809435790",
0x94b654cc,"MainField_Item_Mushroom_A_2833471475",
0x3a2e0c44,"MainField_Item_Mushroom_A_2860181128",
0x3855986e,"MainField_Item_Mushroom_A_2865055090",
0x53ecd4d2,"MainField_Item_Mushroom_A_2876279396",
0xc2f99fd2,"MainField_Item_Mushroom_A_2904447164",
0x39095b9f,"MainField_Item_Mushroom_A_2935446692",
0xdb5a0467,"MainField_Item_Mushroom_A_2946929646",
0xe8364808,"MainField_Item_Mushroom_A_2958611287",
0x234a1181,"MainField_Item_Mushroom_A_2972704566",
0x7fc390af,"MainField_Item_Mushroom_A_2978888784",
0x59371b9f,"MainField_Item_Mushroom_A_2988864588",
0x78c8ee82,"MainField_Item_Mushroom_A_2991440775",
0xe7a6280e,"MainField_Item_Mushroom_A_2993246884",
0x6b160601,"MainField_Item_Mushroom_A_299601695",
0x3b518c0b,"MainField_Item_Mushroom_A_3016113472",
0xb4349497,"MainField_Item_Mushroom_A_3021953154",
0xeea33f63,"MainField_Item_Mushroom_A_3044813546",
0x78d3a827,"MainField_Item_Mushroom_A_3050530116",
0x7a9196d7,"MainField_Item_Mushroom_A_3092849721",
0x54e8d0fb,"MainField_Item_Mushroom_A_3093251352",
0xfa42321d,"MainField_Item_Mushroom_A_3097559041",
0xccbd192f,"MainField_Item_Mushroom_A_3106375211",
0xf825b5d5,"MainField_Item_Mushroom_A_3176372580",
0x14ddab91,"MainField_Item_Mushroom_A_3201691569",
0x24ac2486,"MainField_Item_Mushroom_A_3202951149",
0x20a364fa,"MainField_Item_Mushroom_A_3208192413",
0x07cc0121,"MainField_Item_Mushroom_A_3219256104",
0xe71bdfa6,"MainField_Item_Mushroom_A_3224134123",
0x1dc0236d,"MainField_Item_Mushroom_A_3227780474",
0x9f5f9d14,"MainField_Item_Mushroom_A_3235402042",
0x6a491d39,"MainField_Item_Mushroom_A_3247485878",
0x139eeedd,"MainField_Item_Mushroom_A_325344688",
0x4a3c9cd9,"MainField_Item_Mushroom_A_3287625075",
0x684adf9a,"MainField_Item_Mushroom_A_3300500577",
0x16cb80b5,"MainField_Item_Mushroom_A_3380039241",
0x49a05b0b,"MainField_Item_Mushroom_A_3381313054",
0x4542a2f7,"MainField_Item_Mushroom_A_3393317676",
0x6fbddede,"MainField_Item_Mushroom_A_3410691029",
0xa6063715,"MainField_Item_Mushroom_A_3412891234",
0x13870313,"MainField_Item_Mushroom_A_3426071133",
0xc0ee58a6,"MainField_Item_Mushroom_A_3429282029",
0x57b41daa,"MainField_Item_Mushroom_A_3446497320",
0xb34b049b,"MainField_Item_Mushroom_A_34506107",
0xbff54526,"MainField_Item_Mushroom_A_3475067654",
0xf6e74ccf,"MainField_Item_Mushroom_A_3479955792",
0x6c9b393d,"MainField_Item_Mushroom_A_3525692907",
0xadf1980a,"MainField_Item_Mushroom_A_3533762633",
0x40def9e9,"MainField_Item_Mushroom_A_3547886444",
0x1c58e2d3,"MainField_Item_Mushroom_A_35504671",
0x37f9e07c,"MainField_Item_Mushroom_A_3553950375",
0x0f214201,"MainField_Item_Mushroom_A_35645457",
0x448fd70a,"MainField_Item_Mushroom_A_3566355204",
0x92100fdc,"MainField_Item_Mushroom_A_356721431",
0xb764d40f,"MainField_Item_Mushroom_A_3570540613",
0x172f5165,"MainField_Item_Mushroom_A_3578167940",
0x0d5b292f,"MainField_Item_Mushroom_A_3606578706",
0x348db059,"MainField_Item_Mushroom_A_3606615169",
0x4180a2fe,"MainField_Item_Mushroom_A_3622102560",
0xec3ed064,"MainField_Item_Mushroom_A_3671048826",
0x2850b335,"MainField_Item_Mushroom_A_3672089535",
0xddef03e2,"MainField_Item_Mushroom_A_3673336595",
0x781c92ce,"MainField_Item_Mushroom_A_3690338867",
0x80a47ad6,"MainField_Item_Mushroom_A_3693830591",
0x9591acbb,"MainField_Item_Mushroom_A_369904364",
0x69bcfb29,"MainField_Item_Mushroom_A_3708611681",
0x1191ab1e,"MainField_Item_Mushroom_A_371982128",
0xcdf485b5,"MainField_Item_Mushroom_A_373025117",
0xd1a21563,"MainField_Item_Mushroom_A_3735347460",
0xea7b702a,"MainField_Item_Mushroom_A_3744774904",
0xd280057a,"MainField_Item_Mushroom_A_3752767966",
0x79f71f04,"MainField_Item_Mushroom_A_3759651981",
0x82a76224,"MainField_Item_Mushroom_A_3768252602",
0x6cf431f0,"MainField_Item_Mushroom_A_3783410203",
0x20002274,"MainField_Item_Mushroom_A_3807221259",
0xea85e7be,"MainField_Item_Mushroom_A_3807381693",
0xbf3fda4e,"MainField_Item_Mushroom_A_3813573098",
0x405d14b5,"MainField_Item_Mushroom_A_3835275575",
0x941a6024,"MainField_Item_Mushroom_A_3844227386",
0x75aaf8c8,"MainField_Item_Mushroom_A_3850834605",
0xfe999e4b,"MainField_Item_Mushroom_A_3883450261",
0x230aab73,"MainField_Item_Mushroom_A_3893465125",
0xabacd772,"MainField_Item_Mushroom_A_3903904229",
0xdfacab2d,"MainField_Item_Mushroom_A_3906414506",
0x52540243,"MainField_Item_Mushroom_A_3914187909",
0xc2054ba9,"MainField_Item_Mushroom_A_3937565714",
0x9c6a81c3,"MainField_Item_Mushroom_A_394307460",
0x41ae2b26,"MainField_Item_Mushroom_A_395093334",
0x6c96779c,"MainField_Item_Mushroom_A_3959076839",
0x38e36ed5,"MainField_Item_Mushroom_A_397190649",
0x07cc3772,"MainField_Item_Mushroom_A_3973397164",
0x6b97c0fa,"MainField_Item_Mushroom_A_3978457757",
0x30dd1778,"MainField_Item_Mushroom_A_3979878593",
0x11193c14,"MainField_Item_Mushroom_A_3985450566",
0x481ec21f,"MainField_Item_Mushroom_A_3988447869",
0x05f1d231,"MainField_Item_Mushroom_A_3990600885",
0x30ed4858,"MainField_Item_Mushroom_A_4013321366",
0xd251735d,"MainField_Item_Mushroom_A_4017721992",
0x574043e0,"MainField_Item_Mushroom_A_4049777178",
0xa2d48fe7,"MainField_Item_Mushroom_A_4053545050",
0xa24b0dae,"MainField_Item_Mushroom_A_4066245281",
0x0cefb33d,"MainField_Item_Mushroom_A_4082087898",
0x42f1269c,"MainField_Item_Mushroom_A_408840624",
0x96d21879,"MainField_Item_Mushroom_A_4103180754",
0x15bba289,"MainField_Item_Mushroom_A_4124047502",
0x5891b912,"MainField_Item_Mushroom_A_4136006133",
0x70e23cdb,"MainField_Item_Mushroom_A_4146135108",
0x1b5f2549,"MainField_Item_Mushroom_A_4148549564",
0xc5835a62,"MainField_Item_Mushroom_A_415200493",
0x3a768b16,"MainField_Item_Mushroom_A_4193476824",
0x1387b628,"MainField_Item_Mushroom_A_420353764",
0xaddc6b0d,"MainField_Item_Mushroom_A_4209501784",
0x1cbe6d83,"MainField_Item_Mushroom_A_4248971703",
0x8c3f063f,"MainField_Item_Mushroom_A_4266325912",
0xc704754f,"MainField_Item_Mushroom_A_4272854702",
0xf2b317d4,"MainField_Item_Mushroom_A_4275554999",
0xd83bd2a4,"MainField_Item_Mushroom_A_474366760",
0x62761fb7,"MainField_Item_Mushroom_A_504709062",
0x0fe290e1,"MainField_Item_Mushroom_A_507643037",
0x4363c8f8,"MainField_Item_Mushroom_A_509942802",
0x0641866c,"MainField_Item_Mushroom_A_525736165",
0x000f916c,"MainField_Item_Mushroom_A_526977815",
0x3b9e847c,"MainField_Item_Mushroom_A_537031236",
0x07f87d66,"MainField_Item_Mushroom_A_55122830",
0x118f33bb,"MainField_Item_Mushroom_A_552180550",
0x039bcb52,"MainField_Item_Mushroom_A_579766719",
0xd0194422,"MainField_Item_Mushroom_A_584941794",
0x2ef3ff35,"MainField_Item_Mushroom_A_594639815",
0x74b6e38f,"MainField_Item_Mushroom_A_597717535",
0x33e2d076,"MainField_Item_Mushroom_A_60569201",
0x12e81be7,"MainField_Item_Mushroom_A_633647040",
0x5495a0de,"MainField_Item_Mushroom_A_636255554",
0x805ba807,"MainField_Item_Mushroom_A_636647795",
0x4d8557ee,"MainField_Item_Mushroom_A_642130291",
0xbcd7ce27,"MainField_Item_Mushroom_A_655137588",
0x53b0310f,"MainField_Item_Mushroom_A_668019907",
0x93a8b0d5,"MainField_Item_Mushroom_A_683315042",
0x5b300938,"MainField_Item_Mushroom_A_716122346",
0x100fef27,"MainField_Item_Mushroom_A_737373049",
0x0ef2bb6b,"MainField_Item_Mushroom_A_738047681",
0x4a62a1fc,"MainField_Item_Mushroom_A_742029175",
0xf008084c,"MainField_Item_Mushroom_A_752534577",
0xe30d3959,"MainField_Item_Mushroom_A_755275239",
0x5b6d2fac,"MainField_Item_Mushroom_A_772060965",
0x8af1fb81,"MainField_Item_Mushroom_A_785877099",
0x4b4f5fa0,"MainField_Item_Mushroom_A_801261865",
0x08edbe89,"MainField_Item_Mushroom_A_809745447",
0x65c43ede,"MainField_Item_Mushroom_A_830316784",
0x7cee20da,"MainField_Item_Mushroom_A_845062975",
0xabdadcd9,"MainField_Item_Mushroom_A_863285991",
0x6ab3e93a,"MainField_Item_Mushroom_A_86616179",
0x6d2a16d0,"MainField_Item_Mushroom_A_879177976",
0xbac17e52,"MainField_Item_Mushroom_A_912291637",
0x1a518cf1,"MainField_Item_Mushroom_A_916229187",
0x03817886,"MainField_Item_Mushroom_A_929604282",
0xb9334be3,"MainField_Item_Mushroom_A_933465046",
0xf9122d91,"MainField_Item_Mushroom_A_954531421",
0x99925be8,"MainField_Item_Mushroom_A_966060714",
0x17347c8f,"MainField_Item_Mushroom_A_966153555",
0x45032b4d,"MainField_Item_Mushroom_A_985457295",
0x500e77df,"MainField_Item_Mushroom_B_1007828044",
0xd5ebb6ef,"MainField_Item_Mushroom_B_1024866417",
0x80f07795,"MainField_Item_Mushroom_B_1034093325",
0xb9a41922,"MainField_Item_Mushroom_B_1044206600",
0xb055072c,"MainField_Item_Mushroom_B_106358822",
0xa9498578,"MainField_Item_Mushroom_B_111262638",
0x2db5db9d,"MainField_Item_Mushroom_B_1139412917",
0xc171f296,"MainField_Item_Mushroom_B_1155887230",
0xedd6aeaf,"MainField_Item_Mushroom_B_1158481996",
0x0445d2fe,"MainField_Item_Mushroom_B_1169340356",
0x6402c6fc,"MainField_Item_Mushroom_B_1184180051",
0xbf8dc2b6,"MainField_Item_Mushroom_B_1216745778",
0x1a04358a,"MainField_Item_Mushroom_B_1218993868",
0x3fb87c8f,"MainField_Item_Mushroom_B_1253117076",
0x4b2473a9,"MainField_Item_Mushroom_B_1296321743",
0xae76348d,"MainField_Item_Mushroom_B_1305767111",
0xa79f275d,"MainField_Item_Mushroom_B_1307914315",
0xfb055aa7,"MainField_Item_Mushroom_B_1314115403",
0xf078e609,"MainField_Item_Mushroom_B_1351141600",
0xacb52993,"MainField_Item_Mushroom_B_1355654666",
0xdebdbe97,"MainField_Item_Mushroom_B_1362390103",
0x748eba2f,"MainField_Item_Mushroom_B_1366308800",
0xfdd64d73,"MainField_Item_Mushroom_B_1384588453",
0xfd804acb,"MainField_Item_Mushroom_B_1410865603",
0x43ee74a6,"MainField_Item_Mushroom_B_1433417092",
0x10874c82,"MainField_Item_Mushroom_B_1448882330",
0x5f319536,"MainField_Item_Mushroom_B_1471379025",
0x69f08e14,"MainField_Item_Mushroom_B_1477006309",
0x34fd1d4e,"MainField_Item_Mushroom_B_149405855",
0xfec36248,"MainField_Item_Mushroom_B_15309364",
0xa23d8ee7,"MainField_Item_Mushroom_B_1533198116",
0xa4bb22f8,"MainField_Item_Mushroom_B_1543630748",
0xd82e2ce3,"MainField_Item_Mushroom_B_1552329411",
0x8eb953d3,"MainField_Item_Mushroom_B_1556505134",
0x9dd0380a,"MainField_Item_Mushroom_B_1567502546",
0x35856299,"MainField_Item_Mushroom_B_1575785103",
0xf2366d86,"MainField_Item_Mushroom_B_1612065980",
0x3c998037,"MainField_Item_Mushroom_B_1632274213",
0xfde8e1e2,"MainField_Item_Mushroom_B_1711339206",
0xa185f2ed,"MainField_Item_Mushroom_B_1717179844",
0xc7df2b0b,"MainField_Item_Mushroom_B_1737826243",
0x4ca77c5d,"MainField_Item_Mushroom_B_1738546981",
0xa763595a,"MainField_Item_Mushroom_B_1745442605",
0xa8339a6a,"MainField_Item_Mushroom_B_1760000623",
0x0284590b,"MainField_Item_Mushroom_B_1762181898",
0x114705e9,"MainField_Item_Mushroom_B_1777054860",
0x96f527f2,"MainField_Item_Mushroom_B_1789592906",
0x95091731,"MainField_Item_Mushroom_B_1793548449",
0x5983bed4,"MainField_Item_Mushroom_B_1794328393",
0x67529e35,"MainField_Item_Mushroom_B_1794615756",
0x8661b450,"MainField_Item_Mushroom_B_183205056",
0x39509b84,"MainField_Item_Mushroom_B_1837649828",
0xd718fc54,"MainField_Item_Mushroom_B_1862045531",
0x0811c8d4,"MainField_Item_Mushroom_B_1873214277",
0xd5eedde2,"MainField_Item_Mushroom_B_1883681658",
0xf5cdd42f,"MainField_Item_Mushroom_B_188963282",
0x0a79cd04,"MainField_Item_Mushroom_B_1925462127",
0xd0db2bf0,"MainField_Item_Mushroom_B_1933623894",
0x0e8ccfad,"MainField_Item_Mushroom_B_1934124125",
0x6f93020c,"MainField_Item_Mushroom_B_1950321048",
0xf5dd27b2,"MainField_Item_Mushroom_B_1960068005",
0xdc89f0dc,"MainField_Item_Mushroom_B_196533630",
0x863e76ea,"MainField_Item_Mushroom_B_19759462",
0x090dbaa5,"MainField_Item_Mushroom_B_1987462223",
0xb1647e86,"MainField_Item_Mushroom_B_1991422129",
0x60d001d5,"MainField_Item_Mushroom_B_1992282079",
0xe6c5c1a2,"MainField_Item_Mushroom_B_2018641836",
0xe9113a97,"MainField_Item_Mushroom_B_2019743059",
0x8826ca4b,"MainField_Item_Mushroom_B_2023377314",
0x0e81d39a,"MainField_Item_Mushroom_B_2028185447",
0x9472a482,"MainField_Item_Mushroom_B_2032828252",
0x44319a79,"MainField_Item_Mushroom_B_2037937662",
0x88a659d5,"MainField_Item_Mushroom_B_203823246",
0x75809322,"MainField_Item_Mushroom_B_2077027618",
0x935737cf,"MainField_Item_Mushroom_B_2079572608",
0x5c5431c5,"MainField_Item_Mushroom_B_2085165653",
0x180f4c27,"MainField_Item_Mushroom_B_2107611577",
0x732376e8,"MainField_Item_Mushroom_B_2112025671",
0x5bc30cc3,"MainField_Item_Mushroom_B_211950844",
0x6bc9e25d,"MainField_Item_Mushroom_B_2122194972",
0x157a73ef,"MainField_Item_Mushroom_B_2127818222",
0x6d96cd35,"MainField_Item_Mushroom_B_2131456048",
0x270c0162,"MainField_Item_Mushroom_B_2142114218",
0x96f0fcf0,"MainField_Item_Mushroom_B_2156656890",
0xf1558645,"MainField_Item_Mushroom_B_2177396180",
0x3f1d6a05,"MainField_Item_Mushroom_B_2181579542",
0x42cac6be,"MainField_Item_Mushroom_B_2182639537",
0xd6f58d66,"MainField_Item_Mushroom_B_2187025777",
0xd63bdc7c,"MainField_Item_Mushroom_B_2192200921",
0xf5766809,"MainField_Item_Mushroom_B_2225538564",
0x1a672520,"MainField_Item_Mushroom_B_2228303503",
0x16917504,"MainField_Item_Mushroom_B_2231223309",
0x4ff15263,"MainField_Item_Mushroom_B_2232828719",
0x0958e065,"MainField_Item_Mushroom_B_2271576704",
0x7a458dd2,"MainField_Item_Mushroom_B_2271760930",
0xdc7771e2,"MainField_Item_Mushroom_B_2277045089",
0xf019173d,"MainField_Item_Mushroom_B_2290753470",
0xf4f97fcb,"MainField_Item_Mushroom_B_2306599588",
0x7fe76cfb,"MainField_Item_Mushroom_B_2342392924",
0x8f96c991,"MainField_Item_Mushroom_B_2343222190",
0x7e0f22f3,"MainField_Item_Mushroom_B_2346676623",
0x6eb31bc9,"MainField_Item_Mushroom_B_2372406528",
0x259fecff,"MainField_Item_Mushroom_B_2375651027",
0x937791d7,"MainField_Item_Mushroom_B_2382557672",
0xc78494ef,"MainField_Item_Mushroom_B_2405723612",
0xab0d196a,"MainField_Item_Mushroom_B_2433392486",
0xf0b3f758,"MainField_Item_Mushroom_B_2453314995",
0x2225aa8d,"MainField_Item_Mushroom_B_2455037512",
0x3b65d1e3,"MainField_Item_Mushroom_B_2457443598",
0xb5cfd360,"MainField_Item_Mushroom_B_24666733",
0x8555562e,"MainField_Item_Mushroom_B_2476958690",
0xad0ec744,"MainField_Item_Mushroom_B_2515435426",
0x38b613b6,"MainField_Item_Mushroom_B_2518791148",
0xf03fb1ef,"MainField_Item_Mushroom_B_2525124522",
0xa1ee17dc,"MainField_Item_Mushroom_B_2561704739",
0xaa332658,"MainField_Item_Mushroom_B_2611593033",
0x791d7864,"MainField_Item_Mushroom_B_2629701620",
0xd209ff1e,"MainField_Item_Mushroom_B_2647700389",
0x7370cdee,"MainField_Item_Mushroom_B_2649085887",
0x40c1b998,"MainField_Item_Mushroom_B_2653180747",
0x89ab0694,"MainField_Item_Mushroom_B_2663871291",
0x65212ae8,"MainField_Item_Mushroom_B_2680598445",
0x4d408e8a,"MainField_Item_Mushroom_B_2692600986",
0xbe659154,"MainField_Item_Mushroom_B_2700830471",
0x500f7608,"MainField_Item_Mushroom_B_2705950736",
0xc57cb6ee,"MainField_Item_Mushroom_B_2739462374",
0xca69e5fc,"MainField_Item_Mushroom_B_2789323074",
0x5eb0f7c5,"MainField_Item_Mushroom_B_2799485319",
0x836fcfa7,"MainField_Item_Mushroom_B_280006088",
0x1b3e84bf,"MainField_Item_Mushroom_B_2807901789",
0x0abd73b1,"MainField_Item_Mushroom_B_2812808046",
0xed0c2ad5,"MainField_Item_Mushroom_B_283202431",
0x2082a28b,"MainField_Item_Mushroom_B_2844796656",
0x103155ef,"MainField_Item_Mushroom_B_2859488639",
0x4d07d0b6,"MainField_Item_Mushroom_B_2870285912",
0x8a71292e,"MainField_Item_Mushroom_B_2883116978",
0x3dcc1669,"MainField_Item_Mushroom_B_2889457997",
0x950372f3,"MainField_Item_Mushroom_B_2901149152",
0xaceb1bd4,"MainField_Item_Mushroom_B_2908655764",
0xfc038319,"MainField_Item_Mushroom_B_2909841130",
0x45d6be2b,"MainField_Item_Mushroom_B_2914516577",
0x273c532f,"MainField_Item_Mushroom_B_2928818195",
0x277ad248,"MainField_Item_Mushroom_B_2984109783",
0x05584bc2,"MainField_Item_Mushroom_B_2990976470",
0xa1711dbd,"MainField_Item_Mushroom_B_2998626353",
0x07f3543a,"MainField_Item_Mushroom_B_3006574891",
0x48cd14e6,"MainField_Item_Mushroom_B_3027143025",
0x312e549b,"MainField_Item_Mushroom_B_3075216852",
0x58ced51c,"MainField_Item_Mushroom_B_3086734703",
0x3ebe2ace,"MainField_Item_Mushroom_B_3086980879",
0xc2c09965,"MainField_Item_Mushroom_B_3099181123",
0x944cb85e,"MainField_Item_Mushroom_B_3116500122",
0x090a675e,"MainField_Item_Mushroom_B_3156315918",
0xddf9107a,"MainField_Item_Mushroom_B_3161729341",
0x573e0642,"MainField_Item_Mushroom_B_31680460",
0x36f80931,"MainField_Item_Mushroom_B_3183581385",
0x477bb92a,"MainField_Item_Mushroom_B_3204373449",
0xcae56e56,"MainField_Item_Mushroom_B_3221415724",
0x5c42f8ed,"MainField_Item_Mushroom_B_32349617",
0x563829cb,"MainField_Item_Mushroom_B_3244122033",
0xef4a1fd4,"MainField_Item_Mushroom_B_3253764204",
0xad3bbd8c,"MainField_Item_Mushroom_B_3272234396",
0xc48b23be,"MainField_Item_Mushroom_B_3283225199",
0x7a91aba5,"MainField_Item_Mushroom_B_3293954285",
0x4859b791,"MainField_Item_Mushroom_B_3294474446",
0x7a856500,"MainField_Item_Mushroom_B_3297185057",
0x4bef85a5,"MainField_Item_Mushroom_B_3354010660",
0x1a6937f3,"MainField_Item_Mushroom_B_3362824380",
0xf60818b2,"MainField_Item_Mushroom_B_3367634058",
0x23436e9b,"MainField_Item_Mushroom_B_3400440637",
0x2e5cba48,"MainField_Item_Mushroom_B_3408770593",
0x4919088a,"MainField_Item_Mushroom_B_3423091205",
0x4c30da2d,"MainField_Item_Mushroom_B_3423348954",
0xb0b73326,"MainField_Item_Mushroom_B_3434067365",
0xa373de48,"MainField_Item_Mushroom_B_3439215200",
0x20c4c202,"MainField_Item_Mushroom_B_3446332164",
0xfc6cce9d,"MainField_Item_Mushroom_B_3474490348",
0xf0af10ba,"MainField_Item_Mushroom_B_3505072460",
0x4f94c252,"MainField_Item_Mushroom_B_3509007423",
0x243fd341,"MainField_Item_Mushroom_B_3514677255",
0x7794b9de,"MainField_Item_Mushroom_B_3514728359",
0xd0d311fc,"MainField_Item_Mushroom_B_3534142294",
0xa89351e5,"MainField_Item_Mushroom_B_3535734770",
0x849fe198,"MainField_Item_Mushroom_B_3553975137",
0xc5ec987e,"MainField_Item_Mushroom_B_3565075496",
0xfdcb41d2,"MainField_Item_Mushroom_B_3595166528",
0x471d53d9,"MainField_Item_Mushroom_B_3608995681",
0x44f65173,"MainField_Item_Mushroom_B_3646651590",
0x02865f44,"MainField_Item_Mushroom_B_3653527249",
0xdd9fdf6a,"MainField_Item_Mushroom_B_3658574414",
0x5278c4f5,"MainField_Item_Mushroom_B_3678876432",
0x6b386032,"MainField_Item_Mushroom_B_3680033496",
0xac4f903f,"MainField_Item_Mushroom_B_3689547808",
0x9767b096,"MainField_Item_Mushroom_B_3703637515",
0xe3eab4c8,"MainField_Item_Mushroom_B_3725995030",
0x4ad1f3c2,"MainField_Item_Mushroom_B_3759295265",
0x485e82a7,"MainField_Item_Mushroom_B_3774792988",
0x50324fa5,"MainField_Item_Mushroom_B_3781535336",
0x45257370,"MainField_Item_Mushroom_B_3814000815",
0xb3df0e97,"MainField_Item_Mushroom_B_382115556",
0x2931a30c,"MainField_Item_Mushroom_B_3821840580",
0x6a4effac,"MainField_Item_Mushroom_B_3849216520",
0xe8777b0e,"MainField_Item_Mushroom_B_3853774299",
0x2235a323,"MainField_Item_Mushroom_B_3857836824",
0xa502d00e,"MainField_Item_Mushroom_B_3863370709",
0xf585a082,"MainField_Item_Mushroom_B_3886324430",
0x382e8268,"MainField_Item_Mushroom_B_3900049366",
0x22133807,"MainField_Item_Mushroom_B_3908220030",
0x96713aec,"MainField_Item_Mushroom_B_3919209712",
0xdb56ef86,"MainField_Item_Mushroom_B_3926232583",
0xe820d90a,"MainField_Item_Mushroom_B_393197171",
0x73ce5218,"MainField_Item_Mushroom_B_394119985",
0x8f541a1e,"MainField_Item_Mushroom_B_3944002068",
0xec41b319,"MainField_Item_Mushroom_B_3963216378",
0xb0d9187f,"MainField_Item_Mushroom_B_3975185734",
0x669d2911,"MainField_Item_Mushroom_B_3988014979",
0x4d9183e1,"MainField_Item_Mushroom_B_3990052796",
0x3197d51c,"MainField_Item_Mushroom_B_4000068492",
0x9a8aa885,"MainField_Item_Mushroom_B_40069138",
0x9e8ee066,"MainField_Item_Mushroom_B_4018833707",
0x2f36829c,"MainField_Item_Mushroom_B_4063288361",
0x22704276,"MainField_Item_Mushroom_B_4069661390",
0xea4da99c,"MainField_Item_Mushroom_B_4095258772",
0xb4859c4e,"MainField_Item_Mushroom_B_4115556136",
0xe63812c9,"MainField_Item_Mushroom_B_4120923485",
0xa61c09d7,"MainField_Item_Mushroom_B_4128124933",
0xcb67d37e,"MainField_Item_Mushroom_B_4131015785",
0xe453e5b0,"MainField_Item_Mushroom_B_4131853723",
0x5d47830e,"MainField_Item_Mushroom_B_4180686649",
0x32e70fe6,"MainField_Item_Mushroom_B_4220986069",
0x9efa6f6d,"MainField_Item_Mushroom_B_4221150218",
0x8f3904e0,"MainField_Item_Mushroom_B_422193170",
0xffee8c90,"MainField_Item_Mushroom_B_4231166422",
0xfcb94fd2,"MainField_Item_Mushroom_B_4248067379",
0x9e0fd47f,"MainField_Item_Mushroom_B_4258415414",
0x4dd22df1,"MainField_Item_Mushroom_B_4263717048",
0x86ee9fc1,"MainField_Item_Mushroom_B_427202461",
0x103bc3b9,"MainField_Item_Mushroom_B_4286687885",
0x5d1fcc2a,"MainField_Item_Mushroom_B_4291297764",
0x52282f78,"MainField_Item_Mushroom_B_447895951",
0x482d8fe9,"MainField_Item_Mushroom_B_448312961",
0x6993a7e6,"MainField_Item_Mushroom_B_468793568",
0xe1fe5d97,"MainField_Item_Mushroom_B_481945637",
0xcc4870b6,"MainField_Item_Mushroom_B_487969084",
0xda7c66f5,"MainField_Item_Mushroom_B_490236769",
0xaa48130e,"MainField_Item_Mushroom_B_505481275",
0xbdf1e94b,"MainField_Item_Mushroom_B_51172497",
0x00fcffb7,"MainField_Item_Mushroom_B_524131835",
0x4696136a,"MainField_Item_Mushroom_B_530572187",
0x82d0d321,"MainField_Item_Mushroom_B_543762894",
0x4b374dc9,"MainField_Item_Mushroom_B_544099952",
0xbcc62090,"MainField_Item_Mushroom_B_556332434",
0x91b80cea,"MainField_Item_Mushroom_B_559768667",
0x92ef7121,"MainField_Item_Mushroom_B_566650816",
0x629c43ec,"MainField_Item_Mushroom_B_59411411",
0xe9438495,"MainField_Item_Mushroom_B_594768717",
0x9092ea1c,"MainField_Item_Mushroom_B_596666974",
0xdcddd708,"MainField_Item_Mushroom_B_616776641",
0x5a8187b7,"MainField_Item_Mushroom_B_631804747",
0x442ef02e,"MainField_Item_Mushroom_B_65453335",
0xc0473067,"MainField_Item_Mushroom_B_655798753",
0x63393ab9,"MainField_Item_Mushroom_B_661129086",
0x13f68336,"MainField_Item_Mushroom_B_727464516",
0x095eaf98,"MainField_Item_Mushroom_B_761128457",
0xb6f32293,"MainField_Item_Mushroom_B_762295015",
0xa1e4ee27,"MainField_Item_Mushroom_B_762737580",
0x89022a35,"MainField_Item_Mushroom_B_787150719",
0xd41d7ec3,"MainField_Item_Mushroom_B_794686304",
0x3524b017,"MainField_Item_Mushroom_B_802037983",
0x8965c00e,"MainField_Item_Mushroom_B_830580078",
0xa5c55f70,"MainField_Item_Mushroom_B_839843052",
0x300002b3,"MainField_Item_Mushroom_B_856096318",
0x74fb5d1c,"MainField_Item_Mushroom_B_859473789",
0x3f7f6954,"MainField_Item_Mushroom_B_860353846",
0x310662b5,"MainField_Item_Mushroom_B_865609634",
0xddfdd715,"MainField_Item_Mushroom_B_870819768",
0x726ad8f1,"MainField_Item_Mushroom_B_873623494",
0xd1476839,"MainField_Item_Mushroom_B_881706766",
0x92db50ba,"MainField_Item_Mushroom_B_892683463",
0xe2136ff3,"MainField_Item_Mushroom_B_929024116",
0x22ad6a2f,"MainField_Item_Mushroom_B_932024916",
0xfc30cc80,"MainField_Item_Mushroom_B_947053464",
0x874daf92,"MainField_Item_Mushroom_B_947669773",
0x6477d4b8,"MainField_Item_Mushroom_B_958725409",
0x346a5f88,"MainField_Item_Mushroom_B_965151534",
0xcf5b6feb,"MainField_Item_Mushroom_B_986272714",
0x02fef8ea,"MainField_Item_Mushroom_B_994584971",
0xfc762df5,"MainField_Item_Mushroom_C_1003535773",
0x574cc47e,"MainField_Item_Mushroom_C_1138946124",
0xa9eb0dc0,"MainField_Item_Mushroom_C_1145736982",
0x37ad4ff2,"MainField_Item_Mushroom_C_115200499",
0x978733c4,"MainField_Item_Mushroom_C_12249961",
0xfc7d3dc8,"MainField_Item_Mushroom_C_1240605817",
0xe9d07141,"MainField_Item_Mushroom_C_1255917040",
0x0beaffa5,"MainField_Item_Mushroom_C_1260425555",
0x1c464d82,"MainField_Item_Mushroom_C_1267854391",
0xaf932273,"MainField_Item_Mushroom_C_134781959",
0x52266050,"MainField_Item_Mushroom_C_1389458317",
0x1e7f0d90,"MainField_Item_Mushroom_C_1390313227",
0x80bb1910,"MainField_Item_Mushroom_C_147571177",
0x12d26b97,"MainField_Item_Mushroom_C_1478791257",
0x1066a4b6,"MainField_Item_Mushroom_C_1528533418",
0x41f85c7a,"MainField_Item_Mushroom_C_1535217600",
0x5056c5f3,"MainField_Item_Mushroom_C_1572412037",
0x2ace9e48,"MainField_Item_Mushroom_C_15749104",
0xa71de9da,"MainField_Item_Mushroom_C_1582876957",
0xc254ec51,"MainField_Item_Mushroom_C_1601961575",
0xc68eb49c,"MainField_Item_Mushroom_C_1607891641",
0x4b8e2ff3,"MainField_Item_Mushroom_C_1671429871",
0xb159b773,"MainField_Item_Mushroom_C_1680326516",
0xb30004f5,"MainField_Item_Mushroom_C_1746776613",
0x7b03f65e,"MainField_Item_Mushroom_C_1758290210",
0x52e99bdf,"MainField_Item_Mushroom_C_1778684235",
0x70b2019f,"MainField_Item_Mushroom_C_1797768176",
0x33031b05,"MainField_Item_Mushroom_C_1800120004",
0x10f0d74e,"MainField_Item_Mushroom_C_1951224926",
0x48714789,"MainField_Item_Mushroom_C_1963109707",
0xbf432f06,"MainField_Item_Mushroom_C_1971247795",
0x9e961875,"MainField_Item_Mushroom_C_1977946124",
0x73fb4560,"MainField_Item_Mushroom_C_1999827640",
0xcce96508,"MainField_Item_Mushroom_C_2105737956",
0x4838426c,"MainField_Item_Mushroom_C_2114961691",
0x286a49df,"MainField_Item_Mushroom_C_2119749486",
0x0ac1c6cd,"MainField_Item_Mushroom_C_2172534384",
0x8cb6760b,"MainField_Item_Mushroom_C_2234284747",
0x6bfc5a07,"MainField_Item_Mushroom_C_2238546799",
0x6d8f753d,"MainField_Item_Mushroom_C_2277968520",
0x71c1be1f,"MainField_Item_Mushroom_C_2289204651",
0x4afe9b4a,"MainField_Item_Mushroom_C_2289286895",
0x9256f740,"MainField_Item_Mushroom_C_2319779947",
0x66c8544d,"MainField_Item_Mushroom_C_2354863578",
0x6108ae88,"MainField_Item_Mushroom_C_2377978949",
0x368fc6b3,"MainField_Item_Mushroom_C_2511460590",
0x0a64e28b,"MainField_Item_Mushroom_C_2540945616",
0x25d66470,"MainField_Item_Mushroom_C_2545729758",
0x1e20f115,"MainField_Item_Mushroom_C_2677301597",
0x236883cc,"MainField_Item_Mushroom_C_2767912086",
0x42118ae0,"MainField_Item_Mushroom_C_2771316863",
0x08b3e334,"MainField_Item_Mushroom_C_2872550044",
0x24c6a076,"MainField_Item_Mushroom_C_2872738470",
0x6babc3c2,"MainField_Item_Mushroom_C_290450439",
0x8b10b24f,"MainField_Item_Mushroom_C_2946675478",
0x406b8340,"MainField_Item_Mushroom_C_2949435532",
0xaaa57c27,"MainField_Item_Mushroom_C_3032686129",
0x96e72a31,"MainField_Item_Mushroom_C_3062896575",
0x2cd34e59,"MainField_Item_Mushroom_C_3091589289",
0xedc44231,"MainField_Item_Mushroom_C_3118619991",
0xb22d8183,"MainField_Item_Mushroom_C_314439574",
0x5eed99cf,"MainField_Item_Mushroom_C_3179984854",
0x54b2ab45,"MainField_Item_Mushroom_C_3249134660",
0xeeeb7387,"MainField_Item_Mushroom_C_3277585144",
0x40caef5e,"MainField_Item_Mushroom_C_3321482076",
0x7f542746,"MainField_Item_Mushroom_C_3329518459",
0x44b9e402,"MainField_Item_Mushroom_C_3343397457",
0x936c3660,"MainField_Item_Mushroom_C_3370277912",
0xdf881d06,"MainField_Item_Mushroom_C_3393174851",
0x8f6f4dc1,"MainField_Item_Mushroom_C_3424224757",
0xf6d55963,"MainField_Item_Mushroom_C_3470625404",
0xb7cd1c93,"MainField_Item_Mushroom_C_3510970403",
0x7778bd25,"MainField_Item_Mushroom_C_3525951721",
0x787d1854,"MainField_Item_Mushroom_C_3547113799",
0x929cbc3d,"MainField_Item_Mushroom_C_3558140191",
0xa3d9e3d2,"MainField_Item_Mushroom_C_3558346243",
0x0df78da7,"MainField_Item_Mushroom_C_3561675593",
0x8196782e,"MainField_Item_Mushroom_C_3568335943",
0xe4bccb49,"MainField_Item_Mushroom_C_3613888329",
0x2d0d563a,"MainField_Item_Mushroom_C_3645438023",
0x8627d606,"MainField_Item_Mushroom_C_3654917616",
0x5e81709d,"MainField_Item_Mushroom_C_3740596547",
0x15134757,"MainField_Item_Mushroom_C_3785275300",
0xf6cf5f5e,"MainField_Item_Mushroom_C_3799771420",
0xa5135004,"MainField_Item_Mushroom_C_3802673436",
0x207bcfab,"MainField_Item_Mushroom_C_3818393980",
0xfd7cbeb2,"MainField_Item_Mushroom_C_3851077762",
0x26dd720f,"MainField_Item_Mushroom_C_3916127524",
0xae1f9aa4,"MainField_Item_Mushroom_C_3946413940",
0xa65d3b1e,"MainField_Item_Mushroom_C_3964159150",
0xbd465da2,"MainField_Item_Mushroom_C_4007304991",
0xdf373d2f,"MainField_Item_Mushroom_C_4028202352",
0xdd96273e,"MainField_Item_Mushroom_C_4076263266",
0x733b855b,"MainField_Item_Mushroom_C_4083062548",
0x91abedf6,"MainField_Item_Mushroom_C_4090674785",
0xa0d56ed1,"MainField_Item_Mushroom_C_4124732330",
0x84f30388,"MainField_Item_Mushroom_C_4127480240",
0x6d8737af,"MainField_Item_Mushroom_C_4151530268",
0x18a28d9a,"MainField_Item_Mushroom_C_4160942248",
0x96ed4cdd,"MainField_Item_Mushroom_C_4207025106",
0x48b294a0,"MainField_Item_Mushroom_C_457161475",
0x69862dc3,"MainField_Item_Mushroom_C_458267674",
0xdaff43ac,"MainField_Item_Mushroom_C_624122068",
0xb78060a3,"MainField_Item_Mushroom_C_631093872",
0xd0372b3c,"MainField_Item_Mushroom_C_632438343",
0x049c853c,"MainField_Item_Mushroom_C_640091614",
0xfe2d7480,"MainField_Item_Mushroom_C_726098069",
0x719ca37e,"MainField_Item_Mushroom_C_749268468",
0xd92be0c1,"MainField_Item_Mushroom_C_756154874",
0x183d21dd,"MainField_Item_Mushroom_C_770918947",
0x5345df32,"MainField_Item_Mushroom_C_793821315",
0x533208bf,"MainField_Item_Mushroom_C_800354558",
0xbf5e8cd7,"MainField_Item_Mushroom_C_860952698",
0x11b01f68,"MainField_Item_Mushroom_C_884224248",
0x8495d54c,"MainField_Item_Mushroom_C_930291300",
0x8edb8931,"MainField_Item_Mushroom_C_952653127",
0x5db9c006,"MainField_Item_Mushroom_C_96780030",
0x51d8832b,"MainField_Item_Mushroom_C_973616783",
0x08aea19f,"MainField_Item_Mushroom_D_1002132843",
0x8e621a8d,"MainField_Item_Mushroom_D_1004036283",
0x2d3cfc07,"MainField_Item_Mushroom_D_1004103129",
0x76e70a5d,"MainField_Item_Mushroom_D_1008123699",
0xd875263c,"MainField_Item_Mushroom_D_1012260959",
0xdf2d8739,"MainField_Item_Mushroom_D_1013171727",
0x9a6282ce,"MainField_Item_Mushroom_D_1018949013",
0x9c2c8850,"MainField_Item_Mushroom_D_1020033632",
0x921f5838,"MainField_Item_Mushroom_D_1022521107",
0x6f55aafa,"MainField_Item_Mushroom_D_1022716312",
0xc9339d2d,"MainField_Item_Mushroom_D_1023829945",
0xffbb52c4,"MainField_Item_Mushroom_D_1024745022",
0xb4f235ee,"MainField_Item_Mushroom_D_1025250148",
0x2e63816c,"MainField_Item_Mushroom_D_1026928398",
0x4232b1bf,"MainField_Item_Mushroom_D_102790595",
0xd34b0943,"MainField_Item_Mushroom_D_1032591719",
0x715459e1,"MainField_Item_Mushroom_D_1035043404",
0xd37a4d31,"MainField_Item_Mushroom_D_1035327930",
0xe6f4f15d,"MainField_Item_Mushroom_D_103735894",
0x9cbbd6ed,"MainField_Item_Mushroom_D_1038447507",
0x013423b3,"MainField_Item_Mushroom_D_1039450604",
0x0c0e4a27,"MainField_Item_Mushroom_D_1040632812",
0x789e603f,"MainField_Item_Mushroom_D_1042201790",
0xc66435f8,"MainField_Item_Mushroom_D_1043852589",
0x12f7d821,"MainField_Item_Mushroom_D_104860806",
0x6ca19f6c,"MainField_Item_Mushroom_D_1051657029",
0x5983e6da,"MainField_Item_Mushroom_D_1053819088",
0xbdbe9270,"MainField_Item_Mushroom_D_1053919006",
0x714f9beb,"MainField_Item_Mushroom_D_1057154775",
0x5c565b17,"MainField_Item_Mushroom_D_1057310969",
0x73519e06,"MainField_Item_Mushroom_D_1060346976",
0xecd78360,"MainField_Item_Mushroom_D_1061470102",
0xae844340,"MainField_Item_Mushroom_D_1063942315",
0xf88c3b8c,"MainField_Item_Mushroom_D_1066191015",
0x831fece4,"MainField_Item_Mushroom_D_1066776757",
0x51d980f4,"MainField_Item_Mushroom_D_1067529097",
0x8ab4332f,"MainField_Item_Mushroom_D_1071725275",
0x2903c919,"MainField_Item_Mushroom_D_1076843100",
0x9fa644ea,"MainField_Item_Mushroom_D_1078102442",
0x56934f6a,"MainField_Item_Mushroom_D_1078235514",
0x979e4bcb,"MainField_Item_Mushroom_D_1078610751",
0x1e2fe501,"MainField_Item_Mushroom_D_107906285",
0x39cfab28,"MainField_Item_Mushroom_D_1089292890",
0xeab081a5,"MainField_Item_Mushroom_D_1093940031",
0xcfec38e5,"MainField_Item_Mushroom_D_1095648169",
0xc92aeee2,"MainField_Item_Mushroom_D_110050895",
0x9f1be0c8,"MainField_Item_Mushroom_D_1101160738",
0x7e3b6fcd,"MainField_Item_Mushroom_D_1103460500",
0x9e9a80fa,"MainField_Item_Mushroom_D_110366710",
0x883bcea2,"MainField_Item_Mushroom_D_1103889021",
0xd916177b,"MainField_Item_Mushroom_D_1109712707",
0x50ed48bd,"MainField_Item_Mushroom_D_1111093354",
0xbc1b579a,"MainField_Item_Mushroom_D_1112009967",
0xacb12be0,"MainField_Item_Mushroom_D_1112427490",
0x9c31bf5c,"MainField_Item_Mushroom_D_1114484526",
0xd5bcd954,"MainField_Item_Mushroom_D_111503503",
0xe8444383,"MainField_Item_Mushroom_D_111527160",
0x2dba73fb,"MainField_Item_Mushroom_D_1118162740",
0x83fd6a92,"MainField_Item_Mushroom_D_1118887244",
0xb801e8e1,"MainField_Item_Mushroom_D_1119462704",
0x56d71fb5,"MainField_Item_Mushroom_D_1119746503",
0xff9a2b01,"MainField_Item_Mushroom_D_1119817962",
0x53feb3ae,"MainField_Item_Mushroom_D_1121180745",
0xb6390748,"MainField_Item_Mushroom_D_1126953179",
0x1c07dbbc,"MainField_Item_Mushroom_D_1128523904",
0xf88e4c5d,"MainField_Item_Mushroom_D_1128902960",
0x5352a9a1,"MainField_Item_Mushroom_D_1129221868",
0xea554692,"MainField_Item_Mushroom_D_1130088263",
0x981be624,"MainField_Item_Mushroom_D_1131026289",
0x133dd04e,"MainField_Item_Mushroom_D_1131080639",
0x62a181b3,"MainField_Item_Mushroom_D_1131942919",
0xc9059780,"MainField_Item_Mushroom_D_1132368621",
0xefd5b875,"MainField_Item_Mushroom_D_1132843195",
0x88cf7753,"MainField_Item_Mushroom_D_1133119357",
0x4a800bf5,"MainField_Item_Mushroom_D_1136199870",
0xa96f0f42,"MainField_Item_Mushroom_D_1137410414",
0xf8042f14,"MainField_Item_Mushroom_D_1138206401",
0x5cd172b1,"MainField_Item_Mushroom_D_1138489611",
0xd823d3ad,"MainField_Item_Mushroom_D_1139900566",
0x0d7810b0,"MainField_Item_Mushroom_D_1142177999",
0x0d7325c0,"MainField_Item_Mushroom_D_1143180249",
0x4df55147,"MainField_Item_Mushroom_D_114563522",
0x34a7f822,"MainField_Item_Mushroom_D_1145918019",
0xdde8b54d,"MainField_Item_Mushroom_D_1149912869",
0x9749adde,"MainField_Item_Mushroom_D_1151110822",
0x6bbb9cf8,"MainField_Item_Mushroom_D_1152171634",
0xbd47ca68,"MainField_Item_Mushroom_D_115628865",
0x55f071a2,"MainField_Item_Mushroom_D_1160917174",
0x406a6389,"MainField_Item_Mushroom_D_1161146519",
0xa3af4081,"MainField_Item_Mushroom_D_1163108400",
0x9f5c0f85,"MainField_Item_Mushroom_D_1164159813",
0x30f29fe2,"MainField_Item_Mushroom_D_1168276421",
0x5bd0f92c,"MainField_Item_Mushroom_D_1169166124",
0x78e7ead5,"MainField_Item_Mushroom_D_1169843858",
0x0ee64246,"MainField_Item_Mushroom_D_1173177542",
0xd45c9652,"MainField_Item_Mushroom_D_1175075935",
0x4c04c6f7,"MainField_Item_Mushroom_D_1177107342",
0xed9fc463,"MainField_Item_Mushroom_D_118513108",
0x7a73e492,"MainField_Item_Mushroom_D_1185161368",
0x47202fca,"MainField_Item_Mushroom_D_1193522789",
0x3073c8e3,"MainField_Item_Mushroom_D_1193613781",
0x7b370e3d,"MainField_Item_Mushroom_D_1194668460",
0x5c9cd0c0,"MainField_Item_Mushroom_D_1196073398",
0x77cec8ae,"MainField_Item_Mushroom_D_1198261709",
0x97f8a67e,"MainField_Item_Mushroom_D_1198402590",
0x554dc7d6,"MainField_Item_Mushroom_D_1201444909",
0x105d01e7,"MainField_Item_Mushroom_D_1204606850",
0xcc092669,"MainField_Item_Mushroom_D_1206412332",
0x9b532814,"MainField_Item_Mushroom_D_1207209051",
0x331456dd,"MainField_Item_Mushroom_D_1208368627",
0xb1a2521d,"MainField_Item_Mushroom_D_1212363552",
0x97aa7e66,"MainField_Item_Mushroom_D_12174343",
0xa4ebb4ef,"MainField_Item_Mushroom_D_1218479711",
0x945a992d,"MainField_Item_Mushroom_D_1220351121",
0x94e493c6,"MainField_Item_Mushroom_D_1223396134",
0x1bbb36fd,"MainField_Item_Mushroom_D_1224146694",
0xc21a5d69,"MainField_Item_Mushroom_D_1225070851",
0xec970d67,"MainField_Item_Mushroom_D_1226485279",
0x95afb234,"MainField_Item_Mushroom_D_1228077227",
0x77fc6c03,"MainField_Item_Mushroom_D_1228163379",
0xb65ec580,"MainField_Item_Mushroom_D_1232017940",
0x9eeb84ac,"MainField_Item_Mushroom_D_123306269",
0x17bbaee2,"MainField_Item_Mushroom_D_123384075",
0x159b6f01,"MainField_Item_Mushroom_D_1234524723",
0xecaa0bc6,"MainField_Item_Mushroom_D_123600838",
0xcac02d1c,"MainField_Item_Mushroom_D_1236148085",
0x21963151,"MainField_Item_Mushroom_D_1239462802",
0xd1857095,"MainField_Item_Mushroom_D_1243342994",
0x9ebb2edf,"MainField_Item_Mushroom_D_124409699",
0x70fad16a,"MainField_Item_Mushroom_D_1246324013",
0x1e3e8249,"MainField_Item_Mushroom_D_1249433823",
0xea4a07f9,"MainField_Item_Mushroom_D_125032705",
0x66089e65,"MainField_Item_Mushroom_D_1254501868",
0x0c13cf9e,"MainField_Item_Mushroom_D_1254629159",
0x97916a92,"MainField_Item_Mushroom_D_1255284945",
0xb76e2012,"MainField_Item_Mushroom_D_1255342241",
0x89168c96,"MainField_Item_Mushroom_D_1257900993",
0x2678efa1,"MainField_Item_Mushroom_D_1259164539",
0xbc3e1a9c,"MainField_Item_Mushroom_D_1260624796",
0xe83fa00e,"MainField_Item_Mushroom_D_1263754638",
0xc9619499,"MainField_Item_Mushroom_D_1270506290",
0x24123fdb,"MainField_Item_Mushroom_D_1273425523",
0xc2d9c330,"MainField_Item_Mushroom_D_1277469659",
0x5a2872fb,"MainField_Item_Mushroom_D_1279611943",
0x58d703b8,"MainField_Item_Mushroom_D_1282774887",
0x14573a58,"MainField_Item_Mushroom_D_1285045524",
0xfed0ff70,"MainField_Item_Mushroom_D_1285828229",
0xbcb51958,"MainField_Item_Mushroom_D_1286611692",
0x824f0354,"MainField_Item_Mushroom_D_1289511558",
0x49eefcbf,"MainField_Item_Mushroom_D_1290203211",
0x21e9ffc2,"MainField_Item_Mushroom_D_1292211159",
0x471dcdba,"MainField_Item_Mushroom_D_1294352430",
0x0e6acc35,"MainField_Item_Mushroom_D_1297100309",
0xbf56fbe1,"MainField_Item_Mushroom_D_1297150300",
0x9ab85db4,"MainField_Item_Mushroom_D_1300654354",
0xb0c2e646,"MainField_Item_Mushroom_D_1301831101",
0x83378b1d,"MainField_Item_Mushroom_D_1301867223",
0x1db0ae02,"MainField_Item_Mushroom_D_1305230224",
0xbcf6edd4,"MainField_Item_Mushroom_D_1310328792",
0x40cb57ae,"MainField_Item_Mushroom_D_1310399408",
0x6402d5fe,"MainField_Item_Mushroom_D_1312866051",
0xd5f10126,"MainField_Item_Mushroom_D_1313568897",
0x145a2742,"MainField_Item_Mushroom_D_1313606411",
0x18ae6f18,"MainField_Item_Mushroom_D_1313818313",
0x294009ea,"MainField_Item_Mushroom_D_1320328086",
0x3039df66,"MainField_Item_Mushroom_D_1320431140",
0x185f7066,"MainField_Item_Mushroom_D_1323927622",
0xd5381009,"MainField_Item_Mushroom_D_1324275873",
0xc2a47054,"MainField_Item_Mushroom_D_1326319868",
0xc30dea2e,"MainField_Item_Mushroom_D_1326449946",
0x77b8495c,"MainField_Item_Mushroom_D_132731406",
0x978f9c4b,"MainField_Item_Mushroom_D_1327639913",
0xd0ffb241,"MainField_Item_Mushroom_D_1328413744",
0xf45b0afe,"MainField_Item_Mushroom_D_1330050488",
0x9cfda79f,"MainField_Item_Mushroom_D_1332919907",
0x942e43c2,"MainField_Item_Mushroom_D_1332955065",
0xd0f5152f,"MainField_Item_Mushroom_D_1333367917",
0x23c3b59d,"MainField_Item_Mushroom_D_1333823940",
0xf4b3f3f3,"MainField_Item_Mushroom_D_133450450",
0x4d12592d,"MainField_Item_Mushroom_D_133581497",
0x6deb46c4,"MainField_Item_Mushroom_D_1338131652",
0x6bad20cc,"MainField_Item_Mushroom_D_1341588107",
0xb5156a2b,"MainField_Item_Mushroom_D_1345283754",
0x13e95006,"MainField_Item_Mushroom_D_1345972646",
0xa2de38fe,"MainField_Item_Mushroom_D_134839715",
0xfa157a58,"MainField_Item_Mushroom_D_1349726798",
0x27ff59a8,"MainField_Item_Mushroom_D_135025790",
0x159bcf97,"MainField_Item_Mushroom_D_1350444467",
0x2a98931a,"MainField_Item_Mushroom_D_1351272157",
0xdb2565f7,"MainField_Item_Mushroom_D_1352048578",
0x712eafcd,"MainField_Item_Mushroom_D_1352910736",
0xbbbc69e3,"MainField_Item_Mushroom_D_1353059851",
0x10c76032,"MainField_Item_Mushroom_D_1355546812",
0xd1129fb5,"MainField_Item_Mushroom_D_1356511668",
0x059092bc,"MainField_Item_Mushroom_D_1357906247",
0x60dd09d3,"MainField_Item_Mushroom_D_1361262549",
0x6f057ab6,"MainField_Item_Mushroom_D_1362026949",
0x7c2826e8,"MainField_Item_Mushroom_D_1362618005",
0x7a686110,"MainField_Item_Mushroom_D_1363648514",
0x2831b405,"MainField_Item_Mushroom_D_1364202642",
0x3f384d91,"MainField_Item_Mushroom_D_1375880080",
0x7ea7ecf0,"MainField_Item_Mushroom_D_1377905822",
0x073be481,"MainField_Item_Mushroom_D_1377914986",
0xfd624cf5,"MainField_Item_Mushroom_D_1385120181",
0x5f507a20,"MainField_Item_Mushroom_D_138529577",
0xa44ca095,"MainField_Item_Mushroom_D_1387234929",
0x79e523eb,"MainField_Item_Mushroom_D_1387502757",
0xd62b6aa1,"MainField_Item_Mushroom_D_1388916054",
0xf562fd18,"MainField_Item_Mushroom_D_1390532365",
0x44e22612,"MainField_Item_Mushroom_D_1392034809",
0xf9a60b2f,"MainField_Item_Mushroom_D_1392394104",
0xef58d7a3,"MainField_Item_Mushroom_D_1392943790",
0xc228c26d,"MainField_Item_Mushroom_D_1394756527",
0x3e256ffd,"MainField_Item_Mushroom_D_1397719380",
0xc6e52569,"MainField_Item_Mushroom_D_1398144882",
0x41276910,"MainField_Item_Mushroom_D_1400026408",
0x0b5ec160,"MainField_Item_Mushroom_D_1403772882",
0x4a221c0e,"MainField_Item_Mushroom_D_140431067",
0x0aa5fb7b,"MainField_Item_Mushroom_D_1404318652",
0x5ec08369,"MainField_Item_Mushroom_D_1405948308",
0x3748fa07,"MainField_Item_Mushroom_D_1410241302",
0x4d4c6c76,"MainField_Item_Mushroom_D_1413362600",
0xc318b113,"MainField_Item_Mushroom_D_1413973972",
0x6204d189,"MainField_Item_Mushroom_D_1414443",
0xe27653f5,"MainField_Item_Mushroom_D_1415280118",
0x32a9d6fd,"MainField_Item_Mushroom_D_1415292487",
0x6f49aff5,"MainField_Item_Mushroom_D_141593019",
0x6d73968d,"MainField_Item_Mushroom_D_1416198094",
0xf258493a,"MainField_Item_Mushroom_D_1417129161",
0x87619394,"MainField_Item_Mushroom_D_1419637032",
0x4b8f1f66,"MainField_Item_Mushroom_D_1420343754",
0x2047cf78,"MainField_Item_Mushroom_D_1420780860",
0x371c8c0b,"MainField_Item_Mushroom_D_1420940966",
0x9509a092,"MainField_Item_Mushroom_D_1422093470",
0x47650564,"MainField_Item_Mushroom_D_1424603665",
0x468efd15,"MainField_Item_Mushroom_D_1425995668",
0xd28c7c29,"MainField_Item_Mushroom_D_1426335545",
0x7e9ba45d,"MainField_Item_Mushroom_D_1428066033",
0x97e046cd,"MainField_Item_Mushroom_D_1429853119",
0x2865fa7b,"MainField_Item_Mushroom_D_1430465152",
0x9fed920c,"MainField_Item_Mushroom_D_1432494746",
0x0f218f7f,"MainField_Item_Mushroom_D_1434073432",
0xe6fd3ca2,"MainField_Item_Mushroom_D_1436247196",
0x6bec704b,"MainField_Item_Mushroom_D_1442302521",
0x22dc85bc,"MainField_Item_Mushroom_D_1449419916",
0x90ab6192,"MainField_Item_Mushroom_D_1450906767",
0xc79339ba,"MainField_Item_Mushroom_D_1452342094",
0x1ddf97b2,"MainField_Item_Mushroom_D_1454064489",
0x397f6fd9,"MainField_Item_Mushroom_D_1457577426",
0x5e485722,"MainField_Item_Mushroom_D_1457732520",
0xd49cfed2,"MainField_Item_Mushroom_D_1457983293",
0xce07a1d0,"MainField_Item_Mushroom_D_145958164",
0x4d2890c1,"MainField_Item_Mushroom_D_1463032178",
0x2cb5a34f,"MainField_Item_Mushroom_D_1463373853",
0x76990dbb,"MainField_Item_Mushroom_D_1463552376",
0xa48394c3,"MainField_Item_Mushroom_D_1465135952",
0x1d182c75,"MainField_Item_Mushroom_D_1465736902",
0xdf3abe44,"MainField_Item_Mushroom_D_1469704251",
0xae4b46a0,"MainField_Item_Mushroom_D_1469777498",
0x6efe0823,"MainField_Item_Mushroom_D_1471643053",
0xfde8de29,"MainField_Item_Mushroom_D_1471764431",
0xcba73465,"MainField_Item_Mushroom_D_1473631163",
0x42cc3542,"MainField_Item_Mushroom_D_1476877343",
0x8d7fc1d3,"MainField_Item_Mushroom_D_148053313",
0xa2c76164,"MainField_Item_Mushroom_D_1481998630",
0x0b80c093,"MainField_Item_Mushroom_D_1482352662",
0x44bd2343,"MainField_Item_Mushroom_D_1483211650",
0x38e581b6,"MainField_Item_Mushroom_D_1484129536",
0x7167eee4,"MainField_Item_Mushroom_D_1486418037",
0x82ade847,"MainField_Item_Mushroom_D_148667456",
0x3bb949e7,"MainField_Item_Mushroom_D_1489127625",
0xca93e0a4,"MainField_Item_Mushroom_D_149110097",
0xf2c83d7a,"MainField_Item_Mushroom_D_1493180992",
0x819ed995,"MainField_Item_Mushroom_D_1494765873",
0x65dfb17c,"MainField_Item_Mushroom_D_1497082679",
0xfa8dd41b,"MainField_Item_Mushroom_D_1497444073",
0xadeaa863,"MainField_Item_Mushroom_D_1500652414",
0xd9d4e329,"MainField_Item_Mushroom_D_1501894914",
0xe148b2c2,"MainField_Item_Mushroom_D_1504228657",
0x6a188a12,"MainField_Item_Mushroom_D_1504522014",
0xf20093ee,"MainField_Item_Mushroom_D_1507003686",
0xb2e878ae,"MainField_Item_Mushroom_D_1511778289",
0x46ae8823,"MainField_Item_Mushroom_D_1515029890",
0x39fa9467,"MainField_Item_Mushroom_D_1515323329",
0xf71ec5ba,"MainField_Item_Mushroom_D_1516323885",
0x0ac1953f,"MainField_Item_Mushroom_D_1516695193",
0x23a8f107,"MainField_Item_Mushroom_D_1517682915",
0x219f12aa,"MainField_Item_Mushroom_D_151816960",
0xb960a3c2,"MainField_Item_Mushroom_D_151836666",
0x0d682d8c,"MainField_Item_Mushroom_D_151842828",
0xc4bc6c0d,"MainField_Item_Mushroom_D_1518628039",
0x9807abbf,"MainField_Item_Mushroom_D_1518655593",
0x0141c022,"MainField_Item_Mushroom_D_1519637764",
0xfa571606,"MainField_Item_Mushroom_D_1521471742",
0x1a6d8c60,"MainField_Item_Mushroom_D_1522247853",
0xf7496989,"MainField_Item_Mushroom_D_1522513768",
0x8a5333bc,"MainField_Item_Mushroom_D_1525787769",
0xc37ca277,"MainField_Item_Mushroom_D_1527518957",
0x239dd3ac,"MainField_Item_Mushroom_D_1530441309",
0x95564bf5,"MainField_Item_Mushroom_D_153101043",
0xee19c160,"MainField_Item_Mushroom_D_1532318817",
0x90fe2485,"MainField_Item_Mushroom_D_1534532187",
0xc05ab7e4,"MainField_Item_Mushroom_D_1534651941",
0x61ea72f9,"MainField_Item_Mushroom_D_1537631459",
0xff4bc541,"MainField_Item_Mushroom_D_1539921908",
0xf97e4a82,"MainField_Item_Mushroom_D_1546203621",
0xd291b7cf,"MainField_Item_Mushroom_D_1546971514",
0x1322f49a,"MainField_Item_Mushroom_D_1547321118",
0x66ebc0fd,"MainField_Item_Mushroom_D_1548164845",
0x5caa3ffa,"MainField_Item_Mushroom_D_1548478259",
0x4f684d7a,"MainField_Item_Mushroom_D_1551926994",
0x90258a37,"MainField_Item_Mushroom_D_1552067131",
0x9aa97c61,"MainField_Item_Mushroom_D_1558218783",
0x180ae242,"MainField_Item_Mushroom_D_1558336017",
0xf3071616,"MainField_Item_Mushroom_D_1563115436",
0x9db97584,"MainField_Item_Mushroom_D_1564863726",
0xf39c5dc6,"MainField_Item_Mushroom_D_1565724346",
0xb3be9e0b,"MainField_Item_Mushroom_D_1566450425",
0xe7f5ed61,"MainField_Item_Mushroom_D_1568012317",
0x72ba7d4d,"MainField_Item_Mushroom_D_1569949642",
0x07024a65,"MainField_Item_Mushroom_D_1572984559",
0x35f81fe5,"MainField_Item_Mushroom_D_1573440640",
0xf70ca1e1,"MainField_Item_Mushroom_D_1578827436",
0x9f34bbce,"MainField_Item_Mushroom_D_1580560052",
0x7148132e,"MainField_Item_Mushroom_D_1589253172",
0x9ff972dd,"MainField_Item_Mushroom_D_1595978717",
0x31b92065,"MainField_Item_Mushroom_D_1596358636",
0xfc714a2e,"MainField_Item_Mushroom_D_1600553070",
0x3539a064,"MainField_Item_Mushroom_D_1602463852",
0xa07ab5ac,"MainField_Item_Mushroom_D_1603898146",
0xf27c3534,"MainField_Item_Mushroom_D_1607721898",
0xed89de41,"MainField_Item_Mushroom_D_1607921616",
0x1e47db47,"MainField_Item_Mushroom_D_1611016287",
0x00849c48,"MainField_Item_Mushroom_D_1612176192",
0x557e9f3c,"MainField_Item_Mushroom_D_1613897857",
0x78608316,"MainField_Item_Mushroom_D_1618102626",
0xb25db4bd,"MainField_Item_Mushroom_D_1618108209",
0x02910ee2,"MainField_Item_Mushroom_D_1618245029",
0x767a8580,"MainField_Item_Mushroom_D_1619271487",
0x29c2de8b,"MainField_Item_Mushroom_D_161978309",
0xe71b851d,"MainField_Item_Mushroom_D_1619811002",
0x2578719c,"MainField_Item_Mushroom_D_1620432634",
0x878f21cc,"MainField_Item_Mushroom_D_1625599642",
0x8d8c58c8,"MainField_Item_Mushroom_D_1631885658",
0xf950b011,"MainField_Item_Mushroom_D_1632837102",
0x1863ca62,"MainField_Item_Mushroom_D_1633074395",
0xcc378c3a,"MainField_Item_Mushroom_D_1639892672",
0x10fa88b3,"MainField_Item_Mushroom_D_1642126723",
0x0ef431c5,"MainField_Item_Mushroom_D_1642575116",
0x20a077c4,"MainField_Item_Mushroom_D_1643542395",
0xbda9b8a1,"MainField_Item_Mushroom_D_1645149914",
0x2cb9e046,"MainField_Item_Mushroom_D_1645698526",
0x7c339b19,"MainField_Item_Mushroom_D_1648444618",
0x2066ee0b,"MainField_Item_Mushroom_D_1652172145",
0xa184d1c4,"MainField_Item_Mushroom_D_1653063554",
0x2c0c4dd4,"MainField_Item_Mushroom_D_1655281367",
0xa97ef05b,"MainField_Item_Mushroom_D_1655802680",
0x0d5ad3e0,"MainField_Item_Mushroom_D_1656860554",
0x910f75d9,"MainField_Item_Mushroom_D_1661441410",
0x882b7daf,"MainField_Item_Mushroom_D_1662053528",
0x55873b28,"MainField_Item_Mushroom_D_1666466743",
0xad23d7c0,"MainField_Item_Mushroom_D_1669266161",
0xf78e6317,"MainField_Item_Mushroom_D_1670987978",
0x119b9d05,"MainField_Item_Mushroom_D_1672418477",
0xcf59e4d7,"MainField_Item_Mushroom_D_1672494701",
0x0d36fc70,"MainField_Item_Mushroom_D_1672501897",
0xa3eda1f1,"MainField_Item_Mushroom_D_1673201360",
0xa62d386e,"MainField_Item_Mushroom_D_1673830355",
0xfcf94ecd,"MainField_Item_Mushroom_D_1675115435",
0x58e71194,"MainField_Item_Mushroom_D_1675858622",
0x6ee3cabe,"MainField_Item_Mushroom_D_1679356372",
0x6c67e749,"MainField_Item_Mushroom_D_1684133706",
0x470ec331,"MainField_Item_Mushroom_D_1684247709",
0xa39ea6f4,"MainField_Item_Mushroom_D_1684788683",
0xd5fbe6d5,"MainField_Item_Mushroom_D_1686473815",
0x1bc3d57b,"MainField_Item_Mushroom_D_1688060529",
0xdc58dc76,"MainField_Item_Mushroom_D_1688865064",
0xf218592e,"MainField_Item_Mushroom_D_1691337986",
0x0c1e7a69,"MainField_Item_Mushroom_D_169411765",
0x79fc4237,"MainField_Item_Mushroom_D_1694418888",
0xf5223861,"MainField_Item_Mushroom_D_1699478399",
0x7c0c6e24,"MainField_Item_Mushroom_D_1700550932",
0x7876b280,"MainField_Item_Mushroom_D_1703916860",
0x32de9d9f,"MainField_Item_Mushroom_D_1705685503",
0x8f99987b,"MainField_Item_Mushroom_D_1708006589",
0x0c0e6d4e,"MainField_Item_Mushroom_D_1709228829",
0x57c7b520,"MainField_Item_Mushroom_D_1710794769",
0x4ed5a973,"MainField_Item_Mushroom_D_1711487028",
0xc0d2cfb3,"MainField_Item_Mushroom_D_1715757096",
0x7ae378ea,"MainField_Item_Mushroom_D_1720179949",
0xd29be382,"MainField_Item_Mushroom_D_1720857400",
0xcd0f3a78,"MainField_Item_Mushroom_D_1725849803",
0xc72f6471,"MainField_Item_Mushroom_D_1726145488",
0x67081b29,"MainField_Item_Mushroom_D_1727122273",
0xa7fea73a,"MainField_Item_Mushroom_D_1732114006",
0x1c2eea63,"MainField_Item_Mushroom_D_173407125",
0x028ccf06,"MainField_Item_Mushroom_D_1734132005",
0x0941f107,"MainField_Item_Mushroom_D_1737624070",
0xf4f005be,"MainField_Item_Mushroom_D_1737785142",
0xa374f9be,"MainField_Item_Mushroom_D_1743492600",
0x51bcb258,"MainField_Item_Mushroom_D_1743622836",
0x7d1bd234,"MainField_Item_Mushroom_D_1746299646",
0x4e8674da,"MainField_Item_Mushroom_D_1746664487",
0x3f52c9e8,"MainField_Item_Mushroom_D_1748222296",
0x39c32250,"MainField_Item_Mushroom_D_1755021084",
0x349214a0,"MainField_Item_Mushroom_D_1761295208",
0x9d71168f,"MainField_Item_Mushroom_D_1761853153",
0x7f8c0168,"MainField_Item_Mushroom_D_1769375100",
0x67b6e592,"MainField_Item_Mushroom_D_1773328274",
0xd7045609,"MainField_Item_Mushroom_D_1774166340",
0x5ebcb615,"MainField_Item_Mushroom_D_1775491104",
0xb3eb4530,"MainField_Item_Mushroom_D_1775989623",
0xc178eaf1,"MainField_Item_Mushroom_D_1776111067",
0xf3e79c00,"MainField_Item_Mushroom_D_1780158080",
0x461c7cf4,"MainField_Item_Mushroom_D_1780646381",
0xe1126dc9,"MainField_Item_Mushroom_D_1780801093",
0x41049c17,"MainField_Item_Mushroom_D_1782654766",
0x282da977,"MainField_Item_Mushroom_D_1782729860",
0xccb1cfa5,"MainField_Item_Mushroom_D_1786165869",
0xff1588b7,"MainField_Item_Mushroom_D_1787903316",
0x38bc6ff4,"MainField_Item_Mushroom_D_1791898142",
0xef308719,"MainField_Item_Mushroom_D_1792511307",
0x4c744067,"MainField_Item_Mushroom_D_1793409527",
0x29b94ee7,"MainField_Item_Mushroom_D_1793867590",
0x725b081f,"MainField_Item_Mushroom_D_1798014432",
0x7e313ba4,"MainField_Item_Mushroom_D_1801032536",
0x0c3130e6,"MainField_Item_Mushroom_D_1803962567",
0x6b4dc226,"MainField_Item_Mushroom_D_1806414594",
0xe52d02b8,"MainField_Item_Mushroom_D_1816185809",
0x58d4ae43,"MainField_Item_Mushroom_D_1819169563",
0x2656e4a9,"MainField_Item_Mushroom_D_1821760476",
0x60e542c4,"MainField_Item_Mushroom_D_1825809418",
0x577a3697,"MainField_Item_Mushroom_D_1828546564",
0x0d49570c,"MainField_Item_Mushroom_D_1830204737",
0xf6e42f13,"MainField_Item_Mushroom_D_1833548498",
0xaefabdfc,"MainField_Item_Mushroom_D_1834782295",
0x02cf05e6,"MainField_Item_Mushroom_D_1835628874",
0x984e48cc,"MainField_Item_Mushroom_D_1836654541",
0xda7c48c9,"MainField_Item_Mushroom_D_1836718782",
0x9d9b1eb0,"MainField_Item_Mushroom_D_1837617064",
0xd57c9fb5,"MainField_Item_Mushroom_D_1838800705",
0x122b2175,"MainField_Item_Mushroom_D_1841255601",
0x07baa5df,"MainField_Item_Mushroom_D_1841735162",
0x78baaef2,"MainField_Item_Mushroom_D_1843382161",
0x1a374b7a,"MainField_Item_Mushroom_D_1843811996",
0x5f47682e,"MainField_Item_Mushroom_D_1845541140",
0xcbacbe76,"MainField_Item_Mushroom_D_1851202524",
0x1293526c,"MainField_Item_Mushroom_D_1852143241",
0xac77f4ce,"MainField_Item_Mushroom_D_1855436562",
0xc1d79482,"MainField_Item_Mushroom_D_185954025",
0xecd80098,"MainField_Item_Mushroom_D_1861505449",
0xf526df85,"MainField_Item_Mushroom_D_186209810",
0xbd2e66ac,"MainField_Item_Mushroom_D_1863504040",
0xd1e7b4c9,"MainField_Item_Mushroom_D_1863654625",
0xb9a25f6c,"MainField_Item_Mushroom_D_186429012",
0x0546588c,"MainField_Item_Mushroom_D_1869790589",
0xfc82af63,"MainField_Item_Mushroom_D_1870153741",
0xacdc600a,"MainField_Item_Mushroom_D_18741088",
0x62b5dd92,"MainField_Item_Mushroom_D_1878228860",
0x5ada0fe9,"MainField_Item_Mushroom_D_1882868597",
0xf03d2965,"MainField_Item_Mushroom_D_1883133454",
0xb8813852,"MainField_Item_Mushroom_D_1885698490",
0x3e6674ce,"MainField_Item_Mushroom_D_1886160296",
0x21d9bad5,"MainField_Item_Mushroom_D_188698933",
0xf76c3a86,"MainField_Item_Mushroom_D_1887407118",
0x8dcd7042,"MainField_Item_Mushroom_D_1890968800",
0xfb2aa124,"MainField_Item_Mushroom_D_1893321811",
0xca1f673d,"MainField_Item_Mushroom_D_1896086039",
0x4a7b49d2,"MainField_Item_Mushroom_D_1897410980",
0x18fc0711,"MainField_Item_Mushroom_D_1899967483",
0xa4ae63e8,"MainField_Item_Mushroom_D_1907508949",
0x0770ac2e,"MainField_Item_Mushroom_D_1908074102",
0x5430a7da,"MainField_Item_Mushroom_D_1909512802",
0x4ce79f64,"MainField_Item_Mushroom_D_1910297681",
0x99e5c3e1,"MainField_Item_Mushroom_D_1912738262",
0x9d3bbb31,"MainField_Item_Mushroom_D_191586855",
0x343de333,"MainField_Item_Mushroom_D_1917557198",
0xc5acc44d,"MainField_Item_Mushroom_D_1921375536",
0x0963bcab,"MainField_Item_Mushroom_D_1923298818",
0x87b3777c,"MainField_Item_Mushroom_D_1924237997",
0x2ccd6154,"MainField_Item_Mushroom_D_1925177596",
0x4094ee55,"MainField_Item_Mushroom_D_1925648164",
0x50212bd0,"MainField_Item_Mushroom_D_1926373637",
0xe6336b72,"MainField_Item_Mushroom_D_1927012418",
0x847d2471,"MainField_Item_Mushroom_D_1927051552",
0xd53e4700,"MainField_Item_Mushroom_D_1928948260",
0x41aff915,"MainField_Item_Mushroom_D_1933889566",
0xb7c2903a,"MainField_Item_Mushroom_D_1934437215",
0x3cc282a9,"MainField_Item_Mushroom_D_1935033364",
0x8dd7be41,"MainField_Item_Mushroom_D_1936038600",
0x40e927c9,"MainField_Item_Mushroom_D_193896484",
0x7b2dbfec,"MainField_Item_Mushroom_D_1939915299",
0x4752da4e,"MainField_Item_Mushroom_D_194125093",
0x195b5411,"MainField_Item_Mushroom_D_194399519",
0x6f24cc4a,"MainField_Item_Mushroom_D_1949033384",
0x7cc05ff0,"MainField_Item_Mushroom_D_195411529",
0xd108a7d1,"MainField_Item_Mushroom_D_1957650795",
0x989e8af6,"MainField_Item_Mushroom_D_1959330144",
0x4f27b307,"MainField_Item_Mushroom_D_1962384863",
0x32420f47,"MainField_Item_Mushroom_D_1966095821",
0x6121187d,"MainField_Item_Mushroom_D_1967555669",
0xbbefb3b5,"MainField_Item_Mushroom_D_1974113974",
0xebd7d5bd,"MainField_Item_Mushroom_D_1975682010",
0x2678a465,"MainField_Item_Mushroom_D_1975784792",
0x756557f5,"MainField_Item_Mushroom_D_1976010736",
0xc4b12b13,"MainField_Item_Mushroom_D_1976296334",
0xadb501f2,"MainField_Item_Mushroom_D_1982222556",
0xd8dcaa2f,"MainField_Item_Mushroom_D_1983480658",
0x53a5a743,"MainField_Item_Mushroom_D_1989383160",
0x7a5122b7,"MainField_Item_Mushroom_D_1989394198",
0x3add6372,"MainField_Item_Mushroom_D_1992962060",
0x592d66b4,"MainField_Item_Mushroom_D_199339055",
0x636e9a06,"MainField_Item_Mushroom_D_199555382",
0x83b75775,"MainField_Item_Mushroom_D_1997082686",
0xcc5a438b,"MainField_Item_Mushroom_D_1998573516",
0xa42a1d85,"MainField_Item_Mushroom_D_1998663686",
0xb7689ea8,"MainField_Item_Mushroom_D_2001494323",
0x42031419,"MainField_Item_Mushroom_D_2003531858",
0x37b43967,"MainField_Item_Mushroom_D_200547764",
0x1c928a81,"MainField_Item_Mushroom_D_2005899850",
0x125c52bc,"MainField_Item_Mushroom_D_2009427368",
0x047de5be,"MainField_Item_Mushroom_D_201151025",
0xef35d635,"MainField_Item_Mushroom_D_2011532003",
0x5bb4aeea,"MainField_Item_Mushroom_D_201203643",
0x6a53ee92,"MainField_Item_Mushroom_D_2013916655",
0xbaf874b1,"MainField_Item_Mushroom_D_2014381369",
0x13bc4b8c,"MainField_Item_Mushroom_D_2015554428",
0x13ed3fab,"MainField_Item_Mushroom_D_2017543308",
0xdf735b6f,"MainField_Item_Mushroom_D_2018214805",
0x2fb13c63,"MainField_Item_Mushroom_D_202061926",
0x985beb87,"MainField_Item_Mushroom_D_2023041259",
0xf9affbce,"MainField_Item_Mushroom_D_2035005389",
0xc721ffec,"MainField_Item_Mushroom_D_2037101331",
0x713d166f,"MainField_Item_Mushroom_D_2038310477",
0x36994526,"MainField_Item_Mushroom_D_2039260170",
0x1971d27b,"MainField_Item_Mushroom_D_2040013041",
0x18cb73d8,"MainField_Item_Mushroom_D_2040090724",
0x2cf3e688,"MainField_Item_Mushroom_D_204279908",
0x3043eb17,"MainField_Item_Mushroom_D_2043217180",
0xbdaef6eb,"MainField_Item_Mushroom_D_2044045723",
0x4a88be63,"MainField_Item_Mushroom_D_2045439703",
0x258e2863,"MainField_Item_Mushroom_D_2045907106",
0x9d31c16c,"MainField_Item_Mushroom_D_2047149571",
0xf37701dd,"MainField_Item_Mushroom_D_2048703206",
0x923edd48,"MainField_Item_Mushroom_D_2049732673",
0x8d707807,"MainField_Item_Mushroom_D_2051807133",
0x1400a145,"MainField_Item_Mushroom_D_2052314331",
0x600e1755,"MainField_Item_Mushroom_D_2054722966",
0x7a70e415,"MainField_Item_Mushroom_D_2055869877",
0xde1fcac9,"MainField_Item_Mushroom_D_2057365263",
0x40c8959f,"MainField_Item_Mushroom_D_2058493014",
0x74e467b9,"MainField_Item_Mushroom_D_205980215",
0x06ea17aa,"MainField_Item_Mushroom_D_2062165713",
0x361efc54,"MainField_Item_Mushroom_D_206248850",
0x68b1f2ec,"MainField_Item_Mushroom_D_2065160414",
0x116b9b38,"MainField_Item_Mushroom_D_2065493172",
0x98fd0df7,"MainField_Item_Mushroom_D_2065753840",
0x97abb6be,"MainField_Item_Mushroom_D_2068409686",
0x6667bac0,"MainField_Item_Mushroom_D_2069651679",
0xa0e0b615,"MainField_Item_Mushroom_D_2069766153",
0x3a0e8f7a,"MainField_Item_Mushroom_D_2072564464",
0xdce82424,"MainField_Item_Mushroom_D_2073283569",
0x3d6886cf,"MainField_Item_Mushroom_D_2075273569",
0x980e477e,"MainField_Item_Mushroom_D_2075852102",
0xd0455662,"MainField_Item_Mushroom_D_2079428367",
0xad1e85f3,"MainField_Item_Mushroom_D_2084690016",
0xcb4be9f1,"MainField_Item_Mushroom_D_2087556236",
0xe0389838,"MainField_Item_Mushroom_D_2087873714",
0x4de7ce3e,"MainField_Item_Mushroom_D_2095423039",
0x4f5256b5,"MainField_Item_Mushroom_D_2097210525",
0xd9f0166c,"MainField_Item_Mushroom_D_2100230948",
0xc560fadc,"MainField_Item_Mushroom_D_2100710931",
0xb2ff8edd,"MainField_Item_Mushroom_D_2101614867",
0x44886bc3,"MainField_Item_Mushroom_D_2102184914",
0x98dc88c0,"MainField_Item_Mushroom_D_2105508006",
0x718cd188,"MainField_Item_Mushroom_D_2107780630",
0x8fad26a8,"MainField_Item_Mushroom_D_2115023084",
0xf822e4d7,"MainField_Item_Mushroom_D_2115641697",
0x57b6a716,"MainField_Item_Mushroom_D_2119625538",
0x1a9ccc55,"MainField_Item_Mushroom_D_2121316094",
0xe78694ac,"MainField_Item_Mushroom_D_2123599559",
0x08489ca5,"MainField_Item_Mushroom_D_212646255",
0xedc6615a,"MainField_Item_Mushroom_D_2127678050",
0xd884b242,"MainField_Item_Mushroom_D_2134537403",
0x111d042b,"MainField_Item_Mushroom_D_2135001532",
0x7e4f53eb,"MainField_Item_Mushroom_D_2135243166",
0x9643b31e,"MainField_Item_Mushroom_D_2135524731",
0x8bf57141,"MainField_Item_Mushroom_D_2137544668",
0x11cad439,"MainField_Item_Mushroom_D_2140228090",
0x913eaa1f,"MainField_Item_Mushroom_D_2140566987",
0x8acfc142,"MainField_Item_Mushroom_D_2141826523",
0xa4d9cb87,"MainField_Item_Mushroom_D_21428491",
0x39469a68,"MainField_Item_Mushroom_D_2144438353",
0x4eceac3e,"MainField_Item_Mushroom_D_2146594363",
0x45d33b3f,"MainField_Item_Mushroom_D_215032448",
0xed5c380e,"MainField_Item_Mushroom_D_2155598326",
0x0641a0fe,"MainField_Item_Mushroom_D_2159041565",
0x0325efe4,"MainField_Item_Mushroom_D_2159415247",
0x5656aba0,"MainField_Item_Mushroom_D_216177625",
0xe14c0fdc,"MainField_Item_Mushroom_D_2161849364",
0xeeb41283,"MainField_Item_Mushroom_D_2162641556",
0x4da46f00,"MainField_Item_Mushroom_D_2162901279",
0xc1b2d354,"MainField_Item_Mushroom_D_2163224096",
0x25a05def,"MainField_Item_Mushroom_D_2163557740",
0xf7f248e7,"MainField_Item_Mushroom_D_2167627601",
0x4da46473,"MainField_Item_Mushroom_D_2169400928",
0xfe142e09,"MainField_Item_Mushroom_D_217216247",
0x9f26d351,"MainField_Item_Mushroom_D_2173150745",
0xedf85d49,"MainField_Item_Mushroom_D_2174052332",
0xef5b6919,"MainField_Item_Mushroom_D_2181452690",
0xa1b49d3d,"MainField_Item_Mushroom_D_2181800246",
0xdc23cf84,"MainField_Item_Mushroom_D_2182677590",
0xab6bd21f,"MainField_Item_Mushroom_D_2185294787",
0xeab59a11,"MainField_Item_Mushroom_D_2185940108",
0xebd96454,"MainField_Item_Mushroom_D_2187104355",
0x4a6c162b,"MainField_Item_Mushroom_D_2190048922",
0x6a6bff95,"MainField_Item_Mushroom_D_2190626129",
0x194ac32b,"MainField_Item_Mushroom_D_2190727797",
0x88fac5f1,"MainField_Item_Mushroom_D_2191194580",
0xd812863c,"MainField_Item_Mushroom_D_2198933579",
0xb0a15c3e,"MainField_Item_Mushroom_D_219903017",
0x13ac7543,"MainField_Item_Mushroom_D_2201879818",
0x251618fb,"MainField_Item_Mushroom_D_2202533475",
0xe0b0aae6,"MainField_Item_Mushroom_D_2202791164",
0x6b6613ae,"MainField_Item_Mushroom_D_2203298560",
0x04b0e97a,"MainField_Item_Mushroom_D_2203715127",
0xe965c240,"MainField_Item_Mushroom_D_2205526824",
0xb6bc7fdd,"MainField_Item_Mushroom_D_2206379849",
0x4de482b3,"MainField_Item_Mushroom_D_2209088272",
0x283cf70e,"MainField_Item_Mushroom_D_2214258877",
0x28993a56,"MainField_Item_Mushroom_D_2214797545",
0x178a54d7,"MainField_Item_Mushroom_D_2216183726",
0xa0f0658c,"MainField_Item_Mushroom_D_2220938484",
0x3f14ba74,"MainField_Item_Mushroom_D_2223014105",
0xc9433ce1,"MainField_Item_Mushroom_D_2223640741",
0xfe90be80,"MainField_Item_Mushroom_D_2226132758",
0x18dcad22,"MainField_Item_Mushroom_D_2226458161",
0x875d42ae,"MainField_Item_Mushroom_D_2227682136",
0xe4e653a4,"MainField_Item_Mushroom_D_2228357167",
0x07047e68,"MainField_Item_Mushroom_D_2230604883",
0xc92f6e50,"MainField_Item_Mushroom_D_2230689787",
0x6d68d00c,"MainField_Item_Mushroom_D_2231093826",
0x31fb230d,"MainField_Item_Mushroom_D_2231358049",
0xc71cc2de,"MainField_Item_Mushroom_D_2233088955",
0xc4809ea2,"MainField_Item_Mushroom_D_2233238824",
0x619f9aa0,"MainField_Item_Mushroom_D_2235430406",
0x78ebdc7a,"MainField_Item_Mushroom_D_223666269",
0x162d82d3,"MainField_Item_Mushroom_D_2236909334",
0xc84895cc,"MainField_Item_Mushroom_D_2237039620",
0x214b644d,"MainField_Item_Mushroom_D_2238571193",
0x241c8ce4,"MainField_Item_Mushroom_D_2240821908",
0xaa6ada43,"MainField_Item_Mushroom_D_2242317241",
0x550f342a,"MainField_Item_Mushroom_D_2243840909",
0x7e66d96e,"MainField_Item_Mushroom_D_2245024673",
0xbacf9c1f,"MainField_Item_Mushroom_D_2245591104",
0xdaa7a1c6,"MainField_Item_Mushroom_D_2247461062",
0x02e5aaf9,"MainField_Item_Mushroom_D_2248915245",
0xcfebcb60,"MainField_Item_Mushroom_D_225212983",
0x04358ba2,"MainField_Item_Mushroom_D_2255059365",
0xec05ccff,"MainField_Item_Mushroom_D_2259410749",
0xba52a0ca,"MainField_Item_Mushroom_D_2260225612",
0x6147b348,"MainField_Item_Mushroom_D_2260376171",
0x3937c2bf,"MainField_Item_Mushroom_D_2260640787",
0xa4b4bea9,"MainField_Item_Mushroom_D_2261504452",
0x0cbf502f,"MainField_Item_Mushroom_D_2262442741",
0x7d0c47ef,"MainField_Item_Mushroom_D_2263653251",
0xda186cbc,"MainField_Item_Mushroom_D_2266074780",
0xe0a9edfa,"MainField_Item_Mushroom_D_2268398777",
0x9a63bf0b,"MainField_Item_Mushroom_D_2269537836",
0xdc87e700,"MainField_Item_Mushroom_D_2270946599",
0xc9e370b0,"MainField_Item_Mushroom_D_2282074496",
0x9f010b6f,"MainField_Item_Mushroom_D_2282626122",
0x24bdf220,"MainField_Item_Mushroom_D_2283488561",
0xb61662c8,"MainField_Item_Mushroom_D_2283526373",
0x9d535c10,"MainField_Item_Mushroom_D_2284475838",
0x9e47507a,"MainField_Item_Mushroom_D_228470852",
0x93c6848f,"MainField_Item_Mushroom_D_2287126403",
0x2fcb5ad3,"MainField_Item_Mushroom_D_2289169890",
0xcbce6ba0,"MainField_Item_Mushroom_D_2289353983",
0xbc2ca3d1,"MainField_Item_Mushroom_D_2292739307",
0x6659f472,"MainField_Item_Mushroom_D_2292833894",
0xdda17375,"MainField_Item_Mushroom_D_2294724239",
0xbf0dd12b,"MainField_Item_Mushroom_D_2295564530",
0x3f996b28,"MainField_Item_Mushroom_D_2296598360",
0x8619832e,"MainField_Item_Mushroom_D_2296742720",
0xc3e90ed8,"MainField_Item_Mushroom_D_2298303847",
0x95eb462d,"MainField_Item_Mushroom_D_2299218282",
0x5af51534,"MainField_Item_Mushroom_D_2300839045",
0x371be9ad,"MainField_Item_Mushroom_D_2301446278",
0x03785326,"MainField_Item_Mushroom_D_2302331558",
0xc0400041,"MainField_Item_Mushroom_D_2303107075",
0x9490a084,"MainField_Item_Mushroom_D_2303438786",
0x29d9bc59,"MainField_Item_Mushroom_D_230563875",
0x91111630,"MainField_Item_Mushroom_D_2307010512",
0x0a3fde76,"MainField_Item_Mushroom_D_230742852",
0x47f1157a,"MainField_Item_Mushroom_D_2309127340",
0x9a3a1ad1,"MainField_Item_Mushroom_D_231113055",
0x650e90f7,"MainField_Item_Mushroom_D_2314550768",
0x046200bc,"MainField_Item_Mushroom_D_2314797569",
0x5ef81310,"MainField_Item_Mushroom_D_2315454842",
0x88eea66c,"MainField_Item_Mushroom_D_2315575035",
0x931ff165,"MainField_Item_Mushroom_D_2316223113",
0xee9f900d,"MainField_Item_Mushroom_D_231865167",
0xea0cfb8e,"MainField_Item_Mushroom_D_2319321111",
0xd406532a,"MainField_Item_Mushroom_D_2320072118",
0xf9965c6e,"MainField_Item_Mushroom_D_232094728",
0x3d00df6c,"MainField_Item_Mushroom_D_2322891569",
0xdff4df13,"MainField_Item_Mushroom_D_2323004681",
0x6d1791cf,"MainField_Item_Mushroom_D_2323011855",
0x0d0c7888,"MainField_Item_Mushroom_D_2324528263",
0x1dec4bb1,"MainField_Item_Mushroom_D_2333745557",
0x01c03bc4,"MainField_Item_Mushroom_D_2336035204",
0x6fac7150,"MainField_Item_Mushroom_D_233814273",
0x6cf94422,"MainField_Item_Mushroom_D_2338880552",
0x7681ce3a,"MainField_Item_Mushroom_D_2341249251",
0x60cb56f7,"MainField_Item_Mushroom_D_234157877",
0xdfb30b85,"MainField_Item_Mushroom_D_2343194285",
0x57232f7c,"MainField_Item_Mushroom_D_2349824980",
0xa14449ef,"MainField_Item_Mushroom_D_2352364244",
0xbecfc242,"MainField_Item_Mushroom_D_2352874230",
0xa94b0869,"MainField_Item_Mushroom_D_235731667",
0x8efc22ee,"MainField_Item_Mushroom_D_2359051262",
0x471b1d73,"MainField_Item_Mushroom_D_2359065385",
0x5e3724ca,"MainField_Item_Mushroom_D_2364992854",
0xe4e80626,"MainField_Item_Mushroom_D_2369922226",
0x3470c015,"MainField_Item_Mushroom_D_2372002707",
0xba3cc439,"MainField_Item_Mushroom_D_2375705183",
0x90d18cb2,"MainField_Item_Mushroom_D_2378259988",
0x671f60fe,"MainField_Item_Mushroom_D_2379792798",
0x95064db5,"MainField_Item_Mushroom_D_2379841928",
0x9642bc1a,"MainField_Item_Mushroom_D_2383157305",
0xdd7d0d12,"MainField_Item_Mushroom_D_2384684708",
0xbe6c2d6b,"MainField_Item_Mushroom_D_2385162329",
0x1cc24f9d,"MainField_Item_Mushroom_D_2389442104",
0x75bc7b9c,"MainField_Item_Mushroom_D_2389515116",
0xbc6c588d,"MainField_Item_Mushroom_D_2390294212",
0xecd61998,"MainField_Item_Mushroom_D_2390497019",
0xec3313cc,"MainField_Item_Mushroom_D_2394529067",
0xc7a2ed32,"MainField_Item_Mushroom_D_2398873092",
0xa103c861,"MainField_Item_Mushroom_D_2399604206",
0x3c91db8e,"MainField_Item_Mushroom_D_2403028197",
0x054c09be,"MainField_Item_Mushroom_D_2403320124",
0x0f33c42f,"MainField_Item_Mushroom_D_2403771294",
0xb695052c,"MainField_Item_Mushroom_D_2405458813",
0xf9968267,"MainField_Item_Mushroom_D_2406184949",
0x15733f56,"MainField_Item_Mushroom_D_240737055",
0xd77a7b4d,"MainField_Item_Mushroom_D_2408588933",
0xf33993eb,"MainField_Item_Mushroom_D_2408729238",
0x49d7241f,"MainField_Item_Mushroom_D_2409823772",
0xcffe823e,"MainField_Item_Mushroom_D_2412224998",
0x6c9501a8,"MainField_Item_Mushroom_D_2413000448",
0xaee44cc1,"MainField_Item_Mushroom_D_2413025157",
0x88e83f8a,"MainField_Item_Mushroom_D_2413796266",
0xf0e22a1b,"MainField_Item_Mushroom_D_2417802378",
0x3bcaa2ba,"MainField_Item_Mushroom_D_2423769431",
0x47a70ece,"MainField_Item_Mushroom_D_2424234917",
0x57f889a6,"MainField_Item_Mushroom_D_2424916438",
0xef556324,"MainField_Item_Mushroom_D_2425751415",
0x95e6bafc,"MainField_Item_Mushroom_D_2426828310",
0x79e23386,"MainField_Item_Mushroom_D_2427582484",
0x9e244ebd,"MainField_Item_Mushroom_D_2427648459",
0x48efdac2,"MainField_Item_Mushroom_D_2430430353",
0x3d611c4a,"MainField_Item_Mushroom_D_2432896314",
0xa0c8558a,"MainField_Item_Mushroom_D_2433895024",
0xb7c1ef31,"MainField_Item_Mushroom_D_2435540424",
0x557dfd86,"MainField_Item_Mushroom_D_2437669867",
0x5d0f17ae,"MainField_Item_Mushroom_D_2439161625",
0x7f8778ed,"MainField_Item_Mushroom_D_2441351583",
0x1806d36a,"MainField_Item_Mushroom_D_2442931886",
0x48523b5c,"MainField_Item_Mushroom_D_2444580391",
0x5973d13d,"MainField_Item_Mushroom_D_2445258120",
0x7b28d5a8,"MainField_Item_Mushroom_D_2447845106",
0x1603c3d3,"MainField_Item_Mushroom_D_244894364",
0xe7ed1f9f,"MainField_Item_Mushroom_D_2449976362",
0x347a34e1,"MainField_Item_Mushroom_D_2450799606",
0xed632d6d,"MainField_Item_Mushroom_D_2458218098",
0x9264a629,"MainField_Item_Mushroom_D_2458473334",
0x678b7035,"MainField_Item_Mushroom_D_2460486773",
0x14cbf284,"MainField_Item_Mushroom_D_2461354329",
0xee72cd84,"MainField_Item_Mushroom_D_2466336931",
0xded9145c,"MainField_Item_Mushroom_D_2467185202",
0xdb08a399,"MainField_Item_Mushroom_D_2468246493",
0x230664cd,"MainField_Item_Mushroom_D_2470011587",
0x7e41f57b,"MainField_Item_Mushroom_D_2472598413",
0xe27142a9,"MainField_Item_Mushroom_D_2473441612",
0x2cad3d2c,"MainField_Item_Mushroom_D_2473830325",
0xbc89e98d,"MainField_Item_Mushroom_D_2473981373",
0x87eb853e,"MainField_Item_Mushroom_D_247558588",
0x200b841d,"MainField_Item_Mushroom_D_2475854861",
0xd1289ffe,"MainField_Item_Mushroom_D_2476616446",
0x6506b528,"MainField_Item_Mushroom_D_2477427534",
0x97d60ce0,"MainField_Item_Mushroom_D_2477447583",
0x807a5427,"MainField_Item_Mushroom_D_2478275046",
0x4761a8cd,"MainField_Item_Mushroom_D_2480396088",
0x1252fd25,"MainField_Item_Mushroom_D_2480398205",
0x93fd3d6d,"MainField_Item_Mushroom_D_2480735898",
0x27b131b8,"MainField_Item_Mushroom_D_2481684736",
0xec72b59c,"MainField_Item_Mushroom_D_248384592",
0x587c61b4,"MainField_Item_Mushroom_D_248420260",
0xe39cfcda,"MainField_Item_Mushroom_D_2484475065",
0xb4ce8c29,"MainField_Item_Mushroom_D_2490815089",
0x0f879bdf,"MainField_Item_Mushroom_D_2491051924",
0xb34d93d8,"MainField_Item_Mushroom_D_2491620135",
0x9fe0fc08,"MainField_Item_Mushroom_D_2493380147",
0xbb451d5c,"MainField_Item_Mushroom_D_2494980681",
0xb52fb3ab,"MainField_Item_Mushroom_D_249517900",
0x19cae8c9,"MainField_Item_Mushroom_D_2500989507",
0x7762f115,"MainField_Item_Mushroom_D_2503954363",
0x43b28f0a,"MainField_Item_Mushroom_D_2507433927",
0x3fec4acd,"MainField_Item_Mushroom_D_2509922885",
0xdf4cc388,"MainField_Item_Mushroom_D_2511773057",
0x5ba39062,"MainField_Item_Mushroom_D_251337289",
0x2032824a,"MainField_Item_Mushroom_D_2514715053",
0x7f6210ea,"MainField_Item_Mushroom_D_2515013686",
0x0b0ab687,"MainField_Item_Mushroom_D_25158514",
0x07872741,"MainField_Item_Mushroom_D_2516746813",
0x2bc86288,"MainField_Item_Mushroom_D_2518742454",
0x93c715db,"MainField_Item_Mushroom_D_2522178914",
0x8fdf5d56,"MainField_Item_Mushroom_D_2522623355",
0xcc1848d2,"MainField_Item_Mushroom_D_2531104670",
0x8c7d77e2,"MainField_Item_Mushroom_D_253457244",
0xd57624bf,"MainField_Item_Mushroom_D_253526347",
0x4cd043fb,"MainField_Item_Mushroom_D_2535492974",
0x68cf2776,"MainField_Item_Mushroom_D_2536183413",
0xa1fbd8d8,"MainField_Item_Mushroom_D_2537111070",
0xf36df2d0,"MainField_Item_Mushroom_D_2539232372",
0x2f8bb9e8,"MainField_Item_Mushroom_D_2544179966",
0x6fdd6311,"MainField_Item_Mushroom_D_254450026",
0x1827af37,"MainField_Item_Mushroom_D_2545309198",
0xf396bb9b,"MainField_Item_Mushroom_D_2545702650",
0xf65f6204,"MainField_Item_Mushroom_D_2549304159",
0x34d0152b,"MainField_Item_Mushroom_D_2549441514",
0x31b39044,"MainField_Item_Mushroom_D_254973083",
0xc5031bca,"MainField_Item_Mushroom_D_2560367958",
0x6b19eda0,"MainField_Item_Mushroom_D_2560428319",
0x90da8e62,"MainField_Item_Mushroom_D_2561229083",
0xdc65aaaa,"MainField_Item_Mushroom_D_2562194886",
0x7dab3fc3,"MainField_Item_Mushroom_D_2564189665",
0xe9a940b3,"MainField_Item_Mushroom_D_256817294",
0x723581df,"MainField_Item_Mushroom_D_2571977375",
0xc71aa3c0,"MainField_Item_Mushroom_D_2572975849",
0x90adbcd6,"MainField_Item_Mushroom_D_2574205343",
0x86b6d912,"MainField_Item_Mushroom_D_2574823810",
0x91f0b1f2,"MainField_Item_Mushroom_D_2575624759",
0x090579fb,"MainField_Item_Mushroom_D_2576955139",
0x080a5732,"MainField_Item_Mushroom_D_2580935644",
0xad2950aa,"MainField_Item_Mushroom_D_2583018839",
0xc4810316,"MainField_Item_Mushroom_D_2583802546",
0xe99929b8,"MainField_Item_Mushroom_D_2585175",
0xbedba698,"MainField_Item_Mushroom_D_2586898488",
0x7ef7715d,"MainField_Item_Mushroom_D_2587218620",
0x37b1cc2c,"MainField_Item_Mushroom_D_2589151482",
0x2f308375,"MainField_Item_Mushroom_D_2589789694",
0xf5a9543a,"MainField_Item_Mushroom_D_2590319980",
0x0bd78caf,"MainField_Item_Mushroom_D_2591009786",
0x7ededd32,"MainField_Item_Mushroom_D_2602168766",
0x33fa4672,"MainField_Item_Mushroom_D_2606222114",
0x9f64114e,"MainField_Item_Mushroom_D_2606408401",
0xaa724f89,"MainField_Item_Mushroom_D_2607383010",
0x53429210,"MainField_Item_Mushroom_D_260755034",
0x40b1530a,"MainField_Item_Mushroom_D_2608899392",
0x944524e0,"MainField_Item_Mushroom_D_2611184860",
0xc6481b4b,"MainField_Item_Mushroom_D_2612425407",
0x5a518981,"MainField_Item_Mushroom_D_2612860966",
0xde8d8f25,"MainField_Item_Mushroom_D_2613879822",
0xef346ecb,"MainField_Item_Mushroom_D_2616908005",
0x56695428,"MainField_Item_Mushroom_D_2618593379",
0x23b25dd1,"MainField_Item_Mushroom_D_2627551464",
0xe03bb88d,"MainField_Item_Mushroom_D_2627845707",
0xe560df54,"MainField_Item_Mushroom_D_2628634887",
0x7a68256e,"MainField_Item_Mushroom_D_2632159063",
0x00af0878,"MainField_Item_Mushroom_D_2634667715",
0x0f478873,"MainField_Item_Mushroom_D_2635816716",
0x9b133063,"MainField_Item_Mushroom_D_2636217329",
0x935872bb,"MainField_Item_Mushroom_D_2639695862",
0x40414c9d,"MainField_Item_Mushroom_D_2644332242",
0x08fc76de,"MainField_Item_Mushroom_D_2646130089",
0x4fcc52f2,"MainField_Item_Mushroom_D_2653053192",
0x3ebed43a,"MainField_Item_Mushroom_D_26564446",
0x08c2002d,"MainField_Item_Mushroom_D_2656761698",
0x35783f23,"MainField_Item_Mushroom_D_2661796289",
0x1dc6f5a1,"MainField_Item_Mushroom_D_2662866226",
0xa8d14ab3,"MainField_Item_Mushroom_D_2669446144",
0x32ef84be,"MainField_Item_Mushroom_D_2672884986",
0xceb45231,"MainField_Item_Mushroom_D_2674008182",
0xefc79f03,"MainField_Item_Mushroom_D_2676317158",
0x01ccfd70,"MainField_Item_Mushroom_D_267756199",
0xad7364cf,"MainField_Item_Mushroom_D_2680187932",
0x93c3c768,"MainField_Item_Mushroom_D_2680304303",
0x106f6be3,"MainField_Item_Mushroom_D_2681156907",
0x537635c7,"MainField_Item_Mushroom_D_2682975815",
0x27c4a850,"MainField_Item_Mushroom_D_2683726439",
0xf4318dd6,"MainField_Item_Mushroom_D_2683976605",
0x50da73b1,"MainField_Item_Mushroom_D_2684455863",
0xd094107d,"MainField_Item_Mushroom_D_2686134233",
0xcdb85ab3,"MainField_Item_Mushroom_D_2694685661",
0xf94ae957,"MainField_Item_Mushroom_D_2696255153",
0xa48833b2,"MainField_Item_Mushroom_D_2696536879",
0xbe163751,"MainField_Item_Mushroom_D_2698269916",
0x8248175d,"MainField_Item_Mushroom_D_2698483032",
0xf1c0f240,"MainField_Item_Mushroom_D_2700007976",
0x40b0904f,"MainField_Item_Mushroom_D_2701456287",
0x5e5eba2f,"MainField_Item_Mushroom_D_2703270664",
0xb8339b4e,"MainField_Item_Mushroom_D_2704317895",
0x5849b0e5,"MainField_Item_Mushroom_D_2705525290",
0xb48e3d54,"MainField_Item_Mushroom_D_2707311962",
0x725670ba,"MainField_Item_Mushroom_D_2707748864",
0x734eeae8,"MainField_Item_Mushroom_D_2709606190",
0xad902fbc,"MainField_Item_Mushroom_D_2712078778",
0xe3a2ecac,"MainField_Item_Mushroom_D_271468032",
0xabbae84d,"MainField_Item_Mushroom_D_2719063527",
0xca9dcf1d,"MainField_Item_Mushroom_D_2721294382",
0xe24af271,"MainField_Item_Mushroom_D_2721389508",
0x5e2d8bbb,"MainField_Item_Mushroom_D_2725330921",
0x31705e2f,"MainField_Item_Mushroom_D_2725700247",
0x2f0aa696,"MainField_Item_Mushroom_D_272678093",
0x0bb6d25d,"MainField_Item_Mushroom_D_2727198129",
0x375776e1,"MainField_Item_Mushroom_D_2731389914",
0x477d646f,"MainField_Item_Mushroom_D_2731781496",
0x7393684e,"MainField_Item_Mushroom_D_273315503",
0x6f7cc4ff,"MainField_Item_Mushroom_D_2733369740",
0x19702f96,"MainField_Item_Mushroom_D_2738623626",
0xb62a7e2b,"MainField_Item_Mushroom_D_2739229261",
0xf6337ff7,"MainField_Item_Mushroom_D_2739603706",
0xe88b740f,"MainField_Item_Mushroom_D_2740199949",
0x2dd5880a,"MainField_Item_Mushroom_D_2740473187",
0xc46fcc91,"MainField_Item_Mushroom_D_2745367521",
0x25fb950d,"MainField_Item_Mushroom_D_274564998",
0xde217d3b,"MainField_Item_Mushroom_D_2747160249",
0x0af41c58,"MainField_Item_Mushroom_D_2748128308",
0xd0e32858,"MainField_Item_Mushroom_D_2748440667",
0xc78585d0,"MainField_Item_Mushroom_D_2752047037",
0xce95b9a8,"MainField_Item_Mushroom_D_2756050106",
0x14c29b87,"MainField_Item_Mushroom_D_2756959348",
0x37014b18,"MainField_Item_Mushroom_D_275995330",
0xb0a41155,"MainField_Item_Mushroom_D_2760524345",
0x8e55f4d9,"MainField_Item_Mushroom_D_2761792907",
0x0ad78b0e,"MainField_Item_Mushroom_D_2761900342",
0x3392c2a7,"MainField_Item_Mushroom_D_2762488988",
0xdf9870a1,"MainField_Item_Mushroom_D_2763316850",
0x9125deed,"MainField_Item_Mushroom_D_2763371861",
0x3bd3bb64,"MainField_Item_Mushroom_D_2764625968",
0xffc40312,"MainField_Item_Mushroom_D_2764858402",
0xc19e0e7f,"MainField_Item_Mushroom_D_2766566431",
0xda6aaea3,"MainField_Item_Mushroom_D_2768792183",
0x1825edd4,"MainField_Item_Mushroom_D_2769649177",
0x5e2ae8f9,"MainField_Item_Mushroom_D_2770546341",
0x4597dd11,"MainField_Item_Mushroom_D_2773992824",
0x71297904,"MainField_Item_Mushroom_D_2776070748",
0x4b8f0911,"MainField_Item_Mushroom_D_2776181923",
0x6199082b,"MainField_Item_Mushroom_D_2778116593",
0xbf22797c,"MainField_Item_Mushroom_D_2779651125",
0x1bed437d,"MainField_Item_Mushroom_D_2780019793",
0x206f290d,"MainField_Item_Mushroom_D_2782335583",
0xd047ba41,"MainField_Item_Mushroom_D_27828863",
0xfd48a3d9,"MainField_Item_Mushroom_D_2789233519",
0x4f803339,"MainField_Item_Mushroom_D_2791516309",
0x49336bdf,"MainField_Item_Mushroom_D_279250508",
0x38378e14,"MainField_Item_Mushroom_D_2793245839",
0xbcfe73b8,"MainField_Item_Mushroom_D_2793568793",
0x69d8a949,"MainField_Item_Mushroom_D_2793803958",
0x224c2c63,"MainField_Item_Mushroom_D_2796726973",
0xe20a9b25,"MainField_Item_Mushroom_D_2796811828",
0xad03deb4,"MainField_Item_Mushroom_D_2799779992",
0xad2e7c84,"MainField_Item_Mushroom_D_2803301085",
0x4f2a4ea5,"MainField_Item_Mushroom_D_2805039920",
0x9c8de42a,"MainField_Item_Mushroom_D_2805909642",
0x868bf449,"MainField_Item_Mushroom_D_2808969450",
0x7a6fb3c5,"MainField_Item_Mushroom_D_2811302238",
0x934ae78d,"MainField_Item_Mushroom_D_2812506572",
0x98ce6781,"MainField_Item_Mushroom_D_281425244",
0x054f29c9,"MainField_Item_Mushroom_D_2817411755",
0x7dfa3e07,"MainField_Item_Mushroom_D_2818587719",
0xe3a5451a,"MainField_Item_Mushroom_D_282022034",
0xc7265c20,"MainField_Item_Mushroom_D_282076070",
0x891e312c,"MainField_Item_Mushroom_D_2822043491",
0x4723c5aa,"MainField_Item_Mushroom_D_2822679907",
0xa26d908f,"MainField_Item_Mushroom_D_2822785658",
0x2a08cf55,"MainField_Item_Mushroom_D_2824793613",
0x0de6d392,"MainField_Item_Mushroom_D_2827327116",
0xf989a95b,"MainField_Item_Mushroom_D_2828074872",
0xfdf4bf4e,"MainField_Item_Mushroom_D_2829854956",
0xdc678304,"MainField_Item_Mushroom_D_2834462704",
0x13559790,"MainField_Item_Mushroom_D_2835846402",
0xe620d315,"MainField_Item_Mushroom_D_2839452410",
0xf51bfc44,"MainField_Item_Mushroom_D_2840646448",
0x5b9afe4e,"MainField_Item_Mushroom_D_2842084857",
0x325c3afb,"MainField_Item_Mushroom_D_2843896960",
0x772a3688,"MainField_Item_Mushroom_D_2844640328",
0x5e9b07d3,"MainField_Item_Mushroom_D_2844905761",
0x99574cd9,"MainField_Item_Mushroom_D_2847184633",
0xc64d4f79,"MainField_Item_Mushroom_D_2847296347",
0x98bbbbc1,"MainField_Item_Mushroom_D_2847420682",
0x8fe46adc,"MainField_Item_Mushroom_D_2847437552",
0x154ebeea,"MainField_Item_Mushroom_D_2850955503",
0x8cafbc7d,"MainField_Item_Mushroom_D_2852883413",
0x4353da2f,"MainField_Item_Mushroom_D_2853134782",
0x9b333e48,"MainField_Item_Mushroom_D_285708003",
0xae6f8b9d,"MainField_Item_Mushroom_D_2859652122",
0x558b6d07,"MainField_Item_Mushroom_D_2860405038",
0x558ec22a,"MainField_Item_Mushroom_D_2861081191",
0x1a8d4cc3,"MainField_Item_Mushroom_D_2863976725",
0x9e3ec31c,"MainField_Item_Mushroom_D_286453124",
0xa7c047cf,"MainField_Item_Mushroom_D_2865489343",
0x70ae1c41,"MainField_Item_Mushroom_D_2868369953",
0x5014ead4,"MainField_Item_Mushroom_D_2870726704",
0x3ab303ec,"MainField_Item_Mushroom_D_2871791786",
0x3a2a53c2,"MainField_Item_Mushroom_D_2872858031",
0x7fc48210,"MainField_Item_Mushroom_D_2878235119",
0x5e1a4fd1,"MainField_Item_Mushroom_D_2880103978",
0xd4afd14d,"MainField_Item_Mushroom_D_28840801",
0x24b48ee7,"MainField_Item_Mushroom_D_2886915776",
0x2a8ec1d4,"MainField_Item_Mushroom_D_2887697811",
0xfdadc84c,"MainField_Item_Mushroom_D_2889640313",
0xbcc7fdb4,"MainField_Item_Mushroom_D_2889770493",
0xdeda0f99,"MainField_Item_Mushroom_D_2891982588",
0x128f5738,"MainField_Item_Mushroom_D_2894014083",
0x3fe14532,"MainField_Item_Mushroom_D_289559967",
0x655b9d0a,"MainField_Item_Mushroom_D_2895886684",
0x0e25d2c4,"MainField_Item_Mushroom_D_2896293924",
0x8175dbc7,"MainField_Item_Mushroom_D_2896841893",
0x5a8c1929,"MainField_Item_Mushroom_D_2901417784",
0x634c2a7b,"MainField_Item_Mushroom_D_2905432374",
0xd91efdd0,"MainField_Item_Mushroom_D_2908408566",
0x037f94b5,"MainField_Item_Mushroom_D_2913586237",
0x53f9e018,"MainField_Item_Mushroom_D_2915336761",
0x735ab231,"MainField_Item_Mushroom_D_2918619810",
0x3926f876,"MainField_Item_Mushroom_D_2919015644",
0xc1609ce1,"MainField_Item_Mushroom_D_2920081104",
0x229535f3,"MainField_Item_Mushroom_D_292032941",
0x39490a02,"MainField_Item_Mushroom_D_2922497602",
0x313b0c41,"MainField_Item_Mushroom_D_2923849669",
0xfbdd8145,"MainField_Item_Mushroom_D_2924773447",
0x06086593,"MainField_Item_Mushroom_D_2930956474",
0xdbfda02b,"MainField_Item_Mushroom_D_2934005956",
0x326ec1e5,"MainField_Item_Mushroom_D_2934254493",
0xcf9d518d,"MainField_Item_Mushroom_D_2934840095",
0x9c0472f2,"MainField_Item_Mushroom_D_2935073857",
0x34b673f5,"MainField_Item_Mushroom_D_2940029362",
0x00bfb337,"MainField_Item_Mushroom_D_2941906369",
0xc1b8c651,"MainField_Item_Mushroom_D_2943473421",
0x768c4473,"MainField_Item_Mushroom_D_2944041478",
0xc2d4b8cf,"MainField_Item_Mushroom_D_2946301345",
0x4b7b6fdc,"MainField_Item_Mushroom_D_2946359152",
0x2b204480,"MainField_Item_Mushroom_D_2947998746",
0x974a498f,"MainField_Item_Mushroom_D_2949757136",
0x8e1892c9,"MainField_Item_Mushroom_D_2954781399",
0xd17b413c,"MainField_Item_Mushroom_D_2956585591",
0xa77ebdd1,"MainField_Item_Mushroom_D_2956943870",
0x7f16ece3,"MainField_Item_Mushroom_D_2957884423",
0x92819a56,"MainField_Item_Mushroom_D_2959809390",
0xb9d2accc,"MainField_Item_Mushroom_D_2960922713",
0xcbfe5782,"MainField_Item_Mushroom_D_2963264150",
0xe551402d,"MainField_Item_Mushroom_D_2963498270",
0x1380f137,"MainField_Item_Mushroom_D_2966036598",
0xe80c4b2c,"MainField_Item_Mushroom_D_297005331",
0xd9effe9e,"MainField_Item_Mushroom_D_2970890471",
0x92969838,"MainField_Item_Mushroom_D_2971067869",
0x294c7c4d,"MainField_Item_Mushroom_D_29711791",
0x009f1d01,"MainField_Item_Mushroom_D_2976406798",
0xd695228c,"MainField_Item_Mushroom_D_2978185986",
0xe7b3ab44,"MainField_Item_Mushroom_D_2978354141",
0x9db6c180,"MainField_Item_Mushroom_D_2980621968",
0xcc775c71,"MainField_Item_Mushroom_D_2982341170",
0xe950c93a,"MainField_Item_Mushroom_D_29868131",
0x4b3bd192,"MainField_Item_Mushroom_D_2988208986",
0x4fc2b485,"MainField_Item_Mushroom_D_2989276726",
0x1aeaceea,"MainField_Item_Mushroom_D_2989777219",
0xaed41d88,"MainField_Item_Mushroom_D_2991298107",
0xba72273b,"MainField_Item_Mushroom_D_2991526908",
0xced0108a,"MainField_Item_Mushroom_D_2996738186",
0x82cf6dbd,"MainField_Item_Mushroom_D_2997681708",
0x947db23b,"MainField_Item_Mushroom_D_2997720365",
0x4a023cbf,"MainField_Item_Mushroom_D_2998273513",
0x40c7fba3,"MainField_Item_Mushroom_D_2998855305",
0x76d887ce,"MainField_Item_Mushroom_D_3001451705",
0xe0e52b84,"MainField_Item_Mushroom_D_3001576063",
0x4088f02b,"MainField_Item_Mushroom_D_3008593701",
0x923e4886,"MainField_Item_Mushroom_D_3009157173",
0xf643c0af,"MainField_Item_Mushroom_D_3013747724",
0x857cc213,"MainField_Item_Mushroom_D_3014064216",
0x3d43e9e0,"MainField_Item_Mushroom_D_3016582910",
0x32996239,"MainField_Item_Mushroom_D_3019856138",
0xaac33bc9,"MainField_Item_Mushroom_D_3021066123",
0x01e59b22,"MainField_Item_Mushroom_D_3023343768",
0x6a1b2f4a,"MainField_Item_Mushroom_D_3023430142",
0xa1f8f39a,"MainField_Item_Mushroom_D_302355604",
0x3c344f24,"MainField_Item_Mushroom_D_3024207045",
0xd5ac41b2,"MainField_Item_Mushroom_D_3029206786",
0xa68ecb21,"MainField_Item_Mushroom_D_3030618093",
0x8d9a590e,"MainField_Item_Mushroom_D_3036181844",
0x6ffce72e,"MainField_Item_Mushroom_D_3037232932",
0x2671bbfc,"MainField_Item_Mushroom_D_3038045810",
0x2ec6f17b,"MainField_Item_Mushroom_D_3039655039",
0x1c6fd00f,"MainField_Item_Mushroom_D_3040211014",
0xd02d59f5,"MainField_Item_Mushroom_D_3045162899",
0x22eadc5c,"MainField_Item_Mushroom_D_3047138444",
0xaee5aa6e,"MainField_Item_Mushroom_D_3047328507",
0x5ecce077,"MainField_Item_Mushroom_D_3047383716",
0xfcf712a2,"MainField_Item_Mushroom_D_3047434591",
0xbd72207b,"MainField_Item_Mushroom_D_3047771291",
0x26c28c70,"MainField_Item_Mushroom_D_3049896366",
0x9183ab85,"MainField_Item_Mushroom_D_3052529739",
0x0dbaca78,"MainField_Item_Mushroom_D_305534142",
0xf3e92c71,"MainField_Item_Mushroom_D_3057900641",
0x1708cdc7,"MainField_Item_Mushroom_D_30606499",
0xb43dd17f,"MainField_Item_Mushroom_D_3061174548",
0xd86b6508,"MainField_Item_Mushroom_D_3061741899",
0x8135841d,"MainField_Item_Mushroom_D_3062406186",
0xfd2b099f,"MainField_Item_Mushroom_D_3064494507",
0xed39844c,"MainField_Item_Mushroom_D_3066284847",
0x564ce428,"MainField_Item_Mushroom_D_3067340511",
0x22f28766,"MainField_Item_Mushroom_D_3067997622",
0x65b3e403,"MainField_Item_Mushroom_D_306831335",
0x28e010a2,"MainField_Item_Mushroom_D_3070387578",
0x9302b978,"MainField_Item_Mushroom_D_3071103099",
0xff6d1cc4,"MainField_Item_Mushroom_D_3073295447",
0x2074fa8a,"MainField_Item_Mushroom_D_3074048172",
0x3601b160,"MainField_Item_Mushroom_D_3074670553",
0x8f767914,"MainField_Item_Mushroom_D_3076565997",
0xcf4832e1,"MainField_Item_Mushroom_D_3078063661",
0x7a7672f3,"MainField_Item_Mushroom_D_3078779877",
0x1031046d,"MainField_Item_Mushroom_D_3079977988",
0xca3ec363,"MainField_Item_Mushroom_D_3081446781",
0x498b98c2,"MainField_Item_Mushroom_D_3081522712",
0x3aa8151b,"MainField_Item_Mushroom_D_308187934",
0xa487b2dd,"MainField_Item_Mushroom_D_3083752658",
0xa987e139,"MainField_Item_Mushroom_D_3084692866",
0xe682d9dd,"MainField_Item_Mushroom_D_3089363947",
0xdfdad2b3,"MainField_Item_Mushroom_D_3089988304",
0x903b6c8d,"MainField_Item_Mushroom_D_3094114297",
0xb7211033,"MainField_Item_Mushroom_D_309421720",
0x703133a7,"MainField_Item_Mushroom_D_3096481487",
0xa3447c5e,"MainField_Item_Mushroom_D_3097515211",
0xe622b5d5,"MainField_Item_Mushroom_D_3099314741",
0x3eb3cc1f,"MainField_Item_Mushroom_D_3100111734",
0xe925511e,"MainField_Item_Mushroom_D_3100112706",
0x4a95ca86,"MainField_Item_Mushroom_D_3101262398",
0x4ab33168,"MainField_Item_Mushroom_D_3101815760",
0xdc8bdf35,"MainField_Item_Mushroom_D_3102769043",
0x9bc897f4,"MainField_Item_Mushroom_D_3104257169",
0x7d39ab80,"MainField_Item_Mushroom_D_3104472787",
0xfb20fc06,"MainField_Item_Mushroom_D_3105675927",
0x3e511a85,"MainField_Item_Mushroom_D_3106884564",
0xfb706f7e,"MainField_Item_Mushroom_D_3107352213",
0xdd70455b,"MainField_Item_Mushroom_D_3109723848",
0x56c88e9f,"MainField_Item_Mushroom_D_3112948521",
0x3ae2d116,"MainField_Item_Mushroom_D_3114073543",
0xb4805042,"MainField_Item_Mushroom_D_3114267446",
0x5a1dadaf,"MainField_Item_Mushroom_D_3114374013",
0x962efe83,"MainField_Item_Mushroom_D_3119044957",
0x36bba64e,"MainField_Item_Mushroom_D_3119679234",
0xa25b7de7,"MainField_Item_Mushroom_D_3120362210",
0x62cf92fa,"MainField_Item_Mushroom_D_3121815979",
0x863b681c,"MainField_Item_Mushroom_D_3123627603",
0x910f8b4b,"MainField_Item_Mushroom_D_3130223044",
0x8b7d497c,"MainField_Item_Mushroom_D_3132238600",
0x67ce39f0,"MainField_Item_Mushroom_D_3132286350",
0x5a6aab7b,"MainField_Item_Mushroom_D_3132938462",
0x4ef9405e,"MainField_Item_Mushroom_D_3133636314",
0x6393621b,"MainField_Item_Mushroom_D_3134443671",
0x1111e7da,"MainField_Item_Mushroom_D_3135596706",
0xe4abd601,"MainField_Item_Mushroom_D_3136404929",
0x0f6058a9,"MainField_Item_Mushroom_D_3142547454",
0x3539e397,"MainField_Item_Mushroom_D_3146272322",
0x4fe7a205,"MainField_Item_Mushroom_D_3153516182",
0x19de5db9,"MainField_Item_Mushroom_D_3153654876",
0xc06af9e2,"MainField_Item_Mushroom_D_315437139",
0xd7e7d4f3,"MainField_Item_Mushroom_D_3154450119",
0x7a92fadb,"MainField_Item_Mushroom_D_3155121579",
0x5f157631,"MainField_Item_Mushroom_D_3156299418",
0xa8b7f598,"MainField_Item_Mushroom_D_3156887303",
0xc1009276,"MainField_Item_Mushroom_D_316126885",
0x0ea8d0c6,"MainField_Item_Mushroom_D_3162463000",
0xf49084e6,"MainField_Item_Mushroom_D_3162837367",
0x369efd5d,"MainField_Item_Mushroom_D_3169861253",
0x1718edff,"MainField_Item_Mushroom_D_3173951682",
0xc1e53b75,"MainField_Item_Mushroom_D_3176711411",
0xbdb46377,"MainField_Item_Mushroom_D_3177666160",
0xf60930e7,"MainField_Item_Mushroom_D_31786618",
0x3612b603,"MainField_Item_Mushroom_D_318042522",
0xa8a9fb1b,"MainField_Item_Mushroom_D_318044740",
0x0b13ca8f,"MainField_Item_Mushroom_D_3180727154",
0x20be67a9,"MainField_Item_Mushroom_D_3186840594",
0xff84627e,"MainField_Item_Mushroom_D_3190944457",
0x2f78ae76,"MainField_Item_Mushroom_D_319239370",
0x03554f92,"MainField_Item_Mushroom_D_3198046076",
0xeb8ea34d,"MainField_Item_Mushroom_D_3199026419",
0x5b68c236,"MainField_Item_Mushroom_D_3199967762",
0x5de938bf,"MainField_Item_Mushroom_D_3201516922",
0x59ab5f89,"MainField_Item_Mushroom_D_3201620120",
0x79914eaa,"MainField_Item_Mushroom_D_3203170487",
0x2e49b7f4,"MainField_Item_Mushroom_D_3204592774",
0xbae38035,"MainField_Item_Mushroom_D_3204829288",
0x231dac5b,"MainField_Item_Mushroom_D_3207255520",
0xc071a12b,"MainField_Item_Mushroom_D_320841919",
0x5f8dafd0,"MainField_Item_Mushroom_D_3209176516",
0x9111d512,"MainField_Item_Mushroom_D_3210047683",
0x5b54e6d7,"MainField_Item_Mushroom_D_3215292097",
0x49d3cf77,"MainField_Item_Mushroom_D_3216029113",
0xed0e8f2e,"MainField_Item_Mushroom_D_3217851514",
0x32807e75,"MainField_Item_Mushroom_D_321952098",
0x1bb4b4cb,"MainField_Item_Mushroom_D_3220787646",
0x216cc724,"MainField_Item_Mushroom_D_3223142091",
0x3ae7a248,"MainField_Item_Mushroom_D_3223231303",
0xa49aadac,"MainField_Item_Mushroom_D_322516954",
0x77e65efc,"MainField_Item_Mushroom_D_3225492271",
0x812cbcb4,"MainField_Item_Mushroom_D_322567531",
0xe3e6e781,"MainField_Item_Mushroom_D_322593279",
0x961ebb06,"MainField_Item_Mushroom_D_3226742156",
0x48daff83,"MainField_Item_Mushroom_D_3227399731",
0x8f23f0ae,"MainField_Item_Mushroom_D_323059304",
0x2a86f479,"MainField_Item_Mushroom_D_3234090920",
0x97525f24,"MainField_Item_Mushroom_D_3237829968",
0x2e55d278,"MainField_Item_Mushroom_D_3239577769",
0x4b0aefc0,"MainField_Item_Mushroom_D_3241172522",
0x3a05c850,"MainField_Item_Mushroom_D_3247992303",
0x3f1a2819,"MainField_Item_Mushroom_D_3249790331",
0xbea3d943,"MainField_Item_Mushroom_D_325243229",
0x77fb0fb1,"MainField_Item_Mushroom_D_3252734214",
0x89282202,"MainField_Item_Mushroom_D_325659633",
0x9caa3778,"MainField_Item_Mushroom_D_3262267943",
0xa3f0cb5a,"MainField_Item_Mushroom_D_3263216119",
0x8c89b548,"MainField_Item_Mushroom_D_3263688475",
0x28cd7da7,"MainField_Item_Mushroom_D_3267453051",
0x8212135f,"MainField_Item_Mushroom_D_3267985236",
0xc8c13adc,"MainField_Item_Mushroom_D_3270154986",
0x9198f730,"MainField_Item_Mushroom_D_3272300206",
0xf1ce6310,"MainField_Item_Mushroom_D_3272405195",
0xc87dba44,"MainField_Item_Mushroom_D_3273341860",
0xd47d32d3,"MainField_Item_Mushroom_D_3273391662",
0xbc4d81e6,"MainField_Item_Mushroom_D_3277894858",
0xa750c78a,"MainField_Item_Mushroom_D_3278052307",
0x8efe3038,"MainField_Item_Mushroom_D_3282419501",
0x6c0e5ba5,"MainField_Item_Mushroom_D_3282619977",
0x9a9b854a,"MainField_Item_Mushroom_D_3283025287",
0xa40efaba,"MainField_Item_Mushroom_D_3287150473",
0xf81d8740,"MainField_Item_Mushroom_D_3287194188",
0x1d6bc3c7,"MainField_Item_Mushroom_D_328747260",
0x6144db23,"MainField_Item_Mushroom_D_3287949466",
0x3ab8ff7b,"MainField_Item_Mushroom_D_3291064057",
0x050fcea5,"MainField_Item_Mushroom_D_3291388227",
0x4eb5005d,"MainField_Item_Mushroom_D_3291840642",
0x9f3845dc,"MainField_Item_Mushroom_D_329346660",
0x8a6f89b1,"MainField_Item_Mushroom_D_3294003041",
0x6fca4178,"MainField_Item_Mushroom_D_3294032408",
0x9920cd74,"MainField_Item_Mushroom_D_3294615184",
0xd243f2cf,"MainField_Item_Mushroom_D_3295640426",
0x5a41bfc5,"MainField_Item_Mushroom_D_3297496787",
0x53948ad8,"MainField_Item_Mushroom_D_3299506680",
0x6ce1151a,"MainField_Item_Mushroom_D_3304358848",
0x63126ae5,"MainField_Item_Mushroom_D_3306394205",
0xfc7b8128,"MainField_Item_Mushroom_D_331100787",
0x4ec19ac3,"MainField_Item_Mushroom_D_3314568945",
0x985ee3d8,"MainField_Item_Mushroom_D_3316082126",
0x4c732dd8,"MainField_Item_Mushroom_D_332006820",
0xb56aa8c0,"MainField_Item_Mushroom_D_3322464542",
0x2f28ed8d,"MainField_Item_Mushroom_D_3322975886",
0x9a2961c5,"MainField_Item_Mushroom_D_3324015064",
0xedba3df8,"MainField_Item_Mushroom_D_3328726991",
0xf139f42b,"MainField_Item_Mushroom_D_332879017",
0x61aeef5e,"MainField_Item_Mushroom_D_3329974698",
0x4a078384,"MainField_Item_Mushroom_D_3330357640",
0x1193a680,"MainField_Item_Mushroom_D_3331824968",
0xed8c88fc,"MainField_Item_Mushroom_D_3331885168",
0x2f2c944f,"MainField_Item_Mushroom_D_3334299349",
0x75101ed1,"MainField_Item_Mushroom_D_3334986770",
0xc2b71600,"MainField_Item_Mushroom_D_3337833725",
0x5f945710,"MainField_Item_Mushroom_D_3337960684",
0x7da8abb0,"MainField_Item_Mushroom_D_3338090362",
0x0bdcc932,"MainField_Item_Mushroom_D_3338749387",
0x0eb7bb2f,"MainField_Item_Mushroom_D_3340379137",
0x0c7eef91,"MainField_Item_Mushroom_D_3343371278",
0xbd485b28,"MainField_Item_Mushroom_D_3346811525",
0x783d0331,"MainField_Item_Mushroom_D_3347671556",
0x012e649d,"MainField_Item_Mushroom_D_3348174179",
0x55d0020e,"MainField_Item_Mushroom_D_3349241928",
0xba9c3673,"MainField_Item_Mushroom_D_3350783766",
0x9f387621,"MainField_Item_Mushroom_D_3351441998",
0xb79536aa,"MainField_Item_Mushroom_D_3354493912",
0x2b11f13a,"MainField_Item_Mushroom_D_3357948232",
0x9b29f744,"MainField_Item_Mushroom_D_3358548012",
0x15124271,"MainField_Item_Mushroom_D_3358830033",
0x98410ac0,"MainField_Item_Mushroom_D_336490623",
0xee8129ba,"MainField_Item_Mushroom_D_3366031532",
0xcab74c2d,"MainField_Item_Mushroom_D_3370653666",
0xdc362f61,"MainField_Item_Mushroom_D_3370711575",
0xee0a966b,"MainField_Item_Mushroom_D_3371593054",
0x42ed861d,"MainField_Item_Mushroom_D_3372941036",
0xb5625be4,"MainField_Item_Mushroom_D_337524810",
0x17d0e2fe,"MainField_Item_Mushroom_D_3379450423",
0x3d33a185,"MainField_Item_Mushroom_D_3382733478",
0x8b079c26,"MainField_Item_Mushroom_D_3385239051",
0x8965bfd9,"MainField_Item_Mushroom_D_3386107918",
0xb89251e8,"MainField_Item_Mushroom_D_3391237902",
0xf3dbca5c,"MainField_Item_Mushroom_D_3392387165",
0x9aeaabed,"MainField_Item_Mushroom_D_3393734345",
0x89ad83ce,"MainField_Item_Mushroom_D_3394997676",
0xe51efeee,"MainField_Item_Mushroom_D_3398190750",
0x8cda2096,"MainField_Item_Mushroom_D_339865742",
0x7978c932,"MainField_Item_Mushroom_D_3399790452",
0x03de56d3,"MainField_Item_Mushroom_D_3399906143",
0x6bce071c,"MainField_Item_Mushroom_D_3403214862",
0xcd3b5cb5,"MainField_Item_Mushroom_D_340836546",
0x6b6cb57a,"MainField_Item_Mushroom_D_3410678541",
0x1509af02,"MainField_Item_Mushroom_D_3418750269",
0x9b1ea2b9,"MainField_Item_Mushroom_D_3420888773",
0x3d3115bd,"MainField_Item_Mushroom_D_3422603092",
0x04aad662,"MainField_Item_Mushroom_D_3423509193",
0xe8bfc650,"MainField_Item_Mushroom_D_3423929099",
0x0d341153,"MainField_Item_Mushroom_D_3424052763",
0x02f1c07a,"MainField_Item_Mushroom_D_3425222703",
0xac6ed871,"MainField_Item_Mushroom_D_3425863322",
0x7d869a91,"MainField_Item_Mushroom_D_3426235158",
0x7be94000,"MainField_Item_Mushroom_D_3432068675",
0xf130efae,"MainField_Item_Mushroom_D_3433465250",
0xdb339949,"MainField_Item_Mushroom_D_3435028427",
0xe85e2145,"MainField_Item_Mushroom_D_3435289165",
0x71305de7,"MainField_Item_Mushroom_D_3436043348",
0x5d68c86a,"MainField_Item_Mushroom_D_343962495",
0x133254ac,"MainField_Item_Mushroom_D_3440195301",
0x9a1d93e0,"MainField_Item_Mushroom_D_3443456770",
0xd1a73e77,"MainField_Item_Mushroom_D_3444563350",
0xd5af0b5f,"MainField_Item_Mushroom_D_3445559312",
0xf2b586a9,"MainField_Item_Mushroom_D_345176520",
0xe6dfc2f4,"MainField_Item_Mushroom_D_3451942440",
0x8f809571,"MainField_Item_Mushroom_D_3453981921",
0x0963514d,"MainField_Item_Mushroom_D_3458284456",
0xe54d65db,"MainField_Item_Mushroom_D_345944438",
0xa4854bc3,"MainField_Item_Mushroom_D_346010424",
0xa30981c1,"MainField_Item_Mushroom_D_3463622897",
0xb2763419,"MainField_Item_Mushroom_D_3465700469",
0xfe9944a4,"MainField_Item_Mushroom_D_3466721652",
0xf02c8186,"MainField_Item_Mushroom_D_3467324052",
0xeb1cb68c,"MainField_Item_Mushroom_D_3468569623",
0xf3bf12e1,"MainField_Item_Mushroom_D_3469919926",
0xae1ad951,"MainField_Item_Mushroom_D_3470112702",
0xbace15af,"MainField_Item_Mushroom_D_3471089497",
0x9c06ed27,"MainField_Item_Mushroom_D_3473071181",
0x2f6b1afd,"MainField_Item_Mushroom_D_3475317278",
0x6e9cf29d,"MainField_Item_Mushroom_D_3476831004",
0x9e533577,"MainField_Item_Mushroom_D_347819496",
0x2ecad88e,"MainField_Item_Mushroom_D_3479871121",
0x10d2c140,"MainField_Item_Mushroom_D_3480186008",
0xe7d07af8,"MainField_Item_Mushroom_D_3483206409",
0xf9eb1bb9,"MainField_Item_Mushroom_D_348483285",
0x417598db,"MainField_Item_Mushroom_D_3487488769",
0x34719d6e,"MainField_Item_Mushroom_D_3488344675",
0x9a9c5ec4,"MainField_Item_Mushroom_D_3489173391",
0x9a3effcb,"MainField_Item_Mushroom_D_3489695765",
0x2aef6a14,"MainField_Item_Mushroom_D_3491259354",
0xf5b9d68e,"MainField_Item_Mushroom_D_3495007896",
0xb902ff91,"MainField_Item_Mushroom_D_3496082825",
0x227ac781,"MainField_Item_Mushroom_D_3505528707",
0xa7ac3ac0,"MainField_Item_Mushroom_D_3508047912",
0xaab572a0,"MainField_Item_Mushroom_D_350862815",
0x52d419a0,"MainField_Item_Mushroom_D_3509656347",
0x33c577e5,"MainField_Item_Mushroom_D_3512144526",
0xab5f8858,"MainField_Item_Mushroom_D_3520719189",
0xedaccba8,"MainField_Item_Mushroom_D_3521664026",
0x62b92318,"MainField_Item_Mushroom_D_3523469958",
0x498637bf,"MainField_Item_Mushroom_D_3524971094",
0xa861aa71,"MainField_Item_Mushroom_D_3525443336",
0x3b426243,"MainField_Item_Mushroom_D_3526600067",
0x4f075901,"MainField_Item_Mushroom_D_3526637519",
0xf27bb75e,"MainField_Item_Mushroom_D_3527339363",
0x37e79867,"MainField_Item_Mushroom_D_3531230914",
0x1afc5290,"MainField_Item_Mushroom_D_3532510611",
0x1e84efa2,"MainField_Item_Mushroom_D_3533946660",
0x21047637,"MainField_Item_Mushroom_D_3535882346",
0x338febda,"MainField_Item_Mushroom_D_3539275053",
0x5040d5a9,"MainField_Item_Mushroom_D_3540176146",
0x895af11f,"MainField_Item_Mushroom_D_3540785041",
0xf1032a22,"MainField_Item_Mushroom_D_3544631699",
0x7aadd03c,"MainField_Item_Mushroom_D_3545833745",
0x85814fac,"MainField_Item_Mushroom_D_3547093098",
0xb4d1b711,"MainField_Item_Mushroom_D_354892108",
0x22b655d9,"MainField_Item_Mushroom_D_3549184521",
0xf8fa8acb,"MainField_Item_Mushroom_D_3552205168",
0xf036bced,"MainField_Item_Mushroom_D_3553779883",
0x6d0db17b,"MainField_Item_Mushroom_D_3555353234",
0x7639c425,"MainField_Item_Mushroom_D_3556167013",
0x6d66acf5,"MainField_Item_Mushroom_D_355638111",
0x24edf656,"MainField_Item_Mushroom_D_3557090195",
0xa607ff8d,"MainField_Item_Mushroom_D_3558620301",
0x7a8151a5,"MainField_Item_Mushroom_D_3561461033",
0x0a6a1328,"MainField_Item_Mushroom_D_35632469",
0x7d401b46,"MainField_Item_Mushroom_D_357035680",
0x78c6ae5d,"MainField_Item_Mushroom_D_3571201661",
0x8d79a3ea,"MainField_Item_Mushroom_D_3574583304",
0x6a5c3a21,"MainField_Item_Mushroom_D_3575831830",
0xe83edb90,"MainField_Item_Mushroom_D_3576210312",
0xe4070733,"MainField_Item_Mushroom_D_3577732098",
0x5035def9,"MainField_Item_Mushroom_D_3579304036",
0x12365e1f,"MainField_Item_Mushroom_D_3579859190",
0xa63ca28e,"MainField_Item_Mushroom_D_3580434682",
0x64942e59,"MainField_Item_Mushroom_D_3582807664",
0xedb9ebf5,"MainField_Item_Mushroom_D_3587678964",
0xc2590058,"MainField_Item_Mushroom_D_3588267009",
0x42ff4728,"MainField_Item_Mushroom_D_3589107483",
0x20388857,"MainField_Item_Mushroom_D_3589117804",
0x5794589a,"MainField_Item_Mushroom_D_3592444396",
0xd38086b5,"MainField_Item_Mushroom_D_3592519474",
0x6e7f125b,"MainField_Item_Mushroom_D_3593108921",
0xc978d6b4,"MainField_Item_Mushroom_D_3596183887",
0x6061f575,"MainField_Item_Mushroom_D_3596350060",
0xf22665a0,"MainField_Item_Mushroom_D_3601266642",
0x52ce4875,"MainField_Item_Mushroom_D_3601548727",
0x0aec5be5,"MainField_Item_Mushroom_D_3604449177",
0x918128a6,"MainField_Item_Mushroom_D_3605258585",
0x01ef7680,"MainField_Item_Mushroom_D_3608724253",
0xbc5ccf40,"MainField_Item_Mushroom_D_3610361547",
0xe956acfc,"MainField_Item_Mushroom_D_3612671931",
0x98acb5eb,"MainField_Item_Mushroom_D_3613710901",
0x530db2dc,"MainField_Item_Mushroom_D_3617107624",
0xbb72fd8b,"MainField_Item_Mushroom_D_3619126029",
0x89ee37da,"MainField_Item_Mushroom_D_3620918569",
0x2f60ea56,"MainField_Item_Mushroom_D_3624901428",
0x9f547af4,"MainField_Item_Mushroom_D_3625728814",
0xe2b509cb,"MainField_Item_Mushroom_D_3626321385",
0xbeab5d03,"MainField_Item_Mushroom_D_3630223882",
0xcbed53f1,"MainField_Item_Mushroom_D_3635977438",
0x0065d90c,"MainField_Item_Mushroom_D_3637276699",
0x94b56ead,"MainField_Item_Mushroom_D_3638819085",
0x672cb253,"MainField_Item_Mushroom_D_3639020675",
0xd427b5f2,"MainField_Item_Mushroom_D_364030202",
0x46cb5287,"MainField_Item_Mushroom_D_3641573976",
0xe158fac2,"MainField_Item_Mushroom_D_3642060970",
0x4732f352,"MainField_Item_Mushroom_D_364245633",
0xd2910ead,"MainField_Item_Mushroom_D_3645975303",
0x91a28d72,"MainField_Item_Mushroom_D_3646347276",
0xd5a865e7,"MainField_Item_Mushroom_D_364700885",
0x1de07bb9,"MainField_Item_Mushroom_D_3648421567",
0x96df5105,"MainField_Item_Mushroom_D_3648422187",
0x8833869e,"MainField_Item_Mushroom_D_3648650149",
0x5dcb05ee,"MainField_Item_Mushroom_D_364948520",
0xa1352b83,"MainField_Item_Mushroom_D_3655004408",
0xbc5fdd32,"MainField_Item_Mushroom_D_3655172497",
0x85a5d76f,"MainField_Item_Mushroom_D_3656943787",
0x97da7796,"MainField_Item_Mushroom_D_3659079481",
0xc7a198f5,"MainField_Item_Mushroom_D_3663831249",
0x3ccb4883,"MainField_Item_Mushroom_D_3664014517",
0xbad05c1e,"MainField_Item_Mushroom_D_3664259925",
0xdf1febdd,"MainField_Item_Mushroom_D_3664394926",
0x1342cb29,"MainField_Item_Mushroom_D_3664744154",
0xd091be49,"MainField_Item_Mushroom_D_3666117782",
0x9a419425,"MainField_Item_Mushroom_D_3666720948",
0x60b1fb42,"MainField_Item_Mushroom_D_366850127",
0x7e33a706,"MainField_Item_Mushroom_D_3668509707",
0xe32f8b3c,"MainField_Item_Mushroom_D_3668681558",
0x5d17cb82,"MainField_Item_Mushroom_D_3668837395",
0x3672fc07,"MainField_Item_Mushroom_D_3669321314",
0xece20d34,"MainField_Item_Mushroom_D_3672137768",
0x1d042e1e,"MainField_Item_Mushroom_D_3675870082",
0xe60d989e,"MainField_Item_Mushroom_D_3675937409",
0xf6c58922,"MainField_Item_Mushroom_D_3676358569",
0xf84e35c6,"MainField_Item_Mushroom_D_3678068237",
0x93940db0,"MainField_Item_Mushroom_D_3679060989",
0x67843472,"MainField_Item_Mushroom_D_3679720827",
0xcc9418ff,"MainField_Item_Mushroom_D_3680711385",
0x7e9000e0,"MainField_Item_Mushroom_D_3682470334",
0x62593f65,"MainField_Item_Mushroom_D_3688299946",
0x1a2d9a0e,"MainField_Item_Mushroom_D_3689149261",
0x21852031,"MainField_Item_Mushroom_D_3689692838",
0xc65cbe8d,"MainField_Item_Mushroom_D_3691042049",
0xca70c485,"MainField_Item_Mushroom_D_3691289729",
0xf0f36893,"MainField_Item_Mushroom_D_3692906993",
0x8fcbb7bb,"MainField_Item_Mushroom_D_3693747006",
0xccac2190,"MainField_Item_Mushroom_D_3696190821",
0x355f918a,"MainField_Item_Mushroom_D_3697348619",
0x731d2479,"MainField_Item_Mushroom_D_3700928863",
0x0cef984c,"MainField_Item_Mushroom_D_3701361624",
0xb7e56e62,"MainField_Item_Mushroom_D_370297269",
0x70abdedf,"MainField_Item_Mushroom_D_3703292517",
0x580b8010,"MainField_Item_Mushroom_D_3704092638",
0x59ac40b1,"MainField_Item_Mushroom_D_3711195502",
0xc7cf7f50,"MainField_Item_Mushroom_D_3711512970",
0xc396db8e,"MainField_Item_Mushroom_D_3715138596",
0x1ea3511e,"MainField_Item_Mushroom_D_3715708950",
0x1ed021c2,"MainField_Item_Mushroom_D_371616992",
0x44cd1b46,"MainField_Item_Mushroom_D_3717644280",
0xb2fcdcd2,"MainField_Item_Mushroom_D_3718584746",
0x0ccbd9a3,"MainField_Item_Mushroom_D_3721449494",
0xed8e2805,"MainField_Item_Mushroom_D_3721799574",
0x8ac1f040,"MainField_Item_Mushroom_D_3722240363",
0x3b7d2e5e,"MainField_Item_Mushroom_D_3723298253",
0x87dcee07,"MainField_Item_Mushroom_D_3724184111",
0x8838ac98,"MainField_Item_Mushroom_D_3725036998",
0x7c2c4b33,"MainField_Item_Mushroom_D_3725706805",
0x4955132e,"MainField_Item_Mushroom_D_3727919213",
0x8d7840e9,"MainField_Item_Mushroom_D_3730932453",
0x6200482b,"MainField_Item_Mushroom_D_3732215345",
0x436161c4,"MainField_Item_Mushroom_D_3732263195",
0x4f5d8550,"MainField_Item_Mushroom_D_3732345096",
0xaeef6b55,"MainField_Item_Mushroom_D_3734001221",
0xf5d0ff00,"MainField_Item_Mushroom_D_3734584797",
0x4119614e,"MainField_Item_Mushroom_D_3736017082",
0xa99aee2f,"MainField_Item_Mushroom_D_3736178822",
0x7d26b19f,"MainField_Item_Mushroom_D_3736666867",
0xd11754ef,"MainField_Item_Mushroom_D_373836469",
0xea5a860c,"MainField_Item_Mushroom_D_3739561281",
0xc7a18d81,"MainField_Item_Mushroom_D_3743793135",
0xf349d3f7,"MainField_Item_Mushroom_D_375161714",
0xa1c8c5ce,"MainField_Item_Mushroom_D_375235917",
0x9148b1b0,"MainField_Item_Mushroom_D_3753877100",
0xfc1afac6,"MainField_Item_Mushroom_D_3754147563",
0x2be19791,"MainField_Item_Mushroom_D_3755840955",
0xf135dfb6,"MainField_Item_Mushroom_D_3756557947",
0x2dbb9b1d,"MainField_Item_Mushroom_D_3757632866",
0xdf11c754,"MainField_Item_Mushroom_D_3759180377",
0x813bc6d0,"MainField_Item_Mushroom_D_3759886742",
0x8d8333ef,"MainField_Item_Mushroom_D_3762713536",
0x3e5de02a,"MainField_Item_Mushroom_D_3762932742",
0xb7bfc2cd,"MainField_Item_Mushroom_D_3763238672",
0xda583eaa,"MainField_Item_Mushroom_D_3763307941",
0x6b20b4ea,"MainField_Item_Mushroom_D_376366540",
0x55da4588,"MainField_Item_Mushroom_D_3766928421",
0xbae38126,"MainField_Item_Mushroom_D_3768110052",
0x988a206d,"MainField_Item_Mushroom_D_3772471166",
0x50759207,"MainField_Item_Mushroom_D_377526426",
0xbe565523,"MainField_Item_Mushroom_D_3776798882",
0x180f563b,"MainField_Item_Mushroom_D_3778021584",
0x37630c17,"MainField_Item_Mushroom_D_3781693476",
0xc420659e,"MainField_Item_Mushroom_D_378358396",
0x91ab8978,"MainField_Item_Mushroom_D_3784999832",
0x318949f8,"MainField_Item_Mushroom_D_3789209844",
0x76b72ddc,"MainField_Item_Mushroom_D_3792966731",
0x01b975f8,"MainField_Item_Mushroom_D_3793397041",
0x943807d6,"MainField_Item_Mushroom_D_3793686931",
0x0d312623,"MainField_Item_Mushroom_D_3793808969",
0x587934f9,"MainField_Item_Mushroom_D_3793974221",
0x00f17f02,"MainField_Item_Mushroom_D_3794790090",
0xbd646c3f,"MainField_Item_Mushroom_D_3795356866",
0x9b3e3937,"MainField_Item_Mushroom_D_379799282",
0x08776688,"MainField_Item_Mushroom_D_3799894232",
0x1a6412de,"MainField_Item_Mushroom_D_3801374712",
0xff77a51d,"MainField_Item_Mushroom_D_3805822619",
0xf60ede25,"MainField_Item_Mushroom_D_380765836",
0x9418b96e,"MainField_Item_Mushroom_D_3809992854",
0xd9aa0424,"MainField_Item_Mushroom_D_3813495258",
0xf3452587,"MainField_Item_Mushroom_D_3814422558",
0x3261b4a7,"MainField_Item_Mushroom_D_3817799395",
0x3757f49c,"MainField_Item_Mushroom_D_3818356868",
0x67fbb6f1,"MainField_Item_Mushroom_D_3822930586",
0xed37f9d4,"MainField_Item_Mushroom_D_3825315613",
0x6bafb3aa,"MainField_Item_Mushroom_D_3826800383",
0x555cef2c,"MainField_Item_Mushroom_D_3828609209",
0xefa53662,"MainField_Item_Mushroom_D_3830059061",
0xa684a48e,"MainField_Item_Mushroom_D_3830978046",
0x6cd64356,"MainField_Item_Mushroom_D_3831827385",
0x4cc05821,"MainField_Item_Mushroom_D_3833088216",
0xba339f2b,"MainField_Item_Mushroom_D_3833839065",
0x9ce430ff,"MainField_Item_Mushroom_D_3834279756",
0xb94cba83,"MainField_Item_Mushroom_D_3836246275",
0x4ab45688,"MainField_Item_Mushroom_D_3841756211",
0x95766c40,"MainField_Item_Mushroom_D_3842929931",
0x3d674886,"MainField_Item_Mushroom_D_3844866647",
0xdcc0adf4,"MainField_Item_Mushroom_D_3845699987",
0x2edd4dbb,"MainField_Item_Mushroom_D_3846590119",
0x80849fed,"MainField_Item_Mushroom_D_3848072126",
0x617923d2,"MainField_Item_Mushroom_D_3849603225",
0x432da8bb,"MainField_Item_Mushroom_D_3850050896",
0xb6a069cb,"MainField_Item_Mushroom_D_3850136691",
0xadae68cd,"MainField_Item_Mushroom_D_3853613622",
0x2196a591,"MainField_Item_Mushroom_D_3853936502",
0xc4932b2b,"MainField_Item_Mushroom_D_3856983349",
0xbee57ba3,"MainField_Item_Mushroom_D_3858812293",
0xb53c852a,"MainField_Item_Mushroom_D_3859041177",
0x74f6ede0,"MainField_Item_Mushroom_D_3860376483",
0x68516479,"MainField_Item_Mushroom_D_3866614031",
0x3fa26830,"MainField_Item_Mushroom_D_3870072226",
0x58350d27,"MainField_Item_Mushroom_D_3870190740",
0xc524e153,"MainField_Item_Mushroom_D_3870564611",
0x70d7d1ec,"MainField_Item_Mushroom_D_387659193",
0x4a59ae58,"MainField_Item_Mushroom_D_3876736727",
0x4bf09b17,"MainField_Item_Mushroom_D_3880182275",
0xab9fb733,"MainField_Item_Mushroom_D_3881570584",
0x0f9cbe53,"MainField_Item_Mushroom_D_3884674896",
0x5431eca9,"MainField_Item_Mushroom_D_3886109999",
0xe0d91b5c,"MainField_Item_Mushroom_D_3888192003",
0x8eeee1e4,"MainField_Item_Mushroom_D_3889944640",
0xcbdf081e,"MainField_Item_Mushroom_D_3890167912",
0x7091db39,"MainField_Item_Mushroom_D_3891208789",
0x4cce2172,"MainField_Item_Mushroom_D_3896259796",
0x68525866,"MainField_Item_Mushroom_D_3897481764",
0x3f4cc1a5,"MainField_Item_Mushroom_D_3908246850",
0x189c3534,"MainField_Item_Mushroom_D_3909279857",
0x8abae9b9,"MainField_Item_Mushroom_D_3909425434",
0x946482fe,"MainField_Item_Mushroom_D_3910150162",
0x094a3932,"MainField_Item_Mushroom_D_3911425564",
0x8c4669e1,"MainField_Item_Mushroom_D_3915951595",
0x016dd5f3,"MainField_Item_Mushroom_D_391935372",
0x552d8c91,"MainField_Item_Mushroom_D_3920221209",
0xb3879665,"MainField_Item_Mushroom_D_392286886",
0xbdaf7134,"MainField_Item_Mushroom_D_3926799849",
0x9f5693c7,"MainField_Item_Mushroom_D_3927150247",
0x1b99eb1f,"MainField_Item_Mushroom_D_3927416043",
0x56293de0,"MainField_Item_Mushroom_D_3928147519",
0x348edd5f,"MainField_Item_Mushroom_D_3929427609",
0xf17ed96b,"MainField_Item_Mushroom_D_3930341087",
0x70c7427c,"MainField_Item_Mushroom_D_3932401099",
0x175dbb82,"MainField_Item_Mushroom_D_3932868770",
0x6c0b5b42,"MainField_Item_Mushroom_D_3935783211",
0x3a731136,"MainField_Item_Mushroom_D_3937467449",
0x7f8867a8,"MainField_Item_Mushroom_D_3940263180",
0x5b23523e,"MainField_Item_Mushroom_D_3942281919",
0xc54e6349,"MainField_Item_Mushroom_D_3943030677",
0xf1d38ad1,"MainField_Item_Mushroom_D_3948528697",
0xed30defa,"MainField_Item_Mushroom_D_395064302",
0x03629a71,"MainField_Item_Mushroom_D_3950926090",
0xa0d5eb00,"MainField_Item_Mushroom_D_3951685609",
0xacc81c0b,"MainField_Item_Mushroom_D_3954131636",
0x97bcc8bb,"MainField_Item_Mushroom_D_3955447703",
0x62dac4ea,"MainField_Item_Mushroom_D_3962717548",
0x69335a04,"MainField_Item_Mushroom_D_3963252364",
0xf28ec80b,"MainField_Item_Mushroom_D_3964428104",
0xbe5805f4,"MainField_Item_Mushroom_D_3965334117",
0x486350a8,"MainField_Item_Mushroom_D_3965715330",
0x29a036e8,"MainField_Item_Mushroom_D_3969585480",
0xe4d30309,"MainField_Item_Mushroom_D_396973931",
0x7ea6e0a6,"MainField_Item_Mushroom_D_3969989672",
0x4c381193,"MainField_Item_Mushroom_D_3970025402",
0x343a9d45,"MainField_Item_Mushroom_D_397069339",
0x406ceedd,"MainField_Item_Mushroom_D_3971425622",
0x606ba355,"MainField_Item_Mushroom_D_3973785372",
0xad49480c,"MainField_Item_Mushroom_D_3975169169",
0xe7931a49,"MainField_Item_Mushroom_D_3978857317",
0xdffcda5b,"MainField_Item_Mushroom_D_3981497378",
0xcedc0c64,"MainField_Item_Mushroom_D_3982267324",
0xc463e5f9,"MainField_Item_Mushroom_D_398361318",
0x996c705d,"MainField_Item_Mushroom_D_3984466211",
0x24821a6f,"MainField_Item_Mushroom_D_398705819",
0xf2832c66,"MainField_Item_Mushroom_D_3989676810",
0x7de09b66,"MainField_Item_Mushroom_D_3991872290",
0xccfc5922,"MainField_Item_Mushroom_D_3993822820",
0xe7e2c691,"MainField_Item_Mushroom_D_399598185",
0x878ac8d9,"MainField_Item_Mushroom_D_3996146300",
0x751fdb2e,"MainField_Item_Mushroom_D_3998524249",
0xad1f4782,"MainField_Item_Mushroom_D_399929821",
0xaed58aa0,"MainField_Item_Mushroom_D_3999346656",
0x2b73ab51,"MainField_Item_Mushroom_D_4000055901",
0x083e3985,"MainField_Item_Mushroom_D_4000592364",
0x6e923778,"MainField_Item_Mushroom_D_4002535705",
0x008db575,"MainField_Item_Mushroom_D_4009259632",
0x5c340886,"MainField_Item_Mushroom_D_4010445544",
0xff2dd4c3,"MainField_Item_Mushroom_D_4012953947",
0xc91c0a1b,"MainField_Item_Mushroom_D_4015939906",
0x0bc03e04,"MainField_Item_Mushroom_D_4018141322",
0x08e354c8,"MainField_Item_Mushroom_D_4018970580",
0xa9669134,"MainField_Item_Mushroom_D_4019038198",
0x63ae9f55,"MainField_Item_Mushroom_D_4025436238",
0x89e24306,"MainField_Item_Mushroom_D_4029340113",
0x1451cfaf,"MainField_Item_Mushroom_D_4030811635",
0x88c9fc75,"MainField_Item_Mushroom_D_403481717",
0xc049101f,"MainField_Item_Mushroom_D_4036795472",
0x03c8520e,"MainField_Item_Mushroom_D_40372298",
0x5a23f3bc,"MainField_Item_Mushroom_D_4038276756",
0x8d05d6f5,"MainField_Item_Mushroom_D_4038875312",
0xa03270d4,"MainField_Item_Mushroom_D_4041125891",
0x6866fd79,"MainField_Item_Mushroom_D_4043078500",
0xb9ffb6bd,"MainField_Item_Mushroom_D_4046357061",
0x20097ab5,"MainField_Item_Mushroom_D_4046953356",
0x0e65271a,"MainField_Item_Mushroom_D_4047328522",
0x8c8c8954,"MainField_Item_Mushroom_D_4048227375",
0x036aed47,"MainField_Item_Mushroom_D_404957799",
0xfc8a11cd,"MainField_Item_Mushroom_D_405194480",
0xcbe64951,"MainField_Item_Mushroom_D_4052632617",
0x3b6fff80,"MainField_Item_Mushroom_D_4053017978",
0x4aeb43df,"MainField_Item_Mushroom_D_4054985450",
0x80dcf701,"MainField_Item_Mushroom_D_4058145568",
0x8083c138,"MainField_Item_Mushroom_D_4063720281",
0xfa556eb1,"MainField_Item_Mushroom_D_4064100623",
0x41da50f4,"MainField_Item_Mushroom_D_406481010",
0xf04db81f,"MainField_Item_Mushroom_D_4065363794",
0xe6c1ca3f,"MainField_Item_Mushroom_D_4070903705",
0x649ea92a,"MainField_Item_Mushroom_D_4070912950",
0x237f95d1,"MainField_Item_Mushroom_D_4071917923",
0xccc15aaf,"MainField_Item_Mushroom_D_4072056645",
0x3b4d457c,"MainField_Item_Mushroom_D_4074720255",
0xdb56e7fb,"MainField_Item_Mushroom_D_4076913118",
0x8c40d11f,"MainField_Item_Mushroom_D_4077660885",
0x7cb72e15,"MainField_Item_Mushroom_D_407803515",
0xaf42aa26,"MainField_Item_Mushroom_D_4082063798",
0xdc9a8a00,"MainField_Item_Mushroom_D_4084077354",
0x12b8800c,"MainField_Item_Mushroom_D_4086655586",
0xdbc03a49,"MainField_Item_Mushroom_D_408724882",
0x4fdc5bb9,"MainField_Item_Mushroom_D_4087632567",
0x345475e3,"MainField_Item_Mushroom_D_4087724340",
0x21dd5c44,"MainField_Item_Mushroom_D_4090560793",
0x32a64618,"MainField_Item_Mushroom_D_4093378078",
0x067921f9,"MainField_Item_Mushroom_D_4095421585",
0x02df81be,"MainField_Item_Mushroom_D_4095724708",
0xefba3bf4,"MainField_Item_Mushroom_D_4101504134",
0xa4b5299c,"MainField_Item_Mushroom_D_4105988107",
0xb8faffd0,"MainField_Item_Mushroom_D_4106247425",
0x052b8180,"MainField_Item_Mushroom_D_4106425708",
0xb8d879a6,"MainField_Item_Mushroom_D_4106586830",
0xf63eb6a6,"MainField_Item_Mushroom_D_4108796293",
0x7cb7ced4,"MainField_Item_Mushroom_D_4110391492",
0xbe3030d0,"MainField_Item_Mushroom_D_4112259091",
0x207990b4,"MainField_Item_Mushroom_D_4113474336",
0x508089e0,"MainField_Item_Mushroom_D_4115325585",
0x566e6898,"MainField_Item_Mushroom_D_4121195347",
0x50a1a3b2,"MainField_Item_Mushroom_D_4122860131",
0xfded02f0,"MainField_Item_Mushroom_D_4123548287",
0xe346541b,"MainField_Item_Mushroom_D_4125739919",
0x6af31260,"MainField_Item_Mushroom_D_4127620047",
0xf5198e11,"MainField_Item_Mushroom_D_4127788086",
0x92aa460d,"MainField_Item_Mushroom_D_4130477849",
0x79e184bf,"MainField_Item_Mushroom_D_4131959109",
0xf70c7ac6,"MainField_Item_Mushroom_D_4132844788",
0x6485a549,"MainField_Item_Mushroom_D_4132980898",
0x72eb315b,"MainField_Item_Mushroom_D_4134038892",
0x4394c952,"MainField_Item_Mushroom_D_4134467151",
0x9b76c4b1,"MainField_Item_Mushroom_D_4135806279",
0xe354d2f8,"MainField_Item_Mushroom_D_4136941626",
0xafa3ba68,"MainField_Item_Mushroom_D_4141059213",
0xd3e08b67,"MainField_Item_Mushroom_D_4141928290",
0x1df5617c,"MainField_Item_Mushroom_D_4142191832",
0x6beb2b24,"MainField_Item_Mushroom_D_4145422529",
0x72b74df1,"MainField_Item_Mushroom_D_4145439181",
0x87bee28b,"MainField_Item_Mushroom_D_4145588474",
0xcd7b6d9d,"MainField_Item_Mushroom_D_4145693411",
0x41560ada,"MainField_Item_Mushroom_D_4146293070",
0xb4a006df,"MainField_Item_Mushroom_D_4150748611",
0xa8c10c0f,"MainField_Item_Mushroom_D_4151162849",
0x1bd720c2,"MainField_Item_Mushroom_D_4152485453",
0x45023dfb,"MainField_Item_Mushroom_D_415451918",
0x519d4070,"MainField_Item_Mushroom_D_4154597903",
0xb8081594,"MainField_Item_Mushroom_D_4160114662",
0x790e3635,"MainField_Item_Mushroom_D_4161217878",
0xde30af5b,"MainField_Item_Mushroom_D_4162141437",
0x74bc6098,"MainField_Item_Mushroom_D_4162731150",
0x99bf7ad2,"MainField_Item_Mushroom_D_4167907303",
0xbc5fc36a,"MainField_Item_Mushroom_D_4168056114",
0xf4bc1dcf,"MainField_Item_Mushroom_D_417218640",
0x07414438,"MainField_Item_Mushroom_D_4172640704",
0x536ff489,"MainField_Item_Mushroom_D_4173315468",
0x6c9a4288,"MainField_Item_Mushroom_D_4174756994",
0x09dd47b8,"MainField_Item_Mushroom_D_417638554",
0x4512e5a2,"MainField_Item_Mushroom_D_4176960496",
0xb603a8c7,"MainField_Item_Mushroom_D_4178552359",
0xcb184b8b,"MainField_Item_Mushroom_D_4179966319",
0xced3f16a,"MainField_Item_Mushroom_D_4196288233",
0x5c722d10,"MainField_Item_Mushroom_D_4197124200",
0xe328367f,"MainField_Item_Mushroom_D_4198104579",
0xdeefe559,"MainField_Item_Mushroom_D_419950721",
0x2e556f07,"MainField_Item_Mushroom_D_4207809912",
0x97f3aa13,"MainField_Item_Mushroom_D_4208346401",
0xc965b0a5,"MainField_Item_Mushroom_D_4209419159",
0xa4550deb,"MainField_Item_Mushroom_D_4213049787",
0x19afa7ed,"MainField_Item_Mushroom_D_4213841967",
0x7ec18db8,"MainField_Item_Mushroom_D_4214677920",
0x28e65f3f,"MainField_Item_Mushroom_D_4217635951",
0x6a0c431c,"MainField_Item_Mushroom_D_4219626096",
0x05fea11d,"MainField_Item_Mushroom_D_4219695020",
0xfd376fda,"MainField_Item_Mushroom_D_4221462661",
0xd886d748,"MainField_Item_Mushroom_D_4223300623",
0x11e603de,"MainField_Item_Mushroom_D_4223557153",
0xdf491236,"MainField_Item_Mushroom_D_4227852360",
0xb6c2223a,"MainField_Item_Mushroom_D_4228024189",
0xc71539d5,"MainField_Item_Mushroom_D_4228385689",
0x817d0e24,"MainField_Item_Mushroom_D_4230377930",
0x8a5a28b0,"MainField_Item_Mushroom_D_4230892384",
0xad170fdc,"MainField_Item_Mushroom_D_4231698414",
0x88d6dd16,"MainField_Item_Mushroom_D_4231811670",
0x0a5977e0,"MainField_Item_Mushroom_D_4232439992",
0xf1475ae3,"MainField_Item_Mushroom_D_4232921346",
0x13b32716,"MainField_Item_Mushroom_D_423387044",
0x38e3e54c,"MainField_Item_Mushroom_D_423531667",
0x99935482,"MainField_Item_Mushroom_D_4235326014",
0xf4dfc7be,"MainField_Item_Mushroom_D_4236941512",
0x7f0ac523,"MainField_Item_Mushroom_D_4239326471",
0x2c3ce423,"MainField_Item_Mushroom_D_4239394233",
0xc57ce455,"MainField_Item_Mushroom_D_4250977400",
0x5921c804,"MainField_Item_Mushroom_D_4254769917",
0xf5d255f9,"MainField_Item_Mushroom_D_4255073384",
0x1ce3ea16,"MainField_Item_Mushroom_D_4256543300",
0xa8f68c13,"MainField_Item_Mushroom_D_425674550",
0x66db5cff,"MainField_Item_Mushroom_D_4260613026",
0x0bf89c19,"MainField_Item_Mushroom_D_4260852863",
0xefa765fe,"MainField_Item_Mushroom_D_4262044893",
0xfb1a33b0,"MainField_Item_Mushroom_D_4263864319",
0x252a50ee,"MainField_Item_Mushroom_D_426490362",
0xc06eb180,"MainField_Item_Mushroom_D_4265083212",
0x1d983edf,"MainField_Item_Mushroom_D_4266707937",
0x20fd88be,"MainField_Item_Mushroom_D_4275056641",
0x499ea142,"MainField_Item_Mushroom_D_4279422349",
0xcb4cb675,"MainField_Item_Mushroom_D_4279874004",
0xe7a4507b,"MainField_Item_Mushroom_D_4280977989",
0x39d10a0e,"MainField_Item_Mushroom_D_4284355153",
0x25684cba,"MainField_Item_Mushroom_D_4284938265",
0xc8474c49,"MainField_Item_Mushroom_D_4285478165",
0xb8761654,"MainField_Item_Mushroom_D_428976487",
0x8d8b41d8,"MainField_Item_Mushroom_D_4290502853",
0x00b2a7c4,"MainField_Item_Mushroom_D_4291264909",
0x187d6bdf,"MainField_Item_Mushroom_D_4294041696",
0xa2b13bb3,"MainField_Item_Mushroom_D_4294310258",
0xa42a0803,"MainField_Item_Mushroom_D_4294557484",
0xbaf61fe2,"MainField_Item_Mushroom_D_430944225",
0x59c550d8,"MainField_Item_Mushroom_D_434585031",
0x93814d7e,"MainField_Item_Mushroom_D_437970544",
0x6afa03c3,"MainField_Item_Mushroom_D_440838223",
0x2fa8e757,"MainField_Item_Mushroom_D_445186900",
0x0c4911a2,"MainField_Item_Mushroom_D_445488307",
0x5669f9b5,"MainField_Item_Mushroom_D_446140595",
0xea69a09b,"MainField_Item_Mushroom_D_446662094",
0x4c758895,"MainField_Item_Mushroom_D_448366343",
0xc7f9af81,"MainField_Item_Mushroom_D_452480707",
0xc1c833b7,"MainField_Item_Mushroom_D_459486766",
0xa0e27c2a,"MainField_Item_Mushroom_D_460361916",
0xf3b91ce4,"MainField_Item_Mushroom_D_462191616",
0xcb1a7066,"MainField_Item_Mushroom_D_462645495",
0xbefd6cc4,"MainField_Item_Mushroom_D_46315241",
0xa841ba73,"MainField_Item_Mushroom_D_465675974",
0x7eae50a0,"MainField_Item_Mushroom_D_466409145",
0xe44a3d12,"MainField_Item_Mushroom_D_472744620",
0x460be83c,"MainField_Item_Mushroom_D_47609411",
0xea12c4ae,"MainField_Item_Mushroom_D_477128801",
0x4da2109f,"MainField_Item_Mushroom_D_479560830",
0xe4b4d63e,"MainField_Item_Mushroom_D_479928663",
0x32964b73,"MainField_Item_Mushroom_D_480408803",
0x7bfc5934,"MainField_Item_Mushroom_D_480717920",
0xebf711a8,"MainField_Item_Mushroom_D_482829311",
0x5a87eecb,"MainField_Item_Mushroom_D_482978340",
0x195622f4,"MainField_Item_Mushroom_D_485512405",
0x749387ee,"MainField_Item_Mushroom_D_486706926",
0x3fd197cf,"MainField_Item_Mushroom_D_486931905",
0x6c4d8a40,"MainField_Item_Mushroom_D_494804284",
0x59b8afd2,"MainField_Item_Mushroom_D_496627820",
0xe9a3723a,"MainField_Item_Mushroom_D_496719802",
0x6b1bdd0b,"MainField_Item_Mushroom_D_499306840",
0xd7636c64,"MainField_Item_Mushroom_D_50194445",
0xcabe21de,"MainField_Item_Mushroom_D_503417747",
0xbc71aec3,"MainField_Item_Mushroom_D_507050813",
0xfb009c4d,"MainField_Item_Mushroom_D_51016067",
0xed7271ce,"MainField_Item_Mushroom_D_512220585",
0xb195293f,"MainField_Item_Mushroom_D_513229575",
0x77f3829a,"MainField_Item_Mushroom_D_52169539",
0x7599b982,"MainField_Item_Mushroom_D_522675667",
0x0612082f,"MainField_Item_Mushroom_D_523295796",
0xe72ee95b,"MainField_Item_Mushroom_D_525639786",
0xabb1997a,"MainField_Item_Mushroom_D_53317875",
0x44be7b9d,"MainField_Item_Mushroom_D_533744372",
0xc78272ae,"MainField_Item_Mushroom_D_534699138",
0xc59762e1,"MainField_Item_Mushroom_D_539944281",
0xa03c90aa,"MainField_Item_Mushroom_D_541435483",
0x8810fafa,"MainField_Item_Mushroom_D_544327495",
0xa8dca121,"MainField_Item_Mushroom_D_546089771",
0x5adf2225,"MainField_Item_Mushroom_D_54802719",
0x052264c5,"MainField_Item_Mushroom_D_548076614",
0x3eda0642,"MainField_Item_Mushroom_D_548425019",
0xc8ab8cc9,"MainField_Item_Mushroom_D_549936791",
0x90a27fb0,"MainField_Item_Mushroom_D_550476953",
0x8df0a24b,"MainField_Item_Mushroom_D_550996312",
0x05710123,"MainField_Item_Mushroom_D_5529018",
0x4fb65b79,"MainField_Item_Mushroom_D_554307194",
0xc1574278,"MainField_Item_Mushroom_D_55461437",
0x5a20f720,"MainField_Item_Mushroom_D_554786494",
0xfedd6ccd,"MainField_Item_Mushroom_D_555239027",
0x4a33fe8e,"MainField_Item_Mushroom_D_555682386",
0xafff2127,"MainField_Item_Mushroom_D_556467828",
0x8809967b,"MainField_Item_Mushroom_D_556813468",
0x3442012c,"MainField_Item_Mushroom_D_557347348",
0x680e6b9a,"MainField_Item_Mushroom_D_55870402",
0x03710f14,"MainField_Item_Mushroom_D_561051269",
0xfbc50bb9,"MainField_Item_Mushroom_D_561233833",
0x35edab73,"MainField_Item_Mushroom_D_563959515",
0xf336067c,"MainField_Item_Mushroom_D_568919964",
0xb5ee6a9f,"MainField_Item_Mushroom_D_570679407",
0x52960b30,"MainField_Item_Mushroom_D_573939697",
0x1dcd1604,"MainField_Item_Mushroom_D_574171049",
0x01a05395,"MainField_Item_Mushroom_D_577899448",
0x5926220e,"MainField_Item_Mushroom_D_579599973",
0xb473d04e,"MainField_Item_Mushroom_D_579870374",
0xfdc63fdc,"MainField_Item_Mushroom_D_580398864",
0xfd0d980e,"MainField_Item_Mushroom_D_580981181",
0x7b5b5ef8,"MainField_Item_Mushroom_D_581023960",
0x78ccc9a3,"MainField_Item_Mushroom_D_581411430",
0x520aed0d,"MainField_Item_Mushroom_D_581956822",
0xed10f676,"MainField_Item_Mushroom_D_59109125",
0x4f8acb41,"MainField_Item_Mushroom_D_593341552",
0x5eeada6b,"MainField_Item_Mushroom_D_597091255",
0xada6736e,"MainField_Item_Mushroom_D_59836428",
0x0112b1a1,"MainField_Item_Mushroom_D_59936668",
0x7bf07c3e,"MainField_Item_Mushroom_D_60265442",
0xade89c82,"MainField_Item_Mushroom_D_606401953",
0x75017b06,"MainField_Item_Mushroom_D_607642092",
0x605966ce,"MainField_Item_Mushroom_D_608154598",
0x4ec593d9,"MainField_Item_Mushroom_D_609553187",
0x894769e8,"MainField_Item_Mushroom_D_610248328",
0x452b303e,"MainField_Item_Mushroom_D_615012374",
0xcb97ded0,"MainField_Item_Mushroom_D_618017556",
0x24160575,"MainField_Item_Mushroom_D_618317147",
0x305f1196,"MainField_Item_Mushroom_D_628564018",
0x251be93d,"MainField_Item_Mushroom_D_628838696",
0x8ad4d471,"MainField_Item_Mushroom_D_629072521",
0x2af6b59c,"MainField_Item_Mushroom_D_631932580",
0x65d3b976,"MainField_Item_Mushroom_D_632817079",
0x42c67a1f,"MainField_Item_Mushroom_D_638169336",
0x0b847bdf,"MainField_Item_Mushroom_D_641360082",
0x146c8bc1,"MainField_Item_Mushroom_D_642142156",
0x528d3b8e,"MainField_Item_Mushroom_D_642460139",
0x5b6f0cc2,"MainField_Item_Mushroom_D_644729324",
0x365ce0f1,"MainField_Item_Mushroom_D_646306201",
0x0255f28c,"MainField_Item_Mushroom_D_646825421",
0xed917c40,"MainField_Item_Mushroom_D_648816891",
0x26ce6c9a,"MainField_Item_Mushroom_D_65363311",
0x25f2491f,"MainField_Item_Mushroom_D_654887990",
0x4a2c0b19,"MainField_Item_Mushroom_D_655626222",
0x84bf1fb1,"MainField_Item_Mushroom_D_655775659",
0x096eca6a,"MainField_Item_Mushroom_D_657075004",
0x8b9841e6,"MainField_Item_Mushroom_D_657466892",
0x5d230e45,"MainField_Item_Mushroom_D_658869571",
0x6880cc62,"MainField_Item_Mushroom_D_662799660",
0x45599391,"MainField_Item_Mushroom_D_662884870",
0xef910092,"MainField_Item_Mushroom_D_667879563",
0x36cadf6e,"MainField_Item_Mushroom_D_668829986",
0xf7ec858e,"MainField_Item_Mushroom_D_669101473",
0x43a55c4a,"MainField_Item_Mushroom_D_669454784",
0xafd96216,"MainField_Item_Mushroom_D_675503133",
0xd4fe19ff,"MainField_Item_Mushroom_D_676898107",
0xa46677d4,"MainField_Item_Mushroom_D_679878472",
0xd5890d8d,"MainField_Item_Mushroom_D_681156417",
0x82c39ea4,"MainField_Item_Mushroom_D_682734145",
0x02d5d7c6,"MainField_Item_Mushroom_D_683674451",
0x68b945a7,"MainField_Item_Mushroom_D_686813049",
0x10eee404,"MainField_Item_Mushroom_D_686934329",
0xd25f0cf0,"MainField_Item_Mushroom_D_687712093",
0xe0291457,"MainField_Item_Mushroom_D_688642346",
0x15c612b7,"MainField_Item_Mushroom_D_690503978",
0x81ee227e,"MainField_Item_Mushroom_D_697946399",
0x2e11d54a,"MainField_Item_Mushroom_D_699785481",
0x508d6f23,"MainField_Item_Mushroom_D_70115735",
0x19368696,"MainField_Item_Mushroom_D_701661474",
0x7ebe480a,"MainField_Item_Mushroom_D_709882146",
0xa05aa58f,"MainField_Item_Mushroom_D_71363822",
0xf5130f79,"MainField_Item_Mushroom_D_71473191",
0x3ebd8e58,"MainField_Item_Mushroom_D_71753252",
0xd2ed5c7a,"MainField_Item_Mushroom_D_718260243",
0x118a062e,"MainField_Item_Mushroom_D_719003670",
0xdbdb46ec,"MainField_Item_Mushroom_D_720800040",
0xfb5f330b,"MainField_Item_Mushroom_D_72435470",
0xe4f62726,"MainField_Item_Mushroom_D_725726572",
0x6efa54b0,"MainField_Item_Mushroom_D_726545836",
0xd9873cdc,"MainField_Item_Mushroom_D_728658319",
0x64acf6ad,"MainField_Item_Mushroom_D_729040968",
0xa5a1f0c5,"MainField_Item_Mushroom_D_731050764",
0x2a4cd072,"MainField_Item_Mushroom_D_73190718",
0x2cd5fb04,"MainField_Item_Mushroom_D_732522772",
0x777170d1,"MainField_Item_Mushroom_D_733886161",
0xcc3d7ef1,"MainField_Item_Mushroom_D_73619391",
0xd0467076,"MainField_Item_Mushroom_D_737184243",
0x55676ef0,"MainField_Item_Mushroom_D_741021795",
0x1c483208,"MainField_Item_Mushroom_D_745232309",
0x86bbad21,"MainField_Item_Mushroom_D_746613817",
0x08a55ab3,"MainField_Item_Mushroom_D_746781315",
0xf1d0dab5,"MainField_Item_Mushroom_D_747623510",
0x113af64b,"MainField_Item_Mushroom_D_749788643",
0xbdf2f10f,"MainField_Item_Mushroom_D_750042837",
0x40de8202,"MainField_Item_Mushroom_D_754097050",
0x44ee07d3,"MainField_Item_Mushroom_D_754126050",
0x582bb8b8,"MainField_Item_Mushroom_D_754172065",
0x3cdae931,"MainField_Item_Mushroom_D_754524379",
0xcfb645d2,"MainField_Item_Mushroom_D_756163395",
0x721953fc,"MainField_Item_Mushroom_D_757410229",
0x2535e026,"MainField_Item_Mushroom_D_758643654",
0x370e9cec,"MainField_Item_Mushroom_D_762964183",
0xefb4a2bf,"MainField_Item_Mushroom_D_763360659",
0x3327a5cb,"MainField_Item_Mushroom_D_763773348",
0xe3512b48,"MainField_Item_Mushroom_D_766546590",
0x27e36cd6,"MainField_Item_Mushroom_D_76960361",
0xcbf79412,"MainField_Item_Mushroom_D_770021718",
0x84886ef4,"MainField_Item_Mushroom_D_771880557",
0xdd84c022,"MainField_Item_Mushroom_D_772214599",
0x5cdbe5d4,"MainField_Item_Mushroom_D_773726124",
0x31aeb5c8,"MainField_Item_Mushroom_D_774366712",
0xff9d7553,"MainField_Item_Mushroom_D_777161526",
0xa07f2181,"MainField_Item_Mushroom_D_780791932",
0x4caff9bd,"MainField_Item_Mushroom_D_78148293",
0x3a0d8314,"MainField_Item_Mushroom_D_781828451",
0x43cfe3ff,"MainField_Item_Mushroom_D_784011016",
0x65384e1a,"MainField_Item_Mushroom_D_786987197",
0x8347669d,"MainField_Item_Mushroom_D_790518661",
0xb05e2a12,"MainField_Item_Mushroom_D_791727553",
0x1484ab88,"MainField_Item_Mushroom_D_793493227",
0xa9d9c8b8,"MainField_Item_Mushroom_D_793813287",
0xc61b7e61,"MainField_Item_Mushroom_D_794178013",
0x8cd91633,"MainField_Item_Mushroom_D_795907048",
0x79dc3b23,"MainField_Item_Mushroom_D_798402705",
0xb26de102,"MainField_Item_Mushroom_D_79948916",
0xfa8881a4,"MainField_Item_Mushroom_D_804040420",
0xc1ad546b,"MainField_Item_Mushroom_D_805026300",
0xe1939dfb,"MainField_Item_Mushroom_D_806475283",
0xca5ec9fc,"MainField_Item_Mushroom_D_806514317",
0x35437db9,"MainField_Item_Mushroom_D_8090314",
0xd94f4d24,"MainField_Item_Mushroom_D_809083195",
0x40604498,"MainField_Item_Mushroom_D_811086512",
0xd357dce6,"MainField_Item_Mushroom_D_811459960",
0xe4d98064,"MainField_Item_Mushroom_D_814171390",
0x1ea0568e,"MainField_Item_Mushroom_D_81619096",
0x4c28b8c1,"MainField_Item_Mushroom_D_81665556",
0x3de7fdb1,"MainField_Item_Mushroom_D_817193986",
0x96ff9798,"MainField_Item_Mushroom_D_817736169",
0x912833a8,"MainField_Item_Mushroom_D_81968429",
0xf1260ca1,"MainField_Item_Mushroom_D_821758280",
0x84e25b64,"MainField_Item_Mushroom_D_826733765",
0x273ed062,"MainField_Item_Mushroom_D_828032705",
0x1dfcab79,"MainField_Item_Mushroom_D_83067882",
0xe956ac3e,"MainField_Item_Mushroom_D_831181693",
0x46201920,"MainField_Item_Mushroom_D_831851275",
0x79aeab21,"MainField_Item_Mushroom_D_832446153",
0xd80770d9,"MainField_Item_Mushroom_D_83295917",
0xe4223fff,"MainField_Item_Mushroom_D_833540320",
0xbcc9187e,"MainField_Item_Mushroom_D_83380134",
0x7a2473a3,"MainField_Item_Mushroom_D_840147456",
0xd481aa2a,"MainField_Item_Mushroom_D_847614471",
0x767689e0,"MainField_Item_Mushroom_D_848231366",
0x4a2bdb18,"MainField_Item_Mushroom_D_848831738",
0xe71ca40e,"MainField_Item_Mushroom_D_849199617",
0xf90a451c,"MainField_Item_Mushroom_D_850143931",
0x32d6d3e3,"MainField_Item_Mushroom_D_852304938",
0xc9f71470,"MainField_Item_Mushroom_D_852817056",
0x85ea4c69,"MainField_Item_Mushroom_D_853049362",
0x76aab5b1,"MainField_Item_Mushroom_D_853292669",
0x1974bd3b,"MainField_Item_Mushroom_D_855301216",
0xf9cc3fc0,"MainField_Item_Mushroom_D_855718484",
0x1d911b81,"MainField_Item_Mushroom_D_856881703",
0xebd9a0d0,"MainField_Item_Mushroom_D_858998089",
0x7aca456c,"MainField_Item_Mushroom_D_85904903",
0x550aa155,"MainField_Item_Mushroom_D_8592117",
0x938791df,"MainField_Item_Mushroom_D_863394972",
0x5349b933,"MainField_Item_Mushroom_D_868503769",
0x04ef42c7,"MainField_Item_Mushroom_D_870392902",
0xc786203e,"MainField_Item_Mushroom_D_870900138",
0xa4b750fc,"MainField_Item_Mushroom_D_872053369",
0xb5c4de68,"MainField_Item_Mushroom_D_872378641",
0xacd80d32,"MainField_Item_Mushroom_D_874022429",
0x576bf057,"MainField_Item_Mushroom_D_87405882",
0xa77e3254,"MainField_Item_Mushroom_D_877227551",
0x43a91bd2,"MainField_Item_Mushroom_D_877767529",
0x8444a6da,"MainField_Item_Mushroom_D_878701094",
0x320f0589,"MainField_Item_Mushroom_D_880815584",
0x2fe53ff5,"MainField_Item_Mushroom_D_884960415",
0x61dede8e,"MainField_Item_Mushroom_D_886194087",
0x66581d3b,"MainField_Item_Mushroom_D_8873651",
0x525a83b6,"MainField_Item_Mushroom_D_890604798",
0xd45041f8,"MainField_Item_Mushroom_D_894178397",
0x08d0e3b8,"MainField_Item_Mushroom_D_898530043",
0x7315f17b,"MainField_Item_Mushroom_D_899682309",
0xcf81a359,"MainField_Item_Mushroom_D_902211193",
0x20d49563,"MainField_Item_Mushroom_D_904868023",
0x4ace9485,"MainField_Item_Mushroom_D_906645457",
0x0f8c661f,"MainField_Item_Mushroom_D_914136974",
0xeea95a3b,"MainField_Item_Mushroom_D_917211516",
0x86db1a4e,"MainField_Item_Mushroom_D_920507177",
0xe78a2e23,"MainField_Item_Mushroom_D_920636067",
0xa045114a,"MainField_Item_Mushroom_D_921452040",
0x259ddcfa,"MainField_Item_Mushroom_D_92904963",
0x336db54e,"MainField_Item_Mushroom_D_9299542",
0x5ffb29d7,"MainField_Item_Mushroom_D_935907257",
0xf200388c,"MainField_Item_Mushroom_D_93757064",
0x73ce44e1,"MainField_Item_Mushroom_D_937684917",
0xe4e8ae51,"MainField_Item_Mushroom_D_938076018",
0x2d2a4575,"MainField_Item_Mushroom_D_938602279",
0x9461bf2a,"MainField_Item_Mushroom_D_939424071",
0x56a4e315,"MainField_Item_Mushroom_D_945362047",
0xd2f41912,"MainField_Item_Mushroom_D_950873900",
0xb6e70c58,"MainField_Item_Mushroom_D_953731616",
0x945c60d3,"MainField_Item_Mushroom_D_955112151",
0xb440c213,"MainField_Item_Mushroom_D_956260217",
0xf44e5635,"MainField_Item_Mushroom_D_957409478",
0x3f046e8c,"MainField_Item_Mushroom_D_962926063",
0x55b7ed76,"MainField_Item_Mushroom_D_963668657",
0x23a060dd,"MainField_Item_Mushroom_D_967820748",
0x2e1cc2f7,"MainField_Item_Mushroom_D_970120693",
0x0d22ad0f,"MainField_Item_Mushroom_D_973707111",
0x67b0c957,"MainField_Item_Mushroom_D_975711654",
0x39581c78,"MainField_Item_Mushroom_D_976454683",
0x7d79cb17,"MainField_Item_Mushroom_D_978302881",
0xbb59b6f3,"MainField_Item_Mushroom_D_981022736",
0x77f08020,"MainField_Item_Mushroom_D_982171434",
0xe955e14a,"MainField_Item_Mushroom_D_985084255",
0x463b525b,"MainField_Item_Mushroom_D_985871743",
0xdccfe8cf,"MainField_Item_Mushroom_D_98726405",
0xbd4d9c16,"MainField_Item_Mushroom_D_987418745",
0x9740581a,"MainField_Item_Mushroom_D_987925679",
0x49298070,"MainField_Item_Mushroom_D_989213789",
0x310aefc2,"MainField_Item_Mushroom_D_992615685",
0xbdc422ef,"MainField_Item_Mushroom_D_994992900",
0x09e7db03,"MainField_Item_Mushroom_D_995124776",
0xca8d628c,"MainField_Item_Mushroom_D_995474743",
0x6bd23068,"MainField_Item_Mushroom_D_995587955",
0x768bc260,"MainField_Item_Mushroom_D_998263760",
0x152cef6f,"MainField_Item_Mushroom_D_99828804",
0x18291a2f,"MainField_Item_Mushroom_E_1009329122",
0x29313b07,"MainField_Item_Mushroom_E_1018356548",
0x1dcbad41,"MainField_Item_Mushroom_E_1018395913",
0xd9bfe3c2,"MainField_Item_Mushroom_E_1023007906",
0x22bafbdd,"MainField_Item_Mushroom_E_1026119987",
0x9a8571ce,"MainField_Item_Mushroom_E_102953760",
0xeecc560b,"MainField_Item_Mushroom_E_103258028",
0x6bda2643,"MainField_Item_Mushroom_E_1048093545",
0x1a618e8f,"MainField_Item_Mushroom_E_1052612287",
0xeb13817a,"MainField_Item_Mushroom_E_1076879005",
0xbbb38b4b,"MainField_Item_Mushroom_E_1079015747",
0x3c8db2f7,"MainField_Item_Mushroom_E_1095233466",
0x65e842de,"MainField_Item_Mushroom_E_1102909011",
0x11f28250,"MainField_Item_Mushroom_E_1108028268",
0x2fdcfa01,"MainField_Item_Mushroom_E_1125260296",
0x116396f0,"MainField_Item_Mushroom_E_1134652414",
0xac7951fb,"MainField_Item_Mushroom_E_1139028184",
0x57d98547,"MainField_Item_Mushroom_E_1142546300",
0xf519f92e,"MainField_Item_Mushroom_E_1158764683",
0x01521c06,"MainField_Item_Mushroom_E_1165991236",
0xd9594cbb,"MainField_Item_Mushroom_E_1167104235",
0xdfd48ca2,"MainField_Item_Mushroom_E_1181262059",
0xa85bb3a6,"MainField_Item_Mushroom_E_1210369102",
0x2e2c7c54,"MainField_Item_Mushroom_E_1211550538",
0x74731bf1,"MainField_Item_Mushroom_E_1229265323",
0xe8508ec4,"MainField_Item_Mushroom_E_1261310836",
0x84021a94,"MainField_Item_Mushroom_E_1263271832",
0x0f047220,"MainField_Item_Mushroom_E_1263841899",
0xd9001b92,"MainField_Item_Mushroom_E_1266834999",
0x918ec737,"MainField_Item_Mushroom_E_1278752230",
0x38ee5438,"MainField_Item_Mushroom_E_1295842728",
0xa10b5823,"MainField_Item_Mushroom_E_1302553935",
0x25bf6ba6,"MainField_Item_Mushroom_E_1334305104",
0x1e82becc,"MainField_Item_Mushroom_E_134728948",
0x01a2b546,"MainField_Item_Mushroom_E_1353402095",
0xbf1806cd,"MainField_Item_Mushroom_E_1356280908",
0x6630e753,"MainField_Item_Mushroom_E_1356665280",
0x967ca279,"MainField_Item_Mushroom_E_1370534706",
0xf9ae30d3,"MainField_Item_Mushroom_E_1374030099",
0x3a812671,"MainField_Item_Mushroom_E_1379185517",
0xae768c0d,"MainField_Item_Mushroom_E_1380456057",
0xdc4594c1,"MainField_Item_Mushroom_E_1381781926",
0x7ed6afdf,"MainField_Item_Mushroom_E_1388450686",
0x120775a4,"MainField_Item_Mushroom_E_139183378",
0x1c5ff34b,"MainField_Item_Mushroom_E_1403782065",
0x52de0be8,"MainField_Item_Mushroom_E_14105662",
0x39ed4606,"MainField_Item_Mushroom_E_1418106376",
0xe19d422b,"MainField_Item_Mushroom_E_144108820",
0x64782fa7,"MainField_Item_Mushroom_E_1445660240",
0x5924b9ed,"MainField_Item_Mushroom_E_1454862195",
0xb60a10be,"MainField_Item_Mushroom_E_1462292113",
0x2e472bcb,"MainField_Item_Mushroom_E_1462732604",
0x1fac4d93,"MainField_Item_Mushroom_E_1465673501",
0x9fdf6350,"MainField_Item_Mushroom_E_1467998747",
0x82dc1862,"MainField_Item_Mushroom_E_1479260787",
0x441a486e,"MainField_Item_Mushroom_E_1481211558",
0x7d0af0bd,"MainField_Item_Mushroom_E_1485489091",
0x520bf1ac,"MainField_Item_Mushroom_E_1496544746",
0xcae654b8,"MainField_Item_Mushroom_E_1501830839",
0xb5b987cd,"MainField_Item_Mushroom_E_1502775061",
0x8a155f80,"MainField_Item_Mushroom_E_1508593844",
0x61a6ffde,"MainField_Item_Mushroom_E_1510756088",
0x42c1a99a,"MainField_Item_Mushroom_E_15259800",
0x8c7727ee,"MainField_Item_Mushroom_E_1535340062",
0x4c24512b,"MainField_Item_Mushroom_E_1536948369",
0x2461cc8a,"MainField_Item_Mushroom_E_1538425373",
0x057b9469,"MainField_Item_Mushroom_E_1539088258",
0xf1596247,"MainField_Item_Mushroom_E_1542312348",
0x0c5e2643,"MainField_Item_Mushroom_E_1547351565",
0x286cce7a,"MainField_Item_Mushroom_E_1558316639",
0x83a2c773,"MainField_Item_Mushroom_E_1568115034",
0x8219b44d,"MainField_Item_Mushroom_E_157162278",
0x5d9a9dee,"MainField_Item_Mushroom_E_1586768482",
0x9a4062e4,"MainField_Item_Mushroom_E_1611534380",
0x6fceab08,"MainField_Item_Mushroom_E_1616150119",
0x731bd257,"MainField_Item_Mushroom_E_1616382319",
0x2748a2b2,"MainField_Item_Mushroom_E_1630690573",
0xc47ee0ae,"MainField_Item_Mushroom_E_1641007973",
0x58a93142,"MainField_Item_Mushroom_E_1641521129",
0xdaa1b2a5,"MainField_Item_Mushroom_E_1642258838",
0xc8760c30,"MainField_Item_Mushroom_E_164503549",
0xd369902b,"MainField_Item_Mushroom_E_1653674453",
0x8e61f152,"MainField_Item_Mushroom_E_1656124728",
0x2351a8ce,"MainField_Item_Mushroom_E_1656771933",
0x0c2ef65d,"MainField_Item_Mushroom_E_1669334537",
0xa142e663,"MainField_Item_Mushroom_E_1674886600",
0x213f3fa3,"MainField_Item_Mushroom_E_1675448005",
0x439b9c64,"MainField_Item_Mushroom_E_1689226689",
0x98daba1e,"MainField_Item_Mushroom_E_1691848468",
0x640a896d,"MainField_Item_Mushroom_E_1720418412",
0xf8c0e102,"MainField_Item_Mushroom_E_1730481594",
0x4ecbd473,"MainField_Item_Mushroom_E_1735207012",
0xf132108e,"MainField_Item_Mushroom_E_174076675",
0x4e382843,"MainField_Item_Mushroom_E_1742269583",
0x2cd847e8,"MainField_Item_Mushroom_E_1757767549",
0x989d2199,"MainField_Item_Mushroom_E_1760059331",
0x5cef4c02,"MainField_Item_Mushroom_E_1760262321",
0x0085a510,"MainField_Item_Mushroom_E_1761129414",
0xc17d313a,"MainField_Item_Mushroom_E_1770054979",
0x59173e00,"MainField_Item_Mushroom_E_1773407898",
0x8960f561,"MainField_Item_Mushroom_E_1773759227",
0x3ec97c06,"MainField_Item_Mushroom_E_1781152160",
0x4f075b2f,"MainField_Item_Mushroom_E_1786954769",
0x3b4a47ba,"MainField_Item_Mushroom_E_179572097",
0x838e8985,"MainField_Item_Mushroom_E_1798841285",
0x0245c3a8,"MainField_Item_Mushroom_E_1801022249",
0xdce79e99,"MainField_Item_Mushroom_E_1807892492",
0x8a3a69fb,"MainField_Item_Mushroom_E_1814479542",
0xf130a7fd,"MainField_Item_Mushroom_E_1819834757",
0xac3601ed,"MainField_Item_Mushroom_E_1836036597",
0xce8f273f,"MainField_Item_Mushroom_E_1846573537",
0x12f2ea1e,"MainField_Item_Mushroom_E_185027194",
0x7d8f05fd,"MainField_Item_Mushroom_E_1850863334",
0x26196c86,"MainField_Item_Mushroom_E_188081202",
0x6915aca2,"MainField_Item_Mushroom_E_1897630093",
0xfe62b698,"MainField_Item_Mushroom_E_1901188714",
0x8e56640f,"MainField_Item_Mushroom_E_1912940209",
0x34030c1d,"MainField_Item_Mushroom_E_1913856433",
0xc784e0bd,"MainField_Item_Mushroom_E_191452382",
0x0582b2dc,"MainField_Item_Mushroom_E_1946813031",
0xa2f61d19,"MainField_Item_Mushroom_E_1949228696",
0x8492d5ad,"MainField_Item_Mushroom_E_1953235381",
0x8800334d,"MainField_Item_Mushroom_E_196051561",
0xf296683f,"MainField_Item_Mushroom_E_1966273188",
0x9500ba29,"MainField_Item_Mushroom_E_1970763319",
0x1f5b1117,"MainField_Item_Mushroom_E_1978149918",
0x196a070a,"MainField_Item_Mushroom_E_1983171725",
0xa647059f,"MainField_Item_Mushroom_E_2000699528",
0xadd991d7,"MainField_Item_Mushroom_E_2002583158",
0x66e5f55e,"MainField_Item_Mushroom_E_2031263772",
0x1521e139,"MainField_Item_Mushroom_E_2036286203",
0xcaedc873,"MainField_Item_Mushroom_E_2036879175",
0xb11776e2,"MainField_Item_Mushroom_E_2046700105",
0x50316e3c,"MainField_Item_Mushroom_E_2070914947",
0xbdd6da6a,"MainField_Item_Mushroom_E_2087882657",
0x5a20d631,"MainField_Item_Mushroom_E_2106765",
0x6b8556fa,"MainField_Item_Mushroom_E_2108710835",
0x6d5868ac,"MainField_Item_Mushroom_E_2108898583",
0x3318ddce,"MainField_Item_Mushroom_E_2109556079",
0xb27e300d,"MainField_Item_Mushroom_E_211114296",
0xaf08f9e8,"MainField_Item_Mushroom_E_2116921463",
0x0556d866,"MainField_Item_Mushroom_E_2119943089",
0x363b2f2e,"MainField_Item_Mushroom_E_2133712986",
0x1a74d3dd,"MainField_Item_Mushroom_E_2134394193",
0x4937a9ea,"MainField_Item_Mushroom_E_2149219109",
0x5fc87d58,"MainField_Item_Mushroom_E_2163563074",
0x9fa89c7d,"MainField_Item_Mushroom_E_2163897575",
0x26071187,"MainField_Item_Mushroom_E_2164859019",
0xe79e97b3,"MainField_Item_Mushroom_E_216601662",
0x7e14627e,"MainField_Item_Mushroom_E_2171050324",
0x9067d4a7,"MainField_Item_Mushroom_E_2178555838",
0x4ea1a75d,"MainField_Item_Mushroom_E_2180475723",
0x9db3d05e,"MainField_Item_Mushroom_E_2196534764",
0x13cff4ef,"MainField_Item_Mushroom_E_2197597365",
0xaa6fb6d6,"MainField_Item_Mushroom_E_2244058532",
0x7a46bf36,"MainField_Item_Mushroom_E_2245974676",
0x01cae140,"MainField_Item_Mushroom_E_2247332092",
0xc5f0834b,"MainField_Item_Mushroom_E_2253494881",
0x21452013,"MainField_Item_Mushroom_E_2254942116",
0xa3416b5f,"MainField_Item_Mushroom_E_2259564371",
0x517e1a8b,"MainField_Item_Mushroom_E_2261879500",
0xe2c37625,"MainField_Item_Mushroom_E_2271434251",
0xe460ff03,"MainField_Item_Mushroom_E_2272729658",
0x3dcdba21,"MainField_Item_Mushroom_E_2273316542",
0x427782e3,"MainField_Item_Mushroom_E_2288384742",
0x6fe088f3,"MainField_Item_Mushroom_E_229313546",
0x9b86033c,"MainField_Item_Mushroom_E_2302767659",
0x28f7fe6f,"MainField_Item_Mushroom_E_2309715323",
0x5c502745,"MainField_Item_Mushroom_E_2317445751",
0x3f87b801,"MainField_Item_Mushroom_E_2342923244",
0xede9540d,"MainField_Item_Mushroom_E_2375760930",
0xcb6eb13d,"MainField_Item_Mushroom_E_2376747926",
0x0cd380c5,"MainField_Item_Mushroom_E_2377943659",
0x966afd22,"MainField_Item_Mushroom_E_2378570593",
0xf0143def,"MainField_Item_Mushroom_E_2385978438",
0x30c1a6b4,"MainField_Item_Mushroom_E_2407443002",
0x968de556,"MainField_Item_Mushroom_E_2408037110",
0xda5c58f2,"MainField_Item_Mushroom_E_2412144583",
0x8b1b158d,"MainField_Item_Mushroom_E_2412158358",
0xa8b71383,"MainField_Item_Mushroom_E_2418777955",
0x0572d533,"MainField_Item_Mushroom_E_2426369971",
0x71e939f1,"MainField_Item_Mushroom_E_2429622980",
0xb84a9b67,"MainField_Item_Mushroom_E_2436989887",
0xc7d9735b,"MainField_Item_Mushroom_E_2443594980",
0x90314a17,"MainField_Item_Mushroom_E_2448840960",
0x861fdf05,"MainField_Item_Mushroom_E_2452493937",
0x737520b2,"MainField_Item_Mushroom_E_2455287002",
0x4c7bb8e9,"MainField_Item_Mushroom_E_2465548098",
0xf86167a4,"MainField_Item_Mushroom_E_2470176584",
0xfeb1d97a,"MainField_Item_Mushroom_E_2487069050",
0x191a72dc,"MainField_Item_Mushroom_E_2491698326",
0x6f4c479d,"MainField_Item_Mushroom_E_2503995779",
0x8f6cce73,"MainField_Item_Mushroom_E_2504962408",
0xb872ad39,"MainField_Item_Mushroom_E_2520343702",
0x001362c9,"MainField_Item_Mushroom_E_2520711399",
0x43296afe,"MainField_Item_Mushroom_E_2522240331",
0x4c2ec89e,"MainField_Item_Mushroom_E_2522616586",
0xc0138ff4,"MainField_Item_Mushroom_E_2524703804",
0xe8cb30a1,"MainField_Item_Mushroom_E_253919548",
0xd6afb72a,"MainField_Item_Mushroom_E_2549562036",
0x52bf1eee,"MainField_Item_Mushroom_E_2552182027",
0x8715f973,"MainField_Item_Mushroom_E_2557736304",
0x81bcd03f,"MainField_Item_Mushroom_E_2560923109",
0x4adfa20e,"MainField_Item_Mushroom_E_2603369625",
0xe3e90ecf,"MainField_Item_Mushroom_E_2615537166",
0xfa646858,"MainField_Item_Mushroom_E_2621422710",
0xcc8b917e,"MainField_Item_Mushroom_E_2625910913",
0xbb3cc433,"MainField_Item_Mushroom_E_2638129617",
0x4620fc95,"MainField_Item_Mushroom_E_2642131469",
0xf3b6b74a,"MainField_Item_Mushroom_E_2649540800",
0x80896ebf,"MainField_Item_Mushroom_E_2652329256",
0xf96be977,"MainField_Item_Mushroom_E_2663850440",
0x3f1958d9,"MainField_Item_Mushroom_E_2670853921",
0x21b5d061,"MainField_Item_Mushroom_E_2670951279",
0x2d4c061b,"MainField_Item_Mushroom_E_2673814908",
0x516ed721,"MainField_Item_Mushroom_E_2676619806",
0x88f6c78f,"MainField_Item_Mushroom_E_2682703446",
0xc22b8566,"MainField_Item_Mushroom_E_2686793171",
0x9158e0ae,"MainField_Item_Mushroom_E_2689106258",
0x030bc3d8,"MainField_Item_Mushroom_E_2689350421",
0x3dd29e62,"MainField_Item_Mushroom_E_2696716451",
0x7a1658df,"MainField_Item_Mushroom_E_2707348968",
0x8c9c020c,"MainField_Item_Mushroom_E_2726664282",
0xdff3b20f,"MainField_Item_Mushroom_E_2728613251",
0xd8597f0a,"MainField_Item_Mushroom_E_2730850430",
0x315c64ac,"MainField_Item_Mushroom_E_273166512",
0x777dcdc3,"MainField_Item_Mushroom_E_2732757109",
0xbdd7abb0,"MainField_Item_Mushroom_E_2739200037",
0x0ff138c3,"MainField_Item_Mushroom_E_2741935930",
0x5e0df3eb,"MainField_Item_Mushroom_E_2752205695",
0xc1d458d5,"MainField_Item_Mushroom_E_2773354140",
0x95c339a8,"MainField_Item_Mushroom_E_2803047793",
0xd6cef83e,"MainField_Item_Mushroom_E_2816384236",
0x9528e90b,"MainField_Item_Mushroom_E_2816990907",
0x390a8541,"MainField_Item_Mushroom_E_2817369032",
0x4079742e,"MainField_Item_Mushroom_E_2842711393",
0xddc9de3f,"MainField_Item_Mushroom_E_2845260093",
0x5a5442fd,"MainField_Item_Mushroom_E_2846109042",
0xb3da8a6d,"MainField_Item_Mushroom_E_286279945",
0x96070103,"MainField_Item_Mushroom_E_2878283225",
0xd8bc154c,"MainField_Item_Mushroom_E_288094582",
0x7c52fa29,"MainField_Item_Mushroom_E_2885417578",
0xce36b288,"MainField_Item_Mushroom_E_2892429035",
0x5db480f6,"MainField_Item_Mushroom_E_2896121976",
0x3f7ca0d3,"MainField_Item_Mushroom_E_2899058592",
0x2dce510b,"MainField_Item_Mushroom_E_2922258986",
0xf1dc813a,"MainField_Item_Mushroom_E_2930024149",
0xc18b8473,"MainField_Item_Mushroom_E_2946367382",
0xb1e66b3a,"MainField_Item_Mushroom_E_2947699588",
0x4e5e9fdf,"MainField_Item_Mushroom_E_2948053433",
0x2f7fafba,"MainField_Item_Mushroom_E_2967751194",
0xd402951c,"MainField_Item_Mushroom_E_2969475108",
0x0ea6b802,"MainField_Item_Mushroom_E_2970865867",
0x772ac04c,"MainField_Item_Mushroom_E_2976034721",
0x0f0a4a79,"MainField_Item_Mushroom_E_2991022016",
0x492af1b7,"MainField_Item_Mushroom_E_3001076523",
0xcf92e7c7,"MainField_Item_Mushroom_E_3002274043",
0x76392fb2,"MainField_Item_Mushroom_E_3012211975",
0x13e44228,"MainField_Item_Mushroom_E_3012922215",
0xc1242685,"MainField_Item_Mushroom_E_3013328594",
0x5d4749c9,"MainField_Item_Mushroom_E_3016845408",
0x06553859,"MainField_Item_Mushroom_E_3043167081",
0x30a63c53,"MainField_Item_Mushroom_E_3046552975",
0x85c018c3,"MainField_Item_Mushroom_E_3089542010",
0x54a843de,"MainField_Item_Mushroom_E_3093041157",
0x7c8ee16e,"MainField_Item_Mushroom_E_3095215776",
0xf62ca625,"MainField_Item_Mushroom_E_3100165321",
0x1742712b,"MainField_Item_Mushroom_E_3100737141",
0x3e918899,"MainField_Item_Mushroom_E_3109143829",
0x5bb279bf,"MainField_Item_Mushroom_E_3109160497",
0x525656be,"MainField_Item_Mushroom_E_3118884671",
0x82b1c745,"MainField_Item_Mushroom_E_3148532797",
0x9d7d954e,"MainField_Item_Mushroom_E_315048585",
0xcc666316,"MainField_Item_Mushroom_E_3152237874",
0xe108ff13,"MainField_Item_Mushroom_E_3162513451",
0xf3a96f27,"MainField_Item_Mushroom_E_3170244155",
0x34ce1c9f,"MainField_Item_Mushroom_E_3180799497",
0xfdbb6e43,"MainField_Item_Mushroom_E_3209924078",
0x1585eaf7,"MainField_Item_Mushroom_E_3217740108",
0xbe2e20b7,"MainField_Item_Mushroom_E_3218223503",
0x45b59e85,"MainField_Item_Mushroom_E_3263840164",
0xd47d7b7d,"MainField_Item_Mushroom_E_3264373824",
0x9081fd37,"MainField_Item_Mushroom_E_3281188161",
0xc6fb72e2,"MainField_Item_Mushroom_E_3289519315",
0xa83ae6b8,"MainField_Item_Mushroom_E_3295424005",
0x553b2218,"MainField_Item_Mushroom_E_3302866115",
0x14dd4eae,"MainField_Item_Mushroom_E_3320314813",
0xc41d3f0b,"MainField_Item_Mushroom_E_3325478777",
0xef481c9e,"MainField_Item_Mushroom_E_332589744",
0x8a874fa5,"MainField_Item_Mushroom_E_3331193607",
0x1a0c1220,"MainField_Item_Mushroom_E_3334259201",
0x91e46ac4,"MainField_Item_Mushroom_E_3334745498",
0x8a260bf2,"MainField_Item_Mushroom_E_3340096367",
0xc4059064,"MainField_Item_Mushroom_E_3353574028",
0x96f61770,"MainField_Item_Mushroom_E_3355164023",
0x0b6b022b,"MainField_Item_Mushroom_E_336462690",
0x913802e0,"MainField_Item_Mushroom_E_3378146635",
0xf4a82ced,"MainField_Item_Mushroom_E_3389130290",
0xeab244d6,"MainField_Item_Mushroom_E_338989737",
0x29a5f30d,"MainField_Item_Mushroom_E_3407305826",
0x66f95977,"MainField_Item_Mushroom_E_3410863931",
0x82436426,"MainField_Item_Mushroom_E_3411382958",
0xf6d9635e,"MainField_Item_Mushroom_E_3411416256",
0xe9a54877,"MainField_Item_Mushroom_E_3422537328",
0x661a9235,"MainField_Item_Mushroom_E_3423781508",
0x39861407,"MainField_Item_Mushroom_E_3438087072",
0x62e76f3f,"MainField_Item_Mushroom_E_3439123660",
0x0498da7b,"MainField_Item_Mushroom_E_3440824341",
0xc198e0f4,"MainField_Item_Mushroom_E_3451300869",
0x71eb6d8d,"MainField_Item_Mushroom_E_3461691047",
0x92639396,"MainField_Item_Mushroom_E_3462268674",
0x50acf293,"MainField_Item_Mushroom_E_3463222033",
0xf5e790aa,"MainField_Item_Mushroom_E_3464662662",
0xe1a5b0fa,"MainField_Item_Mushroom_E_3468798570",
0x1be75280,"MainField_Item_Mushroom_E_3476691003",
0x487855e3,"MainField_Item_Mushroom_E_3478392226",
0xa2ec712f,"MainField_Item_Mushroom_E_3480882742",
0xc691761e,"MainField_Item_Mushroom_E_3490586045",
0xa0432135,"MainField_Item_Mushroom_E_3496977031",
0xc1426e82,"MainField_Item_Mushroom_E_3501863242",
0x18181df9,"MainField_Item_Mushroom_E_3505856790",
0x27c5266c,"MainField_Item_Mushroom_E_3518550626",
0x08c17ce2,"MainField_Item_Mushroom_E_3530730125",
0xca024708,"MainField_Item_Mushroom_E_3532630526",
0x39affd9b,"MainField_Item_Mushroom_E_3540455874",
0xedcc48a4,"MainField_Item_Mushroom_E_3540616096",
0x24a2bd9c,"MainField_Item_Mushroom_E_3551485584",
0xf243ba0d,"MainField_Item_Mushroom_E_3561915161",
0xd7996c72,"MainField_Item_Mushroom_E_3570093535",
0x07ee282a,"MainField_Item_Mushroom_E_3583314593",
0xd82f9867,"MainField_Item_Mushroom_E_3600310032",
0x6d1450c7,"MainField_Item_Mushroom_E_3601033879",
0xedd35b5b,"MainField_Item_Mushroom_E_3612019587",
0xf30c0065,"MainField_Item_Mushroom_E_3621374051",
0x31c7cf5b,"MainField_Item_Mushroom_E_3628554116",
0x1794a603,"MainField_Item_Mushroom_E_3634024192",
0x9a76b3ed,"MainField_Item_Mushroom_E_3634585341",
0x13d3eb6b,"MainField_Item_Mushroom_E_3636196681",
0x93072bff,"MainField_Item_Mushroom_E_3640376195",
0xe84bb1e6,"MainField_Item_Mushroom_E_3640696379",
0x4f46e7dc,"MainField_Item_Mushroom_E_3642968256",
0x1c936a3c,"MainField_Item_Mushroom_E_3651786839",
0x6ae84f32,"MainField_Item_Mushroom_E_3652879724",
0x80ec2f7a,"MainField_Item_Mushroom_E_3664021073",
0x8c1a3241,"MainField_Item_Mushroom_E_3664956290",
0xbcc1d451,"MainField_Item_Mushroom_E_367106298",
0x4f77c382,"MainField_Item_Mushroom_E_3673191027",
0xfaa121b2,"MainField_Item_Mushroom_E_3683716837",
0x2fb0e040,"MainField_Item_Mushroom_E_3686092007",
0xb2936108,"MainField_Item_Mushroom_E_3700757821",
0x27df88ef,"MainField_Item_Mushroom_E_3704595508",
0x965c0381,"MainField_Item_Mushroom_E_3716252572",
0x0e6c5c75,"MainField_Item_Mushroom_E_3754534303",
0xbde11043,"MainField_Item_Mushroom_E_3758356149",
0x119172f7,"MainField_Item_Mushroom_E_3769475202",
0x58d6b1ef,"MainField_Item_Mushroom_E_3770156436",
0x5d51b9ec,"MainField_Item_Mushroom_E_3778546260",
0x578ec766,"MainField_Item_Mushroom_E_3779095125",
0x6c4af604,"MainField_Item_Mushroom_E_3788709768",
0xd1fd3ef0,"MainField_Item_Mushroom_E_3790809406",
0x29890473,"MainField_Item_Mushroom_E_3790975265",
0x01a32387,"MainField_Item_Mushroom_E_3812268524",
0xbb32c8e9,"MainField_Item_Mushroom_E_3813365345",
0xee9bd054,"MainField_Item_Mushroom_E_3819634980",
0xccce2923,"MainField_Item_Mushroom_E_3828795616",
0x1324cda0,"MainField_Item_Mushroom_E_3831500304",
0xf0cb73fd,"MainField_Item_Mushroom_E_3832465923",
0x582b2b1e,"MainField_Item_Mushroom_E_383771582",
0xffe01075,"MainField_Item_Mushroom_E_3838710155",
0xf437de40,"MainField_Item_Mushroom_E_3842991549",
0x068a2617,"MainField_Item_Mushroom_E_3846955583",
0xf2101fef,"MainField_Item_Mushroom_E_3848336480",
0x71bb5f4e,"MainField_Item_Mushroom_E_3850970036",
0xef2587ea,"MainField_Item_Mushroom_E_3881746196",
0x3211082a,"MainField_Item_Mushroom_E_3887918552",
0x33db1d8b,"MainField_Item_Mushroom_E_3890376364",
0x7e073a60,"MainField_Item_Mushroom_E_3890445087",
0x49df5f9a,"MainField_Item_Mushroom_E_3892844171",
0xb5316e0e,"MainField_Item_Mushroom_E_3899668767",
0x25956ab1,"MainField_Item_Mushroom_E_3904360072",
0xcacf7a8c,"MainField_Item_Mushroom_E_392530226",
0xe99e0f4c,"MainField_Item_Mushroom_E_3933093992",
0xa3a068d7,"MainField_Item_Mushroom_E_3938090611",
0xd81f9a7c,"MainField_Item_Mushroom_E_3941647375",
0x4643e9a3,"MainField_Item_Mushroom_E_3946133625",
0x546fe8eb,"MainField_Item_Mushroom_E_3970622638",
0xf9db683f,"MainField_Item_Mushroom_E_3972895280",
0xb74df54d,"MainField_Item_Mushroom_E_3989964872",
0xe7c6843f,"MainField_Item_Mushroom_E_3998105729",
0xf7c6469d,"MainField_Item_Mushroom_E_4000910126",
0x8b09838a,"MainField_Item_Mushroom_E_4006319010",
0x22cdc675,"MainField_Item_Mushroom_E_4015029542",
0x9fed68a1,"MainField_Item_Mushroom_E_4020528700",
0x59e9cd7b,"MainField_Item_Mushroom_E_4020648380",
0xf8987f4e,"MainField_Item_Mushroom_E_4023119179",
0x383cd6aa,"MainField_Item_Mushroom_E_4023681682",
0xbf7cc57f,"MainField_Item_Mushroom_E_4028992254",
0xa781b809,"MainField_Item_Mushroom_E_4037138556",
0x778bd7e4,"MainField_Item_Mushroom_E_4048347818",
0x49d844ca,"MainField_Item_Mushroom_E_4050219675",
0x5a6211fd,"MainField_Item_Mushroom_E_4058140469",
0x51b0c779,"MainField_Item_Mushroom_E_409285644",
0x8c134254,"MainField_Item_Mushroom_E_4109940702",
0x0198d209,"MainField_Item_Mushroom_E_4136939077",
0x52ccb115,"MainField_Item_Mushroom_E_4153639806",
0xe957d973,"MainField_Item_Mushroom_E_4154308079",
0xe667b2b2,"MainField_Item_Mushroom_E_4158729176",
0xce9fbd84,"MainField_Item_Mushroom_E_41645673",
0xd9f10f95,"MainField_Item_Mushroom_E_4169116801",
0x9f499a57,"MainField_Item_Mushroom_E_4177070036",
0xcd72a2a8,"MainField_Item_Mushroom_E_4177445309",
0x8fc542fc,"MainField_Item_Mushroom_E_4179707981",
0x2919d8f6,"MainField_Item_Mushroom_E_419699496",
0xc19ad5fc,"MainField_Item_Mushroom_E_4197217073",
0x4102adc2,"MainField_Item_Mushroom_E_4202779841",
0x15b09e8b,"MainField_Item_Mushroom_E_4213403109",
0xe2760998,"MainField_Item_Mushroom_E_4229920501",
0xa6b0880f,"MainField_Item_Mushroom_E_423286040",
0x5b364288,"MainField_Item_Mushroom_E_4234289879",
0xf388f262,"MainField_Item_Mushroom_E_4237934729",
0xa3b7b178,"MainField_Item_Mushroom_E_4239281040",
0x5ca42b20,"MainField_Item_Mushroom_E_4243638391",
0x1986e000,"MainField_Item_Mushroom_E_4251318656",
0xf8028242,"MainField_Item_Mushroom_E_4255885241",
0xbb416a57,"MainField_Item_Mushroom_E_4274821305",
0xb6993157,"MainField_Item_Mushroom_E_4291028727",
0x63304613,"MainField_Item_Mushroom_E_435345718",
0x5d38b613,"MainField_Item_Mushroom_E_442644708",
0x809f40c6,"MainField_Item_Mushroom_E_442660934",
0xc09a238e,"MainField_Item_Mushroom_E_449409990",
0x573f0fa7,"MainField_Item_Mushroom_E_457164912",
0xc7357717,"MainField_Item_Mushroom_E_469584556",
0xf1c57c5a,"MainField_Item_Mushroom_E_470061830",
0xd2bf9a52,"MainField_Item_Mushroom_E_470978740",
0x0607d424,"MainField_Item_Mushroom_E_478769728",
0x8da1f626,"MainField_Item_Mushroom_E_481354177",
0xd5558c22,"MainField_Item_Mushroom_E_489549676",
0x05be2cc6,"MainField_Item_Mushroom_E_495123704",
0xb9d97746,"MainField_Item_Mushroom_E_502423361",
0xc6fd1c53,"MainField_Item_Mushroom_E_511205469",
0x9748e422,"MainField_Item_Mushroom_E_525099251",
0x71545a7c,"MainField_Item_Mushroom_E_525943635",
0xfc16d824,"MainField_Item_Mushroom_E_542857099",
0xacb080a2,"MainField_Item_Mushroom_E_55402127",
0xa1e3e3b9,"MainField_Item_Mushroom_E_58452251",
0x101831c3,"MainField_Item_Mushroom_E_597151346",
0xb2b2bc5d,"MainField_Item_Mushroom_E_608117134",
0xa8dfbb76,"MainField_Item_Mushroom_E_609864118",
0x4e77a300,"MainField_Item_Mushroom_E_624709864",
0xbcaf51f8,"MainField_Item_Mushroom_E_637898218",
0x156d4e35,"MainField_Item_Mushroom_E_638315057",
0x10fc456b,"MainField_Item_Mushroom_E_641535826",
0x63a0f39a,"MainField_Item_Mushroom_E_650658720",
0x6d540386,"MainField_Item_Mushroom_E_662905702",
0x8c425495,"MainField_Item_Mushroom_E_67786523",
0xe7ee9a81,"MainField_Item_Mushroom_E_678703345",
0x31311029,"MainField_Item_Mushroom_E_679256180",
0x09497a44,"MainField_Item_Mushroom_E_682406520",
0xc78a03de,"MainField_Item_Mushroom_E_692876843",
0x610d678b,"MainField_Item_Mushroom_E_696939736",
0xe3eb797f,"MainField_Item_Mushroom_E_701668804",
0x1ca84b2f,"MainField_Item_Mushroom_E_705442368",
0x860cd3f4,"MainField_Item_Mushroom_E_708226809",
0x299060ba,"MainField_Item_Mushroom_E_713193594",
0x06f42dc2,"MainField_Item_Mushroom_E_71705123",
0xef8b39b0,"MainField_Item_Mushroom_E_733202815",
0x3d24f583,"MainField_Item_Mushroom_E_739817937",
0x50b28da2,"MainField_Item_Mushroom_E_746872456",
0x92eb0556,"MainField_Item_Mushroom_E_756682265",
0xbf1461cd,"MainField_Item_Mushroom_E_758729255",
0xe85c4a35,"MainField_Item_Mushroom_E_760970711",
0x45f91efb,"MainField_Item_Mushroom_E_765026568",
0x68b6df70,"MainField_Item_Mushroom_E_767349433",
0x9a5cc400,"MainField_Item_Mushroom_E_771900632",
0x6bc8698b,"MainField_Item_Mushroom_E_777583359",
0x4ac19a4c,"MainField_Item_Mushroom_E_787713181",
0x4fd2b990,"MainField_Item_Mushroom_E_788796797",
0x6fed8e41,"MainField_Item_Mushroom_E_793254303",
0x28fe4b6d,"MainField_Item_Mushroom_E_798273979",
0x2cc457a6,"MainField_Item_Mushroom_E_809185209",
0x4a7be1a0,"MainField_Item_Mushroom_E_821241476",
0x0888e796,"MainField_Item_Mushroom_E_836630154",
0xce2660e0,"MainField_Item_Mushroom_E_841885508",
0xa7ce65c0,"MainField_Item_Mushroom_E_848750367",
0x52cf2bc4,"MainField_Item_Mushroom_E_852069401",
0xb7cedbb2,"MainField_Item_Mushroom_E_85618828",
0xedc67e2b,"MainField_Item_Mushroom_E_859057840",
0xa418b034,"MainField_Item_Mushroom_E_861908569",
0xae257385,"MainField_Item_Mushroom_E_896363867",
0x5ea64bd9,"MainField_Item_Mushroom_E_897118466",
0x30d9fca4,"MainField_Item_Mushroom_E_91071727",
0xd706642c,"MainField_Item_Mushroom_E_913548687",
0x8e0d4dde,"MainField_Item_Mushroom_E_929085478",
0xea2b3450,"MainField_Item_Mushroom_E_93635093",
0x318bc96f,"MainField_Item_Mushroom_E_94428373",
0x6a08f1d7,"MainField_Item_Mushroom_E_959966261",
0x9bf2f9bb,"MainField_Item_Mushroom_E_970393645",
0xdfa07160,"MainField_Item_Mushroom_E_991793150",
0xe9527195,"MainField_Item_Mushroom_F_1021747198",
0x9c9f89ba,"MainField_Item_Mushroom_F_1028045646",
0x764bbb14,"MainField_Item_Mushroom_F_1076555741",
0x8d745632,"MainField_Item_Mushroom_F_1078452953",
0x4a7e8f9d,"MainField_Item_Mushroom_F_1079256922",
0xbe95240c,"MainField_Item_Mushroom_F_1110185751",
0x322dc285,"MainField_Item_Mushroom_F_1110874007",
0x750d1819,"MainField_Item_Mushroom_F_1128250992",
0xa7f39324,"MainField_Item_Mushroom_F_1138097675",
0x666a56f6,"MainField_Item_Mushroom_F_1139016926",
0x55d044cc,"MainField_Item_Mushroom_F_1145304675",
0xdff82661,"MainField_Item_Mushroom_F_1149200679",
0x39070d16,"MainField_Item_Mushroom_F_1178771472",
0xff11f2f1,"MainField_Item_Mushroom_F_1179366447",
0x48f2f682,"MainField_Item_Mushroom_F_1183020911",
0x3722563e,"MainField_Item_Mushroom_F_1186928972",
0x5d8a0789,"MainField_Item_Mushroom_F_11884291",
0x43b25ad2,"MainField_Item_Mushroom_F_1225222625",
0x33d9ff08,"MainField_Item_Mushroom_F_1290322943",
0xc6710074,"MainField_Item_Mushroom_F_1295564383",
0x31001edf,"MainField_Item_Mushroom_F_1312724610",
0xae2703f8,"MainField_Item_Mushroom_F_1318193210",
0x4e472fe3,"MainField_Item_Mushroom_F_1337550300",
0x3cdcb254,"MainField_Item_Mushroom_F_1344464229",
0x71cc2011,"MainField_Item_Mushroom_F_1356584938",
0x328eba4b,"MainField_Item_Mushroom_F_1359034466",
0x0ba6fff1,"MainField_Item_Mushroom_F_1366776681",
0x257fd5d4,"MainField_Item_Mushroom_F_1366957615",
0x8a52dc58,"MainField_Item_Mushroom_F_1371810511",
0x55eb6466,"MainField_Item_Mushroom_F_1373376802",
0x8afe4907,"MainField_Item_Mushroom_F_1376312844",
0x08323416,"MainField_Item_Mushroom_F_1382798359",
0x33263e05,"MainField_Item_Mushroom_F_1396224511",
0xa0438b8b,"MainField_Item_Mushroom_F_1402893471",
0x423edc54,"MainField_Item_Mushroom_F_1456386759",
0x403af09b,"MainField_Item_Mushroom_F_1485876078",
0xca7dec55,"MainField_Item_Mushroom_F_1487824328",
0x37d0b607,"MainField_Item_Mushroom_F_1494032763",
0x62ed9a2e,"MainField_Item_Mushroom_F_1515070827",
0xf49ac829,"MainField_Item_Mushroom_F_1521505956",
0x2b8a5914,"MainField_Item_Mushroom_F_1524770198",
0x06bf4002,"MainField_Item_Mushroom_F_1524983335",
0xe760a0e0,"MainField_Item_Mushroom_F_1544997667",
0x78229c16,"MainField_Item_Mushroom_F_1546807704",
0x271c730e,"MainField_Item_Mushroom_F_1553149512",
0x98a327c0,"MainField_Item_Mushroom_F_1555216675",
0xa7b7cc94,"MainField_Item_Mushroom_F_1597317985",
0x16e4e0c2,"MainField_Item_Mushroom_F_1598217296",
0xdd5b7088,"MainField_Item_Mushroom_F_1611436303",
0x73721afe,"MainField_Item_Mushroom_F_1621767529",
0x79e4a76b,"MainField_Item_Mushroom_F_1653318023",
0x54dce054,"MainField_Item_Mushroom_F_1666892515",
0x9594f49f,"MainField_Item_Mushroom_F_16675099",
0x8184c48d,"MainField_Item_Mushroom_F_1679910077",
0x7b6553af,"MainField_Item_Mushroom_F_1730795418",
0xc04a1cb9,"MainField_Item_Mushroom_F_1743394133",
0x13ff9b95,"MainField_Item_Mushroom_F_175068692",
0xcd085e12,"MainField_Item_Mushroom_F_1771319140",
0x23ac3592,"MainField_Item_Mushroom_F_1771946362",
0xecaf20a1,"MainField_Item_Mushroom_F_1789873930",
0x78791513,"MainField_Item_Mushroom_F_179596695",
0x59798f47,"MainField_Item_Mushroom_F_1818230089",
0x4c060f2a,"MainField_Item_Mushroom_F_1840794374",
0xd5839f37,"MainField_Item_Mushroom_F_1851528167",
0xa04b1777,"MainField_Item_Mushroom_F_1855358269",
0x0b43cf33,"MainField_Item_Mushroom_F_1856338728",
0xcc360f7c,"MainField_Item_Mushroom_F_1862719122",
0x8868a312,"MainField_Item_Mushroom_F_1867730680",
0x4e1711bd,"MainField_Item_Mushroom_F_1869354884",
0x43c9127b,"MainField_Item_Mushroom_F_1873627387",
0x905369e3,"MainField_Item_Mushroom_F_1896579415",
0xfba6ee68,"MainField_Item_Mushroom_F_1899092684",
0x252925e0,"MainField_Item_Mushroom_F_1910378141",
0x588bc958,"MainField_Item_Mushroom_F_1912039928",
0x2f7e6bf1,"MainField_Item_Mushroom_F_191425450",
0x3219544e,"MainField_Item_Mushroom_F_1929257416",
0x92ffb2c9,"MainField_Item_Mushroom_F_1929724999",
0x52a62ecf,"MainField_Item_Mushroom_F_1952143299",
0xb7e0dd63,"MainField_Item_Mushroom_F_2017158024",
0x83cb3013,"MainField_Item_Mushroom_F_2036648436",
0x14814643,"MainField_Item_Mushroom_F_2046538548",
0xb47c0b28,"MainField_Item_Mushroom_F_2047351574",
0x2da73fc9,"MainField_Item_Mushroom_F_2048760829",
0x451f523b,"MainField_Item_Mushroom_F_2060764922",
0xdd804a2c,"MainField_Item_Mushroom_F_2070880893",
0x982dab93,"MainField_Item_Mushroom_F_2114258781",
0xea5a2801,"MainField_Item_Mushroom_F_2128565982",
0x6e0feb86,"MainField_Item_Mushroom_F_2157758199",
0x38e9fcbf,"MainField_Item_Mushroom_F_2162706151",
0x9322da54,"MainField_Item_Mushroom_F_2192248671",
0x9945b58b,"MainField_Item_Mushroom_F_2223243538",
0x83acb84e,"MainField_Item_Mushroom_F_2229360571",
0xdfdee3dc,"MainField_Item_Mushroom_F_2232724529",
0xdafbe920,"MainField_Item_Mushroom_F_2292107240",
0xddfda835,"MainField_Item_Mushroom_F_2293201948",
0xe1516ea4,"MainField_Item_Mushroom_F_2304101545",
0x23a1a44d,"MainField_Item_Mushroom_F_230450043",
0x771d5108,"MainField_Item_Mushroom_F_2344070488",
0x5b71b6bf,"MainField_Item_Mushroom_F_2345481853",
0x21dc82c6,"MainField_Item_Mushroom_F_2349606337",
0xe73c8d62,"MainField_Item_Mushroom_F_2357600888",
0x03912cb1,"MainField_Item_Mushroom_F_2363649977",
0xefa32a45,"MainField_Item_Mushroom_F_2368205045",
0x0739f1c3,"MainField_Item_Mushroom_F_2383826438",
0xecd39016,"MainField_Item_Mushroom_F_2439625554",
0xd9db888d,"MainField_Item_Mushroom_F_2445721963",
0x333e349c,"MainField_Item_Mushroom_F_2451869794",
0xcd8d9483,"MainField_Item_Mushroom_F_2457869170",
0xa8518d91,"MainField_Item_Mushroom_F_247635674",
0xbf458762,"MainField_Item_Mushroom_F_2480777593",
0x442002a0,"MainField_Item_Mushroom_F_2481312992",
0x879ed8f8,"MainField_Item_Mushroom_F_2491168872",
0x6133d392,"MainField_Item_Mushroom_F_2519702342",
0xf618ad00,"MainField_Item_Mushroom_F_2559798643",
0x17c5114d,"MainField_Item_Mushroom_F_2567562915",
0x6f0acfa7,"MainField_Item_Mushroom_F_2579963793",
0x740875b4,"MainField_Item_Mushroom_F_2586034555",
0x2217d1c2,"MainField_Item_Mushroom_F_2597922508",
0x3ae495a9,"MainField_Item_Mushroom_F_2617196406",
0x997e0237,"MainField_Item_Mushroom_F_2637207028",
0x54eee08a,"MainField_Item_Mushroom_F_2686808612",
0xbae70644,"MainField_Item_Mushroom_F_2725019788",
0xa5fbf63e,"MainField_Item_Mushroom_F_2776214614",
0x8817708c,"MainField_Item_Mushroom_F_2783838134",
0x439bb491,"MainField_Item_Mushroom_F_2784249917",
0xe36277ab,"MainField_Item_Mushroom_F_2820684596",
0x5ca9ff54,"MainField_Item_Mushroom_F_2831766279",
0x930985bd,"MainField_Item_Mushroom_F_2835930722",
0x38975478,"MainField_Item_Mushroom_F_2848408586",
0x0d846c70,"MainField_Item_Mushroom_F_2870012744",
0xfef1328f,"MainField_Item_Mushroom_F_2888727718",
0x779ca25a,"MainField_Item_Mushroom_F_29373321",
0xd9be67e7,"MainField_Item_Mushroom_F_2940050854",
0x372b86a7,"MainField_Item_Mushroom_F_2957858766",
0x3c767687,"MainField_Item_Mushroom_F_2958070218",
0xe21a82ae,"MainField_Item_Mushroom_F_2973100843",
0xc10632fd,"MainField_Item_Mushroom_F_2985027625",
0xe94be8c4,"MainField_Item_Mushroom_F_3006806902",
0x95cd17f2,"MainField_Item_Mushroom_F_3013248337",
0x41b25418,"MainField_Item_Mushroom_F_3020180368",
0x3dbf1732,"MainField_Item_Mushroom_F_3021696942",
0xf08f5f20,"MainField_Item_Mushroom_F_3043551188",
0x2b85b6f8,"MainField_Item_Mushroom_F_3075128823",
0x5010a526,"MainField_Item_Mushroom_F_3082665226",
0x49b490ae,"MainField_Item_Mushroom_F_3086166924",
0x9c79e71a,"MainField_Item_Mushroom_F_3099526666",
0x9cf34796,"MainField_Item_Mushroom_F_3100003692",
0x22b438b3,"MainField_Item_Mushroom_F_3215117035",
0x80cad942,"MainField_Item_Mushroom_F_3249755916",
0x3cc0f6aa,"MainField_Item_Mushroom_F_3265835466",
0x554457de,"MainField_Item_Mushroom_F_327123140",
0xf7e9060e,"MainField_Item_Mushroom_F_3294276921",
0xa3b7f99d,"MainField_Item_Mushroom_F_336047828",
0x10611f1b,"MainField_Item_Mushroom_F_3375792015",
0xe550eb45,"MainField_Item_Mushroom_F_3397913699",
0x5b06a20a,"MainField_Item_Mushroom_F_3416975727",
0x0f166ee0,"MainField_Item_Mushroom_F_3423511060",
0xd20972e5,"MainField_Item_Mushroom_F_3442809499",
0x3a0d1c95,"MainField_Item_Mushroom_F_3513126681",
0x268ac348,"MainField_Item_Mushroom_F_3547132323",
0xc16b11c3,"MainField_Item_Mushroom_F_3550281376",
0x65f45138,"MainField_Item_Mushroom_F_3570237814",
0x7dcedf13,"MainField_Item_Mushroom_F_3575866142",
0xd9c8e90b,"MainField_Item_Mushroom_F_3607017448",
0x522df09d,"MainField_Item_Mushroom_F_3614241837",
0xce4f5f0c,"MainField_Item_Mushroom_F_3619435523",
0xa97104f5,"MainField_Item_Mushroom_F_3625511762",
0x39778024,"MainField_Item_Mushroom_F_3655078713",
0xf47809fd,"MainField_Item_Mushroom_F_3663393274",
0x4bc4bc83,"MainField_Item_Mushroom_F_3678339114",
0x939533e8,"MainField_Item_Mushroom_F_3693935296",
0xe248c23e,"MainField_Item_Mushroom_F_3722852754",
0xd34c4cfd,"MainField_Item_Mushroom_F_3730490208",
0x867a1cc5,"MainField_Item_Mushroom_F_3748884391",
0x851f1b02,"MainField_Item_Mushroom_F_376462478",
0x92c20310,"MainField_Item_Mushroom_F_3786436049",
0x12641b9b,"MainField_Item_Mushroom_F_380641729",
0xebadcb84,"MainField_Item_Mushroom_F_3837526862",
0x3f9d54aa,"MainField_Item_Mushroom_F_3864701233",
0xa4949f00,"MainField_Item_Mushroom_F_3871796930",
0x418441ad,"MainField_Item_Mushroom_F_3871867608",
0xa851d510,"MainField_Item_Mushroom_F_3878505102",
0x29ba9ac4,"MainField_Item_Mushroom_F_3904563410",
0x3df9d962,"MainField_Item_Mushroom_F_3935186624",
0x31228efc,"MainField_Item_Mushroom_F_3939429085",
0x559216e9,"MainField_Item_Mushroom_F_3940086763",
0xb4f39110,"MainField_Item_Mushroom_F_3943895326",
0xc30a36b2,"MainField_Item_Mushroom_F_3984089191",
0x4cef1feb,"MainField_Item_Mushroom_F_3985032169",
0xb044e224,"MainField_Item_Mushroom_F_3987157896",
0x64098fc1,"MainField_Item_Mushroom_F_3991294710",
0xbd8ec0b5,"MainField_Item_Mushroom_F_3992651603",
0xc42b1c61,"MainField_Item_Mushroom_F_4015520724",
0x508fb97b,"MainField_Item_Mushroom_F_4045937296",
0x216cb798,"MainField_Item_Mushroom_F_4049714098",
0x8ffacf9e,"MainField_Item_Mushroom_F_4051611701",
0xc07c07dd,"MainField_Item_Mushroom_F_4057333726",
0x6371a0ab,"MainField_Item_Mushroom_F_4058485132",
0x45871afd,"MainField_Item_Mushroom_F_4066670132",
0x0903cb26,"MainField_Item_Mushroom_F_4086169963",
0x1ce51806,"MainField_Item_Mushroom_F_408624554",
0xad91e863,"MainField_Item_Mushroom_F_4101179860",
0xd1bb590b,"MainField_Item_Mushroom_F_412033895",
0xee30278a,"MainField_Item_Mushroom_F_4128079400",
0x166134cd,"MainField_Item_Mushroom_F_4146740729",
0xcbfd7cbf,"MainField_Item_Mushroom_F_4153581128",
0x43d42e9c,"MainField_Item_Mushroom_F_4176395863",
0x917b83d1,"MainField_Item_Mushroom_F_4200699987",
0xdb2a49c6,"MainField_Item_Mushroom_F_4218087657",
0xdc2ace02,"MainField_Item_Mushroom_F_4237330628",
0xcb3a7ce0,"MainField_Item_Mushroom_F_4260252297",
0x507ceb44,"MainField_Item_Mushroom_F_4277135374",
0xc6f79421,"MainField_Item_Mushroom_F_445672599",
0x38e49e6a,"MainField_Item_Mushroom_F_447752101",
0x34c90ed9,"MainField_Item_Mushroom_F_449602330",
0x120039f5,"MainField_Item_Mushroom_F_449610103",
0x7a7f216b,"MainField_Item_Mushroom_F_458213618",
0xf0614471,"MainField_Item_Mushroom_F_489991783",
0xa77e5d90,"MainField_Item_Mushroom_F_519393659",
0xd30bffaf,"MainField_Item_Mushroom_F_523542080",
0x3ed67b20,"MainField_Item_Mushroom_F_526005097",
0x5440ace5,"MainField_Item_Mushroom_F_550783929",
0xb6cf2c67,"MainField_Item_Mushroom_F_574800788",
0xe3621c44,"MainField_Item_Mushroom_F_638664094",
0x2bf4473c,"MainField_Item_Mushroom_F_641788081",
0x942786d6,"MainField_Item_Mushroom_F_64848285",
0xd882d0fc,"MainField_Item_Mushroom_F_649099657",
0xc04951f7,"MainField_Item_Mushroom_F_707496916",
0xabad08ca,"MainField_Item_Mushroom_F_720039203",
0x4aabf5e3,"MainField_Item_Mushroom_F_729729184",
0xc0a2a303,"MainField_Item_Mushroom_F_786263402",
0x8cbe714b,"MainField_Item_Mushroom_F_788816473",
0xbfa9e22a,"MainField_Item_Mushroom_F_8250724",
0xc2be453d,"MainField_Item_Mushroom_F_830066401",
0xa5840518,"MainField_Item_Mushroom_F_841284453",
0xb68f377f,"MainField_Item_Mushroom_F_849558944",
0xb524df9e,"MainField_Item_Mushroom_F_861446755",
0xe15f5c07,"MainField_Item_Mushroom_F_862486876",
0x4cae1ebd,"MainField_Item_Mushroom_F_866116027",
0xfe8d8e76,"MainField_Item_Mushroom_F_912409036",
0xad579e4a,"MainField_Item_Mushroom_F_916135628",
0x98d40c34,"MainField_Item_Mushroom_F_942594226",
0x30890a5e,"MainField_Item_Mushroom_F_980194030",
0x7d97499a,"MainField_Item_Mushroom_F_992897904",
0x4874e37e,"MainField_Item_Mushroom_H_1025958392",
0xa516cfe4,"MainField_Item_Mushroom_H_1032444545",
0x63fd1758,"MainField_Item_Mushroom_H_110008704",
0xff706f3f,"MainField_Item_Mushroom_H_1107623022",
0xdaba70f6,"MainField_Item_Mushroom_H_1169280423",
0x02a7cc4f,"MainField_Item_Mushroom_H_1173679750",
0x624d7fd4,"MainField_Item_Mushroom_H_1209969378",
0xb8ba744e,"MainField_Item_Mushroom_H_1211161878",
0x0bbb69dc,"MainField_Item_Mushroom_H_1230132688",
0x356e45f3,"MainField_Item_Mushroom_H_1238589616",
0x90f87fb7,"MainField_Item_Mushroom_H_1307003892",
0x928fdaf7,"MainField_Item_Mushroom_H_1309613917",
0x34e81b93,"MainField_Item_Mushroom_H_1316734628",
0xd2568cda,"MainField_Item_Mushroom_H_1323740075",
0x098a4f62,"MainField_Item_Mushroom_H_1330419769",
0xcceb2261,"MainField_Item_Mushroom_H_1353224550",
0x8d1f5666,"MainField_Item_Mushroom_H_136894915",
0xce252ed1,"MainField_Item_Mushroom_H_1393252162",
0x2169aeb9,"MainField_Item_Mushroom_H_1424609334",
0xfe8d6399,"MainField_Item_Mushroom_H_1454444089",
0xab15c8a6,"MainField_Item_Mushroom_H_1458693151",
0xdb561c3a,"MainField_Item_Mushroom_H_147730627",
0x1291df27,"MainField_Item_Mushroom_H_1534100076",
0x07c6f5cf,"MainField_Item_Mushroom_H_15404831",
0xd3bb8c41,"MainField_Item_Mushroom_H_1547644271",
0xdb713c01,"MainField_Item_Mushroom_H_1571707316",
0x5a541541,"MainField_Item_Mushroom_H_1631650339",
0x921eb5bf,"MainField_Item_Mushroom_H_1664464259",
0x374d3f0b,"MainField_Item_Mushroom_H_1687784485",
0x39c29909,"MainField_Item_Mushroom_H_169373786",
0xd2477a7f,"MainField_Item_Mushroom_H_1714957389",
0x096b2658,"MainField_Item_Mushroom_H_1715580298",
0xe6333b46,"MainField_Item_Mushroom_H_1732241373",
0xe014bcd7,"MainField_Item_Mushroom_H_175280127",
0xde0da565,"MainField_Item_Mushroom_H_1789562867",
0x09707ba4,"MainField_Item_Mushroom_H_182464585",
0xb661b5da,"MainField_Item_Mushroom_H_1848543025",
0xb5fffce2,"MainField_Item_Mushroom_H_1861263116",
0xe3fe38a9,"MainField_Item_Mushroom_H_1869173630",
0x9d0260ea,"MainField_Item_Mushroom_H_188683357",
0x1b4c64c7,"MainField_Item_Mushroom_H_1897505660",
0x46e933d8,"MainField_Item_Mushroom_H_1904122129",
0x51c0041b,"MainField_Item_Mushroom_H_1910036941",
0xbde43e13,"MainField_Item_Mushroom_H_1928267005",
0xc4cdefe0,"MainField_Item_Mushroom_H_1934330307",
0xaeedec61,"MainField_Item_Mushroom_H_1944547159",
0x4439adc7,"MainField_Item_Mushroom_H_1982916426",
0x73ae3029,"MainField_Item_Mushroom_H_2004680436",
0x50c61366,"MainField_Item_Mushroom_H_2028775841",
0xd838ee40,"MainField_Item_Mushroom_H_2029140156",
0xbf592835,"MainField_Item_Mushroom_H_2042822124",
0xab81cd47,"MainField_Item_Mushroom_H_207346411",
0xfb8a1015,"MainField_Item_Mushroom_H_2088729672",
0x02d2a5d4,"MainField_Item_Mushroom_H_2093643162",
0x0e70b117,"MainField_Item_Mushroom_H_2095172520",
0x693e8674,"MainField_Item_Mushroom_H_2115766843",
0xf3c610a6,"MainField_Item_Mushroom_H_2121708554",
0xc79c4760,"MainField_Item_Mushroom_H_2130466280",
0x7a14ff1c,"MainField_Item_Mushroom_H_2140767421",
0xd93f5d14,"MainField_Item_Mushroom_H_2168762191",
0x92edab83,"MainField_Item_Mushroom_H_2180640077",
0x2c66e6d9,"MainField_Item_Mushroom_H_2194561849",
0xbfd6c361,"MainField_Item_Mushroom_H_2197295491",
0xf8411b20,"MainField_Item_Mushroom_H_2207432884",
0x2f50c8df,"MainField_Item_Mushroom_H_2223607852",
0xb2e9b92d,"MainField_Item_Mushroom_H_2225011799",
0x83dc61a0,"MainField_Item_Mushroom_H_2225123781",
0x8a4e6393,"MainField_Item_Mushroom_H_2226155541",
0xfe1858d7,"MainField_Item_Mushroom_H_2233838291",
0xa8c29bf9,"MainField_Item_Mushroom_H_2250676434",
0x55eeedff,"MainField_Item_Mushroom_H_2254715201",
0xa43ed868,"MainField_Item_Mushroom_H_2254965451",
0xaae4b436,"MainField_Item_Mushroom_H_2281729493",
0xbeae01cb,"MainField_Item_Mushroom_H_2300922777",
0xce3ddf24,"MainField_Item_Mushroom_H_2330261572",
0x4b948946,"MainField_Item_Mushroom_H_2333925604",
0x6c361573,"MainField_Item_Mushroom_H_2334172068",
0x827d3a07,"MainField_Item_Mushroom_H_2347811324",
0x466bf465,"MainField_Item_Mushroom_H_2375789922",
0xfbe06a37,"MainField_Item_Mushroom_H_2399947776",
0xc260c923,"MainField_Item_Mushroom_H_2436276155",
0xa5903211,"MainField_Item_Mushroom_H_2471781719",
0x79fbf7e1,"MainField_Item_Mushroom_H_2477628082",
0xac9b1330,"MainField_Item_Mushroom_H_250120059",
0x527c7cd3,"MainField_Item_Mushroom_H_2514990972",
0x51e6e3a2,"MainField_Item_Mushroom_H_252365902",
0x87637d8a,"MainField_Item_Mushroom_H_2538220057",
0x38965ef6,"MainField_Item_Mushroom_H_2558600154",
0x64bb106a,"MainField_Item_Mushroom_H_256334560",
0x44014dcf,"MainField_Item_Mushroom_H_2610446218",
0xb27112ec,"MainField_Item_Mushroom_H_2685717960",
0x9297739d,"MainField_Item_Mushroom_H_2697786652",
0xabedefaf,"MainField_Item_Mushroom_H_2708029864",
0x5af2a8f4,"MainField_Item_Mushroom_H_2713611379",
0x91f42350,"MainField_Item_Mushroom_H_2716602717",
0x2aa19d04,"MainField_Item_Mushroom_H_2796909263",
0xc0755c5e,"MainField_Item_Mushroom_H_2834841600",
0x0df85adb,"MainField_Item_Mushroom_H_2887233314",
0x5e591515,"MainField_Item_Mushroom_H_2918364069",
0x1e8a8367,"MainField_Item_Mushroom_H_2933021940",
0x116b2023,"MainField_Item_Mushroom_H_2968944866",
0xd7750cc3,"MainField_Item_Mushroom_H_2977659601",
0x6fa5c5e2,"MainField_Item_Mushroom_H_3014526041",
0x28208a01,"MainField_Item_Mushroom_H_3014545180",
0x6c1fc84f,"MainField_Item_Mushroom_H_3051345046",
0x5a41120a,"MainField_Item_Mushroom_H_3064423998",
0x1c86c5d9,"MainField_Item_Mushroom_H_310563431",
0xc629677a,"MainField_Item_Mushroom_H_3129512446",
0x54574539,"MainField_Item_Mushroom_H_3192123782",
0x2341a8a7,"MainField_Item_Mushroom_H_324532415",
0xc6c2c091,"MainField_Item_Mushroom_H_3254868185",
0x70bee56b,"MainField_Item_Mushroom_H_3284189137",
0x26d07ec0,"MainField_Item_Mushroom_H_328778550",
0x0fec1c3f,"MainField_Item_Mushroom_H_3301342300",
0x3c853782,"MainField_Item_Mushroom_H_3325490080",
0x52fb0d7f,"MainField_Item_Mushroom_H_3365809686",
0x6e62b201,"MainField_Item_Mushroom_H_3382321314",
0xe6105d3e,"MainField_Item_Mushroom_H_3389205223",
0xe8a99d90,"MainField_Item_Mushroom_H_3406628616",
0x0f05dbfa,"MainField_Item_Mushroom_H_3464703138",
0x71b325e6,"MainField_Item_Mushroom_H_3474341687",
0x0bcf01a7,"MainField_Item_Mushroom_H_3479158929",
0x10482e4b,"MainField_Item_Mushroom_H_350128048",
0x310c8bb5,"MainField_Item_Mushroom_H_350675548",
0x36a25219,"MainField_Item_Mushroom_H_3507061032",
0x9c391548,"MainField_Item_Mushroom_H_351315085",
0x8b9f3785,"MainField_Item_Mushroom_H_3525458308",
0x3b4920e2,"MainField_Item_Mushroom_H_3535362829",
0x826ec4e8,"MainField_Item_Mushroom_H_3540865139",
0xce098a8b,"MainField_Item_Mushroom_H_3545480437",
0x8e252279,"MainField_Item_Mushroom_H_3558496993",
0x04afebbc,"MainField_Item_Mushroom_H_355902826",
0x191479ce,"MainField_Item_Mushroom_H_3585900181",
0x4e8accdd,"MainField_Item_Mushroom_H_3592973325",
0x85c037f1,"MainField_Item_Mushroom_H_3594462090",
0x7354adda,"MainField_Item_Mushroom_H_3635468625",
0xb77a67f2,"MainField_Item_Mushroom_H_3654355129",
0xf30de676,"MainField_Item_Mushroom_H_3656169994",
0x1a509da1,"MainField_Item_Mushroom_H_3677323200",
0x9b1bb700,"MainField_Item_Mushroom_H_3683900744",
0xbdff4108,"MainField_Item_Mushroom_H_3713247770",
0xf919ed27,"MainField_Item_Mushroom_H_3728429675",
0xb7c1d444,"MainField_Item_Mushroom_H_3816238165",
0x68f8d843,"MainField_Item_Mushroom_H_3819946620",
0x366b83b8,"MainField_Item_Mushroom_H_3832344782",
0x8f744828,"MainField_Item_Mushroom_H_3868702375",
0x1b3a4b66,"MainField_Item_Mushroom_H_3878468604",
0x6b70721f,"MainField_Item_Mushroom_H_3883410098",
0xf290ac14,"MainField_Item_Mushroom_H_3896440645",
0xb39bb52d,"MainField_Item_Mushroom_H_3915269558",
0x584e55f9,"MainField_Item_Mushroom_H_3921714616",
0xe730ef9b,"MainField_Item_Mushroom_H_3951589216",
0xa4790ab3,"MainField_Item_Mushroom_H_3971551084",
0xca4474a5,"MainField_Item_Mushroom_H_4032939815",
0x6b92af9e,"MainField_Item_Mushroom_H_4039619474",
0x4a197608,"MainField_Item_Mushroom_H_40624020",
0xe2571089,"MainField_Item_Mushroom_H_4079267546",
0x30d8a647,"MainField_Item_Mushroom_H_4128994135",
0xefc8be99,"MainField_Item_Mushroom_H_4166416001",
0xce5b3082,"MainField_Item_Mushroom_H_4170259313",
0xfbb1bd0f,"MainField_Item_Mushroom_H_418306713",
0xbbd15862,"MainField_Item_Mushroom_H_4231887664",
0xa1042c99,"MainField_Item_Mushroom_H_424803663",
0x21f300ff,"MainField_Item_Mushroom_H_4271943763",
0xb20d4413,"MainField_Item_Mushroom_H_4272645833",
0xc2b2968d,"MainField_Item_Mushroom_H_4284642953",
0xb7ef9c7d,"MainField_Item_Mushroom_H_47107509",
0x2aa98fd6,"MainField_Item_Mushroom_H_511118625",
0x4e90bebf,"MainField_Item_Mushroom_H_514181808",
0x50228575,"MainField_Item_Mushroom_H_53883475",
0x7ee2eb4c,"MainField_Item_Mushroom_H_595846371",
0xbf3c7ce8,"MainField_Item_Mushroom_H_61166911",
0x44089dad,"MainField_Item_Mushroom_H_62144154",
0xcdb08ce6,"MainField_Item_Mushroom_H_642902883",
0x0edc428b,"MainField_Item_Mushroom_H_702152030",
0x12b1bcec,"MainField_Item_Mushroom_H_711363901",
0xb02219c0,"MainField_Item_Mushroom_H_720994674",
0x0e0feb67,"MainField_Item_Mushroom_H_729793588",
0x1086a68f,"MainField_Item_Mushroom_H_736097101",
0x59c8461b,"MainField_Item_Mushroom_H_743631977",
0xbd65b096,"MainField_Item_Mushroom_H_745403643",
0x9bf374cd,"MainField_Item_Mushroom_H_746392874",
0x3033230c,"MainField_Item_Mushroom_H_761665686",
0x132c1857,"MainField_Item_Mushroom_H_772024811",
0x84442e57,"MainField_Item_Mushroom_H_805653410",
0x5d87223f,"MainField_Item_Mushroom_H_813033220",
0xb1c9e296,"MainField_Item_Mushroom_H_84241193",
0x9243facc,"MainField_Item_Mushroom_H_852183019",
0xe052ffff,"MainField_Item_Mushroom_H_94716090",
0x086ad90c,"MainField_Item_Mushroom_H_978987348",
0xcbeff952,"MainField_Item_Mushroom_J_1015380325",
0xfa817c00,"MainField_Item_Mushroom_J_1024787635",
0x873f31d0,"MainField_Item_Mushroom_J_1040079111",
0x6a84fde1,"MainField_Item_Mushroom_J_1043415202",
0x60701c6b,"MainField_Item_Mushroom_J_1060054278",
0x3bf5caf6,"MainField_Item_Mushroom_J_1064644359",
0xf4d9902e,"MainField_Item_Mushroom_J_1073682969",
0xacf592ec,"MainField_Item_Mushroom_J_1075082384",
0x47f3aaa3,"MainField_Item_Mushroom_J_1080226038",
0xeb32c143,"MainField_Item_Mushroom_J_1080335448",
0x731cbf18,"MainField_Item_Mushroom_J_1082761181",
0x5ad8dc34,"MainField_Item_Mushroom_J_1088631724",
0xe4be8754,"MainField_Item_Mushroom_J_1114193281",
0x201d6111,"MainField_Item_Mushroom_J_1119508887",
0xf432a64b,"MainField_Item_Mushroom_J_1143282261",
0x2c86fc46,"MainField_Item_Mushroom_J_1144308427",
0x6944fb1e,"MainField_Item_Mushroom_J_1147970804",
0x18569c56,"MainField_Item_Mushroom_J_115544703",
0x3c3e25dd,"MainField_Item_Mushroom_J_1167408294",
0x7b40f7fe,"MainField_Item_Mushroom_J_117167707",
0xb853416a,"MainField_Item_Mushroom_J_1178470314",
0x9cb93d97,"MainField_Item_Mushroom_J_1181163959",
0xc48b15b5,"MainField_Item_Mushroom_J_1184654088",
0xf5941fbe,"MainField_Item_Mushroom_J_1185141625",
0x5123eb54,"MainField_Item_Mushroom_J_1193037518",
0x2223cb08,"MainField_Item_Mushroom_J_1197392448",
0x77eb3812,"MainField_Item_Mushroom_J_1200558191",
0x340ed863,"MainField_Item_Mushroom_J_1201784762",
0x16c11e70,"MainField_Item_Mushroom_J_1242706981",
0x031cf58b,"MainField_Item_Mushroom_J_1250022345",
0x5650a396,"MainField_Item_Mushroom_J_1252748330",
0x562ba480,"MainField_Item_Mushroom_J_1271668702",
0x2c23c926,"MainField_Item_Mushroom_J_1272917666",
0x13d1269f,"MainField_Item_Mushroom_J_1275811227",
0x72c1b235,"MainField_Item_Mushroom_J_128885873",
0x55e50b36,"MainField_Item_Mushroom_J_1299446649",
0x948f3039,"MainField_Item_Mushroom_J_1300230204",
0x334b9ede,"MainField_Item_Mushroom_J_1311021759",
0xabf5e713,"MainField_Item_Mushroom_J_1331198614",
0xbd478ecf,"MainField_Item_Mushroom_J_1331819791",
0xcbd674eb,"MainField_Item_Mushroom_J_1352807343",
0x4d6573e8,"MainField_Item_Mushroom_J_1355053641",
0xa015c580,"MainField_Item_Mushroom_J_1359900732",
0x3daacdaa,"MainField_Item_Mushroom_J_136280207",
0xa54da99f,"MainField_Item_Mushroom_J_1363790226",
0x73e885fd,"MainField_Item_Mushroom_J_1402090810",
0xd15dea1c,"MainField_Item_Mushroom_J_1405871956",
0x551bcdff,"MainField_Item_Mushroom_J_1413519478",
0xcf26771f,"MainField_Item_Mushroom_J_1424436805",
0x78070cb5,"MainField_Item_Mushroom_J_1435219187",
0x7a550ce5,"MainField_Item_Mushroom_J_1456309935",
0xc5cb48f7,"MainField_Item_Mushroom_J_1464276773",
0xba646d51,"MainField_Item_Mushroom_J_1465006716",
0x3f573a60,"MainField_Item_Mushroom_J_1466211699",
0x0f51bfa1,"MainField_Item_Mushroom_J_1482440026",
0xaa7b20b8,"MainField_Item_Mushroom_J_1493184651",
0x2d15e411,"MainField_Item_Mushroom_J_1506218545",
0x08809906,"MainField_Item_Mushroom_J_1534009757",
0x13531f54,"MainField_Item_Mushroom_J_153737603",
0x0df5c333,"MainField_Item_Mushroom_J_1546575073",
0xccf6747c,"MainField_Item_Mushroom_J_1551644419",
0xcc06cbf5,"MainField_Item_Mushroom_J_155902148",
0x578713b1,"MainField_Item_Mushroom_J_1562508054",
0x832d5e1f,"MainField_Item_Mushroom_J_1564633026",
0x8b0159c3,"MainField_Item_Mushroom_J_1564787257",
0xd9b2fac9,"MainField_Item_Mushroom_J_1578726821",
0xa69ee116,"MainField_Item_Mushroom_J_1584495584",
0x4bd16cd4,"MainField_Item_Mushroom_J_1594469582",
0x6dc4bd1d,"MainField_Item_Mushroom_J_1612341369",
0x2a2078c4,"MainField_Item_Mushroom_J_1636257108",
0xd80adb38,"MainField_Item_Mushroom_J_1639920978",
0xf31367de,"MainField_Item_Mushroom_J_1663873103",
0x5c053890,"MainField_Item_Mushroom_J_1664930990",
0x1911c28e,"MainField_Item_Mushroom_J_1668740790",
0xacc71037,"MainField_Item_Mushroom_J_1674047602",
0xc082f9c5,"MainField_Item_Mushroom_J_1692693064",
0x33e8b5fe,"MainField_Item_Mushroom_J_1695860363",
0x30ae7e45,"MainField_Item_Mushroom_J_1716297317",
0x1628423c,"MainField_Item_Mushroom_J_1727860024",
0x3bedd4f2,"MainField_Item_Mushroom_J_1742522111",
0x3f0ca6c1,"MainField_Item_Mushroom_J_1760205525",
0xf3f5e179,"MainField_Item_Mushroom_J_1765110639",
0xdfb609e6,"MainField_Item_Mushroom_J_1766072672",
0x7d5aecd6,"MainField_Item_Mushroom_J_1775549371",
0x71aba0a6,"MainField_Item_Mushroom_J_1779459004",
0x084f0f3d,"MainField_Item_Mushroom_J_178158188",
0x00f05b97,"MainField_Item_Mushroom_J_1783336863",
0x607e788b,"MainField_Item_Mushroom_J_1788672242",
0xa095c7a0,"MainField_Item_Mushroom_J_1792314422",
0x3e48a2b9,"MainField_Item_Mushroom_J_179603835",
0xa0ce294c,"MainField_Item_Mushroom_J_1800381237",
0xa6785ffb,"MainField_Item_Mushroom_J_1822563495",
0x54e0df62,"MainField_Item_Mushroom_J_1852847018",
0x21edd366,"MainField_Item_Mushroom_J_185824817",
0x92123aa1,"MainField_Item_Mushroom_J_1862740928",
0x200e865c,"MainField_Item_Mushroom_J_1869112971",
0xa0beb69b,"MainField_Item_Mushroom_J_1871303417",
0xe5d09393,"MainField_Item_Mushroom_J_1873295233",
0x83a3c950,"MainField_Item_Mushroom_J_1874197175",
0x510806fa,"MainField_Item_Mushroom_J_189347180",
0x000985ac,"MainField_Item_Mushroom_J_1916643352",
0x9cc91596,"MainField_Item_Mushroom_J_1918145140",
0x3356d984,"MainField_Item_Mushroom_J_1918426400",
0x945f79f2,"MainField_Item_Mushroom_J_1926277622",
0x4144e97c,"MainField_Item_Mushroom_J_1944359055",
0x30ddc55f,"MainField_Item_Mushroom_J_1951084415",
0x7d1802ad,"MainField_Item_Mushroom_J_1955151794",
0xe9790810,"MainField_Item_Mushroom_J_1957095083",
0x1eb0d84c,"MainField_Item_Mushroom_J_1958101414",
0xdb0bed90,"MainField_Item_Mushroom_J_1966691949",
0xfc3f23f7,"MainField_Item_Mushroom_J_1976663494",
0x4f7a40d6,"MainField_Item_Mushroom_J_1987361806",
0xf1276e2c,"MainField_Item_Mushroom_J_198777626",
0x41280bb7,"MainField_Item_Mushroom_J_1990465701",
0x64c10c24,"MainField_Item_Mushroom_J_2001677058",
0x10aa8dd7,"MainField_Item_Mushroom_J_2010717982",
0x8f9d6e13,"MainField_Item_Mushroom_J_20193432",
0xcd22c7ed,"MainField_Item_Mushroom_J_2032261906",
0x5f597a5a,"MainField_Item_Mushroom_J_2068319382",
0x68ba0564,"MainField_Item_Mushroom_J_2070310352",
0xddee12d3,"MainField_Item_Mushroom_J_2072401030",
0xf82d7358,"MainField_Item_Mushroom_J_2087245569",
0x009dcaa7,"MainField_Item_Mushroom_J_2087322351",
0xb3486552,"MainField_Item_Mushroom_J_2088897815",
0x304e1d37,"MainField_Item_Mushroom_J_2095066107",
0x24e57613,"MainField_Item_Mushroom_J_2097206726",
0xf2fb3d43,"MainField_Item_Mushroom_J_2102578214",
0x8f8d6bb4,"MainField_Item_Mushroom_J_2106173873",
0x9af56983,"MainField_Item_Mushroom_J_2120420744",
0xefb41374,"MainField_Item_Mushroom_J_2123684123",
0xf85adf54,"MainField_Item_Mushroom_J_2125090937",
0x4c1e6d8d,"MainField_Item_Mushroom_J_213356377",
0xfe9cfdb4,"MainField_Item_Mushroom_J_213420953",
0x043aa408,"MainField_Item_Mushroom_J_2136702504",
0xe520c957,"MainField_Item_Mushroom_J_2136786231",
0x07853830,"MainField_Item_Mushroom_J_2138376616",
0x49de37f2,"MainField_Item_Mushroom_J_2140249583",
0xb045dacc,"MainField_Item_Mushroom_J_2162696178",
0x2a3f1a8a,"MainField_Item_Mushroom_J_2170311958",
0x5ae5731e,"MainField_Item_Mushroom_J_2172012108",
0x791b9184,"MainField_Item_Mushroom_J_2179429661",
0x45ecafdd,"MainField_Item_Mushroom_J_2182747669",
0xf13c1010,"MainField_Item_Mushroom_J_2183923200",
0x72e71700,"MainField_Item_Mushroom_J_2183956951",
0x8c325278,"MainField_Item_Mushroom_J_2184941007",
0x2ef06fdf,"MainField_Item_Mushroom_J_2185523463",
0xa34bef38,"MainField_Item_Mushroom_J_2191650268",
0xf22aacd9,"MainField_Item_Mushroom_J_219177478",
0xaaf149d5,"MainField_Item_Mushroom_J_2205618737",
0x48f9663f,"MainField_Item_Mushroom_J_2207781409",
0xe9054849,"MainField_Item_Mushroom_J_2211156788",
0x4313b99e,"MainField_Item_Mushroom_J_2235679257",
0xb25a7f28,"MainField_Item_Mushroom_J_2242338788",
0x38a2d7b3,"MainField_Item_Mushroom_J_2247505109",
0xbe25cd91,"MainField_Item_Mushroom_J_2265711394",
0x8ababbc8,"MainField_Item_Mushroom_J_2276289713",
0xe8c08295,"MainField_Item_Mushroom_J_2286447767",
0x67242d78,"MainField_Item_Mushroom_J_2287138331",
0x0893b980,"MainField_Item_Mushroom_J_2289999905",
0x14a06ed3,"MainField_Item_Mushroom_J_2295355874",
0xaae3a81b,"MainField_Item_Mushroom_J_2301532588",
0x5e20b061,"MainField_Item_Mushroom_J_2311089904",
0x30bb0ac1,"MainField_Item_Mushroom_J_233105450",
0x7b9bd462,"MainField_Item_Mushroom_J_2335781378",
0xb1d3175c,"MainField_Item_Mushroom_J_2336614174",
0xd8847f56,"MainField_Item_Mushroom_J_2342144555",
0x80593ad0,"MainField_Item_Mushroom_J_2344167887",
0x4c509e44,"MainField_Item_Mushroom_J_2344329284",
0x73ccc9cc,"MainField_Item_Mushroom_J_2349630946",
0xceaaf246,"MainField_Item_Mushroom_J_2352603772",
0x698057b1,"MainField_Item_Mushroom_J_2369056091",
0x76ef9d97,"MainField_Item_Mushroom_J_2441481660",
0x2dff8281,"MainField_Item_Mushroom_J_2442282547",
0x8b9e7187,"MainField_Item_Mushroom_J_2443696826",
0x6500bd82,"MainField_Item_Mushroom_J_245038071",
0x41500a37,"MainField_Item_Mushroom_J_2457959274",
0xb690e50c,"MainField_Item_Mushroom_J_2458761316",
0xbee66b64,"MainField_Item_Mushroom_J_2462064443",
0x514c8625,"MainField_Item_Mushroom_J_247018357",
0xda3fe365,"MainField_Item_Mushroom_J_2471915197",
0x2ef34b66,"MainField_Item_Mushroom_J_2479833049",
0x1b9c173b,"MainField_Item_Mushroom_J_2484589379",
0xaa5eff29,"MainField_Item_Mushroom_J_2485458099",
0x74b9b2d0,"MainField_Item_Mushroom_J_2487492697",
0x03b6bae8,"MainField_Item_Mushroom_J_2495728643",
0xe89f3fab,"MainField_Item_Mushroom_J_2497823372",
0xe1528025,"MainField_Item_Mushroom_J_2529436780",
0xece2f71a,"MainField_Item_Mushroom_J_2541469695",
0x7dde2fa3,"MainField_Item_Mushroom_J_2553151933",
0xa86dc448,"MainField_Item_Mushroom_J_2556310924",
0x41ea6484,"MainField_Item_Mushroom_J_2566866591",
0x0a9c2f7e,"MainField_Item_Mushroom_J_2571010407",
0x79e74959,"MainField_Item_Mushroom_J_2571034114",
0xed13f460,"MainField_Item_Mushroom_J_2585946003",
0xd1d2bece,"MainField_Item_Mushroom_J_2587605138",
0xb89e8db1,"MainField_Item_Mushroom_J_2589204596",
0xe30d60cd,"MainField_Item_Mushroom_J_2592924681",
0x13eecc0a,"MainField_Item_Mushroom_J_2604087105",
0x2286929d,"MainField_Item_Mushroom_J_2619687428",
0xe667006f,"MainField_Item_Mushroom_J_2623206232",
0x3f0f4c9f,"MainField_Item_Mushroom_J_2648979431",
0x115d5f81,"MainField_Item_Mushroom_J_2658588616",
0x03e5800a,"MainField_Item_Mushroom_J_2682101900",
0xf7d14834,"MainField_Item_Mushroom_J_2682803758",
0xe5cbcb9c,"MainField_Item_Mushroom_J_2692337608",
0x6488145e,"MainField_Item_Mushroom_J_2709113979",
0x9a60f255,"MainField_Item_Mushroom_J_2727868256",
0x2459e3a2,"MainField_Item_Mushroom_J_2732759185",
0xa8bbaecc,"MainField_Item_Mushroom_J_2735446849",
0xb3eccd81,"MainField_Item_Mushroom_J_2742128912",
0x66f6a02c,"MainField_Item_Mushroom_J_2752450867",
0xdd0ac055,"MainField_Item_Mushroom_J_2753320201",
0x373ab089,"MainField_Item_Mushroom_J_2753390006",
0x3a4d512f,"MainField_Item_Mushroom_J_2770982476",
0x25538466,"MainField_Item_Mushroom_J_2789044279",
0xeca94d5f,"MainField_Item_Mushroom_J_2801147449",
0x7f01be86,"MainField_Item_Mushroom_J_2812255140",
0x9e9fc7ef,"MainField_Item_Mushroom_J_2812266077",
0x25bec76a,"MainField_Item_Mushroom_J_2813299345",
0x971f0c63,"MainField_Item_Mushroom_J_2821362386",
0xefbbc24f,"MainField_Item_Mushroom_J_2825981372",
0xc92b3bb8,"MainField_Item_Mushroom_J_2852817199",
0x6c20bf9c,"MainField_Item_Mushroom_J_2857260573",
0x072a73d7,"MainField_Item_Mushroom_J_2865225019",
0xdf729a1b,"MainField_Item_Mushroom_J_2868935642",
0x76b62d1e,"MainField_Item_Mushroom_J_2870828022",
0xbf2753f6,"MainField_Item_Mushroom_J_2872283181",
0x07c27762,"MainField_Item_Mushroom_J_2879892738",
0xc38afa4b,"MainField_Item_Mushroom_J_2881630790",
0x283421ad,"MainField_Item_Mushroom_J_2889850162",
0xea19ce83,"MainField_Item_Mushroom_J_2896750981",
0x9c9e6568,"MainField_Item_Mushroom_J_2926350810",
0x89a6b3b0,"MainField_Item_Mushroom_J_2935290512",
0x055937a5,"MainField_Item_Mushroom_J_2942479076",
0xf37dd52a,"MainField_Item_Mushroom_J_2956297357",
0xa2ba2007,"MainField_Item_Mushroom_J_2966934155",
0x3be845e8,"MainField_Item_Mushroom_J_2970145022",
0x763ba776,"MainField_Item_Mushroom_J_2976881107",
0x6fd0e747,"MainField_Item_Mushroom_J_2977035379",
0xd0db5fa7,"MainField_Item_Mushroom_J_2979987465",
0x0749cff9,"MainField_Item_Mushroom_J_300348623",
0xdd58efb2,"MainField_Item_Mushroom_J_3013584115",
0xa5d8a3f9,"MainField_Item_Mushroom_J_3017600122",
0x1bfa03be,"MainField_Item_Mushroom_J_3020893208",
0x4d2f54f7,"MainField_Item_Mushroom_J_3056075525",
0x17d089f3,"MainField_Item_Mushroom_J_3067913666",
0xfccaa096,"MainField_Item_Mushroom_J_307170568",
0x12051070,"MainField_Item_Mushroom_J_3079687083",
0x7e550080,"MainField_Item_Mushroom_J_3106991075",
0x8bc251d5,"MainField_Item_Mushroom_J_3110465777",
0xd816d342,"MainField_Item_Mushroom_J_3119366610",
0xe37ca5cc,"MainField_Item_Mushroom_J_3119690898",
0xce65aa15,"MainField_Item_Mushroom_J_3128826281",
0x9f18e382,"MainField_Item_Mushroom_J_3156830227",
0x649c24f1,"MainField_Item_Mushroom_J_3163146096",
0x34c773db,"MainField_Item_Mushroom_J_316500294",
0x58f56ee0,"MainField_Item_Mushroom_J_3170421262",
0x108a1220,"MainField_Item_Mushroom_J_3181561033",
0x49fbb6dd,"MainField_Item_Mushroom_J_3192010693",
0x29e18ad6,"MainField_Item_Mushroom_J_32076554",
0x9adaefe4,"MainField_Item_Mushroom_J_3217962918",
0xafe82227,"MainField_Item_Mushroom_J_3230245586",
0xe3ee3492,"MainField_Item_Mushroom_J_323088550",
0x1a82450e,"MainField_Item_Mushroom_J_3258854378",
0x240f88f1,"MainField_Item_Mushroom_J_3265628164",
0x5bfe3957,"MainField_Item_Mushroom_J_3268110090",
0x56f2e48b,"MainField_Item_Mushroom_J_3302743276",
0x54b927ab,"MainField_Item_Mushroom_J_331299635",
0x0643de75,"MainField_Item_Mushroom_J_3316006141",
0x6b4f6d44,"MainField_Item_Mushroom_J_3316996664",
0xdd8496a7,"MainField_Item_Mushroom_J_3323210325",
0xe1d387ad,"MainField_Item_Mushroom_J_3330055255",
0x14c9837c,"MainField_Item_Mushroom_J_3340567297",
0x4da48b35,"MainField_Item_Mushroom_J_3363226620",
0x48a158b5,"MainField_Item_Mushroom_J_3368301214",
0x737f59bb,"MainField_Item_Mushroom_J_3369993513",
0x8551e244,"MainField_Item_Mushroom_J_3373985659",
0xd64649a1,"MainField_Item_Mushroom_J_3374068547",
0xf59feb44,"MainField_Item_Mushroom_J_3383999322",
0x47bad6e3,"MainField_Item_Mushroom_J_3386892286",
0x0a878caa,"MainField_Item_Mushroom_J_3391336180",
0x9b1413cc,"MainField_Item_Mushroom_J_3399188233",
0xf9a0e014,"MainField_Item_Mushroom_J_3401919432",
0xc864bc8f,"MainField_Item_Mushroom_J_3414587416",
0x2f8bb5a6,"MainField_Item_Mushroom_J_3414691813",
0x07fccb74,"MainField_Item_Mushroom_J_3426941885",
0xf4293efe,"MainField_Item_Mushroom_J_3428776716",
0x7c9908da,"MainField_Item_Mushroom_J_3429332792",
0x6772e820,"MainField_Item_Mushroom_J_3433957339",
0x1f8c9b6b,"MainField_Item_Mushroom_J_3455867908",
0xbdb5fa45,"MainField_Item_Mushroom_J_3476611858",
0x9d8151aa,"MainField_Item_Mushroom_J_347937471",
0xab6fb2b8,"MainField_Item_Mushroom_J_3491773683",
0xd6c0daa2,"MainField_Item_Mushroom_J_3496331569",
0x8d0672b8,"MainField_Item_Mushroom_J_3507940809",
0xe5f23946,"MainField_Item_Mushroom_J_3520012398",
0xe84ef2c6,"MainField_Item_Mushroom_J_3541788063",
0x4d6721af,"MainField_Item_Mushroom_J_3552731207",
0x32200895,"MainField_Item_Mushroom_J_3576714818",
0x5e83611f,"MainField_Item_Mushroom_J_3582454669",
0xfdfa9e67,"MainField_Item_Mushroom_J_3587827055",
0x6db707da,"MainField_Item_Mushroom_J_3594688093",
0x19a9b5b0,"MainField_Item_Mushroom_J_3595482481",
0xce5954e0,"MainField_Item_Mushroom_J_3598129136",
0x433ad864,"MainField_Item_Mushroom_J_3601193557",
0xa9d2b757,"MainField_Item_Mushroom_J_3607143452",
0xa764a5e7,"MainField_Item_Mushroom_J_3618925757",
0x2aab15c9,"MainField_Item_Mushroom_J_3635104744",
0x67a9fc17,"MainField_Item_Mushroom_J_3640312238",
0x98837c10,"MainField_Item_Mushroom_J_3644766546",
0x033f773e,"MainField_Item_Mushroom_J_3647709516",
0xb5f30499,"MainField_Item_Mushroom_J_3659991169",
0xac6c5f4e,"MainField_Item_Mushroom_J_3663606150",
0xa303155f,"MainField_Item_Mushroom_J_3664452833",
0x201c4f39,"MainField_Item_Mushroom_J_3666192392",
0x73f6999d,"MainField_Item_Mushroom_J_3678843156",
0x99f73f16,"MainField_Item_Mushroom_J_369648020",
0x8d5ca652,"MainField_Item_Mushroom_J_3701442430",
0x86f6a5a3,"MainField_Item_Mushroom_J_3704679627",
0x7b08c1b8,"MainField_Item_Mushroom_J_370887730",
0xaf328775,"MainField_Item_Mushroom_J_3718891380",
0x90d5550f,"MainField_Item_Mushroom_J_3720408493",
0x4ae5054f,"MainField_Item_Mushroom_J_3728552763",
0x6dcb5428,"MainField_Item_Mushroom_J_372979359",
0x238b8d5f,"MainField_Item_Mushroom_J_3730157016",
0x1f6aae41,"MainField_Item_Mushroom_J_3734289899",
0x81f99bb0,"MainField_Item_Mushroom_J_373668475",
0xf66d2c8a,"MainField_Item_Mushroom_J_3743676236",
0xb6709547,"MainField_Item_Mushroom_J_3746358649",
0x6dfaebe9,"MainField_Item_Mushroom_J_3752453579",
0x2a09f28a,"MainField_Item_Mushroom_J_3762480445",
0x90aaf1a1,"MainField_Item_Mushroom_J_37644027",
0x7deaf488,"MainField_Item_Mushroom_J_3777971668",
0xb1cd7ba1,"MainField_Item_Mushroom_J_3805083526",
0xbc409398,"MainField_Item_Mushroom_J_3806436775",
0x294646f6,"MainField_Item_Mushroom_J_3813574695",
0x4c95ff22,"MainField_Item_Mushroom_J_382082146",
0x11e50205,"MainField_Item_Mushroom_J_3833412989",
0xdd28a5d8,"MainField_Item_Mushroom_J_3834928538",
0xeb14fd89,"MainField_Item_Mushroom_J_3858975169",
0x13dfdddd,"MainField_Item_Mushroom_J_3860812033",
0x59398232,"MainField_Item_Mushroom_J_3866109639",
0xcbcd4209,"MainField_Item_Mushroom_J_3873724204",
0x7fa624b0,"MainField_Item_Mushroom_J_3876798222",
0x1176f530,"MainField_Item_Mushroom_J_3877452271",
0xeddd8bfd,"MainField_Item_Mushroom_J_3901056523",
0x8919fc46,"MainField_Item_Mushroom_J_3906408066",
0x0a0befb0,"MainField_Item_Mushroom_J_3916209007",
0x2843279e,"MainField_Item_Mushroom_J_3925880506",
0x923b9c56,"MainField_Item_Mushroom_J_3930708480",
0x1fc35d75,"MainField_Item_Mushroom_J_3933164811",
0x9a61fba0,"MainField_Item_Mushroom_J_393677139",
0x856a323d,"MainField_Item_Mushroom_J_3948790734",
0xd39be369,"MainField_Item_Mushroom_J_3962570711",
0x84118723,"MainField_Item_Mushroom_J_3989679998",
0x58ad44c7,"MainField_Item_Mushroom_J_3991005526",
0x6952d998,"MainField_Item_Mushroom_J_3996620289",
0x667d64ca,"MainField_Item_Mushroom_J_4000039414",
0x1fad838e,"MainField_Item_Mushroom_J_4008900574",
0xcc6f2c9f,"MainField_Item_Mushroom_J_4010121825",
0xfec754e6,"MainField_Item_Mushroom_J_4010707012",
0x0820216c,"MainField_Item_Mushroom_J_4038600486",
0x50639413,"MainField_Item_Mushroom_J_4040947198",
0xc03cf6fb,"MainField_Item_Mushroom_J_4046921017",
0xc9658d59,"MainField_Item_Mushroom_J_4053119250",
0x0c1fe545,"MainField_Item_Mushroom_J_4059372051",
0x1faa9df5,"MainField_Item_Mushroom_J_4063124052",
0x425fd5c9,"MainField_Item_Mushroom_J_4064735887",
0xa0ef25ae,"MainField_Item_Mushroom_J_4066955103",
0x503ef214,"MainField_Item_Mushroom_J_4067690363",
0x0ac05801,"MainField_Item_Mushroom_J_4075077628",
0xcaf6a882,"MainField_Item_Mushroom_J_4082378528",
0x523bed76,"MainField_Item_Mushroom_J_4085436919",
0x900933f2,"MainField_Item_Mushroom_J_4105405275",
0x25a9691b,"MainField_Item_Mushroom_J_4105539474",
0x23ab7853,"MainField_Item_Mushroom_J_4112641866",
0xef60d5b2,"MainField_Item_Mushroom_J_4113249942",
0x4cc458c9,"MainField_Item_Mushroom_J_4135662220",
0xe52b71bc,"MainField_Item_Mushroom_J_4142074407",
0xa7cd1634,"MainField_Item_Mushroom_J_4144048433",
0xb779b180,"MainField_Item_Mushroom_J_4157832411",
0x8d29f093,"MainField_Item_Mushroom_J_4160888000",
0xbc4dd5ab,"MainField_Item_Mushroom_J_4165187314",
0x3182abe9,"MainField_Item_Mushroom_J_4166142831",
0x8815157e,"MainField_Item_Mushroom_J_4182263170",
0xddfecc39,"MainField_Item_Mushroom_J_4189232248",
0xd11d06e7,"MainField_Item_Mushroom_J_4192583801",
0x0b1f3821,"MainField_Item_Mushroom_J_4201714248",
0x49215c03,"MainField_Item_Mushroom_J_4215212744",
0xd02b8d74,"MainField_Item_Mushroom_J_4228844668",
0xa222d9a6,"MainField_Item_Mushroom_J_4238179005",
0xca7a7eca,"MainField_Item_Mushroom_J_4248252728",
0x9ff206a9,"MainField_Item_Mushroom_J_4269591421",
0xd1458920,"MainField_Item_Mushroom_J_440715879",
0x7f808526,"MainField_Item_Mushroom_J_44148069",
0x71c7791d,"MainField_Item_Mushroom_J_44219590",
0x608cf6fa,"MainField_Item_Mushroom_J_444906511",
0x2a20074f,"MainField_Item_Mushroom_J_474274726",
0x70e7a34d,"MainField_Item_Mushroom_J_474518435",
0xe66b28f9,"MainField_Item_Mushroom_J_475380701",
0x637616df,"MainField_Item_Mushroom_J_475697259",
0x55eff5bd,"MainField_Item_Mushroom_J_477123125",
0x03de086c,"MainField_Item_Mushroom_J_479682527",
0x13266be3,"MainField_Item_Mushroom_J_482689862",
0x56c05e41,"MainField_Item_Mushroom_J_488651215",
0xd99ac218,"MainField_Item_Mushroom_J_488849586",
0x1ef713ad,"MainField_Item_Mushroom_J_495187070",
0xdacab7bb,"MainField_Item_Mushroom_J_498251637",
0x24c873e3,"MainField_Item_Mushroom_J_505359634",
0x6cff1037,"MainField_Item_Mushroom_J_518377160",
0xd8c301bd,"MainField_Item_Mushroom_J_521348806",
0xa381610b,"MainField_Item_Mushroom_J_532674671",
0x210dd6f3,"MainField_Item_Mushroom_J_564837470",
0xa90afdb9,"MainField_Item_Mushroom_J_567793032",
0x33611b1e,"MainField_Item_Mushroom_J_568396700",
0x552e973f,"MainField_Item_Mushroom_J_572350702",
0xcd9c528e,"MainField_Item_Mushroom_J_576758973",
0x932a177a,"MainField_Item_Mushroom_J_58325912",
0x6ea0cea7,"MainField_Item_Mushroom_J_621065915",
0xe74f3f37,"MainField_Item_Mushroom_J_624931122",
0xd5b009f8,"MainField_Item_Mushroom_J_634711440",
0xbe7b79f8,"MainField_Item_Mushroom_J_638617465",
0x87709fee,"MainField_Item_Mushroom_J_64492264",
0x310f4e91,"MainField_Item_Mushroom_J_662977671",
0xc8a2f3a0,"MainField_Item_Mushroom_J_670529632",
0x7ceab214,"MainField_Item_Mushroom_J_679760830",
0xaaae1cc6,"MainField_Item_Mushroom_J_685070936",
0x826c0ad4,"MainField_Item_Mushroom_J_686685985",
0xb440613f,"MainField_Item_Mushroom_J_688979106",
0x57941ae0,"MainField_Item_Mushroom_J_696827201",
0x7b61d087,"MainField_Item_Mushroom_J_697019433",
0x39bc3674,"MainField_Item_Mushroom_J_71550073",
0x6feb098f,"MainField_Item_Mushroom_J_716289213",
0x50e3a36c,"MainField_Item_Mushroom_J_716350712",
0xf099408c,"MainField_Item_Mushroom_J_7177008",
0xf4119bdc,"MainField_Item_Mushroom_J_718921127",
0xa7e60e65,"MainField_Item_Mushroom_J_720759255",
0x14477110,"MainField_Item_Mushroom_J_730036666",
0x48b6f3c7,"MainField_Item_Mushroom_J_731520251",
0x1e5ac944,"MainField_Item_Mushroom_J_733319365",
0x487708d9,"MainField_Item_Mushroom_J_744055217",
0x3b5a9ae0,"MainField_Item_Mushroom_J_753412055",
0xda88282c,"MainField_Item_Mushroom_J_764374418",
0xa25e7b81,"MainField_Item_Mushroom_J_766820673",
0x45625657,"MainField_Item_Mushroom_J_778209751",
0xd2adf592,"MainField_Item_Mushroom_J_808389885",
0x184f802c,"MainField_Item_Mushroom_J_811306774",
0x5f2ae62b,"MainField_Item_Mushroom_J_818606063",
0x1f81db76,"MainField_Item_Mushroom_J_834696004",
0x6304793e,"MainField_Item_Mushroom_J_850769631",
0x017a961a,"MainField_Item_Mushroom_J_869102227",
0x2142115f,"MainField_Item_Mushroom_J_873197122",
0xca0b91f3,"MainField_Item_Mushroom_J_879472921",
0xad816d1f,"MainField_Item_Mushroom_J_883719642",
0xb5a8da6b,"MainField_Item_Mushroom_J_89367266",
0x9e8a2822,"MainField_Item_Mushroom_J_894156063",
0x78a21128,"MainField_Item_Mushroom_J_897524707",
0xfc2fb79e,"MainField_Item_Mushroom_J_898418234",
0x0dbb049d,"MainField_Item_Mushroom_J_899031053",
0x33745997,"MainField_Item_Mushroom_J_913331671",
0x44aa6658,"MainField_Item_Mushroom_J_917600645",
0x572c13be,"MainField_Item_Mushroom_J_925636901",
0x050c9f20,"MainField_Item_Mushroom_J_929132350",
0x9f5498a7,"MainField_Item_Mushroom_J_929415264",
0x3310a397,"MainField_Item_Mushroom_J_937815541",
0x91ac4273,"MainField_Item_Mushroom_J_957200798",
0x0a12a394,"MainField_Item_Mushroom_J_959140144",
0x4c075564,"MainField_Item_Mushroom_J_960939941",
0x9605ea1b,"MainField_Item_Mushroom_J_980992170",
0xa042b9d7,"MainField_Item_Mushroom_J_98756191",
0x8dd9d02b,"MainField_Item_Mushroom_J_99715663",
0x82524088,"MainField_Item_Mushroom_L_1020233056",
0x57c88b97,"MainField_Item_Mushroom_L_1053437782",
0x7fa620b6,"MainField_Item_Mushroom_L_1064103226",
0xf651208c,"MainField_Item_Mushroom_L_1076584820",
0x28f0d5b7,"MainField_Item_Mushroom_L_1113956121",
0x082ab41f,"MainField_Item_Mushroom_L_1135250051",
0x341ed0de,"MainField_Item_Mushroom_L_1135375737",
0xaa6e27b8,"MainField_Item_Mushroom_L_11608304",
0x4968f1bc,"MainField_Item_Mushroom_L_1182531812",
0x8d674a82,"MainField_Item_Mushroom_L_1197748638",
0x01b087c4,"MainField_Item_Mushroom_L_1215587000",
0x55ae8178,"MainField_Item_Mushroom_L_1220194028",
0xb5d991c1,"MainField_Item_Mushroom_L_1220719080",
0xe3411b64,"MainField_Item_Mushroom_L_1232638680",
0x2fd98bf6,"MainField_Item_Mushroom_L_1278387468",
0xc8fe86b7,"MainField_Item_Mushroom_L_1290928134",
0x980bda4b,"MainField_Item_Mushroom_L_1321340450",
0x382539c8,"MainField_Item_Mushroom_L_1324994983",
0xd18e287c,"MainField_Item_Mushroom_L_1331304942",
0x2f69d378,"MainField_Item_Mushroom_L_1372406650",
0xef472964,"MainField_Item_Mushroom_L_1374386416",
0x0410c2f3,"MainField_Item_Mushroom_L_1392321762",
0xb5f24be3,"MainField_Item_Mushroom_L_1409572009",
0x459be1aa,"MainField_Item_Mushroom_L_1420361572",
0x5b2d0d62,"MainField_Item_Mushroom_L_1460752216",
0x41c922b3,"MainField_Item_Mushroom_L_1461770107",
0xa156a1a0,"MainField_Item_Mushroom_L_1467641230",
0xc3243a89,"MainField_Item_Mushroom_L_1468358423",
0xdb277f54,"MainField_Item_Mushroom_L_1497804058",
0x3d95fcaf,"MainField_Item_Mushroom_L_1498191654",
0x6eb808fb,"MainField_Item_Mushroom_L_1507654028",
0x11f9467d,"MainField_Item_Mushroom_L_1518410197",
0xda6083ba,"MainField_Item_Mushroom_L_1526131167",
0x2a5c0f18,"MainField_Item_Mushroom_L_1559738011",
0x5b1f8419,"MainField_Item_Mushroom_L_1573681879",
0xdb69c8e4,"MainField_Item_Mushroom_L_1585404229",
0x050c16e9,"MainField_Item_Mushroom_L_159625707",
0xfa9bcac0,"MainField_Item_Mushroom_L_1599806462",
0x1f779560,"MainField_Item_Mushroom_L_1643339569",
0x833cdc8a,"MainField_Item_Mushroom_L_166387677",
0xf172962a,"MainField_Item_Mushroom_L_1675202358",
0xfd4f3b13,"MainField_Item_Mushroom_L_1703719581",
0x3c028440,"MainField_Item_Mushroom_L_1714141399",
0x51ea0474,"MainField_Item_Mushroom_L_1719034502",
0x89f82de7,"MainField_Item_Mushroom_L_1719953700",
0x068c8499,"MainField_Item_Mushroom_L_1730140403",
0x70a9370b,"MainField_Item_Mushroom_L_1731204033",
0x80d59659,"MainField_Item_Mushroom_L_1740000034",
0xb5899d0b,"MainField_Item_Mushroom_L_1757018502",
0x5e76119c,"MainField_Item_Mushroom_L_1793554070",
0x0b1639d6,"MainField_Item_Mushroom_L_1869590712",
0x3078f267,"MainField_Item_Mushroom_L_1885439253",
0x9d38afa1,"MainField_Item_Mushroom_L_189649535",
0x5b1826d0,"MainField_Item_Mushroom_L_1897013168",
0x3e3f3344,"MainField_Item_Mushroom_L_1899206030",
0x9733d61d,"MainField_Item_Mushroom_L_192759919",
0x403b3c87,"MainField_Item_Mushroom_L_1940727322",
0x01db65dd,"MainField_Item_Mushroom_L_1971072683",
0x3024b915,"MainField_Item_Mushroom_L_1997850730",
0x68173463,"MainField_Item_Mushroom_L_200809110",
0x9a8a3450,"MainField_Item_Mushroom_L_2010836350",
0x4f4b1d4d,"MainField_Item_Mushroom_L_2018159248",
0x6e168220,"MainField_Item_Mushroom_L_2026033366",
0x8f15d13c,"MainField_Item_Mushroom_L_2037557744",
0xfde36b24,"MainField_Item_Mushroom_L_2040604413",
0x3df5b149,"MainField_Item_Mushroom_L_2053541647",
0xbf871b7d,"MainField_Item_Mushroom_L_207704408",
0x0ea61eeb,"MainField_Item_Mushroom_L_2090174721",
0xae51b2d6,"MainField_Item_Mushroom_L_2132400307",
0xe42d5271,"MainField_Item_Mushroom_L_2142206886",
0x5eefc0f0,"MainField_Item_Mushroom_L_2150150291",
0x0b90b245,"MainField_Item_Mushroom_L_2152901543",
0x9633c41a,"MainField_Item_Mushroom_L_2169397440",
0x5832f6dd,"MainField_Item_Mushroom_L_2228483818",
0x2fc20d6b,"MainField_Item_Mushroom_L_2248949562",
0x42a8e9ab,"MainField_Item_Mushroom_L_2264106018",
0xcc61f00f,"MainField_Item_Mushroom_L_228828108",
0x9da976f8,"MainField_Item_Mushroom_L_2291120149",
0x9cb5cbd9,"MainField_Item_Mushroom_L_2308359704",
0xe9cca3a8,"MainField_Item_Mushroom_L_2333217661",
0xa414ff50,"MainField_Item_Mushroom_L_2334907047",
0xad8be045,"MainField_Item_Mushroom_L_2344881182",
0xa20e6a83,"MainField_Item_Mushroom_L_2368069271",
0xd0500756,"MainField_Item_Mushroom_L_2381906728",
0x30fd8c13,"MainField_Item_Mushroom_L_2385274231",
0xd3485be3,"MainField_Item_Mushroom_L_2396265536",
0x5f017a0f,"MainField_Item_Mushroom_L_2417923057",
0x18a8115f,"MainField_Item_Mushroom_L_2481551589",
0xc61c50d6,"MainField_Item_Mushroom_L_2502724539",
0x105a719f,"MainField_Item_Mushroom_L_2520842551",
0x77d243d1,"MainField_Item_Mushroom_L_2524876739",
0x7b7e3c83,"MainField_Item_Mushroom_L_2544556431",
0x658601a7,"MainField_Item_Mushroom_L_2546405788",
0xa7ac5473,"MainField_Item_Mushroom_L_2550828239",
0x1e25afe3,"MainField_Item_Mushroom_L_2553076841",
0xcb5bd3c2,"MainField_Item_Mushroom_L_2582277000",
0xdfcf49b9,"MainField_Item_Mushroom_L_2592397027",
0x2cc8d891,"MainField_Item_Mushroom_L_2602062393",
0xd71ed900,"MainField_Item_Mushroom_L_263336282",
0xb96a864e,"MainField_Item_Mushroom_L_2684839952",
0x126dbf64,"MainField_Item_Mushroom_L_2696910591",
0xfa90a556,"MainField_Item_Mushroom_L_2743574357",
0x141246e0,"MainField_Item_Mushroom_L_2819582956",
0xbae9ee98,"MainField_Item_Mushroom_L_2844040538",
0x9953915c,"MainField_Item_Mushroom_L_2846622444",
0x37151a24,"MainField_Item_Mushroom_L_286746427",
0x2dc3af2f,"MainField_Item_Mushroom_L_2874381863",
0xb5028e2c,"MainField_Item_Mushroom_L_289129322",
0x1893b81d,"MainField_Item_Mushroom_L_2901420783",
0xd765d77d,"MainField_Item_Mushroom_L_2901677213",
0x794852d7,"MainField_Item_Mushroom_L_2927460753",
0xd76c2a3d,"MainField_Item_Mushroom_L_2943946403",
0x0d2bb38c,"MainField_Item_Mushroom_L_2947847532",
0xd1a9762a,"MainField_Item_Mushroom_L_2999562661",
0x7f6122cb,"MainField_Item_Mushroom_L_3035784570",
0x16d8bff6,"MainField_Item_Mushroom_L_3037176329",
0x06616592,"MainField_Item_Mushroom_L_3054373926",
0x2fcca7b8,"MainField_Item_Mushroom_L_3101339048",
0x683686dd,"MainField_Item_Mushroom_L_311436208",
0xc0f4d692,"MainField_Item_Mushroom_L_3114751367",
0x856d5644,"MainField_Item_Mushroom_L_3138943191",
0x5859bfe1,"MainField_Item_Mushroom_L_3191445657",
0xc15f1e70,"MainField_Item_Mushroom_L_3208475042",
0x0437a48e,"MainField_Item_Mushroom_L_3210490096",
0x487392c3,"MainField_Item_Mushroom_L_3215991438",
0xca69bd2a,"MainField_Item_Mushroom_L_3230934780",
0x45da4307,"MainField_Item_Mushroom_L_3259636393",
0x2d2b753f,"MainField_Item_Mushroom_L_3262681121",
0x420f17a9,"MainField_Item_Mushroom_L_329273813",
0x85e4c9cd,"MainField_Item_Mushroom_L_329546382",
0x7ae65b2b,"MainField_Item_Mushroom_L_3303572305",
0x48fd14fe,"MainField_Item_Mushroom_L_3323859618",
0xa3432bd0,"MainField_Item_Mushroom_L_3324080783",
0x6926263c,"MainField_Item_Mushroom_L_3333702645",
0x05cb8373,"MainField_Item_Mushroom_L_3367483163",
0x1cae0dd9,"MainField_Item_Mushroom_L_3385900267",
0xa2076cf2,"MainField_Item_Mushroom_L_3397585568",
0xd98c3543,"MainField_Item_Mushroom_L_3411234386",
0x9599ce0e,"MainField_Item_Mushroom_L_3417334332",
0x389b2480,"MainField_Item_Mushroom_L_3464152633",
0xf1dcc7f9,"MainField_Item_Mushroom_L_3488956567",
0x29c105e4,"MainField_Item_Mushroom_L_3515518610",
0x141bf769,"MainField_Item_Mushroom_L_3530156562",
0xd457f30d,"MainField_Item_Mushroom_L_3557773757",
0x7567c843,"MainField_Item_Mushroom_L_3581845430",
0xc9cb4c6d,"MainField_Item_Mushroom_L_3664231158",
0xabe18850,"MainField_Item_Mushroom_L_3713700644",
0xc79aa6a0,"MainField_Item_Mushroom_L_372224218",
0x6e9cff5a,"MainField_Item_Mushroom_L_3737342573",
0x9a436fdb,"MainField_Item_Mushroom_L_374287701",
0x5ab72f0b,"MainField_Item_Mushroom_L_3743556805",
0x5155a4c9,"MainField_Item_Mushroom_L_3751771842",
0x12b45e9f,"MainField_Item_Mushroom_L_3778131120",
0x0b61174a,"MainField_Item_Mushroom_L_3781181745",
0xb07d20a9,"MainField_Item_Mushroom_L_3796652764",
0x8684e9e6,"MainField_Item_Mushroom_L_380979274",
0x29641a0b,"MainField_Item_Mushroom_L_3818222085",
0xb18afc42,"MainField_Item_Mushroom_L_3836002700",
0xad023d87,"MainField_Item_Mushroom_L_3839355696",
0x95b9e724,"MainField_Item_Mushroom_L_3840784997",
0xb79a7f42,"MainField_Item_Mushroom_L_3880756815",
0x14404292,"MainField_Item_Mushroom_L_3908568994",
0x80f992de,"MainField_Item_Mushroom_L_3938567707",
0x22175eda,"MainField_Item_Mushroom_L_3941324652",
0xb498cd02,"MainField_Item_Mushroom_L_3950246942",
0x1773fbe4,"MainField_Item_Mushroom_L_3958289342",
0x18aba01c,"MainField_Item_Mushroom_L_396476403",
0xde3bb6db,"MainField_Item_Mushroom_L_3972420893",
0xa0dce445,"MainField_Item_Mushroom_L_3986208958",
0x80b899ba,"MainField_Item_Mushroom_L_3995115502",
0x9c25a28c,"MainField_Item_Mushroom_L_4009032170",
0x2cf61d1c,"MainField_Item_Mushroom_L_4012310915",
0xe610de9b,"MainField_Item_Mushroom_L_4012884121",
0x596d1a32,"MainField_Item_Mushroom_L_4043239875",
0xe434fb9f,"MainField_Item_Mushroom_L_404777737",
0x734e243d,"MainField_Item_Mushroom_L_4051756188",
0xef7e97fd,"MainField_Item_Mushroom_L_4058587136",
0x9846fc7c,"MainField_Item_Mushroom_L_409714579",
0xacb0ab10,"MainField_Item_Mushroom_L_4113276079",
0xc01732bf,"MainField_Item_Mushroom_L_4113650403",
0xa538cece,"MainField_Item_Mushroom_L_4144963822",
0x7f2f0511,"MainField_Item_Mushroom_L_422959123",
0xb003d907,"MainField_Item_Mushroom_L_4270347594",
0x5c1c49e5,"MainField_Item_Mushroom_L_4292363670",
0xeee454a2,"MainField_Item_Mushroom_L_456898058",
0x06cf1f46,"MainField_Item_Mushroom_L_480134929",
0xe8310468,"MainField_Item_Mushroom_L_49642472",
0xed580b44,"MainField_Item_Mushroom_L_51439201",
0x9bf5a212,"MainField_Item_Mushroom_L_534896576",
0x86b99cca,"MainField_Item_Mushroom_L_554221579",
0x10048a07,"MainField_Item_Mushroom_L_565056706",
0xd57f2798,"MainField_Item_Mushroom_L_571354128",
0x78af79d9,"MainField_Item_Mushroom_L_572531480",
0x35bf0044,"MainField_Item_Mushroom_L_582104981",
0xcb3f9557,"MainField_Item_Mushroom_L_584045023",
0x8f81d00b,"MainField_Item_Mushroom_L_591766851",
0xd0623519,"MainField_Item_Mushroom_L_624357408",
0xfefd3545,"MainField_Item_Mushroom_L_624404802",
0x0e8204f2,"MainField_Item_Mushroom_L_63145238",
0x79e1c208,"MainField_Item_Mushroom_L_652083475",
0xf6b345fd,"MainField_Item_Mushroom_L_670077748",
0x037fab9e,"MainField_Item_Mushroom_L_695371805",
0x0da21cbc,"MainField_Item_Mushroom_L_708109361",
0x0cc5a1c6,"MainField_Item_Mushroom_L_724762965",
0x79c43c78,"MainField_Item_Mushroom_L_729196757",
0x0f87f579,"MainField_Item_Mushroom_L_736116203",
0xe35ec313,"MainField_Item_Mushroom_L_758153682",
0x2b43c187,"MainField_Item_Mushroom_L_769895304",
0x9a6529c8,"MainField_Item_Mushroom_L_793401028",
0x649078aa,"MainField_Item_Mushroom_L_796205180",
0x7f78b6b7,"MainField_Item_Mushroom_L_79668123",
0x9ec3c9d4,"MainField_Item_Mushroom_L_800511228",
0x055a1f6e,"MainField_Item_Mushroom_L_821713595",
0x6d8c292c,"MainField_Item_Mushroom_L_827714220",
0x1fc7aa32,"MainField_Item_Mushroom_L_839678494",
0x5fbbb50d,"MainField_Item_Mushroom_L_852361678",
0x0cc39518,"MainField_Item_Mushroom_L_888234841",
0xb5054b6b,"MainField_Item_Mushroom_L_925388689",
0x05bd1797,"MainField_Item_Mushroom_L_933122688",
0x94e1e6d6,"MainField_Item_Mushroom_L_957118174",
0xa5fa9741,"MainField_Item_Mushroom_L_972228199",
0x4fc0bcef,"MainField_Item_Mushroom_L_97455165",
0xa7aa9546,"MainField_Item_Mushroom_L_977643775",
0x5d0b1179,"MainField_Item_Mushroom_L_978374833",
0xe667b9a0,"MainField_Item_Mushroom_M_1022354661",
0x7d66c9f7,"MainField_Item_Mushroom_M_1043314434",
0x8ba1cace,"MainField_Item_Mushroom_M_1049408618",
0x57947d26,"MainField_Item_Mushroom_M_1057334125",
0x60b5457c,"MainField_Item_Mushroom_M_1062147425",
0xa4d88d59,"MainField_Item_Mushroom_M_1065813761",
0xa62cf8c2,"MainField_Item_Mushroom_M_1081023944",
0x71f1344e,"MainField_Item_Mushroom_M_1084960734",
0x92d0970e,"MainField_Item_Mushroom_M_1089262315",
0x22fd0278,"MainField_Item_Mushroom_M_1092270946",
0x236e85c8,"MainField_Item_Mushroom_M_1111638526",
0x179b23c7,"MainField_Item_Mushroom_M_1114082558",
0x7499af0d,"MainField_Item_Mushroom_M_1153541190",
0xff20ac39,"MainField_Item_Mushroom_M_1172714055",
0x5f5cd7a0,"MainField_Item_Mushroom_M_1181616930",
0x070d687d,"MainField_Item_Mushroom_M_1237339251",
0x1744adfa,"MainField_Item_Mushroom_M_1255624173",
0xcaf27468,"MainField_Item_Mushroom_M_1259489879",
0xb85fac81,"MainField_Item_Mushroom_M_1265821147",
0x9822aad4,"MainField_Item_Mushroom_M_1268622556",
0x061eb5b1,"MainField_Item_Mushroom_M_1274852913",
0x24f70c22,"MainField_Item_Mushroom_M_1295564963",
0x71f8a8b3,"MainField_Item_Mushroom_M_1298933094",
0x580f8bd2,"MainField_Item_Mushroom_M_1325326905",
0x35834684,"MainField_Item_Mushroom_M_1344479916",
0xb43c98d7,"MainField_Item_Mushroom_M_1380347644",
0x88a240fd,"MainField_Item_Mushroom_M_1388889182",
0xaf937f0a,"MainField_Item_Mushroom_M_1421277372",
0x85cd7112,"MainField_Item_Mushroom_M_1441431570",
0x2ac8a8e4,"MainField_Item_Mushroom_M_1448437639",
0xfcd79f41,"MainField_Item_Mushroom_M_1472282671",
0xb3f2bb6c,"MainField_Item_Mushroom_M_1482524498",
0x0c5bd78e,"MainField_Item_Mushroom_M_1483554205",
0xc8366249,"MainField_Item_Mushroom_M_1485539323",
0x362bb3f0,"MainField_Item_Mushroom_M_1487214835",
0xdbcf72c5,"MainField_Item_Mushroom_M_1571522215",
0x152588cd,"MainField_Item_Mushroom_M_1584770659",
0x56ceec6d,"MainField_Item_Mushroom_M_1592655215",
0x1ef3ca95,"MainField_Item_Mushroom_M_1596488907",
0x4e397de0,"MainField_Item_Mushroom_M_1599407864",
0x959a6c72,"MainField_Item_Mushroom_M_1617958545",
0xbe13a37f,"MainField_Item_Mushroom_M_1694906843",
0x6111b0e2,"MainField_Item_Mushroom_M_1697200687",
0xa096053f,"MainField_Item_Mushroom_M_1712217276",
0xc7f556c1,"MainField_Item_Mushroom_M_1728171536",
0xfc5b85e5,"MainField_Item_Mushroom_M_1737037322",
0x659dc24f,"MainField_Item_Mushroom_M_1764639351",
0x80e1dbd5,"MainField_Item_Mushroom_M_1779072533",
0x3de89ecc,"MainField_Item_Mushroom_M_178138360",
0xdb3a9610,"MainField_Item_Mushroom_M_1782514567",
0xb055efd7,"MainField_Item_Mushroom_M_1799964599",
0xe253bd54,"MainField_Item_Mushroom_M_1808718887",
0xa0c0ecdd,"MainField_Item_Mushroom_M_1824124525",
0x1f29e7ed,"MainField_Item_Mushroom_M_1854318089",
0x0981abca,"MainField_Item_Mushroom_M_1855128151",
0x812f366b,"MainField_Item_Mushroom_M_1857896391",
0x2db8b797,"MainField_Item_Mushroom_M_1865624112",
0x133dfc94,"MainField_Item_Mushroom_M_1891608294",
0x2e92fa57,"MainField_Item_Mushroom_M_1893257854",
0xbfe88096,"MainField_Item_Mushroom_M_189737586",
0xdb48933b,"MainField_Item_Mushroom_M_1940311165",
0xb56fc9f7,"MainField_Item_Mushroom_M_194070947",
0xaf44f906,"MainField_Item_Mushroom_M_1964284045",
0x28ecc571,"MainField_Item_Mushroom_M_1990728406",
0xcf0726a3,"MainField_Item_Mushroom_M_2002167366",
0xdc38e416,"MainField_Item_Mushroom_M_200792874",
0x91081c6c,"MainField_Item_Mushroom_M_2012260124",
0x49fe5a9a,"MainField_Item_Mushroom_M_2022441101",
0xffbf5a27,"MainField_Item_Mushroom_M_2040503951",
0xb3399fd4,"MainField_Item_Mushroom_M_2064194847",
0xdde2fe76,"MainField_Item_Mushroom_M_2085981480",
0x051adc4d,"MainField_Item_Mushroom_M_2092617980",
0x719a0d7c,"MainField_Item_Mushroom_M_2096847433",
0x2de3e7b9,"MainField_Item_Mushroom_M_2098676958",
0xd63074f0,"MainField_Item_Mushroom_M_209904663",
0x9dddd4b9,"MainField_Item_Mushroom_M_2119891862",
0x9f9e5703,"MainField_Item_Mushroom_M_2137568557",
0x307bc543,"MainField_Item_Mushroom_M_2162255743",
0x3061b638,"MainField_Item_Mushroom_M_2164475720",
0x43b62b87,"MainField_Item_Mushroom_M_2164977831",
0x880c9233,"MainField_Item_Mushroom_M_2200272467",
0x07aafab7,"MainField_Item_Mushroom_M_2215687412",
0x55474c06,"MainField_Item_Mushroom_M_2220543437",
0xfbd953c4,"MainField_Item_Mushroom_M_2232314536",
0xb37cbe4d,"MainField_Item_Mushroom_M_2252402070",
0x2fa86d88,"MainField_Item_Mushroom_M_2262537353",
0x034857e1,"MainField_Item_Mushroom_M_2297579783",
0xbef5325c,"MainField_Item_Mushroom_M_2325439051",
0x09e35f44,"MainField_Item_Mushroom_M_2329394098",
0x54e2d07c,"MainField_Item_Mushroom_M_2329816950",
0xc89343a3,"MainField_Item_Mushroom_M_2331711454",
0x9d879c4e,"MainField_Item_Mushroom_M_2342244662",
0x9415c6cd,"MainField_Item_Mushroom_M_2353646002",
0x43512d84,"MainField_Item_Mushroom_M_23574848",
0xf51d0e76,"MainField_Item_Mushroom_M_2376327743",
0x484b658c,"MainField_Item_Mushroom_M_2425711072",
0xb72b8769,"MainField_Item_Mushroom_M_2429549326",
0xf9a50024,"MainField_Item_Mushroom_M_2438282943",
0x444c43ee,"MainField_Item_Mushroom_M_2451906584",
0xfea18f54,"MainField_Item_Mushroom_M_2458537819",
0x7043ddd3,"MainField_Item_Mushroom_M_2470532076",
0xf455baf2,"MainField_Item_Mushroom_M_2474115123",
0xe72c9e4b,"MainField_Item_Mushroom_M_2475441904",
0x16b64757,"MainField_Item_Mushroom_M_2492983443",
0x53775215,"MainField_Item_Mushroom_M_252796407",
0x08537906,"MainField_Item_Mushroom_M_2594756948",
0x5d63b4b2,"MainField_Item_Mushroom_M_2599246696",
0x80c49cd8,"MainField_Item_Mushroom_M_2614147360",
0x3a9c2e90,"MainField_Item_Mushroom_M_2633622836",
0xe7d38621,"MainField_Item_Mushroom_M_264012319",
0x2b7f2141,"MainField_Item_Mushroom_M_2644519204",
0x21c1c799,"MainField_Item_Mushroom_M_2673371598",
0x9c56c39b,"MainField_Item_Mushroom_M_2680903420",
0xa78a4957,"MainField_Item_Mushroom_M_271342316",
0x801e40df,"MainField_Item_Mushroom_M_2737517044",
0x62a86d09,"MainField_Item_Mushroom_M_2760370067",
0x2e00b3a8,"MainField_Item_Mushroom_M_2772828466",
0x0f909b53,"MainField_Item_Mushroom_M_2791872210",
0xdcf70247,"MainField_Item_Mushroom_M_2799611131",
0xeef6292b,"MainField_Item_Mushroom_M_2823020811",
0xd0aa3f92,"MainField_Item_Mushroom_M_2829024301",
0xe0a09591,"MainField_Item_Mushroom_M_2829698327",
0xbf91066c,"MainField_Item_Mushroom_M_2840515757",
0x65cc90d6,"MainField_Item_Mushroom_M_2863870670",
0x62f31dd6,"MainField_Item_Mushroom_M_2879435907",
0x8a402d5c,"MainField_Item_Mushroom_M_288041004",
0x30317aa7,"MainField_Item_Mushroom_M_2883890137",
0xdaf79d0d,"MainField_Item_Mushroom_M_2920821238",
0x6b14a602,"MainField_Item_Mushroom_M_2921266743",
0x230976a7,"MainField_Item_Mushroom_M_2924841391",
0xa6181628,"MainField_Item_Mushroom_M_2933079893",
0xb8f26d75,"MainField_Item_Mushroom_M_2935749344",
0xe03b0672,"MainField_Item_Mushroom_M_2944334574",
0x2beffae4,"MainField_Item_Mushroom_M_2956773369",
0x2b91d464,"MainField_Item_Mushroom_M_2975743818",
0xb36c9d8d,"MainField_Item_Mushroom_M_2982901743",
0xee887a92,"MainField_Item_Mushroom_M_2988010776",
0x8a2396d2,"MainField_Item_Mushroom_M_2989003197",
0x09eb148d,"MainField_Item_Mushroom_M_3052685790",
0x57c2e147,"MainField_Item_Mushroom_M_3059456861",
0x93abc62d,"MainField_Item_Mushroom_M_3072898643",
0xc03808c1,"MainField_Item_Mushroom_M_3081588788",
0x09981aed,"MainField_Item_Mushroom_M_3082496530",
0x729ad379,"MainField_Item_Mushroom_M_308581327",
0x7381f3d5,"MainField_Item_Mushroom_M_3113658756",
0x76cf51e9,"MainField_Item_Mushroom_M_3141448271",
0x9b01c0d7,"MainField_Item_Mushroom_M_3148169077",
0x5fb66737,"MainField_Item_Mushroom_M_3159328905",
0x950a8d3c,"MainField_Item_Mushroom_M_3170411678",
0xb73ce184,"MainField_Item_Mushroom_M_3170981094",
0xc405ec0f,"MainField_Item_Mushroom_M_3171626661",
0x6b1f4a5a,"MainField_Item_Mushroom_M_3174087143",
0xf530cae8,"MainField_Item_Mushroom_M_3190676748",
0xd641879c,"MainField_Item_Mushroom_M_3215713413",
0x5f9bea4e,"MainField_Item_Mushroom_M_323143179",
0xbcf4d749,"MainField_Item_Mushroom_M_3237629026",
0x5157cf98,"MainField_Item_Mushroom_M_3239121585",
0xdc0f9f5c,"MainField_Item_Mushroom_M_3274148786",
0x061f4c70,"MainField_Item_Mushroom_M_3298370057",
0x1f3247f3,"MainField_Item_Mushroom_M_3318375804",
0x76d799d9,"MainField_Item_Mushroom_M_3330971496",
0x5be5e48c,"MainField_Item_Mushroom_M_3350603392",
0x3fa79757,"MainField_Item_Mushroom_M_3366178667",
0xc9f2ed39,"MainField_Item_Mushroom_M_3374672041",
0xd0e1a505,"MainField_Item_Mushroom_M_339278880",
0x38d32884,"MainField_Item_Mushroom_M_3411196075",
0x31a650a5,"MainField_Item_Mushroom_M_3442462141",
0x35c5ef6f,"MainField_Item_Mushroom_M_3462151064",
0x843c6bd6,"MainField_Item_Mushroom_M_3513564966",
0x2d663f0a,"MainField_Item_Mushroom_M_352985557",
0x928d8784,"MainField_Item_Mushroom_M_3530257603",
0x45b77089,"MainField_Item_Mushroom_M_3536509710",
0x40b543ab,"MainField_Item_Mushroom_M_3565637897",
0x976ea01d,"MainField_Item_Mushroom_M_3569466311",
0xeb0f3d77,"MainField_Item_Mushroom_M_3591611103",
0xb5eb0f2f,"MainField_Item_Mushroom_M_3605894982",
0x016ef9ba,"MainField_Item_Mushroom_M_3608977203",
0x43b77f2a,"MainField_Item_Mushroom_M_3670011311",
0x133fe676,"MainField_Item_Mushroom_M_3697212537",
0x211cc04d,"MainField_Item_Mushroom_M_3698031283",
0xb46574b8,"MainField_Item_Mushroom_M_3724519695",
0xf2cb8ba8,"MainField_Item_Mushroom_M_3776680367",
0xcc80cb98,"MainField_Item_Mushroom_M_3791485735",
0x584bf855,"MainField_Item_Mushroom_M_3802011286",
0x565d0691,"MainField_Item_Mushroom_M_3805673210",
0xf9cb342e,"MainField_Item_Mushroom_M_3815705265",
0xb1738829,"MainField_Item_Mushroom_M_3834724821",
0x3a9c4502,"MainField_Item_Mushroom_M_3839694610",
0xca1cf1d8,"MainField_Item_Mushroom_M_3872025158",
0x79ad0c7f,"MainField_Item_Mushroom_M_38781115",
0xec865003,"MainField_Item_Mushroom_M_389186378",
0x1a873b45,"MainField_Item_Mushroom_M_3895669945",
0xc4a19155,"MainField_Item_Mushroom_M_3897325933",
0x3a0b5fb1,"MainField_Item_Mushroom_M_3946574063",
0xd6293c5e,"MainField_Item_Mushroom_M_3968498900",
0x1fe0614d,"MainField_Item_Mushroom_M_397309251",
0xb3ece7c4,"MainField_Item_Mushroom_M_4004398096",
0x94d21b6b,"MainField_Item_Mushroom_M_400974526",
0xf921c0ec,"MainField_Item_Mushroom_M_4010368080",
0x188a2515,"MainField_Item_Mushroom_M_4010408285",
0xb21452e4,"MainField_Item_Mushroom_M_4017851741",
0x9276bd9f,"MainField_Item_Mushroom_M_4037866327",
0x74942538,"MainField_Item_Mushroom_M_4047598307",
0x7a46b812,"MainField_Item_Mushroom_M_4059302781",
0x383804a9,"MainField_Item_Mushroom_M_406086214",
0x226b7f03,"MainField_Item_Mushroom_M_406504363",
0xad5b3382,"MainField_Item_Mushroom_M_4069554226",
0x62beb721,"MainField_Item_Mushroom_M_4096318460",
0xc3a54d25,"MainField_Item_Mushroom_M_4099538527",
0xcf7a7a2e,"MainField_Item_Mushroom_M_4122094166",
0x6a71f3ef,"MainField_Item_Mushroom_M_4138806628",
0xf01337c0,"MainField_Item_Mushroom_M_4140349668",
0x1f35025f,"MainField_Item_Mushroom_M_4161645774",
0xe088ac97,"MainField_Item_Mushroom_M_4186939711",
0xb6cad9cb,"MainField_Item_Mushroom_M_4213263842",
0x7de434dc,"MainField_Item_Mushroom_M_4229928530",
0x4136808c,"MainField_Item_Mushroom_M_4256479638",
0xac7848e5,"MainField_Item_Mushroom_M_4257217390",
0x28304cf6,"MainField_Item_Mushroom_M_4262472153",
0x23981a11,"MainField_Item_Mushroom_M_432804859",
0x42abe8f0,"MainField_Item_Mushroom_M_438627603",
0xf3898099,"MainField_Item_Mushroom_M_438929128",
0x127fded5,"MainField_Item_Mushroom_M_442437618",
0xf3c3920d,"MainField_Item_Mushroom_M_462304912",
0x99a78089,"MainField_Item_Mushroom_M_482234555",
0x3b173450,"MainField_Item_Mushroom_M_489060476",
0x55d89246,"MainField_Item_Mushroom_M_509982979",
0x77ffd81f,"MainField_Item_Mushroom_M_514590087",
0x5e3538fb,"MainField_Item_Mushroom_M_535334917",
0xfc420bdc,"MainField_Item_Mushroom_M_558723862",
0x5001eda9,"MainField_Item_Mushroom_M_567124569",
0x8ad8713a,"MainField_Item_Mushroom_M_572945633",
0xa2948281,"MainField_Item_Mushroom_M_583887678",
0xe9e4a060,"MainField_Item_Mushroom_M_590089916",
0x71999c5b,"MainField_Item_Mushroom_M_598842527",
0x89e2ad4f,"MainField_Item_Mushroom_M_599580165",
0xd85df8c2,"MainField_Item_Mushroom_M_625760165",
0x02429586,"MainField_Item_Mushroom_M_626508684",
0x71828678,"MainField_Item_Mushroom_M_663594569",
0xf53cc5fc,"MainField_Item_Mushroom_M_670067656",
0x9bcfe21b,"MainField_Item_Mushroom_M_676854930",
0x5edcf7af,"MainField_Item_Mushroom_M_678553656",
0xbda23918,"MainField_Item_Mushroom_M_696900956",
0xf9bcfdb1,"MainField_Item_Mushroom_M_701168533",
0xd0e328ca,"MainField_Item_Mushroom_M_707012106",
0x39a40adf,"MainField_Item_Mushroom_M_712454373",
0xfd2eb947,"MainField_Item_Mushroom_M_727650255",
0x252d6903,"MainField_Item_Mushroom_M_760115074",
0xc237dae9,"MainField_Item_Mushroom_M_878525931",
0xd140ffd7,"MainField_Item_Mushroom_M_884165992",
0x19c0d570,"MainField_Item_Mushroom_M_884722478",
0xd8ebcd23,"MainField_Item_Mushroom_M_890087877",
0x9961681c,"MainField_Item_Mushroom_M_893107764",
0x685b977e,"MainField_Item_Mushroom_M_908791460",
0x54dc2e79,"MainField_Item_Mushroom_M_919277936",
0xbef28045,"MainField_Item_Mushroom_M_941170974",
0x8f3761e4,"MainField_Item_Mushroom_M_947445024",
0x74443992,"MainField_Item_Mushroom_M_949614927",
0x57a417e9,"MainField_Item_Mushroom_M_956194652",
0xe9a9e470,"MainField_Item_Mushroom_M_965285156",
0x2b92ca97,"MainField_Item_Mushroom_M_980292966",
0x6ce89a54,"MainField_Item_Mushroom_N_100220698",
0x0f1f4cb2,"MainField_Item_Mushroom_N_1003890350",
0xff651165,"MainField_Item_Mushroom_N_1071065670",
0xf68a0374,"MainField_Item_Mushroom_N_1143462770",
0xdc3ab25a,"MainField_Item_Mushroom_N_132011963",
0xd145ebf1,"MainField_Item_Mushroom_N_1387161676",
0x843246de,"MainField_Item_Mushroom_N_1456167142",
0x423cf052,"MainField_Item_Mushroom_N_1523974632",
0x301e86dd,"MainField_Item_Mushroom_N_1557849524",
0x4da92525,"MainField_Item_Mushroom_N_1573742726",
0xd0f9ea7b,"MainField_Item_Mushroom_N_1648270697",
0xa9702dbd,"MainField_Item_Mushroom_N_1741394293",
0x898a2fd9,"MainField_Item_Mushroom_N_1921382454",
0xbafb0b66,"MainField_Item_Mushroom_N_1976286413",
0x909ee2b7,"MainField_Item_Mushroom_N_1986703538",
0x2b058196,"MainField_Item_Mushroom_N_2007960208",
0xd041d668,"MainField_Item_Mushroom_N_2056409060",
0x8dd0ea69,"MainField_Item_Mushroom_N_2192725408",
0x4edfb0c8,"MainField_Item_Mushroom_N_2202843985",
0x069f64c7,"MainField_Item_Mushroom_N_2304671031",
0xde9068ee,"MainField_Item_Mushroom_N_2308180040",
0x85221cc4,"MainField_Item_Mushroom_N_2329350819",
0x8b81c63c,"MainField_Item_Mushroom_N_2371605137",
0xa4b780fc,"MainField_Item_Mushroom_N_2382370472",
0x583caa26,"MainField_Item_Mushroom_N_2511761649",
0xec47c2e3,"MainField_Item_Mushroom_N_2554117528",
0x0c02c0c6,"MainField_Item_Mushroom_N_2555636776",
0xa4fc083c,"MainField_Item_Mushroom_N_2678324073",
0x90acddfa,"MainField_Item_Mushroom_N_2726778847",
0xd1f138ec,"MainField_Item_Mushroom_N_2734078843",
0x23be6189,"MainField_Item_Mushroom_N_2787571721",
0x8721a242,"MainField_Item_Mushroom_N_2868837533",
0x769851b6,"MainField_Item_Mushroom_N_2887300741",
0x4e49dcf9,"MainField_Item_Mushroom_N_2945894518",
0x355cadb1,"MainField_Item_Mushroom_N_3023748480",
0x5557e01b,"MainField_Item_Mushroom_N_3042589747",
0xa85b7b99,"MainField_Item_Mushroom_N_309529388",
0xd89dbc5b,"MainField_Item_Mushroom_N_3190985764",
0xf16990ed,"MainField_Item_Mushroom_N_3357026585",
0xce360218,"MainField_Item_Mushroom_N_3439432135",
0xee84b435,"MainField_Item_Mushroom_N_3457638580",
0xbb7a62f0,"MainField_Item_Mushroom_N_3589802664",
0x38f8df41,"MainField_Item_Mushroom_N_3649489230",
0xcbcceba6,"MainField_Item_Mushroom_N_3780294245",
0x47e12275,"MainField_Item_Mushroom_N_3849680623",
0xbd46bd10,"MainField_Item_Mushroom_N_3867420997",
0xc0103d05,"MainField_Item_Mushroom_N_4056447568",
0x0ba4372d,"MainField_Item_Mushroom_N_4097389668",
0x98728889,"MainField_Item_Mushroom_N_4136779112",
0x957dfffe,"MainField_Item_Mushroom_N_4173114513",
0x5aa40629,"MainField_Item_Mushroom_N_4252563738",
0x55122300,"MainField_Item_Mushroom_N_4263490435",
0x7577eb40,"MainField_Item_Mushroom_N_525165832",
0x3c2a0320,"MainField_Item_Mushroom_N_543110295",
0xf4ce2d78,"MainField_Item_Mushroom_N_658844043",
0x4d0b27b7,"MainField_Item_Mushroom_N_736673038",
0xc9eb1803,"MainField_Item_Mushroom_N_798357288",
0xee6e3270,"MainField_Item_Mushroom_N_811086366",
0x80fc502a,"MainField_Item_Mushroom_N_94708911",
0x56751a4a,"MainField_Item_Mushroom_O_1071226390",
0xccd5c727,"MainField_Item_Mushroom_O_1136469444",
0xfc284148,"MainField_Item_Mushroom_O_1155040075",
0x3d988e5c,"MainField_Item_Mushroom_O_1181936869",
0x57909528,"MainField_Item_Mushroom_O_1192718910",
0x20f56cfe,"MainField_Item_Mushroom_O_1206797422",
0xd832708f,"MainField_Item_Mushroom_O_1259367613",
0x4df7d96f,"MainField_Item_Mushroom_O_129878997",
0xc0a39755,"MainField_Item_Mushroom_O_1317664841",
0x154fa799,"MainField_Item_Mushroom_O_1371284570",
0xe653328d,"MainField_Item_Mushroom_O_1395549501",
0xb35b23da,"MainField_Item_Mushroom_O_1400838262",
0xbe14c6d0,"MainField_Item_Mushroom_O_1502244919",
0x0b547038,"MainField_Item_Mushroom_O_1551305830",
0x5cc116bc,"MainField_Item_Mushroom_O_1594794335",
0xe951a3f3,"MainField_Item_Mushroom_O_1655968073",
0x997e78cb,"MainField_Item_Mushroom_O_1694567109",
0x999ec804,"MainField_Item_Mushroom_O_1734598365",
0x6c83e5e5,"MainField_Item_Mushroom_O_1779635859",
0xd42690e3,"MainField_Item_Mushroom_O_1801770535",
0x79fd73ef,"MainField_Item_Mushroom_O_1969515253",
0x2aa54c11,"MainField_Item_Mushroom_O_2046388415",
0x129328cf,"MainField_Item_Mushroom_O_2059762338",
0xb279ba2d,"MainField_Item_Mushroom_O_2134305562",
0xf9c319c3,"MainField_Item_Mushroom_O_2190694954",
0x851fa732,"MainField_Item_Mushroom_O_2259201787",
0xfb0bf307,"MainField_Item_Mushroom_O_2340588779",
0x0c835060,"MainField_Item_Mushroom_O_2392268215",
0xcaed5a8b,"MainField_Item_Mushroom_O_2393554210",
0x53390af5,"MainField_Item_Mushroom_O_2428480204",
0x5bc9e62f,"MainField_Item_Mushroom_O_2455162438",
0xa678621e,"MainField_Item_Mushroom_O_2468319749",
0xb773e8dc,"MainField_Item_Mushroom_O_2497871104",
0x14b58d17,"MainField_Item_Mushroom_O_2506127632",
0x1296d5f0,"MainField_Item_Mushroom_O_2511590889",
0x36463d68,"MainField_Item_Mushroom_O_2521411483",
0x22be8f86,"MainField_Item_Mushroom_O_2530047964",
0x096e0151,"MainField_Item_Mushroom_O_2574785809",
0xe0b501bc,"MainField_Item_Mushroom_O_2588066724",
0x72ca22f4,"MainField_Item_Mushroom_O_2590495060",
0x20fffcde,"MainField_Item_Mushroom_O_2621620684",
0x4cc4ed4d,"MainField_Item_Mushroom_O_263632912",
0xde8e4445,"MainField_Item_Mushroom_O_2692962012",
0x6954e407,"MainField_Item_Mushroom_O_2733655821",
0xbc001515,"MainField_Item_Mushroom_O_2776468737",
0x16ea8237,"MainField_Item_Mushroom_O_281379264",
0x43a3e744,"MainField_Item_Mushroom_O_2819145354",
0x2a9e2e6f,"MainField_Item_Mushroom_O_2828872132",
0xf4d74bd5,"MainField_Item_Mushroom_O_2898995995",
0x65e5b07f,"MainField_Item_Mushroom_O_2963858900",
0xa41146ae,"MainField_Item_Mushroom_O_3040209861",
0xb08bd380,"MainField_Item_Mushroom_O_3077444072",
0x42bc7bb6,"MainField_Item_Mushroom_O_3165100279",
0xa10b67d7,"MainField_Item_Mushroom_O_3172964779",
0xb5c6160d,"MainField_Item_Mushroom_O_317579297",
0xbf187112,"MainField_Item_Mushroom_O_3289450285",
0xc8437a66,"MainField_Item_Mushroom_O_3316444013",
0x940fe640,"MainField_Item_Mushroom_O_3388901606",
0xb2d07dbb,"MainField_Item_Mushroom_O_3452999068",
0xad53d5dd,"MainField_Item_Mushroom_O_3498853273",
0x43db5c4d,"MainField_Item_Mushroom_O_3570035297",
0x383a7ae0,"MainField_Item_Mushroom_O_3621437473",
0x862462a7,"MainField_Item_Mushroom_O_3647111288",
0xce91ae72,"MainField_Item_Mushroom_O_3665720501",
0xd0256d9d,"MainField_Item_Mushroom_O_3822361975",
0x6956ee3e,"MainField_Item_Mushroom_O_3876953801",
0x036a47f1,"MainField_Item_Mushroom_O_3972953412",
0x0abef210,"MainField_Item_Mushroom_O_4003640288",
0x95bd2265,"MainField_Item_Mushroom_O_4074458565",
0xeb435c43,"MainField_Item_Mushroom_O_4114935729",
0x94cbb493,"MainField_Item_Mushroom_O_4221422067",
0xf5ab96e4,"MainField_Item_Mushroom_O_454397539",
0x9e1d7506,"MainField_Item_Mushroom_O_535151153",
0xc9d99ac4,"MainField_Item_Mushroom_O_585908633",
0x43ea3dde,"MainField_Item_Mushroom_O_63579910",
0x174126d7,"MainField_Item_Mushroom_O_702842759",
0x0402eb47,"MainField_Item_Mushroom_O_711376397",
0x8b55220c,"MainField_Item_Mushroom_O_731044953",
0x7611b5dc,"MainField_Item_Mushroom_O_732060277",
0x35e69527,"MainField_Item_Mushroom_O_791844469",
0x6b9f0e69,"MainField_Item_Mushroom_O_829229549",
0x4f14ff6a,"MainField_Item_Mushroom_O_85154754",
0x13a67133,"MainField_Item_Mushroom_O_87229950",
0x4d44f120,"MainField_Item_Mushroom_O_928769965",
0xe91c1b0f,"MainField_Item_Mushroom_O_959030242",
0xd9e37544,"MainField_Item_Mushroom_O_975918795",
0x341f4248,"MainField_Item_Mushroom_O_985697804",
0x82451ff4,"MainField_Item_Ore_A_1830628255",
0xc539dfc8,"MainField_Item_Ore_B_1059390520",
0x70363c02,"MainField_Item_Ore_B_3400810864",
0xc32f2e02,"MainField_Item_Ore_C_1559572738",
0x27ef8158,"MainField_Item_Ore_D_3540450516",
0x65568454,"MainField_Item_Ore_D_4202838710",
0x4e388b14,"MainField_Item_Ore_E_3561899886",
0x887687fa,"MainField_Item_Ore_F_2621931342",
0xbe8629dd,"MainField_Item_Ore_G_1394799571",
0x887d9c25,"MainField_Item_Ore_G_1538408249",
0x7f2a2aae,"MainField_Item_Ore_G_1909465135",
0x17c614f2,"MainField_Item_Ore_G_2243727295",
0x447e52d6,"MainField_Item_Ore_G_2354277809",
0xb6f07a93,"MainField_Item_Ore_G_2558537994",
0x29a49b2f,"MainField_Item_Ore_G_2583115452",
0xc62ff8bc,"MainField_Item_Ore_G_3138290245",
0x581c1511,"MainField_Item_Ore_G_3684456225",
0x2403e039,"MainField_Item_Ore_G_3783466223",
0x06d2a55e,"MainField_Item_Ore_G_3922471677",
0x1d587b60,"MainField_Item_Ore_G_3951873882",
0x9b86343c,"MainField_Item_Ore_H_2180257902",
0x1e7c58a5,"MainField_Item_Ore_H_2221039470",
0xd828e411,"MainField_Item_Ore_H_2233157358",
0x5387c311,"MainField_Item_Ore_H_2425727025",
0x2853f903,"MainField_Item_Ore_H_2581445106",
0x70b4e004,"MainField_Item_Ore_H_2699587822",
0xa1c90cff,"MainField_Item_Ore_H_2912207186",
0xd5c164e5,"MainField_Item_Ore_H_3081517818",
0x2557289a,"MainField_Item_Ore_H_35471494",
0xf70345b7,"MainField_Item_Ore_H_542004613",
0x354b23a3,"MainField_Item_Ore_I_1129929604",
0x2bca13a7,"MainField_Item_Ore_I_1149191317",
0xe01f11d4,"MainField_Item_Ore_I_1444056112",
0x5b682078,"MainField_Item_Ore_I_1825007262",
0x9f6342b9,"MainField_Item_Ore_I_1999057924",
0x0db2f817,"MainField_Item_Ore_I_260940383",
0xadcf768e,"MainField_Item_Ore_I_2973815753",
0x02d994be,"MainField_Item_Ore_I_3045461527",
0x28b6e190,"MainField_Item_Ore_I_3255828207",
0x334a818c,"MainField_Item_Ore_I_3383398458",
0xc87f6ca8,"MainField_Item_Ore_I_607789393",
0x23bdae83,"MainField_Item_Ore_I_875256204",
0x9825ba71,"MainField_Item_PlantGet_B_867159465",
0x8894af59,"MainField_Item_PlantGet_J_1647535332",
0x4a989a46,"MainField_Item_PlantGet_J_1670725175",
0xb7dcc20a,"MainField_Item_PlantGet_J_1873819232",
0x34024eee,"MainField_Item_PlantGet_J_1956763819",
0xfde1c2c5,"MainField_Item_PlantGet_J_2243443699",
0x6d470a66,"MainField_Item_PlantGet_J_3062089469",
0x22a0df52,"MainField_Item_PlantGet_J_3214963590",
0x6a9fdca5,"MainField_Item_PlantGet_J_3344123994",
0x6ea39be9,"MainField_Item_PlantGet_J_3473063263",
0xd8f94c7d,"MainField_Item_PlantGet_J_3613643571",
0x60175c74,"MainField_Item_PlantGet_J_4065964348",
0xcc8e1c38,"MainField_Item_PlantGet_J_463646823",
0x0442c21d,"MainField_Item_PlantGet_M_1467506359",
0xf0ea60ab,"MainField_Item_PlantGet_M_1872147435",
0x06553d94,"MainField_Item_PlantGet_M_2007291247",
0xb13f9f6f,"MainField_Item_PlantGet_M_2976538619",
0x23b1815d,"MainField_Item_PlantGet_M_3042258939",
0x5c1a167b,"MainField_Item_PlantGet_M_3124829824",
0xd611c95c,"MainField_Item_PlantGet_M_3640152823",
0x59daf5d5,"MainField_Item_PlantGet_M_3717680747",
0x6715f8ec,"MainField_Item_PlantGet_M_3768117189",
0x29cb65d7,"MainField_Item_PlantGet_M_3812284741",
0x5d819c6f,"MainField_Item_PlantGet_M_3907311845",
0x38520fbf,"MainField_Item_PlantGet_M_505115208",
0x0c5b8e5d,"MainField_Item_PlantGet_M_527242471",
0x6fe323f3,"MainField_Item_PlantGet_O_2233581755",
0xb5865494,"MainField_Item_PlantGet_O_2735356229",
0x567fa399,"MainField_Item_PlantGet_O_2884448878",
0x9fa35ace,"MainField_Item_PlantGet_O_3705445624",
0x28d421c2,"MainField_Item_Plant_A_1000078037",
0xa75c1f99,"MainField_Item_Plant_A_1011387551",
0x2da1a9ab,"MainField_Item_Plant_A_1014047803",
0xee85006e,"MainField_Item_Plant_A_1019262420",
0xda2ddbb4,"MainField_Item_Plant_A_1021610471",
0x852367e6,"MainField_Item_Plant_A_1027348735",
0x894dedb8,"MainField_Item_Plant_A_1028201253",
0x4c489894,"MainField_Item_Plant_A_1038198110",
0x6b5e6638,"MainField_Item_Plant_A_1044001174",
0xa5df0287,"MainField_Item_Plant_A_1056430481",
0x31167267,"MainField_Item_Plant_A_1060550713",
0x70895498,"MainField_Item_Plant_A_1081643322",
0x827936f9,"MainField_Item_Plant_A_1089370449",
0xb04d44bf,"MainField_Item_Plant_A_1093070636",
0x1d5c9196,"MainField_Item_Plant_A_1101133498",
0x148bd5b8,"MainField_Item_Plant_A_1117968046",
0xa0a265bc,"MainField_Item_Plant_A_1140352920",
0xefef2382,"MainField_Item_Plant_A_1153531388",
0xcccce7c0,"MainField_Item_Plant_A_1154681390",
0x3390fbe7,"MainField_Item_Plant_A_1163349720",
0x7637fe92,"MainField_Item_Plant_A_1195623109",
0x7de0dd2f,"MainField_Item_Plant_A_1197131648",
0xb86c6312,"MainField_Item_Plant_A_1203125684",
0xcfff7726,"MainField_Item_Plant_A_1205186017",
0x9374021c,"MainField_Item_Plant_A_1213849580",
0xbfb813b0,"MainField_Item_Plant_A_121904738",
0x7f6d5a6b,"MainField_Item_Plant_A_1225732431",
0xc7d0f4c2,"MainField_Item_Plant_A_1247103707",
0x158444fa,"MainField_Item_Plant_A_1258477410",
0x6d5d1337,"MainField_Item_Plant_A_1263226693",
0x0ab9c4de,"MainField_Item_Plant_A_1271366978",
0x191d6165,"MainField_Item_Plant_A_1284740545",
0xea77d35c,"MainField_Item_Plant_A_1294200746",
0xd26f1407,"MainField_Item_Plant_A_130710895",
0x07db7869,"MainField_Item_Plant_A_1320195240",
0xb2dbc35c,"MainField_Item_Plant_A_1323491275",
0xf6596dfb,"MainField_Item_Plant_A_1326156438",
0x06322d5b,"MainField_Item_Plant_A_1348151326",
0xbae88ce6,"MainField_Item_Plant_A_1350794166",
0xc7195857,"MainField_Item_Plant_A_1355309947",
0xcdccdcd3,"MainField_Item_Plant_A_1362307387",
0xe009de0d,"MainField_Item_Plant_A_1380736107",
0x3860c1fa,"MainField_Item_Plant_A_1388313676",
0x3ed696fd,"MainField_Item_Plant_A_139877024",
0x29e214e9,"MainField_Item_Plant_A_1431828557",
0x5087adb5,"MainField_Item_Plant_A_1474759089",
0xb6dcb12a,"MainField_Item_Plant_A_1490889462",
0xdcd3f6ec,"MainField_Item_Plant_A_1524848345",
0xd2feeb31,"MainField_Item_Plant_A_1525263437",
0x13a62bb3,"MainField_Item_Plant_A_1548656471",
0xdf4e2a51,"MainField_Item_Plant_A_1557797754",
0xf9bb0b48,"MainField_Item_Plant_A_1559483771",
0xc957d499,"MainField_Item_Plant_A_1568672831",
0x30a13614,"MainField_Item_Plant_A_1582122480",
0x0c840ae7,"MainField_Item_Plant_A_1606134947",
0x3e32597b,"MainField_Item_Plant_A_1614841328",
0xfed16f0b,"MainField_Item_Plant_A_1651839231",
0xb0fe2578,"MainField_Item_Plant_A_1658268260",
0x08e0fd5d,"MainField_Item_Plant_A_1662071380",
0x8002c689,"MainField_Item_Plant_A_1666584364",
0x4e3c3364,"MainField_Item_Plant_A_1676751377",
0x3e7faf0e,"MainField_Item_Plant_A_1717394785",
0x385e49ae,"MainField_Item_Plant_A_1734619368",
0xc17419b4,"MainField_Item_Plant_A_1743949287",
0x63ee5de9,"MainField_Item_Plant_A_1769566525",
0x26a9b860,"MainField_Item_Plant_A_1773839678",
0x58f4c574,"MainField_Item_Plant_A_1799608097",
0x4752826b,"MainField_Item_Plant_A_18052512",
0xb5d3ba26,"MainField_Item_Plant_A_1821633846",
0xc929b7f0,"MainField_Item_Plant_A_1831137476",
0x417593d7,"MainField_Item_Plant_A_1846004354",
0xf3547835,"MainField_Item_Plant_A_1860416806",
0xac774f09,"MainField_Item_Plant_A_1870484794",
0x68f82676,"MainField_Item_Plant_A_1876174076",
0x910a86ae,"MainField_Item_Plant_A_1881613270",
0x5051317c,"MainField_Item_Plant_A_1885247672",
0xd532d655,"MainField_Item_Plant_A_190842527",
0xb20e1697,"MainField_Item_Plant_A_1917152360",
0x0566ddcc,"MainField_Item_Plant_A_1929909184",
0xd49826ca,"MainField_Item_Plant_A_1944180010",
0x28209ae0,"MainField_Item_Plant_A_1946756254",
0x58da2569,"MainField_Item_Plant_A_1955105743",
0xb1f85a4a,"MainField_Item_Plant_A_1958539353",
0xd57b7d65,"MainField_Item_Plant_A_1960420533",
0xbb40c03a,"MainField_Item_Plant_A_1962535542",
0x9835ac41,"MainField_Item_Plant_A_196521695",
0x25535781,"MainField_Item_Plant_A_1969691931",
0xd23eb644,"MainField_Item_Plant_A_1972097003",
0x9826210a,"MainField_Item_Plant_A_1977455301",
0x9a220ac0,"MainField_Item_Plant_A_1981772423",
0x169fe49b,"MainField_Item_Plant_A_1991813534",
0xeb8f9cb2,"MainField_Item_Plant_A_1998218124",
0xcb4a18fd,"MainField_Item_Plant_A_2003389320",
0x20acd81c,"MainField_Item_Plant_A_2007424446",
0x1ee0023d,"MainField_Item_Plant_A_2022003226",
0x9f4c1d64,"MainField_Item_Plant_A_2076099017",
0x7f5180ca,"MainField_Item_Plant_A_2102200389",
0x2d38f53b,"MainField_Item_Plant_A_2111216851",
0xf612c411,"MainField_Item_Plant_A_2117474878",
0x4011cf87,"MainField_Item_Plant_A_2120002585",
0xb5aec509,"MainField_Item_Plant_A_212142252",
0x91647905,"MainField_Item_Plant_A_2138239529",
0x54f31fc6,"MainField_Item_Plant_A_2171978562",
0x63a85af4,"MainField_Item_Plant_A_2179427752",
0xb5e1c792,"MainField_Item_Plant_A_2185589092",
0xe9682603,"MainField_Item_Plant_A_2198673291",
0x1b6e826b,"MainField_Item_Plant_A_220307223",
0x4e2ed221,"MainField_Item_Plant_A_2225244625",
0xba71a346,"MainField_Item_Plant_A_2237618412",
0x2a0eb55f,"MainField_Item_Plant_A_2238376323",
0x610d041a,"MainField_Item_Plant_A_227365020",
0xa6900edb,"MainField_Item_Plant_A_2299769957",
0x1f3bb7e0,"MainField_Item_Plant_A_2325071771",
0xe8d6433d,"MainField_Item_Plant_A_2333828820",
0xa5fa8ce0,"MainField_Item_Plant_A_2378766618",
0xfee490e8,"MainField_Item_Plant_A_2392409099",
0xc097027f,"MainField_Item_Plant_A_2417183554",
0x982022fa,"MainField_Item_Plant_A_2423287968",
0x352f2984,"MainField_Item_Plant_A_2424603549",
0x601202e0,"MainField_Item_Plant_A_2433373276",
0x207b981a,"MainField_Item_Plant_A_2436579800",
0xfab60cda,"MainField_Item_Plant_A_2440428500",
0x2148205d,"MainField_Item_Plant_A_2442965512",
0xf3f508e3,"MainField_Item_Plant_A_2446054751",
0x62f07dbc,"MainField_Item_Plant_A_2446921671",
0x4a51bdc5,"MainField_Item_Plant_A_2450394135",
0x5d34f077,"MainField_Item_Plant_A_2465204583",
0xb1118b20,"MainField_Item_Plant_A_2480356287",
0x68a5658b,"MainField_Item_Plant_A_2490188541",
0xa0f8bad9,"MainField_Item_Plant_A_2510982203",
0xef47d619,"MainField_Item_Plant_A_2538192954",
0xa4bba62c,"MainField_Item_Plant_A_254109121",
0x88f403a3,"MainField_Item_Plant_A_2557397860",
0xde95fbdc,"MainField_Item_Plant_A_256518337",
0x28902220,"MainField_Item_Plant_A_2567656431",
0x548d3f56,"MainField_Item_Plant_A_257487700",
0xa7af46b1,"MainField_Item_Plant_A_260653017",
0xe68032d6,"MainField_Item_Plant_A_2614317042",
0xf7e37413,"MainField_Item_Plant_A_2617699534",
0x7efc377e,"MainField_Item_Plant_A_2632258835",
0x3b89290a,"MainField_Item_Plant_A_2637085304",
0xfaea1b86,"MainField_Item_Plant_A_2637181417",
0x04485e81,"MainField_Item_Plant_A_2640991244",
0x4ced2825,"MainField_Item_Plant_A_266645641",
0x8f3d4d78,"MainField_Item_Plant_A_2677015492",
0xf24fd4b0,"MainField_Item_Plant_A_270082746",
0x34788d03,"MainField_Item_Plant_A_2703337696",
0xaa74b531,"MainField_Item_Plant_A_272702611",
0xe7b22bcf,"MainField_Item_Plant_A_2728665207",
0x3d82dff0,"MainField_Item_Plant_A_2736508565",
0xe2d70072,"MainField_Item_Plant_A_2737671814",
0x795cd3ee,"MainField_Item_Plant_A_2743031087",
0x87a05ba6,"MainField_Item_Plant_A_2744782318",
0x07531525,"MainField_Item_Plant_A_2749331608",
0x6cacb668,"MainField_Item_Plant_A_2755676831",
0xa34ae2a3,"MainField_Item_Plant_A_2756346837",
0x97529d3d,"MainField_Item_Plant_A_2775972156",
0xf5c94d11,"MainField_Item_Plant_A_2781471854",
0x14f33b79,"MainField_Item_Plant_A_2795078293",
0x8887dc45,"MainField_Item_Plant_A_279688977",
0xcaab3ebe,"MainField_Item_Plant_A_2800357395",
0xf6d9f2b2,"MainField_Item_Plant_A_2801508350",
0x517612ad,"MainField_Item_Plant_A_2805991669",
0x9755332b,"MainField_Item_Plant_A_2826268959",
0xd9855d7e,"MainField_Item_Plant_A_2845929156",
0x6429eb52,"MainField_Item_Plant_A_2851545272",
0x7be2aa9d,"MainField_Item_Plant_A_2862902214",
0x17885c96,"MainField_Item_Plant_A_2868370311",
0xe75670e4,"MainField_Item_Plant_A_2886507161",
0x019b38d3,"MainField_Item_Plant_A_2931635261",
0x2ce2ddd1,"MainField_Item_Plant_A_2934736842",
0xf5c6a436,"MainField_Item_Plant_A_295334710",
0xef9cbf68,"MainField_Item_Plant_A_2973990139",
0xda09c538,"MainField_Item_Plant_A_2974494252",
0xcc3e0fb8,"MainField_Item_Plant_A_30229934",
0x19fae056,"MainField_Item_Plant_A_3039765570",
0x79f95878,"MainField_Item_Plant_A_3054126462",
0xe6ebbdbd,"MainField_Item_Plant_A_3075112155",
0xe95b8361,"MainField_Item_Plant_A_3103844962",
0x34484a3d,"MainField_Item_Plant_A_3113671302",
0x2a6a15b6,"MainField_Item_Plant_A_3120031819",
0xb260ba49,"MainField_Item_Plant_A_3120292057",
0x22503622,"MainField_Item_Plant_A_3173060515",
0x07d58fd2,"MainField_Item_Plant_A_3206329282",
0x624ec754,"MainField_Item_Plant_A_3215041358",
0x078be347,"MainField_Item_Plant_A_3218678003",
0x9b28100e,"MainField_Item_Plant_A_3238307447",
0xcbf11d70,"MainField_Item_Plant_A_3257651738",
0x89419ecc,"MainField_Item_Plant_A_3264536597",
0x721a444e,"MainField_Item_Plant_A_3279280164",
0x20449a48,"MainField_Item_Plant_A_3281521004",
0x4154c5fe,"MainField_Item_Plant_A_3282249404",
0xeb6605c4,"MainField_Item_Plant_A_3282419533",
0xb91bf8fc,"MainField_Item_Plant_A_3287988381",
0xfaceac97,"MainField_Item_Plant_A_3294222270",
0x56dcc0da,"MainField_Item_Plant_A_3300947567",
0xef17a4c6,"MainField_Item_Plant_A_3313921969",
0xe9280eb3,"MainField_Item_Plant_A_3332436990",
0x958a419e,"MainField_Item_Plant_A_3339716839",
0x083a4e5b,"MainField_Item_Plant_A_3341747342",
0xdf6663e0,"MainField_Item_Plant_A_3342155630",
0xcc2076d5,"MainField_Item_Plant_A_3352863689",
0xbecc4195,"MainField_Item_Plant_A_3374133804",
0xd2f98cb8,"MainField_Item_Plant_A_3387582994",
0x133ca5a2,"MainField_Item_Plant_A_3411689083",
0xd81d05fd,"MainField_Item_Plant_A_3453328838",
0x286072f7,"MainField_Item_Plant_A_3455083776",
0xc1d800db,"MainField_Item_Plant_A_346939867",
0x49da24df,"MainField_Item_Plant_A_3492286769",
0xaef5d755,"MainField_Item_Plant_A_3497416525",
0xff0d7d7b,"MainField_Item_Plant_A_3502816096",
0x713a0ccd,"MainField_Item_Plant_A_350458660",
0x240e4e5d,"MainField_Item_Plant_A_3536473464",
0xdd34924c,"MainField_Item_Plant_A_3536770296",
0x295ff66a,"MainField_Item_Plant_A_3537092119",
0x4d275ad2,"MainField_Item_Plant_A_3544676873",
0x960e9c63,"MainField_Item_Plant_A_3558624611",
0xc2751c32,"MainField_Item_Plant_A_3559570960",
0x9df5aac0,"MainField_Item_Plant_A_3571553114",
0x57ccb71f,"MainField_Item_Plant_A_3580585658",
0xd36d811f,"MainField_Item_Plant_A_3583752206",
0xd0379fe4,"MainField_Item_Plant_A_3584233697",
0x84b17c30,"MainField_Item_Plant_A_3595383167",
0x1460aaba,"MainField_Item_Plant_A_3600912575",
0x294b3ef1,"MainField_Item_Plant_A_3602300276",
0xa3493a50,"MainField_Item_Plant_A_3614967779",
0xe00b39ab,"MainField_Item_Plant_A_3626471363",
0x17d38d48,"MainField_Item_Plant_A_3664702108",
0xac349dc5,"MainField_Item_Plant_A_3665621233",
0x782e482b,"MainField_Item_Plant_A_3676107798",
0x6812f09b,"MainField_Item_Plant_A_370948342",
0x2fd91eea,"MainField_Item_Plant_A_3757686702",
0xdcd5017e,"MainField_Item_Plant_A_3761635807",
0xd5c11d05,"MainField_Item_Plant_A_3784730834",
0x1a9f2ce5,"MainField_Item_Plant_A_3790582829",
0x769177fd,"MainField_Item_Plant_A_3793893061",
0x1445e090,"MainField_Item_Plant_A_3800664803",
0xe5399167,"MainField_Item_Plant_A_3801139657",
0xab0d74f6,"MainField_Item_Plant_A_3803145636",
0xaed27c20,"MainField_Item_Plant_A_3806762195",
0x4f399d41,"MainField_Item_Plant_A_3820003393",
0x6b93a758,"MainField_Item_Plant_A_387364104",
0x598ec775,"MainField_Item_Plant_A_389582469",
0x02c3aefe,"MainField_Item_Plant_A_3897809710",
0x430ddbd1,"MainField_Item_Plant_A_3899784175",
0x7e658680,"MainField_Item_Plant_A_3923189988",
0x46c13da6,"MainField_Item_Plant_A_3929226863",
0x370ca699,"MainField_Item_Plant_A_3932266142",
0x5c3a45d5,"MainField_Item_Plant_A_3942767704",
0xfbd7ea61,"MainField_Item_Plant_A_3949719716",
0xcf6a81e6,"MainField_Item_Plant_A_397190334",
0x5ff98712,"MainField_Item_Plant_A_3972375213",
0x365af88d,"MainField_Item_Plant_A_3974105909",
0xbb255060,"MainField_Item_Plant_A_3983482210",
0x17b3e2d9,"MainField_Item_Plant_A_404562900",
0x6e0cab65,"MainField_Item_Plant_A_4069440635",
0xe6392814,"MainField_Item_Plant_A_4076098274",
0xa445b952,"MainField_Item_Plant_A_4086978093",
0xd2fe1434,"MainField_Item_Plant_A_4087786963",
0xc00d1799,"MainField_Item_Plant_A_409703136",
0xc29dd766,"MainField_Item_Plant_A_410181373",
0x03e7527d,"MainField_Item_Plant_A_4102229032",
0x21182050,"MainField_Item_Plant_A_4112723045",
0x12632aa5,"MainField_Item_Plant_A_4133515706",
0xec49cba8,"MainField_Item_Plant_A_417120767",
0xb4f729e2,"MainField_Item_Plant_A_4173637507",
0x88ae391e,"MainField_Item_Plant_A_4173803464",
0x1eb8a32c,"MainField_Item_Plant_A_4197738325",
0xa799d6fb,"MainField_Item_Plant_A_4206011219",
0xb0e6971d,"MainField_Item_Plant_A_4227368598",
0x2de69059,"MainField_Item_Plant_A_4251376016",
0x9e89268a,"MainField_Item_Plant_A_4271304010",
0x2c5a0808,"MainField_Item_Plant_A_4281630206",
0xa1413ba4,"MainField_Item_Plant_A_4283269726",
0xc80538ca,"MainField_Item_Plant_A_4286467794",
0x3acd3e37,"MainField_Item_Plant_A_4294745034",
0x89a78753,"MainField_Item_Plant_A_430441074",
0xb2a04e15,"MainField_Item_Plant_A_435766970",
0x11651be3,"MainField_Item_Plant_A_468411044",
0xdc09b130,"MainField_Item_Plant_A_49455313",
0x6ab2175b,"MainField_Item_Plant_A_511212360",
0xfb321014,"MainField_Item_Plant_A_513812612",
0x0e2d301c,"MainField_Item_Plant_A_514886870",
0xef574869,"MainField_Item_Plant_A_51513439",
0x1d52c5a8,"MainField_Item_Plant_A_51874251",
0x9f90feec,"MainField_Item_Plant_A_528972231",
0x21a69133,"MainField_Item_Plant_A_549795407",
0xaa01050d,"MainField_Item_Plant_A_556547720",
0xe8426e99,"MainField_Item_Plant_A_572745382",
0x547ad68b,"MainField_Item_Plant_A_57950214",
0x56115694,"MainField_Item_Plant_A_598097848",
0xff7eb944,"MainField_Item_Plant_A_607171693",
0x81d3b3c7,"MainField_Item_Plant_A_616575270",
0x6081af77,"MainField_Item_Plant_A_617707983",
0x3093c5a3,"MainField_Item_Plant_A_619352792",
0x18eafbd2,"MainField_Item_Plant_A_647276379",
0x10880c45,"MainField_Item_Plant_A_669088759",
0x884e8da3,"MainField_Item_Plant_A_671020006",
0x0b598d18,"MainField_Item_Plant_A_674392044",
0xcb75031d,"MainField_Item_Plant_A_688147075",
0x8af1c91c,"MainField_Item_Plant_A_69118632",
0x1c115895,"MainField_Item_Plant_A_713669948",
0x1105689b,"MainField_Item_Plant_A_737713080",
0x15eb9ab0,"MainField_Item_Plant_A_74226694",
0xfa861b59,"MainField_Item_Plant_A_747705479",
0x90933b02,"MainField_Item_Plant_A_757783717",
0x52ec9a78,"MainField_Item_Plant_A_769943119",
0x67d5b95c,"MainField_Item_Plant_A_776939389",
0x3f691571,"MainField_Item_Plant_A_785114419",
0x4e9b2aa9,"MainField_Item_Plant_A_801075718",
0xf797342b,"MainField_Item_Plant_A_809824851",
0x70da4e32,"MainField_Item_Plant_A_819586849",
0xddbf5c52,"MainField_Item_Plant_A_843583719",
0x322061b6,"MainField_Item_Plant_A_866445881",
0xac25af9b,"MainField_Item_Plant_A_877039238",
0x81691887,"MainField_Item_Plant_A_888734684",
0xa4ef9f0c,"MainField_Item_Plant_A_889149198",
0x32b28c13,"MainField_Item_Plant_A_922696501",
0xfe748db3,"MainField_Item_Plant_A_936558461",
0xdb797bf1,"MainField_Item_Plant_A_958435765",
0x8766eb60,"MainField_Item_Plant_A_975494972",
0x72b397d4,"MainField_Item_Plant_A_989878967",
0x6b389dd8,"MainField_Item_Plant_A_991960141",
0xfddb591b,"MainField_Item_Plant_B_1033533522",
0xb3b7eaaa,"MainField_Item_Plant_B_1067214694",
0x8f004546,"MainField_Item_Plant_B_1074648051",
0x3e834683,"MainField_Item_Plant_B_1081498942",
0x1c79a640,"MainField_Item_Plant_B_1105941972",
0x74638efb,"MainField_Item_Plant_B_1110452280",
0xa30b74c4,"MainField_Item_Plant_B_1134262675",
0xa64ad3d5,"MainField_Item_Plant_B_1150641343",
0x5aa09b5d,"MainField_Item_Plant_B_1168961420",
0x18277495,"MainField_Item_Plant_B_1180742459",
0xb4a3359b,"MainField_Item_Plant_B_1200951560",
0x08253f97,"MainField_Item_Plant_B_1201512126",
0x51cce6ea,"MainField_Item_Plant_B_1214970629",
0x3f303149,"MainField_Item_Plant_B_1231238148",
0x5b757ea3,"MainField_Item_Plant_B_1252069032",
0x05370ba8,"MainField_Item_Plant_B_1274910264",
0xb661e693,"MainField_Item_Plant_B_1281293438",
0x8953430d,"MainField_Item_Plant_B_1292260005",
0x76731a52,"MainField_Item_Plant_B_1297717691",
0x95bd417c,"MainField_Item_Plant_B_1299800178",
0xfd92b2a8,"MainField_Item_Plant_B_1302892713",
0xe5378d24,"MainField_Item_Plant_B_1317052769",
0x5048c11e,"MainField_Item_Plant_B_1332049544",
0x2761efa5,"MainField_Item_Plant_B_1352466448",
0x3aa8043d,"MainField_Item_Plant_B_1353468224",
0x5c97422c,"MainField_Item_Plant_B_1354714306",
0x6773ed98,"MainField_Item_Plant_B_1382472059",
0xe77cd462,"MainField_Item_Plant_B_1406310343",
0xc3fbce79,"MainField_Item_Plant_B_1424666563",
0x68a92b73,"MainField_Item_Plant_B_1440885332",
0xddf61286,"MainField_Item_Plant_B_146860849",
0x2c33c00e,"MainField_Item_Plant_B_1494796020",
0x7723268d,"MainField_Item_Plant_B_1495486458",
0x0202aeef,"MainField_Item_Plant_B_1510623572",
0xb503fe1c,"MainField_Item_Plant_B_1537133985",
0x83e3971a,"MainField_Item_Plant_B_1538467244",
0x81c82178,"MainField_Item_Plant_B_1602990673",
0xb3229611,"MainField_Item_Plant_B_1623693657",
0xe3967c5f,"MainField_Item_Plant_B_1645432799",
0xcfffd786,"MainField_Item_Plant_B_1646084650",
0x34c5ad79,"MainField_Item_Plant_B_1680308393",
0x38b12f29,"MainField_Item_Plant_B_1696017455",
0x078fb909,"MainField_Item_Plant_B_1733336291",
0x115ceb52,"MainField_Item_Plant_B_1744834685",
0xc6086fd9,"MainField_Item_Plant_B_1806100093",
0xfa325864,"MainField_Item_Plant_B_1813137724",
0x8d44b20b,"MainField_Item_Plant_B_183897335",
0x80c4612b,"MainField_Item_Plant_B_1858980784",
0x944befef,"MainField_Item_Plant_B_1888131744",
0xd6b4c16a,"MainField_Item_Plant_B_1907523210",
0xf8b8ddc3,"MainField_Item_Plant_B_1911842645",
0xa6506fd9,"MainField_Item_Plant_B_1918145740",
0x94723c9b,"MainField_Item_Plant_B_192993819",
0x7a843f0f,"MainField_Item_Plant_B_197915736",
0xfc8a3717,"MainField_Item_Plant_B_2003711361",
0x263d44be,"MainField_Item_Plant_B_2023237533",
0x487c4bb4,"MainField_Item_Plant_B_2024306434",
0x122212ea,"MainField_Item_Plant_B_2059882480",
0xb053d95a,"MainField_Item_Plant_B_2086931599",
0x285c04fd,"MainField_Item_Plant_B_211349092",
0xf00067ef,"MainField_Item_Plant_B_2137243401",
0x827ece94,"MainField_Item_Plant_B_2140161303",
0xa1ade38d,"MainField_Item_Plant_B_2152244865",
0x889e9c09,"MainField_Item_Plant_B_2171260017",
0x4a91eb91,"MainField_Item_Plant_B_2177759270",
0xf12d76ff,"MainField_Item_Plant_B_2216435606",
0xdd738547,"MainField_Item_Plant_B_2218307740",
0x6d5598e3,"MainField_Item_Plant_B_2221700680",
0x5630d38a,"MainField_Item_Plant_B_222997725",
0x336e4425,"MainField_Item_Plant_B_2258537203",
0x2c1d4a28,"MainField_Item_Plant_B_2259876822",
0x587ebe91,"MainField_Item_Plant_B_2322441218",
0xeb6b22fa,"MainField_Item_Plant_B_2386971575",
0x71f5a376,"MainField_Item_Plant_B_2395951930",
0xc24edb94,"MainField_Item_Plant_B_240229754",
0xa92c1a4c,"MainField_Item_Plant_B_2404391678",
0x5a392e78,"MainField_Item_Plant_B_2426093474",
0xa2295195,"MainField_Item_Plant_B_2438172806",
0x6221099a,"MainField_Item_Plant_B_2438522854",
0x67a25e95,"MainField_Item_Plant_B_2524965158",
0x5481fc12,"MainField_Item_Plant_B_2553113716",
0x12a319cd,"MainField_Item_Plant_B_2557009198",
0x380778d2,"MainField_Item_Plant_B_2629869847",
0x091afeb7,"MainField_Item_Plant_B_2700763707",
0x5ad88094,"MainField_Item_Plant_B_271825365",
0xcbe8f302,"MainField_Item_Plant_B_2764873999",
0x4b25133f,"MainField_Item_Plant_B_2805157770",
0xcd122a3d,"MainField_Item_Plant_B_2819335623",
0xe1bcd68f,"MainField_Item_Plant_B_2819653650",
0xa06c73df,"MainField_Item_Plant_B_2828854337",
0x0360507a,"MainField_Item_Plant_B_2847760049",
0xeafc648f,"MainField_Item_Plant_B_2862177556",
0x4d6fd0c7,"MainField_Item_Plant_B_2870289042",
0x70aaa341,"MainField_Item_Plant_B_2884738159",
0x26fc7af5,"MainField_Item_Plant_B_2911861925",
0x72ae4ff7,"MainField_Item_Plant_B_2933776603",
0x535d28df,"MainField_Item_Plant_B_2959823190",
0x259220ca,"MainField_Item_Plant_B_2985140970",
0xd427fdb3,"MainField_Item_Plant_B_304895556",
0x72eb0072,"MainField_Item_Plant_B_3053684147",
0x8a73a572,"MainField_Item_Plant_B_3088315350",
0x9532f526,"MainField_Item_Plant_B_3102937501",
0x31ab6641,"MainField_Item_Plant_B_3171612687",
0xe07431b2,"MainField_Item_Plant_B_3191628585",
0xaf29442d,"MainField_Item_Plant_B_3217012513",
0xd3430b2b,"MainField_Item_Plant_B_3243738197",
0x60351d07,"MainField_Item_Plant_B_3263121982",
0x80e1eec4,"MainField_Item_Plant_B_3268964557",
0xbd54b535,"MainField_Item_Plant_B_3282186252",
0xc8c5deec,"MainField_Item_Plant_B_3289152656",
0x667f855d,"MainField_Item_Plant_B_3327435123",
0x3c72d560,"MainField_Item_Plant_B_3364110068",
0x17e41e65,"MainField_Item_Plant_B_3368522688",
0x2850a717,"MainField_Item_Plant_B_3372128911",
0x6289778a,"MainField_Item_Plant_B_339830862",
0x94f13958,"MainField_Item_Plant_B_3416419930",
0x8fcf4885,"MainField_Item_Plant_B_3417045013",
0x37750cf9,"MainField_Item_Plant_B_3426346842",
0x76fce643,"MainField_Item_Plant_B_3467980091",
0xdf141728,"MainField_Item_Plant_B_3484837455",
0xb29e2835,"MainField_Item_Plant_B_3487542919",
0x8bc0c7c0,"MainField_Item_Plant_B_3497216984",
0x06c75b0c,"MainField_Item_Plant_B_3508563838",
0xc404ba8a,"MainField_Item_Plant_B_3556893467",
0xce973de1,"MainField_Item_Plant_B_3584080859",
0x32a89aeb,"MainField_Item_Plant_B_3587332307",
0x762a7bd6,"MainField_Item_Plant_B_3596600108",
0x6f42536a,"MainField_Item_Plant_B_3599208181",
0xe8956296,"MainField_Item_Plant_B_3622746449",
0x08339e90,"MainField_Item_Plant_B_3639244184",
0x7d08fec0,"MainField_Item_Plant_B_3672575098",
0x687e7a58,"MainField_Item_Plant_B_3682570382",
0x2a8ed49e,"MainField_Item_Plant_B_3686797447",
0xe9d4f3e7,"MainField_Item_Plant_B_3706809757",
0x3153a6aa,"MainField_Item_Plant_B_3725178244",
0xe3d92c65,"MainField_Item_Plant_B_3728593186",
0x171321b4,"MainField_Item_Plant_B_3756755361",
0xe91c47d4,"MainField_Item_Plant_B_3772730794",
0xb9e8f501,"MainField_Item_Plant_B_3804616072",
0x95732076,"MainField_Item_Plant_B_3820426624",
0x9c3cad1f,"MainField_Item_Plant_B_3834616455",
0x51d1a0f6,"MainField_Item_Plant_B_3848430749",
0xb7133368,"MainField_Item_Plant_B_3854060472",
0x161c64cf,"MainField_Item_Plant_B_3912244493",
0xaa090f12,"MainField_Item_Plant_B_3928416211",
0x8f6cb100,"MainField_Item_Plant_B_4004092572",
0x25b205fd,"MainField_Item_Plant_B_402257527",
0x1d9ef591,"MainField_Item_Plant_B_4086186755",
0xcf8c2358,"MainField_Item_Plant_B_4117396059",
0x639fcb20,"MainField_Item_Plant_B_4133202564",
0xdea42356,"MainField_Item_Plant_B_4148275384",
0xd1711d3b,"MainField_Item_Plant_B_4173390055",
0x14ff2c76,"MainField_Item_Plant_B_418659746",
0xcaea25f1,"MainField_Item_Plant_B_4190250292",
0xa713cf01,"MainField_Item_Plant_B_4225177359",
0xdbfd7bfa,"MainField_Item_Plant_B_4233824865",
0x9b93ecd1,"MainField_Item_Plant_B_4278111185",
0x8114d4c5,"MainField_Item_Plant_B_465582222",
0xf26ea5ab,"MainField_Item_Plant_B_484486268",
0x56a35041,"MainField_Item_Plant_B_487362453",
0xdf117756,"MainField_Item_Plant_B_509605685",
0x2b8cc8c2,"MainField_Item_Plant_B_530367065",
0x8ee055f0,"MainField_Item_Plant_B_53520214",
0x32723bea,"MainField_Item_Plant_B_591235491",
0x721ef3d0,"MainField_Item_Plant_B_60147194",
0x2084bbb2,"MainField_Item_Plant_B_615163088",
0x0e960ee5,"MainField_Item_Plant_B_662681861",
0x92014bae,"MainField_Item_Plant_B_692297006",
0xe0b095a4,"MainField_Item_Plant_B_694427960",
0x6f2d824c,"MainField_Item_Plant_B_723392059",
0xc46e595f,"MainField_Item_Plant_B_734179846",
0x543daad8,"MainField_Item_Plant_B_768168818",
0xb2edf6f9,"MainField_Item_Plant_B_82369025",
0xc4ce2869,"MainField_Item_Plant_B_839064171",
0xd6c43e5f,"MainField_Item_Plant_B_875484270",
0x97a18a06,"MainField_Item_Plant_B_921917700",
0x4094b0c5,"MainField_Item_Plant_B_991276822",
0x5a7e3e27,"MainField_Item_Plant_C_1156550743",
0xfc46462c,"MainField_Item_Plant_C_1220765928",
0xf6717477,"MainField_Item_Plant_C_1262496275",
0xd7c162bd,"MainField_Item_Plant_C_1281484327",
0x6e01ff9f,"MainField_Item_Plant_C_131335692",
0xd2b1975f,"MainField_Item_Plant_C_1382788897",
0x249fe88f,"MainField_Item_Plant_C_1471691737",
0xe1d26b5c,"MainField_Item_Plant_C_1507772615",
0xf014224a,"MainField_Item_Plant_C_15295871",
0x8b899116,"MainField_Item_Plant_C_1577538126",
0x84c1ba8d,"MainField_Item_Plant_C_1618268655",
0x2620815f,"MainField_Item_Plant_C_1742674266",
0x49ed3802,"MainField_Item_Plant_C_1746202838",
0xb2970f42,"MainField_Item_Plant_C_1811551021",
0xa30b5873,"MainField_Item_Plant_C_1883751907",
0x6161c278,"MainField_Item_Plant_C_1961946668",
0x25b92070,"MainField_Item_Plant_C_1970610819",
0x04085737,"MainField_Item_Plant_C_1975839161",
0x78dba0ce,"MainField_Item_Plant_C_1996512015",
0x1845c500,"MainField_Item_Plant_C_2174769216",
0x938a1c5c,"MainField_Item_Plant_C_2197790372",
0x33195189,"MainField_Item_Plant_C_2211400504",
0x76ee082a,"MainField_Item_Plant_C_2245521512",
0x1c38c9b8,"MainField_Item_Plant_C_2447426982",
0x59bb9b35,"MainField_Item_Plant_C_2573412167",
0xe302b558,"MainField_Item_Plant_C_263168165",
0x09e30851,"MainField_Item_Plant_C_2645828141",
0x48136161,"MainField_Item_Plant_C_2709458432",
0x6924c186,"MainField_Item_Plant_C_2825048510",
0xc0d51515,"MainField_Item_Plant_C_2925590667",
0xda2563d5,"MainField_Item_Plant_C_2958490909",
0x89a6db3b,"MainField_Item_Plant_C_2979064187",
0xbb111d0d,"MainField_Item_Plant_C_3013497493",
0x7a3b589b,"MainField_Item_Plant_C_3043798749",
0x508a3a95,"MainField_Item_Plant_C_3051669103",
0x40895718,"MainField_Item_Plant_C_3129309801",
0x38420dad,"MainField_Item_Plant_C_3274514351",
0x1ee97214,"MainField_Item_Plant_C_336463587",
0xc54a182f,"MainField_Item_Plant_C_3383176325",
0x219f33ce,"MainField_Item_Plant_C_3422648774",
0x8ad1af41,"MainField_Item_Plant_C_3461766955",
0x80f0e7e7,"MainField_Item_Plant_C_3543833388",
0x88d908f5,"MainField_Item_Plant_C_3563559822",
0x047c956c,"MainField_Item_Plant_C_3616712339",
0x80958de7,"MainField_Item_Plant_C_3846813610",
0xb3141665,"MainField_Item_Plant_C_3865832107",
0x00837d30,"MainField_Item_Plant_C_3876332666",
0x076690b4,"MainField_Item_Plant_C_3985102570",
0x018712ed,"MainField_Item_Plant_C_4137857059",
0x59d45251,"MainField_Item_Plant_C_4169156698",
0x8cf49666,"MainField_Item_Plant_C_4179705580",
0x9fe9909a,"MainField_Item_Plant_C_4223045257",
0xb9ada3da,"MainField_Item_Plant_C_4229035573",
0x2d1bfe91,"MainField_Item_Plant_C_501003629",
0x0720c5a1,"MainField_Item_Plant_C_638337655",
0x7f0004c2,"MainField_Item_Plant_C_644084864",
0xc406c202,"MainField_Item_Plant_C_873153988",
0x474ae5a4,"MainField_Item_Plant_C_891432895",
0x722e4a37,"MainField_Item_Plant_C_900926371",
0x6ba1fd25,"MainField_Item_Plant_C_916149962",
0x1fecc598,"MainField_Item_Plant_C_923005148",
0x65297727,"MainField_Item_Plant_C_972137598",
0x512cfb51,"MainField_Item_Plant_C_993359958",
0x184a230a,"MainField_Item_Plant_E_100219909",
0x4fa787d2,"MainField_Item_Plant_E_1003244195",
0x311464e6,"MainField_Item_Plant_E_1005796462",
0xf0c43747,"MainField_Item_Plant_E_1033440484",
0x1b1c5429,"MainField_Item_Plant_E_1034098997",
0x3ae05580,"MainField_Item_Plant_E_1059792137",
0xd7682246,"MainField_Item_Plant_E_1085156407",
0x576f602d,"MainField_Item_Plant_E_1092602921",
0xf385ec94,"MainField_Item_Plant_E_109401082",
0x20443a97,"MainField_Item_Plant_E_1115068608",
0xc769fcff,"MainField_Item_Plant_E_1154406824",
0xbf6e8526,"MainField_Item_Plant_E_1168093023",
0xab43384f,"MainField_Item_Plant_E_1171348474",
0x80349f55,"MainField_Item_Plant_E_1172223232",
0x952f85ec,"MainField_Item_Plant_E_1211606684",
0xca28607e,"MainField_Item_Plant_E_1226328145",
0x8042c0ab,"MainField_Item_Plant_E_1230049621",
0xee6c0047,"MainField_Item_Plant_E_1326716845",
0x07319009,"MainField_Item_Plant_E_13768346",
0x3e464486,"MainField_Item_Plant_E_1391613723",
0x4be2eda5,"MainField_Item_Plant_E_1392076217",
0xa5114ca1,"MainField_Item_Plant_E_1401718283",
0x79f343bc,"MainField_Item_Plant_E_1408144070",
0xb6fa6931,"MainField_Item_Plant_E_1433129358",
0x334cefac,"MainField_Item_Plant_E_1436554858",
0x588b9a82,"MainField_Item_Plant_E_1443508667",
0x19337588,"MainField_Item_Plant_E_1450001535",
0xcc68b279,"MainField_Item_Plant_E_1457240793",
0x3f2bf22b,"MainField_Item_Plant_E_1461297224",
0x9d2e50da,"MainField_Item_Plant_E_1464571621",
0xde906684,"MainField_Item_Plant_E_1490622058",
0x23d3f3d8,"MainField_Item_Plant_E_1520666181",
0x3d2d6c16,"MainField_Item_Plant_E_1528748036",
0x1c2e7751,"MainField_Item_Plant_E_1538452543",
0xf2f87bea,"MainField_Item_Plant_E_1549152402",
0xa14dddec,"MainField_Item_Plant_E_155917635",
0xe13e52e7,"MainField_Item_Plant_E_1562765237",
0x03c69457,"MainField_Item_Plant_E_1564227096",
0xd2de3d6d,"MainField_Item_Plant_E_1574983446",
0xd3c859eb,"MainField_Item_Plant_E_1586586942",
0x10daef7c,"MainField_Item_Plant_E_1589466536",
0x56eb7098,"MainField_Item_Plant_E_1589808607",
0x491372ec,"MainField_Item_Plant_E_1605361579",
0x2cf910e6,"MainField_Item_Plant_E_1653490731",
0x9715f547,"MainField_Item_Plant_E_1668445072",
0xd62570db,"MainField_Item_Plant_E_1668687036",
0xdbbf947e,"MainField_Item_Plant_E_1671366884",
0x425ff628,"MainField_Item_Plant_E_1690516913",
0x36effc53,"MainField_Item_Plant_E_1716255899",
0x69eec647,"MainField_Item_Plant_E_1718277650",
0xdf429f5e,"MainField_Item_Plant_E_1740631343",
0xdf2ff673,"MainField_Item_Plant_E_1749253373",
0xc1feed82,"MainField_Item_Plant_E_1752083866",
0xf7d8c7e1,"MainField_Item_Plant_E_1785850626",
0x5772096c,"MainField_Item_Plant_E_179700237",
0x291330fa,"MainField_Item_Plant_E_1822988150",
0xe0f2f17f,"MainField_Item_Plant_E_1825815778",
0x8a03542e,"MainField_Item_Plant_E_1835375400",
0x69fe32ff,"MainField_Item_Plant_E_1837001484",
0x6efcbece,"MainField_Item_Plant_E_184995996",
0x1710da74,"MainField_Item_Plant_E_1868796317",
0x2e9abb3d,"MainField_Item_Plant_E_1877934839",
0xd676a646,"MainField_Item_Plant_E_191627920",
0x49d84da8,"MainField_Item_Plant_E_1924979831",
0x2047aa1a,"MainField_Item_Plant_E_1936611422",
0x238930cc,"MainField_Item_Plant_E_1944708163",
0xcd3b948d,"MainField_Item_Plant_E_1951139652",
0xd330c331,"MainField_Item_Plant_E_1955027711",
0x22ee49ad,"MainField_Item_Plant_E_1956849230",
0x1f274994,"MainField_Item_Plant_E_1973222143",
0xcc991e43,"MainField_Item_Plant_E_1978461149",
0x8a56959c,"MainField_Item_Plant_E_2000544827",
0x11d14c03,"MainField_Item_Plant_E_2000625917",
0x5c2a6529,"MainField_Item_Plant_E_2000980597",
0x11adbba5,"MainField_Item_Plant_E_2001200743",
0x17b5e7d4,"MainField_Item_Plant_E_2025130817",
0xaee2503e,"MainField_Item_Plant_E_2027013600",
0x386fda9c,"MainField_Item_Plant_E_2034280128",
0xb6e5db17,"MainField_Item_Plant_E_2081425018",
0xde80ed50,"MainField_Item_Plant_E_2087295785",
0xe9073f2c,"MainField_Item_Plant_E_2090719727",
0x97d5de27,"MainField_Item_Plant_E_2094387260",
0xe0385498,"MainField_Item_Plant_E_2098984087",
0x08cb6302,"MainField_Item_Plant_E_2099933316",
0x5d6769ca,"MainField_Item_Plant_E_2163448045",
0x41e8b7d4,"MainField_Item_Plant_E_217257051",
0xf7d60ebf,"MainField_Item_Plant_E_2189081430",
0xaff8cb5d,"MainField_Item_Plant_E_2198676590",
0x46c40561,"MainField_Item_Plant_E_2199412677",
0xd6beb446,"MainField_Item_Plant_E_2202397550",
0x7db40d94,"MainField_Item_Plant_E_2218502387",
0xb2458615,"MainField_Item_Plant_E_2255247491",
0x53ae87e6,"MainField_Item_Plant_E_2271320069",
0x37bea0f7,"MainField_Item_Plant_E_2279119108",
0x83184808,"MainField_Item_Plant_E_2280333623",
0x32706497,"MainField_Item_Plant_E_2306037333",
0xcb0f302c,"MainField_Item_Plant_E_2319273778",
0xc7be9b9e,"MainField_Item_Plant_E_2328015017",
0xb6425f04,"MainField_Item_Plant_E_2334677255",
0xf13a3e79,"MainField_Item_Plant_E_2336562283",
0x22161db6,"MainField_Item_Plant_E_2352781760",
0xe4c8da00,"MainField_Item_Plant_E_2355763425",
0x31eb8f9d,"MainField_Item_Plant_E_2355768321",
0x38d2e89a,"MainField_Item_Plant_E_2357897943",
0x966f9b1a,"MainField_Item_Plant_E_2373479036",
0xd0818768,"MainField_Item_Plant_E_2375237365",
0xe2a9ee62,"MainField_Item_Plant_E_2401302971",
0x3eae81b3,"MainField_Item_Plant_E_2407915817",
0x0319dd88,"MainField_Item_Plant_E_2412030178",
0x7837c366,"MainField_Item_Plant_E_2418317647",
0x033c94d6,"MainField_Item_Plant_E_2420456511",
0x5bf913d3,"MainField_Item_Plant_E_2429163317",
0x4ffb7a93,"MainField_Item_Plant_E_2430333529",
0xec5e7ab8,"MainField_Item_Plant_E_2437421072",
0x29f5eb4b,"MainField_Item_Plant_E_2448496158",
0x8cca37bf,"MainField_Item_Plant_E_2475940987",
0x66c3ffed,"MainField_Item_Plant_E_2545020946",
0x88e26719,"MainField_Item_Plant_E_255634490",
0x615c9f13,"MainField_Item_Plant_E_2575403743",
0x1fbece0d,"MainField_Item_Plant_E_2578492264",
0x4e7eeee7,"MainField_Item_Plant_E_2590394841",
0x1bb08b4c,"MainField_Item_Plant_E_2598365897",
0xf10e53e3,"MainField_Item_Plant_E_2603315796",
0x6a11f7a9,"MainField_Item_Plant_E_2608244349",
0x2492ee03,"MainField_Item_Plant_E_2627780747",
0x0fad8ec0,"MainField_Item_Plant_E_2661374291",
0x647c07b5,"MainField_Item_Plant_E_2672081808",
0xa6eb959e,"MainField_Item_Plant_E_2674195226",
0xa53276dd,"MainField_Item_Plant_E_2717436928",
0x258040f4,"MainField_Item_Plant_E_2718249009",
0xb909b9f3,"MainField_Item_Plant_E_2733436407",
0x467d1ab3,"MainField_Item_Plant_E_2738651692",
0xb2c6702c,"MainField_Item_Plant_E_274754674",
0xfb0af708,"MainField_Item_Plant_E_2751052341",
0x2242f278,"MainField_Item_Plant_E_2771989946",
0xc2cbf6d6,"MainField_Item_Plant_E_2796103759",
0xbbc065fc,"MainField_Item_Plant_E_284370943",
0x30bf7469,"MainField_Item_Plant_E_2844959940",
0x518914bd,"MainField_Item_Plant_E_2853690129",
0x08e21e22,"MainField_Item_Plant_E_2862275882",
0x063bfa67,"MainField_Item_Plant_E_2874945772",
0x1acd89d9,"MainField_Item_Plant_E_2878731109",
0xe7cd6170,"MainField_Item_Plant_E_288630680",
0x7bf6e292,"MainField_Item_Plant_E_2889002730",
0x342006fd,"MainField_Item_Plant_E_28896372",
0xfbdd3b6d,"MainField_Item_Plant_E_2929508245",
0x2d69b2c0,"MainField_Item_Plant_E_2949366562",
0x4c4c453c,"MainField_Item_Plant_E_2978925",
0x5440a609,"MainField_Item_Plant_E_2999339284",
0x256973f7,"MainField_Item_Plant_E_3041133605",
0xce7cff11,"MainField_Item_Plant_E_3053364022",
0xcede0986,"MainField_Item_Plant_E_3056906051",
0x1bd7b1c2,"MainField_Item_Plant_E_3061239357",
0x9a099ba5,"MainField_Item_Plant_E_3061899142",
0x100e5929,"MainField_Item_Plant_E_3073856736",
0x80cffcd2,"MainField_Item_Plant_E_307966713",
0x6ccb9076,"MainField_Item_Plant_E_3081389471",
0x75c4a814,"MainField_Item_Plant_E_3119272213",
0xdde71867,"MainField_Item_Plant_E_3134797594",
0x47e0c48b,"MainField_Item_Plant_E_3142886564",
0x434658c9,"MainField_Item_Plant_E_3146373354",
0xbeb32446,"MainField_Item_Plant_E_3162719497",
0x808d729c,"MainField_Item_Plant_E_3164613233",
0xcbc819c0,"MainField_Item_Plant_E_3179869981",
0xb9c641c2,"MainField_Item_Plant_E_3184597795",
0xacbdeec3,"MainField_Item_Plant_E_319676247",
0x30aad084,"MainField_Item_Plant_E_3204342977",
0xbc83076c,"MainField_Item_Plant_E_3211558093",
0xfaa0d756,"MainField_Item_Plant_E_3215810909",
0x7308bec5,"MainField_Item_Plant_E_3240804187",
0xd02d7b84,"MainField_Item_Plant_E_3243850717",
0x0cbc23b3,"MainField_Item_Plant_E_3245722373",
0x19ec7457,"MainField_Item_Plant_E_3250615574",
0x8745f6b8,"MainField_Item_Plant_E_3251265468",
0x3b88a586,"MainField_Item_Plant_E_3257036810",
0x4d31732e,"MainField_Item_Plant_E_3267340243",
0x35d69f95,"MainField_Item_Plant_E_3282518449",
0xcf63006f,"MainField_Item_Plant_E_3283100830",
0xc96f1a65,"MainField_Item_Plant_E_3317504267",
0xf8d79580,"MainField_Item_Plant_E_3322690834",
0xebdbd7a5,"MainField_Item_Plant_E_3329097901",
0x108c1d41,"MainField_Item_Plant_E_3331237550",
0x1bd8d07c,"MainField_Item_Plant_E_3356061377",
0x6ed3eb1b,"MainField_Item_Plant_E_3389671582",
0x880b6063,"MainField_Item_Plant_E_3389790455",
0xd2bfbba5,"MainField_Item_Plant_E_3408558155",
0x76416086,"MainField_Item_Plant_E_34195626",
0x454a7026,"MainField_Item_Plant_E_3424677937",
0xb1e40e45,"MainField_Item_Plant_E_3433284825",
0x8d5ed2ab,"MainField_Item_Plant_E_3433643389",
0xb7e38e80,"MainField_Item_Plant_E_3466011561",
0x369af1b9,"MainField_Item_Plant_E_3467113911",
0x0bce051d,"MainField_Item_Plant_E_3485238947",
0x29519554,"MainField_Item_Plant_E_3515185426",
0xc7bc61c6,"MainField_Item_Plant_E_352924017",
0x5dab0f7b,"MainField_Item_Plant_E_3533002506",
0xdfd53756,"MainField_Item_Plant_E_3544862884",
0x5a0f0be5,"MainField_Item_Plant_E_3591291685",
0x166718dc,"MainField_Item_Plant_E_3596782797",
0xd976bfdb,"MainField_Item_Plant_E_3597035790",
0x8de09b8a,"MainField_Item_Plant_E_3612947004",
0xdc4ea0d5,"MainField_Item_Plant_E_3615338894",
0xc2157f54,"MainField_Item_Plant_E_3638707344",
0xda751f8f,"MainField_Item_Plant_E_3641678332",
0x1785d261,"MainField_Item_Plant_E_3672186367",
0x69914e4d,"MainField_Item_Plant_E_3680812231",
0x71e1d3bc,"MainField_Item_Plant_E_369649624",
0xc1b5d6d4,"MainField_Item_Plant_E_3718315692",
0x6699b03a,"MainField_Item_Plant_E_3720633986",
0x7ac26955,"MainField_Item_Plant_E_3722203254",
0x6569a9dc,"MainField_Item_Plant_E_3724485232",
0x68f52ef7,"MainField_Item_Plant_E_3744536776",
0xa0cec2f8,"MainField_Item_Plant_E_3750981184",
0x6c3f729b,"MainField_Item_Plant_E_3761374596",
0xae64c58e,"MainField_Item_Plant_E_3768207559",
0xa27ba93b,"MainField_Item_Plant_E_3791161415",
0x21891432,"MainField_Item_Plant_E_3806891793",
0x66ac0c95,"MainField_Item_Plant_E_3822435008",
0x32c4b929,"MainField_Item_Plant_E_3826677266",
0x038ddcdb,"MainField_Item_Plant_E_3836194512",
0x915fdbf1,"MainField_Item_Plant_E_3843320246",
0x90e267d9,"MainField_Item_Plant_E_3872837448",
0x3f7d1f09,"MainField_Item_Plant_E_3877542323",
0x1e0e54f1,"MainField_Item_Plant_E_3881849731",
0x1a6cd98f,"MainField_Item_Plant_E_3883077517",
0x6a70dda5,"MainField_Item_Plant_E_3887219338",
0xff89d8a4,"MainField_Item_Plant_E_3901960966",
0x59db799b,"MainField_Item_Plant_E_3911092826",
0xb4b05e1b,"MainField_Item_Plant_E_3926633968",
0xcd13dec0,"MainField_Item_Plant_E_3927218989",
0x859b13ee,"MainField_Item_Plant_E_396309878",
0x0b63afb5,"MainField_Item_Plant_E_3972305340",
0xb3db16b9,"MainField_Item_Plant_E_3972374373",
0x5c3b7104,"MainField_Item_Plant_E_4000631209",
0x6c0ea49b,"MainField_Item_Plant_E_400247640",
0xa08d4ec7,"MainField_Item_Plant_E_4072666170",
0x002daed4,"MainField_Item_Plant_E_4073570490",
0xea33120d,"MainField_Item_Plant_E_4081442783",
0xfba7ec88,"MainField_Item_Plant_E_4097973976",
0x5f4036e9,"MainField_Item_Plant_E_4110574353",
0x84476651,"MainField_Item_Plant_E_4111045124",
0xad52516a,"MainField_Item_Plant_E_4132380614",
0xd0129818,"MainField_Item_Plant_E_4145820917",
0x08ac3571,"MainField_Item_Plant_E_4153071440",
0xf1698327,"MainField_Item_Plant_E_4153328199",
0x779a1dcc,"MainField_Item_Plant_E_4171442004",
0x5254b713,"MainField_Item_Plant_E_4173001563",
0x56209025,"MainField_Item_Plant_E_4188863890",
0x756074e0,"MainField_Item_Plant_E_4189996138",
0x860c3ee2,"MainField_Item_Plant_E_4205410760",
0x9e4169d5,"MainField_Item_Plant_E_4208293990",
0x7b5e457f,"MainField_Item_Plant_E_4211944448",
0x2e54cb26,"MainField_Item_Plant_E_4225825989",
0xcb59b774,"MainField_Item_Plant_E_4257607034",
0x24e18f73,"MainField_Item_Plant_E_4264655014",
0xba4f2c98,"MainField_Item_Plant_E_4272229270",
0x2c70876c,"MainField_Item_Plant_E_4279227475",
0x01e2cf2e,"MainField_Item_Plant_E_4279784841",
0x4d95c638,"MainField_Item_Plant_E_4284589060",
0xd01745de,"MainField_Item_Plant_E_431728205",
0x6eabf64b,"MainField_Item_Plant_E_434490299",
0xfdede9a2,"MainField_Item_Plant_E_451535449",
0x03e5f8f3,"MainField_Item_Plant_E_46189970",
0x7495ba9e,"MainField_Item_Plant_E_476253776",
0x14dd1339,"MainField_Item_Plant_E_477839138",
0x4a8d9028,"MainField_Item_Plant_E_491187685",
0xeb61019a,"MainField_Item_Plant_E_495902559",
0x33c5ec2a,"MainField_Item_Plant_E_503027422",
0xd2220f5d,"MainField_Item_Plant_E_520581909",
0x21bd3d57,"MainField_Item_Plant_E_528233505",
0x6c17faf1,"MainField_Item_Plant_E_529089080",
0xc6466b57,"MainField_Item_Plant_E_538690007",
0xc409a689,"MainField_Item_Plant_E_554911053",
0x88bb69e3,"MainField_Item_Plant_E_563530060",
0x4d266465,"MainField_Item_Plant_E_597324522",
0x787384d1,"MainField_Item_Plant_E_609982583",
0x617afdd9,"MainField_Item_Plant_E_618604963",
0x39c764a6,"MainField_Item_Plant_E_629604189",
0x734ca9c6,"MainField_Item_Plant_E_648365043",
0xfa8860ac,"MainField_Item_Plant_E_65344333",
0x032e38cb,"MainField_Item_Plant_E_679804912",
0x879dcbdd,"MainField_Item_Plant_E_744433965",
0xc3290b2b,"MainField_Item_Plant_E_746522789",
0x438b58c2,"MainField_Item_Plant_E_748773168",
0x8d895c90,"MainField_Item_Plant_E_77592652",
0x836d5926,"MainField_Item_Plant_E_791184964",
0x39c67544,"MainField_Item_Plant_E_791760300",
0xa0fbdd99,"MainField_Item_Plant_E_792134824",
0xb9d4ce57,"MainField_Item_Plant_E_803868902",
0x3471d8a3,"MainField_Item_Plant_E_821922106",
0xa624b8f8,"MainField_Item_Plant_E_834326380",
0x9730d50d,"MainField_Item_Plant_E_839062340",
0xde50acf4,"MainField_Item_Plant_E_848025953",
0xb24a7ca1,"MainField_Item_Plant_E_851955274",
0xeb330227,"MainField_Item_Plant_E_863700567",
0xfccc4141,"MainField_Item_Plant_E_891753397",
0xa5f3d0c3,"MainField_Item_Plant_E_89668131",
0x1df16438,"MainField_Item_Plant_E_906363366",
0xccf4bcbd,"MainField_Item_Plant_E_910362550",
0xb9b2e35c,"MainField_Item_Plant_E_917427045",
0x70023503,"MainField_Item_Plant_E_925927750",
0x665429af,"MainField_Item_Plant_E_980019530",
0x729dea89,"MainField_Item_Plant_E_980853070",
0xcd2be659,"MainField_Item_Plant_E_982490967",
0x5574f084,"MainField_Item_Plant_E_989855873",
0x2008aa57,"MainField_Item_Plant_F_1193610548",
0x72889337,"MainField_Item_Plant_F_1232606634",
0xf949fcae,"MainField_Item_Plant_F_1297383971",
0x56b04257,"MainField_Item_Plant_F_1300172261",
0x00af97b4,"MainField_Item_Plant_F_1334620358",
0xa52136b2,"MainField_Item_Plant_F_1345210795",
0x2cf430aa,"MainField_Item_Plant_F_1433162166",
0x693dd30e,"MainField_Item_Plant_F_1447825921",
0x5e137579,"MainField_Item_Plant_F_1521475258",
0x53a17338,"MainField_Item_Plant_F_1555538097",
0xd3970e27,"MainField_Item_Plant_F_1640360537",
0x8f9cf046,"MainField_Item_Plant_F_1720804764",
0x501cd4cc,"MainField_Item_Plant_F_1751676159",
0xcbbe065d,"MainField_Item_Plant_F_197341565",
0x1558f8c7,"MainField_Item_Plant_F_1981122597",
0x334643f4,"MainField_Item_Plant_F_1987958375",
0x8f3d206f,"MainField_Item_Plant_F_2016461247",
0x7901d3dc,"MainField_Item_Plant_F_2113290472",
0xeafff1e6,"MainField_Item_Plant_F_2176374531",
0xaa057cd5,"MainField_Item_Plant_F_2227860145",
0x24bcc1b4,"MainField_Item_Plant_F_2261740416",
0x9092afb6,"MainField_Item_Plant_F_2332725552",
0xf77f8cc1,"MainField_Item_Plant_F_2413455893",
0x21123bec,"MainField_Item_Plant_F_2425975666",
0x239dbd58,"MainField_Item_Plant_F_244264122",
0xb5c274fc,"MainField_Item_Plant_F_2462918849",
0xadf41646,"MainField_Item_Plant_F_2531237134",
0x423bba83,"MainField_Item_Plant_F_2668085633",
0x788fe11e,"MainField_Item_Plant_F_2712996771",
0xfeb06634,"MainField_Item_Plant_F_2744690929",
0x7f672f8f,"MainField_Item_Plant_F_2746354074",
0xe5901a54,"MainField_Item_Plant_F_2761399144",
0xe2350b58,"MainField_Item_Plant_F_2866102106",
0xb8ff9e1e,"MainField_Item_Plant_F_2923465875",
0xd78205a3,"MainField_Item_Plant_F_3002060287",
0x6d32590d,"MainField_Item_Plant_F_3041616714",
0xf8249dfe,"MainField_Item_Plant_F_3046819965",
0x45648b92,"MainField_Item_Plant_F_304711447",
0xbf884c7d,"MainField_Item_Plant_F_3065277390",
0xdfa70831,"MainField_Item_Plant_F_3121201125",
0x59a4a3f0,"MainField_Item_Plant_F_3176729595",
0x4b35c4cc,"MainField_Item_Plant_F_3181948391",
0xca076e47,"MainField_Item_Plant_F_3198872503",
0xdb2bf358,"MainField_Item_Plant_F_3326933030",
0x4bbbfb33,"MainField_Item_Plant_F_3335250881",
0xe564f9fe,"MainField_Item_Plant_F_3417126889",
0xe7141c01,"MainField_Item_Plant_F_3419166110",
0x5f659fd3,"MainField_Item_Plant_F_3506294386",
0xcf7c43dc,"MainField_Item_Plant_F_3517403107",
0xbaf268e4,"MainField_Item_Plant_F_3535112602",
0x4dfa3f59,"MainField_Item_Plant_F_3556154396",
0xda76ec6f,"MainField_Item_Plant_F_3606563207",
0xe83b48a7,"MainField_Item_Plant_F_3701247156",
0x6a907ef2,"MainField_Item_Plant_F_3710335599",
0xb6cd8c01,"MainField_Item_Plant_F_3765435996",
0x9fac36b2,"MainField_Item_Plant_F_3782352632",
0x96b3879c,"MainField_Item_Plant_F_3837346399",
0x8a36238a,"MainField_Item_Plant_F_3848351206",
0x77856d6e,"MainField_Item_Plant_F_387871931",
0x409d8e26,"MainField_Item_Plant_F_3918529726",
0x9272ff87,"MainField_Item_Plant_F_399127802",
0xf389b90c,"MainField_Item_Plant_F_4053624090",
0xa0e64c8b,"MainField_Item_Plant_F_4113140551",
0x5b796efa,"MainField_Item_Plant_F_4175485016",
0xa7dee78a,"MainField_Item_Plant_F_516600801",
0xb8f9f83c,"MainField_Item_Plant_F_52978243",
0x85d02bc4,"MainField_Item_Plant_F_565593812",
0x80e9b13a,"MainField_Item_Plant_F_581897334",
0xec97f86f,"MainField_Item_Plant_F_601375577",
0xa9146ef9,"MainField_Item_Plant_F_604864861",
0x3ff45e17,"MainField_Item_Plant_F_631596512",
0x7ded258e,"MainField_Item_Plant_F_714876337",
0x79b0ea92,"MainField_Item_Plant_F_740517613",
0xc08e7498,"MainField_Item_Plant_F_753057140",
0x0cee13ee,"MainField_Item_Plant_F_785365528",
0x436d663c,"MainField_Item_Plant_F_859057603",
0x38f4eb0a,"MainField_Item_Plant_F_891545829",
0x0ad116d1,"MainField_Item_Plant_F_954383787",
0xab9b9029,"MainField_Item_Plant_G_1023627706",
0xf1491a42,"MainField_Item_Plant_G_1080792603",
0xafabefe8,"MainField_Item_Plant_G_1085370349",
0xa3171ae0,"MainField_Item_Plant_G_1116804454",
0xecb9deff,"MainField_Item_Plant_G_1235182797",
0x5a16a05b,"MainField_Item_Plant_G_1279365573",
0xec6225e6,"MainField_Item_Plant_G_1325977067",
0x51b0da45,"MainField_Item_Plant_G_137285148",
0xbcbd0372,"MainField_Item_Plant_G_1377201362",
0x1a73c508,"MainField_Item_Plant_G_1386920832",
0xe6dd6cf6,"MainField_Item_Plant_G_1405971632",
0xd15d077f,"MainField_Item_Plant_G_1424110988",
0xd04abfce,"MainField_Item_Plant_G_1515639202",
0x4cd4b687,"MainField_Item_Plant_G_1533920902",
0x2f9a36f9,"MainField_Item_Plant_G_1543732686",
0xdff8235e,"MainField_Item_Plant_G_1700389301",
0x18e40799,"MainField_Item_Plant_G_1870746740",
0x812eb76b,"MainField_Item_Plant_G_188668088",
0x13bbd137,"MainField_Item_Plant_G_1890929556",
0xd7c35fbd,"MainField_Item_Plant_G_1919072471",
0x3c716c3b,"MainField_Item_Plant_G_1927924588",
0xf074004c,"MainField_Item_Plant_G_1937567560",
0x74f93ba6,"MainField_Item_Plant_G_2006290566",
0x15ad5b34,"MainField_Item_Plant_G_2096416059",
0x10d350b5,"MainField_Item_Plant_G_2112273883",
0x0c12be39,"MainField_Item_Plant_G_2138014047",
0x3df9d6fe,"MainField_Item_Plant_G_2158368205",
0xf09186c6,"MainField_Item_Plant_G_2215688442",
0x8843bf42,"MainField_Item_Plant_G_2402944512",
0x9c1b1252,"MainField_Item_Plant_G_2456147999",
0x8d9e6609,"MainField_Item_Plant_G_2538109286",
0x8a5b1284,"MainField_Item_Plant_G_2551263814",
0xd043a961,"MainField_Item_Plant_G_2555154157",
0x2ae1c36d,"MainField_Item_Plant_G_2590026163",
0x1e2451f8,"MainField_Item_Plant_G_2700401733",
0xf2b1c619,"MainField_Item_Plant_G_2738288858",
0xc729e5dd,"MainField_Item_Plant_G_2800789688",
0x1a2c1778,"MainField_Item_Plant_G_2909579039",
0x9c530ed2,"MainField_Item_Plant_G_2911074817",
0xb90cd3c6,"MainField_Item_Plant_G_2923175636",
0xba023a8a,"MainField_Item_Plant_G_2931938301",
0x4d446581,"MainField_Item_Plant_G_2983619154",
0x1596be04,"MainField_Item_Plant_G_325078815",
0x6e82e1b5,"MainField_Item_Plant_G_3385397863",
0xc4d94cca,"MainField_Item_Plant_G_3385622154",
0xffe91552,"MainField_Item_Plant_G_3495224483",
0xa1f69137,"MainField_Item_Plant_G_3638994832",
0xb32b339e,"MainField_Item_Plant_G_3654518826",
0x403c135c,"MainField_Item_Plant_G_3721126287",
0xe80bb1db,"MainField_Item_Plant_G_3780146829",
0x1f8023c9,"MainField_Item_Plant_G_3784835884",
0x03afa1c4,"MainField_Item_Plant_G_3801178372",
0x9dea4198,"MainField_Item_Plant_G_3808564121",
0xaa5e239a,"MainField_Item_Plant_G_3816502829",
0xe7d8f278,"MainField_Item_Plant_G_3901431746",
0x7ce7ba66,"MainField_Item_Plant_G_3925401553",
0x6ee61263,"MainField_Item_Plant_G_3965916511",
0x7dca2667,"MainField_Item_Plant_G_3967998671",
0xe0d757e3,"MainField_Item_Plant_G_399356441",
0x5119d481,"MainField_Item_Plant_G_3993749578",
0x2bf32dda,"MainField_Item_Plant_G_4063257561",
0x93dc818e,"MainField_Item_Plant_G_4090410584",
0xad365db2,"MainField_Item_Plant_G_4122201254",
0x5ca590c7,"MainField_Item_Plant_G_4125136745",
0x8307e86d,"MainField_Item_Plant_G_4206040894",
0xdd7f709e,"MainField_Item_Plant_G_4217625946",
0x92e2981f,"MainField_Item_Plant_G_436757478",
0x9fc076e5,"MainField_Item_Plant_G_444118799",
0x03207d76,"MainField_Item_Plant_G_482559377",
0x3034da84,"MainField_Item_Plant_G_483654828",
0x24df5f06,"MainField_Item_Plant_G_491273504",
0xe7bc5870,"MainField_Item_Plant_G_552840321",
0x4b8312ae,"MainField_Item_Plant_G_599922164",
0xd95b4bf7,"MainField_Item_Plant_G_623430569",
0x6ba91f9a,"MainField_Item_Plant_G_637674452",
0xdbf52c26,"MainField_Item_Plant_G_641328503",
0x07cd7bc5,"MainField_Item_Plant_G_655349429",
0xe4c6ccbc,"MainField_Item_Plant_G_693429408",
0xe88a7203,"MainField_Item_Plant_G_716975127",
0x50e600cd,"MainField_Item_Plant_G_727515288",
0x5df57d63,"MainField_Item_Plant_G_763021161",
0x660ff9d5,"MainField_Item_Plant_G_769826163",
0xcb9ee3a6,"MainField_Item_Plant_G_813964513",
0x18437e5f,"MainField_Item_Plant_G_916432870",
0x4edee6ab,"MainField_Item_Plant_G_918970795",
0xeca4ba15,"MainField_Item_Plant_G_957246709",
0x3e318b20,"MainField_Item_Plant_H_1080179403",
0x96fd2163,"MainField_Item_Plant_H_1103786480",
0x2b2f19d6,"MainField_Item_Plant_H_1167656205",
0xa8026eb4,"MainField_Item_Plant_H_1174498948",
0x07e17aa9,"MainField_Item_Plant_H_1212104621",
0x7c081994,"MainField_Item_Plant_H_1223174172",
0x634958f3,"MainField_Item_Plant_H_1235082554",
0x8a0773f6,"MainField_Item_Plant_H_1235938123",
0xbc07d95f,"MainField_Item_Plant_H_1242151793",
0xd3c91e7f,"MainField_Item_Plant_H_1302364146",
0xa0691fbf,"MainField_Item_Plant_H_1327922403",
0xdab7d8aa,"MainField_Item_Plant_H_1360233129",
0x56fbeaf1,"MainField_Item_Plant_H_1432060206",
0x4a410488,"MainField_Item_Plant_H_1463134056",
0xdfed7df8,"MainField_Item_Plant_H_1567104823",
0x70dac2ee,"MainField_Item_Plant_H_1585000284",
0xbc8923b3,"MainField_Item_Plant_H_1648347357",
0xf8724f2d,"MainField_Item_Plant_H_1742877271",
0x60c417c6,"MainField_Item_Plant_H_1748782378",
0xf6e8247d,"MainField_Item_Plant_H_1843079325",
0x98396503,"MainField_Item_Plant_H_185178208",
0xee483b06,"MainField_Item_Plant_H_1861113495",
0x05b1a7cd,"MainField_Item_Plant_H_1965866308",
0x0ba3568c,"MainField_Item_Plant_H_21180532",
0xe70bf79c,"MainField_Item_Plant_H_2127583407",
0xbf0cd962,"MainField_Item_Plant_H_2156080630",
0x6a59a5b5,"MainField_Item_Plant_H_215736614",
0x318c9724,"MainField_Item_Plant_H_2173012741",
0x5e89c7bf,"MainField_Item_Plant_H_2252619276",
0xbd1b4eec,"MainField_Item_Plant_H_2310257680",
0xe169c61b,"MainField_Item_Plant_H_2334230379",
0xb70b9760,"MainField_Item_Plant_H_2373684997",
0x3856663b,"MainField_Item_Plant_H_2375279578",
0x6770da0c,"MainField_Item_Plant_H_2388823472",
0x41a0c4ca,"MainField_Item_Plant_H_2433957793",
0x56f91964,"MainField_Item_Plant_H_2439822981",
0x9b14e8ad,"MainField_Item_Plant_H_244404011",
0x90d269b2,"MainField_Item_Plant_H_2479773850",
0x16b17ca8,"MainField_Item_Plant_H_2505759775",
0x23c4f3c2,"MainField_Item_Plant_H_2519631942",
0xaafdc541,"MainField_Item_Plant_H_2534386552",
0x572391f9,"MainField_Item_Plant_H_2577053056",
0x796f398c,"MainField_Item_Plant_H_2733964818",
0x83fe45e6,"MainField_Item_Plant_H_2792468665",
0x23810cc8,"MainField_Item_Plant_H_2842432740",
0xcb4ee3a4,"MainField_Item_Plant_H_2905055639",
0x58516e79,"MainField_Item_Plant_H_2912934298",
0xccf1f981,"MainField_Item_Plant_H_3017854841",
0x201f8081,"MainField_Item_Plant_H_3032884010",
0x97d1b2d9,"MainField_Item_Plant_H_3091460597",
0x4ee3bc94,"MainField_Item_Plant_H_3251313935",
0x8c8cf23c,"MainField_Item_Plant_H_3275711518",
0x41c8f945,"MainField_Item_Plant_H_3373821410",
0x3fbd8a24,"MainField_Item_Plant_H_3384991732",
0x0bb6d5a0,"MainField_Item_Plant_H_3498344947",
0xbe0be2aa,"MainField_Item_Plant_H_3524561868",
0xe7ce0822,"MainField_Item_Plant_H_3547462465",
0xd68df6bf,"MainField_Item_Plant_H_3558325635",
0x86acab9d,"MainField_Item_Plant_H_35817236",
0xf9e6075f,"MainField_Item_Plant_H_3583852699",
0x4faebfdc,"MainField_Item_Plant_H_3586523243",
0xfeb3f5a7,"MainField_Item_Plant_H_361865429",
0xcb9432e3,"MainField_Item_Plant_H_3641394857",
0x9c49b33d,"MainField_Item_Plant_H_3645119863",
0xcd6f0986,"MainField_Item_Plant_H_3662376046",
0x6ec00589,"MainField_Item_Plant_H_3677043893",
0x2ffe8e09,"MainField_Item_Plant_H_3740335374",
0xa9975531,"MainField_Item_Plant_H_3781882762",
0xfe88cb9a,"MainField_Item_Plant_H_3834243353",
0x09eb7d2c,"MainField_Item_Plant_H_3883296893",
0xd41dbc37,"MainField_Item_Plant_H_394679576",
0xd25d193f,"MainField_Item_Plant_H_3975421551",
0x1ab7fd5e,"MainField_Item_Plant_H_4023457871",
0x7b6efdce,"MainField_Item_Plant_H_4029501294",
0xd311fdab,"MainField_Item_Plant_H_4040965168",
0x56dbe609,"MainField_Item_Plant_H_4151289192",
0xabd72ab2,"MainField_Item_Plant_H_4181760932",
0xa9d000d2,"MainField_Item_Plant_H_4207028548",
0x4819bbd9,"MainField_Item_Plant_H_4214958557",
0xf7a0ff8d,"MainField_Item_Plant_H_4248176776",
0x23a22d74,"MainField_Item_Plant_H_4259760794",
0x3c452d12,"MainField_Item_Plant_H_4288560266",
0x56d79579,"MainField_Item_Plant_H_459511281",
0x495824aa,"MainField_Item_Plant_H_479083741",
0xbd5df6a8,"MainField_Item_Plant_H_538024000",
0x9143284b,"MainField_Item_Plant_H_541106891",
0xca996909,"MainField_Item_Plant_H_543260074",
0xee184e45,"MainField_Item_Plant_H_560341345",
0x47b7272c,"MainField_Item_Plant_H_560540898",
0x57d1fdd9,"MainField_Item_Plant_H_564780762",
0x94a31eab,"MainField_Item_Plant_H_569221366",
0x615af9e5,"MainField_Item_Plant_H_624321635",
0x83df77d3,"MainField_Item_Plant_H_63725843",
0x514f3a90,"MainField_Item_Plant_H_640924267",
0x89ed258d,"MainField_Item_Plant_H_664858903",
0xf6474551,"MainField_Item_Plant_H_703827245",
0xfd908239,"MainField_Item_Plant_H_706118505",
0xe5f2a9b6,"MainField_Item_Plant_H_856161896",
0xcd1aa5fc,"MainField_Item_Plant_H_920842861",
0x16d1668f,"MainField_Item_Plant_H_947809273",
0xb296562d,"MainField_Item_Plant_H_983071329",
0x720f4b57,"MainField_Item_Plant_H_991750451",
0xa78732ba,"MainField_Item_Plant_H_995474468",
0x15e03cb3,"MainField_Item_Plant_I_1003289484",
0x1bc43b0e,"MainField_Item_Plant_I_1007785634",
0x2805495b,"MainField_Item_Plant_I_1047362065",
0xb67ab2d0,"MainField_Item_Plant_I_1047413227",
0x25a1e5c4,"MainField_Item_Plant_I_1053779319",
0xf8e5635f,"MainField_Item_Plant_I_1100192111",
0x32b03e5c,"MainField_Item_Plant_I_1153295188",
0x36c73a76,"MainField_Item_Plant_I_1166401697",
0xac2de429,"MainField_Item_Plant_I_1222007486",
0x498a148b,"MainField_Item_Plant_I_1226981700",
0xd1562cab,"MainField_Item_Plant_I_1233691467",
0xa8b7a518,"MainField_Item_Plant_I_1246636979",
0x80debd9f,"MainField_Item_Plant_I_1302735623",
0x81d6c98c,"MainField_Item_Plant_I_1385132843",
0x87ddf33d,"MainField_Item_Plant_I_1444161781",
0xae5a889b,"MainField_Item_Plant_I_1496133043",
0x12a34176,"MainField_Item_Plant_I_1506015398",
0xee086bcf,"MainField_Item_Plant_I_1522215798",
0xe8ccd147,"MainField_Item_Plant_I_1620698669",
0xb7e0f9a0,"MainField_Item_Plant_I_1656237452",
0xbd5837d4,"MainField_Item_Plant_I_1677346837",
0x7f195ac7,"MainField_Item_Plant_I_171498006",
0xa43ef0e6,"MainField_Item_Plant_I_1746313319",
0xbcf38f84,"MainField_Item_Plant_I_1757582850",
0x620a0cfa,"MainField_Item_Plant_I_1792543826",
0x517bc84f,"MainField_Item_Plant_I_1800869514",
0x3ede9788,"MainField_Item_Plant_I_1829493924",
0xc9c700d4,"MainField_Item_Plant_I_1851395263",
0xa5fe850b,"MainField_Item_Plant_I_1856000326",
0xa5cbe717,"MainField_Item_Plant_I_1967442665",
0x7d9ac829,"MainField_Item_Plant_I_1973308021",
0xca76daed,"MainField_Item_Plant_I_2007815002",
0x2b26276d,"MainField_Item_Plant_I_2022721082",
0x7891754b,"MainField_Item_Plant_I_2189143955",
0xc2277930,"MainField_Item_Plant_I_2202698319",
0xff8522c9,"MainField_Item_Plant_I_2208341334",
0x86820abd,"MainField_Item_Plant_I_2211547447",
0x9ac6b954,"MainField_Item_Plant_I_2294223858",
0x8ab2ea3a,"MainField_Item_Plant_I_2397529575",
0x4c059a06,"MainField_Item_Plant_I_2431907217",
0x78f31ee1,"MainField_Item_Plant_I_244766506",
0x9fc50767,"MainField_Item_Plant_I_2479616056",
0x94b9e5c4,"MainField_Item_Plant_I_2575373176",
0x76a821a1,"MainField_Item_Plant_I_2642396485",
0xccbe79b6,"MainField_Item_Plant_I_2676053555",
0x9c1fcd49,"MainField_Item_Plant_I_2699159710",
0x446eb4fd,"MainField_Item_Plant_I_270678889",
0xda51cf6f,"MainField_Item_Plant_I_2714649877",
0xae76410f,"MainField_Item_Plant_I_272556726",
0x4d6afea5,"MainField_Item_Plant_I_2727541789",
0x0c1f6d2e,"MainField_Item_Plant_I_2738101491",
0x5c7b3bde,"MainField_Item_Plant_I_2745715243",
0x35d4b577,"MainField_Item_Plant_I_2768545479",
0xdf06d95d,"MainField_Item_Plant_I_2814291776",
0x2f23076e,"MainField_Item_Plant_I_282783204",
0x925990dc,"MainField_Item_Plant_I_2842127341",
0x9d0c1537,"MainField_Item_Plant_I_2852433992",
0x85c55b85,"MainField_Item_Plant_I_285760867",
0xd612ae15,"MainField_Item_Plant_I_2893976847",
0xd0666190,"MainField_Item_Plant_I_2897930490",
0x32655775,"MainField_Item_Plant_I_2908989732",
0x37da7d10,"MainField_Item_Plant_I_2928789785",
0xa34e5490,"MainField_Item_Plant_I_2957409044",
0xd822b0f0,"MainField_Item_Plant_I_2992402784",
0xbde126fb,"MainField_Item_Plant_I_3017933334",
0xc8a8fd52,"MainField_Item_Plant_I_3026488748",
0x6997f06c,"MainField_Item_Plant_I_3031343067",
0xfca02a89,"MainField_Item_Plant_I_3073097485",
0x49bd0229,"MainField_Item_Plant_I_3082160476",
0x207d3bdf,"MainField_Item_Plant_I_3111724535",
0x7d8d5bff,"MainField_Item_Plant_I_3124647042",
0xda0aabb3,"MainField_Item_Plant_I_3159263882",
0xfa718d7b,"MainField_Item_Plant_I_3172061872",
0x12bafb16,"MainField_Item_Plant_I_3191558655",
0x9009d2af,"MainField_Item_Plant_I_3209277095",
0x7ffa20c6,"MainField_Item_Plant_I_3232635500",
0xcb6b04b7,"MainField_Item_Plant_I_3286920257",
0xeedb7669,"MainField_Item_Plant_I_3357603909",
0x41dc4d5a,"MainField_Item_Plant_I_3383925514",
0x498a0da0,"MainField_Item_Plant_I_339729355",
0x92b53d3c,"MainField_Item_Plant_I_342152111",
0xc6b1c891,"MainField_Item_Plant_I_3447922288",
0x6a928141,"MainField_Item_Plant_I_3472577796",
0xd9846870,"MainField_Item_Plant_I_3491882791",
0xbb9353ed,"MainField_Item_Plant_I_3496671599",
0xaba96cb9,"MainField_Item_Plant_I_3500017323",
0x3309e7a6,"MainField_Item_Plant_I_3556545634",
0x71689868,"MainField_Item_Plant_I_3589042784",
0x2761064a,"MainField_Item_Plant_I_3603100874",
0xafc341d2,"MainField_Item_Plant_I_3679731597",
0xdb33d66a,"MainField_Item_Plant_I_3739296982",
0xa0c823bb,"MainField_Item_Plant_I_3769316753",
0x76855737,"MainField_Item_Plant_I_3808616498",
0x55c360c0,"MainField_Item_Plant_I_3822103859",
0xbfd0e324,"MainField_Item_Plant_I_3839491598",
0x8d96bd84,"MainField_Item_Plant_I_3843010639",
0x2ca37f63,"MainField_Item_Plant_I_3935741053",
0x4ff381b4,"MainField_Item_Plant_I_3942725229",
0xfddc0bd8,"MainField_Item_Plant_I_3946881924",
0xfd364467,"MainField_Item_Plant_I_3998002092",
0x3c24126a,"MainField_Item_Plant_I_4004982350",
0x0898488f,"MainField_Item_Plant_I_4037331724",
0x2281280c,"MainField_Item_Plant_I_4042864590",
0x644499b0,"MainField_Item_Plant_I_4116817203",
0x2281f13f,"MainField_Item_Plant_I_4122816249",
0x93352727,"MainField_Item_Plant_I_4124139167",
0x33ead509,"MainField_Item_Plant_I_4163478420",
0x8ef4993e,"MainField_Item_Plant_I_4178206432",
0x0c5d621e,"MainField_Item_Plant_I_4269582637",
0x683870d2,"MainField_Item_Plant_I_451322612",
0x1de51426,"MainField_Item_Plant_I_469955037",
0x6969d421,"MainField_Item_Plant_I_474754002",
0xf5f3d83d,"MainField_Item_Plant_I_506560858",
0xd3890add,"MainField_Item_Plant_I_508438384",
0xc6e65d71,"MainField_Item_Plant_I_51768321",
0x4188ce9a,"MainField_Item_Plant_I_59407827",
0x679aa7cb,"MainField_Item_Plant_I_609299036",
0xb7201269,"MainField_Item_Plant_I_635578870",
0xbe2e2c0f,"MainField_Item_Plant_I_693054126",
0xf33f39e4,"MainField_Item_Plant_I_70222668",
0x702e123d,"MainField_Item_Plant_I_734228553",
0xf375f7e1,"MainField_Item_Plant_I_762729676",
0x3d7d3acb,"MainField_Item_Plant_I_825063954",
0x35d0e038,"MainField_Item_Plant_I_834807147",
0x24bd8f16,"MainField_Item_Plant_I_840803303",
0x943ba83e,"MainField_Item_Plant_I_865827184",
0xa68772bd,"MainField_Item_Plant_I_878627940",
0xdd136721,"MainField_Item_Plant_I_908102535",
0x43e90d6a,"MainField_Item_Plant_I_933519192",
0xcb3775b6,"MainField_Item_Plant_I_960512110",
0x35777bd0,"MainField_Item_Plant_I_983636881",
0x1b1ab13b,"MainField_Item_Plant_J_1471397644",
0x95b51d89,"MainField_Item_Plant_J_1526482848",
0x647a3652,"MainField_Item_Plant_J_1598939842",
0xbf22d1a9,"MainField_Item_Plant_J_1645678582",
0xd7c17b12,"MainField_Item_Plant_J_1709704234",
0xc24cc9bb,"MainField_Item_Plant_J_1738763659",
0xb50d275b,"MainField_Item_Plant_J_1837118606",
0x8b1654d0,"MainField_Item_Plant_J_1902024990",
0x2b7d9017,"MainField_Item_Plant_J_1968165856",
0x897edba0,"MainField_Item_Plant_J_1998345477",
0x2c048739,"MainField_Item_Plant_J_2020240091",
0xd09ec594,"MainField_Item_Plant_J_2187095956",
0x8a3d0ad8,"MainField_Item_Plant_J_2190366726",
0xbfcfba8d,"MainField_Item_Plant_J_2191111630",
0xb3da6fd9,"MainField_Item_Plant_J_2238433007",
0xc52ea024,"MainField_Item_Plant_J_2413888551",
0xdab114c1,"MainField_Item_Plant_J_2525651868",
0xe4c6a4a3,"MainField_Item_Plant_J_3012709970",
0xabcd29b2,"MainField_Item_Plant_J_3266410010",
0x9b8d0528,"MainField_Item_Plant_J_3305734781",
0xf00cc772,"MainField_Item_Plant_J_3412441889",
0x74518258,"MainField_Item_Plant_J_3514498367",
0x5f837821,"MainField_Item_Plant_J_3568270013",
0x9d6759fc,"MainField_Item_Plant_J_3860635973",
0x7bc8f816,"MainField_Item_Plant_J_3993825967",
0xa7c2d872,"MainField_Item_Plant_J_4100904948",
0x75bff852,"MainField_Item_Plant_J_4195072129",
0x532a453b,"MainField_Item_Plant_J_4212083254",
0xa82194e0,"MainField_Item_Plant_L_1002928377",
0x4af4887d,"MainField_Item_Plant_L_1007319771",
0xe9129be3,"MainField_Item_Plant_L_119628884",
0xef4fcd8e,"MainField_Item_Plant_L_1207446271",
0xe62a4ed3,"MainField_Item_Plant_L_1244617781",
0x0569b29e,"MainField_Item_Plant_L_1296722406",
0xadcb4421,"MainField_Item_Plant_L_1311815944",
0xd3b3e35a,"MainField_Item_Plant_L_1393345616",
0x0c55bee4,"MainField_Item_Plant_L_139623637",
0x5b3a1fbd,"MainField_Item_Plant_L_1407330758",
0x553f5e87,"MainField_Item_Plant_L_1414803766",
0xba4078fb,"MainField_Item_Plant_L_1484617036",
0xd3c08ea7,"MainField_Item_Plant_L_1512223052",
0xe359a150,"MainField_Item_Plant_L_153535691",
0xe8697117,"MainField_Item_Plant_L_1559971486",
0x64904b18,"MainField_Item_Plant_L_1606069943",
0x0dce5483,"MainField_Item_Plant_L_1606490343",
0x32821736,"MainField_Item_Plant_L_1611750446",
0x9d520ac3,"MainField_Item_Plant_L_1642823055",
0x40033be5,"MainField_Item_Plant_L_1665618043",
0x6730fa14,"MainField_Item_Plant_L_1705542534",
0x9150c160,"MainField_Item_Plant_L_1709673144",
0x140a3ca4,"MainField_Item_Plant_L_1884547388",
0x1bc64be4,"MainField_Item_Plant_L_1967352365",
0x67ca0f32,"MainField_Item_Plant_L_2051612564",
0x3887feed,"MainField_Item_Plant_L_2275832177",
0x8165545d,"MainField_Item_Plant_L_2278309943",
0xce2e7794,"MainField_Item_Plant_L_2443925794",
0xe3820b5e,"MainField_Item_Plant_L_2453102728",
0x9f980eba,"MainField_Item_Plant_L_2471477021",
0xa0ea6573,"MainField_Item_Plant_L_249763241",
0xfb5a9611,"MainField_Item_Plant_L_249981040",
0xd986a129,"MainField_Item_Plant_L_2551473326",
0x5a813916,"MainField_Item_Plant_L_2572780496",
0x9488c4f9,"MainField_Item_Plant_L_2599004929",
0x02e48216,"MainField_Item_Plant_L_2636312545",
0xe9c59aec,"MainField_Item_Plant_L_276704524",
0x5827ef72,"MainField_Item_Plant_L_279129769",
0x31495586,"MainField_Item_Plant_L_2817849901",
0x429978c5,"MainField_Item_Plant_L_2859690640",
0x631367c4,"MainField_Item_Plant_L_2941325162",
0x56488520,"MainField_Item_Plant_L_2965163572",
0x0075f5c6,"MainField_Item_Plant_L_2994726607",
0x66916cca,"MainField_Item_Plant_L_3006007502",
0xf4a8f0a7,"MainField_Item_Plant_L_3025135234",
0xd98c2ee6,"MainField_Item_Plant_L_3070753619",
0xd7d6c472,"MainField_Item_Plant_L_3095108457",
0x31716a17,"MainField_Item_Plant_L_3220661680",
0x467ca9f3,"MainField_Item_Plant_L_3232577250",
0x9fdd784e,"MainField_Item_Plant_L_3308633915",
0x50298b52,"MainField_Item_Plant_L_3355968819",
0x42d1f729,"MainField_Item_Plant_L_3392408122",
0xb83e9585,"MainField_Item_Plant_L_3445424369",
0xb31a6cc6,"MainField_Item_Plant_L_3529606799",
0xf9207b75,"MainField_Item_Plant_L_3530275713",
0x824270ce,"MainField_Item_Plant_L_3570352464",
0x3cdec069,"MainField_Item_Plant_L_361734462",
0x977d6708,"MainField_Item_Plant_L_3619504141",
0x3a375f85,"MainField_Item_Plant_L_3678623585",
0x5c482ce2,"MainField_Item_Plant_L_3808483841",
0x19b79553,"MainField_Item_Plant_L_3812969435",
0x4db564a6,"MainField_Item_Plant_L_381944383",
0x3fbf6cfb,"MainField_Item_Plant_L_3899927166",
0x7a916de8,"MainField_Item_Plant_L_3994384605",
0x1cd754e9,"MainField_Item_Plant_L_3998921251",
0xad78244b,"MainField_Item_Plant_L_4007208305",
0x752b6e59,"MainField_Item_Plant_L_411469761",
0xc1bbd7c2,"MainField_Item_Plant_L_4140338536",
0xf626126d,"MainField_Item_Plant_L_4154261037",
0xc3825d18,"MainField_Item_Plant_L_4156748997",
0xe603ba8d,"MainField_Item_Plant_L_4276991491",
0x695d1173,"MainField_Item_Plant_L_438694822",
0x09de65d8,"MainField_Item_Plant_L_540789507",
0x897973f9,"MainField_Item_Plant_L_546071465",
0x8230ea83,"MainField_Item_Plant_L_547479694",
0x7afed125,"MainField_Item_Plant_L_573118324",
0xbb243083,"MainField_Item_Plant_L_579083521",
0x31840946,"MainField_Item_Plant_L_632492511",
0x524bfea2,"MainField_Item_Plant_L_696424696",
0x369305d1,"MainField_Item_Plant_L_754853173",
0xc78a8ca9,"MainField_Item_Plant_L_767153166",
0x4bf2dec5,"MainField_Item_Plant_L_77463031",
0xec710e8c,"MainField_Item_Plant_L_776470332",
0x3ea75a92,"MainField_Item_Plant_L_781363428",
0x3d50796a,"MainField_Item_Plant_L_858046665",
0x8873b817,"MainField_Item_Plant_L_888716994",
0xc7093268,"MainField_Item_Plant_L_91744980",
0x05355980,"MainField_Item_Plant_L_971507967",
0xec30e482,"MainField_Item_Plant_L_99348255",
0x14b3fe39,"MainField_Item_Plant_M_1023470491",
0x97a68d6b,"MainField_Item_Plant_M_1455400772",
0xc9b2d4cf,"MainField_Item_Plant_M_1821916874",
0xf66e5c01,"MainField_Item_Plant_M_1928252306",
0xd4dbf686,"MainField_Item_Plant_M_2190893030",
0x24a957f7,"MainField_Item_Plant_M_2504897157",
0x37cc1879,"MainField_Item_Plant_M_2505500309",
0x2051c1fb,"MainField_Item_Plant_M_2875346352",
0x3036616d,"MainField_Item_Plant_M_2968511466",
0x73285d7c,"MainField_Item_Plant_M_3045586959",
0x87baf18f,"MainField_Item_Plant_M_3797014019",
0xb78f8b0e,"MainField_Item_Plant_M_3957824040",
0xb1a73997,"MainField_Item_Plant_M_4119949168",
0x8bc3cb57,"MainField_Item_Plant_M_4169654739",
0xab66ac8c,"MainField_Item_Plant_M_4223172269",
0xfa7d9e93,"MainField_Item_Plant_M_463408732",
0x9ecd08a8,"MainField_Item_Plant_M_740189079",
0x0791ec7e,"MainField_Item_Plant_M_784640199",
0xb3aa6b2c,"MainField_Item_Plant_M_99220228",
0x87188d97,"MainField_Item_Plant_O_1003339251",
0xac3409be,"MainField_Item_Plant_O_1005235963",
0xc9043e6c,"MainField_Item_Plant_O_100663604",
0x0fb08d5c,"MainField_Item_Plant_O_1009947073",
0xff69e4f7,"MainField_Item_Plant_O_1010969153",
0x54a14a89,"MainField_Item_Plant_O_1011513025",
0xb47d863a,"MainField_Item_Plant_O_1012150102",
0x7e1bbe4a,"MainField_Item_Plant_O_1017544983",
0xad0b6a53,"MainField_Item_Plant_O_1023070111",
0xa0fb2c23,"MainField_Item_Plant_O_1035766463",
0xdb49fbf4,"MainField_Item_Plant_O_1037360203",
0xecc96967,"MainField_Item_Plant_O_1037614963",
0x7dab0191,"MainField_Item_Plant_O_1039052004",
0x230328db,"MainField_Item_Plant_O_1041583401",
0x6b3b10df,"MainField_Item_Plant_O_1048980080",
0x0fbcff6c,"MainField_Item_Plant_O_1049129090",
0x2070c0ca,"MainField_Item_Plant_O_1049444922",
0xad6d0d7e,"MainField_Item_Plant_O_1057751978",
0x9faf2428,"MainField_Item_Plant_O_1058242947",
0x4d9d9e0b,"MainField_Item_Plant_O_1064728977",
0x0645c1ab,"MainField_Item_Plant_O_1065604888",
0x2ad073e4,"MainField_Item_Plant_O_1084900569",
0x6a090402,"MainField_Item_Plant_O_1086130439",
0x18e348f6,"MainField_Item_Plant_O_1090167398",
0xc12adff1,"MainField_Item_Plant_O_1092146239",
0xeb8849c4,"MainField_Item_Plant_O_1097027690",
0x86cf1f31,"MainField_Item_Plant_O_1109770876",
0xaf972a98,"MainField_Item_Plant_O_1111165272",
0x7b80861b,"MainField_Item_Plant_O_1113209827",
0xfb21ec19,"MainField_Item_Plant_O_1130256632",
0x58812a23,"MainField_Item_Plant_O_113109556",
0xab505848,"MainField_Item_Plant_O_1134910222",
0x9e2acf8c,"MainField_Item_Plant_O_1136701688",
0xe5216c6c,"MainField_Item_Plant_O_1142468779",
0x26f86f5d,"MainField_Item_Plant_O_1146185818",
0x296c43c0,"MainField_Item_Plant_O_1148963320",
0x45c8385f,"MainField_Item_Plant_O_1149876692",
0x8268a09c,"MainField_Item_Plant_O_1151467116",
0xa032e855,"MainField_Item_Plant_O_1155905585",
0x9a43deae,"MainField_Item_Plant_O_1156376533",
0x6b8c9a70,"MainField_Item_Plant_O_1161161453",
0x9fa0e60c,"MainField_Item_Plant_O_1168900987",
0x7fbcf9ab,"MainField_Item_Plant_O_1169482575",
0xcb049d76,"MainField_Item_Plant_O_1172796405",
0x57f9d758,"MainField_Item_Plant_O_1176831575",
0xe602273a,"MainField_Item_Plant_O_1177139263",
0x9f2096a1,"MainField_Item_Plant_O_1178834903",
0xcae3e312,"MainField_Item_Plant_O_1181603111",
0xc2a25987,"MainField_Item_Plant_O_1182001938",
0x65acf493,"MainField_Item_Plant_O_1182888269",
0x3ff66dd3,"MainField_Item_Plant_O_1186043150",
0x7c6acabd,"MainField_Item_Plant_O_1189344694",
0x8826ced0,"MainField_Item_Plant_O_1190428999",
0xc2fcfbe9,"MainField_Item_Plant_O_1198648149",
0x5ea993a5,"MainField_Item_Plant_O_1199539399",
0x914b4e2e,"MainField_Item_Plant_O_1200456380",
0x70969bf5,"MainField_Item_Plant_O_1200732523",
0x21ae9ef7,"MainField_Item_Plant_O_1205552548",
0x5a1d165c,"MainField_Item_Plant_O_1206258783",
0x382ebe06,"MainField_Item_Plant_O_1211496263",
0x6b469360,"MainField_Item_Plant_O_1216771816",
0x60ad9045,"MainField_Item_Plant_O_1221210973",
0xd60058e3,"MainField_Item_Plant_O_1224666624",
0x5d903864,"MainField_Item_Plant_O_1230748794",
0x0269e1f8,"MainField_Item_Plant_O_1236002348",
0x91575e4d,"MainField_Item_Plant_O_1244996532",
0xb945a331,"MainField_Item_Plant_O_1247300787",
0x44c9c5d9,"MainField_Item_Plant_O_1252107653",
0xec195b55,"MainField_Item_Plant_O_1254513559",
0x1b5e9ca5,"MainField_Item_Plant_O_1265655142",
0x4cf4114f,"MainField_Item_Plant_O_1266243415",
0x2e1b606c,"MainField_Item_Plant_O_126789198",
0x4ee6e5a9,"MainField_Item_Plant_O_1281183343",
0xd61a46da,"MainField_Item_Plant_O_1281797200",
0xe32530b4,"MainField_Item_Plant_O_1282008474",
0x6ccd09cc,"MainField_Item_Plant_O_1282642542",
0xa7075635,"MainField_Item_Plant_O_1287569192",
0x04dcc538,"MainField_Item_Plant_O_1290989031",
0x8a9248e2,"MainField_Item_Plant_O_1291711117",
0xfc38cf85,"MainField_Item_Plant_O_1298209780",
0xa535adb4,"MainField_Item_Plant_O_1305472593",
0x412767e2,"MainField_Item_Plant_O_1311730029",
0xeab30a2c,"MainField_Item_Plant_O_1316841990",
0xb165f8a8,"MainField_Item_Plant_O_1317111836",
0xeb66480a,"MainField_Item_Plant_O_1320919212",
0xd614b832,"MainField_Item_Plant_O_1321132334",
0xa53fbd5f,"MainField_Item_Plant_O_1321667889",
0x6b930e2c,"MainField_Item_Plant_O_1324605500",
0xf65af445,"MainField_Item_Plant_O_1334812363",
0xb85e67cc,"MainField_Item_Plant_O_1336935492",
0xf54d5159,"MainField_Item_Plant_O_134307318",
0x98d9a55d,"MainField_Item_Plant_O_1349974976",
0x5a00f59b,"MainField_Item_Plant_O_1351806993",
0x2d6a6219,"MainField_Item_Plant_O_1360946461",
0x3b6981fb,"MainField_Item_Plant_O_1365715717",
0xa28730dc,"MainField_Item_Plant_O_1377361353",
0xbf27d7ca,"MainField_Item_Plant_O_1385918698",
0x7de937a4,"MainField_Item_Plant_O_1394181102",
0x95bc7cd1,"MainField_Item_Plant_O_1407309312",
0x438ee872,"MainField_Item_Plant_O_1408795804",
0x6f936354,"MainField_Item_Plant_O_1409393796",
0x6b4654dc,"MainField_Item_Plant_O_141338667",
0xf9e4ce8e,"MainField_Item_Plant_O_1415014142",
0xcf8398fa,"MainField_Item_Plant_O_1418863114",
0xd4090937,"MainField_Item_Plant_O_1421656683",
0x87d0fe61,"MainField_Item_Plant_O_1422509638",
0x8c02c26c,"MainField_Item_Plant_O_1428558647",
0xa482c63a,"MainField_Item_Plant_O_1431300539",
0xfd19932b,"MainField_Item_Plant_O_1436756181",
0xfef08128,"MainField_Item_Plant_O_1437095090",
0x53fb8ec1,"MainField_Item_Plant_O_1447803033",
0x4f7e7b72,"MainField_Item_Plant_O_1450869578",
0x694b3a7b,"MainField_Item_Plant_O_1451009772",
0x326ace16,"MainField_Item_Plant_O_1452201221",
0xbf737deb,"MainField_Item_Plant_O_1456882266",
0x60a9e7ca,"MainField_Item_Plant_O_1457197106",
0x9c22b7fe,"MainField_Item_Plant_O_1461508971",
0x07a7ffc4,"MainField_Item_Plant_O_1470061619",
0x12d08f1a,"MainField_Item_Plant_O_1477910003",
0xa1c5d45c,"MainField_Item_Plant_O_1480145038",
0x958fabb1,"MainField_Item_Plant_O_148157771",
0x9ba399e7,"MainField_Item_Plant_O_1487389110",
0x4746c3a6,"MainField_Item_Plant_O_1490331754",
0x70db9e3e,"MainField_Item_Plant_O_1492076479",
0x3d89f7da,"MainField_Item_Plant_O_14936301",
0x17325662,"MainField_Item_Plant_O_149460052",
0xe2f58da4,"MainField_Item_Plant_O_1495359432",
0xa40311a5,"MainField_Item_Plant_O_1502445386",
0x7b75505d,"MainField_Item_Plant_O_1502992282",
0x3e8ebdc6,"MainField_Item_Plant_O_1505802219",
0x3fc966fa,"MainField_Item_Plant_O_1506701981",
0x26e72ae9,"MainField_Item_Plant_O_1518313979",
0x16aaa56e,"MainField_Item_Plant_O_1520658867",
0x6e0c8196,"MainField_Item_Plant_O_1521700977",
0xd244f79c,"MainField_Item_Plant_O_152942616",
0x89b7c8a6,"MainField_Item_Plant_O_153547597",
0xb821bbb4,"MainField_Item_Plant_O_1537965686",
0xf6b4f212,"MainField_Item_Plant_O_1540193506",
0x5055595a,"MainField_Item_Plant_O_1541476324",
0x0d4e7b0f,"MainField_Item_Plant_O_1547645518",
0xcccea585,"MainField_Item_Plant_O_1550990657",
0xa6c8dc20,"MainField_Item_Plant_O_1552285450",
0xf0f2cfd0,"MainField_Item_Plant_O_1553186410",
0x8dcd80c6,"MainField_Item_Plant_O_1553843489",
0x4aeab8cb,"MainField_Item_Plant_O_1553915564",
0x2186dc3b,"MainField_Item_Plant_O_1554364687",
0x9335721d,"MainField_Item_Plant_O_1555022645",
0x1f66fb14,"MainField_Item_Plant_O_1565581210",
0x8169079c,"MainField_Item_Plant_O_1565723965",
0xda6ed5c4,"MainField_Item_Plant_O_1565837220",
0x55e30ca7,"MainField_Item_Plant_O_1566406400",
0xf8166f6a,"MainField_Item_Plant_O_1567560615",
0x8f98881c,"MainField_Item_Plant_O_1569172221",
0xbb08bdc2,"MainField_Item_Plant_O_1569870343",
0xb5cbe9fa,"MainField_Item_Plant_O_1570182572",
0x1b15979b,"MainField_Item_Plant_O_1570212946",
0x84d7867b,"MainField_Item_Plant_O_1570774253",
0x514e4095,"MainField_Item_Plant_O_158401342",
0x95f6a9fa,"MainField_Item_Plant_O_1588464176",
0x630eea01,"MainField_Item_Plant_O_1589635784",
0x115d01d7,"MainField_Item_Plant_O_1590058797",
0x7d7a0e85,"MainField_Item_Plant_O_1591520176",
0x51f03c57,"MainField_Item_Plant_O_1591786323",
0x892f1f85,"MainField_Item_Plant_O_1591976167",
0x10ad17a2,"MainField_Item_Plant_O_1595681921",
0x061cefc9,"MainField_Item_Plant_O_1596729965",
0x70108c9e,"MainField_Item_Plant_O_1606899916",
0x44460402,"MainField_Item_Plant_O_1620342872",
0x471c125f,"MainField_Item_Plant_O_1626022088",
0x253cf08b,"MainField_Item_Plant_O_1628941574",
0xf3809da1,"MainField_Item_Plant_O_163277806",
0x4c809e31,"MainField_Item_Plant_O_164033123",
0x2550687c,"MainField_Item_Plant_O_1645240824",
0x1d03d0a5,"MainField_Item_Plant_O_1645395400",
0xbc3a8f5e,"MainField_Item_Plant_O_1649804166",
0x0bc7199e,"MainField_Item_Plant_O_1655387770",
0x53afe77f,"MainField_Item_Plant_O_1661898815",
0xb2078a7e,"MainField_Item_Plant_O_1667773958",
0x46770f3c,"MainField_Item_Plant_O_1670399763",
0x24594af8,"MainField_Item_Plant_O_1674621989",
0x22213c99,"MainField_Item_Plant_O_1675059889",
0x155ba315,"MainField_Item_Plant_O_1676810398",
0x578ae431,"MainField_Item_Plant_O_1684384171",
0x41918906,"MainField_Item_Plant_O_1688480209",
0xa6675ebe,"MainField_Item_Plant_O_169107834",
0x5c7432b7,"MainField_Item_Plant_O_1693410772",
0x46c57020,"MainField_Item_Plant_O_1693980751",
0x3a172687,"MainField_Item_Plant_O_1695572097",
0x51bfeee8,"MainField_Item_Plant_O_1710236690",
0x8e8944a0,"MainField_Item_Plant_O_1713661716",
0x41364ac8,"MainField_Item_Plant_O_1716177448",
0x7b94c825,"MainField_Item_Plant_O_17175408",
0xea83eadf,"MainField_Item_Plant_O_1718748231",
0xa1a5c031,"MainField_Item_Plant_O_1726163233",
0x0b790ece,"MainField_Item_Plant_O_1729821552",
0xdc88db2a,"MainField_Item_Plant_O_1732634281",
0xb240f07e,"MainField_Item_Plant_O_174035785",
0x85a4341d,"MainField_Item_Plant_O_1746551038",
0x83585341,"MainField_Item_Plant_O_1747228968",
0x49e923be,"MainField_Item_Plant_O_1749058020",
0xab6a1c8d,"MainField_Item_Plant_O_1751475671",
0x605a4a29,"MainField_Item_Plant_O_1753926104",
0x650a4f4d,"MainField_Item_Plant_O_1755107593",
0x0489dbef,"MainField_Item_Plant_O_1757221740",
0x062b8ee9,"MainField_Item_Plant_O_1760994559",
0xd3ca8fbf,"MainField_Item_Plant_O_1769790416",
0x45de95e1,"MainField_Item_Plant_O_1770783245",
0xc3bbbd06,"MainField_Item_Plant_O_1771147427",
0x70bd7e78,"MainField_Item_Plant_O_177432497",
0x3f94440e,"MainField_Item_Plant_O_1775623181",
0xad0100b1,"MainField_Item_Plant_O_1778233953",
0xacafa7bf,"MainField_Item_Plant_O_178508517",
0x0e769483,"MainField_Item_Plant_O_1786982040",
0xf39f29fa,"MainField_Item_Plant_O_1788454627",
0x29cfccc9,"MainField_Item_Plant_O_1790869493",
0x5cbf19fd,"MainField_Item_Plant_O_1791731019",
0xababe2d5,"MainField_Item_Plant_O_1799432992",
0xaee58f62,"MainField_Item_Plant_O_1799546788",
0xa045fd7f,"MainField_Item_Plant_O_180066235",
0x8024d3f3,"MainField_Item_Plant_O_1801748537",
0x2d463eec,"MainField_Item_Plant_O_1802532462",
0x0098c3af,"MainField_Item_Plant_O_1804121678",
0x3dac3b7a,"MainField_Item_Plant_O_1805173266",
0xc40dddf4,"MainField_Item_Plant_O_1808920545",
0x4582d7a8,"MainField_Item_Plant_O_1818110847",
0x7642013e,"MainField_Item_Plant_O_1824092890",
0x00174eea,"MainField_Item_Plant_O_1827097623",
0x7101a93b,"MainField_Item_Plant_O_1827624211",
0xaf100f07,"MainField_Item_Plant_O_1827668155",
0xaeb9d92a,"MainField_Item_Plant_O_1830937140",
0x8efb35bf,"MainField_Item_Plant_O_1831172780",
0x87154e84,"MainField_Item_Plant_O_1843753370",
0x8a153168,"MainField_Item_Plant_O_184526934",
0xa2ccf653,"MainField_Item_Plant_O_1845634673",
0x4f8251fd,"MainField_Item_Plant_O_1848591276",
0xfd6b3088,"MainField_Item_Plant_O_1852216467",
0x81c78f34,"MainField_Item_Plant_O_1855138048",
0xd6a13eae,"MainField_Item_Plant_O_1856740523",
0xd6f63b03,"MainField_Item_Plant_O_1860867087",
0xad7f989a,"MainField_Item_Plant_O_1860871420",
0x42bf8e1b,"MainField_Item_Plant_O_1862836686",
0x410b9290,"MainField_Item_Plant_O_1872302074",
0x0433b061,"MainField_Item_Plant_O_1874944114",
0x32ffa228,"MainField_Item_Plant_O_1877706454",
0xffda79b8,"MainField_Item_Plant_O_1881103702",
0xa2454ac9,"MainField_Item_Plant_O_1884360379",
0x431fbab4,"MainField_Item_Plant_O_188467427",
0xeaa3a012,"MainField_Item_Plant_O_1885221031",
0x33ede9ed,"MainField_Item_Plant_O_1885930840",
0x955d82d1,"MainField_Item_Plant_O_18895817",
0xf04f72b0,"MainField_Item_Plant_O_1894483614",
0x9e5ed6cd,"MainField_Item_Plant_O_1894897866",
0x86a22b84,"MainField_Item_Plant_O_1897289457",
0xc6babcda,"MainField_Item_Plant_O_1901032438",
0xc339a7d3,"MainField_Item_Plant_O_1903493994",
0xbbb2d696,"MainField_Item_Plant_O_190406303",
0x27504f52,"MainField_Item_Plant_O_1906985267",
0x51330fe2,"MainField_Item_Plant_O_1927742675",
0xdf9022cf,"MainField_Item_Plant_O_1929632703",
0x916e11cd,"MainField_Item_Plant_O_1930528883",
0x9b8f4cb0,"MainField_Item_Plant_O_1931438159",
0xe5f6233e,"MainField_Item_Plant_O_1931465078",
0xb367b4c6,"MainField_Item_Plant_O_1940475513",
0xc6f24603,"MainField_Item_Plant_O_1951339271",
0x818ededc,"MainField_Item_Plant_O_1954969974",
0xac8bc75b,"MainField_Item_Plant_O_1957103739",
0x6b595ffe,"MainField_Item_Plant_O_1960866778",
0x2fc2737f,"MainField_Item_Plant_O_1961218683",
0xf3f1335b,"MainField_Item_Plant_O_196432109",
0x2f4b0243,"MainField_Item_Plant_O_197046772",
0x726e948c,"MainField_Item_Plant_O_1974809033",
0x4a846eeb,"MainField_Item_Plant_O_1978338622",
0x45c56446,"MainField_Item_Plant_O_1979014736",
0x5dbc643b,"MainField_Item_Plant_O_1983132224",
0x91c371ec,"MainField_Item_Plant_O_1988270086",
0x335389e3,"MainField_Item_Plant_O_2000142878",
0x272b0c4a,"MainField_Item_Plant_O_200555992",
0xf677168c,"MainField_Item_Plant_O_2018046624",
0xa24ded64,"MainField_Item_Plant_O_2026521812",
0x1c550d1a,"MainField_Item_Plant_O_2028668649",
0x2595058a,"MainField_Item_Plant_O_2031050134",
0xd9f633f7,"MainField_Item_Plant_O_2036553464",
0x1cc4f5bd,"MainField_Item_Plant_O_2044268617",
0x24466770,"MainField_Item_Plant_O_204791984",
0x353a593b,"MainField_Item_Plant_O_2055435269",
0xdf5aa443,"MainField_Item_Plant_O_2057080042",
0x279053e5,"MainField_Item_Plant_O_2060311859",
0x1a57f119,"MainField_Item_Plant_O_2071812661",
0x9b37f1d0,"MainField_Item_Plant_O_2078254178",
0xba30511e,"MainField_Item_Plant_O_2078576699",
0x2d891ae1,"MainField_Item_Plant_O_2079017443",
0xab3165e1,"MainField_Item_Plant_O_2080512187",
0x857aad74,"MainField_Item_Plant_O_2085955124",
0x6905baed,"MainField_Item_Plant_O_2086214567",
0x0919ae24,"MainField_Item_Plant_O_2098143902",
0x1718e382,"MainField_Item_Plant_O_209817505",
0xe1ce7e38,"MainField_Item_Plant_O_2098895216",
0x1adca59b,"MainField_Item_Plant_O_2101111208",
0x1a9d8834,"MainField_Item_Plant_O_2102501856",
0x9875114b,"MainField_Item_Plant_O_2106218881",
0xf50bc5f2,"MainField_Item_Plant_O_2107530622",
0x6ae6adff,"MainField_Item_Plant_O_2108875480",
0xd9f129a2,"MainField_Item_Plant_O_2109591639",
0xbc1b1c0c,"MainField_Item_Plant_O_2109605996",
0xdc4f7707,"MainField_Item_Plant_O_2110030876",
0xe0f26040,"MainField_Item_Plant_O_2110084715",
0x6d364327,"MainField_Item_Plant_O_2111656788",
0x2af50cc2,"MainField_Item_Plant_O_2114094919",
0x4c27fa5a,"MainField_Item_Plant_O_2122455300",
0x87b82d79,"MainField_Item_Plant_O_2125612965",
0xa4474339,"MainField_Item_Plant_O_2126197061",
0xa701ca26,"MainField_Item_Plant_O_2127049187",
0x559e718b,"MainField_Item_Plant_O_2127090186",
0xd1b45210,"MainField_Item_Plant_O_2131759599",
0x8f5da4a5,"MainField_Item_Plant_O_2132403802",
0xe5837ac9,"MainField_Item_Plant_O_2136224493",
0xb66b2ea9,"MainField_Item_Plant_O_214119366",
0xa0eaaa13,"MainField_Item_Plant_O_2153326916",
0x30ab2e35,"MainField_Item_Plant_O_2153980095",
0x0a912c50,"MainField_Item_Plant_O_215520500",
0x94957bb2,"MainField_Item_Plant_O_2158406327",
0x097143ed,"MainField_Item_Plant_O_2159942931",
0xfd6684e3,"MainField_Item_Plant_O_2160890830",
0x8d04249c,"MainField_Item_Plant_O_2161976295",
0xb641096a,"MainField_Item_Plant_O_2164703458",
0xb1ebc2c5,"MainField_Item_Plant_O_2182318458",
0xbed77b2c,"MainField_Item_Plant_O_2197070330",
0x117755a5,"MainField_Item_Plant_O_2210068083",
0x815409d2,"MainField_Item_Plant_O_2210455506",
0x05be5ebb,"MainField_Item_Plant_O_2216918133",
0xf0c81bc7,"MainField_Item_Plant_O_2221171849",
0x38bac3d9,"MainField_Item_Plant_O_2228281393",
0x02f6e646,"MainField_Item_Plant_O_2229131165",
0x20ae25e4,"MainField_Item_Plant_O_2234871209",
0xdf60281e,"MainField_Item_Plant_O_2237304035",
0x464bb543,"MainField_Item_Plant_O_2238363486",
0xc4bce5dd,"MainField_Item_Plant_O_2239907791",
0x30bbcafb,"MainField_Item_Plant_O_2241155802",
0xdf8ee56e,"MainField_Item_Plant_O_2241913524",
0x7e21bc8e,"MainField_Item_Plant_O_2245383784",
0xf8f5e78a,"MainField_Item_Plant_O_2246431762",
0x68c42e72,"MainField_Item_Plant_O_2247690124",
0xafb8ee55,"MainField_Item_Plant_O_2249043107",
0x0912e230,"MainField_Item_Plant_O_2250891977",
0xb00f6b87,"MainField_Item_Plant_O_2253693088",
0x05cfa7dd,"MainField_Item_Plant_O_2254741630",
0x02d86df5,"MainField_Item_Plant_O_2255972902",
0xd8cb6e27,"MainField_Item_Plant_O_2265161101",
0x94fa5992,"MainField_Item_Plant_O_2266797450",
0xbb2123e8,"MainField_Item_Plant_O_2269028756",
0xfdbc49ed,"MainField_Item_Plant_O_2273999187",
0xc13c71d7,"MainField_Item_Plant_O_2277221725",
0x7d408cf4,"MainField_Item_Plant_O_2282467101",
0xe46c6905,"MainField_Item_Plant_O_2291688466",
0xbcab16b9,"MainField_Item_Plant_O_2295743363",
0x5a4b9fb3,"MainField_Item_Plant_O_2307027323",
0xa271fbb1,"MainField_Item_Plant_O_2308137064",
0xf1a04141,"MainField_Item_Plant_O_2317498054",
0xff6e2ee5,"MainField_Item_Plant_O_2326778719",
0xc82b22a8,"MainField_Item_Plant_O_232866069",
0x4178d1ac,"MainField_Item_Plant_O_2328713794",
0x9b0bb58b,"MainField_Item_Plant_O_2340401606",
0x90aad18b,"MainField_Item_Plant_O_234690242",
0x0216cb4b,"MainField_Item_Plant_O_2352705065",
0x0a199854,"MainField_Item_Plant_O_2353842745",
0xc2f51f06,"MainField_Item_Plant_O_235592460",
0x98b854c3,"MainField_Item_Plant_O_2356767211",
0x520b0099,"MainField_Item_Plant_O_2360839456",
0x0f5a9341,"MainField_Item_Plant_O_236175441",
0xf0829031,"MainField_Item_Plant_O_2364381841",
0x1e365ae4,"MainField_Item_Plant_O_2369892590",
0x9960a52d,"MainField_Item_Plant_O_2373817075",
0x68015dbc,"MainField_Item_Plant_O_2373861335",
0x5d26b334,"MainField_Item_Plant_O_2377096940",
0xda385987,"MainField_Item_Plant_O_2389707363",
0x871fb99a,"MainField_Item_Plant_O_2393252583",
0x36b51a4c,"MainField_Item_Plant_O_2404514156",
0xb7ee9349,"MainField_Item_Plant_O_2404637443",
0x5c74990f,"MainField_Item_Plant_O_2405765249",
0x181ab970,"MainField_Item_Plant_O_2407918962",
0x95156a40,"MainField_Item_Plant_O_2410626140",
0x50aa77c6,"MainField_Item_Plant_O_2412788087",
0x19a47b4c,"MainField_Item_Plant_O_2423481263",
0x9a936836,"MainField_Item_Plant_O_2424317066",
0x47c0c354,"MainField_Item_Plant_O_2429052321",
0xa5e4e477,"MainField_Item_Plant_O_2434389476",
0x3bd61add,"MainField_Item_Plant_O_2440593678",
0xb3f22758,"MainField_Item_Plant_O_244305367",
0xb6578c7a,"MainField_Item_Plant_O_245012878",
0x24f3649b,"MainField_Item_Plant_O_245149655",
0xe0c860d8,"MainField_Item_Plant_O_2456868894",
0xf70a9964,"MainField_Item_Plant_O_2457189570",
0x6c7fb018,"MainField_Item_Plant_O_2467153280",
0xc4535da7,"MainField_Item_Plant_O_2468417889",
0x8d7392fb,"MainField_Item_Plant_O_2470161567",
0x1bf5ecff,"MainField_Item_Plant_O_2470283663",
0x39f5fc17,"MainField_Item_Plant_O_2482794583",
0x03ecc1db,"MainField_Item_Plant_O_2484448371",
0xd0957769,"MainField_Item_Plant_O_2490496988",
0xffb8647f,"MainField_Item_Plant_O_2494533391",
0xacc29962,"MainField_Item_Plant_O_2495022613",
0x5a538ecf,"MainField_Item_Plant_O_2498044566",
0x24529885,"MainField_Item_Plant_O_2499848736",
0x64be9027,"MainField_Item_Plant_O_2504646379",
0xe7b75301,"MainField_Item_Plant_O_2505637714",
0x263ff93a,"MainField_Item_Plant_O_2509705428",
0x7f715db2,"MainField_Item_Plant_O_2513552718",
0x104a4f84,"MainField_Item_Plant_O_2514298355",
0xec14fdca,"MainField_Item_Plant_O_2515601166",
0x2574bb54,"MainField_Item_Plant_O_2516031173",
0x51c4683c,"MainField_Item_Plant_O_2518883688",
0x69668f60,"MainField_Item_Plant_O_2524070084",
0xe7dd30c5,"MainField_Item_Plant_O_2525812730",
0x1701a5d0,"MainField_Item_Plant_O_2528580923",
0xab91fbb3,"MainField_Item_Plant_O_2535024582",
0x3e6854c0,"MainField_Item_Plant_O_2544372701",
0xa0c8ca91,"MainField_Item_Plant_O_2551476802",
0x4904f814,"MainField_Item_Plant_O_2551498464",
0x5bec2252,"MainField_Item_Plant_O_2552739504",
0x6421b2b1,"MainField_Item_Plant_O_2555335534",
0x91345cd2,"MainField_Item_Plant_O_2556772049",
0x3b1fe349,"MainField_Item_Plant_O_2558880328",
0x71a1635f,"MainField_Item_Plant_O_255910988",
0x59eb8327,"MainField_Item_Plant_O_25612128",
0x0173d192,"MainField_Item_Plant_O_2562105255",
0x8ce214a0,"MainField_Item_Plant_O_2563328802",
0x178c0c41,"MainField_Item_Plant_O_2564820914",
0xd19b1c43,"MainField_Item_Plant_O_2566539662",
0x57397875,"MainField_Item_Plant_O_25719630",
0x76bf0b80,"MainField_Item_Plant_O_2574257424",
0x4cda59d9,"MainField_Item_Plant_O_2575890564",
0x864d7a73,"MainField_Item_Plant_O_2580867251",
0xe286d1ee,"MainField_Item_Plant_O_2586853867",
0xb33b987c,"MainField_Item_Plant_O_2588914848",
0x994f2c51,"MainField_Item_Plant_O_2597551632",
0x01b4135b,"MainField_Item_Plant_O_2597905411",
0x68b0612f,"MainField_Item_Plant_O_2598928250",
0x2293bbf9,"MainField_Item_Plant_O_259922966",
0x0896e747,"MainField_Item_Plant_O_2601555856",
0x1e2626d7,"MainField_Item_Plant_O_260902202",
0x2a15c6b2,"MainField_Item_Plant_O_2617203806",
0xe194f926,"MainField_Item_Plant_O_2617498738",
0xd47a5749,"MainField_Item_Plant_O_2619008922",
0x33fe6ae8,"MainField_Item_Plant_O_2619161451",
0x7b5ddaac,"MainField_Item_Plant_O_2624003525",
0x03463f75,"MainField_Item_Plant_O_2624031077",
0x31b40c34,"MainField_Item_Plant_O_2625226627",
0x65e3914f,"MainField_Item_Plant_O_2625384194",
0x3e065543,"MainField_Item_Plant_O_2628273171",
0x4c052f21,"MainField_Item_Plant_O_2631887181",
0x9a9f7354,"MainField_Item_Plant_O_2634306622",
0x91b0424a,"MainField_Item_Plant_O_2634394201",
0x64fc419c,"MainField_Item_Plant_O_2634448961",
0x3ecec610,"MainField_Item_Plant_O_2639087041",
0x4dc4d398,"MainField_Item_Plant_O_2644198823",
0xdca23d6c,"MainField_Item_Plant_O_264646164",
0x7a863ebe,"MainField_Item_Plant_O_2649474868",
0x0caec909,"MainField_Item_Plant_O_2649659557",
0xbc47b25d,"MainField_Item_Plant_O_2651106165",
0x4acc1fe2,"MainField_Item_Plant_O_2651773946",
0xb9cd90ec,"MainField_Item_Plant_O_2653216800",
0x427c74ee,"MainField_Item_Plant_O_2664969192",
0x1a9191c4,"MainField_Item_Plant_O_2668844553",
0x6e553baa,"MainField_Item_Plant_O_2669024760",
0x11ded8ca,"MainField_Item_Plant_O_2669395948",
0x3b149ac3,"MainField_Item_Plant_O_2671954715",
0xf13da658,"MainField_Item_Plant_O_2672702721",
0x137bbcb6,"MainField_Item_Plant_O_2677079342",
0x6cdb5872,"MainField_Item_Plant_O_2679498675",
0x9f02577f,"MainField_Item_Plant_O_268343133",
0x3d6a2701,"MainField_Item_Plant_O_268473697",
0xb4b0bdd1,"MainField_Item_Plant_O_2687630099",
0x84afc1f5,"MainField_Item_Plant_O_2706328039",
0x519141f9,"MainField_Item_Plant_O_2710857876",
0x80bdde0e,"MainField_Item_Plant_O_2711153001",
0xbd5dba5b,"MainField_Item_Plant_O_2711554674",
0xafef1b23,"MainField_Item_Plant_O_272102306",
0x721f66e9,"MainField_Item_Plant_O_2721051840",
0xd468e92f,"MainField_Item_Plant_O_2722715856",
0x536588ad,"MainField_Item_Plant_O_2730030741",
0x3232f812,"MainField_Item_Plant_O_273387607",
0x37f69b96,"MainField_Item_Plant_O_2736442556",
0x508beaed,"MainField_Item_Plant_O_2738483226",
0xbebcbf59,"MainField_Item_Plant_O_2738823708",
0x3e87ffb1,"MainField_Item_Plant_O_2739409629",
0xc018baea,"MainField_Item_Plant_O_2742253067",
0x44b6313f,"MainField_Item_Plant_O_2752105306",
0xaa3e2d9e,"MainField_Item_Plant_O_275698387",
0xebcf5f91,"MainField_Item_Plant_O_2757651429",
0xefce44bc,"MainField_Item_Plant_O_2761842333",
0x2cc5545b,"MainField_Item_Plant_O_2762799703",
0xf040bb24,"MainField_Item_Plant_O_276482186",
0xfef060ae,"MainField_Item_Plant_O_2771729132",
0x5fc5ac87,"MainField_Item_Plant_O_2772449131",
0x565af296,"MainField_Item_Plant_O_277554896",
0xbf540cf2,"MainField_Item_Plant_O_2777722479",
0xe18095ad,"MainField_Item_Plant_O_2781479270",
0x3c935e3d,"MainField_Item_Plant_O_2788517172",
0x4a16d673,"MainField_Item_Plant_O_2789737321",
0x37103c3d,"MainField_Item_Plant_O_2793744316",
0x5149f311,"MainField_Item_Plant_O_2796953839",
0x8c489f22,"MainField_Item_Plant_O_2797800079",
0xcc24d7e2,"MainField_Item_Plant_O_2800298210",
0xcd374ba5,"MainField_Item_Plant_O_2802773533",
0xf80ab6a5,"MainField_Item_Plant_O_2802945326",
0x297b3ad3,"MainField_Item_Plant_O_2803071225",
0xb32afaea,"MainField_Item_Plant_O_2804674916",
0xcec7aad7,"MainField_Item_Plant_O_2805192776",
0xe95d0953,"MainField_Item_Plant_O_2805317163",
0xbe325459,"MainField_Item_Plant_O_2809897204",
0x0376e11a,"MainField_Item_Plant_O_2810835659",
0x341ebcd0,"MainField_Item_Plant_O_2811982333",
0xfcb20a75,"MainField_Item_Plant_O_2814927160",
0x3a61f455,"MainField_Item_Plant_O_2815373077",
0xbe86c589,"MainField_Item_Plant_O_2819799994",
0xf63bd057,"MainField_Item_Plant_O_282016718",
0x2b57332e,"MainField_Item_Plant_O_2822445616",
0x9a9d9a2f,"MainField_Item_Plant_O_2827223148",
0x3a5d9563,"MainField_Item_Plant_O_2831730945",
0x416bd47b,"MainField_Item_Plant_O_2836550419",
0x812f1ab1,"MainField_Item_Plant_O_2837219452",
0xa80d1c9e,"MainField_Item_Plant_O_2840489323",
0x4e0b6ddf,"MainField_Item_Plant_O_2842564964",
0x48559514,"MainField_Item_Plant_O_2843653295",
0xabe73a50,"MainField_Item_Plant_O_2845730605",
0x45607a71,"MainField_Item_Plant_O_2849969694",
0xbac44503,"MainField_Item_Plant_O_285149336",
0xfb37c484,"MainField_Item_Plant_O_2853797031",
0x0c60391d,"MainField_Item_Plant_O_2856663075",
0x6c70b575,"MainField_Item_Plant_O_2857621091",
0xd1648a00,"MainField_Item_Plant_O_2858748790",
0xe2294e8b,"MainField_Item_Plant_O_285962722",
0xd544da07,"MainField_Item_Plant_O_2863636288",
0x414dfe97,"MainField_Item_Plant_O_2869642484",
0x01b9343c,"MainField_Item_Plant_O_2874685454",
0x21b17745,"MainField_Item_Plant_O_2880992869",
0xf7d7e8cf,"MainField_Item_Plant_O_2883262961",
0x5e0c23f5,"MainField_Item_Plant_O_2889921599",
0x2b6f9f97,"MainField_Item_Plant_O_2894142004",
0x9e99b280,"MainField_Item_Plant_O_2895697232",
0x87f10ae4,"MainField_Item_Plant_O_2899179135",
0xbd69f684,"MainField_Item_Plant_O_2900967836",
0x2aeaff50,"MainField_Item_Plant_O_2904467445",
0x64a8259e,"MainField_Item_Plant_O_2907429650",
0x45a6b51f,"MainField_Item_Plant_O_2910557463",
0x30ae1b72,"MainField_Item_Plant_O_2914906170",
0x9baa7321,"MainField_Item_Plant_O_2915195737",
0xfb5e3d2a,"MainField_Item_Plant_O_2917103919",
0xad066194,"MainField_Item_Plant_O_2917775354",
0xd5769def,"MainField_Item_Plant_O_2920494763",
0x29dac9e2,"MainField_Item_Plant_O_2925155193",
0xb194c0fa,"MainField_Item_Plant_O_292822762",
0xb9155344,"MainField_Item_Plant_O_2928942616",
0xb6ffc113,"MainField_Item_Plant_O_2936405389",
0xebc18418,"MainField_Item_Plant_O_2941369247",
0xcfebf58e,"MainField_Item_Plant_O_2946850229",
0x000c2475,"MainField_Item_Plant_O_2949142884",
0xbdfa722e,"MainField_Item_Plant_O_2954122331",
0x4029bb43,"MainField_Item_Plant_O_2954397995",
0x615360f0,"MainField_Item_Plant_O_2957185619",
0xff68e675,"MainField_Item_Plant_O_2960491383",
0x5b01e455,"MainField_Item_Plant_O_296076508",
0xcd376982,"MainField_Item_Plant_O_2966375861",
0xab848acd,"MainField_Item_Plant_O_2968443519",
0x53818dd0,"MainField_Item_Plant_O_2973503415",
0xd291051a,"MainField_Item_Plant_O_2979278928",
0xaf46da4f,"MainField_Item_Plant_O_2984458135",
0xd4d9fe1a,"MainField_Item_Plant_O_2990675298",
0xb8f38824,"MainField_Item_Plant_O_2991309166",
0x5f0e0130,"MainField_Item_Plant_O_2993234981",
0xaf2c1abc,"MainField_Item_Plant_O_2993844383",
0x31284e7c,"MainField_Item_Plant_O_2994699492",
0x3a7ce493,"MainField_Item_Plant_O_2995007723",
0x4014f887,"MainField_Item_Plant_O_2996810991",
0xf05d3186,"MainField_Item_Plant_O_3004838211",
0x1665d061,"MainField_Item_Plant_O_3006480973",
0x50823e12,"MainField_Item_Plant_O_3010028382",
0x664c9827,"MainField_Item_Plant_O_3022209217",
0xeacfd90a,"MainField_Item_Plant_O_3027528317",
0x41267db9,"MainField_Item_Plant_O_3035113409",
0x06d56f3f,"MainField_Item_Plant_O_3035373543",
0x33834f87,"MainField_Item_Plant_O_3036517330",
0xde833798,"MainField_Item_Plant_O_3039130729",
0xfefcb521,"MainField_Item_Plant_O_3048939139",
0x37c4275a,"MainField_Item_Plant_O_3050187410",
0x7c99dfdf,"MainField_Item_Plant_O_3054103410",
0x412b345d,"MainField_Item_Plant_O_3071884581",
0x09d00064,"MainField_Item_Plant_O_3076955066",
0xfb1512e6,"MainField_Item_Plant_O_3078319855",
0xab63ceab,"MainField_Item_Plant_O_307857109",
0x80019dd1,"MainField_Item_Plant_O_3079165711",
0x629f221e,"MainField_Item_Plant_O_3080231851",
0x61f1204a,"MainField_Item_Plant_O_3080452674",
0xf24b6839,"MainField_Item_Plant_O_3081855052",
0x99f12942,"MainField_Item_Plant_O_3082865246",
0x5b01a9ed,"MainField_Item_Plant_O_3087239796",
0x388b3fc3,"MainField_Item_Plant_O_3091310392",
0xea2871d2,"MainField_Item_Plant_O_3093606320",
0xf0f85ea0,"MainField_Item_Plant_O_3095785576",
0x164a3dc9,"MainField_Item_Plant_O_3098816584",
0x6733774f,"MainField_Item_Plant_O_3102181791",
0x1dcaf3ed,"MainField_Item_Plant_O_3102893411",
0x812eb0f2,"MainField_Item_Plant_O_3108805650",
0xbadd6fc7,"MainField_Item_Plant_O_3109729830",
0xa795d0b2,"MainField_Item_Plant_O_3112690179",
0xf2855cdb,"MainField_Item_Plant_O_3120575705",
0x9787c2cc,"MainField_Item_Plant_O_3121360048",
0x49173f79,"MainField_Item_Plant_O_3122119259",
0x9f1c8974,"MainField_Item_Plant_O_3122474547",
0x86257663,"MainField_Item_Plant_O_312275937",
0x85d9edda,"MainField_Item_Plant_O_3123018014",
0x9858ec1b,"MainField_Item_Plant_O_3123493509",
0xe5fb368e,"MainField_Item_Plant_O_3123814226",
0xb1aa4392,"MainField_Item_Plant_O_3124452002",
0x4a20c2b8,"MainField_Item_Plant_O_3136598630",
0x31415dcb,"MainField_Item_Plant_O_3138193229",
0xf11c4fe7,"MainField_Item_Plant_O_3146837724",
0x2f608373,"MainField_Item_Plant_O_3149754201",
0x068d6f9b,"MainField_Item_Plant_O_3160273771",
0xa3e9b193,"MainField_Item_Plant_O_3160981522",
0x1ff7b865,"MainField_Item_Plant_O_3168631390",
0x4a82a258,"MainField_Item_Plant_O_3176177173",
0x8fbcec85,"MainField_Item_Plant_O_3182403283",
0xbac4efa2,"MainField_Item_Plant_O_3195575020",
0x77c28678,"MainField_Item_Plant_O_3201876213",
0xc6344fc1,"MainField_Item_Plant_O_3202311154",
0x570763ec,"MainField_Item_Plant_O_3207060379",
0x03d80ea0,"MainField_Item_Plant_O_320835158",
0xc2431cfc,"MainField_Item_Plant_O_3211007149",
0x13125acb,"MainField_Item_Plant_O_3216868120",
0x443775cb,"MainField_Item_Plant_O_3217223621",
0xf75a64cb,"MainField_Item_Plant_O_3222247169",
0xdce097aa,"MainField_Item_Plant_O_3225514400",
0xab597b7a,"MainField_Item_Plant_O_3234232235",
0x8f8b4815,"MainField_Item_Plant_O_3237069510",
0x6cda2759,"MainField_Item_Plant_O_3240783816",
0xecb5fbd2,"MainField_Item_Plant_O_3241408162",
0x4a70b650,"MainField_Item_Plant_O_3244111768",
0xdf40b05f,"MainField_Item_Plant_O_3245610832",
0x3527af68,"MainField_Item_Plant_O_3246877475",
0xc4c8d690,"MainField_Item_Plant_O_3247712238",
0x49e1856f,"MainField_Item_Plant_O_3253940280",
0xa1310c9a,"MainField_Item_Plant_O_3255735916",
0x692f0930,"MainField_Item_Plant_O_3256169687",
0xdd6ad994,"MainField_Item_Plant_O_3259094427",
0x32b21b2f,"MainField_Item_Plant_O_3263677541",
0x277b88db,"MainField_Item_Plant_O_326488332",
0x899ecdd6,"MainField_Item_Plant_O_3266560126",
0xbb3b8345,"MainField_Item_Plant_O_3270988770",
0x9b8d4bc3,"MainField_Item_Plant_O_3272115002",
0x20eb759f,"MainField_Item_Plant_O_3272704359",
0xf1a4cbba,"MainField_Item_Plant_O_3273489024",
0x9bbc6435,"MainField_Item_Plant_O_3277386406",
0x1acca792,"MainField_Item_Plant_O_3278816647",
0x94d8bb54,"MainField_Item_Plant_O_3283740733",
0xa0299637,"MainField_Item_Plant_O_3287756111",
0x407843f8,"MainField_Item_Plant_O_3287804314",
0x20af19ce,"MainField_Item_Plant_O_3289516259",
0x2a30d938,"MainField_Item_Plant_O_329284576",
0x429c958b,"MainField_Item_Plant_O_3299346337",
0x8d18ac45,"MainField_Item_Plant_O_3302411924",
0x532ad1a3,"MainField_Item_Plant_O_3303311635",
0x041ad31a,"MainField_Item_Plant_O_3308555472",
0xe6f60e78,"MainField_Item_Plant_O_330897681",
0x6b871780,"MainField_Item_Plant_O_3311373010",
0xbc71810f,"MainField_Item_Plant_O_3311478204",
0x5b19bff6,"MainField_Item_Plant_O_3319589482",
0x40f5b178,"MainField_Item_Plant_O_3328831314",
0x8219a633,"MainField_Item_Plant_O_3330037106",
0xcabdf853,"MainField_Item_Plant_O_3331020091",
0x4fa1afae,"MainField_Item_Plant_O_333675155",
0xf3f38822,"MainField_Item_Plant_O_3338269063",
0x67df4717,"MainField_Item_Plant_O_333969827",
0x31e0ec83,"MainField_Item_Plant_O_3353996009",
0x91bc2f3a,"MainField_Item_Plant_O_3370746511",
0x7144eafc,"MainField_Item_Plant_O_3378184559",
0x196e70d6,"MainField_Item_Plant_O_3378474481",
0x3a20ec31,"MainField_Item_Plant_O_3380587130",
0xd8db0188,"MainField_Item_Plant_O_3380588807",
0x6b47c638,"MainField_Item_Plant_O_33810975",
0xdd18f6a7,"MainField_Item_Plant_O_3385247993",
0xfe93b77c,"MainField_Item_Plant_O_3386015209",
0xaca76eec,"MainField_Item_Plant_O_3386127255",
0xaf8240d6,"MainField_Item_Plant_O_3388596804",
0xecec451b,"MainField_Item_Plant_O_3412952099",
0xdb7042a0,"MainField_Item_Plant_O_3416127559",
0x47c06f23,"MainField_Item_Plant_O_3416753954",
0xc7f6c17b,"MainField_Item_Plant_O_3420402406",
0x03e8b590,"MainField_Item_Plant_O_3420411951",
0x11b746b7,"MainField_Item_Plant_O_342195435",
0xc21f29e1,"MainField_Item_Plant_O_3422566985",
0xf0216453,"MainField_Item_Plant_O_342476970",
0x67348ffd,"MainField_Item_Plant_O_3426694093",
0x11b9b285,"MainField_Item_Plant_O_3428933470",
0x8ad67885,"MainField_Item_Plant_O_3429281036",
0xcf052fe1,"MainField_Item_Plant_O_3433731463",
0xa1cf7c64,"MainField_Item_Plant_O_3433910964",
0x06846200,"MainField_Item_Plant_O_3438539142",
0x13da30e2,"MainField_Item_Plant_O_3440876633",
0xbe26cebd,"MainField_Item_Plant_O_3441150288",
0x7df2bbc9,"MainField_Item_Plant_O_3443907230",
0x5fa6fc8e,"MainField_Item_Plant_O_3444246106",
0xf217cdb2,"MainField_Item_Plant_O_3444806518",
0xb7ef5e39,"MainField_Item_Plant_O_3446038746",
0x8b2d57ca,"MainField_Item_Plant_O_3450898762",
0x0d475c41,"MainField_Item_Plant_O_3452738810",
0x7df044b7,"MainField_Item_Plant_O_3457314403",
0x4cf19a40,"MainField_Item_Plant_O_345818086",
0x1341484a,"MainField_Item_Plant_O_3458464199",
0x8f5fffdb,"MainField_Item_Plant_O_3458519989",
0xf39359c8,"MainField_Item_Plant_O_3459408130",
0xb0a93ee2,"MainField_Item_Plant_O_3462152545",
0xe01ac852,"MainField_Item_Plant_O_3462375106",
0xb2053947,"MainField_Item_Plant_O_3464051399",
0xb0b20ae2,"MainField_Item_Plant_O_3475261463",
0x43d0c20b,"MainField_Item_Plant_O_3476420907",
0x8923b084,"MainField_Item_Plant_O_347758658",
0xdd58274a,"MainField_Item_Plant_O_3487850689",
0x3681eaa8,"MainField_Item_Plant_O_3502655032",
0x49e763ca,"MainField_Item_Plant_O_3503023939",
0x4b99d419,"MainField_Item_Plant_O_3504436555",
0x4b1c68f3,"MainField_Item_Plant_O_3513015116",
0xdbfbe8da,"MainField_Item_Plant_O_3515589318",
0x78de8ba7,"MainField_Item_Plant_O_3521652948",
0xd88b07e8,"MainField_Item_Plant_O_3527340344",
0xfd86c5ef,"MainField_Item_Plant_O_3531618633",
0xe2a8cb46,"MainField_Item_Plant_O_3536474053",
0x341bcbc7,"MainField_Item_Plant_O_3539267771",
0xf020d96b,"MainField_Item_Plant_O_3540595139",
0xe6f988c7,"MainField_Item_Plant_O_3547110149",
0x6e2bf676,"MainField_Item_Plant_O_3547216179",
0x76d39c13,"MainField_Item_Plant_O_3550090805",
0xf9578ce3,"MainField_Item_Plant_O_3552817561",
0x2b05c9e0,"MainField_Item_Plant_O_3556763936",
0x405f0781,"MainField_Item_Plant_O_3557785607",
0x90b40a37,"MainField_Item_Plant_O_3564310539",
0x63c001e3,"MainField_Item_Plant_O_3565964207",
0xc20c7b55,"MainField_Item_Plant_O_3566374352",
0xfa80179b,"MainField_Item_Plant_O_3566454184",
0x59656ec8,"MainField_Item_Plant_O_357236783",
0x594991b9,"MainField_Item_Plant_O_357416743",
0x5b684f77,"MainField_Item_Plant_O_3577651754",
0x4bef3c84,"MainField_Item_Plant_O_358345698",
0x87a8e26f,"MainField_Item_Plant_O_3584446076",
0x77bbe9c1,"MainField_Item_Plant_O_3584690088",
0x891c8f91,"MainField_Item_Plant_O_3589232682",
0x66f38549,"MainField_Item_Plant_O_3593796320",
0xb3de9867,"MainField_Item_Plant_O_3594304149",
0xc1dbdaa1,"MainField_Item_Plant_O_3607243798",
0x801961aa,"MainField_Item_Plant_O_3610824869",
0x49558806,"MainField_Item_Plant_O_3616787682",
0x800b47e6,"MainField_Item_Plant_O_3621159648",
0x89aafd5d,"MainField_Item_Plant_O_3626289022",
0xa6e6cd60,"MainField_Item_Plant_O_3627348057",
0xe7df9838,"MainField_Item_Plant_O_3631081928",
0x7fe0c610,"MainField_Item_Plant_O_3631376679",
0x0040f561,"MainField_Item_Plant_O_3634686363",
0x504c7aae,"MainField_Item_Plant_O_3636313244",
0x1f3cfd06,"MainField_Item_Plant_O_3639083831",
0x67e316c1,"MainField_Item_Plant_O_3640889871",
0x0fb47a43,"MainField_Item_Plant_O_3641848317",
0xaafff002,"MainField_Item_Plant_O_3650098331",
0x76bbf032,"MainField_Item_Plant_O_3652619821",
0xd80c4768,"MainField_Item_Plant_O_3655015266",
0x29e31bc3,"MainField_Item_Plant_O_3656673808",
0x465629e9,"MainField_Item_Plant_O_3657965969",
0xf360edef,"MainField_Item_Plant_O_3664932350",
0x8cd48b94,"MainField_Item_Plant_O_3666331138",
0xbc2d7acb,"MainField_Item_Plant_O_366997759",
0x2779dc83,"MainField_Item_Plant_O_3681285311",
0x7b6ee4bf,"MainField_Item_Plant_O_3682513379",
0xa9b8e5c9,"MainField_Item_Plant_O_3683563712",
0x5bf5440c,"MainField_Item_Plant_O_3687998490",
0xd93eb1b8,"MainField_Item_Plant_O_36954119",
0x573c0d46,"MainField_Item_Plant_O_370055451",
0x526995a8,"MainField_Item_Plant_O_3705321182",
0xa54f62c0,"MainField_Item_Plant_O_3717649352",
0xea84f2e7,"MainField_Item_Plant_O_3718819798",
0x587f83f8,"MainField_Item_Plant_O_3720655560",
0x049dd8b4,"MainField_Item_Plant_O_3723117509",
0xd96d2f54,"MainField_Item_Plant_O_3724891835",
0x606eb40c,"MainField_Item_Plant_O_372715021",
0xe88a6e50,"MainField_Item_Plant_O_3727345229",
0xafb3dc99,"MainField_Item_Plant_O_3729847107",
0x5009831a,"MainField_Item_Plant_O_3730939172",
0xd0ab3af4,"MainField_Item_Plant_O_3741944263",
0x0c360e43,"MainField_Item_Plant_O_3743033742",
0x5421ace3,"MainField_Item_Plant_O_3748240628",
0x6142800a,"MainField_Item_Plant_O_3750725845",
0x7af19e76,"MainField_Item_Plant_O_3756622520",
0x9a57ba63,"MainField_Item_Plant_O_3763789983",
0xf3291d73,"MainField_Item_Plant_O_3768407219",
0xd768cb35,"MainField_Item_Plant_O_3770861140",
0x9413d38c,"MainField_Item_Plant_O_377097444",
0x83d38002,"MainField_Item_Plant_O_377240035",
0xe88bbcb4,"MainField_Item_Plant_O_3782229713",
0x01c42ec8,"MainField_Item_Plant_O_3784533325",
0x8af3b405,"MainField_Item_Plant_O_3785373170",
0xd52f2cf2,"MainField_Item_Plant_O_3791467117",
0xb8a121a5,"MainField_Item_Plant_O_3798103169",
0x0088dd30,"MainField_Item_Plant_O_3800498506",
0xbf83d113,"MainField_Item_Plant_O_3801797865",
0xf55c4f10,"MainField_Item_Plant_O_3801915688",
0xe22106ea,"MainField_Item_Plant_O_3805738456",
0x75e18b3b,"MainField_Item_Plant_O_3816022862",
0x005f5425,"MainField_Item_Plant_O_3817908564",
0x4addf34f,"MainField_Item_Plant_O_3823570240",
0xbec367be,"MainField_Item_Plant_O_3825048438",
0xaa8c22c2,"MainField_Item_Plant_O_3827967391",
0x683fac7a,"MainField_Item_Plant_O_3835128150",
0x2db06540,"MainField_Item_Plant_O_3836972378",
0x0924d25f,"MainField_Item_Plant_O_3839309664",
0x21915d51,"MainField_Item_Plant_O_3850582510",
0xb2c55e12,"MainField_Item_Plant_O_3853456100",
0xa542f15a,"MainField_Item_Plant_O_3853633057",
0xfa268003,"MainField_Item_Plant_O_3855506569",
0x7b37b66e,"MainField_Item_Plant_O_3858584787",
0xd289440c,"MainField_Item_Plant_O_3863552023",
0x9eb8fe24,"MainField_Item_Plant_O_3867012489",
0xa51ee20e,"MainField_Item_Plant_O_3867393617",
0xe435539a,"MainField_Item_Plant_O_38678767",
0xe4de96cf,"MainField_Item_Plant_O_3868570097",
0x7100a201,"MainField_Item_Plant_O_3868738572",
0xf7c3c13f,"MainField_Item_Plant_O_3875441288",
0x675b9dc2,"MainField_Item_Plant_O_387561162",
0x715369eb,"MainField_Item_Plant_O_3877692850",
0x174ce85a,"MainField_Item_Plant_O_3879970476",
0xbc1a079b,"MainField_Item_Plant_O_3882170771",
0x01c808eb,"MainField_Item_Plant_O_3882253648",
0x1fbf8400,"MainField_Item_Plant_O_3884240292",
0xdc037019,"MainField_Item_Plant_O_3886806010",
0x741cefda,"MainField_Item_Plant_O_3889660194",
0x8a41ce00,"MainField_Item_Plant_O_3898779080",
0x00664846,"MainField_Item_Plant_O_3901498568",
0x89cd5181,"MainField_Item_Plant_O_3909874340",
0x1e95a301,"MainField_Item_Plant_O_3916757264",
0xa98888a2,"MainField_Item_Plant_O_3917765723",
0xc417ef85,"MainField_Item_Plant_O_3923473338",
0x49792106,"MainField_Item_Plant_O_3931881723",
0x52757dc3,"MainField_Item_Plant_O_3938709365",
0xa7c6df59,"MainField_Item_Plant_O_3949071641",
0xd22d2d47,"MainField_Item_Plant_O_395231717",
0xf17367a9,"MainField_Item_Plant_O_3955456224",
0x60a640ba,"MainField_Item_Plant_O_3955638713",
0x6f454544,"MainField_Item_Plant_O_3955883814",
0x260e9d05,"MainField_Item_Plant_O_3957920131",
0x7a7bfcd7,"MainField_Item_Plant_O_3961920957",
0x8d1a0088,"MainField_Item_Plant_O_3963984174",
0x901429ce,"MainField_Item_Plant_O_396545494",
0x8389085e,"MainField_Item_Plant_O_3980419565",
0xd6728e02,"MainField_Item_Plant_O_3984220448",
0x6b355bd9,"MainField_Item_Plant_O_3984563769",
0xd8fe33bf,"MainField_Item_Plant_O_3985422581",
0xd8f1474f,"MainField_Item_Plant_O_3988907569",
0x3756c32e,"MainField_Item_Plant_O_3993964552",
0x82d46932,"MainField_Item_Plant_O_399400696",
0x0cd074b2,"MainField_Item_Plant_O_3997175120",
0xb08b1188,"MainField_Item_Plant_O_4000256947",
0xc8c6b357,"MainField_Item_Plant_O_4003868937",
0xddf67c29,"MainField_Item_Plant_O_4007105100",
0xef1650a0,"MainField_Item_Plant_O_4011041530",
0x9c74309e,"MainField_Item_Plant_O_4013041180",
0x987c50fb,"MainField_Item_Plant_O_4016510485",
0xab4a3ba7,"MainField_Item_Plant_O_4017766016",
0x19d7c18f,"MainField_Item_Plant_O_4018200978",
0xae2e638f,"MainField_Item_Plant_O_4018449595",
0xdefa9aa3,"MainField_Item_Plant_O_4021265130",
0xb79b6bf3,"MainField_Item_Plant_O_4026283149",
0xad7a1f62,"MainField_Item_Plant_O_4033947062",
0x22fce442,"MainField_Item_Plant_O_4034051767",
0x7483798a,"MainField_Item_Plant_O_4035434014",
0x7c1bd7b6,"MainField_Item_Plant_O_403647475",
0x4ed16a96,"MainField_Item_Plant_O_403673432",
0x9bb223da,"MainField_Item_Plant_O_4041595079",
0xc1436caa,"MainField_Item_Plant_O_4049062418",
0x41fdae2d,"MainField_Item_Plant_O_4057357047",
0x545b1aff,"MainField_Item_Plant_O_4057921994",
0xda5bae07,"MainField_Item_Plant_O_4068225763",
0xf3635369,"MainField_Item_Plant_O_407082815",
0x13e7d344,"MainField_Item_Plant_O_407223475",
0x0413e597,"MainField_Item_Plant_O_4073646298",
0x06f3e984,"MainField_Item_Plant_O_407649060",
0x3253334a,"MainField_Item_Plant_O_4080777871",
0xf4a3e9e3,"MainField_Item_Plant_O_4081630947",
0x500a9b35,"MainField_Item_Plant_O_4082929494",
0xb687e46c,"MainField_Item_Plant_O_4089223579",
0xcbe3224e,"MainField_Item_Plant_O_4091501905",
0x3c347168,"MainField_Item_Plant_O_4094570998",
0x7a7c3fc4,"MainField_Item_Plant_O_4095373847",
0xf7db8394,"MainField_Item_Plant_O_4096058044",
0x98d1b0ba,"MainField_Item_Plant_O_4112856579",
0x6783013c,"MainField_Item_Plant_O_4113991047",
0x5df0dbcc,"MainField_Item_Plant_O_4115044029",
0xf93e03a1,"MainField_Item_Plant_O_4116816587",
0x273f787b,"MainField_Item_Plant_O_4117126497",
0x2ec3697a,"MainField_Item_Plant_O_4118700175",
0xc4b6d23e,"MainField_Item_Plant_O_4118934461",
0xd833fd4b,"MainField_Item_Plant_O_4120518984",
0x5c3481db,"MainField_Item_Plant_O_4120664371",
0xf5c8f1e7,"MainField_Item_Plant_O_4125526110",
0x1cdcf12c,"MainField_Item_Plant_O_4127322228",
0x7dcf38ba,"MainField_Item_Plant_O_4129426307",
0x61ea8015,"MainField_Item_Plant_O_4129513296",
0xdf94ec11,"MainField_Item_Plant_O_4130610470",
0x6fecb519,"MainField_Item_Plant_O_4132243986",
0x13d10a3c,"MainField_Item_Plant_O_4132769641",
0x2471e0ef,"MainField_Item_Plant_O_4133972060",
0xf1aacc51,"MainField_Item_Plant_O_4138319354",
0x8c131f60,"MainField_Item_Plant_O_4142874266",
0x5045a4b3,"MainField_Item_Plant_O_4146619390",
0x409c5c5d,"MainField_Item_Plant_O_4149851075",
0xe86828ab,"MainField_Item_Plant_O_4152690249",
0xa4bbf719,"MainField_Item_Plant_O_4156135636",
0x6a52a54e,"MainField_Item_Plant_O_4160491005",
0xe3ad48be,"MainField_Item_Plant_O_4161397644",
0xfe6fb205,"MainField_Item_Plant_O_4168982121",
0xef3b9666,"MainField_Item_Plant_O_4171114484",
0xc74a8e14,"MainField_Item_Plant_O_4171575674",
0x66f3e250,"MainField_Item_Plant_O_4181975521",
0x1dcb322a,"MainField_Item_Plant_O_4183815702",
0xd1844402,"MainField_Item_Plant_O_4186598151",
0x02213b88,"MainField_Item_Plant_O_4187648404",
0xcd00baa2,"MainField_Item_Plant_O_419492440",
0xe366a868,"MainField_Item_Plant_O_4195303463",
0x8cf72dcf,"MainField_Item_Plant_O_4195970091",
0xcdf5720f,"MainField_Item_Plant_O_4202393904",
0xe5a73861,"MainField_Item_Plant_O_420883272",
0xf59c57a6,"MainField_Item_Plant_O_4213865652",
0x4b065f3f,"MainField_Item_Plant_O_421798866",
0x16bd5043,"MainField_Item_Plant_O_4226275124",
0x3f2d5cf2,"MainField_Item_Plant_O_4227483763",
0x1810566d,"MainField_Item_Plant_O_423110464",
0x49ddc323,"MainField_Item_Plant_O_4232406661",
0x4b3391b7,"MainField_Item_Plant_O_4234240025",
0xc08d6cad,"MainField_Item_Plant_O_4241363020",
0x2a9b1465,"MainField_Item_Plant_O_4241866317",
0x0302fe33,"MainField_Item_Plant_O_4244443388",
0x02e97abd,"MainField_Item_Plant_O_4247997075",
0x259b014e,"MainField_Item_Plant_O_4250470742",
0xfbaad3ef,"MainField_Item_Plant_O_425107758",
0xc63dbfb8,"MainField_Item_Plant_O_4253317039",
0x2af07d02,"MainField_Item_Plant_O_4254653487",
0x54b7a67d,"MainField_Item_Plant_O_4256872325",
0x08212c27,"MainField_Item_Plant_O_4261952353",
0xb69324b6,"MainField_Item_Plant_O_4267768569",
0x72407420,"MainField_Item_Plant_O_4277834283",
0x773a465f,"MainField_Item_Plant_O_4279847572",
0xc2ffb192,"MainField_Item_Plant_O_4290231391",
0x0ddd0ca5,"MainField_Item_Plant_O_4291915264",
0x4bc38373,"MainField_Item_Plant_O_435818074",
0xbfc43ef6,"MainField_Item_Plant_O_437230024",
0x8c2b0ef6,"MainField_Item_Plant_O_438044417",
0xd37bc4fe,"MainField_Item_Plant_O_439629694",
0xd1f9ae0b,"MainField_Item_Plant_O_439801260",
0xc4a57ce9,"MainField_Item_Plant_O_441972949",
0x01b6df52,"MainField_Item_Plant_O_442776787",
0x1375ec43,"MainField_Item_Plant_O_445867412",
0xeeead21c,"MainField_Item_Plant_O_446300929",
0x8697bdca,"MainField_Item_Plant_O_44871265",
0x21e3b9fd,"MainField_Item_Plant_O_450367224",
0x4327b95e,"MainField_Item_Plant_O_451817534",
0x3ce36737,"MainField_Item_Plant_O_46354069",
0x5f787078,"MainField_Item_Plant_O_468824901",
0xbdb71c4f,"MainField_Item_Plant_O_471015594",
0xa3bac362,"MainField_Item_Plant_O_471760868",
0x094f5a35,"MainField_Item_Plant_O_47733024",
0xf99fa28e,"MainField_Item_Plant_O_477462521",
0x267d8d11,"MainField_Item_Plant_O_479738606",
0x9cdcb56d,"MainField_Item_Plant_O_480216077",
0x5a0ee6d9,"MainField_Item_Plant_O_48113671",
0x5f1ba275,"MainField_Item_Plant_O_481782340",
0x4e7a86a3,"MainField_Item_Plant_O_486951144",
0xa279a1d8,"MainField_Item_Plant_O_489192125",
0x650fae9a,"MainField_Item_Plant_O_49232357",
0x8998823d,"MainField_Item_Plant_O_494958846",
0x4c43412e,"MainField_Item_Plant_O_497024891",
0x6b2158da,"MainField_Item_Plant_O_498700756",
0xca971514,"MainField_Item_Plant_O_50936712",
0x385e6ade,"MainField_Item_Plant_O_511572636",
0xcc7ba509,"MainField_Item_Plant_O_512000029",
0xc534a849,"MainField_Item_Plant_O_518185938",
0xeba519f1,"MainField_Item_Plant_O_521166033",
0x424008b4,"MainField_Item_Plant_O_521361663",
0xd8eb269c,"MainField_Item_Plant_O_522443799",
0xf4d7c550,"MainField_Item_Plant_O_522444941",
0xd9c14d09,"MainField_Item_Plant_O_527398263",
0xd76cbc5c,"MainField_Item_Plant_O_528991106",
0x11223371,"MainField_Item_Plant_O_533797343",
0x526cb0ba,"MainField_Item_Plant_O_535039511",
0x6321cbef,"MainField_Item_Plant_O_535639913",
0xc3e5843e,"MainField_Item_Plant_O_537366901",
0x98fca8b6,"MainField_Item_Plant_O_539719704",
0xba741159,"MainField_Item_Plant_O_542686862",
0x1b241d2a,"MainField_Item_Plant_O_543679711",
0x27d87f22,"MainField_Item_Plant_O_545001552",
0x006886e7,"MainField_Item_Plant_O_545445364",
0xb5289147,"MainField_Item_Plant_O_548981358",
0xbd5cfb45,"MainField_Item_Plant_O_550092813",
0x526b27ea,"MainField_Item_Plant_O_55408765",
0x257a3c99,"MainField_Item_Plant_O_555726095",
0x10e17f2d,"MainField_Item_Plant_O_557020271",
0x3d1a541f,"MainField_Item_Plant_O_558208545",
0xfd895161,"MainField_Item_Plant_O_560021830",
0x5ee72788,"MainField_Item_Plant_O_56488986",
0xb9e94f68,"MainField_Item_Plant_O_568775351",
0x22651087,"MainField_Item_Plant_O_574976981",
0x75fd2e99,"MainField_Item_Plant_O_579397969",
0x4550d726,"MainField_Item_Plant_O_583766263",
0x51a45229,"MainField_Item_Plant_O_585384024",
0x7fd49702,"MainField_Item_Plant_O_590707028",
0x370c0290,"MainField_Item_Plant_O_592923269",
0xec7dd780,"MainField_Item_Plant_O_598058341",
0x84928ca7,"MainField_Item_Plant_O_598558734",
0x1d094970,"MainField_Item_Plant_O_599974064",
0xe495baf4,"MainField_Item_Plant_O_609952090",
0xd7b0416e,"MainField_Item_Plant_O_617262760",
0x5e0e363f,"MainField_Item_Plant_O_619333479",
0xcb64a93d,"MainField_Item_Plant_O_621782774",
0x894b9ebf,"MainField_Item_Plant_O_629750648",
0xb4fbbcbe,"MainField_Item_Plant_O_634319606",
0x32083417,"MainField_Item_Plant_O_654591642",
0x5905c38f,"MainField_Item_Plant_O_661046155",
0xe773bacd,"MainField_Item_Plant_O_661769046",
0x39eb2f91,"MainField_Item_Plant_O_665639376",
0xbef5e399,"MainField_Item_Plant_O_668259639",
0x93a4f2af,"MainField_Item_Plant_O_669686353",
0xfbfca797,"MainField_Item_Plant_O_671816874",
0xa60e59fc,"MainField_Item_Plant_O_675972930",
0x36b5c2b0,"MainField_Item_Plant_O_677256266",
0xbf875bac,"MainField_Item_Plant_O_67740740",
0x2c5cff90,"MainField_Item_Plant_O_677704452",
0xba1b73bb,"MainField_Item_Plant_O_680539906",
0x32ac7473,"MainField_Item_Plant_O_688073803",
0xee0856cd,"MainField_Item_Plant_O_690924636",
0x968ef88e,"MainField_Item_Plant_O_691395101",
0x406c25b2,"MainField_Item_Plant_O_695543635",
0xd54df760,"MainField_Item_Plant_O_703040453",
0x0ef9c4b7,"MainField_Item_Plant_O_704255362",
0xe4c992e9,"MainField_Item_Plant_O_707923703",
0x4b668e50,"MainField_Item_Plant_O_708077823",
0x99a37b4c,"MainField_Item_Plant_O_718271794",
0x748881e1,"MainField_Item_Plant_O_71922845",
0x47605695,"MainField_Item_Plant_O_719535667",
0x28782fad,"MainField_Item_Plant_O_719768394",
0x3a1fe1d1,"MainField_Item_Plant_O_721739151",
0xb90e9cc2,"MainField_Item_Plant_O_722100664",
0xda5e2867,"MainField_Item_Plant_O_726768735",
0x90f8eea3,"MainField_Item_Plant_O_728339872",
0x06a9b6a6,"MainField_Item_Plant_O_730762421",
0x4cb81277,"MainField_Item_Plant_O_732377255",
0xb0fe7d2e,"MainField_Item_Plant_O_732382578",
0x5409cc66,"MainField_Item_Plant_O_733194183",
0x6cb819d2,"MainField_Item_Plant_O_734643826",
0x7d582d19,"MainField_Item_Plant_O_735582003",
0x4d5eac45,"MainField_Item_Plant_O_743480072",
0xede5400c,"MainField_Item_Plant_O_746161600",
0x7e4f08a0,"MainField_Item_Plant_O_7473773",
0xd62ae3dd,"MainField_Item_Plant_O_756747054",
0x042f46a2,"MainField_Item_Plant_O_763313223",
0x7db57285,"MainField_Item_Plant_O_763611838",
0x69d58904,"MainField_Item_Plant_O_764932044",
0x4927633c,"MainField_Item_Plant_O_767407166",
0x58563da3,"MainField_Item_Plant_O_767714500",
0xedf91530,"MainField_Item_Plant_O_767833695",
0x6f23800b,"MainField_Item_Plant_O_772084066",
0x55a9cd24,"MainField_Item_Plant_O_774881796",
0xd93c4001,"MainField_Item_Plant_O_777414936",
0x361f7cb2,"MainField_Item_Plant_O_780104383",
0x0401958c,"MainField_Item_Plant_O_78452116",
0x9c7005ad,"MainField_Item_Plant_O_78462986",
0x2a425d56,"MainField_Item_Plant_O_790017240",
0x271eb4ef,"MainField_Item_Plant_O_790079304",
0x11fdbbfb,"MainField_Item_Plant_O_790717870",
0x5d7c96ef,"MainField_Item_Plant_O_791632530",
0xf39b1b4f,"MainField_Item_Plant_O_794631136",
0x812f2627,"MainField_Item_Plant_O_796915834",
0x4264ae49,"MainField_Item_Plant_O_79746830",
0xd62788d2,"MainField_Item_Plant_O_800724041",
0xeb645965,"MainField_Item_Plant_O_805380834",
0xffe7cd4f,"MainField_Item_Plant_O_807821171",
0x740ac324,"MainField_Item_Plant_O_81081266",
0x58eeeb0c,"MainField_Item_Plant_O_811149668",
0x24f97f23,"MainField_Item_Plant_O_811606252",
0xf620ef38,"MainField_Item_Plant_O_814526302",
0x548dce93,"MainField_Item_Plant_O_817100285",
0x993c887c,"MainField_Item_Plant_O_820716216",
0x329a63b4,"MainField_Item_Plant_O_821968903",
0x37887bf0,"MainField_Item_Plant_O_824266029",
0x549415da,"MainField_Item_Plant_O_825169587",
0xfcc7e9a9,"MainField_Item_Plant_O_827584185",
0xae91239c,"MainField_Item_Plant_O_828146402",
0xc7c0be4a,"MainField_Item_Plant_O_830479992",
0x498cf948,"MainField_Item_Plant_O_831067735",
0x6e310819,"MainField_Item_Plant_O_841164527",
0x3160f3aa,"MainField_Item_Plant_O_846221458",
0x04d1b2e8,"MainField_Item_Plant_O_85365642",
0x911d8c0b,"MainField_Item_Plant_O_855496145",
0xf36f47bf,"MainField_Item_Plant_O_861239167",
0x649abb91,"MainField_Item_Plant_O_866569978",
0x2356a229,"MainField_Item_Plant_O_885157518",
0xb212d800,"MainField_Item_Plant_O_899305376",
0x0d1c8ab3,"MainField_Item_Plant_O_905363409",
0xee41323e,"MainField_Item_Plant_O_906792055",
0xe2737849,"MainField_Item_Plant_O_910247624",
0xa24cee21,"MainField_Item_Plant_O_913166204",
0x7f7c7b51,"MainField_Item_Plant_O_917058897",
0x1e15e0c2,"MainField_Item_Plant_O_922744903",
0x86c83607,"MainField_Item_Plant_O_924450168",
0x461d8667,"MainField_Item_Plant_O_939804059",
0x9eda893a,"MainField_Item_Plant_O_942397491",
0x1c2ab5be,"MainField_Item_Plant_O_947303843",
0x5d217af5,"MainField_Item_Plant_O_954237674",
0x4bfbab07,"MainField_Item_Plant_O_954838927",
0xca0acffe,"MainField_Item_Plant_O_955424842",
0x711b3907,"MainField_Item_Plant_O_955809336",
0x26394b76,"MainField_Item_Plant_O_963364211",
0xa5f7de6b,"MainField_Item_Plant_O_966167057",
0x6088db0d,"MainField_Item_Plant_O_969263598",
0xfe0069a0,"MainField_Item_Plant_O_969379198",
0xce06be9b,"MainField_Item_Plant_O_974160593",
0xdbbd5c3d,"MainField_Item_Plant_O_975986546",
0xa3ea0da9,"MainField_Item_Plant_O_976199697",
0xcd951947,"MainField_Item_Plant_O_976656672",
0x09edd12b,"MainField_Item_Plant_O_979761513",
0x9e447477,"MainField_Item_Plant_O_987648387",
0xa12e7104,"MainField_Item_Plant_O_991875697",
0xba84df6a,"MainField_Item_Plant_O_993390686",
0x6f9836e7,"MainField_Item_Plant_O_994801206",
0xca074596,"MainField_Item_Plant_O_998081387",
0x08e8af2c,"MainField_Item_Plant_O_999108862",
0x324461b8,"MainField_Item_Plant_Q_128807493",
0x90f7a32f,"MainField_Item_Plant_Q_1537254454",
0xa8cada80,"MainField_Item_Plant_Q_1554368201",
0x50fc7cd6,"MainField_Item_Plant_Q_1693528204",
0x216c227d,"MainField_Item_Plant_Q_1803940503",
0xaca9e0b2,"MainField_Item_Plant_Q_2196836121",
0x0656d522,"MainField_Item_Plant_Q_2430138756",
0x96ad152a,"MainField_Item_Plant_Q_2466299735",
0x3c651a64,"MainField_Item_Plant_Q_2526114765",
0x485df0b5,"MainField_Item_Plant_Q_266400020",
0x8324e838,"MainField_Item_Plant_Q_269954847",
0x3c738e8e,"MainField_Item_Plant_Q_2767887810",
0x320e5f1a,"MainField_Item_Plant_Q_2834187103",
0xb175a204,"MainField_Item_Plant_Q_2894488946",
0x45713f94,"MainField_Item_Plant_Q_3054669075",
0x30245de5,"MainField_Item_Plant_Q_3068150930",
0x29c345be,"MainField_Item_Plant_Q_3168203505",
0x5e34654f,"MainField_Item_Plant_Q_3250189839",
0x33f5d5ac,"MainField_Item_Plant_Q_3493404633",
0xbdb39f05,"MainField_Item_Plant_Q_3505763980",
0x862749b9,"MainField_Item_Plant_Q_3532653824",
0x0b5b5802,"MainField_Item_Plant_Q_3639374775",
0x9b5a7cec,"MainField_Item_Plant_Q_3917356337",
0xeefcaa5b,"MainField_Item_Plant_Q_3928804437",
0x25e21e75,"MainField_Item_Plant_Q_4054911675",
0x0bc6c69e,"MainField_Item_Plant_Q_4076101518",
0x2894b12e,"MainField_Item_Plant_Q_4190897668",
0x1aede82a,"MainField_Item_Plant_Q_557304585",
0x0b62788e,"MainField_Item_Plant_Q_948919049",
0xecec01b0,"MainField_Item_Plant_Q_988233822",
0x64f32def,"MainField_Item_RoastFish_01_1051838192",
0x92acd280,"MainField_Item_RoastFish_01_1122912528",
0x11d16a68,"MainField_Item_RoastFish_01_1187372764",
0xa269a8a7,"MainField_Item_RoastFish_01_1193226931",
0xe3ea5dd3,"MainField_Item_RoastFish_01_126801777",
0x4437d213,"MainField_Item_RoastFish_01_1371458207",
0x62fa0ec4,"MainField_Item_RoastFish_01_1410632949",
0x1d666495,"MainField_Item_RoastFish_01_1581080067",
0x1a83df43,"MainField_Item_RoastFish_01_1843817717",
0xce8ce346,"MainField_Item_RoastFish_01_1964907975",
0xde91ae3d,"MainField_Item_RoastFish_01_2022959891",
0xa8ac2c9e,"MainField_Item_RoastFish_01_2090123661",
0x5aef14f1,"MainField_Item_RoastFish_01_2399564746",
0x641b4c11,"MainField_Item_RoastFish_01_2454129901",
0x8efac452,"MainField_Item_RoastFish_01_2578136544",
0xe5a7de7b,"MainField_Item_RoastFish_01_2592079598",
0x7995c3a5,"MainField_Item_RoastFish_01_2798569272",
0x844e625b,"MainField_Item_RoastFish_01_2897181652",
0x0029f980,"MainField_Item_RoastFish_01_2956902675",
0x6ac1a258,"MainField_Item_RoastFish_01_3280047061",
0xba72eb41,"MainField_Item_RoastFish_01_3280928100",
0xbc96b448,"MainField_Item_RoastFish_01_3310251123",
0xb3eea23d,"MainField_Item_RoastFish_01_3551676342",
0x5f30c968,"MainField_Item_RoastFish_01_3650976608",
0xbbf87a48,"MainField_Item_RoastFish_01_3930474902",
0x04ab7f71,"MainField_Item_RoastFish_01_3968260225",
0x324b204c,"MainField_Item_RoastFish_01_4032155538",
0x2027e856,"MainField_Item_RoastFish_01_4208287357",
0xafefe8f0,"MainField_Item_RoastFish_01_560915035",
0xf845ab4c,"MainField_Item_RoastFish_01_658737144",
0x28793a40,"MainField_Item_RoastFish_01_677211251",
0x70ef228c,"MainField_Item_RoastFish_01_68905805",
0x7985b85d,"MainField_Item_RoastFish_01_722055319",
0xde38072a,"MainField_Item_RoastFish_01_875562841",
0xeec44689,"MainField_Item_RoastFish_01_98049476",
0xcc871f60,"MainField_Item_RoastFish_09_2807839760",
0xefcb0976,"MainField_Item_RoastFish_09_3495627910",
0x0ae5db94,"MainField_Item_RoastFish_09_960014771",
0xdf3fbe41,"MainField_Item_Roast_01_3607071685",
0xdb611383,"MainField_Item_Roast_01_3765228500",
0x20660bf5,"MainField_Item_Roast_01_4294905484",
0x8b742010,"MainField_Item_Roast_03_1467127567",
0xb412f99f,"MainField_Item_Roast_03_713217406",
0x043db580,"MainField_Item_Roast_05_1406541343",
0xb954e07c,"MainField_Item_Roast_05_3132472106",
0x1a46482a,"MainField_Item_Roast_05_617688713",
0x3e055b85,"MainField_Item_Roast_06_3465535150",
0xae357bb0,"MainField_Item_Roast_40_3742218106",
0xffe5aac9,"MainField_Item_Roast_45_808608095",
0x396d50d8,"MainField_KibakoDesert_Contain_01_1411429550",
0x175bf38d,"MainField_KibakoDesert_Contain_01_1510395536",
0x3824fd8e,"MainField_KibakoDesert_Contain_01_1802914511",
0xf68297b9,"MainField_KibakoDesert_Contain_01_184713871",
0x65e3388f,"MainField_KibakoDesert_Contain_01_1919456900",
0xc71873ee,"MainField_KibakoDesert_Contain_01_1953114313",
0x875cb114,"MainField_KibakoDesert_Contain_01_2250748567",
0x7e5924fe,"MainField_KibakoDesert_Contain_01_2452667924",
0xa9252857,"MainField_KibakoDesert_Contain_01_2592737972",
0x3132f9a1,"MainField_KibakoDesert_Contain_01_2816232017",
0x0dcc067e,"MainField_KibakoDesert_Contain_01_2992003171",
0x6124bc65,"MainField_KibakoDesert_Contain_01_3129587637",
0x5e7d5a27,"MainField_KibakoDesert_Contain_01_3250999717",
0x6c90f535,"MainField_KibakoDesert_Contain_01_3297450486",
0x91d06427,"MainField_KibakoDesert_Contain_01_3612634129",
0x8b245b02,"MainField_KibakoDesert_Contain_01_3654876673",
0xfece6cdd,"MainField_KibakoDesert_Contain_01_3681256037",
0x1af643e7,"MainField_KibakoDesert_Contain_01_3858320684",
0x73a3175b,"MainField_KibakoDesert_Contain_01_394006478",
0x36ab86cd,"MainField_KibakoDesert_Contain_01_4124349752",
0x3b07509e,"MainField_KibakoDesert_Contain_01_462276861",
0x3c1c7341,"MainField_KibakoDesert_Contain_01_474360764",
0x048c7480,"MainField_KibakoDesert_Contain_01_581410231",
0x0265deed,"MainField_KibakoDesert_Contain_01_675433661",
0x8e424c1a,"MainField_KibakoDesert_Contain_01_677549230",
0x7b251727,"MainField_KibakoDesert_Contain_01_693325234",
0x055b9c2a,"MainField_KibakoDesert_Contain_01_872853590",
0x00ab5a27,"MainField_KibakoDesert_Contain_01_895869769",
0xfc11c9d7,"MainField_KibakoJungle_Contain_01_1116727761",
0x0e6084ce,"MainField_KibakoJungle_Contain_01_1220620539",
0x0a947f34,"MainField_KibakoJungle_Contain_01_1760933970",
0xfc7e0a7a,"MainField_KibakoJungle_Contain_01_179841540",
0xa5ebe053,"MainField_KibakoJungle_Contain_01_2132166262",
0xb9f73f1a,"MainField_KibakoJungle_Contain_01_2434623083",
0x321c0f50,"MainField_KibakoJungle_Contain_01_3032979768",
0x1b49e692,"MainField_KibakoJungle_Contain_01_3643450814",
0xc0fbfe25,"MainField_KibakoJungle_Contain_01_3754902555",
0xf67732da,"MainField_KibakoJungle_Contain_01_3866804145",
0x63027376,"MainField_KibakoJungle_Contain_01_39427486",
0x19de6d50,"MainField_KibakoJungle_Contain_01_669877370",
0x5d972895,"MainField_KibakoJungle_Contain_01_718323503",
0x19c18be5,"MainField_KibakoJungle_Contain_01_795006295",
0x8d534133,"MainField_KibakoSeaside_Contain_01_102817948",
0x03a92640,"MainField_KibakoSeaside_Contain_01_1309910929",
0x0dde9aa2,"MainField_KibakoSeaside_Contain_01_1947846369",
0x63be1da2,"MainField_KibakoSeaside_Contain_01_2409461841",
0x9e91296f,"MainField_KibakoSeaside_Contain_01_2432859003",
0xea1184d0,"MainField_KibakoSeaside_Contain_01_2463573123",
0x8a25ab54,"MainField_KibakoSeaside_Contain_01_2857291983",
0x14e7b3e3,"MainField_KibakoSeaside_Contain_01_3046578465",
0x946879ee,"MainField_KibakoSeaside_Contain_01_3135123655",
0x4b67c488,"MainField_KibakoSeaside_Contain_01_3298647704",
0xfac87d8f,"MainField_KibakoSeaside_Contain_01_3711766862",
0x730ce3c1,"MainField_KibakoSeaside_Contain_01_3807730272",
0x6c70c153,"MainField_KibakoSeaside_Contain_01_3939964251",
0x76084749,"MainField_KibakoSeaside_Contain_01_4179061990",
0x6a85862f,"MainField_KibakoSeaside_Contain_01_516855466",
0x0f7ccce2,"MainField_KibakoSeaside_Contain_01_659711214",
0x10b1b77f,"MainField_KibakoSnowMountain_Contain_01_1053520315",
0xd2d655fc,"MainField_KibakoSnowMountain_Contain_01_1291609880",
0x51554778,"MainField_KibakoSnowMountain_Contain_01_2011494902",
0xb7fd7612,"MainField_KibakoSnowMountain_Contain_01_2156149555",
0xe02ecc6e,"MainField_KibakoSnowMountain_Contain_01_2177310997",
0x2c23814c,"MainField_KibakoSnowMountain_Contain_01_250682107",
0x429ddc07,"MainField_KibakoSnowMountain_Contain_01_2562488847",
0x7814c9b7,"MainField_KibakoSnowMountain_Contain_01_2874397106",
0xe80bb623,"MainField_KibakoSnowMountain_Contain_01_2927340601",
0x4cb99677,"MainField_KibakoSnowMountain_Contain_01_364131206",
0xde211b98,"MainField_KibakoSnowMountain_Contain_01_3725362038",
0x3668c067,"MainField_KibakoSnowMountain_Contain_01_373919763",
0x3482054c,"MainField_KibakoSnowMountain_Contain_01_3832261336",
0x8109d601,"MainField_KibakoSnowMountain_Contain_01_4075340619",
0xf024884f,"MainField_KibakoSnowMountain_Contain_01_4108061530",
0x195c5e80,"MainField_KibakoSnowMountain_Contain_01_4184604181",
0x399879fd,"MainField_KibakoSnowMountain_Contain_01_4218202863",
0x5e60e824,"MainField_KibakoSnowMountain_Contain_01_678232134",
0xda66244e,"MainField_KibakoSnowMountain_Contain_01_686808727",
0x951c0423,"MainField_KibakoSnowMountain_Contain_01_746646242",
0xe9ee8438,"MainField_KibakoSnowMountain_Contain_01_972911453",
0x0c5a92a4,"MainField_KibakoSnowMountain_Contain_01_984376751",
0xc0e88b5a,"MainField_KibakoVolcano_Contain_01_1002703571",
0x6260fb1b,"MainField_KibakoVolcano_Contain_01_1147451185",
0x86192286,"MainField_KibakoVolcano_Contain_01_1811640718",
0xf24d9c63,"MainField_KibakoVolcano_Contain_01_1877211230",
0x5052e85d,"MainField_KibakoVolcano_Contain_01_2123394366",
0xb2171e59,"MainField_KibakoVolcano_Contain_01_2662044815",
0x91de4278,"MainField_KibakoVolcano_Contain_01_2743013553",
0xdb4db062,"MainField_KibakoVolcano_Contain_01_2989279192",
0x37ff5b20,"MainField_KibakoVolcano_Contain_01_3041594672",
0x7e7c6cf7,"MainField_KibakoVolcano_Contain_01_3345651287",
0x5e107649,"MainField_KibakoVolcano_Contain_01_3714390953",
0x4e8f99f9,"MainField_KibakoVolcano_Contain_01_384332111",
0x3610be04,"MainField_KibakoVolcano_Contain_01_4038986592",
0xedd04c08,"MainField_KibakoVolcano_Contain_01_4045443332",
0x0a7cf620,"MainField_KibakoVolcano_Contain_01_608905800",
0xf1148ce4,"MainField_KibakoZora_Contain_01_1018707094",
0x0d0fff11,"MainField_KibakoZora_Contain_01_1101753765",
0x77f63c3c,"MainField_KibakoZora_Contain_01_1164952629",
0x40dc12d1,"MainField_KibakoZora_Contain_01_1210920595",
0x1de2957f,"MainField_KibakoZora_Contain_01_152692347",
0xf2775ce9,"MainField_KibakoZora_Contain_01_1529149458",
0x9893b37f,"MainField_KibakoZora_Contain_01_1660969172",
0x55fd81a7,"MainField_KibakoZora_Contain_01_1742263399",
0x18afce46,"MainField_KibakoZora_Contain_01_177224138",
0xcb3c0795,"MainField_KibakoZora_Contain_01_1776693160",
0x5e65cbe9,"MainField_KibakoZora_Contain_01_1805215627",
0x5c2e7b96,"MainField_KibakoZora_Contain_01_1820448060",
0x980f869f,"MainField_KibakoZora_Contain_01_1920236504",
0xb1f8ca64,"MainField_KibakoZora_Contain_01_2142979406",
0x2e29423e,"MainField_KibakoZora_Contain_01_2162577391",
0xa2f9f3f7,"MainField_KibakoZora_Contain_01_2436199845",
0x1bac4a7c,"MainField_KibakoZora_Contain_01_248992983",
0x21cb799f,"MainField_KibakoZora_Contain_01_2534786527",
0xd005d2d3,"MainField_KibakoZora_Contain_01_2611696217",
0xc645c5f1,"MainField_KibakoZora_Contain_01_2683581292",
0x46d400ad,"MainField_KibakoZora_Contain_01_2725057926",
0x4bd76a41,"MainField_KibakoZora_Contain_01_2773233462",
0xc4611b26,"MainField_KibakoZora_Contain_01_27733635",
0x77cc9e02,"MainField_KibakoZora_Contain_01_2885664230",
0xd1be663c,"MainField_KibakoZora_Contain_01_2933645948",
0x629ac2da,"MainField_KibakoZora_Contain_01_2989964149",
0xf3020e3b,"MainField_KibakoZora_Contain_01_336023618",
0x9c07b21f,"MainField_KibakoZora_Contain_01_3405176456",
0x6243028c,"MainField_KibakoZora_Contain_01_3651387342",
0xed650673,"MainField_KibakoZora_Contain_01_3661125930",
0x707abc4d,"MainField_KibakoZora_Contain_01_3694262022",
0xdc5dad60,"MainField_KibakoZora_Contain_01_4146370899",
0xe915a5f3,"MainField_KibakoZora_Contain_01_4208407825",
0x839c07de,"MainField_KibakoZora_Contain_01_4288154905",
0xc114f4bc,"MainField_KibakoZora_Contain_01_485220101",
0x6449b945,"MainField_KibakoZora_Contain_01_502063354",
0xe07ae738,"MainField_KibakoZora_Contain_01_530675030",
0x002f4cd5,"MainField_KibakoZora_Contain_01_639087407",
0x71ffea83,"MainField_KibakoZora_Contain_01_841646437",
0xe9de0d17,"MainField_KibakoZora_Contain_01_856863764",
0x9d456156,"MainField_KibakoZora_Contain_01_896493946",
0x2c4e78fe,"MainField_Kibako_Contain_01_1001329756",
0x834e6944,"MainField_Kibako_Contain_01_1004461870",
0x3bbc8851,"MainField_Kibako_Contain_01_1048541555",
0x8053ecdb,"MainField_Kibako_Contain_01_105403273",
0x094f8981,"MainField_Kibako_Contain_01_1099853630",
0x30afa2f7,"MainField_Kibako_Contain_01_1102656914",
0x82e0613e,"MainField_Kibako_Contain_01_1111387978",
0xc0785148,"MainField_Kibako_Contain_01_1114317931",
0x97f64ef3,"MainField_Kibako_Contain_01_1137361653",
0x020a118e,"MainField_Kibako_Contain_01_1150147848",
0xff9c85ef,"MainField_Kibako_Contain_01_1169251941",
0xcd710992,"MainField_Kibako_Contain_01_1172594900",
0x670e3d65,"MainField_Kibako_Contain_01_118913410",
0x6b714f35,"MainField_Kibako_Contain_01_1200608680",
0xa37789c3,"MainField_Kibako_Contain_01_1233803464",
0x6092b616,"MainField_Kibako_Contain_01_1244848750",
0xc43c6d00,"MainField_Kibako_Contain_01_124701987",
0xc6bab4c1,"MainField_Kibako_Contain_01_1249850725",
0x98ba3e91,"MainField_Kibako_Contain_01_1285953164",
0xe7e05137,"MainField_Kibako_Contain_01_1313253961",
0x6fb0cdcc,"MainField_Kibako_Contain_01_1332695171",
0x953f372f,"MainField_Kibako_Contain_01_134197396",
0x7aea4bea,"MainField_Kibako_Contain_01_1374941397",
0x633144a4,"MainField_Kibako_Contain_01_1396777642",
0xb573cf4b,"MainField_Kibako_Contain_01_1397575010",
0x1a7deef3,"MainField_Kibako_Contain_01_1407947968",
0xd946549f,"MainField_Kibako_Contain_01_1412611741",
0xca5b8eac,"MainField_Kibako_Contain_01_142099182",
0x14e0962b,"MainField_Kibako_Contain_01_1425750910",
0x0d7f825e,"MainField_Kibako_Contain_01_1453816974",
0xfd4c3fc0,"MainField_Kibako_Contain_01_1465591274",
0xcad5c34e,"MainField_Kibako_Contain_01_1479222355",
0x0dcc9016,"MainField_Kibako_Contain_01_1505035343",
0xab43dd43,"MainField_Kibako_Contain_01_151332195",
0x0c3fe7fa,"MainField_Kibako_Contain_01_1518049427",
0x293146fd,"MainField_Kibako_Contain_01_1519129715",
0xfc5adcfc,"MainField_Kibako_Contain_01_1521316464",
0x1acbb2b6,"MainField_Kibako_Contain_01_1523345043",
0xcf041d32,"MainField_Kibako_Contain_01_155548675",
0x77455b09,"MainField_Kibako_Contain_01_1574346108",
0xe295dfb4,"MainField_Kibako_Contain_01_1577848465",
0x345b7015,"MainField_Kibako_Contain_01_1581088848",
0x4bd85c2b,"MainField_Kibako_Contain_01_1589756134",
0x2f7e23fa,"MainField_Kibako_Contain_01_1593373416",
0xa4c3b5a5,"MainField_Kibako_Contain_01_1597726706",
0x93454cef,"MainField_Kibako_Contain_01_161184655",
0xfea16833,"MainField_Kibako_Contain_01_162783410",
0x747a81e8,"MainField_Kibako_Contain_01_1638409232",
0x4405cdf4,"MainField_Kibako_Contain_01_1644400221",
0x3a6da51d,"MainField_Kibako_Contain_01_1675465223",
0xaa2f6afd,"MainField_Kibako_Contain_01_1680836014",
0x8f9acd5a,"MainField_Kibako_Contain_01_1704398267",
0x9622fa0a,"MainField_Kibako_Contain_01_172568503",
0x0a56b783,"MainField_Kibako_Contain_01_1733518453",
0x274c70ce,"MainField_Kibako_Contain_01_1739625168",
0x745812ce,"MainField_Kibako_Contain_01_1742671678",
0x99853f8a,"MainField_Kibako_Contain_01_1748102866",
0x97a21a84,"MainField_Kibako_Contain_01_1759877658",
0x713645d1,"MainField_Kibako_Contain_01_1762586303",
0x474e3271,"MainField_Kibako_Contain_01_1763220254",
0xeb397409,"MainField_Kibako_Contain_01_1773028352",
0x46aed3c7,"MainField_Kibako_Contain_01_1820219634",
0xa088cf71,"MainField_Kibako_Contain_01_1821742338",
0x7f975717,"MainField_Kibako_Contain_01_1827037200",
0xc428408d,"MainField_Kibako_Contain_01_1828045060",
0x73125176,"MainField_Kibako_Contain_01_183518962",
0xd4af25e1,"MainField_Kibako_Contain_01_1860255113",
0x911b2d57,"MainField_Kibako_Contain_01_1874303798",
0xd738518c,"MainField_Kibako_Contain_01_1886604510",
0x5cb0b96f,"MainField_Kibako_Contain_01_191658737",
0xc5b60980,"MainField_Kibako_Contain_01_1931258303",
0xcaf5f914,"MainField_Kibako_Contain_01_1931646694",
0x60285877,"MainField_Kibako_Contain_01_1934317608",
0x6b94e622,"MainField_Kibako_Contain_01_1940653690",
0x1af1fa1c,"MainField_Kibako_Contain_01_1941122145",
0xf1fcaaed,"MainField_Kibako_Contain_01_1942213633",
0x397347f2,"MainField_Kibako_Contain_01_1957753829",
0xcfe5f817,"MainField_Kibako_Contain_01_1974316407",
0x1a84d6ef,"MainField_Kibako_Contain_01_1992651955",
0xb5f27626,"MainField_Kibako_Contain_01_2001966506",
0xdff30e00,"MainField_Kibako_Contain_01_2022464653",
0x7a4a435a,"MainField_Kibako_Contain_01_2025053488",
0xfa8d9012,"MainField_Kibako_Contain_01_2025984200",
0xaddd8881,"MainField_Kibako_Contain_01_2035057835",
0x7aa97cb6,"MainField_Kibako_Contain_01_2035442445",
0xbf851e4c,"MainField_Kibako_Contain_01_2040902188",
0xff3e23ba,"MainField_Kibako_Contain_01_2055837272",
0xc2505dba,"MainField_Kibako_Contain_01_2076638813",
0xf23921ee,"MainField_Kibako_Contain_01_2089978156",
0x67b722e5,"MainField_Kibako_Contain_01_2117448776",
0xec458a04,"MainField_Kibako_Contain_01_2129451862",
0x6d156ebd,"MainField_Kibako_Contain_01_213128364",
0xd63c4090,"MainField_Kibako_Contain_01_2133172426",
0x70f9c99e,"MainField_Kibako_Contain_01_2139058808",
0xcf88afc2,"MainField_Kibako_Contain_01_2141555138",
0xeaa96df7,"MainField_Kibako_Contain_01_220107193",
0x500c1d45,"MainField_Kibako_Contain_01_2222932241",
0x5fead4b5,"MainField_Kibako_Contain_01_2231567048",
0xe7d745d1,"MainField_Kibako_Contain_01_2233489494",
0xcdacc8a5,"MainField_Kibako_Contain_01_2249737121",
0xa27df6a0,"MainField_Kibako_Contain_01_2249991250",
0xbee4b1bd,"MainField_Kibako_Contain_01_2259516458",
0xde5d32db,"MainField_Kibako_Contain_01_2264962637",
0xce420f09,"MainField_Kibako_Contain_01_2266680822",
0xb02e03a1,"MainField_Kibako_Contain_01_2269443963",
0x23564072,"MainField_Kibako_Contain_01_2273008717",
0x97899c88,"MainField_Kibako_Contain_01_227339714",
0x63c9d749,"MainField_Kibako_Contain_01_2279983495",
0x443c5284,"MainField_Kibako_Contain_01_23079500",
0x178b7ff5,"MainField_Kibako_Contain_01_2308536964",
0xd59b4d10,"MainField_Kibako_Contain_01_2316965523",
0xa60da322,"MainField_Kibako_Contain_01_2318222146",
0xfbe966a8,"MainField_Kibako_Contain_01_2318893081",
0xd07c2d10,"MainField_Kibako_Contain_01_2374564079",
0xb4b5d9f7,"MainField_Kibako_Contain_01_2384606230",
0x934cfad0,"MainField_Kibako_Contain_01_2388000271",
0x6b54dea9,"MainField_Kibako_Contain_01_2395560866",
0xb6c11bb4,"MainField_Kibako_Contain_01_2409156330",
0xb9115166,"MainField_Kibako_Contain_01_2410216308",
0x51888445,"MainField_Kibako_Contain_01_2423556133",
0x5fcb5bc0,"MainField_Kibako_Contain_01_2425021072",
0xe9e58a0f,"MainField_Kibako_Contain_01_2447172160",
0xd4c69a05,"MainField_Kibako_Contain_01_24657510",
0x59cb82ed,"MainField_Kibako_Contain_01_2467159755",
0xcf0c3c0d,"MainField_Kibako_Contain_01_2488111774",
0x51f59858,"MainField_Kibako_Contain_01_2528308068",
0x5193cf73,"MainField_Kibako_Contain_01_2531717747",
0xd5950856,"MainField_Kibako_Contain_01_2535375624",
0xd292a491,"MainField_Kibako_Contain_01_2540293075",
0x139feca1,"MainField_Kibako_Contain_01_2560146947",
0xbb04ab7a,"MainField_Kibako_Contain_01_2571285744",
0x9e24253f,"MainField_Kibako_Contain_01_2591984613",
0x414e23e1,"MainField_Kibako_Contain_01_2610410982",
0x2f906e6d,"MainField_Kibako_Contain_01_2638119945",
0x375b6297,"MainField_Kibako_Contain_01_263815352",
0xc62043d0,"MainField_Kibako_Contain_01_2649563110",
0xd4f4a009,"MainField_Kibako_Contain_01_2664141540",
0x4a597e3f,"MainField_Kibako_Contain_01_2674469098",
0xcb86c4fa,"MainField_Kibako_Contain_01_268094959",
0xd5abcc83,"MainField_Kibako_Contain_01_2686909392",
0xb4c79d94,"MainField_Kibako_Contain_01_2700772391",
0xde32d74c,"MainField_Kibako_Contain_01_2710092576",
0x3d93dbfe,"MainField_Kibako_Contain_01_2712123177",
0xbd6e9e6a,"MainField_Kibako_Contain_01_271661321",
0xfdd6d124,"MainField_Kibako_Contain_01_271729143",
0xe4842721,"MainField_Kibako_Contain_01_2748388279",
0x5e17c2dc,"MainField_Kibako_Contain_01_2760781110",
0xede3b118,"MainField_Kibako_Contain_01_2773797197",
0xf563c709,"MainField_Kibako_Contain_01_2779945856",
0xe5737df4,"MainField_Kibako_Contain_01_2784848555",
0x3ff27971,"MainField_Kibako_Contain_01_2812384256",
0x0f43349b,"MainField_Kibako_Contain_01_2817326541",
0xeeb6d74a,"MainField_Kibako_Contain_01_2852452559",
0xf0dc2493,"MainField_Kibako_Contain_01_286565114",
0x8719b0a3,"MainField_Kibako_Contain_01_2883504005",
0xfdf7d95b,"MainField_Kibako_Contain_01_2897123574",
0xf34681a9,"MainField_Kibako_Contain_01_2917504347",
0xc5273ca8,"MainField_Kibako_Contain_01_2931026111",
0x2766eda7,"MainField_Kibako_Contain_01_2931099295",
0x527789ea,"MainField_Kibako_Contain_01_294023519",
0x747f330e,"MainField_Kibako_Contain_01_2941609738",
0x12f3ebbf,"MainField_Kibako_Contain_01_2954118721",
0x729683e2,"MainField_Kibako_Contain_01_2956495282",
0x43e7bf1e,"MainField_Kibako_Contain_01_2962361409",
0x0a6145a8,"MainField_Kibako_Contain_01_2966856058",
0x2f4aa21a,"MainField_Kibako_Contain_01_2974031843",
0xf557824c,"MainField_Kibako_Contain_01_2988524211",
0x412f3930,"MainField_Kibako_Contain_01_2992853834",
0x05c0a616,"MainField_Kibako_Contain_01_2995158760",
0x0216d0ae,"MainField_Kibako_Contain_01_2995943039",
0x399949a0,"MainField_Kibako_Contain_01_2997262060",
0x7c3ee438,"MainField_Kibako_Contain_01_3013918732",
0xef71fa9c,"MainField_Kibako_Contain_01_3017110246",
0x3b7b00a9,"MainField_Kibako_Contain_01_3033033493",
0x540d9e61,"MainField_Kibako_Contain_01_3058084008",
0x309aa53c,"MainField_Kibako_Contain_01_3088774011",
0x98b1edec,"MainField_Kibako_Contain_01_3091429361",
0x7ce94e2a,"MainField_Kibako_Contain_01_3097808341",
0x25150d6b,"MainField_Kibako_Contain_01_3101356911",
0x7453aea4,"MainField_Kibako_Contain_01_3104288233",
0xa151fc96,"MainField_Kibako_Contain_01_310600945",
0xdbae374d,"MainField_Kibako_Contain_01_31116990",
0x0f9c1db0,"MainField_Kibako_Contain_01_31180682",
0xfd84668d,"MainField_Kibako_Contain_01_312779394",
0xcf514740,"MainField_Kibako_Contain_01_3127866413",
0x40e973b1,"MainField_Kibako_Contain_01_3130059316",
0x6f4d01e3,"MainField_Kibako_Contain_01_3155215681",
0xab05e9c8,"MainField_Kibako_Contain_01_3169146215",
0x62883a3f,"MainField_Kibako_Contain_01_3190472809",
0x2d167a26,"MainField_Kibako_Contain_01_3193057877",
0x053ef032,"MainField_Kibako_Contain_01_3219216566",
0x4fe4d2e2,"MainField_Kibako_Contain_01_3221066233",
0xe8c21b56,"MainField_Kibako_Contain_01_3233242947",
0x3009c66a,"MainField_Kibako_Contain_01_3245961221",
0x4ad45d8f,"MainField_Kibako_Contain_01_3251083620",
0x276e53ab,"MainField_Kibako_Contain_01_3254428108",
0x7e2482c2,"MainField_Kibako_Contain_01_3260583450",
0x8f12db0e,"MainField_Kibako_Contain_01_3278606716",
0x3d5a9028,"MainField_Kibako_Contain_01_3280889916",
0xdcec0d47,"MainField_Kibako_Contain_01_3302113904",
0xdf21466a,"MainField_Kibako_Contain_01_3304818507",
0xd8cf380c,"MainField_Kibako_Contain_01_3314894569",
0x8bb049b5,"MainField_Kibako_Contain_01_3317371976",
0x39070b59,"MainField_Kibako_Contain_01_3327805036",
0xcf051a9e,"MainField_Kibako_Contain_01_3329478931",
0x8219c629,"MainField_Kibako_Contain_01_3342852466",
0xbb84f396,"MainField_Kibako_Contain_01_3360610479",
0xe9fcfaaf,"MainField_Kibako_Contain_01_3368788931",
0x12c19b38,"MainField_Kibako_Contain_01_3375231231",
0x02f1d2af,"MainField_Kibako_Contain_01_3378929281",
0x9e2e541a,"MainField_Kibako_Contain_01_3418126620",
0x21746437,"MainField_Kibako_Contain_01_3440670559",
0x2f06d9e9,"MainField_Kibako_Contain_01_3482037305",
0x0a9a39e3,"MainField_Kibako_Contain_01_3488485689",
0x457509be,"MainField_Kibako_Contain_01_349281624",
0x0af221b9,"MainField_Kibako_Contain_01_3495765465",
0x2ce4a5e3,"MainField_Kibako_Contain_01_3498248562",
0x9ecebfa9,"MainField_Kibako_Contain_01_3501544880",
0x4ce67807,"MainField_Kibako_Contain_01_3519014601",
0x7da52919,"MainField_Kibako_Contain_01_3538063817",
0xec6caa97,"MainField_Kibako_Contain_01_3588482598",
0x0365560a,"MainField_Kibako_Contain_01_3606750068",
0x9c77b862,"MainField_Kibako_Contain_01_3612660601",
0x4bea51bd,"MainField_Kibako_Contain_01_3619641040",
0x248d43b2,"MainField_Kibako_Contain_01_3629881450",
0xfdbf82e2,"MainField_Kibako_Contain_01_36464301",
0x89c0bd77,"MainField_Kibako_Contain_01_3647210835",
0xa3bfe33c,"MainField_Kibako_Contain_01_3649439647",
0x74a82b5d,"MainField_Kibako_Contain_01_36582799",
0x55ce2f48,"MainField_Kibako_Contain_01_366318169",
0xb04b6c8c,"MainField_Kibako_Contain_01_3715811785",
0x6c1c281b,"MainField_Kibako_Contain_01_3718989200",
0x33476c1e,"MainField_Kibako_Contain_01_3720509899",
0x0bbf2a40,"MainField_Kibako_Contain_01_3720648683",
0x0ed51d0c,"MainField_Kibako_Contain_01_3736411004",
0x21ff7a4a,"MainField_Kibako_Contain_01_3739767480",
0xf65325fe,"MainField_Kibako_Contain_01_3742277454",
0xfd6fa52e,"MainField_Kibako_Contain_01_3743651441",
0xd00d0aad,"MainField_Kibako_Contain_01_3789669093",
0xc59cb086,"MainField_Kibako_Contain_01_3791715907",
0xbbb73302,"MainField_Kibako_Contain_01_379693774",
0xed917bf1,"MainField_Kibako_Contain_01_3811119330",
0xac077219,"MainField_Kibako_Contain_01_382583371",
0x027b32b0,"MainField_Kibako_Contain_01_3831541949",
0xb776a5d5,"MainField_Kibako_Contain_01_3861412647",
0x185a761b,"MainField_Kibako_Contain_01_3870880953",
0x1856716e,"MainField_Kibako_Contain_01_3883120819",
0x07cf0485,"MainField_Kibako_Contain_01_3884050780",
0x07554a15,"MainField_Kibako_Contain_01_38843070",
0x38bcfcaf,"MainField_Kibako_Contain_01_3888550719",
0x91aa0fb5,"MainField_Kibako_Contain_01_3894119790",
0x574051c3,"MainField_Kibako_Contain_01_3938671600",
0x7755b7f6,"MainField_Kibako_Contain_01_3948053634",
0x56f34576,"MainField_Kibako_Contain_01_397472227",
0x83bef7ea,"MainField_Kibako_Contain_01_3978324147",
0xe7bd867d,"MainField_Kibako_Contain_01_3979333003",
0xf0797825,"MainField_Kibako_Contain_01_3981612516",
0x2c157f6c,"MainField_Kibako_Contain_01_4010854720",
0x4d1dd367,"MainField_Kibako_Contain_01_4080067016",
0x19f62561,"MainField_Kibako_Contain_01_4081112693",
0x60c2b960,"MainField_Kibako_Contain_01_4081642257",
0xb5cbb6f1,"MainField_Kibako_Contain_01_4085851373",
0x74b3bc13,"MainField_Kibako_Contain_01_4103218707",
0x9fb39596,"MainField_Kibako_Contain_01_4112432598",
0x365e64dd,"MainField_Kibako_Contain_01_4145624223",
0x1b64885b,"MainField_Kibako_Contain_01_4149251435",
0x19105499,"MainField_Kibako_Contain_01_4156821405",
0xec19e634,"MainField_Kibako_Contain_01_4179952768",
0xa0ec1778,"MainField_Kibako_Contain_01_4188455474",
0xda5ca889,"MainField_Kibako_Contain_01_4222530374",
0x47c371e4,"MainField_Kibako_Contain_01_4247368507",
0xc5142839,"MainField_Kibako_Contain_01_4259395112",
0xab36d92d,"MainField_Kibako_Contain_01_4260193702",
0xa3cab1c8,"MainField_Kibako_Contain_01_4290908155",
0x9907cbd1,"MainField_Kibako_Contain_01_4292311721",
0x93c454cc,"MainField_Kibako_Contain_01_432767870",
0x12fcd1f6,"MainField_Kibako_Contain_01_438235464",
0xb03ade40,"MainField_Kibako_Contain_01_43967580",
0x05ffe178,"MainField_Kibako_Contain_01_445800147",
0xe1b89be7,"MainField_Kibako_Contain_01_454813135",
0xe57f6b9f,"MainField_Kibako_Contain_01_472451564",
0xb214f055,"MainField_Kibako_Contain_01_502427042",
0xa4ad34cb,"MainField_Kibako_Contain_01_514421910",
0xae38a773,"MainField_Kibako_Contain_01_52538535",
0x35bd222a,"MainField_Kibako_Contain_01_52675321",
0x221e1afa,"MainField_Kibako_Contain_01_55119397",
0x89abdf22,"MainField_Kibako_Contain_01_562416917",
0x4341f9aa,"MainField_Kibako_Contain_01_569939322",
0xff1344cd,"MainField_Kibako_Contain_01_611055013",
0xf4f57d51,"MainField_Kibako_Contain_01_655479587",
0xf4a5f62a,"MainField_Kibako_Contain_01_657861497",
0xcec47725,"MainField_Kibako_Contain_01_667316185",
0xc8a9b587,"MainField_Kibako_Contain_01_675049316",
0x91ee1185,"MainField_Kibako_Contain_01_681955094",
0x9ee7ea54,"MainField_Kibako_Contain_01_68917545",
0xb9a60076,"MainField_Kibako_Contain_01_697651886",
0x953a3eb8,"MainField_Kibako_Contain_01_702542016",
0x5c65e4ac,"MainField_Kibako_Contain_01_723082470",
0x1b8600ab,"MainField_Kibako_Contain_01_725458019",
0xd7234e02,"MainField_Kibako_Contain_01_734104218",
0x1a4c874f,"MainField_Kibako_Contain_01_735688749",
0x2ff3a8c4,"MainField_Kibako_Contain_01_744114422",
0x696e5873,"MainField_Kibako_Contain_01_745219474",
0x9721bd61,"MainField_Kibako_Contain_01_750005318",
0x0c8da4c4,"MainField_Kibako_Contain_01_766653631",
0xbc89079c,"MainField_Kibako_Contain_01_837593607",
0xedd13ed4,"MainField_Kibako_Contain_01_847646917",
0x9547679d,"MainField_Kibako_Contain_01_863861303",
0x04b8a038,"MainField_Kibako_Contain_01_872653589",
0x4988f855,"MainField_Kibako_Contain_01_881877526",
0x8f2f26c2,"MainField_Kibako_Contain_01_887124942",
0x690b261c,"MainField_Kibako_Contain_01_887769033",
0x8db19012,"MainField_Kibako_Contain_01_892952540",
0xc3624b41,"MainField_Kibako_Contain_01_896532738",
0x844e6f04,"MainField_Kibako_Contain_01_902325857",
0xafec3d8c,"MainField_Kibako_Contain_01_917616131",
0x32e3ccf8,"MainField_Kibako_Contain_01_920996797",
0x9d92598b,"MainField_Kibako_Contain_01_979782966",
0xfe615f63,"MainField_Kibako_Contain_01_997575132",
0x7fbe0fb5,"MainField_LinkTagAnd_01002540431",
0x03d7569b,"MainField_LinkTagAnd_01005623247",
0x07986a05,"MainField_LinkTagAnd_01009457797",
0xb68aeb36,"MainField_LinkTagAnd_01067358634",
0x429650a8,"MainField_LinkTagAnd_0107370503",
0x48db219e,"MainField_LinkTagAnd_01077323509",
0xcf09da77,"MainField_LinkTagAnd_01108922455",
0x6bf1e4dd,"MainField_LinkTagAnd_01110143446",
0xdbb36199,"MainField_LinkTagAnd_01139991139",
0x9aba28ef,"MainField_LinkTagAnd_01147960305",
0x64e979c1,"MainField_LinkTagAnd_01151755015",
0xc0401094,"MainField_LinkTagAnd_01155150498",
0x025fe40f,"MainField_LinkTagAnd_01162214799",
0x6053463a,"MainField_LinkTagAnd_01180568223",
0xe8efbad5,"MainField_LinkTagAnd_01201417370",
0x8c339031,"MainField_LinkTagAnd_0120171934",
0x7d2a6bfe,"MainField_LinkTagAnd_01225283792",
0x7aa6bba2,"MainField_LinkTagAnd_01240857350",
0x31159a1f,"MainField_LinkTagAnd_01245567603",
0x3f82d474,"MainField_LinkTagAnd_01275763877",
0xf3dec9fb,"MainField_LinkTagAnd_01295599091",
0xf8036707,"MainField_LinkTagAnd_01333433076",
0xec58faa1,"MainField_LinkTagAnd_01335980755",
0x659321cd,"MainField_LinkTagAnd_01338825639",
0xa167ff54,"MainField_LinkTagAnd_01341435321",
0x5d5adddb,"MainField_LinkTagAnd_01370645311",
0x293ea06e,"MainField_LinkTagAnd_01383783422",
0xde877bd5,"MainField_LinkTagAnd_01392065488",
0xb86fb307,"MainField_LinkTagAnd_01407614687",
0x2fd75e9b,"MainField_LinkTagAnd_01427602107",
0xcdaf3b01,"MainField_LinkTagAnd_01478556559",
0x02bbdb73,"MainField_LinkTagAnd_01486576328",
0x11f84162,"MainField_LinkTagAnd_01493297976",
0x3ef23acd,"MainField_LinkTagAnd_01524482677",
0x244cab55,"MainField_LinkTagAnd_01538623154",
0x9245cfe4,"MainField_LinkTagAnd_01545523969",
0xb15e7ad9,"MainField_LinkTagAnd_01546036511",
0x1ef4a696,"MainField_LinkTagAnd_01546212509",
0xc387813f,"MainField_LinkTagAnd_01580526378",
0x624066a2,"MainField_LinkTagAnd_01659021918",
0xf299b3bd,"MainField_LinkTagAnd_01756704575",
0x495cae97,"MainField_LinkTagAnd_01771874625",
0xfdcf3a3f,"MainField_LinkTagAnd_01787028082",
0x28b12fb5,"MainField_LinkTagAnd_01793245543",
0xc03d5f9d,"MainField_LinkTagAnd_018753695",
0x214d0c92,"MainField_LinkTagAnd_01879878746",
0x880f5c63,"MainField_LinkTagAnd_0188533078",
0xb78d6b00,"MainField_LinkTagAnd_01942138564",
0xd7e447fd,"MainField_LinkTagAnd_01979001797",
0xb70a13ac,"MainField_LinkTagAnd_01981356041",
0xf008cc18,"MainField_LinkTagAnd_02000902417",
0x3a8dedd8,"MainField_LinkTagAnd_02033821087",
0x9c67bc06,"MainField_LinkTagAnd_02034550617",
0xb6fb201a,"MainField_LinkTagAnd_02073892428",
0xca1e1a17,"MainField_LinkTagAnd_02090521937",
0xdaa02780,"MainField_LinkTagAnd_02092287629",
0x59f83d2e,"MainField_LinkTagAnd_0211964100",
0x30551ec3,"MainField_LinkTagAnd_02128750805",
0x9d840ff5,"MainField_LinkTagAnd_02148291071",
0xd64acb51,"MainField_LinkTagAnd_0216482844",
0xe496ba0f,"MainField_LinkTagAnd_02183151160",
0x9724031c,"MainField_LinkTagAnd_0218491383",
0x4a47f956,"MainField_LinkTagAnd_02256746509",
0x14acd7b1,"MainField_LinkTagAnd_02288308444",
0xe0c2a9bb,"MainField_LinkTagAnd_02306154684",
0x360e6a8a,"MainField_LinkTagAnd_02328696454",
0xdb2e2c84,"MainField_LinkTagAnd_02376127135",
0xe40582aa,"MainField_LinkTagAnd_02376967958",
0x373465c2,"MainField_LinkTagAnd_02396245808",
0xd96cb143,"MainField_LinkTagAnd_02398882725",
0x24e60d7a,"MainField_LinkTagAnd_02437673162",
0xff58e497,"MainField_LinkTagAnd_02440647719",
0x0e9411f6,"MainField_LinkTagAnd_02461264617",
0xeab79d23,"MainField_LinkTagAnd_02490939093",
0x476c9560,"MainField_LinkTagAnd_02530575023",
0xf4158a32,"MainField_LinkTagAnd_02558839002",
0x2e06476e,"MainField_LinkTagAnd_02589792719",
0xa1c0de21,"MainField_LinkTagAnd_02595112004",
0x9da98a31,"MainField_LinkTagAnd_02646550548",
0x1a039017,"MainField_LinkTagAnd_02669815638",
0xc9c42485,"MainField_LinkTagAnd_02693957714",
0x72685e13,"MainField_LinkTagAnd_02696844741",
0x08e33910,"MainField_LinkTagAnd_0270535179",
0xf8176678,"MainField_LinkTagAnd_02713893515",
0x1d59f54c,"MainField_LinkTagAnd_02718604286",
0xba2b89f3,"MainField_LinkTagAnd_02739968260",
0x3c6b99f3,"MainField_LinkTagAnd_02760787",
0x2d31384d,"MainField_LinkTagAnd_02760835406",
0x5019819b,"MainField_LinkTagAnd_02770488127",
0x1015f80a,"MainField_LinkTagAnd_02840823483",
0xb54a8edb,"MainField_LinkTagAnd_02844937654",
0x8889cfa8,"MainField_LinkTagAnd_0284585202",
0x210eb4c8,"MainField_LinkTagAnd_02866041742",
0x4c808ff5,"MainField_LinkTagAnd_02869904116",
0x7cf5d94c,"MainField_LinkTagAnd_02873859235",
0xe34c6e31,"MainField_LinkTagAnd_02881856494",
0xc18df69f,"MainField_LinkTagAnd_02892138221",
0x2381e004,"MainField_LinkTagAnd_02894606454",
0x53921c99,"MainField_LinkTagAnd_02909463907",
0x241a05be,"MainField_LinkTagAnd_02914621335",
0xc341d253,"MainField_LinkTagAnd_02922684986",
0xc16a8cb0,"MainField_LinkTagAnd_02936747904",
0x8ccc1d05,"MainField_LinkTagAnd_02938209339",
0x95ac22d5,"MainField_LinkTagAnd_02951157137",
0xd2f75f2e,"MainField_LinkTagAnd_02956153581",
0xf66cec74,"MainField_LinkTagAnd_02992425643",
0xdf7f2bb8,"MainField_LinkTagAnd_02997102279",
0xb7a83ea4,"MainField_LinkTagAnd_03033768793",
0x04999590,"MainField_LinkTagAnd_03044285034",
0x3409103f,"MainField_LinkTagAnd_03062123777",
0x7529c31f,"MainField_LinkTagAnd_03076429556",
0xfe2e2212,"MainField_LinkTagAnd_03093025742",
0x342f84bd,"MainField_LinkTagAnd_03103868495",
0x72880157,"MainField_LinkTagAnd_03113010733",
0xcf801bc3,"MainField_LinkTagAnd_03116637411",
0x18733abc,"MainField_LinkTagAnd_03129082587",
0x59e46c27,"MainField_LinkTagAnd_03144992423",
0x6a0ef264,"MainField_LinkTagAnd_031697731",
0xb3fae020,"MainField_LinkTagAnd_03201612348",
0x1e5b4917,"MainField_LinkTagAnd_03204415321",
0x6e7fd6d7,"MainField_LinkTagAnd_03250260873",
0xda396e97,"MainField_LinkTagAnd_03269158601",
0x54be75dd,"MainField_LinkTagAnd_0330584179",
0x041971bc,"MainField_LinkTagAnd_03306652347",
0x0af03300,"MainField_LinkTagAnd_03338719236",
0x53cbd0b2,"MainField_LinkTagAnd_03355479079",
0x304cbad3,"MainField_LinkTagAnd_0336514375",
0x8fa86c5a,"MainField_LinkTagAnd_0338348513",
0xa6b6f311,"MainField_LinkTagAnd_03387686068",
0x0b877543,"MainField_LinkTagAnd_03426572369",
0xe5144045,"MainField_LinkTagAnd_03468275652",
0x8ba26ca1,"MainField_LinkTagAnd_03468897397",
0x004b727e,"MainField_LinkTagAnd_03479038283",
0x6cb44579,"MainField_LinkTagAnd_03497170417",
0x3dc91ba8,"MainField_LinkTagAnd_03506219708",
0x87562bbe,"MainField_LinkTagAnd_03525671849",
0x1420ca73,"MainField_LinkTagAnd_03534439652",
0x5da623e0,"MainField_LinkTagAnd_03536013736",
0x810b9178,"MainField_LinkTagAnd_03537107536",
0x45a850d6,"MainField_LinkTagAnd_03577992039",
0x90838ec5,"MainField_LinkTagAnd_03590835670",
0x03db3508,"MainField_LinkTagAnd_03626415277",
0x21da7326,"MainField_LinkTagAnd_03664182765",
0xfeabd82f,"MainField_LinkTagAnd_03676599490",
0x9b7bcf30,"MainField_LinkTagAnd_03696012341",
0xf70ec5f3,"MainField_LinkTagAnd_03703739256",
0x703db806,"MainField_LinkTagAnd_03713762532",
0x222d160e,"MainField_LinkTagAnd_03726328429",
0x636e89a7,"MainField_LinkTagAnd_03730093613",
0xffbbca94,"MainField_LinkTagAnd_03742580047",
0x84b31d13,"MainField_LinkTagAnd_03781431453",
0xe49326fc,"MainField_LinkTagAnd_03786357512",
0x571df8a6,"MainField_LinkTagAnd_037941669",
0x2962a8de,"MainField_LinkTagAnd_03890968943",
0xac69d175,"MainField_LinkTagAnd_03901983881",
0x65376526,"MainField_LinkTagAnd_03905231271",
0x7f2b524b,"MainField_LinkTagAnd_03942434517",
0x9bc4e68c,"MainField_LinkTagAnd_03944724366",
0xb4833c35,"MainField_LinkTagAnd_03980530580",
0xce387034,"MainField_LinkTagAnd_04002547756",
0x80bafbc8,"MainField_LinkTagAnd_04029252746",
0x511e3820,"MainField_LinkTagAnd_04041116183",
0x7796ecf6,"MainField_LinkTagAnd_04087322967",
0x05c4c06a,"MainField_LinkTagAnd_04098013112",
0x4116eb40,"MainField_LinkTagAnd_04110761178",
0xa790e447,"MainField_LinkTagAnd_04128936201",
0xcbb832df,"MainField_LinkTagAnd_04144718185",
0xc3d07b70,"MainField_LinkTagAnd_04177361368",
0xdcfb065b,"MainField_LinkTagAnd_04196706509",
0x08f4e9df,"MainField_LinkTagAnd_04208457030",
0x7fef7ac8,"MainField_LinkTagAnd_04227703569",
0x219fed48,"MainField_LinkTagAnd_04252530324",
0xfb3b676a,"MainField_LinkTagAnd_04277940071",
0xcf1eb2e9,"MainField_LinkTagAnd_0431180054",
0xd30afb9a,"MainField_LinkTagAnd_0467456336",
0xeaa0c003,"MainField_LinkTagAnd_0472955180",
0xf070b429,"MainField_LinkTagAnd_0497650039",
0xe83b479c,"MainField_LinkTagAnd_0546826384",
0xf2d8e294,"MainField_LinkTagAnd_0568642343",
0x17b511a8,"MainField_LinkTagAnd_0591206962",
0xe7c588d4,"MainField_LinkTagAnd_0597129013",
0xa958af13,"MainField_LinkTagAnd_0605184980",
0x90b31f4d,"MainField_LinkTagAnd_0638475309",
0x910cf693,"MainField_LinkTagAnd_0638512395",
0x9f60da98,"MainField_LinkTagAnd_0666177356",
0x2ae982bb,"MainField_LinkTagAnd_0678356975",
0xa23e8853,"MainField_LinkTagAnd_0685289148",
0x21779ed2,"MainField_LinkTagAnd_0709861992",
0xc15a303e,"MainField_LinkTagAnd_0716650234",
0xfa666d36,"MainField_LinkTagAnd_0723116951",
0x0a2ef0a1,"MainField_LinkTagAnd_073449509",
0xe6970045,"MainField_LinkTagAnd_0797777083",
0xcb2e1125,"MainField_LinkTagAnd_0800418469",
0x41c9c4ed,"MainField_LinkTagAnd_0843238681",
0xff675f05,"MainField_LinkTagAnd_0873556643",
0x146e3f4e,"MainField_LinkTagAnd_0892274944",
0xf21f5215,"MainField_LinkTagAnd_0913348730",
0x05e610d1,"MainField_LinkTagAnd_092411184",
0x711e5d62,"MainField_LinkTagAnd_0925996643",
0x6f58566a,"MainField_LinkTagAnd_098450793",
0x33e1310b,"MainField_LinkTagAnd_0994947659",
0xc529a4d7,"MainField_LinkTagAnd_1009457797",
0xcaefdd13,"MainField_LinkTagAnd_1013825278",
0x6fce770e,"MainField_LinkTagAnd_1055740597",
0x743b25e4,"MainField_LinkTagAnd_1067358634",
0xc45daec8,"MainField_LinkTagAnd_1069226064",
0x8a6aef4c,"MainField_LinkTagAnd_1077323509",
0x0db814a5,"MainField_LinkTagAnd_1108922455",
0x7e070a86,"MainField_LinkTagAnd_1148220128",
0xea02e58d,"MainField_LinkTagAnd_120171934",
0xbf9ba52c,"MainField_LinkTagAnd_1225283792",
0xb0773850,"MainField_LinkTagAnd_1286122902",
0x316f0729,"MainField_LinkTagAnd_1295599091",
0x2ee93473,"MainField_LinkTagAnd_1335980755",
0xeb8f6ebc,"MainField_LinkTagAnd_1383783422",
0x7ade7dd5,"MainField_LinkTagAnd_1407614687",
0xc2b91187,"MainField_LinkTagAnd_1418486775",
0x0f1ef5d3,"MainField_LinkTagAnd_1478556559",
0xd3498fb0,"MainField_LinkTagAnd_1493297976",
0xdc456844,"MainField_LinkTagAnd_1546212509",
0x2b889574,"MainField_LinkTagAnd_1595319596",
0xb56f3f3e,"MainField_LinkTagAnd_1605353149",
0x7bb2c5c7,"MainField_LinkTagAnd_1607972082",
0x88c0de3f,"MainField_LinkTagAnd_1739614725",
0x8bed6045,"MainField_LinkTagAnd_1771874625",
0xe4f3d9f8,"MainField_LinkTagAnd_1777346767",
0xefa64817,"MainField_LinkTagAnd_1802826795",
0xf7732947,"MainField_LinkTagAnd_18753695",
0xe3fcc240,"MainField_LinkTagAnd_1879878746",
0x1555892f,"MainField_LinkTagAnd_1979001797",
0xf83c230a,"MainField_LinkTagAnd_2033821087",
0x3a4a802a,"MainField_LinkTagAnd_2041493791",
0x8b54108e,"MainField_LinkTagAnd_207645734",
0x08afd4c5,"MainField_LinkTagAnd_2090521937",
0x1811e952,"MainField_LinkTagAnd_2092287629",
0x89768e23,"MainField_LinkTagAnd_2113499337",
0x3fc94892,"MainField_LinkTagAnd_211964100",
0x262774dd,"MainField_LinkTagAnd_2183151160",
0x20465cd0,"MainField_LinkTagAnd_2240661935",
0x584a6002,"MainField_LinkTagAnd_224202054",
0xf35bf338,"MainField_LinkTagAnd_2270809650",
0x22736769,"MainField_LinkTagAnd_2306154684",
0x4aeca7f2,"MainField_LinkTagAnd_236562491",
0x199fe256,"MainField_LinkTagAnd_2376127135",
0xe9c77983,"MainField_LinkTagAnd_2392251368",
0x1bdd7f91,"MainField_LinkTagAnd_2398882725",
0x3de92a45,"MainField_LinkTagAnd_2440647719",
0xadb4452c,"MainField_LinkTagAnd_2443394409",
0xbe469a6b,"MainField_LinkTagAnd_245991817",
0xcc25df24,"MainField_LinkTagAnd_2461264617",
0x7d90cdef,"MainField_LinkTagAnd_246178287",
0x85dd5bb2,"MainField_LinkTagAnd_2530575023",
0xe2b2d7a1,"MainField_LinkTagAnd_2546684649",
0xba48fb58,"MainField_LinkTagAnd_2579355965",
0x637110f3,"MainField_LinkTagAnd_2595112004",
0x17c0aafb,"MainField_LinkTagAnd_2625497178",
0x25cc163b,"MainField_LinkTagAnd_2630792535",
0x5f1844e3,"MainField_LinkTagAnd_2646550548",
0xd8b25ec5,"MainField_LinkTagAnd_2669815638",
0xe41cc438,"MainField_LinkTagAnd_2682980147",
0x0b75ea57,"MainField_LinkTagAnd_2693957714",
0xb0d990c1,"MainField_LinkTagAnd_2696844741",
0x789a4721,"MainField_LinkTagAnd_2739968260",
0x092e9fd3,"MainField_LinkTagAnd_276128240",
0x92a84f49,"MainField_LinkTagAnd_2770488127",
0xd2a436d8,"MainField_LinkTagAnd_2840823483",
0xeeb8ba14,"MainField_LinkTagAnd_284585202",
0x21fda0e3,"MainField_LinkTagAnd_2881856494",
0x9123d24b,"MainField_LinkTagAnd_2909463907",
0x03db4262,"MainField_LinkTagAnd_2936747904",
0x1dcee56a,"MainField_LinkTagAnd_2997102279",
0x9cb5c671,"MainField_LinkTagAnd_3065580062",
0xf69e4a6f,"MainField_LinkTagAnd_3103868495",
0xdac2f46e,"MainField_LinkTagAnd_3129082587",
0x0b410da8,"MainField_LinkTagAnd_3151936815",
0x1888a045,"MainField_LinkTagAnd_3269158601",
0x567dcf6f,"MainField_LinkTagAnd_336514375",
0xe99919e6,"MainField_LinkTagAnd_338348513",
0x64073dc3,"MainField_LinkTagAnd_3387686068",
0xa1ddd639,"MainField_LinkTagAnd_3394401417",
0xc936bb91,"MainField_LinkTagAnd_3426572369",
0x27a58e97,"MainField_LinkTagAnd_3468275652",
0x4913a273,"MainField_LinkTagAnd_3468897397",
0xc2fabcac,"MainField_LinkTagAnd_3479038283",
0xae058bab,"MainField_LinkTagAnd_3497170417",
0x45e7e56c,"MainField_LinkTagAnd_3525671849",
0x43ba5faa,"MainField_LinkTagAnd_3537107536",
0xa2753e11,"MainField_LinkTagAnd_3544787347",
0x8c31df2c,"MainField_LinkTagAnd_3580835731",
0x2ce6b7ee,"MainField_LinkTagAnd_364904822",
0xfc9f663e,"MainField_LinkTagAnd_3662080192",
0x3c1a16fd,"MainField_LinkTagAnd_3676599490",
0xc4a7c53b,"MainField_LinkTagAnd_3694429241",
0x59ca01e2,"MainField_LinkTagAnd_3696012341",
0x35bf0b21,"MainField_LinkTagAnd_3703739256",
0xb28c76d4,"MainField_LinkTagAnd_3713762532",
0xa1df4775,"MainField_LinkTagAnd_3730093613",
0x05a126e2,"MainField_LinkTagAnd_3742262380",
0x053eab1f,"MainField_LinkTagAnd_3815322076",
0xebd3660c,"MainField_LinkTagAnd_3890968943",
0x6ed81fa7,"MainField_LinkTagAnd_3901983881",
0xa786abf4,"MainField_LinkTagAnd_3905231271",
0x7632f2e7,"MainField_LinkTagAnd_3980530580",
0x0c89bee6,"MainField_LinkTagAnd_4002547756",
0x420b351a,"MainField_LinkTagAnd_4029252746",
0x93aff6f2,"MainField_LinkTagAnd_4041116183",
0x5c2cdbd7,"MainField_LinkTagAnd_4060996176",
0xc7750eb8,"MainField_LinkTagAnd_4098013112",
0x839aebf6,"MainField_LinkTagAnd_4135847647",
0x0161b5a2,"MainField_LinkTagAnd_4177361368",
0x3744e2e4,"MainField_LinkTagAnd_4236251543",
0xe32e239a,"MainField_LinkTagAnd_4252530324",
0xb53b8e26,"MainField_LinkTagAnd_467456336",
0x8c91b5bf,"MainField_LinkTagAnd_472955180",
0x8e0a3220,"MainField_LinkTagAnd_546826384",
0x71846414,"MainField_LinkTagAnd_591206962",
0x81f4fd68,"MainField_LinkTagAnd_597129013",
0xb51ef1f4,"MainField_LinkTagAnd_598887480",
0xf951af24,"MainField_LinkTagAnd_666177356",
0x4cd8f707,"MainField_LinkTagAnd_678356975",
0xb479408e,"MainField_LinkTagAnd_681398491",
0xc40ffdef,"MainField_LinkTagAnd_685289148",
0xfc034ade,"MainField_LinkTagAnd_695434284",
0xa76b4582,"MainField_LinkTagAnd_716650234",
0x387c8c2a,"MainField_LinkTagAnd_909434108",
0x942e27a9,"MainField_LinkTagAnd_913348730",
0xaf8b2bc6,"MainField_LinkTagAnd_921934664",
0x32a8660b,"MainField_LinkTagAnd_92411184",
0xef0e6d3b,"MainField_LinkTagAnd_933421743",
0x86eba65c,"MainField_LinkTagAnd_966107842",
0x581620b0,"MainField_LinkTagAnd_98450793",
0xfdad902c,"MainField_LinkTagAnd_999899210",
0x33e2fbb7,"MainField_LinkTagNOr_01049954121",
0x1a75a799,"MainField_LinkTagNOr_01401742823",
0x4a0b46ee,"MainField_LinkTagNOr_01433633134",
0x7466ef2e,"MainField_LinkTagNOr_01764870054",
0x8d586cda,"MainField_LinkTagNOr_01856666644",
0xf986ef55,"MainField_LinkTagNOr_01885375232",
0xb464a8c9,"MainField_LinkTagNOr_01956258705",
0xd0722bb8,"MainField_LinkTagNOr_01959322116",
0xe3321d13,"MainField_LinkTagNOr_01968520356",
0x65879a6a,"MainField_LinkTagNOr_01975322236",
0x9a989c91,"MainField_LinkTagNOr_02076793710",
0x78e7828b,"MainField_LinkTagNOr_02213901973",
0xef7725e6,"MainField_LinkTagNOr_02264065472",
0xc673a1c3,"MainField_LinkTagNOr_02525115333",
0x821f8ecc,"MainField_LinkTagNOr_02579829064",
0x9bfd8968,"MainField_LinkTagNOr_02738614663",
0x07c86b1b,"MainField_LinkTagNOr_02852442770",
0x58ce72e8,"MainField_LinkTagNOr_0296346266",
0xade9dbf9,"MainField_LinkTagNOr_03045020604",
0x3a5e4eb5,"MainField_LinkTagNOr_03218753334",
0xc8f01943,"MainField_LinkTagNOr_03276503480",
0xeaf0b61f,"MainField_LinkTagNOr_03290888984",
0xdab452ff,"MainField_LinkTagNOr_03315260381",
0x83024f0c,"MainField_LinkTagNOr_03334293656",
0x8edaddf3,"MainField_LinkTagNOr_03358946592",
0x1dad6e8f,"MainField_LinkTagNOr_03420245346",
0x2f7d3163,"MainField_LinkTagNOr_03622941357",
0x31ab0b9d,"MainField_LinkTagNOr_04024777835",
0x68eb8879,"MainField_LinkTagNOr_04128362397",
0x6fc2802d,"MainField_LinkTagNOr_04181792968",
0x8348a543,"MainField_LinkTagNOr_0418211655",
0xb8190372,"MainField_LinkTagNOr_04272871055",
0xd3cd1d20,"MainField_LinkTagNOr_0585122453",
0xa87dbe0e,"MainField_LinkTagNOr_0646184891",
0xe5d1b8e1,"MainField_LinkTagNOr_0885811464",
0xf4542dbb,"MainField_LinkTagNOr_0961537687",
0x16b634dd,"MainField_LinkTagNOr_0999468100",
0x5e8d584c,"MainField_LinkTagNOr_120857055",
0x8f6c9e01,"MainField_LinkTagNOr_1278059438",
0xd12ccc5b,"MainField_LinkTagNOr_134752554",
0x139c24e5,"MainField_LinkTagNOr_1401742823",
0x919b27cf,"MainField_LinkTagNOr_1416231322",
0x063476c2,"MainField_LinkTagNOr_1758437583",
0x6d70d37f,"MainField_LinkTagNOr_1760344275",
0xe6946ec2,"MainField_LinkTagNOr_1837774171",
0x11e5480f,"MainField_LinkTagNOr_1846743466",
0x84b1efa6,"MainField_LinkTagNOr_1856666644",
0x5895e7f4,"MainField_LinkTagNOr_187674055",
0xf06f6c29,"MainField_LinkTagNOr_1885375232",
0xbd8d2bb5,"MainField_LinkTagNOr_1956258705",
0xd0969d0c,"MainField_LinkTagNOr_1985063515",
0x93711fed,"MainField_LinkTagNOr_2076793710",
0x710e01f7,"MainField_LinkTagNOr_2213901973",
0x851d5392,"MainField_LinkTagNOr_2214836812",
0x53403545,"MainField_LinkTagNOr_2220828342",
0xfa1b840a,"MainField_LinkTagNOr_2258849886",
0x45966f33,"MainField_LinkTagNOr_2280272251",
0xcf9a22bf,"MainField_LinkTagNOr_2525115333",
0x1c8e6281,"MainField_LinkTagNOr_2593104498",
0xeda29f9d,"MainField_LinkTagNOr_2670126457",
0x2a2754a3,"MainField_LinkTagNOr_2713496053",
0x92140a14,"MainField_LinkTagNOr_2738614663",
0x0bcc08ec,"MainField_LinkTagNOr_2781046169",
0x9a9a0ffa,"MainField_LinkTagNOr_2955697642",
0xc5a57805,"MainField_LinkTagNOr_2963339867",
0x31daa6af,"MainField_LinkTagNOr_2981787662",
0x34059b1b,"MainField_LinkTagNOr_3032747760",
0xa4005885,"MainField_LinkTagNOr_3045020604",
0x33b7cdc9,"MainField_LinkTagNOr_3218753334",
0xc1199a3f,"MainField_LinkTagNOr_3276503480",
0xe3193563,"MainField_LinkTagNOr_3290888984",
0x87335e8f,"MainField_LinkTagNOr_3358946592",
0x57ddb6f4,"MainField_LinkTagNOr_3390571582",
0x1444edf3,"MainField_LinkTagNOr_3420245346",
0x42dc6d21,"MainField_LinkTagNOr_3536640936",
0x19a30282,"MainField_LinkTagNOr_3553234376",
0x5a88d203,"MainField_LinkTagNOr_3556677988",
0x06a733a9,"MainField_LinkTagNOr_3652150295",
0xfe7f85fc,"MainField_LinkTagNOr_3667473166",
0x989a259e,"MainField_LinkTagNOr_3692276289",
0x90500c37,"MainField_LinkTagNOr_3731093067",
0xbd39e6e0,"MainField_LinkTagNOr_373427150",
0xee2b6477,"MainField_LinkTagNOr_3767274712",
0x6c17bc30,"MainField_LinkTagNOr_3965079656",
0xad9e40a5,"MainField_LinkTagNOr_3996747459",
0xae1ec898,"MainField_LinkTagNOr_415431862",
0xb1f0800e,"MainField_LinkTagNOr_4272871055",
0x375e4321,"MainField_LinkTagNOr_627722233",
0x12b8382e,"MainField_LinkTagNOr_643179111",
0x76e7865f,"MainField_LinkTagNOr_682830260",
0xd4d3e5af,"MainField_LinkTagNOr_698985441",
0x9a9203fb,"MainField_LinkTagNOr_808155407",
0x7310ffac,"MainField_LinkTagNOr_837517284",
0x01862c05,"MainField_LinkTagNOr_840813421",
0x2a0694f1,"MainField_LinkTagNOr_841135733",
0xba1eefed,"MainField_LinkTagNOr_885811464",
0xfbedb37c,"MainField_LinkTagNOr_896857775",
0x5e1b5838,"MainField_LinkTagNOr_983524051",
0x497963d1,"MainField_LinkTagNOr_999468100",
0xa248396b,"MainField_LinkTagOr_01014649139",
0xfd2b216f,"MainField_LinkTagOr_01117647763",
0xa2a280b6,"MainField_LinkTagOr_01146430649",
0x8021344c,"MainField_LinkTagOr_01396195395",
0x4b38eada,"MainField_LinkTagOr_01478517093",
0xba51b225,"MainField_LinkTagOr_0209495830",
0x765cc90c,"MainField_LinkTagOr_02311366954",
0xf10a7524,"MainField_LinkTagOr_02502238107",
0x77638a22,"MainField_LinkTagOr_02526439863",
0x3f71fb09,"MainField_LinkTagOr_02764001523",
0x0ed8a981,"MainField_LinkTagOr_02836016696",
0xcbdd89f8,"MainField_LinkTagOr_02892943525",
0xb152801b,"MainField_LinkTagOr_03006941669",
0x37c1b3bd,"MainField_LinkTagOr_03162336104",
0x792d1613,"MainField_LinkTagOr_0334619734",
0xd1ca1e27,"MainField_LinkTagOr_03487402071",
0xc8dfb9a5,"MainField_LinkTagOr_0414897360",
0xf40075b2,"MainField_LinkTagOr_1000494017",
0x5f9bc9ea,"MainField_LinkTagOr_1014649139",
0x66bb3984,"MainField_LinkTagOr_1049968401",
0x34b1e378,"MainField_LinkTagOr_1066658161",
0x00f8d1ee,"MainField_LinkTagOr_1117647763",
0x5f717037,"MainField_LinkTagOr_1146430649",
0x630c9c0b,"MainField_LinkTagOr_1289984812",
0x0a271576,"MainField_LinkTagOr_1329778457",
0x7df2c4cd,"MainField_LinkTagOr_1396195395",
0xb6eb1a5b,"MainField_LinkTagOr_1478517093",
0x760dfc82,"MainField_LinkTagOr_1558530775",
0xbf114aa6,"MainField_LinkTagOr_1567846195",
0x1744de81,"MainField_LinkTagOr_1623511099",
0x3d323ea1,"MainField_LinkTagOr_1767671957",
0x7cbc9ade,"MainField_LinkTagOr_2010157227",
0x0b58493f,"MainField_LinkTagOr_209495830",
0x370275e6,"MainField_LinkTagOr_2161444738",
0xa6fbc7fd,"MainField_LinkTagOr_2240844310",
0x8b8f398d,"MainField_LinkTagOr_2311366954",
0x0cd985a5,"MainField_LinkTagOr_2502238107",
0x8ab07aa3,"MainField_LinkTagOr_2526439863",
0xc2a20b88,"MainField_LinkTagOr_2764001523",
0xf30b5900,"MainField_LinkTagOr_2836016696",
0x360e7979,"MainField_LinkTagOr_2892943525",
0x4c81709a,"MainField_LinkTagOr_3006941669",
0x08c270a4,"MainField_LinkTagOr_315409454",
0xca12433c,"MainField_LinkTagOr_3162336104",
0x6d24b887,"MainField_LinkTagOr_3320585069",
0xc824ed09,"MainField_LinkTagOr_334619734",
0xe2087ac1,"MainField_LinkTagOr_3358946592",
0xe0483a72,"MainField_LinkTagOr_3406368958",
0x2c19eea6,"MainField_LinkTagOr_3487402071",
0xb463ee8c,"MainField_LinkTagOr_3584832237",
0x4f768a18,"MainField_LinkTagOr_3595280035",
0x21a03cc0,"MainField_LinkTagOr_3626746998",
0xfe5dd285,"MainField_LinkTagOr_3847691251",
0x79d642bf,"MainField_LinkTagOr_414897360",
0x462f51fa,"MainField_Mannequin_001_Head_1464690557",
0x39e6254e,"MainField_Mannequin_001_Head_3083729277",
0x7455e6f8,"MainField_Mannequin_001_Lower_1505990737",
0x408bdb87,"MainField_Mannequin_001_Lower_3498351279",
0xee6635f4,"MainField_Mannequin_001_Upper_3234515435",
0xbf403aa5,"MainField_Mannequin_001_Upper_476338033",
0x6f767243,"MainField_Mannequin_002_Head_3151128676",
0xf9da3bda,"MainField_Mannequin_002_Lower_3622970673",
0xb8b1ad78,"MainField_Mannequin_002_Upper_3340583869",
0xc8f153e6,"MainField_Mannequin_003_Head_3970723426",
0x7f02ea4c,"MainField_Mannequin_003_Lower_1710226079",
0x8df56c19,"MainField_Mannequin_003_Upper_2611699444",
0x32686b65,"MainField_Mannequin_004_Head_1119266109",
0xee6c0a14,"MainField_Mannequin_004_Lower_900830635",
0xa64fe419,"MainField_Mannequin_004_Upper_2897789969",
0x6dfc48ec,"MainField_Mannequin_005_Head_2778091608",
0xd446eaa9,"MainField_Mannequin_005_Lower_3101984204",
0xfec8b3a6,"MainField_Mannequin_005_Upper_3006369983",
0x8ef013d5,"MainField_Mannequin_006_Head_2279837677",
0xca5a4e1b,"MainField_Mannequin_006_Head_3433564697",
0xe4ec0a8f,"MainField_Mannequin_006_Lower_158107662",
0x9fe4b78a,"MainField_Mannequin_006_Lower_1899234696",
0xd6826eab,"MainField_Mannequin_006_Upper_1263112051",
0x65dbfb33,"MainField_Mannequin_006_Upper_2723173023",
0x659a472e,"MainField_Mannequin_007_Head_4020240563",
0xf04f6f51,"MainField_Mannequin_007_Lower_781748965",
0x8b1fe2d2,"MainField_Mannequin_007_Upper_103485299",
0x6a7b35ab,"MainField_Mannequin_008_Head_1970588510",
0xb588f61b,"MainField_Mannequin_008_Lower_4254996732",
0xcc61ce15,"MainField_Mannequin_008_Upper_356621356",
0x77c94a47,"MainField_Mannequin_009_Upper_323319186",
0xb67b41cc,"MainField_Necklace_2_Enemy_Giant_Junior_1602515170",
0xecdf23f0,"MainField_Necklace_2_Enemy_Giant_Junior_1833514674",
0xc501e749,"MainField_Necklace_2_Enemy_Giant_Junior_1903632265",
0x447dd2bf,"MainField_Necklace_2_Enemy_Giant_Junior_3337886442",
0x5fdd0556,"MainField_Necklace_2_Enemy_Giant_Junior_3350013125",
0x5556f599,"MainField_Necklace_2_Enemy_Giant_Junior_373348007",
0xf697cee9,"MainField_Necklace_2_Enemy_Giant_Junior_4099815924",
0x3884f8bf,"MainField_Necklace_2_Enemy_Giant_Middle_1266282060",
0x222c1abf,"MainField_Necklace_2_Enemy_Giant_Middle_1384742674",
0xacbc63b7,"MainField_Necklace_2_Enemy_Giant_Middle_1925665429",
0x5973b7ba,"MainField_Necklace_2_Enemy_Giant_Middle_1949698080",
0x29bc8f28,"MainField_Necklace_2_Enemy_Giant_Middle_2083136285",
0x2e1b9584,"MainField_Necklace_2_Enemy_Giant_Middle_2472409077",
0xdaefd1b5,"MainField_Necklace_2_Enemy_Giant_Middle_2530315670",
0x623416cf,"MainField_Necklace_2_Enemy_Giant_Middle_3272527560",
0x8e6ed136,"MainField_Necklace_2_Enemy_Giant_Middle_3572226454",
0xcd8f0bb1,"MainField_Necklace_2_Enemy_Giant_Middle_3627901807",
0xc325e6b1,"MainField_Necklace_2_Enemy_Giant_Middle_3637193649",
0x22e03ac3,"MainField_Necklace_2_Enemy_Giant_Middle_3706280648",
0x49d9da30,"MainField_Necklace_2_Enemy_Giant_Middle_3744176720",
0xd8a1ecd0,"MainField_Necklace_2_Enemy_Giant_Middle_3831867213",
0x693bd414,"MainField_Necklace_2_Enemy_Giant_Middle_4080554254",
0x791a9984,"MainField_Necklace_2_Enemy_Giant_Middle_936329231",
0x898b59ae,"MainField_Necklace_2_Enemy_Giant_Senior_1028264440",
0x11cfb6dc,"MainField_Necklace_2_Enemy_Giant_Senior_2009832062",
0x7c2c499c,"MainField_Necklace_2_Enemy_Giant_Senior_2853871544",
0x0d527450,"MainField_Necklace_2_Enemy_Giant_Senior_3410381997",
0x88397e0d,"MainField_Necklace_2_Enemy_Giant_Senior_3741497810",
0xb2457869,"MainField_Necklace_2_Enemy_Giant_Senior_3785639248",
0x0c8679fc,"MainField_Necklace_2_Enemy_Giant_Senior_851170738",
0xb89ca91d,"MainField_Necklace_3_Enemy_Giant_Junior_1602515170",
0xe238cb21,"MainField_Necklace_3_Enemy_Giant_Junior_1833514674",
0xcbe60f98,"MainField_Necklace_3_Enemy_Giant_Junior_1903632265",
0x4a9a3a6e,"MainField_Necklace_3_Enemy_Giant_Junior_3337886442",
0x513aed87,"MainField_Necklace_3_Enemy_Giant_Junior_3350013125",
0x69361691,"MainField_Necklace_3_Enemy_Giant_Junior_373348007",
0xf8702638,"MainField_Necklace_3_Enemy_Giant_Junior_4099815924",
0x3663106e,"MainField_Necklace_3_Enemy_Giant_Middle_1266282060",
0x2ccbf26e,"MainField_Necklace_3_Enemy_Giant_Middle_1384742674",
0xa25b8b66,"MainField_Necklace_3_Enemy_Giant_Middle_1925665429",
0x57945f6b,"MainField_Necklace_3_Enemy_Giant_Middle_1949698080",
0x275b67f9,"MainField_Necklace_3_Enemy_Giant_Middle_2083136285",
0x20fc7d55,"MainField_Necklace_3_Enemy_Giant_Middle_2472409077",
0xd4083964,"MainField_Necklace_3_Enemy_Giant_Middle_2530315670",
0x6cd3fe1e,"MainField_Necklace_3_Enemy_Giant_Middle_3272527560",
0x808939e7,"MainField_Necklace_3_Enemy_Giant_Middle_3572226454",
0xc368e360,"MainField_Necklace_3_Enemy_Giant_Middle_3627901807",
0xcdc20e60,"MainField_Necklace_3_Enemy_Giant_Middle_3637193649",
0x2c07d212,"MainField_Necklace_3_Enemy_Giant_Middle_3706280648",
0x473e32e1,"MainField_Necklace_3_Enemy_Giant_Middle_3744176720",
0xd6460401,"MainField_Necklace_3_Enemy_Giant_Middle_3831867213",
0x67dc3cc5,"MainField_Necklace_3_Enemy_Giant_Middle_4080554254",
0x457a7a8c,"MainField_Necklace_3_Enemy_Giant_Middle_936329231",
0x876cb17f,"MainField_Necklace_3_Enemy_Giant_Senior_1028264440",
0x1f285e0d,"MainField_Necklace_3_Enemy_Giant_Senior_2009832062",
0x72cba14d,"MainField_Necklace_3_Enemy_Giant_Senior_2853871544",
0x03b59c81,"MainField_Necklace_3_Enemy_Giant_Senior_3410381997",
0x86de96dc,"MainField_Necklace_3_Enemy_Giant_Senior_3741497810",
0xbca290b8,"MainField_Necklace_3_Enemy_Giant_Senior_3785639248",
0x30e69af4,"MainField_Necklace_3_Enemy_Giant_Senior_851170738",
0x902b332a,"MainField_Necklace_4_Enemy_Giant_Junior_1602515170",
0xca8f5116,"MainField_Necklace_4_Enemy_Giant_Junior_1833514674",
0xe35195af,"MainField_Necklace_4_Enemy_Giant_Junior_1903632265",
0x622da059,"MainField_Necklace_4_Enemy_Giant_Junior_3337886442",
0x798d77b0,"MainField_Necklace_4_Enemy_Giant_Junior_3350013125",
0xdc14bfa9,"MainField_Necklace_4_Enemy_Giant_Junior_373348007",
0xd0c7bc0f,"MainField_Necklace_4_Enemy_Giant_Junior_4099815924",
0x1ed48a59,"MainField_Necklace_4_Enemy_Giant_Middle_1266282060",
0x047c6859,"MainField_Necklace_4_Enemy_Giant_Middle_1384742674",
0x8aec1151,"MainField_Necklace_4_Enemy_Giant_Middle_1925665429",
0x7f23c55c,"MainField_Necklace_4_Enemy_Giant_Middle_1949698080",
0x0fecfdce,"MainField_Necklace_4_Enemy_Giant_Middle_2083136285",
0x084be762,"MainField_Necklace_4_Enemy_Giant_Middle_2472409077",
0xfcbfa353,"MainField_Necklace_4_Enemy_Giant_Middle_2530315670",
0x44646429,"MainField_Necklace_4_Enemy_Giant_Middle_3272527560",
0xa83ea3d0,"MainField_Necklace_4_Enemy_Giant_Middle_3572226454",
0xebdf7957,"MainField_Necklace_4_Enemy_Giant_Middle_3627901807",
0xe5759457,"MainField_Necklace_4_Enemy_Giant_Middle_3637193649",
0x04b04825,"MainField_Necklace_4_Enemy_Giant_Middle_3706280648",
0x6f89a8d6,"MainField_Necklace_4_Enemy_Giant_Middle_3744176720",
0xfef19e36,"MainField_Necklace_4_Enemy_Giant_Middle_3831867213",
0x4f6ba6f2,"MainField_Necklace_4_Enemy_Giant_Middle_4080554254",
0xf058d3b4,"MainField_Necklace_4_Enemy_Giant_Middle_936329231",
0xafdb2b48,"MainField_Necklace_4_Enemy_Giant_Senior_1028264440",
0x379fc43a,"MainField_Necklace_4_Enemy_Giant_Senior_2009832062",
0x5a7c3b7a,"MainField_Necklace_4_Enemy_Giant_Senior_2853871544",
0x2b0206b6,"MainField_Necklace_4_Enemy_Giant_Senior_3410381997",
0xae690ceb,"MainField_Necklace_4_Enemy_Giant_Senior_3741497810",
0x94150a8f,"MainField_Necklace_4_Enemy_Giant_Senior_3785639248",
0x85c433cc,"MainField_Necklace_4_Enemy_Giant_Senior_851170738",
0xc96c6d4a,"MainField_Npc_Assassin_003_2854671073",
0xb468b493,"MainField_Npc_Assassin_003_4167850397",
0xb8c3e00d,"MainField_Npc_Assassin_004_4124138202",
0x23a16544,"MainField_Npc_Assassin_004_871248885",
0xe0b090f0,"MainField_Npc_Assassin_005_1745518528",
0x9722c059,"MainField_Npc_Assassin_005_647902910",
0x5e93a914,"MainField_Npc_Assassin_006_1176364745",
0xeb1ee78c,"MainField_Npc_Assassin_006_2864604240",
0x0edef9a0,"MainField_Npc_Assassin_007_1300569185",
0x39b68d31,"MainField_Npc_Assassin_007_2951346853",
0x9500df49,"MainField_Npc_Assassin_008_2195089996",
0x04baeb78,"MainField_Npc_Assassin_008_2370630445",
0x83d82826,"MainField_Npc_Assassin_009_1734138914",
0x21d89ad8,"MainField_Npc_Assassin_009_3289491899",
0xa85e84cf,"MainField_Npc_Assassin_010_2186816156",
0x54af7c8e,"MainField_Npc_Assassin_010_2644587388",
0x40491760,"MainField_Obj_ArrowBundle_A_01_104326536",
0xc4fca91a,"MainField_Obj_ArrowBundle_A_01_1143254632",
0x47f1d8a4,"MainField_Obj_ArrowBundle_A_01_1181169881",
0x8954eaa9,"MainField_Obj_ArrowBundle_A_01_1206640374",
0x8004cc1e,"MainField_Obj_ArrowBundle_A_01_1217052511",
0x892d4773,"MainField_Obj_ArrowBundle_A_01_1250568235",
0x4952b17e,"MainField_Obj_ArrowBundle_A_01_1375034529",
0x9ff4b538,"MainField_Obj_ArrowBundle_A_01_1625979581",
0xf0905072,"MainField_Obj_ArrowBundle_A_01_1693734294",
0xd8456189,"MainField_Obj_ArrowBundle_A_01_1968613294",
0x48bbb8fb,"MainField_Obj_ArrowBundle_A_01_2059197917",
0x617c4593,"MainField_Obj_ArrowBundle_A_01_2078675370",
0xcef47a51,"MainField_Obj_ArrowBundle_A_01_2229672066",
0x09fe31f7,"MainField_Obj_ArrowBundle_A_01_2335650137",
0xbb666189,"MainField_Obj_ArrowBundle_A_01_2366597095",
0xc401658a,"MainField_Obj_ArrowBundle_A_01_2490393731",
0xa147f45e,"MainField_Obj_ArrowBundle_A_01_2651607220",
0x94f3f6ba,"MainField_Obj_ArrowBundle_A_01_2737686192",
0x29eba4cf,"MainField_Obj_ArrowBundle_A_01_2954360101",
0xeeaa2616,"MainField_Obj_ArrowBundle_A_01_3045297429",
0xff10d0b2,"MainField_Obj_ArrowBundle_A_01_3226607141",
0x2afc0c53,"MainField_Obj_ArrowBundle_A_01_3693696969",
0xc29f7936,"MainField_Obj_ArrowBundle_A_01_4192414471",
0x845dc73b,"MainField_Obj_ArrowBundle_A_01_4209733671",
0xf0456f0e,"MainField_Obj_ArrowBundle_A_01_4281097401",
0x3fff8abb,"MainField_Obj_ArrowBundle_A_01_4282945147",
0xcf8b22e0,"MainField_Obj_ArrowBundle_A_01_52521870",
0x5748064f,"MainField_Obj_ArrowBundle_A_01_530188445",
0x131cb357,"MainField_Obj_ArrowBundle_A_01_604621328",
0xc4949e43,"MainField_Obj_ArrowBundle_A_01_842198494",
0x8692f6b5,"MainField_Obj_ArrowBundle_A_01_953811842",
0x4a860caf,"MainField_Obj_ArrowBundle_A_02_1565871891",
0x20233a61,"MainField_Obj_ArrowBundle_A_02_2448731010",
0xd88f278b,"MainField_Obj_ArrowBundle_A_02_3671553760",
0xb80ffb32,"MainField_Obj_ArrowBundle_A_02_3678533569",
0xe730a9c7,"MainField_Obj_ArrowBundle_A_02_4213327015",
0x1af943c5,"MainField_Obj_ArrowBundle_A_02_781734643",
0xe36cb70d,"MainField_Obj_ArrowNormal_A_01_1001259404",
0xa27ca427,"MainField_Obj_ArrowNormal_A_01_1104975964",
0x3de4d5b9,"MainField_Obj_ArrowNormal_A_01_1159691569",
0x0fd28157,"MainField_Obj_ArrowNormal_A_01_1219181513",
0x4cc1e0a1,"MainField_Obj_ArrowNormal_A_01_1286156570",
0xd9c21952,"MainField_Obj_ArrowNormal_A_01_1288657734",
0x53c0b318,"MainField_Obj_ArrowNormal_A_01_1352031416",
0x29bcd15a,"MainField_Obj_ArrowNormal_A_01_138157657",
0x3e435145,"MainField_Obj_ArrowNormal_A_01_1420714779",
0x00ffcfc3,"MainField_Obj_ArrowNormal_A_01_1548825902",
0x07906ac8,"MainField_Obj_ArrowNormal_A_01_171020671",
0x7ebe595d,"MainField_Obj_ArrowNormal_A_01_1767893739",
0x305100d4,"MainField_Obj_ArrowNormal_A_01_1794513050",
0xab1b0f71,"MainField_Obj_ArrowNormal_A_01_1818988013",
0x32856f68,"MainField_Obj_ArrowNormal_A_01_194140051",
0x3398b2de,"MainField_Obj_ArrowNormal_A_01_209754865",
0xfb32c884,"MainField_Obj_ArrowNormal_A_01_2100732393",
0xddc1bf25,"MainField_Obj_ArrowNormal_A_01_2205287810",
0x738aa5ed,"MainField_Obj_ArrowNormal_A_01_2270268359",
0xd220f7c2,"MainField_Obj_ArrowNormal_A_01_230419787",
0x2ecc553a,"MainField_Obj_ArrowNormal_A_01_232808380",
0xf7c6365d,"MainField_Obj_ArrowNormal_A_01_2366079943",
0x4be154db,"MainField_Obj_ArrowNormal_A_01_2469970117",
0x449769f0,"MainField_Obj_ArrowNormal_A_01_2481390089",
0x6a5fc445,"MainField_Obj_ArrowNormal_A_01_2490239251",
0xa1f4b2a9,"MainField_Obj_ArrowNormal_A_01_2494741745",
0x55da3f2a,"MainField_Obj_ArrowNormal_A_01_2617197286",
0x5737205d,"MainField_Obj_ArrowNormal_A_01_2736175636",
0xbde9e738,"MainField_Obj_ArrowNormal_A_01_2870025245",
0xac473575,"MainField_Obj_ArrowNormal_A_01_2887517999",
0x423e7c49,"MainField_Obj_ArrowNormal_A_01_3140810162",
0xd424bb6d,"MainField_Obj_ArrowNormal_A_01_3202921479",
0xf340c305,"MainField_Obj_ArrowNormal_A_01_3253909725",
0x7ed0cc0f,"MainField_Obj_ArrowNormal_A_01_3261761776",
0x2be29435,"MainField_Obj_ArrowNormal_A_01_3333114615",
0x44dad0da,"MainField_Obj_ArrowNormal_A_01_342164231",
0x0e8dc10b,"MainField_Obj_ArrowNormal_A_01_3473740196",
0xac0ed578,"MainField_Obj_ArrowNormal_A_01_3485440572",
0x08e27d96,"MainField_Obj_ArrowNormal_A_01_3496487899",
0xc6d6bbc3,"MainField_Obj_ArrowNormal_A_01_3539340131",
0x328318e0,"MainField_Obj_ArrowNormal_A_01_3622408058",
0x05f09a9a,"MainField_Obj_ArrowNormal_A_01_3682330770",
0x404f25a5,"MainField_Obj_ArrowNormal_A_01_3692440715",
0x6a63be7d,"MainField_Obj_ArrowNormal_A_01_3796935703",
0x01ef6832,"MainField_Obj_ArrowNormal_A_01_3803281416",
0x22c9a579,"MainField_Obj_ArrowNormal_A_01_3820325991",
0x14a9cc98,"MainField_Obj_ArrowNormal_A_01_3825440511",
0xc34720fe,"MainField_Obj_ArrowNormal_A_01_3930597103",
0xf1efea2c,"MainField_Obj_ArrowNormal_A_01_4011195624",
0x3b4f5302,"MainField_Obj_ArrowNormal_A_01_4097441548",
0x1f13b09d,"MainField_Obj_ArrowNormal_A_01_4103822331",
0xc4ce0b8b,"MainField_Obj_ArrowNormal_A_01_4115013171",
0xe4f6afba,"MainField_Obj_ArrowNormal_A_01_4224572748",
0xbd344ca5,"MainField_Obj_ArrowNormal_A_01_434280214",
0xf3b637b4,"MainField_Obj_ArrowNormal_A_01_490530031",
0x198deb36,"MainField_Obj_ArrowNormal_A_01_494332192",
0x7dba7c94,"MainField_Obj_ArrowNormal_A_01_502343692",
0x6dbc6bb3,"MainField_Obj_ArrowNormal_A_01_509139581",
0x94ef980b,"MainField_Obj_ArrowNormal_A_01_538713299",
0x12d62eb2,"MainField_Obj_ArrowNormal_A_01_599060572",
0xe213b28f,"MainField_Obj_ArrowNormal_A_01_599392247",
0x48e7f424,"MainField_Obj_ArrowNormal_A_01_64311030",
0x3bc35a46,"MainField_Obj_ArrowNormal_A_01_696507855",
0xd2627c15,"MainField_Obj_ArrowNormal_A_01_730154593",
0xeac0890f,"MainField_Obj_ArrowNormal_A_01_811795757",
0x7faa6d9a,"MainField_Obj_ArrowNormal_A_01_840453543",
0x91a9d953,"MainField_Obj_ArrowNormal_A_01_954223568",
0x76a4d73c,"MainField_Obj_ArrowNormal_A_01_991778216",
0xcafe7507,"MainField_Obj_BombArrow_A_01_1080870958",
0x040e993f,"MainField_Obj_BombArrow_A_01_2471941070",
0x3ee203df,"MainField_Obj_BombArrow_A_01_3076876445",
0xc9f5114d,"MainField_Obj_BombArrow_A_01_595390082",
0x91f8ac34,"MainField_Obj_BombArrow_A_01_704007747",
0x177212c7,"MainField_Obj_BombArrow_A_01_982280357",
0xa297d454,"MainField_Obj_BombArrow_A_02_1466808496",
0x30359316,"MainField_Obj_BombArrow_A_02_2222155631",
0x57000ebb,"MainField_Obj_BombArrow_A_02_2397313937",
0x5a2c64ad,"MainField_Obj_BombArrow_A_02_4084488185",
0x60f71a9d,"MainField_Obj_BombArrow_A_02_494548693",
0x1ed4d138,"MainField_Obj_BombArrow_A_03_1824347534",
0x1b95b547,"MainField_Obj_BombArrow_A_03_225064014",
0x37a9e148,"MainField_Obj_BombArrow_A_03_2837080832",
0x66280568,"MainField_Obj_BombArrow_A_03_2870820858",
0x2594d2b1,"MainField_Obj_BombArrow_A_03_3317184289",
0x0075eaac,"MainField_Obj_BombArrow_A_03_3402260026",
0x4b3055d0,"MainField_Obj_BombArrow_A_03_3917545227",
0x32e051f0,"MainField_Obj_BombArrow_A_03_402822343",
0xf9e337d8,"MainField_Obj_BombArrow_A_03_4271435072",
0x1aa003d4,"MainField_Obj_BombArrow_A_04_2199728615",
0x9825c502,"MainField_Obj_BombArrow_A_04_3796181444",
0xcf805155,"MainField_Obj_BombArrow_A_04_4035843768",
0xb67660ee,"MainField_Obj_BreakBoxIron_100149117",
0x854b201b,"MainField_Obj_BreakBoxIron_101984067",
0x8f8736d4,"MainField_Obj_BreakBoxIron_1061708421",
0x34faa4ca,"MainField_Obj_BreakBoxIron_1085494598",
0x57076d3c,"MainField_Obj_BreakBoxIron_111064574",
0xda056e13,"MainField_Obj_BreakBoxIron_1112416019",
0xa4d90801,"MainField_Obj_BreakBoxIron_1118600506",
0xb4b8a1f4,"MainField_Obj_BreakBoxIron_1124302297",
0x5e4ec2a1,"MainField_Obj_BreakBoxIron_1132603902",
0xa4f67f03,"MainField_Obj_BreakBoxIron_1136968874",
0xa3bd7115,"MainField_Obj_BreakBoxIron_1185119684",
0x52ad869b,"MainField_Obj_BreakBoxIron_1211283308",
0xc40ec618,"MainField_Obj_BreakBoxIron_1227167884",
0xd5b069fe,"MainField_Obj_BreakBoxIron_1227395689",
0x8daf7c0f,"MainField_Obj_BreakBoxIron_1243127097",
0xb4ce5be7,"MainField_Obj_BreakBoxIron_1250022001",
0xf85a354e,"MainField_Obj_BreakBoxIron_1271958282",
0x7d78f178,"MainField_Obj_BreakBoxIron_1282772419",
0x41388444,"MainField_Obj_BreakBoxIron_1340744002",
0xbf559673,"MainField_Obj_BreakBoxIron_1347322053",
0x9941eac0,"MainField_Obj_BreakBoxIron_1350459389",
0xec7a917e,"MainField_Obj_BreakBoxIron_1352796921",
0x97958c77,"MainField_Obj_BreakBoxIron_1360871543",
0xeccf4bf7,"MainField_Obj_BreakBoxIron_1361730607",
0x6426aaa5,"MainField_Obj_BreakBoxIron_1364153708",
0x8723dd6a,"MainField_Obj_BreakBoxIron_1383293154",
0x3f61b5f7,"MainField_Obj_BreakBoxIron_1385078942",
0x02437759,"MainField_Obj_BreakBoxIron_1424946315",
0xfdcbb2d4,"MainField_Obj_BreakBoxIron_1430251340",
0x188b311d,"MainField_Obj_BreakBoxIron_144247192",
0x7d0d8fca,"MainField_Obj_BreakBoxIron_1453665421",
0x37e6ffcc,"MainField_Obj_BreakBoxIron_1453985221",
0xc3c6bd2c,"MainField_Obj_BreakBoxIron_1463530555",
0x551c4781,"MainField_Obj_BreakBoxIron_1482125081",
0xebfb5d31,"MainField_Obj_BreakBoxIron_1489336740",
0xe37cff99,"MainField_Obj_BreakBoxIron_1507867657",
0xf0701289,"MainField_Obj_BreakBoxIron_152684024",
0x30ae50d9,"MainField_Obj_BreakBoxIron_1559118902",
0xe6a3c267,"MainField_Obj_BreakBoxIron_1586825463",
0x23d1abe7,"MainField_Obj_BreakBoxIron_161053602",
0x4ce710c8,"MainField_Obj_BreakBoxIron_1632243812",
0x363fdde1,"MainField_Obj_BreakBoxIron_1668685218",
0xb8fafa60,"MainField_Obj_BreakBoxIron_1687394027",
0xd750ad6b,"MainField_Obj_BreakBoxIron_1697498068",
0x43a6f081,"MainField_Obj_BreakBoxIron_1724603969",
0xe263ade5,"MainField_Obj_BreakBoxIron_1756009823",
0x62b39340,"MainField_Obj_BreakBoxIron_1780094834",
0xfe5157e3,"MainField_Obj_BreakBoxIron_1783212173",
0xb8469895,"MainField_Obj_BreakBoxIron_1835403361",
0xc7866b78,"MainField_Obj_BreakBoxIron_1842754325",
0xeae7d244,"MainField_Obj_BreakBoxIron_1848484337",
0x6d60c46d,"MainField_Obj_BreakBoxIron_1855955344",
0x0766f6f7,"MainField_Obj_BreakBoxIron_1892327565",
0x3e07b6f1,"MainField_Obj_BreakBoxIron_1902159255",
0x874a8c36,"MainField_Obj_BreakBoxIron_1916461131",
0x26fc6ab9,"MainField_Obj_BreakBoxIron_1917025837",
0x08b2db27,"MainField_Obj_BreakBoxIron_1932821353",
0x1d3dd05e,"MainField_Obj_BreakBoxIron_1939139986",
0x4145df78,"MainField_Obj_BreakBoxIron_1968015419",
0x8fa90443,"MainField_Obj_BreakBoxIron_1971887837",
0x2a7de1f6,"MainField_Obj_BreakBoxIron_2013661223",
0x5f537885,"MainField_Obj_BreakBoxIron_2037150242",
0xc25aafdf,"MainField_Obj_BreakBoxIron_2079360932",
0x0923cd4a,"MainField_Obj_BreakBoxIron_2095812072",
0x0e9477a1,"MainField_Obj_BreakBoxIron_2114116548",
0xadf65121,"MainField_Obj_BreakBoxIron_2123456638",
0xe995646c,"MainField_Obj_BreakBoxIron_2132873279",
0xf0838166,"MainField_Obj_BreakBoxIron_2141075726",
0x06d758eb,"MainField_Obj_BreakBoxIron_2141498647",
0x4561aca1,"MainField_Obj_BreakBoxIron_2172021456",
0xdbbdea4e,"MainField_Obj_BreakBoxIron_2185894762",
0x8dce71d6,"MainField_Obj_BreakBoxIron_2195253322",
0xfd9df3a3,"MainField_Obj_BreakBoxIron_2196477685",
0xc2faa56a,"MainField_Obj_BreakBoxIron_2208929119",
0x82450716,"MainField_Obj_BreakBoxIron_2213060970",
0x37f25e89,"MainField_Obj_BreakBoxIron_2239927031",
0x4852b245,"MainField_Obj_BreakBoxIron_2255451655",
0x2c616be0,"MainField_Obj_BreakBoxIron_2313787867",
0xa6cbdae2,"MainField_Obj_BreakBoxIron_2324038279",
0xb4cbeb51,"MainField_Obj_BreakBoxIron_2351623378",
0xbb8095ee,"MainField_Obj_BreakBoxIron_2377709389",
0x9e30dee0,"MainField_Obj_BreakBoxIron_2388983426",
0xb2a93900,"MainField_Obj_BreakBoxIron_2390209924",
0xb779b28a,"MainField_Obj_BreakBoxIron_2401692927",
0xe5cd2d88,"MainField_Obj_BreakBoxIron_2401773920",
0xb1e65890,"MainField_Obj_BreakBoxIron_2408168732",
0xc82785b6,"MainField_Obj_BreakBoxIron_2425593783",
0x89d29060,"MainField_Obj_BreakBoxIron_2482880290",
0x3d6ae8b4,"MainField_Obj_BreakBoxIron_2498259938",
0x577284e3,"MainField_Obj_BreakBoxIron_2506506552",
0xbaff2108,"MainField_Obj_BreakBoxIron_2515123787",
0x7f9943d0,"MainField_Obj_BreakBoxIron_2516885506",
0x253898dc,"MainField_Obj_BreakBoxIron_2533229204",
0x9f41531d,"MainField_Obj_BreakBoxIron_2533486054",
0xbabd237f,"MainField_Obj_BreakBoxIron_2535102922",
0x5ce8c893,"MainField_Obj_BreakBoxIron_2536239949",
0x0ee611f8,"MainField_Obj_BreakBoxIron_2551447736",
0x46398d4b,"MainField_Obj_BreakBoxIron_2563555784",
0x085647b3,"MainField_Obj_BreakBoxIron_257151201",
0x0e98e6b1,"MainField_Obj_BreakBoxIron_2579014416",
0x06a6eb44,"MainField_Obj_BreakBoxIron_2626656116",
0x89b11d1c,"MainField_Obj_BreakBoxIron_2632264320",
0x43177197,"MainField_Obj_BreakBoxIron_2638042270",
0x53fe1a4e,"MainField_Obj_BreakBoxIron_2644268167",
0xfe09866a,"MainField_Obj_BreakBoxIron_2660404632",
0x385be966,"MainField_Obj_BreakBoxIron_2660607068",
0x6c4e8b5d,"MainField_Obj_BreakBoxIron_26774448",
0xfb867327,"MainField_Obj_BreakBoxIron_2683766125",
0x7999f737,"MainField_Obj_BreakBoxIron_2683955972",
0x032058eb,"MainField_Obj_BreakBoxIron_2690854135",
0x9851edc8,"MainField_Obj_BreakBoxIron_2709345389",
0xdff28a2f,"MainField_Obj_BreakBoxIron_2715574094",
0x17cea0b2,"MainField_Obj_BreakBoxIron_2720350902",
0xd8b47221,"MainField_Obj_BreakBoxIron_2751501761",
0xdc6fa56d,"MainField_Obj_BreakBoxIron_2767007466",
0x15656f07,"MainField_Obj_BreakBoxIron_2768269695",
0x4ea296fb,"MainField_Obj_BreakBoxIron_2791048796",
0xcea2916d,"MainField_Obj_BreakBoxIron_2792703537",
0xd457709d,"MainField_Obj_BreakBoxIron_2815520021",
0xdba65f9a,"MainField_Obj_BreakBoxIron_2827544787",
0xb96e2c9b,"MainField_Obj_BreakBoxIron_2846527263",
0x01a338e7,"MainField_Obj_BreakBoxIron_2851871269",
0xf723f847,"MainField_Obj_BreakBoxIron_2876429561",
0x9697cc58,"MainField_Obj_BreakBoxIron_2970164140",
0x228b4aa2,"MainField_Obj_BreakBoxIron_299536998",
0x53991ef5,"MainField_Obj_BreakBoxIron_3020308765",
0x71f26c9f,"MainField_Obj_BreakBoxIron_3055021622",
0x86adc6e5,"MainField_Obj_BreakBoxIron_3077229305",
0xc1e935e2,"MainField_Obj_BreakBoxIron_3099269713",
0xf3438293,"MainField_Obj_BreakBoxIron_3110646883",
0x1afd1d6a,"MainField_Obj_BreakBoxIron_3115094244",
0xb3a9368c,"MainField_Obj_BreakBoxIron_3119686070",
0xb69cf767,"MainField_Obj_BreakBoxIron_3133581985",
0x86415dc2,"MainField_Obj_BreakBoxIron_3163117150",
0x98361931,"MainField_Obj_BreakBoxIron_3190958267",
0xe1425018,"MainField_Obj_BreakBoxIron_3193420074",
0x8aaf41e8,"MainField_Obj_BreakBoxIron_3208237358",
0x27f30f78,"MainField_Obj_BreakBoxIron_3208338878",
0x5fa493de,"MainField_Obj_BreakBoxIron_322700932",
0x709e4d45,"MainField_Obj_BreakBoxIron_3236523443",
0x944d3c2c,"MainField_Obj_BreakBoxIron_3275828963",
0xa12348ec,"MainField_Obj_BreakBoxIron_3277862347",
0x59a91856,"MainField_Obj_BreakBoxIron_3283294669",
0x8912e072,"MainField_Obj_BreakBoxIron_3287425910",
0xa7c5e247,"MainField_Obj_BreakBoxIron_3323686864",
0x5bab4d88,"MainField_Obj_BreakBoxIron_3333910237",
0x75a3e35f,"MainField_Obj_BreakBoxIron_3339741758",
0xd66f889b,"MainField_Obj_BreakBoxIron_3362356923",
0x5cffd47c,"MainField_Obj_BreakBoxIron_3371584564",
0xa6b4d330,"MainField_Obj_BreakBoxIron_340094043",
0x4f4cdbb5,"MainField_Obj_BreakBoxIron_3403198460",
0x3987c118,"MainField_Obj_BreakBoxIron_3406251670",
0x9f7ac48b,"MainField_Obj_BreakBoxIron_340954461",
0xe7873fa3,"MainField_Obj_BreakBoxIron_3416340641",
0xdb2707d2,"MainField_Obj_BreakBoxIron_3464819248",
0x9f9231df,"MainField_Obj_BreakBoxIron_3466242851",
0x531467b3,"MainField_Obj_BreakBoxIron_3467304705",
0x462273f3,"MainField_Obj_BreakBoxIron_3483663136",
0x5ee62868,"MainField_Obj_BreakBoxIron_3530661859",
0x2458704d,"MainField_Obj_BreakBoxIron_3531625699",
0xd2c836c1,"MainField_Obj_BreakBoxIron_3566148733",
0x5cba93c6,"MainField_Obj_BreakBoxIron_3596655020",
0xecfa6f30,"MainField_Obj_BreakBoxIron_3602754361",
0xb95dfd38,"MainField_Obj_BreakBoxIron_3616774124",
0xcfb08199,"MainField_Obj_BreakBoxIron_3620782508",
0xb192122f,"MainField_Obj_BreakBoxIron_3626352758",
0x7a586531,"MainField_Obj_BreakBoxIron_3638979530",
0x5ab4a019,"MainField_Obj_BreakBoxIron_3656078204",
0x007040d4,"MainField_Obj_BreakBoxIron_3656539378",
0x17b71479,"MainField_Obj_BreakBoxIron_365691785",
0x9dccc805,"MainField_Obj_BreakBoxIron_3664672440",
0x85246a0d,"MainField_Obj_BreakBoxIron_3681108143",
0x985e8daa,"MainField_Obj_BreakBoxIron_3716647952",
0xad9cecfb,"MainField_Obj_BreakBoxIron_3761030107",
0x493cdf46,"MainField_Obj_BreakBoxIron_3782068687",
0x18bfb486,"MainField_Obj_BreakBoxIron_3819761072",
0xbb6ce057,"MainField_Obj_BreakBoxIron_3846511378",
0x6a73a3cc,"MainField_Obj_BreakBoxIron_3848734432",
0x6dad332a,"MainField_Obj_BreakBoxIron_391018105",
0x8bed0f77,"MainField_Obj_BreakBoxIron_395288765",
0x8723f9ec,"MainField_Obj_BreakBoxIron_3966736013",
0xdc8fb949,"MainField_Obj_BreakBoxIron_396926713",
0x3e4f00e3,"MainField_Obj_BreakBoxIron_3997968360",
0x269ca72d,"MainField_Obj_BreakBoxIron_4008474161",
0xe63e3c59,"MainField_Obj_BreakBoxIron_401491564",
0x6104ba13,"MainField_Obj_BreakBoxIron_402038370",
0xe3254571,"MainField_Obj_BreakBoxIron_4021743415",
0xa6a235f0,"MainField_Obj_BreakBoxIron_4076518212",
0x79b73b4c,"MainField_Obj_BreakBoxIron_4077684674",
0xb5a475d5,"MainField_Obj_BreakBoxIron_410657532",
0x41b76a41,"MainField_Obj_BreakBoxIron_4114549282",
0x90c975c7,"MainField_Obj_BreakBoxIron_4115475452",
0x3d1a35f6,"MainField_Obj_BreakBoxIron_4130966385",
0x951f8fd3,"MainField_Obj_BreakBoxIron_4136425534",
0xd643de4d,"MainField_Obj_BreakBoxIron_4159927629",
0x5aae3e9c,"MainField_Obj_BreakBoxIron_416969010",
0x58696815,"MainField_Obj_BreakBoxIron_4172270562",
0x96407765,"MainField_Obj_BreakBoxIron_4174598605",
0xefb2053b,"MainField_Obj_BreakBoxIron_4175561539",
0x71250787,"MainField_Obj_BreakBoxIron_4179337224",
0xcefd0d47,"MainField_Obj_BreakBoxIron_4209990667",
0x638805f5,"MainField_Obj_BreakBoxIron_4219238497",
0x5908d528,"MainField_Obj_BreakBoxIron_424598170",
0x74e77a8f,"MainField_Obj_BreakBoxIron_4273894307",
0xc4d4978a,"MainField_Obj_BreakBoxIron_4275366208",
0x34be9c41,"MainField_Obj_BreakBoxIron_451846098",
0xc18d40ec,"MainField_Obj_BreakBoxIron_456759154",
0x74555592,"MainField_Obj_BreakBoxIron_517470054",
0x29d4cd46,"MainField_Obj_BreakBoxIron_540837037",
0x45ed3721,"MainField_Obj_BreakBoxIron_564272155",
0xee7dea45,"MainField_Obj_BreakBoxIron_574477042",
0x41c045f1,"MainField_Obj_BreakBoxIron_58161867",
0xbf908112,"MainField_Obj_BreakBoxIron_589959829",
0x9c5b74a7,"MainField_Obj_BreakBoxIron_614243649",
0x398e4d41,"MainField_Obj_BreakBoxIron_699920643",
0xf4fe1311,"MainField_Obj_BreakBoxIron_703536012",
0xb257ae03,"MainField_Obj_BreakBoxIron_704879948",
0x9ac1ae37,"MainField_Obj_BreakBoxIron_72501506",
0x4ea75044,"MainField_Obj_BreakBoxIron_766681689",
0x1dc76c1e,"MainField_Obj_BreakBoxIron_786893983",
0x31f564d9,"MainField_Obj_BreakBoxIron_795632321",
0xc4ba48b0,"MainField_Obj_BreakBoxIron_814287136",
0x1cab1214,"MainField_Obj_BreakBoxIron_832691566",
0x5a13b8b5,"MainField_Obj_BreakBoxIron_849305686",
0x91adcf44,"MainField_Obj_BreakBoxIron_849479959",
0x36e7603c,"MainField_Obj_BreakBoxIron_873460232",
0xf757c51b,"MainField_Obj_BreakBoxIron_877846422",
0xe338e7c3,"MainField_Obj_BreakBoxIron_885257276",
0x14b386f1,"MainField_Obj_BreakBoxIron_902174933",
0xb57dbd90,"MainField_Obj_BreakBoxIron_902290123",
0xf721764f,"MainField_Obj_BreakBoxIron_955375461",
0x7365714b,"MainField_Obj_BreakBoxIron_964693721",
0x686748c8,"MainField_Obj_BreakBoxIron_969280642",
0xa1ac0092,"MainField_Obj_BreakBoxIron_986917964",
0xa89db78b,"MainField_Obj_CursedStatue_2173265156",
0xd7ff168f,"MainField_Obj_ElectricArrow_A_01_1044907851",
0x9c6e9cf8,"MainField_Obj_ElectricArrow_A_01_1159902432",
0x53695293,"MainField_Obj_ElectricArrow_A_01_1302573441",
0x32889efb,"MainField_Obj_ElectricArrow_A_01_1364837345",
0x58d0e5c3,"MainField_Obj_ElectricArrow_A_01_1570295101",
0xdf368e56,"MainField_Obj_ElectricArrow_A_01_1648146414",
0x3d4022ac,"MainField_Obj_ElectricArrow_A_01_1775425580",
0x1dd1fbfe,"MainField_Obj_ElectricArrow_A_01_1804270533",
0x4261d65a,"MainField_Obj_ElectricArrow_A_01_1878381173",
0xdab14ebd,"MainField_Obj_ElectricArrow_A_01_1890919000",
0xeac3acea,"MainField_Obj_ElectricArrow_A_01_1961020829",
0xd3c826e2,"MainField_Obj_ElectricArrow_A_01_1979453085",
0xcd50ec3d,"MainField_Obj_ElectricArrow_A_01_2176002702",
0xedb5af3f,"MainField_Obj_ElectricArrow_A_01_2200824882",
0xd0b1f459,"MainField_Obj_ElectricArrow_A_01_249226888",
0xf8f5fd51,"MainField_Obj_ElectricArrow_A_01_2616356785",
0xa7cdbccd,"MainField_Obj_ElectricArrow_A_01_2697014978",
0x1955e11e,"MainField_Obj_ElectricArrow_A_01_2887482578",
0xb6235dba,"MainField_Obj_ElectricArrow_A_01_2894466686",
0xb68010cd,"MainField_Obj_ElectricArrow_A_01_2898239923",
0x3aea140e,"MainField_Obj_ElectricArrow_A_01_2958766099",
0x3e199883,"MainField_Obj_ElectricArrow_A_01_3198158322",
0xbba3d821,"MainField_Obj_ElectricArrow_A_01_3423936384",
0xe3f81cd9,"MainField_Obj_ElectricArrow_A_01_3442737001",
0xefd701c7,"MainField_Obj_ElectricArrow_A_01_3559906240",
0xd024c004,"MainField_Obj_ElectricArrow_A_01_356222840",
0x53eae2f7,"MainField_Obj_ElectricArrow_A_01_3672025857",
0x703bb4c2,"MainField_Obj_ElectricArrow_A_01_3685731609",
0x98fff9cb,"MainField_Obj_ElectricArrow_A_01_3835410778",
0x2f27ca07,"MainField_Obj_ElectricArrow_A_01_3952646974",
0xa202c281,"MainField_Obj_ElectricArrow_A_01_3975503655",
0xac80016f,"MainField_Obj_ElectricArrow_A_01_4019692309",
0x46a34775,"MainField_Obj_ElectricArrow_A_01_404790932",
0xdfda9c93,"MainField_Obj_ElectricArrow_A_01_4059555831",
0xf9acc2f6,"MainField_Obj_ElectricArrow_A_01_4060917725",
0x47d66899,"MainField_Obj_ElectricArrow_A_01_4068715135",
0x7c2c23cc,"MainField_Obj_ElectricArrow_A_01_419860334",
0xdfffc0cd,"MainField_Obj_ElectricArrow_A_01_474452737",
0x9bf5c638,"MainField_Obj_ElectricArrow_A_01_548383416",
0x1ed3a760,"MainField_Obj_ElectricArrow_A_01_616270904",
0x838d1a70,"MainField_Obj_ElectricArrow_A_01_633972021",
0x9984d6ba,"MainField_Obj_ElectricArrow_A_01_713771261",
0xd9e24bb2,"MainField_Obj_ElectricArrow_A_01_874940154",
0x52cc46d6,"MainField_Obj_ElectricArrow_A_01_892319556",
0x35991d00,"MainField_Obj_ElectricArrow_A_01_916924898",
0x3624d898,"MainField_Obj_ElectricArrow_A_01_956128016",
0x2e8041eb,"MainField_Obj_ElectricArrow_A_02_1587378202",
0x5dc7cd1e,"MainField_Obj_ElectricArrow_A_02_1869963911",
0x2d865d12,"MainField_Obj_ElectricArrow_A_02_2686552731",
0x0779396e,"MainField_Obj_ElectricArrow_A_02_2865912841",
0x579cecda,"MainField_Obj_ElectricArrow_A_02_2904790203",
0xa950a1ac,"MainField_Obj_ElectricArrow_A_02_3167129310",
0x085e192a,"MainField_Obj_ElectricArrow_A_02_3257507155",
0x5833412c,"MainField_Obj_ElectricArrow_A_02_3372378419",
0x1dda945e,"MainField_Obj_ElectricArrow_A_02_3521419955",
0x258fcd17,"MainField_Obj_ElectricArrow_A_02_3588598609",
0x6228456d,"MainField_Obj_ElectricArrow_A_02_4176114606",
0xc21fda63,"MainField_Obj_ElectricArrow_A_03_3669839288",
0x3c02a3d8,"MainField_Obj_ElectricArrow_A_03_3818853889",
0xfa3e028b,"MainField_Obj_ElectricArrow_A_03_4085716448",
0x168f5faa,"MainField_Obj_ElectricArrow_B_01_2882035209",
0x61fdb96a,"MainField_Obj_ElectricArrow_B_01_3016000036",
0x57f3b8f6,"MainField_Obj_ElectricArrow_B_01_3315793647",
0xae143b6c,"MainField_Obj_ElectricArrow_B_01_428988414",
0x2aa1feb9,"MainField_Obj_ElectricArrow_B_01_467689076",
0xecafd4cc,"MainField_Obj_FireArrow_A_01_1665160369",
0x8cc60684,"MainField_Obj_FireArrow_A_01_1690970245",
0xb12218db,"MainField_Obj_FireArrow_A_01_1860829",
0x64bc5e39,"MainField_Obj_FireArrow_A_01_2068878617",
0x0a0433ca,"MainField_Obj_FireArrow_A_01_2328119721",
0x8ef95a67,"MainField_Obj_FireArrow_A_01_2370712989",
0x60e1991c,"MainField_Obj_FireArrow_A_01_2793920243",
0xfd990a1e,"MainField_Obj_FireArrow_A_01_3211562123",
0xa82b3a72,"MainField_Obj_FireArrow_A_01_332208147",
0x0a0b097d,"MainField_Obj_FireArrow_A_01_340223015",
0xb0a16162,"MainField_Obj_FireArrow_A_01_3605932796",
0x642bf5f7,"MainField_Obj_FireArrow_A_01_4101322004",
0x330da895,"MainField_Obj_FireArrow_A_01_4199089419",
0xdce9c6a7,"MainField_Obj_FireArrow_A_01_4247044370",
0xee9047f4,"MainField_Obj_FireArrow_A_02_1719164055",
0xbd9e16b3,"MainField_Obj_FireArrow_A_02_2231344185",
0x8209a558,"MainField_Obj_FireArrow_A_02_2453108868",
0x530dbeef,"MainField_Obj_FireArrow_A_02_2749325849",
0xd87ddc8c,"MainField_Obj_FireArrow_A_02_3326680720",
0x42845db9,"MainField_Obj_FireArrow_A_02_3390858056",
0x69852bef,"MainField_Obj_FireArrow_A_02_876639024",
0x2ad4e59d,"MainField_Obj_FireArrow_A_02_94923181",
0x381e7b55,"MainField_Obj_FireArrow_A_03_1680707752",
0x66b809b0,"MainField_Obj_FireArrow_A_03_2376699312",
0x8b5fb9dc,"MainField_Obj_FireWoodBundle_1024084511",
0x858788ea,"MainField_Obj_FireWoodBundle_1082478980",
0xb607d10c,"MainField_Obj_FireWoodBundle_1141651084",
0x5fcb95c2,"MainField_Obj_FireWoodBundle_123787489",
0xb2b89933,"MainField_Obj_FireWoodBundle_124827830",
0x0033fa29,"MainField_Obj_FireWoodBundle_1265957682",
0xfc890235,"MainField_Obj_FireWoodBundle_1499709072",
0xe1616c9e,"MainField_Obj_FireWoodBundle_1551621666",
0x3dd2f6d1,"MainField_Obj_FireWoodBundle_1586754001",
0x24950e06,"MainField_Obj_FireWoodBundle_1596777380",
0x643fedeb,"MainField_Obj_FireWoodBundle_1636492269",
0x8af6404f,"MainField_Obj_FireWoodBundle_1638812826",
0xa4f6ad84,"MainField_Obj_FireWoodBundle_1716831356",
0x684d0196,"MainField_Obj_FireWoodBundle_1809945329",
0x55242a83,"MainField_Obj_FireWoodBundle_1820657436",
0x37b04d34,"MainField_Obj_FireWoodBundle_1885994421",
0x14e6e567,"MainField_Obj_FireWoodBundle_1951610583",
0xdfd84e88,"MainField_Obj_FireWoodBundle_195501587",
0x74e48a7b,"MainField_Obj_FireWoodBundle_198599831",
0x46317166,"MainField_Obj_FireWoodBundle_1995785739",
0x316ea0ce,"MainField_Obj_FireWoodBundle_2003834661",
0xb0f88eac,"MainField_Obj_FireWoodBundle_2175426792",
0x1fdbd0ff,"MainField_Obj_FireWoodBundle_2242468712",
0x373c54da,"MainField_Obj_FireWoodBundle_2331660654",
0xf929448d,"MainField_Obj_FireWoodBundle_2417273870",
0x83ea8b7c,"MainField_Obj_FireWoodBundle_2484053286",
0x1c886c4c,"MainField_Obj_FireWoodBundle_2620331439",
0x9bb30a28,"MainField_Obj_FireWoodBundle_2650029472",
0x73332a66,"MainField_Obj_FireWoodBundle_265911868",
0x8e13dcd0,"MainField_Obj_FireWoodBundle_2914493107",
0x50650622,"MainField_Obj_FireWoodBundle_3004280971",
0x6a1ffc39,"MainField_Obj_FireWoodBundle_3005679063",
0x88477f2a,"MainField_Obj_FireWoodBundle_3016534064",
0x03d2a670,"MainField_Obj_FireWoodBundle_3099888654",
0x200623c2,"MainField_Obj_FireWoodBundle_3174693002",
0xa298cae9,"MainField_Obj_FireWoodBundle_3209180155",
0xfadb4120,"MainField_Obj_FireWoodBundle_332384990",
0x0639b814,"MainField_Obj_FireWoodBundle_3330446218",
0x20a2f6f4,"MainField_Obj_FireWoodBundle_3551537802",
0x0f93ab0f,"MainField_Obj_FireWoodBundle_3629080301",
0xa112a00c,"MainField_Obj_FireWoodBundle_3653874056",
0xfb77ad45,"MainField_Obj_FireWoodBundle_3730914611",
0xf47e80ee,"MainField_Obj_FireWoodBundle_3772178468",
0xb03b55d9,"MainField_Obj_FireWoodBundle_3839045275",
0xe4a3e0da,"MainField_Obj_FireWoodBundle_384177386",
0xb3029e7e,"MainField_Obj_FireWoodBundle_38502412",
0x80fd879c,"MainField_Obj_FireWoodBundle_3906327336",
0x00f22c47,"MainField_Obj_FireWoodBundle_3945276729",
0xab934a6f,"MainField_Obj_FireWoodBundle_4022704257",
0x2f626f06,"MainField_Obj_FireWoodBundle_4065365242",
0x1455777e,"MainField_Obj_FireWoodBundle_4217278304",
0x7447a31d,"MainField_Obj_FireWoodBundle_4231955495",
0xab22c5ce,"MainField_Obj_FireWoodBundle_448697528",
0x865cc344,"MainField_Obj_FireWoodBundle_461518730",
0xaa5540ef,"MainField_Obj_FireWoodBundle_484876903",
0xb2daade3,"MainField_Obj_FireWoodBundle_58367987",
0xdeee83e9,"MainField_Obj_FireWoodBundle_599503119",
0x796ab72c,"MainField_Obj_FireWoodBundle_63584719",
0xe57745b1,"MainField_Obj_FireWoodBundle_769330334",
0x89d61638,"MainField_Obj_FireWoodBundle_814555103",
0x13a69268,"MainField_Obj_HeartUtuwa_A_01_1752524829",
0x7a71a850,"MainField_Obj_HeartUtuwa_A_01_1966408948",
0x0f9e2957,"MainField_Obj_HeartUtuwa_A_01_2008015156",
0x285e2141,"MainField_Obj_HeartUtuwa_A_01_2048666970",
0xcbe13433,"MainField_Obj_HeartUtuwa_A_01_2669318871",
0x11605356,"MainField_Obj_HeartUtuwa_A_01_2876298557",
0xfe3d23cc,"MainField_Obj_HeartUtuwa_A_01_2899155860",
0x90a68168,"MainField_Obj_HeartUtuwa_A_01_2969929706",
0x439215b8,"MainField_Obj_HeartUtuwa_A_01_3000711256",
0x77f7b9fc,"MainField_Obj_HeartUtuwa_A_01_3060687117",
0x8d254883,"MainField_Obj_HeartUtuwa_A_01_3100641397",
0x4856f46b,"MainField_Obj_HeartUtuwa_A_01_3208703741",
0xb5b4a6a1,"MainField_Obj_HeartUtuwa_A_01_3290157087",
0xbcb01e5f,"MainField_Obj_HeartUtuwa_A_01_3353329220",
0x574f2fb1,"MainField_Obj_HeartUtuwa_A_01_3390309880",
0xaacf01f7,"MainField_Obj_HeartUtuwa_A_01_3544034218",
0xf3588ea8,"MainField_Obj_HeartUtuwa_A_01_3631701225",
0x2698bd97,"MainField_Obj_HeartUtuwa_A_01_3758954541",
0xb729fc61,"MainField_Obj_IceArrow_A_01_1752985112",
0x2a4659e7,"MainField_Obj_IceArrow_A_01_1888385130",
0x26cf7da4,"MainField_Obj_IceArrow_A_01_212342133",
0x0091146d,"MainField_Obj_IceArrow_A_01_2835255053",
0x8961d713,"MainField_Obj_IceArrow_A_01_3011486537",
0x7ae43ca9,"MainField_Obj_IceArrow_A_01_331262396",
0x4ecfbb51,"MainField_Obj_IceArrow_A_01_3971405820",
0xe0b8f533,"MainField_Obj_IceArrow_A_01_670202578",
0x92883dbd,"MainField_Obj_IceArrow_A_01_992175632",
0xc5c43c32,"MainField_Obj_IceArrow_A_02_128925015",
0x5d5f1b0d,"MainField_Obj_IceArrow_A_02_1469072906",
0x1fb74d44,"MainField_Obj_IceArrow_A_02_2715301603",
0x27ca3e6e,"MainField_Obj_IceArrow_A_02_3063251121",
0x3f2ce560,"MainField_Obj_IceArrow_A_02_3097084216",
0x36da611d,"MainField_Obj_IceArrow_A_02_3222230819",
0x58da2734,"MainField_Obj_IceArrow_A_02_403990989",
0xf49ccc67,"MainField_Obj_IceArrow_A_02_4058467774",
0x12050221,"MainField_Obj_IceArrow_A_03_222971291",
0xf0d2218c,"MainField_Obj_IceArrow_A_03_2936112579",
0x3e33838f,"MainField_Obj_IceArrow_A_03_4139604285",
0x8c9d58e3,"MainField_Obj_LiftRockEldin_A_01_1128071282",
0x0275de21,"MainField_Obj_LiftRockEldin_A_01_1237778080",
0x8e3a467e,"MainField_Obj_LiftRockEldin_A_01_1481290765",
0x7715cca7,"MainField_Obj_LiftRockEldin_A_01_1532898288",
0x145f5540,"MainField_Obj_LiftRockEldin_A_01_1673920388",
0xfb26e1d6,"MainField_Obj_LiftRockEldin_A_01_171892770",
0x6ed6e1b3,"MainField_Obj_LiftRockEldin_A_01_1729804154",
0xf0b18983,"MainField_Obj_LiftRockEldin_A_01_1831705542",
0x48c44e24,"MainField_Obj_LiftRockEldin_A_01_1842547049",
0x61086b36,"MainField_Obj_LiftRockEldin_A_01_2050949223",
0x36bcfebe,"MainField_Obj_LiftRockEldin_A_01_2083902405",
0x2c28684a,"MainField_Obj_LiftRockEldin_A_01_2100940980",
0xb5d20a3e,"MainField_Obj_LiftRockEldin_A_01_2162452786",
0x7817b91d,"MainField_Obj_LiftRockEldin_A_01_2319917637",
0xe409c5ac,"MainField_Obj_LiftRockEldin_A_01_2657266377",
0x8ccf8266,"MainField_Obj_LiftRockEldin_A_01_2733506551",
0xc09ee0de,"MainField_Obj_LiftRockEldin_A_01_2811257338",
0x96e7ba8b,"MainField_Obj_LiftRockEldin_A_01_2882343065",
0xa7530aa4,"MainField_Obj_LiftRockEldin_A_01_3003303136",
0xc4bedad5,"MainField_Obj_LiftRockEldin_A_01_3022054256",
0xe137aed7,"MainField_Obj_LiftRockEldin_A_01_3117942625",
0x1a79bb4f,"MainField_Obj_LiftRockEldin_A_01_3233422504",
0x4b71974f,"MainField_Obj_LiftRockEldin_A_01_3268687130",
0xf9ad39fe,"MainField_Obj_LiftRockEldin_A_01_3355030435",
0x37fe2afc,"MainField_Obj_LiftRockEldin_A_01_3373853036",
0xe346dc67,"MainField_Obj_LiftRockEldin_A_01_3470218984",
0x2a72ad80,"MainField_Obj_LiftRockEldin_A_01_3476897372",
0x2cf4e1da,"MainField_Obj_LiftRockEldin_A_01_3828384014",
0xf97c8108,"MainField_Obj_LiftRockEldin_A_01_3874391593",
0x6627377e,"MainField_Obj_LiftRockEldin_A_01_4029110040",
0x56ce1f8d,"MainField_Obj_LiftRockEldin_A_01_403752600",
0x240ff830,"MainField_Obj_LiftRockEldin_A_01_4037706201",
0x46d439d0,"MainField_Obj_LiftRockEldin_A_01_4080292031",
0xd3982443,"MainField_Obj_LiftRockEldin_A_01_4089886629",
0x6bd5cb0b,"MainField_Obj_LiftRockEldin_A_01_4100676014",
0xd1d862f8,"MainField_Obj_LiftRockEldin_A_01_4173135948",
0xa14d67f5,"MainField_Obj_LiftRockEldin_A_01_4272235308",
0xe1a1bebd,"MainField_Obj_LiftRockEldin_A_01_441497790",
0x7fa9cb68,"MainField_Obj_LiftRockEldin_A_01_462195136",
0xce123114,"MainField_Obj_LiftRockEldin_A_01_483148832",
0x73832f89,"MainField_Obj_LiftRockEldin_A_01_731607420",
0x65d77d69,"MainField_Obj_LiftRockEldin_A_01_752462086",
0x906fa0a0,"MainField_Obj_LiftRockEldin_A_01_757707466",
0x63f39ce7,"MainField_Obj_LiftRockEldin_A_01_806288025",
0x57d883fe,"MainField_Obj_LiftRockEldin_A_01_816369909",
0x8e356172,"MainField_Obj_LiftRockEldin_A_01_817120263",
0x7e35a77f,"MainField_Obj_LiftRockEldin_A_01_824377303",
0x23d92b7c,"MainField_Obj_LiftRockEldin_A_01_872191371",
0x68e56b9d,"MainField_Obj_LiftRockEldin_A_01_93303305",
0x75725f3c,"MainField_Obj_LiftRockEldin_A_01_968001826",
0x857a426b,"MainField_Obj_LiftRockEldin_A_01_988150546",
0x7e79260a,"MainField_Obj_LiftRockGerudo_A_01_1005882231",
0x7f531639,"MainField_Obj_LiftRockGerudo_A_01_1035259983",
0x752ef8ea,"MainField_Obj_LiftRockGerudo_A_01_1045615450",
0x03d253ce,"MainField_Obj_LiftRockGerudo_A_01_111863822",
0x820e45be,"MainField_Obj_LiftRockGerudo_A_01_1127209165",
0xb55a0fb8,"MainField_Obj_LiftRockGerudo_A_01_1200516094",
0x87eb4e11,"MainField_Obj_LiftRockGerudo_A_01_1214750225",
0x8e4851e0,"MainField_Obj_LiftRockGerudo_A_01_1306404032",
0xd201d893,"MainField_Obj_LiftRockGerudo_A_01_1312303061",
0x2955d706,"MainField_Obj_LiftRockGerudo_A_01_1335828203",
0xe08b3f55,"MainField_Obj_LiftRockGerudo_A_01_1403572632",
0x2e44bd92,"MainField_Obj_LiftRockGerudo_A_01_1431436999",
0xb541e790,"MainField_Obj_LiftRockGerudo_A_01_1443982153",
0x78ec20f9,"MainField_Obj_LiftRockGerudo_A_01_1461917332",
0x698712b4,"MainField_Obj_LiftRockGerudo_A_01_1586846399",
0x8ce68ef9,"MainField_Obj_LiftRockGerudo_A_01_1592741310",
0x159ca5b1,"MainField_Obj_LiftRockGerudo_A_01_162934740",
0x1da0639f,"MainField_Obj_LiftRockGerudo_A_01_171640860",
0x168e41f4,"MainField_Obj_LiftRockGerudo_A_01_1737816463",
0x88e34dea,"MainField_Obj_LiftRockGerudo_A_01_1766814141",
0x23a39a60,"MainField_Obj_LiftRockGerudo_A_01_1823286885",
0xeeedfb59,"MainField_Obj_LiftRockGerudo_A_01_184620872",
0x60bc26e7,"MainField_Obj_LiftRockGerudo_A_01_1847768484",
0x3b8b4147,"MainField_Obj_LiftRockGerudo_A_01_193711321",
0x8947cfbf,"MainField_Obj_LiftRockGerudo_A_01_2028805384",
0x4e2f5a38,"MainField_Obj_LiftRockGerudo_A_01_2052193427",
0xbd61aa1b,"MainField_Obj_LiftRockGerudo_A_01_2080778206",
0x0149ac52,"MainField_Obj_LiftRockGerudo_A_01_2095161331",
0xbf97bab3,"MainField_Obj_LiftRockGerudo_A_01_2101098634",
0x0a76500b,"MainField_Obj_LiftRockGerudo_A_01_2128026526",
0x334a85d2,"MainField_Obj_LiftRockGerudo_A_01_2235335326",
0x893801f8,"MainField_Obj_LiftRockGerudo_A_01_223792133",
0xbd84f117,"MainField_Obj_LiftRockGerudo_A_01_2269188241",
0x8f0deaa2,"MainField_Obj_LiftRockGerudo_A_01_2275368698",
0x6c9844f5,"MainField_Obj_LiftRockGerudo_A_01_2314199071",
0xf4bfdd24,"MainField_Obj_LiftRockGerudo_A_01_2357113770",
0x2917ac82,"MainField_Obj_LiftRockGerudo_A_01_2430171826",
0x011b2dca,"MainField_Obj_LiftRockGerudo_A_01_2489294271",
0xe8cbe910,"MainField_Obj_LiftRockGerudo_A_01_2498099004",
0x6d5979aa,"MainField_Obj_LiftRockGerudo_A_01_2502057708",
0x3d5c653d,"MainField_Obj_LiftRockGerudo_A_01_2571279733",
0x769967ba,"MainField_Obj_LiftRockGerudo_A_01_2592347533",
0xfbbcfcd1,"MainField_Obj_LiftRockGerudo_A_01_2687911673",
0xbc253f19,"MainField_Obj_LiftRockGerudo_A_01_2693846779",
0x84808dff,"MainField_Obj_LiftRockGerudo_A_01_2761422282",
0x4b496e23,"MainField_Obj_LiftRockGerudo_A_01_2763882997",
0x351ce50c,"MainField_Obj_LiftRockGerudo_A_01_278120729",
0x4d2c86fa,"MainField_Obj_LiftRockGerudo_A_01_2844289115",
0x459155db,"MainField_Obj_LiftRockGerudo_A_01_2852008051",
0x51be1319,"MainField_Obj_LiftRockGerudo_A_01_2871179027",
0x6956ec30,"MainField_Obj_LiftRockGerudo_A_01_2928931946",
0x4ea829ad,"MainField_Obj_LiftRockGerudo_A_01_2941867190",
0xdf9b1f68,"MainField_Obj_LiftRockGerudo_A_01_3021012127",
0x505ec299,"MainField_Obj_LiftRockGerudo_A_01_3071362983",
0x95dc82d8,"MainField_Obj_LiftRockGerudo_A_01_3133895853",
0xb5101e0a,"MainField_Obj_LiftRockGerudo_A_01_3181818036",
0x440c1032,"MainField_Obj_LiftRockGerudo_A_01_3191911329",
0x6888fd6a,"MainField_Obj_LiftRockGerudo_A_01_3222697777",
0x7cef376b,"MainField_Obj_LiftRockGerudo_A_01_3273116681",
0x429a5ede,"MainField_Obj_LiftRockGerudo_A_01_3333322456",
0x4409f5e0,"MainField_Obj_LiftRockGerudo_A_01_3399581730",
0x7330dc74,"MainField_Obj_LiftRockGerudo_A_01_3452238471",
0x2068ecc6,"MainField_Obj_LiftRockGerudo_A_01_3452716091",
0x29b8ba94,"MainField_Obj_LiftRockGerudo_A_01_3456387558",
0xccde9fea,"MainField_Obj_LiftRockGerudo_A_01_3458877230",
0x9dda64d4,"MainField_Obj_LiftRockGerudo_A_01_3506053748",
0x3da4f242,"MainField_Obj_LiftRockGerudo_A_01_3549480284",
0x6b5a585c,"MainField_Obj_LiftRockGerudo_A_01_3552227683",
0xa0dd64d8,"MainField_Obj_LiftRockGerudo_A_01_360178192",
0x741e5e20,"MainField_Obj_LiftRockGerudo_A_01_3610343023",
0xaee53c44,"MainField_Obj_LiftRockGerudo_A_01_3650405628",
0x8b35ed1d,"MainField_Obj_LiftRockGerudo_A_01_3700792764",
0x2f90d7df,"MainField_Obj_LiftRockGerudo_A_01_3733936333",
0x87562bb0,"MainField_Obj_LiftRockGerudo_A_01_3752220051",
0xe9a18e67,"MainField_Obj_LiftRockGerudo_A_01_3782312722",
0xb84c6585,"MainField_Obj_LiftRockGerudo_A_01_3814239529",
0x2886c8fe,"MainField_Obj_LiftRockGerudo_A_01_3828705584",
0x4dfa1019,"MainField_Obj_LiftRockGerudo_A_01_3842995812",
0xa05470b8,"MainField_Obj_LiftRockGerudo_A_01_3845749168",
0x061e8390,"MainField_Obj_LiftRockGerudo_A_01_3857239625",
0xb70bec41,"MainField_Obj_LiftRockGerudo_A_01_3890104868",
0x0a6313b1,"MainField_Obj_LiftRockGerudo_A_01_3993895635",
0x34439aae,"MainField_Obj_LiftRockGerudo_A_01_3996177759",
0x40edc67c,"MainField_Obj_LiftRockGerudo_A_01_4021384409",
0xa65e51a8,"MainField_Obj_LiftRockGerudo_A_01_4118961366",
0x91584623,"MainField_Obj_LiftRockGerudo_A_01_4198734952",
0xec07f0eb,"MainField_Obj_LiftRockGerudo_A_01_421889330",
0x2434d283,"MainField_Obj_LiftRockGerudo_A_01_464131827",
0xa36b9d9f,"MainField_Obj_LiftRockGerudo_A_01_513150784",
0xf9b51bd3,"MainField_Obj_LiftRockGerudo_A_01_513531292",
0xd13eb5c2,"MainField_Obj_LiftRockGerudo_A_01_539239938",
0x95d78f30,"MainField_Obj_LiftRockGerudo_A_01_615506190",
0xfab78464,"MainField_Obj_LiftRockGerudo_A_01_620560167",
0xe9f583a5,"MainField_Obj_LiftRockGerudo_A_01_645636038",
0xde5e1648,"MainField_Obj_LiftRockGerudo_A_01_665737782",
0x1e60ce28,"MainField_Obj_LiftRockGerudo_A_01_814308200",
0x91b3ff91,"MainField_Obj_LiftRockGerudo_A_01_849083536",
0x1a781564,"MainField_Obj_LiftRockGerudo_A_01_875096155",
0x949b53ed,"MainField_Obj_LiftRockGerudo_A_01_930296651",
0x5b5db75f,"MainField_Obj_LiftRockGerudo_A_01_987305046",
0x23414049,"MainField_Obj_LiftRockWhite_A_01_100079826",
0x1b8175cf,"MainField_Obj_LiftRockWhite_A_01_1002081754",
0x4eab9131,"MainField_Obj_LiftRockWhite_A_01_1006047328",
0xcd0f6b9a,"MainField_Obj_LiftRockWhite_A_01_1006072767",
0x23e249ed,"MainField_Obj_LiftRockWhite_A_01_1011596770",
0xcfc68ae3,"MainField_Obj_LiftRockWhite_A_01_1011608767",
0x3f2126ee,"MainField_Obj_LiftRockWhite_A_01_1013024693",
0x3061912c,"MainField_Obj_LiftRockWhite_A_01_1017765806",
0x09c714c7,"MainField_Obj_LiftRockWhite_A_01_1018053968",
0x6f682381,"MainField_Obj_LiftRockWhite_A_01_1020452623",
0xc48d5110,"MainField_Obj_LiftRockWhite_A_01_1031137513",
0x30baf806,"MainField_Obj_LiftRockWhite_A_01_103117383",
0x95f03820,"MainField_Obj_LiftRockWhite_A_01_1033201376",
0x1224ea33,"MainField_Obj_LiftRockWhite_A_01_103911466",
0xddbbff6f,"MainField_Obj_LiftRockWhite_A_01_1042641961",
0x66df7709,"MainField_Obj_LiftRockWhite_A_01_1048000123",
0x31f734cb,"MainField_Obj_LiftRockWhite_A_01_1053222995",
0x571540e4,"MainField_Obj_LiftRockWhite_A_01_1057805209",
0x716c6fec,"MainField_Obj_LiftRockWhite_A_01_1058766880",
0xa174fa69,"MainField_Obj_LiftRockWhite_A_01_1060375349",
0x519b8c96,"MainField_Obj_LiftRockWhite_A_01_10617204",
0xf2be8d14,"MainField_Obj_LiftRockWhite_A_01_1063300743",
0x7e6a22e7,"MainField_Obj_LiftRockWhite_A_01_1069676810",
0xc0a5b972,"MainField_Obj_LiftRockWhite_A_01_1070347864",
0x55fd004c,"MainField_Obj_LiftRockWhite_A_01_107285057",
0x24bdd8e2,"MainField_Obj_LiftRockWhite_A_01_1074847265",
0x9e118c2d,"MainField_Obj_LiftRockWhite_A_01_1079865334",
0xa0c0eedb,"MainField_Obj_LiftRockWhite_A_01_1084599022",
0xbaf86f1a,"MainField_Obj_LiftRockWhite_A_01_1089286890",
0xcc2e53dd,"MainField_Obj_LiftRockWhite_A_01_1090952857",
0x1e4a017b,"MainField_Obj_LiftRockWhite_A_01_1091104103",
0x51cddead,"MainField_Obj_LiftRockWhite_A_01_109186538",
0xed34af08,"MainField_Obj_LiftRockWhite_A_01_1093831427",
0x8e14c52a,"MainField_Obj_LiftRockWhite_A_01_1098730213",
0xfcb51d55,"MainField_Obj_LiftRockWhite_A_01_1100224865",
0xc6857241,"MainField_Obj_LiftRockWhite_A_01_1100955701",
0x76ce2ab7,"MainField_Obj_LiftRockWhite_A_01_1101050458",
0xf542a554,"MainField_Obj_LiftRockWhite_A_01_1106825431",
0xe6389f16,"MainField_Obj_LiftRockWhite_A_01_1112121226",
0xdcb212a2,"MainField_Obj_LiftRockWhite_A_01_1112571866",
0x7f792f10,"MainField_Obj_LiftRockWhite_A_01_1123717122",
0x97236591,"MainField_Obj_LiftRockWhite_A_01_1127314093",
0xabc16c9e,"MainField_Obj_LiftRockWhite_A_01_1127592733",
0x477a0ec3,"MainField_Obj_LiftRockWhite_A_01_1130945178",
0xf1fa64e9,"MainField_Obj_LiftRockWhite_A_01_1132978790",
0x06a7b795,"MainField_Obj_LiftRockWhite_A_01_1133066509",
0xae4583f7,"MainField_Obj_LiftRockWhite_A_01_1133104575",
0x9ddc7b40,"MainField_Obj_LiftRockWhite_A_01_1135188445",
0x30f2fdff,"MainField_Obj_LiftRockWhite_A_01_1150314186",
0xce3c76de,"MainField_Obj_LiftRockWhite_A_01_1150997131",
0x6e1bb2e3,"MainField_Obj_LiftRockWhite_A_01_1160010643",
0xc6ecc370,"MainField_Obj_LiftRockWhite_A_01_1161309479",
0xe8fc5a55,"MainField_Obj_LiftRockWhite_A_01_1163819222",
0x8c7fcd21,"MainField_Obj_LiftRockWhite_A_01_1165538899",
0x2b847d26,"MainField_Obj_LiftRockWhite_A_01_1172031830",
0x0dfb263c,"MainField_Obj_LiftRockWhite_A_01_1173508063",
0xaed768dd,"MainField_Obj_LiftRockWhite_A_01_1174988406",
0x0b6c9f14,"MainField_Obj_LiftRockWhite_A_01_1178072478",
0x927f1f50,"MainField_Obj_LiftRockWhite_A_01_1181895535",
0x814f25cf,"MainField_Obj_LiftRockWhite_A_01_1186807269",
0x1bc8f4c3,"MainField_Obj_LiftRockWhite_A_01_1188226713",
0xc517801c,"MainField_Obj_LiftRockWhite_A_01_1188394370",
0x959b44ec,"MainField_Obj_LiftRockWhite_A_01_1190780280",
0x78f8a4b7,"MainField_Obj_LiftRockWhite_A_01_119208656",
0x0f92815e,"MainField_Obj_LiftRockWhite_A_01_1197927254",
0x4deb4e6a,"MainField_Obj_LiftRockWhite_A_01_1198445764",
0x3621b6b1,"MainField_Obj_LiftRockWhite_A_01_1199634163",
0xe8aa63a5,"MainField_Obj_LiftRockWhite_A_01_1210649630",
0xa1b46ddb,"MainField_Obj_LiftRockWhite_A_01_1220987292",
0x61263837,"MainField_Obj_LiftRockWhite_A_01_1222848392",
0xf149efc5,"MainField_Obj_LiftRockWhite_A_01_1226452150",
0xeb76a1be,"MainField_Obj_LiftRockWhite_A_01_1232078573",
0xb38f1352,"MainField_Obj_LiftRockWhite_A_01_123983143",
0x470f7c79,"MainField_Obj_LiftRockWhite_A_01_1244008999",
0x5f9167b0,"MainField_Obj_LiftRockWhite_A_01_1248639487",
0xb8cc1d11,"MainField_Obj_LiftRockWhite_A_01_1251359579",
0xc58d9902,"MainField_Obj_LiftRockWhite_A_01_1272723901",
0x5f3ef313,"MainField_Obj_LiftRockWhite_A_01_1275228591",
0x5d7463e5,"MainField_Obj_LiftRockWhite_A_01_1276670159",
0x368958fa,"MainField_Obj_LiftRockWhite_A_01_1277237296",
0xba620ea0,"MainField_Obj_LiftRockWhite_A_01_1279468476",
0x91af032a,"MainField_Obj_LiftRockWhite_A_01_1280585178",
0x2daaa064,"MainField_Obj_LiftRockWhite_A_01_1285348097",
0xa173d8f3,"MainField_Obj_LiftRockWhite_A_01_128705788",
0x87dcd375,"MainField_Obj_LiftRockWhite_A_01_1288860487",
0x4015f8e5,"MainField_Obj_LiftRockWhite_A_01_1290805494",
0x0a5bc872,"MainField_Obj_LiftRockWhite_A_01_1290814249",
0x7df35877,"MainField_Obj_LiftRockWhite_A_01_1292628584",
0xcb8cb7a4,"MainField_Obj_LiftRockWhite_A_01_1295699954",
0x37654dc9,"MainField_Obj_LiftRockWhite_A_01_12984549",
0x24fde36b,"MainField_Obj_LiftRockWhite_A_01_130147579",
0xdcefa138,"MainField_Obj_LiftRockWhite_A_01_1303836869",
0xeafd051b,"MainField_Obj_LiftRockWhite_A_01_1305157848",
0x48c5f9c5,"MainField_Obj_LiftRockWhite_A_01_1307848445",
0xc1b42294,"MainField_Obj_LiftRockWhite_A_01_130794006",
0x77f4759e,"MainField_Obj_LiftRockWhite_A_01_1310448039",
0x5e071062,"MainField_Obj_LiftRockWhite_A_01_1320015935",
0x8c2bfb42,"MainField_Obj_LiftRockWhite_A_01_1320969168",
0x5d581dd5,"MainField_Obj_LiftRockWhite_A_01_1323489766",
0x347aa5ae,"MainField_Obj_LiftRockWhite_A_01_1326088827",
0xdd25f4bc,"MainField_Obj_LiftRockWhite_A_01_1326644949",
0x8718a2a2,"MainField_Obj_LiftRockWhite_A_01_1328007734",
0x4ae7e6e1,"MainField_Obj_LiftRockWhite_A_01_1329907719",
0xe165a57c,"MainField_Obj_LiftRockWhite_A_01_1329914902",
0x0d9de4df,"MainField_Obj_LiftRockWhite_A_01_1333408599",
0xbc457b0a,"MainField_Obj_LiftRockWhite_A_01_1344982928",
0xabe2dc18,"MainField_Obj_LiftRockWhite_A_01_1348525125",
0xcd02083b,"MainField_Obj_LiftRockWhite_A_01_1349203884",
0x116ddf23,"MainField_Obj_LiftRockWhite_A_01_134984380",
0x5bee43e8,"MainField_Obj_LiftRockWhite_A_01_1354689596",
0x34906ae3,"MainField_Obj_LiftRockWhite_A_01_1360088021",
0x1a297ff2,"MainField_Obj_LiftRockWhite_A_01_1366601052",
0x7a7369ed,"MainField_Obj_LiftRockWhite_A_01_1366693539",
0xc133ae8f,"MainField_Obj_LiftRockWhite_A_01_1366974671",
0x34e766d7,"MainField_Obj_LiftRockWhite_A_01_1367613542",
0x09afb512,"MainField_Obj_LiftRockWhite_A_01_136796106",
0x5ac51c0f,"MainField_Obj_LiftRockWhite_A_01_1370513844",
0x9d753f03,"MainField_Obj_LiftRockWhite_A_01_1371509735",
0xdf836320,"MainField_Obj_LiftRockWhite_A_01_1374252297",
0x3e1ea86e,"MainField_Obj_LiftRockWhite_A_01_1377169042",
0x1d8ffdda,"MainField_Obj_LiftRockWhite_A_01_1381494120",
0x05dde225,"MainField_Obj_LiftRockWhite_A_01_13830281",
0x5b380b93,"MainField_Obj_LiftRockWhite_A_01_1389855297",
0x8bb28fd7,"MainField_Obj_LiftRockWhite_A_01_13921150",
0x57c0e0fb,"MainField_Obj_LiftRockWhite_A_01_1396740113",
0xb5df42e4,"MainField_Obj_LiftRockWhite_A_01_1401341816",
0x46572c2c,"MainField_Obj_LiftRockWhite_A_01_1405465518",
0x957b91f4,"MainField_Obj_LiftRockWhite_A_01_1409710359",
0xdf4c0734,"MainField_Obj_LiftRockWhite_A_01_1420016372",
0x75fb6a41,"MainField_Obj_LiftRockWhite_A_01_1426851406",
0x7b68b290,"MainField_Obj_LiftRockWhite_A_01_1429875057",
0x5f1b224e,"MainField_Obj_LiftRockWhite_A_01_1430227939",
0xbbbb2350,"MainField_Obj_LiftRockWhite_A_01_1440085249",
0xd63df4f2,"MainField_Obj_LiftRockWhite_A_01_1441468855",
0x81a41266,"MainField_Obj_LiftRockWhite_A_01_1443327205",
0x2f67284b,"MainField_Obj_LiftRockWhite_A_01_1447641390",
0x1a71b48b,"MainField_Obj_LiftRockWhite_A_01_1455289238",
0x949c4ef8,"MainField_Obj_LiftRockWhite_A_01_1457366445",
0x03c9e9b1,"MainField_Obj_LiftRockWhite_A_01_1457604512",
0x0d71d55d,"MainField_Obj_LiftRockWhite_A_01_1460073397",
0xf38bfc7e,"MainField_Obj_LiftRockWhite_A_01_1460535928",
0x42911a6a,"MainField_Obj_LiftRockWhite_A_01_1463227728",
0x8002a1d8,"MainField_Obj_LiftRockWhite_A_01_1464273801",
0x904e9d08,"MainField_Obj_LiftRockWhite_A_01_1464865998",
0xbf840df4,"MainField_Obj_LiftRockWhite_A_01_1467256775",
0x48aebeae,"MainField_Obj_LiftRockWhite_A_01_1467318618",
0x36880ba1,"MainField_Obj_LiftRockWhite_A_01_146890804",
0xea4343df,"MainField_Obj_LiftRockWhite_A_01_1471795578",
0xd94b6d15,"MainField_Obj_LiftRockWhite_A_01_1476105048",
0xc89f2d6c,"MainField_Obj_LiftRockWhite_A_01_1476491199",
0xad044221,"MainField_Obj_LiftRockWhite_A_01_1479829916",
0x7e7be7b8,"MainField_Obj_LiftRockWhite_A_01_148027641",
0x46a487e9,"MainField_Obj_LiftRockWhite_A_01_1480830110",
0x06378af3,"MainField_Obj_LiftRockWhite_A_01_1482787926",
0xa0950fb8,"MainField_Obj_LiftRockWhite_A_01_1483453686",
0x50a3cd71,"MainField_Obj_LiftRockWhite_A_01_14835310",
0x7b1c0b38,"MainField_Obj_LiftRockWhite_A_01_148850077",
0x90550769,"MainField_Obj_LiftRockWhite_A_01_1492630499",
0xae7cea0d,"MainField_Obj_LiftRockWhite_A_01_150587261",
0x14066adc,"MainField_Obj_LiftRockWhite_A_01_1507687303",
0x27305930,"MainField_Obj_LiftRockWhite_A_01_1514085330",
0xd59748ab,"MainField_Obj_LiftRockWhite_A_01_1517156280",
0x26bc23be,"MainField_Obj_LiftRockWhite_A_01_152047223",
0xc5d2a551,"MainField_Obj_LiftRockWhite_A_01_152062500",
0x69abbcd1,"MainField_Obj_LiftRockWhite_A_01_1523557961",
0xbb6c38fb,"MainField_Obj_LiftRockWhite_A_01_1531311491",
0xdd24fcda,"MainField_Obj_LiftRockWhite_A_01_1532432846",
0xd6c86a48,"MainField_Obj_LiftRockWhite_A_01_1540871713",
0x380f10fb,"MainField_Obj_LiftRockWhite_A_01_1541513539",
0xde8b7198,"MainField_Obj_LiftRockWhite_A_01_1542181081",
0xd62f6a5a,"MainField_Obj_LiftRockWhite_A_01_1543472077",
0xe2afc9ea,"MainField_Obj_LiftRockWhite_A_01_1545114709",
0xcc4c7ea7,"MainField_Obj_LiftRockWhite_A_01_1546009419",
0xa28d2295,"MainField_Obj_LiftRockWhite_A_01_1550043292",
0xb8510587,"MainField_Obj_LiftRockWhite_A_01_1552687369",
0xc081ca45,"MainField_Obj_LiftRockWhite_A_01_1552692231",
0x3375b7af,"MainField_Obj_LiftRockWhite_A_01_1556958855",
0xb4051ec3,"MainField_Obj_LiftRockWhite_A_01_1560204113",
0x912373e3,"MainField_Obj_LiftRockWhite_A_01_1562485615",
0x92b99e00,"MainField_Obj_LiftRockWhite_A_01_1562918134",
0x26c5e835,"MainField_Obj_LiftRockWhite_A_01_1564689989",
0x64eab021,"MainField_Obj_LiftRockWhite_A_01_1566176027",
0x057a0d33,"MainField_Obj_LiftRockWhite_A_01_1567303927",
0x4faaf272,"MainField_Obj_LiftRockWhite_A_01_1569413003",
0x26de04e0,"MainField_Obj_LiftRockWhite_A_01_1571809938",
0x8d959d64,"MainField_Obj_LiftRockWhite_A_01_1575256929",
0xdc4c0c84,"MainField_Obj_LiftRockWhite_A_01_1580942874",
0x7c16cbba,"MainField_Obj_LiftRockWhite_A_01_1583063828",
0xf8329cdb,"MainField_Obj_LiftRockWhite_A_01_1583754105",
0xee71b5cc,"MainField_Obj_LiftRockWhite_A_01_1584349112",
0x97a50523,"MainField_Obj_LiftRockWhite_A_01_1587267506",
0x50a5ac4b,"MainField_Obj_LiftRockWhite_A_01_1594839435",
0x9afddac6,"MainField_Obj_LiftRockWhite_A_01_1595630405",
0x27e9fb3f,"MainField_Obj_LiftRockWhite_A_01_1596636977",
0xcb7e548f,"MainField_Obj_LiftRockWhite_A_01_1606785937",
0xf188e166,"MainField_Obj_LiftRockWhite_A_01_1608707129",
0xe107732e,"MainField_Obj_LiftRockWhite_A_01_1611187153",
0x4554c34c,"MainField_Obj_LiftRockWhite_A_01_1612319190",
0x53ba0e44,"MainField_Obj_LiftRockWhite_A_01_1618137037",
0x6157d8cb,"MainField_Obj_LiftRockWhite_A_01_1621770665",
0xf25f9bdc,"MainField_Obj_LiftRockWhite_A_01_1622805616",
0x1e348e56,"MainField_Obj_LiftRockWhite_A_01_163106328",
0x4ce72210,"MainField_Obj_LiftRockWhite_A_01_1631774270",
0xa9e7cd31,"MainField_Obj_LiftRockWhite_A_01_1635884961",
0xce570298,"MainField_Obj_LiftRockWhite_A_01_1638522543",
0x7c0cad51,"MainField_Obj_LiftRockWhite_A_01_1638966778",
0xbeb46665,"MainField_Obj_LiftRockWhite_A_01_1640852662",
0xce8503d8,"MainField_Obj_LiftRockWhite_A_01_1642201738",
0xf7bb7856,"MainField_Obj_LiftRockWhite_A_01_1647325301",
0x1b636c15,"MainField_Obj_LiftRockWhite_A_01_1649792466",
0xd2c27de2,"MainField_Obj_LiftRockWhite_A_01_1650992141",
0x18b97653,"MainField_Obj_LiftRockWhite_A_01_1652698494",
0x08a010b2,"MainField_Obj_LiftRockWhite_A_01_1656666187",
0x1ae8ce3a,"MainField_Obj_LiftRockWhite_A_01_1667169230",
0xc6b9dfe7,"MainField_Obj_LiftRockWhite_A_01_1676714351",
0x864473bb,"MainField_Obj_LiftRockWhite_A_01_1681682733",
0x9dbee26c,"MainField_Obj_LiftRockWhite_A_01_1684969419",
0x10b02323,"MainField_Obj_LiftRockWhite_A_01_1689733345",
0x839c0289,"MainField_Obj_LiftRockWhite_A_01_1696275496",
0xe968626d,"MainField_Obj_LiftRockWhite_A_01_1697681719",
0xc2f2dcbe,"MainField_Obj_LiftRockWhite_A_01_1699545822",
0x4a0f6709,"MainField_Obj_LiftRockWhite_A_01_1707623730",
0x665a3e2c,"MainField_Obj_LiftRockWhite_A_01_1714364967",
0x7e6556ad,"MainField_Obj_LiftRockWhite_A_01_1723586160",
0x58884554,"MainField_Obj_LiftRockWhite_A_01_1726334634",
0x9bd2cf8e,"MainField_Obj_LiftRockWhite_A_01_17282399",
0x8319fc50,"MainField_Obj_LiftRockWhite_A_01_1729683081",
0xbf510221,"MainField_Obj_LiftRockWhite_A_01_1735187614",
0x204924f9,"MainField_Obj_LiftRockWhite_A_01_1735359594",
0xb498e75b,"MainField_Obj_LiftRockWhite_A_01_1735606217",
0x65348247,"MainField_Obj_LiftRockWhite_A_01_1738952313",
0x0d83f8a5,"MainField_Obj_LiftRockWhite_A_01_1739688130",
0x9b8bf25e,"MainField_Obj_LiftRockWhite_A_01_1748241928",
0x815de7b9,"MainField_Obj_LiftRockWhite_A_01_1757216874",
0x7e7ac71b,"MainField_Obj_LiftRockWhite_A_01_1758205244",
0xd2c6135a,"MainField_Obj_LiftRockWhite_A_01_1758258516",
0xb6e32484,"MainField_Obj_LiftRockWhite_A_01_1760151777",
0x2fd6a144,"MainField_Obj_LiftRockWhite_A_01_1763446146",
0xf3e2d05e,"MainField_Obj_LiftRockWhite_A_01_176399835",
0xb618d696,"MainField_Obj_LiftRockWhite_A_01_1765285809",
0xc0d76aa6,"MainField_Obj_LiftRockWhite_A_01_1769127172",
0xfe6c0734,"MainField_Obj_LiftRockWhite_A_01_1771324192",
0xd80e9902,"MainField_Obj_LiftRockWhite_A_01_1775748033",
0xa25fd178,"MainField_Obj_LiftRockWhite_A_01_1775870285",
0x5509d99f,"MainField_Obj_LiftRockWhite_A_01_1776983065",
0x79228178,"MainField_Obj_LiftRockWhite_A_01_177889169",
0x63f61cd4,"MainField_Obj_LiftRockWhite_A_01_1779975621",
0xb291ef8e,"MainField_Obj_LiftRockWhite_A_01_1780026465",
0xd196ae23,"MainField_Obj_LiftRockWhite_A_01_1781626679",
0xf16b59be,"MainField_Obj_LiftRockWhite_A_01_1787339121",
0x54b179dc,"MainField_Obj_LiftRockWhite_A_01_1800710137",
0x61cec50a,"MainField_Obj_LiftRockWhite_A_01_1805218221",
0xd9f5b224,"MainField_Obj_LiftRockWhite_A_01_1806986229",
0x28c3efb7,"MainField_Obj_LiftRockWhite_A_01_1809737766",
0x25779624,"MainField_Obj_LiftRockWhite_A_01_1813623472",
0xd2160958,"MainField_Obj_LiftRockWhite_A_01_1818608243",
0x14297ab3,"MainField_Obj_LiftRockWhite_A_01_1823319344",
0x80389394,"MainField_Obj_LiftRockWhite_A_01_1828351332",
0x3f820b1d,"MainField_Obj_LiftRockWhite_A_01_1829519492",
0xdffa867c,"MainField_Obj_LiftRockWhite_A_01_1835183534",
0xcde43f38,"MainField_Obj_LiftRockWhite_A_01_1835242788",
0x6a188db7,"MainField_Obj_LiftRockWhite_A_01_1837659946",
0x09ad474f,"MainField_Obj_LiftRockWhite_A_01_1838581699",
0x1420be6e,"MainField_Obj_LiftRockWhite_A_01_1844757302",
0x32ccb359,"MainField_Obj_LiftRockWhite_A_01_1848803262",
0xadefdd9e,"MainField_Obj_LiftRockWhite_A_01_1852950939",
0xca83fdd9,"MainField_Obj_LiftRockWhite_A_01_1855353270",
0x52ba320b,"MainField_Obj_LiftRockWhite_A_01_185535784",
0x1a9c1894,"MainField_Obj_LiftRockWhite_A_01_1857316180",
0xc1aa38a8,"MainField_Obj_LiftRockWhite_A_01_1859543148",
0xb6b9cc2a,"MainField_Obj_LiftRockWhite_A_01_1861546903",
0xe6839a45,"MainField_Obj_LiftRockWhite_A_01_186271634",
0x4ea465f6,"MainField_Obj_LiftRockWhite_A_01_1864973692",
0xd68d0b19,"MainField_Obj_LiftRockWhite_A_01_1866773651",
0xa4695d84,"MainField_Obj_LiftRockWhite_A_01_1866876781",
0x549ce986,"MainField_Obj_LiftRockWhite_A_01_1871501190",
0x71f7691c,"MainField_Obj_LiftRockWhite_A_01_1876934996",
0xa4c6a71a,"MainField_Obj_LiftRockWhite_A_01_1881000518",
0x3b7f91b1,"MainField_Obj_LiftRockWhite_A_01_188466794",
0x17223990,"MainField_Obj_LiftRockWhite_A_01_1886327500",
0xd9dd3e6e,"MainField_Obj_LiftRockWhite_A_01_1890374762",
0xe8c3ffcb,"MainField_Obj_LiftRockWhite_A_01_1892479616",
0xf94fa034,"MainField_Obj_LiftRockWhite_A_01_1901743305",
0x2f142f5a,"MainField_Obj_LiftRockWhite_A_01_1907553337",
0x0c62e6a9,"MainField_Obj_LiftRockWhite_A_01_1908275161",
0x82e000d3,"MainField_Obj_LiftRockWhite_A_01_1912655561",
0x4e4ba5ae,"MainField_Obj_LiftRockWhite_A_01_1924301341",
0x10f268f9,"MainField_Obj_LiftRockWhite_A_01_1927641422",
0xcac217d1,"MainField_Obj_LiftRockWhite_A_01_1928341572",
0xfc83ca54,"MainField_Obj_LiftRockWhite_A_01_1932967259",
0x94efebc7,"MainField_Obj_LiftRockWhite_A_01_1933616828",
0xfcdfd9da,"MainField_Obj_LiftRockWhite_A_01_1933907169",
0xb1d0d91a,"MainField_Obj_LiftRockWhite_A_01_1936845276",
0xc68a8356,"MainField_Obj_LiftRockWhite_A_01_1946059378",
0xb4922e56,"MainField_Obj_LiftRockWhite_A_01_194764309",
0x695eb74f,"MainField_Obj_LiftRockWhite_A_01_1948697070",
0xb868d185,"MainField_Obj_LiftRockWhite_A_01_1948832326",
0x6ecef38d,"MainField_Obj_LiftRockWhite_A_01_1953824326",
0xd5236071,"MainField_Obj_LiftRockWhite_A_01_1955808875",
0xfc6ae477,"MainField_Obj_LiftRockWhite_A_01_1957697257",
0x3170b0c2,"MainField_Obj_LiftRockWhite_A_01_19577347",
0x98a9b0c2,"MainField_Obj_LiftRockWhite_A_01_1969261744",
0xa68b5fdd,"MainField_Obj_LiftRockWhite_A_01_1971798904",
0x8fd68626,"MainField_Obj_LiftRockWhite_A_01_1973213100",
0xa65ffe5c,"MainField_Obj_LiftRockWhite_A_01_1974117457",
0x004ba2b6,"MainField_Obj_LiftRockWhite_A_01_1977732106",
0xe63eb38b,"MainField_Obj_LiftRockWhite_A_01_1986366127",
0xef80cabb,"MainField_Obj_LiftRockWhite_A_01_1987274331",
0x675e65f2,"MainField_Obj_LiftRockWhite_A_01_1995655777",
0x6c86978d,"MainField_Obj_LiftRockWhite_A_01_1998386600",
0x4e5dca2e,"MainField_Obj_LiftRockWhite_A_01_2009141363",
0x11a55581,"MainField_Obj_LiftRockWhite_A_01_2010340328",
0xc1c84f26,"MainField_Obj_LiftRockWhite_A_01_2013265987",
0xaf553425,"MainField_Obj_LiftRockWhite_A_01_2018064963",
0xde58a5df,"MainField_Obj_LiftRockWhite_A_01_2018899530",
0x458af694,"MainField_Obj_LiftRockWhite_A_01_2024510651",
0xa40eb96b,"MainField_Obj_LiftRockWhite_A_01_2026238126",
0x041e2a8d,"MainField_Obj_LiftRockWhite_A_01_2027159211",
0x20493700,"MainField_Obj_LiftRockWhite_A_01_2030746612",
0x60a2917d,"MainField_Obj_LiftRockWhite_A_01_2030893442",
0xda38a208,"MainField_Obj_LiftRockWhite_A_01_2034150120",
0x807dbf33,"MainField_Obj_LiftRockWhite_A_01_203458165",
0x2f4e35fe,"MainField_Obj_LiftRockWhite_A_01_20348520",
0x977579b5,"MainField_Obj_LiftRockWhite_A_01_2037624919",
0xb542dcef,"MainField_Obj_LiftRockWhite_A_01_203770030",
0x2dd46daf,"MainField_Obj_LiftRockWhite_A_01_2045766022",
0xa566a332,"MainField_Obj_LiftRockWhite_A_01_2052139310",
0xc77e6c77,"MainField_Obj_LiftRockWhite_A_01_2057514071",
0x4754f6e4,"MainField_Obj_LiftRockWhite_A_01_2059339885",
0xecc2d381,"MainField_Obj_LiftRockWhite_A_01_2062592798",
0xdd86179f,"MainField_Obj_LiftRockWhite_A_01_2066200291",
0x9ffb101c,"MainField_Obj_LiftRockWhite_A_01_207748809",
0x06d88d5c,"MainField_Obj_LiftRockWhite_A_01_2079014285",
0x8f6223ce,"MainField_Obj_LiftRockWhite_A_01_2082882134",
0x7caaaa4b,"MainField_Obj_LiftRockWhite_A_01_2083233910",
0x7532d516,"MainField_Obj_LiftRockWhite_A_01_2083688761",
0x20d0de59,"MainField_Obj_LiftRockWhite_A_01_2087889400",
0xcda74aa8,"MainField_Obj_LiftRockWhite_A_01_2093961219",
0x9319739e,"MainField_Obj_LiftRockWhite_A_01_2097858071",
0x5172d9cd,"MainField_Obj_LiftRockWhite_A_01_2104122196",
0x1255a7ca,"MainField_Obj_LiftRockWhite_A_01_2107469575",
0x0a66feaa,"MainField_Obj_LiftRockWhite_A_01_2108269100",
0x0d8df94c,"MainField_Obj_LiftRockWhite_A_01_2110765453",
0xe5946a3e,"MainField_Obj_LiftRockWhite_A_01_2115467954",
0x9f640306,"MainField_Obj_LiftRockWhite_A_01_212182156",
0x8178e58f,"MainField_Obj_LiftRockWhite_A_01_2126561997",
0x78ce277a,"MainField_Obj_LiftRockWhite_A_01_2131383058",
0x802d1589,"MainField_Obj_LiftRockWhite_A_01_2131526186",
0x5995f30b,"MainField_Obj_LiftRockWhite_A_01_2131746824",
0x2a3ff45a,"MainField_Obj_LiftRockWhite_A_01_2131836708",
0xba971cc6,"MainField_Obj_LiftRockWhite_A_01_2139351573",
0x61442873,"MainField_Obj_LiftRockWhite_A_01_2139733689",
0xfca32cb8,"MainField_Obj_LiftRockWhite_A_01_2143399071",
0x9566f2f8,"MainField_Obj_LiftRockWhite_A_01_2146809040",
0x444b7503,"MainField_Obj_LiftRockWhite_A_01_2146862857",
0xb8d9516b,"MainField_Obj_LiftRockWhite_A_01_2147416043",
0x13057688,"MainField_Obj_LiftRockWhite_A_01_2147969404",
0x8781084a,"MainField_Obj_LiftRockWhite_A_01_2148115909",
0x1f2d53c5,"MainField_Obj_LiftRockWhite_A_01_2148603953",
0xd6b5922e,"MainField_Obj_LiftRockWhite_A_01_2150868066",
0x0560d2a8,"MainField_Obj_LiftRockWhite_A_01_2154669845",
0x9805b167,"MainField_Obj_LiftRockWhite_A_01_2155849911",
0x549272f3,"MainField_Obj_LiftRockWhite_A_01_2157449370",
0x50cd5089,"MainField_Obj_LiftRockWhite_A_01_2158266029",
0x70a388d0,"MainField_Obj_LiftRockWhite_A_01_2159674488",
0x00b9c06c,"MainField_Obj_LiftRockWhite_A_01_2165164873",
0x8188921d,"MainField_Obj_LiftRockWhite_A_01_2165867944",
0x989f418d,"MainField_Obj_LiftRockWhite_A_01_2174160672",
0x4f7aefb2,"MainField_Obj_LiftRockWhite_A_01_217434318",
0xa12cd1d5,"MainField_Obj_LiftRockWhite_A_01_2175935330",
0x7ac2d820,"MainField_Obj_LiftRockWhite_A_01_2175953887",
0x1e34ab7c,"MainField_Obj_LiftRockWhite_A_01_2178929013",
0x42b6ce21,"MainField_Obj_LiftRockWhite_A_01_2184724051",
0x9527ea13,"MainField_Obj_LiftRockWhite_A_01_2185535537",
0xa57711fe,"MainField_Obj_LiftRockWhite_A_01_2185668092",
0x7d8c96cc,"MainField_Obj_LiftRockWhite_A_01_2187157158",
0x0622d437,"MainField_Obj_LiftRockWhite_A_01_2191770652",
0x46f48853,"MainField_Obj_LiftRockWhite_A_01_2194392151",
0x5dea110a,"MainField_Obj_LiftRockWhite_A_01_2195578981",
0xb1797831,"MainField_Obj_LiftRockWhite_A_01_2196462616",
0xfdcbd80f,"MainField_Obj_LiftRockWhite_A_01_2197954984",
0x5a0050de,"MainField_Obj_LiftRockWhite_A_01_2198794011",
0x2ef23d9a,"MainField_Obj_LiftRockWhite_A_01_2204216875",
0x8c65c72d,"MainField_Obj_LiftRockWhite_A_01_2204945538",
0x202292a8,"MainField_Obj_LiftRockWhite_A_01_2205530422",
0xf0eb5951,"MainField_Obj_LiftRockWhite_A_01_2206847323",
0x4c9633b1,"MainField_Obj_LiftRockWhite_A_01_220878156",
0x1a417981,"MainField_Obj_LiftRockWhite_A_01_2211122690",
0xe6e53615,"MainField_Obj_LiftRockWhite_A_01_221151581",
0xc1c7ead7,"MainField_Obj_LiftRockWhite_A_01_2212290040",
0x9be6da4e,"MainField_Obj_LiftRockWhite_A_01_2215333115",
0xe7c94a17,"MainField_Obj_LiftRockWhite_A_01_2215538145",
0x6b7d1a89,"MainField_Obj_LiftRockWhite_A_01_2223289472",
0x73dc5429,"MainField_Obj_LiftRockWhite_A_01_2224413084",
0xc006b260,"MainField_Obj_LiftRockWhite_A_01_2226880213",
0x227eba98,"MainField_Obj_LiftRockWhite_A_01_2229563117",
0x1213ad5b,"MainField_Obj_LiftRockWhite_A_01_2232416224",
0x8d7d1ae4,"MainField_Obj_LiftRockWhite_A_01_223533147",
0xcdcdd6de,"MainField_Obj_LiftRockWhite_A_01_2235617598",
0x55be9e02,"MainField_Obj_LiftRockWhite_A_01_2238053146",
0x88808cd8,"MainField_Obj_LiftRockWhite_A_01_2242602168",
0x8c429e1d,"MainField_Obj_LiftRockWhite_A_01_2244870149",
0x39051d2e,"MainField_Obj_LiftRockWhite_A_01_2246387029",
0x44cc5c28,"MainField_Obj_LiftRockWhite_A_01_2248706301",
0x14cbd816,"MainField_Obj_LiftRockWhite_A_01_2253678701",
0x66902ca4,"MainField_Obj_LiftRockWhite_A_01_2255831473",
0x8497acf6,"MainField_Obj_LiftRockWhite_A_01_2257189220",
0xee8aa578,"MainField_Obj_LiftRockWhite_A_01_2261261921",
0x5d9270d3,"MainField_Obj_LiftRockWhite_A_01_2262080313",
0x001920c2,"MainField_Obj_LiftRockWhite_A_01_2266131630",
0x6b2d0ca2,"MainField_Obj_LiftRockWhite_A_01_2268495517",
0xa5d3ea88,"MainField_Obj_LiftRockWhite_A_01_2279063649",
0x49cfa532,"MainField_Obj_LiftRockWhite_A_01_2279931189",
0x321eae89,"MainField_Obj_LiftRockWhite_A_01_2281992009",
0x43c91104,"MainField_Obj_LiftRockWhite_A_01_2284675646",
0x29df7825,"MainField_Obj_LiftRockWhite_A_01_2287676588",
0x28d6e92e,"MainField_Obj_LiftRockWhite_A_01_2288784790",
0xff8d5e7b,"MainField_Obj_LiftRockWhite_A_01_2295699123",
0x68038ffa,"MainField_Obj_LiftRockWhite_A_01_2298620639",
0x99783ddd,"MainField_Obj_LiftRockWhite_A_01_2301516977",
0x441944a0,"MainField_Obj_LiftRockWhite_A_01_2302850735",
0x26d3eeee,"MainField_Obj_LiftRockWhite_A_01_2304801861",
0x991921b4,"MainField_Obj_LiftRockWhite_A_01_2321610471",
0x3ea5f7da,"MainField_Obj_LiftRockWhite_A_01_2325183011",
0x0d741b2b,"MainField_Obj_LiftRockWhite_A_01_2326048689",
0x098beaa0,"MainField_Obj_LiftRockWhite_A_01_2329655331",
0x1aa46823,"MainField_Obj_LiftRockWhite_A_01_2341372108",
0x399c5e6c,"MainField_Obj_LiftRockWhite_A_01_2343040573",
0xf392a186,"MainField_Obj_LiftRockWhite_A_01_234331016",
0xc374e47d,"MainField_Obj_LiftRockWhite_A_01_2344849654",
0xe29bd18b,"MainField_Obj_LiftRockWhite_A_01_2346813515",
0xfc30583b,"MainField_Obj_LiftRockWhite_A_01_2354820350",
0xe969df07,"MainField_Obj_LiftRockWhite_A_01_2355223350",
0x09107c00,"MainField_Obj_LiftRockWhite_A_01_2355512609",
0x6e720292,"MainField_Obj_LiftRockWhite_A_01_2356694062",
0x23762709,"MainField_Obj_LiftRockWhite_A_01_2361457786",
0x0acaafbc,"MainField_Obj_LiftRockWhite_A_01_2364612177",
0x336d013b,"MainField_Obj_LiftRockWhite_A_01_2366777055",
0xf1347448,"MainField_Obj_LiftRockWhite_A_01_2367979133",
0x41ce659a,"MainField_Obj_LiftRockWhite_A_01_2368973607",
0x753530d1,"MainField_Obj_LiftRockWhite_A_01_2370222670",
0xd40cd96d,"MainField_Obj_LiftRockWhite_A_01_2374907196",
0xaa177dc3,"MainField_Obj_LiftRockWhite_A_01_2375154590",
0x85f93163,"MainField_Obj_LiftRockWhite_A_01_2378104083",
0x5b0451db,"MainField_Obj_LiftRockWhite_A_01_2378641587",
0xc12d30ce,"MainField_Obj_LiftRockWhite_A_01_2387454827",
0x276e2cbd,"MainField_Obj_LiftRockWhite_A_01_2389700294",
0xe97a2627,"MainField_Obj_LiftRockWhite_A_01_2390535905",
0x26b2649d,"MainField_Obj_LiftRockWhite_A_01_2396435749",
0xe1fcfa24,"MainField_Obj_LiftRockWhite_A_01_2398683252",
0x09e81579,"MainField_Obj_LiftRockWhite_A_01_2398784730",
0xcd891338,"MainField_Obj_LiftRockWhite_A_01_2407949215",
0x38974ec0,"MainField_Obj_LiftRockWhite_A_01_2408820339",
0x65cdf0f3,"MainField_Obj_LiftRockWhite_A_01_2413146197",
0x450f725b,"MainField_Obj_LiftRockWhite_A_01_2417582029",
0x94f78690,"MainField_Obj_LiftRockWhite_A_01_2418391156",
0x0fe2ad90,"MainField_Obj_LiftRockWhite_A_01_242142746",
0x9ecd3bf8,"MainField_Obj_LiftRockWhite_A_01_2422961903",
0x4d410fad,"MainField_Obj_LiftRockWhite_A_01_2425109461",
0xb1149fed,"MainField_Obj_LiftRockWhite_A_01_243109437",
0xbccac215,"MainField_Obj_LiftRockWhite_A_01_2431577461",
0x8eab086d,"MainField_Obj_LiftRockWhite_A_01_2432724437",
0xd4a72d6e,"MainField_Obj_LiftRockWhite_A_01_2437371198",
0x2306568e,"MainField_Obj_LiftRockWhite_A_01_2445654976",
0x05912093,"MainField_Obj_LiftRockWhite_A_01_2445816206",
0xcbd7bd4a,"MainField_Obj_LiftRockWhite_A_01_2453560591",
0x5401bfc8,"MainField_Obj_LiftRockWhite_A_01_2456583984",
0x1c73a316,"MainField_Obj_LiftRockWhite_A_01_2459597836",
0xd4560978,"MainField_Obj_LiftRockWhite_A_01_2472200563",
0x2a229303,"MainField_Obj_LiftRockWhite_A_01_2472660538",
0xcc4da428,"MainField_Obj_LiftRockWhite_A_01_2472819320",
0x7185a86c,"MainField_Obj_LiftRockWhite_A_01_2474394221",
0xb5b28f0f,"MainField_Obj_LiftRockWhite_A_01_2475888171",
0x693ec37a,"MainField_Obj_LiftRockWhite_A_01_2476666089",
0x562a0570,"MainField_Obj_LiftRockWhite_A_01_2477625063",
0x3e1840f0,"MainField_Obj_LiftRockWhite_A_01_2481587550",
0xd9d265f4,"MainField_Obj_LiftRockWhite_A_01_2482486322",
0x137099c5,"MainField_Obj_LiftRockWhite_A_01_2483233930",
0x9873f9ef,"MainField_Obj_LiftRockWhite_A_01_2486204338",
0x5e03bd54,"MainField_Obj_LiftRockWhite_A_01_2488500782",
0xb506ea96,"MainField_Obj_LiftRockWhite_A_01_2488847710",
0x6021b8aa,"MainField_Obj_LiftRockWhite_A_01_2492811297",
0xd7bcaaac,"MainField_Obj_LiftRockWhite_A_01_2493926429",
0xfd87d34f,"MainField_Obj_LiftRockWhite_A_01_2494385531",
0x2449e076,"MainField_Obj_LiftRockWhite_A_01_2496251099",
0x4453606b,"MainField_Obj_LiftRockWhite_A_01_2498237679",
0x196aa95e,"MainField_Obj_LiftRockWhite_A_01_2498324142",
0x51af4ae0,"MainField_Obj_LiftRockWhite_A_01_2499725874",
0x0ad64d35,"MainField_Obj_LiftRockWhite_A_01_2510593139",
0xcb338017,"MainField_Obj_LiftRockWhite_A_01_2510606961",
0xb972ae40,"MainField_Obj_LiftRockWhite_A_01_2512778039",
0xb7d38a3d,"MainField_Obj_LiftRockWhite_A_01_2520981013",
0x5c8beb34,"MainField_Obj_LiftRockWhite_A_01_252129493",
0x2b621d5e,"MainField_Obj_LiftRockWhite_A_01_2522027532",
0xd0831347,"MainField_Obj_LiftRockWhite_A_01_2524092945",
0x7cb46aee,"MainField_Obj_LiftRockWhite_A_01_2527545537",
0xff8faba4,"MainField_Obj_LiftRockWhite_A_01_2530660316",
0x41f667dc,"MainField_Obj_LiftRockWhite_A_01_2536136098",
0xd103366d,"MainField_Obj_LiftRockWhite_A_01_2541019015",
0xe480f063,"MainField_Obj_LiftRockWhite_A_01_2543976692",
0x0dff440c,"MainField_Obj_LiftRockWhite_A_01_2545333822",
0x93a77094,"MainField_Obj_LiftRockWhite_A_01_2547272535",
0x13caa087,"MainField_Obj_LiftRockWhite_A_01_2550017086",
0x76914ee4,"MainField_Obj_LiftRockWhite_A_01_25514148",
0xe9206758,"MainField_Obj_LiftRockWhite_A_01_2552325165",
0xdc7ef8a4,"MainField_Obj_LiftRockWhite_A_01_2554483459",
0x27140ec2,"MainField_Obj_LiftRockWhite_A_01_2560117339",
0x354e5334,"MainField_Obj_LiftRockWhite_A_01_2562081871",
0x997eb3bb,"MainField_Obj_LiftRockWhite_A_01_2566728525",
0x90cdc17c,"MainField_Obj_LiftRockWhite_A_01_2577411956",
0x8ee03188,"MainField_Obj_LiftRockWhite_A_01_2579724538",
0x8f386663,"MainField_Obj_LiftRockWhite_A_01_2580509023",
0xea312e47,"MainField_Obj_LiftRockWhite_A_01_2583728005",
0x9c70dcb4,"MainField_Obj_LiftRockWhite_A_01_2587988946",
0x3ebb421d,"MainField_Obj_LiftRockWhite_A_01_2596122213",
0x5b8c9453,"MainField_Obj_LiftRockWhite_A_01_2600024793",
0xeb1c1a73,"MainField_Obj_LiftRockWhite_A_01_260561921",
0x2d84b730,"MainField_Obj_LiftRockWhite_A_01_2612589917",
0x0cbd5b9b,"MainField_Obj_LiftRockWhite_A_01_2613880184",
0xb4e50aed,"MainField_Obj_LiftRockWhite_A_01_2616946900",
0xee16bbff,"MainField_Obj_LiftRockWhite_A_01_2618997021",
0xa1075f45,"MainField_Obj_LiftRockWhite_A_01_2622440812",
0xad476ba0,"MainField_Obj_LiftRockWhite_A_01_2626249399",
0x18283fd6,"MainField_Obj_LiftRockWhite_A_01_262990440",
0x65cc2c8a,"MainField_Obj_LiftRockWhite_A_01_2631854700",
0xe40711db,"MainField_Obj_LiftRockWhite_A_01_2632535192",
0x1b070025,"MainField_Obj_LiftRockWhite_A_01_2637699145",
0x0eea8cce,"MainField_Obj_LiftRockWhite_A_01_263896920",
0xedffe4c2,"MainField_Obj_LiftRockWhite_A_01_2640193964",
0x16a5ba75,"MainField_Obj_LiftRockWhite_A_01_2640463658",
0xa945b8ad,"MainField_Obj_LiftRockWhite_A_01_264421432",
0xa7ab0abf,"MainField_Obj_LiftRockWhite_A_01_2644529044",
0x732ada56,"MainField_Obj_LiftRockWhite_A_01_2651918491",
0x1da78b0a,"MainField_Obj_LiftRockWhite_A_01_2655249869",
0x63d71505,"MainField_Obj_LiftRockWhite_A_01_2659634125",
0xdf4f884e,"MainField_Obj_LiftRockWhite_A_01_2662229182",
0xe5725248,"MainField_Obj_LiftRockWhite_A_01_2670403317",
0xd277a4a8,"MainField_Obj_LiftRockWhite_A_01_2670470141",
0x64184e1c,"MainField_Obj_LiftRockWhite_A_01_2673986246",
0x36df6dc9,"MainField_Obj_LiftRockWhite_A_01_2677071645",
0x5ce2058f,"MainField_Obj_LiftRockWhite_A_01_2678500094",
0xeb924316,"MainField_Obj_LiftRockWhite_A_01_2679936277",
0xe7164171,"MainField_Obj_LiftRockWhite_A_01_2682784157",
0xa8813a48,"MainField_Obj_LiftRockWhite_A_01_2684426223",
0x3b9dc430,"MainField_Obj_LiftRockWhite_A_01_2685641688",
0x4e9b3641,"MainField_Obj_LiftRockWhite_A_01_2686145045",
0x17aa6e26,"MainField_Obj_LiftRockWhite_A_01_2690935685",
0x29145b8d,"MainField_Obj_LiftRockWhite_A_01_269187941",
0x8ca27c1b,"MainField_Obj_LiftRockWhite_A_01_2692992419",
0x7bb60d39,"MainField_Obj_LiftRockWhite_A_01_2693162887",
0x684f508b,"MainField_Obj_LiftRockWhite_A_01_2693609800",
0x94f86e0d,"MainField_Obj_LiftRockWhite_A_01_269572346",
0xb4d99f1b,"MainField_Obj_LiftRockWhite_A_01_270023490",
0xcadb55da,"MainField_Obj_LiftRockWhite_A_01_2701452119",
0x70f2ef3d,"MainField_Obj_LiftRockWhite_A_01_2703862554",
0x167bf633,"MainField_Obj_LiftRockWhite_A_01_2706015546",
0x9de40f7e,"MainField_Obj_LiftRockWhite_A_01_2706024747",
0xd1679e6e,"MainField_Obj_LiftRockWhite_A_01_2712183803",
0x22a70f1c,"MainField_Obj_LiftRockWhite_A_01_2712473862",
0xad838ff5,"MainField_Obj_LiftRockWhite_A_01_2721020188",
0xe794ad80,"MainField_Obj_LiftRockWhite_A_01_2728630843",
0x5111c7c6,"MainField_Obj_LiftRockWhite_A_01_2729383477",
0xb324c960,"MainField_Obj_LiftRockWhite_A_01_2730258242",
0xf3ed9715,"MainField_Obj_LiftRockWhite_A_01_2734969223",
0xbe11a1ea,"MainField_Obj_LiftRockWhite_A_01_2737291247",
0x030b5e09,"MainField_Obj_LiftRockWhite_A_01_2748486567",
0xf18d4106,"MainField_Obj_LiftRockWhite_A_01_275493530",
0x4a5031a5,"MainField_Obj_LiftRockWhite_A_01_2761395144",
0x196c01c0,"MainField_Obj_LiftRockWhite_A_01_2767870737",
0x7ab3a1c8,"MainField_Obj_LiftRockWhite_A_01_2772614405",
0xaec40e32,"MainField_Obj_LiftRockWhite_A_01_2777697922",
0x04513793,"MainField_Obj_LiftRockWhite_A_01_2778389300",
0xa755b256,"MainField_Obj_LiftRockWhite_A_01_278269061",
0xec150e7f,"MainField_Obj_LiftRockWhite_A_01_2786631517",
0x1cfe09b7,"MainField_Obj_LiftRockWhite_A_01_2787314784",
0xe3006138,"MainField_Obj_LiftRockWhite_A_01_2787590435",
0x039713a1,"MainField_Obj_LiftRockWhite_A_01_2787914854",
0xbce83049,"MainField_Obj_LiftRockWhite_A_01_2791964477",
0x92af67f7,"MainField_Obj_LiftRockWhite_A_01_2796002966",
0xa94a48d3,"MainField_Obj_LiftRockWhite_A_01_2797655327",
0x7c6a1756,"MainField_Obj_LiftRockWhite_A_01_2797955042",
0xb57ab21f,"MainField_Obj_LiftRockWhite_A_01_2799630135",
0x09f7d135,"MainField_Obj_LiftRockWhite_A_01_2800899989",
0x67a4b967,"MainField_Obj_LiftRockWhite_A_01_280265897",
0x870bc59a,"MainField_Obj_LiftRockWhite_A_01_2803445388",
0x4cd57d41,"MainField_Obj_LiftRockWhite_A_01_280410196",
0x97dad91c,"MainField_Obj_LiftRockWhite_A_01_2805342971",
0x94226b14,"MainField_Obj_LiftRockWhite_A_01_2805494212",
0xb6b7ed69,"MainField_Obj_LiftRockWhite_A_01_2807373805",
0xcfa797a3,"MainField_Obj_LiftRockWhite_A_01_2812091608",
0x098268a6,"MainField_Obj_LiftRockWhite_A_01_2819000154",
0x5e748867,"MainField_Obj_LiftRockWhite_A_01_2820038433",
0xbe3256f7,"MainField_Obj_LiftRockWhite_A_01_2828916698",
0x76361929,"MainField_Obj_LiftRockWhite_A_01_2838462964",
0x13b208da,"MainField_Obj_LiftRockWhite_A_01_2838506412",
0x4cbe52ec,"MainField_Obj_LiftRockWhite_A_01_2839729519",
0x782a5359,"MainField_Obj_LiftRockWhite_A_01_2847719493",
0x72f924b3,"MainField_Obj_LiftRockWhite_A_01_2849662681",
0x54998428,"MainField_Obj_LiftRockWhite_A_01_2851410971",
0xff8cf8fa,"MainField_Obj_LiftRockWhite_A_01_2851737447",
0x0fcb50e3,"MainField_Obj_LiftRockWhite_A_01_2854075214",
0xe62d10dc,"MainField_Obj_LiftRockWhite_A_01_2856380560",
0x30d5007b,"MainField_Obj_LiftRockWhite_A_01_2857768488",
0xefe83208,"MainField_Obj_LiftRockWhite_A_01_2862483430",
0x099df73d,"MainField_Obj_LiftRockWhite_A_01_2864017333",
0x351bdef5,"MainField_Obj_LiftRockWhite_A_01_2865358759",
0xa047ff0b,"MainField_Obj_LiftRockWhite_A_01_2867488906",
0x21b4f69d,"MainField_Obj_LiftRockWhite_A_01_2869554197",
0x0aab9a32,"MainField_Obj_LiftRockWhite_A_01_2871671487",
0x7cdb03de,"MainField_Obj_LiftRockWhite_A_01_2874153466",
0xce2b4942,"MainField_Obj_LiftRockWhite_A_01_2875479402",
0x3912f322,"MainField_Obj_LiftRockWhite_A_01_287838897",
0x42b230bc,"MainField_Obj_LiftRockWhite_A_01_2882906234",
0x470dc8f4,"MainField_Obj_LiftRockWhite_A_01_2888598045",
0xea23923a,"MainField_Obj_LiftRockWhite_A_01_2896336960",
0xac9dcc99,"MainField_Obj_LiftRockWhite_A_01_2902575694",
0x0ce85aac,"MainField_Obj_LiftRockWhite_A_01_2904939326",
0x4f61b59b,"MainField_Obj_LiftRockWhite_A_01_2911279303",
0xe219a1e8,"MainField_Obj_LiftRockWhite_A_01_2915544048",
0x74e9f8ac,"MainField_Obj_LiftRockWhite_A_01_2916594420",
0xe3309f98,"MainField_Obj_LiftRockWhite_A_01_2922280658",
0x33db3101,"MainField_Obj_LiftRockWhite_A_01_2934133702",
0x004877de,"MainField_Obj_LiftRockWhite_A_01_2938190321",
0x13a294df,"MainField_Obj_LiftRockWhite_A_01_2938679142",
0x60b7310f,"MainField_Obj_LiftRockWhite_A_01_2946136141",
0xc6cc16d0,"MainField_Obj_LiftRockWhite_A_01_294913265",
0xf59b5ba0,"MainField_Obj_LiftRockWhite_A_01_2952054239",
0x7471db2b,"MainField_Obj_LiftRockWhite_A_01_2956087567",
0x27b7b31c,"MainField_Obj_LiftRockWhite_A_01_2957334622",
0x55df3753,"MainField_Obj_LiftRockWhite_A_01_2957738309",
0xea95fd36,"MainField_Obj_LiftRockWhite_A_01_295806415",
0x2dba087b,"MainField_Obj_LiftRockWhite_A_01_2963267185",
0x631d164b,"MainField_Obj_LiftRockWhite_A_01_2966269021",
0x135fdf99,"MainField_Obj_LiftRockWhite_A_01_2969786458",
0x55938dc8,"MainField_Obj_LiftRockWhite_A_01_2985069845",
0x52474944,"MainField_Obj_LiftRockWhite_A_01_2985115986",
0x90559a52,"MainField_Obj_LiftRockWhite_A_01_2992274360",
0xf48aa3d4,"MainField_Obj_LiftRockWhite_A_01_2993137012",
0x74769c16,"MainField_Obj_LiftRockWhite_A_01_299520373",
0x6fd029e2,"MainField_Obj_LiftRockWhite_A_01_300086844",
0xd92c8811,"MainField_Obj_LiftRockWhite_A_01_3009759707",
0x7b1470c5,"MainField_Obj_LiftRockWhite_A_01_3011851943",
0x5f568288,"MainField_Obj_LiftRockWhite_A_01_301515137",
0x0fa5c180,"MainField_Obj_LiftRockWhite_A_01_3016178440",
0x76b8f519,"MainField_Obj_LiftRockWhite_A_01_3018096977",
0x905b3a41,"MainField_Obj_LiftRockWhite_A_01_3023630055",
0xe576f8aa,"MainField_Obj_LiftRockWhite_A_01_3024022143",
0x6dafb904,"MainField_Obj_LiftRockWhite_A_01_3027702834",
0x9a6dac52,"MainField_Obj_LiftRockWhite_A_01_3035165374",
0x5b104514,"MainField_Obj_LiftRockWhite_A_01_3036315678",
0x80fd0ff5,"MainField_Obj_LiftRockWhite_A_01_3036369519",
0xb9e5d5a0,"MainField_Obj_LiftRockWhite_A_01_3037209366",
0x2c6bdac5,"MainField_Obj_LiftRockWhite_A_01_3038747707",
0xe20c1969,"MainField_Obj_LiftRockWhite_A_01_304120737",
0x8a33b6a7,"MainField_Obj_LiftRockWhite_A_01_3041629359",
0x0776b17b,"MainField_Obj_LiftRockWhite_A_01_3042876418",
0xf37f68f5,"MainField_Obj_LiftRockWhite_A_01_3050416321",
0x5893cae9,"MainField_Obj_LiftRockWhite_A_01_3058164256",
0x97519ae2,"MainField_Obj_LiftRockWhite_A_01_3058298345",
0x9cf94992,"MainField_Obj_LiftRockWhite_A_01_3064516407",
0x74cc2eaf,"MainField_Obj_LiftRockWhite_A_01_3075309627",
0xf2cf8ef1,"MainField_Obj_LiftRockWhite_A_01_307940316",
0xdd61aee3,"MainField_Obj_LiftRockWhite_A_01_3082510546",
0x6990c73c,"MainField_Obj_LiftRockWhite_A_01_3088891499",
0x1ef55bbe,"MainField_Obj_LiftRockWhite_A_01_3089876839",
0x0f05a5e8,"MainField_Obj_LiftRockWhite_A_01_3092249920",
0xc0e9a637,"MainField_Obj_LiftRockWhite_A_01_3096709372",
0xa76f78b5,"MainField_Obj_LiftRockWhite_A_01_3098776250",
0x9a2adb11,"MainField_Obj_LiftRockWhite_A_01_3100527829",
0xd7539fe2,"MainField_Obj_LiftRockWhite_A_01_3103795323",
0x4c96a7c6,"MainField_Obj_LiftRockWhite_A_01_3103810388",
0x1e5310a5,"MainField_Obj_LiftRockWhite_A_01_3104346777",
0x9013c469,"MainField_Obj_LiftRockWhite_A_01_3108596389",
0x1c802b6d,"MainField_Obj_LiftRockWhite_A_01_3110383149",
0xc8b092b5,"MainField_Obj_LiftRockWhite_A_01_3116034736",
0xe5b0bd60,"MainField_Obj_LiftRockWhite_A_01_3117879999",
0x6c70e2e1,"MainField_Obj_LiftRockWhite_A_01_3119903348",
0xcc628484,"MainField_Obj_LiftRockWhite_A_01_3125432036",
0x6f499503,"MainField_Obj_LiftRockWhite_A_01_3128741486",
0xe3c4998a,"MainField_Obj_LiftRockWhite_A_01_3133995330",
0x479e2a8f,"MainField_Obj_LiftRockWhite_A_01_3135739790",
0x1d1e2f16,"MainField_Obj_LiftRockWhite_A_01_3141904747",
0xde47a8ff,"MainField_Obj_LiftRockWhite_A_01_3149145972",
0x749e2697,"MainField_Obj_LiftRockWhite_A_01_3152375963",
0x9da6dd76,"MainField_Obj_LiftRockWhite_A_01_3157145358",
0x8df4aff3,"MainField_Obj_LiftRockWhite_A_01_3159005355",
0xa462bd85,"MainField_Obj_LiftRockWhite_A_01_3159985220",
0x21cf3da3,"MainField_Obj_LiftRockWhite_A_01_3168396141",
0x7aaca910,"MainField_Obj_LiftRockWhite_A_01_3170752644",
0x03c6bf84,"MainField_Obj_LiftRockWhite_A_01_3173574375",
0x32f307ab,"MainField_Obj_LiftRockWhite_A_01_3177641651",
0x42406d80,"MainField_Obj_LiftRockWhite_A_01_3192794796",
0xa284da93,"MainField_Obj_LiftRockWhite_A_01_3194312320",
0xa75c0c16,"MainField_Obj_LiftRockWhite_A_01_3197646445",
0x143312d5,"MainField_Obj_LiftRockWhite_A_01_3197661321",
0x872c14aa,"MainField_Obj_LiftRockWhite_A_01_3205903866",
0x9c94f158,"MainField_Obj_LiftRockWhite_A_01_3207085108",
0xd281d2a8,"MainField_Obj_LiftRockWhite_A_01_320878941",
0xc570c82f,"MainField_Obj_LiftRockWhite_A_01_3212911844",
0xb0c26b14,"MainField_Obj_LiftRockWhite_A_01_3216900248",
0xe385eaaa,"MainField_Obj_LiftRockWhite_A_01_3219019427",
0x19520a70,"MainField_Obj_LiftRockWhite_A_01_3220223861",
0xd82ea39b,"MainField_Obj_LiftRockWhite_A_01_322446064",
0x31bc2f24,"MainField_Obj_LiftRockWhite_A_01_3225850935",
0xe59bc9f2,"MainField_Obj_LiftRockWhite_A_01_3226120365",
0x0cf2d3e6,"MainField_Obj_LiftRockWhite_A_01_3233519943",
0x4aa906e7,"MainField_Obj_LiftRockWhite_A_01_3234242058",
0xa623de9b,"MainField_Obj_LiftRockWhite_A_01_3235176001",
0xde44b789,"MainField_Obj_LiftRockWhite_A_01_3239408932",
0xc22eca52,"MainField_Obj_LiftRockWhite_A_01_3239900623",
0xa2e5cd85,"MainField_Obj_LiftRockWhite_A_01_32514489",
0x963d962a,"MainField_Obj_LiftRockWhite_A_01_3254612528",
0x0dc21e8f,"MainField_Obj_LiftRockWhite_A_01_3256396973",
0x2c5fadc1,"MainField_Obj_LiftRockWhite_A_01_325691229",
0x354c0577,"MainField_Obj_LiftRockWhite_A_01_3260645885",
0xab010698,"MainField_Obj_LiftRockWhite_A_01_3260921973",
0xd59f376f,"MainField_Obj_LiftRockWhite_A_01_3261636560",
0x4a6fb50f,"MainField_Obj_LiftRockWhite_A_01_326756146",
0x105efd6f,"MainField_Obj_LiftRockWhite_A_01_3278667685",
0x53a7087a,"MainField_Obj_LiftRockWhite_A_01_327957637",
0x093137e8,"MainField_Obj_LiftRockWhite_A_01_3282853769",
0xcb7a7905,"MainField_Obj_LiftRockWhite_A_01_3287269928",
0x54b159ab,"MainField_Obj_LiftRockWhite_A_01_3291017941",
0x828875b5,"MainField_Obj_LiftRockWhite_A_01_3294787917",
0x4dfc093a,"MainField_Obj_LiftRockWhite_A_01_3296855601",
0x00f0276b,"MainField_Obj_LiftRockWhite_A_01_3302187733",
0xe10ea6b8,"MainField_Obj_LiftRockWhite_A_01_330434669",
0x915b6bd6,"MainField_Obj_LiftRockWhite_A_01_3308096284",
0xb5367aa6,"MainField_Obj_LiftRockWhite_A_01_3310165674",
0x1fba344d,"MainField_Obj_LiftRockWhite_A_01_3314301468",
0x711c6d0f,"MainField_Obj_LiftRockWhite_A_01_3322662745",
0xbf067f65,"MainField_Obj_LiftRockWhite_A_01_3322998532",
0x78a30c36,"MainField_Obj_LiftRockWhite_A_01_3323571018",
0x1f58df58,"MainField_Obj_LiftRockWhite_A_01_3324406064",
0x3c9b945d,"MainField_Obj_LiftRockWhite_A_01_332613663",
0xda4a65ee,"MainField_Obj_LiftRockWhite_A_01_3327229064",
0x29e1aecb,"MainField_Obj_LiftRockWhite_A_01_3329078467",
0xabc11f45,"MainField_Obj_LiftRockWhite_A_01_3330121611",
0x0ce2ab33,"MainField_Obj_LiftRockWhite_A_01_3335325939",
0xd578616e,"MainField_Obj_LiftRockWhite_A_01_3350177092",
0x6798dafc,"MainField_Obj_LiftRockWhite_A_01_3350958378",
0x566b1673,"MainField_Obj_LiftRockWhite_A_01_3351944395",
0x08c0af75,"MainField_Obj_LiftRockWhite_A_01_3354503966",
0x660b0d5b,"MainField_Obj_LiftRockWhite_A_01_3367579662",
0x0aad8b6f,"MainField_Obj_LiftRockWhite_A_01_3368075696",
0x0a309e82,"MainField_Obj_LiftRockWhite_A_01_3369380199",
0x1be9308d,"MainField_Obj_LiftRockWhite_A_01_3369690677",
0x80ab4daf,"MainField_Obj_LiftRockWhite_A_01_3375104126",
0xd4f02f0c,"MainField_Obj_LiftRockWhite_A_01_3375566724",
0x1a5e82df,"MainField_Obj_LiftRockWhite_A_01_3378669078",
0x07596c34,"MainField_Obj_LiftRockWhite_A_01_3380480728",
0x9c6b8505,"MainField_Obj_LiftRockWhite_A_01_3386050827",
0x1fe4ba15,"MainField_Obj_LiftRockWhite_A_01_3389169238",
0x9a7db79a,"MainField_Obj_LiftRockWhite_A_01_3389474696",
0x291d2b55,"MainField_Obj_LiftRockWhite_A_01_339284590",
0x2a26aee6,"MainField_Obj_LiftRockWhite_A_01_3398407874",
0x5c84f76f,"MainField_Obj_LiftRockWhite_A_01_3404614002",
0x496618f2,"MainField_Obj_LiftRockWhite_A_01_3406150708",
0x9eb4d1d8,"MainField_Obj_LiftRockWhite_A_01_3406906526",
0x8937b2dc,"MainField_Obj_LiftRockWhite_A_01_3410104538",
0x0403054b,"MainField_Obj_LiftRockWhite_A_01_3411983570",
0xd91be43f,"MainField_Obj_LiftRockWhite_A_01_3417960368",
0x3210f1b3,"MainField_Obj_LiftRockWhite_A_01_3422218258",
0xb5c1ccc3,"MainField_Obj_LiftRockWhite_A_01_3425532528",
0x84e5208e,"MainField_Obj_LiftRockWhite_A_01_3425650627",
0x362b2677,"MainField_Obj_LiftRockWhite_A_01_3425951931",
0x9de4219b,"MainField_Obj_LiftRockWhite_A_01_3434727496",
0xc3df6352,"MainField_Obj_LiftRockWhite_A_01_3437199341",
0xf533694a,"MainField_Obj_LiftRockWhite_A_01_3441754499",
0x873487cd,"MainField_Obj_LiftRockWhite_A_01_3444407922",
0xddae2bc9,"MainField_Obj_LiftRockWhite_A_01_3448382764",
0xad852fe8,"MainField_Obj_LiftRockWhite_A_01_345415447",
0x993796a1,"MainField_Obj_LiftRockWhite_A_01_3456586690",
0x30a62268,"MainField_Obj_LiftRockWhite_A_01_3457131023",
0xfba3f58e,"MainField_Obj_LiftRockWhite_A_01_3460678562",
0xe19c7ca8,"MainField_Obj_LiftRockWhite_A_01_3461241395",
0x04d152c7,"MainField_Obj_LiftRockWhite_A_01_3462943962",
0xf222a6f9,"MainField_Obj_LiftRockWhite_A_01_3469746729",
0x78d6b28d,"MainField_Obj_LiftRockWhite_A_01_3472016098",
0xb063b031,"MainField_Obj_LiftRockWhite_A_01_3478808717",
0x8f5cb99e,"MainField_Obj_LiftRockWhite_A_01_3481616118",
0xbf045abd,"MainField_Obj_LiftRockWhite_A_01_348311314",
0xd1d94a06,"MainField_Obj_LiftRockWhite_A_01_3484721708",
0xc424cfda,"MainField_Obj_LiftRockWhite_A_01_3485238223",
0xb31bcb92,"MainField_Obj_LiftRockWhite_A_01_3490931483",
0x855581e8,"MainField_Obj_LiftRockWhite_A_01_3491666319",
0xa75b6722,"MainField_Obj_LiftRockWhite_A_01_3492876909",
0xa9aa51d0,"MainField_Obj_LiftRockWhite_A_01_3492912537",
0x0c348450,"MainField_Obj_LiftRockWhite_A_01_3494477501",
0x988a6b06,"MainField_Obj_LiftRockWhite_A_01_3494589674",
0xfd5d51c5,"MainField_Obj_LiftRockWhite_A_01_3497613143",
0x3b7790f5,"MainField_Obj_LiftRockWhite_A_01_3499589246",
0x7a27a341,"MainField_Obj_LiftRockWhite_A_01_3507336648",
0x2ed97a63,"MainField_Obj_LiftRockWhite_A_01_3508514763",
0xb82a1f1d,"MainField_Obj_LiftRockWhite_A_01_3508559268",
0x21861a99,"MainField_Obj_LiftRockWhite_A_01_3508564405",
0x2b9f5154,"MainField_Obj_LiftRockWhite_A_01_3509257456",
0x9a648b4c,"MainField_Obj_LiftRockWhite_A_01_3517907710",
0x6c7a70d9,"MainField_Obj_LiftRockWhite_A_01_3519174516",
0x810ba0db,"MainField_Obj_LiftRockWhite_A_01_3519197862",
0x9743a0e1,"MainField_Obj_LiftRockWhite_A_01_3519538569",
0xd0547fea,"MainField_Obj_LiftRockWhite_A_01_3521766237",
0x5f815c1c,"MainField_Obj_LiftRockWhite_A_01_3527773587",
0x56e1c199,"MainField_Obj_LiftRockWhite_A_01_3528702672",
0x3cd1ad1c,"MainField_Obj_LiftRockWhite_A_01_3539572368",
0x4cd243e9,"MainField_Obj_LiftRockWhite_A_01_3540833929",
0x573237cd,"MainField_Obj_LiftRockWhite_A_01_354126115",
0x9830fa8f,"MainField_Obj_LiftRockWhite_A_01_3543676731",
0x71c5e1cb,"MainField_Obj_LiftRockWhite_A_01_3545057332",
0xcc2d8d81,"MainField_Obj_LiftRockWhite_A_01_3546153355",
0xdda921ae,"MainField_Obj_LiftRockWhite_A_01_3547167005",
0x4e4bc74d,"MainField_Obj_LiftRockWhite_A_01_3549477726",
0xa20264d1,"MainField_Obj_LiftRockWhite_A_01_3549734787",
0x598eb9d4,"MainField_Obj_LiftRockWhite_A_01_3560681212",
0x1b7a4274,"MainField_Obj_LiftRockWhite_A_01_3567764320",
0x106b12b7,"MainField_Obj_LiftRockWhite_A_01_356925478",
0xd3e12b5b,"MainField_Obj_LiftRockWhite_A_01_3569273887",
0x49fa8f0a,"MainField_Obj_LiftRockWhite_A_01_3570781057",
0x97513af1,"MainField_Obj_LiftRockWhite_A_01_3574276117",
0x7c7725c0,"MainField_Obj_LiftRockWhite_A_01_3584038706",
0x876dbc36,"MainField_Obj_LiftRockWhite_A_01_3584960375",
0xef0d6a8b,"MainField_Obj_LiftRockWhite_A_01_3593378700",
0x746636be,"MainField_Obj_LiftRockWhite_A_01_3595409853",
0x3866896f,"MainField_Obj_LiftRockWhite_A_01_359601307",
0xd2a3d835,"MainField_Obj_LiftRockWhite_A_01_3600461950",
0x38fe9d67,"MainField_Obj_LiftRockWhite_A_01_3602381799",
0x32727ed3,"MainField_Obj_LiftRockWhite_A_01_3607504761",
0x95324a57,"MainField_Obj_LiftRockWhite_A_01_3608165722",
0x6dcce568,"MainField_Obj_LiftRockWhite_A_01_3608449870",
0x7a4f1b4f,"MainField_Obj_LiftRockWhite_A_01_361045480",
0xe3660d7b,"MainField_Obj_LiftRockWhite_A_01_3616332434",
0x62725dd9,"MainField_Obj_LiftRockWhite_A_01_3622021009",
0x88e74426,"MainField_Obj_LiftRockWhite_A_01_3624849464",
0x8f536bb3,"MainField_Obj_LiftRockWhite_A_01_3627355454",
0xe08ee721,"MainField_Obj_LiftRockWhite_A_01_3632029316",
0x381ada75,"MainField_Obj_LiftRockWhite_A_01_363267413",
0xab4dc759,"MainField_Obj_LiftRockWhite_A_01_3634186459",
0x5101a41f,"MainField_Obj_LiftRockWhite_A_01_3640804362",
0x21d1d9ad,"MainField_Obj_LiftRockWhite_A_01_3641251679",
0xa3346017,"MainField_Obj_LiftRockWhite_A_01_3644563949",
0x00763b1a,"MainField_Obj_LiftRockWhite_A_01_3645505366",
0xdaf7fc20,"MainField_Obj_LiftRockWhite_A_01_3646176844",
0x3206c229,"MainField_Obj_LiftRockWhite_A_01_3651199940",
0xa93aff8e,"MainField_Obj_LiftRockWhite_A_01_3655623504",
0xac173923,"MainField_Obj_LiftRockWhite_A_01_3660017232",
0xbf567c0d,"MainField_Obj_LiftRockWhite_A_01_3660401639",
0xe957fb38,"MainField_Obj_LiftRockWhite_A_01_3662445857",
0xf2544b76,"MainField_Obj_LiftRockWhite_A_01_3665861637",
0x335db909,"MainField_Obj_LiftRockWhite_A_01_3672579392",
0x0ea2d682,"MainField_Obj_LiftRockWhite_A_01_3685339075",
0x9aee343f,"MainField_Obj_LiftRockWhite_A_01_3687984110",
0x95afded8,"MainField_Obj_LiftRockWhite_A_01_369201786",
0x0bdbe2d8,"MainField_Obj_LiftRockWhite_A_01_369501572",
0x47ed602e,"MainField_Obj_LiftRockWhite_A_01_3696699756",
0xb9484a94,"MainField_Obj_LiftRockWhite_A_01_371160874",
0x21d93f78,"MainField_Obj_LiftRockWhite_A_01_3712197775",
0xe0d00131,"MainField_Obj_LiftRockWhite_A_01_3712267916",
0x8bd5bb84,"MainField_Obj_LiftRockWhite_A_01_3713083070",
0xc3b8acf2,"MainField_Obj_LiftRockWhite_A_01_371703405",
0x040c5259,"MainField_Obj_LiftRockWhite_A_01_3717805936",
0x49b7ae81,"MainField_Obj_LiftRockWhite_A_01_3721848611",
0x5089537b,"MainField_Obj_LiftRockWhite_A_01_3730379542",
0xfacaeda5,"MainField_Obj_LiftRockWhite_A_01_3731961598",
0x13146926,"MainField_Obj_LiftRockWhite_A_01_3733563209",
0xe5d1151c,"MainField_Obj_LiftRockWhite_A_01_3734251162",
0x3704ba08,"MainField_Obj_LiftRockWhite_A_01_3739296264",
0x16dc8fd5,"MainField_Obj_LiftRockWhite_A_01_3740322626",
0xb2358a8f,"MainField_Obj_LiftRockWhite_A_01_3746319746",
0x9ab7a117,"MainField_Obj_LiftRockWhite_A_01_3765694225",
0x862d77f5,"MainField_Obj_LiftRockWhite_A_01_3768768471",
0x9c306a9d,"MainField_Obj_LiftRockWhite_A_01_3777613881",
0xffa11086,"MainField_Obj_LiftRockWhite_A_01_377963207",
0xafd6134b,"MainField_Obj_LiftRockWhite_A_01_3781695369",
0x77599d7c,"MainField_Obj_LiftRockWhite_A_01_3782584967",
0x5f0ebcab,"MainField_Obj_LiftRockWhite_A_01_3784446634",
0x216a9e13,"MainField_Obj_LiftRockWhite_A_01_378483237",
0x17a0d2bf,"MainField_Obj_LiftRockWhite_A_01_3785540865",
0x04361677,"MainField_Obj_LiftRockWhite_A_01_3785701692",
0xf46d6c66,"MainField_Obj_LiftRockWhite_A_01_3793189974",
0x3030ee0f,"MainField_Obj_LiftRockWhite_A_01_3796390022",
0xe16f2b36,"MainField_Obj_LiftRockWhite_A_01_3796511787",
0xeca8571d,"MainField_Obj_LiftRockWhite_A_01_3797557230",
0x0bfe4f05,"MainField_Obj_LiftRockWhite_A_01_3797947713",
0x5cba5e22,"MainField_Obj_LiftRockWhite_A_01_3798224377",
0x066184b9,"MainField_Obj_LiftRockWhite_A_01_3799303200",
0x73334f6c,"MainField_Obj_LiftRockWhite_A_01_3811388327",
0x69bb7a00,"MainField_Obj_LiftRockWhite_A_01_3813978808",
0xe47def55,"MainField_Obj_LiftRockWhite_A_01_381698103",
0x691d86d2,"MainField_Obj_LiftRockWhite_A_01_3819309108",
0x403e517e,"MainField_Obj_LiftRockWhite_A_01_3819601389",
0x50211448,"MainField_Obj_LiftRockWhite_A_01_382360608",
0x1b4c8fb4,"MainField_Obj_LiftRockWhite_A_01_3823801912",
0xbf552af0,"MainField_Obj_LiftRockWhite_A_01_3824047672",
0xf1244f39,"MainField_Obj_LiftRockWhite_A_01_3824810660",
0x245f05f9,"MainField_Obj_LiftRockWhite_A_01_3825169167",
0x0aa9952f,"MainField_Obj_LiftRockWhite_A_01_3831696110",
0x52618734,"MainField_Obj_LiftRockWhite_A_01_3834920637",
0x40c35788,"MainField_Obj_LiftRockWhite_A_01_3838294516",
0xb5b2999e,"MainField_Obj_LiftRockWhite_A_01_3840056110",
0x470e87df,"MainField_Obj_LiftRockWhite_A_01_3840751048",
0x8c5affa1,"MainField_Obj_LiftRockWhite_A_01_384119324",
0xe4c572c6,"MainField_Obj_LiftRockWhite_A_01_3856634145",
0xf7763121,"MainField_Obj_LiftRockWhite_A_01_3859686530",
0x496a6525,"MainField_Obj_LiftRockWhite_A_01_3874989649",
0x7e759305,"MainField_Obj_LiftRockWhite_A_01_3881199854",
0x2a705512,"MainField_Obj_LiftRockWhite_A_01_3881624085",
0xf76f80d1,"MainField_Obj_LiftRockWhite_A_01_3882632825",
0xb8d6deb4,"MainField_Obj_LiftRockWhite_A_01_3889645334",
0x2363cfb7,"MainField_Obj_LiftRockWhite_A_01_3890731210",
0xc51eb09c,"MainField_Obj_LiftRockWhite_A_01_3895137131",
0x3d134440,"MainField_Obj_LiftRockWhite_A_01_389945242",
0x2b94a238,"MainField_Obj_LiftRockWhite_A_01_3901453349",
0x2a90748e,"MainField_Obj_LiftRockWhite_A_01_3901711967",
0x5a7327aa,"MainField_Obj_LiftRockWhite_A_01_3904448073",
0x1825526d,"MainField_Obj_LiftRockWhite_A_01_3904611715",
0x38a928ea,"MainField_Obj_LiftRockWhite_A_01_3908741917",
0x3f7233c0,"MainField_Obj_LiftRockWhite_A_01_3915491668",
0x1b6ee43b,"MainField_Obj_LiftRockWhite_A_01_3918381728",
0x818820b5,"MainField_Obj_LiftRockWhite_A_01_3923174173",
0xd328eae8,"MainField_Obj_LiftRockWhite_A_01_3929445246",
0x651309f0,"MainField_Obj_LiftRockWhite_A_01_3930898941",
0xa64566c9,"MainField_Obj_LiftRockWhite_A_01_3931161491",
0xa65c76aa,"MainField_Obj_LiftRockWhite_A_01_3932666241",
0x429a686c,"MainField_Obj_LiftRockWhite_A_01_3935155254",
0x985f04a8,"MainField_Obj_LiftRockWhite_A_01_393609051",
0x5909ba97,"MainField_Obj_LiftRockWhite_A_01_3936845570",
0xb8d15ad6,"MainField_Obj_LiftRockWhite_A_01_3936892363",
0xa4831265,"MainField_Obj_LiftRockWhite_A_01_3942110152",
0x15d51c9a,"MainField_Obj_LiftRockWhite_A_01_3948673004",
0xb2f11feb,"MainField_Obj_LiftRockWhite_A_01_3952645399",
0x8e8372c2,"MainField_Obj_LiftRockWhite_A_01_3958798216",
0x8e6233f2,"MainField_Obj_LiftRockWhite_A_01_3961803788",
0x08d22164,"MainField_Obj_LiftRockWhite_A_01_3970992083",
0x8d40e4a6,"MainField_Obj_LiftRockWhite_A_01_397242687",
0x24aab55e,"MainField_Obj_LiftRockWhite_A_01_3974238867",
0x94ffe93c,"MainField_Obj_LiftRockWhite_A_01_3978780364",
0xef01eaa6,"MainField_Obj_LiftRockWhite_A_01_398277862",
0xb9eb1565,"MainField_Obj_LiftRockWhite_A_01_3984458026",
0x04ad3c7f,"MainField_Obj_LiftRockWhite_A_01_3990146390",
0x837b78bb,"MainField_Obj_LiftRockWhite_A_01_3992214338",
0xb79c577a,"MainField_Obj_LiftRockWhite_A_01_3992848975",
0x0e411bf7,"MainField_Obj_LiftRockWhite_A_01_3995120638",
0x9727f159,"MainField_Obj_LiftRockWhite_A_01_4006101449",
0xbced86da,"MainField_Obj_LiftRockWhite_A_01_4006545190",
0xe9bb86ee,"MainField_Obj_LiftRockWhite_A_01_4011942744",
0xa9003f77,"MainField_Obj_LiftRockWhite_A_01_4015142175",
0xeb64741b,"MainField_Obj_LiftRockWhite_A_01_4020860564",
0xbd0b0480,"MainField_Obj_LiftRockWhite_A_01_4026145755",
0x1f8c59e8,"MainField_Obj_LiftRockWhite_A_01_4026864097",
0xc01d02cd,"MainField_Obj_LiftRockWhite_A_01_4027931704",
0x9b335635,"MainField_Obj_LiftRockWhite_A_01_4029763083",
0x476de4f8,"MainField_Obj_LiftRockWhite_A_01_4033609567",
0x6871a149,"MainField_Obj_LiftRockWhite_A_01_40352806",
0x35f93e3d,"MainField_Obj_LiftRockWhite_A_01_4037625750",
0x4b0e41b0,"MainField_Obj_LiftRockWhite_A_01_403821586",
0xc4a0cd45,"MainField_Obj_LiftRockWhite_A_01_4040020572",
0xcdb5e338,"MainField_Obj_LiftRockWhite_A_01_4044339307",
0xecb64b2a,"MainField_Obj_LiftRockWhite_A_01_4044695202",
0xe985fb4a,"MainField_Obj_LiftRockWhite_A_01_4050809127",
0xf121ac9d,"MainField_Obj_LiftRockWhite_A_01_4056237190",
0xe1faefe5,"MainField_Obj_LiftRockWhite_A_01_4058933301",
0x38c81ffc,"MainField_Obj_LiftRockWhite_A_01_4068638059",
0x7bedfcea,"MainField_Obj_LiftRockWhite_A_01_4068697683",
0x5679c4c4,"MainField_Obj_LiftRockWhite_A_01_4093972798",
0x467e58b3,"MainField_Obj_LiftRockWhite_A_01_4096867960",
0xc6a598ab,"MainField_Obj_LiftRockWhite_A_01_4108503920",
0xacc8f5c7,"MainField_Obj_LiftRockWhite_A_01_4110368082",
0x5d35296d,"MainField_Obj_LiftRockWhite_A_01_411821840",
0xfbd8c889,"MainField_Obj_LiftRockWhite_A_01_4127974957",
0xcc80bfb5,"MainField_Obj_LiftRockWhite_A_01_4129273150",
0x3e2f10d8,"MainField_Obj_LiftRockWhite_A_01_4145424551",
0x65c156aa,"MainField_Obj_LiftRockWhite_A_01_4145763162",
0x3313bea4,"MainField_Obj_LiftRockWhite_A_01_4150172834",
0x8e1dc253,"MainField_Obj_LiftRockWhite_A_01_4151875617",
0x607d4570,"MainField_Obj_LiftRockWhite_A_01_4154400780",
0x0cb72d88,"MainField_Obj_LiftRockWhite_A_01_4154866150",
0x89713e4a,"MainField_Obj_LiftRockWhite_A_01_4155027816",
0xd5bf9dac,"MainField_Obj_LiftRockWhite_A_01_4156372206",
0x28aba6d4,"MainField_Obj_LiftRockWhite_A_01_4159581191",
0xd1c4713c,"MainField_Obj_LiftRockWhite_A_01_4160710762",
0x46a848a2,"MainField_Obj_LiftRockWhite_A_01_4163500315",
0x88dc902a,"MainField_Obj_LiftRockWhite_A_01_4167998406",
0x867afb4e,"MainField_Obj_LiftRockWhite_A_01_4170489573",
0x03aa6b06,"MainField_Obj_LiftRockWhite_A_01_4171296721",
0x0a994829,"MainField_Obj_LiftRockWhite_A_01_4171355835",
0xdf20e2ac,"MainField_Obj_LiftRockWhite_A_01_4172647505",
0xc5db5876,"MainField_Obj_LiftRockWhite_A_01_4172936256",
0xfba4a5d0,"MainField_Obj_LiftRockWhite_A_01_4180369073",
0x3896bcb7,"MainField_Obj_LiftRockWhite_A_01_4184399440",
0xaf45e10b,"MainField_Obj_LiftRockWhite_A_01_4188591565",
0x71fe7871,"MainField_Obj_LiftRockWhite_A_01_4189380151",
0x54ed817e,"MainField_Obj_LiftRockWhite_A_01_4189502319",
0xb6e866cc,"MainField_Obj_LiftRockWhite_A_01_4204803108",
0x92ad742c,"MainField_Obj_LiftRockWhite_A_01_4205271124",
0x743ba355,"MainField_Obj_LiftRockWhite_A_01_4206030057",
0x39b95f3c,"MainField_Obj_LiftRockWhite_A_01_4217144424",
0x941306f7,"MainField_Obj_LiftRockWhite_A_01_4217468343",
0xe0ccef14,"MainField_Obj_LiftRockWhite_A_01_4219436216",
0xddd76485,"MainField_Obj_LiftRockWhite_A_01_4222769764",
0x66a63b56,"MainField_Obj_LiftRockWhite_A_01_4224946452",
0x647d64df,"MainField_Obj_LiftRockWhite_A_01_4227379010",
0xd5e074d3,"MainField_Obj_LiftRockWhite_A_01_422924072",
0x786dbc37,"MainField_Obj_LiftRockWhite_A_01_4231643277",
0xba7f2f92,"MainField_Obj_LiftRockWhite_A_01_4234869615",
0x353f417a,"MainField_Obj_LiftRockWhite_A_01_4240744544",
0xfde396b8,"MainField_Obj_LiftRockWhite_A_01_4242979037",
0x67445352,"MainField_Obj_LiftRockWhite_A_01_424864574",
0x430bf2c7,"MainField_Obj_LiftRockWhite_A_01_4251436657",
0x2e740779,"MainField_Obj_LiftRockWhite_A_01_4254859750",
0x73de32b8,"MainField_Obj_LiftRockWhite_A_01_4261747273",
0x4e771b0d,"MainField_Obj_LiftRockWhite_A_01_4264278312",
0x84402ee6,"MainField_Obj_LiftRockWhite_A_01_4264638239",
0x88102c51,"MainField_Obj_LiftRockWhite_A_01_4265969209",
0x626514b9,"MainField_Obj_LiftRockWhite_A_01_4266940899",
0x8d1887e8,"MainField_Obj_LiftRockWhite_A_01_4268086198",
0x2986afe1,"MainField_Obj_LiftRockWhite_A_01_427227405",
0x78b82af6,"MainField_Obj_LiftRockWhite_A_01_4273107472",
0xf9af105e,"MainField_Obj_LiftRockWhite_A_01_427845782",
0x73f8704c,"MainField_Obj_LiftRockWhite_A_01_4280631371",
0x7c5158d6,"MainField_Obj_LiftRockWhite_A_01_4285289197",
0xa9fd92f2,"MainField_Obj_LiftRockWhite_A_01_428604943",
0xa2f6885a,"MainField_Obj_LiftRockWhite_A_01_4292096089",
0x3bd49929,"MainField_Obj_LiftRockWhite_A_01_4294531836",
0x6e44dc5b,"MainField_Obj_LiftRockWhite_A_01_431183298",
0x01556109,"MainField_Obj_LiftRockWhite_A_01_436957202",
0xb646d9ab,"MainField_Obj_LiftRockWhite_A_01_442803634",
0x9ea95034,"MainField_Obj_LiftRockWhite_A_01_442867000",
0x2fc3825b,"MainField_Obj_LiftRockWhite_A_01_444660558",
0xe079bf3b,"MainField_Obj_LiftRockWhite_A_01_445201189",
0x5da07cb7,"MainField_Obj_LiftRockWhite_A_01_445597628",
0x0610eb53,"MainField_Obj_LiftRockWhite_A_01_44930247",
0x3c735e6b,"MainField_Obj_LiftRockWhite_A_01_451753519",
0xbc6fea30,"MainField_Obj_LiftRockWhite_A_01_45196519",
0xd2b98461,"MainField_Obj_LiftRockWhite_A_01_459469541",
0x160aa866,"MainField_Obj_LiftRockWhite_A_01_461147495",
0x9d006992,"MainField_Obj_LiftRockWhite_A_01_474639547",
0x90993b77,"MainField_Obj_LiftRockWhite_A_01_483778940",
0xe34c5340,"MainField_Obj_LiftRockWhite_A_01_48692380",
0x96fa896d,"MainField_Obj_LiftRockWhite_A_01_498179780",
0x3ec5acb8,"MainField_Obj_LiftRockWhite_A_01_498330576",
0x63b654ad,"MainField_Obj_LiftRockWhite_A_01_498934377",
0x5eee525d,"MainField_Obj_LiftRockWhite_A_01_499540955",
0x85d24f66,"MainField_Obj_LiftRockWhite_A_01_505101588",
0x9c792122,"MainField_Obj_LiftRockWhite_A_01_517010263",
0x80c7e3ec,"MainField_Obj_LiftRockWhite_A_01_517902811",
0x340c9bc2,"MainField_Obj_LiftRockWhite_A_01_519223034",
0x52940b66,"MainField_Obj_LiftRockWhite_A_01_52278068",
0x66d9debd,"MainField_Obj_LiftRockWhite_A_01_528224267",
0x0a702613,"MainField_Obj_LiftRockWhite_A_01_532357772",
0x512f7907,"MainField_Obj_LiftRockWhite_A_01_533407146",
0xd4a6e3e2,"MainField_Obj_LiftRockWhite_A_01_535754871",
0xa47f998e,"MainField_Obj_LiftRockWhite_A_01_540026310",
0x7eb820d5,"MainField_Obj_LiftRockWhite_A_01_541080351",
0xfc1dcfeb,"MainField_Obj_LiftRockWhite_A_01_544387532",
0x15f226b4,"MainField_Obj_LiftRockWhite_A_01_545425501",
0x27b340f5,"MainField_Obj_LiftRockWhite_A_01_545659516",
0x12ee202c,"MainField_Obj_LiftRockWhite_A_01_551234049",
0x61f2771f,"MainField_Obj_LiftRockWhite_A_01_552279887",
0x795cab1a,"MainField_Obj_LiftRockWhite_A_01_553427918",
0x1726b27d,"MainField_Obj_LiftRockWhite_A_01_554266230",
0xc5ec29d4,"MainField_Obj_LiftRockWhite_A_01_572849887",
0xd391d34b,"MainField_Obj_LiftRockWhite_A_01_573218196",
0x56842604,"MainField_Obj_LiftRockWhite_A_01_581260713",
0x235d3a71,"MainField_Obj_LiftRockWhite_A_01_58314617",
0x0e91e861,"MainField_Obj_LiftRockWhite_A_01_6061633",
0xc04dd64c,"MainField_Obj_LiftRockWhite_A_01_608542095",
0xf19f6ee6,"MainField_Obj_LiftRockWhite_A_01_625188313",
0x7a71bd57,"MainField_Obj_LiftRockWhite_A_01_626056702",
0x0532de44,"MainField_Obj_LiftRockWhite_A_01_630560804",
0xb80a8626,"MainField_Obj_LiftRockWhite_A_01_634434263",
0xc86122f8,"MainField_Obj_LiftRockWhite_A_01_638343174",
0x49c3f5e1,"MainField_Obj_LiftRockWhite_A_01_640135838",
0x2cf4a85d,"MainField_Obj_LiftRockWhite_A_01_644098180",
0x935c7e13,"MainField_Obj_LiftRockWhite_A_01_649548066",
0x5704d266,"MainField_Obj_LiftRockWhite_A_01_651075740",
0x1f9f4503,"MainField_Obj_LiftRockWhite_A_01_651626265",
0x19cf663f,"MainField_Obj_LiftRockWhite_A_01_651872331",
0xb706c23a,"MainField_Obj_LiftRockWhite_A_01_654674785",
0x3ebdcb78,"MainField_Obj_LiftRockWhite_A_01_657325830",
0x7dbd2fb3,"MainField_Obj_LiftRockWhite_A_01_662342099",
0x5ed55188,"MainField_Obj_LiftRockWhite_A_01_66694997",
0x1ea094ea,"MainField_Obj_LiftRockWhite_A_01_671539317",
0x4998afe5,"MainField_Obj_LiftRockWhite_A_01_671637789",
0xf398e56b,"MainField_Obj_LiftRockWhite_A_01_67564878",
0xc505732e,"MainField_Obj_LiftRockWhite_A_01_67836595",
0xe8750eec,"MainField_Obj_LiftRockWhite_A_01_678875153",
0x21dfde77,"MainField_Obj_LiftRockWhite_A_01_680260615",
0xab513921,"MainField_Obj_LiftRockWhite_A_01_6850659",
0x410ac822,"MainField_Obj_LiftRockWhite_A_01_694685486",
0x6aff02e4,"MainField_Obj_LiftRockWhite_A_01_695197061",
0xb9783a86,"MainField_Obj_LiftRockWhite_A_01_698152740",
0x594fa472,"MainField_Obj_LiftRockWhite_A_01_698466010",
0x7f841a26,"MainField_Obj_LiftRockWhite_A_01_701914500",
0x09da2e1d,"MainField_Obj_LiftRockWhite_A_01_706777545",
0x52a9db2d,"MainField_Obj_LiftRockWhite_A_01_711673600",
0xae7988e5,"MainField_Obj_LiftRockWhite_A_01_711727201",
0x5ed1efe0,"MainField_Obj_LiftRockWhite_A_01_713844509",
0x21d57c72,"MainField_Obj_LiftRockWhite_A_01_71423907",
0x0d8d66b1,"MainField_Obj_LiftRockWhite_A_01_71725410",
0x72c4336c,"MainField_Obj_LiftRockWhite_A_01_721847631",
0xc36ad538,"MainField_Obj_LiftRockWhite_A_01_724500748",
0xb6ce7c52,"MainField_Obj_LiftRockWhite_A_01_728279900",
0xfd5b7c49,"MainField_Obj_LiftRockWhite_A_01_730530961",
0xced5c7a8,"MainField_Obj_LiftRockWhite_A_01_734450799",
0x8dbf0bcc,"MainField_Obj_LiftRockWhite_A_01_741845267",
0x19da7de8,"MainField_Obj_LiftRockWhite_A_01_742573333",
0x044f4a9a,"MainField_Obj_LiftRockWhite_A_01_74649821",
0x6200c165,"MainField_Obj_LiftRockWhite_A_01_74933504",
0xf6a0bc3b,"MainField_Obj_LiftRockWhite_A_01_753308117",
0x2173d9da,"MainField_Obj_LiftRockWhite_A_01_756149459",
0xb511d487,"MainField_Obj_LiftRockWhite_A_01_758312156",
0x1cf39307,"MainField_Obj_LiftRockWhite_A_01_759066869",
0x2f7905ff,"MainField_Obj_LiftRockWhite_A_01_764539771",
0xb2f7e8f8,"MainField_Obj_LiftRockWhite_A_01_769762052",
0xd3e32f27,"MainField_Obj_LiftRockWhite_A_01_785638693",
0x805385d0,"MainField_Obj_LiftRockWhite_A_01_786058001",
0xc680edea,"MainField_Obj_LiftRockWhite_A_01_790223862",
0xaa769d27,"MainField_Obj_LiftRockWhite_A_01_79080853",
0x17a79c38,"MainField_Obj_LiftRockWhite_A_01_795640928",
0x08725835,"MainField_Obj_LiftRockWhite_A_01_815242341",
0x2ecf479a,"MainField_Obj_LiftRockWhite_A_01_823121632",
0xc8f25c01,"MainField_Obj_LiftRockWhite_A_01_826649682",
0xf5cb7d9d,"MainField_Obj_LiftRockWhite_A_01_827272033",
0x5f9e2b18,"MainField_Obj_LiftRockWhite_A_01_829932590",
0x9bbfd70d,"MainField_Obj_LiftRockWhite_A_01_834276723",
0x18240431,"MainField_Obj_LiftRockWhite_A_01_837201120",
0x19824f89,"MainField_Obj_LiftRockWhite_A_01_838790638",
0x18e353a1,"MainField_Obj_LiftRockWhite_A_01_841181957",
0x6fe508ed,"MainField_Obj_LiftRockWhite_A_01_844712000",
0xd4b71baa,"MainField_Obj_LiftRockWhite_A_01_844779057",
0x1cb62ded,"MainField_Obj_LiftRockWhite_A_01_844839234",
0xe4214e6f,"MainField_Obj_LiftRockWhite_A_01_84539893",
0x604e5f3c,"MainField_Obj_LiftRockWhite_A_01_849033089",
0x30393a1d,"MainField_Obj_LiftRockWhite_A_01_856863343",
0xa70e2e5e,"MainField_Obj_LiftRockWhite_A_01_860911464",
0xeac7333e,"MainField_Obj_LiftRockWhite_A_01_865556060",
0x1c13ee0c,"MainField_Obj_LiftRockWhite_A_01_86735986",
0xbfbfcd4a,"MainField_Obj_LiftRockWhite_A_01_875608740",
0x51488052,"MainField_Obj_LiftRockWhite_A_01_876270831",
0x6eecb960,"MainField_Obj_LiftRockWhite_A_01_877250866",
0x0dbc08eb,"MainField_Obj_LiftRockWhite_A_01_881786153",
0xbf24194a,"MainField_Obj_LiftRockWhite_A_01_894349084",
0xc726b29b,"MainField_Obj_LiftRockWhite_A_01_895735950",
0xc9181632,"MainField_Obj_LiftRockWhite_A_01_896594943",
0x86fde12b,"MainField_Obj_LiftRockWhite_A_01_897295562",
0x70f2b754,"MainField_Obj_LiftRockWhite_A_01_897736178",
0x1aca04b9,"MainField_Obj_LiftRockWhite_A_01_903499025",
0x1a4e3640,"MainField_Obj_LiftRockWhite_A_01_904731642",
0xde6bfd41,"MainField_Obj_LiftRockWhite_A_01_905814164",
0x4042746f,"MainField_Obj_LiftRockWhite_A_01_908877828",
0x6a94b3a6,"MainField_Obj_LiftRockWhite_A_01_911850899",
0x7dc83e6e,"MainField_Obj_LiftRockWhite_A_01_912204915",
0xd5c469c2,"MainField_Obj_LiftRockWhite_A_01_913001545",
0x5eab58d8,"MainField_Obj_LiftRockWhite_A_01_913492610",
0x8dc8b5ee,"MainField_Obj_LiftRockWhite_A_01_913896871",
0xeec168c4,"MainField_Obj_LiftRockWhite_A_01_913996403",
0x83d875a9,"MainField_Obj_LiftRockWhite_A_01_915308298",
0xd1e2fcdd,"MainField_Obj_LiftRockWhite_A_01_915614199",
0xb915c411,"MainField_Obj_LiftRockWhite_A_01_915959971",
0x3002c6e5,"MainField_Obj_LiftRockWhite_A_01_916947660",
0xdfb9c38d,"MainField_Obj_LiftRockWhite_A_01_920144599",
0x7b08c5d7,"MainField_Obj_LiftRockWhite_A_01_921848236",
0x7ce34d0b,"MainField_Obj_LiftRockWhite_A_01_92564029",
0x13a9c3f4,"MainField_Obj_LiftRockWhite_A_01_928669536",
0x71f2b854,"MainField_Obj_LiftRockWhite_A_01_931741546",
0x9cd8114d,"MainField_Obj_LiftRockWhite_A_01_938562490",
0x1cfe1908,"MainField_Obj_LiftRockWhite_A_01_939194284",
0x7ec5d1b7,"MainField_Obj_LiftRockWhite_A_01_940421869",
0xebc038af,"MainField_Obj_LiftRockWhite_A_01_943969408",
0x4826f0d3,"MainField_Obj_LiftRockWhite_A_01_945700504",
0x07762daf,"MainField_Obj_LiftRockWhite_A_01_950811923",
0x8a35d3a9,"MainField_Obj_LiftRockWhite_A_01_957481903",
0xc09c2dcf,"MainField_Obj_LiftRockWhite_A_01_959871009",
0xef19bf8d,"MainField_Obj_LiftRockWhite_A_01_965590376",
0x845a3bbe,"MainField_Obj_LiftRockWhite_A_01_968187156",
0x9838cf4b,"MainField_Obj_LiftRockWhite_A_01_970789997",
0xdb7a1234,"MainField_Obj_LiftRockWhite_A_01_972379442",
0x56814e58,"MainField_Obj_LiftRockWhite_A_01_975247991",
0x4e3662ac,"MainField_Obj_LiftRockWhite_A_01_975518820",
0x7eeffe68,"MainField_Obj_LiftRockWhite_A_01_975818333",
0x5d1f6bb4,"MainField_Obj_LiftRockWhite_A_01_985942486",
0xdc233a8c,"MainField_Obj_LiftRockWhite_A_01_986174749",
0x2ef8d105,"MainField_Obj_LiftRockWhite_A_01_986264083",
0x7033345c,"MainField_Obj_LiftRockWhite_A_01_992356518",
0x3f007318,"MainField_Obj_LiftRockWhite_A_01_995728039",
0x29866288,"MainField_Obj_MineralBury_A_01_1004400691",
0xc1fe72f1,"MainField_Obj_MineralBury_A_01_1005865361",
0x49af28d3,"MainField_Obj_MineralBury_A_01_1008829954",
0x6fcd8861,"MainField_Obj_MineralBury_A_01_1015337329",
0x3da663da,"MainField_Obj_MineralBury_A_01_1026768623",
0x336333f2,"MainField_Obj_MineralBury_A_01_1035297490",
0xb2d41f51,"MainField_Obj_MineralBury_A_01_1036499183",
0xafcf4d32,"MainField_Obj_MineralBury_A_01_104481960",
0x576cc4e3,"MainField_Obj_MineralBury_A_01_1045402935",
0x9c3730d1,"MainField_Obj_MineralBury_A_01_1047184793",
0xd44b5cb6,"MainField_Obj_MineralBury_A_01_1050080093",
0xe6dc5a8c,"MainField_Obj_MineralBury_A_01_1058112038",
0xb0efe716,"MainField_Obj_MineralBury_A_01_106281004",
0xd3ccf64d,"MainField_Obj_MineralBury_A_01_1086837881",
0xaacb76be,"MainField_Obj_MineralBury_A_01_1095598956",
0x83026de7,"MainField_Obj_MineralBury_A_01_1097672601",
0xab4e720e,"MainField_Obj_MineralBury_A_01_1099451169",
0x763b2aae,"MainField_Obj_MineralBury_A_01_1101658743",
0x2af68175,"MainField_Obj_MineralBury_A_01_110186569",
0x5c53e7c1,"MainField_Obj_MineralBury_A_01_1104884351",
0x9e134d63,"MainField_Obj_MineralBury_A_01_1106066391",
0x64f21e49,"MainField_Obj_MineralBury_A_01_111703418",
0x3470a489,"MainField_Obj_MineralBury_A_01_1118991123",
0xf26a9c7a,"MainField_Obj_MineralBury_A_01_1131230934",
0x07a6d081,"MainField_Obj_MineralBury_A_01_1138180289",
0xbea3dd77,"MainField_Obj_MineralBury_A_01_1145054022",
0x515cfb36,"MainField_Obj_MineralBury_A_01_1148309002",
0x7829b593,"MainField_Obj_MineralBury_A_01_1153428094",
0x288189b6,"MainField_Obj_MineralBury_A_01_1162396786",
0x03705991,"MainField_Obj_MineralBury_A_01_1167902409",
0x87b6b7fc,"MainField_Obj_MineralBury_A_01_1170668553",
0xe0c842d7,"MainField_Obj_MineralBury_A_01_1173114083",
0x14931f6b,"MainField_Obj_MineralBury_A_01_1174247028",
0x02ebd2a7,"MainField_Obj_MineralBury_A_01_1176975775",
0xb8581feb,"MainField_Obj_MineralBury_A_01_1182054093",
0x22bca37e,"MainField_Obj_MineralBury_A_01_1182429251",
0x2db662c3,"MainField_Obj_MineralBury_A_01_118380301",
0xd0b2d2a3,"MainField_Obj_MineralBury_A_01_1192401342",
0x347a5739,"MainField_Obj_MineralBury_A_01_1194605780",
0xfa78e80e,"MainField_Obj_MineralBury_A_01_1194873599",
0xcf3c3b0c,"MainField_Obj_MineralBury_A_01_1197126701",
0x1508ae43,"MainField_Obj_MineralBury_A_01_1200933523",
0x579217c2,"MainField_Obj_MineralBury_A_01_1212472829",
0x75b7bef0,"MainField_Obj_MineralBury_A_01_1218181200",
0x3ed5175f,"MainField_Obj_MineralBury_A_01_1219040068",
0x33e85fa6,"MainField_Obj_MineralBury_A_01_1232609293",
0x8b19de12,"MainField_Obj_MineralBury_A_01_1235247481",
0x1cff93a3,"MainField_Obj_MineralBury_A_01_1235345197",
0x864a26bf,"MainField_Obj_MineralBury_A_01_1238464840",
0x62b5699e,"MainField_Obj_MineralBury_A_01_124123915",
0xcced45f0,"MainField_Obj_MineralBury_A_01_1244508453",
0x1e41b734,"MainField_Obj_MineralBury_A_01_1245495791",
0x12e53228,"MainField_Obj_MineralBury_A_01_1253123704",
0xfeab39ad,"MainField_Obj_MineralBury_A_01_1260240374",
0xa546cce5,"MainField_Obj_MineralBury_A_01_1260974537",
0x5c63b4f1,"MainField_Obj_MineralBury_A_01_126495936",
0x3e08d765,"MainField_Obj_MineralBury_A_01_1265208502",
0x99c0b8df,"MainField_Obj_MineralBury_A_01_1267800266",
0xb7955a97,"MainField_Obj_MineralBury_A_01_1281267739",
0x9a0c9233,"MainField_Obj_MineralBury_A_01_1289542818",
0x50f78424,"MainField_Obj_MineralBury_A_01_1295709733",
0x5729d1fd,"MainField_Obj_MineralBury_A_01_1295923503",
0x00fa99f0,"MainField_Obj_MineralBury_A_01_1297894409",
0xb059b734,"MainField_Obj_MineralBury_A_01_1298336437",
0xd2dc9244,"MainField_Obj_MineralBury_A_01_1303720987",
0x9a1e7d13,"MainField_Obj_MineralBury_A_01_1304930377",
0xad894736,"MainField_Obj_MineralBury_A_01_1305404224",
0x5a8a6e84,"MainField_Obj_MineralBury_A_01_130957733",
0x947f55b6,"MainField_Obj_MineralBury_A_01_1311153020",
0xc83dd5ac,"MainField_Obj_MineralBury_A_01_1319289297",
0x0853198a,"MainField_Obj_MineralBury_A_01_1319904382",
0x180041bc,"MainField_Obj_MineralBury_A_01_1325356562",
0x972341b2,"MainField_Obj_MineralBury_A_01_1326991643",
0x9cd13291,"MainField_Obj_MineralBury_A_01_1330241467",
0x176fe001,"MainField_Obj_MineralBury_A_01_1334439518",
0x47304f89,"MainField_Obj_MineralBury_A_01_1335796850",
0xda52e332,"MainField_Obj_MineralBury_A_01_134613470",
0x18d51c56,"MainField_Obj_MineralBury_A_01_1347732854",
0x635df9fd,"MainField_Obj_MineralBury_A_01_1349621285",
0xb8a5490c,"MainField_Obj_MineralBury_A_01_1351799906",
0x229edef0,"MainField_Obj_MineralBury_A_01_135474329",
0x4cdfb436,"MainField_Obj_MineralBury_A_01_1356879003",
0x23ac04df,"MainField_Obj_MineralBury_A_01_1362245969",
0xbcb91a8c,"MainField_Obj_MineralBury_A_01_1367156929",
0x4d236da0,"MainField_Obj_MineralBury_A_01_1367314176",
0xdd528e3a,"MainField_Obj_MineralBury_A_01_1367693784",
0xd483a08a,"MainField_Obj_MineralBury_A_01_1368318946",
0x47c4a3cb,"MainField_Obj_MineralBury_A_01_1375253287",
0x4adfe473,"MainField_Obj_MineralBury_A_01_1377735024",
0x41db7b6a,"MainField_Obj_MineralBury_A_01_1380812124",
0x30e48278,"MainField_Obj_MineralBury_A_01_1384294159",
0x53df6e43,"MainField_Obj_MineralBury_A_01_1396768823",
0xc22188a0,"MainField_Obj_MineralBury_A_01_1413420419",
0x444acdbb,"MainField_Obj_MineralBury_A_01_1415541409",
0xa4bcac2b,"MainField_Obj_MineralBury_A_01_1427084661",
0x047287df,"MainField_Obj_MineralBury_A_01_1428871534",
0xbcadc466,"MainField_Obj_MineralBury_A_01_1429821547",
0xbb67af83,"MainField_Obj_MineralBury_A_01_145067819",
0x45294c14,"MainField_Obj_MineralBury_A_01_1454906278",
0xc81376a5,"MainField_Obj_MineralBury_A_01_1455395194",
0x2cd59b12,"MainField_Obj_MineralBury_A_01_1461242806",
0xb2bffb33,"MainField_Obj_MineralBury_A_01_1464000196",
0xe306c51a,"MainField_Obj_MineralBury_A_01_1466252594",
0x64f507a5,"MainField_Obj_MineralBury_A_01_1467994550",
0x670e729b,"MainField_Obj_MineralBury_A_01_1469951798",
0x5c5da68f,"MainField_Obj_MineralBury_A_01_1470677910",
0xeb268e06,"MainField_Obj_MineralBury_A_01_1472849427",
0xf2afcaa5,"MainField_Obj_MineralBury_A_01_1472918408",
0xa256c50d,"MainField_Obj_MineralBury_A_01_1474112581",
0x4b33edea,"MainField_Obj_MineralBury_A_01_1475578442",
0x1fa8f454,"MainField_Obj_MineralBury_A_01_147729260",
0x1679c07e,"MainField_Obj_MineralBury_A_01_147900911",
0x725661e8,"MainField_Obj_MineralBury_A_01_148897876",
0x2b83d433,"MainField_Obj_MineralBury_A_01_1490805087",
0x6d78d684,"MainField_Obj_MineralBury_A_01_1492747982",
0x1ade45f7,"MainField_Obj_MineralBury_A_01_1494023969",
0xe25a6be0,"MainField_Obj_MineralBury_A_01_1495283009",
0xc7c16514,"MainField_Obj_MineralBury_A_01_1496284016",
0x67bf622f,"MainField_Obj_MineralBury_A_01_1496846007",
0xa5e10890,"MainField_Obj_MineralBury_A_01_1497344792",
0xf8acf3f4,"MainField_Obj_MineralBury_A_01_1506184897",
0xcf525937,"MainField_Obj_MineralBury_A_01_150949196",
0xf66fe1e8,"MainField_Obj_MineralBury_A_01_1513872083",
0x11b36ecd,"MainField_Obj_MineralBury_A_01_1515155145",
0x18467db7,"MainField_Obj_MineralBury_A_01_1516291902",
0x2a6d7441,"MainField_Obj_MineralBury_A_01_1519070748",
0x93e09e9c,"MainField_Obj_MineralBury_A_01_1521607152",
0x4023e9a9,"MainField_Obj_MineralBury_A_01_152602712",
0x97d5f534,"MainField_Obj_MineralBury_A_01_152900250",
0x9c65671a,"MainField_Obj_MineralBury_A_01_1529844205",
0x40bbc079,"MainField_Obj_MineralBury_A_01_153101268",
0xb6450183,"MainField_Obj_MineralBury_A_01_1536686224",
0x92ca3aea,"MainField_Obj_MineralBury_A_01_1539434854",
0x32d58268,"MainField_Obj_MineralBury_A_01_1542143395",
0xfb380005,"MainField_Obj_MineralBury_A_01_1546281658",
0xe78bbee0,"MainField_Obj_MineralBury_A_01_1548053930",
0xe301924d,"MainField_Obj_MineralBury_A_01_1548253448",
0x6cff30b9,"MainField_Obj_MineralBury_A_01_1549816165",
0xe706e33d,"MainField_Obj_MineralBury_A_01_1551501629",
0x57ccb04c,"MainField_Obj_MineralBury_A_01_155347289",
0xeff2e603,"MainField_Obj_MineralBury_A_01_1556732607",
0x14e9eeb4,"MainField_Obj_MineralBury_A_01_1564181663",
0xf4514edb,"MainField_Obj_MineralBury_A_01_1564493957",
0x53069d3f,"MainField_Obj_MineralBury_A_01_1570235932",
0xc2ae47be,"MainField_Obj_MineralBury_A_01_1575977696",
0x3102db2a,"MainField_Obj_MineralBury_A_01_1585954409",
0xfd403661,"MainField_Obj_MineralBury_A_01_1586983353",
0x144566df,"MainField_Obj_MineralBury_A_01_1592153361",
0x09341817,"MainField_Obj_MineralBury_A_01_1592370945",
0x2e1a36b0,"MainField_Obj_MineralBury_A_01_159769451",
0x53b8cc3c,"MainField_Obj_MineralBury_A_01_1605774237",
0x5316ca5a,"MainField_Obj_MineralBury_A_01_1606568933",
0xa64c2744,"MainField_Obj_MineralBury_A_01_1607183841",
0x8b52f824,"MainField_Obj_MineralBury_A_01_1610063370",
0x7fc142de,"MainField_Obj_MineralBury_A_01_1610910192",
0x631b6bfb,"MainField_Obj_MineralBury_A_01_161388119",
0xcd672183,"MainField_Obj_MineralBury_A_01_1615012383",
0x4938c0aa,"MainField_Obj_MineralBury_A_01_1619459323",
0x6d73e408,"MainField_Obj_MineralBury_A_01_1623774976",
0x594ce713,"MainField_Obj_MineralBury_A_01_1624957455",
0x88e57fac,"MainField_Obj_MineralBury_A_01_1625621945",
0xe136b3f2,"MainField_Obj_MineralBury_A_01_1626994943",
0x97aa2626,"MainField_Obj_MineralBury_A_01_1639469158",
0x6d59fe55,"MainField_Obj_MineralBury_A_01_1643726208",
0xe014a592,"MainField_Obj_MineralBury_A_01_1644647581",
0x4981ccfa,"MainField_Obj_MineralBury_A_01_1647114353",
0xabf2f899,"MainField_Obj_MineralBury_A_01_1647137715",
0x972329bc,"MainField_Obj_MineralBury_A_01_1647610123",
0xf871b256,"MainField_Obj_MineralBury_A_01_1649660338",
0x106a2cbc,"MainField_Obj_MineralBury_A_01_1651763402",
0x6702f9fb,"MainField_Obj_MineralBury_A_01_1654983597",
0xbd9592f9,"MainField_Obj_MineralBury_A_01_1665234271",
0xf7e68f97,"MainField_Obj_MineralBury_A_01_1666320592",
0xc67c8f2c,"MainField_Obj_MineralBury_A_01_1670836294",
0x3221b328,"MainField_Obj_MineralBury_A_01_1673672587",
0x6cb4d13c,"MainField_Obj_MineralBury_A_01_1680121638",
0x46aea192,"MainField_Obj_MineralBury_A_01_1688505087",
0x2075b12e,"MainField_Obj_MineralBury_A_01_1697963194",
0x059e450a,"MainField_Obj_MineralBury_A_01_1699324148",
0xe23c4219,"MainField_Obj_MineralBury_A_01_1704263874",
0x68105973,"MainField_Obj_MineralBury_A_01_1704500429",
0x5d97ccc4,"MainField_Obj_MineralBury_A_01_1705770565",
0x48cd3d56,"MainField_Obj_MineralBury_A_01_1709126988",
0x9fa55278,"MainField_Obj_MineralBury_A_01_1709599527",
0x7fe49270,"MainField_Obj_MineralBury_A_01_1716539924",
0x71693862,"MainField_Obj_MineralBury_A_01_1729267650",
0xedacb8dd,"MainField_Obj_MineralBury_A_01_1730346433",
0x1d7bc52a,"MainField_Obj_MineralBury_A_01_1730854062",
0x9f0850f0,"MainField_Obj_MineralBury_A_01_1733871625",
0x226276b4,"MainField_Obj_MineralBury_A_01_1736768514",
0xd42d4071,"MainField_Obj_MineralBury_A_01_1737357263",
0x2bf6061a,"MainField_Obj_MineralBury_A_01_1737514559",
0x6ab35880,"MainField_Obj_MineralBury_A_01_1746043853",
0xba7ae5f5,"MainField_Obj_MineralBury_A_01_1747198109",
0x12e1184f,"MainField_Obj_MineralBury_A_01_1767191506",
0x409fe8b5,"MainField_Obj_MineralBury_A_01_1768397538",
0x8be34c31,"MainField_Obj_MineralBury_A_01_1771114322",
0x367cfea1,"MainField_Obj_MineralBury_A_01_1772661932",
0x7e8494a4,"MainField_Obj_MineralBury_A_01_1786645368",
0x20c04a6d,"MainField_Obj_MineralBury_A_01_1789002270",
0x833f06ed,"MainField_Obj_MineralBury_A_01_1791444268",
0x7e91ed05,"MainField_Obj_MineralBury_A_01_1795108366",
0x824a38e9,"MainField_Obj_MineralBury_A_01_1795253226",
0xb4a1e370,"MainField_Obj_MineralBury_A_01_1796803633",
0xe194a727,"MainField_Obj_MineralBury_A_01_1797823075",
0x2639aafa,"MainField_Obj_MineralBury_A_01_1798854465",
0xa7b0825a,"MainField_Obj_MineralBury_A_01_180201807",
0x896071d3,"MainField_Obj_MineralBury_A_01_1802767729",
0x69a3ef2b,"MainField_Obj_MineralBury_A_01_1806215545",
0x9c90ad6f,"MainField_Obj_MineralBury_A_01_1806347978",
0x460f7ffe,"MainField_Obj_MineralBury_A_01_180797804",
0xb7fde139,"MainField_Obj_MineralBury_A_01_1815612971",
0xc81cf709,"MainField_Obj_MineralBury_A_01_1817244976",
0x95d2a6ae,"MainField_Obj_MineralBury_A_01_1822035111",
0x770ef456,"MainField_Obj_MineralBury_A_01_1828010040",
0x8fb787eb,"MainField_Obj_MineralBury_A_01_1828704779",
0x03aac7f3,"MainField_Obj_MineralBury_A_01_1835123386",
0x39288f77,"MainField_Obj_MineralBury_A_01_1835766451",
0xf5b9b5df,"MainField_Obj_MineralBury_A_01_1855588541",
0x7b337703,"MainField_Obj_MineralBury_A_01_1856974318",
0x821aa4e3,"MainField_Obj_MineralBury_A_01_1860473738",
0x0781fad7,"MainField_Obj_MineralBury_A_01_1861574260",
0x31dab913,"MainField_Obj_MineralBury_A_01_1864548671",
0x9918aa47,"MainField_Obj_MineralBury_A_01_1864777088",
0x2c78a2c6,"MainField_Obj_MineralBury_A_01_1865216033",
0x4d4ef8d5,"MainField_Obj_MineralBury_A_01_1865343806",
0x43c14886,"MainField_Obj_MineralBury_A_01_186675089",
0x65bbce4a,"MainField_Obj_MineralBury_A_01_1870336394",
0xaa206af7,"MainField_Obj_MineralBury_A_01_1878981044",
0xdeece69f,"MainField_Obj_MineralBury_A_01_1884572406",
0x2f2388e5,"MainField_Obj_MineralBury_A_01_1892938939",
0x0c3f63f6,"MainField_Obj_MineralBury_A_01_1895386046",
0x2be59a43,"MainField_Obj_MineralBury_A_01_1896294980",
0xcbfb34c6,"MainField_Obj_MineralBury_A_01_1906157250",
0x40f6bf28,"MainField_Obj_MineralBury_A_01_190625677",
0xf65f3cb6,"MainField_Obj_MineralBury_A_01_1912149899",
0xe1e6725e,"MainField_Obj_MineralBury_A_01_1916204536",
0x76fb9d30,"MainField_Obj_MineralBury_A_01_1924329804",
0x7d9f1c64,"MainField_Obj_MineralBury_A_01_1925431621",
0x278fc787,"MainField_Obj_MineralBury_A_01_1928332954",
0xcd616092,"MainField_Obj_MineralBury_A_01_1933107715",
0x86388c8a,"MainField_Obj_MineralBury_A_01_1933977801",
0xc49cc5f0,"MainField_Obj_MineralBury_A_01_1940614715",
0xea3cfd9a,"MainField_Obj_MineralBury_A_01_1942411499",
0xa1246e54,"MainField_Obj_MineralBury_A_01_194510224",
0x13394b32,"MainField_Obj_MineralBury_A_01_1947449804",
0x11c49994,"MainField_Obj_MineralBury_A_01_1959691253",
0x736c8f9d,"MainField_Obj_MineralBury_A_01_196490342",
0x7307f776,"MainField_Obj_MineralBury_A_01_1974553097",
0x9d0a1c3f,"MainField_Obj_MineralBury_A_01_1976680135",
0x84935cfb,"MainField_Obj_MineralBury_A_01_1979979837",
0xb7076078,"MainField_Obj_MineralBury_A_01_1981750883",
0x66d55062,"MainField_Obj_MineralBury_A_01_2000745633",
0x4fb7f281,"MainField_Obj_MineralBury_A_01_2006481992",
0x8f7069c2,"MainField_Obj_MineralBury_A_01_2017025388",
0x9b801c17,"MainField_Obj_MineralBury_A_01_2022469882",
0xf3bfcdf8,"MainField_Obj_MineralBury_A_01_2026527048",
0x8a92d69f,"MainField_Obj_MineralBury_A_01_203495828",
0x047e3339,"MainField_Obj_MineralBury_A_01_2035136805",
0x3ec68791,"MainField_Obj_MineralBury_A_01_2044363135",
0xbd11335e,"MainField_Obj_MineralBury_A_01_2044871930",
0xe4d49ff0,"MainField_Obj_MineralBury_A_01_205087524",
0x80afae18,"MainField_Obj_MineralBury_A_01_2054054458",
0xb5a23b24,"MainField_Obj_MineralBury_A_01_205711160",
0x1ebad582,"MainField_Obj_MineralBury_A_01_2057494317",
0x540e6e3f,"MainField_Obj_MineralBury_A_01_2067064235",
0xc2a71cb4,"MainField_Obj_MineralBury_A_01_207192125",
0x2e57af3b,"MainField_Obj_MineralBury_A_01_2073417638",
0x4f1c8573,"MainField_Obj_MineralBury_A_01_2076650971",
0xcaab65d0,"MainField_Obj_MineralBury_A_01_2078215262",
0xe5558c0e,"MainField_Obj_MineralBury_A_01_2080836337",
0xcc5223d0,"MainField_Obj_MineralBury_A_01_2085169550",
0x54ad5d0d,"MainField_Obj_MineralBury_A_01_208885376",
0xaaebd0b6,"MainField_Obj_MineralBury_A_01_2093270201",
0x83dc9450,"MainField_Obj_MineralBury_A_01_2099506601",
0x8b7445a0,"MainField_Obj_MineralBury_A_01_2099937216",
0x213a396b,"MainField_Obj_MineralBury_A_01_2106843083",
0x536cc370,"MainField_Obj_MineralBury_A_01_2106851533",
0x2c099dd0,"MainField_Obj_MineralBury_A_01_2115209856",
0x092e885a,"MainField_Obj_MineralBury_A_01_2119625773",
0xee8c79ed,"MainField_Obj_MineralBury_A_01_2124482870",
0x0fbe7553,"MainField_Obj_MineralBury_A_01_212754082",
0xe425ea9b,"MainField_Obj_MineralBury_A_01_2127950056",
0x29929a8a,"MainField_Obj_MineralBury_A_01_2134535940",
0x20b437ec,"MainField_Obj_MineralBury_A_01_2134794398",
0x6cf9948b,"MainField_Obj_MineralBury_A_01_213515057",
0x12c430c1,"MainField_Obj_MineralBury_A_01_2135284786",
0x6177d4eb,"MainField_Obj_MineralBury_A_01_2140830868",
0xd2caab31,"MainField_Obj_MineralBury_A_01_2150986333",
0x546ee543,"MainField_Obj_MineralBury_A_01_2152758846",
0x42c169f2,"MainField_Obj_MineralBury_A_01_2156668857",
0x845dafe3,"MainField_Obj_MineralBury_A_01_2161020513",
0xe67fe648,"MainField_Obj_MineralBury_A_01_2161943983",
0x38239628,"MainField_Obj_MineralBury_A_01_2181505828",
0x1fc0625f,"MainField_Obj_MineralBury_A_01_2181879239",
0x79606ea8,"MainField_Obj_MineralBury_A_01_2184026649",
0xf4b1e9ac,"MainField_Obj_MineralBury_A_01_2196783623",
0xa2aac6fb,"MainField_Obj_MineralBury_A_01_2199605494",
0x333eae9d,"MainField_Obj_MineralBury_A_01_2202907496",
0x5ef39340,"MainField_Obj_MineralBury_A_01_2206053232",
0xe1be48d0,"MainField_Obj_MineralBury_A_01_2212209299",
0x21d88efa,"MainField_Obj_MineralBury_A_01_2220632009",
0x84ad4a0f,"MainField_Obj_MineralBury_A_01_2221306157",
0xbb6781d0,"MainField_Obj_MineralBury_A_01_2221846025",
0x5ce0a549,"MainField_Obj_MineralBury_A_01_2226257929",
0x9473091f,"MainField_Obj_MineralBury_A_01_2227449465",
0xd1f14836,"MainField_Obj_MineralBury_A_01_2229577631",
0x18b4bc21,"MainField_Obj_MineralBury_A_01_2231121514",
0x760074da,"MainField_Obj_MineralBury_A_01_223422744",
0x4989d6c4,"MainField_Obj_MineralBury_A_01_2235976059",
0x92336fa3,"MainField_Obj_MineralBury_A_01_2236399876",
0xd783338c,"MainField_Obj_MineralBury_A_01_223658160",
0x9122b6d2,"MainField_Obj_MineralBury_A_01_2237443017",
0x838a22f0,"MainField_Obj_MineralBury_A_01_224248019",
0x0fa3854e,"MainField_Obj_MineralBury_A_01_2245462893",
0x04ca8d44,"MainField_Obj_MineralBury_A_01_2246128633",
0x3e15a70f,"MainField_Obj_MineralBury_A_01_2247897665",
0x27989001,"MainField_Obj_MineralBury_A_01_224914430",
0x4ed44e08,"MainField_Obj_MineralBury_A_01_2251257407",
0x9a3b61f1,"MainField_Obj_MineralBury_A_01_2253419010",
0x55435128,"MainField_Obj_MineralBury_A_01_2265933556",
0xaddae3ee,"MainField_Obj_MineralBury_A_01_2267361328",
0x25b6349f,"MainField_Obj_MineralBury_A_01_226749581",
0x3d4d3587,"MainField_Obj_MineralBury_A_01_2274997260",
0x38b99272,"MainField_Obj_MineralBury_A_01_2275349878",
0xad49c70d,"MainField_Obj_MineralBury_A_01_2277000521",
0x695da9e9,"MainField_Obj_MineralBury_A_01_2279714901",
0xd9071c36,"MainField_Obj_MineralBury_A_01_2280144203",
0x4886b038,"MainField_Obj_MineralBury_A_01_2281182397",
0x96d24633,"MainField_Obj_MineralBury_A_01_2282868260",
0x6808923a,"MainField_Obj_MineralBury_A_01_2283110131",
0x675dc992,"MainField_Obj_MineralBury_A_01_2285653280",
0x089132ef,"MainField_Obj_MineralBury_A_01_2287028417",
0x15526e0e,"MainField_Obj_MineralBury_A_01_228892400",
0x67a6e8dd,"MainField_Obj_MineralBury_A_01_229232096",
0x0137f5f7,"MainField_Obj_MineralBury_A_01_2297903084",
0x384356e3,"MainField_Obj_MineralBury_A_01_2305427969",
0x7130f5ab,"MainField_Obj_MineralBury_A_01_2319044619",
0xa77239f3,"MainField_Obj_MineralBury_A_01_2325571698",
0x5760cc54,"MainField_Obj_MineralBury_A_01_2338581059",
0x7418f120,"MainField_Obj_MineralBury_A_01_2338864849",
0x7b7359a2,"MainField_Obj_MineralBury_A_01_2345925116",
0xf4b4f9ec,"MainField_Obj_MineralBury_A_01_2347056415",
0x416f1419,"MainField_Obj_MineralBury_A_01_2347871177",
0x4614d9d4,"MainField_Obj_MineralBury_A_01_2349454366",
0x1366a978,"MainField_Obj_MineralBury_A_01_2354877368",
0x11ba2197,"MainField_Obj_MineralBury_A_01_2361009371",
0x8429d192,"MainField_Obj_MineralBury_A_01_237048577",
0xd918bb61,"MainField_Obj_MineralBury_A_01_2378392012",
0xf39be5d3,"MainField_Obj_MineralBury_A_01_2379128169",
0x3ae40df4,"MainField_Obj_MineralBury_A_01_2382198670",
0xf2f95ef8,"MainField_Obj_MineralBury_A_01_2382369616",
0xb48258be,"MainField_Obj_MineralBury_A_01_2392232527",
0x565e4839,"MainField_Obj_MineralBury_A_01_2393007390",
0xd8bc340d,"MainField_Obj_MineralBury_A_01_2393639034",
0x4447f6b6,"MainField_Obj_MineralBury_A_01_2398630084",
0x1098de10,"MainField_Obj_MineralBury_A_01_2412025764",
0x3ae3c193,"MainField_Obj_MineralBury_A_01_2415869106",
0x173c252c,"MainField_Obj_MineralBury_A_01_24186702",
0x07bb3398,"MainField_Obj_MineralBury_A_01_2421228125",
0x603367b7,"MainField_Obj_MineralBury_A_01_2423239871",
0x8a8f3d30,"MainField_Obj_MineralBury_A_01_2428543014",
0x591d8e14,"MainField_Obj_MineralBury_A_01_2435872361",
0x30401aee,"MainField_Obj_MineralBury_A_01_2437572344",
0xc099c1eb,"MainField_Obj_MineralBury_A_01_2440255923",
0xe6041af2,"MainField_Obj_MineralBury_A_01_2443239909",
0x914bd09c,"MainField_Obj_MineralBury_A_01_2448047149",
0x4d4395d7,"MainField_Obj_MineralBury_A_01_2450072070",
0x7aa58db4,"MainField_Obj_MineralBury_A_01_2451097464",
0x7d111f3d,"MainField_Obj_MineralBury_A_01_245599776",
0x0063f273,"MainField_Obj_MineralBury_A_01_2457312580",
0x7e79496a,"MainField_Obj_MineralBury_A_01_246009284",
0xe5229de2,"MainField_Obj_MineralBury_A_01_2460631235",
0x0c4fb4a1,"MainField_Obj_MineralBury_A_01_2467787606",
0xe556dceb,"MainField_Obj_MineralBury_A_01_2471804109",
0x30e5016a,"MainField_Obj_MineralBury_A_01_2472664734",
0x4f10d044,"MainField_Obj_MineralBury_A_01_2476799818",
0x86263e42,"MainField_Obj_MineralBury_A_01_2480215721",
0xa1f1b0fe,"MainField_Obj_MineralBury_A_01_2480720993",
0x1b9e30c0,"MainField_Obj_MineralBury_A_01_2482632224",
0xd618f21e,"MainField_Obj_MineralBury_A_01_2483002263",
0x5d104e71,"MainField_Obj_MineralBury_A_01_2485181780",
0xeb808e31,"MainField_Obj_MineralBury_A_01_2494166928",
0x5428f27c,"MainField_Obj_MineralBury_A_01_2494516695",
0xeed9f78f,"MainField_Obj_MineralBury_A_01_2503999780",
0x7857b8d7,"MainField_Obj_MineralBury_A_01_2507342061",
0x5bb39f75,"MainField_Obj_MineralBury_A_01_2509828894",
0x4e62c749,"MainField_Obj_MineralBury_A_01_2512791347",
0xa91a2cb9,"MainField_Obj_MineralBury_A_01_2515801082",
0x3b68e256,"MainField_Obj_MineralBury_A_01_2517856909",
0x502e747c,"MainField_Obj_MineralBury_A_01_2518570351",
0xfcc61f0c,"MainField_Obj_MineralBury_A_01_2524797946",
0xf5b8fae7,"MainField_Obj_MineralBury_A_01_2525277432",
0xd30de7e6,"MainField_Obj_MineralBury_A_01_2530783165",
0x4f6200b7,"MainField_Obj_MineralBury_A_01_2533771295",
0x6006adba,"MainField_Obj_MineralBury_A_01_2538485204",
0x48dd1d5e,"MainField_Obj_MineralBury_A_01_254490129",
0xe7984ff6,"MainField_Obj_MineralBury_A_01_254563989",
0x55b287a5,"MainField_Obj_MineralBury_A_01_2548534496",
0xd93b0b37,"MainField_Obj_MineralBury_A_01_2552988352",
0xc6092375,"MainField_Obj_MineralBury_A_01_2561904670",
0x1cacec53,"MainField_Obj_MineralBury_A_01_2585286013",
0xc4edb0ce,"MainField_Obj_MineralBury_A_01_2589175646",
0x36961351,"MainField_Obj_MineralBury_A_01_2594027350",
0x2ea30068,"MainField_Obj_MineralBury_A_01_259695971",
0x6dc8443f,"MainField_Obj_MineralBury_A_01_259866990",
0xd4f47f01,"MainField_Obj_MineralBury_A_01_2601579756",
0xac7d16af,"MainField_Obj_MineralBury_A_01_2603109564",
0x4f5be05e,"MainField_Obj_MineralBury_A_01_260402807",
0xce0f29b1,"MainField_Obj_MineralBury_A_01_2604375678",
0xa18203ad,"MainField_Obj_MineralBury_A_01_2610313062",
0x33a9d5d2,"MainField_Obj_MineralBury_A_01_2611900300",
0xcc8ea4a4,"MainField_Obj_MineralBury_A_01_2621856287",
0xd90fcfc5,"MainField_Obj_MineralBury_A_01_2623040765",
0xaae86d6e,"MainField_Obj_MineralBury_A_01_2623308353",
0x8e542a53,"MainField_Obj_MineralBury_A_01_2630087473",
0x7b47eb0e,"MainField_Obj_MineralBury_A_01_2631617987",
0x134f960a,"MainField_Obj_MineralBury_A_01_2641535615",
0xa8870f14,"MainField_Obj_MineralBury_A_01_2643411754",
0xd9a8a26f,"MainField_Obj_MineralBury_A_01_2649755639",
0x94bf5d7d,"MainField_Obj_MineralBury_A_01_2650773214",
0x0ae50cf8,"MainField_Obj_MineralBury_A_01_265174997",
0x8051a524,"MainField_Obj_MineralBury_A_01_2656225173",
0x0908b966,"MainField_Obj_MineralBury_A_01_266533517",
0x03fd8b8c,"MainField_Obj_MineralBury_A_01_2667366339",
0xb25ef6ee,"MainField_Obj_MineralBury_A_01_2669858709",
0x710b8acd,"MainField_Obj_MineralBury_A_01_2672034887",
0xc61430ca,"MainField_Obj_MineralBury_A_01_2673240979",
0xfbc39872,"MainField_Obj_MineralBury_A_01_267938954",
0x52e9615f,"MainField_Obj_MineralBury_A_01_2679593243",
0x7c2982a1,"MainField_Obj_MineralBury_A_01_2684356674",
0x6ea25aec,"MainField_Obj_MineralBury_A_01_2688043541",
0xe2aad9ea,"MainField_Obj_MineralBury_A_01_2692803106",
0xddc64661,"MainField_Obj_MineralBury_A_01_2698030784",
0x7f665e11,"MainField_Obj_MineralBury_A_01_2699679252",
0x0eca6c9c,"MainField_Obj_MineralBury_A_01_2702720156",
0xb38b5b4f,"MainField_Obj_MineralBury_A_01_2707956577",
0x607a97d6,"MainField_Obj_MineralBury_A_01_2708287894",
0x1c108d1d,"MainField_Obj_MineralBury_A_01_272123859",
0x9e9e1737,"MainField_Obj_MineralBury_A_01_2721246542",
0x93f43ece,"MainField_Obj_MineralBury_A_01_2723456043",
0x5fcfa03c,"MainField_Obj_MineralBury_A_01_2731620966",
0x9bf43243,"MainField_Obj_MineralBury_A_01_2732479762",
0xc6b55008,"MainField_Obj_MineralBury_A_01_2733459028",
0xa5311b0f,"MainField_Obj_MineralBury_A_01_2735440554",
0xc7e22bce,"MainField_Obj_MineralBury_A_01_2744221760",
0x8c229464,"MainField_Obj_MineralBury_A_01_2759961130",
0x536c3d0a,"MainField_Obj_MineralBury_A_01_2765902638",
0xb1d2bf1d,"MainField_Obj_MineralBury_A_01_2767247547",
0x835c6cc9,"MainField_Obj_MineralBury_A_01_2779740503",
0xdf3b58a5,"MainField_Obj_MineralBury_A_01_2782162410",
0x616ee1bb,"MainField_Obj_MineralBury_A_01_2785373523",
0xcde3de81,"MainField_Obj_MineralBury_A_01_2785725405",
0x96752917,"MainField_Obj_MineralBury_A_01_2790816741",
0x13869196,"MainField_Obj_MineralBury_A_01_2800397866",
0x4ad6612e,"MainField_Obj_MineralBury_A_01_281296305",
0xcc835856,"MainField_Obj_MineralBury_A_01_2814430952",
0x1c303907,"MainField_Obj_MineralBury_A_01_282155121",
0x28940323,"MainField_Obj_MineralBury_A_01_2821909286",
0x468000cd,"MainField_Obj_MineralBury_A_01_2828462375",
0xdaaaf367,"MainField_Obj_MineralBury_A_01_283045694",
0x57900bb8,"MainField_Obj_MineralBury_A_01_2830585792",
0x42361d83,"MainField_Obj_MineralBury_A_01_2832220226",
0xe5566901,"MainField_Obj_MineralBury_A_01_2836430019",
0x2198f8b4,"MainField_Obj_MineralBury_A_01_2838740963",
0x6c627510,"MainField_Obj_MineralBury_A_01_2840559566",
0x5b3c0738,"MainField_Obj_MineralBury_A_01_2844879266",
0xafa4b4aa,"MainField_Obj_MineralBury_A_01_2849199881",
0x17acf8ed,"MainField_Obj_MineralBury_A_01_2852980711",
0x9fd0a569,"MainField_Obj_MineralBury_A_01_2853199006",
0x6e6c1368,"MainField_Obj_MineralBury_A_01_2854810337",
0xac71e547,"MainField_Obj_MineralBury_A_01_2862960753",
0x68d9c203,"MainField_Obj_MineralBury_A_01_286683059",
0xf78cae9f,"MainField_Obj_MineralBury_A_01_2867084916",
0xc7bd8ac6,"MainField_Obj_MineralBury_A_01_2871356533",
0x7c32091e,"MainField_Obj_MineralBury_A_01_2875402095",
0x585e41cd,"MainField_Obj_MineralBury_A_01_2877805347",
0xea7a20aa,"MainField_Obj_MineralBury_A_01_2885978913",
0xa4737167,"MainField_Obj_MineralBury_A_01_2904730727",
0x8c01cb92,"MainField_Obj_MineralBury_A_01_2907409551",
0x66815414,"MainField_Obj_MineralBury_A_01_2920055109",
0x2fda47d5,"MainField_Obj_MineralBury_A_01_2920596237",
0xaa3a3f38,"MainField_Obj_MineralBury_A_01_2921517610",
0x8cf41002,"MainField_Obj_MineralBury_A_01_2930670288",
0x8c48477f,"MainField_Obj_MineralBury_A_01_2934539228",
0xe8983871,"MainField_Obj_MineralBury_A_01_2936476527",
0x55328172,"MainField_Obj_MineralBury_A_01_2936581532",
0x31682263,"MainField_Obj_MineralBury_A_01_2946457262",
0xc9b3427b,"MainField_Obj_MineralBury_A_01_2952056235",
0xa1e4cbbf,"MainField_Obj_MineralBury_A_01_295554773",
0xd96dbf8e,"MainField_Obj_MineralBury_A_01_2956869493",
0x651881fa,"MainField_Obj_MineralBury_A_01_2960008436",
0x212ab35f,"MainField_Obj_MineralBury_A_01_2974297476",
0x6e367e65,"MainField_Obj_MineralBury_A_01_2980772549",
0x7369023e,"MainField_Obj_MineralBury_A_01_2981674197",
0xbd6d1eae,"MainField_Obj_MineralBury_A_01_2984349179",
0xd28e4677,"MainField_Obj_MineralBury_A_01_2985951572",
0xe8858e93,"MainField_Obj_MineralBury_A_01_2987578054",
0xda5dd90e,"MainField_Obj_MineralBury_A_01_2987968559",
0xf475c8c0,"MainField_Obj_MineralBury_A_01_2989365069",
0x31da9878,"MainField_Obj_MineralBury_A_01_2989851074",
0x9d7a8e95,"MainField_Obj_MineralBury_A_01_2991453088",
0x38c51514,"MainField_Obj_MineralBury_A_01_299256924",
0x062e1596,"MainField_Obj_MineralBury_A_01_3007140049",
0xd7d4d603,"MainField_Obj_MineralBury_A_01_3008965303",
0x58a87555,"MainField_Obj_MineralBury_A_01_30090629",
0x04fa6bea,"MainField_Obj_MineralBury_A_01_3013663039",
0x9086c353,"MainField_Obj_MineralBury_A_01_3016424866",
0xe0626ca9,"MainField_Obj_MineralBury_A_01_3016454205",
0xaa4b916e,"MainField_Obj_MineralBury_A_01_3022549578",
0x3e875b86,"MainField_Obj_MineralBury_A_01_3027903311",
0xa998f635,"MainField_Obj_MineralBury_A_01_3028474007",
0x3f14c102,"MainField_Obj_MineralBury_A_01_3041271937",
0x0d20b74e,"MainField_Obj_MineralBury_A_01_3041766901",
0x7a5d2429,"MainField_Obj_MineralBury_A_01_3044938097",
0x588553e3,"MainField_Obj_MineralBury_A_01_3052286092",
0x47b4cc4e,"MainField_Obj_MineralBury_A_01_3054425767",
0x1858cb55,"MainField_Obj_MineralBury_A_01_305602655",
0xec84d266,"MainField_Obj_MineralBury_A_01_3057833628",
0x93372516,"MainField_Obj_MineralBury_A_01_3060169542",
0x330bf857,"MainField_Obj_MineralBury_A_01_306243288",
0x9e61627f,"MainField_Obj_MineralBury_A_01_3063682032",
0x8d9769ad,"MainField_Obj_MineralBury_A_01_3065402484",
0x99114584,"MainField_Obj_MineralBury_A_01_3070086340",
0xf8554df0,"MainField_Obj_MineralBury_A_01_3071807288",
0xbb09890e,"MainField_Obj_MineralBury_A_01_3076057419",
0xe66e229a,"MainField_Obj_MineralBury_A_01_3076435842",
0xaf431eed,"MainField_Obj_MineralBury_A_01_3078488110",
0x1ef01e4b,"MainField_Obj_MineralBury_A_01_307873876",
0x2c538fba,"MainField_Obj_MineralBury_A_01_3086051155",
0x4afb6809,"MainField_Obj_MineralBury_A_01_3096174161",
0xa23e693b,"MainField_Obj_MineralBury_A_01_3096698738",
0x65043bec,"MainField_Obj_MineralBury_A_01_31140806",
0x0c4377fb,"MainField_Obj_MineralBury_A_01_311622231",
0x08ec7683,"MainField_Obj_MineralBury_A_01_3118694360",
0x7b33be13,"MainField_Obj_MineralBury_A_01_3126731070",
0x862041a6,"MainField_Obj_MineralBury_A_01_3130633958",
0xeaf4c5af,"MainField_Obj_MineralBury_A_01_3138409478",
0xdb58b437,"MainField_Obj_MineralBury_A_01_3141692905",
0xb0dcc543,"MainField_Obj_MineralBury_A_01_315682768",
0x534b1f47,"MainField_Obj_MineralBury_A_01_3158993806",
0xad3aad3d,"MainField_Obj_MineralBury_A_01_3162215297",
0x2a9b8b47,"MainField_Obj_MineralBury_A_01_3164921608",
0x109ba7bd,"MainField_Obj_MineralBury_A_01_3167536490",
0xf3bd2c0e,"MainField_Obj_MineralBury_A_01_3171930067",
0xe1ce8257,"MainField_Obj_MineralBury_A_01_317294110",
0xb40c45d5,"MainField_Obj_MineralBury_A_01_3176265890",
0x4534ce70,"MainField_Obj_MineralBury_A_01_3184514809",
0x3692e049,"MainField_Obj_MineralBury_A_01_3185634568",
0x8acadd07,"MainField_Obj_MineralBury_A_01_3185940334",
0x85f92bc1,"MainField_Obj_MineralBury_A_01_3188984051",
0x3ca87046,"MainField_Obj_MineralBury_A_01_3197354998",
0x06c6889f,"MainField_Obj_MineralBury_A_01_3198597214",
0xf24d6ff3,"MainField_Obj_MineralBury_A_01_3200522281",
0xe464dea7,"MainField_Obj_MineralBury_A_01_3202957480",
0x21faac5c,"MainField_Obj_MineralBury_A_01_3214258256",
0x64e0547c,"MainField_Obj_MineralBury_A_01_321469490",
0x2ea712d1,"MainField_Obj_MineralBury_A_01_3214762802",
0x98fa0700,"MainField_Obj_MineralBury_A_01_3222576778",
0x164f91e1,"MainField_Obj_MineralBury_A_01_3226346227",
0xd2181f3f,"MainField_Obj_MineralBury_A_01_323600074",
0x6581031f,"MainField_Obj_MineralBury_A_01_3241455857",
0xb509d0d2,"MainField_Obj_MineralBury_A_01_324149509",
0x69f6fb51,"MainField_Obj_MineralBury_A_01_3243136262",
0x874e4a63,"MainField_Obj_MineralBury_A_01_3248078817",
0xa3a337b3,"MainField_Obj_MineralBury_A_01_3251652971",
0x9ca7d4ce,"MainField_Obj_MineralBury_A_01_3253054715",
0x67418fdf,"MainField_Obj_MineralBury_A_01_3257968174",
0x34f4dfc7,"MainField_Obj_MineralBury_A_01_3269004238",
0xb337e47c,"MainField_Obj_MineralBury_A_01_3269848943",
0x239ef280,"MainField_Obj_MineralBury_A_01_3280973566",
0x130d3cd5,"MainField_Obj_MineralBury_A_01_328265876",
0xc953fa42,"MainField_Obj_MineralBury_A_01_3285724648",
0x20409b3a,"MainField_Obj_MineralBury_A_01_3292347746",
0x539c0962,"MainField_Obj_MineralBury_A_01_3293686005",
0xd58af19b,"MainField_Obj_MineralBury_A_01_3302943035",
0x032922c9,"MainField_Obj_MineralBury_A_01_3303242643",
0xd38a1251,"MainField_Obj_MineralBury_A_01_3306579112",
0x3b7ee156,"MainField_Obj_MineralBury_A_01_3312272957",
0x965082c7,"MainField_Obj_MineralBury_A_01_3315999117",
0xd962856f,"MainField_Obj_MineralBury_A_01_3316654328",
0x39b9cb60,"MainField_Obj_MineralBury_A_01_3316824060",
0xbbb9b432,"MainField_Obj_MineralBury_A_01_3322328246",
0xe7c1f1bd,"MainField_Obj_MineralBury_A_01_3324586363",
0x6dc1009a,"MainField_Obj_MineralBury_A_01_332991598",
0xcf748378,"MainField_Obj_MineralBury_A_01_3352981915",
0x0c5b7e3b,"MainField_Obj_MineralBury_A_01_3356333134",
0x3a2a5d64,"MainField_Obj_MineralBury_A_01_3357523087",
0x3693e288,"MainField_Obj_MineralBury_A_01_3366599611",
0x2aa3a2b7,"MainField_Obj_MineralBury_A_01_3369041857",
0x5430060b,"MainField_Obj_MineralBury_A_01_3372813390",
0x4193de87,"MainField_Obj_MineralBury_A_01_3384046515",
0x0d1c80fb,"MainField_Obj_MineralBury_A_01_3387453617",
0x363b7e2a,"MainField_Obj_MineralBury_A_01_3393437013",
0x02036120,"MainField_Obj_MineralBury_A_01_3401339291",
0x9ea333e4,"MainField_Obj_MineralBury_A_01_3407255387",
0x5aab8a02,"MainField_Obj_MineralBury_A_01_3410796014",
0xd83f44af,"MainField_Obj_MineralBury_A_01_3413221018",
0xf2abbb58,"MainField_Obj_MineralBury_A_01_3414616581",
0x74add789,"MainField_Obj_MineralBury_A_01_3415133584",
0xda6b1de2,"MainField_Obj_MineralBury_A_01_3419077914",
0xcb844401,"MainField_Obj_MineralBury_A_01_3421328023",
0x768d0e94,"MainField_Obj_MineralBury_A_01_3424235178",
0x705bcf17,"MainField_Obj_MineralBury_A_01_3427728106",
0x4d82a6d5,"MainField_Obj_MineralBury_A_01_3432287251",
0x6b7264fb,"MainField_Obj_MineralBury_A_01_3435324650",
0xe3f2c38b,"MainField_Obj_MineralBury_A_01_3437964828",
0x7bec7c2d,"MainField_Obj_MineralBury_A_01_344151346",
0xdf203bdd,"MainField_Obj_MineralBury_A_01_3442839191",
0x3a94c28b,"MainField_Obj_MineralBury_A_01_3445266489",
0x9412af1e,"MainField_Obj_MineralBury_A_01_3445519102",
0x659a2c4f,"MainField_Obj_MineralBury_A_01_3447062833",
0x61954ae0,"MainField_Obj_MineralBury_A_01_3453764085",
0xbbaa12aa,"MainField_Obj_MineralBury_A_01_3455237546",
0x9cb3adb3,"MainField_Obj_MineralBury_A_01_3457951543",
0x40184e76,"MainField_Obj_MineralBury_A_01_3458123686",
0x04f6a8e6,"MainField_Obj_MineralBury_A_01_3458355837",
0x8834e1e9,"MainField_Obj_MineralBury_A_01_3462562745",
0x1aef1b41,"MainField_Obj_MineralBury_A_01_3472838387",
0xd12341e0,"MainField_Obj_MineralBury_A_01_3473746607",
0xb45dc98a,"MainField_Obj_MineralBury_A_01_3475183466",
0x2fe9fa12,"MainField_Obj_MineralBury_A_01_3476749415",
0x99b10825,"MainField_Obj_MineralBury_A_01_3477073293",
0xfe8b3653,"MainField_Obj_MineralBury_A_01_348136098",
0x53433716,"MainField_Obj_MineralBury_A_01_3481465481",
0x1a8f9134,"MainField_Obj_MineralBury_A_01_348298641",
0xe0db579d,"MainField_Obj_MineralBury_A_01_3487422808",
0xe22078a7,"MainField_Obj_MineralBury_A_01_3489850920",
0xc8258016,"MainField_Obj_MineralBury_A_01_3499774222",
0xec0bb930,"MainField_Obj_MineralBury_A_01_3515591392",
0xf07e4dbc,"MainField_Obj_MineralBury_A_01_3518524659",
0xda751265,"MainField_Obj_MineralBury_A_01_351990061",
0x168ee063,"MainField_Obj_MineralBury_A_01_352333661",
0x43ee6152,"MainField_Obj_MineralBury_A_01_3524208173",
0x654c12bc,"MainField_Obj_MineralBury_A_01_3525585746",
0x8cdddc1e,"MainField_Obj_MineralBury_A_01_3529677904",
0xabf86972,"MainField_Obj_MineralBury_A_01_3530023806",
0x8c2c9eb3,"MainField_Obj_MineralBury_A_01_3544406106",
0x65a85f69,"MainField_Obj_MineralBury_A_01_3554736551",
0x86704ab6,"MainField_Obj_MineralBury_A_01_3559544026",
0xe17a2a2e,"MainField_Obj_MineralBury_A_01_3564539618",
0x71fd63de,"MainField_Obj_MineralBury_A_01_3566246057",
0xb0891717,"MainField_Obj_MineralBury_A_01_3576607628",
0x2aa0c686,"MainField_Obj_MineralBury_A_01_3583684708",
0x7c07366e,"MainField_Obj_MineralBury_A_01_3593926417",
0xef32402a,"MainField_Obj_MineralBury_A_01_3605438996",
0x07ab5929,"MainField_Obj_MineralBury_A_01_360983543",
0x489be0c9,"MainField_Obj_MineralBury_A_01_3615236215",
0x7270d81c,"MainField_Obj_MineralBury_A_01_3617520688",
0x09c4f391,"MainField_Obj_MineralBury_A_01_3622579594",
0x8478b160,"MainField_Obj_MineralBury_A_01_3624023563",
0xa360e75f,"MainField_Obj_MineralBury_A_01_3627069425",
0x1ef7308a,"MainField_Obj_MineralBury_A_01_3628129579",
0x572e2f60,"MainField_Obj_MineralBury_A_01_3628858360",
0x7a889dc0,"MainField_Obj_MineralBury_A_01_3636607097",
0x31070f2b,"MainField_Obj_MineralBury_A_01_3639533833",
0xca68e5eb,"MainField_Obj_MineralBury_A_01_3639875030",
0x696a1390,"MainField_Obj_MineralBury_A_01_3641296762",
0x7fb95963,"MainField_Obj_MineralBury_A_01_3654453341",
0x8aedb6e2,"MainField_Obj_MineralBury_A_01_365589275",
0x9b1cebf6,"MainField_Obj_MineralBury_A_01_3656060699",
0x90e6ca69,"MainField_Obj_MineralBury_A_01_3660072398",
0x2b926277,"MainField_Obj_MineralBury_A_01_3662805679",
0x0c32320c,"MainField_Obj_MineralBury_A_01_3671530761",
0x248abc4a,"MainField_Obj_MineralBury_A_01_367848536",
0x84bc9ee5,"MainField_Obj_MineralBury_A_01_3679135928",
0xe079ae51,"MainField_Obj_MineralBury_A_01_3680673094",
0x480b993c,"MainField_Obj_MineralBury_A_01_3682037829",
0x4266aaa7,"MainField_Obj_MineralBury_A_01_3690291896",
0x5f3be4ac,"MainField_Obj_MineralBury_A_01_3690779504",
0x8e678be5,"MainField_Obj_MineralBury_A_01_3694247773",
0x93842c80,"MainField_Obj_MineralBury_A_01_3715206753",
0x5db0e23f,"MainField_Obj_MineralBury_A_01_3718105522",
0xcf3f2a26,"MainField_Obj_MineralBury_A_01_3720322883",
0x72ab0445,"MainField_Obj_MineralBury_A_01_3728523462",
0x2959f343,"MainField_Obj_MineralBury_A_01_373021506",
0x8df53a51,"MainField_Obj_MineralBury_A_01_3733221728",
0x44ae3196,"MainField_Obj_MineralBury_A_01_3735816502",
0x61275057,"MainField_Obj_MineralBury_A_01_3740065722",
0x5eaac85b,"MainField_Obj_MineralBury_A_01_3745315417",
0x88d5a5ed,"MainField_Obj_MineralBury_A_01_3754414407",
0x80cbc99a,"MainField_Obj_MineralBury_A_01_3761369645",
0x0c608055,"MainField_Obj_MineralBury_A_01_3767686229",
0x541f61ca,"MainField_Obj_MineralBury_A_01_3769861235",
0x9c915398,"MainField_Obj_MineralBury_A_01_3771887312",
0xd1519f68,"MainField_Obj_MineralBury_A_01_3775949676",
0xa719f687,"MainField_Obj_MineralBury_A_01_3777552198",
0x44fcda75,"MainField_Obj_MineralBury_A_01_379217804",
0xc484808f,"MainField_Obj_MineralBury_A_01_3805990657",
0x4ba2485f,"MainField_Obj_MineralBury_A_01_381212422",
0xe28dc60b,"MainField_Obj_MineralBury_A_01_3813056323",
0x1b4dd402,"MainField_Obj_MineralBury_A_01_3813821456",
0x48353897,"MainField_Obj_MineralBury_A_01_3816148993",
0x62259dd5,"MainField_Obj_MineralBury_A_01_3816166012",
0xe0c68b61,"MainField_Obj_MineralBury_A_01_3817082469",
0x95429b99,"MainField_Obj_MineralBury_A_01_3818232756",
0xafd5043f,"MainField_Obj_MineralBury_A_01_3825100886",
0xfa2358b3,"MainField_Obj_MineralBury_A_01_3825222269",
0x36587e36,"MainField_Obj_MineralBury_A_01_3826964940",
0x6858c6f9,"MainField_Obj_MineralBury_A_01_3831599824",
0xe6513b79,"MainField_Obj_MineralBury_A_01_3836941371",
0x6842367b,"MainField_Obj_MineralBury_A_01_3839687383",
0xd0ab40be,"MainField_Obj_MineralBury_A_01_3839827956",
0xd87aaf5e,"MainField_Obj_MineralBury_A_01_3840469819",
0xa2ba2842,"MainField_Obj_MineralBury_A_01_3843467032",
0x768a94c0,"MainField_Obj_MineralBury_A_01_3843480120",
0x9df794e6,"MainField_Obj_MineralBury_A_01_3850371704",
0x78b7073c,"MainField_Obj_MineralBury_A_01_3851767688",
0x03e58a6e,"MainField_Obj_MineralBury_A_01_385745144",
0x544fc465,"MainField_Obj_MineralBury_A_01_3858947296",
0x5f98a076,"MainField_Obj_MineralBury_A_01_3864833913",
0x9a590322,"MainField_Obj_MineralBury_A_01_3864923685",
0xf027abc5,"MainField_Obj_MineralBury_A_01_3867365903",
0xaa86847f,"MainField_Obj_MineralBury_A_01_3873725203",
0x466d8b2c,"MainField_Obj_MineralBury_A_01_3885588171",
0xfa154f5b,"MainField_Obj_MineralBury_A_01_3886761428",
0xafc3751b,"MainField_Obj_MineralBury_A_01_389262562",
0x523f6ec8,"MainField_Obj_MineralBury_A_01_3895032199",
0xe9c08fba,"MainField_Obj_MineralBury_A_01_3895104666",
0xe5c8a135,"MainField_Obj_MineralBury_A_01_3898375352",
0xe8031a84,"MainField_Obj_MineralBury_A_01_3898628516",
0xbc705146,"MainField_Obj_MineralBury_A_01_3900402632",
0x4aa86600,"MainField_Obj_MineralBury_A_01_3906929829",
0xc7cdd0bd,"MainField_Obj_MineralBury_A_01_3911978881",
0x46490cd3,"MainField_Obj_MineralBury_A_01_3912132809",
0x7cde2b94,"MainField_Obj_MineralBury_A_01_3912206855",
0xd16b6d17,"MainField_Obj_MineralBury_A_01_3915296519",
0x5a06a1f6,"MainField_Obj_MineralBury_A_01_3917251261",
0x7f692592,"MainField_Obj_MineralBury_A_01_3922651970",
0x65b9f8d2,"MainField_Obj_MineralBury_A_01_3924762",
0x01e78618,"MainField_Obj_MineralBury_A_01_3927481994",
0x02e3ec20,"MainField_Obj_MineralBury_A_01_3928048355",
0x82c6a1b2,"MainField_Obj_MineralBury_A_01_3930063122",
0x7bb74d8d,"MainField_Obj_MineralBury_A_01_3930963200",
0x3cb56f96,"MainField_Obj_MineralBury_A_01_3934940894",
0x86c60785,"MainField_Obj_MineralBury_A_01_3944405910",
0x191341d0,"MainField_Obj_MineralBury_A_01_3948531529",
0xf94bf5a5,"MainField_Obj_MineralBury_A_01_394899233",
0xf74a789e,"MainField_Obj_MineralBury_A_01_395114547",
0x2d3848ff,"MainField_Obj_MineralBury_A_01_3957981521",
0x33be0e81,"MainField_Obj_MineralBury_A_01_3958675968",
0xe15f8c4d,"MainField_Obj_MineralBury_A_01_3963459605",
0xb75a88fa,"MainField_Obj_MineralBury_A_01_3964707174",
0x18f40eae,"MainField_Obj_MineralBury_A_01_3966217149",
0x309fb192,"MainField_Obj_MineralBury_A_01_3977781238",
0xa13c5db6,"MainField_Obj_MineralBury_A_01_3979734176",
0xc7f173c4,"MainField_Obj_MineralBury_A_01_3981777263",
0xde1c1348,"MainField_Obj_MineralBury_A_01_4000202428",
0x5239af0d,"MainField_Obj_MineralBury_A_01_400205205",
0x31eb0518,"MainField_Obj_MineralBury_A_01_4006350014",
0xfd94ec00,"MainField_Obj_MineralBury_A_01_4009651284",
0x1fb2fea8,"MainField_Obj_MineralBury_A_01_4015353626",
0xbe3b88db,"MainField_Obj_MineralBury_A_01_4018131001",
0xdb6b86da,"MainField_Obj_MineralBury_A_01_4024618897",
0x1a8555eb,"MainField_Obj_MineralBury_A_01_4029379117",
0xb738b7e7,"MainField_Obj_MineralBury_A_01_4029880562",
0xaf017840,"MainField_Obj_MineralBury_A_01_4032418194",
0xe8be0873,"MainField_Obj_MineralBury_A_01_4035472689",
0x7b671abf,"MainField_Obj_MineralBury_A_01_4039395990",
0x4bdc2811,"MainField_Obj_MineralBury_A_01_4040480744",
0x567a4467,"MainField_Obj_MineralBury_A_01_4044331768",
0x256bb42b,"MainField_Obj_MineralBury_A_01_4046787005",
0xcad81a6a,"MainField_Obj_MineralBury_A_01_4052155386",
0x6d85d093,"MainField_Obj_MineralBury_A_01_4064045503",
0x266d81de,"MainField_Obj_MineralBury_A_01_4066282060",
0xc708093c,"MainField_Obj_MineralBury_A_01_4067017744",
0x4c177276,"MainField_Obj_MineralBury_A_01_4070372798",
0xe70aa502,"MainField_Obj_MineralBury_A_01_4071682951",
0x468a8e65,"MainField_Obj_MineralBury_A_01_4072861300",
0xf95531ff,"MainField_Obj_MineralBury_A_01_4073987797",
0xfee62ac1,"MainField_Obj_MineralBury_A_01_4077851623",
0x565db89a,"MainField_Obj_MineralBury_A_01_4078675175",
0x38fea32d,"MainField_Obj_MineralBury_A_01_4082211633",
0x87e2eb9a,"MainField_Obj_MineralBury_A_01_4082394436",
0xa1a69d7f,"MainField_Obj_MineralBury_A_01_4089168619",
0x4c268c87,"MainField_Obj_MineralBury_A_01_4089952793",
0xbc72ad5f,"MainField_Obj_MineralBury_A_01_4093443302",
0xb7592cef,"MainField_Obj_MineralBury_A_01_4094285045",
0x8f2e1b3b,"MainField_Obj_MineralBury_A_01_4105863566",
0xb5a2d4a3,"MainField_Obj_MineralBury_A_01_4111525577",
0x13dce03d,"MainField_Obj_MineralBury_A_01_4114250681",
0x5944df47,"MainField_Obj_MineralBury_A_01_4116682872",
0x02780803,"MainField_Obj_MineralBury_A_01_4117873039",
0x1f12ff6d,"MainField_Obj_MineralBury_A_01_4128444607",
0x38905090,"MainField_Obj_MineralBury_A_01_4129264962",
0x37c095a0,"MainField_Obj_MineralBury_A_01_4138095823",
0xfe7cb4bd,"MainField_Obj_MineralBury_A_01_4143150939",
0xa7328bcb,"MainField_Obj_MineralBury_A_01_4147026423",
0x9a872f4d,"MainField_Obj_MineralBury_A_01_4154438130",
0xcee0532d,"MainField_Obj_MineralBury_A_01_4157510824",
0xbeb82325,"MainField_Obj_MineralBury_A_01_4157578759",
0xb0f21d5a,"MainField_Obj_MineralBury_A_01_4162714035",
0xc650c94b,"MainField_Obj_MineralBury_A_01_416505439",
0xd7da4301,"MainField_Obj_MineralBury_A_01_4165926986",
0x7e829773,"MainField_Obj_MineralBury_A_01_4167086454",
0x94e1e2be,"MainField_Obj_MineralBury_A_01_4167461334",
0xc21ef25d,"MainField_Obj_MineralBury_A_01_4176580977",
0xd93a6b19,"MainField_Obj_MineralBury_A_01_4176807394",
0x5dec7e10,"MainField_Obj_MineralBury_A_01_4178332449",
0x19e89088,"MainField_Obj_MineralBury_A_01_4180815728",
0x7908c8a7,"MainField_Obj_MineralBury_A_01_4183469657",
0x4ab73322,"MainField_Obj_MineralBury_A_01_4183972946",
0x18dde00b,"MainField_Obj_MineralBury_A_01_4184308411",
0x3208a699,"MainField_Obj_MineralBury_A_01_4190524852",
0xdffae9af,"MainField_Obj_MineralBury_A_01_4190957999",
0x281e6d8a,"MainField_Obj_MineralBury_A_01_4193539254",
0xf85482fd,"MainField_Obj_MineralBury_A_01_4195390838",
0xb3add3e3,"MainField_Obj_MineralBury_A_01_419891106",
0xefb48b17,"MainField_Obj_MineralBury_A_01_4201023451",
0x78f7d91f,"MainField_Obj_MineralBury_A_01_420480932",
0x3caea554,"MainField_Obj_MineralBury_A_01_4204942952",
0x6d12b95b,"MainField_Obj_MineralBury_A_01_4220575490",
0xcc1ee970,"MainField_Obj_MineralBury_A_01_4221988073",
0x110c4976,"MainField_Obj_MineralBury_A_01_4223407470",
0xd6070a30,"MainField_Obj_MineralBury_A_01_4224782663",
0xaa46b693,"MainField_Obj_MineralBury_A_01_4234136260",
0x8a3568e0,"MainField_Obj_MineralBury_A_01_4239093691",
0x858c436c,"MainField_Obj_MineralBury_A_01_4240802711",
0xdd5e85f3,"MainField_Obj_MineralBury_A_01_4249565508",
0xe6c4dba4,"MainField_Obj_MineralBury_A_01_4249808316",
0xdd8712af,"MainField_Obj_MineralBury_A_01_424984876",
0x54e791c0,"MainField_Obj_MineralBury_A_01_4258779898",
0x11add4e8,"MainField_Obj_MineralBury_A_01_4259161941",
0x62195149,"MainField_Obj_MineralBury_A_01_4261996086",
0xa24d3337,"MainField_Obj_MineralBury_A_01_4262256350",
0x06d83db9,"MainField_Obj_MineralBury_A_01_426526601",
0x188a0a91,"MainField_Obj_MineralBury_A_01_4270576812",
0xdec95787,"MainField_Obj_MineralBury_A_01_4271656948",
0xb2173c49,"MainField_Obj_MineralBury_A_01_427931862",
0xd7f936a5,"MainField_Obj_MineralBury_A_01_4279531267",
0xca6ab7ad,"MainField_Obj_MineralBury_A_01_428145006",
0x0adba88b,"MainField_Obj_MineralBury_A_01_4281642798",
0x800f2a54,"MainField_Obj_MineralBury_A_01_4283482474",
0x7355c3c9,"MainField_Obj_MineralBury_A_01_4284862866",
0x51bc80ea,"MainField_Obj_MineralBury_A_01_435056692",
0xb3f9ca79,"MainField_Obj_MineralBury_A_01_438258145",
0x9526c59b,"MainField_Obj_MineralBury_A_01_449264556",
0x823859c3,"MainField_Obj_MineralBury_A_01_450136879",
0x33269094,"MainField_Obj_MineralBury_A_01_453417769",
0x5fe97afc,"MainField_Obj_MineralBury_A_01_454040370",
0x740159af,"MainField_Obj_MineralBury_A_01_456098680",
0x1058217e,"MainField_Obj_MineralBury_A_01_457405611",
0xd65ab879,"MainField_Obj_MineralBury_A_01_464550561",
0xf47bfc16,"MainField_Obj_MineralBury_A_01_465667343",
0x3e4b1cd9,"MainField_Obj_MineralBury_A_01_478375635",
0x3ec7ac41,"MainField_Obj_MineralBury_A_01_486366247",
0x6129831d,"MainField_Obj_MineralBury_A_01_490154323",
0xb548cad8,"MainField_Obj_MineralBury_A_01_493607142",
0xe9c92456,"MainField_Obj_MineralBury_A_01_503655785",
0x699a802f,"MainField_Obj_MineralBury_A_01_510619905",
0x360338a4,"MainField_Obj_MineralBury_A_01_517905440",
0xbc47293a,"MainField_Obj_MineralBury_A_01_534652571",
0xa4430fb8,"MainField_Obj_MineralBury_A_01_534954943",
0x840a3db8,"MainField_Obj_MineralBury_A_01_536324483",
0x1fca7daa,"MainField_Obj_MineralBury_A_01_544047821",
0x978ad9bd,"MainField_Obj_MineralBury_A_01_547874670",
0x68b409eb,"MainField_Obj_MineralBury_A_01_548208945",
0xc707711c,"MainField_Obj_MineralBury_A_01_556889504",
0xb75560a9,"MainField_Obj_MineralBury_A_01_561021960",
0xc16474d7,"MainField_Obj_MineralBury_A_01_575261499",
0x58500133,"MainField_Obj_MineralBury_A_01_576421761",
0x28d72397,"MainField_Obj_MineralBury_A_01_577728804",
0x8079cfab,"MainField_Obj_MineralBury_A_01_578207038",
0xa806a678,"MainField_Obj_MineralBury_A_01_579186276",
0x1d1429ff,"MainField_Obj_MineralBury_A_01_579666289",
0x3bd7a644,"MainField_Obj_MineralBury_A_01_580720788",
0x094df33d,"MainField_Obj_MineralBury_A_01_581928099",
0x8e87769d,"MainField_Obj_MineralBury_A_01_582354578",
0x8f9eee08,"MainField_Obj_MineralBury_A_01_584231060",
0x33e786fc,"MainField_Obj_MineralBury_A_01_587978171",
0xdecc2c3e,"MainField_Obj_MineralBury_A_01_594445738",
0xc9dd5172,"MainField_Obj_MineralBury_A_01_59512578",
0x759b0c63,"MainField_Obj_MineralBury_A_01_597633244",
0xd77ceea3,"MainField_Obj_MineralBury_A_01_600284560",
0xa76bfeed,"MainField_Obj_MineralBury_A_01_602135243",
0x34129259,"MainField_Obj_MineralBury_A_01_605791666",
0x60117444,"MainField_Obj_MineralBury_A_01_612134925",
0xec5bf4ea,"MainField_Obj_MineralBury_A_01_617414566",
0xcb421fff,"MainField_Obj_MineralBury_A_01_62079236",
0xbee8668f,"MainField_Obj_MineralBury_A_01_628043939",
0x2537ab23,"MainField_Obj_MineralBury_A_01_631920686",
0x9871d814,"MainField_Obj_MineralBury_A_01_633312378",
0xda7d8ea6,"MainField_Obj_MineralBury_A_01_638060852",
0xe0a790d9,"MainField_Obj_MineralBury_A_01_651615176",
0x5b579706,"MainField_Obj_MineralBury_A_01_652142544",
0xe810bc3f,"MainField_Obj_MineralBury_A_01_656020263",
0xa09010b1,"MainField_Obj_MineralBury_A_01_660821920",
0xb3121b8c,"MainField_Obj_MineralBury_A_01_663113294",
0x36a3408b,"MainField_Obj_MineralBury_A_01_668722154",
0xe64f19a3,"MainField_Obj_MineralBury_A_01_672086714",
0xc727ceb4,"MainField_Obj_MineralBury_A_01_674923604",
0x35b80de9,"MainField_Obj_MineralBury_A_01_676024022",
0x9aeafe38,"MainField_Obj_MineralBury_A_01_681403041",
0x6b77ce38,"MainField_Obj_MineralBury_A_01_684191993",
0xb3f9ecc0,"MainField_Obj_MineralBury_A_01_689152982",
0xc55bad5e,"MainField_Obj_MineralBury_A_01_693185831",
0xdc2befb3,"MainField_Obj_MineralBury_A_01_694011188",
0x7b2de778,"MainField_Obj_MineralBury_A_01_694053471",
0x96dcae79,"MainField_Obj_MineralBury_A_01_69630963",
0x37843070,"MainField_Obj_MineralBury_A_01_696850210",
0xae7cfb97,"MainField_Obj_MineralBury_A_01_704795114",
0xe74d08ad,"MainField_Obj_MineralBury_A_01_706057133",
0xbd47c831,"MainField_Obj_MineralBury_A_01_708572772",
0x17351f21,"MainField_Obj_MineralBury_A_01_719127000",
0xa4033c24,"MainField_Obj_MineralBury_A_01_719929485",
0xa1aa843b,"MainField_Obj_MineralBury_A_01_725860966",
0xdce8822d,"MainField_Obj_MineralBury_A_01_732637299",
0x7e37e687,"MainField_Obj_MineralBury_A_01_738536456",
0x746adcfc,"MainField_Obj_MineralBury_A_01_74630346",
0xe0a96f2c,"MainField_Obj_MineralBury_A_01_750173491",
0xd3039e8e,"MainField_Obj_MineralBury_A_01_750271166",
0xf2f2ac68,"MainField_Obj_MineralBury_A_01_753770478",
0xd7fb8c1f,"MainField_Obj_MineralBury_A_01_758672808",
0x69ae026f,"MainField_Obj_MineralBury_A_01_759268494",
0x009a10ee,"MainField_Obj_MineralBury_A_01_766924145",
0x99076279,"MainField_Obj_MineralBury_A_01_767238179",
0xfe00530a,"MainField_Obj_MineralBury_A_01_76751011",
0xd2427200,"MainField_Obj_MineralBury_A_01_769879963",
0xb9185cf6,"MainField_Obj_MineralBury_A_01_770312732",
0x4b503463,"MainField_Obj_MineralBury_A_01_771177119",
0xfbff1da7,"MainField_Obj_MineralBury_A_01_773130730",
0xb6728ebd,"MainField_Obj_MineralBury_A_01_773298840",
0x0216d5e5,"MainField_Obj_MineralBury_A_01_773884351",
0xea34b150,"MainField_Obj_MineralBury_A_01_785383523",
0x4fa4be6e,"MainField_Obj_MineralBury_A_01_786176236",
0x3f820237,"MainField_Obj_MineralBury_A_01_786690896",
0x577d5f38,"MainField_Obj_MineralBury_A_01_79561692",
0x0f82408d,"MainField_Obj_MineralBury_A_01_796991617",
0x761a8694,"MainField_Obj_MineralBury_A_01_803507324",
0x9522a060,"MainField_Obj_MineralBury_A_01_805297176",
0x7d286eaa,"MainField_Obj_MineralBury_A_01_806663106",
0xd1614f9f,"MainField_Obj_MineralBury_A_01_80979250",
0xe8411db5,"MainField_Obj_MineralBury_A_01_822690035",
0xff7419ec,"MainField_Obj_MineralBury_A_01_837542359",
0x0b7e53ff,"MainField_Obj_MineralBury_A_01_842494266",
0x0486b91d,"MainField_Obj_MineralBury_A_01_844375763",
0xc9ddc434,"MainField_Obj_MineralBury_A_01_846620075",
0x9778279b,"MainField_Obj_MineralBury_A_01_84960146",
0x7c0191a6,"MainField_Obj_MineralBury_A_01_850097818",
0xd82b4507,"MainField_Obj_MineralBury_A_01_853281598",
0xe5e92615,"MainField_Obj_MineralBury_A_01_854402294",
0x929c3c0d,"MainField_Obj_MineralBury_A_01_854559758",
0x086f4cce,"MainField_Obj_MineralBury_A_01_855826941",
0x1290d0a5,"MainField_Obj_MineralBury_A_01_858902987",
0x3507f1d6,"MainField_Obj_MineralBury_A_01_864114136",
0x68a597e3,"MainField_Obj_MineralBury_A_01_864375457",
0x4cbaf377,"MainField_Obj_MineralBury_A_01_864735327",
0x4f8adfd0,"MainField_Obj_MineralBury_A_01_872746272",
0x1631905f,"MainField_Obj_MineralBury_A_01_87649932",
0xd9636103,"MainField_Obj_MineralBury_A_01_878864881",
0x8db33085,"MainField_Obj_MineralBury_A_01_884762630",
0x8b1b0daf,"MainField_Obj_MineralBury_A_01_885914286",
0xeb9bddbe,"MainField_Obj_MineralBury_A_01_887642167",
0x961775a8,"MainField_Obj_MineralBury_A_01_890196173",
0x883019e8,"MainField_Obj_MineralBury_A_01_891030280",
0x538e34fc,"MainField_Obj_MineralBury_A_01_903298793",
0xbdd025ee,"MainField_Obj_MineralBury_A_01_903812535",
0xa557cad8,"MainField_Obj_MineralBury_A_01_904615586",
0xe81126c5,"MainField_Obj_MineralBury_A_01_906217284",
0xc5d1251b,"MainField_Obj_MineralBury_A_01_911353168",
0x861573c2,"MainField_Obj_MineralBury_A_01_916397552",
0x5fa99dd5,"MainField_Obj_MineralBury_A_01_919265168",
0xdfc6a31a,"MainField_Obj_MineralBury_A_01_91960531",
0x05fb7fd6,"MainField_Obj_MineralBury_A_01_924879775",
0x92d76357,"MainField_Obj_MineralBury_A_01_927979564",
0x423e3131,"MainField_Obj_MineralBury_A_01_928251923",
0x181e726d,"MainField_Obj_MineralBury_A_01_928661438",
0x79ebc18f,"MainField_Obj_MineralBury_A_01_929675187",
0xcfd11bde,"MainField_Obj_MineralBury_A_01_930009903",
0xcfad9745,"MainField_Obj_MineralBury_A_01_930508409",
0x375f51b9,"MainField_Obj_MineralBury_A_01_931420764",
0xacd7ea20,"MainField_Obj_MineralBury_A_01_94326859",
0xc434c0b1,"MainField_Obj_MineralBury_A_01_943587238",
0x3b746ffa,"MainField_Obj_MineralBury_A_01_944977402",
0x0fcf4d69,"MainField_Obj_MineralBury_A_01_945269667",
0x481018a4,"MainField_Obj_MineralBury_A_01_947144545",
0x326fd0f9,"MainField_Obj_MineralBury_A_01_947351234",
0xa21000f5,"MainField_Obj_MineralBury_A_01_953674203",
0xc1fb4dee,"MainField_Obj_MineralBury_A_01_961006334",
0xde5001e2,"MainField_Obj_MineralBury_A_01_962523610",
0xfbccd044,"MainField_Obj_MineralBury_A_01_966713867",
0x72cecc21,"MainField_Obj_MineralBury_A_01_966789134",
0x06d66311,"MainField_Obj_MineralBury_A_01_969242606",
0x159b2560,"MainField_Obj_MineralBury_A_01_973085213",
0x067ad254,"MainField_Obj_MineralBury_A_01_973455843",
0x0c6f5470,"MainField_Obj_MineralBury_A_01_973893648",
0x8fdfabb2,"MainField_Obj_MineralBury_A_01_982180856",
0xd384d4b1,"MainField_Obj_MineralBury_A_01_99400868",
0xaf0abf79,"MainField_Obj_MineralBury_A_01_998090138",
0xe8095698,"MainField_Obj_MineralBury_A_01_998923061",
0x8102dfa4,"MainField_Obj_Mineral_A_01_1004530197",
0x7ca59099,"MainField_Obj_Mineral_A_01_1010683511",
0x145fd42e,"MainField_Obj_Mineral_A_01_1012243988",
0xbd88bfbb,"MainField_Obj_Mineral_A_01_1013659751",
0x5decb862,"MainField_Obj_Mineral_A_01_1015198957",
0xa7bb088a,"MainField_Obj_Mineral_A_01_1018192749",
0x129b6a54,"MainField_Obj_Mineral_A_01_1023433213",
0x8313893a,"MainField_Obj_Mineral_A_01_1027660501",
0x468e04e4,"MainField_Obj_Mineral_A_01_1038841101",
0xcc77963f,"MainField_Obj_Mineral_A_01_104406441",
0xeca3e845,"MainField_Obj_Mineral_A_01_1058393261",
0x8c801ce0,"MainField_Obj_Mineral_A_01_1065027390",
0xda6031c6,"MainField_Obj_Mineral_A_01_1070420049",
0x385902a6,"MainField_Obj_Mineral_A_01_1079229863",
0xd88e4d07,"MainField_Obj_Mineral_A_01_1081421622",
0xa9629c0e,"MainField_Obj_Mineral_A_01_1085439672",
0x63f66fd4,"MainField_Obj_Mineral_A_01_1085448544",
0xb572bebb,"MainField_Obj_Mineral_A_01_1093784318",
0xca4e6c09,"MainField_Obj_Mineral_A_01_1096224438",
0x62146f58,"MainField_Obj_Mineral_A_01_1101182995",
0x77ca8c6c,"MainField_Obj_Mineral_A_01_110493523",
0x4644c836,"MainField_Obj_Mineral_A_01_1107304866",
0x48378b06,"MainField_Obj_Mineral_A_01_1116357900",
0x6c1be295,"MainField_Obj_Mineral_A_01_1118791808",
0xae04cc47,"MainField_Obj_Mineral_A_01_1123095022",
0x743e57be,"MainField_Obj_Mineral_A_01_112610868",
0x89a98025,"MainField_Obj_Mineral_A_01_113124622",
0x789fed1b,"MainField_Obj_Mineral_A_01_113853638",
0x2e78646d,"MainField_Obj_Mineral_A_01_1140739885",
0xf0b50f7b,"MainField_Obj_Mineral_A_01_1146196467",
0xee4d5577,"MainField_Obj_Mineral_A_01_1146213164",
0x81ad9f16,"MainField_Obj_Mineral_A_01_1160807451",
0xf286adb8,"MainField_Obj_Mineral_A_01_116750449",
0x1efb677f,"MainField_Obj_Mineral_A_01_1172851382",
0x5efc5dfb,"MainField_Obj_Mineral_A_01_1173909204",
0x50bf2346,"MainField_Obj_Mineral_A_01_1176368567",
0x8a64546c,"MainField_Obj_Mineral_A_01_1181407688",
0x3df0151f,"MainField_Obj_Mineral_A_01_1186457354",
0x1e75f281,"MainField_Obj_Mineral_A_01_1189212162",
0xbc8e2d9b,"MainField_Obj_Mineral_A_01_118930637",
0x7a00a05c,"MainField_Obj_Mineral_A_01_1191720503",
0x3cdd7b94,"MainField_Obj_Mineral_A_01_1192870473",
0x808e197e,"MainField_Obj_Mineral_A_01_1196074443",
0x7ac9e631,"MainField_Obj_Mineral_A_01_1196879410",
0xc101d022,"MainField_Obj_Mineral_A_01_1198225073",
0xea5d649b,"MainField_Obj_Mineral_A_01_1209328525",
0x68909b45,"MainField_Obj_Mineral_A_01_1209914048",
0xa2a7d496,"MainField_Obj_Mineral_A_01_1219214359",
0x6f1a9b68,"MainField_Obj_Mineral_A_01_122052797",
0xc8fdc8a2,"MainField_Obj_Mineral_A_01_1222111239",
0x0638a086,"MainField_Obj_Mineral_A_01_1222901170",
0x3bf83a2d,"MainField_Obj_Mineral_A_01_1223893185",
0xc2dad489,"MainField_Obj_Mineral_A_01_1225515446",
0x878d1d9c,"MainField_Obj_Mineral_A_01_1232338507",
0x93c9ca3a,"MainField_Obj_Mineral_A_01_1236532863",
0x30ada98b,"MainField_Obj_Mineral_A_01_1239249302",
0x2b8b0917,"MainField_Obj_Mineral_A_01_1246471231",
0x03fb4a43,"MainField_Obj_Mineral_A_01_1251546748",
0x11fd52a2,"MainField_Obj_Mineral_A_01_1256870519",
0xeebdf971,"MainField_Obj_Mineral_A_01_1260205587",
0xb1a68a8c,"MainField_Obj_Mineral_A_01_126078558",
0xe58d62f8,"MainField_Obj_Mineral_A_01_1261446836",
0xf54c4655,"MainField_Obj_Mineral_A_01_1263405423",
0x37b1bbd1,"MainField_Obj_Mineral_A_01_1263676213",
0xc43aa2fa,"MainField_Obj_Mineral_A_01_1265631309",
0xb3760d1f,"MainField_Obj_Mineral_A_01_1265707291",
0x31d6c4cb,"MainField_Obj_Mineral_A_01_1270568782",
0x05f43976,"MainField_Obj_Mineral_A_01_1271951143",
0xe88a81c1,"MainField_Obj_Mineral_A_01_1273728850",
0x5b9678c4,"MainField_Obj_Mineral_A_01_1280666516",
0x069eedcd,"MainField_Obj_Mineral_A_01_1283025023",
0x38eca04f,"MainField_Obj_Mineral_A_01_1283180765",
0x1da7b632,"MainField_Obj_Mineral_A_01_1293257569",
0x8064c509,"MainField_Obj_Mineral_A_01_1293509517",
0x44a60c82,"MainField_Obj_Mineral_A_01_1294619296",
0x5aae710a,"MainField_Obj_Mineral_A_01_129772210",
0x64af7097,"MainField_Obj_Mineral_A_01_1310946129",
0x9cf3097b,"MainField_Obj_Mineral_A_01_1315482345",
0x815c3030,"MainField_Obj_Mineral_A_01_1326750244",
0xd86d541a,"MainField_Obj_Mineral_A_01_1331467311",
0x9a2c6b28,"MainField_Obj_Mineral_A_01_1338952405",
0xaa1b4737,"MainField_Obj_Mineral_A_01_1340660319",
0x6eca2c1a,"MainField_Obj_Mineral_A_01_1344600398",
0x72dbc419,"MainField_Obj_Mineral_A_01_1348355727",
0x3c23825d,"MainField_Obj_Mineral_A_01_1353072093",
0xf3d22f33,"MainField_Obj_Mineral_A_01_1353113309",
0x93ac655d,"MainField_Obj_Mineral_A_01_1353777936",
0x443ca33f,"MainField_Obj_Mineral_A_01_1359825369",
0x36fe99d9,"MainField_Obj_Mineral_A_01_1361334451",
0xb06b8d19,"MainField_Obj_Mineral_A_01_1366041529",
0x29c3c901,"MainField_Obj_Mineral_A_01_1368791740",
0x163fe41c,"MainField_Obj_Mineral_A_01_1369169411",
0x01a6c341,"MainField_Obj_Mineral_A_01_1371332794",
0x8fba8069,"MainField_Obj_Mineral_A_01_137416677",
0x0d0a4431,"MainField_Obj_Mineral_A_01_1378629016",
0x8322fc8a,"MainField_Obj_Mineral_A_01_1385151423",
0x8c851f13,"MainField_Obj_Mineral_A_01_1386552310",
0x143879fb,"MainField_Obj_Mineral_A_01_1386751298",
0xe187e714,"MainField_Obj_Mineral_A_01_1387316709",
0xdbb8e22c,"MainField_Obj_Mineral_A_01_1391285128",
0x37869519,"MainField_Obj_Mineral_A_01_1391959655",
0x950708f4,"MainField_Obj_Mineral_A_01_1397917526",
0x3fdd715a,"MainField_Obj_Mineral_A_01_1404115154",
0xa1ea8254,"MainField_Obj_Mineral_A_01_141067604",
0xa587289b,"MainField_Obj_Mineral_A_01_1413137071",
0x4062c6cd,"MainField_Obj_Mineral_A_01_1414235124",
0x7058f90f,"MainField_Obj_Mineral_A_01_141930195",
0x3b25e662,"MainField_Obj_Mineral_A_01_1421033147",
0x05fefb46,"MainField_Obj_Mineral_A_01_1422997741",
0x6c6547be,"MainField_Obj_Mineral_A_01_1424627607",
0x8d96bd1a,"MainField_Obj_Mineral_A_01_1425835401",
0x46ea1174,"MainField_Obj_Mineral_A_01_1428020292",
0xcd3c9cd2,"MainField_Obj_Mineral_A_01_1428301843",
0x33c11eb2,"MainField_Obj_Mineral_A_01_1432561292",
0x5f0c84f4,"MainField_Obj_Mineral_A_01_1435563075",
0x4ab061c0,"MainField_Obj_Mineral_A_01_144226766",
0xa2f54d16,"MainField_Obj_Mineral_A_01_1444974581",
0xdbcec14b,"MainField_Obj_Mineral_A_01_1447831912",
0x22450dd7,"MainField_Obj_Mineral_A_01_1448212175",
0x199a7481,"MainField_Obj_Mineral_A_01_145265099",
0xa4d96d43,"MainField_Obj_Mineral_A_01_145287404",
0x0cfc8bd3,"MainField_Obj_Mineral_A_01_1455962544",
0x0c22e94e,"MainField_Obj_Mineral_A_01_1460949441",
0x22b9d538,"MainField_Obj_Mineral_A_01_1468946239",
0xc9fcda03,"MainField_Obj_Mineral_A_01_1479108965",
0x1e08b9c8,"MainField_Obj_Mineral_A_01_1479507920",
0x49e596b7,"MainField_Obj_Mineral_A_01_1482876259",
0x3e0b5f4d,"MainField_Obj_Mineral_A_01_1488089157",
0x0e7081c0,"MainField_Obj_Mineral_A_01_1491327595",
0x398d8559,"MainField_Obj_Mineral_A_01_1494165177",
0x9a61cf49,"MainField_Obj_Mineral_A_01_1496578981",
0xc4977b52,"MainField_Obj_Mineral_A_01_149991826",
0xb35b8969,"MainField_Obj_Mineral_A_01_1501680156",
0xb20fa917,"MainField_Obj_Mineral_A_01_1504205135",
0xc0796240,"MainField_Obj_Mineral_A_01_1505669573",
0xabf8c479,"MainField_Obj_Mineral_A_01_1507804940",
0x17e3fa94,"MainField_Obj_Mineral_A_01_1509560296",
0x66d04ed9,"MainField_Obj_Mineral_A_01_151003755",
0x8501ec38,"MainField_Obj_Mineral_A_01_1514835993",
0x5425006e,"MainField_Obj_Mineral_A_01_151529509",
0xae12232b,"MainField_Obj_Mineral_A_01_1521653161",
0x5623b3c7,"MainField_Obj_Mineral_A_01_1523360797",
0xedfdb000,"MainField_Obj_Mineral_A_01_1526687077",
0xa9861a20,"MainField_Obj_Mineral_A_01_1527155150",
0x80282af7,"MainField_Obj_Mineral_A_01_1529974669",
0x070010d8,"MainField_Obj_Mineral_A_01_153699797",
0xd03092a2,"MainField_Obj_Mineral_A_01_1545770084",
0x06561a1f,"MainField_Obj_Mineral_A_01_1548095606",
0x63bc36fe,"MainField_Obj_Mineral_A_01_1556546995",
0x633f945c,"MainField_Obj_Mineral_A_01_1563615961",
0xf8cb7a4f,"MainField_Obj_Mineral_A_01_1563798685",
0x3e2383ee,"MainField_Obj_Mineral_A_01_1567771674",
0x4d0e64aa,"MainField_Obj_Mineral_A_01_1580439335",
0x65e6c1d4,"MainField_Obj_Mineral_A_01_1593051768",
0x28bc66a6,"MainField_Obj_Mineral_A_01_1593425862",
0xa21ba2f3,"MainField_Obj_Mineral_A_01_1595874887",
0x39e0e341,"MainField_Obj_Mineral_A_01_1596089819",
0x53e84b79,"MainField_Obj_Mineral_A_01_1596940707",
0xdad63ba4,"MainField_Obj_Mineral_A_01_1607317519",
0xf611e18f,"MainField_Obj_Mineral_A_01_1607904536",
0x94e98afe,"MainField_Obj_Mineral_A_01_1608325779",
0x9ff75252,"MainField_Obj_Mineral_A_01_1610389967",
0x3a7bbfc9,"MainField_Obj_Mineral_A_01_1614195027",
0xb4997f5b,"MainField_Obj_Mineral_A_01_1637963845",
0x73014f37,"MainField_Obj_Mineral_A_01_1638000352",
0xeb58cc88,"MainField_Obj_Mineral_A_01_1642077342",
0x096c6fb0,"MainField_Obj_Mineral_A_01_1643592218",
0x4934f217,"MainField_Obj_Mineral_A_01_1656279485",
0xdddf6730,"MainField_Obj_Mineral_A_01_1659220348",
0xebca8c47,"MainField_Obj_Mineral_A_01_1665695680",
0xfa7ce8b8,"MainField_Obj_Mineral_A_01_1668823926",
0x81f6ad4f,"MainField_Obj_Mineral_A_01_1674299214",
0xe1cd41e2,"MainField_Obj_Mineral_A_01_1676641429",
0x735a756a,"MainField_Obj_Mineral_A_01_1678755012",
0x3c7ad3bf,"MainField_Obj_Mineral_A_01_1679213007",
0x6d366607,"MainField_Obj_Mineral_A_01_168091485",
0xac076349,"MainField_Obj_Mineral_A_01_1682138816",
0xe4034251,"MainField_Obj_Mineral_A_01_1683383447",
0xd512b6b6,"MainField_Obj_Mineral_A_01_1684341744",
0xf427263c,"MainField_Obj_Mineral_A_01_1690853717",
0x73a1ed4b,"MainField_Obj_Mineral_A_01_1700966885",
0x6e432890,"MainField_Obj_Mineral_A_01_1704232701",
0x4c558783,"MainField_Obj_Mineral_A_01_1706165441",
0x1d119e93,"MainField_Obj_Mineral_A_01_1706704391",
0x9062be0a,"MainField_Obj_Mineral_A_01_1710783030",
0x40de9af9,"MainField_Obj_Mineral_A_01_1715848770",
0x1ff66e10,"MainField_Obj_Mineral_A_01_1718734609",
0x140d830e,"MainField_Obj_Mineral_A_01_172095465",
0x93860c27,"MainField_Obj_Mineral_A_01_1729301020",
0x357ebd63,"MainField_Obj_Mineral_A_01_1729532245",
0x72ef31c0,"MainField_Obj_Mineral_A_01_1729980047",
0x63523289,"MainField_Obj_Mineral_A_01_1736635811",
0xd7e08d3b,"MainField_Obj_Mineral_A_01_1740929824",
0xce10116e,"MainField_Obj_Mineral_A_01_1742401237",
0x28dca484,"MainField_Obj_Mineral_A_01_1745409183",
0xe6939cd2,"MainField_Obj_Mineral_A_01_174665053",
0xc4d8358a,"MainField_Obj_Mineral_A_01_17491283",
0xf403dd4d,"MainField_Obj_Mineral_A_01_1749976049",
0xeea152a3,"MainField_Obj_Mineral_A_01_1754742631",
0xd6167adb,"MainField_Obj_Mineral_A_01_1754897735",
0x65e5d049,"MainField_Obj_Mineral_A_01_1764325777",
0xed449741,"MainField_Obj_Mineral_A_01_1770044826",
0x749cbb29,"MainField_Obj_Mineral_A_01_17710302",
0xa5046828,"MainField_Obj_Mineral_A_01_1774114215",
0x33981860,"MainField_Obj_Mineral_A_01_177906715",
0x14d2538a,"MainField_Obj_Mineral_A_01_1779809039",
0x6bd0e4d3,"MainField_Obj_Mineral_A_01_1791517048",
0x8a3a2c85,"MainField_Obj_Mineral_A_01_1792667813",
0xa4a6c0c4,"MainField_Obj_Mineral_A_01_1795393492",
0xa8eb6a4c,"MainField_Obj_Mineral_A_01_1796218569",
0x28e90e75,"MainField_Obj_Mineral_A_01_179797891",
0x9080ef4c,"MainField_Obj_Mineral_A_01_1807368008",
0xe96b1f2c,"MainField_Obj_Mineral_A_01_1810937474",
0xddfbc6d5,"MainField_Obj_Mineral_A_01_1811191568",
0x47ce6255,"MainField_Obj_Mineral_A_01_1822455344",
0x3ea1c1c0,"MainField_Obj_Mineral_A_01_182321699",
0x3c41d461,"MainField_Obj_Mineral_A_01_1830180322",
0x6e89f20b,"MainField_Obj_Mineral_A_01_1832833168",
0x53430897,"MainField_Obj_Mineral_A_01_1836703065",
0xa0df7973,"MainField_Obj_Mineral_A_01_1841050526",
0xa6964685,"MainField_Obj_Mineral_A_01_184286293",
0xecd8488b,"MainField_Obj_Mineral_A_01_1852369177",
0x24b0f28d,"MainField_Obj_Mineral_A_01_1860597037",
0xe8704766,"MainField_Obj_Mineral_A_01_187074083",
0x003b290c,"MainField_Obj_Mineral_A_01_1877349483",
0x6318bb0a,"MainField_Obj_Mineral_A_01_187889625",
0x8d0d2a18,"MainField_Obj_Mineral_A_01_1880065358",
0x9db20232,"MainField_Obj_Mineral_A_01_1880324123",
0x3495bb23,"MainField_Obj_Mineral_A_01_1881922061",
0x6d891879,"MainField_Obj_Mineral_A_01_1883029510",
0xf001dd9a,"MainField_Obj_Mineral_A_01_1886039857",
0x552fb14e,"MainField_Obj_Mineral_A_01_1892869764",
0x7ef7b9bf,"MainField_Obj_Mineral_A_01_1893866517",
0xe39a48c1,"MainField_Obj_Mineral_A_01_1894886744",
0xaa9d70c9,"MainField_Obj_Mineral_A_01_1895690023",
0x24311e2e,"MainField_Obj_Mineral_A_01_1897546006",
0xbaad1d5c,"MainField_Obj_Mineral_A_01_1897582375",
0xb96e12fd,"MainField_Obj_Mineral_A_01_1901288165",
0x2b4341a7,"MainField_Obj_Mineral_A_01_1906816696",
0xc0dca52f,"MainField_Obj_Mineral_A_01_1913329820",
0x81f0ddcb,"MainField_Obj_Mineral_A_01_1915164387",
0x29f18752,"MainField_Obj_Mineral_A_01_1917726553",
0x833494b0,"MainField_Obj_Mineral_A_01_1919136843",
0xe7d43438,"MainField_Obj_Mineral_A_01_1919835295",
0x44a1ea49,"MainField_Obj_Mineral_A_01_1921343897",
0x26101caa,"MainField_Obj_Mineral_A_01_1922143864",
0x4f4e6d0c,"MainField_Obj_Mineral_A_01_192772183",
0xa0d9bed2,"MainField_Obj_Mineral_A_01_1936292338",
0xa5373f0d,"MainField_Obj_Mineral_A_01_1937138338",
0x3e154265,"MainField_Obj_Mineral_A_01_1940262831",
0x798975e1,"MainField_Obj_Mineral_A_01_1946100677",
0x9a43f0a3,"MainField_Obj_Mineral_A_01_1949509486",
0xdfbc7054,"MainField_Obj_Mineral_A_01_1952212135",
0xb62549a7,"MainField_Obj_Mineral_A_01_1954028279",
0x941beb44,"MainField_Obj_Mineral_A_01_1956156010",
0x6aa69d50,"MainField_Obj_Mineral_A_01_1956964757",
0x7c54f5dc,"MainField_Obj_Mineral_A_01_195731078",
0x6ff2962a,"MainField_Obj_Mineral_A_01_1958342370",
0x41137939,"MainField_Obj_Mineral_A_01_1971213355",
0x60340751,"MainField_Obj_Mineral_A_01_1972811045",
0x60d00631,"MainField_Obj_Mineral_A_01_1979749206",
0xdddc2b8c,"MainField_Obj_Mineral_A_01_1980218271",
0xc393cb0c,"MainField_Obj_Mineral_A_01_1980618181",
0x836bd8f8,"MainField_Obj_Mineral_A_01_1984552118",
0xc41d2291,"MainField_Obj_Mineral_A_01_1999622484",
0x794b9d85,"MainField_Obj_Mineral_A_01_2001083924",
0xc2843a0b,"MainField_Obj_Mineral_A_01_2001634762",
0x976238c8,"MainField_Obj_Mineral_A_01_2002919576",
0xcef06ae0,"MainField_Obj_Mineral_A_01_2006807945",
0xad5a8fb1,"MainField_Obj_Mineral_A_01_2014532506",
0x94a9d51c,"MainField_Obj_Mineral_A_01_2017729325",
0x6aaed3f0,"MainField_Obj_Mineral_A_01_2018010748",
0xb83a2dca,"MainField_Obj_Mineral_A_01_202367645",
0x88f08de5,"MainField_Obj_Mineral_A_01_2025378972",
0x93752dbe,"MainField_Obj_Mineral_A_01_2026675461",
0xb38b473c,"MainField_Obj_Mineral_A_01_2028475310",
0x468e8ff9,"MainField_Obj_Mineral_A_01_2032257013",
0x3065c0b0,"MainField_Obj_Mineral_A_01_2033715635",
0xc65c59a3,"MainField_Obj_Mineral_A_01_2036218987",
0xaed3e61e,"MainField_Obj_Mineral_A_01_2040070635",
0x267c9384,"MainField_Obj_Mineral_A_01_2040999424",
0x5b6e608a,"MainField_Obj_Mineral_A_01_2042682532",
0x847056b5,"MainField_Obj_Mineral_A_01_204462544",
0x6cca7425,"MainField_Obj_Mineral_A_01_2045885752",
0x80f0dd29,"MainField_Obj_Mineral_A_01_2049364177",
0xbc1d39e5,"MainField_Obj_Mineral_A_01_2049616923",
0x4db4e6f6,"MainField_Obj_Mineral_A_01_205482037",
0x51a0401e,"MainField_Obj_Mineral_A_01_206932931",
0x815ab143,"MainField_Obj_Mineral_A_01_2070028612",
0x9dbd3477,"MainField_Obj_Mineral_A_01_2070173271",
0x301639b7,"MainField_Obj_Mineral_A_01_2071397793",
0xab1110fd,"MainField_Obj_Mineral_A_01_2079091478",
0x20912199,"MainField_Obj_Mineral_A_01_210159728",
0x67916e96,"MainField_Obj_Mineral_A_01_2103039662",
0xa6c2962f,"MainField_Obj_Mineral_A_01_2113988168",
0x69a8950d,"MainField_Obj_Mineral_A_01_2116206805",
0x93ff9a96,"MainField_Obj_Mineral_A_01_2129121852",
0x7793e898,"MainField_Obj_Mineral_A_01_2129706470",
0x88e23c3d,"MainField_Obj_Mineral_A_01_2132227306",
0xe701856b,"MainField_Obj_Mineral_A_01_2132542061",
0x36b55352,"MainField_Obj_Mineral_A_01_213895862",
0xc5544e08,"MainField_Obj_Mineral_A_01_2152266369",
0xb1ef46ab,"MainField_Obj_Mineral_A_01_2152274580",
0xc771b22f,"MainField_Obj_Mineral_A_01_2163683441",
0xbf4d70d1,"MainField_Obj_Mineral_A_01_2179663666",
0xbd22ab06,"MainField_Obj_Mineral_A_01_2183259001",
0x8fd06968,"MainField_Obj_Mineral_A_01_2183963259",
0x275af5d0,"MainField_Obj_Mineral_A_01_2184657383",
0x25e9af16,"MainField_Obj_Mineral_A_01_2189970486",
0x6d939c49,"MainField_Obj_Mineral_A_01_2193531028",
0x90d57703,"MainField_Obj_Mineral_A_01_2199449326",
0x73d3631d,"MainField_Obj_Mineral_A_01_219961352",
0xf5f29703,"MainField_Obj_Mineral_A_01_2208628221",
0x3c2f94ce,"MainField_Obj_Mineral_A_01_2208636660",
0x23895647,"MainField_Obj_Mineral_A_01_2209376978",
0x472318f8,"MainField_Obj_Mineral_A_01_2213952118",
0x8a27c843,"MainField_Obj_Mineral_A_01_2222485938",
0x22f9c16f,"MainField_Obj_Mineral_A_01_2229318319",
0xdcb57a2d,"MainField_Obj_Mineral_A_01_2229500437",
0x474d95b0,"MainField_Obj_Mineral_A_01_2239376772",
0xf7e37fce,"MainField_Obj_Mineral_A_01_2243664858",
0xa7ad1151,"MainField_Obj_Mineral_A_01_224715799",
0x4c2e0903,"MainField_Obj_Mineral_A_01_2256656037",
0x69061037,"MainField_Obj_Mineral_A_01_226347055",
0xad86b345,"MainField_Obj_Mineral_A_01_226718688",
0x396715f2,"MainField_Obj_Mineral_A_01_2268675101",
0x27fd7233,"MainField_Obj_Mineral_A_01_2270150736",
0x30b95b4c,"MainField_Obj_Mineral_A_01_2277514212",
0xedeea067,"MainField_Obj_Mineral_A_01_2281446792",
0x1d634a6a,"MainField_Obj_Mineral_A_01_2290855912",
0x58acee0f,"MainField_Obj_Mineral_A_01_2296593946",
0xbebb0c69,"MainField_Obj_Mineral_A_01_2297125708",
0x0b678b82,"MainField_Obj_Mineral_A_01_2301433852",
0x4adf77b1,"MainField_Obj_Mineral_A_01_2316418109",
0x225c7da7,"MainField_Obj_Mineral_A_01_2319717475",
0xce5e8352,"MainField_Obj_Mineral_A_01_2319850529",
0x1bc18c90,"MainField_Obj_Mineral_A_01_2324246184",
0xc24b6c24,"MainField_Obj_Mineral_A_01_2324528948",
0x4388352d,"MainField_Obj_Mineral_A_01_2328507504",
0x92be3962,"MainField_Obj_Mineral_A_01_2330826064",
0x48349a6a,"MainField_Obj_Mineral_A_01_2336255006",
0x0f03373a,"MainField_Obj_Mineral_A_01_2340306073",
0x3ad97648,"MainField_Obj_Mineral_A_01_2357004088",
0xa1253731,"MainField_Obj_Mineral_A_01_2360853623",
0x25948f11,"MainField_Obj_Mineral_A_01_2361706846",
0xb989a74f,"MainField_Obj_Mineral_A_01_2372400621",
0xe5f48655,"MainField_Obj_Mineral_A_01_2374733127",
0x2c5f76bd,"MainField_Obj_Mineral_A_01_2377560036",
0xec6a208e,"MainField_Obj_Mineral_A_01_2379037739",
0xd986fbe4,"MainField_Obj_Mineral_A_01_2384822292",
0x7cfe83a0,"MainField_Obj_Mineral_A_01_2387828332",
0x3ddc0734,"MainField_Obj_Mineral_A_01_2389441787",
0xb84c1be1,"MainField_Obj_Mineral_A_01_2393483473",
0xaacd1242,"MainField_Obj_Mineral_A_01_239496727",
0x979dbcd1,"MainField_Obj_Mineral_A_01_2395137420",
0x3b4e7342,"MainField_Obj_Mineral_A_01_2396077903",
0x9ac98fd0,"MainField_Obj_Mineral_A_01_2399809905",
0x8f0f9e3f,"MainField_Obj_Mineral_A_01_240091145",
0xd342b032,"MainField_Obj_Mineral_A_01_2401384477",
0x8ef5eddc,"MainField_Obj_Mineral_A_01_2403933160",
0x5dfeafee,"MainField_Obj_Mineral_A_01_2405238135",
0x2bab066b,"MainField_Obj_Mineral_A_01_2406619778",
0x04b6d53b,"MainField_Obj_Mineral_A_01_2409702266",
0x47d2c19b,"MainField_Obj_Mineral_A_01_2411155617",
0x8953ae53,"MainField_Obj_Mineral_A_01_2418745779",
0x95e48eb7,"MainField_Obj_Mineral_A_01_2420922937",
0x6d5e7c18,"MainField_Obj_Mineral_A_01_2422921980",
0xac1bd31b,"MainField_Obj_Mineral_A_01_2424277157",
0x6fd011b9,"MainField_Obj_Mineral_A_01_2428564777",
0xe5b6e665,"MainField_Obj_Mineral_A_01_2434047448",
0x6d05555c,"MainField_Obj_Mineral_A_01_2437888695",
0xb86c4149,"MainField_Obj_Mineral_A_01_2440575601",
0x290624aa,"MainField_Obj_Mineral_A_01_2440888169",
0x7753e31f,"MainField_Obj_Mineral_A_01_2443281852",
0x8d910bf5,"MainField_Obj_Mineral_A_01_2447077801",
0x402b9da7,"MainField_Obj_Mineral_A_01_244877631",
0xa9c2bd8b,"MainField_Obj_Mineral_A_01_245319949",
0x31ac721d,"MainField_Obj_Mineral_A_01_245749994",
0x7f85ae6f,"MainField_Obj_Mineral_A_01_246023035",
0xaee987c3,"MainField_Obj_Mineral_A_01_2471314027",
0x68ed4dc6,"MainField_Obj_Mineral_A_01_2474645749",
0x28c55ef4,"MainField_Obj_Mineral_A_01_2476377305",
0x18d9c528,"MainField_Obj_Mineral_A_01_2476829808",
0x86c2d590,"MainField_Obj_Mineral_A_01_248123196",
0xd6dd4a91,"MainField_Obj_Mineral_A_01_2482551230",
0xf69dca22,"MainField_Obj_Mineral_A_01_2488227082",
0x1a151ec7,"MainField_Obj_Mineral_A_01_2495256448",
0x6a617129,"MainField_Obj_Mineral_A_01_2496976820",
0x15a91398,"MainField_Obj_Mineral_A_01_2503197316",
0xb303dcae,"MainField_Obj_Mineral_A_01_2505061431",
0x14e8a4a8,"MainField_Obj_Mineral_A_01_2506565718",
0x4853e0e3,"MainField_Obj_Mineral_A_01_2508026976",
0x9a58f126,"MainField_Obj_Mineral_A_01_2509957412",
0x20ab7285,"MainField_Obj_Mineral_A_01_2510148203",
0xdcb279f9,"MainField_Obj_Mineral_A_01_2516509922",
0xbe6f45cd,"MainField_Obj_Mineral_A_01_2516738604",
0xe875384b,"MainField_Obj_Mineral_A_01_2519205221",
0x6fd71329,"MainField_Obj_Mineral_A_01_2520366542",
0xab91c221,"MainField_Obj_Mineral_A_01_2524763241",
0x17566903,"MainField_Obj_Mineral_A_01_2528556464",
0xd7b86e36,"MainField_Obj_Mineral_A_01_2536164958",
0xd29aa90f,"MainField_Obj_Mineral_A_01_2544076466",
0xd1c98c84,"MainField_Obj_Mineral_A_01_2551438023",
0x3b297ee8,"MainField_Obj_Mineral_A_01_255814790",
0xaa0106b9,"MainField_Obj_Mineral_A_01_2559267672",
0x6145b1bb,"MainField_Obj_Mineral_A_01_2568604325",
0x07cf394f,"MainField_Obj_Mineral_A_01_2571866640",
0xaa028b8c,"MainField_Obj_Mineral_A_01_2573240391",
0xf8ccacb4,"MainField_Obj_Mineral_A_01_2574754077",
0x5d2d0bde,"MainField_Obj_Mineral_A_01_2577079573",
0x3c584db6,"MainField_Obj_Mineral_A_01_2579834051",
0x47348a84,"MainField_Obj_Mineral_A_01_2580524281",
0x5b5714b6,"MainField_Obj_Mineral_A_01_2585827031",
0x17d11a3a,"MainField_Obj_Mineral_A_01_2586618600",
0x05cd49b7,"MainField_Obj_Mineral_A_01_2590013085",
0xb8a33724,"MainField_Obj_Mineral_A_01_2595180846",
0x37ce1f93,"MainField_Obj_Mineral_A_01_2605542198",
0xc0b2a2c3,"MainField_Obj_Mineral_A_01_2606111960",
0x90f62f79,"MainField_Obj_Mineral_A_01_2614519155",
0x764af456,"MainField_Obj_Mineral_A_01_2621111348",
0x03f5a9ea,"MainField_Obj_Mineral_A_01_2623732520",
0xe0189e85,"MainField_Obj_Mineral_A_01_263015489",
0x58b19c95,"MainField_Obj_Mineral_A_01_2637018864",
0x9c80b09a,"MainField_Obj_Mineral_A_01_2637194130",
0x58aed313,"MainField_Obj_Mineral_A_01_264233994",
0xfc650f1d,"MainField_Obj_Mineral_A_01_264473331",
0x21d786c0,"MainField_Obj_Mineral_A_01_2653173017",
0x9491451f,"MainField_Obj_Mineral_A_01_2655137669",
0x266c6795,"MainField_Obj_Mineral_A_01_2671083149",
0xf76352cf,"MainField_Obj_Mineral_A_01_2680936294",
0xa3c0d6f7,"MainField_Obj_Mineral_A_01_2681870292",
0xed43682d,"MainField_Obj_Mineral_A_01_2687143044",
0xd56026dd,"MainField_Obj_Mineral_A_01_2687706474",
0x6e7a9a2f,"MainField_Obj_Mineral_A_01_2688078925",
0x2eff190b,"MainField_Obj_Mineral_A_01_2688777229",
0x034d2999,"MainField_Obj_Mineral_A_01_269556625",
0x0a7e91dd,"MainField_Obj_Mineral_A_01_2696037749",
0x2587dc93,"MainField_Obj_Mineral_A_01_2702678189",
0x6b08f203,"MainField_Obj_Mineral_A_01_2705463703",
0x3e4d5baf,"MainField_Obj_Mineral_A_01_2705829930",
0x88c0f75d,"MainField_Obj_Mineral_A_01_2708669696",
0x44a7ec3b,"MainField_Obj_Mineral_A_01_2710710959",
0x846a4af3,"MainField_Obj_Mineral_A_01_271772513",
0xac773712,"MainField_Obj_Mineral_A_01_271946668",
0x121d48fc,"MainField_Obj_Mineral_A_01_2729860951",
0x8c7ff916,"MainField_Obj_Mineral_A_01_2730896544",
0xe803abf5,"MainField_Obj_Mineral_A_01_273216369",
0xfe2d2bc1,"MainField_Obj_Mineral_A_01_2737108749",
0x2556da0b,"MainField_Obj_Mineral_A_01_2737405740",
0x9c4204c2,"MainField_Obj_Mineral_A_01_2739939575",
0x74a2b9ab,"MainField_Obj_Mineral_A_01_2741657758",
0xef51365b,"MainField_Obj_Mineral_A_01_2743315954",
0x5578aa6f,"MainField_Obj_Mineral_A_01_2744605240",
0xec11aeaa,"MainField_Obj_Mineral_A_01_2745003546",
0x74210cb0,"MainField_Obj_Mineral_A_01_2748559942",
0x13a7a63b,"MainField_Obj_Mineral_A_01_2749128690",
0x5d81eb53,"MainField_Obj_Mineral_A_01_274927067",
0xc9aa7d60,"MainField_Obj_Mineral_A_01_2750818224",
0xc067b071,"MainField_Obj_Mineral_A_01_2758750936",
0x5ef1d7f9,"MainField_Obj_Mineral_A_01_2761490959",
0x91d0420a,"MainField_Obj_Mineral_A_01_2765203643",
0xada3cfbf,"MainField_Obj_Mineral_A_01_2766723322",
0x49e1e8be,"MainField_Obj_Mineral_A_01_2769660802",
0xdf8bf3a2,"MainField_Obj_Mineral_A_01_2785817233",
0xe93085c2,"MainField_Obj_Mineral_A_01_2787599150",
0xe6f2ef45,"MainField_Obj_Mineral_A_01_2798846439",
0xceb2b527,"MainField_Obj_Mineral_A_01_2808014120",
0x1c475cbb,"MainField_Obj_Mineral_A_01_2808859859",
0x8c172530,"MainField_Obj_Mineral_A_01_2808959487",
0x3f4629e5,"MainField_Obj_Mineral_A_01_2819503631",
0x67ee1bd5,"MainField_Obj_Mineral_A_01_2827441374",
0x5b6b467f,"MainField_Obj_Mineral_A_01_2832400291",
0x7f525312,"MainField_Obj_Mineral_A_01_2833950364",
0x075ddd59,"MainField_Obj_Mineral_A_01_2834539711",
0x581af523,"MainField_Obj_Mineral_A_01_2836791344",
0x3f88f945,"MainField_Obj_Mineral_A_01_2838545944",
0x74838527,"MainField_Obj_Mineral_A_01_2841340335",
0x27936b3b,"MainField_Obj_Mineral_A_01_2842592066",
0x74418f9a,"MainField_Obj_Mineral_A_01_2842692840",
0x21af254e,"MainField_Obj_Mineral_A_01_284553338",
0x475b8d26,"MainField_Obj_Mineral_A_01_2860068733",
0x704e7ab7,"MainField_Obj_Mineral_A_01_2867791572",
0x217ab359,"MainField_Obj_Mineral_A_01_2869685890",
0xc9726c1e,"MainField_Obj_Mineral_A_01_2874382739",
0x428d10ad,"MainField_Obj_Mineral_A_01_2883691081",
0xcaef4f36,"MainField_Obj_Mineral_A_01_2893545051",
0xeebbf6e2,"MainField_Obj_Mineral_A_01_2920603459",
0x9817ae88,"MainField_Obj_Mineral_A_01_2923131789",
0x1831496a,"MainField_Obj_Mineral_A_01_2924440299",
0x0ce08f97,"MainField_Obj_Mineral_A_01_2924484947",
0x8c4e2127,"MainField_Obj_Mineral_A_01_2937497176",
0x9f2c26fc,"MainField_Obj_Mineral_A_01_2939323908",
0x9057e75a,"MainField_Obj_Mineral_A_01_2940308028",
0xcc824110,"MainField_Obj_Mineral_A_01_2941309250",
0x391828ca,"MainField_Obj_Mineral_A_01_2942010958",
0x2df66073,"MainField_Obj_Mineral_A_01_294201507",
0xa6e5b712,"MainField_Obj_Mineral_A_01_2947188437",
0x2f9c14c8,"MainField_Obj_Mineral_A_01_2947566747",
0x292c9083,"MainField_Obj_Mineral_A_01_2967769995",
0x4643769e,"MainField_Obj_Mineral_A_01_2970793038",
0x4e931118,"MainField_Obj_Mineral_A_01_2971218028",
0xe1ef820d,"MainField_Obj_Mineral_A_01_2971694950",
0xc722e41c,"MainField_Obj_Mineral_A_01_2975790457",
0xbe77d7af,"MainField_Obj_Mineral_A_01_2977938081",
0x260154ce,"MainField_Obj_Mineral_A_01_2984391168",
0x0fc71cb4,"MainField_Obj_Mineral_A_01_2985791774",
0x622880d4,"MainField_Obj_Mineral_A_01_2986031625",
0x89c7f827,"MainField_Obj_Mineral_A_01_298771818",
0xe9ef2d54,"MainField_Obj_Mineral_A_01_2992975099",
0x5e63dc24,"MainField_Obj_Mineral_A_01_3007025816",
0x4fb4b61c,"MainField_Obj_Mineral_A_01_3014641536",
0xbd9a52ba,"MainField_Obj_Mineral_A_01_3016397269",
0xc18d6ac6,"MainField_Obj_Mineral_A_01_3023386804",
0x282b5832,"MainField_Obj_Mineral_A_01_3031270173",
0xb8d7fb30,"MainField_Obj_Mineral_A_01_3037410858",
0xc5a2e4ed,"MainField_Obj_Mineral_A_01_3037636141",
0x8a20d1e0,"MainField_Obj_Mineral_A_01_3051682659",
0x15f9bc41,"MainField_Obj_Mineral_A_01_3052215291",
0xf5f65459,"MainField_Obj_Mineral_A_01_3053004030",
0xd21c6933,"MainField_Obj_Mineral_A_01_3060586801",
0xd81ec2cb,"MainField_Obj_Mineral_A_01_3061651896",
0xbf9197f1,"MainField_Obj_Mineral_A_01_3062900902",
0xffd72bc5,"MainField_Obj_Mineral_A_01_3064438547",
0x063f1f13,"MainField_Obj_Mineral_A_01_3066217744",
0x7b6ad69b,"MainField_Obj_Mineral_A_01_3069033287",
0x5fd875d5,"MainField_Obj_Mineral_A_01_3069242224",
0xcbe51f64,"MainField_Obj_Mineral_A_01_3072377194",
0x09a56c43,"MainField_Obj_Mineral_A_01_3075584752",
0x46ac3ddc,"MainField_Obj_Mineral_A_01_307627800",
0x89e3fea0,"MainField_Obj_Mineral_A_01_307792770",
0x2a8cf75e,"MainField_Obj_Mineral_A_01_307930779",
0xfdbf4868,"MainField_Obj_Mineral_A_01_3080606947",
0x49afdfa4,"MainField_Obj_Mineral_A_01_3096199091",
0xe781dac6,"MainField_Obj_Mineral_A_01_3097453360",
0xf98bfe60,"MainField_Obj_Mineral_A_01_3116022566",
0xd1781c81,"MainField_Obj_Mineral_A_01_3119999257",
0x1d4c7a8c,"MainField_Obj_Mineral_A_01_3121601876",
0x076a07d1,"MainField_Obj_Mineral_A_01_312553611",
0x5692fb39,"MainField_Obj_Mineral_A_01_3128512877",
0x80679c8e,"MainField_Obj_Mineral_A_01_3132269649",
0x97528248,"MainField_Obj_Mineral_A_01_3132792708",
0x8c825e13,"MainField_Obj_Mineral_A_01_3146256212",
0x137c89b5,"MainField_Obj_Mineral_A_01_3150961458",
0xc645467a,"MainField_Obj_Mineral_A_01_3156436838",
0x9c918ce8,"MainField_Obj_Mineral_A_01_3159827558",
0xbdde05f2,"MainField_Obj_Mineral_A_01_3162844425",
0x2575ac96,"MainField_Obj_Mineral_A_01_3168815408",
0xef3923c9,"MainField_Obj_Mineral_A_01_3168997152",
0x4e908881,"MainField_Obj_Mineral_A_01_3172887523",
0x54d879d6,"MainField_Obj_Mineral_A_01_3175459610",
0x4faf0e7a,"MainField_Obj_Mineral_A_01_3175574678",
0xcaa95577,"MainField_Obj_Mineral_A_01_3181338842",
0x700a4c46,"MainField_Obj_Mineral_A_01_3185779314",
0x5b95c923,"MainField_Obj_Mineral_A_01_3187231588",
0x75ce303f,"MainField_Obj_Mineral_A_01_3187519429",
0x8e6f3e27,"MainField_Obj_Mineral_A_01_3187766418",
0x7aba80d5,"MainField_Obj_Mineral_A_01_3188183429",
0xad893daf,"MainField_Obj_Mineral_A_01_3192924067",
0x7bdf50ab,"MainField_Obj_Mineral_A_01_3201643602",
0x78122df0,"MainField_Obj_Mineral_A_01_3203956317",
0xef17049d,"MainField_Obj_Mineral_A_01_3206812116",
0xe0f6c4c6,"MainField_Obj_Mineral_A_01_3207819742",
0xd45f7e27,"MainField_Obj_Mineral_A_01_3211484703",
0x8273ea4b,"MainField_Obj_Mineral_A_01_3213314397",
0x44e04a4e,"MainField_Obj_Mineral_A_01_32152829",
0xf6c48da0,"MainField_Obj_Mineral_A_01_3215697469",
0x2b65c96e,"MainField_Obj_Mineral_A_01_3219078200",
0xd0b6bc29,"MainField_Obj_Mineral_A_01_3227977960",
0x7427f7c1,"MainField_Obj_Mineral_A_01_3234209628",
0x979ce800,"MainField_Obj_Mineral_A_01_3247729784",
0x25d34105,"MainField_Obj_Mineral_A_01_3258685817",
0x769f088b,"MainField_Obj_Mineral_A_01_3261424250",
0x68d76d6c,"MainField_Obj_Mineral_A_01_3262820976",
0xe289936f,"MainField_Obj_Mineral_A_01_3274235165",
0xfc2d9cd8,"MainField_Obj_Mineral_A_01_327781687",
0xeca18c42,"MainField_Obj_Mineral_A_01_3278463677",
0x7dc72136,"MainField_Obj_Mineral_A_01_3287198648",
0xd47f125f,"MainField_Obj_Mineral_A_01_3298163876",
0xc7e90b55,"MainField_Obj_Mineral_A_01_3298518678",
0x476a387a,"MainField_Obj_Mineral_A_01_3302873046",
0x697899a2,"MainField_Obj_Mineral_A_01_3324383840",
0x283ee19d,"MainField_Obj_Mineral_A_01_3343060604",
0xfb5a73d9,"MainField_Obj_Mineral_A_01_3345150496",
0xdf80e067,"MainField_Obj_Mineral_A_01_3347118662",
0x033004c2,"MainField_Obj_Mineral_A_01_3348380618",
0x0f77c547,"MainField_Obj_Mineral_A_01_3351651483",
0xdf44c0a6,"MainField_Obj_Mineral_A_01_3352052205",
0x4b999692,"MainField_Obj_Mineral_A_01_335436694",
0xf3ac8773,"MainField_Obj_Mineral_A_01_3376555365",
0x162c8fa9,"MainField_Obj_Mineral_A_01_3377786732",
0xf30db179,"MainField_Obj_Mineral_A_01_3382338272",
0xc970893d,"MainField_Obj_Mineral_A_01_3385611455",
0xabb387ef,"MainField_Obj_Mineral_A_01_3387746740",
0x543401e4,"MainField_Obj_Mineral_A_01_3391485811",
0x1f79ce2e,"MainField_Obj_Mineral_A_01_3393622238",
0x6b268617,"MainField_Obj_Mineral_A_01_3395035725",
0xc6eb41d9,"MainField_Obj_Mineral_A_01_3398437781",
0xf8c8d765,"MainField_Obj_Mineral_A_01_3410506124",
0xc22d71b6,"MainField_Obj_Mineral_A_01_3415477603",
0xfbb4e69f,"MainField_Obj_Mineral_A_01_341670504",
0x8ed3e1ae,"MainField_Obj_Mineral_A_01_3416779513",
0xbcf353bf,"MainField_Obj_Mineral_A_01_3421377613",
0xfd3834d9,"MainField_Obj_Mineral_A_01_3421586965",
0xebfbec06,"MainField_Obj_Mineral_A_01_342215855",
0x4d8f70db,"MainField_Obj_Mineral_A_01_3425818292",
0x75a8d223,"MainField_Obj_Mineral_A_01_3428055813",
0x2b9c3b44,"MainField_Obj_Mineral_A_01_3428407569",
0x3057229f,"MainField_Obj_Mineral_A_01_3438005084",
0x9474e38d,"MainField_Obj_Mineral_A_01_3447071399",
0x64e8da14,"MainField_Obj_Mineral_A_01_3447472794",
0x0293f3b9,"MainField_Obj_Mineral_A_01_3450265367",
0x50abfc0b,"MainField_Obj_Mineral_A_01_3453209715",
0xdf5a202d,"MainField_Obj_Mineral_A_01_3459961644",
0xdd0a6ae5,"MainField_Obj_Mineral_A_01_3461565582",
0x5e4f0bf1,"MainField_Obj_Mineral_A_01_3463174534",
0xff14049d,"MainField_Obj_Mineral_A_01_3464633296",
0x868c2888,"MainField_Obj_Mineral_A_01_3468098567",
0xbfbf1fcd,"MainField_Obj_Mineral_A_01_3468563712",
0xe0a22b43,"MainField_Obj_Mineral_A_01_3472113706",
0x0baa4447,"MainField_Obj_Mineral_A_01_3473540257",
0xb23bca7b,"MainField_Obj_Mineral_A_01_3491767501",
0xd9643cf3,"MainField_Obj_Mineral_A_01_3502737953",
0xc0d9d57a,"MainField_Obj_Mineral_A_01_3515053353",
0x39ccf262,"MainField_Obj_Mineral_A_01_3517536640",
0x70eb1db1,"MainField_Obj_Mineral_A_01_3520485263",
0x8f5e6017,"MainField_Obj_Mineral_A_01_3525889435",
0xae94cb85,"MainField_Obj_Mineral_A_01_3526347200",
0xb549836f,"MainField_Obj_Mineral_A_01_3527437518",
0x313c909d,"MainField_Obj_Mineral_A_01_3529580457",
0xcda9a7d8,"MainField_Obj_Mineral_A_01_3537437291",
0x86334ab3,"MainField_Obj_Mineral_A_01_3544788554",
0xc19e4622,"MainField_Obj_Mineral_A_01_3552522547",
0xb2ca0a9d,"MainField_Obj_Mineral_A_01_355444716",
0x827e37ae,"MainField_Obj_Mineral_A_01_3557365760",
0x5c766c59,"MainField_Obj_Mineral_A_01_3566886540",
0x3fb06f13,"MainField_Obj_Mineral_A_01_356798050",
0xfa061b4f,"MainField_Obj_Mineral_A_01_3571339917",
0x09c1e622,"MainField_Obj_Mineral_A_01_3574757398",
0x4bcad2ea,"MainField_Obj_Mineral_A_01_3579198037",
0xfd486a6e,"MainField_Obj_Mineral_A_01_3585059427",
0xccc4c84a,"MainField_Obj_Mineral_A_01_3585077808",
0xf1abbf3d,"MainField_Obj_Mineral_A_01_3588126806",
0x5d9b725b,"MainField_Obj_Mineral_A_01_3596557893",
0x75504b3b,"MainField_Obj_Mineral_A_01_3601830765",
0xe887380f,"MainField_Obj_Mineral_A_01_3607641606",
0x3c8794bd,"MainField_Obj_Mineral_A_01_3615021271",
0xfebe20e5,"MainField_Obj_Mineral_A_01_3616345729",
0xec06039d,"MainField_Obj_Mineral_A_01_3622433699",
0x34762b97,"MainField_Obj_Mineral_A_01_3623507898",
0x3afd4879,"MainField_Obj_Mineral_A_01_3624076997",
0x12a5d7e3,"MainField_Obj_Mineral_A_01_3624509277",
0x9759afa1,"MainField_Obj_Mineral_A_01_3625351938",
0x0e3931ab,"MainField_Obj_Mineral_A_01_3625758260",
0xcb84273e,"MainField_Obj_Mineral_A_01_3626387607",
0x1746349e,"MainField_Obj_Mineral_A_01_3626762835",
0xcd665e3c,"MainField_Obj_Mineral_A_01_3632549160",
0xc984fe54,"MainField_Obj_Mineral_A_01_3632981930",
0x8e9c789c,"MainField_Obj_Mineral_A_01_3633607482",
0x21f497d0,"MainField_Obj_Mineral_A_01_363392274",
0x64298635,"MainField_Obj_Mineral_A_01_3646337217",
0xa0eabb8a,"MainField_Obj_Mineral_A_01_364811725",
0xa9159b4e,"MainField_Obj_Mineral_A_01_3648139533",
0xa03ac529,"MainField_Obj_Mineral_A_01_3648474206",
0xa30d69b7,"MainField_Obj_Mineral_A_01_3650696197",
0x9658aac0,"MainField_Obj_Mineral_A_01_3685703125",
0xd88ca238,"MainField_Obj_Mineral_A_01_3690261757",
0x5204fc05,"MainField_Obj_Mineral_A_01_3700598117",
0xa08ea0e4,"MainField_Obj_Mineral_A_01_3704361323",
0x0e6f7c04,"MainField_Obj_Mineral_A_01_3707290188",
0x62f78998,"MainField_Obj_Mineral_A_01_3714036854",
0x9ea2836a,"MainField_Obj_Mineral_A_01_3715508973",
0x950607d6,"MainField_Obj_Mineral_A_01_3719833728",
0x33cde2e3,"MainField_Obj_Mineral_A_01_3722007996",
0x551a910a,"MainField_Obj_Mineral_A_01_372267434",
0x55e6fb8e,"MainField_Obj_Mineral_A_01_3723908746",
0xf07a1730,"MainField_Obj_Mineral_A_01_3727516205",
0xab6a0876,"MainField_Obj_Mineral_A_01_3728358049",
0xf80d70a1,"MainField_Obj_Mineral_A_01_3728791448",
0xb205ccac,"MainField_Obj_Mineral_A_01_3730536025",
0x70f51e3b,"MainField_Obj_Mineral_A_01_3749145786",
0x4cd66005,"MainField_Obj_Mineral_A_01_3751746765",
0x98885da0,"MainField_Obj_Mineral_A_01_3756902158",
0x3fb78761,"MainField_Obj_Mineral_A_01_3760555614",
0x2f4faf0a,"MainField_Obj_Mineral_A_01_3761352253",
0x018dc8df,"MainField_Obj_Mineral_A_01_3764294189",
0x4c7135d4,"MainField_Obj_Mineral_A_01_3768207830",
0x6d95d083,"MainField_Obj_Mineral_A_01_3777048990",
0x799fda2c,"MainField_Obj_Mineral_A_01_3787890036",
0x0ffe6a10,"MainField_Obj_Mineral_A_01_3790074740",
0xac0951e9,"MainField_Obj_Mineral_A_01_3791941779",
0x3de7f541,"MainField_Obj_Mineral_A_01_3793561096",
0x3feed1f6,"MainField_Obj_Mineral_A_01_3803110711",
0x1fbd269e,"MainField_Obj_Mineral_A_01_3813299693",
0x74c5ab05,"MainField_Obj_Mineral_A_01_3816262969",
0x4ebb41c4,"MainField_Obj_Mineral_A_01_3816644990",
0x08ab9df9,"MainField_Obj_Mineral_A_01_3817865088",
0xfec6348b,"MainField_Obj_Mineral_A_01_3821771007",
0x3174871a,"MainField_Obj_Mineral_A_01_3828501675",
0x2489cab6,"MainField_Obj_Mineral_A_01_3831470633",
0xe945806f,"MainField_Obj_Mineral_A_01_3831541474",
0x317b4f8a,"MainField_Obj_Mineral_A_01_3831913717",
0x9dd06d16,"MainField_Obj_Mineral_A_01_3832247020",
0x784d2d61,"MainField_Obj_Mineral_A_01_3833848342",
0x750eabc6,"MainField_Obj_Mineral_A_01_3837245261",
0x4ce16db0,"MainField_Obj_Mineral_A_01_3837392433",
0x46bb9f28,"MainField_Obj_Mineral_A_01_3838862095",
0xef6fc0cb,"MainField_Obj_Mineral_A_01_383939699",
0x86a63d89,"MainField_Obj_Mineral_A_01_384146359",
0x50126625,"MainField_Obj_Mineral_A_01_3841994929",
0x6f0737b5,"MainField_Obj_Mineral_A_01_3862101322",
0x4ddae8dd,"MainField_Obj_Mineral_A_01_3869361696",
0xcb1b19c2,"MainField_Obj_Mineral_A_01_3872913384",
0x0a4825bb,"MainField_Obj_Mineral_A_01_3878806248",
0x60624f75,"MainField_Obj_Mineral_A_01_3880565484",
0x4251475d,"MainField_Obj_Mineral_A_01_3881606160",
0xd9e73d46,"MainField_Obj_Mineral_A_01_388343457",
0x4de8e85c,"MainField_Obj_Mineral_A_01_3884246444",
0xfcbf169f,"MainField_Obj_Mineral_A_01_3887588298",
0x7bfe9650,"MainField_Obj_Mineral_A_01_3896732848",
0x0977174d,"MainField_Obj_Mineral_A_01_3898092914",
0xfc65304b,"MainField_Obj_Mineral_A_01_3899918371",
0xb9d33c18,"MainField_Obj_Mineral_A_01_3913698322",
0x85fc0b7b,"MainField_Obj_Mineral_A_01_3924530576",
0xafb9991b,"MainField_Obj_Mineral_A_01_3927079044",
0x96b9ea51,"MainField_Obj_Mineral_A_01_3939535734",
0xc1bcf94f,"MainField_Obj_Mineral_A_01_3946722748",
0x0b5adc99,"MainField_Obj_Mineral_A_01_3946892782",
0x78a09c85,"MainField_Obj_Mineral_A_01_3953512278",
0xdc13cfef,"MainField_Obj_Mineral_A_01_3954991909",
0x6ff7c0a1,"MainField_Obj_Mineral_A_01_3959080359",
0x5e17a17a,"MainField_Obj_Mineral_A_01_3962939356",
0x709320fd,"MainField_Obj_Mineral_A_01_3963114209",
0x62bfd128,"MainField_Obj_Mineral_A_01_3987533716",
0xe45ee2cc,"MainField_Obj_Mineral_A_01_3999224039",
0xf92764ce,"MainField_Obj_Mineral_A_01_4004663542",
0xe97e0bc5,"MainField_Obj_Mineral_A_01_4008960085",
0x40fb8ba5,"MainField_Obj_Mineral_A_01_4010782863",
0x0ce09e2f,"MainField_Obj_Mineral_A_01_4023223252",
0x00a54451,"MainField_Obj_Mineral_A_01_4025381790",
0x7b86d573,"MainField_Obj_Mineral_A_01_4029230715",
0xde1734ac,"MainField_Obj_Mineral_A_01_4032504673",
0x405d6d7e,"MainField_Obj_Mineral_A_01_4034468869",
0x0869c6fc,"MainField_Obj_Mineral_A_01_4035912599",
0x83c74ae4,"MainField_Obj_Mineral_A_01_404365907",
0xb3f083a0,"MainField_Obj_Mineral_A_01_4049089686",
0x39359190,"MainField_Obj_Mineral_A_01_4051263384",
0xf40abbef,"MainField_Obj_Mineral_A_01_4053528891",
0x794054e2,"MainField_Obj_Mineral_A_01_4053864461",
0x1199af22,"MainField_Obj_Mineral_A_01_4054273446",
0x14d31148,"MainField_Obj_Mineral_A_01_4056591241",
0x0353efc4,"MainField_Obj_Mineral_A_01_4079795105",
0x1be70323,"MainField_Obj_Mineral_A_01_408184842",
0xd26333de,"MainField_Obj_Mineral_A_01_4083845373",
0x010d2435,"MainField_Obj_Mineral_A_01_4093429254",
0xa786122c,"MainField_Obj_Mineral_A_01_4094943625",
0x23ff64fd,"MainField_Obj_Mineral_A_01_4095071995",
0x1b4e2bb6,"MainField_Obj_Mineral_A_01_4096326201",
0xd2ada381,"MainField_Obj_Mineral_A_01_4108168785",
0x1169c738,"MainField_Obj_Mineral_A_01_410964261",
0x3d976eaa,"MainField_Obj_Mineral_A_01_4112809130",
0x18f0812c,"MainField_Obj_Mineral_A_01_4114825435",
0x176ffaab,"MainField_Obj_Mineral_A_01_4116248749",
0x0fa6136b,"MainField_Obj_Mineral_A_01_4131503466",
0x169c51bd,"MainField_Obj_Mineral_A_01_4143120084",
0xc2ae3d64,"MainField_Obj_Mineral_A_01_4154757306",
0x3c837a88,"MainField_Obj_Mineral_A_01_4160989925",
0xe1fc69a2,"MainField_Obj_Mineral_A_01_4162793515",
0xab1cee82,"MainField_Obj_Mineral_A_01_4176921443",
0x3b586974,"MainField_Obj_Mineral_A_01_4183573847",
0x35ed96dc,"MainField_Obj_Mineral_A_01_4191733818",
0x2722de36,"MainField_Obj_Mineral_A_01_4192840315",
0x6062009b,"MainField_Obj_Mineral_A_01_4197760947",
0x043caea7,"MainField_Obj_Mineral_A_01_4198575738",
0x7fcdf74a,"MainField_Obj_Mineral_A_01_4198910056",
0xc7129763,"MainField_Obj_Mineral_A_01_4199261206",
0xf4010d2d,"MainField_Obj_Mineral_A_01_4211106452",
0xd911a075,"MainField_Obj_Mineral_A_01_4218091405",
0xca4dedec,"MainField_Obj_Mineral_A_01_421847305",
0x83058331,"MainField_Obj_Mineral_A_01_4218656372",
0x819d17a8,"MainField_Obj_Mineral_A_01_4220865205",
0x020ccb7c,"MainField_Obj_Mineral_A_01_4223081610",
0xca6554b1,"MainField_Obj_Mineral_A_01_4223235244",
0x01eaf3a9,"MainField_Obj_Mineral_A_01_4232712909",
0xafc5b946,"MainField_Obj_Mineral_A_01_4234762408",
0xe5272cdd,"MainField_Obj_Mineral_A_01_4248289633",
0x117c784b,"MainField_Obj_Mineral_A_01_425336771",
0xa6b4876a,"MainField_Obj_Mineral_A_01_425954800",
0x2b1f96f8,"MainField_Obj_Mineral_A_01_4259904781",
0x835926c4,"MainField_Obj_Mineral_A_01_4259986917",
0x30eb877f,"MainField_Obj_Mineral_A_01_4262427812",
0x994e27ee,"MainField_Obj_Mineral_A_01_4263775141",
0xe9e44603,"MainField_Obj_Mineral_A_01_428352376",
0xb0383364,"MainField_Obj_Mineral_A_01_4289002481",
0x62d6d2fc,"MainField_Obj_Mineral_A_01_4290127037",
0x6a92a8ca,"MainField_Obj_Mineral_A_01_4291776366",
0x8f3ff8d8,"MainField_Obj_Mineral_A_01_4294638394",
0x940810eb,"MainField_Obj_Mineral_A_01_429797345",
0x6854972e,"MainField_Obj_Mineral_A_01_432332036",
0x08e17f9c,"MainField_Obj_Mineral_A_01_433597895",
0xae33ed25,"MainField_Obj_Mineral_A_01_434526459",
0xaef3b128,"MainField_Obj_Mineral_A_01_436558088",
0x6560c12a,"MainField_Obj_Mineral_A_01_437152635",
0x5555e6c4,"MainField_Obj_Mineral_A_01_438810753",
0xeb58ad47,"MainField_Obj_Mineral_A_01_443957616",
0xe1d5aa47,"MainField_Obj_Mineral_A_01_44533265",
0x230a404a,"MainField_Obj_Mineral_A_01_447508886",
0x084ee42b,"MainField_Obj_Mineral_A_01_451772271",
0x3ab13c78,"MainField_Obj_Mineral_A_01_45485863",
0xb1b7f71a,"MainField_Obj_Mineral_A_01_455016711",
0x341f81f5,"MainField_Obj_Mineral_A_01_460677922",
0x2db064dc,"MainField_Obj_Mineral_A_01_462584274",
0x4395e551,"MainField_Obj_Mineral_A_01_467132562",
0x3c72d046,"MainField_Obj_Mineral_A_01_47452024",
0x371bf565,"MainField_Obj_Mineral_A_01_480597547",
0xf26f6110,"MainField_Obj_Mineral_A_01_481889243",
0x5d99ae9c,"MainField_Obj_Mineral_A_01_484214768",
0xdbf6c921,"MainField_Obj_Mineral_A_01_486546779",
0x6afb6bf1,"MainField_Obj_Mineral_A_01_488093774",
0x0e7de5ba,"MainField_Obj_Mineral_A_01_491444813",
0xbf771c52,"MainField_Obj_Mineral_A_01_493718423",
0x06ca24ec,"MainField_Obj_Mineral_A_01_500594714",
0x56df05bc,"MainField_Obj_Mineral_A_01_500927960",
0x6f69239f,"MainField_Obj_Mineral_A_01_506405515",
0x003b2e6e,"MainField_Obj_Mineral_A_01_512354835",
0xbca9cec1,"MainField_Obj_Mineral_A_01_513609038",
0xa7747c27,"MainField_Obj_Mineral_A_01_514518912",
0xbaae2a4f,"MainField_Obj_Mineral_A_01_524126187",
0xac2118aa,"MainField_Obj_Mineral_A_01_530999385",
0x72f1f73a,"MainField_Obj_Mineral_A_01_532773649",
0x92b95816,"MainField_Obj_Mineral_A_01_533129334",
0x0cb03c05,"MainField_Obj_Mineral_A_01_544106768",
0xea6300bc,"MainField_Obj_Mineral_A_01_545576379",
0x3dea6911,"MainField_Obj_Mineral_A_01_551645351",
0x2def95b8,"MainField_Obj_Mineral_A_01_558718958",
0xd2cdb81c,"MainField_Obj_Mineral_A_01_568866995",
0xf516f980,"MainField_Obj_Mineral_A_01_571293536",
0xde30a37e,"MainField_Obj_Mineral_A_01_571923943",
0x9e33085d,"MainField_Obj_Mineral_A_01_578793215",
0xdb0606b0,"MainField_Obj_Mineral_A_01_581948758",
0x35f9df92,"MainField_Obj_Mineral_A_01_583265606",
0xf5ef4b35,"MainField_Obj_Mineral_A_01_583277781",
0xe4f13ed4,"MainField_Obj_Mineral_A_01_585514497",
0x21b7029a,"MainField_Obj_Mineral_A_01_598376390",
0xcdbe2fff,"MainField_Obj_Mineral_A_01_606149549",
0xe80fb4a9,"MainField_Obj_Mineral_A_01_618267989",
0x020404ad,"MainField_Obj_Mineral_A_01_618759776",
0x3400a7ad,"MainField_Obj_Mineral_A_01_623049171",
0x03f9c567,"MainField_Obj_Mineral_A_01_624516708",
0x55dab02d,"MainField_Obj_Mineral_A_01_632078936",
0x4689ced4,"MainField_Obj_Mineral_A_01_639146966",
0x3899cdfd,"MainField_Obj_Mineral_A_01_644210567",
0xb7538658,"MainField_Obj_Mineral_A_01_64828240",
0x1407b0f3,"MainField_Obj_Mineral_A_01_659870911",
0x0071123f,"MainField_Obj_Mineral_A_01_660105836",
0xfa416024,"MainField_Obj_Mineral_A_01_666830955",
0x6f5f8a03,"MainField_Obj_Mineral_A_01_669443237",
0x57bd5199,"MainField_Obj_Mineral_A_01_670655946",
0x6a8f0eb5,"MainField_Obj_Mineral_A_01_672436347",
0xda9eb26e,"MainField_Obj_Mineral_A_01_677180918",
0x017d882f,"MainField_Obj_Mineral_A_01_677758469",
0x4c2e24bc,"MainField_Obj_Mineral_A_01_680079076",
0x67afe324,"MainField_Obj_Mineral_A_01_687151696",
0x91624f98,"MainField_Obj_Mineral_A_01_688904234",
0xb9941e11,"MainField_Obj_Mineral_A_01_689784023",
0xf1de496b,"MainField_Obj_Mineral_A_01_697596291",
0x23d4aa16,"MainField_Obj_Mineral_A_01_698643080",
0x09e018c5,"MainField_Obj_Mineral_A_01_70299346",
0x01717256,"MainField_Obj_Mineral_A_01_710772569",
0x211c8795,"MainField_Obj_Mineral_A_01_719442535",
0xd0dadbe6,"MainField_Obj_Mineral_A_01_723267214",
0x6af5be64,"MainField_Obj_Mineral_A_01_72535915",
0xea707b5f,"MainField_Obj_Mineral_A_01_726833052",
0xc4e02fe7,"MainField_Obj_Mineral_A_01_727082096",
0x9db62ad7,"MainField_Obj_Mineral_A_01_729253696",
0x12ea5912,"MainField_Obj_Mineral_A_01_72967708",
0x6143b1b3,"MainField_Obj_Mineral_A_01_731050640",
0x5bb4bf5d,"MainField_Obj_Mineral_A_01_741480779",
0xa715f6c5,"MainField_Obj_Mineral_A_01_747156869",
0xeb77e475,"MainField_Obj_Mineral_A_01_748130479",
0x10c4b878,"MainField_Obj_Mineral_A_01_761894747",
0x57e195f2,"MainField_Obj_Mineral_A_01_762383521",
0x6a445c16,"MainField_Obj_Mineral_A_01_77556199",
0xf598d7da,"MainField_Obj_Mineral_A_01_776907967",
0x38d6be11,"MainField_Obj_Mineral_A_01_777238598",
0xeffcebd0,"MainField_Obj_Mineral_A_01_778558215",
0x1d8b7327,"MainField_Obj_Mineral_A_01_780045202",
0xfc7e3ec6,"MainField_Obj_Mineral_A_01_780209243",
0x5af42066,"MainField_Obj_Mineral_A_01_780230144",
0x6c2ce302,"MainField_Obj_Mineral_A_01_790903595",
0x9e6929e6,"MainField_Obj_Mineral_A_01_791545764",
0xe3cbf4f6,"MainField_Obj_Mineral_A_01_796943989",
0x616dfeca,"MainField_Obj_Mineral_A_01_802650073",
0x8adc69d6,"MainField_Obj_Mineral_A_01_803679703",
0xa2d0c97f,"MainField_Obj_Mineral_A_01_80602662",
0x316a4f53,"MainField_Obj_Mineral_A_01_807510595",
0xe3fa8088,"MainField_Obj_Mineral_A_01_810671780",
0xc5829397,"MainField_Obj_Mineral_A_01_812926498",
0xf73878ea,"MainField_Obj_Mineral_A_01_814809244",
0x37584007,"MainField_Obj_Mineral_A_01_815335740",
0x0fe3614f,"MainField_Obj_Mineral_A_01_816805678",
0xd2bc4e9d,"MainField_Obj_Mineral_A_01_817487390",
0x36961f97,"MainField_Obj_Mineral_A_01_822957499",
0xb91a3e3e,"MainField_Obj_Mineral_A_01_825387833",
0x0c92016b,"MainField_Obj_Mineral_A_01_825723564",
0x4d4d4180,"MainField_Obj_Mineral_A_01_830631926",
0xc0ff515c,"MainField_Obj_Mineral_A_01_835595470",
0xb2684dd1,"MainField_Obj_Mineral_A_01_83888347",
0x82a8b26d,"MainField_Obj_Mineral_A_01_843559706",
0xc93e2395,"MainField_Obj_Mineral_A_01_844554807",
0x27644b0d,"MainField_Obj_Mineral_A_01_857785246",
0x20f5d556,"MainField_Obj_Mineral_A_01_875388024",
0x2dd562bf,"MainField_Obj_Mineral_A_01_880405548",
0x9f0457ae,"MainField_Obj_Mineral_A_01_882976287",
0x39b1f9ab,"MainField_Obj_Mineral_A_01_886577484",
0x88dd1054,"MainField_Obj_Mineral_A_01_893225507",
0x972dde1f,"MainField_Obj_Mineral_A_01_898203522",
0x8f2c5786,"MainField_Obj_Mineral_A_01_89943481",
0x7474c6d4,"MainField_Obj_Mineral_A_01_899741656",
0x03fae63c,"MainField_Obj_Mineral_A_01_902983481",
0x6ee1459d,"MainField_Obj_Mineral_A_01_903501403",
0xeeb17a70,"MainField_Obj_Mineral_A_01_904522537",
0x7af33c41,"MainField_Obj_Mineral_A_01_909958302",
0x757f7393,"MainField_Obj_Mineral_A_01_91314597",
0xf29cebcc,"MainField_Obj_Mineral_A_01_915886685",
0xa0f104f4,"MainField_Obj_Mineral_A_01_916408901",
0x6c1ac2e2,"MainField_Obj_Mineral_A_01_920205609",
0x9c389ba7,"MainField_Obj_Mineral_A_01_941609696",
0xc7e50f10,"MainField_Obj_Mineral_A_01_94427902",
0x0fa60acb,"MainField_Obj_Mineral_A_01_950489481",
0xaf930da2,"MainField_Obj_Mineral_A_01_95394739",
0xa9c88583,"MainField_Obj_Mineral_A_01_954843720",
0x1a08a3b1,"MainField_Obj_Mineral_A_01_956980751",
0x686bac77,"MainField_Obj_Mineral_A_01_957834502",
0xd8d3b4b9,"MainField_Obj_Mineral_A_01_961001734",
0x30d91ec2,"MainField_Obj_Mineral_A_01_964751893",
0xde99d406,"MainField_Obj_Mineral_A_01_965490716",
0xe4e2b236,"MainField_Obj_Mineral_A_01_971384887",
0xef97d7cd,"MainField_Obj_Mineral_A_01_971741775",
0x872a1312,"MainField_Obj_Mineral_A_01_977397405",
0x5fd1a0bf,"MainField_Obj_Mineral_A_01_981900138",
0x80fbbed3,"MainField_Obj_Mineral_A_01_993461434",
0xe658dd91,"MainField_Obj_Mineral_A_01_999941234",
0x3e9e72ea,"MainField_Obj_Mineral_B_01_1007455676",
0x584606e1,"MainField_Obj_Mineral_B_01_1050790392",
0x19fc8648,"MainField_Obj_Mineral_B_01_1055002056",
0xcf733f5b,"MainField_Obj_Mineral_B_01_1092284746",
0x74bf984a,"MainField_Obj_Mineral_B_01_111288497",
0xde796cc3,"MainField_Obj_Mineral_B_01_112050776",
0xf474d740,"MainField_Obj_Mineral_B_01_1132920607",
0xb2b18752,"MainField_Obj_Mineral_B_01_1133754454",
0x0bc23189,"MainField_Obj_Mineral_B_01_1156713628",
0xa3469447,"MainField_Obj_Mineral_B_01_1164452025",
0x4c4505a1,"MainField_Obj_Mineral_B_01_1166196161",
0xbe0a96b9,"MainField_Obj_Mineral_B_01_1216594190",
0xf4f85b8b,"MainField_Obj_Mineral_B_01_1220007492",
0x5b85f81c,"MainField_Obj_Mineral_B_01_1264626612",
0x37c14523,"MainField_Obj_Mineral_B_01_1319072158",
0x14f463df,"MainField_Obj_Mineral_B_01_1331523000",
0xc0a74211,"MainField_Obj_Mineral_B_01_1351744774",
0x1b14c3ca,"MainField_Obj_Mineral_B_01_1352030017",
0xb1f9d436,"MainField_Obj_Mineral_B_01_1353250597",
0x4970921b,"MainField_Obj_Mineral_B_01_135859665",
0x2508ffd2,"MainField_Obj_Mineral_B_01_1374566661",
0xf976e05e,"MainField_Obj_Mineral_B_01_138496649",
0x8eb2a799,"MainField_Obj_Mineral_B_01_1414133093",
0x468f98f1,"MainField_Obj_Mineral_B_01_1426047748",
0xd494458f,"MainField_Obj_Mineral_B_01_1453066815",
0x4a7fa56f,"MainField_Obj_Mineral_B_01_1458515555",
0x4fd2fcc8,"MainField_Obj_Mineral_B_01_1460577356",
0x70ed299d,"MainField_Obj_Mineral_B_01_1468420204",
0x99c09509,"MainField_Obj_Mineral_B_01_1470887033",
0x0e2db002,"MainField_Obj_Mineral_B_01_1520962575",
0xd5bafbae,"MainField_Obj_Mineral_B_01_1531755511",
0x48244272,"MainField_Obj_Mineral_B_01_1553965326",
0x92a213a0,"MainField_Obj_Mineral_B_01_1584001054",
0x72084f5d,"MainField_Obj_Mineral_B_01_1588537919",
0x83f3c3de,"MainField_Obj_Mineral_B_01_1589844382",
0x172b8445,"MainField_Obj_Mineral_B_01_1605194278",
0xb7ffc61f,"MainField_Obj_Mineral_B_01_160774889",
0xa4a4a86f,"MainField_Obj_Mineral_B_01_1622761145",
0xe3b5216c,"MainField_Obj_Mineral_B_01_1626226403",
0x78b1794e,"MainField_Obj_Mineral_B_01_1630133461",
0x6ef69b30,"MainField_Obj_Mineral_B_01_1655440360",
0x77986256,"MainField_Obj_Mineral_B_01_1690950180",
0xdcdd8a22,"MainField_Obj_Mineral_B_01_1693609810",
0xcdc6461c,"MainField_Obj_Mineral_B_01_172053444",
0x7e292a7e,"MainField_Obj_Mineral_B_01_1734785993",
0xcedb1311,"MainField_Obj_Mineral_B_01_1742767546",
0xd6debee1,"MainField_Obj_Mineral_B_01_175191410",
0xc51d1879,"MainField_Obj_Mineral_B_01_1758732641",
0xf16091cf,"MainField_Obj_Mineral_B_01_1776142439",
0x8d907569,"MainField_Obj_Mineral_B_01_1805887514",
0xfe3129d6,"MainField_Obj_Mineral_B_01_181039635",
0x09f231f1,"MainField_Obj_Mineral_B_01_1828529199",
0xcc642e3e,"MainField_Obj_Mineral_B_01_183347986",
0x800a72e3,"MainField_Obj_Mineral_B_01_1840009916",
0x258d2b3d,"MainField_Obj_Mineral_B_01_1852887920",
0x074ec1fe,"MainField_Obj_Mineral_B_01_1860404139",
0x7a42bd22,"MainField_Obj_Mineral_B_01_1878329778",
0x63e849fa,"MainField_Obj_Mineral_B_01_1889319719",
0xba28e149,"MainField_Obj_Mineral_B_01_189459028",
0xc6b89a4f,"MainField_Obj_Mineral_B_01_1949148381",
0x9ed2d758,"MainField_Obj_Mineral_B_01_1951000902",
0xe30f35e9,"MainField_Obj_Mineral_B_01_1956174895",
0xdf0b1c44,"MainField_Obj_Mineral_B_01_1998726161",
0xa36c1b55,"MainField_Obj_Mineral_B_01_2033173491",
0x6578fc7b,"MainField_Obj_Mineral_B_01_2050278236",
0x2c938f90,"MainField_Obj_Mineral_B_01_2090477487",
0xe2481755,"MainField_Obj_Mineral_B_01_2137210514",
0xbf4d5438,"MainField_Obj_Mineral_B_01_2139441537",
0x0c6d2a5e,"MainField_Obj_Mineral_B_01_2147615183",
0xafea6577,"MainField_Obj_Mineral_B_01_2158884625",
0x53fe0a89,"MainField_Obj_Mineral_B_01_2181434789",
0x94b5256c,"MainField_Obj_Mineral_B_01_2208655101",
0x1fbd6446,"MainField_Obj_Mineral_B_01_2223112484",
0x593ca278,"MainField_Obj_Mineral_B_01_2231470172",
0x12eb30f5,"MainField_Obj_Mineral_B_01_2258271073",
0x7d3b5544,"MainField_Obj_Mineral_B_01_2264913066",
0xc7f75078,"MainField_Obj_Mineral_B_01_2273158432",
0x6cd86e9d,"MainField_Obj_Mineral_B_01_2274021891",
0xe8f8dc72,"MainField_Obj_Mineral_B_01_2280596681",
0x234100e5,"MainField_Obj_Mineral_B_01_229968978",
0xac9302bd,"MainField_Obj_Mineral_B_01_2305164798",
0x1d6c9d66,"MainField_Obj_Mineral_B_01_2319608466",
0x5c4b6545,"MainField_Obj_Mineral_B_01_2357153069",
0xb7d2d8f4,"MainField_Obj_Mineral_B_01_2371331442",
0xe7922f3d,"MainField_Obj_Mineral_B_01_2406505115",
0x799c4f5c,"MainField_Obj_Mineral_B_01_2434495965",
0x94d935af,"MainField_Obj_Mineral_B_01_2438691632",
0x23161679,"MainField_Obj_Mineral_B_01_2468657726",
0x112565cd,"MainField_Obj_Mineral_B_01_2510692579",
0x2c00de4c,"MainField_Obj_Mineral_B_01_2511829892",
0x73682614,"MainField_Obj_Mineral_B_01_2518953606",
0x692db8df,"MainField_Obj_Mineral_B_01_2527727342",
0xa519615f,"MainField_Obj_Mineral_B_01_2542449979",
0x0658fe57,"MainField_Obj_Mineral_B_01_2598840623",
0x46110cf9,"MainField_Obj_Mineral_B_01_2619555145",
0x249494a1,"MainField_Obj_Mineral_B_01_2697223409",
0x7e7a647e,"MainField_Obj_Mineral_B_01_2709888325",
0x5e84a321,"MainField_Obj_Mineral_B_01_2710494728",
0x02b55043,"MainField_Obj_Mineral_B_01_2714627185",
0x98cedb59,"MainField_Obj_Mineral_B_01_2724766838",
0x916df225,"MainField_Obj_Mineral_B_01_2748273650",
0x69446616,"MainField_Obj_Mineral_B_01_2751301708",
0x34e89a7e,"MainField_Obj_Mineral_B_01_2751513810",
0x7c3d142c,"MainField_Obj_Mineral_B_01_2772055423",
0xb784742e,"MainField_Obj_Mineral_B_01_277736594",
0x33eb0ac6,"MainField_Obj_Mineral_B_01_2783232532",
0xb6a93524,"MainField_Obj_Mineral_B_01_2784397420",
0x3d9d5462,"MainField_Obj_Mineral_B_01_2802527401",
0x6845806b,"MainField_Obj_Mineral_B_01_2813209888",
0xdd721574,"MainField_Obj_Mineral_B_01_2822571471",
0xd1c4ac65,"MainField_Obj_Mineral_B_01_2835573252",
0x018c767b,"MainField_Obj_Mineral_B_01_2844602811",
0x265e9720,"MainField_Obj_Mineral_B_01_2845900385",
0xe2803247,"MainField_Obj_Mineral_B_01_2859601671",
0x2304c081,"MainField_Obj_Mineral_B_01_2871527664",
0x679b2054,"MainField_Obj_Mineral_B_01_2874021620",
0x1defdd60,"MainField_Obj_Mineral_B_01_2876392096",
0x77727aae,"MainField_Obj_Mineral_B_01_2887637808",
0x0da7c721,"MainField_Obj_Mineral_B_01_2893852732",
0xef2bad78,"MainField_Obj_Mineral_B_01_2905268963",
0xe8a256bd,"MainField_Obj_Mineral_B_01_2954483652",
0x0a8e75f2,"MainField_Obj_Mineral_B_01_2962146775",
0xd0059339,"MainField_Obj_Mineral_B_01_2998193039",
0xae5b2eaf,"MainField_Obj_Mineral_B_01_3005451730",
0x9088d038,"MainField_Obj_Mineral_B_01_3014068026",
0x6bb31286,"MainField_Obj_Mineral_B_01_3025437200",
0x040da4ed,"MainField_Obj_Mineral_B_01_303091721",
0x71a57551,"MainField_Obj_Mineral_B_01_3039888797",
0x0b7bf1e1,"MainField_Obj_Mineral_B_01_3050143464",
0x4cf714e2,"MainField_Obj_Mineral_B_01_3054554198",
0xd4d29857,"MainField_Obj_Mineral_B_01_3074575232",
0x6961c841,"MainField_Obj_Mineral_B_01_3106533482",
0x6129fd32,"MainField_Obj_Mineral_B_01_3115380240",
0x4cc20d69,"MainField_Obj_Mineral_B_01_3117084777",
0xe0048113,"MainField_Obj_Mineral_B_01_3133013983",
0x603ae178,"MainField_Obj_Mineral_B_01_3148920795",
0x4394fa8c,"MainField_Obj_Mineral_B_01_3178981147",
0xf0998e27,"MainField_Obj_Mineral_B_01_3185795750",
0x7cf61936,"MainField_Obj_Mineral_B_01_3189017137",
0x5a386a9b,"MainField_Obj_Mineral_B_01_31968618",
0x5bf401c3,"MainField_Obj_Mineral_B_01_3206942160",
0xb6e2ce98,"MainField_Obj_Mineral_B_01_3242028451",
0x5fc0f8fa,"MainField_Obj_Mineral_B_01_3250539840",
0x47cd64af,"MainField_Obj_Mineral_B_01_3265702058",
0x3ac29bc1,"MainField_Obj_Mineral_B_01_327328525",
0x5992eac8,"MainField_Obj_Mineral_B_01_3273540998",
0x48d4e323,"MainField_Obj_Mineral_B_01_3324387273",
0xfff81526,"MainField_Obj_Mineral_B_01_3343558245",
0xb1efd62d,"MainField_Obj_Mineral_B_01_3377927989",
0xead3633e,"MainField_Obj_Mineral_B_01_3380458623",
0x8f279cab,"MainField_Obj_Mineral_B_01_3395704587",
0xf84c8c07,"MainField_Obj_Mineral_B_01_3400139211",
0x053cba18,"MainField_Obj_Mineral_B_01_3400622644",
0xcfc834c4,"MainField_Obj_Mineral_B_01_3409432470",
0x808fbdc1,"MainField_Obj_Mineral_B_01_3419520256",
0x88e3e35f,"MainField_Obj_Mineral_B_01_344801715",
0x3bec6a7a,"MainField_Obj_Mineral_B_01_345004272",
0x9fbf6854,"MainField_Obj_Mineral_B_01_345077899",
0x04419438,"MainField_Obj_Mineral_B_01_3458188038",
0xbd7984f2,"MainField_Obj_Mineral_B_01_3466013139",
0xaeb48f2e,"MainField_Obj_Mineral_B_01_3479574071",
0xe5b6ff8d,"MainField_Obj_Mineral_B_01_3482043261",
0x29217157,"MainField_Obj_Mineral_B_01_3484017763",
0x6a7a22ef,"MainField_Obj_Mineral_B_01_3486574794",
0x919edebe,"MainField_Obj_Mineral_B_01_3500035152",
0xa920d23f,"MainField_Obj_Mineral_B_01_3514721920",
0x1bf4eaf3,"MainField_Obj_Mineral_B_01_3516254936",
0xf17aeb94,"MainField_Obj_Mineral_B_01_3541800825",
0xa85ae9b8,"MainField_Obj_Mineral_B_01_3545893647",
0xa90c38d8,"MainField_Obj_Mineral_B_01_3556948757",
0x686fd870,"MainField_Obj_Mineral_B_01_355707424",
0xaa6a07d3,"MainField_Obj_Mineral_B_01_3582181198",
0x790f537f,"MainField_Obj_Mineral_B_01_360597462",
0x921fecd6,"MainField_Obj_Mineral_B_01_3607701481",
0x23936b7a,"MainField_Obj_Mineral_B_01_3626207507",
0x8dca7d43,"MainField_Obj_Mineral_B_01_3640499627",
0xa0e843af,"MainField_Obj_Mineral_B_01_3657653148",
0x294393c7,"MainField_Obj_Mineral_B_01_3663573619",
0x93d2d02e,"MainField_Obj_Mineral_B_01_3673932469",
0x71878f46,"MainField_Obj_Mineral_B_01_3690797199",
0xeb2294dc,"MainField_Obj_Mineral_B_01_3744408507",
0x86d4f4f9,"MainField_Obj_Mineral_B_01_3754145819",
0xc2f46e10,"MainField_Obj_Mineral_B_01_3762743124",
0x270ec477,"MainField_Obj_Mineral_B_01_3763253449",
0x7b35a16e,"MainField_Obj_Mineral_B_01_378991645",
0x2a91f33d,"MainField_Obj_Mineral_B_01_3799663862",
0xe552879a,"MainField_Obj_Mineral_B_01_3802562713",
0xeb767d85,"MainField_Obj_Mineral_B_01_3847820181",
0x11f618db,"MainField_Obj_Mineral_B_01_3856688303",
0xdf136a5e,"MainField_Obj_Mineral_B_01_3872648065",
0xd84483e7,"MainField_Obj_Mineral_B_01_3875190132",
0x35cf6f23,"MainField_Obj_Mineral_B_01_3886889091",
0x59bd1d19,"MainField_Obj_Mineral_B_01_3902609412",
0x64d3f0d7,"MainField_Obj_Mineral_B_01_3905294715",
0x81acebf7,"MainField_Obj_Mineral_B_01_3912410402",
0xed567905,"MainField_Obj_Mineral_B_01_3926797373",
0xb7f73ba9,"MainField_Obj_Mineral_B_01_3934736561",
0xd1135a85,"MainField_Obj_Mineral_B_01_3941744504",
0xdf75c996,"MainField_Obj_Mineral_B_01_3943372119",
0x6a78d67c,"MainField_Obj_Mineral_B_01_395640132",
0x9fdc49d7,"MainField_Obj_Mineral_B_01_3963590576",
0x37ba398c,"MainField_Obj_Mineral_B_01_3968184361",
0x5585142a,"MainField_Obj_Mineral_B_01_3976110883",
0x00a52ea4,"MainField_Obj_Mineral_B_01_400646827",
0x98d58d93,"MainField_Obj_Mineral_B_01_4009670033",
0xf21b4abd,"MainField_Obj_Mineral_B_01_401094701",
0xa83618b9,"MainField_Obj_Mineral_B_01_4014835577",
0x564c07cc,"MainField_Obj_Mineral_B_01_4027408387",
0x3a967a08,"MainField_Obj_Mineral_B_01_4056329029",
0x696aa360,"MainField_Obj_Mineral_B_01_4060571169",
0xc323ed50,"MainField_Obj_Mineral_B_01_4076663924",
0x50a0ab44,"MainField_Obj_Mineral_B_01_4081421999",
0x5722216d,"MainField_Obj_Mineral_B_01_4092734077",
0x5e702ed3,"MainField_Obj_Mineral_B_01_4106746820",
0xe7dfab97,"MainField_Obj_Mineral_B_01_4109273364",
0x245de552,"MainField_Obj_Mineral_B_01_4114738493",
0x92cf65b0,"MainField_Obj_Mineral_B_01_4151539798",
0x0ffe5a82,"MainField_Obj_Mineral_B_01_4157383602",
0x4b592116,"MainField_Obj_Mineral_B_01_4160666128",
0x70ad5e0b,"MainField_Obj_Mineral_B_01_4162272481",
0xe816f602,"MainField_Obj_Mineral_B_01_4176973932",
0x59a4fa95,"MainField_Obj_Mineral_B_01_4204612649",
0xf8841464,"MainField_Obj_Mineral_B_01_4215191857",
0xf5fffc64,"MainField_Obj_Mineral_B_01_4215631589",
0xa582b88d,"MainField_Obj_Mineral_B_01_4218415192",
0xbcf15d7b,"MainField_Obj_Mineral_B_01_4236893739",
0x80bff252,"MainField_Obj_Mineral_B_01_4287727003",
0x569f0581,"MainField_Obj_Mineral_B_01_431887956",
0x230c7607,"MainField_Obj_Mineral_B_01_478618252",
0x7129c7a2,"MainField_Obj_Mineral_B_01_49839766",
0x98f994d3,"MainField_Obj_Mineral_B_01_501840452",
0x098d96d3,"MainField_Obj_Mineral_B_01_502007966",
0xadbbe6f3,"MainField_Obj_Mineral_B_01_511419623",
0x83cac47f,"MainField_Obj_Mineral_B_01_52642326",
0xe5196064,"MainField_Obj_Mineral_B_01_541566642",
0x504a0458,"MainField_Obj_Mineral_B_01_548293104",
0x49e60088,"MainField_Obj_Mineral_B_01_558389129",
0x06160d16,"MainField_Obj_Mineral_B_01_570855699",
0x6f9d57ee,"MainField_Obj_Mineral_B_01_592827864",
0x8fe41e40,"MainField_Obj_Mineral_B_01_595449540",
0xd1cec7a5,"MainField_Obj_Mineral_B_01_609239162",
0x4b95b30d,"MainField_Obj_Mineral_B_01_626995447",
0xe8afe5c8,"MainField_Obj_Mineral_B_01_633827477",
0xeb211c69,"MainField_Obj_Mineral_B_01_643058722",
0x63b10b99,"MainField_Obj_Mineral_B_01_64314015",
0xe5f16adb,"MainField_Obj_Mineral_B_01_646201858",
0xb09a3904,"MainField_Obj_Mineral_B_01_706785539",
0x8e717e9c,"MainField_Obj_Mineral_B_01_710794997",
0x4143df9b,"MainField_Obj_Mineral_B_01_72147850",
0xa1f97ea9,"MainField_Obj_Mineral_B_01_741768174",
0x3d37437a,"MainField_Obj_Mineral_B_01_767133375",
0x617c5e29,"MainField_Obj_Mineral_B_01_82277293",
0xa93d958e,"MainField_Obj_Mineral_B_01_856918114",
0xb2fcd747,"MainField_Obj_Mineral_B_01_890197789",
0x59183489,"MainField_Obj_Mineral_B_01_907372583",
0x0a56e21d,"MainField_Obj_Mineral_B_01_919638687",
0x5f9010d7,"MainField_Obj_Mineral_B_01_933831495",
0x010ace80,"MainField_Obj_Mineral_B_01_941806889",
0x732d5c3f,"MainField_Obj_Mineral_C_01_1004310062",
0xd0e1a311,"MainField_Obj_Mineral_C_01_1005437974",
0xb7f37faa,"MainField_Obj_Mineral_C_01_1016899000",
0x32ff5fad,"MainField_Obj_Mineral_C_01_102909542",
0x68e60aa1,"MainField_Obj_Mineral_C_01_1034949196",
0x5e135c95,"MainField_Obj_Mineral_C_01_1044593153",
0x0349776c,"MainField_Obj_Mineral_C_01_1044911683",
0x68379671,"MainField_Obj_Mineral_C_01_1065173041",
0x7e465a0a,"MainField_Obj_Mineral_C_01_1079317521",
0xfb6067d7,"MainField_Obj_Mineral_C_01_1082564315",
0xc3b6092d,"MainField_Obj_Mineral_C_01_1083964313",
0x6c7cf8a8,"MainField_Obj_Mineral_C_01_1101623657",
0x9e6a03af,"MainField_Obj_Mineral_C_01_1103225028",
0x0f20cb68,"MainField_Obj_Mineral_C_01_1118269426",
0x650417e0,"MainField_Obj_Mineral_C_01_112351285",
0x35002733,"MainField_Obj_Mineral_C_01_1127517473",
0xc0fad766,"MainField_Obj_Mineral_C_01_113196799",
0xf5b86eff,"MainField_Obj_Mineral_C_01_113898883",
0xbc02fcb8,"MainField_Obj_Mineral_C_01_1142479511",
0xe6587e37,"MainField_Obj_Mineral_C_01_1154939394",
0x8f2815af,"MainField_Obj_Mineral_C_01_1155876609",
0x96056106,"MainField_Obj_Mineral_C_01_1170986375",
0xb65d63f5,"MainField_Obj_Mineral_C_01_1183963454",
0x4dc1e071,"MainField_Obj_Mineral_C_01_119527177",
0x398acf66,"MainField_Obj_Mineral_C_01_1196463998",
0x9dcc2aa9,"MainField_Obj_Mineral_C_01_1214741864",
0xdcfd64e1,"MainField_Obj_Mineral_C_01_122045481",
0xce54b2f4,"MainField_Obj_Mineral_C_01_1220575843",
0x061b671e,"MainField_Obj_Mineral_C_01_1225416496",
0x45ff47ae,"MainField_Obj_Mineral_C_01_1235709364",
0xad6b2bb9,"MainField_Obj_Mineral_C_01_1236889697",
0x75285071,"MainField_Obj_Mineral_C_01_1243301935",
0xc4438bab,"MainField_Obj_Mineral_C_01_1257462131",
0xd65d7589,"MainField_Obj_Mineral_C_01_126757680",
0x16e0ae88,"MainField_Obj_Mineral_C_01_1284560121",
0x13bc2071,"MainField_Obj_Mineral_C_01_1298018615",
0xdda55341,"MainField_Obj_Mineral_C_01_1300384987",
0x1567d3d0,"MainField_Obj_Mineral_C_01_1306751495",
0x93d54400,"MainField_Obj_Mineral_C_01_1308576522",
0x346145ed,"MainField_Obj_Mineral_C_01_1321555688",
0x2acc88af,"MainField_Obj_Mineral_C_01_1336023141",
0x9d41aa20,"MainField_Obj_Mineral_C_01_1339994007",
0xfe19b4d3,"MainField_Obj_Mineral_C_01_134898794",
0x44b7f8db,"MainField_Obj_Mineral_C_01_1395899882",
0xe2a4e64c,"MainField_Obj_Mineral_C_01_141088478",
0xb75fea4f,"MainField_Obj_Mineral_C_01_141661400",
0xbf031ba3,"MainField_Obj_Mineral_C_01_142114472",
0xb910b7bd,"MainField_Obj_Mineral_C_01_142736091",
0xbb1c4acb,"MainField_Obj_Mineral_C_01_1433867934",
0x3f30986a,"MainField_Obj_Mineral_C_01_14415993",
0x182dbece,"MainField_Obj_Mineral_C_01_1454914106",
0x15e92dbf,"MainField_Obj_Mineral_C_01_1468682558",
0xd581dda6,"MainField_Obj_Mineral_C_01_1494066793",
0xcd94fb0f,"MainField_Obj_Mineral_C_01_1494076348",
0xe8d307d9,"MainField_Obj_Mineral_C_01_149524630",
0x18a63196,"MainField_Obj_Mineral_C_01_1502581407",
0x9233efe5,"MainField_Obj_Mineral_C_01_1506442543",
0x97840f5b,"MainField_Obj_Mineral_C_01_1507536723",
0x2e561857,"MainField_Obj_Mineral_C_01_1512295284",
0x53801a30,"MainField_Obj_Mineral_C_01_1529517362",
0x408c096a,"MainField_Obj_Mineral_C_01_1564048146",
0x6d82ede8,"MainField_Obj_Mineral_C_01_1564136628",
0x7c5d1408,"MainField_Obj_Mineral_C_01_1586207694",
0x72d20395,"MainField_Obj_Mineral_C_01_1594112151",
0x78cb95ca,"MainField_Obj_Mineral_C_01_1599056090",
0xb394f5ac,"MainField_Obj_Mineral_C_01_1603339563",
0x889066f9,"MainField_Obj_Mineral_C_01_1606311510",
0x507707d3,"MainField_Obj_Mineral_C_01_161537180",
0xaec0ab3e,"MainField_Obj_Mineral_C_01_1624470636",
0xaa0039b3,"MainField_Obj_Mineral_C_01_1627589967",
0xeac4e76e,"MainField_Obj_Mineral_C_01_1627949279",
0x1ddfb54c,"MainField_Obj_Mineral_C_01_162851395",
0x21e40caf,"MainField_Obj_Mineral_C_01_163734005",
0xf479fbd5,"MainField_Obj_Mineral_C_01_1641257196",
0xeffe0eee,"MainField_Obj_Mineral_C_01_1648298130",
0x78ea5cc1,"MainField_Obj_Mineral_C_01_1650283627",
0x4d239800,"MainField_Obj_Mineral_C_01_1666124086",
0xc5b7dbae,"MainField_Obj_Mineral_C_01_1675346028",
0x13dd972d,"MainField_Obj_Mineral_C_01_1686138841",
0x9a1e05bc,"MainField_Obj_Mineral_C_01_1690479065",
0xb0285e3f,"MainField_Obj_Mineral_C_01_170615145",
0xa01a277d,"MainField_Obj_Mineral_C_01_171141812",
0xc2076859,"MainField_Obj_Mineral_C_01_1724263121",
0xea12e1db,"MainField_Obj_Mineral_C_01_1754355084",
0x79ce709b,"MainField_Obj_Mineral_C_01_1756553492",
0xa7865feb,"MainField_Obj_Mineral_C_01_1756697534",
0x9e89c8c4,"MainField_Obj_Mineral_C_01_1759324594",
0xd6b6bc14,"MainField_Obj_Mineral_C_01_1776435533",
0x1da3bd99,"MainField_Obj_Mineral_C_01_1786262265",
0xee40074a,"MainField_Obj_Mineral_C_01_1791571202",
0x7943d759,"MainField_Obj_Mineral_C_01_1804533589",
0x74512821,"MainField_Obj_Mineral_C_01_1815558746",
0xd8b93b2a,"MainField_Obj_Mineral_C_01_1818947739",
0x85e674fe,"MainField_Obj_Mineral_C_01_182464965",
0x017c04b2,"MainField_Obj_Mineral_C_01_1830969140",
0x1f0fb921,"MainField_Obj_Mineral_C_01_1841204963",
0x346ce6a4,"MainField_Obj_Mineral_C_01_1857137803",
0xca050401,"MainField_Obj_Mineral_C_01_1866648237",
0x3227cec9,"MainField_Obj_Mineral_C_01_186913812",
0x5486eef7,"MainField_Obj_Mineral_C_01_1881453635",
0xd3f04ffd,"MainField_Obj_Mineral_C_01_1888031216",
0xb39cde9f,"MainField_Obj_Mineral_C_01_1895282345",
0x0f0fae44,"MainField_Obj_Mineral_C_01_1909572",
0x52014c1e,"MainField_Obj_Mineral_C_01_191317989",
0x5385e202,"MainField_Obj_Mineral_C_01_1916002654",
0xe475f6a3,"MainField_Obj_Mineral_C_01_1916845774",
0x39975740,"MainField_Obj_Mineral_C_01_1920630432",
0xf2749ef0,"MainField_Obj_Mineral_C_01_1924111438",
0x9ef22b72,"MainField_Obj_Mineral_C_01_194543637",
0x9179ae2d,"MainField_Obj_Mineral_C_01_1947086606",
0xbcaa5980,"MainField_Obj_Mineral_C_01_1959400694",
0x047e1d9d,"MainField_Obj_Mineral_C_01_19703846",
0x1d8fbce0,"MainField_Obj_Mineral_C_01_1972744063",
0x12f794aa,"MainField_Obj_Mineral_C_01_1973485603",
0x0ab22b45,"MainField_Obj_Mineral_C_01_1975785244",
0x601560e5,"MainField_Obj_Mineral_C_01_1983161483",
0x3305c6e9,"MainField_Obj_Mineral_C_01_1992797014",
0x0091a136,"MainField_Obj_Mineral_C_01_1999818885",
0xa97daf65,"MainField_Obj_Mineral_C_01_2005524892",
0x8564013f,"MainField_Obj_Mineral_C_01_2011168759",
0x91198cc2,"MainField_Obj_Mineral_C_01_2019423539",
0xbd01ee66,"MainField_Obj_Mineral_C_01_2026383968",
0x68447d59,"MainField_Obj_Mineral_C_01_2031578508",
0x1900c97a,"MainField_Obj_Mineral_C_01_2073514499",
0xe58dc45d,"MainField_Obj_Mineral_C_01_2084142592",
0x73e5391c,"MainField_Obj_Mineral_C_01_2084690653",
0x0b643ed1,"MainField_Obj_Mineral_C_01_2097412289",
0x1c04ee66,"MainField_Obj_Mineral_C_01_2117030938",
0x56b4c5bb,"MainField_Obj_Mineral_C_01_2119593628",
0x611a9e89,"MainField_Obj_Mineral_C_01_2120756023",
0x912c174b,"MainField_Obj_Mineral_C_01_2131397590",
0xb44828ec,"MainField_Obj_Mineral_C_01_215155387",
0xd4b36657,"MainField_Obj_Mineral_C_01_2172527117",
0xe3dcf164,"MainField_Obj_Mineral_C_01_2174341723",
0x35743962,"MainField_Obj_Mineral_C_01_2176570088",
0x6fda7131,"MainField_Obj_Mineral_C_01_2198051274",
0xc0700f8b,"MainField_Obj_Mineral_C_01_2198599137",
0x286a95d1,"MainField_Obj_Mineral_C_01_2214456561",
0xad593408,"MainField_Obj_Mineral_C_01_2214764563",
0xcca823a6,"MainField_Obj_Mineral_C_01_2216631708",
0xcff76bad,"MainField_Obj_Mineral_C_01_2217170647",
0x3b453e6a,"MainField_Obj_Mineral_C_01_2258530741",
0x6729f03a,"MainField_Obj_Mineral_C_01_2274188353",
0x1f975c85,"MainField_Obj_Mineral_C_01_2278943572",
0x518c40ec,"MainField_Obj_Mineral_C_01_2281715950",
0x597bd6e1,"MainField_Obj_Mineral_C_01_2282706040",
0xd12f2434,"MainField_Obj_Mineral_C_01_2284352931",
0x24f3552f,"MainField_Obj_Mineral_C_01_229250820",
0x236460e4,"MainField_Obj_Mineral_C_01_2304506616",
0xeaa21db8,"MainField_Obj_Mineral_C_01_2307768961",
0x98dc916f,"MainField_Obj_Mineral_C_01_2317039447",
0xebad4407,"MainField_Obj_Mineral_C_01_2325208265",
0x9533c948,"MainField_Obj_Mineral_C_01_234325648",
0x4af74c45,"MainField_Obj_Mineral_C_01_2347136616",
0x074f8ea9,"MainField_Obj_Mineral_C_01_2357887603",
0x80c5400e,"MainField_Obj_Mineral_C_01_2367961288",
0x8a7aa84a,"MainField_Obj_Mineral_C_01_2400938841",
0xf97740b3,"MainField_Obj_Mineral_C_01_2401339077",
0x7eeca23b,"MainField_Obj_Mineral_C_01_240420487",
0x35d15358,"MainField_Obj_Mineral_C_01_2432435716",
0x9c9219dc,"MainField_Obj_Mineral_C_01_243522453",
0xd8dabfe9,"MainField_Obj_Mineral_C_01_2442537058",
0xa7268f6c,"MainField_Obj_Mineral_C_01_2473137066",
0x0a38d986,"MainField_Obj_Mineral_C_01_2476047196",
0x754074e0,"MainField_Obj_Mineral_C_01_248357268",
0xa0c4dfd0,"MainField_Obj_Mineral_C_01_2485143228",
0x20251cc3,"MainField_Obj_Mineral_C_01_2490689356",
0xe70a25fc,"MainField_Obj_Mineral_C_01_2500506589",
0x116ef29c,"MainField_Obj_Mineral_C_01_2503200074",
0x960de830,"MainField_Obj_Mineral_C_01_2511489355",
0x3296743b,"MainField_Obj_Mineral_C_01_2525037526",
0xa63c4dca,"MainField_Obj_Mineral_C_01_2530469444",
0xf72125b1,"MainField_Obj_Mineral_C_01_2543430262",
0xc1c9cc3f,"MainField_Obj_Mineral_C_01_2545969657",
0x0f701243,"MainField_Obj_Mineral_C_01_2559321537",
0xf37128cc,"MainField_Obj_Mineral_C_01_2569639411",
0x730675f3,"MainField_Obj_Mineral_C_01_2579409630",
0xf85f50f1,"MainField_Obj_Mineral_C_01_2580669769",
0x1d093791,"MainField_Obj_Mineral_C_01_258608496",
0x214622e0,"MainField_Obj_Mineral_C_01_2592079378",
0x08fe82d4,"MainField_Obj_Mineral_C_01_2594369437",
0x3ae90c62,"MainField_Obj_Mineral_C_01_2600113688",
0xf4809bcd,"MainField_Obj_Mineral_C_01_2602586528",
0x2921ff46,"MainField_Obj_Mineral_C_01_2604348658",
0x177f9dc3,"MainField_Obj_Mineral_C_01_2606411909",
0xca48c740,"MainField_Obj_Mineral_C_01_2614706477",
0xfa2cdc53,"MainField_Obj_Mineral_C_01_262907035",
0x333b679b,"MainField_Obj_Mineral_C_01_263858748",
0x25f0a970,"MainField_Obj_Mineral_C_01_2640402146",
0x1814b1a1,"MainField_Obj_Mineral_C_01_2649463618",
0x080c3de6,"MainField_Obj_Mineral_C_01_2650977387",
0x44f9045c,"MainField_Obj_Mineral_C_01_267147995",
0xd9ce20fa,"MainField_Obj_Mineral_C_01_2680467503",
0x5966cc77,"MainField_Obj_Mineral_C_01_268124873",
0x5a783d01,"MainField_Obj_Mineral_C_01_268161010",
0x280090d0,"MainField_Obj_Mineral_C_01_2683015426",
0x2ed53507,"MainField_Obj_Mineral_C_01_2702553071",
0x13e670eb,"MainField_Obj_Mineral_C_01_2718104372",
0xace7e5e5,"MainField_Obj_Mineral_C_01_2726759986",
0x23aa158e,"MainField_Obj_Mineral_C_01_272677055",
0x61350e30,"MainField_Obj_Mineral_C_01_2741362702",
0xb93f2ed4,"MainField_Obj_Mineral_C_01_2741478709",
0x377e3771,"MainField_Obj_Mineral_C_01_2747957751",
0x36e90435,"MainField_Obj_Mineral_C_01_2763343427",
0x23c8e7c5,"MainField_Obj_Mineral_C_01_2763998320",
0xcadf2693,"MainField_Obj_Mineral_C_01_2780699533",
0xae0b935e,"MainField_Obj_Mineral_C_01_2783304197",
0x277573bb,"MainField_Obj_Mineral_C_01_2784199981",
0xe6f524f2,"MainField_Obj_Mineral_C_01_2789265568",
0x42cc8794,"MainField_Obj_Mineral_C_01_2789856651",
0xcb728afd,"MainField_Obj_Mineral_C_01_2797647772",
0x8d1a619f,"MainField_Obj_Mineral_C_01_2799763542",
0x4cac9cb6,"MainField_Obj_Mineral_C_01_2810007167",
0xd5889cb8,"MainField_Obj_Mineral_C_01_2830551895",
0x034744a7,"MainField_Obj_Mineral_C_01_2835370228",
0x522e2182,"MainField_Obj_Mineral_C_01_2840513609",
0x98621ae6,"MainField_Obj_Mineral_C_01_286728984",
0x87d36a35,"MainField_Obj_Mineral_C_01_2869223265",
0x539b1aa9,"MainField_Obj_Mineral_C_01_2874473421",
0xfc4a2957,"MainField_Obj_Mineral_C_01_2875549627",
0x19c4ae2b,"MainField_Obj_Mineral_C_01_2878642454",
0xacf95de5,"MainField_Obj_Mineral_C_01_2911705886",
0x51204f94,"MainField_Obj_Mineral_C_01_292370191",
0xec9770c3,"MainField_Obj_Mineral_C_01_2934456361",
0x94894500,"MainField_Obj_Mineral_C_01_2935136438",
0x3c28e712,"MainField_Obj_Mineral_C_01_2938014412",
0xf9fe203d,"MainField_Obj_Mineral_C_01_2940319109",
0xe69a6e21,"MainField_Obj_Mineral_C_01_2962426737",
0x7ef3c299,"MainField_Obj_Mineral_C_01_2973949734",
0x661e1584,"MainField_Obj_Mineral_C_01_2975392022",
0x5606e4a7,"MainField_Obj_Mineral_C_01_2977922523",
0x53faa61b,"MainField_Obj_Mineral_C_01_2996474921",
0x3a603619,"MainField_Obj_Mineral_C_01_2999779491",
0x6fd27e0a,"MainField_Obj_Mineral_C_01_3004543027",
0x30576255,"MainField_Obj_Mineral_C_01_3005191952",
0x1cff9d31,"MainField_Obj_Mineral_C_01_3007259072",
0xac658537,"MainField_Obj_Mineral_C_01_3031357590",
0x7a3e398e,"MainField_Obj_Mineral_C_01_3040659214",
0x52224999,"MainField_Obj_Mineral_C_01_3047602506",
0x276344ae,"MainField_Obj_Mineral_C_01_3058772722",
0x03188f3a,"MainField_Obj_Mineral_C_01_3067341293",
0xec03b380,"MainField_Obj_Mineral_C_01_3074347958",
0x8315d5e2,"MainField_Obj_Mineral_C_01_3083387763",
0xf38908aa,"MainField_Obj_Mineral_C_01_309703124",
0xaff515ee,"MainField_Obj_Mineral_C_01_3097937755",
0x0c37a7f2,"MainField_Obj_Mineral_C_01_3105605882",
0x53b4f360,"MainField_Obj_Mineral_C_01_3120358055",
0xa85ba2b1,"MainField_Obj_Mineral_C_01_3121113092",
0x61f1f272,"MainField_Obj_Mineral_C_01_3132333696",
0xfd6f1a4f,"MainField_Obj_Mineral_C_01_3160307337",
0xd995dd5d,"MainField_Obj_Mineral_C_01_3173379292",
0x93fe7e4e,"MainField_Obj_Mineral_C_01_3177792501",
0x63241c8d,"MainField_Obj_Mineral_C_01_3181496679",
0xb080b003,"MainField_Obj_Mineral_C_01_3215430839",
0x9c60116b,"MainField_Obj_Mineral_C_01_3222672330",
0xdfad963e,"MainField_Obj_Mineral_C_01_3226100956",
0xc590e3b2,"MainField_Obj_Mineral_C_01_3228064449",
0xb9e02400,"MainField_Obj_Mineral_C_01_3241271701",
0xcebcc40e,"MainField_Obj_Mineral_C_01_3242132683",
0xe1808332,"MainField_Obj_Mineral_C_01_3247377999",
0x2c0ea64c,"MainField_Obj_Mineral_C_01_3256979225",
0x6c575635,"MainField_Obj_Mineral_C_01_3262192343",
0xc7099fa7,"MainField_Obj_Mineral_C_01_3269622275",
0xfbe2a2cd,"MainField_Obj_Mineral_C_01_3297989153",
0xbc51779c,"MainField_Obj_Mineral_C_01_3301044994",
0xbbb92f7b,"MainField_Obj_Mineral_C_01_3301045340",
0xce84153c,"MainField_Obj_Mineral_C_01_3306779636",
0x0179c6a3,"MainField_Obj_Mineral_C_01_3313382987",
0xb97a6e8c,"MainField_Obj_Mineral_C_01_331375480",
0x4a2c6d14,"MainField_Obj_Mineral_C_01_3315186389",
0x7da95625,"MainField_Obj_Mineral_C_01_3318826917",
0xf07f7924,"MainField_Obj_Mineral_C_01_3320379025",
0x7e449909,"MainField_Obj_Mineral_C_01_3329049103",
0x18b0a95b,"MainField_Obj_Mineral_C_01_3350270118",
0x05d291fc,"MainField_Obj_Mineral_C_01_3356731301",
0xcb373621,"MainField_Obj_Mineral_C_01_3399276261",
0xb0776b71,"MainField_Obj_Mineral_C_01_3423422287",
0xbb2531fb,"MainField_Obj_Mineral_C_01_3434229464",
0x0b2bc97d,"MainField_Obj_Mineral_C_01_3468821208",
0xb9a6b188,"MainField_Obj_Mineral_C_01_3478235640",
0x9d692ed7,"MainField_Obj_Mineral_C_01_3479528579",
0x319ca426,"MainField_Obj_Mineral_C_01_3484915757",
0x66dcdba1,"MainField_Obj_Mineral_C_01_3485849983",
0x9c092293,"MainField_Obj_Mineral_C_01_3495910910",
0x9563c73f,"MainField_Obj_Mineral_C_01_3497668651",
0xb94050f8,"MainField_Obj_Mineral_C_01_3502215372",
0x17b6f938,"MainField_Obj_Mineral_C_01_3507416707",
0x0f1d6ed8,"MainField_Obj_Mineral_C_01_3508075058",
0xa9089c29,"MainField_Obj_Mineral_C_01_3515923437",
0x7077ffb9,"MainField_Obj_Mineral_C_01_3520760048",
0x33f0f3d8,"MainField_Obj_Mineral_C_01_3549032304",
0xc4eb48a3,"MainField_Obj_Mineral_C_01_3552600004",
0x160c9abd,"MainField_Obj_Mineral_C_01_3554046966",
0x460d1cdf,"MainField_Obj_Mineral_C_01_3564711461",
0xaf2f7bb3,"MainField_Obj_Mineral_C_01_3567060402",
0x1ecc1c52,"MainField_Obj_Mineral_C_01_3572729931",
0x3ab129b9,"MainField_Obj_Mineral_C_01_358241149",
0x8c8d13fd,"MainField_Obj_Mineral_C_01_3587475593",
0x81a9b722,"MainField_Obj_Mineral_C_01_3589010724",
0xe4d91eaa,"MainField_Obj_Mineral_C_01_3590863970",
0x2097f4c5,"MainField_Obj_Mineral_C_01_3595806415",
0xb00b077d,"MainField_Obj_Mineral_C_01_3596219708",
0x64811366,"MainField_Obj_Mineral_C_01_3605283527",
0xd2bfe4ed,"MainField_Obj_Mineral_C_01_3613451324",
0xfdc78fd1,"MainField_Obj_Mineral_C_01_3630611794",
0xb64b4ff6,"MainField_Obj_Mineral_C_01_363812419",
0x52849d7a,"MainField_Obj_Mineral_C_01_3641005789",
0xd840cd6b,"MainField_Obj_Mineral_C_01_3655249616",
0x85a0fa9a,"MainField_Obj_Mineral_C_01_3693057055",
0x5b4cdd79,"MainField_Obj_Mineral_C_01_3693267973",
0xa25fd581,"MainField_Obj_Mineral_C_01_3700167503",
0x42d495ca,"MainField_Obj_Mineral_C_01_3714998501",
0xf635397b,"MainField_Obj_Mineral_C_01_3737599254",
0xf296b399,"MainField_Obj_Mineral_C_01_3740648605",
0x0ea77eb4,"MainField_Obj_Mineral_C_01_3744068190",
0x18349770,"MainField_Obj_Mineral_C_01_3771210186",
0xbf767a2d,"MainField_Obj_Mineral_C_01_3780349513",
0x1f970eaf,"MainField_Obj_Mineral_C_01_3780908380",
0x22921f9c,"MainField_Obj_Mineral_C_01_3785609302",
0xeee64db8,"MainField_Obj_Mineral_C_01_3796469349",
0x67980ac3,"MainField_Obj_Mineral_C_01_379794924",
0x0fea361f,"MainField_Obj_Mineral_C_01_3800044574",
0x8e1ad8e2,"MainField_Obj_Mineral_C_01_3803550314",
0xdbe4f5d0,"MainField_Obj_Mineral_C_01_3808603810",
0x4274c267,"MainField_Obj_Mineral_C_01_3819212047",
0x20bac6f3,"MainField_Obj_Mineral_C_01_382437109",
0x285e06e9,"MainField_Obj_Mineral_C_01_3840587611",
0xc10cfaba,"MainField_Obj_Mineral_C_01_3845187374",
0x45d66231,"MainField_Obj_Mineral_C_01_384878391",
0xa11f50e5,"MainField_Obj_Mineral_C_01_3862106686",
0xb4841dff,"MainField_Obj_Mineral_C_01_3866282043",
0x022dad86,"MainField_Obj_Mineral_C_01_3876663549",
0xef928894,"MainField_Obj_Mineral_C_01_3890232711",
0x8c83d398,"MainField_Obj_Mineral_C_01_3895747346",
0x58697db3,"MainField_Obj_Mineral_C_01_3906796834",
0xc26bc981,"MainField_Obj_Mineral_C_01_3907386481",
0xc67f372d,"MainField_Obj_Mineral_C_01_3926645098",
0xe1f3bfec,"MainField_Obj_Mineral_C_01_3929280815",
0xd423690c,"MainField_Obj_Mineral_C_01_3937533862",
0xf6f161cf,"MainField_Obj_Mineral_C_01_3942706512",
0x047a1bbf,"MainField_Obj_Mineral_C_01_3946056438",
0xdab15631,"MainField_Obj_Mineral_C_01_3953725877",
0x12077e84,"MainField_Obj_Mineral_C_01_3954999294",
0x311d9974,"MainField_Obj_Mineral_C_01_3955208742",
0x4af95782,"MainField_Obj_Mineral_C_01_3958539001",
0x454a5cc3,"MainField_Obj_Mineral_C_01_3980300275",
0x7dbf1530,"MainField_Obj_Mineral_C_01_3986430128",
0xa6970f3a,"MainField_Obj_Mineral_C_01_399457081",
0xfe3b9775,"MainField_Obj_Mineral_C_01_4008407323",
0xabdea891,"MainField_Obj_Mineral_C_01_4010629373",
0x3ab3172b,"MainField_Obj_Mineral_C_01_4017855421",
0x5478a73c,"MainField_Obj_Mineral_C_01_4030807563",
0x6ca51258,"MainField_Obj_Mineral_C_01_4032784037",
0xf55b987e,"MainField_Obj_Mineral_C_01_4050421117",
0xb637b570,"MainField_Obj_Mineral_C_01_4051223349",
0xab8587a5,"MainField_Obj_Mineral_C_01_4077445883",
0xb8ccebf4,"MainField_Obj_Mineral_C_01_4080933154",
0x7184c9d2,"MainField_Obj_Mineral_C_01_4098654327",
0x7540fc1d,"MainField_Obj_Mineral_C_01_4100230859",
0xa951cf1c,"MainField_Obj_Mineral_C_01_4103075511",
0x4d7efcd7,"MainField_Obj_Mineral_C_01_4105703948",
0x83cb50ef,"MainField_Obj_Mineral_C_01_4114996236",
0xe942966f,"MainField_Obj_Mineral_C_01_4129903409",
0x47233845,"MainField_Obj_Mineral_C_01_4141265557",
0x92351889,"MainField_Obj_Mineral_C_01_414373689",
0x991e4c0c,"MainField_Obj_Mineral_C_01_414768063",
0xb24ef49f,"MainField_Obj_Mineral_C_01_4149418037",
0xa2a3e72a,"MainField_Obj_Mineral_C_01_4150742398",
0x42dad7af,"MainField_Obj_Mineral_C_01_4161676345",
0x56ba96e5,"MainField_Obj_Mineral_C_01_4186091486",
0x42550d11,"MainField_Obj_Mineral_C_01_421050739",
0xb738ffec,"MainField_Obj_Mineral_C_01_4227785519",
0x9a695fe0,"MainField_Obj_Mineral_C_01_423288348",
0x7e2fb5bc,"MainField_Obj_Mineral_C_01_4247130343",
0x56db97d7,"MainField_Obj_Mineral_C_01_4254751832",
0xfedcaaa7,"MainField_Obj_Mineral_C_01_4260851106",
0x5b1d1e7c,"MainField_Obj_Mineral_C_01_4285496686",
0xb040c8e4,"MainField_Obj_Mineral_C_01_432255438",
0xb0ab1b8b,"MainField_Obj_Mineral_C_01_435736534",
0x1f33fca2,"MainField_Obj_Mineral_C_01_437020245",
0x05e9447a,"MainField_Obj_Mineral_C_01_460536080",
0x9c330e10,"MainField_Obj_Mineral_C_01_501663635",
0x852b3016,"MainField_Obj_Mineral_C_01_506156002",
0xf43255e1,"MainField_Obj_Mineral_C_01_50750481",
0xc99f69be,"MainField_Obj_Mineral_C_01_516238413",
0x235b66d3,"MainField_Obj_Mineral_C_01_530070624",
0x9fa19c8f,"MainField_Obj_Mineral_C_01_5314523",
0x3d1f4435,"MainField_Obj_Mineral_C_01_556996720",
0x1bbaf601,"MainField_Obj_Mineral_C_01_557486871",
0xcfa498a5,"MainField_Obj_Mineral_C_01_562309029",
0xb2c70f81,"MainField_Obj_Mineral_C_01_569270975",
0xae90e396,"MainField_Obj_Mineral_C_01_573469290",
0x75540bf4,"MainField_Obj_Mineral_C_01_590071729",
0x658c8719,"MainField_Obj_Mineral_C_01_590109553",
0x2e094e4b,"MainField_Obj_Mineral_C_01_592615360",
0x8f49ead2,"MainField_Obj_Mineral_C_01_597319366",
0xb9131ff3,"MainField_Obj_Mineral_C_01_606055424",
0xfd823e4a,"MainField_Obj_Mineral_C_01_607139322",
0xfb75fdf3,"MainField_Obj_Mineral_C_01_610882060",
0xc017185f,"MainField_Obj_Mineral_C_01_627487519",
0x95254456,"MainField_Obj_Mineral_C_01_629280469",
0x3b33aab5,"MainField_Obj_Mineral_C_01_639121328",
0xd0245f70,"MainField_Obj_Mineral_C_01_643635373",
0x1e3462e7,"MainField_Obj_Mineral_C_01_645130760",
0xb458d61f,"MainField_Obj_Mineral_C_01_647463053",
0xda52555e,"MainField_Obj_Mineral_C_01_655508025",
0xf8155c12,"MainField_Obj_Mineral_C_01_669910076",
0xf1646080,"MainField_Obj_Mineral_C_01_675641282",
0xb759b99e,"MainField_Obj_Mineral_C_01_675897613",
0x04e3a2b4,"MainField_Obj_Mineral_C_01_675947204",
0xd93ccc36,"MainField_Obj_Mineral_C_01_677670719",
0x24e3d228,"MainField_Obj_Mineral_C_01_679492656",
0xb6b7637f,"MainField_Obj_Mineral_C_01_681955551",
0xf9f72714,"MainField_Obj_Mineral_C_01_683136672",
0xdbe8c637,"MainField_Obj_Mineral_C_01_683356425",
0xae058a6a,"MainField_Obj_Mineral_C_01_697816803",
0xc88135c5,"MainField_Obj_Mineral_C_01_710522217",
0xc0e05e97,"MainField_Obj_Mineral_C_01_713009721",
0x001dd3ff,"MainField_Obj_Mineral_C_01_717474499",
0xa8ec59ed,"MainField_Obj_Mineral_C_01_721105966",
0xa25ea667,"MainField_Obj_Mineral_C_01_732707824",
0xcd729823,"MainField_Obj_Mineral_C_01_747540982",
0x1bb3dcf2,"MainField_Obj_Mineral_C_01_757398498",
0xa967479a,"MainField_Obj_Mineral_C_01_758008810",
0xd6e7d4a4,"MainField_Obj_Mineral_C_01_770001619",
0x12978ed9,"MainField_Obj_Mineral_C_01_78633",
0x2c25b1ee,"MainField_Obj_Mineral_C_01_793656171",
0x66c005c3,"MainField_Obj_Mineral_C_01_799618036",
0xd76902ad,"MainField_Obj_Mineral_C_01_813664538",
0x18a99c20,"MainField_Obj_Mineral_C_01_815664742",
0xf11e88f8,"MainField_Obj_Mineral_C_01_82451346",
0x7bbe3ca2,"MainField_Obj_Mineral_C_01_83102982",
0x10ddcaa8,"MainField_Obj_Mineral_C_01_831964984",
0x35667e14,"MainField_Obj_Mineral_C_01_84534431",
0x87d12fbb,"MainField_Obj_Mineral_C_01_853101283",
0x76545ce5,"MainField_Obj_Mineral_C_01_86111335",
0xe4aee358,"MainField_Obj_Mineral_C_01_888400178",
0x63747e05,"MainField_Obj_Mineral_C_01_909397230",
0x1b7fcda2,"MainField_Obj_Mineral_C_01_921257234",
0x0f98dc7c,"MainField_Obj_Mineral_C_01_931795484",
0xe5ff79f5,"MainField_Obj_Mineral_C_01_937496294",
0x34ec6248,"MainField_Obj_Mineral_C_01_939529966",
0x223e773c,"MainField_Obj_Mineral_C_01_943455296",
0x98cafaa3,"MainField_Obj_Mineral_C_01_946851434",
0xaeca886d,"MainField_Obj_Mineral_C_01_952822317",
0x95464d38,"MainField_Obj_Mineral_C_01_961508876",
0x26019647,"MainField_Obj_Mineral_C_01_988037777",
0x93c98803,"MainField_Obj_Mineral_C_01_999851370",
0x32b1c72a,"MainField_Obj_RockBroken_A_02_1055413791",
0x521661f0,"MainField_Obj_RockBroken_A_02_1187241548",
0x15c8e4a0,"MainField_Obj_RockBroken_A_02_1214368412",
0x8a7f1cd8,"MainField_Obj_RockBroken_A_02_1216476879",
0x93631a04,"MainField_Obj_RockBroken_A_02_1267416559",
0xe736217f,"MainField_Obj_RockBroken_A_02_1275516757",
0x11c5cf38,"MainField_Obj_RockBroken_A_02_127785352",
0x984c372b,"MainField_Obj_RockBroken_A_02_1287130849",
0xa266b1ea,"MainField_Obj_RockBroken_A_02_1335455710",
0x62e28c97,"MainField_Obj_RockBroken_A_02_1341038132",
0xee8f851a,"MainField_Obj_RockBroken_A_02_1561060605",
0xd410c9b7,"MainField_Obj_RockBroken_A_02_1845164774",
0x7edc4e79,"MainField_Obj_RockBroken_A_02_1848920418",
0x2901ca03,"MainField_Obj_RockBroken_A_02_187085830",
0xc3806b85,"MainField_Obj_RockBroken_A_02_1926971839",
0xb5f3c46d,"MainField_Obj_RockBroken_A_02_1986241752",
0xa94b8269,"MainField_Obj_RockBroken_A_02_2039909671",
0xa8ce1174,"MainField_Obj_RockBroken_A_02_2055687841",
0x80bb47b2,"MainField_Obj_RockBroken_A_02_2158811083",
0xc9efe76c,"MainField_Obj_RockBroken_A_02_2198910671",
0x70fefc99,"MainField_Obj_RockBroken_A_02_2240237407",
0x587a5374,"MainField_Obj_RockBroken_A_02_2262495264",
0x00ebb94d,"MainField_Obj_RockBroken_A_02_2323055119",
0x64f9d040,"MainField_Obj_RockBroken_A_02_2358527021",
0x63144061,"MainField_Obj_RockBroken_A_02_2376100034",
0x8b0a2c08,"MainField_Obj_RockBroken_A_02_2448448014",
0x4d124165,"MainField_Obj_RockBroken_A_02_251492681",
0x88dc7aad,"MainField_Obj_RockBroken_A_02_2645986832",
0xef635d5d,"MainField_Obj_RockBroken_A_02_265327175",
0x621a959c,"MainField_Obj_RockBroken_A_02_2707821196",
0xcbcd7f05,"MainField_Obj_RockBroken_A_02_2758234317",
0x27cc1c72,"MainField_Obj_RockBroken_A_02_2834192983",
0x95fac54f,"MainField_Obj_RockBroken_A_02_2837017341",
0xe1ca1e9c,"MainField_Obj_RockBroken_A_02_2959884018",
0xf62d004d,"MainField_Obj_RockBroken_A_02_3009740333",
0xe87705f1,"MainField_Obj_RockBroken_A_02_304013808",
0x0eb0eed9,"MainField_Obj_RockBroken_A_02_3126963162",
0x81425255,"MainField_Obj_RockBroken_A_02_318185113",
0xfbe2f1eb,"MainField_Obj_RockBroken_A_02_3293249951",
0xf79eab40,"MainField_Obj_RockBroken_A_02_3305109341",
0x3724a6a6,"MainField_Obj_RockBroken_A_02_3363943300",
0x7fcf2e7d,"MainField_Obj_RockBroken_A_02_3434209364",
0x2b9103d2,"MainField_Obj_RockBroken_A_02_3548701470",
0x2e992dde,"MainField_Obj_RockBroken_A_02_3571711315",
0x55249dd9,"MainField_Obj_RockBroken_A_02_367782553",
0x4e0f34d5,"MainField_Obj_RockBroken_A_02_373832974",
0xa1d67006,"MainField_Obj_RockBroken_A_02_3789090689",
0x3aa9339b,"MainField_Obj_RockBroken_A_02_3839781520",
0x2d9a19d9,"MainField_Obj_RockBroken_A_02_3961931559",
0xc007b644,"MainField_Obj_RockBroken_A_02_3991194176",
0xc14b8a70,"MainField_Obj_RockBroken_A_02_4058326385",
0x38586938,"MainField_Obj_RockBroken_A_02_4069170280",
0x2faa33c5,"MainField_Obj_RockBroken_A_02_4092803013",
0x8a3d88f6,"MainField_Obj_RockBroken_A_02_4106338782",
0x7a1161e0,"MainField_Obj_RockBroken_A_02_4197714488",
0xe7f75827,"MainField_Obj_RockBroken_A_02_4207978942",
0x874d1748,"MainField_Obj_RockBroken_A_02_4288931074",
0x82104d9c,"MainField_Obj_RockBroken_A_02_43056858",
0x7215eab6,"MainField_Obj_RockBroken_A_02_605296696",
0xc5e3027b,"MainField_Obj_RockBroken_A_02_64234690",
0xd24a9646,"MainField_Obj_RockBroken_A_02_706882323",
0x24eacd02,"MainField_Obj_RockBroken_A_02_754319320",
0x4df5fa93,"MainField_Obj_RockBroken_A_02_796768755",
0xf58a14c7,"MainField_Obj_RockBroken_A_02_830819744",
0xd7e4f705,"MainField_Obj_RockBroken_A_02_958062886",
0x197194b1,"MainField_Obj_RockBroken_A_02_963700439",
0x7c58022e,"MainField_Obj_RockBroken_A_02_970670533",
0x22637515,"MainField_Obj_StaminaUtuwa_A_01_1066041353",
0x34b1bee5,"MainField_Obj_StaminaUtuwa_A_01_1095486698",
0x783f07db,"MainField_Obj_StaminaUtuwa_A_01_1133580703",
0x9368abd4,"MainField_Obj_StaminaUtuwa_A_01_1571990601",
0x9bfa5e95,"MainField_Obj_StaminaUtuwa_A_01_2078849759",
0x8dfeec01,"MainField_Obj_StaminaUtuwa_A_01_2127738829",
0x27ac1368,"MainField_Obj_StaminaUtuwa_A_01_2410940791",
0x228eed5f,"MainField_Obj_StaminaUtuwa_A_01_2466114040",
0xe58ed210,"MainField_Obj_StaminaUtuwa_A_01_2528160939",
0x3a0a57b9,"MainField_Obj_StaminaUtuwa_A_01_3273771814",
0xec6088b9,"MainField_Obj_StaminaUtuwa_A_01_3316916448",
0x9ada164f,"MainField_Obj_StaminaUtuwa_A_01_3364311362",
0xb0ba7115,"MainField_Obj_StaminaUtuwa_A_01_710535482",
0xe1ac2450,"MainField_Obj_StaminaUtuwa_A_01_954296046",
0x7ef087b3,"MainField_Obj_ThunderRock_1792673843",
0x28df75d8,"MainField_OreDrop0_Enemy_Golem_Fire_1372356658",
0xbb5cfe9a,"MainField_OreDrop0_Enemy_Golem_Fire_2080410037",
0x92482ba1,"MainField_OreDrop0_Enemy_Golem_Fire_2112489205",
0x9291ec6b,"MainField_OreDrop0_Enemy_Golem_Fire_3180486693",
0x76124ac4,"MainField_OreDrop0_Enemy_Golem_Fire_4143112243",
0x29fc1515,"MainField_OreDrop0_Enemy_Golem_Ice_1684711544",
0x01b71b39,"MainField_OreDrop0_Enemy_Golem_Ice_189892196",
0x4dc33c9f,"MainField_OreDrop0_Enemy_Golem_Ice_4007690024",
0xe4606412,"MainField_OreDrop0_Enemy_Golem_Ice_4087740554",
0xcb5c3a05,"MainField_OreDrop0_Enemy_Golem_Ice_648092042",
0xc04fe5c1,"MainField_OreDrop0_Enemy_Golem_Junior_139564765",
0x3e0e1aba,"MainField_OreDrop0_Enemy_Golem_Junior_1450825465",
0xee11a8b5,"MainField_OreDrop0_Enemy_Golem_Junior_2009332164",
0xd70e2932,"MainField_OreDrop0_Enemy_Golem_Junior_3610208070",
0x50e08122,"MainField_OreDrop0_Enemy_Golem_Junior_3688513041",
0xa479376e,"MainField_OreDrop0_Enemy_Golem_Junior_3707687090",
0x0a2e6d3a,"MainField_OreDrop0_Enemy_Golem_Junior_3817875409",
0xece09113,"MainField_OreDrop0_Enemy_Golem_Junior_3873221297",
0xc95d48ac,"MainField_OreDrop0_Enemy_Golem_Junior_4097205930",
0x6ef74d34,"MainField_OreDrop0_Enemy_Golem_Junior_4234397911",
0x9704443a,"MainField_OreDrop0_Enemy_Golem_Junior_481698249",
0x257750e7,"MainField_OreDrop0_Enemy_Golem_Middle_1063952470",
0xb188a2d2,"MainField_OreDrop0_Enemy_Golem_Middle_2367737901",
0x6b9aed60,"MainField_OreDrop0_Enemy_Golem_Middle_2827830463",
0xa8166d77,"MainField_OreDrop0_Enemy_Golem_Middle_3273877672",
0x378ea05b,"MainField_OreDrop0_Enemy_Golem_Middle_3529818719",
0xbf5f8296,"MainField_OreDrop0_Enemy_Golem_Middle_3605751469",
0x22bddd16,"MainField_OreDrop0_Enemy_Golem_Middle_372172601",
0x0ddc97b1,"MainField_OreDrop0_Enemy_Golem_Middle_3812413946",
0x51e5e45f,"MainField_OreDrop0_Enemy_Golem_Middle_3900382371",
0x2d5a4f66,"MainField_OreDrop0_Enemy_Golem_Middle_491623064",
0x2a076d00,"MainField_OreDrop0_Enemy_Golem_Middle_853217538",
0x02faac59,"MainField_OreDrop0_Enemy_Golem_Middle_975119268",
0xf53d61ca,"MainField_OreDrop0_Enemy_Golem_Senior_1943962273",
0xb10e5f87,"MainField_OreDrop0_Enemy_Golem_Senior_2455625785",
0xf4184644,"MainField_OreDrop0_Enemy_Golem_Senior_2530932938",
0x5926d086,"MainField_OreDrop0_Enemy_Golem_Senior_2763213354",
0x02a69188,"MainField_OreDrop0_Enemy_Golem_Senior_321907823",
0x53606fb8,"MainField_OreDrop0_Enemy_Golem_Senior_3869714669",
0xc54f562c,"MainField_OreDrop0_Enemy_Golem_Senior_464316212",
0x4bd8acfc,"MainField_OreDrop1_Enemy_Golem_Fire_1372356658",
0xd85b27be,"MainField_OreDrop1_Enemy_Golem_Fire_2080410037",
0xf14ff285,"MainField_OreDrop1_Enemy_Golem_Fire_2112489205",
0xf196354f,"MainField_OreDrop1_Enemy_Golem_Fire_3180486693",
0x151593e0,"MainField_OreDrop1_Enemy_Golem_Fire_4143112243",
0x2849e808,"MainField_OreDrop1_Enemy_Golem_Ice_1684711544",
0x6f3b0078,"MainField_OreDrop1_Enemy_Golem_Ice_189892196",
0x4c76c182,"MainField_OreDrop1_Enemy_Golem_Ice_4007690024",
0xe5d5990f,"MainField_OreDrop1_Enemy_Golem_Ice_4087740554",
0xa5d02144,"MainField_OreDrop1_Enemy_Golem_Ice_648092042",
0xfc2f06c9,"MainField_OreDrop1_Enemy_Golem_Junior_139564765",
0x30e9f26b,"MainField_OreDrop1_Enemy_Golem_Junior_1450825465",
0xe0f64064,"MainField_OreDrop1_Enemy_Golem_Junior_2009332164",
0xd9e9c1e3,"MainField_OreDrop1_Enemy_Golem_Junior_3610208070",
0x5e0769f3,"MainField_OreDrop1_Enemy_Golem_Junior_3688513041",
0xaa9edfbf,"MainField_OreDrop1_Enemy_Golem_Junior_3707687090",
0x04c985eb,"MainField_OreDrop1_Enemy_Golem_Junior_3817875409",
0xe20779c2,"MainField_OreDrop1_Enemy_Golem_Junior_3873221297",
0xc7baa07d,"MainField_OreDrop1_Enemy_Golem_Junior_4097205930",
0x6010a5e5,"MainField_OreDrop1_Enemy_Golem_Junior_4234397911",
0xab64a732,"MainField_OreDrop1_Enemy_Golem_Junior_481698249",
0x2b90b836,"MainField_OreDrop1_Enemy_Golem_Middle_1063952470",
0xbf6f4a03,"MainField_OreDrop1_Enemy_Golem_Middle_2367737901",
0x657d05b1,"MainField_OreDrop1_Enemy_Golem_Middle_2827830463",
0xa6f185a6,"MainField_OreDrop1_Enemy_Golem_Middle_3273877672",
0x3969488a,"MainField_OreDrop1_Enemy_Golem_Middle_3529818719",
0xb1b86a47,"MainField_OreDrop1_Enemy_Golem_Middle_3605751469",
0x1edd3e1e,"MainField_OreDrop1_Enemy_Golem_Middle_372172601",
0x033b7f60,"MainField_OreDrop1_Enemy_Golem_Middle_3812413946",
0x5f020c8e,"MainField_OreDrop1_Enemy_Golem_Middle_3900382371",
0x113aac6e,"MainField_OreDrop1_Enemy_Golem_Middle_491623064",
0x16678e08,"MainField_OreDrop1_Enemy_Golem_Middle_853217538",
0x3e9a4f51,"MainField_OreDrop1_Enemy_Golem_Middle_975119268",
0xfbda891b,"MainField_OreDrop1_Enemy_Golem_Senior_1943962273",
0xbfe9b756,"MainField_OreDrop1_Enemy_Golem_Senior_2455625785",
0xfaffae95,"MainField_OreDrop1_Enemy_Golem_Senior_2530932938",
0x57c13857,"MainField_OreDrop1_Enemy_Golem_Senior_2763213354",
0x3ec67280,"MainField_OreDrop1_Enemy_Golem_Senior_321907823",
0x5d878769,"MainField_OreDrop1_Enemy_Golem_Senior_3869714669",
0xf92fb524,"MainField_OreDrop1_Enemy_Golem_Senior_464316212",
0xeed0c790,"MainField_OreDrop2_Enemy_Golem_Fire_1372356658",
0x7d534cd2,"MainField_OreDrop2_Enemy_Golem_Fire_2080410037",
0x544799e9,"MainField_OreDrop2_Enemy_Golem_Fire_2112489205",
0x549e5e23,"MainField_OreDrop2_Enemy_Golem_Fire_3180486693",
0xb01df88c,"MainField_OreDrop2_Enemy_Golem_Fire_4143112243",
0x2a97ef2f,"MainField_OreDrop2_Enemy_Golem_Ice_1684711544",
0xdcaf2dbb,"MainField_OreDrop2_Enemy_Golem_Ice_189892196",
0x4ea8c6a5,"MainField_OreDrop2_Enemy_Golem_Ice_4007690024",
0xe70b9e28,"MainField_OreDrop2_Enemy_Golem_Ice_4087740554",
0x16440c87,"MainField_OreDrop2_Enemy_Golem_Ice_648092042",
0xb88e23d1,"MainField_OreDrop2_Enemy_Golem_Junior_139564765",
0x23c1cb18,"MainField_OreDrop2_Enemy_Golem_Junior_1450825465",
0xf3de7917,"MainField_OreDrop2_Enemy_Golem_Junior_2009332164",
0xcac1f890,"MainField_OreDrop2_Enemy_Golem_Junior_3610208070",
0x4d2f5080,"MainField_OreDrop2_Enemy_Golem_Junior_3688513041",
0xb9b6e6cc,"MainField_OreDrop2_Enemy_Golem_Junior_3707687090",
0x17e1bc98,"MainField_OreDrop2_Enemy_Golem_Junior_3817875409",
0xf12f40b1,"MainField_OreDrop2_Enemy_Golem_Junior_3873221297",
0xd492990e,"MainField_OreDrop2_Enemy_Golem_Junior_4097205930",
0x73389c96,"MainField_OreDrop2_Enemy_Golem_Junior_4234397911",
0xefc5822a,"MainField_OreDrop2_Enemy_Golem_Junior_481698249",
0x38b88145,"MainField_OreDrop2_Enemy_Golem_Middle_1063952470",
0xac477370,"MainField_OreDrop2_Enemy_Golem_Middle_2367737901",
0x76553cc2,"MainField_OreDrop2_Enemy_Golem_Middle_2827830463",
0xb5d9bcd5,"MainField_OreDrop2_Enemy_Golem_Middle_3273877672",
0x2a4171f9,"MainField_OreDrop2_Enemy_Golem_Middle_3529818719",
0xa2905334,"MainField_OreDrop2_Enemy_Golem_Middle_3605751469",
0x5a7c1b06,"MainField_OreDrop2_Enemy_Golem_Middle_372172601",
0x10134613,"MainField_OreDrop2_Enemy_Golem_Middle_3812413946",
0x4c2a35fd,"MainField_OreDrop2_Enemy_Golem_Middle_3900382371",
0x559b8976,"MainField_OreDrop2_Enemy_Golem_Middle_491623064",
0x52c6ab10,"MainField_OreDrop2_Enemy_Golem_Middle_853217538",
0x7a3b6a49,"MainField_OreDrop2_Enemy_Golem_Middle_975119268",
0xe8f2b068,"MainField_OreDrop2_Enemy_Golem_Senior_1943962273",
0xacc18e25,"MainField_OreDrop2_Enemy_Golem_Senior_2455625785",
0xe9d797e6,"MainField_OreDrop2_Enemy_Golem_Senior_2530932938",
0x44e90124,"MainField_OreDrop2_Enemy_Golem_Senior_2763213354",
0x7a675798,"MainField_OreDrop2_Enemy_Golem_Senior_321907823",
0x4eafbe1a,"MainField_OreDrop2_Enemy_Golem_Senior_3869714669",
0xbd8e903c,"MainField_OreDrop2_Enemy_Golem_Senior_464316212",
0x2d3e691b,"MainField_Pot_1038440407",
0x1f52fef7,"MainField_Pot_1050803184",
0x62c3baa4,"MainField_Pot_1060825144",
0x6d329057,"MainField_Pot_1138688213",
0x3698e783,"MainField_Pot_1166430471",
0xb9cbd408,"MainField_Pot_1189138232",
0xfed49903,"MainField_Pot_1291850534",
0xaa22cf9d,"MainField_Pot_1299355645",
0xe7d0aa4b,"MainField_Pot_1299479005",
0x39473313,"MainField_Pot_1299817885",
0xa71feeef,"MainField_Pot_1332244173",
0x08e69b74,"MainField_Pot_1385086688",
0xb1e7efc7,"MainField_Pot_1452599201",
0x221084f4,"MainField_Pot_1458299983",
0x164ad533,"MainField_Pot_1563105975",
0x2057302d,"MainField_Pot_1674307494",
0x3448ae4d,"MainField_Pot_1689920271",
0xb9c0d795,"MainField_Pot_1690871764",
0x61bc4d45,"MainField_Pot_1736214005",
0x29eaf3f1,"MainField_Pot_1794249204",
0xe6bc3f4d,"MainField_Pot_1837294989",
0x6a82e4da,"MainField_Pot_1844470102",
0x564dab77,"MainField_Pot_1893078390",
0x2754fd69,"MainField_Pot_1947562894",
0xb4825c21,"MainField_Pot_194908397",
0x7de1ec94,"MainField_Pot_208702579",
0x7187dec0,"MainField_Pot_2133020861",
0xb4cd06d2,"MainField_Pot_228977059",
0x116947a5,"MainField_Pot_2337433061",
0xbda6a107,"MainField_Pot_2455301497",
0x89567162,"MainField_Pot_2468914475",
0xc0c1a5f2,"MainField_Pot_251287004",
0x1799c13a,"MainField_Pot_252465665",
0xac666970,"MainField_Pot_2571449690",
0x38ae5e0c,"MainField_Pot_2598408542",
0xd2e9212b,"MainField_Pot_2673295911",
0x934a2581,"MainField_Pot_2702057162",
0x8900bc18,"MainField_Pot_2779049505",
0x8b865a1d,"MainField_Pot_2832221130",
0xb24c337c,"MainField_Pot_2906241512",
0xd7e3a7ba,"MainField_Pot_2942428984",
0x2648ee1a,"MainField_Pot_2962063784",
0x339152fe,"MainField_Pot_3026009551",
0x5872237d,"MainField_Pot_3111974435",
0x954e451c,"MainField_Pot_311222266",
0x07063fcd,"MainField_Pot_3134068125",
0x8b3ead77,"MainField_Pot_3253519623",
0x5e44a8fc,"MainField_Pot_3262625598",
0x07a07cc5,"MainField_Pot_3281580231",
0x51d71efb,"MainField_Pot_3286922268",
0x6d764451,"MainField_Pot_3402915006",
0x874fcd70,"MainField_Pot_3483219483",
0xc02a01db,"MainField_Pot_3543333346",
0x1fedb3ca,"MainField_Pot_3652943814",
0x7bf6c1a7,"MainField_Pot_3733413405",
0x4cb76501,"MainField_Pot_3752181577",
0x6f568ba3,"MainField_Pot_3755438940",
0x431a75dd,"MainField_Pot_377284172",
0x7974a1c9,"MainField_Pot_3810133612",
0x4ae82811,"MainField_Pot_3810322508",
0x75d7de81,"MainField_Pot_3815277239",
0xf2fab8e7,"MainField_Pot_3991380424",
0x49beebce,"MainField_Pot_4019848397",
0xf709ceaa,"MainField_Pot_4040552732",
0x10431c5e,"MainField_Pot_4086360230",
0x28e46f0e,"MainField_Pot_4177017473",
0x3dfdacb7,"MainField_Pot_4198495238",
0xe2c3abbc,"MainField_Pot_4236085999",
0x02884f1e,"MainField_Pot_4265925139",
0xb8f32164,"MainField_Pot_518328377",
0x697e874a,"MainField_Pot_594104148",
0x609e5cae,"MainField_Pot_65650916",
0x60fefccb,"MainField_Pot_765916229",
0x3a9206f6,"MainField_Pot_915797144",
0x93721754,"MainField_Pot_925738187",
0x0f1cbfb0,"MainField_Pot_979254898",
0x6465c1c0,"MainField_PutRupee_1040969445",
0x622ee1c6,"MainField_PutRupee_1159982928",
0xc3aa5c8f,"MainField_PutRupee_1213807974",
0x906bae95,"MainField_PutRupee_1283477163",
0xe9444520,"MainField_PutRupee_1432054836",
0x8198ec26,"MainField_PutRupee_1461170453",
0x6bc7355d,"MainField_PutRupee_1527455457",
0x86e2d3db,"MainField_PutRupee_1601462832",
0x39037f8d,"MainField_PutRupee_1782761318",
0xb0f0ae76,"MainField_PutRupee_1852551277",
0xe20172db,"MainField_PutRupee_202054766",
0x3c6b7460,"MainField_PutRupee_2126312073",
0x13b300ed,"MainField_PutRupee_2306525407",
0x350cc576,"MainField_PutRupee_2325407173",
0x457dfe3f,"MainField_PutRupee_2487744376",
0xcdc7cdc8,"MainField_PutRupee_2597220176",
0xe97ea3cf,"MainField_PutRupee_2608765863",
0xc6473c89,"MainField_PutRupee_2698334614",
0x345c8f9a,"MainField_PutRupee_2849504542",
0xa9426d53,"MainField_PutRupee_2907967879",
0x75a9c10b,"MainField_PutRupee_3190788589",
0x412f9710,"MainField_PutRupee_3249030714",
0xe91d16a9,"MainField_PutRupee_3361697926",
0xe81b4220,"MainField_PutRupee_3489526322",
0xd365cca5,"MainField_PutRupee_3647779341",
0x71e0c8b2,"MainField_PutRupee_3729864155",
0x253c2882,"MainField_PutRupee_4008895000",
0xa01646ba,"MainField_PutRupee_4262972186",
0x4f7b97ad,"MainField_PutRupee_4275393811",
0x2d670a21,"MainField_PutRupee_4287665837",
0x2cb44f33,"MainField_PutRupee_526351245",
0x58a6f93d,"MainField_PutRupee_606448425",
0xcb98bbed,"MainField_PutRupee_79873720",
0x62023efb,"MainField_PutRupee_821344815",
0x68b62265,"MainField_PutRupee_825346524",
0x54260a3d,"MainField_PutRupee_Blue_106851533",
0xc91df832,"MainField_PutRupee_Blue_1274371889",
0x9903dcfe,"MainField_PutRupee_Blue_1447679489",
0x5157dd5a,"MainField_PutRupee_Blue_1630834295",
0x052f8731,"MainField_PutRupee_Blue_1696787807",
0xc31cafc9,"MainField_PutRupee_Blue_2155257810",
0x7da3f442,"MainField_PutRupee_Blue_2775862597",
0x8a481806,"MainField_PutRupee_Blue_2804691815",
0xd687d243,"MainField_PutRupee_Blue_3058514911",
0xb708a66a,"MainField_PutRupee_Blue_3072939414",
0xac8730a2,"MainField_PutRupee_Blue_3090941416",
0xb14d3279,"MainField_PutRupee_Blue_3194372447",
0xfaf3028a,"MainField_PutRupee_Blue_3221817831",
0xecdaa6ae,"MainField_PutRupee_Blue_3308205033",
0x8a3af41b,"MainField_PutRupee_Blue_3571174137",
0xc14c0245,"MainField_PutRupee_Blue_3618034427",
0x5ee26067,"MainField_PutRupee_Blue_369372691",
0xea29b77c,"MainField_PutRupee_Blue_4003188734",
0xb25ac239,"MainField_PutRupee_Blue_524936855",
0x711634d6,"MainField_PutRupee_Blue_534290251",
0xd0f67bb6,"MainField_PutRupee_Blue_777439277",
0xb3b7514a,"MainField_PutRupee_Gold_2935520467",
0x73099ad8,"MainField_PutRupee_Purple_2581467609",
0x36b4b860,"MainField_PutRupee_Red_1031330376",
0x82b599c5,"MainField_PutRupee_Red_1035686757",
0x81b59907,"MainField_PutRupee_Red_1080492145",
0x7dc263a7,"MainField_PutRupee_Red_1210034034",
0xb0022744,"MainField_PutRupee_Red_1647934741",
0x386bc1bc,"MainField_PutRupee_Red_1668606224",
0xc5578099,"MainField_PutRupee_Red_1697652372",
0x87effa11,"MainField_PutRupee_Red_2278230051",
0x751001d6,"MainField_PutRupee_Red_2396707565",
0x4d4787a8,"MainField_PutRupee_Red_243330295",
0x7ced23a4,"MainField_PutRupee_Red_2811740826",
0xf0c2a92d,"MainField_PutRupee_Red_3002982672",
0xb5fa7490,"MainField_PutRupee_Red_3254111910",
0x063fed2f,"MainField_PutRupee_Red_3358209708",
0xa41673d0,"MainField_PutRupee_Red_3531983281",
0x7ad1d1b7,"MainField_PutRupee_Red_3653948918",
0xa136c959,"MainField_PutRupee_Red_4204357279",
0xa82264bd,"MainField_PutRupee_Red_477889961",
0x22021500,"MainField_PutRupee_Red_613261440",
0x9ed61047,"MainField_PutRupee_Silver_200311821",
0xc5b9f723,"MainField_PutRupee_Silver_2150445967",
0x165b41cb,"MainField_PutRupee_Silver_4185699630",
0xd638f607,"MainField_RemainsFire_Drone_A_01_1182513706",
0x6bf0f9ef,"MainField_RemainsFire_Drone_A_01_1646387036",
0xdc890ace,"MainField_RemainsFire_Drone_A_01_1934542849",
0x16b72283,"MainField_RemainsFire_Drone_A_01_1939486171",
0xb40bb329,"MainField_RemainsFire_Drone_A_01_1948453912",
0x648a9fc4,"MainField_RemainsFire_Drone_A_01_2431370498",
0x460388d7,"MainField_RemainsFire_Drone_A_01_2537119220",
0xfd1fd949,"MainField_RemainsFire_Drone_A_01_2586668340",
0xb55fc7c5,"MainField_RemainsFire_Drone_A_01_2638260525",
0xa3f26b85,"MainField_RemainsFire_Drone_A_01_2843509050",
0x328b7ef0,"MainField_RemainsFire_Drone_A_01_3317240223",
0xda95574f,"MainField_RemainsFire_Drone_A_01_3732648364",
0xed957256,"MainField_RemainsFire_Drone_A_01_3930560955",
0x8365ebc4,"MainField_RemainsFire_Drone_A_01_4014930669",
0x2c61e6db,"MainField_RemainsFire_Drone_A_01_52706446",
0xc3a7e60e,"MainField_RemainsFire_Drone_A_01_624780687",
0x7fe7c231,"MainField_RemainsFire_Drone_A_01_71956631",
0x5ebea308,"MainField_RemainsFire_Drone_A_01_783750458",
0x5df598c8,"MainField_TBox_Field_Enemy_1039277961",
0x6fbb9f7d,"MainField_TBox_Field_Enemy_1046310171",
0x5e5cc69c,"MainField_TBox_Field_Enemy_1081762818",
0x0c929e0a,"MainField_TBox_Field_Enemy_112042483",
0x759881eb,"MainField_TBox_Field_Enemy_1149811241",
0x2bea221f,"MainField_TBox_Field_Enemy_1153264704",
0xee4bb35c,"MainField_TBox_Field_Enemy_1166023328",
0x557459ee,"MainField_TBox_Field_Enemy_1177129274",
0xf325e910,"MainField_TBox_Field_Enemy_1408506157",
0x37fbe18a,"MainField_TBox_Field_Enemy_1449120336",
0xfbcec560,"MainField_TBox_Field_Enemy_1451704573",
0x0abf7fb0,"MainField_TBox_Field_Enemy_1610925690",
0xdee3dda4,"MainField_TBox_Field_Enemy_1649297148",
0x2662ac26,"MainField_TBox_Field_Enemy_1766782611",
0x0a73eba4,"MainField_TBox_Field_Enemy_1769347722",
0xa8b28304,"MainField_TBox_Field_Enemy_1795066482",
0x19b08fb0,"MainField_TBox_Field_Enemy_184482119",
0x1811b9ee,"MainField_TBox_Field_Enemy_1845994126",
0x37f010c4,"MainField_TBox_Field_Enemy_2142154964",
0xedce9582,"MainField_TBox_Field_Enemy_2287855586",
0xb615c993,"MainField_TBox_Field_Enemy_2335430415",
0xc1445ca6,"MainField_TBox_Field_Enemy_2358531767",
0x99eefbd2,"MainField_TBox_Field_Enemy_2495402960",
0xeb9fc740,"MainField_TBox_Field_Enemy_2563698991",
0xb6145b0f,"MainField_TBox_Field_Enemy_2586310741",
0x4a0f53b4,"MainField_TBox_Field_Enemy_2690546993",
0x5e8a8454,"MainField_TBox_Field_Enemy_2746088988",
0x0e1992c6,"MainField_TBox_Field_Enemy_2747618053",
0x0610c16c,"MainField_TBox_Field_Enemy_2786225120",
0xf76a6f2e,"MainField_TBox_Field_Enemy_2800841551",
0x73ccfb45,"MainField_TBox_Field_Enemy_2821016294",
0xc1766a92,"MainField_TBox_Field_Enemy_290764376",
0x23aee0df,"MainField_TBox_Field_Enemy_3023897834",
0xe01b5d11,"MainField_TBox_Field_Enemy_3070830878",
0x3e5a5e8d,"MainField_TBox_Field_Enemy_3101039837",
0xa6d49928,"MainField_TBox_Field_Enemy_3122256932",
0x6f5fe32f,"MainField_TBox_Field_Enemy_3126264210",
0x550bc474,"MainField_TBox_Field_Enemy_3246083737",
0x537d2d76,"MainField_TBox_Field_Enemy_3437107610",
0x740728f1,"MainField_TBox_Field_Enemy_3451085184",
0x58dde0cc,"MainField_TBox_Field_Enemy_3467431944",
0x55212b1c,"MainField_TBox_Field_Enemy_3509188362",
0x62807c2b,"MainField_TBox_Field_Enemy_3587085179",
0x9bedcdf8,"MainField_TBox_Field_Enemy_3605229816",
0x67d9601b,"MainField_TBox_Field_Enemy_3606380063",
0x47f04e9a,"MainField_TBox_Field_Enemy_3746623201",
0xbe343593,"MainField_TBox_Field_Enemy_3851766039",
0x384a5ea1,"MainField_TBox_Field_Enemy_418291669",
0x352ee9af,"MainField_TBox_Field_Enemy_4220741153",
0x6866ae04,"MainField_TBox_Field_Enemy_4284143476",
0x5fa54030,"MainField_TBox_Field_Enemy_432831078",
0xf25a0587,"MainField_TBox_Field_Enemy_464533090",
0x85cfd98d,"MainField_TBox_Field_Enemy_562065515",
0x44690cbb,"MainField_TBox_Field_Enemy_999953490",
0x39951405,"MainField_TBox_Field_Iron_1001778960",
0xdaeae7d3,"MainField_TBox_Field_Iron_100295353",
0xc3cb4830,"MainField_TBox_Field_Iron_1007253771",
0x9b3654ae,"MainField_TBox_Field_Iron_1009245369",
0x85f6f586,"MainField_TBox_Field_Iron_1019895495",
0xc3915686,"MainField_TBox_Field_Iron_1034798336",
0x8503cb93,"MainField_TBox_Field_Iron_1035186669",
0x962eef71,"MainField_TBox_Field_Iron_1039414755",
0xde759c95,"MainField_TBox_Field_Iron_1053938972",
0x316ae1b6,"MainField_TBox_Field_Iron_1054358123",
0x0c48b750,"MainField_TBox_Field_Iron_1058824036",
0x3852a337,"MainField_TBox_Field_Iron_1060034856",
0x9263a8de,"MainField_TBox_Field_Iron_1061698721",
0xec7e4f0d,"MainField_TBox_Field_Iron_1062031406",
0x708ef5c5,"MainField_TBox_Field_Iron_1068442606",
0x65d10179,"MainField_TBox_Field_Iron_1068730519",
0x317e43da,"MainField_TBox_Field_Iron_1069229842",
0xc1acb15d,"MainField_TBox_Field_Iron_1070881855",
0x7d27a529,"MainField_TBox_Field_Iron_1082549397",
0xe6d48fbf,"MainField_TBox_Field_Iron_1085667751",
0x60792a86,"MainField_TBox_Field_Iron_1088776091",
0xb45824b8,"MainField_TBox_Field_Iron_1102353536",
0xfb585339,"MainField_TBox_Field_Iron_1122705485",
0x41d82677,"MainField_TBox_Field_Iron_1152452426",
0xb42a2a09,"MainField_TBox_Field_Iron_1154148229",
0x78915cf7,"MainField_TBox_Field_Iron_1160943436",
0x031fed47,"MainField_TBox_Field_Iron_1164859773",
0x946263ac,"MainField_TBox_Field_Iron_1167755580",
0xd57d3733,"MainField_TBox_Field_Iron_1185882003",
0x45fa9a46,"MainField_TBox_Field_Iron_1197778593",
0xbd202bc3,"MainField_TBox_Field_Iron_1219292146",
0xe1226ec6,"MainField_TBox_Field_Iron_1223229531",
0xb73599ce,"MainField_TBox_Field_Iron_1225528380",
0x89f819ec,"MainField_TBox_Field_Iron_1231558654",
0x4bb5837d,"MainField_TBox_Field_Iron_1239682",
0x9198bb2d,"MainField_TBox_Field_Iron_1248915646",
0xbf247318,"MainField_TBox_Field_Iron_130530844",
0x96f39335,"MainField_TBox_Field_Iron_1305945533",
0x67a18ad1,"MainField_TBox_Field_Iron_1307332435",
0x3c51ee1f,"MainField_TBox_Field_Iron_131922223",
0x741ea6a4,"MainField_TBox_Field_Iron_1320534692",
0x4b1e82b6,"MainField_TBox_Field_Iron_1333419385",
0x6e46d928,"MainField_TBox_Field_Iron_1350638657",
0x7ba25e3d,"MainField_TBox_Field_Iron_1367637753",
0x84fae8b9,"MainField_TBox_Field_Iron_1369310247",
0xc9a69828,"MainField_TBox_Field_Iron_137887917",
0x346c92c9,"MainField_TBox_Field_Iron_1379201832",
0xf53d2c08,"MainField_TBox_Field_Iron_1385678178",
0x408b44ba,"MainField_TBox_Field_Iron_1389340257",
0xb555261c,"MainField_TBox_Field_Iron_139569725",
0xb4c3dde0,"MainField_TBox_Field_Iron_139611962",
0xcc531719,"MainField_TBox_Field_Iron_1400211312",
0x4bdbb104,"MainField_TBox_Field_Iron_1401384830",
0x09828fe6,"MainField_TBox_Field_Iron_1403466912",
0x47cf0454,"MainField_TBox_Field_Iron_1420845522",
0xf79fb578,"MainField_TBox_Field_Iron_1424156752",
0x0250bd01,"MainField_TBox_Field_Iron_1429812883",
0xef2bfece,"MainField_TBox_Field_Iron_1437174372",
0x336687b8,"MainField_TBox_Field_Iron_1438333699",
0x4da390d9,"MainField_TBox_Field_Iron_1444486421",
0x70ff6729,"MainField_TBox_Field_Iron_1453896671",
0x68c86cf0,"MainField_TBox_Field_Iron_1454666335",
0x138b134e,"MainField_TBox_Field_Iron_1469049575",
0xb96b4ef2,"MainField_TBox_Field_Iron_1471186924",
0xd59089f5,"MainField_TBox_Field_Iron_1482345138",
0x5868ee30,"MainField_TBox_Field_Iron_1483366419",
0x757d03e2,"MainField_TBox_Field_Iron_1496117456",
0x9df9a033,"MainField_TBox_Field_Iron_1498077515",
0xc2cbd5d0,"MainField_TBox_Field_Iron_1506497297",
0x3a899804,"MainField_TBox_Field_Iron_1510666715",
0x6b7bc782,"MainField_TBox_Field_Iron_1520451930",
0x81d009f6,"MainField_TBox_Field_Iron_152120795",
0x34acb543,"MainField_TBox_Field_Iron_1529964834",
0xb783038f,"MainField_TBox_Field_Iron_1530743282",
0xfd85b8b3,"MainField_TBox_Field_Iron_1532272583",
0x930ad5df,"MainField_TBox_Field_Iron_1535280777",
0x8cebd331,"MainField_TBox_Field_Iron_1549603485",
0x87b239f0,"MainField_TBox_Field_Iron_1559548426",
0x75065cbb,"MainField_TBox_Field_Iron_1574761993",
0x11008e90,"MainField_TBox_Field_Iron_1594216329",
0x745fc1ce,"MainField_TBox_Field_Iron_160190496",
0x41ca72d9,"MainField_TBox_Field_Iron_1602544782",
0xd60888e5,"MainField_TBox_Field_Iron_1603123707",
0xdf8f1bc8,"MainField_TBox_Field_Iron_1623139313",
0x07e6aced,"MainField_TBox_Field_Iron_1637373433",
0xd410d045,"MainField_TBox_Field_Iron_164330944",
0x48f9c405,"MainField_TBox_Field_Iron_1650509316",
0x6b7abfa4,"MainField_TBox_Field_Iron_1651840378",
0xef344468,"MainField_TBox_Field_Iron_1655137296",
0x0ffc8534,"MainField_TBox_Field_Iron_1655335696",
0x0adfcd27,"MainField_TBox_Field_Iron_1675229188",
0xa1c19ed2,"MainField_TBox_Field_Iron_169212219",
0x8474ef35,"MainField_TBox_Field_Iron_1705997727",
0xacac1e01,"MainField_TBox_Field_Iron_1721250700",
0xf68f33bc,"MainField_TBox_Field_Iron_1722978285",
0xd7e89b29,"MainField_TBox_Field_Iron_1728252078",
0x544174b1,"MainField_TBox_Field_Iron_1731294309",
0x08d10e04,"MainField_TBox_Field_Iron_1732049083",
0xcd79e512,"MainField_TBox_Field_Iron_1737237722",
0xcc479262,"MainField_TBox_Field_Iron_1752241026",
0x46c5183c,"MainField_TBox_Field_Iron_1754409777",
0xb8c36d14,"MainField_TBox_Field_Iron_1756408700",
0xf3c25232,"MainField_TBox_Field_Iron_1756624524",
0xb4b9c960,"MainField_TBox_Field_Iron_1757302122",
0x6b104b66,"MainField_TBox_Field_Iron_1767830151",
0xaeb637e9,"MainField_TBox_Field_Iron_1784715464",
0x9b9dd2cd,"MainField_TBox_Field_Iron_1796411624",
0xfaceec17,"MainField_TBox_Field_Iron_1813492430",
0x48d18527,"MainField_TBox_Field_Iron_1825524602",
0x26780977,"MainField_TBox_Field_Iron_1833151420",
0x51af4ee2,"MainField_TBox_Field_Iron_1833976329",
0x2b9d8623,"MainField_TBox_Field_Iron_1836137588",
0x186a92f1,"MainField_TBox_Field_Iron_1836271291",
0xfbe30f3d,"MainField_TBox_Field_Iron_1848407939",
0xc60b5a60,"MainField_TBox_Field_Iron_1850654508",
0xc4732d2f,"MainField_TBox_Field_Iron_185132469",
0xe2ddb2e3,"MainField_TBox_Field_Iron_1861981528",
0x31b87c0a,"MainField_TBox_Field_Iron_1867978954",
0x60cc2994,"MainField_TBox_Field_Iron_1876238077",
0xba6fc420,"MainField_TBox_Field_Iron_1880894645",
0x7cc78ed1,"MainField_TBox_Field_Iron_1882321148",
0x808e93f3,"MainField_TBox_Field_Iron_1893981910",
0xec4d80c8,"MainField_TBox_Field_Iron_1893983190",
0x4ed8c132,"MainField_TBox_Field_Iron_1900840968",
0x41565a09,"MainField_TBox_Field_Iron_1901053128",
0x56ee2a02,"MainField_TBox_Field_Iron_191517764",
0xb4babffd,"MainField_TBox_Field_Iron_191697808",
0xfa299436,"MainField_TBox_Field_Iron_1930134405",
0x0a70c032,"MainField_TBox_Field_Iron_1933608495",
0x8116f09a,"MainField_TBox_Field_Iron_1937646263",
0x324d6b76,"MainField_TBox_Field_Iron_195317545",
0x1e6ff042,"MainField_TBox_Field_Iron_1963694381",
0xbeb3c37f,"MainField_TBox_Field_Iron_196387272",
0xc2245c53,"MainField_TBox_Field_Iron_1965946491",
0xcce1b01e,"MainField_TBox_Field_Iron_1978005036",
0xff6a46fc,"MainField_TBox_Field_Iron_1982372038",
0xc729d89b,"MainField_TBox_Field_Iron_1992338821",
0x0debba5d,"MainField_TBox_Field_Iron_199399775",
0xcf2324cd,"MainField_TBox_Field_Iron_2008800968",
0xb8970d45,"MainField_TBox_Field_Iron_2011178569",
0x726c803e,"MainField_TBox_Field_Iron_2023665230",
0x8ea1c2f6,"MainField_TBox_Field_Iron_2029607683",
0xeb7e33d8,"MainField_TBox_Field_Iron_203206732",
0x7d06167b,"MainField_TBox_Field_Iron_2053491615",
0x879e543a,"MainField_TBox_Field_Iron_2055514071",
0xed8c0b46,"MainField_TBox_Field_Iron_2055560919",
0xbc5be0db,"MainField_TBox_Field_Iron_2061310186",
0xdbc00e9d,"MainField_TBox_Field_Iron_207450471",
0x9589eede,"MainField_TBox_Field_Iron_2076156603",
0x83f73010,"MainField_TBox_Field_Iron_2082874460",
0x0a1d418c,"MainField_TBox_Field_Iron_2095607697",
0x40111832,"MainField_TBox_Field_Iron_2099137307",
0xee36850b,"MainField_TBox_Field_Iron_2112563469",
0x8189c7b6,"MainField_TBox_Field_Iron_2116087794",
0xc9f54f2b,"MainField_TBox_Field_Iron_212115638",
0x551fde90,"MainField_TBox_Field_Iron_2132275790",
0xe4d8a802,"MainField_TBox_Field_Iron_2136309676",
0x232bbc0a,"MainField_TBox_Field_Iron_2137637124",
0xc4a6c810,"MainField_TBox_Field_Iron_2139413692",
0x415910ff,"MainField_TBox_Field_Iron_2144669829",
0x5bb06d71,"MainField_TBox_Field_Iron_2145001544",
0x303eb0cf,"MainField_TBox_Field_Iron_214791822",
0xcb630d71,"MainField_TBox_Field_Iron_2154725405",
0x7e9d2d6f,"MainField_TBox_Field_Iron_2159174700",
0x62ff5d62,"MainField_TBox_Field_Iron_2161356979",
0x6dc25699,"MainField_TBox_Field_Iron_2170065742",
0x535299d1,"MainField_TBox_Field_Iron_2171645681",
0x3578cb4e,"MainField_TBox_Field_Iron_2188847100",
0x100ff0c1,"MainField_TBox_Field_Iron_2191300509",
0x12e4eb61,"MainField_TBox_Field_Iron_2193651932",
0x395f5057,"MainField_TBox_Field_Iron_2199424046",
0x14e952b2,"MainField_TBox_Field_Iron_2214788523",
0xa3e04a66,"MainField_TBox_Field_Iron_2216253316",
0x00347e6c,"MainField_TBox_Field_Iron_2218727311",
0xff669348,"MainField_TBox_Field_Iron_2218779285",
0xa79f77eb,"MainField_TBox_Field_Iron_222423612",
0xfeba12da,"MainField_TBox_Field_Iron_2233314149",
0x27997a1c,"MainField_TBox_Field_Iron_2238363106",
0x81877db1,"MainField_TBox_Field_Iron_2242850626",
0xfc47c087,"MainField_TBox_Field_Iron_2243579066",
0x4854b7fa,"MainField_TBox_Field_Iron_2249057719",
0xdd32fc6b,"MainField_TBox_Field_Iron_225004724",
0xfe0988db,"MainField_TBox_Field_Iron_2259596828",
0x52202e39,"MainField_TBox_Field_Iron_2270204843",
0x88ec8372,"MainField_TBox_Field_Iron_2283393607",
0xa3a2a8d7,"MainField_TBox_Field_Iron_2291744218",
0xf3a11960,"MainField_TBox_Field_Iron_2292903165",
0xa52b061f,"MainField_TBox_Field_Iron_2297378253",
0xb91f3e21,"MainField_TBox_Field_Iron_2305460517",
0xe794aea3,"MainField_TBox_Field_Iron_231707920",
0x8b796925,"MainField_TBox_Field_Iron_2317217745",
0x1c2ec22f,"MainField_TBox_Field_Iron_2325715544",
0x100f9cf1,"MainField_TBox_Field_Iron_2326456728",
0xc53fb825,"MainField_TBox_Field_Iron_2338066423",
0xe023b9ec,"MainField_TBox_Field_Iron_2340985924",
0xeb34808f,"MainField_TBox_Field_Iron_234388252",
0x1ad25e98,"MainField_TBox_Field_Iron_2345968098",
0x5345dc4e,"MainField_TBox_Field_Iron_235380293",
0x694dd522,"MainField_TBox_Field_Iron_2361495073",
0x7a6f09f9,"MainField_TBox_Field_Iron_2370316807",
0x3f1949dc,"MainField_TBox_Field_Iron_2371181995",
0xaf02c0b3,"MainField_TBox_Field_Iron_2371431414",
0x4ca5e4e4,"MainField_TBox_Field_Iron_2371560225",
0x1ef43a3a,"MainField_TBox_Field_Iron_2372027614",
0xe72879aa,"MainField_TBox_Field_Iron_2382090190",
0x3b7c6270,"MainField_TBox_Field_Iron_2382892770",
0x8336da67,"MainField_TBox_Field_Iron_238676055",
0x83e9a3bc,"MainField_TBox_Field_Iron_2393563249",
0xdccdb969,"MainField_TBox_Field_Iron_2404533720",
0x07bf078a,"MainField_TBox_Field_Iron_2415552866",
0x1ae1dd24,"MainField_TBox_Field_Iron_2415874070",
0xf0468700,"MainField_TBox_Field_Iron_2417429350",
0x9bc6bb46,"MainField_TBox_Field_Iron_2428077626",
0x7001586f,"MainField_TBox_Field_Iron_2442090605",
0x52f030f4,"MainField_TBox_Field_Iron_2470466058",
0x79a8b125,"MainField_TBox_Field_Iron_2471551506",
0x8c605a13,"MainField_TBox_Field_Iron_2476528241",
0xc2e2cc00,"MainField_TBox_Field_Iron_2488863597",
0x1919e609,"MainField_TBox_Field_Iron_249226348",
0x93ac141f,"MainField_TBox_Field_Iron_2495953728",
0xbb43d09f,"MainField_TBox_Field_Iron_2497785740",
0xc198ee4a,"MainField_TBox_Field_Iron_2498863117",
0xdc234d63,"MainField_TBox_Field_Iron_2499069562",
0xd55b43ac,"MainField_TBox_Field_Iron_2503468879",
0x609ccb43,"MainField_TBox_Field_Iron_2505318621",
0x7394fd0d,"MainField_TBox_Field_Iron_2509664128",
0xb0744c03,"MainField_TBox_Field_Iron_2515667806",
0xa9f43343,"MainField_TBox_Field_Iron_2526420444",
0xb240025e,"MainField_TBox_Field_Iron_2534108832",
0xbf4f7ed5,"MainField_TBox_Field_Iron_2542487652",
0x33f2437c,"MainField_TBox_Field_Iron_2547439322",
0xe064a3a9,"MainField_TBox_Field_Iron_2547745601",
0xd65538cc,"MainField_TBox_Field_Iron_2548659108",
0xb2d2f235,"MainField_TBox_Field_Iron_2553047599",
0xba83cc7d,"MainField_TBox_Field_Iron_2560961344",
0x46d647f7,"MainField_TBox_Field_Iron_2569806473",
0x306b6efd,"MainField_TBox_Field_Iron_2571263621",
0x9e67b6fc,"MainField_TBox_Field_Iron_2597637829",
0x713aa08f,"MainField_TBox_Field_Iron_2611443383",
0x0c8b7919,"MainField_TBox_Field_Iron_2620285992",
0x1d0ad45a,"MainField_TBox_Field_Iron_2632235451",
0xf3e0a766,"MainField_TBox_Field_Iron_2632953765",
0xd4d51266,"MainField_TBox_Field_Iron_2634513557",
0x0d4c409d,"MainField_TBox_Field_Iron_2638441283",
0xd2f62685,"MainField_TBox_Field_Iron_26391794",
0xa8443d33,"MainField_TBox_Field_Iron_2672673001",
0x16c0d817,"MainField_TBox_Field_Iron_2675135236",
0xdd76b0d6,"MainField_TBox_Field_Iron_2676100029",
0xb42274b1,"MainField_TBox_Field_Iron_2683795021",
0x9e7a4c3e,"MainField_TBox_Field_Iron_2699962286",
0x94299f97,"MainField_TBox_Field_Iron_2702126533",
0x9272efea,"MainField_TBox_Field_Iron_2712908755",
0x4526145d,"MainField_TBox_Field_Iron_2714343092",
0x0a08bbdc,"MainField_TBox_Field_Iron_2717355335",
0xb9091cf4,"MainField_TBox_Field_Iron_2723142858",
0xcaa19974,"MainField_TBox_Field_Iron_2729324793",
0x096cc084,"MainField_TBox_Field_Iron_2741160429",
0xdad6d15d,"MainField_TBox_Field_Iron_2750500357",
0x6aa88711,"MainField_TBox_Field_Iron_275172052",
0x60bd9df1,"MainField_TBox_Field_Iron_2761495313",
0xab1bd451,"MainField_TBox_Field_Iron_2762828042",
0x1ced5a35,"MainField_TBox_Field_Iron_2785793685",
0xfb4ad923,"MainField_TBox_Field_Iron_278838322",
0x4148b92b,"MainField_TBox_Field_Iron_2788784565",
0x8631fb75,"MainField_TBox_Field_Iron_2811276367",
0x30763d71,"MainField_TBox_Field_Iron_2812509781",
0x55390500,"MainField_TBox_Field_Iron_2816538790",
0xe9b2772d,"MainField_TBox_Field_Iron_2818249630",
0x2fa52200,"MainField_TBox_Field_Iron_2822851856",
0xa71f6907,"MainField_TBox_Field_Iron_2824001260",
0x35db1015,"MainField_TBox_Field_Iron_283478570",
0x8108eb6f,"MainField_TBox_Field_Iron_2845885970",
0xc33ae39c,"MainField_TBox_Field_Iron_2853960406",
0x41aee9b9,"MainField_TBox_Field_Iron_2854121351",
0x0023285f,"MainField_TBox_Field_Iron_286295037",
0x9d3a93a9,"MainField_TBox_Field_Iron_2865478414",
0xd3073402,"MainField_TBox_Field_Iron_287994949",
0x906b0966,"MainField_TBox_Field_Iron_2882463730",
0xf9a41f97,"MainField_TBox_Field_Iron_2882853809",
0xcd23b24f,"MainField_TBox_Field_Iron_2884643165",
0x46774cec,"MainField_TBox_Field_Iron_2887957726",
0x95a0ca1f,"MainField_TBox_Field_Iron_2889988447",
0x21ced620,"MainField_TBox_Field_Iron_289940332",
0x502a44b3,"MainField_TBox_Field_Iron_2901845928",
0x1a057214,"MainField_TBox_Field_Iron_2918461827",
0x4e9eb909,"MainField_TBox_Field_Iron_2921610527",
0xd4bf167a,"MainField_TBox_Field_Iron_2927919877",
0x874206ff,"MainField_TBox_Field_Iron_2928804409",
0x2f01ad64,"MainField_TBox_Field_Iron_2932146880",
0xfc6a1574,"MainField_TBox_Field_Iron_293266321",
0xe135f307,"MainField_TBox_Field_Iron_2937866400",
0xda4b0852,"MainField_TBox_Field_Iron_2939615112",
0xbcd2fe25,"MainField_TBox_Field_Iron_2944058130",
0x94b7ddcf,"MainField_TBox_Field_Iron_2952028510",
0xf3bd4418,"MainField_TBox_Field_Iron_2958086101",
0x033fd618,"MainField_TBox_Field_Iron_2959340874",
0x822648d8,"MainField_TBox_Field_Iron_2961210650",
0x98ad371b,"MainField_TBox_Field_Iron_2961580850",
0xf3b001fe,"MainField_TBox_Field_Iron_2964691193",
0x90467f10,"MainField_TBox_Field_Iron_297334505",
0x2965c3e0,"MainField_TBox_Field_Iron_2985599044",
0xf372d8d8,"MainField_TBox_Field_Iron_2995203455",
0x4cff1860,"MainField_TBox_Field_Iron_2995782211",
0xb470f706,"MainField_TBox_Field_Iron_29985339",
0xdd0f1860,"MainField_TBox_Field_Iron_2999223003",
0x5aa10123,"MainField_TBox_Field_Iron_300311671",
0xdacb06d2,"MainField_TBox_Field_Iron_3007332426",
0x6131d94d,"MainField_TBox_Field_Iron_3007573547",
0xc132eac0,"MainField_TBox_Field_Iron_3016892046",
0xe220ce96,"MainField_TBox_Field_Iron_3018817247",
0x14afc2f9,"MainField_TBox_Field_Iron_3031569224",
0x50b9ab72,"MainField_TBox_Field_Iron_303330436",
0x67027f02,"MainField_TBox_Field_Iron_3034979131",
0xcefe1a58,"MainField_TBox_Field_Iron_303823410",
0x1efb988c,"MainField_TBox_Field_Iron_303953733",
0xeed4a561,"MainField_TBox_Field_Iron_3041662652",
0x5e6054d1,"MainField_TBox_Field_Iron_30547543",
0x402c8943,"MainField_TBox_Field_Iron_305852247",
0xc3f75cf0,"MainField_TBox_Field_Iron_306014001",
0x445683ef,"MainField_TBox_Field_Iron_3069287883",
0x46c7f3bf,"MainField_TBox_Field_Iron_3083660260",
0x39e3e54d,"MainField_TBox_Field_Iron_3090211221",
0xc21b2471,"MainField_TBox_Field_Iron_3092990080",
0xc2a31e7c,"MainField_TBox_Field_Iron_3093088881",
0x78336921,"MainField_TBox_Field_Iron_3095777895",
0x1ccecc52,"MainField_TBox_Field_Iron_3098906856",
0x95906d6b,"MainField_TBox_Field_Iron_3107772853",
0x89e33e1e,"MainField_TBox_Field_Iron_3112313132",
0x4c4de36f,"MainField_TBox_Field_Iron_312182765",
0xbdf454d3,"MainField_TBox_Field_Iron_3139741329",
0xd90642a7,"MainField_TBox_Field_Iron_3144255391",
0xbe22a505,"MainField_TBox_Field_Iron_3155428228",
0x703f8288,"MainField_TBox_Field_Iron_3157885976",
0x297611ed,"MainField_TBox_Field_Iron_3161921041",
0x1d8c0df0,"MainField_TBox_Field_Iron_3175612285",
0x222fcb24,"MainField_TBox_Field_Iron_317834611",
0xf450604a,"MainField_TBox_Field_Iron_3182037294",
0xc62aaa15,"MainField_TBox_Field_Iron_318716168",
0x7925a01c,"MainField_TBox_Field_Iron_320668688",
0x1e3404ca,"MainField_TBox_Field_Iron_3208192332",
0x1ef1ae12,"MainField_TBox_Field_Iron_3217144738",
0x22e6f610,"MainField_TBox_Field_Iron_3218351127",
0x3ae93687,"MainField_TBox_Field_Iron_3226883364",
0x2be833f0,"MainField_TBox_Field_Iron_3233823356",
0x173f7af8,"MainField_TBox_Field_Iron_3235700587",
0x95bf5025,"MainField_TBox_Field_Iron_3244503465",
0x096307f1,"MainField_TBox_Field_Iron_3246815642",
0x58418950,"MainField_TBox_Field_Iron_3253495359",
0x33106804,"MainField_TBox_Field_Iron_3268188250",
0xce2bc168,"MainField_TBox_Field_Iron_3273140529",
0xf88a20ee,"MainField_TBox_Field_Iron_3275774500",
0xf399c350,"MainField_TBox_Field_Iron_3277182306",
0xfbacf65e,"MainField_TBox_Field_Iron_3286432837",
0x803b816c,"MainField_TBox_Field_Iron_3286959761",
0x11056735,"MainField_TBox_Field_Iron_3297443220",
0x3c76f720,"MainField_TBox_Field_Iron_3309195167",
0x169afb51,"MainField_TBox_Field_Iron_3318352307",
0x43c39d77,"MainField_TBox_Field_Iron_3329533659",
0xff4c62f4,"MainField_TBox_Field_Iron_3335282544",
0x42f7e690,"MainField_TBox_Field_Iron_3335660483",
0x62f6f642,"MainField_TBox_Field_Iron_3340848427",
0xc47a9ea7,"MainField_TBox_Field_Iron_334717340",
0x2ec9f4f6,"MainField_TBox_Field_Iron_3349353712",
0x936f3af3,"MainField_TBox_Field_Iron_3350036882",
0x7740e44f,"MainField_TBox_Field_Iron_3350546334",
0xdf87accc,"MainField_TBox_Field_Iron_3375186484",
0x5fde9989,"MainField_TBox_Field_Iron_3381003323",
0x2d0b1fe5,"MainField_TBox_Field_Iron_3382842697",
0x45603102,"MainField_TBox_Field_Iron_3391370987",
0x7b187733,"MainField_TBox_Field_Iron_3396852514",
0x358a3b24,"MainField_TBox_Field_Iron_3399665325",
0xc5a564c8,"MainField_TBox_Field_Iron_3403284843",
0x1263ced6,"MainField_TBox_Field_Iron_3405088721",
0x1d6a5010,"MainField_TBox_Field_Iron_3414893782",
0x05a5aaa2,"MainField_TBox_Field_Iron_3421206150",
0xf8f63fc3,"MainField_TBox_Field_Iron_3425207363",
0x1102ae31,"MainField_TBox_Field_Iron_3440887061",
0x493d7d10,"MainField_TBox_Field_Iron_3460032803",
0xc40b0147,"MainField_TBox_Field_Iron_3463226373",
0x8f7fb249,"MainField_TBox_Field_Iron_3466092750",
0xc9624d41,"MainField_TBox_Field_Iron_3485175216",
0x9c1d1d65,"MainField_TBox_Field_Iron_3486341949",
0xb42a4ada,"MainField_TBox_Field_Iron_3491392994",
0xea6245f2,"MainField_TBox_Field_Iron_3494546458",
0xf80ad180,"MainField_TBox_Field_Iron_3496035908",
0x96094559,"MainField_TBox_Field_Iron_3496408870",
0x7ac99149,"MainField_TBox_Field_Iron_3497628028",
0xd9892195,"MainField_TBox_Field_Iron_3499574832",
0xdfc147dc,"MainField_TBox_Field_Iron_3535823544",
0xd5fb12e8,"MainField_TBox_Field_Iron_3543458045",
0xc978d3fa,"MainField_TBox_Field_Iron_3548323950",
0x0eda8b8e,"MainField_TBox_Field_Iron_3549780177",
0xaa074ef0,"MainField_TBox_Field_Iron_3557695569",
0x602b2c39,"MainField_TBox_Field_Iron_3558527197",
0xf6236e79,"MainField_TBox_Field_Iron_3559532458",
0x3acdf771,"MainField_TBox_Field_Iron_3566746737",
0x66d5ae7e,"MainField_TBox_Field_Iron_3585427801",
0x90206c35,"MainField_TBox_Field_Iron_3599777380",
0xd71aeb3d,"MainField_TBox_Field_Iron_36009935",
0x49cd5286,"MainField_TBox_Field_Iron_3602332415",
0x3826692b,"MainField_TBox_Field_Iron_3602720041",
0x1f3b1da4,"MainField_TBox_Field_Iron_3607058155",
0x136628b0,"MainField_TBox_Field_Iron_3614734282",
0xd641a8b3,"MainField_TBox_Field_Iron_3623496767",
0xa9156701,"MainField_TBox_Field_Iron_3626763773",
0x2240fd49,"MainField_TBox_Field_Iron_3634645256",
0xf9e41a6e,"MainField_TBox_Field_Iron_364857245",
0x5a8041d2,"MainField_TBox_Field_Iron_3653212286",
0x9b15439b,"MainField_TBox_Field_Iron_3659523672",
0xf723fe38,"MainField_TBox_Field_Iron_3660212975",
0x98a73f57,"MainField_TBox_Field_Iron_3681727284",
0x36c0de1a,"MainField_TBox_Field_Iron_3681824517",
0x0984d144,"MainField_TBox_Field_Iron_3685622260",
0x6fa67637,"MainField_TBox_Field_Iron_3686673701",
0x5d44bf43,"MainField_TBox_Field_Iron_3693132813",
0x3746da68,"MainField_TBox_Field_Iron_370374453",
0x319b81da,"MainField_TBox_Field_Iron_370794488",
0xa0b3bf68,"MainField_TBox_Field_Iron_372261438",
0xcc465e96,"MainField_TBox_Field_Iron_3729177928",
0x3938a3e2,"MainField_TBox_Field_Iron_3731117862",
0x718f6e72,"MainField_TBox_Field_Iron_3742409497",
0x046bb71a,"MainField_TBox_Field_Iron_3746546895",
0x88c4d928,"MainField_TBox_Field_Iron_3753184174",
0x229793f2,"MainField_TBox_Field_Iron_3759584426",
0x263cc783,"MainField_TBox_Field_Iron_3763867057",
0xe40c2b54,"MainField_TBox_Field_Iron_377195081",
0x1f6f738b,"MainField_TBox_Field_Iron_3772552958",
0x704b7ebd,"MainField_TBox_Field_Iron_3777493416",
0xd1520aac,"MainField_TBox_Field_Iron_3784387914",
0x82305c2e,"MainField_TBox_Field_Iron_3796176819",
0xb2d8a715,"MainField_TBox_Field_Iron_3799975868",
0x57ce2624,"MainField_TBox_Field_Iron_3800808615",
0xfc6ed37f,"MainField_TBox_Field_Iron_3807088626",
0x29279539,"MainField_TBox_Field_Iron_3830680283",
0x4348908b,"MainField_TBox_Field_Iron_383258217",
0x9e1d4935,"MainField_TBox_Field_Iron_3846967635",
0x69c8b42a,"MainField_TBox_Field_Iron_3849079795",
0x2570bd1a,"MainField_TBox_Field_Iron_3851576114",
0x098c2f7a,"MainField_TBox_Field_Iron_3857385741",
0xe24be8ae,"MainField_TBox_Field_Iron_3857618283",
0x61c3f035,"MainField_TBox_Field_Iron_3858998906",
0xe2c811e6,"MainField_TBox_Field_Iron_3859239268",
0x202f15cc,"MainField_TBox_Field_Iron_3862337175",
0x192b37ef,"MainField_TBox_Field_Iron_3864800790",
0x24048e42,"MainField_TBox_Field_Iron_3883700080",
0xe88782c9,"MainField_TBox_Field_Iron_388443518",
0xcb40d8f0,"MainField_TBox_Field_Iron_3890426604",
0x56f023ee,"MainField_TBox_Field_Iron_3897779397",
0x3b6b4952,"MainField_TBox_Field_Iron_3900592888",
0xfaba1f2d,"MainField_TBox_Field_Iron_3902262908",
0x60c7f51d,"MainField_TBox_Field_Iron_3906843155",
0xd2b2d351,"MainField_TBox_Field_Iron_3907501147",
0x281bb44b,"MainField_TBox_Field_Iron_3921219436",
0xc05585e5,"MainField_TBox_Field_Iron_3931060219",
0x9422f540,"MainField_TBox_Field_Iron_3937024334",
0x443efc58,"MainField_TBox_Field_Iron_3937171367",
0xa135d3c3,"MainField_TBox_Field_Iron_394040126",
0xf7a6e24f,"MainField_TBox_Field_Iron_3943112210",
0x5d49e7a7,"MainField_TBox_Field_Iron_3943516391",
0x67bd1f31,"MainField_TBox_Field_Iron_3945356322",
0x2a951b42,"MainField_TBox_Field_Iron_3951022814",
0xe0d9fa92,"MainField_TBox_Field_Iron_3953433931",
0x8b53a232,"MainField_TBox_Field_Iron_3957956067",
0x6a140369,"MainField_TBox_Field_Iron_3966140947",
0x6dffbde4,"MainField_TBox_Field_Iron_3971228179",
0x6903133e,"MainField_TBox_Field_Iron_3971790898",
0xdf532e07,"MainField_TBox_Field_Iron_3974488821",
0xbe8bd641,"MainField_TBox_Field_Iron_3978900772",
0x79ad188f,"MainField_TBox_Field_Iron_3982980334",
0x63d06f9f,"MainField_TBox_Field_Iron_3995793230",
0x5e6ce9c1,"MainField_TBox_Field_Iron_4011871507",
0x025d3314,"MainField_TBox_Field_Iron_4012685036",
0x14af5b0e,"MainField_TBox_Field_Iron_4022375917",
0xf50a5acd,"MainField_TBox_Field_Iron_4025212558",
0x0358eb12,"MainField_TBox_Field_Iron_4025474747",
0x92a787e0,"MainField_TBox_Field_Iron_4035181578",
0xabc7ca64,"MainField_TBox_Field_Iron_4042994739",
0x131763c5,"MainField_TBox_Field_Iron_4043807207",
0x7e171176,"MainField_TBox_Field_Iron_4064720740",
0x71a2244a,"MainField_TBox_Field_Iron_4070986459",
0x79131bc6,"MainField_TBox_Field_Iron_4077772247",
0x8e6ebd26,"MainField_TBox_Field_Iron_4083934584",
0xb92c28b8,"MainField_TBox_Field_Iron_4092054275",
0xdde9804a,"MainField_TBox_Field_Iron_4093869572",
0x7f1034e2,"MainField_TBox_Field_Iron_4100148598",
0x1562445f,"MainField_TBox_Field_Iron_4110002986",
0x0f544f33,"MainField_TBox_Field_Iron_4111773942",
0x4c722b2e,"MainField_TBox_Field_Iron_4124729366",
0x3d217110,"MainField_TBox_Field_Iron_4148707564",
0xa1d4dcee,"MainField_TBox_Field_Iron_415503958",
0xa307e7b0,"MainField_TBox_Field_Iron_4197427235",
0x26f7f061,"MainField_TBox_Field_Iron_419833188",
0x752bfdc5,"MainField_TBox_Field_Iron_4199442744",
0x4032f7af,"MainField_TBox_Field_Iron_420581026",
0x3e5eca03,"MainField_TBox_Field_Iron_4210769254",
0x23347c05,"MainField_TBox_Field_Iron_4212474385",
0x93364fc0,"MainField_TBox_Field_Iron_4213569891",
0x71863b98,"MainField_TBox_Field_Iron_4217494700",
0xb7c38411,"MainField_TBox_Field_Iron_4222398229",
0x466ddf3a,"MainField_TBox_Field_Iron_4230895209",
0x276744f8,"MainField_TBox_Field_Iron_4233931138",
0x79011226,"MainField_TBox_Field_Iron_4234551321",
0x1190a9ed,"MainField_TBox_Field_Iron_4239885713",
0x16408c26,"MainField_TBox_Field_Iron_4243347982",
0x09d0e207,"MainField_TBox_Field_Iron_4280214225",
0xa1aa6f25,"MainField_TBox_Field_Iron_4287424785",
0xcfa4e22a,"MainField_TBox_Field_Iron_4291705628",
0x959fb9d7,"MainField_TBox_Field_Iron_4292386206",
0x61c04564,"MainField_TBox_Field_Iron_4293395803",
0x26a57bd3,"MainField_TBox_Field_Iron_4293404901",
0x4905c6b1,"MainField_TBox_Field_Iron_4293934302",
0xcf3cc070,"MainField_TBox_Field_Iron_44135018",
0x266bc5ff,"MainField_TBox_Field_Iron_441938591",
0xc9f8b05a,"MainField_TBox_Field_Iron_442485578",
0x08cd8332,"MainField_TBox_Field_Iron_461693195",
0xf8cfd383,"MainField_TBox_Field_Iron_464024192",
0x260e44d1,"MainField_TBox_Field_Iron_467957087",
0xe82a8e59,"MainField_TBox_Field_Iron_472254805",
0x02ed6df2,"MainField_TBox_Field_Iron_477335528",
0x59a794b6,"MainField_TBox_Field_Iron_48319296",
0xc4107730,"MainField_TBox_Field_Iron_484706126",
0xa9631787,"MainField_TBox_Field_Iron_485240907",
0x9c0399b9,"MainField_TBox_Field_Iron_492030687",
0x8590a149,"MainField_TBox_Field_Iron_498854459",
0x98b5f069,"MainField_TBox_Field_Iron_500137107",
0xcfee1ace,"MainField_TBox_Field_Iron_507951821",
0x1104ebd3,"MainField_TBox_Field_Iron_511163824",
0x3db6ab75,"MainField_TBox_Field_Iron_521265924",
0x3d85e286,"MainField_TBox_Field_Iron_523166592",
0xbe5c1206,"MainField_TBox_Field_Iron_528853506",
0x9d2c68ea,"MainField_TBox_Field_Iron_530571635",
0xbc3e10a0,"MainField_TBox_Field_Iron_533479225",
0x5087ddc5,"MainField_TBox_Field_Iron_534660948",
0xbe700f61,"MainField_TBox_Field_Iron_536499972",
0x2fc06c5f,"MainField_TBox_Field_Iron_537942608",
0x9f301902,"MainField_TBox_Field_Iron_546454289",
0x46e34de3,"MainField_TBox_Field_Iron_550777769",
0x10d36283,"MainField_TBox_Field_Iron_554021750",
0xdd76d421,"MainField_TBox_Field_Iron_598465132",
0x27b2fd53,"MainField_TBox_Field_Iron_599487442",
0x29957980,"MainField_TBox_Field_Iron_608422654",
0x3c11e7f9,"MainField_TBox_Field_Iron_614466614",
0x7c449ade,"MainField_TBox_Field_Iron_623513864",
0x81a7c7f1,"MainField_TBox_Field_Iron_628080506",
0xb74619ef,"MainField_TBox_Field_Iron_632077327",
0xf904e718,"MainField_TBox_Field_Iron_64882599",
0x03a251c6,"MainField_TBox_Field_Iron_655045212",
0x27c4019a,"MainField_TBox_Field_Iron_655889537",
0xabcf82ee,"MainField_TBox_Field_Iron_657698018",
0xead0b679,"MainField_TBox_Field_Iron_660146514",
0x91a17874,"MainField_TBox_Field_Iron_662706173",
0x8f8c462b,"MainField_TBox_Field_Iron_664766686",
0xadc80b77,"MainField_TBox_Field_Iron_677118654",
0x094f4c43,"MainField_TBox_Field_Iron_68760609",
0x5da2bb5f,"MainField_TBox_Field_Iron_69905208",
0xc44a245a,"MainField_TBox_Field_Iron_70676703",
0x460f8dbf,"MainField_TBox_Field_Iron_708513020",
0x5a2affe1,"MainField_TBox_Field_Iron_716129908",
0x56f22c2a,"MainField_TBox_Field_Iron_716855822",
0x06f9ad57,"MainField_TBox_Field_Iron_721376267",
0x385ce440,"MainField_TBox_Field_Iron_722284165",
0x117fa32c,"MainField_TBox_Field_Iron_728790229",
0xa958e3c6,"MainField_TBox_Field_Iron_737157778",
0xef5fce1f,"MainField_TBox_Field_Iron_743549766",
0x3a8601ca,"MainField_TBox_Field_Iron_744026530",
0x5c4eb40f,"MainField_TBox_Field_Iron_748227650",
0xc41f0926,"MainField_TBox_Field_Iron_757198158",
0x69941072,"MainField_TBox_Field_Iron_766927520",
0xa13f517b,"MainField_TBox_Field_Iron_767501701",
0x58df9015,"MainField_TBox_Field_Iron_783210943",
0xafc2dd80,"MainField_TBox_Field_Iron_789522239",
0x03e9b3ac,"MainField_TBox_Field_Iron_797448374",
0x5134dc72,"MainField_TBox_Field_Iron_8312663",
0xb644facb,"MainField_TBox_Field_Iron_836749965",
0x06c6649b,"MainField_TBox_Field_Iron_843744120",
0x16378d0b,"MainField_TBox_Field_Iron_85065275",
0x41ba74cf,"MainField_TBox_Field_Iron_852165051",
0xaf0c6b99,"MainField_TBox_Field_Iron_87655617",
0xcbaf4159,"MainField_TBox_Field_Iron_890467781",
0x4e499a5e,"MainField_TBox_Field_Iron_890774752",
0xc74ddb9f,"MainField_TBox_Field_Iron_893358713",
0xb56dbcf1,"MainField_TBox_Field_Iron_910468887",
0x0aa9c2f5,"MainField_TBox_Field_Iron_92601747",
0xc43c6130,"MainField_TBox_Field_Iron_926056347",
0xd95ad3b3,"MainField_TBox_Field_Iron_934506065",
0x64945195,"MainField_TBox_Field_Iron_93487111",
0x056feb7e,"MainField_TBox_Field_Iron_962481589",
0x4deb78cd,"MainField_TBox_Field_Iron_971003604",
0x39107f66,"MainField_TBox_Field_Iron_982603570",
0x2913d6cc,"MainField_TBox_Field_Iron_983742254",
0x09684a34,"MainField_TBox_Field_Iron_984289322",
0x1289d989,"MainField_TBox_Field_Iron_990270704",
0x13f294e5,"MainField_TBox_Field_Iron_996157908",
0x5ff70f32,"MainField_TBox_Field_Iron_997729111",
0xe93c69eb,"MainField_TBox_Field_Iron_GodTree_111780462",
0x44485641,"MainField_TBox_Field_Iron_GodTree_2668399187",
0x1db1f02a,"MainField_TBox_Field_Iron_NoReaction_1101107709",
0x42e0bb6c,"MainField_TBox_Field_Iron_NoReaction_1140709720",
0x5db0aa3d,"MainField_TBox_Field_Iron_NoReaction_1489808006",
0xc99f9283,"MainField_TBox_Field_Iron_NoReaction_2239849716",
0xd03138cd,"MainField_TBox_Field_Iron_NoReaction_2463472452",
0x2eea19cb,"MainField_TBox_Field_Iron_NoReaction_2585267194",
0x6c2a5854,"MainField_TBox_Field_Iron_NoReaction_2625230511",
0xb9051795,"MainField_TBox_Field_Iron_NoReaction_2782068585",
0x45a690f4,"MainField_TBox_Field_Iron_NoReaction_2832405486",
0xa46aabbc,"MainField_TBox_Field_Iron_NoReaction_3014554655",
0x76424d74,"MainField_TBox_Field_Iron_NoReaction_3214244294",
0x1445f4bf,"MainField_TBox_Field_Iron_NoReaction_3224206340",
0xd7e15ed0,"MainField_TBox_Field_Iron_NoReaction_3669008966",
0x074ee680,"MainField_TBox_Field_Iron_NoReaction_3910200627",
0xd8a74a97,"MainField_TBox_Field_Iron_NoReaction_455907821",
0x3aa08a88,"MainField_TBox_Field_Iron_NoReaction_525020201",

/* 3rd March DLC treasures? */
0x66acedc3,"MainField_TBox_Field_Iron_NoReaction_Aoc_4093217196",
0x339be2bc,"MainField_TBox_Field_Iron_NoReaction_Aoc_473644037",
0x211fd058,"MainField_TBox_Field_Iron_NoReaction_Aoc_759164510",

0x4b757c24,"MainField_TBox_Field_Stone_1010046799",
0x3759aca0,"MainField_TBox_Field_Stone_1018768719",
0xb0e325b2,"MainField_TBox_Field_Stone_1037519518",
0x08e4a70c,"MainField_TBox_Field_Stone_1044386518",
0xaee7c294,"MainField_TBox_Field_Stone_1057130287",
0x7ca65fbf,"MainField_TBox_Field_Stone_1093245165",
0xad56d67f,"MainField_TBox_Field_Stone_1094385771",
0x20059cd3,"MainField_TBox_Field_Stone_1099237980",
0x582e8300,"MainField_TBox_Field_Stone_1100553496",
0x7e0eb5d1,"MainField_TBox_Field_Stone_1108287970",
0x985be8e9,"MainField_TBox_Field_Stone_1122331722",
0xb546413a,"MainField_TBox_Field_Stone_1130610796",
0x40d08d30,"MainField_TBox_Field_Stone_1175755810",
0x8d5f9d97,"MainField_TBox_Field_Stone_1195250099",
0x8de80165,"MainField_TBox_Field_Stone_1216984011",
0x0fa95e7d,"MainField_TBox_Field_Stone_1225298978",
0x051ec3a1,"MainField_TBox_Field_Stone_1234325019",
0x5d86f5d1,"MainField_TBox_Field_Stone_1237367464",
0xc090a562,"MainField_TBox_Field_Stone_1253947834",
0xe2b96617,"MainField_TBox_Field_Stone_1254156027",
0xcdd171bc,"MainField_TBox_Field_Stone_1261285928",
0x997f92d7,"MainField_TBox_Field_Stone_1313140003",
0x7eac67bd,"MainField_TBox_Field_Stone_1335244483",
0x4daf0d04,"MainField_TBox_Field_Stone_1336330295",
0x9951a570,"MainField_TBox_Field_Stone_1336487113",
0x4ff5bf5b,"MainField_TBox_Field_Stone_1337874741",
0xc8f92c04,"MainField_TBox_Field_Stone_13508045",
0x0241995a,"MainField_TBox_Field_Stone_1351790075",
0x244efc8b,"MainField_TBox_Field_Stone_1378726307",
0x6983482a,"MainField_TBox_Field_Stone_1397214346",
0x2158fa50,"MainField_TBox_Field_Stone_1397757005",
0x8ee1260b,"MainField_TBox_Field_Stone_139851715",
0xda966601,"MainField_TBox_Field_Stone_1406995855",
0xaf3d6d2e,"MainField_TBox_Field_Stone_1407137022",
0xe068fa02,"MainField_TBox_Field_Stone_141775228",
0xa0fc43e9,"MainField_TBox_Field_Stone_1420026614",
0xd2be3e0f,"MainField_TBox_Field_Stone_1438899360",
0x485e3414,"MainField_TBox_Field_Stone_1441938348",
0x3b8573e7,"MainField_TBox_Field_Stone_1465348166",
0x8efa25cc,"MainField_TBox_Field_Stone_1467932802",
0xeba95015,"MainField_TBox_Field_Stone_1468292364",
0x24059320,"MainField_TBox_Field_Stone_1488683803",
0x4703d691,"MainField_TBox_Field_Stone_1491795805",
0x6477aa1e,"MainField_TBox_Field_Stone_1509788199",
0x0eb15f24,"MainField_TBox_Field_Stone_154457414",
0xffa773c5,"MainField_TBox_Field_Stone_1549979885",
0x0dca666f,"MainField_TBox_Field_Stone_1558154797",
0x97eaaea8,"MainField_TBox_Field_Stone_1567807835",
0x6c528b44,"MainField_TBox_Field_Stone_1574041919",
0xb2a35c58,"MainField_TBox_Field_Stone_1580711597",
0xc3e31fd0,"MainField_TBox_Field_Stone_1582552700",
0x4010c3cc,"MainField_TBox_Field_Stone_1596209266",
0xd159fae7,"MainField_TBox_Field_Stone_1610277516",
0x06f3eccf,"MainField_TBox_Field_Stone_1659104910",
0x36a1ce1c,"MainField_TBox_Field_Stone_1669353966",
0xf7869d3d,"MainField_TBox_Field_Stone_1695514473",
0x95134c4c,"MainField_TBox_Field_Stone_1704474960",
0x46312659,"MainField_TBox_Field_Stone_1732323234",
0x047dac78,"MainField_TBox_Field_Stone_1738703230",
0x9d7e5a1f,"MainField_TBox_Field_Stone_1748775430",
0x6b3c4139,"MainField_TBox_Field_Stone_1761515063",
0xa2051b71,"MainField_TBox_Field_Stone_1775504681",
0x94b4cf19,"MainField_TBox_Field_Stone_1780912248",
0xcc3129bb,"MainField_TBox_Field_Stone_1790025404",
0xb3cf1a51,"MainField_TBox_Field_Stone_1794295881",
0x72a80de8,"MainField_TBox_Field_Stone_1795422602",
0xd17a15b1,"MainField_TBox_Field_Stone_1805177461",
0x394a7993,"MainField_TBox_Field_Stone_1807902579",
0x94cf9bed,"MainField_TBox_Field_Stone_181265501",
0xe9219ab1,"MainField_TBox_Field_Stone_181926113",
0x40e88436,"MainField_TBox_Field_Stone_1827679107",
0xc87eacd3,"MainField_TBox_Field_Stone_1855573219",
0x56cad709,"MainField_TBox_Field_Stone_1857299176",
0x36bc1a56,"MainField_TBox_Field_Stone_186643694",
0x089db49b,"MainField_TBox_Field_Stone_1904619820",
0x62c441ed,"MainField_TBox_Field_Stone_1924960938",
0xc0085050,"MainField_TBox_Field_Stone_1936720710",
0xa1775aaf,"MainField_TBox_Field_Stone_1938601138",
0x47c9914b,"MainField_TBox_Field_Stone_195085511",
0xa2b96820,"MainField_TBox_Field_Stone_1965651867",
0x441541fa,"MainField_TBox_Field_Stone_199240618",
0xe09617de,"MainField_TBox_Field_Stone_1999356160",
0xd3920fc0,"MainField_TBox_Field_Stone_2005265140",
0xc5d48e20,"MainField_TBox_Field_Stone_2009260213",
0x888c00d0,"MainField_TBox_Field_Stone_2009900434",
0x3c496926,"MainField_TBox_Field_Stone_2026648602",
0xf448934e,"MainField_TBox_Field_Stone_2029054705",
0xf60c2436,"MainField_TBox_Field_Stone_2045340421",
0x0e342628,"MainField_TBox_Field_Stone_2055474776",
0xafacc20c,"MainField_TBox_Field_Stone_206270111",
0x86365460,"MainField_TBox_Field_Stone_2069325108",
0x8770e963,"MainField_TBox_Field_Stone_2070850062",
0x39ca891b,"MainField_TBox_Field_Stone_2071543779",
0x875810ff,"MainField_TBox_Field_Stone_2074155839",
0x7c092c84,"MainField_TBox_Field_Stone_2088975630",
0x9972f715,"MainField_TBox_Field_Stone_2094766373",
0x278fe02f,"MainField_TBox_Field_Stone_2108624220",
0xc395b06a,"MainField_TBox_Field_Stone_2110462589",
0x523b244a,"MainField_TBox_Field_Stone_2141755056",
0xab8413f3,"MainField_TBox_Field_Stone_2166831310",
0x078eb776,"MainField_TBox_Field_Stone_217594417",
0xf531ddaf,"MainField_TBox_Field_Stone_2178228879",
0x76f00b59,"MainField_TBox_Field_Stone_2188940025",
0xbd2b26bc,"MainField_TBox_Field_Stone_2211088610",
0xf63a97a6,"MainField_TBox_Field_Stone_2211519133",
0x8e5fd4e3,"MainField_TBox_Field_Stone_2217473617",
0x02df74bd,"MainField_TBox_Field_Stone_2239436877",
0x70c65da9,"MainField_TBox_Field_Stone_2290729921",
0xb37f2003,"MainField_TBox_Field_Stone_2292812350",
0x323ab158,"MainField_TBox_Field_Stone_2305267245",
0x398c2851,"MainField_TBox_Field_Stone_231106372",
0x00aa20e3,"MainField_TBox_Field_Stone_2335392690",
0xcf53a3a3,"MainField_TBox_Field_Stone_2342074180",
0x9770a5c2,"MainField_TBox_Field_Stone_2350905657",
0x1f28f271,"MainField_TBox_Field_Stone_2361584466",
0xf90a8684,"MainField_TBox_Field_Stone_2384741472",
0x18a45e10,"MainField_TBox_Field_Stone_2398190126",
0xf31784b9,"MainField_TBox_Field_Stone_2429085515",
0x37b90a29,"MainField_TBox_Field_Stone_2437411730",
0x03670d9b,"MainField_TBox_Field_Stone_2440821153",
0x0b339866,"MainField_TBox_Field_Stone_2443306083",
0xe7560bcc,"MainField_TBox_Field_Stone_246237684",
0x74611a2e,"MainField_TBox_Field_Stone_2493393346",
0x19567558,"MainField_TBox_Field_Stone_2504145081",
0xec164bb6,"MainField_TBox_Field_Stone_2560875918",
0x90aecb84,"MainField_TBox_Field_Stone_2569362692",
0x7ba9ece5,"MainField_TBox_Field_Stone_2607082874",
0xfe1b2c6e,"MainField_TBox_Field_Stone_2611526643",
0x03e32995,"MainField_TBox_Field_Stone_2627457469",
0xde7c034d,"MainField_TBox_Field_Stone_2646712090",
0x83350792,"MainField_TBox_Field_Stone_2651527466",
0x76db97a5,"MainField_TBox_Field_Stone_2664586440",
0x8c2334fa,"MainField_TBox_Field_Stone_2695063175",
0x6b3b86c8,"MainField_TBox_Field_Stone_2715435148",
0xf8ad9955,"MainField_TBox_Field_Stone_2717641349",
0xe37b5aec,"MainField_TBox_Field_Stone_2722149659",
0xb9a1b34d,"MainField_TBox_Field_Stone_2723421906",
0x2faf29db,"MainField_TBox_Field_Stone_2728429337",
0xe0810fc8,"MainField_TBox_Field_Stone_2730755812",
0x5d4a6258,"MainField_TBox_Field_Stone_2731441477",
0x2296dd01,"MainField_TBox_Field_Stone_2744557682",
0xb09df878,"MainField_TBox_Field_Stone_2748645519",
0x30cd4bd7,"MainField_TBox_Field_Stone_2777891377",
0x2e1c8f5d,"MainField_TBox_Field_Stone_2790780411",
0x0733d77b,"MainField_TBox_Field_Stone_2798152012",
0xcc1e252b,"MainField_TBox_Field_Stone_2821595502",
0xf20fc239,"MainField_TBox_Field_Stone_2826512989",
0x0aa802a8,"MainField_TBox_Field_Stone_2856675622",
0xd22e97e3,"MainField_TBox_Field_Stone_2859393028",
0x36f55ac5,"MainField_TBox_Field_Stone_2892402686",
0xb5570e64,"MainField_TBox_Field_Stone_2920699957",
0x7e062529,"MainField_TBox_Field_Stone_292872964",
0xa0787e8a,"MainField_TBox_Field_Stone_293738793",
0x9d7d4187,"MainField_TBox_Field_Stone_2948269230",
0x6e38efff,"MainField_TBox_Field_Stone_2949130574",
0x3e4ed627,"MainField_TBox_Field_Stone_2951121444",
0x246e5d08,"MainField_TBox_Field_Stone_2958931792",
0x5221027b,"MainField_TBox_Field_Stone_2993853416",
0x28585094,"MainField_TBox_Field_Stone_3004097437",
0x9e92171c,"MainField_TBox_Field_Stone_3034556572",
0x42e393a3,"MainField_TBox_Field_Stone_3034784848",
0xcfb0e4f4,"MainField_TBox_Field_Stone_3042417818",
0xa3023f6c,"MainField_TBox_Field_Stone_304706761",
0xe35865a7,"MainField_TBox_Field_Stone_3047587072",
0x76ccda64,"MainField_TBox_Field_Stone_3063385243",
0xb8862b71,"MainField_TBox_Field_Stone_3064461873",
0x4d4a87df,"MainField_TBox_Field_Stone_306589109",
0xb5225972,"MainField_TBox_Field_Stone_306589417",
0x00f8f4b5,"MainField_TBox_Field_Stone_3068312845",
0xe47da4ca,"MainField_TBox_Field_Stone_3079508418",
0xdb7444c2,"MainField_TBox_Field_Stone_3092787610",
0xc6f0075d,"MainField_TBox_Field_Stone_3128219364",
0x104551a4,"MainField_TBox_Field_Stone_3146080169",
0x9b41af3a,"MainField_TBox_Field_Stone_3150707937",
0x28e511ec,"MainField_TBox_Field_Stone_3152531233",
0xe71ffbab,"MainField_TBox_Field_Stone_3164657485",
0x86909fac,"MainField_TBox_Field_Stone_3167487135",
0x16f0b784,"MainField_TBox_Field_Stone_3184227158",
0x100d5840,"MainField_TBox_Field_Stone_3192340704",
0x717be4fe,"MainField_TBox_Field_Stone_3205722175",
0xafaac7dc,"MainField_TBox_Field_Stone_3222056974",
0xb6f16fab,"MainField_TBox_Field_Stone_3222588741",
0x07aa46b1,"MainField_TBox_Field_Stone_3247442831",
0xaa311bf7,"MainField_TBox_Field_Stone_3247604319",
0x0c9fd63b,"MainField_TBox_Field_Stone_3257654984",
0x76392a26,"MainField_TBox_Field_Stone_3271006069",
0xe987eaaa,"MainField_TBox_Field_Stone_3276934066",
0x643bea4d,"MainField_TBox_Field_Stone_3331856446",
0x6ad6ef6d,"MainField_TBox_Field_Stone_3335843054",
0xe49265b0,"MainField_TBox_Field_Stone_3383508387",
0x5b1861a5,"MainField_TBox_Field_Stone_3451030460",
0xe38d505b,"MainField_TBox_Field_Stone_3459972676",
0x0e315ae0,"MainField_TBox_Field_Stone_3472477842",
0x8c7f2706,"MainField_TBox_Field_Stone_3479397899",
0xe011ca98,"MainField_TBox_Field_Stone_3488762707",
0x35073270,"MainField_TBox_Field_Stone_3492141103",
0xcaaf5a56,"MainField_TBox_Field_Stone_3513756850",
0x6aa5a0c7,"MainField_TBox_Field_Stone_3520607590",
0xb1ff20ac,"MainField_TBox_Field_Stone_3534889990",
0xe2688e5f,"MainField_TBox_Field_Stone_3546218204",
0xe7204691,"MainField_TBox_Field_Stone_3552474534",
0x80a2c2c1,"MainField_TBox_Field_Stone_358523342",
0xef2c6046,"MainField_TBox_Field_Stone_3586739667",
0xd9bbed08,"MainField_TBox_Field_Stone_3588746178",
0x9402292c,"MainField_TBox_Field_Stone_3590540856",
0x8bcfdf31,"MainField_TBox_Field_Stone_3593840904",
0x5c417e05,"MainField_TBox_Field_Stone_3615423266",
0xdcfeeb09,"MainField_TBox_Field_Stone_3616618573",
0x79b437e6,"MainField_TBox_Field_Stone_3641499392",
0xf0cc21f0,"MainField_TBox_Field_Stone_3644975623",
0x480595ca,"MainField_TBox_Field_Stone_3651590101",
0xa6a5cdb6,"MainField_TBox_Field_Stone_3683400264",
0x048e2141,"MainField_TBox_Field_Stone_3701641776",
0x4f06c47a,"MainField_TBox_Field_Stone_3719160080",
0xd90eae52,"MainField_TBox_Field_Stone_3720073072",
0xcc5b07c4,"MainField_TBox_Field_Stone_3723546291",
0xd5a0400e,"MainField_TBox_Field_Stone_3728946584",
0x7897fe65,"MainField_TBox_Field_Stone_3759590144",
0x49f2593c,"MainField_TBox_Field_Stone_3769707659",
0xd78d377d,"MainField_TBox_Field_Stone_3772592165",
0xd07abf30,"MainField_TBox_Field_Stone_3779189787",
0xf59d3bb9,"MainField_TBox_Field_Stone_3791240011",
0xc6573514,"MainField_TBox_Field_Stone_3815956590",
0xf59c559c,"MainField_TBox_Field_Stone_3817004620",
0x49eca90d,"MainField_TBox_Field_Stone_3818405231",
0x68a15b1b,"MainField_TBox_Field_Stone_3835931569",
0x534beb99,"MainField_TBox_Field_Stone_3850783740",
0xea911dea,"MainField_TBox_Field_Stone_3855403820",
0x93fb6987,"MainField_TBox_Field_Stone_3861085089",
0x6b3cecc4,"MainField_TBox_Field_Stone_3880452204",
0x3bca2de0,"MainField_TBox_Field_Stone_3904471551",
0xb07f42c6,"MainField_TBox_Field_Stone_3911656389",
0xddd1a557,"MainField_TBox_Field_Stone_3933297202",
0xf57fabd7,"MainField_TBox_Field_Stone_3956041863",
0x33f26456,"MainField_TBox_Field_Stone_3963465037",
0xdc6bbe4b,"MainField_TBox_Field_Stone_3985186090",
0x474d2b8f,"MainField_TBox_Field_Stone_3986555859",
0x7ad77762,"MainField_TBox_Field_Stone_4011611883",
0xec7e1b0b,"MainField_TBox_Field_Stone_4017030562",
0x6bc7ff2c,"MainField_TBox_Field_Stone_4020217226",
0xf388da8d,"MainField_TBox_Field_Stone_4029984",
0xa65417dd,"MainField_TBox_Field_Stone_4034423928",
0x8dd70415,"MainField_TBox_Field_Stone_4034727745",
0x524a05df,"MainField_TBox_Field_Stone_4036738292",
0x84115040,"MainField_TBox_Field_Stone_406185851",
0xbe5fc90a,"MainField_TBox_Field_Stone_4076098459",
0x92b930bc,"MainField_TBox_Field_Stone_4101208962",
0x3b2d1cbc,"MainField_TBox_Field_Stone_4125758579",
0x1a07a107,"MainField_TBox_Field_Stone_4129291175",
0x1b678b1f,"MainField_TBox_Field_Stone_4141537481",
0x5b88b433,"MainField_TBox_Field_Stone_4144346101",
0x2bd8a636,"MainField_TBox_Field_Stone_4169515330",
0xbc90594a,"MainField_TBox_Field_Stone_4193667768",
0xfcc78f7c,"MainField_TBox_Field_Stone_4201017484",
0x722a77da,"MainField_TBox_Field_Stone_4202917339",
0xf2f65b2f,"MainField_TBox_Field_Stone_4215984636",
0xaf74d6a9,"MainField_TBox_Field_Stone_4269152644",
0x249098f8,"MainField_TBox_Field_Stone_4269878726",
0x0582068b,"MainField_TBox_Field_Stone_4287160948",
0xaf75e504,"MainField_TBox_Field_Stone_43619475",
0x4603aa1e,"MainField_TBox_Field_Stone_468536599",
0xcbc99345,"MainField_TBox_Field_Stone_469170580",
0x159372da,"MainField_TBox_Field_Stone_471032154",
0xf5e71654,"MainField_TBox_Field_Stone_473877286",
0x43c53a39,"MainField_TBox_Field_Stone_479936489",
0xdb6dda4f,"MainField_TBox_Field_Stone_496333431",
0x0fdcfe98,"MainField_TBox_Field_Stone_504874141",
0xdf2ad01e,"MainField_TBox_Field_Stone_536149570",
0x1d064008,"MainField_TBox_Field_Stone_591392381",
0x47315397,"MainField_TBox_Field_Stone_600253898",
0x01ab2301,"MainField_TBox_Field_Stone_606035849",
0x4972be23,"MainField_TBox_Field_Stone_607911630",
0xd7b78861,"MainField_TBox_Field_Stone_611155917",
0xe211a588,"MainField_TBox_Field_Stone_634727440",
0xcb171c77,"MainField_TBox_Field_Stone_636083304",
0x6095093e,"MainField_TBox_Field_Stone_636489135",
0xead1a91d,"MainField_TBox_Field_Stone_646828635",
0x91519b1e,"MainField_TBox_Field_Stone_65242927",
0xfccfb309,"MainField_TBox_Field_Stone_676170857",
0x0a23d585,"MainField_TBox_Field_Stone_68921327",
0x6a26ab54,"MainField_TBox_Field_Stone_693429994",
0x24baa87e,"MainField_TBox_Field_Stone_696793431",
0xbcfe4966,"MainField_TBox_Field_Stone_703644996",
0x2217b52d,"MainField_TBox_Field_Stone_724255579",
0xfa5c4e69,"MainField_TBox_Field_Stone_753815850",
0xd73568c2,"MainField_TBox_Field_Stone_776371786",
0x17e37e17,"MainField_TBox_Field_Stone_793213793",
0xace84991,"MainField_TBox_Field_Stone_793944584",
0xf211f353,"MainField_TBox_Field_Stone_796277443",
0x42df5491,"MainField_TBox_Field_Stone_815609585",
0xe5fcd74a,"MainField_TBox_Field_Stone_820697865",
0xfee8f7e9,"MainField_TBox_Field_Stone_824379463",
0x5e85952c,"MainField_TBox_Field_Stone_828753638",
0x2a7cc148,"MainField_TBox_Field_Stone_835342906",
0x6f45eba6,"MainField_TBox_Field_Stone_859339332",
0x559ee015,"MainField_TBox_Field_Stone_865093810",
0xdd97c317,"MainField_TBox_Field_Stone_876552493",
0xf0e75a34,"MainField_TBox_Field_Stone_883235353",
0xf69a43b3,"MainField_TBox_Field_Stone_88585716",
0x0bbdeaaa,"MainField_TBox_Field_Stone_911152910",
0x01468d32,"MainField_TBox_Field_Stone_91802921",
0x072f9ed0,"MainField_TBox_Field_Stone_932195220",
0xfa581b74,"MainField_TBox_Field_Stone_946822747",
0x5d9793da,"MainField_TBox_Field_Stone_949903176",
0x1fa18abb,"MainField_TBox_Field_Stone_950861039",
0x390811fe,"MainField_TBox_Field_Stone_953870732",
0x2e8cab4f,"MainField_TBox_Field_Stone_NoReaction_1129053593",
0x87b07161,"MainField_TBox_Field_Stone_NoReaction_1761649191",
0xd0a4d1a0,"MainField_TBox_Field_Stone_NoReaction_1994756610",
0xf398900b,"MainField_TBox_Field_Stone_NoReaction_2304114477",
0x12612d06,"MainField_TBox_Field_Stone_NoReaction_2896665769",
0x15891cfa,"MainField_TBox_Field_Stone_NoReaction_3105094027",
0x9fa2659a,"MainField_TBox_Field_Stone_NoReaction_4283842150",
0x3ca169e2,"MainField_TBox_Field_Stone_NoReaction_516117098",
0xdfa7978c,"MainField_TBox_Field_Stone_NoReaction_854114284",
0x48ee29db,"MainField_TBox_Field_Stone_NoReaction_993561984",
0x27092a31,"MainField_TBox_Field_Wood_1004674500",
0xa9e03f86,"MainField_TBox_Field_Wood_1019527254",
0xc06ca735,"MainField_TBox_Field_Wood_104119229",
0x4bab758e,"MainField_TBox_Field_Wood_1042555180",
0x0e910d7b,"MainField_TBox_Field_Wood_1057953325",
0x1910a99c,"MainField_TBox_Field_Wood_1060537711",
0xceb238af,"MainField_TBox_Field_Wood_1087571833",
0xb3129f4e,"MainField_TBox_Field_Wood_1203619059",
0xfb7a001b,"MainField_TBox_Field_Wood_1234441321",
0x9bff7bd2,"MainField_TBox_Field_Wood_1253211792",
0xdd79acf2,"MainField_TBox_Field_Wood_1280223692",
0x3d8796be,"MainField_TBox_Field_Wood_1319867852",
0xac896da4,"MainField_TBox_Field_Wood_1325848240",
0x3a39874b,"MainField_TBox_Field_Wood_1355194819",
0xd552a4bc,"MainField_TBox_Field_Wood_1359342477",
0x7d7ad81f,"MainField_TBox_Field_Wood_1413590562",
0xdd4a0e2a,"MainField_TBox_Field_Wood_1427933274",
0x24bc2e33,"MainField_TBox_Field_Wood_1446344031",
0xce2ebf53,"MainField_TBox_Field_Wood_1454959846",
0xce355b70,"MainField_TBox_Field_Wood_1457770742",
0x429e2c90,"MainField_TBox_Field_Wood_1493548515",
0xcb20e05d,"MainField_TBox_Field_Wood_149982252",
0xda298b23,"MainField_TBox_Field_Wood_1524125403",
0xbc1c09d5,"MainField_TBox_Field_Wood_1542514592",
0x3a74b5bf,"MainField_TBox_Field_Wood_1565885892",
0x3635b76d,"MainField_TBox_Field_Wood_1627044513",
0x64fd3e2e,"MainField_TBox_Field_Wood_1673894958",
0x0aae5233,"MainField_TBox_Field_Wood_1689737867",
0xa077a339,"MainField_TBox_Field_Wood_1732745671",
0xd6e0d2cf,"MainField_TBox_Field_Wood_1764582201",
0x9da9b1f6,"MainField_TBox_Field_Wood_1767134980",
0xb53f1947,"MainField_TBox_Field_Wood_1778606661",
0xe410970f,"MainField_TBox_Field_Wood_1781413350",
0xde89db26,"MainField_TBox_Field_Wood_1782046241",
0xf1ea054c,"MainField_TBox_Field_Wood_1813328737",
0x7f8056c1,"MainField_TBox_Field_Wood_1823339420",
0xb23df113,"MainField_TBox_Field_Wood_1846342251",
0x80d2f50a,"MainField_TBox_Field_Wood_1860880510",
0xdb6a5413,"MainField_TBox_Field_Wood_1866611099",
0xac7c05f3,"MainField_TBox_Field_Wood_1903790114",
0x1e7a6622,"MainField_TBox_Field_Wood_1930979206",
0xc6d6d2f2,"MainField_TBox_Field_Wood_1949315180",
0x88d8282c,"MainField_TBox_Field_Wood_1973681754",
0x974d3ecb,"MainField_TBox_Field_Wood_198140701",
0x6bd9a5a5,"MainField_TBox_Field_Wood_201748986",
0x82f49c12,"MainField_TBox_Field_Wood_2019636385",
0x48412957,"MainField_TBox_Field_Wood_208016799",
0x8b3f99ff,"MainField_TBox_Field_Wood_2080206557",
0x2b298de9,"MainField_TBox_Field_Wood_2086905331",
0xc3a38ff3,"MainField_TBox_Field_Wood_2089566527",
0xe7031981,"MainField_TBox_Field_Wood_2105406180",
0x271e3a68,"MainField_TBox_Field_Wood_2127983644",
0x9788f876,"MainField_TBox_Field_Wood_2129092038",
0x17799e03,"MainField_TBox_Field_Wood_2162469202",
0x8e1fa7cd,"MainField_TBox_Field_Wood_2181546185",
0xaa993b9e,"MainField_TBox_Field_Wood_2223263316",
0x9d2bf638,"MainField_TBox_Field_Wood_2251280312",
0x78d4c62e,"MainField_TBox_Field_Wood_2335175359",
0xe5693b6b,"MainField_TBox_Field_Wood_2390940536",
0xf1411f01,"MainField_TBox_Field_Wood_2404988468",
0xd79e10c9,"MainField_TBox_Field_Wood_240876353",
0x566491e7,"MainField_TBox_Field_Wood_2419136961",
0x7c4ce474,"MainField_TBox_Field_Wood_2425719185",
0x8d3273fa,"MainField_TBox_Field_Wood_2430942843",
0x83b35e02,"MainField_TBox_Field_Wood_2431438792",
0x79b3582e,"MainField_TBox_Field_Wood_2449442138",
0x2f60b612,"MainField_TBox_Field_Wood_2492542738",
0x0f72c610,"MainField_TBox_Field_Wood_2516433116",
0x4be79642,"MainField_TBox_Field_Wood_2538651084",
0x4d10ebad,"MainField_TBox_Field_Wood_2546798802",
0x0cc689cb,"MainField_TBox_Field_Wood_2587816358",
0x3eb56d9b,"MainField_TBox_Field_Wood_2588303677",
0x1b0a1ab4,"MainField_TBox_Field_Wood_2608008029",
0x67dafd2d,"MainField_TBox_Field_Wood_2630033614",
0x4e01e008,"MainField_TBox_Field_Wood_2655278067",
0xa5d883aa,"MainField_TBox_Field_Wood_2665261857",
0x53942ade,"MainField_TBox_Field_Wood_26843095",
0xdfdd1b88,"MainField_TBox_Field_Wood_2722154720",
0x8318f76a,"MainField_TBox_Field_Wood_2725121838",
0x705b6dab,"MainField_TBox_Field_Wood_2731693161",
0xec714d7d,"MainField_TBox_Field_Wood_2759914145",
0x636488e3,"MainField_TBox_Field_Wood_2767370557",
0xa9ef8d30,"MainField_TBox_Field_Wood_2796710778",
0xefbc65ca,"MainField_TBox_Field_Wood_2868175396",
0x7d911fb2,"MainField_TBox_Field_Wood_287602722",
0x3cf631b0,"MainField_TBox_Field_Wood_2934429776",
0x0f1f694b,"MainField_TBox_Field_Wood_293873791",
0x9ad8f609,"MainField_TBox_Field_Wood_298151136",
0xc9e82287,"MainField_TBox_Field_Wood_2990124756",
0x98fd0136,"MainField_TBox_Field_Wood_2994892897",
0x39f07412,"MainField_TBox_Field_Wood_3005657673",
0xdc09c1da,"MainField_TBox_Field_Wood_3006395575",
0xf8b4af67,"MainField_TBox_Field_Wood_300913965",
0x3f265824,"MainField_TBox_Field_Wood_3027193464",
0xe5d90023,"MainField_TBox_Field_Wood_3054626206",
0xe4080fba,"MainField_TBox_Field_Wood_3054883027",
0xb1adf3a7,"MainField_TBox_Field_Wood_3123151764",
0x38a570a3,"MainField_TBox_Field_Wood_3126858690",
0x635c62b0,"MainField_TBox_Field_Wood_3131003854",
0xa4ed01fe,"MainField_TBox_Field_Wood_313309550",
0x121daa20,"MainField_TBox_Field_Wood_3138298400",
0x58e940e2,"MainField_TBox_Field_Wood_3142551221",
0x73d3876c,"MainField_TBox_Field_Wood_3191667538",
0x16da978b,"MainField_TBox_Field_Wood_3238946093",
0x068cc122,"MainField_TBox_Field_Wood_3286156217",
0xe31f7107,"MainField_TBox_Field_Wood_3310771271",
0xe84fdea0,"MainField_TBox_Field_Wood_3342659956",
0x2993137b,"MainField_TBox_Field_Wood_3360742972",
0x15277c46,"MainField_TBox_Field_Wood_3397386231",
0x87fbe0c3,"MainField_TBox_Field_Wood_3409174617",
0x5100641e,"MainField_TBox_Field_Wood_3427519095",
0xf029ebc9,"MainField_TBox_Field_Wood_345113310",
0xc3b44600,"MainField_TBox_Field_Wood_349167694",
0x692bbaa5,"MainField_TBox_Field_Wood_350171171",
0x421fdb64,"MainField_TBox_Field_Wood_3502255245",
0x18a6cd01,"MainField_TBox_Field_Wood_3503225987",
0x5bbaa51a,"MainField_TBox_Field_Wood_3557126828",
0x0ae73c4f,"MainField_TBox_Field_Wood_3581020301",
0xe42361e9,"MainField_TBox_Field_Wood_3608319860",
0x6650189a,"MainField_TBox_Field_Wood_3623835588",
0xc0c41d14,"MainField_TBox_Field_Wood_36541502",
0x80aa500d,"MainField_TBox_Field_Wood_3671578421",
0x562c6d7a,"MainField_TBox_Field_Wood_36837439",
0x6daba0ff,"MainField_TBox_Field_Wood_3697504869",
0xb9959224,"MainField_TBox_Field_Wood_3758213647",
0xb051a3de,"MainField_TBox_Field_Wood_3760398519",
0xd9d056d6,"MainField_TBox_Field_Wood_3762803882",
0x8cc4b495,"MainField_TBox_Field_Wood_3767454055",
0xdf1f21cd,"MainField_TBox_Field_Wood_3773227241",
0x1b28251b,"MainField_TBox_Field_Wood_3780012504",
0x94d1d87e,"MainField_TBox_Field_Wood_383692990",
0xa853da8d,"MainField_TBox_Field_Wood_3844696999",
0xcf335116,"MainField_TBox_Field_Wood_3858828915",
0x07dbd7f3,"MainField_TBox_Field_Wood_3875059148",
0x589b25ec,"MainField_TBox_Field_Wood_3881229906",
0xdfa4e531,"MainField_TBox_Field_Wood_3914513255",
0x590609ad,"MainField_TBox_Field_Wood_3917275651",
0xdb30bba5,"MainField_TBox_Field_Wood_3940677724",
0xe2afc772,"MainField_TBox_Field_Wood_3963691802",
0x51b2fe8a,"MainField_TBox_Field_Wood_3970545864",
0x922cac80,"MainField_TBox_Field_Wood_3990760242",
0x1e54756b,"MainField_TBox_Field_Wood_3995412180",
0xa7a876ff,"MainField_TBox_Field_Wood_4020491445",
0xb23394a5,"MainField_TBox_Field_Wood_4037251314",
0xf8416448,"MainField_TBox_Field_Wood_4046343091",
0x62bd2fe0,"MainField_TBox_Field_Wood_4054147276",
0xf5941ebb,"MainField_TBox_Field_Wood_407446232",
0xcc92173b,"MainField_TBox_Field_Wood_4092278495",
0x6918a6b7,"MainField_TBox_Field_Wood_4158888388",
0x94ad4ed3,"MainField_TBox_Field_Wood_4199145894",
0x6e23378b,"MainField_TBox_Field_Wood_4210547311",
0x336bf9fa,"MainField_TBox_Field_Wood_4214260471",
0x947d7050,"MainField_TBox_Field_Wood_4244277401",
0xe3ca24ef,"MainField_TBox_Field_Wood_430630870",
0x2cf952fd,"MainField_TBox_Field_Wood_441992928",
0xfc0aa4aa,"MainField_TBox_Field_Wood_484243667",
0xb4dd0b52,"MainField_TBox_Field_Wood_494258676",
0xf719fd70,"MainField_TBox_Field_Wood_500243416",
0xfeb83047,"MainField_TBox_Field_Wood_515835868",
0x339ab087,"MainField_TBox_Field_Wood_519268736",
0x93d01bbc,"MainField_TBox_Field_Wood_53105567",
0x4b50a584,"MainField_TBox_Field_Wood_547751767",
0x3bdaccea,"MainField_TBox_Field_Wood_576770187",
0xbb0d4ffc,"MainField_TBox_Field_Wood_599080023",
0x2e56159b,"MainField_TBox_Field_Wood_614595506",
0x2836e1ba,"MainField_TBox_Field_Wood_655823661",
0x20efd5ab,"MainField_TBox_Field_Wood_686576388",
0xf3bb53d5,"MainField_TBox_Field_Wood_693233483",
0x5857175f,"MainField_TBox_Field_Wood_710063442",
0x9f840b29,"MainField_TBox_Field_Wood_747184169",
0xf322d0ef,"MainField_TBox_Field_Wood_755056013",
0x7beb6d74,"MainField_TBox_Field_Wood_776493141",
0xc7232c1e,"MainField_TBox_Field_Wood_776728342",
0x66ddef3d,"MainField_TBox_Field_Wood_885284593",
0x051fa772,"MainField_TBox_Field_Wood_9268528",
0x58433eed,"MainField_TBox_Field_Wood_950706040",
0x44bc13de,"MainField_TBox_Field_Wood_956481320",
0xd37436ba,"MainField_TBox_Field_Wood_962756328",
0x943feca2,"MainField_TBox_Field_Wood_994679130",
0x6e43ef67,"MainField_TBox_Field_Wood_99574487",
0x63f557dc,"MainField_TreasureSpot_00_1030976624",
0xdf6431bf,"MainField_TreasureSpot_00_1032265635",
0x4c911134,"MainField_TreasureSpot_00_1039840665",
0xf0b7f3f1,"MainField_TreasureSpot_00_1041432463",
0x5edd7b6d,"MainField_TreasureSpot_00_1052114056",
0x5b06db0a,"MainField_TreasureSpot_00_1052267589",
0x88f3f53c,"MainField_TreasureSpot_00_1074342136",
0x9e922294,"MainField_TreasureSpot_00_1081177219",
0x42669087,"MainField_TreasureSpot_00_1091607022",
0x1f34356e,"MainField_TreasureSpot_00_109842226",
0x8c4d7b1a,"MainField_TreasureSpot_00_112080104",
0xa4f7d67b,"MainField_TreasureSpot_00_1123505267",
0xba22e45a,"MainField_TreasureSpot_00_1136789709",
0x5cf8e013,"MainField_TreasureSpot_00_1139410203",
0x5ff090c5,"MainField_TreasureSpot_00_1146971406",
0xa9d8d46a,"MainField_TreasureSpot_00_1155896551",
0xcdbdd873,"MainField_TreasureSpot_00_1166068731",
0x29f81fed,"MainField_TreasureSpot_00_1168878696",
0x73e54fe9,"MainField_TreasureSpot_00_1180037759",
0x927c5633,"MainField_TreasureSpot_00_1185769017",
0x34f4edc5,"MainField_TreasureSpot_00_1186901640",
0x0cac827b,"MainField_TreasureSpot_00_1193062554",
0xe0324c77,"MainField_TreasureSpot_00_1194634648",
0x9919718f,"MainField_TreasureSpot_00_1196220727",
0x48d912de,"MainField_TreasureSpot_00_1203477346",
0xb5ff9e0c,"MainField_TreasureSpot_00_1204509986",
0xe305f5a7,"MainField_TreasureSpot_00_1214113050",
0xf7926e39,"MainField_TreasureSpot_00_1226854496",
0x163741de,"MainField_TreasureSpot_00_1238211061",
0x44a9e4c5,"MainField_TreasureSpot_00_1258128655",
0x393e1a63,"MainField_TreasureSpot_00_1260273183",
0x4dd4eb0f,"MainField_TreasureSpot_00_1280560459",
0xe209b570,"MainField_TreasureSpot_00_131079053",
0x2aa00237,"MainField_TreasureSpot_00_1316120740",
0x42f953d5,"MainField_TreasureSpot_00_1321759431",
0x95c959d2,"MainField_TreasureSpot_00_1325943645",
0xf875cf92,"MainField_TreasureSpot_00_1332215843",
0xe8635558,"MainField_TreasureSpot_00_1338517412",
0x54bc9cc5,"MainField_TreasureSpot_00_1338708444",
0x14800f62,"MainField_TreasureSpot_00_1352947090",
0xdaf7417d,"MainField_TreasureSpot_00_138644534",
0xeaadedc8,"MainField_TreasureSpot_00_1390893364",
0xaf3f9616,"MainField_TreasureSpot_00_1415569295",
0x4361f5af,"MainField_TreasureSpot_00_1420775223",
0x583790ad,"MainField_TreasureSpot_00_142103058",
0x0ca64105,"MainField_TreasureSpot_00_1431790700",
0x6ca40b75,"MainField_TreasureSpot_00_1434756577",
0x1a9547db,"MainField_TreasureSpot_00_143747271",
0x36055b03,"MainField_TreasureSpot_00_1455090418",
0x36cef3b0,"MainField_TreasureSpot_00_1456480413",
0x0f9e9ecd,"MainField_TreasureSpot_00_146234542",
0xf4e8f1b4,"MainField_TreasureSpot_00_1472827787",
0x107ecb20,"MainField_TreasureSpot_00_1535685692",
0xb26189ba,"MainField_TreasureSpot_00_1537731128",
0x53ee8ac4,"MainField_TreasureSpot_00_1537874506",
0x7954b63f,"MainField_TreasureSpot_00_1537919850",
0x86db6fd2,"MainField_TreasureSpot_00_154979120",
0xa613cffd,"MainField_TreasureSpot_00_1549818317",
0x271738d4,"MainField_TreasureSpot_00_1561614304",
0x884769ac,"MainField_TreasureSpot_00_1576813059",
0xf3f69a37,"MainField_TreasureSpot_00_1592117896",
0xa3333daa,"MainField_TreasureSpot_00_1600548022",
0x90b49488,"MainField_TreasureSpot_00_1601660258",
0x90274289,"MainField_TreasureSpot_00_1608233795",
0xf16ce11e,"MainField_TreasureSpot_00_1615662846",
0x16fefe0d,"MainField_TreasureSpot_00_1616643803",
0x5c1f1f84,"MainField_TreasureSpot_00_1623335880",
0x5074d856,"MainField_TreasureSpot_00_1648430761",
0xc78a4cfa,"MainField_TreasureSpot_00_1653749520",
0x069aa43b,"MainField_TreasureSpot_00_1686696187",
0x1dea3bab,"MainField_TreasureSpot_00_1687387228",
0x686e25e8,"MainField_TreasureSpot_00_1703743023",
0xdbf2a301,"MainField_TreasureSpot_00_1710056712",
0xb2e8fd1f,"MainField_TreasureSpot_00_1711272713",
0xe81ec37c,"MainField_TreasureSpot_00_1717757095",
0x4cae62d7,"MainField_TreasureSpot_00_1717851077",
0x81129e56,"MainField_TreasureSpot_00_1739581393",
0xdcf726cf,"MainField_TreasureSpot_00_1756982421",
0xc9a994ce,"MainField_TreasureSpot_00_1760516878",
0x2428067c,"MainField_TreasureSpot_00_176673798",
0x0056532f,"MainField_TreasureSpot_00_1782434349",
0x26bab1ba,"MainField_TreasureSpot_00_1789189406",
0x71dfcc60,"MainField_TreasureSpot_00_1795068409",
0xd12d6195,"MainField_TreasureSpot_00_1806803707",
0xf548caae,"MainField_TreasureSpot_00_1810041395",
0xa24a704b,"MainField_TreasureSpot_00_1821298218",
0x32114d3c,"MainField_TreasureSpot_00_182278127",
0x4b45109c,"MainField_TreasureSpot_00_1826528831",
0x84f74b74,"MainField_TreasureSpot_00_1833963060",
0x9d2c0920,"MainField_TreasureSpot_00_1870831383",
0xb7dccc8f,"MainField_TreasureSpot_00_1878903548",
0x3be38bf1,"MainField_TreasureSpot_00_1880088932",
0xb436ff83,"MainField_TreasureSpot_00_1881726539",
0x5143a6c2,"MainField_TreasureSpot_00_1919663213",
0x67748f36,"MainField_TreasureSpot_00_1923439954",
0x230aebc8,"MainField_TreasureSpot_00_1931284196",
0xc9ea6217,"MainField_TreasureSpot_00_1939112170",
0x295a44f0,"MainField_TreasureSpot_00_194056184",
0xa0591d3f,"MainField_TreasureSpot_00_194210798",
0x565e8573,"MainField_TreasureSpot_00_1949739195",
0xaaf5dc8a,"MainField_TreasureSpot_00_1952577784",
0x86b9c7d0,"MainField_TreasureSpot_00_1964303653",
0x1a835c7a,"MainField_TreasureSpot_00_1965168467",
0xf64edad2,"MainField_TreasureSpot_00_198160754",
0x6361c938,"MainField_TreasureSpot_00_1989783126",
0x9874f8ad,"MainField_TreasureSpot_00_1992228834",
0xcc060ec3,"MainField_TreasureSpot_00_1994849213",
0xbb44bc33,"MainField_TreasureSpot_00_2035695551",
0x96282980,"MainField_TreasureSpot_00_2043959896",
0xe64d566b,"MainField_TreasureSpot_00_2072577617",
0xfe642855,"MainField_TreasureSpot_00_2082902189",
0x32718aa9,"MainField_TreasureSpot_00_2090097528",
0xcc15c646,"MainField_TreasureSpot_00_2090221422",
0x32c1880e,"MainField_TreasureSpot_00_2093515236",
0xdfb5f37a,"MainField_TreasureSpot_00_2095366728",
0x416aac4b,"MainField_TreasureSpot_00_2117860262",
0xadd6aba4,"MainField_TreasureSpot_00_212128531",
0x0b8f5723,"MainField_TreasureSpot_00_2126090589",
0x7dc87bd9,"MainField_TreasureSpot_00_2128630208",
0xfc251d5b,"MainField_TreasureSpot_00_2139054724",
0xf9338f90,"MainField_TreasureSpot_00_2151956793",
0xb6b5ef4c,"MainField_TreasureSpot_00_2154427353",
0x8131574d,"MainField_TreasureSpot_00_2161716201",
0xb13de51e,"MainField_TreasureSpot_00_2165576701",
0x13d34937,"MainField_TreasureSpot_00_2182569719",
0x967287db,"MainField_TreasureSpot_00_2189471494",
0x0176fccb,"MainField_TreasureSpot_00_2200654171",
0x8a41c63b,"MainField_TreasureSpot_00_2213166462",
0xb66eba0f,"MainField_TreasureSpot_00_2217978625",
0x7f8da6ea,"MainField_TreasureSpot_00_2226095936",
0x87df5e3b,"MainField_TreasureSpot_00_2243771146",
0xbb53a417,"MainField_TreasureSpot_00_224737949",
0xc6824cdd,"MainField_TreasureSpot_00_2250141476",
0xe6f135da,"MainField_TreasureSpot_00_2264333995",
0xe04df529,"MainField_TreasureSpot_00_2274672254",
0x0ff0a27f,"MainField_TreasureSpot_00_2338491438",
0x3c8cfb7c,"MainField_TreasureSpot_00_2340646659",
0x09cf9bb9,"MainField_TreasureSpot_00_234856123",
0x56e6e241,"MainField_TreasureSpot_00_2359055932",
0x70fb77e3,"MainField_TreasureSpot_00_2363808418",
0x22cba6e7,"MainField_TreasureSpot_00_2366803188",
0xfbd29cc6,"MainField_TreasureSpot_00_2395751140",
0xbd1555c3,"MainField_TreasureSpot_00_2419432074",
0x43624a98,"MainField_TreasureSpot_00_2442793342",
0x870dddab,"MainField_TreasureSpot_00_2455860786",
0xf6f6355a,"MainField_TreasureSpot_00_2478966103",
0xaf1c88d8,"MainField_TreasureSpot_00_248736462",
0xc5989b2f,"MainField_TreasureSpot_00_2494135878",
0x5d875d80,"MainField_TreasureSpot_00_250016826",
0x820205a1,"MainField_TreasureSpot_00_2504340888",
0x529f5ede,"MainField_TreasureSpot_00_2508679037",
0xd899d454,"MainField_TreasureSpot_00_2511176361",
0xacae362f,"MainField_TreasureSpot_00_2515901019",
0xae496f33,"MainField_TreasureSpot_00_2518468529",
0xc2bc7ad7,"MainField_TreasureSpot_00_2523137844",
0x543e8343,"MainField_TreasureSpot_00_2535422255",
0xcda593f0,"MainField_TreasureSpot_00_2535835997",
0x54d19c5d,"MainField_TreasureSpot_00_2566689191",
0xc50d842c,"MainField_TreasureSpot_00_2567025478",
0xa0a56091,"MainField_TreasureSpot_00_2572518887",
0x37f747f9,"MainField_TreasureSpot_00_2575111957",
0x7398cc56,"MainField_TreasureSpot_00_2603382399",
0xdae19eaf,"MainField_TreasureSpot_00_2621551206",
0xb33a8d17,"MainField_TreasureSpot_00_2623921473",
0x70c3a648,"MainField_TreasureSpot_00_2676317832",
0x2695bbfa,"MainField_TreasureSpot_00_2678547794",
0x02b04c5b,"MainField_TreasureSpot_00_2682084055",
0x375e6c60,"MainField_TreasureSpot_00_2698078507",
0x16d0cea5,"MainField_TreasureSpot_00_2702527294",
0x9b2534dc,"MainField_TreasureSpot_00_2705043780",
0x2e3c410a,"MainField_TreasureSpot_00_2725486760",
0x234ac03d,"MainField_TreasureSpot_00_2750522508",
0xb9efcf5c,"MainField_TreasureSpot_00_2797413520",
0x0cd87e9f,"MainField_TreasureSpot_00_2797764230",
0x1c8d2e22,"MainField_TreasureSpot_00_2814638297",
0xbba33394,"MainField_TreasureSpot_00_2826124507",
0x3009eb60,"MainField_TreasureSpot_00_2836582838",
0x75dda4b8,"MainField_TreasureSpot_00_2855514881",
0x3589a576,"MainField_TreasureSpot_00_2862722552",
0xe6d855a4,"MainField_TreasureSpot_00_2878337889",
0x184d2da9,"MainField_TreasureSpot_00_2898790749",
0xf023c32a,"MainField_TreasureSpot_00_2905252472",
0xed2d0317,"MainField_TreasureSpot_00_291726163",
0xec797d56,"MainField_TreasureSpot_00_291962742",
0xb6431a4e,"MainField_TreasureSpot_00_2926949618",
0xf07a86c6,"MainField_TreasureSpot_00_2929080671",
0x84a99235,"MainField_TreasureSpot_00_2935380288",
0x0d88a5b8,"MainField_TreasureSpot_00_2936600214",
0x39c61876,"MainField_TreasureSpot_00_2945888398",
0x7116b782,"MainField_TreasureSpot_00_2947388511",
0xf9ea32f7,"MainField_TreasureSpot_00_2963667902",
0xa45ff1dd,"MainField_TreasureSpot_00_2968234916",
0xa543db71,"MainField_TreasureSpot_00_2976386458",
0xf94eef3c,"MainField_TreasureSpot_00_2988214384",
0x477b3a87,"MainField_TreasureSpot_00_3009818995",
0x4bbcef03,"MainField_TreasureSpot_00_3015128180",
0x0be9ce82,"MainField_TreasureSpot_00_3028801625",
0x88382cd6,"MainField_TreasureSpot_00_3035633081",
0x9a2cd39e,"MainField_TreasureSpot_00_304871974",
0xeb68c841,"MainField_TreasureSpot_00_3078835238",
0xd7ca1101,"MainField_TreasureSpot_00_3086639057",
0xe3e6c11a,"MainField_TreasureSpot_00_310419428",
0xa1e4a286,"MainField_TreasureSpot_00_3109939087",
0xa2c71ed0,"MainField_TreasureSpot_00_311049913",
0x7279d788,"MainField_TreasureSpot_00_3121472215",
0xea06bfd7,"MainField_TreasureSpot_00_3123351814",
0x44d3baa7,"MainField_TreasureSpot_00_3153799630",
0x4b19c4f0,"MainField_TreasureSpot_00_3165237971",
0xf9c53c4b,"MainField_TreasureSpot_00_3172876640",
0x7ccd8d21,"MainField_TreasureSpot_00_3178682505",
0xe17c900c,"MainField_TreasureSpot_00_3189016732",
0x8e9edfec,"MainField_TreasureSpot_00_3196205416",
0x4e525207,"MainField_TreasureSpot_00_3198825662",
0x637617f3,"MainField_TreasureSpot_00_3233432957",
0xe7a706e0,"MainField_TreasureSpot_00_3237834567",
0xe48734a6,"MainField_TreasureSpot_00_3238197525",
0x1379f7f4,"MainField_TreasureSpot_00_3251815131",
0xd281c43d,"MainField_TreasureSpot_00_3253143488",
0xd5e8e1d5,"MainField_TreasureSpot_00_3258174410",
0x552ada4b,"MainField_TreasureSpot_00_3271787900",
0x386b301f,"MainField_TreasureSpot_00_3279421466",
0x0cc40256,"MainField_TreasureSpot_00_3280005327",
0xe1d3039c,"MainField_TreasureSpot_00_328486090",
0xa27e1761,"MainField_TreasureSpot_00_3285215286",
0x82ea9f81,"MainField_TreasureSpot_00_3286089979",
0x87dd8f49,"MainField_TreasureSpot_00_32882717",
0x401f48db,"MainField_TreasureSpot_00_3294691813",
0x58074939,"MainField_TreasureSpot_00_3335709433",
0xc8c8602d,"MainField_TreasureSpot_00_3344722382",
0x24d5bc8c,"MainField_TreasureSpot_00_3354172210",
0x31c23804,"MainField_TreasureSpot_00_3361472617",
0x73f8b7a0,"MainField_TreasureSpot_00_3375668353",
0x2402d909,"MainField_TreasureSpot_00_3380939262",
0xae0e0daa,"MainField_TreasureSpot_00_3431855195",
0x80427d64,"MainField_TreasureSpot_00_3471363559",
0xce0d4b0c,"MainField_TreasureSpot_00_3472028598",
0xc04192c3,"MainField_TreasureSpot_00_3490198402",
0x7c38d861,"MainField_TreasureSpot_00_3502557263",
0x2bf2b327,"MainField_TreasureSpot_00_3511989408",
0x4a5b7fbf,"MainField_TreasureSpot_00_352990005",
0xf8992a84,"MainField_TreasureSpot_00_3550171139",
0xf83a8f97,"MainField_TreasureSpot_00_3553764562",
0x277d8329,"MainField_TreasureSpot_00_3575979433",
0xcabed18c,"MainField_TreasureSpot_00_3576024201",
0x647b9854,"MainField_TreasureSpot_00_3596893563",
0xcf18ce53,"MainField_TreasureSpot_00_35989445",
0x962c297d,"MainField_TreasureSpot_00_3603002910",
0x4b39a20d,"MainField_TreasureSpot_00_3607721921",
0x6488fb00,"MainField_TreasureSpot_00_3621515632",
0xc5068714,"MainField_TreasureSpot_00_3637178965",
0x171c5144,"MainField_TreasureSpot_00_3644043707",
0xbe2ce931,"MainField_TreasureSpot_00_3648160868",
0x69afea88,"MainField_TreasureSpot_00_3655252578",
0x88dd4786,"MainField_TreasureSpot_00_3674080939",
0x8fc8567d,"MainField_TreasureSpot_00_3708398816",
0xbe56a381,"MainField_TreasureSpot_00_3716624328",
0x4850196b,"MainField_TreasureSpot_00_3718511415",
0x2797d990,"MainField_TreasureSpot_00_3722362535",
0xd4ef246a,"MainField_TreasureSpot_00_3725975840",
0x26b672ab,"MainField_TreasureSpot_00_3748924493",
0x83b64dd8,"MainField_TreasureSpot_00_375269299",
0xa2249c46,"MainField_TreasureSpot_00_3776706343",
0x3de06faa,"MainField_TreasureSpot_00_3778290229",
0x28af5349,"MainField_TreasureSpot_00_3825865485",
0x8c722f08,"MainField_TreasureSpot_00_3828079357",
0x7c6029af,"MainField_TreasureSpot_00_3851755372",
0xc4f5ff54,"MainField_TreasureSpot_00_3855054374",
0x21a1e40b,"MainField_TreasureSpot_00_3857552370",
0x2baa96bd,"MainField_TreasureSpot_00_386658913",
0xe6ad6d26,"MainField_TreasureSpot_00_3867771465",
0x4e75f76c,"MainField_TreasureSpot_00_3880990619",
0xbfcf0e3b,"MainField_TreasureSpot_00_389268892",
0x3617fe77,"MainField_TreasureSpot_00_3900861982",
0xeedc6efb,"MainField_TreasureSpot_00_3906628161",
0x93422278,"MainField_TreasureSpot_00_3916122057",
0xba5c766d,"MainField_TreasureSpot_00_3916561633",
0xa5d2e7a5,"MainField_TreasureSpot_00_3920481724",
0x6e45e6df,"MainField_TreasureSpot_00_3934759901",
0xb1599e08,"MainField_TreasureSpot_00_3935478425",
0x17d34592,"MainField_TreasureSpot_00_3940914432",
0x6ca354cf,"MainField_TreasureSpot_00_3947266800",
0xcf9caef6,"MainField_TreasureSpot_00_3949370169",
0xbd892e78,"MainField_TreasureSpot_00_3962259177",
0xbd0cb157,"MainField_TreasureSpot_00_3978678357",
0x0e97abe2,"MainField_TreasureSpot_00_3999925130",
0x49d54e32,"MainField_TreasureSpot_00_400196100",
0xccf711ff,"MainField_TreasureSpot_00_4006805816",
0x9ac8b2e2,"MainField_TreasureSpot_00_4014195994",
0x39fa546f,"MainField_TreasureSpot_00_401740022",
0xfc11d81b,"MainField_TreasureSpot_00_4019257324",
0xb5c8ba99,"MainField_TreasureSpot_00_4021792344",
0x61f76839,"MainField_TreasureSpot_00_4026245425",
0x467a2019,"MainField_TreasureSpot_00_4072278940",
0x3580fadb,"MainField_TreasureSpot_00_4081650777",
0x955b4df4,"MainField_TreasureSpot_00_409004382",
0xd566f6ef,"MainField_TreasureSpot_00_4094165332",
0x6e01edea,"MainField_TreasureSpot_00_4096549325",
0x4a12120f,"MainField_TreasureSpot_00_4099497870",
0x127c3c87,"MainField_TreasureSpot_00_4108072772",
0x0fb58875,"MainField_TreasureSpot_00_4111494753",
0x875f6bcd,"MainField_TreasureSpot_00_4112275242",
0xbba6be35,"MainField_TreasureSpot_00_4130356136",
0x049f3458,"MainField_TreasureSpot_00_4190859890",
0x33e9d3b6,"MainField_TreasureSpot_00_4193100274",
0xb1fa22d9,"MainField_TreasureSpot_00_4207294510",
0x8db33647,"MainField_TreasureSpot_00_4209966875",
0x393cfc17,"MainField_TreasureSpot_00_4217116578",
0x34cc045c,"MainField_TreasureSpot_00_4226390924",
0x01cc357a,"MainField_TreasureSpot_00_4228193808",
0x42b1663d,"MainField_TreasureSpot_00_4234778112",
0xac4314c9,"MainField_TreasureSpot_00_4236533653",
0x4b6e75d4,"MainField_TreasureSpot_00_4244021939",
0xc7b6f83d,"MainField_TreasureSpot_00_4255048597",
0x8b83e6ee,"MainField_TreasureSpot_00_4290036687",
0x96c3e3cb,"MainField_TreasureSpot_00_435087429",
0x2936088a,"MainField_TreasureSpot_00_449816644",
0x021e4901,"MainField_TreasureSpot_00_456524586",
0xe49f3202,"MainField_TreasureSpot_00_462028476",
0x53361cef,"MainField_TreasureSpot_00_464097531",
0x0fb95369,"MainField_TreasureSpot_00_475705094",
0xe3be64f0,"MainField_TreasureSpot_00_481585840",
0xfdf56865,"MainField_TreasureSpot_00_484719269",
0xfa3b9045,"MainField_TreasureSpot_00_492869188",
0x683f0240,"MainField_TreasureSpot_00_515963412",
0xcb636d1c,"MainField_TreasureSpot_00_542217646",
0x75f90b13,"MainField_TreasureSpot_00_549652944",
0x28d07691,"MainField_TreasureSpot_00_550412573",
0x2b4cb685,"MainField_TreasureSpot_00_551988427",
0x18347d80,"MainField_TreasureSpot_00_566304351",
0x31dc3adc,"MainField_TreasureSpot_00_575689978",
0x0264c021,"MainField_TreasureSpot_00_598388902",
0xe29033b2,"MainField_TreasureSpot_00_609231325",
0xc70b2ebc,"MainField_TreasureSpot_00_638385654",
0xd0816d3a,"MainField_TreasureSpot_00_677469216",
0x84f1946c,"MainField_TreasureSpot_00_688046624",
0xbc2b4f93,"MainField_TreasureSpot_00_688452161",
0x68a816eb,"MainField_TreasureSpot_00_70567921",
0x56ea26f7,"MainField_TreasureSpot_00_716426499",
0x6dd1214c,"MainField_TreasureSpot_00_717120974",
0x39c5281c,"MainField_TreasureSpot_00_732280480",
0xeb0bef49,"MainField_TreasureSpot_00_755147984",
0xb4184058,"MainField_TreasureSpot_00_763274095",
0x0800861b,"MainField_TreasureSpot_00_768844247",
0x163419bf,"MainField_TreasureSpot_00_775280491",
0x773a6aaa,"MainField_TreasureSpot_00_800372391",
0xb0f0e2ed,"MainField_TreasureSpot_00_807108620",
0xc0fb620e,"MainField_TreasureSpot_00_833246895",
0xe4a7707c,"MainField_TreasureSpot_00_837985432",
0x282186cb,"MainField_TreasureSpot_00_842590030",
0x3914f355,"MainField_TreasureSpot_00_851171035",
0x40bba423,"MainField_TreasureSpot_00_860908330",
0xd568f0cb,"MainField_TreasureSpot_00_864088468",
0x407c3a3d,"MainField_TreasureSpot_00_871431436",
0xdae6bd41,"MainField_TreasureSpot_00_875545535",
0x31d0e397,"MainField_TreasureSpot_00_879927934",
0xc61c025b,"MainField_TreasureSpot_00_88663004",
0x2a9044b8,"MainField_TreasureSpot_00_887960973",
0x2128d93a,"MainField_TreasureSpot_00_906077996",
0x11b15cac,"MainField_TreasureSpot_00_907520376",
0x81ba320a,"MainField_TreasureSpot_00_9078555",
0xa94c6da1,"MainField_TreasureSpot_00_91995815",
0x400e3357,"MainField_TreasureSpot_00_928612534",
0xb703a05a,"MainField_TreasureSpot_00_950141386",
0xad36f99f,"MainField_TreasureSpot_00_952901962",
0x37d2bf52,"MainField_TreasureSpot_00_953095986",
0xd5d87702,"MainField_TreasureSpot_00_963991602",
0x49bb3c49,"MainField_TreasureSpot_00_968198153",
0xe8fa00e9,"MainField_TreasureSpot_00_970471569",
0x7c33d33e,"MainField_TreasureSpot_00_971728469",
0x82872bed,"MainField_TreasureSpot_00_993789076",
0x7deae2e7,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_1096929616",
0x3edf78df,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_1157258875",
0x998b63e2,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_1167749384",
0x419e2724,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_1298141241",
0x533f370b,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_131241364",
0x5873e0ac,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_1385461003",
0x861e1f77,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_1396408229",
0xb340d154,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_1777595191",
0x4db9f1bb,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_1777673021",
0xe03450c4,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_1810112249",
0x04de7e23,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_1975601821",
0x57a8df7b,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_1989802986",
0xdd39e62f,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_2146818259",
0x01150204,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_2174197046",
0x690ffba1,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_2185383719",
0x178e842a,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_2225320702",
0x2c422de4,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_226477099",
0x63f36176,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_2269183180",
0x80198100,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_2418549536",
0x699492a6,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_2545695561",
0xcb20aa37,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_2593177954",
0xc46ffb1c,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_2834064296",
0x2c3945e9,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_2900007538",
0x25ad5320,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_3099216669",
0x9af1aa5e,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_3126400757",
0x0af68d27,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_3206166012",
0xf1826614,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_3213240670",
0x6dbb523e,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_3263072609",
0x13d1ca15,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_327119818",
0xcf7f2a6e,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_3276731285",
0x275c8c31,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_3349493135",
0x5eb05349,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_3420642010",
0x12789b6e,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_3490823700",
0x5f826520,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_3509275003",
0xd981210c,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_3704373181",
0x5dbb0a11,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_3719438627",
0x2447a3df,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_3819442773",
0xed77b41d,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_395901592",
0x8ee5cc81,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_4181050245",
0x3431bfdb,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_4238302062",
0x18ed566a,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_462818471",
0x99f97ba7,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_694077341",
0xe90096ac,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_827630309",
0xa07a4f35,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_84061779",
0x8e711664,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_87671980",
0x56a74bc2,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_944396945",
0x8462f4c4,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_968110218",
0x988a90f3,"MainField_TwnObj_AncientCandlePoleRoofOff_A_01_995658852",
0x7a6f4080,"MainField_TwnObj_AncientReactorCoreOff_A_01_1168809635",
0x6459aebf,"MainField_TwnObj_AncientReactorCoreOff_A_01_1862424862",
0x5f69c973,"MainField_TwnObj_AncientReactorCore_A_01_2922594937",
0xd7128463,"MainField_TwnObj_AncientReactorCore_A_01_525213852",
0xf31b1373,"MainField_TwnObj_ArtifactObservationPostBed_A_01_1063309129",
0x9f323273,"MainField_TwnObj_City_GerudoPot_A_LL_Act_02_1905159111",
0x4db94457,"MainField_TwnObj_City_GerudoPot_A_LL_Act_02_2268725939",
0x25378e6b,"MainField_TwnObj_City_GerudoPot_A_LL_Act_02_2411389278",
0xf8d274f7,"MainField_TwnObj_City_GerudoPot_A_LL_Act_02_3014060692",
0x0474d85b,"MainField_TwnObj_City_GerudoPot_A_LL_Act_02_3504662392",
0x3d253334,"MainField_TwnObj_City_GerudoPot_A_LL_Act_02_728202805",
0x311b2dc5,"MainField_TwnObj_City_GerudoPot_A_M_Act_01_1772821196",
0xccbacdd7,"MainField_TwnObj_City_GerudoPot_A_M_Act_01_3302563854",
0xcf76cd74,"MainField_TwnObj_City_GerudoPot_A_M_Act_01_524183184",
0xc2d840d1,"MainField_TwnObj_City_GerudoPot_A_S_Act_01_1869825922",
0x18caa0bf,"MainField_TwnObj_City_GerudoPot_A_S_Act_01_60238134",
0xcac407bb,"MainField_TwnObj_City_GoronPot_A_M_Act_01_1055175128",
0x192dbf1c,"MainField_TwnObj_City_GoronPot_A_M_Act_01_1080545147",
0x408edf1d,"MainField_TwnObj_City_GoronPot_A_M_Act_01_108486626",
0x630272c4,"MainField_TwnObj_City_GoronPot_A_M_Act_01_1148725579",
0xcd1957c7,"MainField_TwnObj_City_GoronPot_A_M_Act_01_1231446386",
0x7a0ff405,"MainField_TwnObj_City_GoronPot_A_M_Act_01_1279763502",
0xd4a0a470,"MainField_TwnObj_City_GoronPot_A_M_Act_01_1319205211",
0x40bc4d8a,"MainField_TwnObj_City_GoronPot_A_M_Act_01_1432218545",
0xe82c41cf,"MainField_TwnObj_City_GoronPot_A_M_Act_01_162327443",
0x118635e9,"MainField_TwnObj_City_GoronPot_A_M_Act_01_2089935074",
0x57b1c5bf,"MainField_TwnObj_City_GoronPot_A_M_Act_01_2299793162",
0xd207e4a5,"MainField_TwnObj_City_GoronPot_A_M_Act_01_2440747871",
0x3c646828,"MainField_TwnObj_City_GoronPot_A_M_Act_01_2486102347",
0x25f96875,"MainField_TwnObj_City_GoronPot_A_M_Act_01_2520920475",
0xf7553771,"MainField_TwnObj_City_GoronPot_A_M_Act_01_2587268457",
0x46a37e71,"MainField_TwnObj_City_GoronPot_A_M_Act_01_2645481466",
0xd2774b9f,"MainField_TwnObj_City_GoronPot_A_M_Act_01_2838142904",
0xe36d8c5a,"MainField_TwnObj_City_GoronPot_A_M_Act_01_285611534",
0x1d9ada47,"MainField_TwnObj_City_GoronPot_A_M_Act_01_2887868983",
0x1c12f6d8,"MainField_TwnObj_City_GoronPot_A_M_Act_01_2991113276",
0x91924109,"MainField_TwnObj_City_GoronPot_A_M_Act_01_3008389567",
0x602a8fd3,"MainField_TwnObj_City_GoronPot_A_M_Act_01_3011352174",
0xaa6f26bb,"MainField_TwnObj_City_GoronPot_A_M_Act_01_3245354124",
0x69b1e9b5,"MainField_TwnObj_City_GoronPot_A_M_Act_01_3245824630",
0xc3cf0e9b,"MainField_TwnObj_City_GoronPot_A_M_Act_01_3248737924",
0x8eff8d48,"MainField_TwnObj_City_GoronPot_A_M_Act_01_3452007224",
0x75875a9e,"MainField_TwnObj_City_GoronPot_A_M_Act_01_3645692500",
0x4f710555,"MainField_TwnObj_City_GoronPot_A_M_Act_01_3770866588",
0x862870b4,"MainField_TwnObj_City_GoronPot_A_M_Act_01_3782398629",
0x1e0bdfd7,"MainField_TwnObj_City_GoronPot_A_M_Act_01_385863553",
0x9f2b1170,"MainField_TwnObj_City_GoronPot_A_M_Act_01_3966799435",
0x178c270e,"MainField_TwnObj_City_GoronPot_A_M_Act_01_4287013425",
0x7bb7eac1,"MainField_TwnObj_City_GoronPot_A_M_Act_01_521149395",
0x17f0b8ca,"MainField_TwnObj_City_GoronPot_A_M_Act_01_768570487",
0x725fe186,"MainField_TwnObj_City_GoronPot_A_M_Act_01_814270271",
0xb2046807,"MainField_TwnObj_City_GoronPot_A_M_Act_01_870516654",
0x51554c01,"MainField_TwnObj_City_GoronPot_A_M_Act_01_91288272",
0x3d8c440c,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_1135768782",
0x57a6a0dc,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_1327176971",
0x42b5b9dc,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_1500338246",
0xaf84bb63,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_1640342604",
0xe7cda068,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_1759489455",
0x0dc1d38e,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_19144040",
0x518718c8,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_1918299381",
0x4c08f515,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_2316467396",
0xe0db6347,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_2340561139",
0x0c3cdb1b,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_2382999493",
0x0e26dc72,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_2633144614",
0x31b3a350,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_2740510294",
0x2c6faf7c,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_277512779",
0xfde2b328,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_2784372570",
0x23c251ae,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_2853908048",
0x6ed299e2,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_2919935823",
0x9424db4d,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3158297927",
0x476f9399,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3383049152",
0x1223b88a,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3620304423",
0x5bec379e,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3682368065",
0x1cc560ef,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3723345584",
0xb06929ff,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3799452638",
0xd74d2dd2,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_3944743001",
0xede1e4bb,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_399932127",
0x31655819,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_4087077944",
0xc19a903f,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_749877593",
0xb8272d39,"MainField_TwnObj_GanonGrudgeSolid_Eyeball_A_01_810799956",
0x47afd395,"MainField_TwnObj_GateKeeperBed_A_01_205012717",
0x30f62d7d,"MainField_TwnObj_HeburaLodgeBed_A_01_107167427",
0x0f351cae,"MainField_TwnObj_HeburaLodgeBed_A_01_1224029870",
0x23caa7ef,"MainField_TwnObj_HeburaLodgeBed_A_01_246808931",
0xbed44d52,"MainField_TwnObj_HunterHouseBed_A_01_4092699273",
0x3aaaa531,"MainField_TwnObj_HyruleCastleObject_Wallcrack_Before_A_01_1171501220",
0xc65e3ac0,"MainField_TwnObj_HyruleCastleObject_Wallcrack_Before_A_01_1214323519",
0x88214b81,"MainField_TwnObj_HyruleCastleObject_Wallcrack_Before_A_01_1643320853",
0x5acdca80,"MainField_TwnObj_HyruleCastleObject_Wallcrack_Before_A_01_1813899894",
0xcdc942fd,"MainField_TwnObj_HyruleCastleObject_Wallcrack_Before_A_01_2447643139",
0x93e9a655,"MainField_TwnObj_HyruleCastleObject_Wallcrack_Before_A_01_2501364129",
0x538d9053,"MainField_TwnObj_HyruleCastleObject_Wallcrack_Before_A_01_314192002",
0x2eebdb69,"MainField_TwnObj_HyruleCastleObject_Wallcrack_Before_A_01_3262479335",
0x23d67e81,"MainField_TwnObj_HyruleCastleObject_Wallcrack_Before_A_01_422386106",
0xe08075a5,"MainField_TwnObj_HyruleCity_Tower_WallCrack_A_01_2619633328",
0x6bf45371,"MainField_TwnObj_HyruleCity_Tower_WallCrack_A_01_4156530792",
0x199da2ea,"MainField_TwnObj_Village_FishingPot_A_M_Act_01_1218123517",
0x16a968d1,"MainField_TwnObj_Village_FishingPot_A_M_Act_01_1708650433",
0xc78a8d01,"MainField_TwnObj_Village_FishingPot_A_M_Act_01_2828967171",
0xfed4b601,"MainField_TwnObj_Village_FishingPot_A_M_Act_01_3030801932",
0x964238cf,"MainField_TwnObj_Village_FishingPot_A_M_Act_01_3052366546",
0x4bab3fcc,"MainField_TwnObj_Village_FishingPot_A_M_Act_01_3436193884",
0x238a7723,"MainField_TwnObj_Village_FishingPot_A_M_Act_01_3603960586",
0x058cf523,"MainField_TwnObj_Village_FishingPot_A_M_Act_01_3938909591",
0x4868cef1,"MainField_TwnObj_Village_FishingPot_A_M_Act_01_4078270717",
0xdc6d0211,"MainField_TwnObj_Village_FishingPot_A_M_Act_01_408242977",
0xeb23d170,"MainField_TwnObj_Village_HatenoPot_A_L_Act_01_1079176173",
0x0b405c6e,"MainField_TwnObj_Village_HatenoPot_A_L_Act_01_1210235434",
0xd2c96e3d,"MainField_TwnObj_Village_HatenoPot_A_L_Act_01_1785502424",
0x69385772,"MainField_TwnObj_Village_HatenoPot_A_L_Act_01_2002199064",
0x5f6e7bfd,"MainField_TwnObj_Village_HatenoPot_A_L_Act_01_2736038970",
0x5d6b23f6,"MainField_TwnObj_Village_HatenoPot_A_L_Act_01_2775314897",
0x11322939,"MainField_TwnObj_Village_HatenoPot_A_L_Act_01_3423689864",
0x0cb11a48,"MainField_TwnObj_Village_HatenoPot_A_L_Act_01_3616516044",
0x0d0e9a9c,"MainField_TwnObj_Village_HatenoPot_A_L_Act_01_3852850489",
0x62a35011,"MainField_TwnObj_Village_HatenoPot_A_L_Act_01_4179325492",
0xe213a3cb,"MainField_TwnObj_Village_HatenoPot_A_L_Act_01_735676102",
0x41d9f628,"MainField_TwnObj_Village_HatenoPot_A_L_Act_01_842009019",
0x81decbb0,"MainField_TwnObj_Village_HatenoPot_A_L_Act_01_862572590",
0x94451fd7,"MainField_TwnObj_Village_HatenoPot_A_M_Act_01_1513777196",
0x04e77fdb,"MainField_TwnObj_Village_HatenoPot_A_M_Act_01_153736800",
0x72b03f6f,"MainField_TwnObj_Village_HatenoPot_A_M_Act_01_1970021328",
0xb4fa84d1,"MainField_TwnObj_Village_HatenoPot_A_M_Act_01_2572963614",
0x6d7cbc75,"MainField_TwnObj_Village_HatenoPot_A_M_Act_01_2629442567",
0x4fe1db77,"MainField_TwnObj_Village_HatenoPot_A_M_Act_01_2734589913",
0x00d74f87,"MainField_TwnObj_Village_HatenoPot_A_M_Act_01_2883427906",
0x5bca7f37,"MainField_TwnObj_Village_HatenoPot_A_M_Act_01_3003161766",
0xaa55b32d,"MainField_TwnObj_Village_HatenoPot_A_M_Act_01_3231219046",
0xb206fb45,"MainField_TwnObj_Village_HatenoPot_A_M_Act_01_4058545552",
0xc10c3cbe,"MainField_TwnObj_Village_HatenoPot_A_S_Act_01_1721157005",
0x1f585ecd,"MainField_TwnObj_Village_HatenoPot_A_S_Act_01_2338809574",
0xad49915b,"MainField_TwnObj_Village_HatenoPot_A_S_Act_01_2954787174",
0xad57eddc,"MainField_TwnObj_Village_HatenoPot_A_S_Act_01_3249665187",
0x942a7e07,"MainField_TwnObj_Village_HatenoPot_A_S_Act_01_3691317794",
0x3e8b5f09,"MainField_TwnObj_Village_HatenoPot_A_S_Act_01_385222324",
0x76ca740e,"MainField_TwnObj_Village_HatenoPot_A_S_Act_01_4096202391",
0x37d9bb12,"MainField_TwnObj_Village_HatenoPot_A_S_Act_01_530859117",
0xee327475,"MainField_TwnObj_Village_HatenoPot_A_S_Act_01_532028334",
0x4677ada5,"MainField_TwnObj_Village_HatenoPot_A_S_Act_01_917593417",
0xc01d9208,"MainField_TwnObj_Village_KorokBed_A_01_2474445534",
0x02ab40d1,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_1043384640",
0x9be38a34,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_1185150257",
0xc13dd096,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_123444671",
0x40f41b1e,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_134473415",
0x610d7631,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_1412743601",
0xa9dd4659,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_1553053995",
0xdb3b18a9,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_1583982738",
0x9b0509de,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_2034534709",
0x63c2c3c3,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_2302040771",
0xefff292e,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_2660266466",
0x1e2635d1,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_2684039558",
0x98602bdc,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_2971731350",
0x04f1ff55,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_2999934888",
0x71e93e22,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_3070435209",
0x9ce60478,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_308995843",
0x793a2a1e,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_3236432065",
0xb6093424,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_3493646330",
0x03597bbe,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_3558174169",
0xfe4ce6cd,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_3580821791",
0x1278de5e,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_3618319437",
0xe2d041f9,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_3810524798",
0x99099a22,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_3930893691",
0x7089d214,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_4063080066",
0x96e063f3,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_4191422550",
0x7070de18,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_63559335",
0x8b1b60ea,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_733090830",
0x2c708b02,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_738686593",
0x1a84f6ed,"MainField_TwnObj_Village_KorokPot_A_S_Act_01_787870574",
0xefe680ef,"MainField_TwnObj_Village_RitoPot_A_M_Act_01_1434963853",
0xe7f1072b,"MainField_TwnObj_Village_RitoPot_A_M_Act_01_1628734223",
0x5c5b5898,"MainField_TwnObj_Village_RitoPot_A_M_Act_01_2210961526",
0x72cc9705,"MainField_TwnObj_Village_RitoPot_A_M_Act_01_3450473732",
0xacf1e8ae,"MainField_TwnObj_Village_RitoPot_A_M_Act_01_3814466946",
0x8ca3c6ad,"MainField_TwnObj_Village_RitoPot_A_M_Act_02_1486471114",
0xdf5b547b,"MainField_TwnObj_Village_RitoPot_A_M_Act_02_2071062283",
0x3e02d0ad,"MainField_TwnObj_Village_RitoPot_A_M_Act_02_3726952826",
0xf9206ece,"MainField_TwnObj_Village_RitoPot_A_M_Act_02_4040352252",
0x1b5a434d,"MainField_TwnObj_Village_RitoPot_A_S_Act_01_1921482529",
0xa516f5e8,"MainField_TwnObj_Village_RitoPot_A_S_Act_01_1987024716",
0xf11a52f0,"MainField_TwnObj_Village_RitoPot_A_S_Act_01_2421908568",
0xe75e865d,"MainField_TwnObj_Village_RitoPot_A_S_Act_01_3886675391",
0x2b5c45f9,"MainField_TwnObj_Village_RitoPot_A_S_Act_02_1123867598",
0xe61d1404,"MainField_TwnObj_Village_RitoPot_A_S_Act_02_1340255113",
0xdf372e92,"MainField_TwnObj_Village_RitoPot_A_S_Act_02_1749401357",
0x66b05629,"MainField_TwnObj_Village_RitoPot_A_S_Act_02_2848678093",
0xb63d24ad,"MainField_TwnObj_Village_RitoPot_A_S_Act_02_3378502969",
0x4d2db654,"MainField_TwnObj_Village_RitoPot_A_S_Act_02_3671136583",
0xa0de35d8,"MainField_TwnObj_Village_RitoPot_A_S_Act_02_521177794",
0x9750060a,"MainField_TwnObj_Village_SakuradaBed_A_01_2426892765",
0xdaca925e,"MainField_TwnObj_Village_SheikerPot_A_LL_Act_01_2843902388",
0xbc954e18,"MainField_TwnObj_Village_SheikerPot_A_LL_Act_01_3285248149",
0x98c7e3ff,"MainField_TwnObj_Village_SheikerPot_A_LL_Act_01_3429509774",
0x33db3dc1,"MainField_TwnObj_Village_SheikerPot_A_LL_Act_01_3444208053",
0x1e286346,"MainField_TwnObj_Village_SheikerPot_A_L_Act_01_3668754806",
0xc71b4241,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_1018227618",
0x8d2873f0,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_1531103985",
0xeaac2c6d,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_1765025073",
0x5fb563c0,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_1876612793",
0xcd7eb306,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_2053177528",
0x5bab0221,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_2174553831",
0x4409930f,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_2222522664",
0x3e7b260c,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_270499956",
0xd23dbe03,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_3005199585",
0x9b03eb25,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_3340960151",
0xdf7a11dc,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_3393332461",
0xcb64c2d2,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_3750429647",
0x6285f341,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_4033981145",
0x26e3b593,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_4075766599",
0xad8b9770,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_700745150",
0x5b18eafb,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_729846585",
0x0dc444dd,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_757136586",
0x6cbe6820,"MainField_TwnObj_Village_SheikerPot_A_M_Act_02_965696732",
0x324ee3e1,"MainField_TwnObj_Village_SheikerPot_A_S_Act_01_1221462179",
0x85470986,"MainField_TwnObj_Village_SheikerPot_A_S_Act_01_1333662113",
0x381ffd5e,"MainField_TwnObj_Village_SheikerPot_A_S_Act_01_1580395311",
0x8e0d934b,"MainField_TwnObj_Village_SheikerPot_A_S_Act_01_2170041778",
0x441755a2,"MainField_TwnObj_Village_SheikerPot_A_S_Act_01_3470081406",
0xd7776246,"MainField_TwnObj_Village_SheikerPot_A_S_Act_01_352829014",
0xf452974b,"MainField_TwnObj_Village_SheikerPot_A_S_Act_01_4274001555",
0xb12322fc,"MainField_TwnObj_Village_SheikerPot_A_S_Act_01_622987168",
0xd02fe5be,"MainField_TwnObj_Village_SheikerPot_A_S_Act_01_668384427",
0xf5754ca6,"MainField_TwnObj_Village_ZoraFacilityBed_A_01_1959804623",
0x682d8237,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_1118969189",
0xe122c305,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_1145372268",
0x61c49b4f,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_1187427643",
0xc05a7647,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_1201448359",
0xac747b66,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_1487719864",
0x4d7cecae,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_1508436634",
0x6271ca71,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_1596544956",
0x56dbb5ff,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_1652221656",
0x8a075a80,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_1668080994",
0xc150c617,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_2681027986",
0xe88da940,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_3051763644",
0x39e45557,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_3130435527",
0x360612a2,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_3412494463",
0x99c82e39,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_3761855515",
0xcd74e405,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_3929990454",
0xc046c0d7,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_442695696",
0x3b1a2045,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_571725794",
0xcda1cd3c,"MainField_TwnObj_Village_ZoraPot_A_M_Act_01_744658668",
0x41e8b117,"MainField_VolcanoEruptionField_2534928850",
0x9203a9de,"MainField_VolcanoEruptionField_3190559051",
0x0f2456f4,"MainField_VolcanoEruptionField_3427512644",
0xf3851e42,"MainField_WallCrack_1049984653",
0x703e1502,"MainField_WallCrack_1053145203",
0xa7964323,"MainField_WallCrack_1123627511",
0x15fd2100,"MainField_WallCrack_1181579701",
0xbb4687fd,"MainField_WallCrack_1251951237",
0xec89c837,"MainField_WallCrack_1281757948",
0xc359d5c3,"MainField_WallCrack_1369978044",
0xce4e4aa6,"MainField_WallCrack_1569082577",
0x6e7e2159,"MainField_WallCrack_1588212217",
0x4a26f6e6,"MainField_WallCrack_158849031",
0x6283a76d,"MainField_WallCrack_1685487671",
0xf80ab407,"MainField_WallCrack_1707130878",
0x1b037e91,"MainField_WallCrack_1726043361",
0xf224e6af,"MainField_WallCrack_1769301740",
0x4fb55f9a,"MainField_WallCrack_1791968140",
0xcd9be488,"MainField_WallCrack_1793810263",
0x57d1ad71,"MainField_WallCrack_1818317227",
0x01591782,"MainField_WallCrack_1830418384",
0x65151c66,"MainField_WallCrack_1855149841",
0x49c67fee,"MainField_WallCrack_1861945606",
0x2cb2d3e7,"MainField_WallCrack_189449401",
0x92542596,"MainField_WallCrack_1965253572",
0x5ec80d3c,"MainField_WallCrack_2016472200",
0xfff0a5e6,"MainField_WallCrack_2024286019",
0x0b7ac10f,"MainField_WallCrack_203738156",
0x1cf4b150,"MainField_WallCrack_2064002312",
0xf5704ddf,"MainField_WallCrack_2077860469",
0x9561de45,"MainField_WallCrack_2085172862",
0x50ed1bec,"MainField_WallCrack_2130441648",
0xeab87894,"MainField_WallCrack_2139907134",
0xc3c6ec00,"MainField_WallCrack_2143377566",
0x5e091b1c,"MainField_WallCrack_2165012271",
0xf8ecf737,"MainField_WallCrack_2199361276",
0x6d0ca857,"MainField_WallCrack_221574926",
0xc686413c,"MainField_WallCrack_227012353",
0x76d46829,"MainField_WallCrack_2288991268",
0x6393d2c6,"MainField_WallCrack_2294190914",
0x9739a8c9,"MainField_WallCrack_2337573619",
0x9e123c1a,"MainField_WallCrack_234111856",
0x9db29216,"MainField_WallCrack_2348318710",
0xa0f7aed1,"MainField_WallCrack_2353734929",
0xe271661e,"MainField_WallCrack_2390206526",
0xfd697c41,"MainField_WallCrack_2435721673",
0x62149dfb,"MainField_WallCrack_2450318901",
0x0bc2e52f,"MainField_WallCrack_2500921677",
0x95af963a,"MainField_WallCrack_2502114997",
0x93f27116,"MainField_WallCrack_2504146725",
0x16832643,"MainField_WallCrack_2511460383",
0x2082fbdb,"MainField_WallCrack_2514056567",
0xafbd3fa6,"MainField_WallCrack_2514836261",
0xe330089e,"MainField_WallCrack_2542064663",
0x9590f54c,"MainField_WallCrack_2553563449",
0x95403a97,"MainField_WallCrack_2585390772",
0x5cd3ec82,"MainField_WallCrack_2661154289",
0xe7534c0f,"MainField_WallCrack_2696753735",
0xe4c9cff1,"MainField_WallCrack_2712778963",
0xd669accb,"MainField_WallCrack_272355955",
0xcea88edd,"MainField_WallCrack_2750186397",
0x6dc2ea14,"MainField_WallCrack_2751692253",
0x67d587d3,"MainField_WallCrack_2848948055",
0x9ab34308,"MainField_WallCrack_2852922354",
0x43e8aa15,"MainField_WallCrack_2873490761",
0x2f107138,"MainField_WallCrack_2990926156",
0x502fb259,"MainField_WallCrack_2997926861",
0xb186efe1,"MainField_WallCrack_3009987975",
0x0eb58d34,"MainField_WallCrack_3011326361",
0x334f0ec8,"MainField_WallCrack_3015215605",
0x0222de0a,"MainField_WallCrack_3051815056",
0x3cf992e3,"MainField_WallCrack_3075740545",
0x318c3425,"MainField_WallCrack_3115838229",
0xbfe29a54,"MainField_WallCrack_3124816605",
0x974c5d5a,"MainField_WallCrack_3126479315",
0x0ce86cc9,"MainField_WallCrack_3141304327",
0xc4ce6e8d,"MainField_WallCrack_314490793",
0x99ba883b,"MainField_WallCrack_3244434608",
0x0b74e21a,"MainField_WallCrack_3256111322",
0xb01cc9c4,"MainField_WallCrack_3261533948",
0x4f2828ab,"MainField_WallCrack_3261568942",
0x7039b256,"MainField_WallCrack_3324759584",
0xfefe578d,"MainField_WallCrack_3329889886",
0x1fe1c516,"MainField_WallCrack_3345569263",
0xb294b278,"MainField_WallCrack_3414324396",
0x12f62d4d,"MainField_WallCrack_3422476351",
0x960ed6f4,"MainField_WallCrack_3427407777",
0x6ecb4a99,"MainField_WallCrack_3461011497",
0x56c74927,"MainField_WallCrack_3470663373",
0xb6138440,"MainField_WallCrack_3491832767",
0x0078a28e,"MainField_WallCrack_349428949",
0xe932001d,"MainField_WallCrack_3531759785",
0x751c8937,"MainField_WallCrack_3548059791",
0xf87392c2,"MainField_WallCrack_3552454234",
0x3eab6d31,"MainField_WallCrack_3557006025",
0x57d8f918,"MainField_WallCrack_3629921746",
0xc7ac15aa,"MainField_WallCrack_3662974344",
0x14cf8ba9,"MainField_WallCrack_3715929339",
0x687e2c75,"MainField_WallCrack_3725120779",
0xadfb0e1f,"MainField_WallCrack_3773966902",
0x317a3daf,"MainField_WallCrack_3784379291",
0x7423f6d1,"MainField_WallCrack_3794777564",
0x972d7a5f,"MainField_WallCrack_382719991",
0x725efc2c,"MainField_WallCrack_3879717705",
0xe3af9403,"MainField_WallCrack_3889643866",
0x587eac4d,"MainField_WallCrack_3944769947",
0xde666bda,"MainField_WallCrack_3970927461",
0x9a2413f5,"MainField_WallCrack_4013128111",
0x9cb9764d,"MainField_WallCrack_4034693760",
0xe17b699a,"MainField_WallCrack_4072895477",
0x2a3283d7,"MainField_WallCrack_4118782575",
0x5a92efc2,"MainField_WallCrack_4126322625",
0x915cf3c7,"MainField_WallCrack_4201338911",
0x81ee7898,"MainField_WallCrack_4201470435",
0x766f24f8,"MainField_WallCrack_4202881590",
0x1d7e49a2,"MainField_WallCrack_4203978986",
0xc0941c8c,"MainField_WallCrack_4226799726",
0x877e745b,"MainField_WallCrack_4246958110",
0xbdc97567,"MainField_WallCrack_4249207730",
0xc79760e9,"MainField_WallCrack_4291702956",
0x0a8adac6,"MainField_WallCrack_436891043",
0x5a11f0a4,"MainField_WallCrack_438224710",
0xd0b237c9,"MainField_WallCrack_510809722",
0xf55f088d,"MainField_WallCrack_528992339",
0x7acd6f78,"MainField_WallCrack_549328500",
0x3e6ef876,"MainField_WallCrack_558857876",
0xe2cda790,"MainField_WallCrack_580313836",
0x0ea95b4f,"MainField_WallCrack_593752555",
0x5c4a6788,"MainField_WallCrack_595393022",
0xbdcbb5de,"MainField_WallCrack_618881274",
0x03a4ba40,"MainField_WallCrack_623489684",
0x405e3fbf,"MainField_WallCrack_641597480",
0x78d46199,"MainField_WallCrack_666209013",
0x61d17e33,"MainField_WallCrack_675782922",
0xd5c6359e,"MainField_WallCrack_707704912",
0x32dceada,"MainField_WallCrack_726774329",
0xcd7fe968,"MainField_WallCrack_731489329",
0x2ed8d1db,"MainField_WallCrack_809478316",
0xda3498e6,"MainField_WallCrack_809839696",
0xe31ae9ab,"MainField_WallCrack_84320156",
0xb86e2be6,"MainField_WallCrack_932188530",
0x1bdd98f7,"MainField_WallCrack_932796942",
0x0ab8d433,"MainField_WallCrack_975685220",
0x53c90275,"MainField_WeaponDrop_1001365173",
0x211f5dbf,"MainField_WeaponDrop_1002767823",
0xebfdf70c,"MainField_WeaponDrop_100612012",
0x6825f532,"MainField_WeaponDrop_1006676483",
0xc94279e9,"MainField_WeaponDrop_1011464761",
0xa84ac878,"MainField_WeaponDrop_1013811738",
0xb720b8c8,"MainField_WeaponDrop_1015522689",
0x96a8c21c,"MainField_WeaponDrop_1015583519",
0x9d0d99e8,"MainField_WeaponDrop_1015670154",
0x28ebf063,"MainField_WeaponDrop_1017714556",
0xb0b22c1d,"MainField_WeaponDrop_1017773918",
0xaa4b5722,"MainField_WeaponDrop_101922076",
0x51d2094e,"MainField_WeaponDrop_1019891529",
0x4925f6db,"MainField_WeaponDrop_1021249854",
0x03fc3a1d,"MainField_WeaponDrop_1021674645",
0x4d20e22b,"MainField_WeaponDrop_1022923663",
0x9c5618d9,"MainField_WeaponDrop_1023880069",
0xac86b65e,"MainField_WeaponDrop_102668443",
0x36f35ec3,"MainField_WeaponDrop_1027245261",
0x5f237330,"MainField_WeaponDrop_1031691508",
0x9559d552,"MainField_WeaponDrop_1034399304",
0xde7179cd,"MainField_WeaponDrop_1040659265",
0x74c1db56,"MainField_WeaponDrop_1043246721",
0xcadbdd54,"MainField_WeaponDrop_1043582976",
0xaf8216e0,"MainField_WeaponDrop_1047563228",
0xb52bb3a7,"MainField_WeaponDrop_1051203749",
0x1f411998,"MainField_WeaponDrop_1052826748",
0x1bbe9814,"MainField_WeaponDrop_1058302183",
0x3e7ceb91,"MainField_WeaponDrop_1059947333",
0x46c2093f,"MainField_WeaponDrop_1062355950",
0x404263c9,"MainField_WeaponDrop_1064468066",
0x5250476f,"MainField_WeaponDrop_1065058194",
0x6bd03ead,"MainField_WeaponDrop_1068511187",
0x3d16b350,"MainField_WeaponDrop_1069458457",
0xdc5afdec,"MainField_WeaponDrop_1070219736",
0x2b86c08b,"MainField_WeaponDrop_1070286140",
0x6786376f,"MainField_WeaponDrop_1070764311",
0xb241d0c7,"MainField_WeaponDrop_1070864093",
0x50c83cdf,"MainField_WeaponDrop_1072541836",
0x88f02aa1,"MainField_WeaponDrop_1077238452",
0x297334f7,"MainField_WeaponDrop_107840627",
0x3fb3c3ca,"MainField_WeaponDrop_1078964392",
0x8685c2b1,"MainField_WeaponDrop_1080209254",
0x0f9a1c77,"MainField_WeaponDrop_1082879458",
0xe1787c9f,"MainField_WeaponDrop_1084112513",
0x5427017b,"MainField_WeaponDrop_1090300090",
0x7ed45942,"MainField_WeaponDrop_109283973",
0xe7b142c2,"MainField_WeaponDrop_1092895672",
0x07b03708,"MainField_WeaponDrop_1094548807",
0xea8a071e,"MainField_WeaponDrop_1098506623",
0x2cce88a6,"MainField_WeaponDrop_1099140217",
0x127a21cb,"MainField_WeaponDrop_1103584980",
0xd059bf0c,"MainField_WeaponDrop_1103836040",
0xdb0b3724,"MainField_WeaponDrop_1106240555",
0x26808626,"MainField_WeaponDrop_110671315",
0x9b76d162,"MainField_WeaponDrop_1109555399",
0x9229612f,"MainField_WeaponDrop_1112852429",
0x83e247a7,"MainField_WeaponDrop_1113357472",
0xfb4df4ef,"MainField_WeaponDrop_1114647758",
0x6231d36d,"MainField_WeaponDrop_1115881052",
0xac9e35af,"MainField_WeaponDrop_1125174225",
0x0d57b42c,"MainField_WeaponDrop_1125247524",
0xd99435ab,"MainField_WeaponDrop_1127407760",
0x90b5feeb,"MainField_WeaponDrop_1127667894",
0xab795bc3,"MainField_WeaponDrop_1128785280",
0x6eaa9183,"MainField_WeaponDrop_1131927046",
0x5b7e4feb,"MainField_WeaponDrop_1133377874",
0xfae96b8c,"MainField_WeaponDrop_1134039637",
0xee228896,"MainField_WeaponDrop_1137173288",
0xe317ffd9,"MainField_WeaponDrop_1138226578",
0x094f6725,"MainField_WeaponDrop_1139672194",
0x0f38e039,"MainField_WeaponDrop_1142211222",
0xcfca0beb,"MainField_WeaponDrop_114483849",
0xbd5ff2da,"MainField_WeaponDrop_1148523592",
0x39d96697,"MainField_WeaponDrop_1151077216",
0x3ed513fc,"MainField_WeaponDrop_1151292624",
0xa3fe306a,"MainField_WeaponDrop_1154439433",
0xdcb7fcf8,"MainField_WeaponDrop_1157166271",
0x8ab0d79c,"MainField_WeaponDrop_1162979475",
0x3aa92b50,"MainField_WeaponDrop_1163152111",
0x7445e924,"MainField_WeaponDrop_1163222318",
0x8f1cad7f,"MainField_WeaponDrop_1164501735",
0xfca4a100,"MainField_WeaponDrop_1167117503",
0x2b417e9c,"MainField_WeaponDrop_1171106319",
0x64f4b5a0,"MainField_WeaponDrop_1172815045",
0x33e995e9,"MainField_WeaponDrop_1176469628",
0x68533dca,"MainField_WeaponDrop_118082339",
0x5c5bc092,"MainField_WeaponDrop_1182612283",
0xf846af44,"MainField_WeaponDrop_1182860777",
0x6f34f437,"MainField_WeaponDrop_1183512225",
0x29f6a9d4,"MainField_WeaponDrop_1186162709",
0x42061e87,"MainField_WeaponDrop_1186790006",
0x896f3158,"MainField_WeaponDrop_1187073520",
0xb86efec8,"MainField_WeaponDrop_118954450",
0x1f20d537,"MainField_WeaponDrop_119465492",
0xe54ab03e,"MainField_WeaponDrop_1198917678",
0xecc4e161,"MainField_WeaponDrop_1199028694",
0x101a6727,"MainField_WeaponDrop_1199203567",
0x36aae9ef,"MainField_WeaponDrop_1201480251",
0x7d254705,"MainField_WeaponDrop_1201681337",
0x446d9bb3,"MainField_WeaponDrop_1204321088",
0xeba22bc8,"MainField_WeaponDrop_120445127",
0xb713098d,"MainField_WeaponDrop_1207153239",
0x049c2980,"MainField_WeaponDrop_1208096369",
0x9be25d61,"MainField_WeaponDrop_1209055136",
0x3e8f1d7b,"MainField_WeaponDrop_1215521404",
0xc9faa7ed,"MainField_WeaponDrop_121564161",
0x81e799d7,"MainField_WeaponDrop_1220121743",
0xcce4d862,"MainField_WeaponDrop_1222964551",
0xf5533c4d,"MainField_WeaponDrop_1226875913",
0xa43b0e00,"MainField_WeaponDrop_1229857743",
0xe316479e,"MainField_WeaponDrop_1230299319",
0xb43cbfd3,"MainField_WeaponDrop_1232243279",
0xf1fd8ec2,"MainField_WeaponDrop_1232648292",
0xd4ab3c15,"MainField_WeaponDrop_123317298",
0x3d6533bd,"MainField_WeaponDrop_123321160",
0x6331f3b0,"MainField_WeaponDrop_1234828109",
0x6e1c1672,"MainField_WeaponDrop_1235901510",
0xa8b1c7be,"MainField_WeaponDrop_1243926187",
0x1b288c15,"MainField_WeaponDrop_1246540315",
0x0c04df0a,"MainField_WeaponDrop_1246630497",
0xba4fe28c,"MainField_WeaponDrop_1248430057",
0xc549982a,"MainField_WeaponDrop_125132995",
0xb708c033,"MainField_WeaponDrop_1253011801",
0x3e82b6dd,"MainField_WeaponDrop_1253931799",
0x0e805665,"MainField_WeaponDrop_1263250574",
0x758835da,"MainField_WeaponDrop_1263751526",
0x7b3b98fb,"MainField_WeaponDrop_1264339967",
0x58cdbc81,"MainField_WeaponDrop_1264579188",
0xedb0cb8a,"MainField_WeaponDrop_126518171",
0xc197b93c,"MainField_WeaponDrop_1267651868",
0x76af6c5d,"MainField_WeaponDrop_1268635018",
0x44ce7afa,"MainField_WeaponDrop_1268927125",
0x7a1f7c99,"MainField_WeaponDrop_1270139133",
0x95439406,"MainField_WeaponDrop_1273148419",
0xf871a94c,"MainField_WeaponDrop_1279604155",
0xd4baae0a,"MainField_WeaponDrop_1280311224",
0x24e8ee22,"MainField_WeaponDrop_1280645090",
0x5b185c42,"MainField_WeaponDrop_1284012326",
0xe6edd2ea,"MainField_WeaponDrop_1284218478",
0x13ac1076,"MainField_WeaponDrop_1284892793",
0x9995670e,"MainField_WeaponDrop_1285845600",
0x3624cbdc,"MainField_WeaponDrop_128866435",
0x4455946d,"MainField_WeaponDrop_1289452563",
0xabc32898,"MainField_WeaponDrop_1294194296",
0xcdf9e361,"MainField_WeaponDrop_1294581284",
0xb474a0e7,"MainField_WeaponDrop_1294616395",
0xa2c1b700,"MainField_WeaponDrop_1296050365",
0xac9251b6,"MainField_WeaponDrop_1296276071",
0x259cb27a,"MainField_WeaponDrop_129667315",
0x97937ab1,"MainField_WeaponDrop_1299978509",
0x9ae18846,"MainField_WeaponDrop_1300451783",
0x0f1da542,"MainField_WeaponDrop_1301759138",
0x3784f302,"MainField_WeaponDrop_1305063122",
0x9d2c65bb,"MainField_WeaponDrop_1305153543",
0x48332efc,"MainField_WeaponDrop_1305543746",
0xfaa43245,"MainField_WeaponDrop_1306387652",
0xd30a627d,"MainField_WeaponDrop_1308178254",
0xafa1604f,"MainField_WeaponDrop_131050949",
0xbac3ccfc,"MainField_WeaponDrop_1316072865",
0xddfc9e6f,"MainField_WeaponDrop_1317079596",
0x60837dbf,"MainField_WeaponDrop_1318239363",
0x8f7f6fa4,"MainField_WeaponDrop_1320807407",
0xfbeaf12e,"MainField_WeaponDrop_132089712",
0xc11f1c25,"MainField_WeaponDrop_1322934198",
0x59450bc5,"MainField_WeaponDrop_1327176595",
0x47bc80ad,"MainField_WeaponDrop_1327904397",
0x34940487,"MainField_WeaponDrop_1333480951",
0xf7863368,"MainField_WeaponDrop_1334139335",
0x9dc265dc,"MainField_WeaponDrop_1334736366",
0x2581c9c3,"MainField_WeaponDrop_1336315344",
0xed0544e0,"MainField_WeaponDrop_1337787928",
0xb5d1d23b,"MainField_WeaponDrop_1338138619",
0xefe77202,"MainField_WeaponDrop_1338188329",
0x78d5735c,"MainField_WeaponDrop_1338552694",
0x0ef0c8c6,"MainField_WeaponDrop_1340964342",
0x23348c5b,"MainField_WeaponDrop_1342135723",
0x2767d48b,"MainField_WeaponDrop_1344671327",
0xbea23d36,"MainField_WeaponDrop_1347835625",
0x91a37cc1,"MainField_WeaponDrop_1350975983",
0xf63cfca7,"MainField_WeaponDrop_1351366914",
0x375b48f3,"MainField_WeaponDrop_1352228075",
0x650546ef,"MainField_WeaponDrop_1355641488",
0xd59843ec,"MainField_WeaponDrop_1360093965",
0x84d53838,"MainField_WeaponDrop_1360983144",
0xf75de908,"MainField_WeaponDrop_1362709057",
0x87126fc1,"MainField_WeaponDrop_1363657073",
0x54aafa42,"MainField_WeaponDrop_1364851877",
0xb2368177,"MainField_WeaponDrop_1371283539",
0xf4184532,"MainField_WeaponDrop_1372040185",
0x4b88975a,"MainField_WeaponDrop_1373499562",
0xc7e7f7ee,"MainField_WeaponDrop_1373908271",
0x472e7dad,"MainField_WeaponDrop_1376003649",
0xb3a67fcf,"MainField_WeaponDrop_1377791512",
0x8bd2ecb8,"MainField_WeaponDrop_1384488657",
0xbed4f3f1,"MainField_WeaponDrop_1385555037",
0x417b5906,"MainField_WeaponDrop_1386088465",
0x26203d2d,"MainField_WeaponDrop_1386907804",
0x5160eaaa,"MainField_WeaponDrop_1386921869",
0x41cad5fa,"MainField_WeaponDrop_1390136277",
0x009e454e,"MainField_WeaponDrop_1391151182",
0x514317a2,"MainField_WeaponDrop_1393303184",
0xdf12ff09,"MainField_WeaponDrop_1394329252",
0x02120b5b,"MainField_WeaponDrop_1400316398",
0x626967de,"MainField_WeaponDrop_1401054036",
0x21eb9025,"MainField_WeaponDrop_1402318242",
0x0678c1a1,"MainField_WeaponDrop_1403735714",
0xdbf6db04,"MainField_WeaponDrop_1403815529",
0xfae3865d,"MainField_WeaponDrop_1405792760",
0x531d8fb9,"MainField_WeaponDrop_1406526805",
0x7f9f957d,"MainField_WeaponDrop_1406575917",
0x76c34b81,"MainField_WeaponDrop_1412727752",
0x1b4a2c12,"MainField_WeaponDrop_1415205181",
0x8e711aa8,"MainField_WeaponDrop_141802589",
0x9dbc81e9,"MainField_WeaponDrop_1421307765",
0x51cb5142,"MainField_WeaponDrop_1424822607",
0xa68b5983,"MainField_WeaponDrop_1424992077",
0x05722fc3,"MainField_WeaponDrop_1428723325",
0xf55bacb2,"MainField_WeaponDrop_1429510255",
0xc537de9f,"MainField_WeaponDrop_1432433364",
0x7e767c5b,"MainField_WeaponDrop_1437835106",
0xbbc53e75,"MainField_WeaponDrop_1438315280",
0xf174dd87,"MainField_WeaponDrop_1438426167",
0xe1fcc57b,"MainField_WeaponDrop_1440102021",
0x3dd65648,"MainField_WeaponDrop_1442779785",
0x0aabcdc3,"MainField_WeaponDrop_1443097011",
0x0b16743f,"MainField_WeaponDrop_1446498816",
0x05ff8e32,"MainField_WeaponDrop_1449598780",
0x873b0cec,"MainField_WeaponDrop_1450707008",
0xb17ab759,"MainField_WeaponDrop_1451907795",
0x481362a1,"MainField_WeaponDrop_1456128483",
0x40497b67,"MainField_WeaponDrop_1456360765",
0xbcdfc1cd,"MainField_WeaponDrop_1457730130",
0x0cae6639,"MainField_WeaponDrop_1458375279",
0xfdbbc8de,"MainField_WeaponDrop_1459471412",
0x429f75a3,"MainField_WeaponDrop_1462941879",
0xd865cca7,"MainField_WeaponDrop_1465900130",
0x4f351c16,"MainField_WeaponDrop_1466555059",
0xd194c6a5,"MainField_WeaponDrop_1466630772",
0xcfee2dfa,"MainField_WeaponDrop_1470825265",
0x5f72c203,"MainField_WeaponDrop_1477840888",
0xcd6c2e5a,"MainField_WeaponDrop_1478302003",
0xd537d48a,"MainField_WeaponDrop_1478704496",
0x8d7e5b7d,"MainField_WeaponDrop_1481502521",
0x7ad5090c,"MainField_WeaponDrop_1485122072",
0xdb45e413,"MainField_WeaponDrop_1488735641",
0xa9ea2df8,"MainField_WeaponDrop_1489268323",
0x103f76ae,"MainField_WeaponDrop_1493474973",
0xec127afb,"MainField_WeaponDrop_1494145493",
0x5a4220a7,"MainField_WeaponDrop_1497852493",
0x0f7818bf,"MainField_WeaponDrop_1499207441",
0xbf5f58fb,"MainField_WeaponDrop_1502448273",
0x0843d0a3,"MainField_WeaponDrop_1503282085",
0xc2aa43ca,"MainField_WeaponDrop_1503606483",
0xb56b77fb,"MainField_WeaponDrop_1503816238",
0x25f30f67,"MainField_WeaponDrop_1504712451",
0xbc5476cf,"MainField_WeaponDrop_1509884430",
0xf18e5fd1,"MainField_WeaponDrop_1523403768",
0xfb7e500d,"MainField_WeaponDrop_1524766699",
0x864d3d04,"MainField_WeaponDrop_1525637263",
0xc8322d33,"MainField_WeaponDrop_1528251673",
0xc83cdb28,"MainField_WeaponDrop_1529161689",
0x55406b86,"MainField_WeaponDrop_1532384765",
0xdeb133fc,"MainField_WeaponDrop_1532614312",
0x55bca25e,"MainField_WeaponDrop_1533493071",
0x3932e4e5,"MainField_WeaponDrop_1534948559",
0xfb4fc9f4,"MainField_WeaponDrop_1537013180",
0xfce6ac05,"MainField_WeaponDrop_1537103879",
0x76667764,"MainField_WeaponDrop_1537958621",
0x231fc28b,"MainField_WeaponDrop_1538394284",
0x7116cda7,"MainField_WeaponDrop_1539531052",
0xa4388504,"MainField_WeaponDrop_1543730610",
0x1f103221,"MainField_WeaponDrop_1543885641",
0x39061181,"MainField_WeaponDrop_1545237730",
0x03a85922,"MainField_WeaponDrop_1545563978",
0x3e43e895,"MainField_WeaponDrop_1546795223",
0xe9193e3c,"MainField_WeaponDrop_1548333698",
0xb277d133,"MainField_WeaponDrop_1550472069",
0xda52fda7,"MainField_WeaponDrop_155063",
0xe59a2f1a,"MainField_WeaponDrop_1551356620",
0x2f83a556,"MainField_WeaponDrop_1553986020",
0xed9306b0,"MainField_WeaponDrop_1561452682",
0x9a1038df,"MainField_WeaponDrop_1563464172",
0x6a50e355,"MainField_WeaponDrop_1564038146",
0x51ed4987,"MainField_WeaponDrop_1567215521",
0xca9b5b91,"MainField_WeaponDrop_1567743544",
0x821911a5,"MainField_WeaponDrop_1568430447",
0xb47f2edf,"MainField_WeaponDrop_1570675666",
0x45df72dd,"MainField_WeaponDrop_1571914769",
0xd3c1b196,"MainField_WeaponDrop_1576638466",
0xbe918edc,"MainField_WeaponDrop_1579855444",
0x990ec6f0,"MainField_WeaponDrop_1581651314",
0x169e6f7c,"MainField_WeaponDrop_1583267043",
0x175e44cd,"MainField_WeaponDrop_1586975139",
0x03097a09,"MainField_WeaponDrop_1588365142",
0x48f231dd,"MainField_WeaponDrop_1591834617",
0xc856b1c1,"MainField_WeaponDrop_1592734532",
0x86fdef99,"MainField_WeaponDrop_1593546409",
0xc2a08615,"MainField_WeaponDrop_1593838394",
0x94c1112d,"MainField_WeaponDrop_1596668682",
0x67bbe790,"MainField_WeaponDrop_1597086243",
0x281a373c,"MainField_WeaponDrop_1597399660",
0x10df7cce,"MainField_WeaponDrop_159909367",
0x63fd9c57,"MainField_WeaponDrop_160070374",
0x6c941a58,"MainField_WeaponDrop_1602354287",
0x6138a6da,"MainField_WeaponDrop_1603298937",
0xf2131569,"MainField_WeaponDrop_1605436871",
0x795e2677,"MainField_WeaponDrop_1607020727",
0x70890cd3,"MainField_WeaponDrop_1609063018",
0x4ec5da80,"MainField_WeaponDrop_1609857055",
0x05dca206,"MainField_WeaponDrop_1609920676",
0x47499cf3,"MainField_WeaponDrop_161193723",
0xdeb5e090,"MainField_WeaponDrop_1616574850",
0x4d92e90f,"MainField_WeaponDrop_1617670493",
0x35f4556f,"MainField_WeaponDrop_1618234161",
0x6ab43be3,"MainField_WeaponDrop_1621163733",
0xd12abb90,"MainField_WeaponDrop_1622902906",
0x940871a9,"MainField_WeaponDrop_1623879299",
0xc1d8ec08,"MainField_WeaponDrop_1624090318",
0xb3b820a2,"MainField_WeaponDrop_1626688977",
0xa24c0ba6,"MainField_WeaponDrop_1628494291",
0x34591d42,"MainField_WeaponDrop_1633375806",
0xe15566fa,"MainField_WeaponDrop_1635694359",
0xa7acd5a8,"MainField_WeaponDrop_1636274683",
0xf1c8d762,"MainField_WeaponDrop_1637323296",
0xed31a066,"MainField_WeaponDrop_1637592175",
0x52956d24,"MainField_WeaponDrop_1637904977",
0xb82fc4b8,"MainField_WeaponDrop_1642113969",
0x0f589de8,"MainField_WeaponDrop_1642347852",
0x8fc7daa3,"MainField_WeaponDrop_1646276122",
0xb3ae8e3f,"MainField_WeaponDrop_1648230344",
0x71ba50ca,"MainField_WeaponDrop_1649574144",
0xe2311822,"MainField_WeaponDrop_1650231658",
0x010371c4,"MainField_WeaponDrop_1650780151",
0x35ad199d,"MainField_WeaponDrop_165507875",
0x579c3822,"MainField_WeaponDrop_1656844317",
0x3de7621b,"MainField_WeaponDrop_1658616657",
0xf973a8a4,"MainField_WeaponDrop_1667370476",
0x312b453d,"MainField_WeaponDrop_1668121676",
0x59358754,"MainField_WeaponDrop_1671149722",
0xe8ec3648,"MainField_WeaponDrop_1673255800",
0x5c9ec855,"MainField_WeaponDrop_1674508860",
0xe730897c,"MainField_WeaponDrop_1675407094",
0x10d1e00f,"MainField_WeaponDrop_1677738069",
0xf052ff12,"MainField_WeaponDrop_167828043",
0x157b3593,"MainField_WeaponDrop_1680616240",
0x84ef2730,"MainField_WeaponDrop_1681368831",
0x3b5866e9,"MainField_WeaponDrop_1683018729",
0xf5c2e721,"MainField_WeaponDrop_1685682583",
0x792153cd,"MainField_WeaponDrop_1688324317",
0xcd439d44,"MainField_WeaponDrop_1692288152",
0x2369193b,"MainField_WeaponDrop_1692306815",
0x81030c88,"MainField_WeaponDrop_1697632751",
0xeaf64d16,"MainField_WeaponDrop_1698577073",
0x2f2d4536,"MainField_WeaponDrop_1704283284",
0xa6913c36,"MainField_WeaponDrop_1704609113",
0xcfdcc4b2,"MainField_WeaponDrop_1704933624",
0xe821fd75,"MainField_WeaponDrop_1706018959",
0xcdd06943,"MainField_WeaponDrop_1706910284",
0xca3592d4,"MainField_WeaponDrop_1707676224",
0xbe49ed08,"MainField_WeaponDrop_1708573951",
0xa47142bd,"MainField_WeaponDrop_1710903830",
0xa491101b,"MainField_WeaponDrop_171210282",
0xb14c6061,"MainField_WeaponDrop_1714941956",
0x65b0c0d7,"MainField_WeaponDrop_1714942535",
0xb66fa116,"MainField_WeaponDrop_1721531942",
0xcdba6de7,"MainField_WeaponDrop_1721654732",
0xf6d019a0,"MainField_WeaponDrop_1722172746",
0xa059efd5,"MainField_WeaponDrop_1722227595",
0xec68b992,"MainField_WeaponDrop_1722574269",
0x3a53665a,"MainField_WeaponDrop_1723124751",
0xc5f2f9f9,"MainField_WeaponDrop_1727237127",
0xa7e6b583,"MainField_WeaponDrop_1729262244",
0xc8bea37c,"MainField_WeaponDrop_1729595208",
0xcb3db017,"MainField_WeaponDrop_1729715850",
0x6681cf64,"MainField_WeaponDrop_1730566048",
0x89efedb2,"MainField_WeaponDrop_1730908603",
0xc5fd0209,"MainField_WeaponDrop_1732119069",
0x2891c650,"MainField_WeaponDrop_1734137296",
0x8bb7cb16,"MainField_WeaponDrop_1736834528",
0xd80cb76e,"MainField_WeaponDrop_173740639",
0x02969583,"MainField_WeaponDrop_1739062525",
0x38407425,"MainField_WeaponDrop_1741799751",
0x1e43d127,"MainField_WeaponDrop_1742091984",
0x1c772b66,"MainField_WeaponDrop_1743508121",
0x1dfbcfa7,"MainField_WeaponDrop_1744713586",
0xfff146d6,"MainField_WeaponDrop_1746021691",
0x0d7ef0ac,"MainField_WeaponDrop_174628736",
0x3d09a81f,"MainField_WeaponDrop_1748104237",
0x47a9a724,"MainField_WeaponDrop_1749371295",
0x0ad732a8,"MainField_WeaponDrop_1750285401",
0x71e61969,"MainField_WeaponDrop_1751837898",
0xc881cee4,"MainField_WeaponDrop_1753402347",
0xe86f5d82,"MainField_WeaponDrop_1753774584",
0xcd6527f5,"MainField_WeaponDrop_175564507",
0xc0eda2a1,"MainField_WeaponDrop_1755966861",
0x3bddce94,"MainField_WeaponDrop_1761364823",
0x7a983084,"MainField_WeaponDrop_1762986740",
0x9cd75708,"MainField_WeaponDrop_1763375814",
0x75cec2d9,"MainField_WeaponDrop_1766637772",
0x76e7e088,"MainField_WeaponDrop_1767798226",
0xb671dad9,"MainField_WeaponDrop_1768713804",
0x2409de06,"MainField_WeaponDrop_1769183353",
0x4f12616f,"MainField_WeaponDrop_1770967508",
0x49530a24,"MainField_WeaponDrop_1777496793",
0xf5701e52,"MainField_WeaponDrop_178649063",
0xf855915d,"MainField_WeaponDrop_1786502784",
0x0c02b359,"MainField_WeaponDrop_1790138682",
0x309f142b,"MainField_WeaponDrop_1793300653",
0xbb831a1e,"MainField_WeaponDrop_1793810292",
0xf56fba1d,"MainField_WeaponDrop_1796188467",
0x71e1b3d1,"MainField_WeaponDrop_1797016280",
0x5640a654,"MainField_WeaponDrop_1798092276",
0xa62c38f9,"MainField_WeaponDrop_1800631469",
0xf5f92d7f,"MainField_WeaponDrop_1803827115",
0x2cbeb544,"MainField_WeaponDrop_1805737410",
0xca993d66,"MainField_WeaponDrop_1808034356",
0x133929ca,"MainField_WeaponDrop_1808084475",
0x3659c870,"MainField_WeaponDrop_1809256376",
0xd8fabc68,"MainField_WeaponDrop_1815268906",
0xee5895bd,"MainField_WeaponDrop_1815483415",
0xb4253a42,"MainField_WeaponDrop_181612398",
0x424cd09b,"MainField_WeaponDrop_1817816749",
0xaa47bfa5,"MainField_WeaponDrop_1820735438",
0x514361f1,"MainField_WeaponDrop_1822233271",
0x84c13077,"MainField_WeaponDrop_1824306896",
0x046e4213,"MainField_WeaponDrop_1825124442",
0xb65dd595,"MainField_WeaponDrop_1825809252",
0x3168cd87,"MainField_WeaponDrop_1826871487",
0xf5a6ea68,"MainField_WeaponDrop_1828321815",
0x9b5fcc4a,"MainField_WeaponDrop_1828415426",
0x1cafe16d,"MainField_WeaponDrop_1828639238",
0x4a1a8f65,"MainField_WeaponDrop_1830839689",
0xec2cbe00,"MainField_WeaponDrop_1833120021",
0x4889a90f,"MainField_WeaponDrop_1840647294",
0x0ea2b842,"MainField_WeaponDrop_1841131066",
0x38892116,"MainField_WeaponDrop_1842002486",
0xce5ef361,"MainField_WeaponDrop_1842154516",
0x033dcf82,"MainField_WeaponDrop_1842698976",
0x7b50c897,"MainField_WeaponDrop_1845244796",
0x01b2112d,"MainField_WeaponDrop_1845356714",
0x64eddc15,"MainField_WeaponDrop_1845649274",
0x03ac01b1,"MainField_WeaponDrop_1847162151",
0xeabf4706,"MainField_WeaponDrop_1847314475",
0x9e544d04,"MainField_WeaponDrop_1847483488",
0xd662c131,"MainField_WeaponDrop_1849283355",
0xc6d6be51,"MainField_WeaponDrop_1849304801",
0xf59f77c0,"MainField_WeaponDrop_1849900848",
0xd7a11d1a,"MainField_WeaponDrop_185019670",
0xa49a11bd,"MainField_WeaponDrop_1852340499",
0x18ed3815,"MainField_WeaponDrop_1853862108",
0x980a65bd,"MainField_WeaponDrop_1854848864",
0xe69e54da,"MainField_WeaponDrop_1855094598",
0x396a03c6,"MainField_WeaponDrop_1857170048",
0x4d8c716a,"MainField_WeaponDrop_1860336463",
0xde917458,"MainField_WeaponDrop_1862793938",
0x97ca86d4,"MainField_WeaponDrop_186404819",
0x8d3ff824,"MainField_WeaponDrop_186670059",
0xc8f0a6c7,"MainField_WeaponDrop_1866749864",
0x280f6c3c,"MainField_WeaponDrop_1868231484",
0x0ed4b752,"MainField_WeaponDrop_1880999369",
0xefdb5559,"MainField_WeaponDrop_1883104002",
0x8bf5b12c,"MainField_WeaponDrop_1883353556",
0x1ced4130,"MainField_WeaponDrop_1885307869",
0x430ce30c,"MainField_WeaponDrop_1886488924",
0x1974f82d,"MainField_WeaponDrop_1890501329",
0x6b1cf810,"MainField_WeaponDrop_1892649614",
0x3713e4e6,"MainField_WeaponDrop_1897317715",
0x0df82c01,"MainField_WeaponDrop_1897657612",
0x82ad1305,"MainField_WeaponDrop_1898781307",
0x98a13835,"MainField_WeaponDrop_1899279996",
0x9853e9c7,"MainField_WeaponDrop_1902333944",
0x9fb2f2f5,"MainField_WeaponDrop_1904605396",
0x55c7cc44,"MainField_WeaponDrop_1906649873",
0x17860195,"MainField_WeaponDrop_1907800642",
0x8bfee6a1,"MainField_WeaponDrop_1908293531",
0xf2b35fcc,"MainField_WeaponDrop_1908735941",
0x4139880d,"MainField_WeaponDrop_1909203692",
0x5d021d80,"MainField_WeaponDrop_1912782624",
0x51e0340b,"MainField_WeaponDrop_1913314230",
0x54b76442,"MainField_WeaponDrop_191510122",
0x1c44dffe,"MainField_WeaponDrop_1915475430",
0x98aa3083,"MainField_WeaponDrop_1917352436",
0x17be5add,"MainField_WeaponDrop_1918424871",
0xa3fa40ef,"MainField_WeaponDrop_191916109",
0xf0f16ef2,"MainField_WeaponDrop_1919525600",
0xe0d8aea6,"MainField_WeaponDrop_1920337170",
0x58d21ef8,"MainField_WeaponDrop_192088109",
0x148e9e94,"MainField_WeaponDrop_1923775474",
0x735b5ec8,"MainField_WeaponDrop_1924033738",
0x2100b5c7,"MainField_WeaponDrop_192559341",
0x9e17698a,"MainField_WeaponDrop_1927188256",
0xcf20c235,"MainField_WeaponDrop_1928348481",
0xa5ec9597,"MainField_WeaponDrop_1929989223",
0x3e87ee69,"MainField_WeaponDrop_1933128425",
0xf5b47bf0,"MainField_WeaponDrop_1934931253",
0xb1640cc7,"MainField_WeaponDrop_1935756369",
0xde5c4737,"MainField_WeaponDrop_1938893969",
0xd8e934c4,"MainField_WeaponDrop_1940439370",
0x7ad9cdad,"MainField_WeaponDrop_1946959066",
0x6f78a72d,"MainField_WeaponDrop_1951412392",
0xf069658f,"MainField_WeaponDrop_1951747422",
0x48ebf16e,"MainField_WeaponDrop_1952837341",
0xa680c1c4,"MainField_WeaponDrop_1953264438",
0x7084cd92,"MainField_WeaponDrop_1956979855",
0x898731b1,"MainField_WeaponDrop_1957278813",
0x9bba1c3f,"MainField_WeaponDrop_1960568416",
0xa796dd1c,"MainField_WeaponDrop_1962291562",
0xb64154e9,"MainField_WeaponDrop_196362379",
0x23272872,"MainField_WeaponDrop_1967190464",
0x9d811615,"MainField_WeaponDrop_196781734",
0xfdb31f54,"MainField_WeaponDrop_1972193600",
0xa3de4fd3,"MainField_WeaponDrop_1977324499",
0xcb013cea,"MainField_WeaponDrop_1982589834",
0xa72faa20,"MainField_WeaponDrop_1982843887",
0x6d0fab10,"MainField_WeaponDrop_1983007691",
0x00da1d17,"MainField_WeaponDrop_1985877316",
0xe6eccb25,"MainField_WeaponDrop_1986366784",
0x70eee2de,"MainField_WeaponDrop_198804540",
0x1eb8b1f1,"MainField_WeaponDrop_1990468811",
0x081e4506,"MainField_WeaponDrop_1992271001",
0xe1dd48fa,"MainField_WeaponDrop_1992809093",
0x9122f1c1,"MainField_WeaponDrop_1993787485",
0x0e93e865,"MainField_WeaponDrop_1994007080",
0x8e1d78e5,"MainField_WeaponDrop_1996412574",
0x1dc4144a,"MainField_WeaponDrop_1996844321",
0x27b23bf7,"MainField_WeaponDrop_1998405018",
0xeb87209d,"MainField_WeaponDrop_1999779604",
0x8ae6a35b,"MainField_WeaponDrop_2000020259",
0x889bf3e2,"MainField_WeaponDrop_2000256890",
0x84036975,"MainField_WeaponDrop_2000417074",
0x1c2c82eb,"MainField_WeaponDrop_2003312832",
0x24bb2131,"MainField_WeaponDrop_2010375149",
0xd6f9922c,"MainField_WeaponDrop_2010392263",
0x3d1597c8,"MainField_WeaponDrop_2013579223",
0xe27e0170,"MainField_WeaponDrop_2014415266",
0x7dd54099,"MainField_WeaponDrop_2014895848",
0x54a8605b,"MainField_WeaponDrop_2015083349",
0x876a32c9,"MainField_WeaponDrop_2017199432",
0x775544ee,"MainField_WeaponDrop_2018115166",
0xdd120ee4,"MainField_WeaponDrop_2020294572",
0xa4ba8863,"MainField_WeaponDrop_2022306038",
0x101ab493,"MainField_WeaponDrop_2034362192",
0x69fec3fc,"MainField_WeaponDrop_2034851750",
0x7fe64db6,"MainField_WeaponDrop_2036150519",
0x9bb0e78e,"MainField_WeaponDrop_2041897679",
0x34bac1cf,"MainField_WeaponDrop_2043571034",
0xdf533136,"MainField_WeaponDrop_2044643248",
0xaa99c1ff,"MainField_WeaponDrop_2050629132",
0xb230b568,"MainField_WeaponDrop_2051771833",
0xd5a75106,"MainField_WeaponDrop_2053061311",
0x60ca3cb6,"MainField_WeaponDrop_2054541079",
0x7912b8a4,"MainField_WeaponDrop_2054599949",
0x229db628,"MainField_WeaponDrop_2055079404",
0x79a16f15,"MainField_WeaponDrop_2055094118",
0xe93461d0,"MainField_WeaponDrop_2056815794",
0xea60379d,"MainField_WeaponDrop_2057283343",
0x86bc79c1,"MainField_WeaponDrop_2057560765",
0x5dd57ef6,"MainField_WeaponDrop_2059498382",
0x25e62013,"MainField_WeaponDrop_2062111936",
0x1575f9b7,"MainField_WeaponDrop_2062214693",
0x325e5a52,"MainField_WeaponDrop_2063793320",
0xbcfd3e6e,"MainField_WeaponDrop_2065166989",
0xe91673fa,"MainField_WeaponDrop_2066088197",
0xf41db789,"MainField_WeaponDrop_2067739586",
0x70a7b356,"MainField_WeaponDrop_2068825377",
0x61800204,"MainField_WeaponDrop_2074604875",
0xc5ea2b3a,"MainField_WeaponDrop_2078192644",
0xda7669cf,"MainField_WeaponDrop_2078958192",
0x07bf24c3,"MainField_WeaponDrop_2083791362",
0xc5083add,"MainField_WeaponDrop_2083797768",
0x75152dda,"MainField_WeaponDrop_2086079447",
0xbc1708fb,"MainField_WeaponDrop_2087614592",
0xd8c52a2b,"MainField_WeaponDrop_2087762100",
0x25316a3d,"MainField_WeaponDrop_2087991483",
0xaadfdec2,"MainField_WeaponDrop_2090677285",
0x30a42e6e,"MainField_WeaponDrop_2092134429",
0x0067caf2,"MainField_WeaponDrop_2093758563",
0xa78df973,"MainField_WeaponDrop_2096714846",
0x4a2ac171,"MainField_WeaponDrop_209694102",
0x568b9d67,"MainField_WeaponDrop_2097580945",
0xe193b2f2,"MainField_WeaponDrop_2098083749",
0x6e098de9,"MainField_WeaponDrop_2099885743",
0x43dea5be,"MainField_WeaponDrop_2100512444",
0x1d76c786,"MainField_WeaponDrop_2100520590",
0x5791c24a,"MainField_WeaponDrop_2100709577",
0x036ee998,"MainField_WeaponDrop_2104838801",
0xe4af1c70,"MainField_WeaponDrop_2105510824",
0x907e9926,"MainField_WeaponDrop_2108018285",
0x119470a8,"MainField_WeaponDrop_2108589222",
0x8d9a0556,"MainField_WeaponDrop_2114283371",
0xc5e9b6f4,"MainField_WeaponDrop_211559169",
0x23fc9ef2,"MainField_WeaponDrop_2120870626",
0xdbfe3309,"MainField_WeaponDrop_212274056",
0x058d442f,"MainField_WeaponDrop_2125177745",
0xabb4fb97,"MainField_WeaponDrop_2127113634",
0xc485d16b,"MainField_WeaponDrop_2128233397",
0xa63002bd,"MainField_WeaponDrop_2131165841",
0xd130eef9,"MainField_WeaponDrop_2131371552",
0x27eeef6b,"MainField_WeaponDrop_213147214",
0xd33e4369,"MainField_WeaponDrop_2133989757",
0xfe831234,"MainField_WeaponDrop_2134643683",
0xcfd1276e,"MainField_WeaponDrop_2136066511",
0x01735cd3,"MainField_WeaponDrop_2136845834",
0x0e5ac16c,"MainField_WeaponDrop_2137535942",
0x194b1183,"MainField_WeaponDrop_2138612159",
0x2003401f,"MainField_WeaponDrop_2141826487",
0xfa8e965e,"MainField_WeaponDrop_2144238695",
0xeef84b2a,"MainField_WeaponDrop_2145478520",
0xed267b3d,"MainField_WeaponDrop_2146143232",
0x5b5ac972,"MainField_WeaponDrop_2146942138",
0x90e49c5b,"MainField_WeaponDrop_2148547916",
0x805c3113,"MainField_WeaponDrop_2151389425",
0xf7db22f3,"MainField_WeaponDrop_2151832912",
0x4a213fb2,"MainField_WeaponDrop_2152361388",
0xb35b80cd,"MainField_WeaponDrop_2153855979",
0x6eb031e7,"MainField_WeaponDrop_2155229726",
0xa6b7fe3c,"MainField_WeaponDrop_215581842",
0xf0e0b446,"MainField_WeaponDrop_2158573956",
0xb6aa18d5,"MainField_WeaponDrop_2159613305",
0xc8515972,"MainField_WeaponDrop_2162445582",
0x10900282,"MainField_WeaponDrop_2162556710",
0xc757b656,"MainField_WeaponDrop_2166779339",
0xb79164cf,"MainField_WeaponDrop_2168617790",
0x404c48d6,"MainField_WeaponDrop_217659729",
0x2ed03343,"MainField_WeaponDrop_2176664501",
0x25555031,"MainField_WeaponDrop_2180713761",
0x9c08f98e,"MainField_WeaponDrop_2181188996",
0x9f6dbffd,"MainField_WeaponDrop_2182987881",
0xb42b8760,"MainField_WeaponDrop_218991381",
0x68a3218b,"MainField_WeaponDrop_219076688",
0xd4daa848,"MainField_WeaponDrop_2192906181",
0x626789e6,"MainField_WeaponDrop_2197224257",
0x45750851,"MainField_WeaponDrop_2202720944",
0xb6fa33c5,"MainField_WeaponDrop_2203005248",
0x5f740075,"MainField_WeaponDrop_2203036677",
0x95a34bff,"MainField_WeaponDrop_2207405265",
0xe4494368,"MainField_WeaponDrop_2207458819",
0x910cfc38,"MainField_WeaponDrop_2207749110",
0xc1c1e30d,"MainField_WeaponDrop_221144016",
0xf9632400,"MainField_WeaponDrop_2212108301",
0x538dba54,"MainField_WeaponDrop_2212841258",
0x68f238ab,"MainField_WeaponDrop_2219770993",
0xdc8fb412,"MainField_WeaponDrop_2220233468",
0x79e51c65,"MainField_WeaponDrop_2220778917",
0x1de9df13,"MainField_WeaponDrop_2227521077",
0x9c4a86fa,"MainField_WeaponDrop_2228793224",
0xc98b3cbe,"MainField_WeaponDrop_2229237587",
0xda3cc610,"MainField_WeaponDrop_223282845",
0x04b85dc7,"MainField_WeaponDrop_2237470081",
0x822f5f01,"MainField_WeaponDrop_2239269747",
0x39b6e23e,"MainField_WeaponDrop_224074490",
0x8567ce6a,"MainField_WeaponDrop_2241197830",
0xfbca30c9,"MainField_WeaponDrop_2241228943",
0x2f7be0e4,"MainField_WeaponDrop_2241546783",
0x52ec166e,"MainField_WeaponDrop_2241591825",
0xdb9d3f06,"MainField_WeaponDrop_2247680276",
0xc6913836,"MainField_WeaponDrop_2249931094",
0xd4a56f02,"MainField_WeaponDrop_225233466",
0xdf6a6a57,"MainField_WeaponDrop_2259632802",
0x55abbbb9,"MainField_WeaponDrop_226047354",
0xf4c802c6,"MainField_WeaponDrop_2265198937",
0xad856128,"MainField_WeaponDrop_2266274072",
0x25752746,"MainField_WeaponDrop_2271554321",
0x4691f09d,"MainField_WeaponDrop_2272172473",
0x58721aaa,"MainField_WeaponDrop_2273179989",
0x733c85c9,"MainField_WeaponDrop_2274298077",
0xaf3bd1da,"MainField_WeaponDrop_2274877029",
0x3a3ecd8f,"MainField_WeaponDrop_2281631889",
0x49868505,"MainField_WeaponDrop_2284309825",
0xa0ae4f68,"MainField_WeaponDrop_2285138637",
0x3ffd482f,"MainField_WeaponDrop_228712591",
0x13c6952f,"MainField_WeaponDrop_2288117225",
0x4fcd9d09,"MainField_WeaponDrop_2296281886",
0xac49e2a9,"MainField_WeaponDrop_2299809648",
0x2eb86792,"MainField_WeaponDrop_2300175972",
0x2ae6aa58,"MainField_WeaponDrop_2301577128",
0x9e17e9a8,"MainField_WeaponDrop_2302768252",
0xcfd53878,"MainField_WeaponDrop_2304593766",
0x2d0166d9,"MainField_WeaponDrop_2305790095",
0x939a3a53,"MainField_WeaponDrop_2307325139",
0xdbb30627,"MainField_WeaponDrop_2309224492",
0x18de9249,"MainField_WeaponDrop_2311097034",
0xcf73ed04,"MainField_WeaponDrop_2311411558",
0x79cfecac,"MainField_WeaponDrop_2311530784",
0x9ef6881a,"MainField_WeaponDrop_2317363278",
0xf169c997,"MainField_WeaponDrop_2317415999",
0xc921f146,"MainField_WeaponDrop_231871219",
0xfbb70928,"MainField_WeaponDrop_2321048357",
0xf7684388,"MainField_WeaponDrop_2322393558",
0x6a7bc297,"MainField_WeaponDrop_2326362082",
0xd37ed685,"MainField_WeaponDrop_2327009806",
0x949c42df,"MainField_WeaponDrop_2328299118",
0x601ff5a9,"MainField_WeaponDrop_2329875645",
0xdf2903de,"MainField_WeaponDrop_2329894876",
0x2f4200e6,"MainField_WeaponDrop_2330103885",
0x7093aaed,"MainField_WeaponDrop_2331614943",
0xf4fe441c,"MainField_WeaponDrop_2332108134",
0xb98fd7fe,"MainField_WeaponDrop_2337498919",
0xcd135002,"MainField_WeaponDrop_2342281826",
0x5df64834,"MainField_WeaponDrop_2346821368",
0x6023517e,"MainField_WeaponDrop_2347608143",
0x1bfdc95d,"MainField_WeaponDrop_2349472725",
0x62de4979,"MainField_WeaponDrop_2350777073",
0xe563fe69,"MainField_WeaponDrop_2356207998",
0xb50f394e,"MainField_WeaponDrop_2357034192",
0xd9be6a2e,"MainField_WeaponDrop_2358441937",
0xf9d6670d,"MainField_WeaponDrop_2358552805",
0xd82893f5,"MainField_WeaponDrop_235966606",
0xef3d9f11,"MainField_WeaponDrop_2360887076",
0x95db5e9a,"MainField_WeaponDrop_2364792377",
0x8d07b116,"MainField_WeaponDrop_2365382684",
0x5dbdd435,"MainField_WeaponDrop_236961388",
0xfb4be320,"MainField_WeaponDrop_236982467",
0xcfd56a58,"MainField_WeaponDrop_2371846756",
0x4fdbd2c5,"MainField_WeaponDrop_2371913686",
0x0cd0c901,"MainField_WeaponDrop_2372621697",
0xcfa7d8ad,"MainField_WeaponDrop_2372794855",
0x32ecadda,"MainField_WeaponDrop_2373123040",
0xd174d8ef,"MainField_WeaponDrop_2373280731",
0x03e3680a,"MainField_WeaponDrop_2377042535",
0x0807b9db,"MainField_WeaponDrop_2380115926",
0x2888161a,"MainField_WeaponDrop_2382087900",
0x781997f6,"MainField_WeaponDrop_2382722075",
0xbd4aa1cb,"MainField_WeaponDrop_2383034335",
0x038d4878,"MainField_WeaponDrop_2384048126",
0x25f007b5,"MainField_WeaponDrop_2386137374",
0x44ede5b3,"MainField_WeaponDrop_2386225924",
0x4bc167a1,"MainField_WeaponDrop_2387082804",
0x78bbe39c,"MainField_WeaponDrop_2389254047",
0x34983ad3,"MainField_WeaponDrop_2394648416",
0x28566ad4,"MainField_WeaponDrop_2394988098",
0xfdbd4d8d,"MainField_WeaponDrop_2396315136",
0x9547b9af,"MainField_WeaponDrop_2402333550",
0x1d6f5b8c,"MainField_WeaponDrop_2404622425",
0x34981ed9,"MainField_WeaponDrop_2404963998",
0xcdfdf8d2,"MainField_WeaponDrop_2407415524",
0x0e40a45b,"MainField_WeaponDrop_2407650905",
0x7c689189,"MainField_WeaponDrop_2409215851",
0x7c133e96,"MainField_WeaponDrop_2409231116",
0x44ae6628,"MainField_WeaponDrop_2411315904",
0xee3c65bc,"MainField_WeaponDrop_241160302",
0xfd42924c,"MainField_WeaponDrop_2412300446",
0x6901a7b9,"MainField_WeaponDrop_2412842669",
0x4da43cc8,"MainField_WeaponDrop_2416373648",
0x0413c7b5,"MainField_WeaponDrop_2416641607",
0x12a4e824,"MainField_WeaponDrop_2418124650",
0x5b004f53,"MainField_WeaponDrop_241891525",
0x4320ae7b,"MainField_WeaponDrop_2421122118",
0x7ea57a3b,"MainField_WeaponDrop_2423834846",
0x1f164a27,"MainField_WeaponDrop_2423914139",
0x4dde9d2f,"MainField_WeaponDrop_2424153178",
0xa672e9c9,"MainField_WeaponDrop_2424261588",
0xbd8e6f66,"MainField_WeaponDrop_2429643116",
0x385fb730,"MainField_WeaponDrop_2431283690",
0x330ef0c4,"MainField_WeaponDrop_2435395812",
0xcd9ec579,"MainField_WeaponDrop_2440227174",
0x87283c54,"MainField_WeaponDrop_2449254871",
0x37615b5a,"MainField_WeaponDrop_2449366576",
0xc68f60d6,"MainField_WeaponDrop_2452163153",
0xc3239e0a,"MainField_WeaponDrop_2452286406",
0xbad607e8,"MainField_WeaponDrop_2455660787",
0xead57695,"MainField_WeaponDrop_2456694075",
0xc8dfe9d4,"MainField_WeaponDrop_2457491863",
0x4ab04e8c,"MainField_WeaponDrop_2457667199",
0xdaaa1011,"MainField_WeaponDrop_2458121559",
0x61d881dd,"MainField_WeaponDrop_2462310465",
0x9d94b136,"MainField_WeaponDrop_2464594577",
0xd3f0b29e,"MainField_WeaponDrop_2465499500",
0x2e67291e,"MainField_WeaponDrop_2466735693",
0xdce5d3b9,"MainField_WeaponDrop_2466875362",
0xe26168a6,"MainField_WeaponDrop_2468192473",
0xce4e9c5e,"MainField_WeaponDrop_247283217",
0xc753900e,"MainField_WeaponDrop_2473301115",
0x5aa38ec0,"MainField_WeaponDrop_2475693344",
0x903d838d,"MainField_WeaponDrop_2478711707",
0xa9d94e5e,"MainField_WeaponDrop_248077260",
0x4126bce2,"MainField_WeaponDrop_2480879310",
0xef8eda5f,"MainField_WeaponDrop_2483159598",
0x2a721ce3,"MainField_WeaponDrop_2483526615",
0xd15a7706,"MainField_WeaponDrop_2488340106",
0x9468bbc4,"MainField_WeaponDrop_2488560908",
0x16185142,"MainField_WeaponDrop_2490393792",
0x906aeb84,"MainField_WeaponDrop_2490523841",
0x3853b46a,"MainField_WeaponDrop_2493361451",
0x3320fec7,"MainField_WeaponDrop_249470919",
0x6011be18,"MainField_WeaponDrop_2494886894",
0x7b9f5aa3,"MainField_WeaponDrop_2504604148",
0x3772a6b7,"MainField_WeaponDrop_2506784697",
0x41ad3c38,"MainField_WeaponDrop_2510712385",
0xbae0b76c,"MainField_WeaponDrop_251481765",
0xfd96e885,"MainField_WeaponDrop_2515284034",
0x6c22264c,"MainField_WeaponDrop_2515364563",
0xf88a0a3b,"MainField_WeaponDrop_2515490370",
0xa300c386,"MainField_WeaponDrop_2519208480",
0xd75c7707,"MainField_WeaponDrop_2519986759",
0x0ab42aa3,"MainField_WeaponDrop_2522581781",
0xab2cc5c8,"MainField_WeaponDrop_2522624653",
0x5437cb1c,"MainField_WeaponDrop_2528741463",
0xa1ff471e,"MainField_WeaponDrop_2534934817",
0xeb1464c2,"MainField_WeaponDrop_2535815944",
0xb61de7be,"MainField_WeaponDrop_2536925146",
0x2b702bdd,"MainField_WeaponDrop_2537120311",
0x478a024e,"MainField_WeaponDrop_2539282896",
0xd95eb436,"MainField_WeaponDrop_253979430",
0x3113c30d,"MainField_WeaponDrop_2540019497",
0xd0259e91,"MainField_WeaponDrop_2540756414",
0xe1e09879,"MainField_WeaponDrop_2544652103",
0x81d244ce,"MainField_WeaponDrop_2547910419",
0x59067fcc,"MainField_WeaponDrop_2548336985",
0x44f9db08,"MainField_WeaponDrop_2551172536",
0x4807915a,"MainField_WeaponDrop_2554891418",
0x5a8e35c0,"MainField_WeaponDrop_2555201904",
0xef64d7cf,"MainField_WeaponDrop_2555988607",
0xdfec2d59,"MainField_WeaponDrop_2556820890",
0xad6b2b5d,"MainField_WeaponDrop_2556908994",
0x4d4735b0,"MainField_WeaponDrop_2560697258",
0x2ec1722b,"MainField_WeaponDrop_2560840429",
0x61ffc253,"MainField_WeaponDrop_2563776095",
0xe647c0b8,"MainField_WeaponDrop_2565752333",
0x3e027897,"MainField_WeaponDrop_2567697880",
0x3753b49a,"MainField_WeaponDrop_2569750155",
0x89ecba1d,"MainField_WeaponDrop_2570684564",
0x5bdd9387,"MainField_WeaponDrop_2571443871",
0xd5b08535,"MainField_WeaponDrop_2572492167",
0x9e623378,"MainField_WeaponDrop_2574081664",
0xb96e33c8,"MainField_WeaponDrop_2576475601",
0x52fde5b6,"MainField_WeaponDrop_2576477679",
0x1714bdb2,"MainField_WeaponDrop_2577879478",
0xe45b1960,"MainField_WeaponDrop_2579319404",
0x6f9c0cb2,"MainField_WeaponDrop_2582104290",
0x629c253b,"MainField_WeaponDrop_2583242861",
0x8ff80298,"MainField_WeaponDrop_2583808247",
0xf9bf14ed,"MainField_WeaponDrop_2584566407",
0xf240bed2,"MainField_WeaponDrop_2584719279",
0x07ba185a,"MainField_WeaponDrop_2584776018",
0xcb974489,"MainField_WeaponDrop_2586600814",
0x47c6ff0b,"MainField_WeaponDrop_2588026829",
0x0926f934,"MainField_WeaponDrop_2588488396",
0xa4a68fe6,"MainField_WeaponDrop_2591487752",
0x9043cd18,"MainField_WeaponDrop_2592356580",
0x13635f32,"MainField_WeaponDrop_2595221715",
0xc162f31a,"MainField_WeaponDrop_2598032933",
0x0ffa9663,"MainField_WeaponDrop_2598930816",
0x2cc08c45,"MainField_WeaponDrop_2618908433",
0xfdba7cc0,"MainField_WeaponDrop_2620139367",
0xde38ddaf,"MainField_WeaponDrop_262021965",
0xe24d2bf8,"MainField_WeaponDrop_2621842624",
0x56b30802,"MainField_WeaponDrop_2622397745",
0xcfb44340,"MainField_WeaponDrop_262330296",
0xad0c3949,"MainField_WeaponDrop_2633026988",
0x27616e2f,"MainField_WeaponDrop_2635016419",
0x15655233,"MainField_WeaponDrop_2635473101",
0x221bb3ea,"MainField_WeaponDrop_2635976784",
0xf0a84687,"MainField_WeaponDrop_2645769793",
0x8ed84d42,"MainField_WeaponDrop_2649015914",
0x4e5d39c7,"MainField_WeaponDrop_2649969008",
0x5634f23d,"MainField_WeaponDrop_265035073",
0x7ca3d8ab,"MainField_WeaponDrop_2650759514",
0x90c7b818,"MainField_WeaponDrop_2652312798",
0xcb03711e,"MainField_WeaponDrop_2655449625",
0xf128fa29,"MainField_WeaponDrop_2656111645",
0x01527f5b,"MainField_WeaponDrop_2660121770",
0x8fbee619,"MainField_WeaponDrop_2662885233",
0x0a4810d7,"MainField_WeaponDrop_2669828246",
0x20fde76d,"MainField_WeaponDrop_2671792437",
0x7824d293,"MainField_WeaponDrop_2672569532",
0x41506912,"MainField_WeaponDrop_2676677439",
0x0693636e,"MainField_WeaponDrop_2683160382",
0x0c7af7de,"MainField_WeaponDrop_2684670267",
0x20129f5f,"MainField_WeaponDrop_2689492977",
0xc106a98a,"MainField_WeaponDrop_2691348393",
0x77bf52cb,"MainField_WeaponDrop_269531102",
0x49f25b10,"MainField_WeaponDrop_2697466486",
0xc85f2c2b,"MainField_WeaponDrop_2699268884",
0xcd6a1f76,"MainField_WeaponDrop_2700762744",
0x732a3f0b,"MainField_WeaponDrop_2701290359",
0x78d03cc3,"MainField_WeaponDrop_2701718099",
0xa1a5170d,"MainField_WeaponDrop_2703454877",
0xd0aa0028,"MainField_WeaponDrop_2703888940",
0x850b77a3,"MainField_WeaponDrop_2705502562",
0x0edff9b0,"MainField_WeaponDrop_2705553678",
0x845f857a,"MainField_WeaponDrop_2708071169",
0xb19ab71e,"MainField_WeaponDrop_2710276379",
0x1cd8f85d,"MainField_WeaponDrop_2713400006",
0x5e83b9bd,"MainField_WeaponDrop_2716167837",
0x56b7c087,"MainField_WeaponDrop_2717979786",
0x3852d4f8,"MainField_WeaponDrop_2720230453",
0x1c28dcbf,"MainField_WeaponDrop_2722771774",
0x04c76504,"MainField_WeaponDrop_2723155190",
0x20b71a5b,"MainField_WeaponDrop_2723559686",
0x2195cd0a,"MainField_WeaponDrop_2724363349",
0x1767682a,"MainField_WeaponDrop_2730661306",
0x13d272d4,"MainField_WeaponDrop_2731514393",
0xe8672fdc,"MainField_WeaponDrop_273212448",
0xaa2afdd6,"MainField_WeaponDrop_2732482086",
0xed99a082,"MainField_WeaponDrop_2734324001",
0xe8f84265,"MainField_WeaponDrop_2735075261",
0xdd8d790d,"MainField_WeaponDrop_273617847",
0x7e7f5a47,"MainField_WeaponDrop_2737055335",
0x1cec6f8c,"MainField_WeaponDrop_2740548315",
0x4c00647f,"MainField_WeaponDrop_2746629006",
0x38b6db24,"MainField_WeaponDrop_274706856",
0x79965750,"MainField_WeaponDrop_2748623453",
0x127c1d2d,"MainField_WeaponDrop_2750036023",
0x2d29707b,"MainField_WeaponDrop_2756246755",
0xfab8b9ac,"MainField_WeaponDrop_2761426384",
0x1711895f,"MainField_WeaponDrop_2762023785",
0xf47f6027,"MainField_WeaponDrop_2762580060",
0x1fef17f3,"MainField_WeaponDrop_277007734",
0x6238d422,"MainField_WeaponDrop_2772297089",
0x048af998,"MainField_WeaponDrop_2773442511",
0x34ee82ae,"MainField_WeaponDrop_2777108519",
0x8bf99d1f,"MainField_WeaponDrop_2779839064",
0xec2e719e,"MainField_WeaponDrop_2779855843",
0x0fa154b4,"MainField_WeaponDrop_2780613885",
0x5ba3a58d,"MainField_WeaponDrop_2781029781",
0xa2d977a9,"MainField_WeaponDrop_2782725661",
0x3ec54b02,"MainField_WeaponDrop_2786679204",
0xd207cda8,"MainField_WeaponDrop_2787107504",
0x5d9762e7,"MainField_WeaponDrop_2787713853",
0xc43e50f5,"MainField_WeaponDrop_278887622",
0x686cc2a0,"MainField_WeaponDrop_2789606978",
0x0fa9d521,"MainField_WeaponDrop_2791380456",
0x501933c7,"MainField_WeaponDrop_2793393605",
0xb8f175a8,"MainField_WeaponDrop_2793433267",
0x25c752dc,"MainField_WeaponDrop_2793754688",
0x65490df5,"MainField_WeaponDrop_2795258147",
0x51eb0bf2,"MainField_WeaponDrop_2795938130",
0x5cf5287e,"MainField_WeaponDrop_2802164270",
0x074b93aa,"MainField_WeaponDrop_2804071037",
0x3bd9be2f,"MainField_WeaponDrop_2807954694",
0xc8adf3c6,"MainField_WeaponDrop_2809275598",
0x625e186f,"MainField_WeaponDrop_2810539746",
0xef2e32bb,"MainField_WeaponDrop_2810773673",
0xf65747da,"MainField_WeaponDrop_2812793057",
0xc6e57dca,"MainField_WeaponDrop_2816103143",
0xa178c355,"MainField_WeaponDrop_2817263124",
0x4ef75e6b,"MainField_WeaponDrop_2817900430",
0x7bf2a92d,"MainField_WeaponDrop_2819972064",
0xfafa5326,"MainField_WeaponDrop_2821676041",
0xc1d8d56f,"MainField_WeaponDrop_2824562574",
0xba718d1a,"MainField_WeaponDrop_2826006316",
0xcf1238c2,"MainField_WeaponDrop_2828151193",
0x87342546,"MainField_WeaponDrop_2830622521",
0x02b35e8b,"MainField_WeaponDrop_2832391619",
0xa8d6b271,"MainField_WeaponDrop_2836361466",
0xdf2d0a90,"MainField_WeaponDrop_2837608684",
0x803b0f2d,"MainField_WeaponDrop_2839584475",
0x3461bfeb,"MainField_WeaponDrop_2839887834",
0xc40f1593,"MainField_WeaponDrop_2840820856",
0x2682682d,"MainField_WeaponDrop_2840846277",
0x0a59e2a2,"MainField_WeaponDrop_2840888026",
0xffbd2516,"MainField_WeaponDrop_2843540927",
0xece02068,"MainField_WeaponDrop_2844653022",
0x2cd47700,"MainField_WeaponDrop_2847933787",
0x3f0e0cbb,"MainField_WeaponDrop_2850062014",
0x646dd019,"MainField_WeaponDrop_2850629881",
0xc8c23b8b,"MainField_WeaponDrop_2851155964",
0xca6c34bd,"MainField_WeaponDrop_2851504962",
0x694614c8,"MainField_WeaponDrop_2855243552",
0xcb28c494,"MainField_WeaponDrop_2856575650",
0x789c0e3a,"MainField_WeaponDrop_2857728549",
0xdce05bf0,"MainField_WeaponDrop_285993100",
0x9573894b,"MainField_WeaponDrop_2864246505",
0x6d20dce7,"MainField_WeaponDrop_2865036194",
0x3a645843,"MainField_WeaponDrop_2865948947",
0x653999a4,"MainField_WeaponDrop_2866811557",
0x6f45326b,"MainField_WeaponDrop_2867408805",
0x4e3ad36f,"MainField_WeaponDrop_2872737235",
0xde195780,"MainField_WeaponDrop_2874660024",
0x888c83cb,"MainField_WeaponDrop_2875989112",
0x4853c483,"MainField_WeaponDrop_2877674565",
0xf084f3c7,"MainField_WeaponDrop_2878454987",
0xa683a937,"MainField_WeaponDrop_2879641406",
0xd0d8ef8f,"MainField_WeaponDrop_2882988859",
0x0c827922,"MainField_WeaponDrop_2884178422",
0xd3020fc0,"MainField_WeaponDrop_2885168107",
0x83c74ae7,"MainField_WeaponDrop_2885807471",
0xc9f94a50,"MainField_WeaponDrop_2887317037",
0x8593daa5,"MainField_WeaponDrop_2889069050",
0x0f30ac8b,"MainField_WeaponDrop_2889283611",
0xfb210095,"MainField_WeaponDrop_2890126292",
0x0a2a7fd7,"MainField_WeaponDrop_2890187471",
0xf93ebe5a,"MainField_WeaponDrop_2892933474",
0x2829abad,"MainField_WeaponDrop_2893303322",
0x5855387d,"MainField_WeaponDrop_2894370832",
0x6613f9c0,"MainField_WeaponDrop_2895186566",
0xcfd63cd6,"MainField_WeaponDrop_289523826",
0xde130c54,"MainField_WeaponDrop_2895512918",
0x7bd6d69c,"MainField_WeaponDrop_2901002269",
0xf8868d5e,"MainField_WeaponDrop_290332106",
0x6c0b21d5,"MainField_WeaponDrop_2904374750",
0x81165355,"MainField_WeaponDrop_2904546599",
0x2c9e7561,"MainField_WeaponDrop_2904887688",
0x689448d9,"MainField_WeaponDrop_2909342905",
0x87f3ee9a,"MainField_WeaponDrop_2909479841",
0x8ea3ab17,"MainField_WeaponDrop_2909550894",
0x24ed68b3,"MainField_WeaponDrop_2910364024",
0x3773d8d6,"MainField_WeaponDrop_2912318329",
0x5366896b,"MainField_WeaponDrop_2915271898",
0x2b74a7e8,"MainField_WeaponDrop_2917425872",
0xf7c023dd,"MainField_WeaponDrop_2918382493",
0x969aa9d6,"MainField_WeaponDrop_2919267280",
0x12caeabf,"MainField_WeaponDrop_2919299134",
0xa952691f,"MainField_WeaponDrop_2919738379",
0x4f8194e7,"MainField_WeaponDrop_2920932952",
0x61b58d6c,"MainField_WeaponDrop_2921995136",
0xc93c177c,"MainField_WeaponDrop_2924566726",
0x9b8064a8,"MainField_WeaponDrop_2925833015",
0x16d2ada7,"MainField_WeaponDrop_2931129991",
0x1e62a041,"MainField_WeaponDrop_2931830636",
0xdc9d7385,"MainField_WeaponDrop_2932631985",
0x0a688b15,"MainField_WeaponDrop_293284437",
0x8c7e0dd2,"MainField_WeaponDrop_2935212090",
0xadfd2797,"MainField_WeaponDrop_2935242096",
0x3f9a9a1d,"MainField_WeaponDrop_2944571197",
0xe39d354e,"MainField_WeaponDrop_2947262756",
0x5da29dc9,"MainField_WeaponDrop_2947386777",
0xba491ace,"MainField_WeaponDrop_2949696800",
0xa846efbf,"MainField_WeaponDrop_2951352812",
0x30b35c97,"MainField_WeaponDrop_2951738210",
0xef269059,"MainField_WeaponDrop_2962544063",
0x0d9cf172,"MainField_WeaponDrop_2963523003",
0x031c3734,"MainField_WeaponDrop_2966343952",
0x72ba5805,"MainField_WeaponDrop_2968358968",
0x0c0c4148,"MainField_WeaponDrop_2968634266",
0xbfc6ba31,"MainField_WeaponDrop_2969757916",
0x67d2dfbb,"MainField_WeaponDrop_2972348245",
0xfdf5f944,"MainField_WeaponDrop_2973151238",
0xf0f6bb3b,"MainField_WeaponDrop_2973686139",
0xedd03b95,"MainField_WeaponDrop_2975158469",
0xee17d141,"MainField_WeaponDrop_2976268698",
0x657bea2b,"MainField_WeaponDrop_2980321199",
0xc09ae5db,"MainField_WeaponDrop_2981731551",
0x2cc47970,"MainField_WeaponDrop_298345565",
0x837a065f,"MainField_WeaponDrop_2987111013",
0xa5f4f32d,"MainField_WeaponDrop_2987219153",
0x32b877e1,"MainField_WeaponDrop_298823816",
0xeb12967e,"MainField_WeaponDrop_2993701675",
0xca4f6409,"MainField_WeaponDrop_299681209",
0xc791878d,"MainField_WeaponDrop_3003292851",
0xfe9d23e4,"MainField_WeaponDrop_3004121714",
0xfa04ae3e,"MainField_WeaponDrop_3006294729",
0xd6d31300,"MainField_WeaponDrop_3006446906",
0xa4f0343d,"MainField_WeaponDrop_3009049476",
0x06139685,"MainField_WeaponDrop_3009274147",
0x6ea7d28d,"MainField_WeaponDrop_3010857459",
0x5c28e92a,"MainField_WeaponDrop_3012855551",
0x7c530d3b,"MainField_WeaponDrop_3014511817",
0xb85e4fe2,"MainField_WeaponDrop_301505455",
0xef473816,"MainField_WeaponDrop_3016140340",
0xb79792ec,"MainField_WeaponDrop_301819567",
0x4c3882d1,"MainField_WeaponDrop_3019340409",
0xa7042d7b,"MainField_WeaponDrop_3022555288",
0x455c5027,"MainField_WeaponDrop_3022572588",
0xc32e9d70,"MainField_WeaponDrop_3023415592",
0x898323c6,"MainField_WeaponDrop_3026695031",
0x39e89cc9,"MainField_WeaponDrop_3026910449",
0x1c1071e9,"MainField_WeaponDrop_3027493004",
0x2148cf6a,"MainField_WeaponDrop_3030873147",
0xea3182ae,"MainField_WeaponDrop_3030989118",
0xf07be97b,"MainField_WeaponDrop_3031133983",
0xbaa15bf9,"MainField_WeaponDrop_3033528943",
0xc7cf6184,"MainField_WeaponDrop_3034567657",
0x3f02f406,"MainField_WeaponDrop_303637358",
0x3cc44063,"MainField_WeaponDrop_3036444865",
0xed609a6a,"MainField_WeaponDrop_3038834311",
0x9f930a60,"MainField_WeaponDrop_3043391100",
0x0d523845,"MainField_WeaponDrop_3045332891",
0xc84ef65c,"MainField_WeaponDrop_3046825356",
0x9f1b7544,"MainField_WeaponDrop_3048608782",
0xdf889dc2,"MainField_WeaponDrop_3049561233",
0x51e9f0c3,"MainField_WeaponDrop_3050080220",
0xcd3633b2,"MainField_WeaponDrop_3052558583",
0x1289d0bf,"MainField_WeaponDrop_3056122396",
0xd66d249b,"MainField_WeaponDrop_3056763948",
0xf684d0d8,"MainField_WeaponDrop_3057996548",
0x4997344a,"MainField_WeaponDrop_3058885632",
0xcfd33c4a,"MainField_WeaponDrop_3059415303",
0x71d53a6d,"MainField_WeaponDrop_3060085627",
0x8cf6e558,"MainField_WeaponDrop_3063852857",
0xb81c2360,"MainField_WeaponDrop_3064935288",
0x82084a64,"MainField_WeaponDrop_3067709812",
0x9e184420,"MainField_WeaponDrop_3069779400",
0xac7c7470,"MainField_WeaponDrop_3071197125",
0x541a4eaf,"MainField_WeaponDrop_3073427959",
0x4a8f6eaf,"MainField_WeaponDrop_3077518420",
0xe75612d1,"MainField_WeaponDrop_3078415160",
0x4f24b8a5,"MainField_WeaponDrop_3085514520",
0x6cd633fd,"MainField_WeaponDrop_3085780622",
0xdaee6b29,"MainField_WeaponDrop_3091562255",
0x2f423385,"MainField_WeaponDrop_3092879307",
0x568f15b6,"MainField_WeaponDrop_3098201256",
0x2c51afee,"MainField_WeaponDrop_3101346922",
0x3c5fb09f,"MainField_WeaponDrop_3101886944",
0xd4ae9caf,"MainField_WeaponDrop_3102841606",
0xaa4b978d,"MainField_WeaponDrop_3103473499",
0xb3655ae7,"MainField_WeaponDrop_3104076890",
0xbd63ef85,"MainField_WeaponDrop_310454977",
0x71a07674,"MainField_WeaponDrop_3104814373",
0x4220d465,"MainField_WeaponDrop_3112754297",
0x8a16350a,"MainField_WeaponDrop_3113262814",
0x38a2a80c,"MainField_WeaponDrop_3119994280",
0xba7bdbce,"MainField_WeaponDrop_3121696895",
0x11f7f4b7,"MainField_WeaponDrop_3124135445",
0x7e57c00f,"MainField_WeaponDrop_3126798942",
0x3268b47f,"MainField_WeaponDrop_3128398704",
0xbcfb37ca,"MainField_WeaponDrop_3129523435",
0x1bd86bc0,"MainField_WeaponDrop_3131272015",
0xb7ba8065,"MainField_WeaponDrop_3131719936",
0x3391918c,"MainField_WeaponDrop_3132222466",
0x0560878b,"MainField_WeaponDrop_3135122384",
0xb9e5e37a,"MainField_WeaponDrop_3135846060",
0xb1dfa887,"MainField_WeaponDrop_3136716464",
0x803bb05d,"MainField_WeaponDrop_3136737276",
0x6e0b83cd,"MainField_WeaponDrop_3136750335",
0x21e658cd,"MainField_WeaponDrop_3137938984",
0xbcc1114d,"MainField_WeaponDrop_3139097096",
0xad742564,"MainField_WeaponDrop_3145949422",
0x894241bb,"MainField_WeaponDrop_3146996011",
0xd3adb943,"MainField_WeaponDrop_3147193193",
0xd869c249,"MainField_WeaponDrop_3148904888",
0x3ec5297c,"MainField_WeaponDrop_3149150361",
0xa900ca56,"MainField_WeaponDrop_3149348100",
0xc168160a,"MainField_WeaponDrop_315072520",
0x4681caa3,"MainField_WeaponDrop_3156053062",
0x9c78b38f,"MainField_WeaponDrop_3162464960",
0x01f285eb,"MainField_WeaponDrop_3165552876",
0x8c23ab6c,"MainField_WeaponDrop_3166591788",
0xfb25dd81,"MainField_WeaponDrop_3167588283",
0x67b8e1db,"MainField_WeaponDrop_3170101341",
0x65cf9a64,"MainField_WeaponDrop_317134149",
0x4672519b,"MainField_WeaponDrop_3172135515",
0x66f9335b,"MainField_WeaponDrop_3172713636",
0x18d43daa,"MainField_WeaponDrop_31741041",
0x7279e3ea,"MainField_WeaponDrop_3177717108",
0xc8d387fb,"MainField_WeaponDrop_3182476913",
0xcca3375c,"MainField_WeaponDrop_318255316",
0x1a3004c4,"MainField_WeaponDrop_3182919079",
0xa536be6d,"MainField_WeaponDrop_3184271367",
0x28caa899,"MainField_WeaponDrop_3184279717",
0x5c6e836b,"MainField_WeaponDrop_3185518054",
0xe8b4a451,"MainField_WeaponDrop_3185801020",
0x1f6cde25,"MainField_WeaponDrop_3187723205",
0xf1ae8ed1,"MainField_WeaponDrop_3187994529",
0xf9078075,"MainField_WeaponDrop_3189152414",
0x64c42b9e,"MainField_WeaponDrop_3191634263",
0x38084075,"MainField_WeaponDrop_3193708632",
0xe859c995,"MainField_WeaponDrop_3197180867",
0x9038273a,"MainField_WeaponDrop_3198445206",
0x82a36c16,"MainField_WeaponDrop_3199347402",
0x143ce870,"MainField_WeaponDrop_3201407717",
0x1538bc6e,"MainField_WeaponDrop_3202372938",
0xca39929c,"MainField_WeaponDrop_3204025425",
0x95d28912,"MainField_WeaponDrop_3207719296",
0x024e88ec,"MainField_WeaponDrop_3207893704",
0xc11c3688,"MainField_WeaponDrop_3208430749",
0x90abedcb,"MainField_WeaponDrop_3211522825",
0x5835a97c,"MainField_WeaponDrop_3215812142",
0xbe24ffde,"MainField_WeaponDrop_3218272665",
0x4352ff6b,"MainField_WeaponDrop_3220077514",
0x999e3013,"MainField_WeaponDrop_322128821",
0x1be39331,"MainField_WeaponDrop_3223711880",
0xb085f4cf,"MainField_WeaponDrop_3226114990",
0x7af3c17b,"MainField_WeaponDrop_3227528786",
0x78981aa2,"MainField_WeaponDrop_3228573467",
0x92897da3,"MainField_WeaponDrop_3229014488",
0xb0608647,"MainField_WeaponDrop_322985871",
0xc849b73f,"MainField_WeaponDrop_3231164214",
0xf42b1eea,"MainField_WeaponDrop_3232244706",
0x71fd32a1,"MainField_WeaponDrop_323241358",
0xe20abac9,"MainField_WeaponDrop_323278622",
0xca7d9bd2,"MainField_WeaponDrop_3233752671",
0x703e4e83,"MainField_WeaponDrop_3234954435",
0xae230512,"MainField_WeaponDrop_3235067464",
0x713733fa,"MainField_WeaponDrop_3237221213",
0x9e968cf4,"MainField_WeaponDrop_3241082241",
0xed477a85,"MainField_WeaponDrop_3244517268",
0x907798fb,"MainField_WeaponDrop_3244772377",
0x06892ea3,"MainField_WeaponDrop_3246787644",
0x0eb1ae9e,"MainField_WeaponDrop_3247244978",
0x8917f796,"MainField_WeaponDrop_3247444114",
0xc53b31f9,"MainField_WeaponDrop_3249635121",
0xb968ce61,"MainField_WeaponDrop_3252648862",
0xcc4eccdf,"MainField_WeaponDrop_3254972650",
0x2d1c055e,"MainField_WeaponDrop_3259018571",
0x94f7e7c2,"MainField_WeaponDrop_3260698209",
0xdbe1a7d0,"MainField_WeaponDrop_3261508514",
0x03bcc533,"MainField_WeaponDrop_3262296100",
0x7e74e243,"MainField_WeaponDrop_3262598113",
0x481738d4,"MainField_WeaponDrop_3272619301",
0x0f83e8f4,"MainField_WeaponDrop_3274665976",
0xadd639be,"MainField_WeaponDrop_3277044986",
0x9fc88b0e,"MainField_WeaponDrop_3277457876",
0x26524164,"MainField_WeaponDrop_3278579885",
0x1e4365e4,"MainField_WeaponDrop_3279171758",
0xf7f60c2e,"MainField_WeaponDrop_328185575",
0xc283fbd1,"MainField_WeaponDrop_3282483655",
0xad25c117,"MainField_WeaponDrop_3282859513",
0x4a045e79,"MainField_WeaponDrop_3282979594",
0xea751ffe,"MainField_WeaponDrop_328804142",
0x9a92cf58,"MainField_WeaponDrop_328847149",
0x1e450434,"MainField_WeaponDrop_3290052260",
0x92d3a48f,"MainField_WeaponDrop_3290220542",
0x2e439a1f,"MainField_WeaponDrop_3294180755",
0xda0c9dc8,"MainField_WeaponDrop_3297329962",
0x90c68b83,"MainField_WeaponDrop_3304266295",
0xad6b70a7,"MainField_WeaponDrop_3306852344",
0xc558e16c,"MainField_WeaponDrop_3308112546",
0x77cecc26,"MainField_WeaponDrop_3308149486",
0x2dbeb498,"MainField_WeaponDrop_3308363365",
0x71239aa8,"MainField_WeaponDrop_3311262831",
0x4e3522ec,"MainField_WeaponDrop_3312939965",
0x23595ec7,"MainField_WeaponDrop_3314068401",
0x3dfb0a4f,"MainField_WeaponDrop_3317401079",
0x0258dcd6,"MainField_WeaponDrop_3319858707",
0xde443f20,"MainField_WeaponDrop_3319860032",
0xc6adf741,"MainField_WeaponDrop_3320139539",
0x5951edb4,"MainField_WeaponDrop_3321417595",
0x47cb53b8,"MainField_WeaponDrop_3322871439",
0xd32f6318,"MainField_WeaponDrop_3323495565",
0x6375920f,"MainField_WeaponDrop_3327219888",
0x5429be35,"MainField_WeaponDrop_3328551138",
0xee0a3c60,"MainField_WeaponDrop_333018126",
0x0f904f79,"MainField_WeaponDrop_3332397869",
0x891b042c,"MainField_WeaponDrop_333487850",
0x8997fde0,"MainField_WeaponDrop_3334995480",
0x719cd2f0,"MainField_WeaponDrop_3335291854",
0x1dd821e7,"MainField_WeaponDrop_3339079106",
0xeaa5758d,"MainField_WeaponDrop_3339539781",
0xfaa1f9d2,"MainField_WeaponDrop_3340033859",
0x7bcd4647,"MainField_WeaponDrop_33426205",
0xfcdcf47c,"MainField_WeaponDrop_334353976",
0x395da518,"MainField_WeaponDrop_3343835479",
0xbc2b4db9,"MainField_WeaponDrop_3345203441",
0xa58150bb,"MainField_WeaponDrop_334942394",
0x518a190e,"MainField_WeaponDrop_3350792717",
0x32654319,"MainField_WeaponDrop_335083066",
0xd27f6880,"MainField_WeaponDrop_3355909133",
0x296ea4f5,"MainField_WeaponDrop_3356047293",
0x26375934,"MainField_WeaponDrop_3358111828",
0xbf9f5b87,"MainField_WeaponDrop_3362826554",
0x77a2d459,"MainField_WeaponDrop_33672954",
0x2885484c,"MainField_WeaponDrop_33688736",
0x6f014f11,"MainField_WeaponDrop_3368910095",
0xdc410d2c,"MainField_WeaponDrop_337345172",
0xc37debfd,"MainField_WeaponDrop_3374818351",
0xfe314960,"MainField_WeaponDrop_3376105921",
0x2139f7ed,"MainField_WeaponDrop_338012109",
0xb57adda0,"MainField_WeaponDrop_3382489104",
0x279b2fe3,"MainField_WeaponDrop_3385217840",
0x77e305a0,"MainField_WeaponDrop_3386132984",
0xa4057a14,"MainField_WeaponDrop_3386845549",
0x808340e3,"MainField_WeaponDrop_339089672",
0x13ae592c,"MainField_WeaponDrop_3399222830",
0x7409f703,"MainField_WeaponDrop_3407230243",
0x574f3a77,"MainField_WeaponDrop_3408942335",
0xdc01ce0a,"MainField_WeaponDrop_3409164188",
0xab8b2e8d,"MainField_WeaponDrop_3411566654",
0xbeab921c,"MainField_WeaponDrop_3418425290",
0xff7f297d,"MainField_WeaponDrop_3419296513",
0x4b9a4478,"MainField_WeaponDrop_3420864576",
0x9c4eaec0,"MainField_WeaponDrop_3422416346",
0xe932d527,"MainField_WeaponDrop_342419687",
0x6d3757d4,"MainField_WeaponDrop_3424594270",
0x0f70fcfd,"MainField_WeaponDrop_3428411661",
0xd3a810ba,"MainField_WeaponDrop_3428426765",
0x3bfa7912,"MainField_WeaponDrop_3432346220",
0x352b4f18,"MainField_WeaponDrop_3434324650",
0x8769a902,"MainField_WeaponDrop_3435919254",
0xd2c21844,"MainField_WeaponDrop_3436641902",
0xa76f9259,"MainField_WeaponDrop_3438608270",
0x73ecc824,"MainField_WeaponDrop_3441661312",
0x602249d5,"MainField_WeaponDrop_3442587267",
0x5b7f5cb1,"MainField_WeaponDrop_3446211624",
0xc84c0e45,"MainField_WeaponDrop_3448300281",
0x032ed6f5,"MainField_WeaponDrop_3449248938",
0xb298b058,"MainField_WeaponDrop_3449367158",
0x068fde5c,"MainField_WeaponDrop_3449816076",
0x39b6bc54,"MainField_WeaponDrop_3452000395",
0x54f579a6,"MainField_WeaponDrop_345383439",
0xaa7baa98,"MainField_WeaponDrop_3458554846",
0x6e908359,"MainField_WeaponDrop_3464001476",
0xa163d417,"MainField_WeaponDrop_3466703978",
0x8046c96c,"MainField_WeaponDrop_3471356955",
0x7acea9d0,"MainField_WeaponDrop_3472447239",
0x4a5e89c2,"MainField_WeaponDrop_3475003021",
0x32f67cd3,"MainField_WeaponDrop_3482369321",
0x93d74754,"MainField_WeaponDrop_348534063",
0x4c074be6,"MainField_WeaponDrop_3489610376",
0xa72d513f,"MainField_WeaponDrop_3495043470",
0xf480d448,"MainField_WeaponDrop_3495747646",
0xc8a2486e,"MainField_WeaponDrop_3498955801",
0x711e61a5,"MainField_WeaponDrop_3499141879",
0xc18ca05c,"MainField_WeaponDrop_3499468098",
0xf0daf355,"MainField_WeaponDrop_3499550191",
0xb375f7f2,"MainField_WeaponDrop_3504260577",
0xabacdcce,"MainField_WeaponDrop_3505780117",
0x6a628002,"MainField_WeaponDrop_3517682454",
0xb733bed7,"MainField_WeaponDrop_3518622941",
0xeda4e7a8,"MainField_WeaponDrop_3521477102",
0x069e4997,"MainField_WeaponDrop_3522391437",
0x7df6e068,"MainField_WeaponDrop_3522501949",
0x5b156cde,"MainField_WeaponDrop_3524727021",
0xb811ba1f,"MainField_WeaponDrop_3531396311",
0x7c6e0d4d,"MainField_WeaponDrop_353294566",
0xe7329c9f,"MainField_WeaponDrop_3533861494",
0xf11c86c0,"MainField_WeaponDrop_3535260366",
0x4911db56,"MainField_WeaponDrop_3535639669",
0x0a39c9bb,"MainField_WeaponDrop_3538639251",
0xfa79c3bc,"MainField_WeaponDrop_3543306585",
0x2e54bf00,"MainField_WeaponDrop_3545253920",
0xf2a3a9ad,"MainField_WeaponDrop_355134103",
0x65955367,"MainField_WeaponDrop_3551478024",
0xd6101210,"MainField_WeaponDrop_3554029075",
0x56ed47e7,"MainField_WeaponDrop_3557656033",
0x9d56652f,"MainField_WeaponDrop_3559235234",
0x6bdc5287,"MainField_WeaponDrop_355969194",
0x3ccaaaf5,"MainField_WeaponDrop_3560976514",
0x561dd532,"MainField_WeaponDrop_356173544",
0x20d86ca4,"MainField_WeaponDrop_3566895504",
0xb524e60b,"MainField_WeaponDrop_3571001843",
0x7799a503,"MainField_WeaponDrop_3583519194",
0xfff69aa3,"MainField_WeaponDrop_3584374624",
0xd5041bcd,"MainField_WeaponDrop_3587290090",
0x8be73ccf,"MainField_WeaponDrop_3590131943",
0x6cd22382,"MainField_WeaponDrop_3592700427",
0x5df6a452,"MainField_WeaponDrop_3593116712",
0xf0bca8d7,"MainField_WeaponDrop_3595838113",
0xbc78d738,"MainField_WeaponDrop_3595905537",
0x67db7787,"MainField_WeaponDrop_3597997631",
0x34e82fae,"MainField_WeaponDrop_3612011113",
0xcd858db8,"MainField_WeaponDrop_3614008268",
0x82f71d5f,"MainField_WeaponDrop_3616407325",
0x1478568c,"MainField_WeaponDrop_3620236718",
0x7c6fdbcd,"MainField_WeaponDrop_3622599157",
0x953faf70,"MainField_WeaponDrop_3623813729",
0xdf956826,"MainField_WeaponDrop_3624763946",
0x4ee0679d,"MainField_WeaponDrop_3624992618",
0x2cc56a83,"MainField_WeaponDrop_3625653268",
0x367fde7f,"MainField_WeaponDrop_3625742751",
0xccc83a95,"MainField_WeaponDrop_3630432231",
0xc2f016f9,"MainField_WeaponDrop_3630970289",
0xb02b956d,"MainField_WeaponDrop_3631292818",
0x47ef0e39,"MainField_WeaponDrop_3632614239",
0xb8c2ae23,"MainField_WeaponDrop_3632637867",
0xd5e88aba,"MainField_WeaponDrop_3633293139",
0xd728b8d6,"MainField_WeaponDrop_3633727247",
0x00b8d733,"MainField_WeaponDrop_3638033846",
0x9a6afd86,"MainField_WeaponDrop_363927198",
0xfbf01a7a,"MainField_WeaponDrop_3641231808",
0xfd29f02f,"MainField_WeaponDrop_3642276231",
0x8c2495f1,"MainField_WeaponDrop_3643210438",
0x43ea4e63,"MainField_WeaponDrop_3649395942",
0xc09f901e,"MainField_WeaponDrop_3649516712",
0x1e62f276,"MainField_WeaponDrop_3652737448",
0x0e1fce75,"MainField_WeaponDrop_3654331576",
0xf03a5edc,"MainField_WeaponDrop_3661345066",
0x382a85de,"MainField_WeaponDrop_3664044714",
0xf340109c,"MainField_WeaponDrop_3665891287",
0x719bda8b,"MainField_WeaponDrop_3666582301",
0x420396ed,"MainField_WeaponDrop_3666777562",
0x04be6d81,"MainField_WeaponDrop_3666830740",
0x9ae85e31,"MainField_WeaponDrop_3666953199",
0x6066c87e,"MainField_WeaponDrop_3667133752",
0x6a5b0301,"MainField_WeaponDrop_3667788602",
0xf0f15753,"MainField_WeaponDrop_3671763985",
0xe5943858,"MainField_WeaponDrop_3672330822",
0x19f69860,"MainField_WeaponDrop_3673541387",
0x7497b0ce,"MainField_WeaponDrop_3674929488",
0xd56eed1a,"MainField_WeaponDrop_3677093415",
0x951a280f,"MainField_WeaponDrop_3682015981",
0x2bec8b07,"MainField_WeaponDrop_368255114",
0x77623689,"MainField_WeaponDrop_3687310089",
0xa0049f11,"MainField_WeaponDrop_3689359989",
0xa92d97a5,"MainField_WeaponDrop_3691083335",
0x10c43579,"MainField_WeaponDrop_3691101881",
0xa5299da3,"MainField_WeaponDrop_3692367822",
0x3528e28c,"MainField_WeaponDrop_3692540850",
0x9e0d4314,"MainField_WeaponDrop_3692795368",
0x64f936f8,"MainField_WeaponDrop_369490277",
0xe65d70e6,"MainField_WeaponDrop_3695008700",
0xf71fbc8b,"MainField_WeaponDrop_3695400129",
0xe85424f8,"MainField_WeaponDrop_3697862807",
0x6a6ba3d9,"MainField_WeaponDrop_3699668018",
0x5db3b6e2,"MainField_WeaponDrop_3704018134",
0xa7a1b3f1,"MainField_WeaponDrop_3705527175",
0x96bc54a1,"MainField_WeaponDrop_3706236091",
0x705db3ce,"MainField_WeaponDrop_371124352",
0x73ae1934,"MainField_WeaponDrop_3713097319",
0x88612e2d,"MainField_WeaponDrop_3714119993",
0x112cd32d,"MainField_WeaponDrop_3716675756",
0x432ec261,"MainField_WeaponDrop_371671997",
0xf358bff2,"MainField_WeaponDrop_3716939128",
0xab572108,"MainField_WeaponDrop_3721461503",
0x650a998c,"MainField_WeaponDrop_372186287",
0x99f17cd1,"MainField_WeaponDrop_3721951060",
0xe4ad088e,"MainField_WeaponDrop_3724314614",
0x84320ff6,"MainField_WeaponDrop_3724995139",
0xab6ec1b8,"MainField_WeaponDrop_3725167410",
0xdacb6149,"MainField_WeaponDrop_3728023081",
0xb5f9a8cd,"MainField_WeaponDrop_3728668138",
0xf2688144,"MainField_WeaponDrop_3738021512",
0x1a86c54b,"MainField_WeaponDrop_3740849117",
0x8044b8a2,"MainField_WeaponDrop_3741519184",
0xcb0b5598,"MainField_WeaponDrop_3742700185",
0x96ec5669,"MainField_WeaponDrop_3743194910",
0x1dd7c5dc,"MainField_WeaponDrop_3744086851",
0x4e798e6a,"MainField_WeaponDrop_3746799062",
0x21086dac,"MainField_WeaponDrop_374760175",
0x374769c6,"MainField_WeaponDrop_3747626617",
0x96d86d0c,"MainField_WeaponDrop_3749908010",
0x2fa77b83,"MainField_WeaponDrop_3752446980",
0x6c02d6a4,"MainField_WeaponDrop_3752955641",
0xe68c7c35,"MainField_WeaponDrop_3755408198",
0x42a894bd,"MainField_WeaponDrop_3764255989",
0x12d1b64e,"MainField_WeaponDrop_3764670475",
0x35566e52,"MainField_WeaponDrop_3765836139",
0x0a40d255,"MainField_WeaponDrop_3773466958",
0x552aa3e6,"MainField_WeaponDrop_3774417706",
0x01652a3b,"MainField_WeaponDrop_377547942",
0xfc44a5b8,"MainField_WeaponDrop_3776627622",
0xe276a5a3,"MainField_WeaponDrop_3779129363",
0xc6a13c33,"MainField_WeaponDrop_378046435",
0x1fb3a66c,"MainField_WeaponDrop_3781270006",
0x788fa188,"MainField_WeaponDrop_3782413847",
0xe62451c8,"MainField_WeaponDrop_3785551271",
0x3309d246,"MainField_WeaponDrop_3791877420",
0x4db917ca,"MainField_WeaponDrop_3792560744",
0xe7d9162b,"MainField_WeaponDrop_3793218808",
0x0a0139ca,"MainField_WeaponDrop_3799235236",
0xcd113fc3,"MainField_WeaponDrop_3799907517",
0x2182fa9c,"MainField_WeaponDrop_3801856830",
0xdd645409,"MainField_WeaponDrop_3805799870",
0x49c2b4ee,"MainField_WeaponDrop_3806156227",
0xdf3b4ebc,"MainField_WeaponDrop_3807051988",
0xbf9daeab,"MainField_WeaponDrop_3808424870",
0xb10a49d5,"MainField_WeaponDrop_3809811002",
0x47a4b27e,"MainField_WeaponDrop_3810198838",
0xde2f4ecb,"MainField_WeaponDrop_3811803402",
0x3cfe585f,"MainField_WeaponDrop_3815853720",
0x439a012a,"MainField_WeaponDrop_3816422059",
0xaa23fa23,"MainField_WeaponDrop_3818473767",
0xc83e2321,"MainField_WeaponDrop_3819015096",
0x55c0b3c6,"MainField_WeaponDrop_3820953329",
0xc48a79d4,"MainField_WeaponDrop_3821715452",
0x312a4e64,"MainField_WeaponDrop_3823617803",
0x4b140c71,"MainField_WeaponDrop_3823719232",
0x037733e3,"MainField_WeaponDrop_382378032",
0x3f2e4d63,"MainField_WeaponDrop_3825158129",
0x9fcdcdb8,"MainField_WeaponDrop_3826366479",
0xd892a617,"MainField_WeaponDrop_3826946127",
0x77a7e081,"MainField_WeaponDrop_3832764573",
0x23b96787,"MainField_WeaponDrop_3833114478",
0x3f1df9e0,"MainField_WeaponDrop_383402107",
0x1aa93c12,"MainField_WeaponDrop_3835821821",
0xb7a4c32d,"MainField_WeaponDrop_3838720580",
0xc3181ffa,"MainField_WeaponDrop_3839327210",
0x62f3c6ff,"MainField_WeaponDrop_3844280836",
0x02170cee,"MainField_WeaponDrop_3850683841",
0xeec6846b,"MainField_WeaponDrop_3852013366",
0x5b64943e,"MainField_WeaponDrop_3853620530",
0xc522a433,"MainField_WeaponDrop_3856977829",
0xdc482888,"MainField_WeaponDrop_3857227235",
0x95c056f2,"MainField_WeaponDrop_3857498106",
0xa83c301b,"MainField_WeaponDrop_3861156397",
0x1566737b,"MainField_WeaponDrop_3861158814",
0x553f9d49,"MainField_WeaponDrop_3864588119",
0xc3a0b149,"MainField_WeaponDrop_3865087168",
0xeeacdbe2,"MainField_WeaponDrop_3868394662",
0x12b6fd9b,"MainField_WeaponDrop_387639707",
0x8011aca8,"MainField_WeaponDrop_3876811782",
0xf64706c2,"MainField_WeaponDrop_3879433600",
0x86f977a6,"MainField_WeaponDrop_3885067885",
0x25c818a6,"MainField_WeaponDrop_3889065750",
0xca6f5802,"MainField_WeaponDrop_3892529633",
0x41b7cc49,"MainField_WeaponDrop_3893659623",
0xf1eb5411,"MainField_WeaponDrop_3894925922",
0x87873bf7,"MainField_WeaponDrop_3896122125",
0x6a8195e0,"MainField_WeaponDrop_389756872",
0x191e54ff,"MainField_WeaponDrop_3898366362",
0xbfca4467,"MainField_WeaponDrop_3900774838",
0x49872018,"MainField_WeaponDrop_3901615017",
0x15c95ad0,"MainField_WeaponDrop_3903312517",
0x081c1bb4,"MainField_WeaponDrop_3903428323",
0x9d734706,"MainField_WeaponDrop_3904435620",
0xf967d856,"MainField_WeaponDrop_3905745732",
0xc01d6edd,"MainField_WeaponDrop_3905871963",
0xfc028c8d,"MainField_WeaponDrop_3907284509",
0x6b7a05cc,"MainField_WeaponDrop_3914440242",
0xd5cca5f0,"MainField_WeaponDrop_3915511938",
0x9e5de576,"MainField_WeaponDrop_3919214146",
0x0881e536,"MainField_WeaponDrop_3922677252",
0xfd743866,"MainField_WeaponDrop_3923428249",
0xce34374d,"MainField_WeaponDrop_3924724382",
0xbe04cf96,"MainField_WeaponDrop_3927349617",
0x09ad3407,"MainField_WeaponDrop_3929037981",
0xc7ca284d,"MainField_WeaponDrop_3932271530",
0x85af43b5,"MainField_WeaponDrop_3932727673",
0xa3324476,"MainField_WeaponDrop_3933164219",
0x401b7866,"MainField_WeaponDrop_3937963951",
0xb727dc9f,"MainField_WeaponDrop_3940737997",
0x8faf9076,"MainField_WeaponDrop_3942743694",
0x977bbd11,"MainField_WeaponDrop_3943371449",
0xd9bb56b4,"MainField_WeaponDrop_394593651",
0xaeed02e4,"MainField_WeaponDrop_3946910806",
0x6d39c4d8,"MainField_WeaponDrop_3947980913",
0xf30907e8,"MainField_WeaponDrop_3948760319",
0x4974cf82,"MainField_WeaponDrop_3948785183",
0x6d67116b,"MainField_WeaponDrop_3949489249",
0x39020d9c,"MainField_WeaponDrop_3952210665",
0xcbf694c3,"MainField_WeaponDrop_3953795061",
0xeb1748c7,"MainField_WeaponDrop_3954973619",
0xeb6129ef,"MainField_WeaponDrop_3955037107",
0xb3d52887,"MainField_WeaponDrop_3956589875",
0x08bfcb97,"MainField_WeaponDrop_3956753339",
0x0045a96a,"MainField_WeaponDrop_3958659699",
0x8c3422ec,"MainField_WeaponDrop_3959459280",
0x411eaeab,"MainField_WeaponDrop_3960027373",
0x917b6ff9,"MainField_WeaponDrop_3960364544",
0xb60156d5,"MainField_WeaponDrop_3961580955",
0x8c4a49d0,"MainField_WeaponDrop_3963151458",
0x390c1edd,"MainField_WeaponDrop_3967201154",
0x7c878df9,"MainField_WeaponDrop_3967601672",
0x2e7b52b8,"MainField_WeaponDrop_3968394148",
0x61dce6ea,"MainField_WeaponDrop_3973216837",
0x21a8711d,"MainField_WeaponDrop_3976522554",
0x994f140f,"MainField_WeaponDrop_3978421397",
0x3d9d9ad3,"MainField_WeaponDrop_398151326",
0xfd16e6f1,"MainField_WeaponDrop_3982947025",
0x6f283bad,"MainField_WeaponDrop_3983767224",
0x3dbc72b3,"MainField_WeaponDrop_3983798585",
0xef3260da,"MainField_WeaponDrop_398728545",
0x4514fcea,"MainField_WeaponDrop_3990176033",
0x373c9fa5,"MainField_WeaponDrop_3991036631",
0xe5546dad,"MainField_WeaponDrop_3991592510",
0x43b89ac3,"MainField_WeaponDrop_3991821019",
0xcc31f9c3,"MainField_WeaponDrop_399543717",
0x5602147d,"MainField_WeaponDrop_3996532934",
0x160a4485,"MainField_WeaponDrop_3996820229",
0xf5aa26fb,"MainField_WeaponDrop_3999620644",
0x6614e782,"MainField_WeaponDrop_4004178926",
0x0d7ec146,"MainField_WeaponDrop_4005700062",
0xe65860e2,"MainField_WeaponDrop_4008315791",
0x343bdb7c,"MainField_WeaponDrop_4008555687",
0x655d4b7e,"MainField_WeaponDrop_4011777332",
0xd52176bd,"MainField_WeaponDrop_401250753",
0x6ea5e9ef,"MainField_WeaponDrop_4012732075",
0x2a69225e,"MainField_WeaponDrop_4013942664",
0x4220f0bf,"MainField_WeaponDrop_4015946473",
0x1099ac3e,"MainField_WeaponDrop_4016721203",
0x30d74dc4,"MainField_WeaponDrop_4018050043",
0x541a0448,"MainField_WeaponDrop_4018800762",
0xf2947ee4,"MainField_WeaponDrop_4022246325",
0x43467686,"MainField_WeaponDrop_402377215",
0x10ff7583,"MainField_WeaponDrop_4027168957",
0xc0cdd8f7,"MainField_WeaponDrop_4027406459",
0x8f348019,"MainField_WeaponDrop_4027585672",
0x91a817bd,"MainField_WeaponDrop_4029235517",
0xb325e9f9,"MainField_WeaponDrop_4031280149",
0xc90d7eab,"MainField_WeaponDrop_4032898651",
0x7f54fddd,"MainField_WeaponDrop_4033038466",
0x68495cbe,"MainField_WeaponDrop_4033456431",
0xf84dd445,"MainField_WeaponDrop_4034578883",
0xaa4e50ef,"MainField_WeaponDrop_4035635769",
0x4fe3bd98,"MainField_WeaponDrop_4035828775",
0x4f7f98cb,"MainField_WeaponDrop_4038674057",
0x0c1c14af,"MainField_WeaponDrop_4041456295",
0x9f933b09,"MainField_WeaponDrop_4042990435",
0x6743767a,"MainField_WeaponDrop_4045146560",
0xdfdf9968,"MainField_WeaponDrop_4047310890",
0x33cbfbcf,"MainField_WeaponDrop_4047901245",
0xaa086fdb,"MainField_WeaponDrop_4048065864",
0x89ab9fc0,"MainField_WeaponDrop_4048781083",
0x740d724e,"MainField_WeaponDrop_4052292666",
0xe16371ae,"MainField_WeaponDrop_40529870",
0xfa47f674,"MainField_WeaponDrop_405430044",
0x30161266,"MainField_WeaponDrop_4056566945",
0x3cd0679c,"MainField_WeaponDrop_4057706205",
0x73d620b7,"MainField_WeaponDrop_4060618740",
0x8725935b,"MainField_WeaponDrop_4065855047",
0x55590f9e,"MainField_WeaponDrop_4065994007",
0x1185c054,"MainField_WeaponDrop_4069373007",
0x25970523,"MainField_WeaponDrop_4069859273",
0x8cfc07ac,"MainField_WeaponDrop_407168222",
0x3b3069d1,"MainField_WeaponDrop_4074608034",
0xd99eeaff,"MainField_WeaponDrop_4074970074",
0x3cf8f45b,"MainField_WeaponDrop_4075182467",
0x287ca37e,"MainField_WeaponDrop_4076851586",
0xeb3f6302,"MainField_WeaponDrop_4079018505",
0xcce389e5,"MainField_WeaponDrop_4080357896",
0x16f18311,"MainField_WeaponDrop_4083019170",
0xbfb3399f,"MainField_WeaponDrop_408514009",
0x2be5d47d,"MainField_WeaponDrop_4086047655",
0xd448d808,"MainField_WeaponDrop_4086260839",
0x8856f435,"MainField_WeaponDrop_4088737287",
0xe24484b3,"MainField_WeaponDrop_408989699",
0x14d14a26,"MainField_WeaponDrop_4090427046",
0x289cf0d7,"MainField_WeaponDrop_4091037637",
0xb815cd59,"MainField_WeaponDrop_40912747",
0xc193e440,"MainField_WeaponDrop_4096295992",
0xe3824f18,"MainField_WeaponDrop_4098417800",
0x50c64484,"MainField_WeaponDrop_409995480",
0x7b38c6c8,"MainField_WeaponDrop_4100606948",
0x07606f1e,"MainField_WeaponDrop_4100872014",
0x6333d695,"MainField_WeaponDrop_410228374",
0x95ecd7e1,"MainField_WeaponDrop_4104304317",
0x3f7851e2,"MainField_WeaponDrop_4106962537",
0x882d54d8,"MainField_WeaponDrop_4107394772",
0x87e9c6c8,"MainField_WeaponDrop_4108157736",
0xe7aef132,"MainField_WeaponDrop_4108482492",
0xa1814e5c,"MainField_WeaponDrop_4108664323",
0x38c7092c,"MainField_WeaponDrop_4108911525",
0xf6544529,"MainField_WeaponDrop_4109247023",
0x7745583e,"MainField_WeaponDrop_4110372067",
0x795690a8,"MainField_WeaponDrop_4115537013",
0xbdd69e07,"MainField_WeaponDrop_4119991707",
0x753b84b3,"MainField_WeaponDrop_4120682356",
0x0df674c3,"MainField_WeaponDrop_412761148",
0x7e51e245,"MainField_WeaponDrop_4128444200",
0x55df4258,"MainField_WeaponDrop_4128719056",
0xf5c09294,"MainField_WeaponDrop_412873",
0x4cf22de4,"MainField_WeaponDrop_4131047753",
0x7b51bfb5,"MainField_WeaponDrop_4131736488",
0xf619ba38,"MainField_WeaponDrop_4133849588",
0x920bf3c7,"MainField_WeaponDrop_4139538633",
0x261e2a57,"MainField_WeaponDrop_4140485773",
0xd4276fd2,"MainField_WeaponDrop_41429649",
0x8404b170,"MainField_WeaponDrop_4143726007",
0x5252fb5e,"MainField_WeaponDrop_4144892842",
0x5a9b980d,"MainField_WeaponDrop_4145483226",
0x642c8b27,"MainField_WeaponDrop_4147206591",
0xbec94421,"MainField_WeaponDrop_4147666648",
0x6f3f12f0,"MainField_WeaponDrop_4153956892",
0x96e75cdf,"MainField_WeaponDrop_4155008795",
0x131e42f9,"MainField_WeaponDrop_415569071",
0x89d276ce,"MainField_WeaponDrop_4157018341",
0xf8a9777a,"MainField_WeaponDrop_4158990778",
0x3bb4e651,"MainField_WeaponDrop_4160426790",
0x69162c75,"MainField_WeaponDrop_4163836043",
0x923f835b,"MainField_WeaponDrop_4163984418",
0x74d71623,"MainField_WeaponDrop_4164707434",
0x36e133bc,"MainField_WeaponDrop_416895860",
0xf0edb954,"MainField_WeaponDrop_4169217372",
0x0d058ca9,"MainField_WeaponDrop_4169505790",
0xe46b2805,"MainField_WeaponDrop_4171154330",
0x23525e08,"MainField_WeaponDrop_4172077709",
0xfb99a147,"MainField_WeaponDrop_4173230338",
0xf4c80eb4,"MainField_WeaponDrop_4174209577",
0x054dc7a0,"MainField_WeaponDrop_4175956209",
0xf172056b,"MainField_WeaponDrop_4177870046",
0xf0fa6330,"MainField_WeaponDrop_4177964947",
0x5c5a3da1,"MainField_WeaponDrop_4180656335",
0xdcbb8aa2,"MainField_WeaponDrop_4182226419",
0xab6e972e,"MainField_WeaponDrop_4182230508",
0xddb48413,"MainField_WeaponDrop_4185395208",
0x66985b05,"MainField_WeaponDrop_4185470782",
0x5e48eba7,"MainField_WeaponDrop_4188296079",
0x31f4273d,"MainField_WeaponDrop_4191101345",
0xd6b7c3ef,"MainField_WeaponDrop_4193943826",
0xa31d39e4,"MainField_WeaponDrop_4199159362",
0x7664f225,"MainField_WeaponDrop_4199558693",
0x328287b4,"MainField_WeaponDrop_4201101120",
0xb9a165ce,"MainField_WeaponDrop_4201407767",
0x9ef4970a,"MainField_WeaponDrop_4203931395",
0x648f8ae1,"MainField_WeaponDrop_420427774",
0x8397dee9,"MainField_WeaponDrop_4204884016",
0x0d123120,"MainField_WeaponDrop_4205300208",
0xbbe39dc8,"MainField_WeaponDrop_4206227899",
0xf4d2960a,"MainField_WeaponDrop_420780146",
0x23cc21d3,"MainField_WeaponDrop_4208032330",
0x1f6d0ca9,"MainField_WeaponDrop_4209505592",
0x6ce36c06,"MainField_WeaponDrop_4210247064",
0x0944d28e,"MainField_WeaponDrop_421096694",
0xe4fec38f,"MainField_WeaponDrop_4211944259",
0x0a1949a7,"MainField_WeaponDrop_4212590498",
0x13f17b36,"MainField_WeaponDrop_4214846788",
0x3d18049e,"MainField_WeaponDrop_4215773784",
0x28395cc9,"MainField_WeaponDrop_4218272639",
0x52a645e0,"MainField_WeaponDrop_4219896310",
0x1b48a682,"MainField_WeaponDrop_4222903886",
0x9569c5e2,"MainField_WeaponDrop_4224780126",
0xe35813ca,"MainField_WeaponDrop_4225465059",
0x8bc114ab,"MainField_WeaponDrop_422758731",
0x76022f1c,"MainField_WeaponDrop_423143175",
0x73eb0d51,"MainField_WeaponDrop_4232997408",
0x6acdb181,"MainField_WeaponDrop_4235426495",
0x71455a60,"MainField_WeaponDrop_4240775581",
0xff1719eb,"MainField_WeaponDrop_4244742414",
0x092ad1c5,"MainField_WeaponDrop_4245359492",
0x8c8a19ad,"MainField_WeaponDrop_4248384073",
0xf7f2129f,"MainField_WeaponDrop_4250164172",
0x33ec26df,"MainField_WeaponDrop_4250923257",
0x45e1fb9a,"MainField_WeaponDrop_4253104423",
0x9e093e43,"MainField_WeaponDrop_4254590299",
0x95ec2b35,"MainField_WeaponDrop_4255007088",
0xea97490e,"MainField_WeaponDrop_4255805343",
0x26d9b56f,"MainField_WeaponDrop_4256459416",
0x8c47f5f5,"MainField_WeaponDrop_4256565956",
0x66e30a57,"MainField_WeaponDrop_4262997539",
0xad72190f,"MainField_WeaponDrop_4263433970",
0xa5fde5ff,"MainField_WeaponDrop_4265953918",
0x696b68aa,"MainField_WeaponDrop_4265974524",
0x2f9a9ea9,"MainField_WeaponDrop_4266966794",
0xd0e144ac,"MainField_WeaponDrop_4268269593",
0x0be1fc28,"MainField_WeaponDrop_4268364423",
0xe2ca53cf,"MainField_WeaponDrop_4273653908",
0x4e687a92,"MainField_WeaponDrop_4274734637",
0x2b0c9c8d,"MainField_WeaponDrop_428039658",
0x1de2ba95,"MainField_WeaponDrop_4283589620",
0xc98c8d82,"MainField_WeaponDrop_4290512288",
0x5242aae1,"MainField_WeaponDrop_4291604915",
0x7b739e4b,"MainField_WeaponDrop_4293252327",
0xd36f18c7,"MainField_WeaponDrop_429548850",
0xc1b086bc,"MainField_WeaponDrop_430166592",
0x342382c3,"MainField_WeaponDrop_430307885",
0xfcf3f4aa,"MainField_WeaponDrop_439849452",
0x598e87f5,"MainField_WeaponDrop_439909900",
0xb389d251,"MainField_WeaponDrop_440154027",
0x7cf8a710,"MainField_WeaponDrop_440723203",
0x4b923a74,"MainField_WeaponDrop_440992068",
0xd8dc93af,"MainField_WeaponDrop_44135244",
0x80f085e9,"MainField_WeaponDrop_442162840",
0x810ef9a7,"MainField_WeaponDrop_442816501",
0xe0f6c60d,"MainField_WeaponDrop_449372901",
0xaff1a5a4,"MainField_WeaponDrop_453230663",
0x9f42f46f,"MainField_WeaponDrop_458098675",
0xac5975e6,"MainField_WeaponDrop_458178554",
0xd48b8d7f,"MainField_WeaponDrop_459435312",
0x29601f9d,"MainField_WeaponDrop_460115728",
0xf422cebd,"MainField_WeaponDrop_460605536",
0x9b3abdea,"MainField_WeaponDrop_462089252",
0x015fe9bb,"MainField_WeaponDrop_463568543",
0x08eb3e13,"MainField_WeaponDrop_464048049",
0xcb415efb,"MainField_WeaponDrop_466237002",
0x62d3f485,"MainField_WeaponDrop_467647463",
0xbfc6ee2d,"MainField_WeaponDrop_470788393",
0xcf8e428f,"MainField_WeaponDrop_474701617",
0x0fa90e66,"MainField_WeaponDrop_475707588",
0x5ed8290a,"MainField_WeaponDrop_476088826",
0xaae3c1d9,"MainField_WeaponDrop_478266440",
0x96ecbbd3,"MainField_WeaponDrop_479232562",
0x8b684086,"MainField_WeaponDrop_48063290",
0x63a0ae5e,"MainField_WeaponDrop_481900238",
0x5c38bc45,"MainField_WeaponDrop_482068829",
0x456f8a50,"MainField_WeaponDrop_484264628",
0x39a67ff0,"MainField_WeaponDrop_486354635",
0xb1bcd6b7,"MainField_WeaponDrop_486520671",
0x88d37d29,"MainField_WeaponDrop_489376067",
0xcf191b19,"MainField_WeaponDrop_48957650",
0xaeca765a,"MainField_WeaponDrop_49008408",
0x2a23c6c5,"MainField_WeaponDrop_491027219",
0xa180bd3f,"MainField_WeaponDrop_491049017",
0xea766b02,"MainField_WeaponDrop_498399182",
0x37f2ec81,"MainField_WeaponDrop_501305565",
0xbfb75230,"MainField_WeaponDrop_502557530",
0x36a0a72d,"MainField_WeaponDrop_503528502",
0xab5b6264,"MainField_WeaponDrop_505268816",
0xbef1b8ab,"MainField_WeaponDrop_505543268",
0x5a904137,"MainField_WeaponDrop_505888518",
0x7d956e10,"MainField_WeaponDrop_510128032",
0x4ab55ded,"MainField_WeaponDrop_510961903",
0x193e5376,"MainField_WeaponDrop_514245995",
0xe227eaaf,"MainField_WeaponDrop_514872961",
0x62d7c628,"MainField_WeaponDrop_515436304",
0x2c8074fb,"MainField_WeaponDrop_517850908",
0x6f38d834,"MainField_WeaponDrop_518392637",
0xa88eb32e,"MainField_WeaponDrop_519389775",
0xae0e6674,"MainField_WeaponDrop_519937051",
0xa7c746ab,"MainField_WeaponDrop_520467944",
0x950be38f,"MainField_WeaponDrop_523907031",
0xc5263403,"MainField_WeaponDrop_524690876",
0xd903608b,"MainField_WeaponDrop_525601999",
0x9be4fc18,"MainField_WeaponDrop_526219962",
0x809325a1,"MainField_WeaponDrop_528859891",
0x2990aa5d,"MainField_WeaponDrop_529099813",
0xf35d8938,"MainField_WeaponDrop_531777006",
0x159c5291,"MainField_WeaponDrop_531894364",
0x1a37b610,"MainField_WeaponDrop_536488003",
0x7ecff2cc,"MainField_WeaponDrop_537878327",
0x8fc7a950,"MainField_WeaponDrop_543914872",
0x7023b52e,"MainField_WeaponDrop_543961826",
0x15dd50e0,"MainField_WeaponDrop_544257454",
0x3968b85f,"MainField_WeaponDrop_547171185",
0x215ff4ee,"MainField_WeaponDrop_549618249",
0xc63dd14d,"MainField_WeaponDrop_549822702",
0x434e3b2e,"MainField_WeaponDrop_55249506",
0x81e51113,"MainField_WeaponDrop_554836133",
0x6ab0aa5d,"MainField_WeaponDrop_560511650",
0x60e099ce,"MainField_WeaponDrop_562690853",
0xfcb891b8,"MainField_WeaponDrop_565416702",
0x6d7dc3d2,"MainField_WeaponDrop_567757486",
0xf7d62ead,"MainField_WeaponDrop_570094754",
0x4d6a149c,"MainField_WeaponDrop_573140497",
0xc3466409,"MainField_WeaponDrop_574099397",
0xde51181b,"MainField_WeaponDrop_57543256",
0x456c0391,"MainField_WeaponDrop_576338634",
0x4ed50e1b,"MainField_WeaponDrop_577965410",
0xc10c42cd,"MainField_WeaponDrop_581630282",
0xe74e258f,"MainField_WeaponDrop_582718625",
0xa6e4b0b5,"MainField_WeaponDrop_584015546",
0x50c6dd38,"MainField_WeaponDrop_591110349",
0xfc013f18,"MainField_WeaponDrop_598761443",
0x329d4bbf,"MainField_WeaponDrop_599496708",
0xfa4ebc3c,"MainField_WeaponDrop_602078386",
0x75060dcd,"MainField_WeaponDrop_609705216",
0xc57bc8c4,"MainField_WeaponDrop_613479287",
0xc7623d81,"MainField_WeaponDrop_614671614",
0x3a0871fc,"MainField_WeaponDrop_61476265",
0xb0e530b8,"MainField_WeaponDrop_615786510",
0xbb5e254c,"MainField_WeaponDrop_616400489",
0x1b219129,"MainField_WeaponDrop_617704566",
0x00c0b71a,"MainField_WeaponDrop_618093973",
0x0f9f9e63,"MainField_WeaponDrop_620111792",
0xe29013a1,"MainField_WeaponDrop_627769133",
0x24f88a0d,"MainField_WeaponDrop_628401097",
0x8089eec7,"MainField_WeaponDrop_631350222",
0x75f568de,"MainField_WeaponDrop_63255521",
0x57b1a245,"MainField_WeaponDrop_635013852",
0x53b393f7,"MainField_WeaponDrop_635710903",
0x8c97e133,"MainField_WeaponDrop_637156226",
0xb570496e,"MainField_WeaponDrop_638321301",
0x446318a5,"MainField_WeaponDrop_64172073",
0x783987e1,"MainField_WeaponDrop_641933914",
0xe96f435c,"MainField_WeaponDrop_6420232",
0x7c9655df,"MainField_WeaponDrop_649983064",
0xf7b0d95a,"MainField_WeaponDrop_651042509",
0xa40aa758,"MainField_WeaponDrop_652096048",
0x7183a08c,"MainField_WeaponDrop_656370739",
0x9bfb6c94,"MainField_WeaponDrop_658240119",
0x2e6e1837,"MainField_WeaponDrop_660970525",
0x9073305c,"MainField_WeaponDrop_66161362",
0x8c33e564,"MainField_WeaponDrop_663638345",
0x1cb31335,"MainField_WeaponDrop_664391761",
0x7b9c547c,"MainField_WeaponDrop_669219149",
0x69ff5f65,"MainField_WeaponDrop_675322964",
0xdcfc9cce,"MainField_WeaponDrop_675573397",
0xfce568b6,"MainField_WeaponDrop_675796627",
0xfec317c1,"MainField_WeaponDrop_676192432",
0xac647f30,"MainField_WeaponDrop_678592431",
0x749202e3,"MainField_WeaponDrop_680779429",
0x18517b41,"MainField_WeaponDrop_681825716",
0xa2d66b01,"MainField_WeaponDrop_685720050",
0x31e28feb,"MainField_WeaponDrop_687058618",
0x855211ca,"MainField_WeaponDrop_690896644",
0x913c6e9b,"MainField_WeaponDrop_691002666",
0x1e8a0ed1,"MainField_WeaponDrop_691127437",
0x84a5f405,"MainField_WeaponDrop_692801740",
0x4050f2a5,"MainField_WeaponDrop_694858205",
0x71cb17d8,"MainField_WeaponDrop_696206435",
0x3e7a7e77,"MainField_WeaponDrop_696249181",
0x81232eaa,"MainField_WeaponDrop_698472245",
0xa1c22654,"MainField_WeaponDrop_7007387",
0x6cad69b6,"MainField_WeaponDrop_701662810",
0x5f65cd95,"MainField_WeaponDrop_702094619",
0x733c4b34,"MainField_WeaponDrop_703786444",
0xe3cb33e1,"MainField_WeaponDrop_709379341",
0xe3e032d6,"MainField_WeaponDrop_709646735",
0xe4d15609,"MainField_WeaponDrop_710804719",
0xa50fc7cf,"MainField_WeaponDrop_711518193",
0x251ba6c4,"MainField_WeaponDrop_71313383",
0x2ae27860,"MainField_WeaponDrop_713348493",
0x148c67c2,"MainField_WeaponDrop_714239954",
0xfd30abcd,"MainField_WeaponDrop_714828466",
0x0ec55212,"MainField_WeaponDrop_716341778",
0x41c179d8,"MainField_WeaponDrop_717038457",
0x7a943f19,"MainField_WeaponDrop_720837969",
0xc4e44fe7,"MainField_WeaponDrop_721609695",
0x01555127,"MainField_WeaponDrop_72224583",
0xd7cab434,"MainField_WeaponDrop_728472959",
0xaebeb9bc,"MainField_WeaponDrop_731749771",
0x92297d34,"MainField_WeaponDrop_734329321",
0xc8ec7219,"MainField_WeaponDrop_736408835",
0x22dbc589,"MainField_WeaponDrop_739724916",
0xeb11d666,"MainField_WeaponDrop_742164841",
0x647afdb2,"MainField_WeaponDrop_744366118",
0x4b61c9e0,"MainField_WeaponDrop_744855500",
0xde4dbddf,"MainField_WeaponDrop_745482182",
0x3edde579,"MainField_WeaponDrop_747059457",
0xfb6e605f,"MainField_WeaponDrop_750049145",
0x21c6d918,"MainField_WeaponDrop_751037428",
0x7c7ec0ab,"MainField_WeaponDrop_751063482",
0xbbd16708,"MainField_WeaponDrop_751150750",
0x3dedf254,"MainField_WeaponDrop_752365712",
0x6b1c5ced,"MainField_WeaponDrop_756376350",
0xb20c52f9,"MainField_WeaponDrop_757544675",
0xd68950ce,"MainField_WeaponDrop_760347177",
0x433f3e75,"MainField_WeaponDrop_761657246",
0x44ce8e7d,"MainField_WeaponDrop_773019745",
0x5ad93f62,"MainField_WeaponDrop_77426438",
0x77b59b97,"MainField_WeaponDrop_774296036",
0x04b2f3de,"MainField_WeaponDrop_77458986",
0xe3824a5d,"MainField_WeaponDrop_777725831",
0xe224ed8e,"MainField_WeaponDrop_777962100",
0xd5d98e22,"MainField_WeaponDrop_778162430",
0x8a1225bd,"MainField_WeaponDrop_780471353",
0x694ef3b0,"MainField_WeaponDrop_780543934",
0x4b83c860,"MainField_WeaponDrop_782754550",
0x78928cbd,"MainField_WeaponDrop_783514868",
0xe839cc85,"MainField_WeaponDrop_784788655",
0x0ef305c3,"MainField_WeaponDrop_785821913",
0x68b7eca8,"MainField_WeaponDrop_78639580",
0xb6f23ff6,"MainField_WeaponDrop_786731763",
0x0caa893f,"MainField_WeaponDrop_788165233",
0x410f7f99,"MainField_WeaponDrop_788755615",
0xe1db2d1c,"MainField_WeaponDrop_79011342",
0x4096dd78,"MainField_WeaponDrop_794315804",
0xd03e3323,"MainField_WeaponDrop_795395900",
0x4fd3b65d,"MainField_WeaponDrop_797513132",
0xafa8f854,"MainField_WeaponDrop_798905762",
0x24c00d6b,"MainField_WeaponDrop_800539081",
0x0c6e718b,"MainField_WeaponDrop_800790857",
0x4adf424d,"MainField_WeaponDrop_804123957",
0x86a412bc,"MainField_WeaponDrop_80692187",
0x6a7b0ee0,"MainField_WeaponDrop_807158507",
0x654af0fd,"MainField_WeaponDrop_809779390",
0x6d7b502b,"MainField_WeaponDrop_814973765",
0x2bbe9379,"MainField_WeaponDrop_818738781",
0xeb2eeb04,"MainField_WeaponDrop_818840547",
0x800fe070,"MainField_WeaponDrop_822300682",
0xea1afb9a,"MainField_WeaponDrop_82403539",
0x3d32b189,"MainField_WeaponDrop_832637722",
0x6d6ab29c,"MainField_WeaponDrop_833902723",
0x7b56c714,"MainField_WeaponDrop_835876134",
0xa5dca272,"MainField_WeaponDrop_836634595",
0xab4f48db,"MainField_WeaponDrop_837939551",
0xbdd613e7,"MainField_WeaponDrop_840299716",
0x7b2272da,"MainField_WeaponDrop_842997169",
0x3be0a2a5,"MainField_WeaponDrop_84385002",
0x83803169,"MainField_WeaponDrop_844262840",
0x2fcc13e7,"MainField_WeaponDrop_844535813",
0xd98d9cf3,"MainField_WeaponDrop_847467802",
0x88f2d67b,"MainField_WeaponDrop_851152328",
0x257d047a,"MainField_WeaponDrop_851162926",
0x3c1306ba,"MainField_WeaponDrop_853717408",
0x2f46942f,"MainField_WeaponDrop_854994967",
0xc18af18b,"MainField_WeaponDrop_858828110",
0x75d3ecae,"MainField_WeaponDrop_859714112",
0x155cc9f5,"MainField_WeaponDrop_862704338",
0x270c489b,"MainField_WeaponDrop_864686182",
0x86a53169,"MainField_WeaponDrop_865811446",
0x1e6d9cc1,"MainField_WeaponDrop_869313541",
0xfde2a249,"MainField_WeaponDrop_875961525",
0xe921754c,"MainField_WeaponDrop_877313805",
0x2b5c3407,"MainField_WeaponDrop_877734963",
0x6cdbaba6,"MainField_WeaponDrop_878673290",
0xf5f1b508,"MainField_WeaponDrop_879814288",
0x266efe55,"MainField_WeaponDrop_880622559",
0x2ceb14e9,"MainField_WeaponDrop_882844355",
0xddc877ae,"MainField_WeaponDrop_884650541",
0x08bf0c19,"MainField_WeaponDrop_886079513",
0x921baa43,"MainField_WeaponDrop_893201085",
0xc27a8c50,"MainField_WeaponDrop_89337299",
0x13df9992,"MainField_WeaponDrop_893857558",
0xa7d9eb8e,"MainField_WeaponDrop_89924997",
0x7cd8e4bf,"MainField_WeaponDrop_900218234",
0x48a7b3b8,"MainField_WeaponDrop_902838187",
0x9a2c4ab6,"MainField_WeaponDrop_903085374",
0xe4b801e8,"MainField_WeaponDrop_903357653",
0x90cba06a,"MainField_WeaponDrop_90345466",
0x0f28ac91,"MainField_WeaponDrop_904172580",
0xc206e0f0,"MainField_WeaponDrop_907082154",
0xb148ab8a,"MainField_WeaponDrop_909790673",
0xad71a71c,"MainField_WeaponDrop_910707400",
0xb466dd0a,"MainField_WeaponDrop_911208189",
0x13fd541e,"MainField_WeaponDrop_91305348",
0xa55df871,"MainField_WeaponDrop_913689830",
0xae526db8,"MainField_WeaponDrop_914382311",
0xabe70808,"MainField_WeaponDrop_914412650",
0x725bc3a8,"MainField_WeaponDrop_91488073",
0x82023f9e,"MainField_WeaponDrop_916290142",
0x0a333b2b,"MainField_WeaponDrop_918937002",
0xa9c8be2f,"MainField_WeaponDrop_920231066",
0x957e7d92,"MainField_WeaponDrop_921461692",
0x8f657457,"MainField_WeaponDrop_927336859",
0x91885d94,"MainField_WeaponDrop_927761522",
0x1c42fb3f,"MainField_WeaponDrop_932931692",
0xe5c18cec,"MainField_WeaponDrop_933089512",
0x09eae8f9,"MainField_WeaponDrop_933659928",
0x5ec38695,"MainField_WeaponDrop_936444533",
0xfcf28a82,"MainField_WeaponDrop_936923481",
0x637e94e9,"MainField_WeaponDrop_938169908",
0x7bcc27d4,"MainField_WeaponDrop_939468739",
0x2f00304b,"MainField_WeaponDrop_940799969",
0xdf559623,"MainField_WeaponDrop_941383066",
0xa88b5031,"MainField_WeaponDrop_942016425",
0x09487caa,"MainField_WeaponDrop_943734742",
0x96449c6d,"MainField_WeaponDrop_944105547",
0x14e52277,"MainField_WeaponDrop_946245607",
0xa15c886f,"MainField_WeaponDrop_947278028",
0x7306c7f3,"MainField_WeaponDrop_950083343",
0xf9d02378,"MainField_WeaponDrop_953135122",
0xbef8a3b6,"MainField_WeaponDrop_954380264",
0x843e0a92,"MainField_WeaponDrop_955775802",
0x98c00f4d,"MainField_WeaponDrop_959282854",
0x2c880bce,"MainField_WeaponDrop_961430505",
0xa68f7078,"MainField_WeaponDrop_961698596",
0xadd60a65,"MainField_WeaponDrop_962915971",
0x42c4ceb5,"MainField_WeaponDrop_973004645",
0x8b280eb5,"MainField_WeaponDrop_975912942",
0x859168f1,"MainField_WeaponDrop_977371565",
0xbe3d8561,"MainField_WeaponDrop_977578737",
0x785337dd,"MainField_WeaponDrop_983496084",
0xa4f2ea14,"MainField_WeaponDrop_987142984",
0x4e29883d,"MainField_WeaponDrop_987980991",
0x92258cf6,"MainField_WeaponDrop_988282125",
0x4abd3c75,"MainField_WeaponDrop_988946681",
0xef2a26a5,"MainField_WeaponDrop_996452146",
0x39719fed,"MainField_WeaponDrop_998011909",
0x5504bfc1,"MainField_Weapon_Bow_001_1878496234",
0x314a4924,"MainField_Weapon_Bow_001_1974182254",
0x298e9cbf,"MainField_Weapon_Bow_001_3768604995",
0x8b596c72,"MainField_Weapon_Bow_001_436518968",
0x978f6ee5,"MainField_Weapon_Bow_001_900840757",
0xdc6bde42,"MainField_Weapon_Bow_002_1529216882",
0xae15954f,"MainField_Weapon_Bow_002_1917941768",
0xd0847911,"MainField_Weapon_Bow_002_1986588683",
0xa776c85f,"MainField_Weapon_Bow_002_2138882423",
0x7b3417b1,"MainField_Weapon_Bow_002_2726231268",
0x6437e784,"MainField_Weapon_Bow_002_2746026919",
0x94bb0375,"MainField_Weapon_Bow_002_3693507275",
0xdca2323e,"MainField_Weapon_Bow_003_3812073092",
0xe35a4045,"MainField_Weapon_Bow_004_1376842377",
0x8ebb2bf5,"MainField_Weapon_Bow_004_1474635068",
0xad2d890e,"MainField_Weapon_Bow_004_1557656313",
0xa819911c,"MainField_Weapon_Bow_004_4033647307",
0x9be1f9f3,"MainField_Weapon_Bow_006_1235050403",
0xa80411b2,"MainField_Weapon_Bow_006_2757080695",
0xb0fcd5ba,"MainField_Weapon_Bow_011_596868754",
0x00e4c91d,"MainField_Weapon_Bow_011_941765605",
0x6b86de1a,"MainField_Weapon_Bow_013_1278332844",
0x214d3627,"MainField_Weapon_Bow_013_2922305354",
0xb5492da5,"MainField_Weapon_Bow_014_4292635535",
0x879dfd36,"MainField_Weapon_Bow_015_2548932184",
0x6089e088,"MainField_Weapon_Bow_015_3047012338",
0xfbaf11c6,"MainField_Weapon_Bow_015_3863871963",
0x4c808f62,"MainField_Weapon_Bow_015_504971724",
0x62196bdc,"MainField_Weapon_Bow_016_2831726766",
0xc475e078,"MainField_Weapon_Bow_016_574340301",
0xbab8785f,"MainField_Weapon_Bow_017_2349380862",
0xb1da952b,"MainField_Weapon_Bow_033_277043031",
0xdf33f0d3,"MainField_Weapon_Bow_033_4274460657",
0x8670e73e,"MainField_Weapon_Bow_035_1790118271",
0x5a78d36d,"MainField_Weapon_Bow_035_3041094555",
0x79801a52,"MainField_Weapon_Bow_035_304950860",
0xaa5bb779,"MainField_Weapon_Bow_035_3557053874",
0x25c443a4,"MainField_Weapon_Bow_035_4249593221",
0x857b4edf,"MainField_Weapon_Bow_036_1587619363",
0x4c793af8,"MainField_Weapon_Bow_036_1839855609",
0xc6117b83,"MainField_Weapon_Bow_036_362320067",
0x9988d51b,"MainField_Weapon_Bow_036_910258361",
0xb8f1e443,"MainField_Weapon_Bow_038_1098631094",
0x293ea259,"MainField_Weapon_Bow_038_1265908210",
0x7163625a,"MainField_Weapon_Bow_038_1440538914",
0x15e4f531,"MainField_Weapon_Bow_038_179645897",
0xba28b4a0,"MainField_Weapon_Bow_038_1806973813",
0xe68cc2cd,"MainField_Weapon_Bow_038_2045557689",
0xe75c318e,"MainField_Weapon_Bow_038_2210805983",
0xca9b8307,"MainField_Weapon_Bow_038_2282735397",
0x2848c7e2,"MainField_Weapon_Bow_038_2420384246",
0xe2485469,"MainField_Weapon_Bow_038_2766949308",
0x50a8ed69,"MainField_Weapon_Bow_038_3343479806",
0x07de9685,"MainField_Weapon_Bow_038_3400129643",
0x2a0df268,"MainField_Weapon_Bow_038_3413590466",
0x16d7b105,"MainField_Weapon_Bow_038_3588352720",
0x61b0aacc,"MainField_Weapon_Bow_038_3656385577",
0xe541d7c0,"MainField_Weapon_Bow_038_4138170666",
0x3ee780ba,"MainField_Weapon_Bow_038_834595535",
0xb337a441,"MainField_Weapon_Bow_038_894150865",
0xd9cfe9b7,"MainField_Weapon_Bow_040_3086865982",
0x8c8bd64d,"MainField_Weapon_Bow_040_3545733825",
0xfa6890ab,"MainField_Weapon_Bow_071_2178255681",
0x9103f60f,"MainField_Weapon_Lsword_001_1028776920",
0x2a2c6b0c,"MainField_Weapon_Lsword_001_1032728867",
0xe7936f25,"MainField_Weapon_Lsword_001_1250492853",
0x07722500,"MainField_Weapon_Lsword_001_1306838444",
0xeb400a85,"MainField_Weapon_Lsword_001_1384391352",
0xbbbd3fb8,"MainField_Weapon_Lsword_001_1441392289",
0x8592cd84,"MainField_Weapon_Lsword_001_1586442302",
0x29477ae8,"MainField_Weapon_Lsword_001_1831413168",
0xf730166c,"MainField_Weapon_Lsword_001_2616646907",
0x3882e09e,"MainField_Weapon_Lsword_001_2629945032",
0x6b6dc014,"MainField_Weapon_Lsword_001_2750063744",
0xa8668065,"MainField_Weapon_Lsword_001_2837842583",
0x72179fb0,"MainField_Weapon_Lsword_001_2884364926",
0xc30ca02e,"MainField_Weapon_Lsword_001_2935465034",
0x18df6a5d,"MainField_Weapon_Lsword_001_3136394689",
0x2a387dfc,"MainField_Weapon_Lsword_001_3152512909",
0xfff3d74e,"MainField_Weapon_Lsword_001_3163177876",
0x5b2399f8,"MainField_Weapon_Lsword_001_3415044866",
0x43c4d297,"MainField_Weapon_Lsword_001_3430116814",
0x58cf24f7,"MainField_Weapon_Lsword_001_3455493463",
0x5cda4599,"MainField_Weapon_Lsword_001_3537253407",
0x116b1e85,"MainField_Weapon_Lsword_001_3572339734",
0xcf0d1596,"MainField_Weapon_Lsword_001_3660979236",
0xbd331aac,"MainField_Weapon_Lsword_001_379806432",
0xf0d86aba,"MainField_Weapon_Lsword_001_3955684958",
0x95a90aaf,"MainField_Weapon_Lsword_001_4172088268",
0x300274cf,"MainField_Weapon_Lsword_001_44389227",
0xade20f7a,"MainField_Weapon_Lsword_001_485215461",
0xca9b701e,"MainField_Weapon_Lsword_001_586077751",
0x78aa6728,"MainField_Weapon_Lsword_001_608883353",
0xb289d28c,"MainField_Weapon_Lsword_001_741694981",
0x0689c7db,"MainField_Weapon_Lsword_001_808409901",
0x6cb24068,"MainField_Weapon_Lsword_001_814952937",
0xe7bafb6b,"MainField_Weapon_Lsword_001_872421342",
0x12c7a0af,"MainField_Weapon_Lsword_002_2074956857",
0xe902e881,"MainField_Weapon_Lsword_002_2201849163",
0x405c81d8,"MainField_Weapon_Lsword_002_2820048025",
0x23dc6f36,"MainField_Weapon_Lsword_002_3742361488",
0xc9b9de80,"MainField_Weapon_Lsword_002_4131964018",
0xe4a93bed,"MainField_Weapon_Lsword_002_4161682649",
0xc722a6c5,"MainField_Weapon_Lsword_002_757206084",
0x6d7dca9e,"MainField_Weapon_Lsword_003_1468591707",
0x4dbd42dd,"MainField_Weapon_Lsword_003_2289880323",
0x655d6e52,"MainField_Weapon_Lsword_003_239019860",
0x7211554b,"MainField_Weapon_Lsword_003_2855872001",
0x6db2aac2,"MainField_Weapon_Lsword_003_3267645306",
0xd2af52ed,"MainField_Weapon_Lsword_003_3710935152",
0x00bba364,"MainField_Weapon_Lsword_003_493588007",
0xfcd8e3c7,"MainField_Weapon_Lsword_003_775426429",
0xec68fbdf,"MainField_Weapon_Lsword_004_1068968262",
0xd92eb706,"MainField_Weapon_Lsword_004_1812839565",
0xead6870c,"MainField_Weapon_Lsword_004_2447343537",
0x9e847035,"MainField_Weapon_Lsword_004_3274714769",
0xf98c88c8,"MainField_Weapon_Lsword_004_3772041117",
0x1acf43e1,"MainField_Weapon_Lsword_004_4104386125",
0x00fbe8a6,"MainField_Weapon_Lsword_004_453676059",
0x72ca5be7,"MainField_Weapon_Lsword_004_46068049",
0x0102f0c4,"MainField_Weapon_Lsword_004_730195154",
0xc14f69b6,"MainField_Weapon_Lsword_005_2528287823",
0xb0c3f4ca,"MainField_Weapon_Lsword_005_3440099174",
0x4f5025b9,"MainField_Weapon_Lsword_006_1682703764",
0xe7b6d9bc,"MainField_Weapon_Lsword_006_2308620646",
0xf2a61c3c,"MainField_Weapon_Lsword_011_109426075",
0xd7697faa,"MainField_Weapon_Lsword_011_2648923375",
0x1f56b454,"MainField_Weapon_Lsword_011_4256982684",
0x6b2894e0,"MainField_Weapon_Lsword_011_784633565",
0x99aa4c40,"MainField_Weapon_Lsword_012_3598813288",
0x09ad40e2,"MainField_Weapon_Lsword_012_3922280212",
0x75f9e05e,"MainField_Weapon_Lsword_012_540703076",
0xdf1bde19,"MainField_Weapon_Lsword_020_1002487822",
0x26aa95ba,"MainField_Weapon_Lsword_020_100337587",
0xba258ca8,"MainField_Weapon_Lsword_020_1034014322",
0xc6fb753a,"MainField_Weapon_Lsword_020_1054903453",
0xe8eb76a8,"MainField_Weapon_Lsword_020_1057524311",
0x5ecedf9f,"MainField_Weapon_Lsword_020_1108524058",
0x1853da60,"MainField_Weapon_Lsword_020_1147214610",
0x565c2579,"MainField_Weapon_Lsword_020_1151412900",
0x833e1543,"MainField_Weapon_Lsword_020_1152532368",
0x259eefa5,"MainField_Weapon_Lsword_020_1184503543",
0xf8b611a2,"MainField_Weapon_Lsword_020_1236019577",
0xb728973d,"MainField_Weapon_Lsword_020_1300441212",
0xa6f8e80d,"MainField_Weapon_Lsword_020_1373700145",
0xf1832c17,"MainField_Weapon_Lsword_020_1507816238",
0xd0fa67f0,"MainField_Weapon_Lsword_020_1570068101",
0xa78e5ccf,"MainField_Weapon_Lsword_020_1587312874",
0xa1905c74,"MainField_Weapon_Lsword_020_1714494229",
0xe5082a1b,"MainField_Weapon_Lsword_020_171955666",
0xf9bf2e50,"MainField_Weapon_Lsword_020_1797798192",
0x2dc9e731,"MainField_Weapon_Lsword_020_1807641619",
0x66ac784f,"MainField_Weapon_Lsword_020_1869898359",
0xbd9e8a93,"MainField_Weapon_Lsword_020_1923414253",
0x50d7c570,"MainField_Weapon_Lsword_020_1963450847",
0x437fd4be,"MainField_Weapon_Lsword_020_1978443500",
0xb7848c4f,"MainField_Weapon_Lsword_020_2012498743",
0xfd47b70f,"MainField_Weapon_Lsword_020_2198170347",
0x629bf9c2,"MainField_Weapon_Lsword_020_2247573766",
0x7ecd534d,"MainField_Weapon_Lsword_020_2282861746",
0x5a934746,"MainField_Weapon_Lsword_020_2316487785",
0x2fb8124a,"MainField_Weapon_Lsword_020_2376060589",
0xeed8f63c,"MainField_Weapon_Lsword_020_2395439150",
0x15d96b02,"MainField_Weapon_Lsword_020_2553158176",
0x8138c77d,"MainField_Weapon_Lsword_020_2574696542",
0xb2a0557c,"MainField_Weapon_Lsword_020_2581072694",
0xa86d4794,"MainField_Weapon_Lsword_020_2789133383",
0x57d6e321,"MainField_Weapon_Lsword_020_3011759392",
0x1390f292,"MainField_Weapon_Lsword_020_3045172166",
0x2e73cc16,"MainField_Weapon_Lsword_020_3079040957",
0xfd6be5d5,"MainField_Weapon_Lsword_020_311374230",
0x74845236,"MainField_Weapon_Lsword_020_3134770014",
0x9ee0f7f6,"MainField_Weapon_Lsword_020_3235259028",
0x869c887f,"MainField_Weapon_Lsword_020_3313448078",
0xa0be7664,"MainField_Weapon_Lsword_020_3385619486",
0x63d8c77d,"MainField_Weapon_Lsword_020_33902432",
0x5e5bd605,"MainField_Weapon_Lsword_020_3510358147",
0x56e2fd15,"MainField_Weapon_Lsword_020_3592517667",
0x6636c08d,"MainField_Weapon_Lsword_020_3608583143",
0xaf104eba,"MainField_Weapon_Lsword_020_3639931985",
0x35a46410,"MainField_Weapon_Lsword_020_3697917537",
0xb2274752,"MainField_Weapon_Lsword_020_3749049806",
0xd057ae46,"MainField_Weapon_Lsword_020_3770615367",
0x14718390,"MainField_Weapon_Lsword_020_3947693507",
0x61181712,"MainField_Weapon_Lsword_020_4025584777",
0xfb61ca3e,"MainField_Weapon_Lsword_020_407517327",
0x8791b214,"MainField_Weapon_Lsword_020_4168307124",
0x8a907662,"MainField_Weapon_Lsword_020_463028272",
0x6d22bfc6,"MainField_Weapon_Lsword_020_516105193",
0x319bd90c,"MainField_Weapon_Lsword_020_555325803",
0x07bef23e,"MainField_Weapon_Lsword_020_625771197",
0x2201b08a,"MainField_Weapon_Lsword_020_64468660",
0x75682be1,"MainField_Weapon_Lsword_020_644971915",
0x8d8bd876,"MainField_Weapon_Lsword_020_695143272",
0xb4d6fe3c,"MainField_Weapon_Lsword_020_816756448",
0x8c67bc00,"MainField_Weapon_Lsword_020_821676895",
0x79f7a258,"MainField_Weapon_Lsword_020_90961557",
0x0854b0d2,"MainField_Weapon_Lsword_020_951159655",
0x92315e93,"MainField_Weapon_Lsword_024_1036024282",
0x93afde74,"MainField_Weapon_Lsword_024_1787002457",
0xd00ea293,"MainField_Weapon_Lsword_024_2701391719",
0x1ac59d18,"MainField_Weapon_Lsword_024_3015970362",
0xbd0f0333,"MainField_Weapon_Lsword_027_409100399",
0xdc932c68,"MainField_Weapon_Lsword_029_1857683630",
0x8dc382a8,"MainField_Weapon_Lsword_029_4073077593",
0x991aec4d,"MainField_Weapon_Lsword_029_853939404",
0x15632534,"MainField_Weapon_Lsword_030_1429726699",
0x4eab1235,"MainField_Weapon_Lsword_030_144749449",
0xa1d7d4dc,"MainField_Weapon_Lsword_030_189712426",
0x0c188d61,"MainField_Weapon_Lsword_030_2071528294",
0x3ac92816,"MainField_Weapon_Lsword_030_227161167",
0x0ff28b3a,"MainField_Weapon_Lsword_030_2482040009",
0x669b6056,"MainField_Weapon_Lsword_030_3781362072",
0x88125ca3,"MainField_Weapon_Lsword_030_436408984",
0x24051686,"MainField_Weapon_Lsword_031_1003725872",
0xb2de2767,"MainField_Weapon_Lsword_031_1198226501",
0x0b643322,"MainField_Weapon_Lsword_031_1248142302",
0xcd15c80b,"MainField_Weapon_Lsword_031_1535266639",
0x62544308,"MainField_Weapon_Lsword_031_17048906",
0x04e38a79,"MainField_Weapon_Lsword_031_1737878368",
0xfc6b73e9,"MainField_Weapon_Lsword_031_1916759933",
0x9dde8b53,"MainField_Weapon_Lsword_031_2217205632",
0x52b9a41c,"MainField_Weapon_Lsword_031_2218696150",
0x5e500a70,"MainField_Weapon_Lsword_031_2224771998",
0xc6eec34c,"MainField_Weapon_Lsword_031_2346849858",
0x4c1e95ff,"MainField_Weapon_Lsword_031_2370959706",
0x33bfc702,"MainField_Weapon_Lsword_031_2399039262",
0xc29914f0,"MainField_Weapon_Lsword_031_2404537802",
0x39b88c9f,"MainField_Weapon_Lsword_031_2533194217",
0x012aa773,"MainField_Weapon_Lsword_031_3027841563",
0xa4ecd178,"MainField_Weapon_Lsword_031_3151302503",
0x225f5620,"MainField_Weapon_Lsword_031_3258984759",
0xb2620b2f,"MainField_Weapon_Lsword_031_3280265764",
0x43c25d46,"MainField_Weapon_Lsword_031_3288010983",
0x60ac55a7,"MainField_Weapon_Lsword_031_3407099915",
0x2a19a29d,"MainField_Weapon_Lsword_031_3415298313",
0x92d1daa3,"MainField_Weapon_Lsword_031_3540980823",
0xbc7994b1,"MainField_Weapon_Lsword_031_3541157795",
0xdfe8e1a0,"MainField_Weapon_Lsword_031_3569134434",
0x403f60a3,"MainField_Weapon_Lsword_031_3669855207",
0xe2be37fd,"MainField_Weapon_Lsword_031_3921844277",
0xafbec2e5,"MainField_Weapon_Lsword_031_4078891532",
0x865095f2,"MainField_Weapon_Lsword_031_485161632",
0x1070b3a2,"MainField_Weapon_Lsword_031_617433136",
0x87a1d72b,"MainField_Weapon_Lsword_031_849503904",
0xbb139e92,"MainField_Weapon_Lsword_031_902293433",
0x553e3b62,"MainField_Weapon_Lsword_031_926437243",
0xcab01538,"MainField_Weapon_Lsword_032_1685011161",
0x4eaafe7f,"MainField_Weapon_Lsword_032_1767031697",
0x16885832,"MainField_Weapon_Lsword_032_2156755166",
0x73b0e8fa,"MainField_Weapon_Lsword_032_2190173744",
0x743fc14e,"MainField_Weapon_Lsword_032_2433451959",
0xcabe5ca3,"MainField_Weapon_Lsword_032_2481839421",
0xa2022172,"MainField_Weapon_Lsword_032_2626614553",
0x067972e6,"MainField_Weapon_Lsword_032_2904031173",
0xfb8fc7b3,"MainField_Weapon_Lsword_032_3029378121",
0x13135c9d,"MainField_Weapon_Lsword_032_3068619006",
0x4752f9c8,"MainField_Weapon_Lsword_032_3322660940",
0x5616a998,"MainField_Weapon_Lsword_032_346005870",
0x981182b5,"MainField_Weapon_Lsword_032_3468562022",
0x0ea81d6b,"MainField_Weapon_Lsword_032_3488358072",
0xf5492ab4,"MainField_Weapon_Lsword_032_358434845",
0x1fa2ac36,"MainField_Weapon_Lsword_032_3793099266",
0x7ce5fe76,"MainField_Weapon_Lsword_032_3848565908",
0xa1af803e,"MainField_Weapon_Lsword_032_389719858",
0x77ce8ed0,"MainField_Weapon_Lsword_032_4152642370",
0xa4fa3a9d,"MainField_Weapon_Lsword_032_4191416316",
0x1ff582d1,"MainField_Weapon_Lsword_032_4206820462",
0xddfb6ace,"MainField_Weapon_Lsword_032_480954384",
0xaa3fa76b,"MainField_Weapon_Lsword_032_579469106",
0x13d75c92,"MainField_Weapon_Lsword_032_949044769",
0xe030bd2c,"MainField_Weapon_Lsword_033_1038655950",
0x70a9c3c7,"MainField_Weapon_Lsword_033_2215644113",
0xe4609c27,"MainField_Weapon_Lsword_033_48940628",
0x98f584f8,"MainField_Weapon_Lsword_033_507370149",
0x74ee3133,"MainField_Weapon_Lsword_034_3352957903",
0xdca9b085,"MainField_Weapon_Lsword_035_1759865226",
0x6c08d8cd,"MainField_Weapon_Lsword_036_1317605538",
0x890a4dc4,"MainField_Weapon_Lsword_036_2689012983",
0xbff39d02,"MainField_Weapon_Lsword_037_2125878384",
0xbbade407,"MainField_Weapon_Lsword_038_1556900446",
0x1d4aa92c,"MainField_Weapon_Lsword_038_1801719534",
0xa2287bc1,"MainField_Weapon_Lsword_038_1998312778",
0xe02ba379,"MainField_Weapon_Lsword_038_2090510651",
0xd2ae1a90,"MainField_Weapon_Lsword_038_2333211404",
0x9f18035a,"MainField_Weapon_Lsword_038_2776820103",
0xd8477c3d,"MainField_Weapon_Lsword_038_2979223687",
0xb5cd3edf,"MainField_Weapon_Lsword_038_335700083",
0x0cedcc70,"MainField_Weapon_Lsword_038_3476412993",
0xa2024d89,"MainField_Weapon_Lsword_038_3577438793",
0xdecbb632,"MainField_Weapon_Lsword_045_1152724048",
0x516e4991,"MainField_Weapon_Lsword_045_1928042578",
0x515d4b77,"MainField_Weapon_Lsword_045_2808534571",
0xffeaaf08,"MainField_Weapon_Lsword_045_3882690697",
0x565be77d,"MainField_Weapon_Lsword_045_778349076",
0x8d309ce2,"MainField_Weapon_Lsword_047_2003407569",
0xd4104fc4,"MainField_Weapon_Lsword_055_959777623",
0xcd03c3e8,"MainField_Weapon_Lsword_056_1093945475",
0x6f6e60ed,"MainField_Weapon_Lsword_056_1212608971",
0xccf5c38b,"MainField_Weapon_Lsword_056_1293808698",
0xb00c74dd,"MainField_Weapon_Lsword_056_1326019490",
0xaa6aa8c4,"MainField_Weapon_Lsword_056_170189656",
0xb4e1c242,"MainField_Weapon_Lsword_056_1856105757",
0xea2f2bf3,"MainField_Weapon_Lsword_056_2063791640",
0x6c031a5c,"MainField_Weapon_Lsword_056_2246898764",
0xdb783004,"MainField_Weapon_Lsword_056_2255169459",
0x85ab87d2,"MainField_Weapon_Lsword_056_2618326304",
0xdec8c44d,"MainField_Weapon_Lsword_056_3211706507",
0xef54fbad,"MainField_Weapon_Lsword_056_582435023",
0xc7d2f137,"MainField_Weapon_Lsword_056_85535898",
0x72a9aae4,"MainField_Weapon_Shield_001_1706835627",
0x19a8c583,"MainField_Weapon_Shield_001_1971397593",
0x8f86c83c,"MainField_Weapon_Shield_001_2036382394",
0xae6f5186,"MainField_Weapon_Shield_001_2104118373",
0x5b062446,"MainField_Weapon_Shield_001_2578848983",
0xaec9a5c0,"MainField_Weapon_Shield_001_2585899760",
0x6a7d3244,"MainField_Weapon_Shield_001_2949403616",
0x5884552c,"MainField_Weapon_Shield_001_3246695628",
0xf0b75cfb,"MainField_Weapon_Shield_001_3492833598",
0xd4619884,"MainField_Weapon_Shield_001_352416725",
0x12915337,"MainField_Weapon_Shield_001_3567246617",
0x4bc7f94a,"MainField_Weapon_Shield_001_387104477",
0x8650b74e,"MainField_Weapon_Shield_001_3988620765",
0x29125179,"MainField_Weapon_Shield_001_4161629865",
0x3f7c3024,"MainField_Weapon_Shield_001_4265236055",
0x670d4ae8,"MainField_Weapon_Shield_001_508451459",
0xbc42dbe4,"MainField_Weapon_Shield_001_594516372",
0xa430cd2f,"MainField_Weapon_Shield_001_731070699",
0xd99ad04b,"MainField_Weapon_Shield_002_1137636259",
0xb286f657,"MainField_Weapon_Shield_002_1376854905",
0x5ade5a66,"MainField_Weapon_Shield_002_1782415526",
0xccdaff52,"MainField_Weapon_Shield_002_2237886046",
0xb841d97d,"MainField_Weapon_Shield_002_2330617923",
0xd7168e88,"MainField_Weapon_Shield_002_2413841708",
0x37a8681f,"MainField_Weapon_Shield_002_25360411",
0x809fd1a1,"MainField_Weapon_Shield_002_2863421229",
0x880e29b0,"MainField_Weapon_Shield_002_2982353357",
0xd1f81f0a,"MainField_Weapon_Shield_002_623171509",
0x54bf9809,"MainField_Weapon_Shield_003_1551608170",
0xad344eb4,"MainField_Weapon_Shield_003_1836072978",
0x67bd559e,"MainField_Weapon_Shield_003_2018824967",
0xb722276d,"MainField_Weapon_Shield_003_2474244719",
0x483ccce5,"MainField_Weapon_Shield_003_2750713645",
0x6f409a82,"MainField_Weapon_Shield_003_2776194615",
0xd3f14c81,"MainField_Weapon_Shield_003_3793531503",
0xc71dcf6d,"MainField_Weapon_Shield_003_3804088777",
0x57e85579,"MainField_Weapon_Shield_003_4020283875",
0x2abcf68a,"MainField_Weapon_Shield_003_466887375",
0xebc10997,"MainField_Weapon_Shield_004_1543227815",
0xada9f1d7,"MainField_Weapon_Shield_004_1607584134",
0x33786e38,"MainField_Weapon_Shield_004_1749616083",
0xc76470c0,"MainField_Weapon_Shield_004_1948687409",
0x55444da7,"MainField_Weapon_Shield_004_2299008104",
0x34d153a5,"MainField_Weapon_Shield_004_3008234468",
0xf4853c28,"MainField_Weapon_Shield_004_314001763",
0x3fd811c1,"MainField_Weapon_Shield_004_3342825015",
0xeaa3881f,"MainField_Weapon_Shield_004_3654925152",
0xc385dfac,"MainField_Weapon_Shield_004_3804856587",
0x6cd46d32,"MainField_Weapon_Shield_004_3879491262",
0xc2ce96bd,"MainField_Weapon_Shield_004_525219141",
0x661c2721,"MainField_Weapon_Shield_005_1544197829",
0x8f8a98dc,"MainField_Weapon_Shield_005_452946685",
0xe7cf788b,"MainField_Weapon_Shield_006_1859453947",
0x16981a95,"MainField_Weapon_Shield_006_3688619286",
0x4b0a84ef,"MainField_Weapon_Shield_007_16690007",
0xf6972a27,"MainField_Weapon_Shield_007_3453440345",
0xbac33df9,"MainField_Weapon_Shield_007_4164987768",
0x95d8a00c,"MainField_Weapon_Shield_007_538949773",
0xfa14df0b,"MainField_Weapon_Shield_008_2837900210",
0x60000de4,"MainField_Weapon_Shield_021_1026274418",
0x6963de94,"MainField_Weapon_Shield_021_1190188805",
0x9d147011,"MainField_Weapon_Shield_021_1205080412",
0x7c30cd0f,"MainField_Weapon_Shield_021_1210992092",
0x684527e4,"MainField_Weapon_Shield_021_1501537886",
0x81546ab9,"MainField_Weapon_Shield_021_1623471875",
0x86a7c04b,"MainField_Weapon_Shield_021_1720507825",
0x7c3c651a,"MainField_Weapon_Shield_021_1829795741",
0x44ba7b1d,"MainField_Weapon_Shield_021_1911781620",
0x02560b8a,"MainField_Weapon_Shield_021_1993630992",
0xea794c4e,"MainField_Weapon_Shield_021_2057731097",
0xea280633,"MainField_Weapon_Shield_021_2072451464",
0x67e3819e,"MainField_Weapon_Shield_021_2072508291",
0xfc1f65bc,"MainField_Weapon_Shield_021_2116276360",
0x29d46720,"MainField_Weapon_Shield_021_221948406",
0x0c810157,"MainField_Weapon_Shield_021_2283773733",
0x183f53ea,"MainField_Weapon_Shield_021_232075737",
0x7c833eab,"MainField_Weapon_Shield_021_235461879",
0x74705d6e,"MainField_Weapon_Shield_021_2393897690",
0x7eb13240,"MainField_Weapon_Shield_021_2421107268",
0x00de8ead,"MainField_Weapon_Shield_021_2457001691",
0x1ac7d7e0,"MainField_Weapon_Shield_021_2485830572",
0x014487b6,"MainField_Weapon_Shield_021_2572393317",
0x81dc1dbe,"MainField_Weapon_Shield_021_2611394639",
0xe2cc4d0c,"MainField_Weapon_Shield_021_2680732109",
0x84f3feb3,"MainField_Weapon_Shield_021_2719200175",
0x732d16aa,"MainField_Weapon_Shield_021_2801945870",
0xc1e29a8d,"MainField_Weapon_Shield_021_2894034052",
0x954be2f7,"MainField_Weapon_Shield_021_290115350",
0xe740f53a,"MainField_Weapon_Shield_021_291742865",
0xcfd6df4b,"MainField_Weapon_Shield_021_2966307810",
0xc39d5ae5,"MainField_Weapon_Shield_021_2975544194",
0xefdc2b63,"MainField_Weapon_Shield_021_3043205304",
0x7587f831,"MainField_Weapon_Shield_021_3067636657",
0x7a31fd79,"MainField_Weapon_Shield_021_3112915571",
0x9e3bc1f2,"MainField_Weapon_Shield_021_3392066591",
0x33966d40,"MainField_Weapon_Shield_021_3397126344",
0x97344e32,"MainField_Weapon_Shield_021_3556345803",
0xbf04252a,"MainField_Weapon_Shield_021_3596553082",
0x18f59459,"MainField_Weapon_Shield_021_3603013560",
0x1e4c9967,"MainField_Weapon_Shield_021_3626627887",
0x7423850b,"MainField_Weapon_Shield_021_363835820",
0x80e27960,"MainField_Weapon_Shield_021_3705577957",
0x74329923,"MainField_Weapon_Shield_021_3761939497",
0xd6789514,"MainField_Weapon_Shield_021_394249698",
0x722f0a86,"MainField_Weapon_Shield_021_4049222196",
0x993fdcdf,"MainField_Weapon_Shield_021_4098951282",
0xec0fe598,"MainField_Weapon_Shield_021_4125016520",
0x830cd403,"MainField_Weapon_Shield_021_412858250",
0x07185b7f,"MainField_Weapon_Shield_021_4130449374",
0x351db5ca,"MainField_Weapon_Shield_021_4133321014",
0x1892d127,"MainField_Weapon_Shield_021_41600110",
0xad5e37aa,"MainField_Weapon_Shield_021_4250802393",
0xad8c5724,"MainField_Weapon_Shield_021_4261180890",
0x088baf24,"MainField_Weapon_Shield_021_4270682191",
0x2c40417a,"MainField_Weapon_Shield_021_602175703",
0x85e6249b,"MainField_Weapon_Shield_021_636907837",
0x0f224c2a,"MainField_Weapon_Shield_021_655891830",
0x1cc7114d,"MainField_Weapon_Shield_021_70459680",
0xd3b8c65f,"MainField_Weapon_Shield_021_846010223",
0x0d8458cb,"MainField_Weapon_Shield_021_861119578",
0x95af3780,"MainField_Weapon_Shield_022_2015618507",
0x2832cc0f,"MainField_Weapon_Shield_022_386082466",
0x2d0d2ef9,"MainField_Weapon_Shield_023_1102799451",
0xff5531e1,"MainField_Weapon_Shield_023_2900899402",
0xd5db1bf7,"MainField_Weapon_Shield_025_2462483991",
0x1925ee82,"MainField_Weapon_Shield_026_2607499047",
0x5585a604,"MainField_Weapon_Shield_031_3801681406",
0x36ced704,"MainField_Weapon_Shield_031_3893327396",
0x21d180ac,"MainField_Weapon_Shield_032_1349809041",
0x4352b93d,"MainField_Weapon_Shield_032_1495640543",
0x509fd2e8,"MainField_Weapon_Shield_032_2193224233",
0xb0eb40ee,"MainField_Weapon_Shield_032_679192469",
0x38876ad6,"MainField_Weapon_Shield_033_2722580545",
0x52459c6f,"MainField_Weapon_Shield_033_2842165950",
0x149d3fad,"MainField_Weapon_Shield_034_1143390168",
0x6d4edc4f,"MainField_Weapon_Shield_034_2408116231",
0xd63b7e15,"MainField_Weapon_Shield_034_3517955530",
0xb0cf093d,"MainField_Weapon_Shield_034_765483804",
0x47c73193,"MainField_Weapon_Shield_035_2339210759",
0xc232ae23,"MainField_Weapon_Shield_035_2402879075",
0xf32bcf4a,"MainField_Weapon_Shield_035_2627896173",
0xb9de9e68,"MainField_Weapon_Shield_035_2740788695",
0x7a4bc494,"MainField_Weapon_Shield_035_3052406802",
0x67924300,"MainField_Weapon_Shield_035_4025293028",
0x43729a36,"MainField_Weapon_Shield_035_4039130780",
0x0f33a17b,"MainField_Weapon_Shield_035_76487311",
0xe86c5e0d,"MainField_Weapon_Shield_035_778127876",
0x6db175e8,"MainField_Weapon_Shield_036_2298705486",
0x17cd60fa,"MainField_Weapon_Shield_040_1348374492",
0x86362fa2,"MainField_Weapon_Shield_040_1461701600",
0x2879b221,"MainField_Weapon_Shield_040_1529298769",
0x9e846b25,"MainField_Weapon_Shield_040_1732496797",
0x88b3e269,"MainField_Weapon_Shield_040_2123865193",
0x395d98a9,"MainField_Weapon_Shield_040_246142587",
0x01e02f13,"MainField_Weapon_Shield_040_2553861918",
0x0bd281af,"MainField_Weapon_Shield_040_2574679187",
0xf5584463,"MainField_Weapon_Shield_040_2964296916",
0x2fa938a8,"MainField_Weapon_Shield_040_3322352575",
0xe4570948,"MainField_Weapon_Shield_040_3552672491",
0xa82bd1f7,"MainField_Weapon_Shield_040_394653421",
0xe055e3ff,"MainField_Weapon_Shield_040_450406350",
0x6c9f3312,"MainField_Weapon_Shield_040_494896769",
0x43cd974e,"MainField_Weapon_Shield_040_871017963",
0xfa5a0552,"MainField_Weapon_Shield_042_328092650",
0x18b23e97,"MainField_Weapon_Spear_001_1485032943",
0x90ad3a80,"MainField_Weapon_Spear_001_1702432567",
0x3bc34199,"MainField_Weapon_Spear_001_2033979031",
0x041bf7a2,"MainField_Weapon_Spear_001_2558406262",
0xbb6cf64a,"MainField_Weapon_Spear_001_4109794303",
0x2a52dbee,"MainField_Weapon_Spear_001_697126769",
0x1cac99b4,"MainField_Weapon_Spear_001_802496562",
0x37baadf2,"MainField_Weapon_Spear_002_1294222788",
0x4eafe5e0,"MainField_Weapon_Spear_002_139371048",
0xc50ee062,"MainField_Weapon_Spear_002_1862732601",
0x7d02e5e1,"MainField_Weapon_Spear_002_1916313309",
0x4144c8fd,"MainField_Weapon_Spear_002_1942071887",
0x64fa9c7e,"MainField_Weapon_Spear_002_2116328295",
0x2a3e04e5,"MainField_Weapon_Spear_002_29985905",
0xe379b1d8,"MainField_Weapon_Spear_002_3461007953",
0x23e86a9b,"MainField_Weapon_Spear_002_3711627533",
0x8f7a2ea7,"MainField_Weapon_Spear_003_2067764370",
0x820dae40,"MainField_Weapon_Spear_003_2181366418",
0x7c17e685,"MainField_Weapon_Spear_003_2541548891",
0xe822cbf5,"MainField_Weapon_Spear_003_2853040604",
0x47707379,"MainField_Weapon_Spear_003_3789597752",
0x522b9c9c,"MainField_Weapon_Spear_003_3904590619",
0xca05e72a,"MainField_Weapon_Spear_004_1008850007",
0xdaf2f35b,"MainField_Weapon_Spear_004_1072548040",
0x4c376a5c,"MainField_Weapon_Spear_004_1128065077",
0xda2a0c26,"MainField_Weapon_Spear_004_1179121918",
0x7be61616,"MainField_Weapon_Spear_004_1186995313",
0xd1a5b8d5,"MainField_Weapon_Spear_004_1498478900",
0xc03a7062,"MainField_Weapon_Spear_004_1519710189",
0xbc4337c0,"MainField_Weapon_Spear_004_1549041402",
0x2b3d8ce5,"MainField_Weapon_Spear_004_1602887767",
0x231b6b63,"MainField_Weapon_Spear_004_165822594",
0x0986bd37,"MainField_Weapon_Spear_004_1660867908",
0x95bc52e0,"MainField_Weapon_Spear_004_1681809870",
0xfad397f2,"MainField_Weapon_Spear_004_177207181",
0x3cf72dbd,"MainField_Weapon_Spear_004_1839732886",
0x988b9515,"MainField_Weapon_Spear_004_1871662092",
0x6705823d,"MainField_Weapon_Spear_004_1874563027",
0x7485d91e,"MainField_Weapon_Spear_004_1935432479",
0x70c22d6d,"MainField_Weapon_Spear_004_1965077930",
0xee258e2d,"MainField_Weapon_Spear_004_1973209608",
0xf66e8329,"MainField_Weapon_Spear_004_1973879504",
0xb019b9bc,"MainField_Weapon_Spear_004_2017664332",
0xbf12ec7b,"MainField_Weapon_Spear_004_2270086065",
0x8a7ff181,"MainField_Weapon_Spear_004_2365909677",
0x72fa99df,"MainField_Weapon_Spear_004_2659013035",
0xd59cf7ac,"MainField_Weapon_Spear_004_2701269000",
0x419bb4da,"MainField_Weapon_Spear_004_279360433",
0xffe6aee8,"MainField_Weapon_Spear_004_2823652434",
0xbc990b87,"MainField_Weapon_Spear_004_282914900",
0x152c4fc8,"MainField_Weapon_Spear_004_2906739439",
0xffb4921f,"MainField_Weapon_Spear_004_2945796918",
0xbac25fc9,"MainField_Weapon_Spear_004_3089696425",
0xf5937679,"MainField_Weapon_Spear_004_3146194708",
0xfd97fcd2,"MainField_Weapon_Spear_004_3199492549",
0xf14fc30b,"MainField_Weapon_Spear_004_325091086",
0xa5035684,"MainField_Weapon_Spear_004_3253198237",
0xf8ccafc3,"MainField_Weapon_Spear_004_3280424430",
0xd20c1308,"MainField_Weapon_Spear_004_3298435958",
0x463d26ea,"MainField_Weapon_Spear_004_3356569881",
0xd3c0ad81,"MainField_Weapon_Spear_004_34181678",
0xe6aa738a,"MainField_Weapon_Spear_004_3434163177",
0x8c04db6c,"MainField_Weapon_Spear_004_3503206498",
0xf6de8d05,"MainField_Weapon_Spear_004_3573194646",
0xf324fd6b,"MainField_Weapon_Spear_004_3602414650",
0x83e8b08e,"MainField_Weapon_Spear_004_361228444",
0xff1c9ef8,"MainField_Weapon_Spear_004_3687464874",
0x4e438c67,"MainField_Weapon_Spear_004_3711224462",
0x1ccd1eb7,"MainField_Weapon_Spear_004_3810991989",
0xf66c6104,"MainField_Weapon_Spear_004_4064079723",
0xa8100416,"MainField_Weapon_Spear_004_4067852417",
0x99299d5a,"MainField_Weapon_Spear_004_457986179",
0x51525dfc,"MainField_Weapon_Spear_004_957629606",
0x8f519d3d,"MainField_Weapon_Spear_005_4041862368",
0xb07a7d27,"MainField_Weapon_Spear_007_1186855975",
0xc793dca8,"MainField_Weapon_Spear_007_1757264564",
0xe3ff048d,"MainField_Weapon_Spear_007_1859179689",
0x339f7055,"MainField_Weapon_Spear_007_1901459934",
0x5ceeb597,"MainField_Weapon_Spear_007_2314358408",
0x7365a32e,"MainField_Weapon_Spear_007_2705716107",
0x4912ba0d,"MainField_Weapon_Spear_007_3684785179",
0x26825032,"MainField_Weapon_Spear_007_863043235",
0x52e52da7,"MainField_Weapon_Spear_008_3214366534",
0xc027592f,"MainField_Weapon_Spear_008_3268125890",
0x5567bcd7,"MainField_Weapon_Spear_008_351084550",
0x4413d8d3,"MainField_Weapon_Spear_008_3544913124",
0x958b616e,"MainField_Weapon_Spear_008_3720217149",
0x147a39ec,"MainField_Weapon_Spear_008_4168409737",
0x3889b9ae,"MainField_Weapon_Spear_009_1113055412",
0xf6487652,"MainField_Weapon_Spear_009_1633864555",
0x5126e266,"MainField_Weapon_Spear_009_3505002396",
0x4788575b,"MainField_Weapon_Spear_009_3671433913",
0x2866ec03,"MainField_Weapon_Spear_010_4084191859",
0x6da459a4,"MainField_Weapon_Spear_011_2482358545",
0xece3be57,"MainField_Weapon_Spear_011_3371251221",
0xceec5013,"MainField_Weapon_Spear_012_2199646063",
0x3255b580,"MainField_Weapon_Spear_012_2373238719",
0x52430ff2,"MainField_Weapon_Spear_012_3139872877",
0xd42698ae,"MainField_Weapon_Spear_012_3452026958",
0x485e5086,"MainField_Weapon_Spear_021_1187515891",
0x605edb87,"MainField_Weapon_Spear_021_1202292342",
0x2088625b,"MainField_Weapon_Spear_021_1245650050",
0x9cf87068,"MainField_Weapon_Spear_021_1349958830",
0x0aaca9ae,"MainField_Weapon_Spear_021_1448906168",
0x2d2af0c4,"MainField_Weapon_Spear_021_1451707291",
0x3f1bc8dd,"MainField_Weapon_Spear_021_1513643559",
0xe7039261,"MainField_Weapon_Spear_021_1552334017",
0xf854e645,"MainField_Weapon_Spear_021_1565685674",
0x147f12c2,"MainField_Weapon_Spear_021_1635415352",
0x80e5a3d6,"MainField_Weapon_Spear_021_1649136016",
0xbbc4d8dc,"MainField_Weapon_Spear_021_1782056780",
0x4442775c,"MainField_Weapon_Spear_021_1837152091",
0x883e8040,"MainField_Weapon_Spear_021_1853396883",
0xa78cad1c,"MainField_Weapon_Spear_021_185366284",
0x94ce551b,"MainField_Weapon_Spear_021_1855603921",
0x7c6be505,"MainField_Weapon_Spear_021_1946556624",
0x05c9e774,"MainField_Weapon_Spear_021_2081723260",
0xdde74126,"MainField_Weapon_Spear_021_2112670695",
0x8fc577e8,"MainField_Weapon_Spear_021_2123828106",
0xaead452a,"MainField_Weapon_Spear_021_2190012119",
0xa8ad56d4,"MainField_Weapon_Spear_021_2269912420",
0x5d35b043,"MainField_Weapon_Spear_021_2515959821",
0x3e0d9e9f,"MainField_Weapon_Spear_021_2527925167",
0xaee4dba7,"MainField_Weapon_Spear_021_2697890885",
0x3fdc5e93,"MainField_Weapon_Spear_021_2698697909",
0x57ca92cf,"MainField_Weapon_Spear_021_2818564618",
0x7cf9afde,"MainField_Weapon_Spear_021_2820829596",
0x7080c307,"MainField_Weapon_Spear_021_3229703979",
0x7c3f8d97,"MainField_Weapon_Spear_021_3312178841",
0xf138be4e,"MainField_Weapon_Spear_021_3325638779",
0xa20e4b5b,"MainField_Weapon_Spear_021_3363283685",
0x605a7b68,"MainField_Weapon_Spear_021_3400895360",
0xeabcf999,"MainField_Weapon_Spear_021_3589440996",
0xaa73118f,"MainField_Weapon_Spear_021_3632297827",
0x51ee517a,"MainField_Weapon_Spear_021_3661116961",
0x27e803ae,"MainField_Weapon_Spear_021_3888734910",
0xf2d18b15,"MainField_Weapon_Spear_021_3933797794",
0x04f5fe14,"MainField_Weapon_Spear_021_39639157",
0xeadd642f,"MainField_Weapon_Spear_021_3972089330",
0xc3f01339,"MainField_Weapon_Spear_021_3996608889",
0x906a71a6,"MainField_Weapon_Spear_021_4021340368",
0x8407d62f,"MainField_Weapon_Spear_021_407627143",
0xb9f8331c,"MainField_Weapon_Spear_021_421260041",
0xae802024,"MainField_Weapon_Spear_021_4282566044",
0xda565418,"MainField_Weapon_Spear_021_4293344309",
0xf68ae9aa,"MainField_Weapon_Spear_021_652708931",
0x9e1a4c74,"MainField_Weapon_Spear_021_669451613",
0x639b282e,"MainField_Weapon_Spear_021_715236419",
0xf9f80a37,"MainField_Weapon_Spear_021_781139463",
0xc4f6cc2a,"MainField_Weapon_Spear_021_785177218",
0x059f49bd,"MainField_Weapon_Spear_021_846743701",
0xa3bf2b04,"MainField_Weapon_Spear_021_919109892",
0xd6e371fa,"MainField_Weapon_Spear_022_1167637704",
0x954a15ae,"MainField_Weapon_Spear_022_1201245983",
0x1ef196fd,"MainField_Weapon_Spear_022_1605846467",
0x734337cf,"MainField_Weapon_Spear_022_2423504276",
0x619482c3,"MainField_Weapon_Spear_022_2618448119",
0x977b16e7,"MainField_Weapon_Spear_022_2951605849",
0x667fc125,"MainField_Weapon_Spear_022_3872553955",
0x9636323d,"MainField_Weapon_Spear_022_3948683089",
0xe8ee7eed,"MainField_Weapon_Spear_022_4198469656",
0x6316a754,"MainField_Weapon_Spear_022_715992487",
0x3fe2823f,"MainField_Weapon_Spear_024_3166562877",
0x89d6b14a,"MainField_Weapon_Spear_025_1841342358",
0x731d9e70,"MainField_Weapon_Spear_027_3822044971",
0x9ba6208b,"MainField_Weapon_Spear_027_4292542013",
0x3f49da6b,"MainField_Weapon_Spear_028_3024909116",
0xe4a0f927,"MainField_Weapon_Spear_029_2020879773",
0x11cf0e9f,"MainField_Weapon_Spear_029_3934566842",
0x8b4ed39f,"MainField_Weapon_Spear_030_12345023",
0x63d64105,"MainField_Weapon_Spear_030_2546004187",
0x7ad1d6b7,"MainField_Weapon_Spear_030_2832327872",
0x5b55c917,"MainField_Weapon_Spear_030_3107315143",
0x074fe3f6,"MainField_Weapon_Spear_030_3662671404",
0xc289bc03,"MainField_Weapon_Spear_030_4020411317",
0x30de9b10,"MainField_Weapon_Spear_030_4242486507",
0xd5fea9ec,"MainField_Weapon_Spear_031_472713211",
0xe0334ba5,"MainField_Weapon_Spear_033_100321491",
0x9e9f4fc4,"MainField_Weapon_Spear_036_1752020925",
0xb17b4fee,"MainField_Weapon_Spear_036_2228674143",
0x991448bc,"MainField_Weapon_Spear_036_2610576110",
0x9ab0d46a,"MainField_Weapon_Spear_036_2707616155",
0xbf6822fd,"MainField_Weapon_Spear_036_3310959163",
0x24c5f370,"MainField_Weapon_Spear_038_1590954853",
0x8ce67813,"MainField_Weapon_Spear_038_1882970610",
0x69fa7c76,"MainField_Weapon_Spear_038_2313417513",
0xdba6a733,"MainField_Weapon_Spear_038_3082268240",
0xe40376e4,"MainField_Weapon_Spear_038_3109048199",
0xca1cb443,"MainField_Weapon_Spear_038_385316526",
0x38cc73c6,"MainField_Weapon_Spear_038_701709299",
0x111fce3a,"MainField_Weapon_Spear_047_2892188459",
0xb94e5dee,"MainField_Weapon_Spear_049_1617641678",
0x540ee9f4,"MainField_Weapon_Sword_001_1154531834",
0xd1d34bb9,"MainField_Weapon_Sword_001_1278389905",
0x70fe5597,"MainField_Weapon_Sword_001_1383303370",
0x70a777f2,"MainField_Weapon_Sword_001_1612025419",
0xfb863df7,"MainField_Weapon_Sword_001_1878465865",
0x373a9524,"MainField_Weapon_Sword_001_2236450571",
0x5bd1c967,"MainField_Weapon_Sword_001_2565482610",
0x76afd213,"MainField_Weapon_Sword_001_2834643114",
0x491dd214,"MainField_Weapon_Sword_001_3240633345",
0x42dcdcd6,"MainField_Weapon_Sword_001_4213021332",
0x516b3c78,"MainField_Weapon_Sword_001_4283558764",
0x27c1ef90,"MainField_Weapon_Sword_001_513250660",
0xc38344b9,"MainField_Weapon_Sword_001_684324395",
0x4a305456,"MainField_Weapon_Sword_001_862223036",
0x8563c2e8,"MainField_Weapon_Sword_001_895942674",
0xab860ecb,"MainField_Weapon_Sword_001_987072729",
0x94afb0fd,"MainField_Weapon_Sword_002_1135692838",
0x1f306fdb,"MainField_Weapon_Sword_002_1521547542",
0x1ef0a660,"MainField_Weapon_Sword_002_1523865505",
0x31aa29a1,"MainField_Weapon_Sword_002_1791471750",
0x2ded6166,"MainField_Weapon_Sword_002_2366191482",
0xd74a763d,"MainField_Weapon_Sword_002_3021937063",
0x43d2cd7a,"MainField_Weapon_Sword_002_319295042",
0x4ce12527,"MainField_Weapon_Sword_002_4072730348",
0xacf225f6,"MainField_Weapon_Sword_002_438454197",
0xee2ccdcd,"MainField_Weapon_Sword_002_802292516",
0x6bee7d2f,"MainField_Weapon_Sword_002_860762412",
0x06a717ec,"MainField_Weapon_Sword_003_1601676551",
0x65e53775,"MainField_Weapon_Sword_003_1770685778",
0x58b38e66,"MainField_Weapon_Sword_003_1851909563",
0x9480616e,"MainField_Weapon_Sword_003_2121293970",
0xe12d8af5,"MainField_Weapon_Sword_003_3025162942",
0x4b589063,"MainField_Weapon_Sword_003_920902179",
0x24ee7a4c,"MainField_Weapon_Sword_004_1037677709",
0xd8faa6e7,"MainField_Weapon_Sword_004_1148223985",
0x8dc85672,"MainField_Weapon_Sword_004_121505861",
0xcf4be8c9,"MainField_Weapon_Sword_004_1565859272",
0x14292525,"MainField_Weapon_Sword_004_1756509703",
0xa4f632fb,"MainField_Weapon_Sword_004_1919342319",
0xbeea547b,"MainField_Weapon_Sword_004_1928068136",
0x508348ff,"MainField_Weapon_Sword_004_2128844072",
0x0823ebe5,"MainField_Weapon_Sword_004_2259440452",
0xf1ffccf5,"MainField_Weapon_Sword_004_2476617033",
0x1f86036c,"MainField_Weapon_Sword_004_2591364711",
0x38503707,"MainField_Weapon_Sword_004_283800169",
0xe4c7e0a3,"MainField_Weapon_Sword_004_2842228485",
0xb1244194,"MainField_Weapon_Sword_004_2913580517",
0x31af4bbe,"MainField_Weapon_Sword_004_2944214445",
0x5818856d,"MainField_Weapon_Sword_004_2963383495",
0x50bf1c30,"MainField_Weapon_Sword_004_3421672584",
0xb87e7a51,"MainField_Weapon_Sword_004_3433389509",
0x74327dee,"MainField_Weapon_Sword_004_3456582461",
0x4f93a431,"MainField_Weapon_Sword_004_3513036201",
0x11318d4c,"MainField_Weapon_Sword_004_4054541266",
0xa0e15c58,"MainField_Weapon_Sword_004_4140478773",
0xc5ca560f,"MainField_Weapon_Sword_004_4232079624",
0xa1e96bc8,"MainField_Weapon_Sword_004_4247417812",
0xa70fedb2,"MainField_Weapon_Sword_004_60256358",
0xb98af8b5,"MainField_Weapon_Sword_004_715941349",
0xd6ab9826,"MainField_Weapon_Sword_004_721741992",
0x2a7b2e3f,"MainField_Weapon_Sword_004_726212617",
0x2ddf4249,"MainField_Weapon_Sword_004_904013311",
0x445fdf59,"MainField_Weapon_Sword_005_2805270067",
0xba83626b,"MainField_Weapon_Sword_005_3292532777",
0x2808bfe7,"MainField_Weapon_Sword_006_1111079703",
0xdbd4a44d,"MainField_Weapon_Sword_006_2209834199",
0x3a6ce556,"MainField_Weapon_Sword_006_3452537320",
0xfa83ceaf,"MainField_Weapon_Sword_007_1210450490",
0xf3564291,"MainField_Weapon_Sword_007_1993321627",
0xef602181,"MainField_Weapon_Sword_007_2262353385",
0x2b72bb5b,"MainField_Weapon_Sword_007_3339868224",
0x5e96bf61,"MainField_Weapon_Sword_007_391527232",
0x1da84a47,"MainField_Weapon_Sword_007_623919826",
0x5c14e55b,"MainField_Weapon_Sword_007_718975380",
0x6b6457ba,"MainField_Weapon_Sword_008_1159558445",
0x5e5193cd,"MainField_Weapon_Sword_008_1818521644",
0x61a7df5f,"MainField_Weapon_Sword_008_2962549997",
0x8cf1cc9b,"MainField_Weapon_Sword_008_881840419",
0xf8223cd7,"MainField_Weapon_Sword_009_65380107",
0x0f1071ca,"MainField_Weapon_Sword_019_185224578",
0xc5a034a8,"MainField_Weapon_Sword_019_2231773671",
0xc9101342,"MainField_Weapon_Sword_021_1027557987",
0x0e0f6904,"MainField_Weapon_Sword_021_1028410832",
0xe7092e68,"MainField_Weapon_Sword_021_1114992135",
0x6d3ac8bf,"MainField_Weapon_Sword_021_122047548",
0xa497c4d1,"MainField_Weapon_Sword_021_1294248255",
0x76e5c58c,"MainField_Weapon_Sword_021_131111423",
0xb15ef6cd,"MainField_Weapon_Sword_021_1366707485",
0x1c25d58f,"MainField_Weapon_Sword_021_1411549357",
0x2458de9f,"MainField_Weapon_Sword_021_1496698408",
0x6646885a,"MainField_Weapon_Sword_021_151126172",
0xaa37d259,"MainField_Weapon_Sword_021_1514781468",
0x02f34f17,"MainField_Weapon_Sword_021_1517240306",
0xe90666d8,"MainField_Weapon_Sword_021_1517385728",
0xe1b7ac22,"MainField_Weapon_Sword_021_1640259547",
0xb098e3b9,"MainField_Weapon_Sword_021_1707213344",
0x3a32f215,"MainField_Weapon_Sword_021_1718002010",
0x2457dcbf,"MainField_Weapon_Sword_021_1824032073",
0x76f368de,"MainField_Weapon_Sword_021_1868468147",
0x11989f88,"MainField_Weapon_Sword_021_193754326",
0xd36b9dda,"MainField_Weapon_Sword_021_1944652323",
0x46676a4b,"MainField_Weapon_Sword_021_1986938606",
0x93bd106b,"MainField_Weapon_Sword_021_2056071821",
0x4c77b283,"MainField_Weapon_Sword_021_2188596792",
0x74815b21,"MainField_Weapon_Sword_021_2272113126",
0xc62958f2,"MainField_Weapon_Sword_021_2351247936",
0x76b574ea,"MainField_Weapon_Sword_021_2529509228",
0x00c9e82a,"MainField_Weapon_Sword_021_2603760569",
0x163e50e9,"MainField_Weapon_Sword_021_2604353124",
0x119195b1,"MainField_Weapon_Sword_021_2634563034",
0xce54383a,"MainField_Weapon_Sword_021_2785313309",
0x29130abb,"MainField_Weapon_Sword_021_2851828073",
0xb41cd2d6,"MainField_Weapon_Sword_021_2916093460",
0x87ccb6f4,"MainField_Weapon_Sword_021_3083932162",
0xb29662fe,"MainField_Weapon_Sword_021_3093023553",
0x263e0551,"MainField_Weapon_Sword_021_3098282663",
0x44c972d4,"MainField_Weapon_Sword_021_3132937476",
0x652c4d48,"MainField_Weapon_Sword_021_3341795097",
0xc62ec274,"MainField_Weapon_Sword_021_3350980237",
0xfb4bac4b,"MainField_Weapon_Sword_021_3372444099",
0x1f5f8253,"MainField_Weapon_Sword_021_341487003",
0xa6dc64f6,"MainField_Weapon_Sword_021_3501745213",
0x113ff31b,"MainField_Weapon_Sword_021_3571464209",
0x46dfe80e,"MainField_Weapon_Sword_021_3590017291",
0xf328ef3b,"MainField_Weapon_Sword_021_3676040608",
0x4c88831e,"MainField_Weapon_Sword_021_3686547491",
0x7b4a2da1,"MainField_Weapon_Sword_021_3726979336",
0xcabaaa2e,"MainField_Weapon_Sword_021_3740772979",
0xe4159978,"MainField_Weapon_Sword_021_3783789786",
0x22ffa6cf,"MainField_Weapon_Sword_021_3992971796",
0x751a9eb4,"MainField_Weapon_Sword_021_4101996471",
0xc93b9505,"MainField_Weapon_Sword_021_4176180062",
0x0f4eb95a,"MainField_Weapon_Sword_021_419184373",
0x6f508d74,"MainField_Weapon_Sword_021_4238269733",
0x8d21920c,"MainField_Weapon_Sword_021_434112131",
0x4988b961,"MainField_Weapon_Sword_021_518102224",
0x2c1cf21f,"MainField_Weapon_Sword_021_59463247",
0xe6277083,"MainField_Weapon_Sword_021_619736918",
0x9d1eea46,"MainField_Weapon_Sword_021_655630567",
0xc6bd32d3,"MainField_Weapon_Sword_021_768516789",
0x07c874bc,"MainField_Weapon_Sword_021_806947824",
0x2ef23c1a,"MainField_Weapon_Sword_021_858164246",
0x600eeac4,"MainField_Weapon_Sword_021_916701448",
0xe266f0cb,"MainField_Weapon_Sword_021_97837659",
0xa216bef5,"MainField_Weapon_Sword_022_1454847272",
0x4d8d6f01,"MainField_Weapon_Sword_022_1475473118",
0x231c4751,"MainField_Weapon_Sword_022_1791960524",
0x1bb8a638,"MainField_Weapon_Sword_022_2760349592",
0xb3b3a76f,"MainField_Weapon_Sword_022_3521361767",
0xcb494fb0,"MainField_Weapon_Sword_022_3846019489",
0x31d11e2f,"MainField_Weapon_Sword_022_3985283545",
0xa8c6d6c6,"MainField_Weapon_Sword_022_4108932436",
0xff6de008,"MainField_Weapon_Sword_024_3028936021",
0x50b89387,"MainField_Weapon_Sword_025_1749082723",
0xd413792d,"MainField_Weapon_Sword_027_1398824038",
0xb93886c7,"MainField_Weapon_Sword_029_1596317762",
0x2484adb6,"MainField_Weapon_Sword_029_2170037149",
0xa2d3cab2,"MainField_Weapon_Sword_029_772217572",
0x2e8b526c,"MainField_Weapon_Sword_030_1825436865",
0x47c7bea0,"MainField_Weapon_Sword_035_2248086706",
0x01374be3,"MainField_Weapon_Sword_041_3686635822",
0xcbfc9661,"MainField_Weapon_Sword_043_1046727917",
0xac4be051,"MainField_Weapon_Sword_043_1080478767",
0x04673c80,"MainField_Weapon_Sword_043_1110933843",
0x53039a48,"MainField_Weapon_Sword_043_1116040178",
0xb21e5cab,"MainField_Weapon_Sword_043_1172877291",
0xc24dcdc0,"MainField_Weapon_Sword_043_1268135547",
0xda2f2a56,"MainField_Weapon_Sword_043_1387944112",
0x9407e871,"MainField_Weapon_Sword_043_1491868917",
0xa1238d98,"MainField_Weapon_Sword_043_1521734217",
0xd10e8c53,"MainField_Weapon_Sword_043_1571873686",
0x48c4fea2,"MainField_Weapon_Sword_043_1637930170",
0xf8ba76af,"MainField_Weapon_Sword_043_1672688940",
0xd4995e76,"MainField_Weapon_Sword_043_1722067734",
0xe784722b,"MainField_Weapon_Sword_043_1873237327",
0x0afb3eff,"MainField_Weapon_Sword_043_1874732902",
0x9761a6da,"MainField_Weapon_Sword_043_1935556204",
0x0a5ed349,"MainField_Weapon_Sword_043_208415607",
0xe0b62c6c,"MainField_Weapon_Sword_043_2149902977",
0x36e2fb91,"MainField_Weapon_Sword_043_2171568438",
0xd3654f52,"MainField_Weapon_Sword_043_2193937295",
0x7df5a440,"MainField_Weapon_Sword_043_2277029169",
0x2ab1c691,"MainField_Weapon_Sword_043_2339382476",
0x8c8009c7,"MainField_Weapon_Sword_043_2366179476",
0x624db2dc,"MainField_Weapon_Sword_043_2583359693",
0x1c68acd9,"MainField_Weapon_Sword_043_2635253907",
0x1d63be8c,"MainField_Weapon_Sword_043_2717751223",
0xd40d9c5a,"MainField_Weapon_Sword_043_2845637404",
0x658d9b24,"MainField_Weapon_Sword_043_2884030151",
0x81d27ac1,"MainField_Weapon_Sword_043_2897557164",
0x3612c281,"MainField_Weapon_Sword_043_3060620500",
0xc39d91f8,"MainField_Weapon_Sword_043_314609705",
0x126395a2,"MainField_Weapon_Sword_043_3203435266",
0x4b1e0634,"MainField_Weapon_Sword_043_3287820889",
0x20150405,"MainField_Weapon_Sword_043_3299180686",
0xe8f5699c,"MainField_Weapon_Sword_043_3354818765",
0x6ae142c1,"MainField_Weapon_Sword_043_3376283355",
0x675489ff,"MainField_Weapon_Sword_043_3435092714",
0x5d25ee13,"MainField_Weapon_Sword_043_3486368486",
0xf345cf78,"MainField_Weapon_Sword_043_3619664287",
0x8d8ac831,"MainField_Weapon_Sword_043_3677755242",
0x299da59f,"MainField_Weapon_Sword_043_3696114159",
0xf80fc3bb,"MainField_Weapon_Sword_043_3705798225",
0x3f7426f1,"MainField_Weapon_Sword_043_3816590430",
0x355c5939,"MainField_Weapon_Sword_043_4012016163",
0xbc14397a,"MainField_Weapon_Sword_043_4013478541",
0xb7326507,"MainField_Weapon_Sword_043_4128862195",
0x4652948d,"MainField_Weapon_Sword_043_4142803204",
0x1062cf1e,"MainField_Weapon_Sword_043_4194118815",
0xf7800717,"MainField_Weapon_Sword_043_4234198514",
0xecbb9f8a,"MainField_Weapon_Sword_043_459777025",
0xf64086d3,"MainField_Weapon_Sword_043_62821711",
0x9c2d545a,"MainField_Weapon_Sword_043_68430968",
0x34550f1b,"MainField_Weapon_Sword_043_692636644",
0x4b7711f4,"MainField_Weapon_Sword_043_811211574",
0xb0430cd5,"MainField_Weapon_Sword_043_874088710",
0xe8a2dcf7,"MainField_Weapon_Sword_043_932210875",
0x8d1c32b1,"MainField_Weapon_Sword_043_947616289",
0x802966c5,"MainField_Weapon_Sword_043_988022808",
0x1f752d0c,"MainField_Weapon_Sword_044_1394597338",
0x9f577162,"MainField_Weapon_Sword_044_1399955322",
0x4159cb30,"MainField_Weapon_Sword_044_1465431867",
0x3502db88,"MainField_Weapon_Sword_044_1732135326",
0x6426167f,"MainField_Weapon_Sword_044_1892880072",
0xc49cad7f,"MainField_Weapon_Sword_044_1900407929",
0x6ffcc0a0,"MainField_Weapon_Sword_044_1910541071",
0xe23150df,"MainField_Weapon_Sword_044_1970982377",
0x6ca64187,"MainField_Weapon_Sword_044_2025527455",
0x96b588e0,"MainField_Weapon_Sword_044_2036679395",
0x8c4b6899,"MainField_Weapon_Sword_044_2141441599",
0xc9396078,"MainField_Weapon_Sword_044_2192753339",
0xbe45a2c6,"MainField_Weapon_Sword_044_2246054512",
0x66cb513d,"MainField_Weapon_Sword_044_2311956596",
0x20f8c199,"MainField_Weapon_Sword_044_2431921358",
0x437fc59c,"MainField_Weapon_Sword_044_272201992",
0x2789b202,"MainField_Weapon_Sword_044_2789486374",
0xe1dabddc,"MainField_Weapon_Sword_044_2827070599",
0x7d861ee9,"MainField_Weapon_Sword_044_3047528043",
0x737bc83f,"MainField_Weapon_Sword_044_3109475863",
0xf1b218f3,"MainField_Weapon_Sword_044_3114879794",
0xb9251c2c,"MainField_Weapon_Sword_044_3154057015",
0xad3d3d34,"MainField_Weapon_Sword_044_3267164544",
0xad645bc8,"MainField_Weapon_Sword_044_3602277159",
0xb2c63eef,"MainField_Weapon_Sword_044_3812431918",
0x21cd92d3,"MainField_Weapon_Sword_044_3860349554",
0xddd73cca,"MainField_Weapon_Sword_044_3923402610",
0x304a2409,"MainField_Weapon_Sword_044_3971415913",
0xd4a86a95,"MainField_Weapon_Sword_044_404092506",
0x18cb5ed0,"MainField_Weapon_Sword_044_4057645397",
0x3409ee47,"MainField_Weapon_Sword_044_4155595503",
0x06362c6d,"MainField_Weapon_Sword_044_4255991944",
0xe8226785,"MainField_Weapon_Sword_044_519444861",
0xde34dc74,"MainField_Weapon_Sword_044_543131565",
0xbf74be11,"MainField_Weapon_Sword_044_831263988",
0xdcafc6ae,"MainField_Weapon_Sword_044_835024154",
0xc01931f1,"MainField_Weapon_Sword_044_919943336",
0xc72cc756,"MainField_Weapon_Sword_044_94985308",
0x63a4c13c,"MainField_Weapon_Sword_047_1750182378",
0x59095c03,"MainField_Weapon_Sword_047_926749202",
0xc14c9868,"MainField_Weapon_Sword_048_3070300915",
0xeb4c3e92,"MainField_Weapon_Sword_049_1828295553",
0x8405b071,"MainField_Weapon_Sword_050_1254937349",
0x557c0829,"MainField_Weapon_Sword_050_1434622013",
0x4cc8a04b,"MainField_Weapon_Sword_053_1261643737",
0xb2d16dff,"MainField_Weapon_Sword_053_1645917618",
0x94aeb46c,"MainField_Weapon_Sword_053_2949515945",
0x073a6ac0,"MainField_Weapon_Sword_060_3488038705",
0xd8490552,"MainField_Weapon_Sword_060_809871254",
0x27c7e0a7,"MainField_Weapon_Sword_060_91923013",
0x99774386,"MainField_Weapon_Sword_070_2427157272",
0x45f2ffc3,"MakeVillage_KaidoNPC_Delete",
0x679c9ed6,"MamonoShop_4PointFirst",
0x4fb8df09,"MamonoShop_BigEnemy_Giant_Activated",
0x2ef3b099,"MamonoShop_BigEnemy_Giant_Finish",
0xc94636a7,"MamonoShop_BigEnemy_Giant_Ready",
0x547faa81,"MamonoShop_BigEnemy_Golem_Activated",
0xd21945b7,"MamonoShop_BigEnemy_Golem_Finish",
0x024b32b8,"MamonoShop_BigEnemy_Golem_Mamonoshop_BigEnemy_Golem_Step1",
0x9a3b64fc,"MamonoShop_BigEnemy_Golem_Ready",
0x1e5ecb9d,"MamonoShop_BigEnemy_Golem_Step1",
0x72ff7ba6,"MamonoShop_BigEnemy_Sandworm_Activated",
0x0cdc0f04,"MamonoShop_BigEnemy_Sandworm_Finish",
0xe4ef1211,"MamonoShop_BigEnemy_Sandworm_Ready",
0x5eae5d90,"MamonoShop_ChallengeTalk",
0xf4d6e145,"MamonoShop_Change",
0xa808b666,"MamonoShop_DefaultFlow",
0x18d0bcab,"MamonoShop_First",
0xfb6cbebc,"MamonoShop_FirstTalk",
0xd900133b,"MamonoShop_MamonoMaterial",
0x22a2b029,"MamonoShop_MonsterTalkMenuFirst",
0x5915caee,"MamonoShop_NewItem_01",
0xc01c9b54,"MamonoShop_NewItem_02",
0xb71babc2,"MamonoShop_NewItem_03",
0x297f3e61,"MamonoShop_NewItem_04",
0xc3bef21a,"MamonoShop_OpenStore",
0xf5d7a9f3,"MamonoShop_SelfCard",
0xd47a636d,"MamonoShop_StopTalk",
0x46096fa6,"MamonoShop_WhatsMamo",
0x7bf47ab8,"MapTower_01",
0xe2fd2b02,"MapTower_02",
0xea39e4c0,"MapTower_02_Info",
0x95fa1b94,"MapTower_03",
0x0b9e8e37,"MapTower_04",
0x7c99bea1,"MapTower_05",
0xe590ef1b,"MapTower_06",
0x9297df8d,"MapTower_07",
0xa73f9b84,"MapTower_07_Demo",
0x0228c21c,"MapTower_08",
0x752ff28a,"MapTower_09",
0x15e87b6f,"MapTower_10",
0x62ef4bf9,"MapTower_11",
0xfbe61a43,"MapTower_12",
0x8ce12ad5,"MapTower_13",
0x1285bf76,"MapTower_14",
0x65828fe0,"MapTower_15",
0xdfb7e81f,"MapTower_DemoFirst",
0x5c4ec9c7,"MarioClubTestDungeon_Animal_Boar_A_2190819057",
0xb0810f02,"MarioClubTestDungeon_Animal_Bull_A_4119989863",
0x8e93cacd,"MarioClubTestDungeon_Animal_Fish_A_2769594508",
0x22c7f21a,"MarioClubTestDungeon_Animal_Fish_A_3345421514",
0x34a7063d,"MarioClubTestDungeon_Animal_Fish_B_1118602635",
0x56d738ae,"MarioClubTestDungeon_Animal_Fish_B_551500237",
0x88373997,"MarioClubTestDungeon_Animal_Fish_C_53910398",
0xdb0ce541,"MarioClubTestDungeon_Animal_Fish_C_924811454",
0x60ca0904,"MarioClubTestDungeon_Animal_Fish_D_3641827730",
0x1abef0cb,"MarioClubTestDungeon_Animal_Fish_D_3979934290",
0x5fa81703,"MarioClubTestDungeon_Animal_Fish_E_73097063",
0x00c2affc,"MarioClubTestDungeon_Animal_Fish_E_812795047",
0x881edc6d,"MarioClubTestDungeon_Animal_Fish_F_3732678027",
0x12740426,"MarioClubTestDungeon_Animal_Fish_F_3931455051",
0x1a5ebb66,"MarioClubTestDungeon_Animal_Fish_G_233640780",
0xf36363d0,"MarioClubTestDungeon_Animal_Fish_G_969148556",
0x321a2b6b,"MarioClubTestDungeon_Animal_Fish_H_2019432057",
0x9fca2683,"MarioClubTestDungeon_Animal_Fish_H_439200319",
0x9df78cd5,"MarioClubTestDungeon_Animal_Fish_I_2521855829",
0x1aa2def3,"MarioClubTestDungeon_Animal_Fish_I_4095986451",
0x2f0097a6,"MarioClubTestDungeon_Animal_Fish_J_2134102624",
0x7c38e6a5,"MarioClubTestDungeon_Animal_Fish_J_490761766",
0x3088a9a2,"MarioClubTestDungeon_Animal_Insect_A_4081979146",
0xf5382f2b,"MarioClubTestDungeon_Animal_Insect_B_351674893",
0x4d146b9c,"MarioClubTestDungeon_Animal_Insect_C_822147580",
0x3ee62d28,"MarioClubTestDungeon_Animal_Insect_E_913128933",
0x10e48216,"MarioClubTestDungeon_Animal_Insect_F_3630390473",
0x08be3bcc,"MarioClubTestDungeon_Animal_Insect_G_1071346126",
0xa8c5d02e,"MarioClubTestDungeon_Animal_Insect_H_672909501",
0x796dcdb1,"MarioClubTestDungeon_Animal_Insect_I_3323308433",
0x6db5d62c,"MarioClubTestDungeon_Animal_Insect_K_3245895048",
0xa8f5bd95,"MarioClubTestDungeon_Animal_Insect_M_2944652402",
0x7badc6ba,"MarioClubTestDungeon_Animal_Insect_N_1099806046",
0xf6728745,"MarioClubTestDungeon_Animal_Insect_O_2834174059",
0x9bb37f77,"MarioClubTestDungeon_Barrel_1571343752",
0xb0050915,"MarioClubTestDungeon_Barrel_3121668239",
0x6d338acc,"MarioClubTestDungeon_Barrel_3572193653",
0xfc226999,"MarioClubTestDungeon_Barrel_3797071636",
0x3c73eb1e,"MarioClubTestDungeon_BoarMeat_1631921976",
0xf12df242,"MarioClubTestDungeon_Item_Enemy_02_1089092562",
0xe66779af,"MarioClubTestDungeon_Item_Enemy_02_580487060",
0x4a46d58b,"MarioClubTestDungeon_Item_Enemy_05_102621502",
0x21f62538,"MarioClubTestDungeon_Item_Enemy_05_842332926",
0xf2ee97b0,"MarioClubTestDungeon_Item_Enemy_15_1684954488",
0x16f0c4fc,"MarioClubTestDungeon_Item_Enemy_15_875267004",
0x38be5945,"MarioClubTestDungeon_Item_Enemy_16_2321562708",
0xca87d322,"MarioClubTestDungeon_Item_Enemy_16_3659915920",
0x1427c8ba,"MarioClubTestDungeon_Item_Enemy_17_1033752471",
0x30a4062f,"MarioClubTestDungeon_Item_Enemy_17_1842876755",
0x53cb3df8,"MarioClubTestDungeon_Item_Enemy_24_1137380103",
0x919a0d5d,"MarioClubTestDungeon_Item_Enemy_24_3208354511",
0x962b7d45,"MarioClubTestDungeon_Item_Enemy_26_1151782686",
0xcb06bb82,"MarioClubTestDungeon_Item_Enemy_26_3092668118",
0xbe755345,"MarioClubTestDungeon_Item_Enemy_27_1609462737",
0x3a0148ca,"MarioClubTestDungeon_Item_Enemy_27_2736710169",
0x802dd3ab,"MarioClubTestDungeon_Item_Enemy_28_3034471274",
0xeebda1d3,"MarioClubTestDungeon_Item_Enemy_28_3601721132",
0x6f5d790c,"MarioClubTestDungeon_Item_Enemy_29_1523595846",
0xd1a62ea3,"MarioClubTestDungeon_Item_Enemy_29_950240768",
0x9b9a1471,"MarioClubTestDungeon_Item_Enemy_30_3014893427",
0x80514503,"MarioClubTestDungeon_Item_Enemy_30_3519030069",
0x66cdd10e,"MarioClubTestDungeon_Item_Enemy_31_1070481945",
0x2b34a001,"MarioClubTestDungeon_Item_Enemy_31_1572732511",
0xc8ba47af,"MarioClubTestDungeon_Item_Enemy_40_2032970530",
0xf0672d5e,"MarioClubTestDungeon_Item_Enemy_40_459216740",
0x79f1676a,"MarioClubTestDungeon_Item_Fruit_A_1920374315",
0x6dabc991,"MarioClubTestDungeon_Item_Fruit_A_2342671971",
0x00e39252,"MarioClubTestDungeon_Item_Fruit_B_1813674852",
0xc698a622,"MarioClubTestDungeon_Item_Fruit_B_2513353516",
0xe235c569,"MarioClubTestDungeon_Item_Fruit_C_1240244373",
0x4ff0d560,"MarioClubTestDungeon_Item_Fruit_C_2181956191",
0x7aef7001,"MarioClubTestDungeon_Item_Fruit_D_1811964787",
0x70d332ec,"MarioClubTestDungeon_Item_Fruit_D_2816655801",
0xd52992dc,"MarioClubTestDungeon_Item_Fruit_E_1317102732",
0x537e4a2c,"MarioClubTestDungeon_Item_Fruit_E_2237908550",
0x7d00f304,"MarioClubTestDungeon_Item_Fruit_F_1802348394",
0x11acd467,"MarioClubTestDungeon_Item_Fruit_F_2693727648",
0x71a9978f,"MarioClubTestDungeon_Item_Fruit_G_1194797223",
0x52c5dbb3,"MarioClubTestDungeon_Item_Fruit_G_2362804845",
0xae489ce1,"MarioClubTestDungeon_Item_Fruit_H_1358402004",
0x7999be81,"MarioClubTestDungeon_Item_Fruit_H_93724304",
0xd1846623,"MarioClubTestDungeon_Item_Fruit_I_3204051192",
0x10b044c0,"MarioClubTestDungeon_Item_Fruit_I_3952639932",
0xbdf22148,"MarioClubTestDungeon_Item_Fruit_J_1469732301",
0xcc8c742f,"MarioClubTestDungeon_Item_Fruit_J_50059913",
0xa079908d,"MarioClubTestDungeon_Item_Meat_01_2535639566",
0xe3d6a876,"MarioClubTestDungeon_Item_Meat_01_4115756616",
0x08df6b6c,"MarioClubTestDungeon_Item_Meat_02_1889199881",
0xb0c69799,"MarioClubTestDungeon_Item_Meat_02_317283151",
0x4166e843,"MarioClubTestDungeon_Item_Meat_06_3155139021",
0x6a14f628,"MarioClubTestDungeon_Item_Meat_06_3964290825",
0xfcdca4c2,"MarioClubTestDungeon_Item_Meat_07_1538766026",
0x647092cd,"MarioClubTestDungeon_Item_Meat_07_200436238",
0x46a53c38,"MarioClubTestDungeon_Item_Meat_11_3553751597",
0x8a926d33,"MarioClubTestDungeon_Item_Meat_12_2765537979",
0x643ba4cb,"MarioClubTestDungeon_Item_Mushroom_A_161279553",
0x94d99d8c,"MarioClubTestDungeon_Item_Mushroom_A_576196586",
0x03be18ce,"MarioClubTestDungeon_Item_Mushroom_B_3543753152",
0x99200ac6,"MarioClubTestDungeon_Item_Mushroom_B_417055498",
0x50fe3e08,"MarioClubTestDungeon_Item_Mushroom_C_4055393855",
0x4f4f972c,"MarioClubTestDungeon_Item_Mushroom_C_979027189",
0xc3625f9a,"MarioClubTestDungeon_Item_Mushroom_D_250686040",
0x7a0a4ebc,"MarioClubTestDungeon_Item_Mushroom_D_624282611",
0xb516fbf8,"MarioClubTestDungeon_Item_Mushroom_E_3428216518",
0x40088581,"MarioClubTestDungeon_Item_Mushroom_E_3885141869",
0x6d9bd888,"MarioClubTestDungeon_Item_Mushroom_F_292392737",
0x29570fd4,"MarioClubTestDungeon_Item_Mushroom_F_3666816491",
0x43fd920b,"MarioClubTestDungeon_Item_Mushroom_H_1027014892",
0x753aa18d,"MarioClubTestDungeon_Item_Mushroom_H_4141195814",
0xc6588888,"MarioClubTestDungeon_Item_Mushroom_J_3319800557",
0x0f1537e6,"MarioClubTestDungeon_Item_Mushroom_J_3995386694",
0x0a3e919d,"MarioClubTestDungeon_Item_Ore_A_3113497825",
0x54f89619,"MarioClubTestDungeon_Item_Ore_A_3975527333",
0x078fef81,"MarioClubTestDungeon_Item_Ore_B_1579948518",
0x83615ca6,"MarioClubTestDungeon_Item_Ore_B_189634210",
0x08896ce2,"MarioClubTestDungeon_Item_Ore_C_3614412059",
0x4f7d8366,"MarioClubTestDungeon_Item_Ore_C_479014865",
0x37872317,"MarioClubTestDungeon_Item_Ore_D_4068691709",
0x1c30dcaa,"MarioClubTestDungeon_Item_Ore_D_962718775",
0xbbf51885,"MarioClubTestDungeon_Item_Ore_E_3489810690",
0x66f79844,"MarioClubTestDungeon_Item_Ore_E_467725256",
0xd3fda1d6,"MarioClubTestDungeon_Item_Ore_F_1040984110",
0x14897d4c,"MarioClubTestDungeon_Item_Ore_F_4126051044",
0x0731a0de,"MarioClubTestDungeon_Item_Ore_G_307668963",
0x77475ad6,"MarioClubTestDungeon_Item_Ore_G_3652453673",
0x98720fef,"MarioClubTestDungeon_Item_Ore_H_1696391203",
0xbf8332d4,"MarioClubTestDungeon_Item_Ore_H_3463756890",
0x11331254,"MarioClubTestDungeon_Item_Plant_A_2333245711",
0xfcc53266,"MarioClubTestDungeon_Item_Plant_A_544922998",
0xa197e555,"MarioClubTestDungeon_Item_Plant_B_1651582010",
0xd54feee5,"MarioClubTestDungeon_Item_Plant_B_3373889603",
0xf89915d5,"MarioClubTestDungeon_Item_Plant_C_2357150998",
0xbd6d7df2,"MarioClubTestDungeon_Item_Plant_C_655825263",
0xc63fb35d,"MarioClubTestDungeon_Item_Plant_E_2080887138",
0x1e33dd07,"MarioClubTestDungeon_Item_Plant_E_510946596",
0xcdc64965,"MarioClubTestDungeon_Item_Plant_F_2450108494",
0x92646c2b,"MarioClubTestDungeon_Item_Plant_F_4034530312",
0x11628d4e,"MarioClubTestDungeon_Item_Plant_G_2070549883",
0xaff9c635,"MarioClubTestDungeon_Item_Plant_G_421112125",
0x7a5778ee,"MarioClubTestDungeon_Item_Plant_H_2506388567",
0x747a38bd,"MarioClubTestDungeon_Item_Plant_H_4145530897",
0x5be1b8d1,"MarioClubTestDungeon_Item_Plant_I_1927043408",
0x09a82c5e,"MarioClubTestDungeon_Item_Plant_I_279962902",
0xd9a8a65b,"MarioClubTestDungeon_Item_Plant_J_124738661",
0x7dc84f1d,"MarioClubTestDungeon_Item_Plant_J_1462403745",
0x67910f36,"MarioClubTestDungeon_Item_Plant_L_1346848440",
0xbb9884f9,"MarioClubTestDungeon_Item_Plant_L_170108",
0x27361267,"MarioClubTestDungeon_Item_Plant_M_3192465300",
0x452eb48f,"MarioClubTestDungeon_Item_Plant_M_3993827664",
0xa9f188ab,"MarioClubTestDungeon_Item_Plant_O_1311880160",
0xe4542581,"MarioClubTestDungeon_Item_Plant_O_742526886",
0x5d122380,"MarioClubTestDungeon_Item_RoastFish_01_293050166",
0x44605b92,"MarioClubTestDungeon_Item_RoastFish_01_3903018878",
0x98d0ec01,"MarioClubTestDungeon_Item_RoastFish_02_29422155",
0xbbef8403,"MarioClubTestDungeon_Item_RoastFish_02_4162073091",
0x9a5bff87,"MarioClubTestDungeon_Item_RoastFish_03_370823983",
0xc3e95e7c,"MarioClubTestDungeon_Item_RoastFish_03_4023292775",
0xca046ad0,"MarioClubTestDungeon_Item_RoastFish_04_141999712",
0x892c13a7,"MarioClubTestDungeon_Item_RoastFish_04_4053958184",
0x4b969877,"MarioClubTestDungeon_Item_RoastFish_07_3775857474",
0x23776378,"MarioClubTestDungeon_Item_RoastFish_07_720216456",
0xaec70bf5,"MarioClubTestDungeon_Item_Roast_01_251748974",
0xdfc0f135,"MarioClubTestDungeon_Item_Roast_01_3303260324",
0xe80d7211,"MarioClubTestDungeon_Item_Roast_02_3904458601",
0x6ae3d529,"MarioClubTestDungeon_Item_Roast_02_593227171",
0x92944f7d,"MarioClubTestDungeon_Item_Roast_03_2837664156",
0xfcfc454b,"MarioClubTestDungeon_Item_Roast_03_882595024",
0x539394d4,"MarioClubTestDungeon_Item_Roast_04_1194143920",
0xa655ac27,"MarioClubTestDungeon_Item_Roast_04_3667211772",
0x7e0cf4bd,"MarioClubTestDungeon_Item_Roast_05_2924385669",
0x2c3fd2a8,"MarioClubTestDungeon_Item_Roast_05_871796937",
0x56860fe2,"MarioClubTestDungeon_Item_Roast_06_1077998761",
0xfc455ff1,"MarioClubTestDungeon_Item_Roast_06_3724079589",
0x61a11442,"MarioClubTestDungeon_Item_Roast_07_2818103726",
0x4d9e7402,"MarioClubTestDungeon_Item_Roast_07_977329378",
0xe83f7afe,"MarioClubTestDungeon_Item_Roast_08_2077934103",
0xb6b252b0,"MarioClubTestDungeon_Item_Roast_08_2956484829",
0x68f88f48,"MarioClubTestDungeon_Item_Roast_09_1580597745",
0x092a62f8,"MarioClubTestDungeon_Item_Roast_09_2513740603",
0xacfc0c24,"MarioClubTestDungeon_Item_Roast_10_2092369422",
0x6c9d58c9,"MarioClubTestDungeon_Item_Roast_10_3075843268",
0xa79545dd,"MarioClubTestDungeon_Item_Roast_11_1499186664",
0xd1726ed1,"MarioClubTestDungeon_Item_Roast_11_2461624098",
0x9b77c07b,"MarioClubTestDungeon_Item_Roast_12_1963018789",
0x85fc589d,"MarioClubTestDungeon_Item_Roast_12_3202607343",
0x9a2c22fd,"MarioClubTestDungeon_Item_Roast_13_1656877910",
0x427da2ac,"MarioClubTestDungeon_Item_Roast_13_2601857822",
0x1d5b38f4,"MarioClubTestDungeon_Item_Roast_15_1705785167",
0x1535376f,"MarioClubTestDungeon_Item_Roast_15_2625173255",
0xf266e562,"MarioClubTestDungeon_Kibako_Contain_01_1152646345",
0xfa406152,"MarioClubTestDungeon_Kibako_Contain_01_1400063418",
0x3ed2926a,"MarioClubTestDungeon_Kibako_Contain_01_1411285411",
0x7dc59667,"MarioClubTestDungeon_Kibako_Contain_01_2735460814",
0x9f7a2855,"MarioClubTestDungeon_Kibako_Contain_01_315464525",
0x4c8db036,"MarioClubTestDungeon_Kibako_Contain_01_3179095190",
0xe829db4c,"MarioClubTestDungeon_Kibako_Contain_01_3455084596",
0xb9744609,"MarioClubTestDungeon_Obj_BombArrow_A_01_65151191",
0x0dd20549,"MarioClubTestDungeon_Pot_1707781083",
0x65d3ee87,"MarioClubTestDungeon_Pot_1872670474",
0x3ff9f9ba,"MarioClubTestDungeon_Pot_2284225037",
0x97484df9,"MarioClubTestDungeon_PutRupee_Silver_187765281",
0x742f4530,"MarioClubTestDungeon_PutRupee_Silver_2083922615",
0x665849ee,"MarioClubTestDungeon_PutRupee_Silver_2453160859",
0x8a5d3bd9,"MarioClubTestDungeon_PutRupee_Silver_2609778608",
0x3d7ab2f1,"MarioClubTestDungeon_PutRupee_Silver_3846140685",
0xcf386ee8,"MarioClubTestDungeon_PutRupee_Silver_3968417574",
0x1b90a142,"MarioClubTestDungeon_TBox_Dungeon_Iron_3453265974",
0x64c8d196,"MarioClubTestDungeon_TBox_Dungeon_Iron_3846083256",
0x89862537,"MarioClubTestDungeon_TBox_Dungeon_Wood_4084536344",
0xa28ce85f,"MarioClubTestDungeon_WallCrack_103551723",
0x141d1ad9,"MarioClubTestDungeon_Weapon_Bow_001_1119066836",
0x5a388a5b,"MarioClubTestDungeon_Weapon_Bow_002_2898131960",
0x3077a7ba,"MarioClubTestDungeon_Weapon_Bow_003_2882551777",
0xfc26c2d6,"MarioClubTestDungeon_Weapon_Bow_004_1172201165",
0xf5e1369d,"MarioClubTestDungeon_Weapon_Bow_006_4209466494",
0x43ddc75d,"MarioClubTestDungeon_Weapon_Bow_009_2999652000",
0x34e26960,"MarioClubTestDungeon_Weapon_Bow_011_327481675",
0x7f8482a8,"MarioClubTestDungeon_Weapon_Bow_013_2579108278",
0x3fba2ab5,"MarioClubTestDungeon_Weapon_Bow_014_2274404411",
0xefaac445,"MarioClubTestDungeon_Weapon_Bow_015_2114075825",
0xbe1fed4c,"MarioClubTestDungeon_Weapon_Bow_016_439530848",
0x3c876574,"MarioClubTestDungeon_Weapon_Bow_017_233966611",
0xccac67d2,"MarioClubTestDungeon_Weapon_Bow_026_1433609127",
0x2178877e,"MarioClubTestDungeon_Weapon_Bow_027_1281888998",
0x788777e8,"MarioClubTestDungeon_Weapon_Bow_029_1772013847",
0x16ecbf7f,"MarioClubTestDungeon_Weapon_Bow_030_4253718631",
0xab8bd61c,"MarioClubTestDungeon_Weapon_Bow_032_350829906",
0x1f73b530,"MarioClubTestDungeon_Weapon_Bow_033_2416729501",
0xc66d51c6,"MarioClubTestDungeon_Weapon_Lsword_001_2046300299",
0x837a5b70,"MarioClubTestDungeon_Weapon_Lsword_002_2549510567",
0x8c44066b,"MarioClubTestDungeon_Weapon_Lsword_003_1884183712",
0x6a6371e7,"MarioClubTestDungeon_Weapon_Lsword_004_2397913318",
0xc6024a3b,"MarioClubTestDungeon_Weapon_Lsword_005_1767206369",
0x8c358f7f,"MarioClubTestDungeon_Weapon_Lsword_006_1285797392",
0xfb1b5357,"MarioClubTestDungeon_Weapon_Lsword_010_2729301820",
0x5940bf97,"MarioClubTestDungeon_Weapon_Lsword_011_1271820809",
0xa12546e3,"MarioClubTestDungeon_Weapon_Lsword_012_2780828453",
0xc469210b,"MarioClubTestDungeon_Weapon_Lsword_013_1057999463",
0xd85b4d2d,"MarioClubTestDungeon_Weapon_Lsword_014_2123748498",
0x110d4675,"MarioClubTestDungeon_Weapon_Lsword_015_2426123710",
0xbae06cc9,"MarioClubTestDungeon_Weapon_Lsword_016_1115176482",
0x55eff973,"MarioClubTestDungeon_Weapon_Lsword_017_1438156625",
0x7c468fb8,"MarioClubTestDungeon_Weapon_Lsword_018_3149325949",
0xf357846f,"MarioClubTestDungeon_Weapon_Lsword_024_1737418195",
0x0aabc615,"MarioClubTestDungeon_Weapon_Lsword_027_3635930976",
0xcd53fde8,"MarioClubTestDungeon_Weapon_Lsword_029_221882597",
0x8d51187e,"MarioClubTestDungeon_Weapon_Lsword_031_2306901247",
0x5a1947ba,"MarioClubTestDungeon_Weapon_Lsword_032_3409685215",
0x730c4498,"MarioClubTestDungeon_Weapon_Lsword_033_3525352350",
0x9c401ab4,"MarioClubTestDungeon_Weapon_Lsword_034_3707442092",
0x2abe8035,"MarioClubTestDungeon_Weapon_Lsword_035_994262699",
0x9e6f60e6,"MarioClubTestDungeon_Weapon_Lsword_036_684684052",
0x29baf9a8,"MarioClubTestDungeon_Weapon_Lsword_037_3334541880",
0x190c701c,"MarioClubTestDungeon_Weapon_Lsword_038_1009711794",
0xd6d19130,"MarioClubTestDungeon_Weapon_Lsword_041_3249329697",
0xc646a1ef,"MarioClubTestDungeon_Weapon_Lsword_047_3934356962",
0xb97fe294,"MarioClubTestDungeon_Weapon_Lsword_051_1625498058",
0x1f5aba54,"MarioClubTestDungeon_Weapon_Lsword_054_799192845",
0x0df3b645,"MarioClubTestDungeon_Weapon_Lsword_055_638909222",
0xcf99b26a,"MarioClubTestDungeon_Weapon_Lsword_056_806758498",
0x5f3b023b,"MarioClubTestDungeon_Weapon_Shield_001_3093066334",
0x01029a6b,"MarioClubTestDungeon_Weapon_Shield_002_3207706183",
0x035a8a97,"MarioClubTestDungeon_Weapon_Shield_003_3062364780",
0x9d8fb7d2,"MarioClubTestDungeon_Weapon_Shield_004_427673080",
0x2bf621ae,"MarioClubTestDungeon_Weapon_Shield_005_2787804934",
0xd13d3f87,"MarioClubTestDungeon_Weapon_Shield_006_2946288429",
0x4329beff,"MarioClubTestDungeon_Weapon_Shield_007_3909439884",
0xccbd16ba,"MarioClubTestDungeon_Weapon_Shield_008_247299211",
0x01309a2a,"MarioClubTestDungeon_Weapon_Shield_009_504365537",
0x4e27c56e,"MarioClubTestDungeon_Weapon_Shield_013_824174243",
0xbc304fa4,"MarioClubTestDungeon_Weapon_Shield_014_1448219506",
0xc183b5fb,"MarioClubTestDungeon_Weapon_Shield_015_1363138411",
0x5deaa231,"MarioClubTestDungeon_Weapon_Shield_016_6747321",
0xe12e123b,"MarioClubTestDungeon_Weapon_Shield_017_3999831445",
0x71bbe147,"MarioClubTestDungeon_Weapon_Shield_018_118173856",
0xc8181ea5,"MarioClubTestDungeon_Weapon_Shield_022_1330192947",
0x20d1dc72,"MarioClubTestDungeon_Weapon_Shield_023_671406050",
0xc1669e3a,"MarioClubTestDungeon_Weapon_Shield_025_950643336",
0xc5295c41,"MarioClubTestDungeon_Weapon_Shield_026_3369156348",
0xac758d2f,"MarioClubTestDungeon_Weapon_Shield_030_3601320868",
0xe7d7ff90,"MarioClubTestDungeon_Weapon_Shield_031_396774858",
0xc9559834,"MarioClubTestDungeon_Weapon_Shield_032_2705802015",
0x324e73d8,"MarioClubTestDungeon_Weapon_Shield_033_652173264",
0xd51c75cc,"MarioClubTestDungeon_Weapon_Shield_034_1210377770",
0x2a27a9eb,"MarioClubTestDungeon_Weapon_Shield_035_4028522701",
0xd4719396,"MarioClubTestDungeon_Weapon_Shield_036_565353417",
0x00d8fe27,"MarioClubTestDungeon_Weapon_Shield_037_3485268709",
0xec388a65,"MarioClubTestDungeon_Weapon_Spear_001_2315538577",
0x4b72a0b5,"MarioClubTestDungeon_Weapon_Spear_002_1667757476",
0x204f5e9c,"MarioClubTestDungeon_Weapon_Spear_003_2372506760",
0x0b5daec0,"MarioClubTestDungeon_Weapon_Spear_004_1327981182",
0xe9b3bab7,"MarioClubTestDungeon_Weapon_Spear_004_2230970608",
0x5e272e78,"MarioClubTestDungeon_Weapon_Spear_005_2703819602",
0x5f110e29,"MarioClubTestDungeon_Weapon_Spear_006_1212851815",
0x1ae473c0,"MarioClubTestDungeon_Weapon_Spear_007_952625861",
0xc3e862fb,"MarioClubTestDungeon_Weapon_Spear_008_3749693378",
0xdceda965,"MarioClubTestDungeon_Weapon_Spear_009_3367993009",
0x3b7f686e,"MarioClubTestDungeon_Weapon_Spear_010_1364301606",
0xe0fcdf75,"MarioClubTestDungeon_Weapon_Spear_011_3210737162",
0x8e98f49e,"MarioClubTestDungeon_Weapon_Spear_012_1491589901",
0xa63664a1,"MarioClubTestDungeon_Weapon_Spear_013_2490541513",
0x7eea19ea,"MarioClubTestDungeon_Weapon_Spear_014_1942653134",
0xb62d1c33,"MarioClubTestDungeon_Weapon_Spear_015_1678389693",
0xd23bd7a0,"MarioClubTestDungeon_Weapon_Spear_016_3517206512",
0x39f369ae,"MarioClubTestDungeon_Weapon_Spear_017_1068117724",
0xca1146ae,"MarioClubTestDungeon_Weapon_Spear_018_3603532777",
0xd1cbd1a2,"MarioClubTestDungeon_Weapon_Spear_024_1792129423",
0x1682df4c,"MarioClubTestDungeon_Weapon_Spear_025_3257401935",
0x22e2722e,"MarioClubTestDungeon_Weapon_Spear_027_893211170",
0x14588286,"MarioClubTestDungeon_Weapon_Spear_028_3677565710",
0xdab047ea,"MarioClubTestDungeon_Weapon_Spear_029_740713315",
0x4e21f0cd,"MarioClubTestDungeon_Weapon_Spear_031_844142139",
0x3be1df44,"MarioClubTestDungeon_Weapon_Spear_033_649142173",
0xf74a7752,"MarioClubTestDungeon_Weapon_Spear_034_568122244",
0x32f89fdd,"MarioClubTestDungeon_Weapon_Spear_035_3486694056",
0x11f492f4,"MarioClubTestDungeon_Weapon_Spear_036_2789525323",
0x97aaacad,"MarioClubTestDungeon_Weapon_Spear_037_3697211159",
0x5055f657,"MarioClubTestDungeon_Weapon_Spear_047_3309684310",
0x8d6680dc,"MarioClubTestDungeon_Weapon_Spear_050_1015750153",
0xf3d55b39,"MarioClubTestDungeon_Weapon_Sword_001_3726133582",
0xaad6e4fb,"MarioClubTestDungeon_Weapon_Sword_002_930868347",
0xde9d1a9f,"MarioClubTestDungeon_Weapon_Sword_003_3648359767",
0x00978dae,"MarioClubTestDungeon_Weapon_Sword_004_1754728406",
0x132c5bc2,"MarioClubTestDungeon_Weapon_Sword_005_1298258983",
0x31ec972f,"MarioClubTestDungeon_Weapon_Sword_006_2742009099",
0x9232a74b,"MarioClubTestDungeon_Weapon_Sword_007_3554865577",
0x0ff93ccd,"MarioClubTestDungeon_Weapon_Sword_008_1038913669",
0x1b24ac2e,"MarioClubTestDungeon_Weapon_Sword_009_3566155184",
0xe0b61f32,"MarioClubTestDungeon_Weapon_Sword_013_3338893327",
0xe8415c16,"MarioClubTestDungeon_Weapon_Sword_014_778100026",
0xa84c4016,"MarioClubTestDungeon_Weapon_Sword_015_3228482582",
0xe1570692,"MarioClubTestDungeon_Weapon_Sword_016_1136284693",
0xa8eb7b5a,"MarioClubTestDungeon_Weapon_Sword_017_1417342310",
0xdfefe278,"MarioClubTestDungeon_Weapon_Sword_018_3128200266",
0x33e97a1b,"MarioClubTestDungeon_Weapon_Sword_024_1053663312",
0x50aa97be,"MarioClubTestDungeon_Weapon_Sword_025_3962448844",
0x37e7961a,"MarioClubTestDungeon_Weapon_Sword_027_688724259",
0x379f6729,"MarioClubTestDungeon_Weapon_Sword_029_4065910420",
0x5c9de569,"MarioClubTestDungeon_Weapon_Sword_030_367063955",
0x3174f316,"MarioClubTestDungeon_Weapon_Sword_031_981554332",
0x9f46126e,"MarioClubTestDungeon_Weapon_Sword_033_3711531419",
0x23b30ac4,"MarioClubTestDungeon_Weapon_Sword_034_597321181",
0x48d65711,"MarioClubTestDungeon_Weapon_Sword_035_3405365480",
0x11619c9e,"MarioClubTestDungeon_Weapon_Sword_040_2136456869",
0x36fa408a,"MarioClubTestDungeon_Weapon_Sword_041_217826002",
0x91b4b87d,"MarioClubTestDungeon_Weapon_Sword_043_668373265",
0xa3295df0,"MarioClubTestDungeon_Weapon_Sword_044_3290581210",
0x644aaa8a,"MarioClubTestDungeon_Weapon_Sword_047_475447224",
0x01a82432,"MarioClubTestDungeon_Weapon_Sword_048_2438586249",
0xd9d7e941,"MarioClubTestDungeon_Weapon_Sword_049_3328505475",
0x84f2070c,"MarioClubTestDungeon_Weapon_Sword_050_3947098069",
0x00b4c52c,"MarioClubTestDungeon_Weapon_Sword_051_2017098428",
0x95823f51,"MarioClubTestDungeon_Weapon_Sword_052_35670752",
0x46dcdee7,"MarioClubTestDungeon_Weapon_Sword_053_456883105",
0xc67a3129,"MarioClubTestDungeon_Weapon_Sword_070_2751625490",
0xe8dc9d31,"MarittaMini_BigWhales_Activated",
0x989ecfbf,"MarittaMini_BigWhales_Finish",
0xc2142d98,"MarittaMini_BigWhales_Ready",
0xdc0aeb6a,"MessageGame_Activated",
0x2d11a8e6,"MessageGame_Finish",
0x9dbc4525,"MessageGame_Ready",
0x471bcdd2,"MinakkareHatago_NPC001_YagiYado",
0xb4364a3f,"MinakkareHatago_NPC002_1_First",
0x031ff63d,"MinakkareHatago_NPC002_Check",
0x7dc2c31d,"MinakkareHatago_NPC002_Enokida",
0xade0b479,"MinakkareHatago_NPC002_First",
0x175c4cfc,"MinakkareHatago_NPC002_Ichikara",
0x7ed92468,"MinakkareHatago_NPC002_NormalFirst",
0x9238eec6,"MinakkareHatago_NPC002_Reward",
0xd157c40a,"MinakkareHatago_NPC002_Yoroisou",
0x69f976be,"MinakkareHatago_NPC003_Dislike",
0x0b97bfcd,"MinakkareHatago_NPC003_First",
0x436360b0,"MinakkareHatago_NPC003_Like",
0xa34ffded,"MinakkareHatago_NPC003_NormalFirst",
0xd8dd4e78,"MinakkareHatago_NPC003_Take",
0xafed67c9,"MinakkareMini_Dragonfly_Activated",
0xc2728401,"MinakkareMini_Dragonfly_Dragonfly",
0xf9ebd9f0,"MinakkareMini_Dragonfly_Finish",
0x388f13b8,"MinakkareMini_Dragonfly_Give",
0x4a6d1abc,"MinakkareMini_Dragonfly_LittleSister",
0xdcfdb146,"MinakkareMini_Dragonfly_Ready",
0x145c0442,"MinakkareMini_Dragonfly_Through",
0x679ffced,"MinamihateeluMini_first",
0xcfdf3c31,"MinamihateeluMini_touzoku000",
0xb8d80ca7,"MinamihateeluMini_touzoku001",
0x1aaf0954,"MinamihateeluMini_touzoku_Activated",
0x7b1c805c,"MinamihateeluMini_touzoku_FindCave",
0x3be1c291,"MinamihateeluMini_touzoku_FindTreasure",
0x07db9677,"MinamihateeluMini_touzoku_Finish",
0x38a940de,"MinamihateeluMini_touzoku_Ready",
0x2c0744c1,"MiniGame_Bowling_AreaIn",
0x2a3dc78c,"MiniGame_Bowling_GetPrizeRod",
0x9b4f3e33,"MiniGame_Bowling_IsTalked",
0xb3f9dceb,"MiniGame_GambleTreasureBox_FirstTalkFinish",
0x0d5082e2,"MiniGame_HillTower_BirdMan_ExplanedOnce",
0x3b08bc61,"MiniGame_HillTower_BirdMan_FinishedOnce",
0xcecc7c62,"MiniGame_HillTower_BirdMan_TalkedOnce",
0xdc8ad276,"MiniGame_HorseRace_ClearTime",
0xa063ccba,"MiniGame_HorseRace_First",
0x385f236d,"MiniGame_HorseRace_GetReins",
0xd09a1e6c,"MiniGame_HorseRace_GetSaddle",
0x44037ce3,"MiniGame_HorseRace_GoaledRace",
0xf4a08e31,"MiniGame_HorseRace_MiddleTime",
0x8d1a69de,"MiniGame_HorseRace_PlayedRace",
0x1a87a742,"MiniGame_HorseRace_Ready",
0x58cc8f22,"MiniGame_HorseRace_UptdateRecord",
0x654fd8b7,"MiniGame_HorsebackArchery_HasAlreadyGivenHorseReins",
0xb6c1355d,"MiniGame_HorsebackArchery_HasAlreadyGivenHorseSaddle",
0xc7bb38cf,"MiniGame_HorsebackArchery_IsPlayed",
0x497211ab,"MiniGame_HorsebackArchery_IsTalked",
0x35f36faf,"MiniGame_KitakkareBF_1stClear",
0xbb7c684c,"MiniGame_KitakkareBF_2stClear",
0xd27761fd,"MiniGame_KitakkareBF_3rdClear",
0xf202d568,"MiniGame_KitakkareBF_Fire",
0x1d8e560a,"MiniGame_KitakkareBF_Rain",
0x5b40e2f4,"MiniGame_ParasailArchery_Activated",
0x8692aba1,"MiniGame_ParasailArchery_BombArrow",
0x86ffd76c,"MiniGame_ParasailArchery_EventTimerAppear",
0xe8e17d67,"MiniGame_ParasailArchery_Fail_AreaInAppear",
0xdbc3feeb,"MiniGame_ParasailArchery_Fail_BombArrow",
0xc66abd27,"MiniGame_ParasailArchery_Fail_Bow",
0x9f1b8a80,"MiniGame_ParasailArchery_Fail_Self",
0xb62bf1e0,"MiniGame_ParasailArchery_Fail_TimeOut",
0x9125c993,"MiniGame_ParasailArchery_Playing",
0x218e4b38,"MiniGame_ParasailArchery_Ready",
0x9e37f937,"MiniGame_ParasailArchery_Ready_Teba_First",
0x5f2ee7c8,"MiniGame_ParasailArchery_Ready_Tyuri_First",
0x96629b18,"MiniGame_ParasailArchery_Start",
0x6ba22caa,"MiniGame_ParasailArchery_TargetONOFF",
0xa464e3a7,"MiniGame_ParasailArchery_TimeOver",
0x46c980f1,"MiniGame_ParasailRide_IsPlayed",
0xc800a995,"MiniGame_ParasailRide_IsTalked",
0x0c429cb4,"MiniGame_ShieldSurfing_IsGoalOnce",
0x88785969,"MiniGame_ShieldSurfing_Try",
0x91d654b7,"MiniGame_SmashGolf_IsTalked",
0x3cea2a79,"MlsYmaQueTest01_Activated",
0xf68d77f7,"MlsYmaQueTest01_Finished",
0x1d0b5624,"MlsYmaQueTest01_Ready",
0x6975f7eb,"Moriblin_Complete",
0x83764f5c,"MouthofDragon_Activated",
0xa41fe576,"MouthofDragon_Finish",
0xcc8a3568,"MouthofDragon_Play",
0x40da35bc,"MouthofDragon_Ready",
0xc4bf9add,"MouthofDragon_Step1",
0xb7e9edf9,"MtMotelMini_Landscape_Activated",
0x4bf21dad,"MtMotelMini_Landscape_Finished",
0x7baeb887,"MtMotelMini_Landscape_Ready",
0x9535c127,"Musician_005_Hint",
0x2891e879,"My_Hero_Activated",
0x9b97f575,"My_Hero_Finish",
0x7dcceb1a,"My_Hero_Ready",
0x783a8a46,"NPC_DRCVoice_DispNameFlag",
0xcb2c1b6f,"NPC_GoddessStatue_BraveSpa_First",
0xae98554d,"NPC_GoddessStatue_PowerSpa_First",
0xfbcc30f6,"NPC_GoddessStatue_WisdomSpa_First",
0x957d8d89,"NPC_Goron003_Boss",
0xec24eafe,"NPC_Goron003_ClearFirst",
0x72b21e1d,"NPC_Goron003_Explain",
0xc0b28f8a,"NPC_Goron003_First",
0x3fd27ddf,"NPC_Goron003_Invite",
0x0fbfb314,"NPC_Goron003_Restruction",
0x8f067c62,"NPC_Goron005_FirstBuy",
0xdc86e68d,"NPC_Goron005_FirstHere",
0x3ad3f6d1,"NPC_Goron006_HeroGuard",
0xc2c529af,"NPC_Goron009_Sword054_First",
0xd828265d,"NPC_Goron011_1Comp",
0x9b879e3d,"NPC_Goron011_First",
0x131b02c1,"NPC_Goron011_Hear",
0x31ea4ddc,"NPC_Goron011_Home",
0x852112fc,"NPC_Goron011_You",
0xebf33196,"NPC_Goron012_City",
0x31c95e42,"NPC_Goron012_Clear_First",
0xaa6f84a0,"NPC_Goron012_First",
0xc9bfb19a,"NPC_Goron014_First",
0x24f0458e,"NPC_Goron014_HeroTalk",
0xde8f969b,"NPC_Goron015_ClearFirst",
0x6fc8ba2e,"NPC_Goron015_First",
0x5e20a0b3,"NPC_Goron016_First",
0xc09c242f,"NPC_Goron017_Explain",
0x8e174e06,"NPC_Goron017_Golem",
0x25d005e2,"NPC_Goron017_Reward",
0x20f39d71,"NPC_Goron017_Select",
0xd3fa45ef,"NPC_Goron017_Tegara",
0x31eebaf0,"NPC_Goron018_Diamond",
0x0e1fdbee,"NPC_Goron018_First",
0x2434d900,"NPC_Goron018_First_Burning",
0xab69cdc9,"NPC_Goron018_Ruby",
0xc33d094f,"NPC_Goron018_Sapphire",
0xffe1cd5d,"NPC_Goron018_Sold",
0xad9cc96a,"NPC_Goron018_Topaz",
0x6043c12a,"NPC_Goron021_ClearFirst",
0x150899de,"NPC_Goron021_First",
0xc2585cdf,"NPC_Goron022_AfterFirst",
0x9f547470,"NPC_Goron022_FireRelic",
0x24e08343,"NPC_Goron022_First",
0x829788f7,"NPC_Goron023_First",
0x72a0d985,"NPC_Goron023_Go",
0x135715aa,"NPC_Goron023_IchikaraFirst",
0x623f849c,"NPC_Goron023_Name",
0xe147bdcd,"NPC_Goron025_First",
0x57cb8659,"NPC_Goron025_Go",
0x051d89a9,"NPC_Goron030_Cannon",
0xb5dab8c8,"NPC_Goron030_Cannon2",
0xc47eccc1,"NPC_Goron030_Coco",
0x7fd592f4,"NPC_Goron030_First",
0x1478df2c,"NPC_Goron030_Help",
0x2819aa31,"NPC_Goron030_Help2",
0x5f1e9aa7,"NPC_Goron030_Help3",
0xc17a0f04,"NPC_Goron030_Help4",
0xf7a1b983,"NPC_Goron030_Intro",
0x4b2aca68,"NPC_Goron030_Yunbo",
0x51ee8630,"NPC_Goron033_ClearFirst",
0x4e3d8869,"NPC_Goron033_First",
0xaabf6eb1,"NPC_Goron21_Yunbo",
0xa4453c2b,"NPC_Lanayru001_Bought_Shield_001",
0x3d4c6d91,"NPC_Lanayru001_Bought_Shield_002",
0x4a4b5d07,"NPC_Lanayru001_Bought_Shield_003",
0xb8c11f40,"NPC_Lanayru001_DispNameFlag",
0x85d40865,"NPC_Lanayru001_First",
0x8b37a4ad,"NPC_Lanayru001_SoldOut",
0xc4a03a9b,"NPC_Lanayru002_DispNameFlag",
0xb43c12f8,"NPC_Lanayru002_First",
0x932a0a24,"NPC_SystemTextBoard_DispNameFlag",
0xe5ab7dc1,"NPC_SystemTextBook_DispNameFlag",
0x22d4123c,"NPC_SystemTextDemonStatue_DispNameFlag",
0x26bbdf2c,"NPC_TominagaTest03",
0x54dd9bc9,"NPC_artist_000_DispNameFlag",
0xe65a0c87,"NPC_oasis004_DispNameFlag",
0x92b7a6bc,"NPC_oasis004_fr01_DispNameFlag",
0xeed68367,"NPC_oasis004_fr02_DispNameFlag",
0x73d96211,"NPC_oasis004_fr03_DispNameFlag",
0x1614c8d1,"NPC_oasis004_fr04_DispNameFlag",
0xb5aa808c,"Nakayama_Test_Flag_0",
0x42022020,"NakedIsland_Activated",
0xb9ae83a2,"NakedIsland_AppearDungeon",
0xc5587bc9,"NakedIsland_DemoAppearDungeon",
0xfc4251b4,"NakedIsland_EraseBall_1",
0x654b000e,"NakedIsland_EraseBall_2",
0x124c3098,"NakedIsland_EraseBall_3",
0x39b7eddb,"NakedIsland_Finish",
0xff4be481,"NakedIsland_KillGiant",
0x61109cb6,"NakedIsland_ReachTop",
0x3aa53739,"NakedIsland_Ready",
0x6707ac20,"NakedIsland_SetBall_1",
0xfe0efd9a,"NakedIsland_SetBall_2",
0x8909cd0c,"NakedIsland_SetBall_3",
0xbf7dcb10,"NightStoneBreak",
0xc872e122,"NightStoneDungeonAppear",
0xffb6a59f,"NightStoneRead",
0x3b578fb8,"Normal_DaijinTalk01",
0xf4fccb45,"Normal_FraratTalk01",
0x77ed5c78,"Normal_KingTalk01",
0xcd76e2cf,"Normal_PrinceTalk01",
0x547fb375,"Normal_PrinceTalk02",
0x2add2d9f,"NotGet_Premium_Shield_1",
0xb3d47c25,"NotGet_Premium_Shield_2",
0xc4d34cb3,"NotGet_Premium_Shield_3",
0x5ab7d910,"NotGet_Premium_Shield_4",
0x2db0e986,"NotGet_Premium_Shield_5",
0xb4b9b83c,"NotGet_Premium_Shield_6",
0xc3be88aa,"NotGet_Premium_Shield_7",
0x5301953b,"NotGet_Premium_Shield_8",
0x2406a5ad,"NotGet_Premium_Shield_9",
0x47111a07,"NpcRoad020_First",
0x75313793,"NpcRoad044_First",
0xf466b7df,"Npc_AncientAssistant001_DispNameFlag",
0x15087372,"Npc_AncientAssistant003_DispNameFlag",
0x645db948,"Npc_AncientAssistant003_First",
0xf16479d6,"Npc_AncientAssistant003_FirstClimb",
0x07472f7c,"Npc_AncientAssistant003_FirstPower",
0x352e9e33,"Npc_AncientAssistant003_FirstRubber",
0x70c5d9b2,"Npc_AncientAssistant004_DispNameFlag",
0x8dfe7419,"Npc_AncientDoctor_DispNameFlag",
0xfd8dfa40,"Npc_AncientDoctor_Hateno_DispNameFlag",
0x6f06bad9,"Npc_Assassin_001",
0xf60feb63,"Npc_Assassin_002",
0x8108dbf5,"Npc_Assassin_003",
0x1f6c4e56,"Npc_Assassin_004",
0x686b7ec0,"Npc_Assassin_005",
0xf1622f7a,"Npc_Assassin_006",
0x86651fec,"Npc_Assassin_007",
0x16da027d,"Npc_Assassin_008",
0x61dd32eb,"Npc_Assassin_009",
0x011abb0e,"Npc_Assassin_010",
0xb297b902,"Npc_Assassin_FirstEncounter",
0x212d3c5d,"Npc_Assassin_Talk",
0x8ba2bcca,"Npc_Attacked_001_DispNameFlag",
0x79f96e36,"Npc_Attacked_001_Reward",
0x09c446ce,"Npc_Attacked_001_Saved",
0xf7c39911,"Npc_Attacked_002_DispNameFlag",
0x6acc7867,"Npc_Attacked_003_DispNameFlag",
0x3bdc694b,"Npc_Attacked_003_Reward",
0x9e5b57e7,"Npc_Attacked_003_Saved",
0x0f01d2a7,"Npc_Attacked_004_DispNameFlag",
0x920e33d1,"Npc_Attacked_005_DispNameFlag",
0xfdb360cc,"Npc_Attacked_005_Reward",
0xfd8b62dd,"Npc_Attacked_005_Saved",
0xee6f160a,"Npc_Attacked_006_DispNameFlag",
0x7360f77c,"Npc_Attacked_007_DispNameFlag",
0x25f4438a,"Npc_Attacked_008_DispNameFlag",
0xb8fba2fc,"Npc_Attacked_009_DispNameFlag",
0xaf568654,"Npc_Attacked_010_DispNameFlag",
0x32596722,"Npc_Attacked_011_DispNameFlag",
0x4e3842f9,"Npc_Attacked_012_DispNameFlag",
0xe00d7d96,"Npc_Attacked_012_Reward",
0xf4865ccd,"Npc_Attacked_012_Saved",
0x39b5fc0b,"Npc_BackseatKorok_Air_DispNameFlag",
0x89dcfa57,"Npc_BackseatKorok_Ground_DispNameFlag",
0x10ad4379,"Npc_Bottle_Mes001_DispNameFlag",
0x9d6e179e,"Npc_Bottle_Mes001_NormalFirst",
0x77b3397c,"Npc_Bottle_Mes001_ReadyFirst",
0xcf975e6c,"Npc_Bottle_Mes001_Talk",
0xad075262,"Npc_Bowling_DispNameFlag",
0xe604cf19,"Npc_Brigade_001_Finish",
0xf7b46bf0,"Npc_Challenge093Lady_DispNameFlag",
0x1ab70d92,"Npc_DeathEastHatago_001_DispNameFlag",
0xdecb2b47,"Npc_DeathEastHatago_002_Change",
0x66d62849,"Npc_DeathEastHatago_002_DispNameFlag",
0x6a312a50,"Npc_DeathEastHatago_002_First",
0x84af2d85,"Npc_DeathEastHatago_002_GoronHelmGet",
0xfbd9c93f,"Npc_DeathEastHatago_003_DispNameFlag",
0xcc4621e4,"Npc_DeathEastHatago_003_First",
0x9ae7f54e,"Npc_DeathMt001_DispNameFlag",
0xcf2e2340,"Npc_DeathWestHatago_002_DispNameFlag",
0x3d59a7c3,"Npc_DemonStatue_First",
0xb072df42,"Npc_DemonStatue_FirstSell",
0xdda7fdd3,"Npc_DemonStatue_Ganbari",
0xcfc48a72,"Npc_DemonStatue_Heart",
0xef3ece3b,"Npc_DressFairy_00_DispNameFlag",
0x72312f4d,"Npc_DressFairy_01_DispNameFlag",
0x0e500a96,"Npc_DressFairy_02_DispNameFlag",
0x935febe0,"Npc_DressFairy_03_DispNameFlag",
0x13460aab,"Npc_DungeonPriest_First",
0x66ca1284,"Npc_FaronHatago_001_DispNameFlag",
0xeb1a4652,"Npc_FaronWoods001_DispNameFlag",
0x0a7482ff,"Npc_FaronWoods003_DispNameFlag",
0xc34f3c63,"Npc_FaronWoods003_IsTalked",
0x8ed7ec92,"Npc_FaronWoods006_DispNameFlag",
0x527b49ca,"Npc_FaronWoods006_talk",
0xe1dd9389,"Npc_FaronWoods007_Axe",
0x13d80de4,"Npc_FaronWoods007_DispNameFlag",
0x99279a6f,"Npc_FaronWoods007_talk",
0x454cb912,"Npc_FaronWoods008_DispNameFlag",
0xd8435864,"Npc_FaronWoods009_DispNameFlag",
0xcfee7ccc,"Npc_FaronWoods010_DispNameFlag",
0xa8a1979a,"Npc_FarthestIsland002_DispNameFlag",
0x674d6093,"Npc_FirstColony004_Defeat",
0xacce8863,"Npc_FirstColony004_DefeatFirst",
0x1aa48c81,"Npc_FirstColony004_DispNameFlag",
0x1ad489f1,"Npc_FirstColony004_talk",
0xfdd9bd0c,"Npc_FirstColony010_First",
0xbae957b5,"Npc_FirstColony010_King",
0xe684b544,"Npc_GerudoCanyonHatago001_DispNameFlag",
0x929be8e5,"Npc_GerudoDesert001_DispNameFlag",
0xeefacd3e,"Npc_GerudoDesert002_DispNameFlag",
0x73f52c48,"Npc_GerudoDesert003_DispNameFlag",
0x16388688,"Npc_GerudoDesert004_DispNameFlag",
0xf0d57e68,"Npc_Goron005_DispNameFlag",
0x78dcc96d,"Npc_Goron006_DemoTalk",
0x8cb45bb3,"Npc_Goron006_DispNameFlag",
0x50822a9b,"Npc_Goron011_DispNameFlag",
0x2ce30f40,"Npc_Goron012_DispNameFlag",
0xb1ecee36,"Npc_Goron013_DispNameFlag",
0xd42144f6,"Npc_Goron014_DispNameFlag",
0x492ea580,"Npc_Goron015_DispNameFlag",
0x354f805b,"Npc_Goron016_DispNameFlag",
0xa840612d,"Npc_Goron017_DispNameFlag",
0xfed4d5db,"Npc_Goron018_DispNameFlag",
0x8f159d84,"Npc_Goron018_Reward",
0xdcf0a194,"Npc_Goron020_DispNameFlag",
0x8ae3dbed,"Npc_Goron020_First",
0x770e2042,"Npc_Goron020_Secret",
0x41ff40e2,"Npc_Goron021_DispNameFlag",
0xa4e0fb2e,"Npc_Goron021_Intro",
0x3d9e6539,"Npc_Goron022_DispNameFlag",
0xa091844f,"Npc_Goron023_DispNameFlag",
0x5853cff9,"Npc_Goron025_DispNameFlag",
0x650b7a7c,"Npc_Goron030_DispNameFlag",
0xf8049b0a,"Npc_Goron031_DispNameFlag",
0x8465bed1,"Npc_Goron032_DispNameFlag",
0x196a5fa7,"Npc_Goron033_DispNameFlag",
0x74487ed4,"Npc_Goron109_DispNameFlag",
0x63e55a7c,"Npc_Goron110_DispNameFlag",
0x980b2ebd,"Npc_Goron_Camp001_DispNameFlag",
0xe46a0b66,"Npc_Goron_Camp002_DispNameFlag",
0x7965ea10,"Npc_Goron_Camp003_DispNameFlag",
0xa8bda8c0,"Npc_HatenoGate001_DispNameFlag",
0x6aa7de39,"Npc_HatenoGate001_First",
0xc277db7b,"Npc_HatenoGate001_GuideFinish",
0xe945bf27,"Npc_HatenoGate002",
0xd4dc8d1b,"Npc_HatenoGate002_DispNameFlag",
0x49d36c6d,"Npc_HatenoGate003_DispNameFlag",
0xfd38cf10,"Npc_HatenoGate003_First",
0x5964894d,"Npc_HatenoGate003_second",
0x56d39352,"Npc_HatenoSee00_Get",
0x6b57e982,"Npc_HatenoVillage001_DispNameFlag",
0x1736cc59,"Npc_HatenoVillage002_DispNameFlag",
0x2c568c2c,"Npc_HatenoVillage002_First",
0x8a392d2f,"Npc_HatenoVillage003_DispNameFlag",
0xeff487ef,"Npc_HatenoVillage004_DispNameFlag",
0x72fb6699,"Npc_HatenoVillage005_DispNameFlag",
0x0e9a4342,"Npc_HatenoVillage006_DispNameFlag",
0x9395a234,"Npc_HatenoVillage007_DispNameFlag",
0xc50116c2,"Npc_HatenoVillage008_DispNameFlag",
0x8826d362,"Npc_HatenoVillage008_First",
0x580ef7b4,"Npc_HatenoVillage009_DispNameFlag",
0x4fa3d31c,"Npc_HatenoVillage010_DispNameFlag",
0xd2ac326a,"Npc_HatenoVillage011_DispNameFlag",
0xaecd17b1,"Npc_HatenoVillage012_DispNameFlag",
0x33c2f6c7,"Npc_HatenoVillage013_DispNameFlag",
0x560f5c07,"Npc_HatenoVillage014_DispNameFlag",
0xcb00bd71,"Npc_HatenoVillage015_DispNameFlag",
0xa8162afb,"Npc_HatenoVillage015_talk",
0x6373b7f0,"Npc_HatenoVillage016_Clear",
0xb76198aa,"Npc_HatenoVillage016_DispNameFlag",
0x2e825855,"Npc_HatenoVillage016_talk",
0x2a6e79dc,"Npc_HatenoVillage017_DispNameFlag",
0x7cfacd2a,"Npc_HatenoVillage018_DispNameFlag",
0xe1f52c5c,"Npc_HatenoVillage019_DispNameFlag",
0x08d8489f,"Npc_HatenoVillage019_Etto",
0xe2fbd848,"Npc_HatenoVillage019_First",
0x474bd9c3,"Npc_HatenoVillage019_Kokotte",
0x5edeb965,"Npc_HatenoVillage020_DispNameFlag",
0xc3d15813,"Npc_HatenoVillage021_DispNameFlag",
0xbfb07dc8,"Npc_HatenoVillage022_DispNameFlag",
0x22bf9cbe,"Npc_HatenoVillage023_DispNameFlag",
0x4beb7ebf,"Npc_HatenoVillage024_Country",
0x4772367e,"Npc_HatenoVillage024_DispNameFlag",
0x0da3be6b,"Npc_HatenoVillage024_First",
0x4bbd3d8f,"Npc_HatenoVillage024_ThisFirst",
0x500e0ac4,"Npc_HatenoVillage024_Uotori",
0x90bd1f72,"Npc_HatenoVillage024_UotoriFirst",
0xda7dd708,"Npc_HatenoVillage025_DispNameFlag",
0xa61cf2d3,"Npc_HatenoVillage026_DispNameFlag",
0x3b1313a5,"Npc_HatenoVillage027_DispNameFlag",
0x6d87a753,"Npc_HatenoVillage028_DispNameFlag",
0xf0884625,"Npc_HatenoVillage029_DispNameFlag",
0xe725628d,"Npc_HatenoVillage030_DispNameFlag",
0x7a2a83fb,"Npc_HatenoVillage031_DispNameFlag",
0x064ba620,"Npc_HatenoVillage032_DispNameFlag",
0xd50c0ea9,"Npc_HatenoVillage032_Go",
0x9b444756,"Npc_HatenoVillage033_DispNameFlag",
0x31f7a5b4,"Npc_HatenoVillage_DispNameFlag",
0x4e78199b,"Npc_HighMountain001_DispNameFlag",
0xaf16dd36,"Npc_HighMountain003_DispNameFlag",
0xcadb77f6,"Npc_HighMountain004_DispNameFlag",
0x57d49680,"Npc_HighMountain005_DispNameFlag",
0xe02ee6db,"Npc_HighMountain008_DispNameFlag",
0xe1fb89a4,"Npc_HighMountain008_SoldOut",
0x7d2107ad,"Npc_HighMountain009_DispNameFlag",
0x6a8c2305,"Npc_HighMountain010_DispNameFlag",
0xf783c273,"Npc_HighMountain011_DispNameFlag",
0x8be2e7a8,"Npc_HighMountain012_DispNameFlag",
0x16ed06de,"Npc_HighMountain013_DispNameFlag",
0x7bf1497c,"Npc_HighMountain020_DispNameFlag",
0xe6fea80a,"Npc_HighMountain021_DispNameFlag",
0xff522711,"Npc_HighMountain025_DispNameFlag",
0xc20a9294,"Npc_HighMountain030_DispNameFlag",
0x5f0573e2,"Npc_HighMountain031_DispNameFlag",
0x23645639,"Npc_HighMountain032_DispNameFlag",
0xbe6bb74f,"Npc_HighMountain033_DispNameFlag",
0xdba61d8f,"Npc_HighMountain034_DispNameFlag",
0x46a9fcf9,"Npc_HighMountain035_DispNameFlag",
0x3ac8d922,"Npc_HighMountain036_DispNameFlag",
0xa7c73854,"Npc_HighMountain037_DispNameFlag",
0x590b9d8e,"Npc_HighMountain040_DispNameFlag",
0xe365ee36,"Npc_HighMountain040_First",
0x7415379d,"Npc_HighMountain040_Reward",
0xc4047cf8,"Npc_HighMountain041_DispNameFlag",
0x4512e582,"Npc_HighMountain041_First",
0xb8bf3703,"Npc_HighMountain041_Reward",
0xe99e4144,"Npc_HorseBuyer_DispNameFlag",
0x7f2b7021,"Npc_Hunters_Lost_DispNameFlag",
0xd99d04a7,"Npc_HutagoHatago_001_DispNameFlag",
0xa5fc217c,"Npc_HutagoHatago_002_DispNameFlag",
0x42a1ef3f,"Npc_HutagoHatago_Woman_01_1stGreeting",
0xd5bdd993,"Npc_HutagoHatago_Woman_01_DispNameFlag",
0xbed4a489,"Npc_HutagoHatago_Woman_01_fisrt",
0x98d5ae7f,"Npc_HutagoHatago_Woman_01_gave",
0x7aa97a5a,"Npc_HyruleDepthHatago001_DispNameFlag",
0x06c85f81,"Npc_HyruleDepthHatago002_DispNameFlag",
0x9bc7bef7,"Npc_HyruleDepthHatago003_DispNameFlag",
0xfe0a1437,"Npc_HyruleDepthHatago004_DispNameFlag",
0x6305f541,"Npc_HyruleDepthHatago005_DispNameFlag",
0x1f64d09a,"Npc_HyruleDepthHatago006_DispNameFlag",
0x826b31ec,"Npc_HyruleDepthHatago007_DispNameFlag",
0xc38d0efd,"Npc_HyrulePlain002_talk",
0x08d1dd58,"Npc_HyrulePlain003_talk",
0x571c4213,"Npc_HyrulePlain004_DispNameFlag",
0x15d4ede0,"Npc_HyrulePlain004_talk",
0xca13a365,"Npc_HyrulePlain005_DispNameFlag",
0xb67286be,"Npc_HyrulePlain006_DispNameFlag",
0x447c3b1e,"Npc_HyruleWestHatago001_DispNameFlag",
0x381d1ec5,"Npc_HyruleWestHatago002_DispNameFlag",
0xa512ffb3,"Npc_HyruleWestHatago003_DispNameFlag",
0xc0df5573,"Npc_HyruleWestHatago004_DispNameFlag",
0x5dd0b405,"Npc_HyruleWestHatago005_DispNameFlag",
0x21b191de,"Npc_HyruleWestHatago006_DispNameFlag",
0x1f6dacbb,"Npc_IceVillage011_DispNameFlag",
0x22a7f103,"Npc_Kakariko001_1Shinju",
0x134feb9e,"Npc_Kakariko001_2Shinju",
0x709fdea4,"Npc_Kakariko001_4Shinju",
0x78784f5c,"Npc_Kakariko001_DispNameFlag",
0x4a8f2677,"Npc_Kakariko001_Talk",
0xd7d6165b,"Npc_Kakariko001_TalkEnd",
0xb2ef35e8,"Npc_Kakariko001_eiketu_first",
0x93e2bec5,"Npc_Kakariko001_ms_first",
0x04196a87,"Npc_Kakariko002_DispNameFlag",
0x319ed1a2,"Npc_Kakariko002_First",
0xcc1b54d9,"Npc_Kakariko002_Talk",
0xcc3ddac4,"Npc_Kakariko002_thanks",
0xcd9b7071,"Npc_Kakariko003_Cha_first",
0x8e5028a2,"Npc_Kakariko003_Cooking1_started",
0xb7dd1467,"Npc_Kakariko003_Cooking2_started",
0xa0a60024,"Npc_Kakariko003_Cooking3_started",
0x99168bf1,"Npc_Kakariko003_DispNameFlag",
0x4c5b7b30,"Npc_Kakariko003_Food",
0xd608a676,"Npc_Kakariko003_cooking1_finished",
0x3d3f1d75,"Npc_Kakariko003_cooking2_finished",
0xd2fd764b,"Npc_Kakariko003_cooking3_finished",
0xfcdb2131,"Npc_Kakariko004_DispNameFlag",
0x61d4c047,"Npc_Kakariko005_DispNameFlag",
0xf439ef2c,"Npc_Kakariko005_First",
0x19bc78e9,"Npc_Kakariko005_First_Behind",
0xd11e6461,"Npc_Kakariko005_Talk",
0xa0c4dd47,"Npc_Kakariko005_fisrtkokko",
0x8ecc97d0,"Npc_Kakariko005_reward",
0x1db5e59c,"Npc_Kakariko006_DispNameFlag",
0xec5ae870,"Npc_Kakariko006_GavePumpkin",
0xad6f6a12,"Npc_Kakariko006_SoldOut",
0x578a16cf,"Npc_Kakariko006_Talk",
0x2ca83b90,"Npc_Kakariko006_excuse",
0x0410dab5,"Npc_Kakariko006_first",
0x80ba04ea,"Npc_Kakariko007_DispNameFlag",
0x7da288b0,"Npc_Kakariko007_Talk_Day",
0x258f2049,"Npc_Kakariko007_Talk_Night",
0xd62eb01c,"Npc_Kakariko008_DispNameFlag",
0x5c8c75c2,"Npc_Kakariko010_DispNameFlag",
0x31b778a6,"Npc_Kakariko010_SoldOut",
0x27a4fe66,"Npc_Kakariko010_Talk",
0xc18394b4,"Npc_Kakariko011_DispNameFlag",
0xecf82dc3,"Npc_Kakariko011_Talk",
0x230a5576,"Npc_Kakariko012_Demo",
0xbde2b16f,"Npc_Kakariko012_DispNameFlag",
0x63b3c37a,"Npc_Kakariko012_NormalSuits_first",
0x6e9d555d,"Npc_Kakariko012_Sheikah_first",
0x1f415020,"Npc_Kakariko012_SoldOut",
0xeb5529fd,"Npc_Kakariko012_SoldOut_SeeKer",
0x6a6c5f6d,"Npc_Kakariko012_Talk",
0x7775cbdb,"Npc_Kakariko012_eiketu_first",
0x1a5ea645,"Npc_Kakariko013_ClerkAsk",
0x20ed5019,"Npc_Kakariko013_DispNameFlag",
0x083a4463,"Npc_Kakariko013_SoldOut",
0xa1308cc8,"Npc_Kakariko013_Talk",
0x4520fad9,"Npc_Kakariko014_DispNameFlag",
0x0195826d,"Npc_Kakariko014_PlantM_SoldOut",
0x6c5b29aa,"Npc_Kakariko014_SoldOut",
0xbc35bc70,"Npc_Kakariko014_Talk",
0xd82f1baf,"Npc_Kakariko015_DispNameFlag",
0x77696fd5,"Npc_Kakariko015_Talk",
0x12b8611b,"Npc_Kakariko015_eiketu_first",
0xf952c0b6,"Npc_Kakariko015_first",
0xa44e3e74,"Npc_Kakariko016_DispNameFlag",
0x6ed944c0,"Npc_Kakariko016_eiketu_first",
0x243bbc8f,"Npc_KakarikoWarrior_Open",
0x62dcbf79,"Npc_Kakariko_016_first",
0xe084197a,"Npc_Kakariko_Artist_followed",
0x55dc0ffa,"Npc_King001_Appear",
0xb78daf31,"Npc_King001_Disappear",
0xcfcbc22d,"Npc_King_ChoiceExclude_Diary",
0x2b4c8416,"Npc_King_ChoiceExclude_Finish",
0xd10dde39,"Npc_King_ChoiceExclude_Recipe",
0x5b52db57,"Npc_King_Common_Anataha",
0xc8349f36,"Npc_King_Common_Kokoha",
0x69d2d0d7,"Npc_King_Cook_Kotsu",
0x365cddb5,"Npc_King_Parasail001_Appear",
0x92fd4112,"Npc_King_Parasail001_Disappear",
0x8cc4ac87,"Npc_King_Parasail001_Talk",
0xb8d3da56,"Npc_King_Parasail002_Appear",
0x0b1f2713,"Npc_King_Parasail002_Disappear",
0x9e60e0f4,"Npc_King_Parasail002_Disappear2",
0x0a50de29,"Npc_King_Parasail002_Talk",
0x7479dac8,"Npc_King_Parasail003_Appear",
0x8f1e8d55,"Npc_King_Parasail003_Appear2",
0xca91f8d3,"Npc_King_Parasail003_Disappear",
0xc10c0d8c,"Npc_King_Parasail003_Talk",
0x7ebcd3d1,"Npc_King_Parasail004_Appear",
0xe3aaed50,"Npc_King_Parasail004_Disappear",
0xdc093d34,"Npc_King_Parasail004_Talk",
0xb216d34f,"Npc_King_Parasail005_Appear",
0x22243290,"Npc_King_Parasail005_Disappear",
0x1755ee91,"Npc_King_Parasail005_Talk",
0xbf9ec365,"Npc_King_Tennokoe",
0x067d656f,"Npc_King_Tennokoe_Ready",
0x9f31f2bd,"Npc_King_Vagrant001_Apple",
0x2d6d50e8,"Npc_King_Vagrant001_ChoiceExclude",
0x2dcae8a0,"Npc_King_Vagrant001_DispNameFlag",
0xf3562cd2,"Npc_King_Vagrant001_Move",
0x834cce11,"Npc_King_Vagrant001_Talk",
0xc5e6199a,"Npc_King_Vagrant001_Torch",
0x05d8bcbf,"Npc_King_Vagrant002_Talk",
0xa9ba3eeb,"Npc_King_Vagrant003_BOKANGI",
0xce846f1a,"Npc_King_Vagrant003_Talk",
0x1eeaee63,"Npc_King_Vagrant004_ChoiceExclude",
0xf55ce0a8,"Npc_King_Vagrant004_Diary",
0x4ce53eb4,"Npc_King_Vagrant004_Recipe",
0xd3815fa2,"Npc_King_Vagrant004_Talk",
0xa48cd351,"Npc_King_Vagrant004_Talk_Axe",
0x5d252f18,"Npc_King_Vagrant004_Talk_Cook",
0xa758c9b9,"Npc_King_Vagrant004_ZRock",
0xa711358b,"Npc_King_Vagrant005_ChoiceExclude",
0xd538879a,"Npc_King_Vagrant005_Scout",
0x18dd8c07,"Npc_King_Vagrant005_Talk",
0xbf32f3e8,"Npc_King_Vagrant005_Talk_Bow",
0x9cabf0d8,"Npc_King_Vagrant005_Talk_Cook",
0x9e49fea9,"Npc_King_Vagrant006_Talk",
0xaf7fd6d1,"Npc_King_Vagrant006_Talk_Kotsu",
0x3195afd1,"Npc_King_Vagrant006_Talk_ToshinoKou",
0xd3068e71,"Npc_King_Vagrant006_TooLate",
0x39a13ad2,"Npc_King_Vagrant_Delete",
0x3ff5e4a9,"Npc_Kokiri007_DispNameFlag",
0xad708022,"Npc_Kokiri007_Talk",
0x6961505f,"Npc_Kokiri008_DispNameFlag",
0x5c2632f7,"Npc_Kokiri008_Talk",
0xe3c39581,"Npc_Kokiri010_DispNameFlag",
0x7ecc74f7,"Npc_Kokiri011_DispNameFlag",
0x3ec8a02e,"Npc_LakeSideHatago004_DispNameFlag",
0xa3c74158,"Npc_LakeSideHatago005_DispNameFlag",
0xdfa66483,"Npc_LakeSideHatago006_DispNameFlag",
0x66542580,"Npc_Lanayru003_DispNameFlag",
0xa3369ece,"Npc_Lanayru003_LieFirst",
0x8a69c7dd,"Npc_Lanayru003_Mifa",
0xb8497ede,"Npc_Lanayru003_NormalFirst",
0x63f2e99f,"Npc_Lanayru003_RiverFirst",
0xb3518fe4,"Npc_Lanayru003_Talk",
0x03998f40,"Npc_Lanayru004_DispNameFlag",
0x176a0010,"Npc_Lanayru004_RiverFirst",
0x472b128f,"Npc_MamonoShop",
0xcea1023e,"Npc_MamonoShop_DispNameFlag",
0x9a8be937,"Npc_MiniGame_Crosscountry_DispNameFlag",
0xf92aadd5,"Npc_MiniGame_Golf_DispNameFlag",
0x4c795ce2,"Npc_Minigame_BirdMan001_DispNameFlag",
0x23e800ee,"Npc_Musician_001_DispNameFlag",
0xc42839fb,"Npc_Musician_001_Hint",
0x42bc4b55,"Npc_Musician_002_Hint",
0x89e098f0,"Npc_Musician_003_Hint",
0x94e5a848,"Npc_Musician_004_Hint",
0x5fb97bed,"Npc_Musician_005_Hint",
0xa3afcfed,"Npc_Musician_006_FirstTalk",
0xd92d0943,"Npc_Musician_006_Hint",
0x295c1e8d,"Npc_Musician_007_First",
0x1271dae6,"Npc_Musician_007_Hint",
0xe3276833,"Npc_Musician_008_Hint",
0x287bbb96,"Npc_Musician_009_Hint",
0xa903e1ea,"Npc_Musician_010_Hint",
0x0cda1f2d,"Npc_Musician_Come",
0x26d05df4,"Npc_Musician_First",
0xe5163cbd,"Npc_Musician_LastTalk",
0xf83367bc,"Npc_NorthHatelHatago001_DispNameFlag",
0x9bf114e9,"Npc_NorthHatelHatago001_Talk",
0x84524267,"Npc_NorthHatelHatago002_DispNameFlag",
0x9dfecd7c,"Npc_NorthHatelHatago006_DispNameFlag",
0x68f0382d,"Npc_NorthHateru002_First",
0x4604d2a4,"Npc_NorthHateru003_DispNameFlag",
0xce873399,"Npc_NorthHateru003_First",
0x7aa1b03e,"Npc_NorthHateru003_talk",
0xbec69912,"Npc_NorthHateru005_DispNameFlag",
0x9aa46613,"Npc_NorthHateru_around001_DispNameFlag",
0x3c972314,"Npc_Npc_goron002_SoldOut",
0x2ed295ff,"Npc_OasisGoddess_DispNameFlag",
0x1224fd99,"Npc_OasisGoddess_first",
0xec477307,"Npc_OasisGoron_001_DispNameFlag",
0x9082f286,"Npc_OasisHighMountain_001_DispNameFlag",
0x6400663d,"Npc_OasisHighMountain_001_first",
0x1f993354,"Npc_OasisHylia_001_DispNameFlag",
0xee0c9276,"Npc_OasisHylia_001_first",
0xa32e4303,"Npc_OasisHylia_002_Barefoot",
0x63f8168f,"Npc_OasisHylia_002_DispNameFlag",
0x667c4a1b,"Npc_OasisHylia_002_SandBoots",
0x9c692336,"Npc_OasisHylia_002_SnowBoots",
0xfef7f7f9,"Npc_OasisHylia_003_DispNameFlag",
0x7993835f,"Npc_OasisHylia_003_first",
0xb5b164a8,"Npc_OasisHylia_016_angry",
0xb6a61261,"Npc_OasisMilk_A_DispNameFlag",
0xcac737ba,"Npc_OasisMilk_B_DispNameFlag",
0x57c8d6cc,"Npc_OasisMilk_C_DispNameFlag",
0x32057c0c,"Npc_OasisMilk_D_DispNameFlag",
0xaf0a9d7a,"Npc_OasisMilk_E_DispNameFlag",
0xf52f4930,"Npc_OasisSoldier_A_DispNameFlag",
0x894e6ceb,"Npc_OasisSoldier_B_DispNameFlag",
0x14418d9d,"Npc_OasisSoldier_C_DispNameFlag",
0x31db3df0,"Npc_OasisStep_A_DispNameFlag",
0x4dba182b,"Npc_OasisStep_B_DispNameFlag",
0x02bf926c,"Npc_OasisStudent_A_DispNameFlag",
0x7edeb7b7,"Npc_OasisStudent_B_DispNameFlag",
0x16599d18,"Npc_OasisStudent_B_fisrt",
0xe9866849,"Npc_OasisStudent_B_second",
0xb8c3085d,"Npc_OasisStudent_B_third",
0xe3d156c1,"Npc_OasisStudent_C_DispNameFlag",
0x2a1d9d5a,"Npc_OasisTeacher_DispNameFlag",
0x8fb9bc98,"Npc_OasisTeacher_first",
0xf3b26380,"Npc_OasisTeacher_lesson",
0xf79e8f4e,"Npc_OasisTeacher_lesson2",
0x8099bfd8,"Npc_OasisTeacher_lesson3",
0xda9b6544,"Npc_OasisWaterChannel_DispNameFlag",
0xc53c70bc,"Npc_OldKorok_900First",
0x2a859497,"Npc_OldKorok_Bow01",
0xb38cc52d,"Npc_OldKorok_Bow02",
0xc48bf5bb,"Npc_OldKorok_Bow03",
0x5aef6018,"Npc_OldKorok_Bow04",
0x2de8508e,"Npc_OldKorok_Bow05",
0xb4e10134,"Npc_OldKorok_Bow06",
0xc3e631a2,"Npc_OldKorok_Bow07",
0x53592c33,"Npc_OldKorok_Bow08",
0x245e1ca5,"Npc_OldKorok_Bow09",
0xba07c493,"Npc_OldKorok_DispNameFlag",
0x214706b9,"Npc_OldKorok_First",
0x5e1fd2c8,"Npc_OldKorok_FullComp",
0x42946b2c,"Npc_OldKorok_GetNuts",
0xcb5ae1dd,"Npc_OldKorok_Shield01",
0x5253b067,"Npc_OldKorok_Shield02",
0x255480f1,"Npc_OldKorok_Shield03",
0xbb301552,"Npc_OldKorok_Shield04",
0xcc3725c4,"Npc_OldKorok_Shield05",
0x553e747e,"Npc_OldKorok_Shield06",
0x223944e8,"Npc_OldKorok_Shield07",
0xb2865979,"Npc_OldKorok_Shield08",
0xc58169ef,"Npc_OldKorok_Shield09",
0xa546e00a,"Npc_OldKorok_Shield10",
0xd241d09c,"Npc_OldKorok_Shield11",
0x4b488126,"Npc_OldKorok_Shield12",
0x3c4fb1b0,"Npc_OldKorok_Shield13",
0xa22b2413,"Npc_OldKorok_Shield14",
0xd52c1485,"Npc_OldKorok_Shield15",
0x4c25453f,"Npc_OldKorok_Shield16",
0x8968dbba,"Npc_OldKorok_Top",
0x75b63e40,"Npc_OldKorok_Weapon01",
0xecbf6ffa,"Npc_OldKorok_Weapon02",
0x9bb85f6c,"Npc_OldKorok_Weapon03",
0x05dccacf,"Npc_OldKorok_Weapon04",
0x72dbfa59,"Npc_OldKorok_Weapon05",
0xebd2abe3,"Npc_OldKorok_Weapon06",
0x9cd59b75,"Npc_OldKorok_Weapon07",
0x0c6a86e4,"Npc_OldKorok_Weapon08",
0x7b6db672,"Npc_OldKorok_Weapon09",
0x1baa3f97,"Npc_OldKorok_Weapon10",
0x6cad0f01,"Npc_OldKorok_Weapon11",
0xdac38e47,"Npc_Remains_Fancier001_DispNameFlag",
0x0993ba27,"Npc_RitoHatago001_DispNameFlag",
0x75f29ffc,"Npc_RitoHatago002_DispNameFlag",
0xe8fd7e8a,"Npc_RitoHatago003_DispNameFlag",
0x8d30d44a,"Npc_RitoHatago004_DispNameFlag",
0xf395c0fb,"Npc_RiverSideHatago001_DispNameFlag",
0x75a355e3,"Npc_RiverSideHatago002_Bow",
0x8ff4e520,"Npc_RiverSideHatago002_DispNameFlag",
0x57b1c506,"Npc_RiverSideHatago002_Lsword",
0x8b53e9c5,"Npc_RiverSideHatago002_Reward",
0xdb11497e,"Npc_RiverSideHatago002_Shield",
0xa73ea61b,"Npc_RiverSideHatago002_Spear",
0xa245efb7,"Npc_RiverSideHatago002_Sword",
0x12fb0456,"Npc_RiverSideHatago003_DispNameFlag",
0xdc29cab3,"Npc_Road_001_DispNameFlag",
0xb9034285,"Npc_Road_001_Talk",
0xa048ef68,"Npc_Road_002_DispNameFlag",
0x3f97302b,"Npc_Road_002_Talk",
0x3d470e1e,"Npc_Road_003_DispNameFlag",
0xf4cbe38e,"Npc_Road_003_Talk",
0x588aa4de,"Npc_Road_004_DispNameFlag",
0xe9ced336,"Npc_Road_004_Talk",
0xc58545a8,"Npc_Road_005_DispNameFlag",
0x22920093,"Npc_Road_005_Talk",
0xb9e46073,"Npc_Road_006_DispNameFlag",
0xa406723d,"Npc_Road_006_Talk",
0x24eb8105,"Npc_Road_007_DispNameFlag",
0x6f5aa198,"Npc_Road_007_Talk",
0x727f35f3,"Npc_Road_008_DispNameFlag",
0x9e0c134d,"Npc_Road_008_Talk",
0xef70d485,"Npc_Road_009_DispNameFlag",
0x5550c0e8,"Npc_Road_009_Talk",
0xf8ddf02d,"Npc_Road_010_DispNameFlag",
0xd4289a94,"Npc_Road_010_Talk",
0x65d2115b,"Npc_Road_011_DispNameFlag",
0x1f744931,"Npc_Road_011_Talk",
0x3648d34f,"Npc_Road_011_first_boy",
0xa0f48367,"Npc_Road_011_first_girl",
0x19b33480,"Npc_Road_012_DispNameFlag",
0x99e03b9f,"Npc_Road_012_Talk",
0x84bcd5f6,"Npc_Road_013_DispNameFlag",
0x52bce83a,"Npc_Road_013_Talk",
0xe1717f36,"Npc_Road_014_DispNameFlag",
0x03028d38,"Npc_Road_014_Flag01",
0x4fb9d882,"Npc_Road_014_Talk",
0x7c7e9e40,"Npc_Road_015_DispNameFlag",
0x84e50b27,"Npc_Road_015_Talk",
0x001fbb9b,"Npc_Road_016_DispNameFlag",
0x02717989,"Npc_Road_016_Talk",
0x9d105aed,"Npc_Road_017_DispNameFlag",
0xcb84ee1b,"Npc_Road_018_DispNameFlag",
0x568b0f6d,"Npc_Road_019_DispNameFlag",
0xe9a09a54,"Npc_Road_020_DispNameFlag",
0x74af7b22,"Npc_Road_021_DispNameFlag",
0x08ce5ef9,"Npc_Road_022_DispNameFlag",
0x95c1bf8f,"Npc_Road_023_DispNameFlag",
0xf00c154f,"Npc_Road_024_DispNameFlag",
0x6d03f439,"Npc_Road_025_DispNameFlag",
0x1162d1e2,"Npc_Road_026_DispNameFlag",
0x8c6d3094,"Npc_Road_027_DispNameFlag",
0xdaf98462,"Npc_Road_028_DispNameFlag",
0x47f66514,"Npc_Road_029_DispNameFlag",
0x505b41bc,"Npc_Road_030_DispNameFlag",
0xcd54a0ca,"Npc_Road_031_DispNameFlag",
0xddb9d388,"Npc_Road_031_First",
0xb1358511,"Npc_Road_032_DispNameFlag",
0x2d90e611,"Npc_Road_032_first",
0x2c3a6467,"Npc_Road_033_DispNameFlag",
0x49f7cea7,"Npc_Road_034_DispNameFlag",
0xd4f82fd1,"Npc_Road_035_DispNameFlag",
0xa8990a0a,"Npc_Road_036_DispNameFlag",
0x3596eb7c,"Npc_Road_037_DispNameFlag",
0xcb5a4ea6,"Npc_Road_040_DispNameFlag",
0x5655afd0,"Npc_Road_041_DispNameFlag",
0x2a348a0b,"Npc_Road_042_DispNameFlag",
0xb73b6b7d,"Npc_Road_043_DispNameFlag",
0xd2f6c1bd,"Npc_Road_044_DispNameFlag",
0x4ff920cb,"Npc_Road_045_DispNameFlag",
0xa184e1b5,"Npc_Road_Delete",
0xbe44acd6,"Npc_Road_Firone001_LookDragon",
0x9cc29f54,"Npc_Road_Firone001_Talk",
0x764fe97f,"Npc_Road_Firone001_Talk2",
0x737e960a,"Npc_Road_Firone002_DispNameFlag",
0x76043053,"Npc_Road_Firone002_MaronRevivalTalk",
0x1a56edfa,"Npc_Road_Firone002_Talk",
0xb7ac76d1,"Npc_Road_Firone2_Talk2",
0x537966fc,"Npc_SmallOasis001_DispNameFlag",
0xe926055f,"Npc_SmallOasis001_first",
0x2f184327,"Npc_SmallOasis002_DispNameFlag",
0x190f30c6,"Npc_SmallOasis002_First",
0xb217a251,"Npc_SmallOasis003_DispNameFlag",
0xd7da0891,"Npc_SmallOasis004_DispNameFlag",
0x4ad5e9e7,"Npc_SmallOasis005_DispNameFlag",
0x36b4cc3c,"Npc_SmallOasis006_DispNameFlag",
0xabbb2d4a,"Npc_SmallOasis007_DispNameFlag",
0xfd2f99bc,"Npc_SmallOasis008_DispNameFlag",
0x29ff0186,"Npc_SmallOasis008_Fisrt_boy",
0xfed98d87,"Npc_SmallOasis008_Talk",
0x602078ca,"Npc_SmallOasis009_DispNameFlag",
0x778d5c62,"Npc_SmallOasis010_DispNameFlag",
0xea82bd14,"Npc_SmallOasis011_DispNameFlag",
0xe44d2ac5,"Npc_SmallOasis011_First",
0xae334dcb,"Npc_SmallOasis011_lady_first",
0x96e398cf,"Npc_SmallOasis012_DispNameFlag",
0x92d4e5e2,"Npc_SourthernVillage001_IsTalked",
0xcc6d1ba9,"Npc_SourthernVillage004_EatFish",
0x5d87fbea,"Npc_SourthernVillage004_EatFishTalk",
0x00bf5b35,"Npc_SourthernVillage004_FishTalk",
0x74fd2ea6,"Npc_SourthernVillage004_IsTalked",
0x9b3f4598,"Npc_SourthernVillage005_IsTalked",
0x6ec1ce28,"Npc_SourthernVillage008_IsTalked",
0x2b72e4b6,"Npc_SourthernVillage009_GuideEnd",
0x8103a516,"Npc_SourthernVillage009_IsTalked",
0xb8d74c79,"Npc_SourthernVillage010_CheckLoanRupee",
0xe131e14c,"Npc_SourthernVillage010_FirstLoanRupee",
0xbc98511c,"Npc_SourthernVillage010_IsTalked",
0x3cc891de,"Npc_SourthernVillage010_LoanRupee",
0xb86d8121,"Npc_SourthernVillage012_IsTalked",
0x29210c86,"Npc_SourthernVillage013_TalkIseki",
0x0c8e0725,"Npc_SourthernVillage013_TalkLorel",
0x07db125a,"Npc_SourthernVillage013_TalkSaihate",
0xc8174cb1,"Npc_SourthernVillage013_TalkSea",
0x7500dab7,"Npc_SourthernVillage013_TalkUotori",
0x3392c64b,"Npc_SouthHateru001_talk",
0x635f57f8,"Npc_SouthHateru004_talk",
0xe3ad6861,"Npc_SouthHateru005_DispNameFlag",
0xa803845d,"Npc_SouthHateru005_talk",
0x2a81e815,"Npc_SouthHateru005_talk01",
0x9fcc4dba,"Npc_SouthHateru006_DispNameFlag",
0x2e97f6f3,"Npc_SouthHateru006_talk",
0xd0184fc0,"Npc_SouthHateru007_001",
0x02c3accc,"Npc_SouthHateru007_DispNameFlag",
0x7f4cfad5,"Npc_SouthHateru007_GoodBye",
0xd787c3d2,"Npc_SouthernVillage001_DispNameFlag",
0x5324adbf,"Npc_SouthernVillage004_DispNameFlag",
0xce2b4cc9,"Npc_SouthernVillage005_DispNameFlag",
0xff9bf38f,"Npc_SouthernVillage005_SoldOut",
0xb24a6912,"Npc_SouthernVillage006_DispNameFlag",
0x2f458864,"Npc_SouthernVillage007_DispNameFlag",
0x79d13c92,"Npc_SouthernVillage008_DispNameFlag",
0xe4dedde4,"Npc_SouthernVillage009_DispNameFlag",
0xf373f94c,"Npc_SouthernVillage010_DispNameFlag",
0x121d3de1,"Npc_SouthernVillage012_DispNameFlag",
0x8f12dc97,"Npc_SouthernVillage013_DispNameFlag",
0xeadf7657,"Npc_SouthernVillage014_DispNameFlag",
0x77d09721,"Npc_SouthernVillage015_DispNameFlag",
0x0bb1b2fa,"Npc_SouthernVillage016_DispNameFlag",
0x1e9ec72d,"Npc_SouthernVillageGambler_DispNameFlag",
0xa0fc0259,"Npc_Strange_Beacon_DispNameFlag",
0x0e78e04b,"Npc_TabantaBridgeHatago001_DispNameFlag",
0x7219c590,"Npc_TabantaBridgeHatago002_DispNameFlag",
0xef1624e6,"Npc_TabantaBridgeHatago003_DispNameFlag",
0x8adb8e26,"Npc_TabantaBridgeHatago004_DispNameFlag",
0x17d46f50,"Npc_TabantaBridgeHatago005_DispNameFlag",
0x6bb54a8b,"Npc_TabantaBridgeHatago006_DispNameFlag",
0x249581f7,"Npc_TabantaHatago001_DispNameFlag",
0x58f4a42c,"Npc_TabantaHatago002_DispNameFlag",
0xc5fb455a,"Npc_TabantaHatago003_DispNameFlag",
0xe2f4c6f6,"Npc_TabantaHatago003_Talk",
0xa036ef9a,"Npc_TabantaHatago004_DispNameFlag",
0x34ad25eb,"Npc_TabantaHatago005_Talk",
0xda86f454,"Npc_TamourHatago001_DispNameFlag",
0xa6e7d18f,"Npc_TamourHatago002_DispNameFlag",
0x3be830f9,"Npc_TamourHatago003_DispNameFlag",
0x5e259a39,"Npc_TamourHatago004_DispNameFlag",
0x0569a391,"Npc_TamourHatago004_First",
0x3c410d9b,"Npc_TamourHatago004_Reward",
0xc32a7b4f,"Npc_TamourHatago005_DispNameFlag",
0x0a12eb67,"Npc_TamourHatago2_001_DispNameFlag",
0x10e0481b,"Npc_TamourPlainHatago_001_DispNameFlag",
0x6c816dc0,"Npc_TamourPlainHatago_002_DispNameFlag",
0xf18e8cb6,"Npc_TamourPlainHatago_003_DispNameFlag",
0x94432676,"Npc_TamourPlainHatago_004_DispNameFlag",
0xe8113de7,"Npc_TripMaster_00_DispNameFlag",
0x8494f813,"Npc_TripMaster_00_Talk",
0xbe5958eb,"Npc_TripMaster_Arrow",
0x67b2d438,"Npc_UMiiVillage003_DispNameFlag",
0x7e1e5b23,"Npc_UMiiVillage007_DispNameFlag",
0x288aefd5,"Npc_UMiiVillage008_DispNameFlag",
0xb5850ea3,"Npc_UMiiVillage009_DispNameFlag",
0xa2282a0b,"Npc_UMiiVillage010_DispNameFlag",
0xf41cf550,"Npc_UMiiVillage032_Go",
0x0ea4b55f,"Npc_ValleyVillage010_DispNameFlag",
0x83aaecee,"Npc_Zora001_Call",
0xeda2aa0f,"Npc_Zora001_ClearFirst",
0x8f5f57d6,"Npc_Zora001_DispNameFlag",
0x14ed1840,"Npc_Zora001_First",
0xd09e1dd8,"Npc_Zora001_NormalFirst",
0x13181a42,"Npc_Zora001_Pos1",
0x8a114bf8,"Npc_Zora001_Pos2",
0x38f78dcd,"Npc_Zora001_RainFirst",
0x6922174d,"Npc_Zora001_RiverFirst",
0x9d5afab5,"Npc_Zora001_You",
0xf33e720d,"Npc_Zora002_DispNameFlag",
0x00abff6e,"Npc_Zora002_Money",
0x47fec9d2,"Npc_Zora002_SoldOut",
0x569464c5,"Npc_Zora002_Talk",
0x6e31937b,"Npc_Zora003_DispNameFlag",
0x9dc8b760,"Npc_Zora003_Talk",
0x0bfc39bb,"Npc_Zora004_DispNameFlag",
0x80cd87d8,"Npc_Zora004_Talk",
0x925d296c,"Npc_Zora004_Talk2",
0x96f3d8cd,"Npc_Zora005_DispNameFlag",
0xe0a23c53,"Npc_Zora005_First",
0xdcc4c25e,"Npc_Zora005_Go",
0x5e2f255c,"Npc_Zora005_IchikaraFirst",
0x1027774e,"Npc_Zora005_NormalFirst",
0xea92fd16,"Npc_Zora006_DispNameFlag",
0xbf5efaec,"Npc_Zora006_FinishFirst",
0xd14a26ce,"Npc_Zora006_First",
0xaded1b80,"Npc_Zora006_NormalFirst",
0x09353352,"Npc_Zora007_AfterRelicFirst",
0x779d1c60,"Npc_Zora007_DispNameFlag",
0x773d2d7a,"Npc_Zora007_First",
0x57ae136c,"Npc_Zora007_MifaSong",
0xd042478e,"Npc_Zora007_RainFirst",
0x301b2eb9,"Npc_Zora007_SongFirst",
0x9173ebbf,"Npc_Zora008_A2First",
0x2109a896,"Npc_Zora008_DispNameFlag",
0x81755d93,"Npc_Zora008_First",
0x570b1730,"Npc_Zora008_NormalFirst",
0x43bab348,"Npc_Zora008_RainFirst",
0xf70f47a3,"Npc_Zora008_Talk",
0x55fd4318,"Npc_Zora008_Talk2",
0x22fa738e,"Npc_Zora008_Talk3",
0xbc0649e0,"Npc_Zora009_DispNameFlag",
0x27025627,"Npc_Zora009_First",
0xc5ab570f,"Npc_Zora009_SoldOut",
0x3c539406,"Npc_Zora009_Talk",
0xf38a48ac,"Npc_Zora009_Talk2",
0xabab6d48,"Npc_Zora010_DispNameFlag",
0x82b4c447,"Npc_Zora010_FinishFirst",
0x7e30136a,"Npc_Zora010_First",
0x591b640a,"Npc_Zora010_GetReward",
0x9007252b,"Npc_Zora010_NormalFirst",
0x36a48c3e,"Npc_Zora011_DispNameFlag",
0x5b0b3cca,"Npc_Zora011_Father",
0x5f221dc2,"Npc_Zora011_FinishFirst",
0xd84718de,"Npc_Zora011_First",
0x71c5eb9b,"Npc_Zora011_FrogFinishFirst",
0xbbb526e4,"Npc_Zora011_FrogReadyFirst",
0x40ab50c4,"Npc_Zora011_GotFrog",
0x4d91fcae,"Npc_Zora011_NormalFirst",
0xe84efe99,"Npc_Zora011_ReadyFirst",
0x76771ddf,"Npc_Zora011_Talk",
0x12c9693f,"Npc_Zora012_AfterRelicFirst",
0xf16b6e9f,"Npc_Zora012_Dad",
0x4ac5a9e5,"Npc_Zora012_DispNameFlag",
0x699d3feb,"Npc_Zora012_Falls",
0xe9af0243,"Npc_Zora012_First",
0x62048057,"Npc_Zora012_FlowedWifeFirst",
0x19e5c03a,"Npc_Zora012_Mom",
0x3ab0a7a3,"Npc_Zora012_RainFirst",
0xf0e36f71,"Npc_Zora012_Talk",
0x68154651,"Npc_Zora013_Call",
0x13e62e9d,"Npc_Zora013_CantGetReward",
0xd7ca4893,"Npc_Zora013_DispNameFlag",
0x4fd809f7,"Npc_Zora013_First",
0x2dcd49e5,"Npc_Zora013_NormalFirst",
0x9436cb68,"Npc_Zora013_OreFinishFirst",
0x9d2bd829,"Npc_Zora013_OreReadyFirst",
0x98da785c,"Npc_Zora013_Reward",
0x588f5057,"Npc_Zora013_RiverFirst",
0x3bbfbcd4,"Npc_Zora013_Talk",
0xb207e253,"Npc_Zora014_DispNameFlag",
0x8a7f3779,"Npc_Zora014_First",
0xac7454da,"Npc_Zora014_GetArrow",
0x50be4fbd,"Npc_Zora014_NormalFirst",
0xd2056de0,"Npc_Zora014_RainFirst",
0x26ba8c6c,"Npc_Zora014_Talk",
0x015e2c66,"Npc_Zora015_Armor",
0x2f080325,"Npc_Zora015_DispNameFlag",
0x9f9b7754,"Npc_Zora015_FinishFirst",
0x2c083ccd,"Npc_Zora015_First",
0x4e5ee122,"Npc_Zora015_Impossible",
0x8d289638,"Npc_Zora015_NormalFirst",
0xeb48c5e6,"Npc_Zora015_ReadyFirst",
0x40824f92,"Npc_Zora015_Step10First",
0xede65fc9,"Npc_Zora015_Talk",
0xbad60731,"Npc_Zora020_DispNameFlag",
0x9bf9bcfb,"Npc_Zora020_HappyBed",
0xec6600f0,"Npc_Zora020_NormalFirst",
0x1542cc92,"Npc_Zora020_RainFirst",
0x8cc3d4e7,"Npc_Zora020_Talk",
0xd8ddc816,"Npc_Zora025_AfterRelicFirst",
0xe27652cc,"Npc_Zora025_BeforeRelicFirst",
0x8fa4bfd1,"Npc_Zora025_Call",
0x3e75695c,"Npc_Zora025_DispNameFlag",
0xe9bca655,"Npc_Zora025_FineFirst",
0xa2873b2e,"Npc_Zora025_First",
0xe5360d6d,"Npc_Zora025_Mifa",
0x1f16497d,"Npc_Zora025_Pos1",
0x861f18c7,"Npc_Zora025_Pos2",
0x0a789979,"Npc_Zora025_RiverFirst",
0x4153f8f1,"Npc_Zora025_Skin",
0xdc0e4554,"Npc_Zora025_Talk",
0x96717afc,"Npc_Zora026_BottleLost",
0x09a924e4,"Npc_Zora026_CantGetReward",
0x42144c87,"Npc_Zora026_DispNameFlag",
0x09cd9011,"Npc_Zora026_RiverNormalFirst",
0x0ce158d1,"Npc_Zora027_AfterRelicFirst",
0xba2a71fa,"Npc_Zora027_DIY",
0xdf1badf1,"Npc_Zora027_DispNameFlag",
0x83a6e7c4,"Npc_Zora027_FinishFirst",
0x35182a07,"Npc_Zora027_First",
0x911506a8,"Npc_Zora027_NormalFirst",
0x3c79d911,"Npc_Zora027_RainFirst",
0x38f74af3,"Npc_Zora027_Spear",
0xca636b04,"Npc_Zora028_AfterRelicFirst",
0x11dd9d27,"Npc_Zora028_BeforeRelicFirst",
0x333aac0f,"Npc_Zora028_Call",
0x898f1907,"Npc_Zora028_DispNameFlag",
0x8ce6b676,"Npc_Zora028_FallNear",
0x2228eb52,"Npc_Zora028_FineFirst",
0xc3505aee,"Npc_Zora028_First",
0x94d79897,"Npc_Zora028_RiverFirst",
0xd0cade9c,"Npc_Zora028_ShidoFirst",
0x6090568a,"Npc_Zora028_Talk",
0x249e95d5,"Npc_Zora028_TalkRelic",
0x22ceb512,"Npc_Zora029_AfterDiveFirst",
0x4dc5a047,"Npc_Zora029_AfterRelicFirst",
0x1480f871,"Npc_Zora029_DispNameFlag",
0x6527515a,"Npc_Zora029_First",
0x9c580fe0,"Npc_Zora029_Mt",
0x0f72d4f8,"Npc_Zora029_RiverFirst",
0xf321e783,"Npc_Zora029_Step1First",
0xabcc852f,"Npc_Zora029_Talk",
0x5068c6c1,"Npc_Zora031_ClearFirst",
0x9e223daf,"Npc_Zora031_DispNameFlag",
0xc2c099b8,"Npc_Zora031_FineFirst",
0x9a621fa3,"Npc_Zora031_First",
0xd8d044cf,"Npc_Zora031_Mifa",
0xe1e80cf6,"Npc_Zora031_Talk",
0xb857d76f,"Npc_Zora032_AroundFirst",
0xe2431874,"Npc_Zora032_DispNameFlag",
0xab8a053e,"Npc_Zora032_First",
0x5e443661,"Npc_Zora032_Mifa",
0x113554cd,"Npc_Zora032_NormalFirst",
0xd68b393c,"Npc_Zora032_RainFirst",
0x8248337e,"Npc_Zora032_Second",
0x677c7e58,"Npc_Zora032_Talk",
0x90bbc9ff,"Npc_Zora033_AfterRelicFirst",
0x7f4cf902,"Npc_Zora033_DispNameFlag",
0x0dfd0e8a,"Npc_Zora033_First",
0xcca38d48,"Npc_Zora033_NormalFirst",
0x1705e6fc,"Npc_Zora033_RainFirst",
0x69ad3546,"Npc_Zora033_Spear049",
0x096abca3,"Npc_Zora033_Spear050",
0xac20adfd,"Npc_Zora033_Talk",
0xf8364c61,"Npc_Zora033_doing",
0xc1d3417a,"Npc_Zora033_spear",
0x4a3f8290,"Npc_Zora034_Dam",
0x1a8153c2,"Npc_Zora034_DispNameFlag",
0xe0f9e39d,"Npc_Zora034_FinalDa",
0x948ba817,"Npc_Zora034_FinalDa2",
0xc85a3004,"Npc_Zora034_First",
0x2ee2d0b2,"Npc_Zora034_GetTest",
0xb1d08b10,"Npc_Zora034_NormalFirst",
0x3e3ef37f,"Npc_Zora034_RainFirst",
0x61e4e7ff,"Npc_Zora034_Rutta",
0x47734e78,"Npc_Zora034_Shido",
0xb1259d45,"Npc_Zora034_Talk",
0x4990530b,"Npc_Zora034_Test2",
0xe352f601,"Npc_Zora034_Test33",
0xce5cf91a,"Npc_Zora034_Test444",
0x378f7ef7,"Npc_Zora035_AfterRelicFirst",
0x878eb2b4,"Npc_Zora035_DispNameFlag",
0x6e2d3bb0,"Npc_Zora035_First",
0x6c465295,"Npc_Zora035_NormalFirst",
0xffb02cbf,"Npc_Zora035_RainFirst",
0xfbef976f,"Npc_Zora036_DispNameFlag",
0xc33fdf37,"Npc_Zora036_FinishFirst",
0xd18c3e5b,"Npc_Zora036_NormalFirst",
0xfc8aa25d,"Npc_Zora036_ReadyFirst",
0xa070920c,"Npc_Zora036_RiverFirst",
0x1c26e7f1,"Npc_Zora036_Step10First",
0x2173113b,"Npc_Zora036_WaterActivatedFirst",
0x98222419,"Npc_Zora036_WaterFinishFirst",
0x3fef5443,"Npc_Zora036_WaterRainFirst",
0x3fdf91e2,"Npc_Zora036_WaterReadyFirst",
0xbf556321,"Npc_Zora037_ClearFirst",
0x66e07619,"Npc_Zora037_DispNameFlag",
0x2a7553fb,"Npc_Zora037_FineFirst",
0xf9b22a99,"Npc_Zora037_First",
0xa24166d6,"Npc_Zora037_Neighbor",
0x769b54e5,"Npc_Zora037_Shido",
0xb43365a8,"Npc_Zora037_Talk1",
0x2d3a3412,"Npc_Zora037_Talk2",
0x5a3d0484,"Npc_Zora037_Talk3",
0x2a70c550,"Npc_Zora126_BeforeFirst",
0xec7cdd16,"Npc_Zora126_DispNameFlag",
0x841435f0,"Npc_Zora126_First",
0xb2bf1787,"Npc_Zora126_MeetFirst",
0x2061df54,"Npc_Zora126_RainFirst",
0x96303764,"Npc_Zora126_Talk",
0xc4c58c1a,"Npc_Zora126_You",
0xa8da25ee,"Npc_ZoraB001_DispNameFlag",
0xd402ce6a,"Npc_ZoraGoron001_DeathMt",
0x8889472d,"Npc_ZoraGoron001_First",
0x03d8ac50,"Npc_ZoraGoron001_Stone",
0xb4d5b4e8,"Npc_ZoraGoron_001_DispNameFlag",
0x26cced4b,"Npc_gerudovalley001_DispNameFlag",
0x39ccb56e,"Npc_goron001_ClearFirst",
0x7eb1d4ab,"Npc_goron001_DispNameFlag",
0x02d0f170,"Npc_goron002_DispNameFlag",
0x9fdf1006,"Npc_goron003_DispNameFlag",
0xfa12bac6,"Npc_goron004_DispNameFlag",
0x2991c0fb,"Npc_goron004_SoldOut",
0x3aca8e11,"Npc_goron005_ClearFirst",
0x2d08e9aa,"Npc_goron026_ClearFirst",
0xb3facffa,"Npc_goron026_DispNameFlag",
0x4fa83ddc,"Npc_kokiri001_DispNameFlag",
0x33c91807,"Npc_kokiri002_DispNameFlag",
0xaec6f971,"Npc_kokiri003_DispNameFlag",
0x05697c73,"Npc_kokiri003_SoldOut",
0xfb85baec,"Npc_kokiri003_Talk",
0xcb0b53b1,"Npc_kokiri004_DispNameFlag",
0x610811ba,"Npc_kokiri004_SoldOut",
0xe6808a54,"Npc_kokiri004_Talk",
0x5604b2c7,"Npc_kokiri005_DispNameFlag",
0x2a65971c,"Npc_kokiri006_DispNameFlag",
0xab482b5f,"Npc_kokiri006_Talk",
0x7cf123ea,"Npc_kokiri009_DispNameFlag",
0x8a32c3ef,"Npc_kokiri012_DispNameFlag",
0x96ae62fd,"Npc_kokiri012_Talk",
0x9eca9008,"Npc_oasis001_DispNameFlag",
0xd4ac4f42,"Npc_oasis001_first_4relic",
0xd7f434f9,"Npc_oasis001_talk",
0xfe7a8e9d,"Npc_oasis0020_first",
0xe2abb5d3,"Npc_oasis002_DispNameFlag",
0x51604657,"Npc_oasis002_talk",
0x7fa454a5,"Npc_oasis003_DispNameFlag",
0xe75b01a6,"Npc_oasis003_Raimei_CK",
0x87661f13,"Npc_oasis005_DispNameFlag",
0xfb073ac8,"Npc_oasis006_DispNameFlag",
0x5716c71d,"Npc_oasis006_aroma",
0xcaf10441,"Npc_oasis006_talk",
0x6608dbbe,"Npc_oasis007_DispNameFlag",
0x57827224,"Npc_oasis007_Raimei_CK",
0xd5c88860,"Npc_oasis007_thanks",
0x309c6f48,"Npc_oasis008_DispNameFlag",
0xad938e3e,"Npc_oasis009_DispNameFlag",
0x0cd339cd,"Npc_oasis009_SinjyuClear_Female",
0x6722db45,"Npc_oasis009_SinjyuClear_Normal",
0xba3eaa96,"Npc_oasis010_DispNameFlag",
0x6bb557a7,"Npc_oasis010_SinjyuClear_Female",
0x0044b52f,"Npc_oasis010_SinjyuClear_Normal",
0x27314be0,"Npc_oasis011_DispNameFlag",
0x5b506e3b,"Npc_oasis012_DispNameFlag",
0xc65f8f4d,"Npc_oasis013_DispNameFlag",
0xa392258d,"Npc_oasis014_DispNameFlag",
0x55b81e37,"Npc_oasis014_First",
0x6842c26d,"Npc_oasis014_GJ",
0x55c5584a,"Npc_oasis014_Raimei_CK",
0x671e40f8,"Npc_oasis014_ShinjuGJ",
0x3e9dc4fb,"Npc_oasis015_DispNameFlag",
0xea127d5b,"Npc_oasis015_talk",
0x42fce120,"Npc_oasis016_DispNameFlag",
0xc2270f1e,"Npc_oasis016_First",
0xdff30056,"Npc_oasis017_DispNameFlag",
0x8967b4a0,"Npc_oasis018_DispNameFlag",
0x146855d6,"Npc_oasis019_DispNameFlag",
0xab43c0ef,"Npc_oasis020_DispNameFlag",
0xeeb912c3,"Npc_oasis020_first",
0xdbdeebd1,"Npc_oasis020_first_gerudo",
0x2d26625a,"Npc_oasis020_first_man",
0x788fcc96,"Npc_oasis020_zarashi",
0x364c2199,"Npc_oasis021_DispNameFlag",
0x4a2d0442,"Npc_oasis022_DispNameFlag",
0x792603ea,"Npc_oasis022_first",
0xd722e534,"Npc_oasis023_DispNameFlag",
0xbf02988e,"Npc_oasis023_SinjyuClear_Female",
0xd4f37a06,"Npc_oasis023_SinjyuClear_Normal",
0xb2ef4ff4,"Npc_oasis024_DispNameFlag",
0x5e463e1d,"Npc_oasis024_SinjyuClear_Female",
0x35b7dc95,"Npc_oasis024_SinjyuClear_Normal",
0x2fe0ae82,"Npc_oasis025_DispNameFlag",
0xe3d5557a,"Npc_oasis025_Raimei_CK",
0xbc813d64,"Npc_oasis025_first",
0x95d54cae,"Npc_oasis025_permitted",
0x53818b59,"Npc_oasis026_DispNameFlag",
0xacf43e1b,"Npc_oasis026_gooby",
0xce8e6a2f,"Npc_oasis027_DispNameFlag",
0x981aded9,"Npc_oasis028_DispNameFlag",
0x221eb24a,"Npc_oasis028_thanks",
0x05153faf,"Npc_oasis029_DispNameFlag",
0x7b215710,"Npc_oasis029_first",
0xf3d6dfaa,"Npc_oasis032_DispNameFlag",
0xeff4ac27,"Npc_oasis036_Clear_first",
0x395cc5da,"Npc_oasis036_Deliver_first",
0xea7a50b1,"Npc_oasis036_DispNameFlag",
0x41c32767,"Npc_oasis036_first",
0x7775b1c7,"Npc_oasis037_DispNameFlag",
0xe7b42cd3,"Npc_oasis037_first",
0x3045cd79,"Npc_oasis037_talk",
0x21e10531,"Npc_oasis038_DispNameFlag",
0xd03d733e,"Npc_oasis038_First",
0xbceee447,"Npc_oasis039_DispNameFlag",
0x89b9141d,"Npc_oasis040_DispNameFlag",
0x14b6f56b,"Npc_oasis041_DispNameFlag",
0x68d7d0b0,"Npc_oasis042_DispNameFlag",
0xf5d831c6,"Npc_oasis043_DispNameFlag",
0x6e73b1e1,"Npc_oasis043_talk",
0x90159b06,"Npc_oasis044_DispNameFlag",
0x73768159,"Npc_oasis044_talk",
0x0d1a7a70,"Npc_oasis045_DispNameFlag",
0xb82a52fc,"Npc_oasis045_talk",
0x717b5fab,"Npc_oasis046_DispNameFlag",
0x3ebe2052,"Npc_oasis046_talk",
0xec74bedd,"Npc_oasis047_DispNameFlag",
0xed7125ca,"Npc_oasis047_first",
0x22f894db,"Npc_oasis047_first_rouge",
0xa0ee3be2,"Npc_oasis050_BoyHunt",
0x3042cff5,"Npc_oasis050_DispNameFlag",
0x25bd34de,"Npc_oasis050_First",
0x59a427bd,"Npc_oasis050_Go",
0x36a9096f,"Npc_oasissoldier_A_first",
0x074113f2,"Npc_oasissoldier_B_first",
0xa1361846,"Npc_oasissoldier_C_first",
0x3846c870,"Npc_oasiswaterchannel_thanks",
0x8f328cd9,"OPDemo_Finished",
0xf47ac3c5,"OP_DRC",
0x225ffe02,"Oasis_Arrow_SoldOut",
0xacfe0e40,"Oasis_Ch_Drug_Activated",
0x77143324,"Oasis_Ch_Drug_Finish",
0x9df89cdb,"Oasis_Ch_Drug_Ready",
0xfd9e9b5f,"Oasis_Drug_Challenge_Activated",
0x26a0e223,"Oasis_Drug_Challenge_Finish",
0xf729a1ea,"Oasis_Drug_Challenge_Ready",
0xd408bdfc,"Oasis_Fruits_SoldOut",
0x486070fa,"Oasis_Meat_SoldOut",
0xdb67f0a1,"OldKorok_Fld_1st",
0x3bfa3280,"OldKorok_Fld_2nd",
0xdc4f05ea,"OldKorok_Fld_3rd",
0x86ecf8c2,"OldKorok_Fld_4th",
0x872e92f5,"OldKorok_Fld_5th",
0x96ede8d6,"OldKorok_Help_Activated",
0xa3cdb7eb,"OldKorok_Help_Finish",
0x8cc7049f,"OldKorok_Help_FirstPorch",
0xfeafff71,"OldKorok_Help_KorokFruit",
0x41cf72db,"OldKorok_Help_Maracus",
0x82ff4aeb,"OldKorok_Help_NormalFirst",
0xff4cb1b8,"OldKorok_Help_Ready",
0x873283c5,"OldKorok_Help_SecondPorch",
0xafadc082,"OldKorok_Help_Self",
0x7ebd7418,"OldKorok_Help_snif",
0x94583412,"OneHundred_1st",
0x0898f85e,"OneHundred_Activated",
0x3dd7b2f8,"OneHundred_Finish",
0x575bed54,"OneHundred_Fr02Free",
0x6a3bc4e4,"OneHundred_Fr03Free",
0xd81b18f4,"OneHundred_Fr04Free",
0xfe8eff26,"OneHundred_Fr0Free",
0x81dc01bf,"OneHundred_Ready",
0x05b9aede,"OneHundred_Step1",
0x7f667eea,"Open_AncientLabo",
0x11d23895,"Open_Door",
0x79750bd3,"Open_Dungeon000",
0x0e723b45,"Open_Dungeon001",
0x977b6aff,"Open_Dungeon002",
0xe07c5a69,"Open_Dungeon003",
0x7e18cfca,"Open_Dungeon004",
0x091fff5c,"Open_Dungeon005",
0x9016aee6,"Open_Dungeon006",
0xe7119e70,"Open_Dungeon007",
0x77ae83e1,"Open_Dungeon008",
0x00a9b377,"Open_Dungeon009",
0x606e3a92,"Open_Dungeon010",
0x17690a04,"Open_Dungeon011",
0x8e605bbe,"Open_Dungeon012",
0xf9676b28,"Open_Dungeon013",
0x6703fe8b,"Open_Dungeon014",
0x1004ce1d,"Open_Dungeon015",
0x890d9fa7,"Open_Dungeon016",
0xfe0aaf31,"Open_Dungeon017",
0x6eb5b2a0,"Open_Dungeon018",
0x19b28236,"Open_Dungeon019",
0x4b436951,"Open_Dungeon020",
0x3c4459c7,"Open_Dungeon021",
0xa54d087d,"Open_Dungeon022",
0xd24a38eb,"Open_Dungeon023",
0x4c2ead48,"Open_Dungeon024",
0x3b299dde,"Open_Dungeon025",
0xa220cc64,"Open_Dungeon026",
0xd527fcf2,"Open_Dungeon027",
0x4598e163,"Open_Dungeon028",
0x329fd1f5,"Open_Dungeon029",
0x52585810,"Open_Dungeon030",
0x255f6886,"Open_Dungeon031",
0xbc56393c,"Open_Dungeon032",
0xcb5109aa,"Open_Dungeon033",
0x55359c09,"Open_Dungeon034",
0x2232ac9f,"Open_Dungeon035",
0xbb3bfd25,"Open_Dungeon036",
0xcc3ccdb3,"Open_Dungeon037",
0x5c83d022,"Open_Dungeon038",
0x2b84e0b4,"Open_Dungeon039",
0x1d19ced7,"Open_Dungeon040",
0x6a1efe41,"Open_Dungeon041",
0xf317affb,"Open_Dungeon042",
0x84109f6d,"Open_Dungeon043",
0x1a740ace,"Open_Dungeon044",
0x6d733a58,"Open_Dungeon045",
0xf47a6be2,"Open_Dungeon046",
0x837d5b74,"Open_Dungeon047",
0x13c246e5,"Open_Dungeon048",
0x64c57673,"Open_Dungeon049",
0x0402ff96,"Open_Dungeon050",
0x7305cf00,"Open_Dungeon051",
0xea0c9eba,"Open_Dungeon052",
0x9d0bae2c,"Open_Dungeon053",
0x036f3b8f,"Open_Dungeon054",
0x74680b19,"Open_Dungeon055",
0xed615aa3,"Open_Dungeon056",
0x9a666a35,"Open_Dungeon057",
0x0ad977a4,"Open_Dungeon058",
0x7dde4732,"Open_Dungeon059",
0x2f2fac55,"Open_Dungeon060",
0x58289cc3,"Open_Dungeon061",
0xc121cd79,"Open_Dungeon062",
0xb626fdef,"Open_Dungeon063",
0x2842684c,"Open_Dungeon064",
0x5f4558da,"Open_Dungeon065",
0xc64c0960,"Open_Dungeon066",
0xb14b39f6,"Open_Dungeon067",
0x21f42467,"Open_Dungeon068",
0x56f314f1,"Open_Dungeon069",
0x36349d14,"Open_Dungeon070",
0x4133ad82,"Open_Dungeon071",
0xd83afc38,"Open_Dungeon072",
0xaf3dccae,"Open_Dungeon073",
0x3159590d,"Open_Dungeon074",
0x465e699b,"Open_Dungeon075",
0xdf573821,"Open_Dungeon076",
0xa85008b7,"Open_Dungeon077",
0x38ef1526,"Open_Dungeon078",
0x4fe825b0,"Open_Dungeon079",
0xb1ac81db,"Open_Dungeon080",
0xc6abb14d,"Open_Dungeon081",
0x5fa2e0f7,"Open_Dungeon082",
0x28a5d061,"Open_Dungeon083",
0xb6c145c2,"Open_Dungeon084",
0xc1c67554,"Open_Dungeon085",
0x58cf24ee,"Open_Dungeon086",
0x2fc81478,"Open_Dungeon087",
0xbf7709e9,"Open_Dungeon088",
0xc870397f,"Open_Dungeon089",
0xa8b7b09a,"Open_Dungeon090",
0xdfb0800c,"Open_Dungeon091",
0x46b9d1b6,"Open_Dungeon092",
0x31bee120,"Open_Dungeon093",
0xafda7483,"Open_Dungeon094",
0xd8dd4415,"Open_Dungeon095",
0x41d415af,"Open_Dungeon096",
0x36d32539,"Open_Dungeon097",
0xa66c38a8,"Open_Dungeon098",
0xd16b083e,"Open_Dungeon099",
0x78b761e4,"Open_Dungeon100",
0x0fb05172,"Open_Dungeon101",
0x96b900c8,"Open_Dungeon102",
0xe1be305e,"Open_Dungeon103",
0x7fdaa5fd,"Open_Dungeon104",
0x08dd956b,"Open_Dungeon105",
0x91d4c4d1,"Open_Dungeon106",
0xe6d3f447,"Open_Dungeon107",
0x766ce9d6,"Open_Dungeon108",
0x016bd940,"Open_Dungeon109",
0x61ac50a5,"Open_Dungeon110",
0x16ab6033,"Open_Dungeon111",
0x8fa23189,"Open_Dungeon112",
0xf8a5011f,"Open_Dungeon113",
0x66c194bc,"Open_Dungeon114",
0x11c6a42a,"Open_Dungeon115",
0x88cff590,"Open_Dungeon116",
0xffc8c506,"Open_Dungeon117",
0x6f77d897,"Open_Dungeon118",
0x1870e801,"Open_Dungeon119",
0x158220d7,"Open_FirstDoor",
0x1f2ca100,"Open_FirstDungeon",
0x965a64a8,"Open_HatenoLabo",
0x22195da4,"Open_StartPoint",
0x615da543,"Para_Rock_1",
0xf854f4f9,"Para_Rock_2",
0x8f53c46f,"Para_Rock_3",
0x1bd50e08,"PictureMemory_Activated",
0x71e20117,"PictureMemory_Find11",
0xa9c25693,"PictureMemory_Finish",
0xff4ae785,"PictureMemory_First1",
0x17d069ed,"PictureMemory_GetShirt",
0x8dd6fc6f,"PictureMemory_Last1",
0xf003ee36,"PictureMemory_Ready",
0x343ec131,"PictureMemory_goodjob",
0x67625660,"PlayerColdAlert",
0x85226c52,"PlayerColdMes",
0xbad18dd2,"PlayerHotAlert",
0x29491320,"PlayerHotMes",
0xcc9ca297,"PlayerStaminaDecreaseMes",
0x8b7a1eee,"PlayerStaminaZeroMes",
0x5d093c05,"PlayerUpdateOnlyBottomComebackPos",
0xce5b18d9,"PlayerWeaponFirstGet",
0x5cbf1540,"PortableUnit_PowerOn",
0xfd976a9b,"PowerFirstDemoKill",
0x712702ec,"PowerFountain_GrassBurned",
0xb318375c,"PowerFountain_GrassBurned2",
0x37ef20c9,"Premium_Shield_1",
0xaee67173,"Premium_Shield_2",
0xd9e141e5,"Premium_Shield_3",
0x4785d446,"Premium_Shield_4",
0x3082e4d0,"Premium_Shield_5",
0xa98bb56a,"Premium_Shield_6",
0xde8c85fc,"Premium_Shield_7",
0x4e33986d,"Premium_Shield_8",
0x3934a8fb,"Premium_Shield_9",
0x4dfad380,"Premium_Shield_conp",
0x554959b1,"PrezentationOkamura_Activated",
0xd2290bd6,"PrezentationOkamura_Finish",
0xaa7505fc,"PrezentationOkamura_Ready",
0xc25e7670,"PrinceCheer01",
0x5b5727ca,"PrinceCheer02",
0x2c50175c,"PrinceCheer03",
0xb23482ff,"PrinceCheer04",
0xc533b269,"PrinceCheer05",
0x5c3ae3d3,"PrinceCheer06",
0x7019e683,"PrinceMeeting",
0xadba8983,"PrinceVoiceEvent_town",
0x9a5a3fef,"QueenLetter_Activated",
0xfacf8652,"QueenLetter_Finished",
0x91596112,"QueenLetter_Map",
0x356d8077,"QueenLetter_Parashawl",
0x5be69bc3,"QueenLetter_Ready",
0x1bc52063,"Radio_PowerOnOff",
0xed39b75f,"Relief_Landing_Act",
0x19ba1ae9,"Relief_Landing_Activated",
0x044cac17,"Relief_Landing_Finish",
0x19712c5c,"Relief_Landing_Ready",
0xaed0c71d,"Relief_Landing_paras",
0x5cd5ac39,"Relief_Landing_step1",
0x96967500,"RemainsElectric_1st",
0x760bb721,"RemainsElectric_2nd",
0x8d57be26,"RemainsElectric_FirstRotateCap",
0xcd41f1fe,"RemainsElectric_FirstRotateDrum",
0xd914715d,"RemainsElectric_FirstRotateDrum1",
0x401d20e7,"RemainsElectric_FirstRotateDrum2",
0x371a1071,"RemainsElectric_FirstRotateDrum3",
0x4e6eb48d,"RemainsElectric_FirstRotateNeck",
0x1248d683,"RemainsElectric_FirstRotateTail",
0x37e18f7c,"RemainsElectric_SmallKey00",
0x40e6bfea,"RemainsElectric_SmallKey01",
0xd9efee50,"RemainsElectric_SmallKey02",
0xaee8dec6,"RemainsElectric_SmallKey03",
0x308c4b65,"RemainsElectric_SmallKey04",
0x478b7bf3,"RemainsElectric_SmallKey05",
0xde822a49,"RemainsElectric_SmallKey06",
0xa9851adf,"RemainsElectric_SmallKey07",
0x393a074e,"RemainsElectric_SmallKey08",
0x4e3d37d8,"RemainsElectric_SmallKey09",
0xfe5495f8,"RemainsFire_AncientSwitch",
0x970f1385,"RemainsFire_ClearFirePillar",
0x5baaabe6,"RemainsFire_DoorStopper",
0x754f20e3,"RemainsFire_IronBox",
0x2a5524c4,"RemainsFire_OpenArrowShutter",
0x2f1a9a25,"RemainsFire_OpenBallShutter",
0xe8b810d2,"RemainsFire_OpenIronDoor",
0x5fd915f4,"RemainsFire_OpenWindow",
0xefc967e7,"RemainsFire_SmallKey00",
0x98ce5771,"RemainsFire_SmallKey01",
0x01c706cb,"RemainsFire_SmallKey02",
0x76c0365d,"RemainsFire_SmallKey03",
0xe8a4a3fe,"RemainsFire_SmallKey04",
0x9fa39368,"RemainsFire_SmallKey05",
0x06aac2d2,"RemainsFire_SmallKey06",
0x71adf244,"RemainsFire_SmallKey07",
0xe112efd5,"RemainsFire_SmallKey08",
0x9615df43,"RemainsFire_SmallKey09",
0x6f69f463,"RemainsWater_BlockWater",
0x4de95921,"RemainsWater_ExtinguishFire",
0x4f23bd9c,"RemainsWater_IceGate_SE",
0x6b7025ae,"RemainsWater_ShortCut1",
0x330dafbf,"RemainsWater_SmallKey00",
0x440a9f29,"RemainsWater_SmallKey01",
0xdd03ce93,"RemainsWater_SmallKey02",
0xaa04fe05,"RemainsWater_SmallKey03",
0x34606ba6,"RemainsWater_SmallKey04",
0x43675b30,"RemainsWater_SmallKey05",
0xda6e0a8a,"RemainsWater_SmallKey06",
0xad693a1c,"RemainsWater_SmallKey07",
0x3dd6278d,"RemainsWater_SmallKey08",
0x4ad1171b,"RemainsWater_SmallKey09",
0xbdae46f2,"RemainsWater_WaterTerminal_SE",
0xec9664be,"RemainsWind_AccessDownload",
0x7f8db614,"RemainsWind_BreakCrack_1",
0xe684e7ae,"RemainsWind_BreakCrack_2",
0xa9052bb4,"RemainsWind_Shutter_1",
0xa748700e,"RemainsWind_SmallKey00",
0xd04f4098,"RemainsWind_SmallKey01",
0x49461122,"RemainsWind_SmallKey02",
0x3e4121b4,"RemainsWind_SmallKey03",
0xa025b417,"RemainsWind_SmallKey04",
0xd7228481,"RemainsWind_SmallKey05",
0x4e2bd53b,"RemainsWind_SmallKey06",
0x392ce5ad,"RemainsWind_SmallKey07",
0xa993f83c,"RemainsWind_SmallKey08",
0xde94c8aa,"RemainsWind_SmallKey09",
0xb8460539,"RemainsWind_Stake_1",
0x214f5483,"RemainsWind_Stake_2",
0x00b59db4,"RemainsWind_UnderLeftWing",
0x6e0eeb0c,"RemainsWind_UnderRightWing",
0x7693890b,"RemainsWind_WindSmallRoom",
0x8456f48c,"Remains_Fancier_Activated",
0xc0b7f865,"Remains_Fancier_Appearance",
0x50aa185e,"Remains_Fancier_BallDelete",
0xdb451683,"Remains_Fancier_Finish",
0x34321c38,"Remains_Fancier_Gaved",
0x70247bf0,"Remains_Fancier_Ready",
0xf441d491,"Remains_Fancier_Step1",
0x19881bcb,"Remains_Fancier_Talked",
0x8dc75543,"Remains_Fancier_photo",
0x3c8149f4,"Remains_Fancier_photoA",
0xa588184e,"Remains_Fancier_photoB",
0xd28f28d8,"Remains_Fancier_photoC",
0x55a099b8,"Remains_FirstMessage",
0x80e445d9,"Retry_Check",
0xf9f7d965,"RevivalGreatFairy1",
0x60fe88df,"RevivalGreatFairy2",
0x17f9b849,"RevivalGreatFairy3",
0x899d2dea,"RevivalGreatFairy4",
0xfe9a1d7c,"RevivalGreatFairy5",
0xb59613ca,"RinelMt_Enter",
0x2522cdfe,"RinelSearch_Activated",
0xe09e402d,"RinelSearch_ArmorSend",
0x686d4963,"RinelSearch_Finish",
0x3e01b508,"RinelSearch_Ready",
0xe9c2dc26,"RinelSearch_Step010",
0xba641a69,"RinelSearch_Step1",
0xc65b2468,"Rinel_Complete",
0x56194582,"RitoMini_Cook_Activated",
0x63ddeb53,"RitoMini_Cook_Finish",
0x557ca4c7,"RitoMini_Cook_Ready",
0xef4ff0be,"RitoMini_Flint_Activated",
0x1272e630,"RitoMini_Flint_Finish",
0x4bf44e83,"RitoMini_Flint_Ready",
0x501c358e,"RitoMini_IceGolem_Activated",
0xb16411af,"RitoMini_IceGolem_Beated",
0xb6b66731,"RitoMini_IceGolem_Finish",
0x5eb122e5,"RitoMini_IceGolem_Ready",
0x513b4af8,"RitoRabitMountain_Activated",
0xf3d7074f,"RitoRabitMountain_Finish",
0xa74cbeaa,"RitoRabitMountain_FlyTraining",
0xe08d29c8,"RitoRabitMountain_Ready",
0x50d93289,"RitoRabitMountain_Talk",
0x0c4114d3,"RitoSongMystery_Activated",
0xe84c3418,"RitoSongMystery_Finished",
0xe4334b28,"RitoSongMystery_NPC012_First",
0x31895d7c,"RitoSongMystery_NPC030_First",
0x036c4084,"RitoSongMystery_PanelOn",
0x2deb08eb,"RitoSongMystery_Ready",
0xa4145202,"RitoSongMystery_Shrine",
0x551170f8,"RitoSongMystery_Sister",
0x8575ac1a,"RitoUmayadoMini_HotRecipe_Activated",
0xe1b98ca6,"RitoUmayadoMini_HotRecipe_Finish",
0x39ef9091,"RitoUmayadoMini_HotRecipe_Ready",
0x73212c0e,"Rito_BrosRock_Activated",
0x0d2e04fc,"Rito_BrosRock_Dungeon",
0x2cbe6434,"Rito_BrosRock_Finish",
0xe9019164,"Rito_BrosRock_Ready",
0xbd52ed70,"Rito_BrosRock_Rito_BrosRock_Step1",
0x245bbcca,"Rito_BrosRock_Rito_BrosRock_Step2",
0x06c970b5,"Rito_BrosRock_S1",
0x164abe95,"Rito_BrosRock_S1Next1",
0x8f43ef2f,"Rito_BrosRock_S1Next2",
0xf844dfb9,"Rito_BrosRock_S1Next3",
0x9fc0210f,"Rito_BrosRock_S2",
0xe8c71199,"Rito_BrosRock_S3",
0x76a3843a,"Rito_BrosRock_S4",
0x16a04d8e,"Rito_BrosRock_Singsong",
0x3fc969ad,"Rito_KeelSearch_Activated",
0xc9648ff1,"Rito_KeelSearch_Finish",
0xe65459a9,"Rito_KeelSearch_Ready",
0x6231f6c8,"Rito_KeelSearch_Step1",
0xd2aeccb2,"Rito_NPC001_First",
0xf6160a52,"Rito_NPC001_First_Clear",
0x8e183a26,"Rito_NPC001_First_NG",
0x9eb5474c,"Rito_NPC001_First_OK",
0x8cdeaedb,"Rito_NPC001_First_Start",
0x4531dd9b,"Rito_NPC003_First",
0x8096e315,"Rito_NPC004_First",
0x48fe492f,"Rito_NPC004_First_All",
0xeb39b941,"Rito_NPC004_First_Clear",
0x1ed77964,"Rito_NPC004_First_NPC010",
0xf51dd261,"Rito_NPC004_NormalFirst",
0x11bb9227,"Rito_NPC004_SelfTalk",
0xf713728e,"Rito_NPC005_BombArrow",
0x26e1e8a1,"Rito_NPC005_First",
0x36af60c4,"Rito_NPC005_First_Clear",
0x4c67c44d,"Rito_NPC005_First_Start",
0x96d07efa,"Rito_NPC005_FlightTraining",
0xcc17e3e7,"Rito_NPC005_NPC001",
0xca0eb862,"Rito_NPC009_Bow028_First",
0xe14182d5,"Rito_NPC009_First",
0xac15d93f,"Rito_NPC009_First_Clear",
0xd6dd7db6,"Rito_NPC009_First_Start",
0x99c34815,"Rito_NPC009_Steal",
0xdda39cca,"Rito_NPC010_Bow028_First",
0xb68f32a1,"Rito_NPC010_First_Clear",
0x6fc95e93,"Rito_NPC010_First_Parashawl",
0x864c66de,"Rito_NPC010_Question",
0xec20bc84,"Rito_NPC010p_First",
0x2944257d,"Rito_NPC010p_First_WindRelic",
0x90b0799c,"Rito_NPC010p_Go_UMiiVillage",
0x4c69e879,"Rito_NPC010p_IchikaraFirst",
0xd3259da6,"Rito_NPC010p_MarryFirst",
0xd08ec813,"Rito_NPC011_ChoiceTeba",
0x1e04cc2c,"Rito_NPC011_First",
0x6b19eb24,"Rito_NPC011_First_Clear",
0x11d14fad,"Rito_NPC011_First_Start",
0x2fecd6b1,"Rito_NPC012_First",
0x899bdd05,"Rito_NPC013_First",
0x67b0e128,"Rito_NPC013_First_After",
0x908bcbcf,"Rito_NPC021_First",
0x7b8d71b8,"Rito_NPC021_First_After",
0xfa464a99,"Rito_NPC025_ClearFirst",
0xd3e121e1,"Rito_NPC025_Explain",
0x64c4efdc,"Rito_NPC025_First",
0x10211f58,"Rito_NPC027_Hero",
0xfa56c0e5,"Rito_NPC030_First",
0x26057a18,"Rito_NPC037_FinishFirst",
0x3ff1fe6b,"Rito_NPC037_First",
0x8b91fe62,"Rito_NPC037_GachiRock",
0xfe004264,"Rito_NPC037_HowAreYou",
0x34b69b74,"Rito_NPC037_NormalFirst",
0x2927cac9,"Rito_NPC037_Thanks",
0xf093c9fc,"Rito_NPC040_First",
0x56e4c248,"Rito_NPC041_First",
0x836d7aab,"RiversideMini_CastleWeapon_Activated",
0x98bd4c1f,"RiversideMini_CastleWeapon_Finish",
0xe1978d98,"RiversideMini_CastleWeapon_Ready",
0xc92548fe,"RiversideMini_CastleWeapon_ShowedShield",
0xb7ad36e1,"RiversideMini_RoyalRecipe_Activated",
0xfea656f1,"RiversideMini_RoyalRecipe_Clear00",
0x89a16667,"RiversideMini_RoyalRecipe_Clear01",
0x40b3d773,"RiversideMini_RoyalRecipe_Finish",
0xe2273b30,"RiversideMini_RoyalRecipe_Ready",
0xbbc44a84,"Rose_Broken",
0xfa4249b2,"SandStorm_Activated",
0x8c90bd33,"SandStorm_Finish",
0xcd3aee84,"SandStorm_Ready",
0x447687e2,"SandwormDef_Dungeon",
0x2ab2d1f4,"SandwormDef_Torch_AllOn",
0xff22711a,"SanrokuMini_Get_GoronArmor",
0x099f8d26,"SanrokuMini_Lizard_Activated",
0x43dc61de,"SanrokuMini_Lizard_Finish",
0x3b739ab2,"SanrokuMini_Lizard_Ready",
0x22c143dd,"SearchStone_Activated",
0x7406e4d2,"SearchStone_AfterTalk",
0x0cb0ee1b,"SearchStone_Dungeon",
0x69258575,"SearchStone_Finish",
0xadbca549,"SearchStone_Ready",
0x29d90a28,"SearchStone_Step1",
0xaebdb406,"SearchStone_Stone2end",
0x1601d363,"SearchStone_Stone3end",
0x8bd6ebda,"SearchStone_Stone4end",
0xa184fffc,"SecretClub_rumor",
0xd343a1ea,"SecretofObject_Activated",
0xbead9e8d,"SecretofObject_Finish",
0x2875b3a8,"SecretofObject_Ready",
0xac101cc9,"SecretofObject_Step1",
0x9e65bb99,"SeekerEye_Activated",
0x74ca795e,"SeekerEye_Finish",
0xf550ea59,"SeekerEye_Ready",
0x71354538,"SeekerEye_Step1",
0x66a8ebf7,"Sekimoto_Test01_Activated",
0xbcee8148,"Sekimoto_Test01_Finish",
0x06f97c6b,"Sekimoto_Test01_Ready",
0x62ffc562,"Sekimoto_Test01_Sekimoto_Test01_Step1",
0xc2d11ccb,"Self_HorseAssociationCustum_DispNameFlag",
0x99bdccdb,"SetugenUmayadoMini_Umahonephoto_Activated",
0xe6931d43,"SetugenUmayadoMini_Umahonephoto_Finish",
0x7eba6c95,"SetugenUmayadoMini_Umahonephoto_Ready",
0xb978cb30,"SetugenUmayadoMini_Umahonephoto_SetugenUmayadoMini_Umaphoto",
0x20ed3730,"SetugenUmayadoMini_Umahonephoto_Step01",
0xbaab8dec,"Shadow_Sign_Activated",
0x5f98a32c,"Shadow_Sign_Dungeon",
0xf2fb8287,"Shadow_Sign_Finish",
0x1779e789,"Shadow_Sign_Ready",
0xfbbc8f19,"ShieldofKolog_Activated",
0x40189df4,"ShieldofKolog_Failed",
0xfe5ad59b,"ShieldofKolog_Finish",
0x459efe03,"ShieldofKolog_Minigame",
0x0035c342,"ShieldofKolog_Ready",
0xaa6589d6,"ShieldofKolog_RentalBow",
0xa63b834c,"ShieldofKolog_RentalShield",
0x8f116c38,"ShieldofKolog_RentalSword",
0xfbe38210,"ShieldofKolog_Retire",
0x5a329194,"ShieldofKolog_Step010",
0x01937a9d,"ShieldofKolog_WeaponHide",
0xd05078c6,"ShieldofKolog_mini_Activated",
0x8fea514b,"ShieldofKolog_mini_Check",
0x332bfee6,"ShieldofKolog_mini_Check_Record",
0x26cfdaf3,"ShieldofKolog_mini_Check_Talk",
0x45ca5623,"ShieldofKolog_mini_Finish",
0xfe5b9dd6,"ShieldofKolog_mini_Goal",
0x1d33b3b9,"ShieldofKolog_mini_Minigame",
0x9bf178f7,"ShieldofKolog_mini_Ready",
0x407301a8,"ShieldofKolog_mini_Retire",
0xadd46f21,"SleepingBed",
0x012992a3,"SmallGoddess_Camera_First",
0x38941382,"SnowMountainRescue_Activated",
0x53929971,"SnowMountainRescue_Finished",
0x8f9c9091,"SnowMountainRescue_I7test",
0x6373c6f0,"SnowMountainRescue_Ready",
0x5e55af56,"SnowMountainRescue_TimeUp",
0x3aa42a3e,"Solitary_Maze_Activated",
0xa1681c82,"Solitary_Maze_Finish",
0x3774d27b,"Solitary_Maze_Ready",
0xa53f0b17,"SortTypeArmorPouch",
0x34cb0bff,"SortTypeBowPouch",
0x57a7a149,"SortTypeShieldPouch",
0x0d70fdd7,"SortTypeWeaponPouch",
0x6e74f3c0,"Stalfos_Complete",
0xf7d0a9fd,"StatueofZora_Activated",
0x16f1455f,"StatueofZora_Dungeon",
0xf25abc61,"StatueofZora_Finish",
0xb6470189,"StatueofZora_Ready",
0x3222aee8,"StatueofZora_Step1",
0xa86d3831,"StatueofZora_Weapon_Spear_049",
0x437449a7,"StolenBook_Activated",
0x6eba7954,"StolenBook_Conclusion_NPC008",
0x90196512,"StolenBook_Conclusion_NPC016",
0x8a937a5c,"StolenBook_Destination_NPC007",
0x1a2c67cd,"StolenBook_Destination_NPC008",
0xe48f7b8b,"StolenBook_Destination_NPC016",
0x2cb42617,"StolenBook_DungeonGenerate",
0x5abd3c97,"StolenBook_Finish",
0x30adf3b9,"StolenBook_KaidoNPC_Delete",
0x3bffc7a1,"StolenBook_NakinCheck",
0x3618a224,"StolenBook_Ready",
0xe22a4a88,"StoneDoor_SnowBall",
0x331b2d6b,"StoneDoor_SnowBall02",
0x267be80a,"Strange_Beacon_First",
0xcfe69678,"Strange_Beacon_Recipe",
0x5610f09d,"SunazarashiRace_Activated",
0x46dbc85b,"SunazarashiRace_Demo330_1",
0xdde4b884,"SunazarashiRace_Dungeon",
0x40eac3fd,"SunazarashiRace_Field1",
0x8614ebd1,"SunazarashiRace_Finish",
0xf32640f2,"SunazarashiRace_First",
0x8265c8bc,"SunazarashiRace_Gate01",
0x1b6c9906,"SunazarashiRace_Gate02",
0x6c6ba990,"SunazarashiRace_Gate03",
0xf20f3c33,"SunazarashiRace_Gate04",
0x85080ca5,"SunazarashiRace_Gate05",
0x1c015d1f,"SunazarashiRace_Gate06",
0x6b066d89,"SunazarashiRace_Gate07",
0x49c22b0a,"SunazarashiRace_Ready",
0x44606c8f,"SunazarashiRace_Retry01",
0x03dd5243,"SunazarashiRace_SetBall",
0xcda7846b,"SunazarashiRace_Step1",
0x54aed5d1,"SunazarashiRace_Step2",
0x23a9e547,"SunazarashiRace_Step3",
0x718993b1,"SunazarashiRace_mini_Ready",
0x0aacd74d,"SunazarashiRace_mini_first",
0x2e5448b2,"TabantaBridgeMini_Sundial_Activated",
0x5571f406,"TabantaBridgeMini_Sundial_Finish",
0xef8ee19c,"TabantaBridgeMini_Sundial_Play",
0xfa29076d,"TabantaBridgeMini_Sundial_Ready",
0x58e8b5f0,"TaiyoBox_Appear",
0xc1d6a53c,"Talk_Vagrant_BOKANGI",
0xd3fafe1d,"Talk_Vagrant_Bed",
0x3b72f8e3,"TalkedPump",
0xeecb7b02,"Tartnack_Complete",
0x489f5cd7,"TestQuest_Takano_01_Activated",
0x9e9668db,"TestQuest_Takano_01_Finish",
0x7c0656dd,"TestQuest_Takano_01_Ready",
0xbfd111f9,"TestQuest_Takano_01_TestQuest_Takano_01_Step01",
0x27476d13,"TestQuest_kwz001_Activated",
0xdb83eaf0,"TestQuest_kwz001_Extermination",
0x5a6f66a2,"TestQuest_kwz001_Finished",
0xf264d88e,"TestQuest_kwz001_Ready",
0x5f098aa2,"TestQuest_shimizu01_Activated",
0x8ab093f5,"TestQuest_shimizu01_Finish",
0x5ba137c1,"TestQuest_shimizu01_Ready",
0xab1d13bb,"TestQuest_shimizu01_Step",
0x9da58ecd,"Test_Giant_Clear1",
0x2210cdbb,"Test_Giant_Dead1",
0xa9570d64,"Test_HyruleRace1",
0x305e5cde,"Test_HyruleRace2",
0xfa8f9944,"Test_Imagawa_Activated",
0x11c75137,"Test_Imagawa_Finish",
0x48a14501,"Test_Imagawa_Ready",
0xc13f271c,"Test_Imagawa_STEP1",
0x6180d4fc,"Test_Imagawa_Test_Imagawa_Step1",
0x79b674d1,"Test_Mo2_DispNameFlag",
0x206a82b0,"Test_Mo_DispNameFlag",
0x5b050ad0,"Test_Murakami_Activated",
0xc4e7126c,"Test_Murakami_Finished",
0x65b1d8d8,"Test_Murakami_Ready",
0xd585d15c,"Test_Murakami_Step",
0x680d2ae5,"Test_Musician001",
0xf1047b5f,"Test_Musician002",
0x86034bc9,"Test_Musician003",
0x1867de6a,"Test_Musician004",
0x6f60eefc,"Test_Musician005",
0xf669bf46,"Test_Musician006",
0x816e8fd0,"Test_Musician007",
0x11d19241,"Test_Musician008",
0x66d6a2d7,"Test_Musician009",
0x06112b32,"Test_Musician010",
0x71161ba4,"Test_Musician011",
0xe81f4a1e,"Test_Musician012",
0x9f187a88,"Test_Musician013",
0x017cef2b,"Test_Musician014",
0x767bdfbd,"Test_Musician015",
0xb0c87c71,"Test_Musician_debug_on",
0xba4dd9b5,"Test_Nakayama00_Activated",
0x82ac6f27,"Test_Nakayama00_Finished",
0x20bac5e6,"Test_Nakayama00_Ready",
0xa4df6a87,"Test_Nakayama00_Step1",
0xd39940c4,"Test_Nakayama_Activated",
0x50f7e0b4,"Test_Nakayama_Finished",
0xb4685c3f,"Test_Nakayama_Horn",
0xec6bf990,"Test_Nakayama_Ready",
0x721dcfa6,"Test_Naoto_Michikai_Activated",
0x05ebfc15,"Test_Naoto_Michikai_Finished",
0xe7694ca5,"Test_Naoto_Michikai_Ready",
0x4e240afe,"Test_Tsumita00_Activated",
0x2d1016fb,"Test_Tsumita00_CHK",
0x3fca7531,"Test_Tsumita00_Finished",
0x5f7684e1,"Test_Tsumita00_Playing",
0x500fdf14,"Test_Tsumita00_Ready",
0x8faad53e,"Test_Tsumita01_Activated",
0x10703f4b,"Test_Tsumita01_CHK",
0xb0a81bd8,"Test_Tsumita01_Finish",
0xf678d4a0,"Test_Tsumita01_Ready",
0xbb3540b4,"Test_kkawazoe01_Activated",
0xf6a3769a,"Test_kkawazoe01_Finish",
0x94abef0f,"Test_kkawazoe01_Ready",
0x30dd5ce9,"Test_kkawazoe01_Step01",
0x613f567e,"Test_nakayama01_Activated",
0xcdc80f08,"Test_nakayama01_Finish",
0x4897e669,"Test_nakayama01_Horn1",
0xd19eb7d3,"Test_nakayama01_Horn2",
0xa6998745,"Test_nakayama01_Horn3",
0x38fd12e6,"Test_nakayama01_Horn4",
0x91f2b52f,"Test_nakayama01_Ready",
0x15971a4e,"Test_nakayama01_Step1",
0xac572dbd,"Test_sima_Activated",
0x9ba37a62,"Test_sima_Finish",
0x4943fc1a,"Test_sima_Ready",
0xf17350a8,"ThornyPath_Found",
0xc5ed41e9,"Threebros_giant_Activated",
0x657b4b50,"Threebros_giant_Finish",
0x4569b882,"Threebros_giant_Ready",
0x5ad7ecc6,"Throw_Out_Finish",
0x5cc8f324,"Throw_Out_Ready",
0x5e3e2350,"Throw_Out_Throw_Out",
0x6478de97,"Thunder_Sword_Activated",
0x25940a1d,"Thunder_Sword_Finish",
0x75239368,"Thunder_Sword_Ready",
0x5e32795e,"Thunder_Sword_Step010",
0x8b38f899,"Tincle7Quest_Activated",
0xf20bda3a,"Tincle7Quest_Blue",
0x9903ac89,"Tincle7Quest_Finished",
0xdaf67d16,"Tincle7Quest_Green",
0x047feed3,"Tincle7Quest_Indigo",
0xa57278e4,"Tincle7Quest_Orange",
0x5232bb7a,"Tincle7Quest_Purple",
0x22f91698,"Tincle7Quest_Ready",
0xc1e43cdb,"Tincle7Quest_Red",
0x5e0008f7,"Tincle7Quest_Step01",
0xc709594d,"Tincle7Quest_Step02",
0xb00e69db,"Tincle7Quest_Step03",
0x2e6afc78,"Tincle7Quest_Step04",
0x596dccee,"Tincle7Quest_Step05",
0x68b32503,"Tincle7Quest_Yellow",
0x8df017b6,"TominagaTest_Activated",
0xe7599ccc,"TominagaTest_Finished",
0x786b849e,"TominagaTest_Miss",
0x18b0508d,"TominagaTest_Out",
0xc1b4a349,"TominagaTest_Playing",
0x5e53bfb1,"TominagaTest_Ready",
0x6cce80af,"Tower04_Domino001",
0xefc28e6d,"TurnTestQuest_Activated",
0x10c66970,"TurnTestQuest_Finish",
0x92e80440,"TurnTestQuest_Ready",
0xa0e3b8d7,"TutorialDungeon_Finish",
0x67a1813a,"TutorialDungeon_Ready",
0x05cd750a,"TwoWheels_Activated",
0xa045ad13,"TwoWheels_Dungeon",
0x209fcd5e,"TwoWheels_Finish",
0x7eb3c3af,"TwoWheels_Ready",
0x8a7d1a78,"UMiiMini_GiveCake_Activated",
0x88cb7810,"UMiiMini_GiveCake_Finish",
0x788ac0f6,"UMiiMini_GiveCake_Give",
0x4dd9d802,"UMiiMini_GiveCake_Ready",
0x25718f36,"UMiiMini_MakeVillage_Activated",
0x18b6e526,"UMiiMini_MakeVillage_Bell",
0xe7bff9b2,"UMiiMini_MakeVillage_Finish",
0xbbde4235,"UMiiMini_MakeVillage_Invite01",
0x22d7138f,"UMiiMini_MakeVillage_Invite02",
0x55d02319,"UMiiMini_MakeVillage_Invite03",
0xcbb4b6ba,"UMiiMini_MakeVillage_Invite04",
0xbcb3862c,"UMiiMini_MakeVillage_Invite05",
0x323eab7c,"UMiiMini_MakeVillage_LookFor01",
0xab37fac6,"UMiiMini_MakeVillage_LookFor02",
0xdc30ca50,"UMiiMini_MakeVillage_LookFor03",
0x42545ff3,"UMiiMini_MakeVillage_LookFor04",
0x35536f65,"UMiiMini_MakeVillage_LookFor05",
0xe605a42e,"UMiiMini_MakeVillage_Marry",
0x892f9bd4,"UMiiMini_MakeVillage_Ready",
0xd9f2dc8c,"UMiiMini_MakeVillage_Wood01",
0x40fb8d36,"UMiiMini_MakeVillage_Wood02",
0x37fcbda0,"UMiiMini_MakeVillage_Wood03",
0xa9982803,"UMiiMini_MakeVillage_Wood04",
0x33c76467,"UMiiMini_RichmansHobby_Activated",
0xa611db10,"UMiiMini_RichmansHobby_Beat",
0x4f9de1de,"UMiiMini_RichmansHobby_Finish",
0xcce8a279,"UMiiMini_RichmansHobby_Ready",
0xa6615b0c,"UMiiVillage_NPC003_First",
0xd602d6ab,"UMiiVillage_NPC003_NormalFirst",
0x67f293fe,"UMiiVillage_NPC008_Cook",
0xa4660ff6,"UMiiVillage_NPC008_First",
0x8a7eb9ca,"UMiiVillage_NPC008_Wife",
0x913804d4,"UMiiVillage_NPC009_BeatGuardians",
0x02110442,"UMiiVillage_NPC009_First",
0x5d2ab086,"UMiiVillage_NPC009_Ote",
0x239cf652,"UMiiVillage_NPC009_Reward",
0x5b23410f,"UMiiVillage_NPC010_First",
0xbbcfbfbe,"UMiiVillage_NPC010_OK",
0x2ba8f93a,"UMiiVillage_NPC010_Reward",
0xf3b7ce18,"UMiiVillage_NPC027_Cooker",
0x100b7862,"UMiiVillage_NPC027_First",
0x495b9a3f,"UMiiVillage_NPC027_MamonoCake",
0x0f3dd290,"UMiiVillage_NPC031_Demo370",
0xd88ca757,"UMiiVillage_NPC031_FinishFirst",
0xbf714dc6,"UMiiVillage_NPC031_First",
0xdcb4dbb3,"UMiiVillage_NPC031_Get",
0x3e5094a4,"UMiiVillage_NPC031_Marry_First",
0xc9f4d122,"UMiiVillage_NPC031_Reward",
0x1558b694,"UMiiVillage_NPC050_Demo370",
0xfdd83564,"UMiiVillage_NPC050_FinishFirst",
0xde323f1d,"UMiiVillage_NPC050_Married",
0x7644a836,"UMiiVillage_NPC050_Reward",
0xf7c2ba79,"UmaKyoukai_001",
0x6ecbebc3,"UmaKyoukai_002",
0x749c6090,"UotoriMini_RecipeSea_Activated",
0x0599b273,"UotoriMini_RecipeSea_Finish",
0x171e4e1d,"UotoriMini_RecipeSea_Ready",
0xa648ce9a,"UotoriMini_RecipieSea_FinishDay",
0x903a77d3,"UotoriMini_RecipieSea_GetReward",
0x82252b20,"UotoriMini_RecipieSea_IsTalked",
0x141f42a5,"UotoriMini_RecipieSea_PorchFull",
0xca2370a6,"UotoriMini_RecoverBay_Activated",
0x6d0575c9,"UotoriMini_RecoverBay_Exterminate",
0x46036fbc,"UotoriMini_RecoverBay_Finish",
0x5cb7f7d9,"UotoriMini_RecoverBay_GetReward",
0x03d2b494,"UotoriMini_RecoverBay_IsTalked",
0xe42aeb75,"UotoriMini_RecoverBay_Ready",
0x2b2c82dc,"UotoriMini_SinkTerasure_GetReward",
0x91a5267e,"UotoriMini_SinkTreasure_Activated",
0x70b0c3b3,"UotoriMini_SinkTreasure_Finish",
0x5b2a712b,"UotoriMini_SinkTreasure_IsTalked",
0x54cc12df,"UotoriMini_SinkTreasure_Ready",
0x8ef766d3,"UotoriMini_SinkTreasure_Salvage",
0xcda06bce,"WM_BloodyDay",
0xa089b5c2,"WM_DaytimeFlag",
0x2a7897b5,"WM_IsEveningA",
0xb371c60f,"WM_IsEveningB",
0x1508673e,"WM_IsMorningA",
0x8c013684,"WM_IsMorningB",
0x1c328e7a,"WM_IsNightA",
0x853bdfc0,"WM_IsNightB",
0x7a1405f7,"WM_IsNoonA",
0xe31d544d,"WM_IsNoonB",
0xf97f7a67,"WM_NighttimeFlag",
0x6327eaab,"Water_Relic_Activated",
0xa69b6778,"Water_Relic_ArmorSend",
0x49238bba,"Water_Relic_BoysNotRun",
0x3ec5adce,"Water_Relic_Finished",
0xf67390c3,"Water_Relic_KingTalk01",
0x0fc5f90d,"Water_Relic_RainStop",
0x2e45ec4d,"Water_Relic_Ready",
0xaa20432c,"Water_Relic_Step1",
0xfbc082fa,"Water_Relic_Step1_1",
0x62c9d340,"Water_Relic_Step1_2",
0x33291296,"Water_Relic_Step2",
0x442e2200,"Water_Relic_Step3",
0xda4ab7a3,"Water_Relic_Step4",
0xad4d8735,"Water_Relic_Step5",
0x0aed2999,"Water_Relic_SunnyKingTalk",
0xca51a626,"Water_Relic_firstPrinceRide",
0x8defc7da,"WeaponQuest2_1_3",
0xdcdffda4,"WeaponQuest_1_3",
0xefb6331e,"Whales_Gamyo",
0x11935096,"Whales_Negyui",
0xe8914f88,"Whales_Orak",
0xecd6fd3d,"Wind_Relic_Activated",
0x7115945a,"Wind_Relic_BatteryONOFF",
0xc1549c59,"Wind_Relic_Battery_A_02_Die",
0x5e7756c2,"Wind_Relic_Battery_A_02_Die_Front",
0x9150a6f9,"Wind_Relic_Battery_A_02_Die_L",
0x6b5f9b9a,"Wind_Relic_Battery_A_02_Die_R",
0xea55b8d6,"Wind_Relic_Battery_A_02_Die_Rear",
0xb740ed89,"Wind_Relic_Battle",
0x678d20bb,"Wind_Relic_BattleClear",
0x2bd95097,"Wind_Relic_BattleFailed",
0xdc854e24,"Wind_Relic_BattleFinished",
0x12806f51,"Wind_Relic_BattleFirstDemoEnd",
0x73468369,"Wind_Relic_BattlePlaying",
0x1d458432,"Wind_Relic_BattleStart",
0x748aee36,"Wind_Relic_Battle_Fail_BombArrow",
0xa77bd751,"Wind_Relic_Battle_Fail_Bow",
0xc0840269,"Wind_Relic_BombArrow1",
0x598d53d3,"Wind_Relic_BombArrow2",
0x8a8e095b,"Wind_Relic_Dungeon",
0xaa4cc412,"Wind_Relic_Finished",
0xe3374f6b,"Wind_Relic_MiniGameStart",
0x0d289137,"Wind_Relic_NPC010_Battle_First",
0x1da2bd4d,"Wind_Relic_NPC010_Finish_First",
0x3107e909,"Wind_Relic_NPC010_First",
0xd221d81d,"Wind_Relic_NPC010_Parashowl_First",
0xa7b68296,"Wind_Relic_NPC10_HelpYou",
0x43e142a5,"Wind_Relic_Parashawl",
0xdedef803,"Wind_Relic_ParashawlClear",
0xac4bae6c,"Wind_Relic_ParashawlFinished",
0xd752446c,"Wind_Relic_ParashawlPlaying",
0x89207ba7,"Wind_Relic_ParashawlTimeOver",
0xde44be7a,"Wind_Relic_Parashawl_EventTimerAppear",
0x1c6d735f,"Wind_Relic_Parashawl_Fail_AreaInAppear",
0x2830388c,"Wind_Relic_Parashawl_Fail_BombArrow",
0xdf0cd013,"Wind_Relic_Parashawl_Fail_Bow",
0xbeb61858,"Wind_Relic_Parashawl_Fail_Self",
0xdea1a284,"Wind_Relic_Parashawl_Fail_TimeOut",
0x20ab55c9,"Wind_Relic_Ready",
0xf05151a4,"Wind_Relic_Saki",
0xa4cefaa8,"Wind_Relic_Step1",
0x05668d9d,"Wind_Relic_TBox_Appear",
0xfc92face,"Wind_Relic_TargetONOFF",
0x36766bd3,"Wind_Relic_WindRelicDemo20ONOFF",
0x94574ded,"Wind_Relic_WindRelicONOFF",
0xc59942d9,"WisdomFirstDemoKill",
0xf37e3e58,"WolfLink_DiedToday",
0xce60eccd,"WolfLink_HasBeenUsed",
0x0fd37e5b,"Wood01_OK",
0x1d66d1b5,"Wood02_OK",
0xa5dab6d0,"Wood03_OK",
0x380d8e69,"Wood04_OK",
0xa71b2d59,"YmdQueTest_Activated",
0x6348e737,"YmdQueTest_Finished",
0xd5f3ff5e,"YmdQueTest_Ready",
0x3cd08cf0,"Z_quest_test01_Activated",
0x030c6b80,"Z_quest_test01_Extermination",
0xcc2bfef7,"Z_quest_test01_Finished",
0x408279d8,"Z_quest_test01_Ready",
0x7e3ab448,"ZoraGuide_talk",
0xa7341c7d,"ZoraKing_talk",
0x3e5ef1c5,"ZoraMeet01",
0xa757a07f,"ZoraMeet02",
0xd05090e9,"ZoraMeet03",
0x4e34054a,"ZoraMeet04",
0x393335dc,"ZoraMeet05",
0xa03a6466,"ZoraMeet06",
0xd73d54f0,"ZoraMeet07",
0x33a87959,"ZoraMini_DiveCalenge_Waterfall",
0xde022111,"ZoraMini_DiveChallenge_Activated",
0xe6b1c5e0,"ZoraMini_DiveChallenge_Finish",
0x02f8fdfe,"ZoraMini_DiveChallenge_ItemGet",
0x0388a18e,"ZoraMini_DiveChallenge_ItemNoGet",
0x5c62cf95,"ZoraMini_DiveChallenge_Ready",
0xd80760f4,"ZoraMini_DiveChallenge_Step1",
0x410e314e,"ZoraMini_DiveChallenge_Step2",
0x92998463,"ZoraMini_FlowedWife_Activated",
0xc70c7801,"ZoraMini_FlowedWife_Finish",
0x552b9ba1,"ZoraMini_FlowedWife_Ready",
0x0cb63152,"ZoraMini_HarvestingStone_Activated",
0x35effbf2,"ZoraMini_HarvestingStone_Finish",
0xd4a826f2,"ZoraMini_HarvestingStone_Ready",
0x50cd8993,"ZoraMini_HarvestingStone_Step1",
0x23fa479f,"ZoraMini_HarvestingStone_WaterRelicClearTalk",
0xc0334cb6,"ZoraMini_ReliefSearch_Activated",
0xb9e1be1b,"ZoraMini_ReliefSearch_Finish",
0x12aab5c0,"ZoraMini_ReliefSearch_FirstTalk",
0x09913cac,"ZoraMini_ReliefSearch_Ready",
0xf61a6e9f,"ZoraMini_ReliefSearch_RewardWait",
0x74d50a37,"ZoraMini_ReliefSearch_SearchRelief",
0x268a8ec9,"ZoraPriest_talk",
0x0cbf112c,"ZoraRelief_Check_01",
0x95b64096,"ZoraRelief_Check_02",
0xe2b17000,"ZoraRelief_Check_03",
0x7cd5e5a3,"ZoraRelief_Check_04",
0x0bd2d535,"ZoraRelief_Check_05",
0x92db848f,"ZoraRelief_Check_06",
0xe5dcb419,"ZoraRelief_Check_07",
0x7563a988,"ZoraRelief_Check_08",
0x0264991e,"ZoraRelief_Check_09",
0x62a310fb,"ZoraRelief_Check_10",
0x2b6fd8a0,"Zora_FlogMini_Activated",
0xc8d087ea,"Zora_FlogMini_Finish",
0xea1205ad,"Zora_FlogMini_Getitem",
0x892d44e8,"Zora_FlogMini_Ready",
0x37c4c99b,"climateFirst",
0xcdcf9ea5,"fujibayashi_000",
0x54c6cf1f,"fujibayashi_003",
0xed2fd607,"giant_ball1",
0x742687bd,"giant_ball2",
0x0321b72b,"giant_ball3",
0xbc90b622,"giant_ball_killA",
0x2599e798,"giant_ball_killB",
0x529ed70e,"giant_ball_killC",
0x801d82c9,"giant_clear1",
0x1914d373,"giant_clear2",
0x6e13e3e5,"giant_clear3",
0x14a4027b,"giant_dungeon",
0x2eed7daf,"isso_treasure_magunemita",
0x9835f7d9,"isso_treasure_mini_Activated",
0xc83d80df,"isso_treasure_mini_Finish",
0xe371ab44,"isso_treasure_mini_Flash",
0x642a71e8,"isso_treasure_mini_Ready",
0x79468f33,"isso_treasure_mini_Step2",
0x7dcd81d7,"isso_treasure_mini_katudouba",
0xd2a635bb,"isso_treasure_mini_open",
0xdc560702,"iwagoro_001",
0x455f56b8,"iwagoro_002",
0x3258662e,"iwagoro_003",
0x9efb51e8,"iwagoro_gerudo_002",
0xe9fc617e,"iwagoro_gerudo_003",
0x1b3a678d,"kawazoe_FujitaLearning001_Activated",
0x5a6b0f22,"kawazoe_FujitaLearning001_Finish",
0xe02b1724,"kawazoe_FujitaLearning001_Ready",
0xbc1b33a7,"nakayama_demotest",
0xf5d80b50,"paras_cameradelete",
0xdbb287a7,"subchallnpc000_DispNameFlag",
0x947729be,"subchallnpc000_twin_DispNameFlag",
0xbca832fe,"testZel_Activated",
0xd1f23b62,"testZel_Finished",
0xd98123f9,"testZel_Get",
0xbc65feaa,"testZel_Ready",
0xd7d03f9a,"test_Dummy01",
0xdc1a7d69,"test_imagawa_flag1",
0xed81004f,"test_imagawa_tunobueflag",
0x8771271e,"yorozu_Activated",
0x66c71342,"yorozu_Finished",
0x21129164,"yorozu_Ready"
	]},
	{type:"s32",id:"(v1.3) ",hashes:[
0x4d1f473f,"EquipTime_Armor_171_Head",
0x00d7e634,"EquipTime_Armor_173_Head",
0x020dfe4d,"EquipTime_Weapon_Sword_080",
0x062a0904,"HardMode_HighScore",
0x0d283d8d,"Defeated_Enemy_Lynel_Gold_Num",
0x13974a67,"EquipTime_Armor_174_Lower",
0x1467c740,"TreasureHunt_touzoku03_RemainingBox",
0x1dd2d68c,"EquipTime_Armor_174_Head",
0x216ac17f,"EquipTime_Armor_171_Upper",
0x363a5ad0,"AocField_PartsDrop_1510445978",
0x3cde0293,"Defeated_Enemy_AirOctarock_Num",
0x41af65c0,"EquipTime_Armor_171_Lower",
0x501a7787,"EquipTime_Armor_176_Head",
0x579819ff,"PictureBookSize_Enemy_AirOctarock",
0x5cd16cbb,"AocField_PartsDrop_1895229974",
0x5ec90fed,"PosTrackEndPointBlockNumber",
0x60057b96,"Defeated_Enemy_Bokoblin_Gold_Num",
0x69304d18,"PorchTime_Armor_172_Head",
0x6a1fff01,"MasterSword_Add_BeamPower",
0x6a465a2b,"PictureBookSize_Enemy_Lizalfos_Gold",
0x6c558bd5,"Defeated_Enemy_Moriblin_Gold_Num",
0x6e86df09,"Defeated_Enemy_Lizalfos_Gold_Num",
0x71aa6d80,"TreasureHunt_touzoku04_RemainingBox",
0x7352eed8,"EquipTime_Armor_174_Upper",
0x746364da,"PictureBookSize_Enemy_Moriblin_Gold",
0x750acedb,"EquipTime_Weapon_Sword_081",
0x7b248982,"MasterSword_Add_Power",
0x7bfc963f,"PosTrackEndPointRecordIndex",
0x85888b2f,"PictureBookSize_Enemy_Lynel_Gold",
0x8751ca9b,"PorchTime_Armor_174_Lower",
0x8bb63be6,"PictureBookSize_Enemy_Bokoblin_Gold",
0xa26c9ebd,"PorchTime_Armor_173_Head",
0xb5ac4183,"PorchTime_Armor_171_Upper",
0xb69286fa,"PorchTime_Weapon_Sword_080",
0xbf69ae05,"PorchTime_Armor_174_Head",
0xc195b66c,"PorchTime_Weapon_Sword_081",
0xcacc5789,"AocField_PartsDrop_418190687",
0xcb8b3591,"EquipTime_Armor_172_Head",
0xd0ea5e87,"PosTrackEndPointPosIndex",
0xd569e53c,"PorchTime_Armor_171_Lower",
0xe7946e24,"PorchTime_Armor_174_Upper",
0xefa43fb6,"PorchTime_Armor_171_Head",
0xf0a375f5,"AocField_PartsDrop_3148495283",
0xf2a10f0e,"PorchTime_Armor_176_Head",
	]},
	{type:"vector3f",id:"(v1.3) ",hashes:[0x51f28649,"WarpDLC_DestPos"]},
	{type:"f32",id:"(v1.3) ",hashes:[0xc1cbed1e,"WarpDLC_DestDegree"]},
	{type:"bool",id:"(v1.3) ",hashes:[

0x003d1c6a,"AocField_Enemy_Golem_Little_Ice_3009239476",
0x010d3f8c,"AocField_Obj_BreakBoxIron_100enemy_265164775",
0x01390a61,"AocField_Kibako_Contain_100enemy_808502728",
0x013b0423,"AocField_Enemy_Bokoblin_Junior_2543478489",
0x015737a2,"AocField_Obj_BreakBoxIron_100enemy_3228380297",
0x0170561a,"MainField_WeaponDrop_243070540",
0x01905468,"MainField_WeaponDrop_4246752370",
0x01948c06,"AocField_Animal_Fish_A_3359327701",
0x019b391c,"AocField_TBox_Dungeon_Stone_3509277471",
0x01d6400c,"AocField_WeaponDrop_231918048",
0x021064aa,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_4154249897",
0x0230ec81,"MainField_WeaponDrop_387501742",
0x0290c84f,"AocField_Kibako_Contain_100enemy_640297540",
0x02b4808d,"IsGet_Armor_173_Head",
0x02bd16df,"MainField_WeaponDrop_1618111426",
0x02d5c233,"TreasureHunt01_Finish",
0x02d7fea6,"MainField_WeaponDrop_2293041085",
0x02df9aee,"100enemy_Active3",
0x02f5e025,"MainField_TBox_Field_Iron_3402792126",
0x03870199,"MainField_TBox_Field_Iron_2215561456",
0x03b773f5,"AocField_Item_Fruit_I_1485438886",
0x04056327,"AocField_TBox_Dungeon_Wood_AreaCulling_813105602",
0x04065300,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3198555883",
0x047373cb,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3918019498",
0x04a15c05,"MainField_Enemy_Lizalfos_Junior_1486392406",
0x04a1aa2f,"MainField_WeaponDrop_1775680432",
0x04c59e07,"AocField_Enemy_Golem_Little_Ice_2013366801",
0x04f0d2e2,"TreasureHunt_touzoku04_Open03",
0x04fddb94,"AocField_Enemy_Chuchu_Electric_Senior_623988554",
0x05426d8c,"AocField_Obj_LiftRockWhite_100enemy_1591065961",
0x06161036,"AocField_Kibako_Contain_100enemy_1093614773",
0x064eb603,"AocField_Animal_Insect_F_3020550838",
0x06c6f462,"AocField_Kibako_Contain_100enemy_612027213",
0x070e53c3,"AocField_Weapon_Lsword_024_2920120443",
0x07420d84,"TreasureHunt02_Ready",
0x07aa0130,"AocField_Enemy_Bokoblin_Junior_3910381286",
0x07c5985b,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3989916214",
0x080981c5,"AocField_WeaponDrop_458827803",
0x081557c9,"AocField_OreDrop2_Enemy_Golem_Ice_3122104937",
0x090e1477,"MainField_TBox_Field_Wood_1338350447",
0x092421df,"AocField_Enemy_Chuchu_Ice_Middle_3169733684",
0x096295eb,"AocField_Kibako_Contain_100enemy_2622977794",
0x0999ad0d,"AocField_Kibako_Contain_100enemy_34662492",
0x099c9ad3,"MainField_Enemy_Lizalfos_Junior_540083681",
0x0a7a8813,"AocField_WeaponDrop_2697030370",
0x0a82fed0,"AocField_WeaponDrop_3910381286",
0x0a8b0020,"MainField_WeaponDrop_1811035589",
0x0aa31e57,"AocField_Weapon_Bow_038_3003747403",
0x0ab4b2be,"AocField_Kibako_Contain_100enemy_1712990013",
0x0acb50ee,"AocField_OreDrop1_Enemy_Golem_Ice_3122104937",
0x0ad4522e,"TreasureHunt_touzoku04_Activated",
0x0ad9d32d,"MainField_TBox_Field_Wood_3841257233",
0x0af93ec3,"FirstTry_100enemy_Senior",
0x0b7eadf3,"AocField_OreDrop0_Enemy_Golem_Ice_3122104937",
0x0b7ec92d,"AocField_WeaponDrop_3741193877",
0x0b9880a0,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_2028366933",
0x0be4b6b5,"AocField_TBox_Dungeon_Stone_3176382602",
0x0c13fbc3,"AocField_WeaponDrop_2543478489",
0x0cb9e6c2,"AocField_Enemy_Guardian_A_3148495283",
0x0d8675c5,"AocField_Weapon_Sword_047_191869960",
0x0df91c3a,"MainField_WeaponDrop_4010333502",
0x0dfa3790,"AocField_Kibako_Contain_100enemy_2611137501",
0x0e35dbfb,"TreasureHunt_touzoku03_Open03",
0x0e63034f,"AocField_Kibako_Contain_100enemy_798726197",
0x0e6866ad,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_352538035",
0x0f06a632,"AocField_Obj_BreakBoxIron_100enemy_495069595",
0x0f17b1b1,"AocField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_4196419116",
0x0f393603,"AocField_Kibako_Contain_100enemy_3977922659",
0x0f800192,"MainField_Enemy_Moriblin_Junior_2986027363",
0x10522329,"AocField_Enemy_Wizzrobe_Fire_Senior_3947906964",
0x105c8385,"MainField_WeaponDrop_389373855",
0x10627fe1,"AocField_Obj_LiftRockWhite_100enemy_1181269523",
0x1070f4f6,"AocField_Obj_LiftRockWhite_100enemy_3231733775",
0x10848b09,"AocField_TBox_Dungeon_Iron_469590919",
0x10b5f4f4,"AocField_Enemy_Wizzrobe_Fire_1294421024",
0x10d05789,"MainField_WeaponDrop_910514622",
0x1193b12b,"AocField_Obj_BreakBoxIron_100enemy_630306700",
0x11c0a5ee,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1867845270",
0x11e85c10,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_4010333502",
0x11fb8972,"AocField_Weapon_Sword_019_2638959266",
0x120b580f,"AocField_Enemy_Lizalfos_Fire_3006758860",
0x1221dedb,"AocField_Obj_LiftRockWhite_100enemy_2242676557",
0x123950c2,"AocField_WeaponDrop_2117518451",
0x1241a50f,"AocField_Kibako_Contain_100enemy_4021438356",
0x129319de,"AocField_TBox_Dungeon_Wood_AreaCulling_1979524167",
0x12a0613e,"AocField_WeaponDrop_3002604607",
0x12e699b9,"MainField_WeaponDrop_3486101056",
0x12e705eb,"AocField_Item_Fruit_I_2233411107",
0x130b661a,"AocField_TBox_Dungeon_Stone_1362194037",
0x1325cf63,"AocField_WeaponDrop_828224922",
0x1343d94a,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1217080480",
0x1369d36f,"AocField_Item_Mushroom_J_1210153151",
0x136ca508,"AocField_Kibako_Contain_100enemy_3014815639",
0x137395ff,"AocField_Weapon_Sword_043_1125572089",
0x137d1932,"AocField_Enemy_Keese_Fire_AllDay_4218451764",
0x13dbb542,"MainField_KibakoDesert_Contain_01_1602936431",
0x14593af4,"AocField_Enemy_Bokoblin_Bone_Junior_AllDay_874645658",
0x145c1285,"AocField_Enemy_Bokoblin_Middle_629989137",
0x147f1ade,"AocField_Animal_Insect_F_872060129",
0x14c84da0,"AocField_Animal_Fish_H_4196503137",
0x14fe0770,"AocField_Obj_BreakBoxIron_100enemy_838837819",
0x1503cf0f,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1277357706",
0x150b4e96,"AocField_WeaponDrop_1616738441",
0x15c210aa,"AocField_Obj_LiftRockWhite_100enemy_1975623471",
0x15f7b355,"AocField_Kibako_Contain_100enemy_1844099892",
0x1649d61a,"AocField_Enemy_Bokoblin_Junior_3671710827",
0x16636d05,"AocField_Animal_Insect_F_1549174229",
0x16771ece,"AocField_Obj_BreakBoxIron_100enemy_3153404463",
0x16791089,"AocField_Weapon_Lsword_027_1747186377",
0x167f18ee,"MainField_TBox_Field_Iron_NoReaction_Aoc_1243080332",
0x169656fa,"AocField_Weapon_Sword_021_315869825",
0x16b5c216,"AocField_WeaponDrop_214379543",
0x16fa9164,"AocField_Item_Material_04_2589463568",
0x173a35c2,"AocField_Weapon_Sword_008_605323027",
0x1789c08a,"MainField_WeaponDrop_2028366933",
0x178c63f4,"AocField_Enemy_Chuchu_Fire_Junior_1835025589",
0x17e261f1,"AocField_WeaponDrop_3324670781",
0x181e73dc,"IsPlayed_Demo202_2",
0x182ca758,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1399783231",
0x19116812,"AocField_TBox_Dungeon_Wood_AreaCulling_452454445",
0x199a189b,"AocField_Obj_BreakBoxIron_100enemy_1701205100",
0x19adbcff,"MiniGame100enemy_Ready",
0x19f682c4,"100enemy_Clear_Middle",
0x19faeb44,"MainField_TBox_Field_Wood_1613917961",
0x1a217343,"AocField_Weapon_Lsword_032_4243993761",
0x1a5172a5,"AocField_Weapon_Lsword_020_751852807",
0x1a7f36cd,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2969876181",
0x1aabb4d8,"MainField_WeaponDrop_3241035409",
0x1af7b9b7,"AocField_Obj_LiftRockWhite_100enemy_1338768991",
0x1b17758f,"MainField_TBox_Field_Wood_392677133",
0x1b2a7def,"MainField_TBox_Field_Iron_2973292086",
0x1b5be3f3,"AocField_Animal_Fish_B_1958173438",
0x1b5dc155,"AocField_Weapon_Lsword_030_969423704",
0x1b6129fa,"AocField_WeaponDrop_3671710827",
0x1b79fa0e,"100enemy_VanishPreist",
0x1ba79ddf,"MainField_WeaponDrop_2663640029",
0x1bd4e199,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3838349298",
0x1bff8135,"MainField_WeaponDrop_3308598352",
0x1c500f0f,"AocField_Enemy_Guardian_Mini_Junior_5990461",
0x1c92bf0c,"MainField_TBox_Field_Wood_1272405920",
0x1cbe7bf2,"MainField_KibakoDesert_Contain_01_1263379544",
0x1d4bd979,"MainField_TBox_Field_Wood_1370307555",
0x1d822fa0,"AocField_Item_Fruit_A_2918433842",
0x1dcdebc0,"AocField_WeaponDrop_846222648",
0x1de8d8b2,"MainField_WeaponDrop_540083681",
0x1dff878a,"AocField_Enemy_Bokoblin_Middle_239572651",
0x1e8d1344,"AocField_Enemy_Guardian_Mini_Junior_833843872",
0x1eed0dcc,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_485819731",
0x1f11af22,"AocField_Enemy_Bokoblin_Junior_2117518451",
0x1f24634c,"AocField_Barrel_100enemy_4244734681",
0x1fb1b035,"IsGet_Armor_174_Head",
0x1feb8d8a,"MainField_WeaponDrop_3683406634",
0x1ffb5862,"AocField_WeaponDrop_1231095740",
0x20185f97,"AocField_BoarMeat_4171413213",
0x202e8a9a,"MainField_TBox_Field_Iron_2139859020",
0x209028c4,"AocField_Enemy_Moriblin_Senior_Volcano_3002604607",
0x20bf9f64,"MainField_WeaponDrop_4004061010",
0x20dcc411,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_143431898",
0x2105d43a,"MainField_TBox_Field_Wood_3054835428",
0x215ad04f,"AocField_WeaponDrop_845395278",
0x216a392e,"AocField_TBox_Dungeon_Wood_AreaCulling_2434597244",
0x21771546,"MainField_WeaponDrop_3701576696",
0x21983d19,"AocField_Enemy_Moriblin_Bone_Junior_4011199496",
0x21f53fa1,"AocField_TBox_Dungeon_Stone_3345699953",
0x22b8d7e5,"AocField_Enemy_Bokoblin_Dark_1147346802",
0x231be2c9,"MainField_Enemy_Wizzrobe_Electric_3344676505",
0x233926bc,"AocField_Animal_Fish_H_1393897586",
0x23768e24,"MainField_KibakoDesert_Contain_01_4133248812",
0x2386d6d6,"MainField_TBox_Field_Iron_4230016077",
0x239fd385,"MainField_TBox_Field_Wood_3239192178",
0x23a91436,"AocField_Item_Mushroom_A_361943504",
0x23ef47ad,"TreasureHunt_touzoku03_Activated",
0x24300d42,"AocField_Kibako_Contain_100enemy_2997771875",
0x245179ad,"MainField_WeaponDrop_1808684326",
0x249c5fa5,"100enemy_VanishShield",
0x249f0e55,"AocField_Weapon_Shield_021_2559576631",
0x24b770aa,"MainField_WeaponDrop_3898937639",
0x24f0609b,"AocField_Obj_LiftRockWhite_100enemy_385465106",
0x24f148e0,"AocField_WeaponDrop_3475091277",
0x2514f414,"AocField_Enemy_Keese_Fire_AllDay_3281343417",
0x25437a7e,"AocField_WeaponDrop_1291264366",
0x25690653,"AocField_Item_Fruit_A_3386936300",
0x257981d6,"AocField_Kibako_Contain_100enemy_1649356389",
0x25cf7bb7,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1463562875",
0x25cff64c,"MainField_WeaponDrop_352538035",
0x266525b5,"AocField_Enemy_Chuchu_Fire_Junior_2620847712",
0x26a0dc82,"MainField_TBox_Field_Wood_1190033678",
0x26cdd970,"AocField_Enemy_Lizalfos_Junior_4186334990",
0x27047e48,"AocField_Enemy_Guardian_A_Fixed_Moss_2116069978",
0x27268454,"MainField_WeaponDrop_1848240742",
0x27307d2c,"AocField_WeaponDrop_3349486980",
0x2766b277,"AocField_Weapon_Sword_004_1023772755",
0x276959a2,"AocField_TBox_Dungeon_Stone_1125891243",
0x28107df2,"AocField_Item_Mushroom_F_2807698529",
0x2820a34e,"IsRegisteredPictureBook_Enemy_Lizalfos_Gold",
0x282da823,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_2383519508",
0x2869917c,"AocField_Barrel_100enemy_913295401",
0x2879ca4a,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3476407294",
0x287d021d,"AocField_Obj_BreakBoxIron_100enemy_2212936760",
0x289331b7,"MainField_WeaponDrop_2045168746",
0x2913acb9,"AocField_Obj_BreakBoxIron_100enemy_1653578772",
0x291a8058,"MainField_WeaponDrop_2434795977",
0x2967c7d1,"AocField_Barrel_100enemy_4190477766",
0x298e9347,"AocField_Obj_LiftRockWhite_100enemy_1962554776",
0x29ab55bf,"AocField_Enemy_Lizalfos_Fire_2007603243",
0x29b542a4,"AocField_Enemy_Guardian_Mini_Baby_3255367254",
0x29c4e320,"MainField_WeaponDrop_2686098826",
0x29cfca6f,"TreasureHunt_touzoku04_Ready",
0x2a37c8ec,"MainField_TBox_Field_Wood_1123805299",
0x2a41a91d,"100enemy_Clear_Senior",
0x2a5927dd,"AocField_Obj_BreakBoxIron_100enemy_3808954744",
0x2a87ea9c,"AocField_Weapon_Spear_009_1666158935",
0x2ad7c6fb,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_243070540",
0x2b53600a,"MainField_TBox_Field_Iron_1979810948",
0x2bb44109,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3063344506",
0x2bcc3f74,"AocField_TBox_Dungeon_Stone_3982205396",
0x2c385bf3,"AocField_WeaponDrop_4275721638",
0x2c79dba0,"MainField_WeaponDrop_4020411446",
0x2ce47896,"MainField_WeaponDrop_3609046300",
0x2d0314df,"AocField_WeaponDrop_2807187952",
0x2d068698,"AocField_IceWall_100enemy_1641950716",
0x2d2dc5bd,"AocField_WeaponDrop_3083454260",
0x2d8c49a3,"MainField_WeaponDrop_465634981",
0x2d92b934,"MainField_TBox_Field_Iron_1378458810",
0x2d9611f2,"AocField_KibakoSnowMountain_Contain_100enemy_3943302584",
0x2dc3a496,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3516993228",
0x2ddd4fe4,"AocField_Animal_Fish_H_2582756187",
0x2e33b798,"AocField_Enemy_Bokoblin_Junior_133290624",
0x2e4c650f,"AocField_TBox_Dungeon_Stone_4112942550",
0x2e50960e,"AocField_Weapon_Spear_038_1249463883",
0x2e512344,"AocField_Animal_Insect_F_3770894722",
0x2e5fe9b7,"AocField_Kibako_Contain_100enemy_1031406338",
0x2e7855bb,"AocField_Animal_Fish_J_1658724370",
0x2eac93be,"AocField_Kibako_Contain_100enemy_2285249992",
0x2ecd0cc2,"AocField_Weapon_Lsword_020_1055326595",
0x2ede230f,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3818466056",
0x2f01bd99,"AocField_Animal_Fish_J_853629775",
0x2f05418e,"AocField_Item_Mushroom_E_3002820133",
0x2f057b38,"MainField_Enemy_Moriblin_Junior_790636477",
0x2f21e7e3,"AocField_WeaponDrop_407182387",
0x2fa26a2a,"AocField_Enemy_Guardian_B_55729667",
0x2fddd029,"MainField_WeaponDrop_2497584981",
0x30629742,"100enemy_ToDeku",
0x30689b8a,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_4020411446",
0x3090a5e8,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1094541451",
0x309d73df,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1941299093",
0x30f538bc,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3609046300",
0x30fe5fb3,"AocField_WeaponDrop_2120842754",
0x3163035e,"MainField_TBox_Field_Wood_1594546169",
0x31800c64,"AocField_Enemy_Bokoblin_Junior_3043629014",
0x31bba513,"AocField_Enemy_Guardian_C_418190687",
0x325f6da0,"AocField_Enemy_Moriblin_Junior_3320096340",
0x32931b1d,"AocField_Obj_LiftRockWhite_100enemy_2954940154",
0x329ac590,"AocField_Kibako_Contain_100enemy_453154998",
0x32e4012e,"TreasureHunt_Aoc1_RunAutoOrder",
0x330bbeca,"MainField_TBox_Field_Iron_3102980308",
0x33225edf,"MainField_TBox_Field_Wood_266157088",
0x33b1af04,"MainField_Enemy_Lynel_Senior_1288562591",
0x33cc9003,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_2497584981",
0x33dc0f7a,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3779788102",
0x3429d57e,"MainField_TBox_Field_Wood_1766076031",
0x343ce809,"MainField_WeaponDrop_2383519508",
0x344277b2,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1066146870",
0x34d039ee,"MainField_WeaponDrop_2285582472",
0x3505e6b5,"MainField_KibakoDesert_Contain_01_171086335",
0x351b78c9,"AocField_Enemy_Lizalfos_Fire_3551161764",
0x35281b70,"AocField_Weapon_Bow_003_64962118",
0x353257f0,"TreasureHunt_touzoku01_Step01",
0x353a34ed,"AocField_Enemy_Bokoblin_Junior_3210898062",
0x3564d062,"MainField_TBox_Field_Iron_2194167583",
0x35b97b7c,"AocField_Enemy_Octarock_421414577",
0x35d5a30a,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_2686098826",
0x35e19549,"AocField_WeaponDrop_1008818560",
0x35f1aeee,"MainField_WeaponDrop_2157699647",
0x35f718f6,"AocField_Enemy_Bokoblin_Middle_1394437603",
0x36059dbf,"IsRegisteredPictureBook_Enemy_Moriblin_Gold",
0x36432eeb,"AocField_Weapon_Sword_043_4196123312",
0x368cae52,"MainField_WeaponDrop_4173853490",
0x369ee7ef,"AocField_WeaponDrop_2557148372",
0x36aa1719,"TreasureHunt01_Ready",
0x36b6937a,"AocField_TBox_Dungeon_Stone_2228603863",
0x37117fdd,"MainField_WeaponDrop_3326076576",
0x37c5dd0a,"AocField_Kibako_Contain_100enemy_1955634808",
0x37d99bc5,"AocField_Enemy_Lynel_Middle_148282207",
0x37f3bb9e,"MainField_WeaponDrop_2150075210",
0x38112eaf,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2113781419",
0x3812cb0d,"AocField_WeaponDrop_3210898062",
0x3874df99,"MainField_TBox_Field_Iron_2560994464",
0x38a63080,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3898937639",
0x38a83428,"MainField_TBox_Field_Iron_1116013200",
0x38bb6379,"AocField_Kibako_Contain_100enemy_2747079091",
0x391c82a9,"100enemy_Activated",
0x394e151a,"FirstTry_100enemy_Middle",
0x395d709e,"AocField_Weapon_Sword_008_1945450297",
0x3971be4c,"100enemy_ClearScene_7",
0x397bb4a3,"AocField_Enemy_Wizzrobe_Ice_Senior_3040193959",
0x39a29070,"AocField_Weapon_Sword_044_4104971746",
0x39c7b1a5,"AocField_TBox_Dungeon_Stone_456421641",
0x39f9c24d,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3219348996",
0x39fead0f,"AocField_Barrel_100enemy_3215386173",
0x3a1f4a9e,"AocField_Obj_LiftRockWhite_100enemy_2004919438",
0x3a27c4de,"MainField_TBox_Field_Iron_422501104",
0x3a3c7572,"AocField_Enemy_Guardian_Mini_Baby_1686109666",
0x3a62cb30,"AocField_KibakoSnowMountain_Contain_100enemy_1028908709",
0x3ab42882,"AocField_TBox_Dungeon_Wood_AreaCulling_2179207388",
0x3ac29b63,"AocField_Enemy_Bokoblin_Middle_4247305728",
0x3b1889be,"AocField_Weapon_Bow_035_1838654353",
0x3b37c47e,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1848240742",
0x3bb84d73,"AocField_Obj_BreakBoxIron_100enemy_2944589637",
0x3bc6a24e,"AocField_Obj_BreakBoxIron_100enemy_1533870992",
0x3c2ae974,"AocField_TBox_Dungeon_Iron_4293408431",
0x3c4faeae,"AocField_WeaponDrop_837643629",
0x3ca2814b,"AocField_Weapon_Spear_004_2559734140",
0x3ca8f384,"AocField_WeaponDrop_3043629014",
0x3ce85135,"AocField_WeaponDrop_1879072025",
0x3d66556c,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3701576696",
0x3d716cb5,"AocField_Animal_Insect_F_3316835336",
0x3dd1a65c,"MainField_WeaponDrop_1068462048",
0x3de9b928,"MainField_Enemy_Bokoblin_Junior_1089892972",
0x3e007989,"MainField_WeaponDrop_1486392406",
0x3e1c7a55,"100enemy_ClearScene_3",
0x3e285bbf,"AocField_Kibako_Contain_100enemy_3217837533",
0x3e327182,"AocField_Weapon_Bow_027_1817538661",
0x3e3a3275,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_4264205954",
0x3e421d1f,"AocField_Kibako_Contain_100enemy_2657797592",
0x3e7e8995,"AocField_Weapon_Spear_024_284462119",
0x3ed6d7eb,"AocField_Kibako_Contain_100enemy_3881318145",
0x3f5c94ec,"MainField_TBox_Field_Iron_3527834409",
0x3fc39862,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1609910351",
0x403e738f,"AocField_Obj_BreakBoxIron_100enemy_3626309397",
0x405cd940,"AocField_Weapon_Shield_021_3219760744",
0x40932584,"AocField_TBox_Dungeon_Stone_174323478",
0x40efcdda,"AocField_Weapon_Sword_043_2038534219",
0x40f6d733,"AocField_TBox_Dungeon_Iron_3423461010",
0x4123eb24,"AocField_Weapon_Sword_047_394956366",
0x415fc757,"AocField_Item_Mushroom_O_1432198",
0x41c890b8,"AocField_Enemy_Lizalfos_Middle_378885091",
0x420f9987,"AocField_Enemy_Bokoblin_Guard_Junior_1548594885",
0x42341b38,"AocField_Kibako_Contain_100enemy_1463078659",
0x4297fa78,"MainField_TBox_Field_Iron_2279135900",
0x434ca0a3,"AocField_WeaponDrop_1549225368",
0x437b0366,"AocField_Obj_LiftRockWhite_100enemy_1177823905",
0x438206f3,"AocField_WeaponDrop_2765819885",
0x43c2906e,"AocField_WeaponDrop_4247305728",
0x43d89002,"AocField_Kibako_Contain_100enemy_4047750327",
0x440a1075,"MainField_TBox_Field_Iron_2170253242",
0x444407d7,"MainField_TBox_Field_Iron_1099152663",
0x445bf9f0,"MainField_TBox_Field_Iron_3632983213",
0x445ec650,"AocField_Enemy_Bokoblin_Guard_Junior_Ambush_1879072025",
0x44c6e820,"AocField_Weapon_Sword_043_2476186223",
0x44ce9c7d,"AocField_TBox_Dungeon_Stone_1274667194",
0x44d082f7,"AocField_Weapon_Sword_021_1412952091",
0x44e5ac3d,"MainField_TBox_Field_Iron_158219254",
0x44f3d1a6,"MainField_WeaponDrop_2441464160",
0x44fc309d,"AocField_Item_RoastFish_01_1353716392",
0x451d79c4,"AocField_Item_Mushroom_L_1895101132",
0x459bd8ae,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2494770958",
0x45a29f82,"AocField_Kibako_Contain_100enemy_4012251368",
0x45d4d080,"AocField_Enemy_Bokoblin_Junior_3296885962",
0x45e7d69b,"MainField_WeaponDrop_2720829350",
0x4638bf40,"MainField_WeaponDrop_2597747360",
0x467592fa,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3326313096",
0x474406a5,"MainField_TBox_Field_Iron_2674008497",
0x4746348b,"AocField_Item_Mushroom_M_2199092408",
0x47692909,"AocField_Item_Fruit_I_594029975",
0x48534141,"AocField_KibakoSnowMountain_Contain_100enemy_3150856332",
0x487c9e2e,"AocField_Kibako_Contain_100enemy_349787607",
0x488a5f61,"AocField_Enemy_Lizalfos_Bone_Junior_3324670781",
0x48ea3fb2,"AocField_Barrel_100enemy_588177354",
0x48fc2f60,"AocField_WeaponDrop_3296885962",
0x4905d0c6,"AocField_Kibako_Contain_100enemy_4129738919",
0x491b4ac3,"100enemy_ClearScene_2",
0x4976afc3,"AocField_Item_Mushroom_E_344747409",
0x497dfeb6,"AocField_Weapon_Sword_021_3260611739",
0x499fc9af,"AocField_Barrel_100enemy_2839653736",
0x49d2e711,"AocField_Enemy_Bokoblin_Bone_Junior_AllDay_2455044910",
0x49e43b08,"AocField_Enemy_Chuchu_Fire_Middle_3949999987",
0x49fe54be,"AocField_Enemy_Bokoblin_Middle_2120842754",
0x4a00b5e1,"AocField_WeaponDrop_4011199496",
0x4a1fff55,"TreasureHunt_touzoku02_Ready",
0x4a24efa3,"TreasureHunt02_Step01",
0x4a3599a9,"AocField_WeaponDrop_3551161764",
0x4a4c819c,"AocField_Enemy_Lizalfos_Middle_1207782276",
0x4a637006,"AocField_Enemy_Lizalfos_Bone_Junior_1616738441",
0x4ad2581b,"MainField_TBox_Field_Iron_1633197501",
0x4b09d632,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3881851610",
0x4bb7ec73,"AocField_Obj_LiftRockWhite_100enemy_3522670394",
0x4bbdc70c,"AocField_Enemy_Guardian_Mini_Junior_Wipe_1967057286",
0x4bc51424,"AocField_Obj_BreakBoxIron_100enemy_1178698422",
0x4bc880c4,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3580607483",
0x4bc8fc67,"MainField_WeaponDrop_3346705999",
0x4bd62d8e,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1697234423",
0x4be782a9,"AocField_WeaponDrop_3261757555",
0x4cdb72f7,"MainField_TBox_Field_Iron_3096228215",
0x4ceacbcc,"IsPlayed_Demo211_0",
0x4cf06113,"AocField_Enemy_Guardian_C_1895229974",
0x4cf0e47f,"AocField_Item_Material_04_3873263637",
0x4cf713fb,"AocField_WeaponDrop_1394437603",
0x4d052928,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1455489018",
0x4d06cad7,"MainField_WeaponDrop_1218994038",
0x4d28a1fb,"IsPlayed_Demo210_0",
0x4d43dbcb,"AocField_Animal_Insect_F_1967244461",
0x4d7fb4c8,"AocField_Kibako_Contain_100enemy_3087304988",
0x4da60218,"AocField_KibakoSnowMountain_Contain_100enemy_4128191744",
0x4da895b9,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1640931976",
0x4e54c1d0,"AocField_Enemy_Keese_Fire_AllDay_1709332493",
0x4e768eda,"100enemy_ClearScene_6",
0x4e8a2c07,"AocField_Item_Mushroom_M_3688152102",
0x4eaaf913,"AocField_Enemy_Bokoblin_Junior_2765819885",
0x4ebae192,"AocField_Kibako_Contain_100enemy_818218312",
0x4f666881,"AocField_Obj_BreakBoxIron_100enemy_1165951692",
0x4f7c2186,"IsGet_Armor_171_Head",
0x4fdc02e6,"MainField_TBox_Field_Iron_1777878794",
0x50ad01f2,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_465634981",
0x50b4ae87,"MainField_WeaponDrop_1281550913",
0x50ff1abf,"AocField_Enemy_Bokoblin_Bone_Junior_AllDay_2710096289",
0x51013284,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2038154970",
0x510e4182,"AocField_WeaponDrop_3541684118",
0x51421904,"AocField_Animal_Fish_A_4191379272",
0x51774d76,"MainField_WeaponDrop_502637932",
0x5279113e,"IsGet_Armor_176_Head",
0x52d43720,"AocField_Kibako_Contain_100enemy_3852961381",
0x5333aa4f,"MainField_WeaponDrop_790636477",
0x5345f754,"MainField_WeaponDrop_2521681360",
0x53574cc6,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2538656462",
0x535bb72a,"MainField_WeaponDrop_3730939662",
0x53bf5cb0,"AocField_Enemy_Bokoblin_Guard_Junior_Ambush_602266139",
0x54000758,"FirstRetireMessage_100enemy",
0x541ed94a,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2480859927",
0x542dceb0,"AocField_Enemy_Bokoblin_Middle_3083454260",
0x5453991a,"TreasureHunt01_Activated",
0x545fbd2a,"AocField_Enemy_Bokoblin_Junior_2206056164",
0x54c6e40b,"AocField_Weapon_Sword_021_3883432977",
0x54d8efbd,"AocField_Enemy_Moriblin_Senior_Volcano_2104267058",
0x550d5033,"AocField_Weapon_Spear_004_1651801548",
0x551ac8de,"AocField_Kibako_Contain_100enemy_419810380",
0x553850fe,"AocField_Enemy_Bokoblin_Middle_4275721638",
0x558f1999,"MainField_TBox_Field_Wood_674475238",
0x55b306dd,"AocField_Enemy_Chuchu_Ice_Middle_1551265163",
0x55be9e96,"AocField_TBox_Dungeon_Iron_3429370975",
0x55c255a1,"AocField_WeaponDrop_2214319269",
0x56682011,"AocField_Kibako_Contain_100enemy_2776517222",
0x566a9231,"AocField_Barrel_100enemy_4235731015",
0x5685b4df,"AocField_WeaponDrop_2007603243",
0x56a0b1a9,"MainField_TBox_Field_Iron_3102932533",
0x56a5189d,"AocField_Kibako_Contain_100enemy_1131083373",
0x57189618,"MainField_WeaponDrop_3881851610",
0x572052a1,"AocField_Item_Fruit_A_879251830",
0x5752b44f,"AocField_Enemy_Giant_Middle_2405663079",
0x5752c47a,"AocField_WeaponDrop_1968731723",
0x5783b07b,"AocField_TBox_Dungeon_Stone_1482414257",
0x5799f788,"MainField_WeaponDrop_3395792517",
0x57aa31f8,"AocField_Item_Fruit_A_3559881878",
0x58168df8,"AocField_Item_Mushroom_A_3738042997",
0x5858354a,"AocField_Enemy_Bokoblin_Junior_846222648",
0x588fbbeb,"AocField_Kibako_Contain_100enemy_1555219203",
0x58dd9184,"MainField_WeaponDrop_3839482753",
0x58e2918c,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_2441464160",
0x5930c85f,"AocField_Kibako_Contain_100enemy_2877665971",
0x597742ca,"AocField_WeaponDrop_2206056164",
0x5a6d9b1e,"AocField_Enemy_Lizalfos_Fire_1291264366",
0x5a8bdbf0,"AocField_Animal_Fish_H_4116855750",
0x5a8e6ac1,"MainField_TBox_Field_Iron_695521123",
0x5a9e7301,"AocField_WeaponDrop_2536160814",
0x5b53d953,"MainField_TBox_Field_Wood_1382540366",
0x5ba5da22,"AocField_TBox_Dungeon_Stone_2733554147",
0x5bb261b5,"AocField_WeaponDrop_2904612702",
0x5be9d29f,"AocField_Enemy_Moriblin_Senior_2403323095",
0x5c26be62,"AocField_Enemy_Bokoblin_Junior_3541684118",
0x5c375c45,"AocField_Obj_LiftRockWhite_100enemy_2508811980",
0x5c612ba7,"AocField_TBox_Dungeon_Iron_2715337580",
0x5caee2c8,"AocField_TBox_Dungeon_Wood_AreaCulling_4102891730",
0x5cbe63fd,"MainField_TBox_Field_Wood_2945309755",
0x5cc5ce0e,"AocField_WeaponDrop_1679651783",
0x5d4d30a1,"AocField_Weapon_Spear_008_626470634",
0x5d577f8e,"AocField_Weapon_Spear_002_818808426",
0x5d62fa32,"AocField_WeaponDrop_2606251326",
0x5d7dd506,"AocField_Weapon_Sword_043_1550633220",
0x5d96baa4,"AocField_Animal_Fish_A_1604605180",
0x5d9a9fad,"AocField_Animal_Insect_F_2382599110",
0x5da84807,"AocField_Item_Mushroom_L_2924948081",
0x5dfc505a,"AocField_IsDefeat_Enemy_Golem_Junior_732702802",
0x5dfd8c40,"MainField_WeaponDrop_143431898",
0x5e307621,"AocField_Enemy_Bokoblin_Middle_3349486980",
0x5e9dfb33,"MainField_WeaponDrop_1176137530",
0x5ec8cebd,"AocField_Enemy_Bokoblin_Bone_Junior_AllDay_1026504330",
0x5f1dc5bd,"AocField_Kibako_Contain_100enemy_2501257626",
0x5fa2294a,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2349472777",
0x602d98ea,"AocField_Enemy_Lizalfos_Guard_Middle_1932916481",
0x60b01fc9,"MainField_WeaponDrop_1855292179",
0x60b0ce8b,"MainField_TBox_Field_Wood_2353666242",
0x60c1bbb6,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2159353477",
0x60c82c65,"MainField_TBox_Field_Iron_3382781565",
0x60e641d5,"AocField_Enemy_Moriblin_Bone_Junior_3741193877",
0x60f5e30c,"MainField_Enemy_Bokoblin_Junior_3773799888",
0x610235ff,"AocField_Weapon_Shield_004_3752716053",
0x611ce635,"AocField_Weapon_Lsword_032_4133051382",
0x6147f226,"AocField_Weapon_Sword_005_3455635625",
0x615b97b1,"AocField_Enemy_Guardian_Mini_Baby_3510930809",
0x617f5925,"AocField_TBox_Dungeon_Stone_3182591387",
0x619393e6,"AocField_Weapon_Sword_043_2141512552",
0x61b2d6f6,"AocField_Weapon_Sword_021_3120205306",
0x61b43a99,"AocField_Enemy_Giant_Middle_3878288352",
0x624d9cd1,"AocField_Weapon_Lsword_002_3535992885",
0x62513492,"AocField_Enemy_Bokoblin_Guard_Middle_4194395711",
0x627be25a,"AocField_Barrel_100enemy_2239353982",
0x628de95e,"MainField_WeaponDrop_3938225711",
0x628e5565,"100enemy_OnceRetire",
0x62c75fd9,"AocField_Weapon_Shield_003_2798350388",
0x62eecc8c,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1232681151",
0x633091f0,"AocField_TBox_Dungeon_Iron_2107255720",
0x633ca4ee,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1735539233",
0x63632e83,"AocField_Weapon_Sword_070_1177494317",
0x63cc459d,"MainField_WeaponDrop_485819731",
0x6466ccb6,"AocField_Obj_BreakBoxIron_100enemy_4052039188",
0x64a3b169,"AocField_Weapon_Sword_043_3848394317",
0x6501704d,"AocField_Weapon_Bow_011_1321287105",
0x652cdb03,"AocField_Item_Material_04_3041332113",
0x654b8b60,"AocField_Obj_LiftRockWhite_100enemy_3197979786",
0x6555507a,"AocField_WeaponDrop_4117390935",
0x65628d80,"AocField_Enemy_Moriblin_Senior_Volcano_1968731723",
0x65bab544,"AocField_Enemy_Moriblin_Junior_3475091277",
0x66055f6c,"AocField_WeaponDrop_2059752448",
0x66412694,"AocField_Barrel_100enemy_3223828672",
0x66870b79,"MainField_WeaponDrop_4157417392",
0x66e8a647,"AocField_WeaponDrop_2104267058",
0x66eadec0,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1259286104",
0x66ecd482,"AocField_Animal_Fish_A_3981109343",
0x670628c5,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3758988995",
0x673d72d0,"AocField_OreDrop2_Enemy_Golem_Fire_1669867293",
0x676bf9cf,"MainField_TBox_Field_Wood_1125107508",
0x67727e29,"MainField_TBox_Field_Wood_3820232600",
0x67958977,"AocField_Kibako_Contain_100enemy_1265451991",
0x67bca42e,"AocField_Weapon_Sword_070_1103301631",
0x67c986cb,"AocField_Enemy_Lizalfos_Middle_2186966282",
0x67f46cbe,"MainField_TBox_Field_Iron_873002914",
0x68035f57,"MainField_WeaponDrop_2120813408",
0x69fd5784,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2899011447",
0x6a796a01,"AocField_Item_Mushroom_A_737234309",
0x6a9361cb,"AocField_Item_Fruit_I_2661433689",
0x6ab43969,"AocField_Enemy_Bokoblin_Junior_407182387",
0x6abf9da9,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2795912794",
0x6acfe49b,"AocField_Kibako_Contain_100enemy_1321024398",
0x6b47cfb1,"MainField_WeaponDrop_2636666805",
0x6b577ded,"AocField_Kibako_Contain_100enemy_510886568",
0x6b96107f,"AocField_Weapon_Spear_004_2488555540",
0x6ba66912,"AocField_WeaponDrop_133290624",
0x6bd2358a,"MainField_TBox_Field_Wood_3107083095",
0x6c09a2f8,"AocField_Item_RoastFish_01_2368129837",
0x6c6413e1,"MainField_WeaponDrop_1648654830",
0x6c77343f,"AocField_IceWall_1996418491",
0x6ce99ddf,"AocField_Enemy_Octarock_Stone_785844733",
0x6d25b96f,"AocField_WeaponDrop_3006758860",
0x6d67de48,"AocField_IsDefeat_Enemy_Golem_Ice_3122104937",
0x6d7dcbd4,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_389373855",
0x6df11fd8,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_910514622",
0x6dfeab18,"AocField_Barrel_100enemy_2417542045",
0x6e25b68a,"AocField_Enemy_Wizzrobe_Electric_2542879058",
0x6e786dcc,"100enemy_Clear_Junior",
0x6f1393b8,"MainField_WeaponDrop_1368399329",
0x6f19434a,"IsPlayed_Demo202_3",
0x6f27c2ef,"MainField_WeaponDrop_1824929331",
0x6f37a08a,"AocField_WeaponDrop_97128967",
0x6f5b1423,"AocField_Kibako_Contain_100enemy_1442330905",
0x7030b14b,"IsNewPictureBook_Enemy_Bokoblin_Gold",
0x707553cb,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1648654830",
0x7078a806,"AocField_WeaponDrop_1932916481",
0x710d8837,"MainField_TBox_Field_Iron_3763855960",
0x71a1e9c8,"AocField_Item_Mushroom_L_1280027919",
0x71e247e2,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3523641314",
0x71f044be,"AocField_Weapon_Sword_004_3949923198",
0x72147edf,"AocField_WeaponDrop_2366002857",
0x7280363f,"MainField_TBox_Field_Wood_2961022681",
0x72ba7785,"AocField_Barrel_100enemy_3437860932",
0x72dcf710,"AocField_Kibako_Contain_100enemy_2147989396",
0x72e8967e,"AocField_Enemy_Giant_Bone_AllDay_173392836",
0x72f4dda5,"MainField_WeaponDrop_4031610821",
0x7302d392,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1368399329",
0x73149004,"AocField_WeaponDrop_3320096340",
0x731cd96b,"AocField_Enemy_Keese_Electric_AllDay_695342366",
0x733682c5,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1824929331",
0x7362bbc9,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3193935694",
0x7370cc72,"MainField_Enemy_Moriblin_Junior_3205864279",
0x7375acd6,"AocField_Enemy_Guardian_Mini_Baby_2604315419",
0x7379bdb5,"FirstTry_100enemy",
0x737efec8,"AocField_Enemy_Bokoblin_Junior_2835212048",
0x73a57a23,"AocField_WeaponDrop_2528793453",
0x73aa9236,"MainField_WeaponDrop_3514989850",
0x73f7e274,"TreasureHunt_touzoku04_Open02",
0x74121f7d,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_2120813408",
0x7450353c,"MainField_TBox_Field_Iron_558118130",
0x7463d09a,"AocField_Enemy_Moriblin_Bone_Junior_1231095740",
0x747e8ed4,"MainField_WeaponDrop_2883241798",
0x74d268fc,"AocField_IceWall_100enemy_3741836726",
0x74f0e228,"AocField_BoarMeat_3760972191",
0x75546973,"AocField_Enemy_Lizalfos_Fire_2697030370",
0x75930a1a,"AocField_Barrel_100enemy_1192616857",
0x7595907b,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_4155649555",
0x7599b2fa,"AocField_Item_Fruit_G_3424627023",
0x75d60f1c,"AocField_Item_Mushroom_L_4150619964",
0x75d8aa78,"100enemy_Active2",
0x761e5901,"AocField_Animal_Insect_F_2405359660",
0x76269558,"AocField_Kibako_Contain_100enemy_3753798038",
0x766e34b7,"AocField_Enemy_Golem_Junior_732702802",
0x76b57dbe,"MainField_TBox_Field_Wood_1701427516",
0x7717508d,"TreasureHunt_touzoku03_Step01",
0x780d1849,"MainField_TBox_Field_Iron_NoReaction_Aoc_4123439869",
0x784319d5,"AocField_Weapon_Sword_044_1072707143",
0x787008d3,"AocField_Item_Mushroom_M_2107881362",
0x78a70275,"AocField_Weapon_Shield_021_4181616312",
0x78ca4f85,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2716371349",
0x78d74344,"AocField_Enemy_Bokoblin_Bone_Junior_AllDay_2588793033",
0x7932eb6d,"TreasureHunt_touzoku03_Open02",
0x7938b211,"MainField_WeaponDrop_4211944607",
0x79674505,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3161032034",
0x79ab883e,"AocField_Obj_LiftRockWhite_100enemy_2988939776",
0x79ac26f4,"AocField_TBox_Dungeon_Stone_1841209160",
0x79dd3dfe,"AocField_Enemy_Chuchu_Fire_Junior_2967198789",
0x7a444489,"MainField_TBox_Field_Wood_3071302336",
0x7a964b53,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_4157417392",
0x7ad0dd97,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_502637932",
0x7b1768ef,"MainField_WeaponDrop_3758988995",
0x7b1c3fc6,"MainField_TBox_Field_Wood_837372312",
0x7b3a0793,"MainField_TBox_Field_Iron_3090346856",
0x7b3bd254,"AocField_Kibako_Contain_100enemy_3636398579",
0x7b83fe6c,"TreasureHunt_touzoku01_Activated",
0x7bb548ff,"MainField_WeaponDrop_2325419409",
0x7bc39074,"AocField_Obj_LiftRockWhite_100enemy_2372466948",
0x7bdd4ce8,"AocField_Kibako_Contain_100enemy_3636231680",
0x7bf7e5c8,"TreasureHunt_touzoku01_Ready",
0x7bfd28b9,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1020292918",
0x7c3a9620,"AocField_Obj_LiftRockWhite_100enemy_2423114172",
0x7c65b89c,"AocField_Weapon_Spear_021_1035587835",
0x7d22dc0b,"AocField_WeaponDrop_1810257339",
0x7d407358,"AocField_Enemy_Bokoblin_Bone_Junior_AllDay_2879767124",
0x7dc37a60,"TreasureHunt_touzoku02_Finish",
0x7dd25994,"TreasureHunt_touzoku04_Step01",
0x7deb22cd,"AocField_Kibako_Contain_100enemy_3411395211",
0x7e560128,"AocField_WeaponDrop_2835212048",
0x7e8b72c7,"IsNewPictureBook_Enemy_Lynel_Gold",
0x7ea389aa,"MainField_TBox_Field_Iron_NoReaction_Aoc_1305595304",
0x7eff8ca6,"MainField_WeaponDrop_1232681151",
0x7f0b039d,"AocField_Enemy_Guardian_A_Fixed_Moss_3194647116",
0x7f11a4d0,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_387501742",
0x7f2de4c4,"MainField_WeaponDrop_1735539233",
0x7f68e870,"AocField_Weapon_Shield_021_873499762",
0x7f7b7637,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3033977400",
0x7ffd3533,"AocField_Enemy_Chuchu_Fire_Middle_726310483",
0x8006a85c,"AocField_Enemy_Bokoblin_Bone_Junior_AllDay_3261757555",
0x80169f2f,"AocField_Enemy_Lizalfos_Junior_1008818560",
0x8020d6de,"AocField_Weapon_Bow_003_1230043626",
0x80bf82f6,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3395792517",
0x80f5bfa2,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_487037258",
0x81172266,"IsPlayed_Demo202_1",
0x8139f4ef,"AocField_Barrel_100enemy_1253622649",
0x814f39bc,"AocField_Enemy_Bokoblin_Bone_Junior_AllDay_231918048",
0x81aec515,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_528473653",
0x81b44d9c,"AocField_Kibako_Contain_100enemy_2246174763",
0x81ec9b09,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3407924480",
0x81f33880,"AocField_WeaponDrop_2290900596",
0x8233cde4,"AocField_WeaponDrop_2455044910",
0x824b4a14,"AocField_Weapon_Spear_021_1556796322",
0x8299ce25,"MainField_TBox_Field_Wood_3325939146",
0x82b190c2,"AocField_TBox_Dungeon_Stone_565925996",
0x82c23c4d,"MainField_TBox_Field_Iron_NoReaction_Aoc_135294763",
0x832b0072,"AocField_Item_Mushroom_A_2374387249",
0x8338ac34,"MainField_WeaponDrop_2480859927",
0x836a7ff7,"AocField_Weapon_Lsword_020_2483433364",
0x837bde8c,"AocField_IceWall_100enemy_2424872745",
0x83a3ac30,"AocField_Obj_BreakBoxIron_100enemy_301051486",
0x83b2d696,"AocField_Enemy_Golem_Little_Fire_2326222515",
0x83cc46d9,"MainField_TBox_Field_Iron_2276860653",
0x83ee7ca2,"AocField_Item_Mushroom_M_1482322139",
0x841ab2d4,"AocField_Weapon_Sword_043_1186218550",
0x845834e3,"IsPlayed_Demo205_1",
0x8463822a,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2521681360",
0x847139b8,"MainField_WeaponDrop_2538656462",
0x847dc254,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3730939662",
0x84a89e39,"AocField_Kibako_Contain_100enemy_3849801357",
0x84f8cf77,"AocField_Obj_RockBroken_A_02_3513257577",
0x852413d1,"AocField_WeaponDrop_2435412095",
0x853d808f,"AocField_Barrel_100enemy_3781355565",
0x858bc8e8,"MainField_KibakoDesert_Contain_01_2497056202",
0x85b2da36,"MainField_Enemy_Lizalfos_Junior_612842970",
0x862747fa,"MainField_WeaponDrop_2038154970",
0x863d9daf,"AocField_Weapon_Lsword_020_1680865071",
0x86bca8bd,"AocField_Item_Fruit_A_593909390",
0x87184965,"AocField_TBox_Dungeon_Stone_4245255027",
0x874db29b,"IsPlayed_Demo299_0",
0x876f7ea4,"AocField_Item_Fruit_G_569032412",
0x8792dbf9,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1281550913",
0x87e4e283,"AocField_Enemy_Bokoblin_Junior_4207265975",
0x8812c45c,"MainField_TBox_Field_Iron_4039997011",
0x8815adc8,"AocField_Barrel_100enemy_443896522",
0x88845c34,"MainField_WeaponDrop_2349472777",
0x88fbac50,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_587467696",
0x893ad995,"MainField_TBox_Field_Iron_2123040883",
0x893f7c81,"AocField_TBox_Dungeon_Stone_550209937",
0x897b61a2,"AocField_Enemy_Lizalfos_Fire_2179066050",
0x89bb8e4d,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1176137530",
0x89c2b8ee,"AocField_IsDefeat_Enemy_Giant_Middle_2405663079",
0x89ccb074,"AocField_WeaponDrop_210614366",
0x8a271ea1,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1612709432",
0x8a28c878,"AocField_Enemy_Golem_Ice_3122104937",
0x8a620c05,"AocField_WeaponDrop_2403323095",
0x8a7c80c9,"AocField_Barrel_100enemy_2329194751",
0x8acc1d63,"AocField_WeaponDrop_4207265975",
0x8ad169a4,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1482250345",
0x8b099216,"AocField_Kibako_Contain_100enemy_50938322",
0x8b100423,"AocField_Enemy_Guardian_Mini_Baby_2705434512",
0x8b7245e3,"AocField_Kibako_Contain_100enemy_327310132",
0x8b8dbffe,"AocField_TBox_Dungeon_Iron_3631437129",
0x8b937f17,"AocField_WeaponDrop_1207782276",
0x8ba5b61d,"AocField_WeaponDrop_2843928137",
0x8bb26576,"MainField_WeaponDrop_1738786478",
0x8bc36c5b,"AocField_Kibako_Contain_100enemy_19734485",
0x8bffa23e,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_540738285",
0x8c19c5d4,"MainField_TBox_Field_Iron_1680416200",
0x8c5ac5d0,"TreasureHunt02_Finish",
0x8c5ca5e0,"AocField_Barrel_100enemy_124805746",
0x8c84f8d5,"AocField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2059752448",
0x8c9ce3e1,"AocField_Enemy_Guardian_Mini_Baby_920565433",
0x8cac558e,"AocField_Enemy_Octarock_665210248",
0x8cd44cd1,"AocField_Enemy_Bokoblin_Middle_3071031257",
0x8cdbc760,"AocField_Enemy_Bokoblin_Junior_2290900596",
0x8ce665c3,"AocField_Enemy_Lizalfos_Fire_928938955",
0x8ceae7ff,"AocField_Weapon_Lsword_020_2937105079",
0x8d980f17,"AocField_TBox_Dungeon_Stone_223901593",
0x8da1d29c,"IsRegisteredPictureBook_Enemy_AirOctarock",
0x8f164b81,"MainField_TBox_Field_Iron_4269501339",
0x8f432af1,"AocField_WeaponDrop_4194395711",
0x8f524cca,"MainField_TBox_Field_Wood_3222760022",
0x8fa7d5e2,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_4216137812",
0x8fae936d,"AocField_Item_Mushroom_M_2466685822",
0x8fb314ae,"AocField_WeaponDrop_2198865570",
0x8fcffc54,"Open_MasterSword_FullPower",
0x8fe5ee77,"IsNewPictureBook_Enemy_Moriblin_Gold",
0x8ffbe4fa,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3839482753",
0x9032f9b1,"AocField_Weapon_Sword_043_2680107244",
0x9062b9cc,"AocField_Kibako_Contain_100enemy_2850825237",
0x909acb9b,"MainField_TBox_Field_Iron_328352811",
0x90ab24d5,"AocField_Kibako_Contain_100enemy_1747105073",
0x90de70ab,"AocField_Animal_Fish_A_3701666498",
0x91068b65,"AocField_Item_Fruit_G_1102449559",
0x911ec28c,"MainField_TBox_Field_Iron_2411774884",
0x911eca3e,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2597747360",
0x9153e784,"MainField_WeaponDrop_3326313096",
0x916ab5ea,"AocField_Animal_Fish_B_3219884379",
0x91c0d086,"IsNewPictureBook_Enemy_Lizalfos_Gold",
0x91c69857,"MainField_WeaponDrop_612842970",
0x929779d4,"AocField_Weapon_Sword_070_3293659694",
0x92b72ee9,"MainField_TBox_Field_Iron_3908874605",
0x92bbeb48,"AocField_Kibako_Contain_100enemy_1573765722",
0x92bdadd0,"MainField_WeaponDrop_2494770958",
0x92c1a3e5,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2720829350",
0x92c77e39,"AocField_Enemy_Moriblin_Senior_1549225368",
0x9329db1f,"AocField_Enemy_Bokoblin_Bone_Junior_AllDay_97128967",
0x933ad316,"AocField_WeaponDrop_4186334990",
0x9399bfd0,"AocField_Kibako_Contain_100enemy_3730464009",
0x93b695c8,"MainField_WeaponDrop_4216137812",
0x93c161b6,"AocField_OreDrop0_Enemy_Golem_Junior_732702802",
0x93d5a931,"IsGet_Armor_171_Upper",
0x93db2db2,"IsGet_Weapon_Sword_080",
0x93ee456a,"AocField_Weapon_Lsword_005_1267505148",
0x9480d489,"AocField_Enemy_Chuchu_Fire_Middle_4198147441",
0x94bf9746,"AocField_Enemy_Guardian_Mini_Junior_Wipe_3544834610",
0x94c04344,"AocField_WeaponDrop_874645658",
0x94dfa25e,"MainField_TBox_Field_Iron_520058780",
0x9529e448,"AocField_WeaponDrop_1026504330",
0x95b17ba8,"AocField_TBox_Dungeon_Wood_AreaCulling_571657376",
0x962cbba6,"AocField_Enemy_Bokoblin_Bone_Junior_AllDay_214379543",
0x96365e8b,"MainField_WeaponDrop_1612709432",
0x9683d0c7,"AocField_Enemy_Bokoblin_Bone_Junior_AllDay_2606251326",
0x96c0298e,"MainField_WeaponDrop_1482250345",
0x9724e4fb,"AocField_Enemy_Bokoblin_Bone_Junior_AllDay_1679651783",
0x978a8887,"AocField_Kibako_Contain_100enemy_2045405407",
0x97907a85,"AocField_Enemy_Chuchu_Ice_Middle_743998754",
0x97a3255c,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1738786478",
0x97a3813a,"AocField_Weapon_Sword_043_903909851",
0x97a8ec37,"AocField_Enemy_Guardian_Mini_Baby_121023524",
0x97e606f3,"AocField_Enemy_Bokoblin_Junior_1871174728",
0x98048965,"MainField_TBox_Field_Iron_2772519871",
0x9942e844,"Lock_OneHitDungeon002",
0x99d9af36,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_309234090",
0x99fb7b53,"AocField_Enemy_Chuchu_Fire_Middle_1719811928",
0x9a1a2e7a,"MainField_TBox_Field_Iron_232214255",
0x9a20bfa9,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1218994038",
0x9a235c56,"MainField_WeaponDrop_1455489018",
0x9a8ee0c7,"MainField_WeaponDrop_1640931976",
0x9aa845ce,"AocField_Weapon_Spear_021_2820926743",
0x9acef913,"AocField_WeaponDrop_1871174728",
0x9af38ba2,"AocField_Enemy_Guardian_A_Fixed_Moss_4177830736",
0x9b1e304a,"AocField_WeaponDrop_2710096289",
0x9b485b48,"AocField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_19219012",
0x9b7ba518,"AocField_WeaponDrop_59486163",
0x9c120f34,"AocField_TBox_Dungeon_Iron_1291205972",
0x9c1c53b2,"AocField_Kibako_Contain_100enemy_2483510924",
0x9cd0b632,"MainField_KibakoDesert_Contain_01_2591683407",
0x9cee8919,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3346705999",
0x9ceef5ba,"MainField_WeaponDrop_3580607483",
0x9cf058f0,"MainField_WeaponDrop_1697234423",
0x9d31104e,"AocField_Enemy_Guardian_Mini_Baby_1999865549",
0x9d33b2e5,"MainField_TBox_Field_Iron_1167469674",
0x9d7b8802,"AocField_Animal_Fish_A_1262347243",
0x9dfddb23,"MainField_WeaponDrop_3407924480",
0x9e0f64b3,"AocField_IsDefeat_Enemy_Giant_Bone_AllDay_173392836",
0x9eaf9968,"100enemy_Demo209",
0x9f0dca07,"AocField_Barrel_3914622109",
0x9f4c5362,"AocField_Kibako_Contain_100enemy_4223322606",
0x9fb98274,"AocField_Weapon_Spear_007_4153113605",
0x9fb9964c,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_824008620",
0xa04e71f2,"AocField_Kibako_Contain_100enemy_2043240994",
0xa05832df,"MainField_WeaponDrop_540738285",
0xa078eff6,"100enemy_ClearScene_4",
0xa0a2ce34,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_4002179690",
0xa1282229,"IsGet_Armor_174_Lower",
0xa132c098,"AocField_OreDrop0_Enemy_Golem_Fire_1669867293",
0xa151b395,"AocField_Weapon_Sword_044_3602522860",
0xa159d246,"AocField_Obj_RockBroken_A_02_1998521821",
0xa169369f,"MainField_Enemy_Lizalfos_Junior_800331796",
0xa1c6bfac,"AocField_Enemy_Guardian_A_Fixed_Moss_2140197374",
0xa2185b9f,"AocField_Enemy_Chuchu_Electric_Senior_248724009",
0xa2812c85,"MainField_TBox_Field_Wood_1597283440",
0xa2966f2c,"AocField_Kibako_Contain_100enemy_3031399253",
0xa29a3106,"MainField_TBox_Field_Iron_1794283002",
0xa2b3e505,"MainField_WeaponDrop_4155649555",
0xa2eca260,"AocField_WeaponDrop_1147346802",
0xa315aa82,"AocField_IceWall_653692133",
0xa3286646,"AocField_Kibako_Contain_100enemy_2704200455",
0xa358fbaa,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2883241798",
0xa35c3cb1,"MainField_WeaponDrop_587467696",
0xa3e1ffa8,"AocField_Barrel_100enemy_2428945093",
0xa444ceb7,"MainField_WeaponDrop_3193935694",
0xa48ce748,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3514989850",
0xa4daaee6,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_2030358393",
0xa54e2c0c,"AocField_Item_Material_04_1893648754",
0xa59ff6be,"AocField_WeaponDrop_19219012",
0xa5d2a8db,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_4031610821",
0xa5d5ec27,"AocField_Weapon_Lsword_011_904126487",
0xa5d95c74,"MainField_TBox_Field_Iron_2299645687",
0xa6167840,"AocField_WeaponDrop_2186966282",
0xa658ed0c,"AocField_Obj_BreakBoxIron_100enemy_3272695768",
0xa66a5076,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_2551178664",
0xa6c4329c,"MainField_WeaponDrop_3523641314",
0xa76187e7,"MainField_WeaponDrop_1168687682",
0xa7883505,"AocField_Weapon_Spear_018_4194156537",
0xa7b33bfd,"AocField_Weapon_Spear_004_3219301449",
0xa7c57c12,"MainField_TBox_Field_Wood_3338410485",
0xa7c7191b,"AocField_Animal_Insect_F_136323525",
0xa85d0349,"MainField_WeaponDrop_3033977400",
0xa9181684,"AocField_Item_Mushroom_F_1360159880",
0xa94e7530,"AocField_Kibako_Contain_100enemy_406148238",
0xa9cea3dd,"100enemy_ClearScene_8",
0xaa0955f4,"MainField_WeaponDrop_528473653",
0xaa41b24a,"AocField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2807187952",
0xaabe56d7,"AocField_Enemy_Chuchu_Fire_Middle_1552440872",
0xab2db2bd,"MainField_TBox_Field_Iron_3575111635",
0xab522f43,"MainField_WeaponDrop_487037258",
0xab53a696,"MainField_TBox_Field_Iron_NoReaction_Aoc_406427291",
0xabbe158d,"AocField_WeaponDrop_1180494379",
0xac281438,"AocField_TBox_Dungeon_Wood_AreaCulling_938107",
0xac5b1f23,"AocField_Animal_Fish_J_3299421094",
0xac933d81,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2325419409",
0xacdb5dc7,"MainField_WeaponDrop_1020292918",
0xad13db34,"AocField_Enemy_Guardian_A_Fixed_Moss_139596165",
0xadc2a12b,"AocField_Enemy_Wizzrobe_Fire_Senior_4030122309",
0xadda7e52,"AocField_Barrel_100enemy_3425481175",
0xae1ec76f,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_4211944607",
0xae36883b,"AocField_Weapon_Spear_027_2287921509",
0xae41307b,"MainField_WeaponDrop_3161032034",
0xae5462d6,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_2879824750",
0xae85d694,"AocField_Weapon_Lsword_024_1174630503",
0xaeb4c833,"MainField_WeaponDrop_794716415",
0xaec59aa6,"AocField_WeaponDrop_243985572",
0xaecf2f72,"AocField_Item_Fruit_A_1743915762",
0xaefcd506,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_2422696524",
0xaf4104aa,"AocField_Item_Fruit_A_544217849",
0xaf65a20a,"AocField_Weapon_Lsword_020_2666845516",
0xafa182be,"AocField_OreDrop1_Enemy_Golem_Junior_732702802",
0xafd82e91,"AocField_Obj_LiftRockWhite_100enemy_3760174503",
0xafec3afb,"MainField_WeaponDrop_2716371349",
0xaff6f9e4,"AocField_Item_Plant_B_1244015685",
0xb0539c55,"MainField_TBox_Field_Iron_1584660469",
0xb0b3d89b,"AocField_Item_Fruit_G_2619396626",
0xb0dfcc4e,"MainField_KibakoDesert_Contain_01_3980046316",
0xb1697afe,"TreasureHunt_touzoku03_Finish",
0xb1951545,"AocField_Enemy_Bokoblin_Junior_1660254078",
0xb1b129b1,"GuideP_VisitMark",
0xb1ccabbe,"MainField_WeaponDrop_1259286104",
0xb1dc417a,"AocField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2557148372",
0xb1e4a34b,"AocField_Kibako_Contain_100enemy_603394381",
0xb203afcd,"IsGet_Obj_WarpDLC",
0xb22a1bce,"MainField_TBox_Field_Iron_NoReaction_Aoc_408756811",
0xb24522fc,"MainField_WeaponDrop_2879824750",
0xb2746e18,"AocField_Barrel_100enemy_1264997190",
0xb27e3fd7,"MainField_WeaponDrop_309234090",
0xb294a66b,"AocField_Animal_Insect_F_3161349292",
0xb2a53c30,"AocField_Animal_Insect_F_437868312",
0xb2ed952c,"MainField_WeaponDrop_2422696524",
0xb33669b1,"AocField_WeaponDrop_2588793033",
0xb3769e02,"MainField_TBox_Field_Wood_848370940",
0xb382f676,"AocField_Enemy_Bokoblin_Junior_4051681527",
0xb3f9dd5e,"AocField_Obj_BreakBoxIron_100enemy_1780867531",
0xb41e06ad,"MainField_WeaponDrop_824008620",
0xb4de321f,"MainField_TBox_Field_Wood_974581977",
0xb50e07f6,"AocField_Barrel_702369299",
0xb51d74fe,"MainField_WeaponDrop_800331796",
0xb576b4fc,"AocField_Kibako_Contain_100enemy_123328691",
0xb5ab9c20,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3938225711",
0xb6170258,"AocField_WeaponDrop_575634755",
0xb66b03b3,"AocField_Weapon_Sword_021_240445091",
0xb697d284,"MainField_TBox_Field_Iron_2534059605",
0xb6a159ad,"AocField_WeaponDrop_2879767124",
0xb73c143b,"AocField_TBox_Dungeon_Stone_1241851576",
0xb75955c4,"AocField_Animal_Fish_B_964688885",
0xb7726b04,"MainField_TBox_Field_Wood_4041206282",
0xb789f6df,"MainField_TBox_Field_Iron_10976894",
0xb78f7ee4,"MainField_Enemy_Bokoblin_Junior_2157699647",
0xb7b13774,"MainField_TBox_Field_Iron_NoReaction_Aoc_1244520185",
0xb7e7cec8,"MainField_WeaponDrop_2159353477",
0xb87eaee3,"AocField_WeaponDrop_1042151264",
0xb8948c18,"AocField_Enemy_Guardian_A_Fixed_Moss_844656885",
0xb8cbac32,"AocField_WeaponDrop_267775485",
0xb8cbeecc,"MainField_WeaponDrop_2030358393",
0xb8ce6974,"AocField_Enemy_Chuchu_Fire_Middle_380426225",
0xb9a158e3,"MainField_TBox_Field_Iron_2110818875",
0xb9b8728f,"AocField_Kibako_Contain_100enemy_644922400",
0xb9ec6bc5,"AocField_Enemy_Lizalfos_Bone_Junior_1370841620",
0xba499c36,"AocField_Kibako_Contain_100enemy_3598145197",
0xba5bdf83,"AocField_Obj_BreakBoxIron_100enemy_691148884",
0xba698824,"AocField_Kibako_Contain_100enemy_1788777456",
0xba7b105c,"MainField_WeaponDrop_2551178664",
0xba91829c,"AocField_Barrel_100enemy_3977337074",
0xbac4998e,"MainField_TBox_Field_Iron_382308894",
0xbb70c7cd,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1168687682",
0xbb9d8b02,"AocField_TBox_Dungeon_Iron_3179964467",
0xbbac73e7,"TreasureHunt_touzoku04_Finish",
0xbbbd5013,"TreasureHunt_touzoku02_Step01",
0xbc091ef7,"AocField_Item_Fruit_A_531123345",
0xbc2c726b,"AocField_Kibako_Contain_100enemy_237338803",
0xbc61bacf,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2636666805",
0xbc64f74a,"MainField_TBox_Field_Wood_2833258530",
0xbc7801fe,"MainField_TBox_Field_Wood_652873662",
0xbcb38e1e,"MainField_WeaponDrop_4002179690",
0xbcba05e9,"MainField_KibakoDesert_Contain_01_69035756",
0xbcbdeaa5,"AocField_WeaponDrop_1660254078",
0xbcfd2280,"AocField_Enemy_Lizalfos_Middle_1810257339",
0xbd035a6b,"AocField_Weapon_Spear_003_3942895935",
0xbd49b25b,"AocField_TBox_Dungeon_Stone_2509086549",
0xbd4e6ccd,"AocField_WeaponDrop_2361801010",
0xbd6394d6,"100enemy_Active",
0xbd6d1e3c,"MainField_WeaponDrop_3205864279",
0xbd835d54,"AocField_Enemy_Moriblin_Senior_Volcano_2198865570",
0xbd99e8d7,"MainField_WeaponDrop_2795912794",
0xbeaa0996,"AocField_WeaponDrop_4051681527",
0xbebb190c,"AocField_Weapon_Sword_024_17490101",
0xbedb22fa,"MainField_WeaponDrop_2899011447",
0xbef42cd9,"AocField_Enemy_Guardian_Mini_Baby_1436018559",
0xbf0b3510,"AocField_WeaponDrop_2484387578",
0xbf0ba8dc,"AocField_Obj_BreakBoxIron_100enemy_817345459",
0xbf21c161,"AocField_Kibako_Contain_100enemy_228359003",
0xbf23aa87,"AocField_Obj_LiftRockWhite_100enemy_54064611",
0xbf243638,"AocField_IsDefeat_Enemy_Giant_Middle_3878288352",
0xbf976922,"MainField_WeaponDrop_1089892972",
0xbfd2a361,"AocField_Item_Mushroom_M_838713371",
0xc01c32cd,"AocField_Barrel_100enemy_1099457370",
0xc03c3f23,"MainField_TBox_Field_Iron_1894318025",
0xc0467fe5,"AocField_Necklace_4_Enemy_Giant_Middle_3878288352",
0xc0eb505a,"AocField_Weapon_Sword_003_1029321250",
0xc112e762,"MainField_WeaponDrop_671999855",
0xc14c2e83,"AocField_WeaponDrop_239572651",
0xc17ea5ee,"AocField_Enemy_Bokoblin_Middle_1042151264",
0xc19dd3dc,"MainField_WeaponDrop_2986027363",
0xc1ed8696,"IsGet_Armor_174_Upper",
0xc2019293,"AocField_Enemy_Keese_Electric_AllDay_3593952231",
0xc225ba71,"MainField_WeaponDrop_1277357706",
0xc23519bc,"AocField_OreDrop1_Enemy_Golem_Fire_1669867293",
0xc25dd4fa,"FirstRetireAccess_100enemy",
0xc2d1a48d,"AocField_TBox_Dungeon_Wood_AreaCulling_1683482510",
0xc3045c9e,"AocField_Kibako_Contain_100enemy_2939792766",
0xc3118a16,"AocField_Kibako_Contain_100enemy_2868556778",
0xc32dd899,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_4193123081",
0xc333ab8d,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1048447017",
0xc368d4a0,"MainField_TBox_Field_Wood_4058480684",
0xc3a7e458,"AocField_Obj_BreakBoxIron_100enemy_837176038",
0xc3d417b3,"MainField_WeaponDrop_954325039",
0xc40aab82,"AocField_Kibako_Contain_100enemy_1037418601",
0xc4437d55,"MainField_TBox_Field_Wood_2056288946",
0xc465ac34,"MainField_WeaponDrop_1217080480",
0xc4abe840,"TreasureHunt01_Step01",
0xc4e54971,"MainField_TBox_Field_Iron_1752755875",
0xc5022bac,"AocField_Obj_BreakBoxIron_100enemy_3606554340",
0xc523766f,"AocField_WeaponDrop_2534505824",
0xc5c0ecc7,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3486101056",
0xc60630f4,"MainField_TBox_Field_Iron_3488992931",
0xc60b3e1d,"AocField_Enemy_Bokoblin_Middle_2484387578",
0xc611ed04,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_78835459",
0xc6b4e628,"MainField_WeaponDrop_184192573",
0xc6e6d090,"MainField_WeaponDrop_1867845270",
0xc6edefec,"AocField_WeaponDrop_2011116073",
0xc7528b59,"AocField_TBox_Dungeon_Stone_3211970552",
0xc7557316,"AocField_WeaponDrop_3118350165",
0xc7b428ce,"AocField_Item_Fruit_I_4278121490",
0xc7bd6b98,"AocField_Enemy_Keese_Fire_AllDay_221775837",
0xc7c99d39,"AocField_Barrel_100enemy_2825856878",
0xc80abe8f,"AocField_WeaponDrop_928938955",
0xc80b898f,"AocField_Enemy_Bokoblin_Junior_2534505824",
0xc82a897f,"AocField_Enemy_Bokoblin_Senior_2214319269",
0xc82c0371,"AocField_Weapon_Lsword_031_2915531289",
0xc8cdf8f4,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3683406634",
0xc8e5bfc8,"AocField_Enemy_Chuchu_Electric_Middle_1814664382",
0xc8efbb8c,"AocField_WeaponDrop_629989137",
0xc95212be,"AocField_Enemy_Moriblin_Dark_2904612702",
0xc95aa2d8,"AocField_Weapon_Sword_070_753456110",
0xc96ad68b,"MainField_TBox_Field_Iron_2719018821",
0xc9ba8fd4,"FirstWarpHole_100enemy",
0xc9c1e119,"AocField_Weapon_Shield_021_4064345007",
0xc9d0c283,"IsRegisteredPictureBook_Enemy_Bokoblin_Gold",
0xc9e85328,"IsGet_Armor_172_Head",
0xca20c9e7,"AocField_Enemy_Guardian_A_1510445978",
0xca21964e,"AocField_IceWall_100enemy_2463057085",
0xca7290fb,"AocField_Enemy_Guardian_B_986866639",
0xca72a609,"AocField_Enemy_Lizalfos_Senior_3577937077",
0xca7d8cf6,"AocField_Enemy_Bokoblin_Junior_3118350165",
0xcae6aceb,"MainField_TBox_Field_Iron_1495153644",
0xcbb9c797,"MainField_TBox_Field_Iron_1930580661",
0xcbc5100c,"AocField_Enemy_Bokoblin_Junior_2011116073",
0xcbd42826,"AocField_Enemy_Guardian_Mini_Junior_2419120625",
0xcc133ed7,"MainField_TBox_Field_Iron_2711765565",
0xcc2523dd,"MainField_TBox_Field_Iron_2568257269",
0xcc596efe,"AocField_Enemy_Bokoblin_Junior_210614366",
0xcc5db0e2,"MainField_TBox_Field_Iron_NoReaction_Aoc_2377598779",
0xcc81e8a1,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2663640029",
0xccf294e7,"MainField_WeaponDrop_3838349298",
0xcd1e27b1,"MainField_TBox_Field_Iron_3921992523",
0xcd5943b3,"MainField_WeaponDrop_2969876181",
0xcd80b0ba,"AocField_Item_Fruit_A_1051304436",
0xcd8d72da,"AocField_Item_Fruit_A_3405170587",
0xcd8dc1a6,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3241035409",
0xcdb1ff1b,"TreasureHunt02_Activated",
0xcdd26e8a,"AocField_Weapon_Spear_030_3379600416",
0xce107f1c,"MainField_TBox_Field_Wood_4272592660",
0xce2c11d2,"AocField_Enemy_Lizalfos_Senior_3925713782",
0xce3fd736,"AocField_Weapon_Lsword_032_2002540715",
0xcf0ad226,"MainField_WeaponDrop_1399783231",
0xcf96666a,"AocField_Enemy_Bokoblin_Middle_828224922",
0xcfb98aa6,"MainField_KibakoDesert_Contain_01_3245228634",
0xcfca1132,"MainField_TBox_Field_Iron_NoReaction_Aoc_3010491383",
0xcff4de8c,"AocField_Barrel_100enemy_1716564852",
0xd0121b79,"100enemy_ClearScene_1",
0xd0e0c6df,"MainField_TBox_Field_Iron_562547623",
0xd0e3ed25,"MainField_WeaponDrop_3989916214",
0xd0f083d5,"AocField_Necklace_2_Enemy_Giant_Middle_2405663079",
0xd1370069,"AocField_Barrel_2410300839",
0xd157b3af,"AocField_IceWall_2014659349",
0xd1783254,"AocField_Kibako_Contain_100enemy_261828513",
0xd17a9cb2,"MainField_TBox_Field_Iron_773393274",
0xd1fb8717,"AocField_Animal_Insect_F_1163870307",
0xd20be881,"MainField_TBox_Field_Iron_3497027845",
0xd239a573,"AocField_Kibako_Contain_100enemy_1536823648",
0xd24fa4bf,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_909791261",
0xd2b7df7c,"AocField_Barrel_100enemy_1664349387",
0xd2be1e80,"AocField_Enemy_Bokoblin_Middle_1180494379",
0xd320267e,"MainField_WeaponDrop_3198555883",
0xd35506b5,"MainField_WeaponDrop_3918019498",
0xd387df51,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1775680432",
0xd3958062,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_794716415",
0xd39b1113,"AocField_Enemy_Guardian_Mini_Baby_1460262777",
0xd3b2bacb,"AocField_Obj_LiftRockWhite_100enemy_1529944601",
0xd3bb5416,"AocField_Barrel_100enemy_1742564392",
0xd4703ad4,"MainField_TBox_Field_Wood_1498602253",
0xd53611d4,"MainField_WeaponDrop_4154249897",
0xd59b63a1,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1618111426",
0xd5db9ebe,"MainField_WeaponDrop_78835459",
0xd5e2537b,"MainField_Enemy_Moriblin_Junior_3308598352",
0xd5f18bd8,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2293041085",
0xd60b3ad5,"AocField_Item_Mushroom_M_627341068",
0xd6585315,"AocField_Weapon_Spear_021_3366941034",
0xd66a1498,"MainField_TBox_Field_Iron_1478906314",
0xd6749b5c,"AocField_TBox_Dungeon_Iron_245520230",
0xd6b483cc,"100enemy_Finish",
0xd6b62116,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_4246752370",
0xd6d6e435,"AocField_Enemy_Moriblin_Bone_Junior_2361801010",
0xd719cb2f,"AocField_Weapon_Spear_005_2620883876",
0xd74b0e32,"MainField_TBox_Field_Iron_1326734489",
0xd74fe369,"AocField_Item_Fruit_A_4113469939",
0xd76aaf7b,"MainField_KibakoDesert_Contain_01_3477189961",
0xd77ae9b8,"AocField_Obj_RockBroken_A_02_1190600512",
0xd77fdf60,"100enemy_ClearScene_5",
0xd7b64ced,"AocField_Weapon_Spear_004_1892493656",
0xd7de2e34,"MainField_TBox_Field_Iron_2973433180",
0xd7f42d96,"AocField_KibakoSnowMountain_Contain_100enemy_1251537497",
0xd92536ac,"AocField_Kibako_Contain_100enemy_2654951279",
0xd9ec28ca,"AocField_Kibako_Contain_100enemy_788223424",
0xda385ee1,"AocField_Enemy_Keese_Fire_AllDay_4068296996",
0xda979ce9,"AocField_Enemy_Lynel_Senior_4060409046",
0xdb1f5e3a,"AocField_Kibako_Contain_100enemy_3178599170",
0xdb357309,"AocField_WeaponDrop_1548594885",
0xdb468743,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_433239886",
0xdb61bee1,"AocField_Enemy_Guardian_A_Fixed_Moss_2301250553",
0xdb7b8520,"AocField_Kibako_Contain_100enemy_1014260166",
0xdbd9cc55,"AocField_Enemy_Golem_Fire_1669867293",
0xdbf73eb0,"AocField_Item_RoastFish_01_2009122717",
0xdc64e0dd,"MainField_Enemy_Lizalfos_Junior_2302834303",
0xdcc1cc1d,"MainField_TBox_Field_Iron_3884079561",
0xdcfb3844,"IsRegisteredPictureBook_Enemy_Lynel_Gold",
0xdd40826e,"AocField_Obj_LiftRockWhite_100enemy_1211299388",
0xddad755e,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1811035589",
0xde176b04,"AocField_Necklace_3_Enemy_Giant_Middle_2405663079",
0xde287cc6,"MainField_KibakoDesert_Contain_01_1345016984",
0xde76860f,"AocField_Weapon_Spear_021_4105411940",
0xdec9934b,"100enemy_ClearScene_9",
0xded6e43a,"AocField_Kibako_Contain_100enemy_3567211536",
0xdf22eba7,"MainField_WeaponDrop_1048447017",
0xdf3c98b3,"MainField_WeaponDrop_4193123081",
0xdf4c0d0b,"AocField_Enemy_Bokoblin_Guard_Junior_730820286",
0xdfa562ce,"AocField_Weapon_Spear_004_784872944",
0xe0370aa3,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_3326076576",
0xe03bbad7,"TreasureHunt_touzoku03_Open01",
0xe0929ba5,"MainField_WeaponDrop_3142203791",
0xe0d17248,"AocField_Item_Mushroom_M_2542455727",
0xe0d5cee0,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2150075210",
0xe0eb4199,"AocField_WeaponDrop_1487756818",
0xe0f219f2,"MainField_TBox_Field_Wood_278435547",
0xe0fc07a7,"AocField_Enemy_Bokoblin_Middle_837643629",
0xe1140f55,"AocField_TBox_Dungeon_Iron_1521240634",
0xe141d63a,"AocField_Kibako_Contain_100enemy_2842316372",
0xe185a473,"AocField_Weapon_Spear_005_2194670812",
0xe1aadb2c,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_4173853490",
0xe24d1721,"AocField_Obj_BreakBoxIron_100enemy_2403668960",
0xe261986d,"TreasureHunt_touzoku02_Activated",
0xe2878f8e,"AocField_Kibako_Contain_100enemy_3124334903",
0xe28b3306,"MainField_WeaponDrop_3773799888",
0xe2cecfc3,"MainField_Enemy_Bokoblin_Junior_1855292179",
0xe2e19bf8,"AocField_WeaponDrop_602266139",
0xe327ac41,"MainField_TBox_Field_Wood_3137618299",
0xe362a9cc,"MainField_TBox_Field_Iron_20632817",
0xe36402cc,"MainField_WeaponDrop_1066146870",
0xe367b962,"MainField_TBox_Field_Iron_3790552559",
0xe3bee988,"AocField_Enemy_Guardian_Mini_Senior_188016384",
0xe3c9522c,"MainField_TBox_Field_Iron_2989622470",
0xe3f64c90,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2285582472",
0xe415204c,"AocField_Enemy_Wizzrobe_Fire_Senior_2330690088",
0xe43571c7,"AocField_Weapon_Lsword_047_4055507029",
0xe44a7b55,"MainField_KibakoDesert_Contain_01_1361387754",
0xe460bd39,"AocField_TBox_Dungeon_Stone_2505276374",
0xe4642391,"AocField_Item_Mushroom_J_3998642955",
0xe4dc1d24,"IsGet_Weapon_Sword_081",
0xe4ddf339,"MainField_TBox_Field_Iron_914656641",
0xe4fa7a04,"MainField_WeaponDrop_3779788102",
0xe584ad5b,"AocField_Animal_Fish_H_2528251644",
0xe590c728,"AocField_Weapon_Spear_012_2258194998",
0xe5961608,"AocField_WeaponDrop_4196419116",
0xe6117a3d,"AocField_Enemy_Bokoblin_Junior_1641158192",
0xe6160d03,"AocField_Necklace_2_Enemy_Giant_Middle_3878288352",
0xe6845555,"AocField_WeaponDrop_1370841620",
0xe6873887,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_2404842641",
0xe69d66eb,"MainField_TBox_Field_Iron_574842933",
0xe6c5c551,"MainField_WeaponDrop_2302834303",
0xe73858e4,"MainField_TBox_Field_Wood_1616967579",
0xe74ffe4b,"AocField_Obj_BreakBoxIron_100enemy_39798493",
0xe7b6d096,"MainField_WeaponDrop_1094541451",
0xe7bb06a1,"MainField_WeaponDrop_1941299093",
0xe808f3dc,"MainField_TBox_Field_Wood_3603745493",
0xe82bae4c,"AocField_Weapon_Shield_021_1069349950",
0xe8738752,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_954325039",
0xe89b1bf1,"AocField_Enemy_Golem_Little_Fire_2444235907",
0xe8bc1ec1,"MainField_WeaponDrop_3002889636",
0xe8bee5c0,"AocField_Kibako_Contain_100enemy_3285187766",
0xe8e5ed1c,"MainField_WeaponDrop_1609910351",
0xe8f1e5d2,"AocField_Necklace_3_Enemy_Giant_Middle_3878288352",
0xe9115d26,"MainField_TBox_Field_Iron_NoReaction_Aoc_long_3048637654",
0xe91c470b,"MainField_WeaponDrop_4264205954",
0xe97a6eff,"MainField_WeaponDrop_1196886240",
0xea1f7e4b,"AocField_Obj_BreakBoxIron_100enemy_2175926356",
0xea4cabe3,"MainField_Enemy_Moriblin_Junior_1808684326",
0xea771228,"AocField_Enemy_Guardian_A_Fixed_Moss_3272524320",
0xeab03b19,"MainField_TBox_Field_Wood_725308796",
0xeab57783,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_671999855",
0xeaf7d322,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_1068462048",
0xeafeb3ce,"TreasureHunt_touzoku04_Open01",
0xeb00a7a6,"AocField_OreDrop2_Enemy_Golem_Junior_732702802",
0xeb1b388a,"AocField_WeaponDrop_3925713782",
0xeb3985dd,"AocField_WeaponDrop_1641158192",
0xeb4d001d,"AocField_Weapon_Lsword_020_1388615376",
0xeb50442c,"AocField_Enemy_Bokoblin_Junior_243985572",
0xeb50781a,"AocField_Item_Mushroom_O_2791493938",
0xeb52f9fd,"AocField_Weapon_Sword_021_1475493280",
0xeb58d402,"AocField_Barrel_100enemy_1472066497",
0xec68f4e1,"TreasureHunt_touzoku03_Ready",
0xecdb0a7a,"100enemy_Ready",
0xecde8ee9,"AocField_Enemy_Guardian_Mini_Junior_4091604171",
0xece70f5b,"MainField_KibakoDesert_Contain_01_3358355792",
0xecf73908,"AocField_WeaponDrop_730820286",
0xed1376c9,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_184192573",
0xed7c4ed0,"AocField_Weapon_Sword_024_288573870",
0xedac749e,"AocField_Item_Fruit_I_953502445",
0xedc3be79,"AocField_Enemy_Bokoblin_Junior_1487756818",
0xee45d8d2,"Lock_OneHitDungeon003",
0xee61c348,"AocField_Barrel_100enemy_2631999588",
0xeedfb733,"MainField_WeaponDrop_3219348996",
0xef375bd1,"MainField_WeaponDrop_2113781419",
0xef458f51,"AocField_WeaponDrop_3577937077",
0xef5d9b92,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3923881587",
0xef83352c,"AocField_Item_Fruit_A_1198856528",
0xef8ca7b8,"AocField_Animal_Insect_F_1329362064",
0xefd58bd0,"MainField_TBox_Field_Iron_2160727846",
0xeffca201,"AocField_Enemy_Bokoblin_Senior_2366002857",
0xf01a07d3,"AocField_TBox_Dungeon_Iron_236853965",
0xf040d62c,"AocField_TBox_Dungeon_Stone_374517867",
0xf07a45fe,"AocField_Enemy_Chuchu_Junior_128431222",
0xf092a559,"AocField_KibakoSnowMountain_Contain_100enemy_3817916423",
0xf0ca2bde,"AocField_TBox_Dungeon_Stone_3605914736",
0xf0cecb4b,"AocField_Obj_LiftRockWhite_100enemy_3362006598",
0xf0d0c34a,"AocField_Kibako_Contain_100enemy_1070438958",
0xf0dbd01b,"IsPlayed_Demo207_0",
0xf0e117a2,"MainField_WeaponDrop_433239886",
0xf0f79fd9,"AocField_Kibako_Contain_100enemy_1830996287",
0xf115335a,"AocField_Weapon_Spear_004_1371091869",
0xf119ba2c,"IsPlayed_Demo206_0",
0xf29d6e42,"IsPlayed_Demo204_0",
0xf2a5bd10,"AocField_Enemy_Bokoblin_Middle_2843928137",
0xf2e90ec9,"MainField_WeaponDrop_1463562875",
0xf2ee2b24,"MainField_Enemy_Wizzrobe_Fire_3896327169",
0xf3100d8e,"IsGet_Armor_171_Lower",
0xf34c7d83,"TreasureHunt_touzoku01_Finish",
0xf34cdbb8,"MainField_WeaponDrop_3923881587",
0xf382dcd2,"AocField_Enemy_Bokoblin_Junior_575634755",
0xf456aca9,"IsPlayed_Demo201_0",
0xf48f6962,"MainField_TBox_Field_Iron_2243965962",
0xf4ad5eeb,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3002889636",
0xf4e7dcb6,"AocField_Enemy_Bokoblin_Guard_Middle_TreeHouseTop_2528793453",
0xf4f0507d,"AocField_Enemy_Guardian_Mini_Junior_2546235668",
0xf52b75cb,"AocField_Enemy_Chuchu_Fire_Junior_2788988688",
0xf52e1360,"AocField_Enemy_Bokoblin_Junior_1460194115",
0xf56b2ed5,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_1196886240",
0xf594c69e,"IsPlayed_Demo200_0",
0xf5d447dc,"AocField_WeaponDrop_3071031257",
0xf5f35644,"AocField_Obj_LiftRockWhite_100enemy_340824500",
0xf61012f0,"IsPlayed_Demo202_0",
0xf65580c2,"AocField_WeaponDrop_2179066050",
0xf659540c,"AocField_Weapon_Shield_033_1914275839",
0xf6a0f133,"AocField_Necklace_4_Enemy_Giant_Middle_2405663079",
0xf6a96273,"AocField_Obj_BreakBoxIron_100enemy_3714636362",
0xf6b26c9c,"MainField_TBox_Field_Wood_4166110215",
0xf77cfa20,"AocField_Kibako_Contain_100enemy_917619246",
0xf799ea1a,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_4004061010",
0xf7b85e08,"AocField_Kibako_Contain_100enemy_1660758767",
0xf7d278c7,"IsPlayed_Demo203_0",
0xf806ec80,"AocField_WeaponDrop_1460194115",
0xf814f62b,"AocField_Obj_LiftRockWhite_100enemy_3718604983",
0xf85df492,"AocField_Kibako_Contain_100enemy_686675665",
0xf8bd8ca4,"AocField_Enemy_Bokoblin_Senior_4117390935",
0xf91d2e22,"AocField_Barrel_100enemy_848907875",
0xf959a51f,"AocField_Obj_LiftRockWhite_100enemy_201100202",
0xf9b2704c,"IsNewPictureBook_Enemy_AirOctarock",
0xf9e8345e,"MainField_WeaponDrop_909791261",
0xf9f85671,"MainField_WeaponDrop_3818466056",
0xfa0af2b1,"AocField_Enemy_Lizalfos_Fire_2435412095",
0xfa15dc94,"AocField_Animal_Fish_H_1065791727",
0xfa234aab,"AocField_Enemy_Chuchu_Electric_Senior_398666979",
0xfa45fd11,"IsPlayed_Demo209_0",
0xfa9678ad,"MainField_WeaponDrop_2404842641",
0xfaaeb286,"AocField_Weapon_Lsword_020_4085134790",
0xfae5d1e8,"MainField_WeaponDrop_3516993228",
0xfaed90c3,"AocField_TBox_Dungeon_Wood_AreaCulling_3770784754",
0xfb2c78f3,"AocField_Weapon_Lsword_018_1882858045",
0xfb879726,"IsPlayed_Demo208_0",
0xfba6938d,"AocField_TBox_Dungeon_Wood_AreaCulling_2153587184",
0xfbd08d95,"AocField_Enemy_Moriblin_Bone_Junior_458827803",
0xfc16d1a8,"AocField_KibakoSnowMountain_Contain_100enemy_684465058",
0xfc1f9215,"MainField_KibakoDesert_Contain_01_57713397",
0xfc83db8f,"MainField_Enemy_Lizalfos_Guard_Junior_LongVisibility_3142203791",
0xfc923477,"MainField_WeaponDrop_3063344506",
0xfd29a556,"MainField_TBox_Field_Iron_2793753996",
0xfd5e72b8,"AocField_Enemy_Bokoblin_Junior_267775485",
0xfde7b3ef,"AocField_IsDefeat_Enemy_Golem_Fire_1669867293",
0xfde97946,"AocField_Enemy_Bokoblin_Middle_845395278",
0xfe3cf526,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2434795977",
0xfe3f6234,"MainField_TBox_Field_Iron_2714499651",
0xfe4d13fd,"MainField_Enemy_Wizzrobe_Ice_1992889235",
0xfe6fd490,"MainField_TBox_Field_Iron_1347548770",
0xfedf6336,"AocField_Enemy_Bokoblin_Middle_59486163",
0xff2bb086,"AocField_WeaponDrop_378885091",
0xff5fbf34,"MainField_WeaponDrop_3476407294",
0xff7e1db8,"AocField_Enemy_Lizalfos_Ice_2536160814",
0xffb09e15,"AocField_Weapon_Sword_021_1859680990",
0xffb544c9,"MainField_Enemy_Bokoblin_Guard_Junior_TreeHouseTop_2045168746",
0xffb7ea84,"AocField_TBox_Dungeon_Wood_AreaCulling_3811317078",
0xffc6fd5f,"AocField_Obj_LiftRockWhite_100enemy_1324373224",
0xfff51a39,"MainField_TBox_Field_Iron_2059226658"
	]}
];

